/*
 * Copyright (C) AU-System AB, 2000.
 * All rights reserved.
 *
 * This software is covered by the license agreement between
 * the end user and AU-System AB, and may be used and copied
 * only in accordance with the terms of the said agreement.
 *
 * AU-System AB does not assume any responsibility or 
 * liability for any errors or inaccuracies in this 
 * software, or any consequential, incidental or indirect 
 * damage arising out of the use of the Generic Layout Engine 
 * software.
 */

/* intern includes from gle */
#include "engine.h"
#include "gleelem.h"
#include "glebreak.h"
#include "glefset.h"
#include "gledef.h"
#include "ansilibs.h"
#ifdef SHOW_KEYS_IN_CONTENT
	#include "glekey.h"
#endif
#include "gleimage.h"
#include "gleinput.h"
#include "gleselct.h"
#include "gleoptn.h"
#include "gleparag.h"
#include "gletable.h"
#include "gletext.h"

#include "aapigle.h"
#include "capigle.h"
#include "gleview.h"
/*****************************/

/*** GWC includes   *********/
#include "aapimmi.h"
#include "capimmi.h"

VOID* WIPAlloc(UINT32 size);
VOID WIPFree( VOID* memory );
  
/*****************************/

GleViewType* gGleViewList=0;
BOOL gAllViewsDisplayImages=FALSE;
BOOL gAllViewsUpdateImages=FALSE;

#ifdef BITMAP_POSITION_CLICK
	UINT16 gGleImageClickXInputID = 0;
	UINT16 gGleImageClickYInputID = 0;
	UINT16 gGleImageClickXCoord = 0;
	UINT16 gGleImageClickYCoord = 0;
#endif

#ifdef DISPLAY_GLEELEMENTLIST_SIZE_IN_CONTENT
	extern WCHAR* char_2_wChar_strcpy(WCHAR* dst, const CHAR* src);
#endif

VOID GLEc_setIntConfig (UINT8 viewId, ConfigInt kind, UINT32 value){
	GleViewType* viewP;
#if (WIPVRSN_1 >= 4)
	//#if (GWCVERSION == 0400)
	if(viewId == ALL_USER_AGENT )
#elif (WIPVRSN_1 > 1)
	if(viewId == GENERAL_VIEW)
#else
	if(1) // Set config for all views if WIPVRSN_1 == 1
#endif
		{
			#if (WIPVRSN_1 > 1)
				CALL_GWC_API(CLNTc_setIntConfig(viewId,kind,value));
			#else
				CALL_GWC_API(CLNTc_setIntConfig(kind,value));
			#endif
			if(kind==configDISPLAY_IMAGES)
				gAllViewsDisplayImages=value;
			else if(kind==configUPDATE_IMAGES)
				gAllViewsUpdateImages=value;
			if(kind==configDISPLAY_IMAGES || kind==configUPDATE_IMAGES){
				// Set value for all current views:
				viewP = gGleViewList;
				while(viewP){
					if(!viewP->mMIa_wait) {
						if(kind==configDISPLAY_IMAGES)
							viewP->displayImages = value;
						else if(kind==configUPDATE_IMAGES)
							viewP->updateImages = value;
						}
					viewP=viewP->next;
					}
				}
		}
	else if(kind==configDISPLAY_IMAGES || kind==configUPDATE_IMAGES){
		viewP = GleViewSearchByViewId(viewId);
		if(!viewP->mMIa_wait) {
			if(kind==configDISPLAY_IMAGES)
				viewP->displayImages = value;
			else if(kind==configUPDATE_IMAGES)
				viewP->updateImages = value;
			#if (WIPVRSN_1 > 1)
				CALL_GWC_API(CLNTc_setIntConfig(viewId,kind,value));
			#else
				CALL_GWC_API(CLNTc_setIntConfig(kind,value));
			#endif
			}
		}
	else {
		viewP = GleViewSearchByViewId(viewId);
		if(viewP)
			if(!viewP->mMIa_wait) 
				#if (WIPVRSN_1 > 1)
					CALL_GWC_API(CLNTc_setIntConfig(viewId,kind,value));
				#else
					CALL_GWC_API(CLNTc_setIntConfig(kind,value));
				#endif
		}
	}
	
BOOL GLEc_formatStringValid(const WCHAR* format, INT16* length, FormatInfoType* info, BOOL* containsConstantChars){
	const WCHAR* f = format;
	WCHAR firstFormat = 0;
	BOOL allSameFormat = TRUE;
	*length=-1;
	*containsConstantChars=FALSE;

	if(!f)
		return FALSE;
	if(!*f)
		return FALSE;
	while(*f){
		if(*f=='\\'){
			*containsConstantChars=TRUE;
			if(!*++f)
				return FALSE;
			else 
				f++;
			}
		else if (*f=='*'){				/* "*f" */

			f++;
			if(!*f)						/* Missing f */
				return FALSE;
			else if (!IS_FORMAT(*f))	/* Invalid f */
				return FALSE;
			else if(*++f != 0)			/* Something after f */
				return FALSE;
			else { 						/* Format string is correct*/
				*length = 0; 			/*Will become -1 after the loop, return FALSE must have *length==-1*/
				break;
				}
			/*else 						
				return(*++f == 0);*/
			}
		else if(IS_DIGIT_1_TO_9(*f)) {	/* "[1-9]f" */
			if(!*++f) 					/* Missing f */
				return FALSE;			
			else {
				if(!IS_FORMAT(*f)) 		/* Invalid f */
					return FALSE;		
				else if(*++f != 0)		/* Something after f */
					return FALSE;
				else 					/* Format string is correct*/
					break;
				/*else 						
					return(*++f == 0);*/
				}
			}
		else
			if(!IS_FORMAT(*f))
				return FALSE;
		while (IS_FORMAT(*f)){f++;}
		}
	/* Format string is valid, now get length... */
	if(!*length)
		*length=-1;
	else 
		*length=0;
		
	f = format;
	if(*length != -1){
		while(*f && !IS_DIGIT_1_TO_9(*f)){
			if(*f=='\\')
				f++;
			(*length)++;
			f++;
			}
		if(IS_DIGIT_1_TO_9(*f))
			(*length) += *f - '0';
		}
	f = format;
	while(*f && !firstFormat){
		if(IS_FORMAT(*f))
			firstFormat = *f;
		f++;
		}
	/* ...and now info */
	if(!firstFormat)
		*info = PureConstant;
	else{
		while(*f && allSameFormat){
			if(IS_FORMAT(*f) && *f!=firstFormat)
				allSameFormat = FALSE;
			f++;
			}
		if(allSameFormat)
			switch(firstFormat){
				case 'A':
					*info = Only_A;
					break;
				case 'a':
					*info = Only_a;
					break;
				case 'N':
					*info = Only_N;
					break;
				case 'X':
					*info = Only_X;
					break;
				case 'x':
					*info = Only_x;
					break;
				case 'M':
					*info = Only_M;
					break;
				case 'm':
					*info = Only_m;
					break;
				}
		else
			*info = Mixed;	
		}
	return TRUE;
	}
	
BOOL GLEc_inputStringValid(const WCHAR* str, const WCHAR* format, FormatErrorType* error){
	BOOL anyNumberOf = FALSE;
	const WCHAR* f = format;
	UINT16 n=0;

	if(!str) {
		*error = NullPointer;
		return FALSE;
		}
		
	while ( (!anyNumberOf && *f) || (anyNumberOf && *str) ){
		
		if (*f==(WCHAR)'*'){ /* *f=='*' any number of characters with the following format is accepted*/
			anyNumberOf = TRUE;
			if(!*str)
				return TRUE;
			f++;
			}
		else if (!anyNumberOf && !*str){ /* str ended before format did*/
			*error = ToShort;
			return FALSE; 
			}
		if (IS_FORMAT_N(*f)){ /* *f==n where n is a character between 1 and 9*/
				n=*f-'0';
				f++;
				}
		else if (*f==(WCHAR)'\\'){ /* *f=='\', means constant character follows*/
			f++;
			if(*f++ != *str++) {
				*error = ConstantReplaced;
				return FALSE;
				}
			}
		else {
			switch (*f) {
				case (WCHAR)'A':
					if(IS_FORMAT_UPPERCASE_A(*str))
						str++; 
					else{
						*error = WrongFormat;
						return FALSE;
						}
					break;
				case (WCHAR)'a':
					if(IS_FORMAT_LOWERCASE_A(*str))
						str++; 
					else{
						*error = WrongFormat;
						return FALSE;
						}
					break;
				case (WCHAR)'N':
					if(IS_FORMAT_N(*str))
						str++;
					else{
						*error = WrongFormat;
						return FALSE;
						}
					break;
				case (WCHAR)'X':
					if(IS_FORMAT_UPPERCASE_X(*str))
						str++;
					else{
						*error = WrongFormat;
						return FALSE;
						}
					break;
				case (WCHAR)'x':
					if(IS_FORMAT_LOWERCASE_X(*str))
						str++;
					else{
						*error = WrongFormat;
						return FALSE;
						}
					break;
				case (WCHAR)'M':
					if(IS_FORMAT_UPPERCASE_M(*str))
						str++;
					else{
						*error = WrongFormat;
						return FALSE;
						}
					break;
				case (WCHAR)'m':
					if(IS_FORMAT_LOWERCASE_M(*str))
						str++;
					else{
						*error = WrongFormat;
						return FALSE;
						}
					break;
				default:
					/*default means an error in this function*/
					break;
				}
			
			if (n){
				n--;
				if(!n)
					return(*str==0);
				}
			else if (!anyNumberOf)
				f++;
			}
		} 
		
	if(*str){
		*error = ToLong;
		return FALSE;
		}
	else
		return TRUE;
	}

INT16 SizeOfGleList(VOID) {
/*	GleElement* p = gFirstGleElement;
	INT16 sum = 0;
	while(p) {		
		sum += p->chunkSize;
		p = p->next;		
		}
*/	return 0;
	}


VOID GleRctSetRectangle (const GleRectanglePtr r, const INT16 left, const INT16 top, const INT16 width, const INT16 height)
{
	r->topLeft.x = left;
	r->topLeft.y = top;
	r->extent.x = width;
	r->extent.y = height;
}

BOOL GleRctPtInRectangle (const INT16 x, const INT16 y, const GleRectanglePtr r)
{
	return (x >= r->topLeft.x &&
			  y >= r->topLeft.y &&
			  x < r->topLeft.x + r->extent.x &&
			  y < r->topLeft.y + r->extent.y);
}

VOID GleRctGetIntersection (const GleRectanglePtr r1, const GleRectanglePtr r2, GleRectanglePtr r3)
{
	INT16 left, right, top, bottom;
	
	left = max (r1->topLeft.x, r2->topLeft.x);
	top = max (r1->topLeft.y, r2->topLeft.y);
	right = min (r1->topLeft.x + r1->extent.x, r2->topLeft.x + r2->extent.x);
	bottom = min (r1->topLeft.y + r1->extent.y, r2->topLeft.y + r2->extent.y);
	
	if ((left < right) && top < bottom)
		GleRctSetRectangle (r3, left, top, right - left, bottom - top);
	else
		GleRctSetRectangle (r3, 0, 0, 0, 0);
}

VOID GleAddToGleList(GleViewType* viewP, GleElement* p)
	{
	if (!viewP->firstGleElement) {
		viewP->firstGleElement=p;
		p->prev=0;
		}		
	else {
		p->prev = viewP->lastGleElement;
		viewP->lastGleElement->next=p;
		}
	viewP->lastGleElement=p;
	p->next=0;
	}

VOID GleGetLinkInfo(GleViewType* viewP, GleElement* p){
	GleText* myText;
	GleImage* myImage;
#ifdef SHOW_KEYS_IN_CONTENT
	GleKey* myKey;
#endif
	GleOption* myOption;
	switch (p->type){
		case Text:
			myText = (GleText*)p;
			if(myText->isLink) {
				if(!myText->image)
					CALL_GWC_API(MMIc_linkInfo(viewP->viewId, WIDGET_TEXTLINK, myText->textId));
				else
					CALL_GWC_API(MMIc_linkInfo(viewP->viewId, WIDGET_IMAGELINK, myText->textId));
				}
			break;
		case Image:
			myImage = (GleImage*)p;
			if(myImage->isLink) 
				CALL_GWC_API(MMIc_linkInfo(viewP->viewId, WIDGET_IMAGELINK, myImage->imageId));
			break;
#ifdef SHOW_KEYS_IN_CONTENT
		case Key:
			myKey = (GleKey*)p;
			CALL_GWC_API(MMIc_linkInfo(viewP->viewId, WIDGET_DOLINK, myKey->keyId));
			break;
#endif
		case Option:
			myOption = (GleOption*)p;
			CALL_GWC_API(MMIc_linkInfo(viewP->viewId, WIDGET_OPTIONLINK, myOption->optionId));
			break;
		}
	}

VOID GleDeleteCard(GleViewType* viewP) {
	GleElement* tempP;
	while (viewP->firstGleElement) {
		tempP = viewP->firstGleElement->next;
		viewP->firstGleElement->Destruct(viewP->firstGleElement);
		viewP->firstGleElement=tempP;
		}
	viewP->markedGleElement = 0;
	}	

BOOL GleRenderCard(GleViewType* viewP, INT16 *xSize, INT16 *ySize) {
	INT16 xSum=0;
	INT16 ySum=0;
	INT16 ascentMax=0;
	INT16 descenderMax=0;
    INT16 columnLeftX=0;
    INT16 columnWidth = viewP->width;

	GleElementType gleType=Break;
	INT16 currentFontLineHeight=14;

	INT16 nbrObjects=0;
	INT8 align=ALIGN_LEFT;
	INT8 alignNext=ALIGN_LEFT;
	BOOL wrap=TRUE;
	INT16 xOffset=0;
	GleText* multipleRowTextPtr=0;
	GleElement* p = viewP->firstGleElement;
	GleElement* p2;
	GleElement* rowStart=0;
	GleElement* fieldSetPtr=0;
	GleElement* GleFieldSetPtr = 0;
	UINT16 k;
	
	*xSize = 0;
	*ySize = 0;
	while (p)
		{
		rowStart=p;
		ascentMax=0;
		descenderMax = 0;
		nbrObjects=0;

		if (multipleRowTextPtr)
			{ /* Uppdatera ascentMax och rowHeightmax fr eventuella andra object p sista raden som ej r full nnu! */
			GleTextGetAcsentAndDescender(multipleRowTextPtr, &ascentMax, &descenderMax);
			}
		while (p->RenderX(p,&xSum,&ySum, &ascentMax, &descenderMax, &gleType, &currentFontLineHeight, &alignNext, &wrap, &columnLeftX, &columnWidth, viewP)) 
			{
			nbrObjects++;
			p=p->next;
			if(!p)
				break;
			}


		*xSize = max(*xSize, xSum);
		viewP->xSumMaxForFieldSet = max(viewP->xSumMaxForFieldSet, xSum);

		switch (align) 
			{
			case ALIGN_CENTER:
				xOffset=columnLeftX+(columnWidth-xSum)/2;
				break;
			case ALIGN_RIGHT:	
				xOffset=columnLeftX+(columnWidth-xSum);
				break;
			default:
				xOffset=columnLeftX;
				break;			
			}
			
		if(xOffset < 0) xOffset = 0; /* center and right align too-large images will be left aligned */
			
		if (multipleRowTextPtr)
			{
			GleTextLastRowRenderY(multipleRowTextPtr, &xSum, &align, &ascentMax, &descenderMax);
			/* Uppdaterar ascentMax och descenderMax, samt justerar sista raden i texten med ascentMax */
			/* multipleRowTextPtr->GetAcsentAndRowHeight(ascentMax, descenderMax); */
		 	multipleRowTextPtr = 0;
	 		if(gleType!=Text)
	 			gleType=NoType;
			/* ascentMax = 0; */
			/* descenderMax = 0; */
			}

		p2=rowStart;
		for ( k=0 ; k<nbrObjects ; k++)
			{
			p2->RenderY(p2,&xOffset,&ySum,&ascentMax);
			p2=p2->next;
			}

		if ((gleType == FieldSet) && p && (xSum==0))
			{			
			fieldSetPtr=p;
			p=p->next;
			
			/* Save FatherFieldSet's extent.x */ 
			if(((GleFieldSet *)viewP->gleFieldSetPtr)->prevFieldSet)
				((GleFieldSet *)viewP->gleFieldSetPtr)->prevFieldSet->bounds.extent.x = max(				
				((GleFieldSet *)viewP->gleFieldSetPtr)->prevFieldSet->bounds.extent.x, viewP->xSumMaxForFieldSet + gle_fieldset_spaceLeft + gle_fieldset_spaceRight);	
				
			viewP->xSumMaxForFieldSet = 0;
			}
		else if ((gleType == CloseFieldSet))
			{

			ySum += descenderMax + ascentMax;		
			ySum += gle_fieldset_spaceBottom;
			
			
			((GleFieldSet *)viewP->gleFieldSetPtr)->bounds.extent.y = ySum - ((GleFieldSet *)viewP->gleFieldSetPtr)->bounds.topLeft.y;
			((GleFieldSet *)viewP->gleFieldSetPtr)->bounds.extent.x = max(((GleFieldSet *)viewP->gleFieldSetPtr)->bounds.extent.x, viewP->xSumMaxForFieldSet + gle_fieldset_spaceLeft + gle_fieldset_spaceRight);
		
			*xSize = max(*xSize ,((GleFieldSet *)viewP->gleFieldSetPtr)->bounds.extent.x);
				
			columnLeftX -= gle_fieldset_spaceLeft;
			columnWidth += gle_fieldset_spaceLeft + gle_fieldset_spaceRight+1;	
 
			/* Is there a Father FieldSet after this ? */ 
			if(((GleFieldSet *)viewP->gleFieldSetPtr)->prevFieldSet) {												
				viewP->xSumMaxForFieldSet = max(((GleFieldSet *)viewP->gleFieldSetPtr)->prevFieldSet->bounds.extent.x - gle_fieldset_spaceRight - gle_fieldset_spaceRight, ((GleFieldSet *)viewP->gleFieldSetPtr)->bounds.extent.x);  
				/* xSize = max(*xSize, viewP->xSumMaxForFieldSet); */
				}
			else {
				/* xSize = max(*xSize, viewP->xSumMaxForFieldSet) */
				viewP->xSumMaxForFieldSet = 0;
				}
				
			viewP->gleFieldSetPtr = ((GleFieldSet *)(viewP->gleFieldSetPtr))->prevFieldSet;

	

			xSum =0;
			descenderMax = 0;
			ascentMax = 0;			

			fieldSetPtr=0;
			if (p)
				p=p->next;
			/*multipleRowTextPtr = 0; */
			}

		else if ((gleType == Select) && p && (xSum==0)) {			
			p=p->next;			
			}
		else if (gleType == Table) {
			while (p->type != CloseTable) {
				p=p->next;
				}
			p=p->next;	
			xSum=0;									
			}	
		else if ((gleType == CloseSelect)) {
			if (p)
				p=p->next;
			#ifdef SHOW_OPTIONS_IN_CONTENT
				xSum = 0;  /* jbi, start on line after Close select*/
			#endif
			}
		else if ((gleType == Text) && p) {
			GleTextMultipleRowCheck((GleText*)p, &xSum, &multipleRowTextPtr, &descenderMax, &columnWidth, xOffset);
			/* Sparar multipleRowTextPtr vid multipla rader samt uppdaterar descenderMax */
			if (multipleRowTextPtr)			
				p=p->next;				
			else {
				if(xSum > *xSize)
					*xSize = xSum;
				xSum=0;
				}
			ySum+=descenderMax + ascentMax;		
			descenderMax = 0;
			ascentMax = 0;			
			}
		else 
			{
			ySum+=descenderMax + ascentMax;		
			descenderMax = 0;
			ascentMax = 0;			
			multipleRowTextPtr=0;
			if(xSum > *xSize)
				*xSize = xSum;
			xSum=0;
			}
		if (gleType == Break)			
			p=p->next;	/* Step over the object */
	
		else if (gleType == Paragraph) 		
			{
			p=p->next;	/* Step over the object */
			align=alignNext;
			}
		}

	if (multipleRowTextPtr)
		{
		ascentMax=0;
		descenderMax=0;
		GleTextGetAcsentAndDescender(multipleRowTextPtr, &ascentMax, &descenderMax);
		GleTextLastRowRenderY(multipleRowTextPtr, &xSum, &align, &ascentMax, &descenderMax);
		ySum += descenderMax + ascentMax;
		}
	*ySize = ySum+1;
	return 1;
	}
	
VOID GleDrawCard(GleViewType* viewP) {
	GleElement* p = viewP->firstGleElement;

	if(GLEa_drawCardRequest(viewP->viewId, viewP->horizOffset, viewP->horizontalSize, viewP->vertOffset, viewP->verticalSize)) {
		GLEa_clearCard(viewP->viewId);
		while(p) {		
			(*p->Draw)(viewP, p);	
			p = p->next;		
			}
		GLEa_drawCardCompleted(viewP->viewId);
		}
	}

VOID GlePerformVerticalScroll(GleViewType* viewP, INT16 pixelsToScroll) {
	INT16 oldVertOffset = viewP->vertOffset;
	if (viewP->verticalSize > viewP->height) {
		viewP->vertOffset+=pixelsToScroll;
		viewP->vertOffset = max(0,viewP->vertOffset);
		viewP->vertOffset = min(viewP->vertOffset,viewP->verticalSize - viewP->height);
		/* Scrollbar updating is made in GleDrawCard */
		}
 	}

VOID GlePerformHorizontalScroll(GleViewType* viewP, INT16 pixelsToScroll) {
	INT16 oldhorizOffset = viewP->horizOffset;
	if (viewP->horizontalSize > viewP->width) {
		viewP->horizOffset += pixelsToScroll;
		viewP->horizOffset = max(0,viewP->horizOffset);
		viewP->horizOffset = min(viewP->horizOffset,viewP->horizontalSize - viewP->width);
		/* Scrollbar updating is made in GleDrawCard */
		}
 	}

VOID GleKeyScrollCard(GleViewType* viewP, GleDirectionType direction) {
	#ifdef SUPPORT_KEY_NAVIGATION
		GleText *markedText = 0;
		GleElement *next = 0;
		BOOL performScroll;
		
		if(!viewP->markedGleElement){
			if(viewP->firstGleElement)
				{			
				if (viewP->firstGleElement->IsMarkable(viewP, viewP->firstGleElement))
					next = viewP->firstGleElement;
				else
					next = GleElementGetNextMarkable(viewP, viewP->firstGleElement, Down);
				}
			}
		else
			next = GleElementGetNextMarkable(viewP, viewP->markedGleElement, direction);	
			
		if(next) {
			viewP->markedGleElement = next;
			performScroll = FALSE;
			}
		else
			performScroll = TRUE;
		
		if(direction==NoDirection)
			performScroll = FALSE;		
	
		if(performScroll) {
			GlePerformVerticalScroll(viewP, (direction == Down) ? gle_scroll_pixelsVertical : -gle_scroll_pixelsVertical);
			if(!next) {
				if(!viewP->markedGleElement) {
					if(direction == Down)		
						next = GleElementGetNextMarkable(viewP, viewP->firstGleElement, Down);
					else
						next = GleElementGetNextMarkable(viewP, viewP->lastGleElement, Up);
					}
				else
					next = GleElementGetNextMarkable(viewP, viewP->markedGleElement, direction);	
				}
	
			if(next) 
				viewP->markedGleElement = next;
			else {		
				if(viewP->markedGleElement) {
					if(!viewP->markedGleElement->IsMarkable(viewP, viewP->markedGleElement))
						viewP->markedGleElement=0;
					}
				}
			}
	#endif
	GleDrawCard(viewP);
	}	

VOID GLEc_verticalScrollbarChanged(UINT8 viewId, UINT16 scrollPos) {
	GleViewType* viewP = GleViewSearchByViewId(viewId);
	if(!viewP->mMIa_wait) {
 		GlePerformVerticalScroll(viewP, scrollPos - viewP->vertOffset);	
		GleDrawCard(viewP);
		}
	}	

VOID GLEc_horizontalScrollbarChanged(UINT8 viewId, UINT16 scrollPos) {
	GleViewType* viewP = GleViewSearchByViewId(viewId);
	if(!viewP->mMIa_wait) {
 		GlePerformHorizontalScroll(viewP, scrollPos - viewP->horizOffset);
		GleDrawCard(viewP);
		}
	}
		
VOID GLEc_openView(UINT8 viewId, UINT8 uaMode, UINT16 width, UINT16 height) {
	GleViewType* newView;
	void* k;
	newView = GleViewCreateAddInit(viewId, width, height);
	k = gGleViewList;
	#if (WIPVRSN_1 >= 4)
	//#if (GWCVERSION == 0400)
		CALL_GWC_API(MMIc_startUserAgent(viewId, uaMode));
	#elif (WIPVRSN_1 > 1)
		CALL_GWC_API(MMIc_openView(viewId, uaMode));
	#else
		CALL_GWC_API(MMIc_openView(viewId));
	#endif
	}

VOID GLEc_stop(UINT8 viewId) {
	GleViewType* viewP = GleViewSearchByViewId(viewId);
	if(viewP->cardComplete)	/* Dont stop cards that are being rendered right now */
		CALL_GWC_API(MMIc_stop(viewId));
	}

VOID GLEc_closeView(UINT8 viewId) {
	GleViewType* viewP = GleViewSearchByViewId(viewId);
	if(viewP) {
		GleViewDelete(viewP);
		#if (WIPVRSN_1 >= 4)
		//#if (GWCVERSION == 0400)
			CALL_GWC_API(MMIc_terminateUserAgent(viewId));
		#else
			CALL_GWC_API(MMIc_closeView(viewId));
		#endif
		}
	}
	
VOID GLEc_inputDialogResponse(GleInput* inputPtr, const GLE_STRING newText, BOOL ok) {
	GleViewType* viewP = GleViewSearchByViewId(inputPtr->viewId);
	if(viewP) {
		if(ok && !viewP->mMIa_wait) {
			INT16 nullChar = 0;
			if(inputPtr->text)
				WIPFree(inputPtr->text);
			inputPtr->text = WIPAlloc(GLEa_sizeofNchars(newText, GLEa_strlen(newText)) + GLEa_sizeofString((WCHAR*)&nullChar));
			GLEa_strcpyGle2Gle(inputPtr->text, newText);
			GleInputDraw(viewP, inputPtr); 
			}
		}
	}

#ifdef BITMAP_POSITION_CLICK
	GleImage* GLEc_getImage(INT8 viewId, INT8 imageId) {
		GleViewType* viewP = GleViewSearchByViewId(viewId);
		if(viewP) {
			GleElement* p = viewP->firstGleElement;
			while (p) {
				if ( ((GleImage*)p)->type == Image) 
					return p;		
				p = p->next;		
				}			
			}
		return 0;
		}
	
	INT16 GLEc_getHorizOffset(INT8 viewId) {
		GleViewType* viewP = GleViewSearchByViewId(viewId);
		if(viewP) 
			return viewP->horizOffset;
		else
			return -1;
		}
	
	INT16 GLEc_getVertOffset(INT8 viewId) {
		GleViewType* viewP = GleViewSearchByViewId(viewId);
		if(viewP) 
			return viewP->vertOffset;
		else
			return -1;
		}
#endif

VOID CALL_GWC_API(MMIa_wait(UINT8 viewId, BOOL isWait)) { 
	GleViewType* viewP = GleViewSearchByViewId(viewId);
	if(viewP) {
		viewP->mMIa_wait = isWait;
		if(!viewP->mMIa_wait && viewP->redrawWhenNotWait){
			GleDrawCard(viewP);
			viewP->redrawWhenNotWait=FALSE;
			}
		viewP->selectedGleElement = 0;
		}
	}
#ifndef SHOW_OPTIONS_IN_CONTENT
	VOID GLEc_selectSetLabel(GleSelect* self, const GLE_STRING newLabel) {
		GleViewType* viewP = GleViewSearchByViewId(self->viewId);
		if(viewP) {
			if(!viewP->mMIa_wait) {

				INT16 nullChar = 0;
				if(self->label)
					WIPFree(self->label);
				self->label = WIPAlloc(GLEa_sizeofNchars(newLabel, GLEa_strlen(newLabel)) + GLEa_sizeofString((WCHAR*)&nullChar));
				GLEa_strcpyGle2Gle(self->label, newLabel);
				}			
			}	
		}

	VOID GLEc_optionSelected(UINT8 viewId, GleOption* self) {
		GleViewType* viewP = GleViewSearchByViewId(viewId);
		if(viewP) {
			if(!viewP->mMIa_wait) {
				if(self->singleSelect)
					((GleSelect*)self->singleSelect)->singleSelectedOption = self;
				else
					self->multipleSelected = !self->multipleSelected;
				CALL_GWC_API(MMIc_optionSelected(viewId, self->optionId));
				}
			}
		}
#endif	
VOID GLEc_start(VOID) {
	gGleViewList = 0;
	CLNTc_start();
	}
	
VOID GLEc_terminate(VOID) {
	while(gGleViewList) {
		GleViewDelete(gGleViewList);
		}		
	}
	
VOID GLEc_resizeView(UINT8 viewId, UINT16 newWidth, UINT16 newHeight) {
	GleViewType* viewP = GleViewSearchByViewId(viewId);
	if(viewP) {
		if(!viewP->mMIa_wait) {
			viewP->width = newWidth;
			viewP->height = newHeight;
			if(viewP->firstGleElement)
				CALL_GWC_API(MMIa_showCard(viewId));
			}
		}
	}

VOID GLEc_drawView(UINT8 viewId) {
	GleViewType* viewP = GleViewSearchByViewId(viewId);
	if(viewP) {
		if(!viewP->mMIa_wait) 
			GleDrawCard(viewP);
		else
			viewP->redrawWhenNotWait=TRUE;
		}
	}
	
VOID GLEc_handleEvent(UINT8 viewId, GleEvent event)
	{
	GleElementType gleType = Key; /* Initiated to something other than Image or Table */
	#ifdef SUPPORT_PEN_NAVIGATION
		GleElement* p;
		GleElement* oldElement;
		BOOL found;
		GleRectangleType r;
	#endif
	GleViewType* viewP = GleViewSearchByViewId(viewId);

	if(!viewP) 
		return;
	if(!viewP->mMIa_wait) {
		switch(event.eventType)
			{
			#ifdef SUPPORT_KEY_NAVIGATION
				case KeyYes:
					if(viewP->markedGleElement)
						viewP->markedGleElement->Selected(viewP, viewP->markedGleElement);
					break;
				case KeyLinkInfo:
					if(viewP->markedGleElement)
						GleGetLinkInfo(viewP, viewP->markedGleElement);
					break;
				case KeyDown:
					GleKeyScrollCard(viewP, Down);
					break;
				case KeyLeft:
					/* GleKeyScrollCard(Left); */
					if(viewP->horizOffset>0) {
						viewP->horizOffset -= gle_scroll_pixelsHorizontal;
						viewP->horizOffset = max(viewP->horizOffset,0);
						GleDrawCard(viewP);
						}
					break;
				case KeyRight:
					/* GleKeyScrollCard(Right); */
					if((viewP->horizontalSize> viewP->width) && (viewP->horizOffset < (viewP->horizontalSize - viewP->width))) {
						viewP->horizOffset += gle_scroll_pixelsHorizontal;
						viewP->horizOffset = min(viewP->horizOffset, viewP->horizontalSize - viewP->width);
						GleDrawCard(viewP);
						}
					break;
				case KeyUp:
					GleKeyScrollCard(viewP, Up);
					break;
			#endif
			#ifdef SUPPORT_PEN_NAVIGATION
				case PenLinkInfo:
			
					/* Test if any controlelement is pressed down */
					found = FALSE;
					p = viewP->firstGleElement;
		
					while((p && !found) && !viewP->mMIa_wait)
						{
						gleType	= p->type;
						found = p->Contains(viewP, p,event.point);
						if (!found)
							p=p->next;
						}	
					if (found)
						GleGetLinkInfo(viewP, p);
					break;
		
				case PenDown:
			
					/* Remember the coordinates in case of ScreenScroll */
					viewP->selectedGleElement=0;
					viewP->oldPenX = event.point.x;
					viewP->oldPenY = event.point.y;
					/* Test if any controlelement is pressed down */
					found = FALSE;
					p = viewP->firstGleElement;
		
					while((p && !found) && !viewP->mMIa_wait)
						{
						gleType	= p->type;
						found = p->Contains(viewP, p,event.point);
						if (!found)
							p=p->next;
						else
							viewP->selectedGleElement=p;
						}	
					if (found)
						{
						oldElement = viewP->markedGleElement;
						viewP->markedGleElement = viewP->selectedGleElement;
		
						if(GLEa_drawCardRequest(viewP->viewId, viewP->horizOffset, viewP->horizontalSize, viewP->vertOffset, viewP->verticalSize)) {
							if(oldElement)
								oldElement->Draw(viewP, oldElement);
							viewP->selectedGleElement->Draw(viewP, viewP->selectedGleElement);
							GLEa_drawCardCompleted(viewP->viewId);
							}	
						}
					
					break;
				case PenMove:
			/*	INT16			minValue;
					INT16			maxValue;
					INT16			value;
					INT16			pageSize;
					ScrollBarPtr	bar;
			*/		
					if(!viewP->mMIa_wait)
						{
						GleRctSetRectangle(&r,0,0, viewP->height, viewP->height);				
						if (viewP->selectedGleElement) {
							/* viewP->selectedGleElement->PenMove(event->point.x, event->point.y); */
							
							BOOL tempContains = viewP->selectedGleElement->Contains(viewP, viewP->selectedGleElement, event.point);
							
												
			
							/* Pen goes out                    Pen comes back */
							if (!tempContains && (viewP->markedGleElement==viewP->selectedGleElement)) {
								viewP->markedGleElement = 0;					
		
								if(GLEa_drawCardRequest(viewP->viewId, viewP->horizOffset, viewP->horizontalSize, viewP->vertOffset, viewP->verticalSize)) {
									viewP->selectedGleElement->Draw(viewP, viewP->selectedGleElement);
									GLEa_drawCardCompleted(viewP->viewId);
									}
								}
							else if(tempContains && (viewP->markedGleElement!=viewP->selectedGleElement)) {
								viewP->markedGleElement = viewP->selectedGleElement;
								if(GLEa_drawCardRequest(viewP->viewId, viewP->horizOffset, viewP->horizontalSize, viewP->vertOffset, viewP->verticalSize)) {
									viewP->selectedGleElement->Draw(viewP, viewP->selectedGleElement);
									GLEa_drawCardCompleted(viewP->viewId);
									}
								}
							}	
						else if (GleRctPtInRectangle(event.point.x, event.point.y, &r))
							{						
		
					/*	bar = (ScrollBarPtr) GetObjectPtr (SurferMainScrollbarScrollBar);
							SclGetScrollBar (bar, &value, &min, &maxValue, &pageSize);		
							short delta = eventP->screenY - viewP->oldPenY;
							value -=delta;
							viewP->oldPenY = eventP->screenY;
							SclSetScrollBar (bar,value, minValue, max, pageSize);
							MainFormScroll(screenScroll);
					*/
							}
						}						
					break;
					
				case PenUp:
					oldElement = viewP->markedGleElement;
					viewP->markedGleElement=0;
		
					if(oldElement) {
						if(GLEa_drawCardRequest(viewP->viewId, viewP->horizOffset, viewP->horizontalSize, viewP->vertOffset, viewP->verticalSize)) {
							oldElement->Draw(viewP, oldElement);	
							GLEa_drawCardCompleted(viewP->viewId);
							}
						}
					if(!viewP->mMIa_wait)
						{					
						if (viewP->selectedGleElement)
							if (viewP->selectedGleElement->Contains(viewP, viewP->selectedGleElement, event.point))
								viewP->selectedGleElement->Selected(viewP, viewP->selectedGleElement);								
						}
					viewP->selectedGleElement=0;
					break;
				#endif
			default:
				break;
			}
		}	
	}

VOID CALL_GWC_API(MMIa_getInputString(UINT8 viewId, UINT8 inputId))
	{
		GleInput* myInputPtr;
		BOOL found = FALSE;
		WCHAR* wideString;
		GleViewType* viewP = GleViewSearchByViewId(viewId);
		GleElement* p = viewP->firstGleElement;

#ifdef BITMAP_POSITION_CLICK
		if ((inputId == gGleImageClickXInputID) || (inputId == gGleImageClickYInputID)){
		
			unsigned char buffer[10];
			WCHAR wBuffer[10];
			UINT16 pos;
			if (inputId == gGleImageClickXInputID) 
				pos = gGleImageClickXCoord;
			else 
				pos = gGleImageClickYCoord;
				
		StrIToA(&buffer, pos);
		GLEa_strcpyGle2Wchar(wBuffer, buffer);
		CALL_GWC_API(MMIc_inputString(viewId, inputId, wBuffer));
		return;
		}
#endif

		while(p && !found)
			{
			/*found = p->InputMatch(inputId);*/
			if (p->type == Input)
				{
				myInputPtr = (GleInput*) p;
				found = (myInputPtr->inputId==inputId);
				}
			else
				found = FALSE;	
			if (!found)
				p=p->next;
			}
		myInputPtr = (GleInput*) p;


		wideString = (WCHAR*) WIPAlloc(2*myInputPtr->nChars+2);
		GLEa_strcpyGle2Wchar(wideString, myInputPtr->text);
		if(*wideString || myInputPtr->emptyOK)
			CALL_GWC_API(MMIc_inputString(viewP->viewId,myInputPtr->inputId,wideString));
		else{
			GLEc_stop(viewId);
			GLEa_error(viewId,InputFieldLeftEmpty);
			}
		WIPFree(wideString);
	}

VOID CALL_GWC_API(MMIa_newBreak(UINT8 viewId)) {
	GleViewType* viewP = GleViewSearchByViewId(viewId);
	GleAddToGleList(viewP, GleBreakConstruct(viewId));
	}


VOID CALL_GWC_API(MMIa_newCard (UINT8 viewId, const WCHAR* title, BOOL isList, BOOL isRefresh, const CHAR *URL, BOOL isBookmarkable, const WCHAR * const *history)) {
	GleViewType* viewP = GleViewSearchByViewId(viewId);
	
	INT32 n1=2;
	GLE_STRING cardTitle = 0;
	viewP->cardComplete = FALSE;
	viewP->waitingForNImages = 0;
	if(title)	
		n1 += GLEa_sizeofString(title);
	cardTitle = (GLE_STRING)WIPAlloc(n1);

	if(title)
		GLEa_strcpyWchar2Gle(cardTitle, title);
	else {
		INT16 noTitle = 0;
		GLEa_strcpyWchar2Gle(cardTitle, (WCHAR*)&noTitle);
		}
	
	GLEa_newCard(viewP->viewId, cardTitle, URL, isBookmarkable, history);
	GleDeleteCard(viewP);

	viewP->vertOffset=0;
	viewP->horizOffset=0;
	viewP->verticalSize=0;	
	viewP->horizontalSize=0;

	WIPFree(cardTitle);
	}

VOID CALL_GWC_API(MMIa_cancelCard (UINT8 viewId)) {
	}
VOID CALL_GWC_API(MMIa_closeFieldSet(UINT8 viewId)) {
	GleViewType* viewP = GleViewSearchByViewId(viewId);
	if(viewP->numberOfNestledFieldSet-- <= gle_fieldset_maxNumberNestled)
		GleAddToGleList(viewP, GleCloseFieldSetConstruct());
	}
	
VOID CALL_GWC_API(MMIa_closeParagraph(UINT8 viewId)) {}

VOID CALL_GWC_API(MMIa_closeSelect(UINT8 viewId)) {
	GleViewType* viewP = GleViewSearchByViewId(viewId);
	GleAddToGleList(viewP, GleCloseSelectConstruct(viewId));	
	}

VOID CALL_GWC_API(MMIa_closeTable (UINT8 viewId)) {
	#ifdef SUPPORT_TABLES
		GleViewType* viewP = GleViewSearchByViewId(viewId);
		GleAddToGleList(viewP, GleCloseTableConstruct(viewId));
	#endif
	}

VOID CALL_GWC_API(MMIa_newFieldSet(UINT8 viewId, const WCHAR *title)) {
	GleViewType* viewP = GleViewSearchByViewId(viewId);
	if(viewP->numberOfNestledFieldSet++ < gle_fieldset_maxNumberNestled) 
		GleAddToGleList(viewP, GleFieldSetConstruct(viewId, title));
	}

VOID GLEc_newFieldSetTest(UINT8 viewId, const GLE_STRING title) {
	WCHAR* p1 = (WCHAR*) WIPAlloc(GLEa_strlen(title)*2+2);
	GLEa_strcpyGle2Wchar(p1,title);	
	CALL_GWC_API(MMIa_newFieldSet(viewId, p1));
	WIPFree(p1);
	}

#if (WIPVRSN_1 <= 2)
//#if (GWCVERSION == 0250) || (GWCVERSION == 0270)
VOID CALL_GWC_API(MMIa_newInput(UINT8 viewId, UINT8 inputId, const WCHAR *title, const WCHAR *text, BOOL isPassword, BOOL emptyOK, const WCHAR *format, INT8 size, INT8 nChars, INT8 tabIndex)) {
#else
VOID CALL_GWC_API(MMIa_newInput(UINT8 viewId, UINT8 inputId, const WCHAR *title, const WCHAR *text, BOOL isPassword, BOOL emptyOK, const WCHAR *format, INT8 size, INT8 nChars, INT8 tabIndex, WCHAR accessKey)) {
#endif
	GleViewType* viewP;

#ifdef BITMAP_POSITION_CLICK
	if(title) {
		char* test = (char*)WIPAlloc(GLEa_sizeofString(title));
		GLEa_strcpyWchar2Gle(test, title);
		if (strcmp(test, BITMAP_POSITION_CLICK_INPUT_TITLE_X) == 0) {
			gGleImageClickXInputID = inputId;
			WIPFree(test);		
			return;
			}
	
		else if (strcmp(test, BITMAP_POSITION_CLICK_INPUT_TITLE_Y) == 0) {
			gGleImageClickYInputID = inputId;
			WIPFree(test);		
			return;
			}
	
		WIPFree(test);		
		}
		
#endif

	viewP = GleViewSearchByViewId(viewId);
	GleAddToGleList(viewP, GleInputConstruct(viewId, inputId, title, text, isPassword, emptyOK, format, size, nChars, tabIndex ));
	}

VOID GLEc_newInputTest(UINT8 viewId, UINT8 inputId, const GLE_STRING title, const GLE_STRING text, BOOL isPassword, BOOL emptyOK, const GLE_STRING format, INT8 size, INT8 nChars, INT8 tabIndex) 
	{
	WCHAR* p1 = (WCHAR*) WIPAlloc(GLEa_strlen(title)*2+2);
	WCHAR* p2 = (WCHAR*) WIPAlloc(GLEa_strlen(text)*2+2);
	WCHAR* p3 = (WCHAR*) WIPAlloc(GLEa_strlen(format)*2+2);
	GLEa_strcpyGle2Wchar(p1,title);	
	GLEa_strcpyGle2Wchar(p2,text);
	GLEa_strcpyGle2Wchar(p3,format);
#if (WIPVRSN_1 <= 2)
//#if (GWCVERSION == 0250) || (GWCVERSION == 0270)
	CALL_GWC_API(MMIa_newInput( viewId,  inputId, p1, p2, isPassword,  emptyOK, p3,  size,  nChars,  tabIndex));
#else
	CALL_GWC_API(MMIa_newInput( viewId,  inputId, p1, p2, isPassword,  emptyOK, p3,  size,  nChars,  tabIndex, 0));
#endif
	WIPFree(p3);
	WIPFree(p2);
	WIPFree(p1);
	}

#ifdef SHOW_KEYS_IN_CONTENT
	VOID CALL_GWC_API(MMIa_newKey(UINT8 viewId, UINT8 keyId, const WCHAR *eventType, const WCHAR *label, BOOL isOptional)) {
		if(!GLEa_newKey (viewId, keyId, eventType, label, isOptional)){
			GleViewType* viewP = GleViewSearchByViewId(viewId);
			GleAddToGleList(viewP, GleKeyConstruct(viewId, keyId, eventType, label, isOptional));
			}
		} 
#endif

VOID GLEc_newKeyTest(UINT8 viewId, UINT8 keyId, const GLE_STRING eventType, const GLE_STRING label, BOOL isOptional){
	WCHAR* p1 = (WCHAR*) WIPAlloc(GLEa_strlen(eventType)*2+2);
	WCHAR* p2 = (WCHAR*) WIPAlloc(GLEa_strlen(label)*2+2);
	GLEa_strcpyGle2Wchar(p1,eventType);	
	GLEa_strcpyGle2Wchar(p2,label);
	CALL_GWC_API(MMIa_newKey(viewId,keyId,p1,p2,isOptional));
	WIPFree(p2); 
	WIPFree(p1);
	}
#if (WIPVRSN_1 <= 2)
//#if (GWCVERSION == 0250) || (GWCVERSION == 0270)
VOID CALL_GWC_API(MMIa_newImage(UINT8 viewId, UINT8 imageId, const CHAR *imageData, UINT16 imageSize, const CHAR *imageType, const WCHAR *altText, const WCHAR *localSrc, BOOL isLink, const WCHAR *linkTitle, INT8 vSpace, INT8 hSpace, INT16 width, INT16 height, INT8 isPercent, INT8 align)) {
#else
VOID CALL_GWC_API(MMIa_newImage(UINT8 viewId, UINT8 imageId, const CHAR *imageData, UINT16 imageSize, const CHAR *imageType, const WCHAR *altText, const WCHAR *localSrc, BOOL isLink, const WCHAR *linkTitle, WCHAR accessKey, INT8 vSpace, INT8 hSpace, INT16 width, INT16 height, INT8 isPercent, INT8 align)) {
#endif
	GleViewType* viewP = GleViewSearchByViewId(viewId);

	if(viewP->displayImages && viewP->updateImages)
		viewP->waitingForNImages++;

	/* Image support */
	if(viewP->displayImages && imageData)
		/* Dynamic imageupdate */
		if(viewP->updateImages) {
			GleElement* imageP = GleImageConstruct(viewId, imageId, imageData, imageSize, imageType, altText, localSrc, isLink, linkTitle, vSpace, hSpace, width, height, isPercent, align);						
			GleAddToGleList(viewP, GleTextConstruct(viewId, imageId, altText, isLink, linkTitle, 0, (GleImage*)imageP));

			}
		else {
			GleElement* imageP = GleImageConstruct(viewId, imageId, imageData, imageSize, imageType, altText, localSrc, isLink, linkTitle, vSpace, hSpace, width, height, isPercent, align);
			
			((GleImage*)imageP)->bitmap = GLEa_imageConvert(imageData, imageSize, imageType, &((GleImage*)imageP)->vSpace, &((GleImage*)imageP)->hSpace, &((GleImage*)imageP)->bounds.extent.x, &((GleImage*)imageP)->bounds.extent.y, isPercent);
			if(((GleImage*)imageP)->bitmap) {
				GleAddToGleList(viewP, imageP);
				GleImageAlign(((GleImage*)imageP), align);
				}
			else {
				GleAddToGleList(viewP, GleTextConstruct(viewId, imageId, altText, isLink, linkTitle, 0, (GleImage*)imageP));
				}
			}	
	else {
		GleElement* imageP = GleImageConstruct(viewId, imageId, imageData, imageSize, imageType, altText, localSrc, isLink, linkTitle, vSpace, hSpace, width, height, isPercent, align);								
		GleAddToGleList(viewP, GleTextConstruct(viewId, imageId, altText, isLink, linkTitle, 0, (GleImage*)imageP));
		}
	}

VOID GLEc_newImageTest(UINT8 viewId, UINT8 imageId, const CHAR *imageData, UINT16 imageSize, const CHAR *imageType, const GLE_STRING altText, const GLE_STRING localSrc, BOOL isLink, const GLE_STRING linkTitle, INT8 vSpace, INT8 hSpace, INT16 width, INT16 height, INT8 isPercent, INT8 align){
	WCHAR* p1 = (WCHAR*)  WIPAlloc(GLEa_strlen(altText)*2+2);
	WCHAR* p2 = (WCHAR*)  WIPAlloc(GLEa_strlen(localSrc)*2+2);
	WCHAR* p3 = (WCHAR*)  WIPAlloc(GLEa_strlen(linkTitle)*2+2);
	GLEa_strcpyGle2Wchar(p1,altText);
	GLEa_strcpyGle2Wchar(p2,localSrc);
	GLEa_strcpyGle2Wchar(p3,linkTitle);
#if (WIPVRSN_1 <= 2)
//#if (GWCVERSION == 0250) || (GWCVERSION == 0270)
	CALL_GWC_API(MMIa_newImage(viewId, imageId, imageData, imageSize, imageType, p1, p2, isLink, p3, vSpace, hSpace, width, height, isPercent, align));
#else
	CALL_GWC_API(MMIa_newImage(viewId, imageId, imageData, imageSize, imageType, p1, p2, isLink, p3, 0, vSpace, hSpace, width, height, isPercent, align));
#endif
	WIPFree(p3);
	WIPFree(p2);
	WIPFree(p1);	
	}

VOID CALL_GWC_API(MMIa_completeImage(UINT8 viewId, UINT8 imageId, const CHAR *imageData, UINT16 imageSize, const CHAR *imageType)) {
	GleViewType* viewP = GleViewSearchByViewId(viewId);
	GleElement* p = viewP->firstGleElement;
	GleImage* imageP = 0;
	BOOL found  = FALSE;
	viewP->markedGleElement=0;
	
	if(viewP->displayImages && viewP->updateImages)
		viewP->waitingForNImages--;
	if(!imageData) {
		if(!viewP->waitingForNImages)
			GLEa_status(viewId, AllImagesDone);
		return;
		}
	if(viewP->displayImages) {
		while(!found) {	
			if(p->type == Text) {
				if(((GleText*)p)->image) {
					if(((GleText*)p)->image->imageId == imageId) {
						imageP = ((GleText*)p)->image;
						found = TRUE;
						continue;
						}
					}
				}
			p=p->next;
			}
		
		if(found) {
			imageP->bitmap = GLEa_imageConvert(imageData, imageSize, imageType, &imageP->vSpace, &imageP->hSpace, &imageP->bounds.extent.x, &imageP->bounds.extent.y, imageP->isPercent);
			if(imageP->bitmap) {
				if(viewP->firstGleElement == p)
					viewP->firstGleElement = (GleElement*)imageP;
				if(viewP->lastGleElement == p)
					viewP->lastGleElement = (GleElement*)imageP;
		
				if(p->prev)
					p->prev->next = (GleElement*)imageP;
				if(p->next)
					p->next->prev = (GleElement*)imageP;
		
				imageP->next = p->next;
				imageP->prev = p->prev;
		
				((GleText*)p)->image = 0;	
				GleTextDestruct((GleText*)p);
				
				GleImageAlign(imageP, imageP->align);
				}
			}
					
		if(viewP->cardComplete) 			
			CALL_GWC_API(MMIa_showCard(viewId));
						
		}
	}

	VOID CALL_GWC_API(MMIa_newOption(UINT8 viewId, UINT8 optionId, const WCHAR *label, const WCHAR *title, BOOL isSelected)) {
		GleViewType* viewP = GleViewSearchByViewId(viewId);
		GleAddToGleList(viewP, GleOptionConstruct(viewId, optionId, label, title, isSelected));
		}
		
	VOID GLEc_newOptionTest(UINT8 viewId, UINT8 optionId, const GLE_STRING label, const GLE_STRING title, BOOL isSelected)
		{
		WCHAR* p1 = (WCHAR*)  WIPAlloc(GLEa_strlen(label)*2+2);
		WCHAR* p2 = (WCHAR*)  WIPAlloc(GLEa_strlen(title)*2+2);
		GLEa_strcpyGle2Wchar(p1,label);	
		GLEa_strcpyGle2Wchar(p2,title);
		CALL_GWC_API(MMIa_newOption(viewId,optionId,p1,p2,isSelected));
		WIPFree(p2);
		WIPFree(p1);	
		}
	
	VOID CALL_GWC_API(MMIa_newOptionGroup(UINT8 viewId, const WCHAR *label)) {
		GleViewType* viewP = GleViewSearchByViewId(viewId);
		GleAddToGleList(viewP, GleOptionGroupConstruct(label));
		}
		
	VOID GLEc_newOptionGroupTest(UINT8 viewId, const GLE_STRING label) {
		WCHAR* p1 = (WCHAR*)  WIPAlloc(GLEa_strlen(label)*2+2);
		GLEa_strcpyGle2Wchar(p1,label);	
		CALL_GWC_API(MMIa_newOptionGroup(viewId,p1));
		WIPFree(p1);	
		}
	
	VOID CALL_GWC_API(MMIa_closeOptionGroup(UINT8 viewId)) {
		GleViewType* viewP = GleViewSearchByViewId(viewId);
		GleAddToGleList(viewP, GleCloseOptionGroupConstruct());
		}
#if (WIPVRSN_1 <= 2)
//#if (GWCVERSION == 0250) || (GWCVERSION == 0270)
VOID CALL_GWC_API(MMIa_newParagraph(UINT8 viewId, INT8 align, BOOL wrap)) {	
#else
VOID CALL_GWC_API(MMIa_newParagraph(UINT8 viewId, INT8 align, BOOL wrap, BOOL preformatted)) {	
#endif
	GleViewType* viewP = GleViewSearchByViewId(viewId);
	GleAddToGleList(viewP, GleParagraphConstruct(align, wrap));
	}

VOID CALL_GWC_API(MMIa_newSelect(UINT8 viewId, const WCHAR* title, BOOL multiSelect, INT8 tabIndex)) {
	GleViewType* viewP = GleViewSearchByViewId(viewId);
		GleAddToGleList(viewP, GleSelectConstruct(viewId, title, multiSelect, tabIndex));
	}

VOID GLEc_newSelectTest(UINT8 viewId, const GLE_STRING title, BOOL multiSelect, INT8 tabIndex) {
	WCHAR* p1 = (WCHAR*) WIPAlloc(GLEa_strlen(title)*2+2);
	GLEa_strcpyGle2Wchar(p1,title);	
	CALL_GWC_API(MMIa_newSelect(viewId, p1, multiSelect, tabIndex));
	WIPFree(p1);
	}
VOID CALL_GWC_API(MMIa_newTable (UINT8 viewId, const WCHAR *title, INT8 noOfColumns, const CHAR *align)) {
	GleViewType* viewP = GleViewSearchByViewId(viewId);
	#ifdef SUPPORT_TABLES
		GleAddToGleList(viewP, GleTableConstruct(viewId, title, noOfColumns, align));
	#else
		viewP->tableNumberColumns=noOfColumns;
		viewP->tableCurrentColumn=-1;
	#endif
	}

VOID GLEc_newTableTest (UINT8 viewId, const GLE_STRING title, INT8 noOfColumns, const CHAR *align) {
	WCHAR* p1 = (WCHAR*) WIPAlloc(GLEa_strlen(title)*2+2);
	GLEa_strcpyGle2Wchar(p1,title);	
	CALL_GWC_API(MMIa_newTable (viewId, p1, noOfColumns, align));
	WIPFree(p1);
	}

VOID CALL_GWC_API(MMIa_newTableData (UINT8 viewId)) {
	GleViewType* viewP = GleViewSearchByViewId(viewId);
	#ifdef SUPPORT_TABLES
		GleAddToGleList(viewP, GleTableDataConstruct(viewId));
	#else
		MMIa_newBreak(viewId);
		//viewP->tableCurrentColumn=(viewP->tableCurrentColumn+1)% viewP->tableNumberColumns;
		viewP->tableCurrentColumn++;
		if(viewP->tableCurrentColumn>=viewP->tableNumberColumns){
			viewP->tableCurrentColumn=0;
			MMIa_newBreak(viewId);
			}
	#endif
	}
#if (WIPVRSN_1 <= 2)
//#if (GWCVERSION == 0250) || (GWCVERSION == 0270)
VOID CALL_GWC_API(MMIa_newText(UINT8 viewId, UINT8 textId, const WCHAR *text, BOOL isLink, const WCHAR *linkTitle, INT8 format)) {
#else
VOID CALL_GWC_API(MMIa_newText(UINT8 viewId, UINT8 textId, const WCHAR *text, BOOL isLink, const WCHAR *linkTitle, WCHAR accessKey, INT8 format)) {
#endif
	GleViewType* viewP = GleViewSearchByViewId(viewId);
	GleAddToGleList(viewP, GleTextConstruct(viewId, textId, text, isLink, linkTitle, format, FALSE));
	}
	
VOID GLEc_newTextTest(UINT8 viewId, UINT8 textId, const GLE_STRING text, BOOL isLink, const GLE_STRING linkTitle, INT8 format) 
	{
	WCHAR* p1 = (WCHAR*) WIPAlloc(GLEa_strlen(text)*2+2);
	WCHAR* p2 = (WCHAR*) WIPAlloc(GLEa_strlen(linkTitle)*2+2);
	GLEa_strcpyGle2Wchar(p1,text);
	GLEa_strcpyGle2Wchar(p2,linkTitle);
#if (WIPVRSN_1 <= 2)
//#if (GWCVERSION == 0250) || (GWCVERSION == 0270)
	CALL_GWC_API(MMIa_newText(viewId,textId,p1,isLink,p2,format));
#else
	CALL_GWC_API(MMIa_newText(viewId,textId,p1,isLink,p2,0,format));
#endif
	WIPFree(p2);
	WIPFree(p1);
	}
	
#if (WIPVRSN_1 > 2)
//#if (GWCVERSION != 0250)
VOID CALL_GWC_API(MMIa_setLanguage (UINT8 viewId, const CHAR *language)) {
	}
#endif
	
VOID CALL_GWC_API(MMIa_showCard(UINT8 viewId)) {
	GleViewType* viewP = GleViewSearchByViewId(viewId);

	#ifdef DISPLAY_GLEELEMENTLIST_SIZE_IN_CONTENT
		CHAR* temp1 = (CHAR*)WIPAlloc(10);
		temp1 = StrIToA(temp1, SizeOfGleList());
		GLEc_newTextTest(1, 1, temp1, FALSE, "", 0);
		WIPFree(temp1);
	#endif

	GleRenderCard(viewP, &viewP->horizontalSize, &viewP->verticalSize);
	GleKeyScrollCard(viewP, NoDirection);
	viewP->cardComplete = TRUE;
	if(!viewP->waitingForNImages)
		GLEa_status(viewId, AllImagesDone);
	}
