/*
 * Copyright (C) AU-System AB, 2000.
 * All rights reserved.
 *
 * This software is covered by the license agreement between
 * the end user and AU-System AB, and may be used and copied
 * only in accordance with the terms of the said agreement.
 *
 * AU-System AB does not assume any responsibility or 
 * liability for any errors or inaccuracies in this 
 * software, or any consequential, incidental or indirect 
 * damage arising out of the use of the Generic Layout Engine 
 * software.
 */

#include "gleview.h"

VOID* WIPAlloc(UINT32 size);
VOID WIPFree( VOID* memory );
extern GleViewType* gGleViewList;
extern BOOL gAllViewsDisplayImages;
extern BOOL gAllViewsUpdateImages;

extern GleDeleteCard(GleViewType* viewP);

GleViewType* GleViewCreateAddInit(UINT8 viewId, UINT16 width, UINT16 height) {
	/* Creates a new View */
	GleViewType* newView = WIPAlloc(sizeof(GleViewType));

	/* Put the new view first in view list */
	newView->next = gGleViewList;
	gGleViewList = newView;

	/* Initalize the variables in new view */
	newView->viewId = viewId;
	newView->displayImages = gAllViewsDisplayImages;
	newView->updateImages = gAllViewsUpdateImages;

	newView->mMIa_wait=FALSE;
	newView->redrawWhenNotWait=FALSE;
	newView->cardComplete=FALSE;
#ifndef	SUPPORT_TABLES
	newView->tableNumberColumns=0;
	newView->tableCurrentColumn=0;
#endif
	newView->waitingForNImages=0;
	newView->vertOffset=0;
	newView->horizOffset=0;
	newView->verticalSize=0;	
	newView->horizontalSize=0;

	newView->width = width;
	newView->height = height;

	newView->oldPenX=0;
	newView->oldPenY=0;
	newView->xSumMaxForFieldSet=0;
	newView->firstGleElement=0;
	newView->lastGleElement=0;
	newView->markedGleElement=0;   /* Marked by keys */	
	newView->selectedGleElement=0; /* Selected by mouse */

	newView->gleSingleSelect =0;
	newView->gleFieldSetPtr =0;
	newView->numberOfNestledFieldSet=0;
	newView->title=0;
	newView->URL=0;
	newView->history=0;
	newView->next=0;

	return(newView);
	}

VOID GleViewDelete(GleViewType* viewP) {
	/* Deletes a View from list */
	GleViewType* prevP = gGleViewList;
	/* If first view in list */
	if(prevP == viewP) {
		gGleViewList = prevP->next;		
		}
	else 
		{
		/* Search the prev view */
		while(prevP->next != viewP)
			prevP = prevP->next;
		prevP->next = viewP->next;
		}
	GleDeleteCard(viewP);
	WIPFree(viewP);	
	}  
 	
GleViewType* GleViewSearchByViewId(UINT8 viewId) {
	/* Returns a pointer to a view by its viewId */
	GleViewType *p = gGleViewList;
	while (p)
		if(p->viewId == viewId)
			return p;
		else
			p = p->next;
	return 0;
	}