/* 
+----------------------------------------------------------------------------- 
|  Project :  
|  Modul   :  
+----------------------------------------------------------------------------- 
|  Copyright 2002 Texas Instruments Berlin, AG 
|                 All rights reserved. 
| 
|                 This file is confidential and a trade secret of Texas 
|                 Instruments Berlin, AG 
|                 The receipt of or possession of this file does not convey 
|                 any rights to reproduce or disclose its contents or to 
|                 manufacture, use, or sell anything it may describe, in 
|                 whole, or in part, without the specific written consent of 
|                 Texas Instruments Berlin, AG. 
+----------------------------------------------------------------------------- 
|  Purpose :  Constants to determine the dimensions of the frame
+----------------------------------------------------------------------------- 
*/ 

#ifndef GPRSCONST_H
#define GPRSCONST_H

/*==== CONSTANTS ============================================================*/

#define MAX_TIMER                  230
#define MAX_SIMULTANEOUS_TIMER     50

#if defined(MMI_PCSYN_ENABLED) && defined(MMI_MMSTASK_ENABLED)
#define MAX_ENTITIES                32 + 2
#define MAX_OS_TASKS                26 + 2
#elif defined(MMI_PCSYN_ENABLED) || defined(MMI_MMSTASK_ENABLED)
#define MAX_ENTITIES                32 + 1
#define MAX_OS_TASKS                26 + 1
#else
#define MAX_ENTITIES                32
#define MAX_OS_TASKS                26
#endif

#define MAX_SEMAPHORES               8
#define MAX_COMMUNICATIONS           MAX_OS_TASKS
#define TST_QUEUE_ENTRIES           50

#ifdef MEMORY_SUPERVISION
#define CONDATPOOL_PS_ADDITION 5000
#else  /* MEMORY_SUPERVISION */
#define CONDATPOOL_PS_ADDITION 0
#endif /* MEMORY_SUPERVISION */

#ifdef WAP
#define CONDATPOOL_WAP_ADDITION 15000
#else  /* WAP */
#define CONDATPOOL_WAP_ADDITION 0
#endif /* WAP */

#ifdef GRR_PPC_IF_PRIM
#define CONDATPOOL_SIZE   (52000 + CONDATPOOL_WAP_ADDITION + CONDATPOOL_PS_ADDITION)
#else  /* #ifdef GRR_PPC_IF_PRIM */
#ifdef _TARGET_
#define CONDATPOOL_SIZE   (50000 + CONDATPOOL_WAP_ADDITION + CONDATPOOL_PS_ADDITION)
#else /* #ifdef _TARGET_ */
#define CONDATPOOL_SIZE   (80000 + CONDATPOOL_WAP_ADDITION + CONDATPOOL_PS_ADDITION)
#endif /* #ifdef _TARGET_ */
#endif /* #ifdef GRR_PPC_IF_PRIM */


/*
 * Partitions for Dynamic Allocation (Primitive Communication)
 */
#define PRIM_PARTITION_0_SIZE		60
#define PRIM_PARTITION_1_SIZE	 128
#define PRIM_PARTITION_2_SIZE	 632
#define PRIM_PARTITION_3_SIZE	1600
#define PRIM_PARTITION_4_SIZE	   0
#define PRIM_PARTITION_5_SIZE	   0

#ifdef WIN32
/*
 * Required for testing LLC acknowledged mode.
 */
#define PRIMPOOL_0_PARTITIONS		100
#define PRIMPOOL_1_PARTITIONS		100
#define PRIMPOOL_2_PARTITIONS		20
#define PRIMPOOL_3_PARTITIONS		20
#define PRIMPOOL_4_PARTITIONS		0
#define PRIMPOOL_5_PARTITIONS		0

#else

#define PRIMPOOL_0_PARTITIONS		130
#define PRIMPOOL_1_PARTITIONS		99
#define PRIMPOOL_2_PARTITIONS		50
#define PRIMPOOL_3_PARTITIONS		7
#define PRIMPOOL_4_PARTITIONS		0
#define PRIMPOOL_5_PARTITIONS		0
#endif

/*
 * Partitions for Dynamic Allocation (no Communication)
 */
#define DMEM_PARTITION_0_SIZE		16
#ifdef _TARGET_
#define DMEM_PARTITION_1_SIZE		1600 /* for non tracing ccd arm7 */
#else
#define DMEM_PARTITION_1_SIZE		2800 /* for non tracing ccd pc */
#endif
#define DMEM_PARTITION_2_SIZE		0
#define DMEM_PARTITION_3_SIZE		0
#define DMEM_PARTITION_4_SIZE		0
#define DMEM_PARTITION_5_SIZE		0

#define DMEMPOOL_0_PARTITIONS		70
#define DMEMPOOL_1_PARTITIONS		1
#define DMEMPOOL_2_PARTITIONS		0
#define DMEMPOOL_3_PARTITIONS		0
#define DMEMPOOL_4_PARTITIONS		0
#define DMEMPOOL_5_PARTITIONS		0

/*
 * Partitions for Test Interface Communication
 */
#define TEST_PARTITION_0_SIZE	  80
#ifdef _TARGET_
 #define TEST_PARTITION_1_SIZE 160
#else
 #define TEST_PARTITION_1_SIZE 260
#endif
#define TEST_PARTITION_2_SIZE	1600
#define TEST_PARTITION_3_SIZE		 0
#define TEST_PARTITION_4_SIZE		 0
#define TEST_PARTITION_5_SIZE		 0

#define TESTPOOL_0_PARTITIONS		10
#define TESTPOOL_1_PARTITIONS		50
#define TESTPOOL_2_PARTITIONS		2
#define TESTPOOL_3_PARTITIONS		0
#define TESTPOOL_4_PARTITIONS		0
#define TESTPOOL_5_PARTITIONS		0

#endif
