/* 
+----------------------------------------------------------------------------- 
|  Project :  
|  Modul   :  J:\g23m-aci\aci\aci_slock.h
+----------------------------------------------------------------------------- 
|  Copyright 2002 Texas Instruments Berlin, AG 
|                 All rights reserved. 
| 
|                 This file is confidential and a trade secret of Texas 
|                 Instruments Berlin, AG 
|                 The receipt of or possession of this file does not convey 
|                 any rights to reproduce or disclose its contents or to 
|                 manufacture, use, or sell anything it may describe, in 
|                 whole, or in part, without the specific written consent of 
|                 Texas Instruments Berlin, AG. 
+----------------------------------------------------------------------------- 
|  Purpose :  
+----------------------------------------------------------------------------- 
*/ 

#ifndef ACI_SLOCK_H
#define ACI_SLOCK_H

/* Shared globals ------------------------------------------------ */

typedef struct{
  T_SIMLOCK_STATUS status[SIMLOCK_FIRST_SIM+1];  /* lint */   /* one status for every personalisation lock type */
  UBYTE            blocked;                 /* blocked flag */
  T_SIMLOCK_TYPE   current_lock;            /* currently checked lock */
}T_ACI_SLOCK_SHARED;

EXTERN T_ACI_SLOCK_SHARED AciSLockShrd;

/* Masking for SIM service table: */
#define NO_ALLOCATED              0
#define ALLOCATED_AND_DEACTIVATED 2
#define ALLOCATED_AND_ACTIVATED   3

/* SIM Servicetable Sservice numbers read from 11.11*/
#define SIMSERVICE_GID1 15
#define SIMSERVICE_GID2 16

#define NOT_PRESENT_8BIT  0xFF

/* SIM configuration information */
typedef struct
{
    UBYTE deper_key [16];
    UBYTE phase;
    UBYTE oper_mode;       /* SIM card functionality   */
    UBYTE pref_lang[5];
    UBYTE access_acm;
    UBYTE access_acmmax;
    UBYTE access_puct;
    UBYTE sim_gidl1[5];
    UBYTE sim_gidl2[5];
} T_ACI_SIM_CONFIG; // This is the same as T_MFW_SIM_CONFIG

/* Prototypes ----------------------------------------------------*/

/*
  Initialising of this module. Has to be called first and *once only* before calling any other method.
*/
void aci_slock_init ( void );

/*
  Checks the lock status of the given lock type.  On a successful
  check the status will be returned. If an error occurred SIMLOCK_FAIL
  will be returned. (Uses the ACI extension "personalisation data access".)
  */
T_SIMLOCK_STATUS aci_slock_checklock ( T_SIMLOCK_TYPE type );

/*
  Unlocks the lock of the given type after verifying the given password.
  The ACI extension for password verifying (see 2.3.9.1) will be used to
  determine if the given password is correct or not.
  On a successful unlock the actual status of the lock will be returned.
  If an error occurred SIMLOCK_FAIL will be returned.
  (Uses the ACI extension "personalisation data access".)
  */
T_SIMLOCK_STATUS aci_slock_unlock ( T_SIMLOCK_TYPE type, char *passwd );

/*
  Locks the lock of the given type.  On a successful lock the actual
  status of the lock will be returned. If an error occurred SIMLOCK_FAIL
  will be returned. This method will use the ACI extension for password
  verifying (see 2.3.9.1) to determine if the given password is correct
  or not. (Uses the ACI extension "personalisation data access".)
  */
T_SIMLOCK_STATUS aci_slock_lock ( T_SIMLOCK_TYPE type, char *passwd );

/*
  Checks for all kinds of personalisation locks. The given IMSI will be checked against the internal 
  personalisation data. Returns SIMLOCK_BLOCKED, if one lock was not matching the given IMSI.
  */
T_SIMLOCK_STATUS aci_slock_checkpersonalisation( void );

/*
  Checks only for service provider and corporate personalisation locks. The given IMSI will be checked against the internal 
  personalisation data. Returns SIMLOCK_BLOCKED, if one lock was not matching the given IMSI.
  */
T_SIMLOCK_STATUS aci_slock_checkpersonalisation_sp_cp( void );

/*
	PURPOSE : Install information found in the primitive into configuration buffers.
*/
void aci_slock_sim_init ( T_SIM_MMI_INSERT_IND *sim_mmi_insert_ind );

/*
  PURPOSE : Checks a service status.
*/
U8 aci_slock_sim_check_service (U8 nr);

/*
  PURPOSE : read SIM group identifier 1 from SIM card

*/
void aci_slock_sim_gid1_cnf(USHORT error, UBYTE *data);

/*
  PURPOSE : read SIM group identifier 2 from SIM card

*/
void aci_slock_sim_gid2_cnf(USHORT error, UBYTE *data);

/*
   PURPOSE :   Call back for SIM read.

*/
void aci_slock_sim_read_sim_cb(SHORT table_id);

/*
   PURPOSE :   Request to read SIM card.

*/
void aci_slock_sim_read_sim(USHORT data_id, UBYTE len, UBYTE max_length);
#endif /* ACI_SLOCK_H */

