/* 
+----------------------------------------------------------------------------- 
|  Project :  GSM-F&D (8411)
|  Modul   :  CUS_ACI
+----------------------------------------------------------------------------- 
|  Copyright 2002 Texas Instruments Berlin, AG 
|                 All rights reserved. 
| 
|                 This file is confidential and a trade secret of Texas 
|                 Instruments Berlin, AG 
|                 The receipt of or possession of this file does not convey 
|                 any rights to reproduce or disclose its contents or to 
|                 manufacture, use, or sell anything it may describe, in 
|                 whole, or in part, without the specific written consent of 
|                 Texas Instruments Berlin, AG. 
+----------------------------------------------------------------------------- 
|  Purpose :  Custom dependent definitions for AT Command Interpreter
|             Use this header for definitions to integrate the
|             ACI in your target system !
+----------------------------------------------------------------------------- 
*/ 

#ifndef CUS_ACI_H
#define CUS_ACI_H

/*==== CONSTANTS ==================================================*/
/*
 * TIMER_VALUES
 *
 * Description :  The constants define the timer values
 *                for the MMI timer TXXX depending on the various
 *                channel types and service access point identifiers.
 *                If your target system uses other units please
 *                change the values.
 */

/*
 * all values in milliseconds
 */
#define TICS_PER_DECIHOURS     (20*60*6*50)
#define TDTE_INACTIV_VALUE     500
#define TRING_VALUE            3000
#define TMPTY_VALUE            10000        /* 10s (5 to 30 seconds) */
#define TECT_VALUE             13500  /* 13,5s (5 to 15 seconds GSM 04.91 and 04.80) */
#define TDTMF_VALUE            3000         /*  3s wait time GSM 02.07 */

/* MAX_ACI_TIMER
 * Description :  The constant defines the number of
 *                timeouts which can be stored. */
#if defined FF_EOTD //&& defined _SIMULATION_
#define MAX_ACI_TIMER 28
#else
#define MAX_ACI_TIMER 23
#endif
/*
 * VERSION
 *
 * Description :  The constants define the type and the value
 *                of a version identification. The version
 *                is part of the monitor struct.
 */

#ifndef SMI
#define T_VERSION   CHAR
#define VERSION_ACI  "ACI 1.0"
#endif

#ifdef SMI
#define T_VERSION   CHAR
#define VERSION_SMI  "SMI 1.0"
#endif

/*
 * VSI_CALLER
 *
 * Description :  For multithread applications the constant VSI_CALLER
 *                must be defined to identify the calling thread of the
 *                VSI-Interface. This must be done correponding to the
 *                type of T_VSI_CALLER in VSI.H. The comma symbol is
 *                neccessary because the vsi-functions are called
 *                like this vsi_xxx (VSI_CALLER par2, par3 ...)
 */

EXTERN T_HANDLE aci_handle;

#define VSI_CALLER aci_handle,
#define VSI_CALLER_SINGLE aci_handle

/*
 * Network operator list
 *
 * Description :  this is the list of all known network operators.
 *                There are five entries for each operator. A long
 *                name (max.24 chars), a short name (max.10 chars),
 *                the mobile country code, the mobile network code
 *                and a language reference. It is important to leave
 *                the EOL indicator at the end of the list to terminate
 *                the list.
 * Source :       The list depends on the GSM Association Infocentre - SE_13
 *                document
 * 
 */
typedef struct operNtry
{
    const char * longName;
    const char * shrtName;
    SHORT  mcc;
    SHORT  mnc;
    UBYTE 	 pnn;  /* PLMN Network Name Source (for EONS) */
    UBYTE 	dcs;  /* PLMN Network Name Data Coding Scheme (for EONS) */
} T_OPER_NTRY;

#ifdef CMH_MMF_C

const T_OPER_NTRY operList[] =
{
    {"Test Network",              "TestNet",          0x001,0x01F   }, /* NOT UPDATED, because of test network            */
    {"Test Net 222",              "Test222",          0x222,0x22F   }, /* NOT UPDATED, because of test network            */
    {"A one",                     "one",              0x232,0x05F   }, /* updated ("one","one")                           */
    {"A1",                        "A1",               0x232,0x01F   }, /* updated ("A1","A1")                             */
    {"T-Mobile A",                "TMO A",            0x232,0x03F   }, /* updated ("A max.","max.")                       */
    {"A tele.ring",               "telering",         0x232,0x07F   }, /* updated ("A tele.ring","telering")              */
    {"SETAR GSM",                 "SETARGSM",         0x363,0x01F   }, /* NOT UPDATED, because of doubeled masks in spec  */
    {"AF AWCC",                   "AWCC",             0x412,0x01F   }, /* updated ("AF AWCC","AWCC")                      */
    {"UNITEL",                    "UNITEL",           0x631,0x02F   }, /* updated ("UNITEL","UNITEL")                     */
    {"AMC - AL",                  "A M C",            0x276,0x01F   }, /* updated ("AMC - AL","A M C")                    */
    {"vodafone AL",               "voda AL",          0x276,0x02F   }, /* updated ("VODAFONE","VAL")                      */
#if !(defined(BT_ADAPTER) && defined(_SIMULATION_))
    /*
     *  UPDATED NETWORKS
     */ 
    {"GR COSMOTE",                "C-OTE",            0x202,0x01F   },
    {"vodafone GR",               "voda GR",          0x202,0x05F   },
    {"GR Q-TELECOM",              "Q-TELCOM",         0x202,0x09F   },
    {"GR TELESTET",               "TLSTET",           0x202,0x10F   },
    {"vodafone NL",               "voda NL",          0x204,0x04F   },
    {"NL KPN",                    "NL KPN",           0x204,0x08F   },
    {"O2 - NL",                   "O2 -NL",           0x204,0x12F   },
    {"T-Mobile NL",               "TMO NL",           0x204,0x16F   },
    {"Orange NL",                 "Orange",           0x204,0x20F   },
    {"BEL PROXIMUS",              "PROXI",            0x206,0x01F   },
    {"B mobistar",                "mobi*",            0x206,0x10F   },
    {"BASE",                      "BASE",             0x206,0x20F   },
    {"Orange F",                  "Orange",           0x208,0x01F   },
    {"F SFR",                     "SFR",              0x208,0x10F   },
    {"F - BOUYGUES TELECOM",      "BYTEL",            0x208,0x20F   },
    {"STA-MOBILAND",              "M-AND",            0x213,0x03F   },
    {"vodafone ES",               "voda ES",          0x214,0x01F   },
    {"E AMENA",                   "AMENA",            0x214,0x03F   },
    {"XFERA",                     "XFERA",            0x214,0x04F   },
    {"MOVISTAR",                  "MSTAR",            0x214,0x07F   },
    {"H PANNON GSM",              "PANNON",           0x216,0x01F   },
    {"WESTEL",                    "WESTEL",           0x216,0x30F   },
    {"vodafone HU",               "voda HU",          0x216,0x70F   },
    {"BA-ERONET",                 "ERONET",           0x218,0x03F   },
    {"MOBI'S",                    "MOBI'S",           0x218,0x05F   },
    {"BH GSMBIH",                 "GSMBIH",           0x218,0x90F   },
    {"HTmobile HR",               "HTmobile",         0x219,0x01F   },
    {"HR VIP",                    "VIP",              0x219,0x10F   },
    {"YU MOBTEL",                 "MOBTEL",           0x220,0x01F   },
    {"ProMonte",                  "ProMonte",         0x220,0x02F   },
    {"YUG 03",                    "YU MTS",           0x220,0x03F   },
    {"MONET",                     "MONET",            0x220,0x04F   },
    {"I TIM",                     "TIM",              0x222,0x01F   },
    {"vodafone IT",               "voda IT",          0x222,0x10F   },
    {"I WIND",                    "I WIND",           0x222,0x88F   },
    {"RO CONNEX",                 "CONNEX",           0x226,0x01F   },
    {"RO Cosmorom",               "Cosmorom",         0x226,0x03F   },
    {"RO ORANGE",                 "ORANGE",           0x226,0x10F   },
    {"SWISS GSM",                 "SWISS",            0x228,0x01F   },
    {"sunrise",                   "sunrise",          0x228,0x02F   },
    {"Orange CH",                 "Orange",           0x228,0x03F   },
    {"T-Mobile CZ",               "TMO CZ",           0x230,0x01F   },
    {"EUROTEL - CZ",              "ET - CZ",          0x230,0x02F   },
    {"OSKAR",                     "OSKAR",            0x230,0x03F   },
    {"Orange SK",                 "Orange",           0x231,0x01F   },
    {"EUROTEL-SK",                "ET-SK",            0x231,0x02F   },
    {"O2 - UK",                   "O2 -UK",           0x234,0x10F   },
    {"vodafone UK",               "voda UK",          0x234,0x15F   },
    {"3 UK",                      "3 UK",             0x234,0x20F   },
    {"T-Mobile UK",               "TMO UK",           0x234,0x30F   },
    {"ORANGE",                    "ORANGE",           0x234,0x33F   },
    {"JT GSM",                    "JT GSM",           0x234,0x50F   },
    {"Cable & Wireless Guernsey", "C&W",              0x234,0x55F   },
    {"DK SONOFON",                "SONO",             0x238,0x02F   },
    {"TELIA DK",                  "TELIA",            0x238,0x20F   },
    {"Orange",                    "Orange",           0x238,0x30F   },
    {"Orange",                    "Orange",           0x238,0x30F   },
    {"TELIA S",                   "TELIA",            0x240,0x01F   },
    {"S COMVIQ",                  "IQ",               0x240,0x07F   },
    {"vodafone SE",               "voda SE",          0x240,0x08F   },
    {"N Telenor",                 "TELENOR",          0x242,0x01F   },
    {"N NetCom GSM",              "N COM",            0x242,0x02F   },
    {"FI TELIA",                  "TELIA",            0x244,0x03F   },
    {"FI RADIOLINJA",             "RL",               0x244,0x05F   },
    {"FI  FINNET",                "FINNET",           0x244,0x09F   },
    {"FI 2G",                     "2G",               0x244,0x12F   },
    {"FI AMT",                    "FI AMT",           0x244,0x14F   },
    {"FI SONERA",                 "SONERA",           0x244,0x91F   },
    {"OMNITEL LT",                "OMT",              0x246,0x01F   },
    {"LT BITE GSM",               "BITE",             0x246,0x02F   },
    {"TELE2",                     "TELE2",            0x246,0x03F   },
    {"LV LMT GSM  LV LMT",        "LMT GSM",          0x247,0x01F   },
    {"LV TELE2",                  "TELE2",            0x247,0x02F   },
    {"EE EMT GSM",                "EMT",              0x248,0x01F   },
    {"248 02/EE 02/RLE/EE RLE/E", "RLE",              0x248,0x02F   },
    {"TELE2",                     "TELE2",            0x248,0x03F   },
    {"MTS-RUS",                   "MTS",              0x250,0x01F   },
    {"MegaFon RUS",               "MegaFon",          0x250,0x02F   },
    {"MegaFon RUS",               "MegaFon",          0x250,0x02F   },
    {"MegaFon RUS",               "MegaFon",          0x250,0x02F   },
    {"SIBCHALLENGE RUS",          "RUS_SCN",          0x250,0x04F   },
    {"RUS DTC",                   "DTC",              0x250,0x10F   },
    {"ORENSOT",                   "ORENSOT",          0x250,0x11F   },
    {"RUS Kuban-GSM",             "KUGSM",            0x250,0x13F   },
    {"RUS16 250 16",              "NTC",              0x250,0x16F   },
    {"RUS 17",                    "ERMAK",            0x250,0x17F   },
    {"EXTEL RUS",                 "EXTEL",            0x250,0x28F   },
    {"RUS North Caucasian GSM",   "NC-GSM",           0x250,0x44F   },
    {"Primetelefone RUS",         "Primtel",          0x250,0x92F   },
    {"UA UMC",                    "UMC",              0x255,0x01F   },
    {"UKR-WellCOM",               "WellCOM",          0x255,0x02F   },
    {"UA-KYIVSTAR",               "UA-KS",            0x255,0x03F   },
    {"UA-GT",                     "GT",               0x255,0x05F   },
    {"BY VELCOM",                 "VELCOM",           0x257,0x01F   },
    {"MD VOXTEL",                 "VOXTEL",           0x259,0x01F   },
    {"MD MOLDCELL",               "MDCELL",           0x259,0x02F   },
    {"Plus GSM",                  "PLUS",             0x260,0x01F   },
    {"Era",                       "Era",              0x260,0x02F   },
    {"IDEA  PL IDEA  PL 03",      "IDEA",             0x260,0x03F   },
    {"T-Mobile D",                "TMO D",            0x262,0x01F   },
    {"Vodafone D2",               "Voda D2",          0x262,0x02F   },
    {"E-Plus",                    "E-Plus",           0x262,0x03F   },
    {"o2 - de",                   "o2 - de",          0x262,0x07F   },
    {"MobilCom",                  "MobilCom",         0x262,0x13F   },
    {"GIBTEL GSM",                "GIBTEL",           0x266,0x01F   },
    {"vodafone P",                "voda P",           0x268,0x01F   },
    {"vodafone P",                "voda P",           0x268,0x01F   },
    {"P OPTIMUS",                 "OPTIM",            0x268,0x03F   },
    {"P TMN",                     "TMN",              0x268,0x06F   },
    {"L LUXGSM",                  "LUXGSM",           0x270,0x01F   },
    {"L TANGO",                   "TANGO",            0x270,0x77F   },
    {"vodafone IE",               "voda IE",          0x272,0x01F   },
    {"02 - IRL",                  "02 -IRL",          0x272,0x02F   },
    {"IRL - METEOR",              "METEOR",           0x272,0x03F   },
    {"IS SIMINN",                 "SIMINN",           0x274,0x01F   },
    {"IS TAL",                    "TAL",              0x274,0x02F   },
    {"Viking",                    "Viking",           0x274,0x04F   },
    {"vodafone MT",               "voda MT",          0x278,0x01F   },
    {"go mobile",                 "gomobile",         0x278,0x21F   },
    {"CY CYTAGSM",                "CY-GSM",           0x280,0x01F   },
    {"GEO-GEOCELL",               "GCELL",            0x282,0x01F   },
    {"MAGTI-GSM-GEO",             "MAGTI",            0x282,0x02F   },
    {"RA-ARMGSM",                 "ARMMO1",           0x283,0x01F   },
    {"M-TEL GSM BG",              "M-TEL",            0x284,0x01F   },
    {"BG GLOBUL",                 "GLOBUL",           0x284,0x05F   },
    {"TR TURKCELL",               "TCELL",            0x286,0x01F   },
    {"TR TELSIM",                 "TELSIM",           0x286,0x02F   },
    {"TR ARIA",                   "ARIA",             0x286,0x03F   },
    {"TR AYCELL",                 "AYCELL",           0x286,0x04F   },
    {"Froya Tele",               "FT-GSM",           0x288,0x01F   },
    {"KALL",                      "KALL",             0x288,0x02F   },
    {"TELE Greenland",            "TELE GRL",         0x290,0x01F   },
    {"SI.MOBIL",                  "SI.MOBIL",         0x293,0x40F   },
    {"SI MOBITEL GSM",            "SI-GSM",           0x293,0x41F   },
    {"SI VEGA 070",               "VEGA 070",         0x293,0x70F   },
    {"MKD-MOBIMAK",               "MOBI-M",           0x294,0x01F   },
    {"MKD COSMOFON",              "COSMOFON",         0x294,0x02F   },
    {"telecom FL",                "FLGSM",            0x295,0x01F   },
    {"Orange FL",                 "OFL",              0x295,0x02F   },
    {"FL1",                       "FL1",              0x295,0x05F   },
    {"LI TANGO",                  "TANGO",            0x295,0x77F   },
    {"Fido",                      "Fido",             0x302,0x370   },
    {"CAN Rogers Wireless Inc.",  "ROGERS",           0x302,0x720   },
    {"Cingular",                  "Cingular",         0x310,0x150   },
    {"T-Mobile",                  "TMO",              0x310,0x200   },
    {"T-Mobile",                  "TMO",              0x310,0x210   },
    {"T-Mobile",                  "TMO",              0x310,0x220   },
    {"T-Mobile",                  "TMO",              0x310,0x230   },
    {"T-Mobile",                  "TMO",              0x310,0x240   },
    {"T-Mobile",                  "TMO",              0x310,0x250   },
    {"T-Mobile",                  "TMO",              0x310,0x260   },
    {"T-Mobile",                  "TMO",              0x310,0x270   },
    {"USA 340",                   "WestLink",         0x310,0x340   },
    {"AT&T Wireless",             "AT&T",             0x310,0x380   },
    {"USA ONELINK",               "ONELINK",          0x310,0x460   },
    {"USA-WVA WIRELESS",          "W V W",            0x310,0x530   },
    {"Cellular One DCS",          "Cell One",         0x310,0x560   },
    {"USA AE Airadigm",           "ARDGMC",           0x310,0x640   },
    {"Iowa Wireless USA",         "IWS",              0x310,0x770   },
    {"USA Pinpoint",              "Pinpoint",         0x310,0x790   },
    {"Mx Telcel GSM",             "TELCEL",           0x334,0x20F   },
    {"JM DIGICEL",                "DIGICEL",          0x338,0x05F   },
    {"F-Orange",                  "Orange",           0x340,0x01F   },
    {"BOUYGTEL-C",                "BOUYG-C",          0x340,0x20F   },
    {"APUA PCS ANTIGUA",          "APUA-PCS",         0x344,0x30F   },
    {"Telcell GSM",               "Telcell",          0x362,0x51F   },
    {"ANT CURACAO TELECOM GSM",   "CT GSM",           0x362,0x69F   },
    {"UTS Wireless Curacao N.V.", "UTS",              0x362,0x91F   },
    {"CU/C_COM",                  "C_COM",            0x368,0x01F   },
    {"orange",                    "orange",           0x370,0x01F   },
    {"TSTT",                      "TSTT",             0x374,0x12F   },
    {"AZE - AZERCELL GSM",        "ACELL",            0x400,0x01F   },
    {"BAKCELL GSM 2000",          "BKCELL",           0x400,0x02F   },
    {"KZ K-MOBILE",               "K-MOBILE",         0x401,0x01F   },
    {"KZ KCELL",                  "KCELL",            0x401,0x02F   },
    {"ESSAR",                     "ESSAR",            0x404,0x01F   },
    {"AirTel",                    "AirTel",           0x404,0x02F   },
    {"AirTel",                    "AirTel",           0x404,0x03F   },
    {"HUTCH",                     "HUTCH",            0x404,0x11F   },
    {"INDEH",                     "ESCOTEL",          0x404,0x12F   },
    {"Hutch",                     "HUTCH",            0x404,0x13F   },
    {"INA SPICE",                 "SPICE",            0x404,0x14F   },
    {"ESSAR",                     "ESSAR",            0x404,0x15F   },
    {"INDEK",                     "ESCOTEL",          0x404,0x19F   },
    {"INA  MaxTouch",             "MAXTCH",           0x404,0x20F   },
    {"BPL MOBILE",                "BPL MOBILE",       0x404,0x21F   },
    {"BPL MOBILE",                "BPL MOBILE",       0x404,0x27F   },
    {"INA HUTCH",                 "HUTCH",            0x404,0x30F   },
    {"AirTel",                    "AirTel",           0x404,0x31F   },
    {"INA AIRTEL",                "AIRTEL",           0x404,0x40F   },
    {"INA RPG",                   "RPG",              0x404,0x41F   },
    {"INA AIRCEL",                "AIRCEL",           0x404,0x42F   },
    {"BPL MOBILE",                "BPL MOB",          0x404,0x43F   },
    {"INA SPICE",                 "SPICE",            0x404,0x44F   },
    {"AirTel",                    "AirTel",           0x404,0x45F   },
    {"BPL MOBILE",                "BPL MOB",          0x404,0x46F   },
    {"AirTel",                    "AirTel",           0x404,0x49F   },
    {"INDEU",                     "ESCOTEL",          0x404,0x56F   },
    {"ESSAR",                     "ESSAR",            0x404,0x60F   },
    {"HUTCH",                     "HUTCH",            0x404,0x86F   },
    {"AirTel",                    "AirTel",           0x404,0x90F   },
    {"AirTel",                    "AirTel",           0x404,0x92F   },
    {"AirTel",                    "AirTel",           0x404,0x93F   },
    {"AirTel",                    "AirTel",           0x404,0x94F   },
    {"AirTel",                    "AirTel",           0x404,0x95F   },
    {"AirTel",                    "AirTel",           0x404,0x96F   },
    {"AirTel",                    "AirTel",           0x404,0x97F   },
    {"AirTel",                    "AirTel",           0x404,0x98F   },
    {"PK-UFONE",                  "UFONE",            0x410,0x03F   },
    {"SRI DIALOG",                "DIALOG",           0x413,0x02F   },
    {"SRI - CELLTEL",             "CELLTEL",          0x413,0x03F   },
    {"MM 900",                    "MPTGSM",           0x414,0x01F   },
    {"RL Cellis",                 "CLLIS",            0x415,0x01F   },
    {"RL LibanCell",              "LibCL",            0x415,0x03F   },
    {"Fastlink",                  "FSTLNK",           0x416,0x01F   },
    {"JO MobCom",                 "MobCom",           0x416,0x77F   },
    {"94 SYRIA",                  "94 SYRIA",         0x417,0x02F   },
    {"SYR MOBILE SYR",            "MOBILE",           0x417,0x09F   },
    {"KT MTCNet",                 "MTC",              0x419,0x02F   },
    {"KT WATANIYA",               "WATANIYA",         0x419,0x03F   },
    {"ALJAWAL",                   "KSA",              0x420,0x01F   },
    {"SPACETEL",                  "SPACETEL",         0x421,0x02F   },
    {"OMAN MOBILE",               "OMAN",             0x422,0x02F   },
    {"UAE ETISALAT",              "ETSLT",            0x424,0x02F   },
    {"IL ORANGE",                 "ORANGE",           0x425,0x01F   },
    {"JAWWAL-PALESTINE",          "JAWWAL",           0x425,0x05F   },
    {"BATELCO",                   "BATELCO",          0x426,0x01F   },
    {"QAT QATARNET",              "Q-NET",            0x427,0x01F   },
    {"MN MobiCom",                "MobiCom",          0x428,0x99F   },
    {"IR-TCI",                    "432 11",           0x432,0x11F   },
    {"IR KISH",                   "KIFZO",            0x432,0x14F   },
    {"UZB DAEWOO-GSM",            "DW-GSM",           0x434,0x04F   },
    {"UZB CSOCOM GSM",            "COSCOM",           0x434,0x05F   },
    {"BITEL KGZ",                 "BITEL",            0x437,0x01F   },
    {"VN MOBIFONE",               "VMS",              0x452,0x01F   },
    {"VN VINAPHONE",              "GPC",              0x452,0x02F   },
    {"CSL",                       "CSL",              0x454,0x00F   },
    {"HK ORANGE",                 "ORANGE",           0x454,0x04F   },
    {"SmarTone",                  "SmarTone",         0x454,0x06F   },
    {"HK NEW WORLD",              "NWPCS",            0x454,0x10F   },
    {"HK PEOPLES",                "PEOPLES",          0x454,0x12F   },
    {"HK SUNDAY",                 "SUNDAY",           0x454,0x16F   },
    {"Macau SMC",                 "SmarTone",         0x455,0x00F   },
    {"MAC-CTMGSM",                "CTMGSM",           0x455,0x01F   },
    {"Hutchison MAC",             "HT Macau",         0x455,0x03F   },
    {"MOBITEL - KHM",             "MT-KHM",           0x456,0x01F   },
    {"KHM-Hello GSM",             "KHM-SM",           0x456,0x02F   },
    {"CAMBODIA SHINAWATRA",       "CAMSHIN",          0x456,0x18F   },
    {"ETL MOBILE NETWORK",        "ETLMNW",           0x457,0x02F   },
    {"CHINA  MOBILE",             "CMCC",             0x460,0x00F   },
    {"CHN-CUGSM",                 "CU-GSM",           0x460,0x01F   },
    {"Far EasTone",               "FET",              0x466,0x01F   },
    {"TWN Tuntex GSM 1800",       "TUNTEX",           0x466,0x06F   },
    {"KGT-Online",                "KGT",              0x466,0x88F   },
    {"TWN Chunghwa",              "Chunghwa",         0x466,0x92F   },
    {"TWN MOBITAI",               "TW MOB",           0x466,0x93F   },
    {"TWN GSM 1800",              "TCC",              0x466,0x97F   },
    {"BGD-GP",                    "GP",               0x470,0x01F   },
    {"BGD AKTEL",                 "AKTEL",            0x470,0x02F   },
    {"BD ShebaWorld",             "SHEBA",            0x470,0x03F   },
    {"MV DHIMOBILE",              "D-MOBILE",         0x472,0x01F   },
    {"MY maxis mobile",           "maxis",            0x502,0x12F   },
    {"MY TMTOUCH",                "TMTOUCH",          0x502,0x13F   },
    {"DiGi",                      "DiGi",             0x502,0x16F   },
    {"MY TIMECel",                "TIMECel",          0x502,0x17F   },
    {"MY CELCOM",                 "CELCOM",           0x502,0x19F   },
    {"Telstra Mobile",            "Telstra",          0x505,0x01F   },
    {"YES OPTUS",                 "Optus",            0x505,0x02F   },
    {"vodafone AU",               "voda AU",          0x505,0x03F   },
    {"3 AUS",                     "3 AUS",            0x505,0x06F   },
    {"IND SATELINDOCEL",          "SAT-C",            0x510,0x01F   },
    {"IND SATELINDOCEL",          "SAT-C",            0x510,0x01F   },
    {"LIPPO TEL",                 "LIPPOTEL",         0x510,0x08F   },
    {"IND TELKOMSEL",             "T-SEL",            0x510,0x10F   },
    {"IND - Excelcom",            "proXL",            0x510,0x11F   },
    {"IND IM3",                   "IM-3",             0x510,0x21F   },
    {"ISLACOM",                   "ISLACOM",          0x515,0x01F   },
    {"Globe Telecom-PH",          "GLOBE",            0x515,0x02F   },
    {"SMART",                     "SMART",            0x515,0x03F   },
    {"PH Sun Cellular",           "SUN",              0x515,0x05F   },
    {"TH GSM",                    "TH GSM",           0x520,0x01F   },
    {"TH ACT 1900",               "ACT-1900",         0x520,0x15F   },
    {"TH-DTAC",                   "DTAC",             0x520,0x18F   },
    {"TH GSM 1800",               "GSM 1800",         0x520,0x23F   },
    {"Orange Th",                 "Orange",           0x520,0x99F   },
    {"SingTel-G9",                "SingTel",          0x525,0x01F   },
    {"SingTel-G18",               "SingTel",          0x525,0x02F   },
    {"SGP M1-GSM",                "M1-GSM",           0x525,0x03F   },
    {"STARHUB-SGP",               "STARHUB",          0x525,0x05F   },
    {"BRU-DSTCom",                "DSTCom",           0x528,0x11F   },
    {"vodafone NZ",               "voda NZ",          0x530,0x01F   },
    {"U-CALL",                    "U-CALL",           0x539,0x01F   },
    {"VUT SMILE",                 "SMILE",            0x541,0x01F   },
    {"FJ VODAFONE",               "VODAFONE",         0x542,0x01F   },
    {"Blue Sky AS",               "AST",              0x544,0x11F   },
    {"NCL MOBILIS",               "MOBNCL",           0x546,0x01F   },
    {"F-VINI",                    "VINI",             0x547,0x20F   },
    {"FSM Telecom",               "FSMTC",            0x550,0x01F   },
    {"EGY MobiNiL",               "MobiNiL",          0x602,0x01F   },
    {"vodafone EG",               "voda EG",          0x602,0x02F   },
    {"ALGERIAN MOBILE NETWORK",   "AMN",              0x603,0x01F   },
    {"Djezzy",                    "Djezzy",           0x603,0x02F   },
    {"MOR MEDITEL",               "MEDITEL",          0x604,0x00F   },
    {"MOR IAM",                   "IAM",              0x604,0x01F   },
    {"TUNISIE TELECOM",           "TUNTEL",           0x605,0x02F   },
    {"GAMCEL",                    "GAMCEL",           0x607,0x01F   },
    {"SN ALIZE",                  "ALIZE",            0x608,0x01F   },
    {"SN-SENTEL SG",              "SENTEL",           0x608,0x02F   },
    {"MALITEL ML",                "MALITEL",          0x610,0x01F   },
    {"GN LAGUI",                  "LAGUI",            0x611,0x02F   },
    {"CI CORA",                   "CORA",             0x612,0x01F   },
    {"Orange CI",                 "Orange",           0x612,0x03F   },
    {"TELECEL-CI",                "TELCEL",           0x612,0x05F   },
    {"BF CELTEL",                 "CELTEL",           0x613,0x02F   },
    {"NE CELTEL",                 "CELTEL",           0x614,0x02F   },
    {"TG-TOGO CELL",              "TGCELL",           0x615,0x01F   },
    {"TELECEL BENIN",             "TLCL-BEN",         0x616,0x02F   },
    {"BJ BENINCELL",              "BENCELL",          0x616,0x03F   },
    {"CELLPLUS-MRU",              "CELL +",           0x617,0x01F   },
    {"EMTEL-MRU",                 "EMTEL",            0x617,0x10F   },
    {"LBR Lonestar Cell",         "LoneStar",         0x618,0x01F   },
    {"GH SPACEFON",               "SPACE",            0x620,0x01F   },
    {"GH ONEtouch",               "ONEtouch",         0x620,0x02F   },
    {"GH-MOBITEL",                "mobitel",          0x620,0x03F   },
    {"ECONET NG",                 "ECONET",           0x621,0x20F   },
    {"MTN - NG",                  "MTN-NG",           0x621,0x30F   },
    {"NG NITEL",                  "NG NITEL",         0x621,0x40F   },
    {"CELTEL TCD",                "CELTEL",           0x622,0x01F   },
    {"TD LIBERTIS",               "LIBERTIS",         0x622,0x02F   },
    {"MTN CAM",                   "62401",            0x624,0x01F   },
    {"MTN CAM",                   "62401",            0x624,0x01F   },
    {"Orange CAM",                "Orange",           0x624,0x02F   },
    {"CPV MOVEL",                 "CMOVEL",           0x625,0x01F   },
    {"628 01/LIBERTIS",           "LIBERTIS",         0x628,0x01F   },
    {"GAB TELECEL",               "TELECEL",          0x628,0x02F   },
    {"CELTEL GA",                 "CELTEL",           0x628,0x03F   },
    {"CELTEL CD",                 "CELTEL",           0x629,0x01F   },
    {"COG LIBERTIS",              "LIBERTIS",         0x629,0x10F   },
    {"CELTEL RC",                 "CELTEL",           0x630,0x02F   },
    {"CELLCO GSM",                "CELLCO",           0x630,0x04F   },
    {"CD OASIS",                  "OASIS",            0x630,0x89F   },
    {"SEYCEL",                    "633-01",           0x633,0x01F   },
    {"SEZ AIRTEL",                "AIRTEL",           0x633,0x10F   },
    {"MobiTel SDN",               "MobiTel",          0x634,0x01F   },
    {"R-CELL",                    "RCELL",            0x635,0x10F   },
    {"ETH-MTN",                   "ET-MTN",           0x636,0x01F   },
    {"SOM BARAKAAT",              "BARAKAAT",         0x637,0x01F   },
    {"Telsom Mobile",             "telsom",           0x637,0x82F   },
    {"Safaricom",                 "SAF-COM",          0x639,0x02F   },
    {"YES!",                      "YES!",             0x639,0x03F   },
    {"MOBITEL - TZ",              "MOBITEL",          0x640,0x02F   },
    {"ZANTEL-TZ",                 "ZANTEL",           0x640,0x03F   },
    {"CELTEL TZ",                 "CELTEL",           0x640,0x05F   },
    {"UG CelTel",                 "CELTEL",           0x641,0x01F   },
    {"MTN-UGANDA",                "MTN-UG",           0x641,0x10F   },
    {"UTL Telecel",               "UTL",              0x641,0x11F   },
    {"Spacetel BI",               "SPACETEL",         0x642,0x01F   },
    {"BUSAFA",                    "SAFARIS",          0x642,0x02F   },
    {"MOZ - mCel",                "mCel",             0x643,0x01F   },
    {"ZM CELTEL",                 "CELTEL",           0x645,0x01F   },
    {"MG Madacom",                "Madacom",          0x646,0x01F   },
    {"MG ANTARIS",                "ANTARIS",          0x646,0x02F   },
    {"Orange re",                 "Orange",           0x647,0x00F   },
    {"F-OMT",                     "OMT",              0x647,0x02F   },
    {"SFR REUNION",               "SFR RU",           0x647,0x10F   },
    {"ZW NET*ONE",                "64801",            0x648,0x01F   },
    {"TELECEL ZW",                "TELECEL",          0x648,0x03F   },
    {"ZW ECONET",                 "ECONET",           0x648,0x04F   },
    {"MTC NAMIBIA",               "MTCNAM",           0x649,0x01F   },
    {"MW CP 900",                 "CP 900",           0x650,0x01F   },
    {"CELTEL MW",                 "CELTEL",           0x650,0x10F   },
    {"VCL COMMS",                 "VCLCOM",           0x651,0x01F   },
    {"LS-ECONET-EZI-CEL",         "EZI-CEL",          0x651,0x02F   },
    {"BW MASCOM",                 "MASCOM",           0x652,0x01F   },
    {"Orange",                    "Orange",           0x652,0x02F   },
    {"Swazi-MTN",                 "SwaziMTN",         0x653,0x10F   },
    {"VodaCom-SA",                "VODA",             0x655,0x01F   },
    {"Cell C",                    "Cell C",           0x655,0x07F   },
    {"MTN-SA",                    "MTN",              0x655,0x10F   },
    {"ESV PERSONAL",              "PERSONAL",         0x706,0x01F   },
    {"Digicel",                   "DIGICEL",          0x706,0x02F   },
    {"I.C.E.",                    "I.C.E.",           0x712,0x01F   },
    {"TIM",                       "TIM PERU",         0x716,0x10F   },
    {"UNIFON",                    "UNIFON",           0x722,0x07F   },
    {"AR PERSONAL",               "AR TP",            0x722,0x34F   },
    {"PORT-HABLE",                "P-HABLE",          0x722,0x35F   },
    {"TIM BRASIL",                "TIM",              0x724,0x02F   },
    {"Oi",                        "Oi",               0x724,0x31F   },
    {"CL ENTEL PCS",              "ENTEL",            0x730,0x01F   },
    {"CL ENTEL PCS",              "ENTEL",            0x730,0x10F   },
    {"VZ INFO",                   "INFONT",           0x734,0x01F   },
    {"DIGITEL",                   "DIGITEL",          0x734,0x02F   },
    {"VIVA",                      "NUEVATEL",         0x736,0x01F   },
    {"BOMOV",                     "EMOVIL",           0x736,0x02F   },
    {"HOLA PARAGUAY S.A.",        "HPGYSA",           0x744,0x01F   },
    {"ICMS SR",                   "ICMS",             0x746,0x01F   },
    {"SR.TELESUR.GSM",            "TeleG",            0x746,0x02F   },
    /*
     *  NOT UPDATED, because of doubeled masks in spec
     */ 
    {"SCS RUS",                   "SCS",              0x250,0x05F   },
    {"SCS RUS",                   "SCS",              0x250,0x05F   },
    {"RUS SCS",                   "SCS",              0x250,0x05F   },
    {"RUS BMT",                   "BMT",              0x250,0x07F   },
    {"RUS 07  RUS SMARTS",        "SMARTS",           0x250,0x07F   },
    {"SIBI-RUS",                  "SIBI",             0x250,0x12F   },
    {"RUS - Sakhalin",            "SAKHALIN",         0x250,0x12F   },
    {"RUS FEast; RUS 12",         "BWC",              0x250,0x12F   },
    {"RF FAR EAST",               "RF FEast",         0x250,0x12F   },
    {"Uraltel",                   "Uraltel",          0x250,0x39F   },
    {"RUS SUCT",                  "SUCT",             0x250,0x39F   },
    {"Bee Line",                  "Bee Line",         0x250,0x99F   },
    {"SETAR GSM",                 "SETARGSM",         0x363,0x01F   },
    {"IL Cellcom",                "Cellcom",          0x425,0x02F   },
    /*
     *  NEW NETWORKS, BUT NOT ADDED
     */ 
     /*
    {"3 ITA",                     "3 ITA",            0x222,0x99F   },
    {"3 AT",                      "3 AT",             0x232,0x10F   },
    {"T-Mobile UK",               "TMO UK",           0x234,0x31F   },
    {"T-Mobile UK",               "TMO UK",           0x234,0x32F   },
    {"Manx Pronto",               "Pronto",           0x234,0x58F   },
    {"TDC MOBIL",                 "DK TDC",           0x238,0x01F   },
    {"3",                         "3",                0x240,0x02F   },
    {"SWEDEN",                    "SWE",              0x240,0x04F   },
    {"SWEDEN",                    "SWE",              0x240,0x04F   },
    {"RUS 07  RUS SMARTS",        "SMARTS",           0x250,0x15F   },
    {"RUS: INDIGO",               "INDIGO",           0x250,0x19F   },
    {"RUS INDIGO",                "INDIGO",           0x250,0x19F   },
    {"RUS.INDIGO",                "INDIGO",           0x250,0x19F   },
    {"DTI",                       "DTIGSM",           0x250,0x19F   },
    {"TELE2",                     "TELE2",            0x250,0x20F   },
    {"TELE2",                     "TELE2",            0x250,0x20F   },
    {"TELE2",                     "TELE2",            0x250,0x20F   },
    {"RUS ECC",                   "ECC",              0x250,0x20F   },
    {"TELE2",                     "TELE2",            0x250,0x20F   },
    {"MTS BY",                    "MTS",              0x257,0x02F   },
    {"IS Islandssimi hf",         "Islandss",         0x274,0x03F   },
    {"USA Wireless 2000 Telepho"  "WTTCKy",           0x310,0x11F   },
    {"T-Mobile",                  "TMO",              0x310,0x26F   },
    {"T-Mobile",                  "TMO",              0x310,0x31F   },
    {"Highland",                  "HIGHLAND",         0x310,0x70F   },
    {"T-Mobile",                  "TMO",              0x310,0x160   },
    {"Cingular",                  "Cingular",         0x310,0x170   },
    {"USA Dutch Harbor",          "D-HARBOR",         0x310,0x190   },
    {"USA - CellularOne",         "Cell",             0x310,0x320   },
    {"Cingular",                  "Cingular",         0x310,0x410   },
    {"Cell One of NE Colorado",   "NECCI",            0x310,0x450   },
    {"T-Mobile",                  "TMO",              0x310,0x660   },
    {"MX MOVISTAR GSM",           "MOVISTAR",         0x334,0x03F   },
    {"C&W",                       "C&W",              0x338,0x180   },
    {"C&W",                       "C&W",              0x342,0x600   },
    {"C&W",                       "C&W",              0x346,0x140   },
    {"TELECOM BDA",               "TELE BDA",         0x350,0x01F   },
    {"BTC MOBILITY LTD.",         "MOBILITY",         0x350,0x02F   },
    {"BaTelCell",                 "BaTelCel",         0x364,0x39F   },
    {"BT B-Mobile",               "B-Mobile",         0x402,0x11F   },
    {"HUTCH",                     "HUTCH",            0x404,0x05F   },
    {"AirTel",                    "AirTel",           0x404,0x10F   },
    {"IDEA",                      "IDEA",             0x404,0x22F   },
    {"IDEA",                      "IDEA",             0x404,0x24F   },
    {"BSNL MOBILE",               "CellOne",          0x404,0x34F   },
    {"BSNL MOBILE",               "CellOne",          0x404,0x38F   },
    {"BSNL MOBILE",               "CellOne",          0x404,0x51F   },
    {"BSNL MOBILE",               "CellOne",          0x404,0x53F   },
    {"BSNL MOBILE",               "CellOne",          0x404,0x54F   },
    {"BSNL MOBILE",               "CellOne",          0x404,0x55F   },
    {"BSNL MOBILE",               "CellOne",          0x404,0x57F   },
    {"BSNL MOBILE",               "CellOne",          0x404,0x58F   },
    {"BSNL MOBILE",               "CellOne",          0x404,0x59F   },
    {"BSNL MOBILE",               "CellOne",          0x404,0x62F   },
    {"BSNL MOBILE",               "CellOne",          0x404,0x64F   },
    {"BSNL MOBILE",               "CellOne",          0x404,0x66F   },
    {"IN-DOLPHIN",                "DOLPHIN",          0x404,0x68F   },
    {"IN-DOLPHIN",                "DOLPHIN",          0x404,0x69F   },
    {"INDH1",                     "Oasis",            0x404,0x70F   },
    {"BSNL MOBILE",               "CellOne",          0x404,0x71F   },
    {"BSNL MOBILE",               "CellOne",          0x404,0x72F   },
    {"BSNL MOBILE",               "CellOne",          0x404,0x73F   },
    {"BSNL MOBILE",               "CellOne",          0x404,0x74F   },
    {"BSNL MOBILE",               "CellOne",          0x404,0x75F   },
    {"BSNL MOBILE",               "CellOne",          0x404,0x76F   },
    {"BSNL MOBILE",               "CellOne",          0x404,0x77F   },
    {"IDEA",                      "IDEA",             0x404,0x78F   },
    {"BSNL MOBILE",               "CellOne",          0x404,0x79F   },
    {"BSNL MOBILE",               "CellOne",          0x404,0x80F   },
    {"BSNL MOBILE",               "CellOne",          0x404,0x81F   },
    {"AF TDCA",                   "TDCA",             0x412,0x20F   },
    {"LK Mobitel",                "MOBITEL",          0x413,0x71F   },
    {"SYRIATEL",                  "SYRIATEL",         0x417,0x01F   },
    {"SabaFon",                   "SABAFON",          0x421,0x01F   },
    {"UZMACOM",                   "UZMGSM",           0x434,0x02F   },
    {"UZB-UZD",                   "UZDGSM",           0x434,0x07F   },
    {"Somoncom",                  "Somoncom",         0x436,0x01F   },
    {"Indigo-T",                  "INDIGO",           0x436,0x02F   },
    {"TJK MLT",                   "MLT",              0x436,0x03F   },
    {"TJT - Tajik Tel",           "TajikTel",         0x436,0x05F   },
    {"JP DoCoMo",                 "DoCoMo",           0x440,0x10F   },
    {"J-PHONE",                   "JPHONE",           0x440,0x20F   },
    {"KR KTF",                    "KTF",              0x450,0x02F   },
    {"KR KTF",                    "KTF",              0x450,0x08F   },
    {"3 HK",                      "3",                0x454,0x03F   },
    {"45703",                     "LATMOBIL",         0x457,0x03F   },
    {"TANGO LAO",                 "TANGO",            0x457,0x08F   },
    {"T3G",                       "T3G",              0x466,0x89F   },
    {"TransAsia",                 "TransAsi",         0x466,0x99F   },
    {"KP SUN",                    "SUNNET",           0x467,0x193   },
    {"PNGBMobile",                "BMobile",          0x537,0x01F   },
    {"KL-Frigate",                "KI-FRIG",          0x545,0x09F   },
    {"TUNISIANA",                 "TUNSIANA",         0x605,0x03F   },
    {"MR MATTEL",                 "MATTEL",           0x609,0x01F   },
    {"IKATEL ML",                 "IKATEL",           0x610,0x02F   },
    {"BELL BENIN COMMUNICATION",  "BBCOM",            0x616,0x04F   },
    {"CELTEL SL",                 "CELTEL",           0x619,0x01F   },
    {"Glo NG",                    "glo",              0x621,0x50F   },
    {"STP CSTmovel",              "CSTmovel",         0x626,0x01F   },
    {"GNQ01",                     "GETESA",           0x627,0x01F   },
    {"VodaCom",                   "VodaCom",          0x640,0x04F   },
    {"BTL",                       "BTL",              0x702,0x67F   },
    {"INTELCO",                   "INTELCO",          0x702,0x68F   },
    {"SERCOM",                    "SERCOM",           0x710,0x73F   },
    {"PANCW",                     "PANCW",            0x714,0x01F   },
    {"TIM BRASIL",                "TIM",              0x724,0x03F   },
    {"TIM BRASIL",                "TIM",              0x724,0x04F   },
    {"BRA TA",                    "TA",               0x724,0x05F   },
    {"BRA TA",                    "TA",               0x724,0x05F   },
    {"BRA TA",                    "TA",               0x724,0x05F   },
    {"TELEFONICA",                "TMOVIL",           0x730,0x02F   },
    {"PORTA GSM",                 "PORTAGSM",         0x740,0x01F   },
    {"PGY Porthable",             "Porth",            0x744,0x02F   },
    */
    /*
     *  NOT EXISTING NETWORKS
     */ 
    {"IT BLU",                    "BLU",              0x222,0x98F   },
    {"Orange",                    "Orange",           0x240,0x03F   },
    {"Telecom XXI RUS",           "TXXI",             0x250,0x93F   },
    {"Quam",                      "Quam ",            0x262,0x14F   },
    {"USA Wireless 2000 Teleph",  "WTTCK",            0x310,0x110   },
    {"T-Mobile USA",              "TMO USA",          0x310,0x310   },
    {"PCS ONE",                   "PCSONE",           0x310,0x580   },
    {"Telemetrix Technologies",   "TLXT",             0x310,0x740   },
    {"INA TATA",                  "TATA",             0x404,0x07F   },
    {"SyriaTel",                  "SYRIATEL",         0x417,0x93F   },
    {"KSA EAE-ALJAWAL",           "KSA EAE",          0x420,0x07F   },
    {"BUZTEL",                    "BUZTEL",           0x434,0x01F   },
    {"HK PACLINK PCS",            "PLINK",            0x454,0x18F   },
    {"HK P PLUS",                 "P PLUS",           0x454,0x22F   },
    {"LAO GSM   ",                "LAO GSM",          0x457,0x01F   },
    {"ACeS",                      "ACeS",             0x466,0x68F   },
    {"ACeS",                      "ACeS",             0x510,0x00F   },
    {"ACeS",                      "ACeS",             0x510,0x00F   },
    {"ACeS",                      "ACeS",             0x515,0x11F   },
    {"ACeS",                      "ACeS",             0x520,0x20F   },
    {"Tritel - TZ",               "TRITEL",           0x640,0x01F   },
#endif
/*--- EOL -------------------------------------------------------*/
    { NULL,                       NULL,                -1,-1   }
};
#else

EXTERN const T_OPER_NTRY operList[];

#endif      /* of #ifdef CMH_MMF_C */

#endif
