/*
+-----------------------------------------------------------------------------
|  Project :
|  Modul   :  J:\g23m-aci\aci_ext\aci_ext_pers.c
+-----------------------------------------------------------------------------
|  Copyright 2002 Texas Instruments Berlin, AG
|                 All rights reserved.
|
|                 This file is confidential and a trade secret of Texas
|                 Instruments Berlin, AG
|                 The receipt of or possession of this file does not convey
|                 any rights to reproduce or disclose its contents or to
|                 manufacture, use, or sell anything it may describe, in
|                 whole, or in part, without the specific written consent of
|                 Texas Instruments Berlin, AG.
+-----------------------------------------------------------------------------
|  Purpose :
+-----------------------------------------------------------------------------
*/
#include "aci_all.h"

#include "aci_cmh.h"
#include "ati_cmd.h"
#include "aci_cmd.h"

#ifdef FAX_AND_DATA
#include "aci_fd.h"
#endif    /* of #ifdef FAX_AND_DATA */

#undef ACI_PERSONALISTION_USE_FFS

#include "aci.h"
#include "psa.h"
#include "psa_sim.h"
#include "psa_sms.h"
#include "psa_mmi.h"
#include "cmh.h"
#include "phb.h"
#include "aoc.h"

#include "psa_sim.h"            /* simShrdPrm */

#ifdef WIN32
/* some testcases need that... some others cannot pass then though */
//#define SIMLOCK
#endif

#include "aci_ext_pers.h"
#include "aci_slock.h"

#include "pcm.h"                /* EF_SIMLCKEXT */

#ifdef MMI_SIMLOCK_TEST
  #define ACI_PERSONALISTION_USE_FFS
#endif

#ifdef ACI_PERSONALISTION_USE_FFS
#include "ffs.h"
#include "ffs_coat.h"
#endif /* ACI_PERSONALISTION_USE_FFS */



#define SLOCK_PCM_NRM 1         /* SIM lock data is stored as SIMLOCK to the PCM */
#define SLOCK_PCM_EXT 2         /* SIM lock data is stored as SIMLOCKEXT to the PCM */

UBYTE aci_slock_sim_lock_field = NOT_PRESENT_8BIT;  /* type of PCM record for sim lock field */

#ifdef _SIMULATION_
EF_SIMLCKEXT simlock_pcm_memory;    /* emulates the PCM memory in WIN32 environment. */
#endif

#include "aci_ext_pers_cus.h" /* the customer specific personalisation data. */

LOCAL UBYTE pcm_get_sim_lock (EF_SIMLCKEXT * simlck);
LOCAL void pcm_set_sim_lock (EF_SIMLCKEXT * simlck);

#ifdef ACI_PERSONALISTION_USE_FFS
LOCAL effs_t flash_read_result = -1; /* remeber last flash read result */
LOCAL effs_t flash_pers_write();
LOCAL effs_t flash_pers_read();
#endif

LOCAL void decodeB2A (U8 *bcd, U8 bcd_len, U8 *ascii);

/*
  Load SIM lock details from PCM and put it into the actual memory structure. Returns PCM_OK if everything was okay.
  The actual PCM record is only supporting one IMSI per lock. This should be extended in the future!
  */
LOCAL UBYTE aci_ext_personalisation_load_pcm( void )
{
  EF_SIMLCKEXT simlck;
  UBYTE result;
  CHAR t_imsi[MAX_IMSI_LEN+1]={'\0'};

  #ifdef ACI_PERSONALISTION_USE_FFS
  flash_read_result = flash_pers_read();
  #endif /* ACI_PERSONALISTION_USE_FFS */

  result = pcm_get_sim_lock(&simlck);
  if (result EQ PCM_OK)
  {
    memcpy(&personalisation_status, &simlck, T_ACI_PERS_STATUS_LEN);

    #ifdef ACI_PERSONALISTION_USE_FFS
    personalisation_sim.count = (personalisation_sim.count EQ 0?1:personalisation_sim.count);
    personalisation_nw.count = (personalisation_nw.count EQ 0?1:personalisation_nw.count);
    personalisation_ns.count = (personalisation_ns.count EQ 0?1:personalisation_ns.count);
    personalisation_sp.count = (personalisation_sp.count EQ 0?1:personalisation_sp.count);
    personalisation_cp.count = (personalisation_cp.count EQ 0?1:personalisation_cp.count);
    #else /* ACI_PERSONALISTION_USE_FFS */
    personalisation_sim.count = 1;
    personalisation_nw.count = 1;
    personalisation_ns.count = 1;
    personalisation_sp.count = 1;
    personalisation_cp.count = 1;
    #endif /* ACI_PERSONALISTION_USE_FFS */
    
    memcpy(t_imsi, simlck.p_imsi, simlck.len_p_imsi);
    t_imsi[simlck.len_p_imsi] = '\0';                  /* make an ASCIIZ string */
    psaSIM_encodeIMSI(t_imsi, &personalisation_sim.data->c_field, personalisation_sim.data->imsi);

    memcpy(t_imsi, simlck.n_imsi, simlck.len_n_imsi);
    t_imsi[simlck.len_n_imsi] = '\0';
    psaSIM_encodeIMSI(t_imsi, &personalisation_nw.data->c_field, personalisation_nw.data->imsi);

    memcpy(t_imsi, simlck.ns_imsi, simlck.len_ns_imsi);
    t_imsi[simlck.len_ns_imsi] = '\0';
    psaSIM_encodeIMSI(t_imsi, &personalisation_ns.data->c_field, personalisation_ns.data->imsi);

    memcpy(t_imsi, simlck.sp_imsi, simlck.len_sp_imsi);
    t_imsi[simlck.len_sp_imsi] = '\0';
    psaSIM_encodeIMSI(t_imsi, &personalisation_sp.data->c_field, personalisation_sp.data->imsi);

    memcpy(t_imsi, simlck.c_imsi, simlck.len_c_imsi);
    t_imsi[simlck.len_c_imsi] = '\0';
    psaSIM_encodeIMSI(t_imsi, &personalisation_cp.data->c_field, personalisation_cp.data->imsi);

    personalisation_sp.data->gid1 = simlck.gidl1;
    personalisation_cp.data->gid1 = simlck.gidl1;
    personalisation_cp.data->gid2 = simlck.gidl2;
  }
  return result;
}

LOCAL void convert_personalisationstruct_to_pcm(EF_SIMLCKEXT* simlck)
{
  CHAR t_imsi[MAX_IMSI_LEN+1]={'\0'};

  memcpy(simlck,&personalisation_status,T_ACI_PERS_STATUS_LEN);

  psaSIM_decodeIMSI(personalisation_sim.data->imsi,personalisation_sim.data->c_field,t_imsi);
  simlck->len_p_imsi = strlen(t_imsi);
  memcpy(simlck->p_imsi, t_imsi, MAX_IMSI_LEN);  /* since x_imsi in ffs is only of size[15] */

  psaSIM_decodeIMSI(personalisation_nw.data->imsi,personalisation_nw.data->c_field,t_imsi);
  simlck->len_n_imsi = strlen(t_imsi);
  memcpy(simlck->n_imsi, t_imsi, MAX_IMSI_LEN);

  psaSIM_decodeIMSI(personalisation_ns.data->imsi,personalisation_ns.data->c_field,t_imsi);
  simlck->len_ns_imsi = strlen(t_imsi);
  memcpy(simlck->ns_imsi, t_imsi, MAX_IMSI_LEN);

  psaSIM_decodeIMSI(personalisation_sp.data->imsi,personalisation_sp.data->c_field,t_imsi);
  simlck->len_sp_imsi = strlen(t_imsi);
  memcpy(simlck->sp_imsi, t_imsi, MAX_IMSI_LEN);

  psaSIM_decodeIMSI(personalisation_cp.data->imsi,personalisation_cp.data->c_field,t_imsi);
  simlck->len_c_imsi = strlen(t_imsi);
  memcpy(simlck->c_imsi, t_imsi, MAX_IMSI_LEN);


  if (FldGet(personalisation_status.locks2,clock) EQ SIMLOCK_ENABLED)
  {
   simlck->gidl1 = personalisation_cp.data->gid1;
   simlck->gidl2 = personalisation_cp.data->gid2;
  }

  if (FldGet(personalisation_status.locks1,splock) EQ SIMLOCK_ENABLED)
  {
   simlck->gidl1 = personalisation_sp.data->gid1;
  }
}

/*
  Save SIM lock details from the actual memory structure and put it into PCM. Returns PCM_OK if everything was okay.
  */
LOCAL void aci_ext_personalisation_save( void )
{
  EF_SIMLCKEXT simlck;
  UBYTE result;

  result = pcm_get_sim_lock(&simlck);
  if (result EQ PCM_OK)
  {
    convert_personalisationstruct_to_pcm(&simlck);
    pcm_set_sim_lock(&simlck);
  }

  #ifdef ACI_PERSONALISTION_USE_FFS
  flash_pers_write();
  #endif
}

/*
  Initialisation of this extension. Might be used to initialise variables, set temporary data to default values and so on.
  This method will be called once before any other public method of this extension.
  */
void aci_ext_personalisation_init( void )
{
#ifndef MMI_SIMLOCK_TEST
  #ifdef WIN32
  /* initialize the faked PCM Data with the compiled data */
  convert_personalisationstruct_to_pcm(&simlock_pcm_memory);
  #endif
  aci_ext_personalisation_load_pcm();
#else /*for mmi testing, just read FFS*/
  #ifdef ACI_PERSONALISTION_USE_FFS
  flash_pers_read();
  #endif
#endif
}


/*
  ACI extension method for retrieving the data for one single personalisation type.
  This personalisation data is stored into a separate customer managed memory area
  and the customer has to implement this extension method to return the actual data
  of the questioned personalisation type.
  */
T_ACI_PERS_DATAS aci_ext_personalisation_get_data( T_SIMLOCK_TYPE personalisation_type )
{
  T_ACI_PERS_DATAS retvalue;

  TRACE_FUNCTION("aci_ext_personalisation_get_data()");

  switch (personalisation_type)
  {
    default:  /* should not occour */
    case SIMLOCK_FIRST_SIM:
    case SIMLOCK_SIM: retvalue.nw = personalisation_sim;break;
    case SIMLOCK_NETWORK: retvalue.nw = personalisation_nw;break;
    case SIMLOCK_NETWORK_SUBSET: retvalue.nw = personalisation_ns;break;
    case SIMLOCK_SERVICE_PROVIDER: retvalue.sp = personalisation_sp;break;
    case SIMLOCK_CORPORATE: retvalue.cp = personalisation_cp;break;
  }
  return retvalue;
}


/*
  ACI extension method for retrieving the status of a single personalisation type.
  The personalisation status is stored in a separate customer managed memory area
  and the customer has to implement this extension method to return the actual status
  of the questioned personalisation type.
  */
T_SIMLOCK_STATUS aci_ext_personalisation_get_status( T_SIMLOCK_TYPE personalisation_type )
{
  TRACE_FUNCTION("aci_ext_personalisation_get_status()");
  switch (personalisation_type)
  {
    case SIMLOCK_SIM:              return FldGet(personalisation_status.locks1,plock);
    case SIMLOCK_NETWORK:          return FldGet(personalisation_status.locks1,nlock);
    case SIMLOCK_NETWORK_SUBSET:   return FldGet(personalisation_status.locks1,nslock);
    case SIMLOCK_SERVICE_PROVIDER: return FldGet(personalisation_status.locks1,splock);
    case SIMLOCK_CORPORATE:        return FldGet(personalisation_status.locks2,clock);
    case SIMLOCK_FIRST_SIM:        return FldGet(personalisation_status.locks2,flock);
    default:                       return SIMLOCK_FAIL;
  }
}

/*
  ACI extension method for retrieving the status of a single personalisation type.
  The personalisation status is stored in a separate customer managed memory area
  and the customer has to implement this extension method to return the actual status
  of the questioned personalisation type.
  */
T_SIMLOCK_STATUS aci_ext_personalisation_set_status( T_SIMLOCK_TYPE personalisation_type,
  T_SIMLOCK_STATUS lock, char* passwd)
{
  U8 exp_pin[16];

  TRACE_FUNCTION("aci_ext_personalisation_set_status()");

  /* check for illegal status */
  if (lock EQ SIMLOCK_FAIL)
    return SIMLOCK_FAIL;

  /* Check, if lock is blocked. If this is the case, compare given PWD with PUK! */
  if (aci_ext_personalisation_get_status(personalisation_type) EQ SIMLOCK_BLOCKED)
    decodeB2A(personalisation_status.PUK, 8, exp_pin);
  else
  {
    switch (personalisation_type)
    {
      case SIMLOCK_SIM: decodeB2A(personalisation_status.PKey, 8, exp_pin);break;
      case SIMLOCK_NETWORK: decodeB2A(personalisation_status.NKey, 8, exp_pin);break;
      case SIMLOCK_NETWORK_SUBSET: decodeB2A(personalisation_status.NSKey, 8, exp_pin);break;
      case SIMLOCK_SERVICE_PROVIDER: decodeB2A(personalisation_status.SPKey, 8, exp_pin);break;
      case SIMLOCK_CORPORATE: decodeB2A(personalisation_status.CKey, 8, exp_pin);break;
      default: return SIMLOCK_FAIL;
    }
  }
  /*use length of stored pin NOT entered pin for comparison*/
  if (memcmp( exp_pin, passwd, strlen((char*)exp_pin) ))
  {
    /* Block personalisation lock, if tried too often! */
    personalisation_status.cnt++;
    if (personalisation_status.cnt >= personalisation_status.maxcnt)
    {
      switch (personalisation_type)
      {
        case SIMLOCK_SIM: personalisation_status.locks1=(UBYTE)FldSet(personalisation_status.locks1,plock,SIMLOCK_BLOCKED); break;
        case SIMLOCK_NETWORK: personalisation_status.locks1=(UBYTE)FldSet(personalisation_status.locks1,nlock,SIMLOCK_BLOCKED); break;
        case SIMLOCK_NETWORK_SUBSET: personalisation_status.locks1=(UBYTE)FldSet(personalisation_status.locks1,nslock,SIMLOCK_BLOCKED); break;
        case SIMLOCK_SERVICE_PROVIDER: personalisation_status.locks1=(UBYTE)FldSet(personalisation_status.locks1,splock,SIMLOCK_BLOCKED); break;
        case SIMLOCK_CORPORATE: personalisation_status.locks2=(UBYTE)FldSet(personalisation_status.locks2,clock,SIMLOCK_BLOCKED); break;
        default: return SIMLOCK_FAIL;
      }
      return SIMLOCK_BLOCKED;
    }
    return SIMLOCK_FAIL;
  }

  switch (personalisation_type)
  { /* the case (UBYTE)lock to help lint is save since if lock==SIMLOCK_FAIL==-1 is checked above! */
    case SIMLOCK_SIM: personalisation_status.locks1 = (UBYTE)FldSet(personalisation_status.locks1,plock,(UBYTE)lock);break;
    case SIMLOCK_NETWORK: personalisation_status.locks1 = (UBYTE)FldSet(personalisation_status.locks1,nlock,(UBYTE)lock);break;
    case SIMLOCK_NETWORK_SUBSET: personalisation_status.locks1 = (UBYTE)FldSet(personalisation_status.locks1,nslock,(UBYTE)lock);break;
    case SIMLOCK_SERVICE_PROVIDER: personalisation_status.locks1 = (UBYTE)FldSet(personalisation_status.locks1,splock,(UBYTE)lock);break;
    case SIMLOCK_CORPORATE: personalisation_status.locks2 = (UBYTE)FldSet(personalisation_status.locks2,clock,(UBYTE)lock);break;
    default: return SIMLOCK_FAIL;
  }

  /* Correct PIN, reset lock count and unblock, if blocked */
  personalisation_status.cnt = 0;    /* Correct PIN, reset lock count */

  /* save changes to PCM here ! */
  aci_ext_personalisation_save();

  return lock;
}

void aci_ext_personalisation_checkfirstsim( )
{
  int imsicount = 0;
  TRACE_FUNCTION("aci_ext_personalisation_checkfirstsim()");

  /* If personalisation on first SIM is enabled, transport the given IMSI to the SIM personalisation area */
  if (FldGet(personalisation_status.locks2,flock) EQ SIMLOCK_ENABLED)
  {
    /* Move IMSI of inserted SIM into the first free "slot" of SIM personalisation. */
    imsicount = personalisation_sim.count;
    personalisation_sim.data[imsicount].c_field = simShrdPrm.imsi.c_field;
    memcpy(&personalisation_sim.data[imsicount].imsi, simShrdPrm.imsi.field,MAX_IMSI-1);
    personalisation_sim.count++;

    personalisation_status.locks1 = (UBYTE)FldSet(personalisation_status.locks1, plock, SIMLOCK_ENABLED);
    personalisation_status.locks2 = (UBYTE)FldSet(personalisation_status.locks2, flock, SIMLOCK_DISABLED);


    /* save changes to PCM here ! */
  aci_ext_personalisation_save();
  }
}

/* internal methods ------------------------------------------------------------- */

/*
  PURPOSE : Fills the extended SIM Lock record with defaults if
            only the simple SIM Lock Record is available.
*/
LOCAL void pcm_fill_sim_lock (EF_SIMLCKEXT * simlck)
{
  TRACE_FUNCTION("pcm_fill_sim_lock()");

  if (aci_slock_sim_lock_field EQ SLOCK_PCM_NRM)
  {
    /*
     * PCM contains SIMLOCK record. Must be mapped to the internal
     * used SIMLOCKEXT record.
     *
     * gidl1, gidl2 to the end of record:
     */
    simlck->gidl1 = simlck->len_sp_imsi;
    simlck->gidl2 = simlck->sp_imsi [0];
    /*
     * IMSI must be copied for the other locks
     * N-Lock   digits 1..5, rest filled with 0x09 to suppress compare
     * SP-Lock  digits 1..5, rest filled with 0x09 to suppress compare
     * NS-Lock  digits 1..7, rest filled with 0x09 to suppress compare
     * C-Lock   digits 1..5, rest filled with 0x09 to suppress compare
     */
    memset (simlck->n_imsi, 0x09, MAX_IMSI_LEN);
    memcpy (simlck->n_imsi, simlck->p_imsi, 5);
    simlck->len_n_imsi = 5;

    memset (simlck->sp_imsi, 0x09, MAX_IMSI_LEN);
    memcpy (simlck->sp_imsi, simlck->p_imsi, 5);
    simlck->len_sp_imsi = 5;

    memset (simlck->ns_imsi, 0x09, MAX_IMSI_LEN);
    memcpy (simlck->ns_imsi, simlck->p_imsi, 7);
    simlck->len_ns_imsi = 7;

    memset (simlck->c_imsi, 0x09, MAX_IMSI_LEN);
    memcpy (simlck->c_imsi, simlck->p_imsi, 5);
    simlck->len_c_imsi = 5;
    /*
     * initialise unblock imsi so that it never will fit
     */
    simlck->len_u_imsi = 0;
  }
}

/*
  * ---------------------------------------------------------------------------
  * PURPOSE : Reads the SIM LOCK Record content from PCM
  * ---------------------------------------------------------------------------
  */
LOCAL UBYTE pcm_get_sim_lock (EF_SIMLCKEXT * simlck)
{
  #ifdef WIN32
  memcpy(simlck, &simlock_pcm_memory, SIZE_EF_SIMLCKEXT);
  return PCM_OK;
  #else

  UBYTE version;
  UBYTE sim_lock_type = 0;

  TRACE_FUNCTION("pcm_get_sim_lock()");


  if (aci_slock_sim_lock_field EQ NOT_PRESENT_8BIT)
  {
    if (pcm_ReadFile((UBYTE *) EF_SIMLCK_ID,
                     SIZE_EF_SIMLCK,
                     (UBYTE *) simlck,
                     &version) EQ PCM_OK)
    {
      aci_slock_sim_lock_field = SLOCK_PCM_NRM;
      pcm_fill_sim_lock (simlck);
      return PCM_OK;
    }
    if (pcm_ReadFile((UBYTE *) EF_SIMLCKEXT_ID,
                     SIZE_EF_SIMLCKEXT,
                     (UBYTE *) simlck,
                     &version) EQ PCM_OK)
    {
      aci_slock_sim_lock_field = SLOCK_PCM_EXT;
      return PCM_OK;
    }
    return PCM_NVRAM_ACCS_FAIL;
  }

  if (aci_slock_sim_lock_field EQ SLOCK_PCM_NRM)
  {
    if (pcm_ReadFile((UBYTE *) EF_SIMLCK_ID,
                     SIZE_EF_SIMLCK,
                     (UBYTE *) simlck,
                     &version) EQ PCM_OK)
    {
      pcm_fill_sim_lock (simlck);
      return PCM_OK;
    }
  }

  if (aci_slock_sim_lock_field EQ SLOCK_PCM_EXT)
  {
    if (pcm_ReadFile((UBYTE *) EF_SIMLCKEXT_ID,
                     SIZE_EF_SIMLCKEXT,
                     (UBYTE *) simlck,
                     &version) EQ PCM_OK)
    {
      return PCM_OK;
    }
  }
  return PCM_NVRAM_ACCS_FAIL;

  #endif
}

/*
  * ---------------------------------------------------------------------------
  PURPOSE : Saves the SIM LOCK Record content into PCM
  * ---------------------------------------------------------------------------
*/

LOCAL void pcm_set_sim_lock (EF_SIMLCKEXT * simlck)
{
  TRACE_FUNCTION("pcm_set_sim_lock()");

  #ifdef WIN32
  memcpy(&simlock_pcm_memory, simlck, SIZE_EF_SIMLCKEXT);
  #else

  if (aci_slock_sim_lock_field EQ SLOCK_PCM_NRM)
  {
    /*
     * set GDL1 and 2 on the expected location
     * => len_sp_imsi = gdl1, sp_imsi[0] = gdl2
     */
    simlck->len_sp_imsi = simlck->gidl1;
    simlck->sp_imsi [0] = simlck->gidl2;
    pcm_WriteFile((UBYTE *) EF_SIMLCK_ID,SIZE_EF_SIMLCK,
                                (UBYTE *) simlck);
  }
  else
    pcm_WriteFile((UBYTE *) EF_SIMLCKEXT_ID,SIZE_EF_SIMLCKEXT,
                                 (UBYTE *) simlck);

  #endif
}

/*
  PURPOSE : convert bcd to ASCII

*/

LOCAL void decodeB2A (U8 *bcd, U8 bcd_len, U8 *ascii)
{
    int i;

    TRACE_FUNCTION("simDecodeB2A()");

    for (i = 0; i < bcd_len * 2; i++)
    {
        ascii[i] = (i & 1) ? (bcd[i/2] >> 4) + '0'
                        : (bcd[i/2] & 0x0f) + '0';
        if (ascii[i] > '9')
        {
            ascii[i] = 0;
            break;
        }
    }
    ascii[i] = 0;
}

#define FFS_MELOCK_STATUS "/gsm/MELOCK/status"
#define FFS_MELOCK_NW "/gsm/MELOCK/Network"
#define FFS_MELOCK_NS "/gsm/MELOCK/NetworkSubnet"
#define FFS_MELOCK_SP "/gsm/MELOCK/Service"
#define FFS_MELOCK_CP "/gsm/MELOCK/Corporate"
#define FFS_MELOCK_SM "/gsm/MELOCK/SIMLock"

#ifdef ACI_PERSONALISTION_USE_FFS
static effs_t flash_pers_write()
{
  effs_t result;
  /*changed to pass addresses of data structures NOT actual data structure*/
  result = FFS_fwrite(FFS_MELOCK_STATUS,  &personalisation_status, sizeof(personalisation_status));
  if (result NEQ EFFS_OK)
    return result;
  result = FFS_fwrite(FFS_MELOCK_NW,  &personalisation_nw, sizeof(personalisation_nw));
  if (result NEQ EFFS_OK)
    return result;
  result = FFS_fwrite(FFS_MELOCK_NS,  &personalisation_ns, sizeof(personalisation_ns));
  if (result NEQ EFFS_OK)
    return result;
  result = FFS_fwrite(FFS_MELOCK_SP,  &personalisation_sp, sizeof(personalisation_sp));
  if (result NEQ EFFS_OK)
    return result;
  result = FFS_fwrite(FFS_MELOCK_CP,  &personalisation_cp, sizeof(personalisation_cp));
  if (result NEQ EFFS_OK)
    return result;
  result = FFS_fwrite(FFS_MELOCK_SM,  &personalisation_sim, sizeof(personalisation_sim));
  if (result NEQ EFFS_OK)
    return result;
  return result;
}

LOCAL effs_t flash_pers_read()
{
  effs_t result;
  result = FFS_fread(FFS_MELOCK_STATUS,  &personalisation_status, sizeof(personalisation_status));
  if (result NEQ EFFS_OK)
    return result;

  /* Optimization: Read personalisation data only, if needed (lock is not disabled) */
  if (aci_ext_personalisation_get_status(SIMLOCK_NETWORK) NEQ SIMLOCK_DISABLED)
  {
    result = FFS_fread(FFS_MELOCK_NW,  &personalisation_nw, sizeof(personalisation_nw));
    if (result NEQ EFFS_OK)
      return result;
  }

  if (aci_ext_personalisation_get_status(SIMLOCK_NETWORK_SUBSET)  NEQ SIMLOCK_DISABLED)
  {
    result = FFS_fread(FFS_MELOCK_NS,  &personalisation_ns, sizeof(personalisation_ns));
    if (result NEQ EFFS_OK)
      return result;
  }
  if (aci_ext_personalisation_get_status(SIMLOCK_SERVICE_PROVIDER) NEQ SIMLOCK_DISABLED)
  {
    result = FFS_fread(FFS_MELOCK_SP,  &personalisation_sp, sizeof(personalisation_sp));
    if (result NEQ EFFS_OK)
      return result;
  }
  if (aci_ext_personalisation_get_status(SIMLOCK_CORPORATE) NEQ SIMLOCK_DISABLED)
  {
    result = FFS_fread(FFS_MELOCK_CP,  &personalisation_cp, sizeof(personalisation_cp));
    if (result NEQ EFFS_OK)
      return result;
  }
  if (aci_ext_personalisation_get_status(SIMLOCK_SIM) NEQ SIMLOCK_DISABLED)
  {
    result = FFS_fread(FFS_MELOCK_SM,  &personalisation_sim, sizeof(personalisation_sim));
    if (result NEQ EFFS_OK)
      return result;
  }
  return result;

}
#endif /* ACI_PERSONALISTION_USE_FFS */


