/*******************************************************************************

          CONDAT (UK)

********************************************************************************

 This software product is the property of Condat (UK) Ltd and may not be
 disclosed to any third party without the express permission of the owner.

********************************************************************************

 $Project name: Basic MMI
 $Project code: BMI (6349)
 $Module:   PhoneBook
 $File:       MmiBookController.c
 $Revision:   1.0

 $Author:   Condat(UK)
 $Date:       25/10/00

********************************************************************************

 Description:

  The book controller module provides the external
  interface to the phone book. It provides ALL external
  entry points to the phone book system.

********************************************************************************

 $History: MmiBookController.c

  25/10/00      Original Condat(UK) BMI version.
  20/08/02     ver0.101  fzq modified

 
 $End

*******************************************************************************/


/*******************************************************************************

                                Include Files

*******************************************************************************/

// MMI phone book specific include files
#define ENTITY_MFW

/* includes */
#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#if defined (NEW_FRAME)

#include "typedefs.h"
#include "vsi.h"
#include "pei.h"
#include "custom.h"
#include "gsm.h"

#else

#include "stddefs.h"
#include "custom.h"
#include "gsm.h"
#include "vsi.h"

#endif
#include "mfw_sys.h"

#include "mfw_mfw.h"
#include "mfw_win.h"
#include "mfw_kbd.h"
#include "mfw_lng.h"
#include "mfw_edt.h"
#include "mfw_icn.h"
#include "mfw_mnu.h"
#include "mfw_tim.h"

#include "mfw_sim.h"
#include "mfw_cm.h"
#include "mfw_nm.h"
#include "mfw_phb.h"
#include "mfw_mme.h"
#include "mfw_sat.h"
#include "mfw_sms.h"

#include "dspl.h"

#include "ksd.h"
#include "psa.h"

#include "MmiMain.h"
#include "MmiBookController.h"
#include "MmiDummy.h"
#include "MmiDialogs.h"
#include "MmiLists.h"



#include "MmiMenu.h"
#include "MmiCall.h"
#include "MmiIcons.h"
#include "MmiIdle.h"

#include "MmiSoftKeys.h"
#include "MmiSounds.h"
#include "MmiIdle.h"
#include "MmiNetwork.h"
#include "MmiSat_i.h"
#include "MmiAoc.h"

#include "gdi.h"
#include "audio.h"

#include "cus_aci.h"
#include "p_sim.h"
#include "MmiTimers.h"


#include "MmiBookController.h"
#include "MmiBookUtils.h"

// MMI specific includes, outside scope of phone book
#include "MmiPins.h"
#include "MmiSmsMenu.h"
#include "MmiSmsSend.h"
#include "MmiCall.h"
#include "MmiIdle.h"
#include "Mmihost.h"

/*******************************************************************************

                                Definitions

*******************************************************************************/
/* Since we want to make sure all of the menu handlers get
   treated the same way, and have a protected handler, define
   a new macro, in terms of the MENU_HANDLER, which will
   invoke the protected function correctly (See below for
   further details)
*/
#define PROTECTED_MENU_HANDLER( Name ) \
static MENU_HANDLER( Protected##Name ); \
MENU_HANDLER( Name ) \
{ \
    return Protected( Name, Protected##Name, Menu, Item ); \
} \
static MENU_HANDLER( Protected##Name )

/*******************************************************************************

                                Local structures

*******************************************************************************/

/* A number of the actions the phone book is required to perform
   can only be activated when the user enters a security PIN number,
   the PIN2 code. In order to make the operation of the module
   clear, we will build a list of the protected actions, and then
   drive all menu operations through a single clearing process.

   The module will use the following action list perform the
   protection, note that the last entry in this list must be
   NULL
*/
typedef tBookStatus (*tProtectFunction)( tBookMfwMenu *Menu, tBookMfwMenuItem *Item );

tProtectFunction ProtectedActions[] = 
{
  bookNameEnter,
  bookEditOne,
  bookDeleteOne, 
  NULL 
};
extern T_idle idle_data;	//add by xzy 2002/12/02
extern voice_info voice_dial[VOICEDIAL_MAX];
extern unsigned char voice_index[VOICEDIAL_MAX];
extern char voice_filename[VOICEDIAL_MAX][12];
extern void PlayVmRecord_cb(void * parameter);
/*******************************************************************************

                                Private methods

*******************************************************************************/

/*******************************************************************************

 $Function:     Mmir_BaseAddress

 $Description:  returns the base address of the Master Index Table

 $Returns:    As above

 $Arguments:  none.

*******************************************************************************/

/* Provide a local routine which will perform the protection
   method. This will scan the list of protected actions, looking
   for the provided function, if it isn't a protected function
   then we will just call it directly, otherwise we will guard
   it with the PIN2 security code if we have a protected book
*/
static int Protected( tProtectFunction FunTag, tProtectFunction FunPtr, tBookMfwMenu *Menu, tBookMfwMenuItem *Item )
{
    pBookMfwWin    win_data = (pBookMfwWin) bookWindowData();
    pBookPhonebook book     = ((pBookStandard)win_data->user)->phbk;
    int i;

  /* Check for a protected book being used (details to be sorted
     out at a later date)
  */
  if ( phb_get_mode() == PHB_RESTRICTED )
  {
    /* Is the requested function one of the ones we need
       to protect ?
    */
    TRACE_FUNCTION("Phone Book in FDN mode:ask for pin 2");
    for ( i = 0; ProtectedActions[i]; i++ )
    {
      if ( FunTag == ProtectedActions[i] )
      {
        /* Yup, so we need to store the pending action for
           this window, this will be invoked by the checking
           operation, indirectly, if the check is successful
        */
        book->pin2_next = (T_VOID_FUNC) FunPtr;

        //We're cheating and passing the current window to the
        //handler via the unused menu parameter -- MC
        book->menu = (tBookMfwMenu*) bookCurrentWindow();
        book->item = Item;

        /* Call out to get the pin2 number checked, this will
           respond with a message back to this object, which
           we will deal with on receipt
        */
        return pin2_check( bookCurrentWindow());
      }
    }
  }

  /* we've come through the list of functions and either don't
     have a protected action to perform, or the book itself
     is not protected, so just invoke the requested action
  */
  return (FunPtr)( (tBookMfwMenu*) bookCurrentWindow()/*Menu*/, Item );
}



/*******************************************************************************

                                Public methods

*******************************************************************************/

/* Menu Handlers

   We are routing all of the phone book functionality through
   this module to allow the functional modules to be developed
   independantly where possible.

   This gives a single interface point for the rest of the
   MMI subsystem.

   All of the following functions use the MENU_HANDLER macro,
   as such they all have a common interface, this is not therefore
   documented in each case. The following information is common
   for each menu handler


   $Returns:  Status from worker routine

   $Arguments:  Menu, the menu from which the function was activated
                Item, the item associated with the call

*/


/*******************************************************************************

 $Function:     bookNameEnter

 $Description:

    Add new name to the phone book

 $Returns:    Refer Menu Handlers Definition Block Above

 $Arguments:  Refer Menu Handlers Definition Block Above

*******************************************************************************/

PROTECTED_MENU_HANDLER( bookNameEnter )
{
  //recast the menu parameter as the current window -- MC
    T_MFW_HND     win         = (T_MFW_HND)Menu;
    T_MFW_WIN     *win_data   = ( (T_MFW_HDR *) win )->data;
    tBookStandard   *data       = (tBookStandard *) win_data->user;
    T_phbk         *Phbk       = data->phbk;

  TRACE_FUNCTION("bookNameEnter");

  if (Phbk->UpdateAction!=ADD_FROM_IDLE&&Phbk->deletenumber!=1)//
  	{
  	memset(Phbk->phbk->edt_buf_foradd,'\0',PHB_number_len+3);
      memset( Phbk->phbk->edt_buf_number, '\0', PHB_number_len+3);
      Phbk->deletenumber=0;
  	}
  else
  	{
  		strncpy((char *) data->phbk->edt_buf_number, (char*)data->phbk->edt_buf_foradd, PHB_number_len );
  	}
  memset( Phbk->phbk->edt_buf_name,   '\0', PHB_name_len+5);
    // Indicate this is an additional entry being created
    
  Phbk->UpdateAction = CREATE_ENTRY;
  Phbk->BookType=     SIM_BOOK_SEL;

  Phbk->phbk->current.status.book=PHB_ADN;
  bookGetCurrentStatus( &Phbk->phbk->current.status );
  if ( Phbk->phbk->current.status.avail_entries )
  {
    Phbk->input_win=bookInputStart(win);
  }
  else
  {        
    tIndexTagNames Tag = ( Phbk->phbk->current.status.used_entries )
        ? TxtPhbkFull : TxtOperationNotAvail;
    bookShowInformation( win, Tag,NULL, NULL );
  }    
  return MFW_EVENT_CONSUMED;
}

/* 2003/12/01 sunsj add number enter */
PROTECTED_MENU_HANDLER( bookNvmNumberEnter )
{
    //recast the menu parameter as the current window -- MC
    T_MFW_HND     win           = (T_MFW_HND)Menu;
    T_MFW_WIN     *win_data   = ( (T_MFW_HDR *) win )->data;
    tBookStandard   *data         = (tBookStandard *) win_data->user;
    T_phbk         *Phbk        = data->phbk;
    char           temp[100];

    if (Phbk->UpdateAction!=ADD_FROM_IDLE&&Phbk->deletenumber!=1)//
  	{
  	    memset(Phbk->phbk->edt_buf_foradd,'\0',PHB_number_len+3);
        memset( Phbk->phbk->edt_buf_number, '\0', PHB_number_len+3 );
        Phbk->deletenumber=0;
  	}
    else
  	{
  		strncpy((char *) data->phbk->edt_buf_number, (char*)data->phbk->edt_buf_foradd, PHB_number_len );
  	}
    memset( Phbk->phbk->edt_buf_name, '\0',PHB_name_len+5);
    memset( Phbk->phbk->edt_buf_home, '\0',PHB_number_len+3);
    memset( Phbk->phbk->edt_buf_office, '\0',PHB_number_len+3);
    memset( Phbk->phbk->edt_buf_memo,'\0',PHB_memo_len+3);
    Phbk->UpdateAction = CREATE_ENTRY;
    Phbk->BookType   =  NVM_BOOK_SEL;
    Phbk->FirstNumber=   NUMBER;
       
    if ( NVMTotal<ALL_NVM_LIST )               
        Phbk->input_win =bookInputStart(win); 
    else
		bookShowInformation( win, TxtPhbkFull,NULL, NULL );  
    
    return MFW_EVENT_CONSUMED;
}

PROTECTED_MENU_HANDLER( bookNumberEnter )
{
  //recast the menu parameter as the current window -- MC
  T_MFW_HND     win           = (T_MFW_HND)Menu;
  T_MFW_WIN     *win_data   = ( (T_MFW_HDR *) win )->data;
  tBookStandard   *data         = (tBookStandard *) win_data->user;
  T_phbk         *Phbk        = data->phbk;
  char           temp[100];

  sprintf(temp,"Number win:%d",(int *)win);
  TRACE_EVENT(temp);

  TRACE_FUNCTION("booknumberEnter");   

  if (Phbk->UpdateAction!=ADD_FROM_IDLE&&Phbk->deletenumber!=1)//
  	{
  	memset(Phbk->phbk->edt_buf_foradd,'\0',PHB_number_len+3);
     memset( Phbk->phbk->edt_buf_number, '\0', PHB_number_len+3 );
     Phbk->deletenumber=0;
  	}
  else
  	{
  		strncpy((char *) data->phbk->edt_buf_number, (char*)data->phbk->edt_buf_foradd, PHB_number_len );
  	}
  memset( Phbk->phbk->edt_buf_name, '\0',PHB_name_len+5);
  memset( Phbk->phbk->edt_buf_home, '\0',PHB_number_len+3);
  memset( Phbk->phbk->edt_buf_office, '\0',PHB_number_len+3);
  memset( Phbk->phbk->edt_buf_memo,'\0',PHB_memo_len+3);
  Phbk->UpdateAction = CREATE_ENTRY;
  Phbk->BookType   =  NVM_BOOK_SEL;
  Phbk->FirstNumber=   NUMBER;
       
  if ( NVMTotal<ALL_NVM_LIST )               
    Phbk->input_win =bookInputStart(win); 
  else bookShowInformation( win, TxtPhbkFull,NULL, NULL );  
    
  return MFW_EVENT_CONSUMED;
}


PROTECTED_MENU_HANDLER( bookHomeEnter )
{  
  T_MFW_HND     win           = (T_MFW_HND)Menu;
  T_MFW_WIN     *win_data   = ( (T_MFW_HDR *) win )->data;
  tBookStandard   *data         = (tBookStandard *) win_data->user;
  T_phbk          *Phbk        = data->phbk;

  TRACE_FUNCTION("bookHomeEnter");   

  if (Phbk->UpdateAction!=ADD_FROM_IDLE&&Phbk->deletenumber!=1)//
  {
     memset( Phbk->phbk->edt_buf_number, '\0', PHB_number_len+3);
     memset(Phbk->phbk->edt_buf_foradd,'\0',PHB_number_len+3);
     memset( Phbk->phbk->edt_buf_home, '\0',PHB_number_len+3);
     Phbk->deletenumber=0;
  }   
  else
  {
     strncpy((char *) data->phbk->edt_buf_home, (char*)data->phbk->edt_buf_foradd, PHB_number_len );
     memset( Phbk->phbk->edt_buf_number, '\0', PHB_number_len);
  }
  memset( Phbk->phbk->edt_buf_name, '\0',PHB_name_len+5);  
  memset( Phbk->phbk->edt_buf_office, '\0',PHB_number_len+3);
  memset( Phbk->phbk->edt_buf_memo,'\0',PHB_memo_len+3);
  Phbk->UpdateAction = CREATE_ENTRY;
  Phbk->BookType   =  NVM_BOOK_SEL;
  Phbk->FirstNumber=   HOME;
     
  if ( NVMTotal<ALL_NVM_LIST )               
    Phbk->input_win =bookInputStart(win); 
  else bookShowInformation( win, TxtPhbkFull,NULL, NULL ); 
    
  return MFW_EVENT_CONSUMED;
}

PROTECTED_MENU_HANDLER( bookOfficeEnter )
{  
  T_MFW_HND       win           = (T_MFW_HND)Menu;
  T_MFW_WIN       *win_data   = ( (T_MFW_HDR *) win )->data;
  tBookStandard   *data         = (tBookStandard *) win_data->user;
  T_phbk          *Phbk        = data->phbk;

  TRACE_FUNCTION("bookOfficeEnter");   

   if (Phbk->UpdateAction!=ADD_FROM_IDLE&&Phbk->deletenumber!=1)//
  {
     memset( Phbk->phbk->edt_buf_number, '\0', PHB_number_len+3);
     memset(Phbk->phbk->edt_buf_foradd,'\0',PHB_number_len+3);
     memset( Phbk->phbk->edt_buf_office, '\0',PHB_number_len+3);
     Phbk->deletenumber=0;
  }   
  else
  {
     strncpy((char *) data->phbk->edt_buf_office, (char*)data->phbk->edt_buf_foradd, PHB_number_len );
     memset( Phbk->phbk->edt_buf_number, '\0', PHB_number_len);
  }	


  memset( Phbk->phbk->edt_buf_name, '\0',PHB_name_len+5);
  memset( Phbk->phbk->edt_buf_home, '\0',PHB_number_len+3);  
  memset( Phbk->phbk->edt_buf_memo,'\0',PHB_memo_len+3);
  Phbk->UpdateAction = CREATE_ENTRY;
  Phbk->BookType   =  NVM_BOOK_SEL;
  Phbk->FirstNumber =   OFFICE;
     
  if ( NVMTotal<ALL_NVM_LIST )               
    Phbk->input_win =bookInputStart(win); 
  else bookShowInformation( win, TxtPhbkFull,NULL, NULL ); 
    
  return MFW_EVENT_CONSUMED;
}
/*******************************************************************************

 $Function:     bookDeleteAll

 $Description:

    Delete name in phone book

 $Returns:    Refer Menu Handlers Definition Block Above

 $Arguments:  Refer Menu Handlers Definition Block Above

*******************************************************************************/

PROTECTED_MENU_HANDLER( bookDeleteAll )
{
  T_MFW_HND     win         = mfwParent( mfw_header() );
      
  return MFW_EVENT_CONSUMED;
}

PROTECTED_MENU_HANDLER( bookSimDeleteAll )
{
    T_MFW_HND     win         = mfwParent( mfw_header() );
    
    bookDeleteAllSIMRecord(win);
    
    return MFW_EVENT_CONSUMED;
}

PROTECTED_MENU_HANDLER( bookNvmDeleteAll )
{
    T_MFW_HND     win         = mfwParent( mfw_header() );
        
  /*  if (!CheckHostkey(PRO_DELALLPHB))    //xsf 2002.08.30
    {
      bookDeleteAllNVMRecord(win);             
    }*/

  CheckHostkey2(win, PRO_DELALLPHB);  //xsf 11.08
        
  return MFW_EVENT_CONSUMED;
}
/*******************************************************************************

 $Function:     bookNameSearch

 $Description:

    Search for name in phone book

 $Returns:    Refer Menu Handlers Definition Block Above

 $Arguments:  Refer Menu Handlers Definition Block Above

*******************************************************************************/

PROTECTED_MENU_HANDLER( bookNameSearch )
{
    T_MFW_HND     win         = mfwParent( mfw_header() );
    T_MFW_WIN     *win_data   = ( (T_MFW_HDR *) win )->data;
    tBookStandard   *data       = (tBookStandard *) win_data->user;

    data->phbk->current.KindOfSearch = SEARCH_BY_NAME;
    data->phbk->current.status.book = /*PHB_ADN;*/bookActiveBook(READ);
    data->phbk->search_win = bookSearchStart( data->phbk->win ); 

    return MFW_EVENT_CONSUMED;
}


PROTECTED_MENU_HANDLER( bookGroupingSetting )
{
    T_MFW_HND       win         = mfwParent( mfw_header() );
    T_MFW_WIN       *win_data   = ( (T_MFW_HDR *) win )->data;
    tBookStandard     *data       = (tBookStandard *) win_data->user;
    
    data->phbk->groupingsetting_win=bookGroupingStart(win);
   
    return MFW_EVENT_CONSUMED;
}

PROTECTED_MENU_HANDLER( bookShortCutDial )
{
    T_MFW_HND      win         = mfwParent( mfw_header() );
    T_MFW_WIN      *win_data   = ( (T_MFW_HDR *) win )->data;
    tBookStandard    *data       = (tBookStandard *) win_data->user;
            
    data->phbk->shortcutdial_win=bookShortCutDialStart(win);
    
    return MFW_EVENT_CONSUMED;
}
    
PROTECTED_MENU_HANDLER( bookCopyOne )
{ 
  T_MFW_HND       win         = (T_MFW_HND)Menu;
  T_MFW_WIN       *win_data   = ( (T_MFW_HDR *) win )->data;
  tBookStandard     *data       = (tBookStandard *) win_data->user;
  T_phbk           *Phbk       = data->phbk;
  char              temp[100];

  if (Phbk->BufferTable==GlobalTable)
  {
    if (l_phb_index[l_phb_sort_index[Phbk->BufferIndex+Phbk->phbk->select_index].index].phonebook==SIM_BOOK_SEL)  
      bookCopyRecord(win,TxtPhbkCopyMess);    
    else if (l_phb_index[l_phb_sort_index[Phbk->BufferIndex+Phbk->phbk->select_index].index].phonebook==NVM_BOOK_SEL)  	  
           bookCopyRecord(win,TxtPhbkCopyMess);  
  }
  else if (Phbk->BufferTable==SearchTable)
  	   {
         if (l_phb_index[bookname[Phbk->BufferIndex+Phbk->phbk->select_index].index].phonebook==SIM_BOOK_SEL)  
           bookCopyRecord(win,TxtPhbkCopyMess);    
         else if (l_phb_index[bookname[Phbk->BufferIndex+Phbk->phbk->select_index].index].phonebook==NVM_BOOK_SEL)  	  
                bookCopyRecord(win,TxtPhbkCopyMess);  
  	   }
        	    
  return MFW_EVENT_CONSUMED;
}

PROTECTED_MENU_HANDLER( bookDeleteOne )
{ 
  T_MFW_HND       win         = (T_MFW_HND)Menu;
  T_MFW_WIN       *win_data   = ( (T_MFW_HDR *) win )->data;
  tBookStandard     *data       = (tBookStandard *) win_data->user;
  T_phbk           *Phbk       = data->phbk;
  
   
  if (l_phb_index[l_phb_sort_index[Phbk->phbk->select_index].index].phonebook==SIM_BOOK_SEL)
  {                  
    bookDeleteRecord(win);  
  }
  else if (l_phb_index[l_phb_sort_index[Phbk->phbk->select_index].index].phonebook==NVM_BOOK_SEL)
       {
         data->phbk->current.status.book = bookActiveBook(READ);

	     if (data->phbk->current.status.book == PHB_ADN_FDN)
		   if (data->phbk->current.entry[ data->phbk->current.selectedName ].book==PHB_FDN)
			 bookActivateFDN(win);
		   else					
		     bookDeleteRecord(win);
	     else		     
	         bookDeleteRecord(win);
	         
       } 
  return MFW_EVENT_CONSUMED;   
}

PROTECTED_MENU_HANDLER( bookEditOne )
{
   T_MFW_HND      win         = (T_MFW_HND)Menu;
   T_MFW_WIN      *win_data   = ( (T_MFW_HDR *) win )->data;
   tBookStandard    *data       = (tBookStandard *) win_data->user;
   T_phbk          *Phbk       = data->phbk;

   T_MFW_PHB_ENTRY  l_phb_Shortentry;   
   T_MFW_PHB_LIST    l_phb_list;	/* phonebook list */   
   T_MFW ret;
   char   temp[100];
     
   Phbk->UpdateAction = MODIFY_EXISTING;
   data->phbk->current.status.book = bookActiveBook(WRITE);   

   memset( Phbk->phbk->edt_buf_name,'\0', PHB_name_len+5);
   memset( Phbk->phbk->edt_buf_number,'\0', PHB_number_len+3);
   memset( Phbk->phbk->edt_buf_home,'\0',PHB_number_len+3);
   memset( Phbk->phbk->edt_buf_office,'\0',PHB_number_len+3);
   memset( Phbk->phbk->edt_buf_memo,'\0',PHB_memo_len+3);
   
   switch ( Phbk->BufferTable )
   {
     case GlobalTable:
     {
          memcpy(Phbk->phbk->edt_buf_name,l_phb_index[l_phb_sort_index[Phbk->phbk->select_index].index].name,PHB_name_len);
           
	      if (l_phb_index[l_phb_sort_index[Phbk->phbk->select_index].index].phonebook==SIM_BOOK_SEL)
	      {
	   
		     memset(&l_phb_Shortentry,0,sizeof(l_phb_Shortentry));
		     memset(&l_phb_list,0,sizeof(l_phb_list));
		     l_phb_list.entry=&l_phb_Shortentry;
		     l_phb_list.num_entries=1;

		     ret=phb_read_entries(PHB_ADN,l_phb_index[l_phb_sort_index[Phbk->phbk->select_index].index].index,MFW_PHB_PHYSICAL,1,&l_phb_list);//l_phb_index[l_phb_sort_index[Phbk->phbk->select_index].index].index
		     if (ret==MFW_PHB_OK)        	       
		     {
		       memcpy(Phbk->phbk->edt_buf_number,l_phb_Shortentry.number,PHB_number_len);
		    
		       sprintf(temp,"Find name:%s",l_phb_Shortentry.name.data);           
		       TRACE_EVENT(temp);         
		     }                 
		     Phbk->BookType=SIM_BOOK_SEL;
		  }  
		  else if (l_phb_index[l_phb_sort_index[Phbk->phbk->select_index].index].phonebook==NVM_BOOK_SEL)
		   	   {
		   	      if (CurrentTable != l_phb_sort_index[Phbk->phbk->select_index].index/PHB_table_len)
		   	      {
		   	        if (flash_PBread(PBTable[l_phb_sort_index[Phbk->phbk->select_index].index/PHB_table_len])!=0)
		   	          CurrentTable=l_phb_sort_index[Phbk->phbk->select_index].index/PHB_table_len;   	                    
		   	      }
		          UnCompressNum((char *)FFS_PBData.NVMBook[l_phb_sort_index[Phbk->phbk->select_index].index%PHB_table_len].number,Phbk->phbk->edt_buf_number);                              
		          UnCompressNum((char *)FFS_PBData.NVMBook[l_phb_sort_index[Phbk->phbk->select_index].index%PHB_table_len].home,Phbk->phbk->edt_buf_home);
		          UnCompressNum((char *)FFS_PBData.NVMBook[l_phb_sort_index[Phbk->phbk->select_index].index%PHB_table_len].office,Phbk->phbk->edt_buf_office);
		          memcpy(Phbk->phbk->edt_buf_memo,(char *)FFS_PBData.NVMBook[l_phb_sort_index[Phbk->phbk->select_index].index%PHB_table_len].memo,PHB_memo_len+1);
		          switch (CheckFirstNumFlag(l_phb_sort_index[Phbk->phbk->select_index].index))
		          {
		            case 0x01:
		            	      Phbk->FirstNumber=NUMBER;
		            	      break;
		            case 0x02:
		            	      Phbk->FirstNumber=HOME;
		            	      break;
		            case 0x03:
		            	      Phbk->FirstNumber=OFFICE;
		            	      break;
		          }                       	
		          Phbk->BookType=NVM_BOOK_SEL;   	       
		   	   }

     }	
     break;
     case SearchTable:
     {
          memcpy(Phbk->phbk->edt_buf_name,l_phb_index[bookname[Phbk->phbk->select_index].index].name,PHB_name_len);
           
	      if (l_phb_index[bookname[Phbk->phbk->select_index].index].phonebook==SIM_BOOK_SEL)
	      {
	   
		     memset(&l_phb_Shortentry,0,sizeof(l_phb_Shortentry));
		     memset(&l_phb_list,0,sizeof(l_phb_list));
		     l_phb_list.entry=&l_phb_Shortentry;
		     l_phb_list.num_entries=1;

		     ret=phb_read_entries(PHB_ADN,l_phb_index[bookname[Phbk->phbk->select_index].index].index,MFW_PHB_PHYSICAL,1,&l_phb_list);//l_phb_index[bookname[Phbk->phbk->select_index].index].index
		     if (ret==MFW_PHB_OK)        	       
		     {
		       memcpy(Phbk->phbk->edt_buf_number,l_phb_Shortentry.number,PHB_number_len);
		    
		       sprintf(temp,"Find name:%s;Index:%d;number:%s",l_phb_Shortentry.name.data,l_phb_Shortentry.index,l_phb_Shortentry.number);           
		       TRACE_EVENT(temp);         
		     }                 
		     Phbk->BookType=SIM_BOOK_SEL;
		  }  
		  else if (l_phb_index[bookname[Phbk->phbk->select_index].index].phonebook==NVM_BOOK_SEL)
		   	   {
		   	      if (CurrentTable != bookname[Phbk->phbk->select_index].index/PHB_table_len)
		   	      {
		   	        if (flash_PBread(PBTable[bookname[Phbk->phbk->select_index].index/PHB_table_len])!=0)
		   	          CurrentTable=bookname[Phbk->phbk->select_index].index/PHB_table_len;   	                    
		   	      }
		          UnCompressNum((char *)FFS_PBData.NVMBook[bookname[Phbk->phbk->select_index].index%PHB_table_len].number,Phbk->phbk->edt_buf_number);                              
		          UnCompressNum((char *)FFS_PBData.NVMBook[bookname[Phbk->phbk->select_index].index%PHB_table_len].home,Phbk->phbk->edt_buf_home);
		          UnCompressNum((char *)FFS_PBData.NVMBook[bookname[Phbk->phbk->select_index].index%PHB_table_len].office,Phbk->phbk->edt_buf_office);
		          memcpy(Phbk->phbk->edt_buf_memo,(char *)FFS_PBData.NVMBook[bookname[Phbk->phbk->select_index].index%PHB_table_len].memo,PHB_memo_len+1);
		          switch (CheckFirstNumFlag(bookname[Phbk->phbk->select_index].index))
		          {
		            case 0x01:
		            	      Phbk->FirstNumber=NUMBER;
		            	      break;
		            case 0x02:
		            	      Phbk->FirstNumber=HOME;
		            	      break;
		            case 0x03:
		            	      Phbk->FirstNumber=OFFICE;
		            	      break;
		          }                       	
		          Phbk->BookType=NVM_BOOK_SEL;   	       
		   	   }            
     }	
     break;
     default: break;     	
   }     
   Phbk->input_win=bookInputStart(win);
   
   return MFW_EVENT_CONSUMED;
}


PROTECTED_MENU_HANDLER ( bookGroupingRename )
{

   T_MFW_HND      win         = (T_MFW_HND)Menu;
   T_MFW_WIN      *win_data   = ( (T_MFW_HDR *) win )->data;
   tBookStandard    *data       = (tBookStandard *) win_data->user;
   T_phbk          *Phbk       = data->phbk;
   char            temp[100];
   T_DISPLAY_DATA  display_info;

   TRACE_EVENT("xzy for rename restart");
   TraceInt((int)idle_data.win);

   sprintf(temp,"Phbk->phbk->select_index:%d; Phbk->select_index:%d",Phbk->phbk->select_index,
   	      Phbk->select_index);
   TRACE_EVENT(temp);


   memset(Phbk->edt_buf_rename,0,sizeof(Phbk->edt_buf_rename));
   memcpy(Phbk->edt_buf_rename,DataItem[Phbk->select_index].name,GroupingName_len); 
   sprintf(temp,"edt_buf_rename:%s,%s",DataItem[Phbk->select_index].name,
   	                             Phbk->edt_buf_rename);
   ////zgzTraceDataFunc("len of name",(unsigned short)wstrlen((char *)Phbk->edt_buf_rename));
   
   TRACE_EVENT(temp);
   
   if (Phbk->select_index == 9)
   	{
        dialog_info_init(&display_info);//gdy add    	
	    display_info.KeyEvents    = KEY_CLEAR;
    	display_info.TextId       = 0;
	    display_info.TextId2      = TxtRenameForbidden;
	    display_info.TextString   = '\0';
		display_info.TextString2  = '\0';
	   	display_info.LeftSoftKey  = TxtNull;
	    display_info.RightSoftKey = TxtNull;
	   	display_info.Time         =ONE_SECS;//
	    display_info.Callback     = NULL;
	    display_info.iconindex=REMIND_FAILURE;
	    info_dialog( win, &display_info );
   	}

   else
  	{
   Phbk->groupingRename_win=bookGroupingRenameStart(win,Phbk->edt_buf_rename); 

	}
   TRACE_EVENT("xzy for rename restart");

   TraceInt((int)idle_data.win);
   
   return MFW_EVENT_CONSUMED;
   
}

PROTECTED_MENU_HANDLER ( bookGroupingAdd )
{
  T_MFW_HND      win         = (T_MFW_HND)Menu;
  T_MFW_WIN      *win_data   = ( (T_MFW_HDR *) win )->data;
  tBookStandard    *data       = (tBookStandard *) win_data->user;
  T_phbk          *Phbk       = data->phbk;
  
  char             temp[100];

  sprintf(temp,"Phbk->phbk->select_index:%d; Phbk->select_index:%d",
  	          Phbk->phbk->select_index,
   	          Phbk->select_index);
  TRACE_EVENT(temp);
  
  Phbk->groupingadd_win= bookGroupingAddStart(win);     
  	
  return MFW_EVENT_CONSUMED;    
}

PROTECTED_MENU_HANDLER( bookGroupingIcon )
{





}

PROTECTED_MENU_HANDLER ( bookGroupingShow )
{
  T_MFW_HND     win         = (T_MFW_HND)Menu;
  T_MFW_WIN     *win_data   = ( (T_MFW_HDR *) win )->data;
  tBookStandard   *data       = (tBookStandard *) win_data->user;
  T_phbk          *Phbk       = data->phbk;

  char             temp[100];

  sprintf(temp,"Phbk->phbk->select_index:%d; Phbk->select_index:%d",
  	          Phbk->phbk->select_index,
   	          Phbk->select_index);
  TRACE_EVENT(temp);
 
  Phbk->groupingbrowser_win=bookGroupingBrowserStart(win);
  
  return MFW_EVENT_CONSUMED;
}

PROTECTED_MENU_HANDLER ( bookCopyAllSIMToNVM )
{
  T_MFW_HND     win         = (T_MFW_HND)Menu;
  bookCopyAllStart(win,SIMToNVM);
  return MFW_EVENT_CONSUMED;
}

PROTECTED_MENU_HANDLER ( bookCopyAllNVMToSIM )
{
  T_MFW_HND     win         = (T_MFW_HND)Menu;
  bookCopyAllStart(win,NVMToSIM);

  return MFW_EVENT_CONSUMED;
}

PROTECTED_MENU_HANDLER(bookChoiceCopySIMToNVM)
{
  T_MFW_HND     win         = (T_MFW_HND)Menu;
  T_MFW_WIN     *win_data   = ( (T_MFW_HDR *) win )->data;
  tBookStandard   *data       = (tBookStandard *) win_data->user;
  T_phbk         *Phbk       = data->phbk;

  if  (!SIMTotal)  
     bookShowInformation( win, TxtPhbkSimEmpty, NULL,NULL);     
  else if (NVMTotal>=ALL_NVM_LIST)
  	     bookShowInformation( win, TxtPhbkNvmFull,NULL,NULL);      
	   else	Phbk->choicecopy_win=bookChoiceCopyStart( win, SIMToNVM ); 	      
  return MFW_EVENT_CONSUMED;
}

PROTECTED_MENU_HANDLER(bookChoiceCopyNVMToSIM)
{
  T_MFW_HND     win         = (T_MFW_HND)Menu;
  T_MFW_WIN     *win_data   = ( (T_MFW_HDR *) win )->data;
  tBookStandard   *data       = (tBookStandard *) win_data->user;
  T_phbk         *Phbk       = data->phbk;
  char            temp[100];  

  T_MFW_PHB_STATUS  status;
     
  if ( !NVMTotal )
      bookShowInformation( win, TxtPhbkNvmEmpty, NULL,NULL);   
  else
  {  		  
	  status.book=PHB_ADN;   
	  if (phb_get_status(&status)==MFW_PHB_OK)
	  {	    	    
	    sprintf(temp,"avail:%d",status.avail_entries);
	    TRACE_EVENT(temp);
	    if (status.avail_entries==0)
	      bookShowInformation( win, TxtPhbkSimFull, NULL,NULL);	    
        else Phbk->choicecopy_win=bookChoiceCopyStart( win , NVMToSIM );             
      }      
	  else bookShowInformation( win, TxtReadError, NULL,NULL);
  }	  
  return MFW_EVENT_CONSUMED;
}

PROTECTED_MENU_HANDLER( bookOwnNumber )
{
  T_MFW_HND     win         = mfwParent( mfw_header() );
  T_MFW_WIN     *win_data   = ( (T_MFW_HDR *) win )->data;
  tBookStandard   *data       = (tBookStandard *) win_data->user;
  char            temp[100];
 
  phb_get_upn();
  data->phbk->upnList=GetOwnNumber();
  sprintf(temp,"count:%d",data->phbk->upnList->count);
  TRACE_EVENT(temp);
  if (data->phbk->upnList->count>0)
  {
    sprintf(temp,"number:%s",data->phbk->upnList->upn[0].number);
    TRACE_EVENT(temp);
    sprintf(temp,"number:%s",data->phbk->upnList->upn[data->phbk->upnList->count-1].number);
    TRACE_EVENT(temp);
  }
  data->phbk->OwnNumber_win=bookOwnNumberStart(win);
  return MFW_EVENT_CONSUMED;  
}

PROTECTED_MENU_HANDLER( bookRecVoice )
{



  return MFW_EVENT_CONSUMED; 
}




/*******************************************************************************

 $Function:     bookNameMessage

 $Description:

 Send message to the number selected

 $Returns:    Refer Menu Handlers Definition Block Above

 $Arguments:  Refer Menu Handlers Definition Block Above

*******************************************************************************/

PROTECTED_MENU_HANDLER( bookSendMessage )
{
    T_MFW_HND       win         = mfwParent( mfw_header() );
    T_MFW_WIN       *win_data   = ( (T_MFW_HDR *) win )->data;
    tBookStandard     *data       = (tBookStandard *) win_data->user;
    T_phbk           *Phbk       = data->phbk;
    T_SmsUserData    SmsData;

    T_MFW_PHB_ENTRY  l_phb_Shortentry;
    PhbkDetailItem      l_phb_NVMentry;    
    char               temp[100];

    TRACE_EVENT("send message from phbk");
    
    SmsData.Send_state= E_REPLY; 
    SmsData.called_from_reply_item=FALSE;
    // Copy the number and name information to the SMS buffer
    if (Phbk->BufferTable==GlobalTable)
    	{
    if (l_phb_index[l_phb_sort_index[Phbk->phbk->select_index].index].phonebook==SIM_BOOK_SEL)
    {
      if (ReadSIMRecord(l_phb_sort_index[Phbk->phbk->select_index].index,&l_phb_Shortentry)==TRUE)
      	memcpy(SmsData.NumberBuffer,l_phb_Shortentry.number,PHB_number_len);
    }
    else if (l_phb_index[l_phb_sort_index[Phbk->phbk->select_index].index].phonebook==NVM_BOOK_SEL)
    	 {
           if (ReadNVMRecord(l_phb_sort_index[Phbk->phbk->select_index].index,&l_phb_NVMentry)==TRUE)
           	 memcpy(SmsData.NumberBuffer,l_phb_NVMentry.number,PHB_number_len);           
    	 }
    	}
    else if (Phbk->BufferTable==SearchTable)
    	{
    		if (l_phb_index[bookname[Phbk->phbk->select_index].index].phonebook==SIM_BOOK_SEL)
    {
      if (ReadSIMRecord(bookname[Phbk->phbk->select_index].index,&l_phb_Shortentry)==TRUE)
      	memcpy(SmsData.NumberBuffer,l_phb_Shortentry.number,PHB_number_len);
    }
    else if (l_phb_index[bookname[Phbk->phbk->select_index].index].phonebook==NVM_BOOK_SEL)
    	 {
           if (ReadNVMRecord(bookname[Phbk->phbk->select_index].index,&l_phb_NVMentry)==TRUE)
           	//TRACE_EVENT((char *)l_phb_NVMentry.number);
           	 memcpy(SmsData.NumberBuffer,l_phb_NVMentry.number,PHB_number_len);           
    	 }
    	}
    // Create a dynamic menu window to perform the SMS sending 
    if(wstrlen((char *)SmsData.NumberBuffer)!=0)
    SmsSend_SEND_start( win, (void *) &SmsData );
	else
		return MFW_EVENT_REJECTED;

    return MFW_EVENT_CONSUMED;

    //  return bookNotImplemented( NULL );
}

/*******************************************************************************

 $Function:     bookRepRedNameEnter

 $Description:

    Enter name associated with Reply-Redial number

 $Returns:    Refer Menu Handlers Definition Block Above

 $Arguments:  Refer Menu Handlers Definition Block Above

*******************************************************************************/

PROTECTED_MENU_HANDLER( bookRepRedNameEnter )
{
    T_MFW_HND     win         = mfwParent( mfw_header() );
    T_MFW_WIN       *win_data   = ( (T_MFW_HDR *) win )->data;
    tBookStandard   *data       = (tBookStandard *) win_data->user;
    T_phbk          *Phbk       = data->phbk;
    
    return 1;
}


/*******************************************************************************

 $Function:     bookRepRedDelete

 $Description:

    Delete name associated with Reply-Redial Number

 $Returns:    Refer Menu Handlers Definition Block Above

 $Arguments:  Refer Menu Handlers Definition Block Above

*******************************************************************************/

PROTECTED_MENU_HANDLER( bookRepRedDelete )
{
    T_MFW_HND     win         = mfwParent( mfw_header() );
    T_MFW_WIN       *win_data   = ( (T_MFW_HDR *) win )->data;
    tBookStandard   *data       = (tBookStandard *) win_data->user;
    T_phbk          *Phbk       = data->phbk;

  deleteFromFile(Phbk,Phbk->current.status.book);
  bookShowInformation( Phbk->win, MmiBookCurrentText(),NULL, NULL );
  bookMenuDestroy(Phbk->menu_call_options_win);
  if(Phbk->current.status.used_entries > 0)
	  SEND_EVENT(Phbk->calls_list_win, SEARCH_SCROLL_UP, 0, 0 );
  SEND_EVENT(Phbk->calls_list_win, CALLS_LIST_INIT, 0, (void*)Phbk->current.status.book );
  return 1;
}


/*******************************************************************************

 $Function:     bookCallListDetails

 $Description:

 Retrieve details for call list

 $Returns:    Refer Menu Handlers Definition Block Above

 $Arguments:  Refer Menu Handlers Definition Block Above

*******************************************************************************/

PROTECTED_MENU_HANDLER( bookCallListDetails )
{
    T_MFW_HND     win         = mfwParent( mfw_header() );
    T_MFW_WIN       *win_data   = ( (T_MFW_HDR *) win )->data;
    tBookStandard   *data       = (tBookStandard *) win_data->user;
    T_phbk          *Phbk       = data->phbk;

    bookCallDetailsStart( Phbk->win);
    return 1;
}

/*******************************************************************************

 $Function:     bookServiceNumbers

 $Description:

 Get service numbers

 $Returns:    Refer Menu Handlers Definition Block Above

 $Arguments:  Refer Menu Handlers Definition Block Above

*******************************************************************************/
PROTECTED_MENU_HANDLER( bookServiceNumbers )
{
    T_MFW_HND     win         = mfwParent( mfw_header() );
    T_MFW_WIN       *win_data   = ( (T_MFW_HDR *) win )->data;
    tBookStandard   *data       = (tBookStandard *) win_data->user;

    data->phbk->current.KindOfSearch = SEARCH_BY_NAME;
    data->phbk->current.status.book = PHB_SDN;
    data->phbk->search_win = bookSearchStart( data->phbk->win );

    return MFW_EVENT_CONSUMED;
}


/*******************************************************************************

 $Function:     bookMemoryStatus

 $Description:

 Show the status of the phone book memory.

 $Returns:    Refer Menu Handlers Definition Block Above

 $Arguments:  Refer Menu Handlers Definition Block Above

*******************************************************************************/
PROTECTED_MENU_HANDLER( bookMemoryStatus )
{
    bookMemoryView();
    return MFW_EVENT_CONSUMED;
}


/*******************************************************************************

 $Function:     bookFDNActivate

 $Description:

   The FDN Activate routine is simply a wrapper for the real
   FDN Activate handler which resides in mmiPins.c

 $Returns:    Refer Menu Handlers Definition Block Above

 $Arguments:  Refer Menu Handlers Definition Block Above

*******************************************************************************/

PROTECTED_MENU_HANDLER( bookFDNActivate )
{
  return fdnActivate( Menu, Item );
}


/*******************************************************************************

 $Function:     bookFDNDeactivate

 $Description:

   The FDN Deactivate routine is simply a wrapper for the real
   FDN Deactivate handler which resides in mmiPins.c

 $Returns:    Refer Menu Handlers Definition Block Above

 $Arguments:  Refer Menu Handlers Definition Block Above

*******************************************************************************/

PROTECTED_MENU_HANDLER( bookFDNDeactivate )
{
  return fdnDeactivate( Menu, Item );
}

/*******************************************************************************

 $Function:     bookEnterLDN

 $Description:

    Enters de LDN List

 $Returns:    Refer Menu Handlers Definition Block Above

 $Arguments:  Refer Menu Handlers Definition Block Above

*******************************************************************************/

PROTECTED_MENU_HANDLER( bookEnterLDN )
{
    T_MFW_HND     win         = mfwParent( mfw_header() );

  bookPhonebookStart(win,PhbkRedialListNormal);
    return MFW_EVENT_CONSUMED;
}
/*******************************************************************************

 $Function:     bookEnterLRN

 $Description:

    Enters de LRN List

 $Returns:    Refer Menu Handlers Definition Block Above

 $Arguments:  Refer Menu Handlers Definition Block Above

*******************************************************************************/

PROTECTED_MENU_HANDLER( bookEnterLRN )
{
    T_MFW_HND     win         = mfwParent( mfw_header() );

  bookPhonebookStart(win,PhbkReplyListNormal);
    return MFW_EVENT_CONSUMED;
}
/*******************************************************************************

 $Function:     bookShowDetails

 $Description:

    show the details of current number

 $Returns:    Refer Menu Handlers Definition Block Above

 $Arguments:  Refer Menu Handlers Definition Block Above

*******************************************************************************/


PROTECTED_MENU_HANDLER( bookShowDetails )
{ 
  T_MFW_HND       win         = (T_MFW_HND)Menu;
  T_MFW_WIN       *win_data   = ( (T_MFW_HDR *) win )->data;
  tBookStandard     *data       = (tBookStandard *) win_data->user;
  T_phbk           *Phbk       = data->phbk;
  char              temp[100];

  bookShowDetailStart(win);
        	    
  return MFW_EVENT_CONSUMED;
}
/*******************************************************************************

 $Function:     bookEnterLMN

 $Description:

 Enters de LMN List

 $Returns:    Refer Menu Handlers Definition Block Above

 $Arguments:  Refer Menu Handlers Definition Block Above

*******************************************************************************/

PROTECTED_MENU_HANDLER( bookEnterLMN )
{
    T_MFW_HND     win         = mfwParent( mfw_header() );

    bookPhonebookStart(win,PhbkMissedCallsListNormal);
    return MFW_EVENT_CONSUMED;
}
//yq add for voice dial
PROTECTED_MENU_HANDLER( bookVoiceCall )
{
T_MFW_HND      win         = (T_MFW_HND)Menu;
   T_MFW_WIN      *win_data   = ( (T_MFW_HDR *) win )->data;
   tBookStandard    *data       = (tBookStandard *) win_data->user;
   T_phbk          *Phbk       = data->phbk;
   int modify_index;
   T_MFW_PHB_ENTRY  l_phb_Shortentry;   
   T_MFW_PHB_LIST    l_phb_list;	/* phonebook list */   
   T_MFW ret;
   
     
   Phbk->UpdateAction = MODIFY_EXISTING;
   data->phbk->current.status.book = bookActiveBook(WRITE);   

   memset( Phbk->phbk->edt_buf_name,'\0', PHB_name_len+5);
   memset( Phbk->phbk->edt_buf_number,'\0', PHB_number_len+3);
   memset( Phbk->phbk->edt_buf_home,'\0',PHB_number_len+3);
   memset( Phbk->phbk->edt_buf_office,'\0',PHB_number_len+3);
   memset( Phbk->phbk->edt_buf_memo,'\0',PHB_memo_len+3);
   //first we get the number an name of current selected index
   switch ( Phbk->BufferTable )
   {
     case GlobalTable:
     {
          memcpy(Phbk->phbk->edt_buf_name,l_phb_index[l_phb_sort_index[Phbk->phbk->select_index].index].name,PHB_name_len);
           
	      if (l_phb_index[l_phb_sort_index[Phbk->phbk->select_index].index].phonebook==SIM_BOOK_SEL)
	      {
	   
		     memset(&l_phb_Shortentry,0,sizeof(l_phb_Shortentry));
		     memset(&l_phb_list,0,sizeof(l_phb_list));
		     l_phb_list.entry=&l_phb_Shortentry;
		     l_phb_list.num_entries=1;

		     ret=phb_read_entries(PHB_ADN,l_phb_index[l_phb_sort_index[Phbk->phbk->select_index].index].index,MFW_PHB_PHYSICAL,1,&l_phb_list);//l_phb_index[l_phb_sort_index[Phbk->phbk->select_index].index].index
		     if (ret==MFW_PHB_OK)        	       
		     {
		       memcpy(Phbk->phbk->edt_buf_number,l_phb_Shortentry.number,PHB_number_len);
		    
		         
		     }                 
		   
		  }  
		  else if (l_phb_index[l_phb_sort_index[Phbk->phbk->select_index].index].phonebook==NVM_BOOK_SEL)
		   	   {
		   	      if (CurrentTable != l_phb_sort_index[Phbk->phbk->select_index].index/PHB_table_len)
		   	      {
		   	        if (flash_PBread(PBTable[l_phb_sort_index[Phbk->phbk->select_index].index/PHB_table_len])!=0)
		   	          CurrentTable=l_phb_sort_index[Phbk->phbk->select_index].index/PHB_table_len;   	                    
		   	      }
		          UnCompressNum((char *)FFS_PBData.NVMBook[l_phb_sort_index[Phbk->phbk->select_index].index%PHB_table_len].number,Phbk->phbk->edt_buf_number);                              
		          UnCompressNum((char *)FFS_PBData.NVMBook[l_phb_sort_index[Phbk->phbk->select_index].index%PHB_table_len].home,Phbk->phbk->edt_buf_home);
		          UnCompressNum((char *)FFS_PBData.NVMBook[l_phb_sort_index[Phbk->phbk->select_index].index%PHB_table_len].office,Phbk->phbk->edt_buf_office);
		         }

     }	
     break;
     case SearchTable:
     {
          memcpy(Phbk->phbk->edt_buf_name,l_phb_index[bookname[Phbk->phbk->select_index].index].name,PHB_name_len);
           
	      if (l_phb_index[bookname[Phbk->phbk->select_index].index].phonebook==SIM_BOOK_SEL)
	      {
	   
		     memset(&l_phb_Shortentry,0,sizeof(l_phb_Shortentry));
		     memset(&l_phb_list,0,sizeof(l_phb_list));
		     l_phb_list.entry=&l_phb_Shortentry;
		     l_phb_list.num_entries=1;
			ret=phb_read_entries(PHB_ADN,l_phb_index[bookname[Phbk->phbk->select_index].index].index,MFW_PHB_PHYSICAL,1,&l_phb_list);//l_phb_index[bookname[Phbk->phbk->select_index].index].index
		     if (ret==MFW_PHB_OK)        	       
		     {
		       memcpy(Phbk->phbk->edt_buf_number,l_phb_Shortentry.number,PHB_number_len);
		    
		       
		     }                 
		  
		  }  
		  else if (l_phb_index[bookname[Phbk->phbk->select_index].index].phonebook==NVM_BOOK_SEL)
		   	   {
		   	      if (CurrentTable != bookname[Phbk->phbk->select_index].index/PHB_table_len)
		   	      {
		   	        if (flash_PBread(PBTable[bookname[Phbk->phbk->select_index].index/PHB_table_len])!=0)
		   	          CurrentTable=bookname[Phbk->phbk->select_index].index/PHB_table_len;   	                    
		   	      }
		          UnCompressNum((char *)FFS_PBData.NVMBook[bookname[Phbk->phbk->select_index].index%PHB_table_len].number,Phbk->phbk->edt_buf_number);                              
		          UnCompressNum((char *)FFS_PBData.NVMBook[bookname[Phbk->phbk->select_index].index%PHB_table_len].home,Phbk->phbk->edt_buf_home);
		          UnCompressNum((char *)FFS_PBData.NVMBook[bookname[Phbk->phbk->select_index].index%PHB_table_len].office,Phbk->phbk->edt_buf_office);
		           
		   	   }            
     }	
     break;
     default: break;     	
   }     
 
//now we start the window to modify or add voice
 
if(strlen((char *)Phbk->phbk->edt_buf_number)!=0)
{
		modify_index=IsNumberIn((char *)Phbk->phbk->edt_buf_number);
		
		bookVoiceModifyStart(win,modify_index,(char *)Phbk->phbk->edt_buf_name,(char *)Phbk->phbk->edt_buf_number);
}
else if(strlen((char *)Phbk->edt_buf_office)!=0)
{
		modify_index=IsNumberIn((char *)Phbk->phbk->edt_buf_office);
		bookVoiceModifyStart(win,modify_index,(char *)Phbk->phbk->edt_buf_name,(char *)Phbk->phbk->edt_buf_office);
}
else if(strlen((char *)Phbk->phbk->edt_buf_home)!=0)
	{
		modify_index=IsNumberIn((char *)Phbk->phbk->edt_buf_home);
		bookVoiceModifyStart(win,modify_index,(char *)Phbk->phbk->edt_buf_name,(char *)Phbk->phbk->edt_buf_home);
}
  return MFW_EVENT_CONSUMED; 
}


PROTECTED_MENU_HANDLER( bookVoiceList )
{

T_MFW_HND     win         = mfwParent( mfw_header() );
  T_MFW_WIN     *win_data   = ( (T_MFW_HDR *) win )->data;
  tBookStandard   *data       = (tBookStandard *) win_data->user;
 TRACE_EVENT("bookvoiceList");
  data->phbk->search_win = bookVoiceListStart(data->phbk->win);
  return MFW_EVENT_CONSUMED;  	
 
}

PROTECTED_MENU_HANDLER( bookVoiceModify )
{
T_MFW_HND      win         = (T_MFW_HND)Menu;
   T_MFW_WIN      *win_data   = ( (T_MFW_HDR *) win )->data;
   tBookStandard    *data       = (tBookStandard *) win_data->user;
   T_phbk          *Phbk       = data->phbk;
   
 
if(wstrlen(voice_dial[voice_index[Phbk->phbk->select_index]].number)!=0
	&&wstrlen(voice_dial[voice_index[Phbk->phbk->select_index]].name)!=0)
	bookVoiceModifyStart(win,voice_index[Phbk->phbk->select_index]+1,
	voice_dial[voice_index[Phbk->phbk->select_index]].name,voice_dial[voice_index[Phbk->phbk->select_index]].number);
	
  
  return MFW_EVENT_CONSUMED; 
}

PROTECTED_MENU_HANDLER( bookVoiceDelete )
{
T_MFW_HND      win         = (T_MFW_HND)Menu;
   T_MFW_WIN      *win_data   = ( (T_MFW_HDR *) win )->data;
   tBookStandard    *data       = (tBookStandard *) win_data->user;
   T_phbk          *Phbk       = data->phbk;
   char delete_file[20];
 if(wstrlen(voice_dial[voice_index[Phbk->phbk->select_index]].number)!=0
	&&wstrlen(voice_dial[voice_index[Phbk->phbk->select_index]].name)!=0)
 	{
		bookDeleteRecordStart(win);
 		
 	}

  return MFW_EVENT_CONSUMED; 
}

PROTECTED_MENU_HANDLER( bookVoiceCalling )
{
T_MFW_HND      win         = (T_MFW_HND)Menu;
   T_MFW_WIN      *win_data   = ( (T_MFW_HDR *) win )->data;
   tBookStandard    *data       = (tBookStandard *) win_data->user;
   T_phbk          *Phbk       = data->phbk;
   
 
if(wstrlen(voice_dial[voice_index[Phbk->phbk->select_index]].number)!=0
	&&wstrlen(voice_dial[voice_index[Phbk->phbk->select_index]].name)!=0)
	callNumber((UBYTE *)voice_dial[voice_index[Phbk->phbk->select_index]].number);
	
  return MFW_EVENT_CONSUMED; 
}

PROTECTED_MENU_HANDLER( bookVoicePlay )
{
T_MFW_HND      win         = (T_MFW_HND)Menu;
   T_MFW_WIN      *win_data   = ( (T_MFW_HDR *) win )->data;
   tBookStandard    *data       = (tBookStandard *) win_data->user;
   T_phbk          *Phbk       = data->phbk;
   char play_file[20];
 
if(wstrlen(voice_dial[voice_index[Phbk->phbk->select_index]].number)!=0
	&&wstrlen(voice_dial[voice_index[Phbk->phbk->select_index]].name)!=0)
{
	wstrcpy(play_file,"/voiceD/");
 	wstrcat(play_file,(char *)voice_filename[voice_index[Phbk->phbk->select_index]]);
 	StartPlayVmRecord((callback_func)PlayVmRecord_cb,(char *)play_file);
}
  return MFW_EVENT_CONSUMED; 
}
//end yq added
//begin xzy add 2002/10/28 for phonebook grouping icon
#if 0
/*******************************************************************************

 $Function:     SetIconClassmate

 $Description:

 

 $Returns:    

 $Arguments:  

*******************************************************************************/
/*
PROTECTED_MENU_HANDLER (SetIconClassmate)
{

   T_MFW_HND      win         = mfwParent( mfw_header() );
 
    
   bookGroupingIconSet(win, classmate);

   return MFW_EVENT_CONSUMED;

}
*/
/*******************************************************************************

 $Function:     SetIconFriend

 $Description:

 

 $Returns:    

 $Arguments:  

*******************************************************************************/
/*
PROTECTED_MENU_HANDLER (SetIconFriend)
{

   T_MFW_HND      win         = mfwParent( mfw_header() );
 
   bookGroupingIconSet(win, friends);

   return MFW_EVENT_CONSUMED;

}
*/
/*******************************************************************************

 $Function:     SetIconFamily

 $Description:

 

 $Returns:    

 $Arguments:  

*******************************************************************************/
/*
PROTECTED_MENU_HANDLER (SetIconFamily)
{

   T_MFW_HND      win         = mfwParent( mfw_header() );
 
   bookGroupingIconSet(win, family);

   return MFW_EVENT_CONSUMED;

}
*/
/*******************************************************************************

 $Function:     SetIconWorkmate

 $Description:

 

 $Returns:    

 $Arguments:  

*******************************************************************************/
/*
PROTECTED_MENU_HANDLER (SetIconWorkmate)
{

   T_MFW_HND      win         = mfwParent( mfw_header() );
     
   bookGroupingIconSet(win, workmate);

   return MFW_EVENT_CONSUMED;

}
*/
#endif

PROTECTED_MENU_HANDLER(SetGroupingIcon)
{
    T_MFW_HND     parent_win   = mfwParent( mfw_header() );
    T_MFW_HND     iconwin;	/* 2004/04/22 sunsj modify */

    T_MFW_WIN      *win_data   = ( (T_MFW_HDR *) parent_win )->data;

    tBookStandard    *data       = (tBookStandard *) win_data->user;
    T_phbk          *Phbk       = data->phbk;
    T_DISPLAY_DATA  display_info;
    
    
    if (Phbk->select_index == 9)
    	{
        dialog_info_init(&display_info);//gdy add    	
	    display_info.KeyEvents    = KEY_CLEAR;
    	display_info.TextId       = 0;
	    display_info.TextId2      = TxtChangeIconForbidden;	/* 2004/04/16 sunsj modify */
	    display_info.TextString   = '\0';
		display_info.TextString2  = '\0';
	   	display_info.LeftSoftKey  = TxtNull;
	    display_info.RightSoftKey = TxtNull;
	   	display_info.Time         =ONE_SECS;
	    display_info.Callback     = NULL;
	    display_info.iconindex=REMIND_FAILURE;
	    info_dialog( parent_win, &display_info );
    	}
   
   else
  	{
	   /* 2004/04/22 sunsj modify */
	   iconwin = iconSelect_create(parent_win);
   		 if(iconwin NEQ NULL)
    	 {
    		SEND_EVENT(iconwin, ICONSELECT_INIT, 0, 0);
    	 }
   	}
    return MFW_EVENT_CONSUMED;

}
//end xzy add 2002/10/28 for phonebook grouping icon

/*******************************************************************************

                                End of File

*******************************************************************************/
