/*******************************************************************************

                    CONDAT (UK)

********************************************************************************                                                                              

 This software product is the property of Condat (UK) Ltd and may not be
 disclosed to any third party without the express permission of the owner.                                 
                                                                              
********************************************************************************

 $Project name:	Basic MMI                                                      
 $Project code:	BMI (6349)                                                           
 $Module:		PhoneBook
 $File:		    MmiBookInputWindow.h
 $Revision:		1.0                                                       
                                                                              
 $Author:		Condat(UK)                                                         
 $Date:		    25/10/00                                                      
                                                                               
********************************************************************************
                                                                              
 Description:

    This module provides the functionality for the input window of the
    phone book module.
                        
********************************************************************************
 $History: MmiBookInputWindow.h

    25/10/00			Original Condat(UK) BMI version.	
    20/08/02     ver0.101  fzq modified  
 $End

*******************************************************************************/

/*******************************************************************************
                                                                              
                                Include Files
                                                                              
*******************************************************************************/
#define ENTITY_MFW

/* includes */
#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#if defined (NEW_FRAME)

    #include "typedefs.h"
    #include "vsi.h"
    #include "pei.h"
    #include "custom.h"
    #include "gsm.h"

#else

    #include "stddefs.h"
    #include "custom.h"
    #include "gsm.h"
    #include "vsi.h"

#endif

#include "mfw_sys.h"

#include "mfw_mfw.h"
#include "mfw_win.h"
#include "mfw_kbd.h"
#include "mfw_lng.h"
#include "mfw_edt.h"
#include "mfw_icn.h"
#include "mfw_mnu.h"
#include "mfw_tim.h"

#include "mfw_sim.h"
#include "mfw_cm.h"
#include "mfw_nm.h"
#include "mfw_phb.h"
#include "mfw_mme.h"
#include "mfw_sat.h"
#include "mfw_sms.h"

#include "dspl.h"

#include "ksd.h"
#include "psa.h"

#include "MmiMain.h"
#include "MmiBookController.h"
#include "MmiDummy.h"
#include "MmiDialogs.h"
#include "MmiLists.h"

#include "MmiMenu.h"
#include "MmiCall.h"
#include "MmiIcons.h"
#include "MmiIdle.h"

#include "MmiSoftKeys.h"
#include "MmiSounds.h"
#include "MmiIdle.h"
#include "MmiNetwork.h"
#include "MmiSat_i.h"
#include "MmiAoc.h"

#include "gdi.h"
#include "audio.h"

#include "cus_aci.h"
#include "p_sim.h"
#include "mfw_ffs.h"
#include "MmiTimers.h"
#include "mmiusersymbolinput.h"

#include "MmiBookShared.h"
#ifdef EASY_TEXT_ENABLED
    #include "mmitextinput.h"		/* 2003/12/04 sunsj modify */
#endif

extern int         upCase;

/*******************************************************************************
                                                                              
                                Private Methods
                                                                              
*******************************************************************************/

/*******************************************************************************

 $Function:		bookInputDialog

 $Description:	Dialog function for the input window
 
 $Returns:		None

 $Arguments:	win, window handle
                event, event to be handled
                value, not used
                parameter, not used
 
*******************************************************************************/

static void bookInputDialog( T_MFW_HND win, USHORT event, SHORT value, void *parameter )
{
    T_MFW_WIN       *win_data   = ((T_MFW_HDR *) win)->data;
    tBookStandard   *data       = (tBookStandard *) win_data->user;    
    T_EDITOR_DATA  *editor_data = (T_EDITOR_DATA *) parameter;

    TRACE_FUNCTION ("bookInputDialog()");

    switch( event )
    {
    case INPUT_INIT:
        {
            // Initialise the input window            				   
            switch( data->phbk->MODE )
            {
            case NUMBER:
            case OFFICE:
            case HOME:
                {
                    editor_attr_init_fullscr(&data->editor_data.editor_attr,1,editor_data->TextId,NULL,NUMBER_LENGTH,edtCurBar1);  
                } 
                break;
            case NAME:
                {
                    editor_attr_init_fullscr(&data->editor_data.editor_attr,1,editor_data->TextId,NULL,PHB_MAX_LEN,edtCurBar1); 
                } 
                break;    
            case MEMO:    
                {
                    editor_attr_init_fullscr(&data->editor_data.editor_attr,1,editor_data->TextId,NULL,PHB_memo_len,edtCurBar1); 
                }                 
                break;

            }

            data->editor_data.hide         = editor_data->hide;
            data->editor_data.min_enter    = editor_data->min_enter;
            data->editor_data.Identifier     = editor_data->Identifier;
            data->editor_data.timeout      = editor_data->timeout;
            data->editor_data.destroyEditor = editor_data->destroyEditor;
            data->editor_data.editor_attr.text=editor_data->editor_attr.text;
            /* 2003/12/04 sunsj remove textid */
            //data->editor_data.TextId           = editor_data->TextId;
            data->editor_data.TextId           = '\0';
            data->editor_data.LeftSoftKey   =editor_data->LeftSoftKey;
            data->editor_data.RightSoftKey  =editor_data->RightSoftKey;
            data->editor_data.mode        =editor_data->mode;
            data->editor_data.Callback      =editor_data->Callback;                     
            /* 20030/12/04 sunsj modify for new editor mode */
            if( data->editor_data.mode == TEXTINPUT_MODE )
            {
                data->editor_data.InputMode    = PINYIN_INPUT_MODE;     /* 2003/12/04 sunsj */
                data->edt                      = text_input(win, &data->editor_data);
            }
            else
            {
                data->edt                   =editor_start(win, &data->editor_data);  /* start the editor */ 
            }                                           

            winShow( data->win );
        }
        break;

    case INPUT_DESTROY:
        {               
            bookInputDestroy( win );
        }
        break;
    }
}

/*******************************************************************************

 $Function:		bookInputWindowCB

 $Description:	Window event handler
 
 $Returns:		MFW_CONSUMED for the visible event, MFW_PASSED otherwise

 $Arguments:	e, event, w, window handle
 
*******************************************************************************/

static int bookInputWindowCB( MfwEvt e, MfwWin *w )
{
    tBookStandard *data = (tBookStandard *) w->user;

    TRACE_FUNCTION ("bookInputWindowCB()");

    // Handle the visible event, otherwise return MFW_EVENT_PASSED	
    switch( e )
    {
    case MfwWinVisible:
        {

        }
        break;

    default:
        {
            // unabel to handle event, pass handling of event back up tree			
            return MFW_EVENT_PASSED;
        }
    }

    // if we get here, we have handled the event	
    return MFW_EVENT_CONSUMED;
}

/*******************************************************************************

 $Function:		bookInputCreate

 $Description:	Create the input window and asociate it's handlers
 
 $Returns:		handle of new window, or NULL if failure

 $Arguments:	parent, handle of the parent window
 
*******************************************************************************/

static T_MFW_HND bookInputCreate( MfwHnd parent )
{
    T_MFW_WIN       *parent_win_data    = ( (T_MFW_HDR *) parent )->data;
    tBookStandard    *parent_data        = (tBookStandard *) parent_win_data->user;
    T_phbk          *phbk               = parent_data->phbk;
    T_MFW_WIN       *win_data;
    tBookStandard   *data;

    TRACE_FUNCTION ("bookInputCreate()");

    // allocate memory for our control block	
    if( ( data = (tBookStandard *) ALLOC_MEMORY( sizeof( tBookStandard ) ) ) == NULL )
        return NULL;

    // Create the window if we can	
    if( ( data->win = win_create( parent, 0, E_WIN_VISIBLE, (T_MFW_CB) bookInputWindowCB ) ) == NULL )
    {
        FREE_MEMORY( (void *)data, sizeof( tBookStandard ) );
        return NULL;
    }

    data->mmi_control.dialog    = (T_DIALOG_FUNC) bookInputDialog;
    data->mmi_control.data      = data;
    win_data                    = ((T_MFW_HDR *)data->win)->data;
    win_data->user              = (void *) data;
    data->phbk                  = phbk;
    data->parent_win            = parent;

    return data->win;
}

/*******************************************************************************
                                                                              
                                Public Methods
                                                                              
*******************************************************************************/

/*******************************************************************************

 $Function:		bookInputStart

 $Description:	Start the input handler
 
 $Returns:		Handle of newly created window, or NULL if failure

 $Arguments:	parent, parent window handle, editor_data, input window property
                control block.
 
*******************************************************************************/

MfwHnd bookInputStart( MfwHnd parent,T_EDITOR_DATA  *editor_data )
{
    T_MFW_HND win;

    TRACE_FUNCTION ("bookInputStart()");

    if( ( win = bookInputCreate( parent ) ) != NULL )
        SEND_EVENT( win, INPUT_INIT, 0, editor_data );
    return win;
}

/*******************************************************************************

 $Function:		bookInputStartNumberEditor

 $Description:	Start the input handler using default editor_data
 
 $Returns:		Handle of newly created window, or NULL if failure

 $Arguments:	parent, parent window handle, buffer, input buffer
 
*******************************************************************************/

MfwHnd bookInputStartNumberEditor( MfwHnd parent, void *buffer )
{  
    T_MFW_WIN      *win_data   = ( (T_MFW_HDR *) parent)->data;
    tBookStandard    *data       = (tBookStandard *) win_data->user;
    T_phbk          *Phbk       = data->phbk;
    T_EDITOR_DATA  DefaultParameters;

    TRACE_FUNCTION ("bookInputStartNumberEditor()");

    Phbk->MODE = NUMBER;

    DefaultParameters.hide=FALSE;
    DefaultParameters.min_enter           =0;   
    DefaultParameters.Identifier            =0xFFFF;
    DefaultParameters.timeout           = FOREVER;  // Avoid returning empty strings
    DefaultParameters.destroyEditor     = FALSE; 
    DefaultParameters.editor_attr.text  = (char *)buffer;
    DefaultParameters.TextId               = TxtEnterNumber;                                             
    DefaultParameters.LeftSoftKey       = TxtSoftOK;
    DefaultParameters.RightSoftKey      = TxtCancel;        
    DefaultParameters.mode              = PHONENUMONLY_MODE;// PARADISE_MODE;
    DefaultParameters.Callback          = (T_EDIT_CB)bookAddNumberCB;

    // allocate window, and if successful initialise it	
    return bookInputStart( parent, &DefaultParameters );
}


MfwHnd bookInputStartOfficeEditor( MfwHnd parent, void *buffer )
{
    T_MFW_HND       win         = parent;
    T_MFW_WIN       *win_data   = ( (T_MFW_HDR *) win )->data;
    tBookStandard   *data       = (tBookStandard *) win_data->user;
    T_phbk          *Phbk       = data->phbk;
    T_EDITOR_DATA  DefaultParameters;

    TRACE_FUNCTION ("bookInputStartNumberEditor()");

    Phbk->MODE = OFFICE;

    DefaultParameters.hide=FALSE;
    DefaultParameters.min_enter           =0;   
    DefaultParameters.Identifier            =0xFFFF;
    DefaultParameters.timeout           = FOREVER;  // Avoid returning empty strings
    DefaultParameters.destroyEditor     = FALSE; 
    DefaultParameters.editor_attr.text  = (char *)buffer;
    DefaultParameters.TextId               = TxtEnterOffice;                                             
    DefaultParameters.LeftSoftKey       = TxtSoftOK;
    DefaultParameters.RightSoftKey      = TxtCancel;        
    DefaultParameters.mode              = PHONENUMONLY_MODE;// PARADISE_MODE;
    DefaultParameters.Callback          = (T_EDIT_CB)bookAddNumberCB;

    // allocate window, and if successful initialise it	
    return bookInputStart( parent, &DefaultParameters );
}

MfwHnd bookInputStartHomeEditor( MfwHnd parent, void *buffer )
{
    T_MFW_HND       win         = parent;
    T_MFW_WIN       *win_data   = ( (T_MFW_HDR *) win )->data;
    tBookStandard   *data       = (tBookStandard *) win_data->user;
    T_phbk          *Phbk       = data->phbk;
    T_EDITOR_DATA  DefaultParameters;

    TRACE_FUNCTION ("bookInputStartNumberEditor()");

    Phbk->MODE = HOME;

    DefaultParameters.hide=FALSE;
    DefaultParameters.min_enter           =0;   
    DefaultParameters.Identifier            =0xFFFF;
    DefaultParameters.timeout           = FOREVER;  // Avoid returning empty strings
    DefaultParameters.destroyEditor     = FALSE; 
    DefaultParameters.editor_attr.text  = (char *)buffer;
    DefaultParameters.TextId               = TxtEnterHome;                                               
    DefaultParameters.LeftSoftKey       = TxtSoftOK;
    DefaultParameters.RightSoftKey      = TxtCancel;        
    DefaultParameters.mode              = PHONENUMONLY_MODE;// PARADISE_MODE;
    DefaultParameters.Callback          = (T_EDIT_CB)bookAddNumberCB;

    // allocate window, and if successful initialise it	
    return bookInputStart( parent, &DefaultParameters );
}

/*******************************************************************************

 $Function:		bookInputStartNameEditor

 $Description:	Start the input handler using default editor_data
 
 $Returns:		Handle of newly created window, or NULL if failure

 $Arguments:	parent, parent window handle, buffer, input buffer
 
*******************************************************************************/

MfwHnd bookInputStartNameEditor( MfwHnd parent, void *buffer )
{
    T_MFW_WIN       *win_data   = ( (T_MFW_HDR *) parent )->data;
    tBookStandard     *data       = (tBookStandard *) win_data->user;
    T_phbk           *Phbk       = data->phbk;
    T_EDITOR_DATA   DefaultParameters;

    TRACE_FUNCTION ("bookInputStartNameEditor()");

    Phbk->MODE=NAME;

    DefaultParameters.hide                 =FALSE;
    DefaultParameters.min_enter            =1;  
    DefaultParameters.Identifier             =0xFFFF;
    DefaultParameters.timeout           = FOREVER;  // Avoid returning empty strings
    DefaultParameters.destroyEditor     = FALSE; 
    DefaultParameters.editor_attr.text  = (char *)buffer;
    DefaultParameters.TextId               = TxtEnterName;                                               
    DefaultParameters.LeftSoftKey       = TxtSoftOK;
    DefaultParameters.RightSoftKey      = TxtCancel;        
    DefaultParameters.mode              = SHUZI_MODE;// PARADISE_MODE;
    DefaultParameters.Callback          = (T_EDIT_CB)bookAddNameCB;
    // allocate window, and if successful initialise it		
    return bookInputStart( parent, &DefaultParameters );
}

MfwHnd bookInputStartMemoEditor( MfwHnd parent, void *buffer )
{
    T_MFW_WIN       *win_data   = ( (T_MFW_HDR *) parent )->data;
    tBookStandard     *data       = (tBookStandard *) win_data->user;
    T_phbk           *Phbk       = data->phbk;
    T_EDITOR_DATA   DefaultParameters;

    TRACE_FUNCTION ("bookInputStartNameEditor()");

    Phbk->MODE=MEMO;

    DefaultParameters.hide                 =FALSE;
    DefaultParameters.min_enter            =0;  
    DefaultParameters.Identifier             =0xFFFF;
    DefaultParameters.timeout           = FOREVER;  // Avoid returning empty strings
    DefaultParameters.destroyEditor     = FALSE; 
    DefaultParameters.editor_attr.text  = (char *)buffer;
    DefaultParameters.TextId               = TxtEnterMemo;                                               
    DefaultParameters.LeftSoftKey       = TxtSoftOK;
    DefaultParameters.RightSoftKey      = TxtCancel;        
    DefaultParameters.mode              = SHUZI_MODE;// PARADISE_MODE;
    DefaultParameters.Callback          = (T_EDIT_CB)bookAddNumberCB;
    // allocate window, and if successful initialise it		
    return bookInputStart( parent, &DefaultParameters );
}



/*******************************************************************************

 $Function:		bookInputDestroy

 $Description:	Destroy Input Window
 
 $Returns:		None

 $Arguments:	Handle of window to be destroyed
 
*******************************************************************************/

void bookInputDestroy( MfwHnd window )
{
    T_MFW_WIN     *win  = ((T_MFW_HDR *) window)->data;
    tBookStandard *data = (tBookStandard *) win->user;

    TRACE_FUNCTION ("bookInputDestroy()");

    if( data )
    {
        //editDeactivate();
        /* 2003/12/04 sunsj modify for new editor mode */
        if( data->editor_data.mode == TEXTINPUT_MODE )
        {
            text_input_destroy(data->edt);
        }
        else
        {
            editor_destroy(data->edt);
        }

        winDelete ( data->win );
        FREE_MEMORY( (void *) data, sizeof( tBookStandard ) );
    }
}
/*******************************************************************************
                                                                              
                                End of File
                                                                              
*******************************************************************************/
