
/*******************************************************************************

					CONDAT (UK)

********************************************************************************                                                                              

 This software product is the property of Condat (UK) Ltd and may not be
 disclosed to any third party without the express permission of the owner.                                 
                                                                              
********************************************************************************

 $Project name:	Basic MMI                                                      
 $Project code:	BMI (6349)                                                           
 $Module:		MMI
 $File:		    MmiExtras.c
 $Revision:		1.0                                                       
                                                                              
 $Author:		Condat(UK)                                                         
 $Date:		    25/10/00                                                      
                                                                               
********************************************************************************
                                                                              
 Description:

    MMI extras handling
                        
********************************************************************************
 $History: MmiExtras.c

	25/10/00			Original Condat(UK) BMI version.	
	   
 $End

*******************************************************************************/




/*******************************************************************************
                                                                              
                                Include Files
                                                                              
*******************************************************************************/
#define ENTITY_MFW

/* includes */
#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#if defined (NEW_FRAME)

#include "typedefs.h"
#include "vsi.h"
#include "pei.h"
#include "custom.h"
#include "gsm.h"

#else

#include "stddefs.h"
#include "custom.h"
#include "gsm.h"
#include "vsi.h"

#endif
#include "nucleus.h"

#include "mfw_sys.h"


#include "mfw_mfw.h"
#include "mfw_win.h"
#include "mfw_kbd.h"
#include "mfw_tim.h"
#include "mfw_edt.h"
#include "mfw_lng.h"
#include "mfw_icn.h"
#include "mfw_mnu.h"

#include "mfw_sim.h"
#include "mfw_nm.h"
#include "mfw_phb.h"
#include "ksd.h"
#include "psa.h"
#include "mfw_sat.h"
#include "mfw_sms.h"

#include "dspl.h"
#include "MmiBookShared.h"
#include "MmiMmi.h"
// #include "MmiLang.h"
#include "MmiMain.h"
#include "MmiStart.h"
#include "MmiPins.h"

#include "MmiDummy.h"
#include "MmiDialogs.h"
#include "MmiLists.h"


#include "MmiMenu.h"
/* kk 011030 file does not exist #include "MmiExtras.h"*/


static int WinEvent  ( MfwEvt e, MfwWin *w );
static int KeyEvent  ( MfwEvt e, MfwKbd *kc );
static int ConfirmMesgTimerEvent( MfwEvt e, MfwTim *tc );
static int TimerEvent( MfwEvt e, MfwTim *tc );

static void extrasInitGame ( void );


static MfwHnd winExtras;              /* our window handler       */
static MfwHnd kbdExtras;              /* our keyboard handler     */
static MfwHnd timerExtras1;           /* our timer handler        */
static MfwHnd timerExtras2;           /* confirm message timer handler */
static MfwHnd oldFocus;               /* store old focus          */

static int extrasSelection;
static int extrasFlipLevel;
static int extrasC4Level;
static int extrasC4Mode;
static int extrasEntryScreen = EXTRAS_MAIN;

//static MmiState nextState;           /* next state when finished */



/*******************************************************************************

 $Function:    	extrasInit

 $Description:	init extras handling
 
 $Returns:		None

 $Arguments:	None
 
*******************************************************************************/
void extrasInit ( void )
{
    winExtras 	   = winCreate( NULL, NULL, MfwWinVisible, (MfwCb)WinEvent );
    winShow(winExtras);
    kbdExtras  	   = kbdCreate( winExtras,  KEY_ALL,       (MfwCb)KeyEvent );
    timerExtras1   = timCreate( winExtras,  GAMES_TIMER,   (MfwCb)TimerEvent );
    timerExtras2   = timCreate( winExtras,  CONFIRMATION_TIMER, (MfwCb)ConfirmMesgTimerEvent );

// initialise the games configurables

    extrasSelection = EXTRAS_MIN;
}


/*******************************************************************************

 $Function:    	extrasExit

 $Description:	exit extras handling
 
 $Returns:		None

 $Arguments:	None
 
*******************************************************************************/
void extrasExit ( void )
{
	kbdDelete( kbdExtras );
	kbdExtras   = 0;

	timStop( timerExtras1 );

    timDelete( timerExtras1 );
    timerExtras1 = 0;

	timStop( timerExtras2 );

    timDelete( timerExtras2 );
    timerExtras2 = 0;


    winDelete( winExtras );
    winExtras   = 0;
}


/*******************************************************************************

 $Function:    	WinEvent

 $Description:	window event handler
 
 $Returns:		Status int

 $Arguments:	Window handler event, win
 
*******************************************************************************/
static int WinEvent ( MfwEvt e, MfwWin *w )
{
    switch ( e )
    {
        case MfwWinVisible:
            break;

        default:
            return 0;
    }

    return 1;
}


/*******************************************************************************

 $Function:    	popMenu

 $Description:	Routine to handle reversion to games menu on confirmation
            timeout or 'C' key press.
 
 $Returns:		None

 $Arguments:	None
 
*******************************************************************************/
static void popMenu( void )
{
    menuPop(1);

/* TO BE SORTED OUT !!!!
	showMenuSoftkeys();
END TO BE SORTED OU */
}



/*******************************************************************************

 $Function:    	keyEvent

 $Description:	keyboard event handler
 
 $Returns:		Status int

 $Arguments:	Window handler event, keyboard control block
 
*******************************************************************************/
static int KeyEvent ( MfwEvt e, MfwKbd *k )
{
	char kp = 0;

	switch( k->code )
	{
	  case KCD_0:
		  kp = '0';
		  break;

  	  case KCD_1:
		  kp = '1';
		  break;

	  case KCD_2:
		  kp = '2';
		  break;

	  case KCD_3:
		  kp = '3';
		  break;

	  case KCD_4:
		  kp = '4';
		  break;

	  case KCD_5:
		  kp = '5';
		  break;

	  case KCD_6:
		  kp = '6';
		  break;

	  case KCD_7:
		  kp = '7';
		  break;

	  case KCD_8:
		  kp = '8';
		  break;

	  case KCD_9:
		  kp = '9';
		  break;

	  case KCD_STAR:
		  kp = '*';
		  break;

	  case KCD_HASH:
		  kp = '#';
		  break;

	  case KCD_LEFT:
		  kp = 'L';
		  break;

	  case KCD_RIGHT:
		  kp = 'R';
		  break;

	  case KCD_HUP:
 		  kp = 'C';
		  break;

	  default:
		  break;
	}

	if( kp != 0 )
	{
		switch ( extrasSelection )
		{
			case EXTRAS_C4 :

				break;

			case EXTRAS_SPS	:

				break;

			case EXTRAS_PIPES :

				break;

			case EXTRAS_FLIP_IT :

				break;

			case EXTRAS_ROLL_EM	:
				break;

			case EXTRAS_LOTTERY	:
				break;

			case EXTRAS_CALCULATOR :
				break;

			case EXTRAS_STOP_WATCH :
				break;

			default:
				break;
		}
	}

    return 1;
}



/*******************************************************************************

 $Function:    	TimerEvent

 $Description:	All extra menu features general timer handler
 
 $Returns:		Status int

 $Arguments:	Window handler event, timer control block
 
*******************************************************************************/
static int TimerEvent( MfwEvt e, MfwTim *tc )
{
    int end_state;

	switch ( extrasSelection )
	{
		case EXTRAS_C4 :

			break;

		case EXTRAS_SPS	:

			break;

		case EXTRAS_PIPES :

			break;

		case EXTRAS_FLIP_IT :

			break;

		case EXTRAS_ROLL_EM	:
			break;

		case EXTRAS_LOTTERY	:
			break;

		case EXTRAS_CALCULATOR :
			break;

		case EXTRAS_STOP_WATCH :
			break;

		default:
			break;
	}

	if ( end_state != TRUE )
	{
	   timStart( timerExtras1 );
	}

    return 1;
}


/*******************************************************************************

 $Function:    	ConfirmMesgTimerEvent

 $Description:	General confirmation (timerExtras2) screen handler
 
 $Returns:		Status int

 $Arguments:	Window handler event, timer control block
 
*******************************************************************************/
static int ConfirmMesgTimerEvent( MfwEvt e, MfwTim *tc )
{
	MfwMnu *mnuDetails;

/* TO BE SORTED OUT !!!!
    mnuDetails = (MfwMnu *)mfwControl( getMenuHandler() );

    // pop the sub-level menu only if there has been no previous
    // intervention - i.e. the 'C'/Back key pressed - we are
    // already where we want to be
    if ( mnuDetails->level == 3 )
    {
        popMenu();
    }
    else
    {
	    showMenuSoftkeys();
	}
END OF TO BE SORTED */

    return 1;
}


/*
 * Extras menu subs:
 */


/*******************************************************************************

 $Function:    	extrasShowDone

 $Description:	Flip It Level selection routines
 
 $Returns:		Status int

 $Arguments:	screen no
 
*******************************************************************************/
static int extrasShowDone( int screen )
{
    extrasEntryScreen = screen;

	dspl_ClearAll();
    timStart( timerExtras2 );
}



/*******************************************************************************

 $Function:    	extrasC4Level1..3

 $Description:	Flip It Level selection routines
 
 $Returns:		Status int

 $Arguments:	menu, item
 
*******************************************************************************/
int extrasC4Level1(MfwMnu* m, MfwMnuItem* i)
{

	return 1;
}

int extrasC4Level2(MfwMnu* m, MfwMnuItem* i)
{

	return 1;
}

int extrasC4Level3(MfwMnu* m, MfwMnuItem* i)
{

	return 1;
}

int extrasC4Phone(MfwMnu* m, MfwMnuItem* i)
{

	return 1;
}

int extrasC4Friend(MfwMnu* m, MfwMnuItem* i)
{

	return 1;
}

int extrasC4Demo(MfwMnu* m, MfwMnuItem* i)
{

	return 1;
}


/*******************************************************************************

 $Function:    	extrasC4Start	

 $Description:	C4 game initiator
 
 $Returns:		Status int

 $Arguments:	menu, item
 
*******************************************************************************/
int extrasC4Start (MfwMnu* m, MfwMnuItem* i)
{
	extrasSelection = EXTRAS_C4;

	return 1;
}



/*******************************************************************************

 $Function:    	extrasSPS 	

 $Description:	Scissor, Paper, and Stone game initiator
 
 $Returns:		Status int

 $Arguments:	menu, item
 
*******************************************************************************/
int extrasSPS (MfwMnu* m, MfwMnuItem* i)
{
	extrasSelection = EXTRAS_SPS;

	return 1;
}


/*******************************************************************************

 $Function:    	extrasPipes   	

 $Description: 	Pipes game initiator
 
 $Returns:		Status int

 $Arguments:	menu, item
 
*******************************************************************************/
int extrasPipes (MfwMnu* m, MfwMnuItem* i)
{
	extrasSelection = EXTRAS_PIPES;


	return 1;
}



/*******************************************************************************

 $Function:    	extrasFlipItLevel1..6   	

 $Description: 	Flip It Level selection routines
 
 $Returns:		Status int

 $Arguments:	menu, item
 
*******************************************************************************/
int extrasFlipItLevel1(MfwMnu* m, MfwMnuItem* i)
{

	return 1;
}

int extrasFlipItLevel2(MfwMnu* m, MfwMnuItem* i)
{

	return 1;
}

int extrasFlipItLevel3(MfwMnu* m, MfwMnuItem* i)
{

	return 1;
}

int extrasFlipItLevel4(MfwMnu* m, MfwMnuItem* i)
{

	return 1;
}

int extrasFlipItLevel5(MfwMnu* m, MfwMnuItem* i)
{

	return 1;
}

int extrasFlipItLevel6(MfwMnu* m, MfwMnuItem* i)
{

	return 1;
}


/*******************************************************************************

 $Function:    	extrasFlipItStart  	

 $Description: 	Flip It  game initiator
 
 $Returns:		Status int

 $Arguments:	menu, item
 
*******************************************************************************/
int extrasFlipItStart(MfwMnu* m, MfwMnuItem* i)
{
	extrasSelection = EXTRAS_FLIP_IT;

	return 1;
}



/*******************************************************************************

 $Function:    	extrasRollEm 	

 $Description: 	Roll Em' game initiator
 
 $Returns:		Status int

 $Arguments:	menu, item
 
*******************************************************************************/
// int extrasRollEm ( void )
//{
//	extrasSelection = EXTRAS_ROLL_EM;
//
//  extrasInitGame();
//
//	return 11;
//}




/*******************************************************************************

 $Function:    	extrasLottery		

 $Description: 	Lottery game initiator
 
 $Returns:		Status int

 $Arguments:	menu, item
 
*******************************************************************************/

// int extrasLottery ( void )
//{
//	extrasSelection = EXTRAS_LOTTERY;
//
//  extrasInitGame();
//
//	return 11;
//}



/*******************************************************************************

 $Function:    	extrasCalculator		

 $Description: 	Calculator initiator
 
 $Returns:		Status int

 $Arguments:	menu, item
 
*******************************************************************************/

int extrasCalculator (MfwMnu* m, MfwMnuItem* i)
{
	extrasSelection = EXTRAS_CALCULATOR;
//
//  extrasInitGame();
//
	return 11;
}



/*******************************************************************************

 $Function:    	extrasStopwatch		

 $Description: 	stopwatch
 
 $Returns:		Status int

 $Arguments:	menu, item
 
*******************************************************************************/
int extrasStopwatch (MfwMnu* m, MfwMnuItem* i)
{
	extrasSelection = EXTRAS_STOP_WATCH;
//
//  extrasInitGame();
//
	return 11;
}




/*******************************************************************************

 $Function:    	extrasInitGame	

 $Description: 	Common games initialisation
 
 $Returns:		void

 $Arguments:	void
 
*******************************************************************************/
static void extrasInitGame ( void )
{
	oldFocus = startWhoIsFocused();

	dspl_ClearAll();

	winFocus(winExtras);
	winShow (winExtras);

    timStart(timerExtras1);
}



/*******************************************************************************

 $Function:    	extrasGamesExit

 $Description: 	Generic games exit function
 
 $Returns:		void

 $Arguments:	level, mode
 
*******************************************************************************/
void extrasGamesExit( int level, int mode )
{
	timStop ( timerExtras1 );
	timStop ( timerExtras2 );

    // preserve associated games configurables
	switch ( extrasSelection )
	{
		case EXTRAS_C4 :
	        extrasC4Level = level;  /* preserve the level on game exit */
	        extrasC4Mode  = mode;   /* preserve the mode on game exit  */
			break;

		case EXTRAS_SPS	:
			break;

		case EXTRAS_PIPES :
			break;

		case EXTRAS_FLIP_IT :
	        extrasFlipLevel = level;  /* preserve the level on game exit */
			break;

		case EXTRAS_ROLL_EM	:
			break;

		case EXTRAS_LOTTERY	:
			break;

		case EXTRAS_CALCULATOR :
			break;

		case EXTRAS_STOP_WATCH :
			break;

		default:
			break;
	}

	extrasEntryScreen = EXTRAS_GAME;

    extrasSelection = EXTRAS_MIN;

    winFocus ( oldFocus );
    winShow  ( oldFocus );
}



/*******************************************************************************

 $Function:    	extrasGamesInitReq

 $Description: 	Function used to initialise games associated data on entry
            to the Extras menu (i.e. games selection menu)
 
 $Returns:		status int

 $Arguments:	None
*******************************************************************************/
int extrasGamesInitReq(void)
{
    return ( extrasEntryScreen != EXTRAS_MAIN );
}


/*******************************************************************************

 $Function:    	extrasResetGamesConfigurables	
 $Description: 	Function used to initialise games associated configurable data
            	on entry to the Extras menu (i.e. games selection menu)
 $Returns:		void

 $Arguments:	None
*******************************************************************************/
void extrasResetGamesConfigurables()
{


    // initialise the entry screen flag to prevent unnecessary calls to
    // reset the configurables.
    extrasEntryScreen = EXTRAS_MAIN;
}


