/**************************************************************
history:
  data             modifaction
  04/08/02         zy Mobile Innovation make gprs enable
*****************************************************************/
#define ENTITY_MFW

/* SH - switch for tracing*/
#define TRACE_MMIGPRS

/* includes */
#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#if defined (NEW_FRAME)

#include "typedefs.h"
#include "vsi.h"
#include "pei.h"
#include "custom.h"
#include "gsm.h"

#else

#include "stddefs.h"
#include "custom.h"
#include "gsm.h"
#include "vsi.h"

#endif
#include "mfw_sys.h"
#include "p_sim.h"
#include "cus_aci.h"

#include "mfw_mfw.h"
#include "mfw_win.h"
#include "mfw_kbd.h"
#include "mfw_edt.h"
#include "mfw_lng.h"
#include "mfw_tim.h"
#include "mfw_icn.h"
#include "mfw_mnu.h"
#include "mfw_phb.h"
#include "mfw_cm.h"
#include "mfw_sim.h"
#include "mfw_nm.h"
#include "mfw_sat.h"
#include "mfw_ss.h" /*for convert*/
#include "mfw_phb.h"
#include "ksd.h"
#include "psa.h"
#include "mfw_sms.h"
#include "mfw_cphs.h"
#include "mfw_sat.h"
#include "aci_cmh.h"  /* include types declared in ACI              */
#include "gaci_cmh.h" /* include types declared in GACI             */
#include "mfw_gprs.h"

#include "dspl.h"

#include "MmiGprs.h" 
#include "MmiBookShared.h"

#include "MmiIcons.h"						// Including this header allows us to access the global icon status, for the GPRS icon.
#include "MmiSmsBroadcast.h"				// Including this to get the info_screen function
#include "MmiEditor.h"						// Need this for counter display
#include "MmiBookUtils.h"
#include "MmiBookMenuWindow.h"				// So we can use bookMenuStart() to create our own menus

/* SPR#1574 - SH - Required to send status to WAP */
#ifdef MMI_WAP_ENABLED
#include "AUIWapExt.h"
#endif
#define ULONG_SIZE_IN_DECIMAL		10		// Max no. of decimal digits a ULONG can have
#define KILOBYTE					1024	// No of bytes in a kilobyte, used for data counter calculation.
#define PLEASEWAIT_TIMEOUT			35000	// Time for "please wait" window to time out

#define TRACE_MMIGPRS 1	//zy make mmigprs trace enable 04/08/02
#ifdef TRACE_MMIGPRS
static char debug[50];
#endif
/* Counter types */

enum
{
	MMI_GPRS_COUNTER_UPLINK,
	MMI_GPRS_COUNTER_DOWNLINK
};

/* GPRS data */

typedef struct
{
	T_MFW_HDR 		*gprsHandle;			// Provide a handle to the GPRS information.
	T_GPRS_CID		cid;					// The context id.  Starts at 1 then increments.
	MfwHnd 			gprsPleaseWait_win;		// Handle for the Please Wait window
	MfwHnd 			gprsEdit_win;			// Handle for editor window
	MfwHnd			menu_win;				// Handler for menu windows
	char			*counterstring;			// String for counter digits
	ULONG			uplink_counter;			// Value of uplink counter
	ULONG			downlink_counter;		// Value of downlink counter
	USHORT			counter_type;			// TRUE if uplink counter has been requested
	T_CGSMS_SERVICE SMSService;				// Type of SMS service
	BOOL			display;				// Whether to display "OK" windows or not
}
T_MMI_GPRS_DATA;

static T_MMI_GPRS_DATA		*mmi_gprs=NULL;


/*******************************************************************************

 LOCAL FUNCTION PROTOTYPES
 
*******************************************************************************/

//static T_MFW_HND mmiPleaseWait(T_MFW_HND parent_win, USHORT TextId1, USHORT TextId2);
static void mmiPleaseWaitCb(T_MFW_HND win, UBYTE identifier, UBYTE reason);
static void mmiPleaseWaitDestroy();
static void mmiGprsShowCounter(T_MFW_HND win, USHORT identifier);
static void mmiGprsDestroyCounter(); /* SPR#1875 - SH */
static char * mmiGprsUpdateCounter(T_MFW_HND win, USHORT identifier);
static void mmiGprsShowCounterCb(T_MFW_HND win, USHORT identifier, USHORT reason);


/*******************************************************************************

 $Function:    	mmiGprsInit

 $Description:	Initialise GPRS

 $Returns:		Handler for MFW GPRS events.

 $Arguments:	None.
 
*******************************************************************************/

void mmiGprsInit(void)
{
#ifdef TRACE_MMIGPRS
	TRACE_FUNCTION("mmiGprsInit()");
#endif

	gprs_init();

	// Allocate memory for mmi data
	mmi_gprs = (T_MMI_GPRS_DATA *)ALLOC_MEMORY(sizeof(T_MMI_GPRS_DATA));

	memset(mmi_gprs, 0, sizeof(T_MMI_GPRS_DATA));
	mmi_gprs->gprsHandle = gprs_create(0, E_MFW_GPRS_ALL_EVENTS, (MfwCb)GPRSMfwCb);
	mmi_gprs->cid = GPRS_CID_OMITTED;
	mmi_gprs->uplink_counter = 0;
	mmi_gprs->downlink_counter = 0;
	mmi_gprs->SMSService = CGSMS_SERVICE_OMITTED;
	mmi_gprs->display = FALSE;

	return;
}


/*******************************************************************************

 $Function:    	mmiGprsExit

 $Description:	Exit GPRS

 $Returns:		None.

 $Arguments:	None.
 
*******************************************************************************/

void mmiGprsExit(void)
{
#ifdef TRACE_MMIGPRS
	TRACE_FUNCTION("mmiGprsExit()");
#endif

	/* Delete any remaining please wait window (just in case!)*/
	mmiPleaseWaitDestroy();		
	/* Delete the MFW module */
	if (mmi_gprs->gprsHandle)
		gprs_delete(mmi_gprs->gprsHandle);

	/* Delete MMI data */
	FREE_MEMORY((void *)mmi_gprs, sizeof(T_MMI_GPRS_DATA));
		
	gprs_exit();
	return;
}


/*******************************************************************************

 $Function:    	mmiGprsStatus

 $Description:	Return status of GPRS.

 $Returns:		GPRS_OFF, GPRS_NOT_ATTACHED, GPRS_ATTACHED

 $Arguments:	None.
 
*******************************************************************************/

int mmiGprsStatus(void)
{
	T_MFW_GPRS *gprs;
	static int previous_result = -1;
	int result;
	
#ifdef TRACE_MMIGPRS
	TRACE_FUNCTION("mmiGprsStatus()");
#endif
	
	/* SPR#1575 - SH - Restructured so it only has one exit point.
	 * Keeps track of last status value.  If status has changed since last update,
	 * informs WAP.
	 * Default state is GPRS OFF */
	iconsDeleteState(iconIdGPRSOn);
    iconsDeleteState(iconIdGPRSAttached);//zy add: Del GPRS Attached Icon state 
	result = GPRS_OFF;
	/* Check if searching or attached */
	
	if (mmi_gprs)
	{
		if (mmi_gprs->gprsHandle)
		{
		 	gprs = mmi_gprs->gprsHandle->data;

			if (gprs)
			{
				gprs->data.regState = gprs_status();
				
#ifdef TRACE_MMIGPRS
				TRACE_EVENT_P1("mmiGprsStatus: regState = %d", gprs->data.regState);
#endif

				if (gprs->data.regState==CGREG_STAT_SEARCHING)
				{
					result = GPRS_SEARCHING;
				}
				
				/* SPR739 - SH - Check regState rather than attached to find GPRS status */
				
/*....if (gprs->data.attached)...............zy test gprs. change for gprs icon in idle win 04/08/02....................................*/
				else if (gprs->data.regState==CGREG_STAT_REG_HOME || gprs->data.regState==CGREG_STAT_REG_ROAM)
		{TRACE_EVENT("gprs->data.attached");
		iconsSetState(iconIdGPRSAttached);//zy set iconGprsAttached state on
					result = GPRS_ATTACHED;
				}
			}
		}
	}
	/*...............................end of zy change for gprs icon 04/08/02....................................*/
#ifdef MMI_WAP_ENABLED
	if (result!=previous_result)
	{
		AUI_wap_gprs_status(result);
	}
#endif

	previous_result = result;
	
	return result;
}


/*******************************************************************************

 $Function:    	mmiPleaseWaitWindow

 $Description:	Creates a "Please Wait" message that will hang around until an "OK" or
 				"Error" message is received by GPRSMfwCb

 $Returns:		GPRS_OFF, GPRS_NOT_ATTACHED, GPRS_ATTACHED

 $Arguments:	parent_win - the parent window
 				TextId1	- the first text string to be shown
 				TextId2  - the second text string to be shown
 
*******************************************************************************/

T_MFW_HND mmiPleaseWaitWindow(T_MFW_HND parent_win, USHORT TextId1, USHORT TextId2,USHORT displayflag)
{
	T_DISPLAY_DATA display_info;

	TRACE_FUNCTION("mmiPleaseWaitWindow()");

	/* Delete any previous window */
	if (mmi_gprs->gprsPleaseWait_win)
	{
		SEND_EVENT(mmi_gprs->gprsPleaseWait_win, DIALOG_DESTROY, NULL, NULL);
		mmi_gprs->gprsPleaseWait_win = NULL;
	}	
	mmi_gprs->display = TRUE;									// Allow OK and Fail messages to appear.
			
	dialog_info_init(&display_info);
	display_info.TextString = NULL;
	display_info.SoftKeyStrings = FALSE;
	display_info.LeftSoftKey = NULL;
	display_info.RightSoftKey = TxtCancel;
	display_info.Time = PLEASEWAIT_TIMEOUT;
	display_info.KeyEvents = KEY_HUP | KEY_RIGHT;
	display_info.TextId = TextId1;
	display_info.TextId2 = TextId2;
	display_info.TextString2 = NULL;
	display_info.Callback = (T_VOID_FUNC)mmiPleaseWaitWindowCb;
    display_info.iconindex    =REMIND_WAITTING;
	mmi_gprs->menu_win = parent_win;							// Store parent window
	if(displayflag)//if now is autoattach
	  return 0;
	return info_dialog(parent_win,&display_info); //information screen
}

/*******************************************************************************

 $Function:    	mmiPleaseWaitCb

 $Description:	Please wait window callback function.

 $Returns:		None

 $Arguments:	win - the parent window
 				identifier - not used
 				reason - the reason for the callback
 
*******************************************************************************/
void mmiPleaseWaitWindowCb(T_MFW_HND win, UBYTE identifier, UBYTE reason)
{
	TRACE_FUNCTION("mmiPleaseWaitWindowCb()");
	
#ifdef TRACE_MMIGPRS
	sprintf(debug, "mmiPleaseWaitWindowCb: reason = %d",reason);
	TRACE_EVENT(debug);
#endif

	switch(reason)
		{
		case INFO_TIMEOUT:
			SEND_EVENT(win, DIALOG_DESTROY, NULL, NULL);
			mmi_gprs->gprsPleaseWait_win = NULL;
			info_screen(mmi_gprs->menu_win, NULL,TxtTimedOut , NULL,REMIND_FAILURE);
		break;
		
		case INFO_KCD_HUP:
		case INFO_KCD_RIGHT:
			SEND_EVENT(win, DIALOG_DESTROY, NULL, NULL);
			mmi_gprs->gprsPleaseWait_win = NULL;
		break;
		}
}
/*******************************************************************************

 $Function:    	mmiPleaseWaitDestroy

 $Description:	Check to see if the "Please Wait" window is present, and destroys it
 				if so.

 $Returns:		None.
 
 $Arguments:	None.
 
*******************************************************************************/

static void mmiPleaseWaitDestroy()
{
	TRACE_FUNCTION("mmiPleaseWaitCb()");

	if (mmi_gprs->gprsPleaseWait_win)
	{
		SEND_EVENT(mmi_gprs->gprsPleaseWait_win, DIALOG_DESTROY, NULL, NULL);
		mmi_gprs->gprsPleaseWait_win = NULL;
	}
	return;
}


/*******************************************************************************

 $Function:    	GPRSMfwCb

 $Description:	Callback function for MFW GPRS events.

 $Returns:		

 $Arguments:	Event number and generic parameter.
 
*******************************************************************************/

int GPRSMfwCb(T_MFW_EVENT event, void* para)
{	
	T_MFW_HND 			win			= mfwParent( mfw_header());
	T_MFW_GPRS_DATA		*gprs_data	= (T_MFW_GPRS_DATA *)para;				// Structure that stores data from mfw
	USHORT				textId;
    int    iconindex;
	TRACE_FUNCTION("GPRSMfwCb()");
	
	switch(event)
	{
		case E_MFW_GPRS_S_CNTXT:
			#ifdef TRACE_MMIGPRS
			TRACE_EVENT("GPRSMfwCb: E_MFW_GPRS_S_CNTXT");
			#endif
			break;

		case E_MFW_GPRS_S_ATT:
			#ifdef TRACE_MMIGPRS
			TRACE_EVENT("GPRSMfwCb: E_MFW_GPRS_S_ATT");
			#endif
			mmiGprsStatus();
			if (mmi_gprs->gprsPleaseWait_win)
			{
				SEND_EVENT(mmi_gprs->gprsPleaseWait_win, DIALOG_DESTROY, NULL, NULL);
			}
			mmi_gprs->gprsPleaseWait_win = mmiPleaseWaitWindow(win, TxtGPRS, TxtConnecting,FlashSettingData.gprsattachmode);
			break;

		case E_MFW_GPRS_S_ACT:
			#ifdef TRACE_MMIGPRS
			TRACE_EVENT("GPRSMfwCb: E_MFW_GPRS_S_ACT");
			#endif
			break;

		case E_MFW_GPRS_R_ACT:
			#ifdef TRACE_MMIGPRS
			TRACE_EVENT("GPRSMfwCb: E_MFW_GPRS_R_ACT");
			#endif
			break;

		case E_MFW_GPRS_S_DATA:
			#ifdef TRACE_MMIGPRS
			TRACE_EVENT("GPRSMfwCb: E_MFW_GPRS_S_DATA");
			#endif
			break;

		case E_MFW_GPRS_R_DATA:
			#ifdef TRACE_MMIGPRS
			TRACE_EVENT("GPRSMfwCb: E_MFW_GPRS_R_DATA");
			#endif
			break;

		case E_MFW_GPRS_S_QOS:
			#ifdef TRACE_MMIGPRS
			TRACE_EVENT("GPRSMfwCb: E_MFW_GPRS_S_QOS");
			#endif
			break;

		case E_MFW_GPRS_R_QOS:
			#ifdef TRACE_MMIGPRS
			TRACE_EVENT("GPRSMfwCb: E_MFW_GPRS_R_QOS");
			#endif
			break;
		
		case E_MFW_GPRS_S_QOS_MIN:
			#ifdef TRACE_MMIGPRS
			TRACE_EVENT("GPRSMfwCb: E_MFW_GPRS_S_QOS_MIN");
			#endif
			break;

		case E_MFW_GPRS_S_CLASS:
			#ifdef TRACE_MMIGPRS
			TRACE_EVENT("GPRSMfwCb: E_MFW_GPRS_S_CLASS");
			#endif
			break;

		case E_MFW_GPRS_S_PDPADDR:
			#ifdef TRACE_MMIGPRS	
			TRACE_EVENT("GPRSMfwCb: E_MFW_GPRS_S_PDPADDR");
			#endif
			break;

		case E_MFW_GPRS_S_AUTORESP:
			#ifdef TRACE_MMIGPRS
			TRACE_EVENT("GPRSMfwCb: E_MFW_GPRS_R_REG");
			#endif
			break;

		case E_MFW_GPRS_S_ANS:
			#ifdef TRACE_MMIGPRS
			TRACE_EVENT("GPRSMfwCb: E_MFW_GPRS_S_AUTORESP");
			#endif
			break;

		case E_MFW_GPRS_R_ANS:
			#ifdef TRACE_MMIGPRS
			TRACE_EVENT("GPRSMfwCb: E_MFW_GPRS_R_ANS");
			#endif
			break;

		case E_MFW_GPRS_S_EREP:
			#ifdef TRACE_MMIGPRS
			TRACE_EVENT("GPRSMfwCb: E_MFW_GPRS_S_EREPG");
			#endif
			break;

		case E_MFW_GPRS_R_EREP_RJ:
			#ifdef TRACE_MMIGPRS
			TRACE_EVENT("GPRSMfwCb: E_MFW_GPRS_R_EREP_RJ");
			#endif
			break;

		case E_MFW_GPRS_R_EREP_ATT:
			#ifdef TRACE_MMIGPRS
			TRACE_EVENT("GPRSMfwCb: E_MFW_GPRS_R_EREP_ATT");
			#endif

			/* Attach status has changed, display message as appropriate */
			if (mmi_gprs->display)
			{
				/* Delete please wait window */
				mmiPleaseWaitDestroy();		
				if (gprs_data->attached)
				{
				textId = TxtAttached;
				iconindex=REMIND_SUCCEED;
				}
				else
				{
			     textId = TxtNotAttached;
			     iconindex=REMIND_SUCCEED;
				}
			
				info_screen(win, TxtGPRS, textId, NULL,iconindex);
			mmi_gprs->display = FALSE;
			}
			mmiGprsStatus();
			break;

		case E_MFW_GPRS_R_EREP_ACT:
			#ifdef TRACE_MMIGPRS
			TRACE_EVENT("GPRSMfwCb: E_MFW_GPRS_R_EREP_ACT");
			#endif
			break;

		case E_MFW_GPRS_R_EREP_DEACT:
			#ifdef TRACE_MMIGPRS
			TRACE_EVENT("GPRSMfwCb: E_MFW_GPRS_R_EREP_DEACT");
			#endif
			break;

		case E_MFW_GPRS_R_EREP_CLASS:
			#ifdef TRACE_MMIGPRS
			TRACE_EVENT("GPRSMfwCb: E_MFW_GPRS_R_EREP_CLASS");
			#endif
			break;

		case E_MFW_GPRS_R_REG:
			#ifdef TRACE_MMIGPRS
			TRACE_EVENT("GPRSMfwCb: E_MFW_GPRS_R_REG");
			TRACE_EVENT_P1("regState: %d", gprs_data->regState);
			#endif
			
			if (mmi_gprs->display)
			{
				textId = TxtError;
				iconindex=REMIND_FAILURE;
				switch (gprs_data->regState)
				{
					case CGREG_STAT_REG_HOME:
					case CGREG_STAT_REG_ROAM:
						textId = TxtAttached;
						iconindex=REMIND_SUCCEED;
#ifdef MMI_HOMEZONE_ENABLED
						/* SPR759 - SH - cell reselection for homezone */
						homezoneUpdate(gprs_data->lac, gprs_data->ci);
#endif
						break;
					
					case CGREG_STAT_NOT_REG:       /* not registered, no searching */
					case CGREG_STAT_REG_DEN:
						textId = TxtNotAttached;
						iconindex=REMIND_SUCCEED;
#ifdef MMI_HOMEZONE_ENABLED
						/* SPR759 - SH - cell reselection for homezone */
						homezoneUpdate(0,0);
#endif
						break;

					case CGREG_STAT_SEARCHING:       /* not registered, searching */
						textId = TxtSearching;
						iconindex=REMIND_NORECORD;
#ifdef MMI_HOMEZONE_ENABLED
						/* SPR759 - SH - cell reselection for homezone */
						homezoneUpdate(0, 0);
#endif
						break;
				}
			}

			if (mmi_gprs->display)
				info_screen(win, TxtGPRS, textId, NULL,iconindex);
			mmi_gprs->display = FALSE;
			mmiGprsStatus();
			if (mmi_gprs->gprsPleaseWait_win)
			{
				SEND_EVENT(mmi_gprs->gprsPleaseWait_win, DIALOG_DESTROY, NULL, NULL);
				mmi_gprs->gprsPleaseWait_win = NULL;
			}		
			break;

		case E_MFW_GPRS_S_SMSSERV:
			#ifdef TRACE_MMIGPRS
			TRACE_EVENT("GPRSMfwCb: E_MFW_GPRS_S_SMSSERV");
			#endif

			/* First display "GPRS Changed" */

			if (mmi_gprs->display)
				info_screen(win, TxtGPRS, TxtChanged, NULL,REMIND_SUCCEED);
			mmi_gprs->display = FALSE;
			mmiGprsStatus();
			
			/* Now destroy the "Please Wait" window underneath */

			if (mmi_gprs->gprsPleaseWait_win)
			{
				SEND_EVENT(mmi_gprs->gprsPleaseWait_win, DIALOG_DESTROY, NULL, NULL);
				mmi_gprs->gprsPleaseWait_win = NULL;
			}
			break;
		
		case E_MFW_GPRS_OK:
			#ifdef TRACE_MMIGPRS
			TRACE_EVENT("GPRSMfwCb: E_MFW_GPRS_OK");
			#endif
			break;
		
		case E_MFW_GPRS_ERROR:
			#ifdef TRACE_MMIGPRS
			TRACE_EVENT("GPRSMfwCb: E_MFW_GPRS_ERROR");
			#endif

			// First display "GPRS Error"

			if (mmi_gprs->display)
				info_screen(win, TxtGPRS, TxtFailed, NULL,REMIND_FAILURE);
			mmi_gprs->display = FALSE;
			mmiGprsStatus();
			
			// Now destroy the "Please Wait" window underneath

			if (mmi_gprs->gprsPleaseWait_win)
			{
				SEND_EVENT(mmi_gprs->gprsPleaseWait_win, DIALOG_DESTROY, NULL, NULL);
				mmi_gprs->gprsPleaseWait_win = NULL;
			}
			break;

		case E_MFW_GPRS_CONNECT:
			#ifdef TRACE_MMIGPRS
			TRACE_EVENT("GPRSMfwCb: E_MFW_GPRS_CONNECT");
			#endif
			/* Display "CONNECTED" window */
			//mmi_gprs->gprsPleaseWait_win = mmiPleaseWaitWindow(win,TxtGPRS, TxtConnecting);
			//info_screen(win, TxtGPRS, TxtConnected, NULL);
			mmi_gprs->gprsPleaseWait_win = mmiPleaseWaitWindow(win,TxtGPRS, TxtConnecting,FlashSettingData.gprsattachmode);
			mmiGprsStatus();
			
			// Now destroy any "Please Wait" window underneath

			if (mmi_gprs->gprsPleaseWait_win)
			{
				SEND_EVENT(mmi_gprs->gprsPleaseWait_win, DIALOG_DESTROY, NULL, NULL);
				mmi_gprs->gprsPleaseWait_win = NULL;
			}
			
			break;

		case E_MFW_GPRS_ERR_CONNECT:
			#ifdef TRACE_MMIGPRS
			TRACE_EVENT("GPRSMfwCb: E_MFW_GPRS_ERR_CONNECT");
			#endif
			// First display "GPRS No Answer"
			if (mmi_gprs->display)
				info_screen(win, TxtGPRS, TxtNoAnswer, NULL,REMIND_FAILURE);
			mmi_gprs->display = FALSE;
			mmiGprsStatus();
			
			// Now destroy the "Please Wait" window underneath

			if (mmi_gprs->gprsPleaseWait_win)
			{
				SEND_EVENT(mmi_gprs->gprsPleaseWait_win, DIALOG_DESTROY, NULL, NULL);
				mmi_gprs->gprsPleaseWait_win = NULL;
			}
			break;

		case E_MFW_GPRS_COUNTER:
			#ifdef TRACE_MMIGPRS
			TRACE_EVENT("GPRSMfwCb: E_MFW_GPRS_COUNTER");
			#endif
			/* SPR#1875 - SH - Now add values returned from ACI
			 * to total counter */
			 
			mmi_gprs->uplink_counter += (ULONG)(gprs_data->counter.octets_uplink);
			mmi_gprs->downlink_counter += (ULONG)(gprs_data->counter.octets_downlink);
			TRACE_EVENT("mmi_gprs->uplink_counter");
	        TraceInt((int) mmi_gprs->uplink_counter);
            TRACE_EVENT("mmi_gprs->downlink_counter");
	        TraceInt((int) mmi_gprs->uplink_counter);
			/* Only display counter if user has requested it */
			
			if (mmi_gprs->display)
			{
			mmi_gprs->display = FALSE;
			mmiGprsStatus();
			/* Delete please wait window */
			mmiPleaseWaitDestroy();
			
			mmiGprsShowCounter(win, mmi_gprs->counter_type);
			}
			break;

		default:
			#ifdef TRACE_MMIGPRS
			sprintf(debug, "GPRSMfwCb: event = %d", event);
			TRACE_EVENT(debug);
			#endif
			break;
	}
			
	if (idleIsFocussed())
	{
		iconsShow();		// Update idle screen icons to show any change in GPRS status
	}
	else
	{
		mmiGprsStatus();	// iconsShow calls mmiGprsStatus; if we don't call that, call this on its own
	}
	return TRUE;
}


/*******************************************************************************

 $Function:    	GPRSAttach

 $Description:	Attempts an attach to the GPRS network.

 $Returns:		

 $Arguments:	Standard menu option parameters
 
*******************************************************************************/
	
int GPRSAttach(MfwMnu* m, MfwMnuItem* i)
{
	T_MFW_HND win = mfw_parent(mfw_header());
	int gprsStatus;

	TRACE_FUNCTION("GPRSAttach()");

	if (!mmi_gprs->gprsHandle)
	{
		mmi_gprs->gprsHandle = gprs_create(0, E_MFW_GPRS_ALL_EVENTS, (MfwCb)GPRSMfwCb);
		mmi_gprs->cid = GPRS_CID_OMITTED;								// Set CID to 0; on increment will become 1.

		// If we've set the SMS service while unattached, change it now
	
		if (mmi_gprs->SMSService != CGSMS_SERVICE_OMITTED)
			gprs_setServiceSMS(mmi_gprs->SMSService);
	
	}

	gprsStatus = mmiGprsStatus();
 	/*zy move 11/26 for Gprs is must attach again avoid no network lose the gprsStatus
	if (gprsStatus==(int)GPRS_ATTACHED)
	{
		info_screen(win, TxtAlreadyAttached, NULL, NULL);
		return;
	}
    */

	mmi_gprs->display = TRUE;
	if (gprs_attach(CGATT_STATE_ATTACHED)==MFW_RES_OK)
	{
		#ifdef TRACE_MMIGPRS
		TRACE_EVENT("GPRSAttach executing, awaiting confirmation.");
		#endif
	  if(!FlashSettingData.gprsattachmode)
		mmi_gprs->gprsPleaseWait_win = mmiPleaseWaitWindow(win, NULL,TxtPleaseWait ,FlashSettingData.gprsattachmode);
	  else
	  	//mmi_gprs->gprsPleaseWait_win = mmiPleaseWaitWindow(win, TxtAttachMode, TxtAttachMode,FlashSettingData.gprsattachmode);
        info_screen(win, TxtAttachMode,TxtAutomatic, NULL,REMIND_NORECORD);
	}
	else
	{
	   if (gprsStatus==(int)GPRS_ATTACHED)
	   {
		info_screen(win,NULL ,TxtAlreadyAttached , NULL,REMIND_NORECORD);
	    TRACE_EVENT("GPRS already attached.");
		return;
	    }
		info_screen(win, TxtNull ,TxtFailed, NULL,REMIND_FAILURE);
		#ifdef TRACE_MMIGPRS
		TRACE_EVENT("GPRSAttach failed.");
		#endif
	}
		
	return;
}

/*******************************************************************************

 $Function:    	GPRSAttach

 $Description:	Attempts an attach to the GPRS network.

 $Returns:		

 $Arguments:	Standard menu option parameters
 
*******************************************************************************/
	
int GPRSAutoAttach()
{
	
	int gprsStatus;

	TRACE_FUNCTION("GPRSAttach()");

	if (!mmi_gprs->gprsHandle)
	{
		mmi_gprs->gprsHandle = gprs_create(0, E_MFW_GPRS_ALL_EVENTS, (MfwCb)GPRSMfwCb);
		mmi_gprs->cid = GPRS_CID_OMITTED;								// Set CID to 0; on increment will become 1.

		// If we've set the SMS service while unattached, change it now
	
		if (mmi_gprs->SMSService != CGSMS_SERVICE_OMITTED)
			gprs_setServiceSMS(mmi_gprs->SMSService);
	
	}

	mmi_gprs->display = FALSE;

    gprs_attach(CGATT_STATE_ATTACHED);
	gprsStatus = mmiGprsStatus();	
	return gprsStatus;
}

/*******************************************************************************

 $Function:    	GPRSDetach

 $Description:	Attempts a detach from the GPRS network.

 $Returns:		

 $Arguments:	Standard menu option parameters
 
*******************************************************************************/

int GPRSDetach(MfwMnu* m, MfwMnuItem* i)
{
	T_MFW_HND win = mfw_parent(mfw_header());
	int gprsStatus;

	TRACE_FUNCTION("GPRSDetach()");

	// Check to make sure gprs handle exists
	
	if (!mmi_gprs->gprsHandle)
	{
	    TRACE_EVENT("zy:detach 1:");
		info_screen(win,NULL ,TxtNotAttached , NULL,REMIND_SUCCEED);
		return;
	}

	gprsStatus = mmiGprsStatus();

	if (gprsStatus==(int)GPRS_OFF)
	{
	    TRACE_EVENT("zy:detach 2:");
		info_screen(win,NULL ,TxtNotAttached , NULL,REMIND_SUCCEED);
		return;
	}

	if (gprsStatus==(int)GPRS_SEARCHING)
	{
	     TRACE_EVENT("zy:detach 3:");
		info_screen(win, TxtGPRS, TxtNotAvailable, NULL,REMIND_FAILURE);
		return;
	}

	// Try to detach
	mmi_gprs->display = TRUE;
	
	if (gprs_attach(CGATT_STATE_DETACHED)==MFW_RES_OK)
	{
		#ifdef TRACE_MMIGPRS
		TRACE_EVENT("GPRSDetach executing, awaiting confirmation.");
		#endif
      if(!FlashSettingData.gprsattachmode)
		mmi_gprs->gprsPleaseWait_win = mmiPleaseWaitWindow(win, TxtPleaseWait, NULL,FlashSettingData.gprsattachmode);
	  else
	  	//mmi_gprs->gprsPleaseWait_win = mmiPleaseWaitWindow(win, TxtAttachMode, TxtAttachMode,FlashSettingData.gprsattachmode);
        info_screen(win, TxtAttachMode,TxtAutomatic, NULL,REMIND_NORECORD);
	}
	else
	{
		#ifdef TRACE_MMIGPRS
		TRACE_EVENT("GPRSDetach failed.");
		#endif
		info_screen(win,TxtNull ,TxtFailed , NULL,REMIND_FAILURE);
	}
	return;
}





/*******************************************************************************

 $Function:    	mmiGprsShowCounter

 $Description:	Generic counter displaying function

 $Returns:		

 $Arguments:	Standard menu option parameters
 
*******************************************************************************/

void mmiGprsShowCounter(T_MFW_HND win, USHORT identifier)
{
	T_EDITOR_DATA	editor_data;
	T_MFW_GPRS		*gprs;
	USHORT			titleId;

	TRACE_FUNCTION("mmiGprsShowCounter()");

	// Make sure GPRS handle exists before we try to read one of its fields...

	editor_data.TextString			= NULL;
	editor_data.LeftSoftKey			= TxtSoftOK;
	editor_data.RightSoftKey		= TxtReset;
	editor_data.AlternateLeftSoftKey	= NULL;
	editor_data.Callback			= (T_EDIT_CB)mmiGprsShowCounterCb;
	editor_data.Identifier			= identifier;
	editor_data.hide				= FALSE;
	editor_data.mode				= READ_ONLY_MODE;
	editor_data.timeout           	= FOREVER;
	editor_data.min_enter			= 0;
	editor_data.destroyEditor		= FALSE;
	editor_data.editor_attr.win.px	= 0;
	editor_data.editor_attr.win.py	= 20;//wufei modified 0912
	editor_data.editor_attr.win.sx	= 84;
	editor_data.editor_attr.win.sy	= 36;//wufei modified 0912
	editor_data.editor_attr.fgColor	= 0;
	editor_data.editor_attr.font	= 0;
	editor_data.editor_attr.mode	= edtCurNone;
	editor_data.editor_attr.controls	= 0;
    editor_data.editor_attr.size	= ULONG_SIZE_IN_DECIMAL;

	switch(identifier)
	{
		case MMI_GPRS_COUNTER_UPLINK:
			editor_data.TextId = TxtOutgoingDataNew;
			break;
		
		case MMI_GPRS_COUNTER_DOWNLINK:
			editor_data.TextId = TxtIncomingData;   //hu binhao 2002-12-23 for 3125
			break;

		default:
			editor_data.TextId = TxtNull;
			break;
	}

	mmi_gprs->counterstring = (char *)ALLOC_MEMORY(ULONG_SIZE_IN_DECIMAL);
	editor_data.editor_attr.text = mmi_gprs->counterstring;
    TRACE_EVENT(mmi_gprs->counterstring);
	mmiGprsUpdateCounter(win, identifier);
    TRACE_EVENT(mmi_gprs->counterstring);
	/* create the dialog handler */
	mmi_gprs->gprsEdit_win = editor_start(win, &editor_data);  /* start the editor */ 

	return;
}

/*******************************************************************************

 $Function:    	mmiGprsDestroyCounter

 $Description:	Destroy counter display
				SPR#1875 - SH - Added

 $Returns:		None

 $Arguments:	None
 
*******************************************************************************/

static void mmiGprsDestroyCounter()
{
	if (mmi_gprs->gprsEdit_win)
	{
		editor_destroy(mmi_gprs->gprsEdit_win);
		FREE_MEMORY((void *)mmi_gprs->counterstring, ULONG_SIZE_IN_DECIMAL);
		mmi_gprs->counterstring = NULL;
		mmi_gprs->gprsEdit_win = NULL;
	}
	
	return;
}

/*******************************************************************************

 $Function:    	mmiGprsUpdateCounter

 $Description:	Change text string for the counter we've just changed

 $Returns:		

 $Arguments:	identifier - the type of counter we're updating
 
*******************************************************************************/

char * mmiGprsUpdateCounter(T_MFW_HND win, USHORT identifier)
{
	T_MFW_GPRS		*gprs;
	ULONG			counter;
    ULONG			initcounter;

	TRACE_FUNCTION("mmiGprsUpdateCounter()");
	
	/* Make sure GPRS handle exists before we try to read one of its fields... */

	counter = 0;
	initcounter =0;
	if (mmi_gprs->gprsHandle)
		gprs = ((T_MFW_HDR *) mmi_gprs->gprsHandle)->data;
	else
		gprs = NULL;
	
	switch(identifier)
	{
		case MMI_GPRS_COUNTER_UPLINK:
			initcounter = mmi_gprs->uplink_counter;
			/*2003-04-16 hubh make it add*/
			counter = initcounter+FlashDCTbData.GPRSOUTCOUNTER;			
			TRACE_EVENT("uplink_counter");
			TraceInt((int) counter);
			break;
		
		case MMI_GPRS_COUNTER_DOWNLINK:
			initcounter = mmi_gprs->downlink_counter;
		    /*2003-04-16 hubh make it add*/
		    counter = initcounter+FlashDCTbData.GPRSINCOUNTER;
		    TRACE_EVENT("downlink_counter");
			TraceInt((int) counter);
		break;
		}
				flash_DCTbwrite();//hubh 2003-04-16
	if (counter<1024)
	{
		sprintf(mmi_gprs->counterstring, "%d bytes",counter);		// Counter is <1K, display bytes
	}
	else
	{
		counter = counter/KILOBYTE;
		if (counter<1024)
			sprintf(mmi_gprs->counterstring, "%dKB",counter);		// Counter is <1M, display Kbytes
		else
		{
			counter = counter/KILOBYTE;
			sprintf(mmi_gprs->counterstring, "%dMB",counter);		// Display Mbytes
		}
	}
	
	return mmi_gprs->counterstring;
}


/*******************************************************************************

 $Function:    	mmiGprsShowCounterCb

 $Description:	Change text string for the counter we've just changed

 $Returns:		nothing.

 $Arguments:	win		- the editor window
 				identifier - the type of counter we're updating
 				reason	- the event that caused the callback
 
*******************************************************************************/

void mmiGprsShowCounterCb(T_MFW_HND win, USHORT identifier, USHORT reason)
{	
	T_MFW_GPRS		*gprs;
	T_DISPLAY_DATA display_info;//wufei add 0912

	TRACE_FUNCTION("mmiGprsShowCounterCb()");
	
#ifdef TRACE_MMIGPRS
	sprintf(debug, "mmiGprsShowCounterCb: reason = %d",reason);
	TRACE_EVENT(debug);
#endif
	
	if (mmi_gprs->gprsHandle)
		gprs = ((T_MFW_HDR *) mmi_gprs->gprsHandle)->data;
	
	switch(reason)
	{
		// Press hangup key or select OK
		case INFO_KCD_LEFT:
		case INFO_KCD_HUP:
			mmiGprsDestroyCounter();
			break;
			
		// Select "reset"
		case INFO_KCD_RIGHT:
			mmiGprsDestroyCounter();
			if (mmi_gprs->gprsHandle)
			{
				if (identifier==MMI_GPRS_COUNTER_UPLINK)
				{
					mmi_gprs->uplink_counter = 0;
				    FlashDCTbData.GPRSOUTCOUNTER=0;
				}
				else
				{
					mmi_gprs->downlink_counter = 0;
					FlashDCTbData.GPRSINCOUNTER=0;	/*2003-04-16 hubh make it add*/
				}
				flash_DCTbwrite();
				mmiGprsShowCounter(win, identifier);
			}
			break;
	}

	return;
}


/*******************************************************************************

 $Function:    	GPRSIncomingDataCount

 $Description:	Show data counter for last incoming call

 $Returns:		

 $Arguments:	Standard menu option parameters
 
*******************************************************************************/

int GPRSIncomingDataCount(MfwMnu* m, MfwMnuItem* i)
{
	T_MFW_HND		win = mfw_parent(mfw_header());
	T_MFW_RES		result;
	
	TRACE_FUNCTION("GPRSIncomingDataCount()");

	mmi_gprs->counter_type = MMI_GPRS_COUNTER_DOWNLINK;	/* SPR#1875 - SH - Now DOWNLINK */
	mmi_gprs->display = TRUE;

	//mmi_gprs->gprsPleaseWait_win = mmiPleaseWait(win, TxtPleaseWait, NULL);  //ganchh primary code , comment out
	mmi_gprs->gprsPleaseWait_win =mmiPleaseWaitWindow(win, NULL,TxtPleaseWait ,0);   //0 means display
	result = gprs_counter(TRUE);

	switch(result)
	{
		case MFW_RES_OK:		/* Send request for counter information; don't reset */
			#ifdef TRACE_MMIGPRS
			TRACE_EVENT("gprs_counter executing, awaiting confirmation.");
			#endif
			break;

		case MfwResDone:
			mmiPleaseWaitDestroy();
			mmi_gprs->display = FALSE;
			mmiGprsShowCounter(win, mmi_gprs->counter_type);
			break;

		default:
			mmiPleaseWaitDestroy();
			mmi_gprs->display = FALSE;
            info_screen(win,NULL ,TxtAlreadyAttached , NULL,REMIND_NORECORD);
          break;
	}
	return;
}


/*******************************************************************************

 $Function:    	GPRSOutgoingDataCount

 $Description:	Show data counter for last incoming call

 $Returns:		

 $Arguments:	Standard menu option parameters
 
*******************************************************************************/

int GPRSOutgoingDataCount(MfwMnu* m, MfwMnuItem* i)
{
	T_MFW_HND		win = mfw_parent(mfw_header());
	T_MFW_RES		result;
	
	TRACE_FUNCTION("GPRSOutgoingDataCount()");

	mmi_gprs->counter_type = MMI_GPRS_COUNTER_UPLINK;
	mmi_gprs->display = TRUE;

	mmi_gprs->gprsPleaseWait_win = mmiPleaseWaitWindow(win, TxtPleaseWait, NULL,FlashSettingData.gprsattachmode);

	result = gprs_counter(TRUE);

	switch(result)
	{
		/* Waiting for rAT_PercentSNCNT callback, don't update display yet */
		
		case MFW_RES_OK:
			#ifdef TRACE_MMIGPRS
			TRACE_EVENT("gprs_counter executing, awaiting confirmation.");
			#endif
			break;

		/* We're not connected - display current total */
		case MfwResDone:
			mmiPleaseWaitDestroy();
			mmi_gprs->display = FALSE;
			mmiGprsShowCounter(win, mmi_gprs->counter_type);
			break;

		/* Error has occurred */
		default:
			mmiPleaseWaitDestroy();
			mmi_gprs->display = FALSE;
			info_screen(win,NULL ,TxtAlreadyAttached , NULL,REMIND_NORECORD);
			break;
	}	
	return;
}



/*******************************************************************************

 $Function:    	GPRSselectSms

 $Description:	Sets cursor position for this menu based on the current settings
 
 $Returns:		

 $Arguments:	menu		- pointer to the current menu
 				item		- pointer to the current menu item
 
*******************************************************************************/

int GPRSselectSms(MfwMnu* menu, MfwMnuItem* item)
{
	T_MFW_HND		parent_win		= mfwParent( mfw_header());					// WAP menu window
	T_MFW_WIN		*win_data   	= ((T_MFW_HDR *)parent_win)->data;
	tBookStandard	*parent_data	= (tBookStandard *)win_data->user;			// data for WAP menu window
	T_MFW_HND		win 			= parent_data->parent_win;					// WAP data window
	T_MFW_GPRS		*gprs;
	
	TRACE_FUNCTION("GPRSselectSms()");
	
	if (mmi_gprs->gprsHandle)
	{
		gprs = mmi_gprs->gprsHandle->data;
		gprs->data.SMSService = gprs_getServiceSMS();
		mmi_gprs->SMSService = gprs->data.SMSService;
	}

	mmi_gprs->menu_win = bookMenuStart(win, SMSServiceAttributes(), 0);
	
	SEND_EVENT(mmi_gprs->menu_win, DEFAULT_OPTION, NULL, &mmi_gprs->SMSService); // Add our own callback
	
	return;
}


/*******************************************************************************

 $Function:    	GPRSselectSms_set

 $Description:	Send SMS by GPRS/CSD
 
 $Returns:		

 $Arguments:	menu		- pointer to the current menu
 				item		- pointer to the current menu item
 
*******************************************************************************/

int GPRSselectSms_set(MfwMnu* menu, MfwMnuItem* item)
{
	T_MFW_HND		parent_win		= mfwParent( mfw_header());					// WAP menu window
	T_MFW_WIN		*win_data   	= ((T_MFW_HDR *)parent_win)->data;
	tBookStandard	*parent_data	= (tBookStandard *)win_data->user;			// data for WAP menu window
	T_MFW_HND		win 			= parent_data->parent_win;					// WAP data window

	TRACE_FUNCTION("GPRSselectSms_set()");

	mmi_gprs->SMSService = menu->lCursor[menu->level];

	if (mmi_gprs->gprsHandle)
	{
		mmi_gprs->display = TRUE;
		if(gprs_setServiceSMS(mmi_gprs->SMSService)==MFW_RES_OK)
		{
			info_screen(win, TxtGPRS, TxtChanged, NULL,REMIND_SUCCEED);
		}
		else
		{
			info_screen(win,NULL ,TxtFailed , NULL,REMIND_FAILURE);
		}
	}
	else
	{
		info_screen(win, TxtWillChangeOn, TxtAttach, NULL,REMIND_NORECORD);						// Changed in memory, will set when attach
	}
	
	return 1;
}

void GPRStest(MfwMnu* m, MfwMnuItem* i)
{
	T_GPRS_CONT_REC context;
	T_ACI_RETURN ret1,ret2;
	UBYTE minsize;

	memset(&context.apn,'\0',MAX_APN_LEN);
	strcpy((char *)&context.apn,"INTERNET");
	strcpy((char*)context.pdp_type,"IP");
	memset(&context.pdp_addr,'\0',MAX_PDP_ADDR_LEN);
	context.d_comp = CGDCONT_D_COMP_OMITTED;
	context.h_comp = CGDCONT_H_COMP_OMITTED;
	memset(&context.qos,0,sizeof(T_QOS));
	memset(&context.min_qos,0,sizeof(T_QOS));

	ret1 = sAT_PlusCGDCONT(CMD_SRC_LCL,1,&context);

    {
         TRACE_EVENT_P1("ret1 %d",ret1);
    }
	switch (ret1)
	{
	    case (AT_CMPL):                         /*operation completed*/
	    case (AT_EXCT):
#ifdef _M188_			//glowing,200406-16, for GPRSTest is not referenced in any place
		sAT_PercentCGPCO (CMD_SRC_LCL,1,ACI_PCO_AUTH_PROT_PAP,"web", "web");
		ret2 = sAT_Dn(CMD_SRC_LCL,"*98**1*1#",D_CLIR_OVRD_Default,D_CUG_CTRL_NotPresent,D_TOC_Data);
#endif		
		switch (ret2)
		{
		    case (AT_CMPL):                         /*operation completed*/
		    case (AT_EXCT):
		    default:
			break;
		}
	}
}
/* SH - Only included for GPRS testing */

#ifdef TEST_MMIGPRS
static T_GPRS_CONT_REC		inputCtxt = {
				"", "", "", CGDCONT_D_COMP_OMITTED, CGDCONT_H_COMP_OMITTED,GPRS_QOS_OMITTED, GPRS_QOS_OMITTED};

enum
{
	QOS_PRECED_HIGH = 1,
	QOS_PRECED_NORMAL,
	QOS_PRECED_LOW
};

enum
{
	QOS_BEST = 1
};
/*******************************************************************************

 $Function:    	GPRSContextDefine

 $Description:	Define a context!

 $Returns:		

 $Arguments:	Standard menu option parameters
 
*******************************************************************************/

int GPRSContextDefine(MfwMnu* m, MfwMnuItem* i)
{
 	SHORT				*cids;
	SHORT				count;
	T_MFW_GPRS			*gprs;
	SHORT				numContexts;	
	T_MFW_HND			win = mfw_parent(mfw_header());
	T_QOS 			Qos = {QOS_PRECED_NORMAL, QOS_BEST, QOS_BEST, QOS_BEST, QOS_BEST};
	T_QOS 			Min_Qos = {QOS_PRECED_LOW, QOS_BEST, QOS_BEST, QOS_BEST, QOS_BEST};
#ifdef TRACE_MMIGPRS
	TRACE_FUNCTION("GPRSContextDefine()");
#endif

	/* Check to make sure gprs handle exists */
	
	if (!mmi_gprs->gprsHandle)
	{
		info_screen(win,NULL ,TxtNotAttached , NULL,REMIND_SUCCEED);
		return;
	}

	gprs = ((T_MFW_HDR *) mmi_gprs->gprsHandle)->data;
	numContexts = gprs->data.numContexts;
	
	strcpy(inputCtxt.apn, "Vizzavi UK");
	strcpy(inputCtxt.pdp_addr, "212.183.137.012");
	strcpy(inputCtxt.pdp_type, "ISDN");
	inputCtxt.d_comp = NULL;
	inputCtxt.h_comp = NULL;
	memcpy(&inputCtxt.qos, &Qos, sizeof(Qos));
	memcpy(&inputCtxt.min_qos, &Min_Qos, sizeof(Qos));
	
	if (gprs_definePDPContext(GPRS_CID_1, &inputCtxt)==MFW_RES_OK)
	{
 		#ifdef TRACE_MMIGPRS
 		TRACE_EVENT("GPRSContextDefine: Activated OK.");
 		#endif
 		info_screen(win,NULL ,TxtActivated , NULL,REMIND_SUCCEED);
	}
 	else
	{
 		#ifdef TRACE_MMIGPRS
		TRACE_EVENT("GPRSContextDefine: Failed.");
 		#endif
 		info_screen(win,NULL ,TxtFailed , NULL,REMIND_FAILURE);
	}
	return 0;
}


/*******************************************************************************

 $Function:    	GPRSActivate

 $Description:	

 $Returns:		

 $Arguments:	Standard menu option parameters
 
*******************************************************************************/

int GPRSActivate(MfwMnu* m, MfwMnuItem* i)
{
 	SHORT			*cids;
	SHORT			count;
	T_MFW_GPRS		*gprs;
	SHORT			numContexts;	
	T_MFW_HND		win = mfw_parent(mfw_header());

	TRACE_FUNCTION("GPRSActivate()");

	// Check to make sure gprs handle exists
	
	if (!mmi_gprs->gprsHandle)
	{
		info_screen(win,NULL ,TxtNotAttached , NULL,REMIND_SUCCEED);
		return;
	}
	
	gprs = ((T_MFW_HDR *) mmi_gprs->gprsHandle)->data;
	numContexts = gprs->data.numContexts;

#ifdef TRACE_MMIGPRS
	TRACE_EVENT_P1("GPRSActivate: No. of contexts: %d", numContexts);
#endif

 	if (numContexts>0)
	{
 		cids = (SHORT *) mfwAlloc(sizeof(SHORT)*numContexts);

		for (count=0; count<numContexts; count++)
			cids[count] = gprs->data.contexts[count].id;
  
 	 	if (gprs_contextActivation(CGACT_STATE_ACTIVATED, cids)==MFW_RES_OK)
 		{
			#ifdef TRACE_MMIGPRS
 	 		TRACE_EVENT("GPRSActivate: Activation executing, awaiting confirmation.");
			#endif
 	 		mmi_gprs->gprsPleaseWait_win = mmiPleaseWaitWindow(win, TxtPleaseWait, NULL,FlashSettingData.gprsattachmode);
 		}
 	 	else
 		{
			#ifdef TRACE_MMIGPRS
			TRACE_EVENT("GPRSActivate: Failed.");
			#endif
 	 		info_screen(win,NULL ,TxtFailed , NULL,REMIND_FAILURE);
 		}
 	 	
		mfwFree((U8 *) cids, sizeof(SHORT)*numContexts);
	}
 	else
	{
		#ifdef TRACE_MMIGPRS
 		TRACE_EVENT("GPRSActivate: No contexts to activate.");
		#endif
 		info_screen(win,NULL ,TxtFailed , NULL,REMIND_FAILURE);
	}
	return 0;
}


/*******************************************************************************

 $Function:    	GPRSDeactivate

 $Description:	

 $Returns:		

 $Arguments:	Standard menu option parameters
 
*******************************************************************************/

int GPRSDeactivate(MfwMnu* m, MfwMnuItem* i)
{
	SHORT 			*cids;
	SHORT			count;
	T_MFW_GPRS		*gprs;
	SHORT			numContexts;	
	T_MFW_HND		win = mfw_parent(mfw_header());

	TRACE_FUNCTION("GPRSDeactivate()");

	if (!mmi_gprs->gprsHandle)
	{
		info_screen(win,NULL ,TxtNotAttached , NULL,REMIND_SUCCEED);
		return;
	}
	
	gprs = ((T_MFW_HDR *) mmi_gprs->gprsHandle)->data;
	numContexts = gprs->data.numContexts;

	if (numContexts>0)
	{
	 	cids = (SHORT *) mfwAlloc(sizeof(SHORT)*numContexts);

	 	for (count=0; count<numContexts; count++)
			cids[count] = gprs->data.contexts[count].id;
	  
		if (gprs_contextActivation(CGACT_STATE_DEACTIVATED, cids)==MFW_RES_OK)
 		{
			#ifdef TRACE_MMIGPRS
 	 		TRACE_EVENT("GPRSDeactivate: Deactivated OK.");
			#endif
 	 		info_screen(win,NULL ,TxtDeActivated , NULL,REMIND_SUCCEED);
 		}
 	 	else
 		{
			#ifdef TRACE_MMIGPRS
			TRACE_EVENT("GPRSDeactivate: Failed.");
			#endif
 	 		info_screen(win,NULL ,TxtFailed , NULL,REMIND_FAILURE);
 		}
 	 	
		mfwFree((U8 *) cids, sizeof(SHORT)*numContexts);
	}
	else
 	{
		#ifdef TRACE_MMIGPRS
 		TRACE_EVENT("GPRSDeactivate: No contexts to deactivate.");
		#endif
 		info_screen(win,NULL ,TxtFailed , NULL,REMIND_FAILURE);
	}
	
	return 0;
}
/*******************************************************************************

 $Function:    	GPRSsetQos

 $Description:	Set quality of service

 $Returns:		

 $Arguments:	Standard menu option parameters
 
*******************************************************************************/

int GPRSsetQos(MfwMnu* m, MfwMnuItem* i)
{
	T_MFW_HND		win = mfw_parent(mfw_header());
	T_QOS 			Qos = {QOS_PRECED_NORMAL, QOS_BEST, QOS_BEST, QOS_BEST, QOS_BEST};

	TRACE_FUNCTION("GPRSsetQos()");

	if (gprs_setQOS(mmi_gprs->cid, &Qos)==MFW_RES_OK)
	{
		#ifdef TRACE_MMIGPRS
		TRACE_EVENT("GPRSsetQos: OK");
		#endif
		info_screen(win,NULL ,TxtActivated , NULL,REMIND_SUCCEED);
	}
	else
	{
		#ifdef TRACE_MMIGPRS
		TRACE_EVENT("GPRSsetQos: Failed");
		#endif
		info_screen(win,NULL ,TxtFailed , NULL,REMIND_FAILURE);
	}
	return;
}


/*******************************************************************************

 $Function:    	GPRSsetMinQos

 $Description:	Set minimum quality of service

 $Returns:		

 $Arguments:	Standard menu option parameters
 
*******************************************************************************/

int GPRSsetMinQos(MfwMnu* m, MfwMnuItem* i)
{
	T_MFW_HND		win = mfw_parent(mfw_header());
	T_QOS 			Qos = {QOS_PRECED_LOW, QOS_BEST, QOS_BEST, QOS_BEST, QOS_BEST};

	TRACE_FUNCTION("GPRSsetMinQos()");

	if (gprs_setQOSMin(mmi_gprs->cid, &Qos)==MFW_RES_OK)
	{
		#ifdef TRACE_MMIGPRS
		TRACE_EVENT("GPRSsetMinQos: OK");
		#endif
		info_screen(win,NULL ,TxtActivated , NULL,REMIND_SUCCEED);
	}
	else
	{
		#ifdef TRACE_MMIGPRS
		TRACE_EVENT("GPRSsetMinQos: Failed");
		#endif
		info_screen(win,NULL ,TxtFailed , NULL,REMIND_FAILURE);
	}
	return;
}

int csdConnection(MfwMnu* m, MfwMnuItem* i)
{
}

int setClassBConnection(MfwMnu* m, MfwMnuItem* i)
{
  
}

int setClasCConnection(MfwMnu* m, MfwMnuItem* i)
{

}

int showIPAddress(MfwMnu* m, MfwMnuItem* i)
{	
}

int GPRSCompressionStatus(MfwMnu* m, MfwMnuItem* i)
{	
}


int GPRSCompressionOn(MfwMnu* m, MfwMnuItem* i)
{

}
int GPRSCompressionOff(MfwMnu* m, MfwMnuItem* i)
{

}
int setClassCVoice(MfwMnu* m, MfwMnuItem* i){}	

int setClassCData(MfwMnu* m, MfwMnuItem* i){}	
#endif
void SaveGprsCounter(void)
{ 
	FlashDCTbData.GPRSOUTCOUNTER=FlashDCTbData.GPRSOUTCOUNTER+mmi_gprs->uplink_counter;
    FlashDCTbData.GPRSINCOUNTER=FlashDCTbData.GPRSINCOUNTER+mmi_gprs->downlink_counter;
    flash_DCTbwrite();
}
