/*******************************************************************************

                    CONDAT (UK)

********************************************************************************                                                                              

 This software product is the property of Condat (UK) Ltd and may not be
 disclosed to any third party without the express permission of the owner.                                 
                                                                              
********************************************************************************

 $Project name:	Basic MMI                                                      
 $Project code:	BMI                                                            
 $Module:		SMS
 $File:		    mmiSatInput.c
 $Revision:		1.0                                                       
                                                                              
 $Author:		Condat(UK)                                                         
 $Date:		    25/10/00                                                      
                                                                               
********************************************************************************
                                                                              
 Description:
 
    Implementation of MMI SIM Application Toolkit (SAT)
                        
********************************************************************************

 $History: mmiSatInput.c

    25/10/00			Original Condat(UK) BMI version.	
    16/08/02            verion0.100 zhaowm modified
       
 $End

*******************************************************************************/

#define ENTITY_MFW

/* includes */
#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#if defined (NEW_FRAME)

    #include "typedefs.h"
    #include "vsi.h"
    #include "pei.h"
    #include "custom.h"
    #include "gsm.h"

#else

    #include "stddefs.h"
    #include "custom.h"
    #include "gsm.h"
    #include "vsi.h"

#endif
#include "mfw_sys.h"

#include "mfw_mfw.h"
#include "mfw_win.h"
#include "mfw_kbd.h"
#include "mfw_edt.h"
#include "mfw_lng.h"
#include "mfw_icn.h"
#include "mfw_phb.h"
#include "mfw_sim.h"
#include "mfw_nm.h"
#include "mfw_sms.h"
#include "mfw_mnu.h"
#include "mfw_sat.h"
#include "mfw_tim.h"
#include "mfw_ss.h"
#include "mfw_str.h"

#include "dspl.h"

#include "MmiMmi.h"
#include "MmiDummy.h"
#include "MmiDialogs.h"
#include "MmiLists.h"

#include "MmiMain.h"
#include "MmiStart.h"
#include "MmiPins.h"
#include "MmiMenu.h"
#include "MmiSoftKeys.h"
#include "MmiSounds.h"
#include "MmiCall.h"

#include "mmiSat_i.h"
#include "mmiEditor.h"
#include "psa_util.h"

#include "cus_aci.h"
#include "p_sim.h"
#include "pcm.h"
#include "mmiusersymbolinput.h"

/* 2004/04/12 sunsj add */
#include "mmitextinput.h"

//#define TDTRACE 1
#include "mmiutilapi.h"

static void sat_editor_cb (T_MFW_HND win, USHORT Identifier, USHORT event);

/*********************************************************************
 *
 * SUB WINDOW  SAT_GET_KEY
 *
 *********************************************************************/
#define SAT_GET_KEY_BUFFER_SIZE (UBYTE)(1+2+2) /* two spares for mfwEdt *///zhaowm 2002/08/05

typedef struct
{
    T_MMI_CONTROL  mmi_control;
    T_MFW_HND      win;
    T_SAT_CMD     *sat_command; /* pointer to sat_command in parent */
    T_EDITOR_DATA  editor_data;
    char          *edt_buffer; /* buffer for editor */ 
    // T_MFW_HND     key_editor;//zhaowm2002/7/30 to be destroy editor_win
    T_MFW_HND      kbd;  /* sbh - keyboard handler, so window can be destroyed by user */
} T_sat_get_key;

static void sat_get_key_destroy (T_MFW_HND own_window);
static void sat_get_key_exec (T_MFW_HND win, USHORT event, SHORT value, T_SAT_CMD * sat_command);

/*******************************************************************************

 $Function:    	sat_get_key_create

 $Description:	Creation of an instance for the SAT GET KEY dialog.Window must 
                be available after reception of SAT command,only one instance.

 
 $Returns:		mfw window handler 

 $Arguments:	parent_window - Parent window handler
 
*******************************************************************************/

T_MFW_HND sat_get_key_create (T_MFW_HND parent_window)
{
    T_sat_get_key * data = (T_sat_get_key *)ALLOC_MEMORY (sizeof (T_sat_get_key));
    T_MFW_WIN     * win;

    data->win = win_create (parent_window, 0, E_WIN_VISIBLE, (T_MFW_CB)sat_win_cb); // c030 rsa

    if( data->win EQ NULL )
        return NULL;

    /*
     * Create window handler
     */
    data->mmi_control.dialog   = (T_DIALOG_FUNC)sat_get_key_exec;
    data->mmi_control.data     = data;
    win                        = ((T_MFW_HDR *)data->win)->data;
    win->user                  = (MfwUserDataPtr)data;

    data->kbd      = kbdCreate( data->win, KEY_ALL, (T_MFW_CB) sat_kbd_cb); /* sbh - add keyboard handler */

    /*
     * return window handle
     */
    win_show(data->win);
    return data->win;
}

/*******************************************************************************

 $Function:    	sat_get_key_destroy

 $Description:	Destroy the sat get inkey dialog.

 
 $Returns:		none

 $Arguments:	own_window - window handler
 
*******************************************************************************/

static void sat_get_key_destroy (T_MFW_HND own_window)
{
    T_MFW_WIN * win_data   = ((T_MFW_HDR *)own_window)->data;
    T_sat_get_key * data = (T_sat_get_key *)win_data->user;

    if( data )
    {
        /*
         * Delete WIN Handler
         */
        win_delete (data->win);
        /*
         * Free Memory
         */
        FREE_MEMORY ((U8 *)data->edt_buffer,
                     SAT_GET_KEY_BUFFER_SIZE * sizeof(char));
        sat_destroy_TEXT_ASCIIZ (data->editor_data.TextString); /* label */
        FREE_MEMORY ((U8 *)data, sizeof (T_sat_get_key));
    }
}

/*******************************************************************************

 $Function:    	sat_get_key_exec

 $Description:	Dialog function for sat_get_key_exec window.

 
 $Returns:		none

 $Arguments:	win - window handler
                event -window event
                value - unique id
                sat_command - Sat command info
 
*******************************************************************************/

static void sat_get_key_exec (T_MFW_HND win, USHORT event, SHORT value, T_SAT_CMD * sat_command)
{
    T_MFW_WIN       * win_data   = ((T_MFW_HDR *)win)->data;
    T_sat_get_key   * data       = (T_sat_get_key *)win_data->user;

    T_SAT_TXT       * txt;
    T_SAT_RES sat_res;

    TRACE_FUNCTION ("sat_get_key_exec()");

    switch( event )
    {
    case SAT_GET_KEY:
        data->sat_command = sat_command; /* save a pointer to the parameter for later use in callbacks */
        txt = (T_SAT_TXT *)&sat_command->c.text;

        /* allocate a buffer to hold the edited chars */
        data->edt_buffer = (char *)ALLOC_MEMORY (
                                                SAT_GET_KEY_BUFFER_SIZE * sizeof(char));

        /*
         * Setup an Editor to get a single key 
         */
        if( (sat_command->qual & SAT_M_INKEY_HELP_AVAIL) NEQ 0 )
        {
            data->editor_data.AlternateLeftSoftKey  = TxtHelp; /* help available */
        }
        else
        {
            data->editor_data.AlternateLeftSoftKey  = TxtNull; /* no help available */
        }
        if( (sat_command->qual & SAT_M_INKEY_YES_NO) NEQ 0 )
        {
            /* yes/no decision */
            data->editor_data.LeftSoftKey    = TxtYes;
            data->editor_data.RightSoftKey   = TxtNo;
            data->editor_data.mode           = ONLYREAD_MODE;//E_EDIT_READ_ONLY_MODE; /* treat as read-only, thus ignoring normal keys */
        }
        else
        {
            /* normal single key request */
            data->editor_data.LeftSoftKey    = TxtSoftOK;
            data->editor_data.RightSoftKey   = TxtDelete;//TxtSoftBack;  //zhaowm 2003.0416
            if( (sat_command->qual & SAT_M_INKEY_ALPHA) EQ 0 ) /* 0=digits, 1=alpha */
            {
                data->editor_data.mode = PHONENUMONLY_MODE;
                ;
            }
            else
            {
                //data->editor_data.mode = E_EDIT_ALPHA_MODE;
                data->editor_data.mode = TEXTINPUT_MODE; /* 2003/12/04 sunsj modify */
				data->editor_data.InputMode = PINYIN_INPUT_MODE;	/* 2004/04/12 sunsj add */
            }
        }
        data->editor_data.TextId                 = TxtNull;
        if( txt->len > 0 )
        {
            data->editor_data.TextString     = sat_create_TEXT_ASCIIZ (txt); /* label */
        }
        else
        {
            data->editor_data.TextString     =  NULL; /* no label */
        }
        data->editor_data.min_enter              = 0; /* get a single key */
        data->editor_data.timeout                = TWENTY_SECS; 
        data->editor_data.Identifier             = event;
        if( (sat_command->qual & SAT_M_INKEY_HELP_AVAIL) NEQ 0 )
        {
            data->editor_data.AlternateLeftSoftKey  = TxtHelp; /* help available */
        }
        else
        {
            data->editor_data.AlternateLeftSoftKey  = 0;       /* no help available */
        }
        data->editor_data.hide                   = FALSE; 


        /* provide an empty zero terminated buffer */
        data->edt_buffer[0] = '\0'; 

        /*
         * Start the Editor
         */
        //        data->key_editor=editor_start_common (win, data->edt_buffer, 3, &data->editor_data, (T_EDIT_CB)sat_editor_cb);//zhaowm2002/7/30
        //        if (data->key_editor== NULL)            
        if( editor_start_common (win, data->edt_buffer, 3, &data->editor_data, (T_EDIT_CB)sat_editor_cb) == NULL )
        {
            sat_res[SAT_ERR_INDEX] = SAT_RES_IMPOSSIBLE;
            sat_res[SAT_AI_INDEX]  = SatResAiNoCause;
            sat_done (sat_command, sat_res, sizeof(T_SAT_RES));
            sat_get_key_destroy (data->win);
        }
        break;

        /* sbh - all window types are being provided with this event to destroy the window */
    case SAT_DESTROY_WINDOW:
        sat_get_key_destroy (data->win);
        break;
        /* ...sbh */

    default:
        TRACE_EVENT ("sim_get_key_exec() unexpected event");
        break;
    }
}

/*********************************************************************
 *
 * SUB WINDOW  SAT_GET_STRING
 *
 *********************************************************************/
#define SAT_GET_STRING_BUFFER_SIZE (UBYTE)255
typedef struct
{
    T_MMI_CONTROL  mmi_control;
    T_MFW_HND      win;
    T_SAT_CMD     *sat_command; /* pointer to sat_command in parent */
    T_EDITOR_DATA  editor_data;
    char          *edt_buffer; /* buffer for editor */ 
    //  T_MFW_HND     string_editor;//zhaowm2002/7/30 to be destroy editor_win
    U16            max_chars;
    T_MFW_HND       kbd;  /* sbh - keyboard handler, so window can be destroyed by user */
} T_sat_get_string;

typedef struct
{
    T_SAT_TXT txt;
    UBYTE     buffer[1]; /* open array */ 
} T_sat_get_string_result; /* NOTE: txt and buffer shall be packed */

static void sat_get_string_destroy (T_MFW_HND own_window);
static void sat_get_string_exec (T_MFW_HND win, USHORT event, SHORT value, T_SAT_CMD * sat_command);

/*******************************************************************************

 $Function:    	sat_get_string_create

 $Description:	Creation of an instance for the SAT GET STRING dialog.Window must 
                be available after reception of SAT command,only one instance.

 
 $Returns:		window 

 $Arguments:	parent_window - parent window
*******************************************************************************/
T_MFW_HND sat_get_string_create (T_MFW_HND parent_window)
{
    T_sat_get_string * data = (T_sat_get_string *)ALLOC_MEMORY (sizeof (T_sat_get_string));
    T_MFW_WIN        * win;

    data->win = win_create (parent_window, 0, E_WIN_VISIBLE, (T_MFW_CB)sat_win_cb);  // c030 rsa

    if( data->win EQ NULL )
        return NULL;

    /*
     * Create window handler
     */
    data->mmi_control.dialog   = (T_DIALOG_FUNC)sat_get_string_exec;
    data->mmi_control.data     = data;
    win                        = ((T_MFW_HDR *)data->win)->data;
    win->user                  = (MfwUserDataPtr)data;


    data->kbd      = kbdCreate( data->win, KEY_ALL, (T_MFW_CB) sat_kbd_cb); /* sbh - add keyboard handler */

    /*
     * return window handle
     */
    win_show(data->win);
    return data->win;
}

/*******************************************************************************

 $Function:    	sat_get_string_destroy

 $Description:	Destroy the sat get string dialog.

 
 $Returns:		none

 $Arguments:	own_window - current window.
*******************************************************************************/
static void sat_get_string_destroy (T_MFW_HND own_window)
{
    T_MFW_WIN * win_data    = ((T_MFW_HDR *)own_window)->data;
    T_sat_get_string * data = (T_sat_get_string *)win_data->user;

    if( data )
    {
        /*
         * Delete WIN Handler
         */
        win_delete (data->win);
        /*
         * Free Memory
         */
        FREE_MEMORY ((U8 *)data->edt_buffer,
                     data->max_chars * sizeof(char));
        sat_destroy_TEXT_ASCIIZ (data->editor_data.TextString); /* label */
        FREE_MEMORY ((U8 *)data, sizeof (T_sat_get_string));
    }
}

/*******************************************************************************

 $Function:    	sat_get_string_exec

 $Description:	Dialog function for sat_get_string window.

 
 $Returns:		none

 $Arguments:	win - window handle
                event - window event
                value - unique id
                sat_command - Sat Command info.
*******************************************************************************/
static void sat_get_string_exec (T_MFW_HND win, USHORT event, SHORT value, T_SAT_CMD * sat_command)
{
    T_MFW_WIN       * win_data   = ((T_MFW_HDR *)win)->data;
    T_sat_get_string   * data       = (T_sat_get_string *)win_data->user;
    T_SAT_INPUT     * inp;
    T_SAT_TXT      * defRsp;
    T_SAT_RES sat_res;

    TRACE_FUNCTION ("sat_get_string_exec()");

    switch( event )
    {
    case SAT_GET_STRING:
        data->sat_command = sat_command; /* save a pointer to the parameter for later use in callbacks */
        inp = (T_SAT_INPUT *)&sat_command->c.inp;

        /* allocate a buffer to hold the edited chars */
        data->max_chars = (U16)(inp->rspMax)+1+2;  //zhaowm 2002/08/05
        data->edt_buffer = (char *)ALLOC_MEMORY (
                                                data->max_chars * sizeof(char));

        /*
         * Setup an Editor to get a string 
         */
        data->editor_data.LeftSoftKey            = TxtSoftOK;
        if( (sat_command->qual & SAT_M_INPUT_HELP_AVAIL) NEQ 0 )
        {
            data->editor_data.AlternateLeftSoftKey  = TxtHelp; /* help available */
        }
        else
        {
            data->editor_data.AlternateLeftSoftKey  = TxtNull; /* no help available */
        }
        //data->editor_data.RightSoftKey           = TxtSoftBack;
        data->editor_data.RightSoftKey           = TxtDelete;    //zhaowm 2003.0416

        data->editor_data.hide                   = ((sat_command->qual & SAT_M_INPUT_NOECHO) NEQ 0);
        if( (sat_command->qual & SAT_M_INPUT_ALPHA) EQ 0 ) /* 0=digits, 1=alpha */
        {
            data->editor_data.mode = PHONENUMONLY_MODE;
        }
        else
        {
            //data->editor_data.mode = E_EDIT_ALPHA_MODE;
            data->editor_data.mode = TEXTINPUT_MODE; /* 2003/12/04 sunsj modify */
        }
        data->editor_data.TextId                 = TxtNull;
        if( inp->prompt.len > 0 )
        {
            data->editor_data.TextString     = sat_create_TEXT_ASCIIZ (&inp->prompt); /* label */
        }
        else
        {
            data->editor_data.TextString     =  NULL; /* no label */
        }
        data->editor_data.min_enter              = inp->rspMin;
        data->editor_data.timeout                = TWENTY_SECS; 
        data->editor_data.Identifier             = event;

        defRsp = &inp->defRsp;
        if( defRsp->len > 0 ) /* default response available? */
        {
            /* provide a zero terminated buffer, initially containing the default response */
            sat_TEXT_to_ASCIIZ (data->edt_buffer, defRsp);
        }
        else
        {
            /* no default response: provide an empty buffer */
            data->edt_buffer[0] = '\0';
        }

        /*
         * Start the Editor
         */
        //     data->string_editor=editor_start_common (win, data->edt_buffer, data->max_chars, &data->editor_data, (T_EDIT_CB)sat_editor_cb);//zhaowm2002/7/30
        //     if (data->string_editor== NULL)
        if( editor_start_common (win, data->edt_buffer, data->max_chars, &data->editor_data, (T_EDIT_CB)sat_editor_cb) == NULL )
        {
            sat_res[SAT_ERR_INDEX] = SAT_RES_IMPOSSIBLE;
            sat_res[SAT_AI_INDEX]  = SatResAiNoCause;
            sat_done (sat_command, sat_res, sizeof(T_SAT_RES));
            sat_get_string_destroy (data->win);
        }
        break;

        /* sbh - all window types are being provided with this event to destroy the window */
    case SAT_DESTROY_WINDOW:
        sat_get_string_destroy (data->win);
        break;
        /* ...sbh */

    default:
        TRACE_EVENT ("sim_get_string_exec() unexpected event");
        break;
    }
}

/*********************************************************************
 *
 * SUB WINDOW  SAT_DISPLAY_TEXT
 *
 *********************************************************************/
typedef struct
{
    T_MMI_CONTROL  mmi_control;
    T_MFW_HND      win;
    T_SAT_CMD     *sat_command; /* pointer to sat_command in parent */
    T_EDITOR_DATA  editor_data;
    char          *info_buffer; /* buffer for info */ 
    // T_MFW_HND     display_editor;//zhaowm2002/12/05 to be destroy editor_win
    T_MFW_HND       kbd;  /* sbh - keyboard handler, so window can be destroyed by user */
} T_sat_display_text;

static void sat_display_text_destroy (T_MFW_HND own_window);
static void sat_display_text_exec (T_MFW_HND win, USHORT event, SHORT value,  T_SAT_CMD * sat_command);

/*******************************************************************************

 $Function:    	sat_get_string_exec

 $Description:	Creation of an instance for the SAT DISPLAY TEXT dialog.Window 
                must be available after reception of SAT command,only one instance.

 $Returns:		Window handle

 $Arguments:	parent_window - parent window
*******************************************************************************/

T_MFW_HND sat_display_text_create (T_MFW_HND parent_window)
{
    T_sat_display_text * data = (T_sat_display_text *)ALLOC_MEMORY (sizeof (T_sat_display_text));
    T_MFW_WIN       * win;

    TRACE_FUNCTION("sat_display_text_create");

    data->win = win_create (parent_window, 0, E_WIN_VISIBLE, (T_MFW_CB)sat_win_cb); // c030 rsa

    if( data->win EQ NULL )
        return NULL;

    /*
     * Create window handler
     */
    data->mmi_control.dialog   = (T_DIALOG_FUNC)sat_display_text_exec;
    data->mmi_control.data     = data;
    win                        = ((T_MFW_HDR *)data->win)->data;
    win->user                  = (MfwUserDataPtr)data;

    data->kbd      = kbdCreate( data->win, KEY_ALL, (T_MFW_CB) sat_kbd_cb); /* sbh - add keyboard handler */

    /*
     * return window handle
     */
    win_show(data->win);
    return data->win;
}

/*******************************************************************************

 $Function:    	sat_display_text_destroy

 $Description:	Destroy the sat play tone dialog.

 $Returns:		none

 $Arguments:	own_window - current window
*******************************************************************************/
static void sat_display_text_destroy (T_MFW_HND own_window)
{
    T_MFW_WIN * win_data   = ((T_MFW_HDR *)own_window)->data;
    T_sat_display_text * data = (T_sat_display_text *)win_data->user;

    TRACE_FUNCTION("sat_display_text_destroy");


    if( data )
    {
        /*
         * Delete WIN Handler
         */
        win_delete (data->win);
        /*
         * Free Memory
         */
        sat_destroy_TEXT_ASCIIZ (data->info_buffer); /* displayed text */
        FREE_MEMORY ((U8 *)data, sizeof (T_sat_display_text));
    }

}

/*******************************************************************************

 $Function:    	sat_display_text_exec

 $Description:	Dialog function for sat_display_text_exec window.

 $Returns:		none

 $Arguments:	win - window
                event - window event
                value - unique id
                sat_command - sat command data.
*******************************************************************************/

static void sat_display_text_exec (T_MFW_HND win, USHORT event, SHORT value,  T_SAT_CMD * sat_command)
{

    T_MFW_WIN       * win_data   = ((T_MFW_HDR *)win)->data;
    T_sat_display_text * data       = (T_sat_display_text *)win_data->user;
    T_SAT_RES sat_res;

    ULONG             time;

    TRACE_FUNCTION ("sat_display_text_exec()");

    switch( event )
    {
    case SAT_DISPLAY_TEXT:
        data->sat_command = sat_command;

        /*
         * Setup an Editor to display a string 
         */
        data->editor_data.LeftSoftKey            = TxtSoftOK;
        data->editor_data.AlternateLeftSoftKey   = TxtNull; 
        data->editor_data.RightSoftKey           = TxtSoftBack;
        data->editor_data.hide                   = FALSE;
        data->editor_data.mode                   = ONLYREAD_MODE;//E_EDIT_READ_ONLY_MODE; 
        data->editor_data.TextId                 = TxtNull;
        data->editor_data.TextString             = NULL; /* no label */
        data->editor_data.min_enter              = 0;
        if( (data->sat_command->qual & SAT_M_TEXT_WAIT) NEQ 0 )
        {
            data->editor_data.timeout         = TWO_MIN;
            data->editor_data.Identifier      = SAT_DISPLAY_TEXT_WAIT;
        }
        else
        {
            data->editor_data.timeout         = TEN_SECS;
            data->editor_data.Identifier      = event;
        }

        /* provide a buffer, containing the string to be displayed */
        memset(data->info_buffer, '\0', sizeof(data->info_buffer));   
        data->info_buffer   = sat_create_TEXT_ASCIIZ (&sat_command->c.text);//zhaowm

        /*
         * Start the Editor
         */

        if( editor_start_common (win, data->info_buffer, sat_command->c.text.len+1, &data->editor_data, (T_EDIT_CB)sat_editor_cb) == NULL )
        //      data->display_editor=editor_start_common (win, data->info_buffer, sat_command->c.text.len+1, &data->editor_data, (T_EDIT_CB)sat_editor_cb);//zhaowm2002/12/05
        //      if (data->display_editor== NULL)
        //if (editor_start_common (win, data->info_buffer, sat_command->c.text.len+1, &data->editor_data, (T_EDIT_CB)sat_editor_cb)== NULL )//zhaowm
        {
            sat_res[SAT_ERR_INDEX] = SAT_RES_IMPOSSIBLE;
            sat_res[SAT_AI_INDEX]  = SatResAiNoCause;
            sat_done (sat_command, sat_res, sizeof(T_SAT_RES));
            sat_display_text_destroy (data->win);
        }
        break;

        /* sbh - all window types are being provided with this event to destroy the window */
    case SAT_DESTROY_WINDOW:
        sat_display_text_destroy (data->win);
        break;
        /* ...sbh */

    default:
        TRACE_EVENT ("sim_display_text_exec() unexpected event");
        break;
    }
}


/*******************************************************************************

 $Function:    	sat_ASCII_to_TEXT

 $Description:	This routine converts an ASCIIZ string into an MFW SAT text 
                descriptor the coding scheme is taken accordign to the code in <txt>

 $Returns:		none

 $Arguments:	txt  - text info
                destination - text destination 
                source - text source

*******************************************************************************/

static void sat_ASCII_to_TEXT (T_SAT_TXT * txt, UBYTE * destination, UBYTE * source)
{
    UBYTE * temp_buffer;

    switch( txt->code )
    {
    case MFW_DCS_7bits:
        temp_buffer = (UBYTE *)ALLOC_MEMORY(txt->len+1); 
        sat_ascii_to_gsm ((char *)temp_buffer, (char *)source, (U16)(txt->len));
        utl_cvt8To7 (temp_buffer, txt->len, destination, 0);
        FREE_MEMORY ((U8 *)temp_buffer, txt->len+1);
        break;
    case MFW_DCS_8bits:
        sat_ascii_to_gsm ((char *)destination, (char *)source, (U16)(txt->len));
        break;
    case MFW_DCS_UCS2:
        sat_ascii_to_ucode ((wchar_t *)destination, (char *)source, (U16)(txt->len));
        break;
    default:
        sat_ascii_to_gsm ((char *)destination, (char *)source, (U16)(txt->len));
        TRACE_EVENT("sat_ASCII_to_TEXT() unexp. DCS");
        break;
    }
}

#ifdef INTEGRATION_SEPT00

/*******************************************************************************

 $Function:    	sat_UCODE_to_TEXT

 $Description:	This routine converts an UCODE string into an MFW SAT text descriptor
            the coding scheme is taken accordign to the code in <txt>


 $Returns:		none

 $Arguments:	txt  - text info
                destination - text destination 
                source - text source

*******************************************************************************/
static void sat_UCODE_to_TEXT (T_SAT_TXT * txt, UBYTE * destination, cp_wstring_t source)
{
    UBYTE * temp_buffer;
    UBYTE size;
    UBYTE outLen;

    switch( txt->code )
    {
    case MFW_DCS_7bits:
        size = (source->length/sizeof(wchar_t))+1;
        temp_buffer = (UBYTE *)ALLOC_MEMORY(size); 
        string_UCS2_to_GSM(
                          MFW_DCS_7bits,  // gsm format 
                          (wstring_t *)source,// source unicode string, includes own length
                          size,           // maximum output size
                          temp_buffer,    // destination byte array
                          &outLen     // actual length in bytes is stored here.
                          );
        txt->len = utl_cvt8To7 (temp_buffer, outLen, destination, 0);
        FREE_MEMORY ((U8 *)temp_buffer, size);
        break;
    case MFW_DCS_8bits:
        string_UCS2_to_GSM(
                          MFW_DCS_7bits,  // gsm format 
                          (wstring_t *)source,// source unicode string, includes own length
                          size,           // maximum output size
                          destination,    // destination byte array
                          &txt->len       // actual length in bytes is stored here.
                          );
        break;
    case MFW_DCS_UCS2:
        txt->len = source->length*sizeof(wchar_t); // txt count is in number of bytes
        memcpy(destination, source, txt->len);
        break;
    default:
        string_UCS2_to_GSM(
                          MFW_DCS_7bits,  // gsm format 
                          (wstring_t *)source,// source unicode string, includes own length
                          size,           // maximum output size
                          destination,    // destination byte array
                          &txt->len       // actual length in bytes is stored here.
                          );
        TRACE_EVENT("sat_UCODE_to_TEXT() unexp. DCS");
        break;
    }
}
#endif /* INTEGRATION_SEPT00 */

/*******************************************************************************

 $Function:    	sat_get_key_send_result

 $Description:	


 $Returns:		none

 $Arguments:	data  - key info

*******************************************************************************/

static void sat_get_key_send_result (T_sat_get_key * data)
{
    T_SAT_RES sat_res;
    struct
    {
        T_SAT_TXT txt;
        UBYTE      buffer[SAT_GET_KEY_BUFFER_SIZE]; /* buffer for result */ 
    } buf; /* NOTE: txt and buffer shall be packed */

    /* update the output parameter */

    /* set the required DCS for the result to be returned */
    if( (data->sat_command->qual & SAT_M_INKEY_UCS2) NEQ 0 )
    {
        buf.txt.code = MFW_DCS_UCS2;
    }
    else
    {
        buf.txt.code = MFW_DCS_8bits; /* use the SMS default alphabet */
    }

    buf.txt.text = sizeof(T_SAT_TXT);
    buf.txt.len = 1;

    /* convert and send the result */
    sat_ASCII_to_TEXT(&buf.txt, buf.buffer, (UBYTE *)data->edt_buffer);
    sat_res[SAT_ERR_INDEX] = SatResSuccess;
    sat_res[SAT_AI_INDEX]  = SatResAiNoCause;
    satString (data->sat_command,  &buf.txt, sat_res, sizeof(T_SAT_RES));
}

/*******************************************************************************

 $Function:    	sat_get_string_send_result

 $Description:	


 $Returns:		none

 $Arguments:	data  - string info

*******************************************************************************/
static void sat_get_string_send_result (T_sat_get_string * data)
{
    T_SAT_RES sat_res;
    T_sat_get_string_result * buf;
    U16 length;
    U16 size;

    /* allocate sufficient temporary space to return the result (avoid huge stack usage) */
    length = strlen(data->edt_buffer);
    size = sizeof(T_sat_get_string_result) + length;
    buf = (T_sat_get_string_result *) ALLOC_MEMORY(size); 

    /* update the output parameter */

    /* set the required DCS for the result to be returned */
    if( (data->sat_command->qual & SAT_M_INPUT_UCS2) NEQ 0 )
    {
        buf->txt.code = MFW_DCS_UCS2;
    }
    else if( (data->sat_command->qual & SAT_M_INPUT_PACKING) NEQ 0 )
    {
        buf->txt.code = MFW_DCS_7bits; /* use the packed SMS alphabet */
    }
    else
    {
        buf->txt.code = MFW_DCS_8bits; /* use the SMS default alphabet */
    }

    buf->txt.text = sizeof(T_SAT_TXT);
    buf->txt.len = length;

    /* convert and send the result */
    sat_ASCII_to_TEXT(&buf->txt, buf->buffer, (UBYTE *)data->edt_buffer);
    sat_res[SAT_ERR_INDEX] = SatResSuccess;
    sat_res[SAT_AI_INDEX]  = SatResAiNoCause;
    satString (data->sat_command,  &buf->txt, sat_res, sizeof(T_SAT_RES));

    FREE_MEMORY((U8 *)buf, size);
}

/*******************************************************************************

 $Function:    	sat_editor_cb

 $Description:	Sat editor call back function.


 $Returns:		none

 $Arguments:	win  - window
                Identifier - unique id
                event - event id
                
*******************************************************************************/
static void sat_editor_cb (T_MFW_HND win, USHORT Identifier, USHORT event)
{
    T_MFW_WIN    * win_data = ((T_MFW_HDR *)win)->data;
    T_SAT_RES sat_res;

    TRACE_EVENT ("sat_editor_cb()");
    {
	/* 2004/04/12 sunsj remove */
        //char temp[60];
        //sprintf(temp,"event number %d",event);
        //TRACE_EVENT(temp);
    }
    if( (win EQ NULL) || (win_data EQ NULL) )
        return;

    switch( Identifier )
    {
    case SAT_GET_KEY:
        {
            T_sat_get_key * data = (T_sat_get_key *)win_data->user;

            if( data EQ NULL )
                return;

            switch( event )
            {
            case INFO_KCD_LEFT: /* OK or single digit key or YES */
                /* update the output parameter */
                if( (data->sat_command->qual & SAT_M_INKEY_YES_NO) NEQ 0 )
                {
                    data->edt_buffer[0] = 0x01; /* means YES */
                    data->edt_buffer[1] = 0x00; /* means YES */
                }

                sat_get_key_send_result (data);
                //editor_destroy(data->key_editor);
                sat_get_key_destroy (data->win);
                break;
            case INFO_KCD_ALTERNATELEFT: /* help */
                sat_res[SAT_ERR_INDEX] = SAT_RES_USER_HELP;
                sat_res[SAT_AI_INDEX]  = SatResAiNoCause;
                sat_done (data->sat_command, sat_res, sizeof(T_SAT_RES));
                //editor_destroy(data->key_editor);
                sat_get_key_destroy (data->win);
                break;
            case INFO_KCD_RIGHT: /* back or NO */
                if( (data->sat_command->qual & SAT_M_INKEY_YES_NO) NEQ 0 )
                {
                    data->edt_buffer[0] = 0x00; /* means NO */
                    data->edt_buffer[1] = 0x00; /* means NO */
                    sat_get_key_send_result (data);
                }
                else
                {
                    sat_res[SAT_ERR_INDEX] = SAT_RES_USER_BACK;
                    sat_res[SAT_AI_INDEX]  = SatResAiNoCause;
                    sat_done (data->sat_command, sat_res, sizeof(T_SAT_RES));
                }
                //editor_destroy(data->key_editor);
                sat_get_key_destroy (data->win);
                break;
            case INFO_KCD_CLEAR: /* clear on empty buffer or long clear */
            case INFO_KCD_HUP:      
                sat_res[SAT_ERR_INDEX] = SAT_RES_USER_ABORT;
                sat_res[SAT_AI_INDEX]  = SatResAiNoCause;
                sat_done (data->sat_command, sat_res, sizeof(T_SAT_RES));
                //editor_destroy(data->key_editor);
                sat_get_key_destroy (data->win);
                break;
            case INFO_TIMEOUT: /* timeout */
                sat_res[SAT_ERR_INDEX] = SAT_RES_USER_NO_RESP;
                sat_res[SAT_AI_INDEX]  = SatResAiNoCause;
                sat_done (data->sat_command, sat_res, sizeof(T_SAT_RES));
                sat_get_key_destroy (data->win);
                break;
            default:
                TRACE_EVENT("sat_editor_cb()_key unexp. event");
                break;
            }
            // sat_get_key_destroy (data->win);
        }
        break;
    case SAT_GET_STRING:
        {
            T_sat_get_string * data = (T_sat_get_string *)win_data->user;

            if( data EQ NULL )
                return;

            switch( event )
            {
            case INFO_KCD_LEFT: /* OK */
                sat_get_string_send_result (data);
                //	editor_destroy(data->string_editor);
                sat_get_string_destroy (data->win);
                break;
            case INFO_KCD_ALTERNATELEFT: /* help */
                sat_res[SAT_ERR_INDEX] = SAT_RES_USER_HELP;
                sat_res[SAT_AI_INDEX]  = SatResAiNoCause;
                sat_done (data->sat_command, sat_res, sizeof(T_SAT_RES));
                //	editor_destroy(data->string_editor);
                sat_get_string_destroy (data->win);
                break;
            case INFO_KCD_RIGHT: /* back */
                sat_res[SAT_ERR_INDEX] = SAT_RES_USER_BACK;
                sat_res[SAT_AI_INDEX]  = SatResAiNoCause;
                sat_done (data->sat_command, sat_res, sizeof(T_SAT_RES));
                //	editor_destroy(data->string_editor);
                sat_get_string_destroy (data->win);
                break;
            case INFO_KCD_CLEAR: /* clear on empty buffer or long clear */
            case INFO_KCD_HUP:      
                sat_res[SAT_ERR_INDEX] = SAT_RES_USER_ABORT;
                sat_res[SAT_AI_INDEX]  = SatResAiNoCause;
                sat_done (data->sat_command, sat_res, sizeof(T_SAT_RES));
                //	editor_destroy(data->string_editor);
                sat_get_string_destroy (data->win);
                break;
            case INFO_TIMEOUT: /* timeout */
                sat_res[SAT_ERR_INDEX] = SAT_RES_USER_NO_RESP;
                sat_res[SAT_AI_INDEX]  = SatResAiNoCause;
                sat_done (data->sat_command, sat_res, sizeof(T_SAT_RES));
                sat_get_string_destroy (data->win);
                break;
            default:
                TRACE_EVENT("sat_editor_cb()_STRING unexp. event");
                break;
            }
            //sat_get_string_destroy (data->win);
        }
        break;

    case SAT_DISPLAY_TEXT:
        {
            T_sat_display_text   * data     = (T_sat_display_text *)win_data->user;

            switch( event )
            {
            case INFO_KCD_RIGHT: /* back */
                sat_res[SAT_ERR_INDEX] = SAT_RES_USER_BACK;
                sat_res[SAT_AI_INDEX]  = SatResAiNoCause;
                sat_done (data->sat_command, sat_res, sizeof(T_SAT_RES));
                // editor_destroy(data->display_editor);         //zhaowm 2002.12.05
                break;
            case INFO_TIMEOUT: /* timeout */
                sat_res[SAT_ERR_INDEX] = SatResSuccess;
                sat_res[SAT_AI_INDEX]  = SatResAiNoCause;
                sat_done (data->sat_command, sat_res, sizeof(T_SAT_RES));
                break;

            case INFO_KCD_LEFT: /* OK */
                sat_res[SAT_ERR_INDEX] = SatResSuccess;
                sat_res[SAT_AI_INDEX]  = SatResAiNoCause;
                sat_done (data->sat_command, sat_res, sizeof(T_SAT_RES));
                //  editor_destroy(data->display_editor);
                break;
            case INFO_KCD_CLEAR: /* abort */
            case INFO_KCD_HUP:      
                sat_res[SAT_ERR_INDEX] = SAT_RES_USER_ABORT;
                sat_res[SAT_AI_INDEX]  = SatResAiNoCause;
                sat_done (data->sat_command, sat_res, sizeof(T_SAT_RES));
                //  editor_destroy(data->display_editor);
                break;
            default:
                TRACE_EVENT("sat_editor_cb() unexp. event");
                break;
            }
            sat_display_text_destroy (data->win);
        }
        break;

    case SAT_DISPLAY_TEXT_WAIT:
        {
            T_sat_display_text   * data     = (T_sat_display_text *)win_data->user;

            switch( event )
            {
            case INFO_TIMEOUT: /* timeout */
                sat_res[SAT_ERR_INDEX] = SAT_RES_USER_NO_RESP;
                sat_res[SAT_AI_INDEX]  = SatResAiNoCause;
                sat_done (data->sat_command, sat_res, sizeof(T_SAT_RES));
                break;
            case INFO_KCD_LEFT: /* OK */
                sat_res[SAT_ERR_INDEX] = SatResSuccess;
                sat_res[SAT_AI_INDEX]  = SatResAiNoCause;
                sat_done (data->sat_command, sat_res, sizeof(T_SAT_RES));
                // editor_destroy(data->display_editor);
                break;
            case INFO_KCD_RIGHT: /* back */
                sat_res[SAT_ERR_INDEX] = SAT_RES_USER_BACK;
                sat_res[SAT_AI_INDEX]  = SatResAiNoCause;
                sat_done (data->sat_command, sat_res, sizeof(T_SAT_RES));
                //  editor_destroy(data->display_editor);
                break;
            case INFO_KCD_CLEAR: /* abort */
            case INFO_KCD_HUP:      
                sat_res[SAT_ERR_INDEX] = SAT_RES_USER_ABORT;
                sat_res[SAT_AI_INDEX]  = SatResAiNoCause;
                sat_done (data->sat_command, sat_res, sizeof(T_SAT_RES));
                //  editor_destroy(data->display_editor);
                break;
            default:
                TRACE_EVENT("sat_editor_cb() unexp. event");
                break;
            }
            sat_display_text_destroy (data->win);
        }
        break;
    default:
        TRACE_EVENT("sat_editor_cb() unexp. Identifier");
        break;
    }
    return;
}
// end c014 rsa
