
/*******************************************************************************

					CONDAT (UK)

********************************************************************************                                                                              

 This software product is the property of Condat (UK) Ltd and may not be
 disclosed to any third party without the express permission of the owner.                                 
                                                                              
********************************************************************************

 $Project name:	Basic MMI                                                      
 $Project code:	BMI (6349)                                                           
 $Module:		SMS Cell Broadcast
 $File:		    MmiSmsBroadcast.c
 $Revision:		1.0                                                       
                                                                              
 $Author:		Condat(UK)                                                         
 $Date:		    22/02/01                                                      
                                                                               
********************************************************************************
                                                                              
 Description:

  Implementation of MMI SMS Cell Broadcast dynamic menu handling

********************************************************************************

 $History: MmiSmsBroadcast.c

	25/10/00			Original Condat(UK) BMI version.	
	19/08/02   ver0.111   wangxia  modified 
	   
 $End

*******************************************************************************/


/*******************************************************************************
                                                                              
                                Include Files
                                                                              
*******************************************************************************/

#define ENTITY_MFW

/* includes */
#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#if defined (NEW_FRAME)

#include "typedefs.h"
#include "vsi.h"
#include "pei.h"
#include "custom.h"
#include "gsm.h"

#else

#include "stddefs.h"
#include "custom.h"
#include "gsm.h"
#include "vsi.h"

#endif
#include "custom.h"
#include "gsm.h"
#include "vsi.h"
#include "p_sim.h"
#include "mfw_sys.h"
#include "mfw_mfw.h"
#include "mfw_win.h"
#include "mfw_kbd.h"
#include "mfw_edt.h"
#include "mfw_lng.h"
#include "mfw_icn.h"
#include "mfw_phb.h"
#include "mfw_sim.h"
#include "mfw_nm.h"
#include "ksd.h"
#include "psa.h"
#include "mfw_sms.h"
#include "mfw_mnu.h"
#include "mfw_sat.h"
#include "mfw_tim.h"

#include "MmiMmi.h"
#include "MmiDummy.h"
#include "MmiDialogs.h"
#include "MmiLists.h"

#include "MmiMain.h"
#include "MmiStart.h"
#include "MmiPins.h"
#include "MmiMenu.h"
#include "MmiSoftKeys.h"

#include "MmiBookController.h"

#include "mmiSmsBroadcast.h"
#include "mmiSmsIdle.h"
#include "MmiEditor.h"
#include "MmiIdle.h"

#include "cus_aci.h"
#include "mmiusersymbolinput.h"
#include "Mmiicons.h"

#define MENU_TEXT_LENGTH 16 /* count of chars to be displayed in one list entry */

/* SH - moved to .h file */

/* storage for Cell Broadcast lists */
typedef struct 
{
    T_SMSCB_ATT 		 cb_attributes[MAX_MIDS];     /* list of attributes of messages stored in fifo */
    CHAR                cb_buffer[CB_STORE_LENGTH]; /* fifo buffer for received cell broadcast messages */

	ListMenuData     	 *cb_list_attributes; 		//atttributes for the menu list

    T_SMSCB_STATUS		current_status;       	       /*read status for all messages    */
    
	UBYTE found_mid;    //indicate the already existing cb in the fifo
	UBYTE clear_mid;     //indicate the deleting process
	UBYTE result_used;	//to update the used-pointer, after deleting the cb message
	UBYTE total_number;  //this include the total number of messages in the fifo; need for Menu-list

	UBYTE multipage_number;  //this include the total page of a particular multipage; need for showing msg.
	UBYTE *multipage;        //pointer to the first page of multipage
	UBYTE start_used;	//indicate the start point of the multipage, used for the searching
	UBYTE end_used;     //indicate the end point of the multipage, used for the searching

	UBYTE used_pointer;  //start point of the free space link list
	UBYTE free_pointer;   //start point of the data link list

	
} T_SMSCB_DATA;



T_SMSCB_DATASTORE g_smscb_data; /* storage for Cell Broadcast lists */
T_SMSCB_DATA smscb_data; /* storage for Cell Broadcast lists */



//Attributes of the list menu
static MfwMnuAttr sms_cb_menuAttrib =
{
    0,
    &smscb_M_CBArea,
    MNU_LEFT | MNU_LIST | MNU_CUR_LINE, /* centered page menu       */
    -1,                                 /* use default font         */
    NULL,                               /* with these items         */
    0                                   /* number of items     */
};


#define OFFSET(index) (index*CB_PAGE_LENGTH)

//this shows traces of some variable
//#define DEBUG_SMS_CB 

#ifdef DEBUG_SMS_CB
	UBYTE DEBUG;
#endif


//these prototypes belongs to saving cb_messages in the fifo
UBYTE sms_cb_count_messages ();
UBYTE sms_cb_count_unread_messages ();
UBYTE sms_cb_delete_message (USHORT mid);
UBYTE sms_cb_find_termination (UBYTE *used);
UBYTE sms_cb_update_free_pointer (UBYTE temp);
UBYTE sms_cb_search_mid (USHORT mid, SHORT sn);
UBYTE sms_cb_store_message (T_MFW_SMS_CB *Message);
UBYTE sms_cb_find_endpoint (USHORT mid, UBYTE end_used);
UBYTE sms_cb_find_startpoint (USHORT mid, UBYTE end_used);
UBYTE sms_cb_give_position_of_msg (UBYTE used, UBYTE positon);
UBYTE sms_cb_overwrite_old_message (T_MFW_SMS_CB *Message);


void sms_cb_show_cb_message(UBYTE link_list_position);
static ListMenuData * sms_cb_create_list (UBYTE number);
static void sms_cb_list_cb(T_MFW_HND win, ListMenuData * ListData);
void   sms_cb_store_attributes (UBYTE position, T_MFW_SMS_CB *Message);
static void sms_cb_editor_cb (T_MFW_HND win, USHORT Identifier, SHORT reason);

void sms_cb_loadEditDefault (T_EDITOR_DATA *editor_data);

/*********************************************************************
**********************************************************************

			cell broadcast declaration

*********************************************************************
**********************************************************************/

typedef struct
{
    int index; /* index of selected item in list */
} T_CB_index_param; 

typedef struct
{
    int index; /* index of selected item in list */
    char name[CB_TAG_LENGTH]; /* buffer for name to be associated with topic */
    char number[CB_MID_LENGTH]; /* buffer for channel number of topic */
} T_CB_EDT_MIDEDT_param;

typedef struct
{
    int index; /* index of selected item in list */
    char text[CB_TAG_LENGTH]; /* buffer for name to be associated with topic ??? size is dummy */
    char number[CB_MID_LENGTH]; /* buffer for channel number of topic ??? size is dummy */
} T_S_param; /* ??? T_S_param not to be defined here, but in "mmiSmsSend.h"! */


/* IDs used for identification of sources of events */
enum {
    smscb_M_CB_ID,
    smscb_M_CB_ACT_ID,
    smscb_M_CB_TOP_ID,
    smscb_M_CB_TOP_CONS_ID,
    smscb_M_CB_TOP_CONS_SEL_ID,
    smscb_CB_ID,
    smscb_CB_TEXTEDT_ID,
    smscb_CB_OPT_ID,
    smscb_CB_OPT_DEL_NOACK_ID,
    smscb_CB_EDT_TEXTEDT_ID,
    smscb_CB_MID,
    smscb_CB_EDT_MIDEDT_ID
} smscb_ID;

enum {
    smscb_CH_ADD,
    smscb_CH_MODIFY,
    smscb_CH_DEL
    }smscb_WIN_ID;

int smscb_win_stat;
void msgsExtractPhoneNumber(UBYTE *InputString, UBYTE *Buffer);
/* forward declaration of local start routines */
static T_MFW_HND smscb_M_CB_ACT_start(T_MFW_HND parent_window, void *dummy);
static T_MFW_HND smscb_M_CB_TOP_start(T_MFW_HND parent_window, void *dummy);
static T_MFW_HND smscb_M_CB_TOP_CONS_start(T_MFW_HND parent_window, void *dummy);
static T_MFW_HND smscb_M_CB_TOP_CONS_SEL_start(T_MFW_HND parent_window, T_CB_index_param * parameter);
static T_MFW_HND smscb_CB_start(T_MFW_HND parent_window, void *dummy);
static T_MFW_HND smscb_CB_TEXTEDT_start(T_MFW_HND parent_window, T_CB_index_param * parameter);
static T_MFW_HND smscb_CB_OPT_start(T_MFW_HND parent_window, T_CB_index_param * parameter);
static T_MFW_HND smscb_CB_OPT_DEL_NOACK_start(T_MFW_HND parent_window, T_CB_index_param * parameter);
static T_MFW_HND smscb_CB_EDT_TEXTEDT_start(T_MFW_HND parent_window, T_CB_index_param * parameter);
static T_MFW_HND smscb_CB_EDT_MIDEDT_start(T_MFW_HND parent_window, T_CB_index_param * parameter);


void info_screen (T_MFW_HND win, USHORT TextId, USHORT TextId2, T_VOID_FUNC  Callback,int iconindex)
    /* present info <TextId> for 3 seconds in a simple infoscreen */
{

    T_DISPLAY_DATA display_info;

    dialog_info_init(&display_info);//gdy add
    display_info.TextString   = 0; 
    display_info.TextString2  = 0;
    display_info.LeftSoftKey  = 0;
    display_info.RightSoftKey = 0;
    display_info.Time         = ONE_SECS;//zy 12/22/2002 change 3sec to 1sec
    display_info.KeyEvents    = KEY_RIGHT|KEY_LEFT|KEY_CLEAR;
    display_info.TextId       = TextId;
    display_info.TextId2      = TextId2;
    display_info.Identifier   = DIALOGS_REASON_NULL;
    display_info.Callback     = (T_VOID_FUNC)Callback;
    display_info.iconindex    =iconindex;
    info_dialog (win, &display_info); 
}

/*********************************************************************
**********************************************************************

			cell broadcast routines DECLARATION

*********************************************************************
**********************************************************************/
static T_MFW smscb_delete_topic(UBYTE MidOffset);
/* SH moved prototype to header file */
#if 0
// c019 rsa
/* this routine is for testing purposes only. It is used to start the submenutree from MmiMenu.c */
int temp_smscb_M_CB_ExeRead (MfwMnu* m, MfwMnuItem* i)
{
    T_MFW_HND       win  = mfwParent((MfwHdr *)mfw_header());

    smscb_InitData();
    return (int)smscb_M_CB_start (win, NULL);
}
// end c019 rsa
static void listCreate(MfwHnd Parent, MfwMnuAttr *a , MfwEvt e, ListCbFunc cbFunc)
{

    ListMenuData list_menu_data;

    /* initialization of the dialog data */
    list_menu_data.Attr           = a;
    list_menu_data.List           = a->items;
    list_menu_data.ListLength     = a->nItems;/* Number of entries in list menu.    */
    list_menu_data.ListPosition   = 1;/* True cursor position in list menu. */
    list_menu_data.CursorPosition = 1;/* Cursor position in list snapshot.  */
    list_menu_data.SnapshotSize   = 4;/* Number of list entries to view.    */
    list_menu_data.Font           = 0;
    list_menu_data.LeftSoftKey    = TxtSoftSelect;
    list_menu_data.RightSoftKey   = TxtSoftBack;
    list_menu_data.KeyEvents      = KEY_CLEAR | KEY_RIGHT | KEY_LEFT | KEY_MNUUP | KEY_MNUDOWN;
    list_menu_data.Reason         = 0;
    list_menu_data.Strings        = FALSE;
	list_menu_data.autoDestroy    = TRUE;
	list_menu_data.ListMenuHeadStr  = (char *) MmiRsrcGetText(TxtBroadcast);
    listDisplayListMenu(Parent, &list_menu_data, cbFunc,0);

}
// c020 rsa
// end c020 rsa
/*********************************************************************
**********************************************************************

			M_CB	 DYNAMIC MENU WINDOW. DECLARATION

*********************************************************************
**********************************************************************/

typedef struct
{
    /* administrative data */

    T_MMI_CONTROL   mmi_control;
    T_MFW_HND       win;
    SHORT           id;
    T_MFW_HND       parent;

    /* associated handlers */

    T_MFW_HND       menu;

    /* internal data */
} T_M_CB;

static T_MFW_HND smscb_M_CB_create (T_MFW_HND parent);
static void smscb_M_CB_destroy (T_MFW_HND window);
static void smscb_M_CB_exec_cb (T_MFW_HND win, USHORT event, SHORT value, void * dummy);
static void smscb_M_CB_mnu_cb (T_MFW_HND * win, ListMenuData * ListData); // c020 rsa

/*********************************************************************
**********************************************************************

			  M_CB   DYNAMIC MENU WINDOW. CONTENTS DECLARATION

*********************************************************************
**********************************************************************/

static T_MFW_MNU_ITEM smscb_M_CBItems[] =
{
    {0,0,0, (char *)TxtActivate,  0, (MenuFunc)smscb_M_CB_ExeActivate,    item_flag_none},
    {0,0,0, (char *)TxtRead,      0, (MenuFunc)sms_cb_select_read,        item_flag_none},
    {0,0,0, (char *)TxtNew,       0, (MenuFunc)smscb_M_CB_TOP_ExeAddNew,  item_flag_none},
    {0,0,0, (char *)TxtList,      0, (MenuFunc)smscb_M_CB_MID_DEL, item_flag_none}
};

static MfwMnuAttr smscb_M_CBAttrib =
{
    &smscb_M_CBArea,
    MNU_LEFT | MNU_LIST | MNU_CUR_LINE, /* centered page menu       */
    -1,                                 /* use default font         */
    smscb_M_CBItems,                      /* with these items         */
    sizeof(smscb_M_CBItems)/sizeof(MfwMnuItem) /* number of items     */
};

/*********************************************************************
**********************************************************************

			 M_CB    DYNAMIC MENU WINDOW. CONTENTS BEHAVIOUR

*********************************************************************
**********************************************************************/
static int smscb_M_CB_ExeActivate (MfwMnu* m, MfwMnuItem* i)
{
    T_MFW_HND       win  = mfwParent((MfwHdr *)mfw_header());

    //	selection "Activate": menucreate M_CB_ACT
    return (int)smscb_M_CB_ACT_start (win, NULL);
}



/*********************************************************************
**********************************************************************

			M_CB	DYNAMIC MENU WINDOW. IMPLEMENTATION

*********************************************************************
**********************************************************************/




/*******************************************************************************

 $Function:    	cellbroadcast_start

 $Description:	Calls from the MainMenu

 $Returns:		none.

 $Arguments:	
 
*******************************************************************************/

int cellbroadcast_start(MfwMnu* m, MfwMnuItem* i)
{
    T_MFW_HND       win  = mfwParent(mfw_header());
  
	smscb_M_CB_start(win, NULL);
	return 1;
}



/*******************************************************************************

 $Function:    	smscb_M_CB_start

 $Description:	

 $Returns:		none.

 $Arguments:	
 
*******************************************************************************/
T_MFW_HND smscb_M_CB_start(T_MFW_HND parent_window, void *dummy)
{
    T_MFW_HND win;

    TRACE_FUNCTION ("smscb_M_CB_start()");

    win = smscb_M_CB_create (parent_window);

    if (win NEQ NULL)
	{
	    SEND_EVENT (win, E_INIT, smscb_M_CB_ID, NULL);
	}
    return win;
}

/*******************************************************************************

 $Function:    	smscb_M_CB_create

 $Description:	

 $Returns:		none.

 $Arguments:	
 
*******************************************************************************/
static T_MFW_HND smscb_M_CB_create(MfwHnd parent_window)
{
    T_M_CB      * data = (T_M_CB *)ALLOC_MEMORY (sizeof (T_M_CB));
    T_MFW_WIN  * win;

    TRACE_FUNCTION ("smscb_M_CB_create()");

    /*
     * Create window handler
     */

    data->win = 
	win_create (parent_window, 0, 0, NULL);
    if (data->win EQ NULL)
	{
	    return NULL;
	}

    /* 
     * connect the dialog data to the MFW-window
     */

    data->mmi_control.dialog = (T_DIALOG_FUNC)smscb_M_CB_exec_cb;
    data->mmi_control.data   = data;
    win                      = ((T_MFW_HDR *)data->win)->data;
    win->user                = (void *)data;
    data->parent             = parent_window;

    /*
     * return window handle
     */

    winShow(data->win);
    return data->win;
}
/*******************************************************************************

 $Function:    	smscb_M_CB_destroy

 $Description:

 $Returns:		none.

 $Arguments:	
 
*******************************************************************************/
static void smscb_M_CB_destroy(MfwHnd own_window)
{
    T_MFW_WIN * win_data;
    T_M_CB     * data;

    TRACE_FUNCTION ("smscb_M_CB_destroy()");

    if (own_window)
	{
	    win_data = ((T_MFW_HDR *)own_window)->data;
	    data = (T_M_CB *)win_data->user;

	    if (data)
		{
		    /*
		     * Delete WIN handler
		     */ 
		    win_delete (data->win);

		    /*     
		     * Free Memory
		     */
		    FREE_MEMORY ((void *)data, sizeof (T_M_CB));
		}
	    else
		{
		    TRACE_EVENT ("smscb_M_CB_destroy() called twice");
		}
	}
}

/*******************************************************************************

 $Function:    	smscb_M_CB_exec_cb

 $Description:	

 $Returns:		none.

 $Arguments:	
 
*******************************************************************************/
void smscb_M_CB_exec_cb (T_MFW_HND win, USHORT event, SHORT value, void * dummy)
    /* callback handler for events sent to to trigger execution */
{	
	//entry submenu "Broadcast": M_CB
	//on INIT: create staticlist (
	//	selection "Read": call smsCB_read()
	//	selection "Activate": menucreate M_CB_ACT

     //nm menu changed 7.2.01
	//	selection "Topics": menucreate M_CB_TOP
	//	selection "Delete All": menucreate M_DEL_ALL(Type=CB)
	//)
	//on EXIT: /* terminate EXIT from M_CB_ACT here */
	//defaultreaction
	//end

    T_MFW_WIN      * win_data = ((T_MFW_HDR *) win)->data;
    T_M_CB          * data = (T_M_CB *)win_data->user;
    T_MFW_HND      * l_parent;
    SHORT            l_id;

    TRACE_FUNCTION ("smscb_M_CB_exec_cb()");

    switch (event)
	{
	case E_INIT:
	    /* initialization of administrative data */
	    data->id = value;

	    /* initialization of the dialog data */

	    /* create the dialog handler */

	    listCreate (data->win,&smscb_M_CBAttrib, E_MNU_ESCAPE, (ListCbFunc)smscb_M_CB_mnu_cb); // c020 rsa
	    break;
	case E_ABORT:  /* defaultreaction */
	    l_parent = data->parent;
	    l_id = data->id;
	    smscb_M_CB_destroy(data->win);
	    SEND_EVENT (l_parent, event, l_id, NULL); /* forward event to parent */
	    break;
	case E_RETURN: /* back from child */
	case E_EXIT: /* terminate EXIT from M_CB_ACT here */
    	    /* winUpdate(data->win); c021 rsa */
	    break;

	case E_BACK:
	default:
	    TRACE_EVENT ("smscb_M_CB_exec_cb() unexpected event");
	    return;
	}
}

/*******************************************************************************

 $Function:    	smscb_M_CB_mnu_cb

 $Description:

 $Returns:		none.

 $Arguments:	
 
*******************************************************************************/
// c020 rsa
static void smscb_M_CB_mnu_cb (T_MFW_HND * win, ListMenuData * ListData)
    /* smscb_M_CB menu event handler */
{
    T_MFW_WIN  * win_data = ((T_MFW_HDR *)win)->data;
    T_M_CB      * data = (T_M_CB *)win_data->user;
    T_MFW_HND  * l_parent;
    SHORT        l_id;
    int index;

    TRACE_FUNCTION ("smscb_M_CB_listmnu_cb()");

    switch (ListData->Reason)
        {
	case E_MNU_SELECT:
	    //on SELECT: menucreate 
	    break;
	case LISTS_REASON_BACK: /* back to previous menu */
	    l_parent = data->parent;
	    l_id = data->id;
	    smscb_M_CB_destroy(data->win);
	    SEND_EVENT (data->parent, E_RETURN, l_id, NULL); // c007 rsa
	    break;
	default:
	    return;
        }
    return;
}
// end c020 rsa

/*********************************************************************
**********************************************************************

			M_CB_ACT	 DYNAMIC MENU WINDOW. DECLARATION

*********************************************************************
**********************************************************************/

typedef struct
{
    /* administrative data */

    T_MMI_CONTROL   mmi_control;
    T_MFW_HND       win;
    SHORT           id;
    T_MFW_HND       parent;

    /* associated handlers */
    T_MFW_HND       kbd;

    T_MFW_HND       menu;

    /* internal data */
} T_M_CB_ACT;
static MfwMnuItem smscb_M_CB_ACTItems [] =
{
    {0,0,0,(char *)TxtOn,0,(MenuFunc)smscb_M_CB_ACT_ExeYes,item_flag_none},
    {0,0,0,(char *)TxtOff,0,(MenuFunc)smscb_M_CB_ACT_ExeNo,item_flag_none},
};

static MfwMnuAttr smscb_M_CB_ACTAttrib =
{
    &smscb_M_CB_ACTArea,
    MNU_LEFT | MNU_LIST | MNU_CUR_LINE, /* centered page menu       */
    -1,                                 /* use default font         */
    smscb_M_CB_ACTItems,                      /* with these items         */
    sizeof(smscb_M_CB_ACTItems)/sizeof(MfwMnuItem) /* number of items     */
};
static T_MFW_HND smscb_M_CB_ACT_create (T_MFW_HND parent);
static void smscb_M_CB_ACT_destroy (T_MFW_HND window);
static void smscb_M_CB_ACT_exec_cb (T_MFW_HND win, USHORT event, SHORT value, void * dummy);
static void smscb_M_CB_ACT_mnu_cb (MfwEvt e, MfwMnu *m);
static int Smscb_M_CB_ACT_kbd_cb (MfwEvt e, MfwKbd *k);
static int Smscb_M_CB_ACT_win_cb (MfwEvt e, MfwWin *w);
static int smscb_M_CB_ACT_ExeNo(MfwMnu* m, MfwMnuItem* i);
static int smscb_M_CB_ACT_ExeYes(MfwMnu* m, MfwMnuItem* i);
/*********************************************************************
**********************************************************************

		M_CB_ACT	     DYNAMIC MENU WINDOW. CONTENTS DECLARATION

*********************************************************************
**********************************************************************/

static MfwMnuItem smscb_M_CB_MIDItems [] =
{
    {&colist, 0,0,(char *)TxtAdd,0,(MenuFunc)smscb_CB_EDT_ExeAdd,item_flag_none},
    {&colist, 0,0,(char *)TxtModify,0,(MenuFunc)smscb_CB_EDT_ExeModify,item_flag_none},
    {&colist, 0,0,(char *)TxtDelete,0,(MenuFunc)smscb_M_CB_MID_DEL,item_flag_none},
};

static MfwMnuAttr smscb_M_CB_MIDAttrib =
{
    0,
    &smscb_M_CB_ACTArea,
    MNU_LEFT | MNU_LIST | MNU_CUR_LINE, /* centered page menu       */
    -1,                                 /* use default font         */
    smscb_M_CB_MIDItems,                      /* with these items         */
    sizeof(smscb_M_CB_MIDItems)/sizeof(MfwMnuItem) /* number of items     */
};
#endif
/*******************************************************************************

 $Function:    	SmsRead_R_dialog_cb

 $Description:	Dialog callback function of the SMS read window (dynamic list of all the SMS)
 				Case where SMS has not been initialized yet
 
 $Returns:		status

 $Arguments:	win - current window handler
 				Identifier - window id
 				Reason - Event cause.
 				
*******************************************************************************/

int SmsCB_dialog_cb(T_MFW_HND win, USHORT Identifier, USHORT Reason)
{
	T_MFW_WIN *win_data = ((T_MFW_HDR *)win)->data;
//	T_M_CB_ACT *data = (T_M_CB_ACT *)win_data->user;
	
    TRACE_FUNCTION ("SmsRead_dialog_cb()");
	
	switch (Reason)
	{
	case INFO_TIMEOUT:
	case INFO_KCD_HUP:
		// Commit suicide
	    SEND_EVENT (win, E_EXIT,0, NULL);
		return MFW_EVENT_CONSUMED;

	default:
		return MFW_EVENT_REJECTED;
	}
}

/******************************************************/
/*                                                    */
/*Function Name: CBLanguage_check                       */
/*Purpose:      CBLanguage_check items check                      */
/*Prarmeter:   struct MfwMnuTag *m, struct MfwMnuAttrTag *ma, struct MfwMnuItemTag *mi */
/*return   :     USHORT                                 */
/*                                                    */
/*Author   :     windy                              */
/*Date     :    Aug.16th,2002                             */
/******************************************************/
USHORT CBONOFF_check (struct MfwMnuTag *m, struct MfwMnuAttrTag *ma, struct MfwMnuItemTag *mi)
{
  	if(FFS_flashData.CBONOFF==1)
    		return FLAGFUNC_TRUE_ICON;
 	else if(FFS_flashData.CBONOFF==0)
    		return FLAGFUNC_FALSE_ICON;
}

/*********************************************************************
**********************************************************************

		M_CB_ACT	     DYNAMIC MENU WINDOW. CONTENTS BEHAVIOUR

*********************************************************************
**********************************************************************/
/*******************************************************************************

 $Function:    	smscb_M_CB_ACT_ExeyNo

 $Description:	

 $Returns:		none.

 $Arguments:	
 
*******************************************************************************/
int smscb_M_CB_ACT_ExeOnOff(MfwMnu* m, MfwMnuItem* i)
{
    T_MFW_HND       win  = mfwParent((MfwHdr *)mfw_header());
  	T_DISPLAY_DATA display_info;

//nm    smscb_setCBsettings(SMSCB_SWITCH_OFF);
  	if(FFS_flashData.CBONOFF==1)
  	{
		sms_set_mt_ind(MT_CB_NO_IDX);
   	 	//		call infoscreen("Deactivated"); 

    	dialog_info_init(&display_info);//gdy add
    	display_info.RightSoftKey = 0;
    	display_info.Time         = ONE_SECS;//zy 12/22/2002 change 3sec to 1sec
    	display_info.KeyEvents    = KEY_RIGHT|KEY_LEFT|KEY_CLEAR;
    	display_info.TextId2      = TxtDeActivated;
    	display_info.Identifier   = DIALOGS_REASON_NULL;
    	display_info.iconindex	=REMIND_COMPLETE;
    	info_dialog (win, &display_info); 
    	//info_screen(win, TxtDeActivated,TxtNull, NULL);//(T_VOID_FUNC)SmsCB_dialog_cb); 
		//statusCBCH(TRUE);
		FFS_flashData.CBONOFF=0;
		flash_write();
  	}
 	 else if(FFS_flashData.CBONOFF==0)
  	{	
  		sms_set_mt_ind(MT_CB_MSG);
  	  	//call infoscreen("Activated"); 
    	dialog_info_init(&display_info);//gdy add
    	display_info.RightSoftKey = 0;
    	display_info.Time         = ONE_SECS;//zy 12/22/2002 change 3sec to 1sec
    	display_info.KeyEvents    = KEY_RIGHT|KEY_LEFT|KEY_CLEAR;
    	display_info.TextId2      = TxtActivated;
    	display_info.Identifier   = DIALOGS_REASON_NULL;
    	display_info.iconindex	=REMIND_COMPLETE;
    	info_dialog (win, &display_info); 
  	 	 //info_screen(win, TxtActivated,TxtNull,NULL);//(T_VOID_FUNC)SmsCB_dialog_cb); 
    	//statusCBCH(FALSE);
		FFS_flashData.CBONOFF=1;
		flash_write();
	}
    return MFW_EVENT_CONSUMED;
}
#if 0
/*********************************************************************
**********************************************************************

		M_CB_ACT	     DYNAMIC MENU WINDOW. CONTENTS BEHAVIOUR

*********************************************************************
**********************************************************************/
/*******************************************************************************

 $Function:    	smscb_M_CB_ACT_ExeNo

 $Description:	

 $Returns:		none.

 $Arguments:	
 
*******************************************************************************/
static int smscb_M_CB_ACT_ExeNo(MfwMnu* m, MfwMnuItem* i)
{
    T_MFW_HND       win  = mfwParent((MfwHdr *)mfw_header());
    T_MFW_WIN     * win_data = ((T_MFW_HDR *)win)->data;
    T_M_CB_ACT         * data = (T_M_CB_ACT *)win_data->user;
    T_MFW_HND      * l_parent;
    SHORT            l_id;

    //		exec sms_cb_activate(Activation=FALSE); 

//nm    smscb_setCBsettings(SMSCB_SWITCH_OFF);
	sms_set_mt_ind(MT_CB_NO_IDX);


    //		call infoscreen("Deactivated"); 
    info_screen(data->win, TxtDeActivated,TxtNull, (T_VOID_FUNC)SmsCB_dialog_cb); 

    statusCBCH(FALSE);

    return MFW_EVENT_CONSUMED;
}

/*******************************************************************************

 $Function:    	smscb_M_CB_ACT_ExeYes

 $Description:	

 $Returns:		none.

 $Arguments:	
 
*******************************************************************************/
static int smscb_M_CB_ACT_ExeYes(MfwMnu* m, MfwMnuItem* i)
{
    T_MFW_HND       win  = mfwParent((MfwHdr *)mfw_header());
    T_MFW_WIN     * win_data = ((T_MFW_HDR *)win)->data;
    T_M_CB_ACT         * data = (T_M_CB_ACT *)win_data->user;
    T_MFW_HND      * l_parent;
    SHORT            l_id;

    //		exec sms_cb_activate(Activation=TRUE); 
//nm    smscb_setCBsettings(SMSCB_SWITCH_ON);
	sms_set_mt_ind(MT_CB_MSG);
    //call infoscreen("Activated"); 
    info_screen(data->win, TxtActivated,TxtNull,(T_VOID_FUNC)SmsCB_dialog_cb); 
	statusCBCH(TRUE);
    return MFW_EVENT_CONSUMED;
}

/*********************************************************************
**********************************************************************

		M_CB_ACT		DYNAMIC MENU WINDOW. IMPLEMENTATION

*********************************************************************
**********************************************************************/

/*******************************************************************************

 $Function:    	smscb_M_CB_ACT_start

 $Description:	

 $Returns:		none.

 $Arguments:	
 
*******************************************************************************/
T_MFW_HND smscb_M_CB_ACT_start(T_MFW_HND parent_window, void *dummy)
{
    T_MFW_HND win;

    TRACE_FUNCTION ("smscb_M_CB_ACT_start()");

    win = smscb_M_CB_ACT_create (parent_window);

    if (win NEQ NULL)
	{
	    SEND_EVENT (win, E_INIT, smscb_M_CB_ACT_ID, NULL);
	}
    return win;
}

/*******************************************************************************

 $Function:    	smscb_M_CB_ACT_create

 $Description:	

 $Returns:		none.

 $Arguments:	
 
*******************************************************************************/
static T_MFW_HND smscb_M_CB_ACT_create(MfwHnd parent_window)
{
    T_M_CB_ACT      * data = (T_M_CB_ACT *)ALLOC_MEMORY (sizeof (T_M_CB_ACT));
    T_MFW_WIN  * win;

    TRACE_FUNCTION ("smscb_M_CB_ACT_create()");

    /*
     * Create window handler
     */

    data->win = win_create (parent_window, 0, E_WIN_VISIBLE, (T_MFW_CB)Smscb_M_CB_ACT_win_cb);
    if (data->win EQ NULL)
	{
	    return NULL;
	}

    /* 
     * connect the dialog data to the MFW-window
     */

    data->mmi_control.dialog = (T_DIALOG_FUNC)smscb_M_CB_ACT_exec_cb;
    data->mmi_control.data   = data;
    win                      = ((T_MFW_HDR *)data->win)->data;
    win->user                = (void *)data;
    data->parent             = parent_window;

    /*
     * return window handle
     */

    winShow(data->win);
    return data->win;
}

/*******************************************************************************

 $Function:    	smscb_M_CB_ACT_destroy

 $Description:	

 $Returns:		none.

 $Arguments:	
 
*******************************************************************************/
static void smscb_M_CB_ACT_destroy(MfwHnd own_window)
{
    T_MFW_WIN * win_data;
    T_M_CB_ACT     * data;

    TRACE_FUNCTION ("smscb_M_CB_ACT_destroy()");

    if (own_window)
	{
	    win_data = ((T_MFW_HDR *)own_window)->data;
	    data = (T_M_CB_ACT *)win_data->user;

	    if (data)
		{
		    /*
		     * Delete WIN handler
		     */ 
		    win_delete (data->win);

		    /*     
		     * Free Memory
		     */
		    FREE_MEMORY ((void *)data, sizeof (T_M_CB_ACT));
		}
	    else
		{
		    TRACE_EVENT ("smscb_M_CB_ACT_destroy() called twice");
		}
	}
}
/*******************************************************************************

 $Function:    	smscb_M_CB_ACT_exec_cb

 $Description:	

 $Returns:		none.

 $Arguments:	
 
*******************************************************************************/
void smscb_M_CB_ACT_exec_cb (T_MFW_HND win, USHORT event, SHORT value, void * dummy)
    /* callback handler for events sent to to trigger execution */
{
	//submenu "Activate": M_CB_ACT
	//on INIT: create staticlist (
	//	selection "Activate": 
	//		exec sms_cb_activate(Activation=TRUE); 
	//		call infoscreen("Activated"); 
	//		selfdestroy; sendevent EXIT
	//	selection "Deactivate": 
	//		exec sms_cb_activate(Activation=FALSE); 
	//		call infoscreen("Deactivated"); 
	//		selfdestroy; sendevent EXIT
	//)
	//defaultreaction
	//end

    T_MFW_WIN      * win_data = ((T_MFW_HDR *) win)->data;
    T_M_CB_ACT          * data = (T_M_CB_ACT *)win_data->user;
    T_MFW_HND      * l_parent;
    SHORT            l_id;
	char *title;

    TRACE_FUNCTION ("smscb_M_CB_ACT_exec_cb()");

    switch (event)
	{
	case E_INIT:
	    /* initialization of administrative data */
	    data->id = value;

	    /* initialization of the dialog data */

	    /* create the dialog handler */
	    data->kbd = kbdCreate(data->win,KEY_ALL, (MfwCb)Smscb_M_CB_ACT_kbd_cb);
		title = (char *) MmiRsrcGetText(TxtBroadcast);  // get the editor title text
    	data->menu = mnuListCreate(data->win,&smscb_M_CB_ACTAttrib, E_MNU_ESCAPE, (MfwCb)smscb_M_CB_ACT_mnu_cb,title);
    	mnuLang(data->menu,mainMmiLng);
    	/* put the (new) dialog window on top of the window stack */
    	mnuUnhide(data->menu);
	    //listCreate (data->win,&smscb_M_CB_ACTAttrib, E_MNU_ESCAPE, (ListCbFunc)smscb_M_CB_ACT_mnu_cb); // c020 rsa
 	    winShow(win);
	    break;
	    // c007 rsa
	case E_RETURN: /* back from child */
    	 /* winUpdate(data->win); c021 rsa */
	    data->kbd = kbdCreate(data->win,KEY_ALL|KEY_MAKE, (MfwCb)Smscb_M_CB_ACT_kbd_cb);
		title = (char *) MmiRsrcGetText(TxtBroadcast);  // get the editor title text
    	data->menu = mnuListCreate(data->win,&smscb_M_CB_ACTAttrib, E_MNU_ESCAPE, (MfwCb)smscb_M_CB_ACT_mnu_cb,title);
    	mnuLang(data->menu,mainMmiLng);
    	/* put the (new) dialog window on top of the window stack */
    	mnuUnhide(data->menu);
	    //listCreate (data->win,&smscb_M_CB_ACTAttrib, E_MNU_ESCAPE, (ListCbFunc)smscb_M_CB_ACT_mnu_cb); // c020 rsa
 	    winShow(win);
	    break;
	    // end c007 rsa
	case E_BACK:
	case E_ABORT:
	case E_EXIT:  /* defaultreaction */
	    l_parent = data->parent;
	    l_id = data->id;
	    smscb_M_CB_ACT_destroy(data->win);
	    SEND_EVENT (l_parent, event, l_id, NULL); /* forward event to parent */
	    break;
	default:
	    return;
	}
}

/*******************************************************************************

 $Function:    	smscb_M_CB_ACT_mnu_cb

 $Description:

 $Returns:		none.

 $Arguments:	
 
*******************************************************************************/
// c020 rsa
static void smscb_M_CB_ACT_mnu_cb (MfwEvt e, MfwMnu *m)
    /* smscb_M_CB_ACT menu event handler */
{
    T_MFW_HND    win  = mfwParent(mfw_header());
    T_MFW_WIN  * win_data = ((T_MFW_HDR *)win)->data;
    T_M_CB_ACT      * data = (T_M_CB_ACT *)win_data->user;
    T_MFW_HND  * l_parent;

    TRACE_FUNCTION ("smscb_M_CB_ACT_mnu_cb()");

    switch (e)
        {
	case E_MNU_SELECT:
	    //on SELECT: menucreate 
	    break;
	case E_MNU_ESCAPE: /* back to previous menu */
	    l_parent = data->parent;
	    smscb_M_CB_ACT_destroy(data->win);
	    SEND_EVENT (l_parent, E_RETURN, data->id, NULL);
	    break;
	default:
	    return;
        }
    return;
}
// end c020 rsa

/*******************************************************************************

 $Function:    	SmsRead_R_OPT_win_cb

 $Description:	Window callback function for the Option window
 			
 $Returns:		status

 $Arguments:	e - mfw event Id
 				w - window handler
*******************************************************************************/

static int Smscb_M_CB_ACT_win_cb (MfwEvt e, MfwWin *w)
    /* SmsRead_R_OPT window event handler */
{
    TRACE_FUNCTION ("SmsRead_R_OPT_win_cb()");

    switch (e)
	{
        case MfwWinVisible:  /* window is visible  */
	    displaySoftKeys(TxtSoftSelect,TxtSoftBack);
	    break;
	case MfwWinFocussed: /* input focus / selected   */
	case MfwWinDelete:   /* window will be deleted   */
        default:
	    return MFW_EVENT_REJECTED;
	}
    return MFW_EVENT_CONSUMED;
}

/*******************************************************************************

 $Function:    	Smscb_M_CB_ACT_kbd_cb

 $Description:	Exec callback function for the Option window
 			
 $Returns:		status

 $Arguments:	e - keyboard event Id
 				k - keyboard data 
*******************************************************************************/

static int Smscb_M_CB_ACT_kbd_cb (MfwEvt e, MfwKbd *k)
    /* SmsRead_R_OPT keyboard event handler */
{
    T_MFW_HND    win  = mfwParent(mfw_header());
    T_MFW_WIN  * win_data = ((T_MFW_HDR *)win)->data;
    T_M_CB_ACT   * data = (T_M_CB_ACT *)win_data->user;

    TRACE_FUNCTION ("SmsRead_R_OPT_kbd_cb()");

	    switch (k->code)
		{
		case KCD_MNUUP: /* highlight previous entry */
		    mnuUp(data->menu);
		    break;
		case KCD_MNUDOWN: /* highlight next entry */
		    mnuDown(data->menu);
		    break;
		case KCD_LEFT: /* activate this entry */
		    mnuSelect(data->menu);
		    break;
		case KCD_HUP: /* back to previous menu */
		case KCD_RIGHT: /* back to previous menu */
		    mnuEscape(data->menu);
		    break;
		default: /* no response to all other keys */
		    return MFW_EVENT_CONSUMED;
		}
    return MFW_EVENT_CONSUMED;
}
/*********************************************************************
**********************************************************************

			M_CB_TOP	 DYNAMIC MENU WINDOW. DECLARATION

*********************************************************************
**********************************************************************/

typedef struct
{
    /* administrative data */

    T_MMI_CONTROL   mmi_control;
    T_MFW_HND       win;
    SHORT           id;
    T_MFW_HND       parent;

    /* associated handlers */

    T_MFW_HND       menu;

    /* internal data */
    T_CB_index_param CB_index_param; /* parameter for called submenus (index in list) */
} T_M_CB_TOP;
/*******************************************************************************

 $Function:    	smscb_M_CB_TOP_ExeAddNew

 $Description:	

 $Returns:		

 $Arguments:	
 
*******************************************************************************/
static int smscb_M_CB_TOP_ExeAddNew(MfwMnu* m, MfwMnuItem* i)
{
    T_MFW_HND      win      = mfwParent((MfwHdr *)mfw_header());
    T_MFW_WIN    * win_data = ((T_MFW_HDR *)win)->data;
    T_M_CB_TOP   * data     = (T_M_CB_TOP *)win_data->user;
    int            index;

    //	selection "Add": call sms_cb_edit_topic(List=CBTopics, Index=ADD)
    index = g_smscb_data.CBTopics.length;/* get the index of the next free entry in list CBTopics */

    if (index > MAXCB)
	{
	    /* all entries are already used */
	    info_screen (win, TxtFull /* ??? */,TxtNull, NULL);
	    return MFW_EVENT_CONSUMED;
	}
    else
	{
	    data->CB_index_param.index = index; 
	    return (int)smscb_CB_EDT_TEXTEDT_start(win, &data->CB_index_param);
	}
}

#endif
/*********************************************************************
**********************************************************************

			M_CB_TOP_CONS	 DYNAMIC MENU WINDOW. DECLARATION

*********************************************************************
**********************************************************************/
typedef struct
{
    /* administrative data */

    T_MMI_CONTROL   mmi_control;
    T_MFW_HND       win;
    SHORT           id;
    T_MFW_HND       parent;

    /* associated handlers */

    /* internal data */
    T_CB_index_param CB_index_param; /* parameter for called submenus (index in list) */
    MfwMnuItem      menuItems[MAX_MIDS]; /* List=CBTopics */
    ListMenuData    list_menu_data;
} T_M_CB_TOP_CONS;
typedef struct
{
    /* administrative data */

    T_MMI_CONTROL   mmi_control;
    T_MFW_HND       win;
    SHORT           id;
    T_MFW_HND       parent;
    T_MFW_HND       menu;
    T_MFW_HND       kbd;
     T_CB_index_param CB_index_param; /* parameter for called submenus (index in list) */
 /* internal data */
} T_M_CB_MID;
static T_MFW_HND smscb_M_CB_TOP_CONS_create (T_MFW_HND parent);
static void smscb_M_CB_TOP_CONS_destroy (T_MFW_HND window);
void smscb_M_CB_MID_DEL_exec_cb (T_MFW_HND win, USHORT event, SHORT value, void * dummy);
static void smscb_M_CB_TOP_CONS_listmnu_cb (T_MFW_HND * win, ListMenuData * ListData);
void ExeDelete_infoscreen_cb (T_MFW_HND win, USHORT identifier, UBYTE reason);
static MfwMnuAttr smscb_M_CB_TOP_CONSAttrib =
{
    0,
    &smscb_M_CB_TOP_CONSArea,
    MNU_LEFT | MNU_LIST | MNU_CUR_LINE, /* centered page menu       */
    -1,                                 /* use default font         */
    NULL,                               /* with these items         */
    0                                   /* number of items     */
};
#if 0
/*******************************************************************************

 $Function:    	smscb_M_CB_MID

 $Description:	

 $Returns:		none.

 $Arguments:	
 
*******************************************************************************/
static int smscb_M_CB_MID(MfwMnu* m, MfwMnuItem* i)
{
    T_MFW_HND       win      = mfwParent((MfwHdr *)mfw_header());

    //	selection "Consult": menucreate M_CB_TOP_CONS
    return (int)smscb_M_CB_MID_start (win, NULL);
}

/*******************************************************************************

 $Function:    	smscb_M_CB_MID_start

 $Description:	

 $Returns:		

 $Arguments:	
 
*******************************************************************************/
T_MFW_HND smscb_M_CB_MID_start(T_MFW_HND parent_window, void *dummy)
{
    T_MFW_HND win;

    TRACE_FUNCTION ("smscb_M_CB_MID_start()");

    win = smscb_M_CB_MID_create (parent_window);

    if (win NEQ NULL)
	{
	    SEND_EVENT (win, E_INIT, smscb_CB_MID, NULL);
	}
    return win;
}

/*******************************************************************************

 $Function:    	smscb_M_CB_MID_create

 $Description:	

 $Returns:		

 $Arguments:	
 
*******************************************************************************/
static T_MFW_HND smscb_M_CB_MID_create(MfwHnd parent_window)
{
    T_M_CB_MID    * data = (T_M_CB_MID *)ALLOC_MEMORY (sizeof (T_M_CB_MID));
    T_MFW_WIN  * win;
    TRACE_FUNCTION ("smscb_M_CB_TOP_CONS_create()");
    /*
     * Create window handler
     */
    data->win =win_create (parent_window, 0, 0, NULL);
    if (data->win EQ NULL)
	{
	    return NULL;
	}
    /* 
     * connect the dialog data to the MFW-window
     */
    data->mmi_control.dialog = (T_DIALOG_FUNC)smscb_M_CB_MID_exec_cb;
    data->mmi_control.data   = data;
    win                      = ((T_MFW_HDR *)data->win)->data;
    win->user                = (void *)data;
    data->parent             = parent_window;
    /*
     * return window handle
     */
    winShow(data->win);
    return data->win;
}

/*******************************************************************************

 $Function:    	smscb_M_CB_MID_exec_cb

 $Description:	

 $Returns:		

 $Arguments:	
 
*******************************************************************************/
void smscb_M_CB_MID_exec_cb (T_MFW_HND win, USHORT event, SHORT value, void * dummy)
{
    T_MFW_WIN         * win_data = ((T_MFW_HDR *) win)->data;
    T_M_CB_MID   * data = (T_M_CB_MID *)win_data->user;
    T_MFW_HND         * l_parent;
    SHORT               l_id;
    UBYTE i;
    char* title;
    TRACE_FUNCTION ("smscb_M_CB_MID_exec_cb()");
    
    switch (event)
	{
	case E_INIT:
	   	 data->id = value;
	    	/* initialization of the dialog data */
	   	 /* create the dialog handler */
	   	 data->kbd = kbdCreate(data->win,KEY_ALL, (MfwCb)Smscb_M_CB_MID_kbd_cb);
		 title = (char *) MmiRsrcGetText(TxtCBChanel);  // get the editor title text
    	 data->menu = mnuListCreate(data->win,&smscb_M_CB_MIDAttrib, E_MNU_ESCAPE, (MfwCb)smscb_M_CB_MID_mnu_cb,title);
    	 mnuLang(data->menu,mainMmiLng);
    		/* put the (new) dialog window on top of the window stack */
    		mnuUnhide(data->menu);
	    	//listCreate (data->win,&smscb_M_CB_ACTAttrib, E_MNU_ESCAPE, (ListCbFunc)smscb_M_CB_ACT_mnu_cb); // c020 rsa
 	   	 winShow(win);
	break;
	    /* NOBREAK */
	case E_RETURN: 
		mnuUpdate(data->menu );
		break;
	case E_EXIT: /* terminate EXIT from CB_EDT_MIDEDT|M_CB_TOP_CONS_SEL here */
	case E_ABORT: /* defaultreaction */
	    TRACE_EVENT ("ABORT in M_CB_MID_exec_cb");	    
	    l_parent = data->parent;
	    l_id = data->id;
	    smscb_M_CB_MID_destroy(data->win);
	    SEND_EVENT (l_parent, event, l_id, NULL); /* forward event to parent */
	    break;
	default:
	    return;
	}
}


/*******************************************************************************

 $Function:    	smscb_M_CB_MID_mnu_cb

 $Description:

 $Returns:		none.

 $Arguments:	
 
*******************************************************************************/
// c020 rsa
static void smscb_M_CB_MID_mnu_cb (MfwEvt e, MfwMnu *m)
    /* smscb_M_CB_ACT menu event handler */
{
    T_MFW_HND    win  = mfwParent(mfw_header());
    T_MFW_WIN  * win_data = ((T_MFW_HDR *)win)->data;
    T_M_CB_MID      * data = (T_M_CB_MID *)win_data->user;
    T_MFW_HND  * l_parent;

    TRACE_FUNCTION ("smscb_M_CB_MID_mnu_cb()");

    switch (e)
        {
	case E_MNU_SELECT:
	    //on SELECT: menucreate 
	    break;
	case E_MNU_ESCAPE: /* back to previous menu */
	    l_parent = data->parent;
	    smscb_M_CB_MID_destroy(data->win);
	    SEND_EVENT (l_parent, E_RETURN, data->id, NULL);
	    break;
	default:
	    return;
        }
    return;
}

/*******************************************************************************

 $Function:    	Smscb_M_CB_ACT_kbd_cb

 $Description:	Exec callback function for the Option window
 			
 $Returns:		status

 $Arguments:	e - keyboard event Id
 				k - keyboard data 
*******************************************************************************/

static int Smscb_M_CB_MID_kbd_cb (MfwEvt e, MfwKbd *k)
    /* SmsRead_R_OPT keyboard event handler */
{
    T_MFW_HND    win  = mfwParent(mfw_header());
    T_MFW_WIN  * win_data = ((T_MFW_HDR *)win)->data;
    T_M_CB_MID   * data = (T_M_CB_MID *)win_data->user;

    TRACE_FUNCTION ("Smscb_M_CB_MID_kbd_cb()");

	    switch (k->code)
		{
		case KCD_MNUUP: /* highlight previous entry */
		    mnuUp(data->menu);
		    break;
		case KCD_MNUDOWN: /* highlight next entry */
		    mnuDown(data->menu);
		    break;
		case KCD_LEFT: /* activate this entry */
		    mnuSelect(data->menu);
		    break;
		case KCD_HUP: /* back to previous menu */
		case KCD_RIGHT: /* back to previous menu */
		    mnuEscape(data->menu);
		    break;
		default: /* no response to all other keys */
		    return MFW_EVENT_CONSUMED;
		}
    return MFW_EVENT_CONSUMED;
}

/*******************************************************************************

 $Function:    	smscb_M_CB_MID_destroy

 $Description:	

 $Returns:		none.

 $Arguments:	
 
*******************************************************************************/
static void smscb_M_CB_MID_destroy(MfwHnd own_window)
{
    T_MFW_WIN * win_data;
    T_M_CB_MID     * data;

    TRACE_FUNCTION ("smscb_M_CB_MID_destroy()");

    if (own_window)
	{
	    win_data = ((T_MFW_HDR *)own_window)->data;
	    data = (T_M_CB_MID *)win_data->user;

	    if (data)
		{
		    /*
		     * Delete WIN handler
		     */ 
		    win_delete (data->win);
		    /*     
		     * Free Memory
		     */
		    FREE_MEMORY ((void *)data, sizeof (T_M_CB_MID));
		}
	    else
		{
		    TRACE_EVENT ("smscb_M_CB_MID_destroy() called twice");
		}
	}
}
#endif
/*********************************************************************
**********************************************************************

		M_CB_TOP	     DYNAMIC MENU WINDOW. CONTENTS BEHAVIOUR

*********************************************************************
**********************************************************************/
/*******************************************************************************

 $Function:    	smscb_M_CB_MID_DEL

 $Description:	

 $Returns:		none.

 $Arguments:	
 
*******************************************************************************/
int smscb_M_CB_MID_DEL(MfwMnu* m, MfwMnuItem* i)
{
    T_MFW_HND       win      = mfwParent((MfwHdr *)mfw_header());
    smscb_win_stat=smscb_CH_DEL;

    //	selection "Consult": menucreate M_CB_TOP_CONS
    return (int)smscb_M_CB_TOP_CONS_start (win, NULL);
}
/*******************************************************************************

 $Function:    	smscb_M_CB_TOP_ExeModify

 $Description:	

 $Returns:		

 $Arguments:	
 
*******************************************************************************/
int smscb_CB_EDT_ExeModify(MfwMnu* m, MfwMnuItem* i)
{
    T_MFW_HND       win      = mfwParent((MfwHdr *)mfw_header());

    //	selection "Consult": menucreate M_CB_TOP_CONS
    //set the state for distinguish the left key's function
    smscb_win_stat=smscb_CH_MODIFY;
    return (int)smscb_M_CB_TOP_CONS_start (win, NULL);
}
/*********************************************************************
**********************************************************************

			M_CB_TOP_CONS	DYNAMIC MENU WINDOW. IMPLEMENTATION

*********************************************************************
**********************************************************************/

/*******************************************************************************

 $Function:    	smscb_M_CB_TOP_CONS_start

 $Description:	

 $Returns:		

 $Arguments:	
 
*******************************************************************************/
T_MFW_HND smscb_M_CB_TOP_CONS_start(T_MFW_HND parent_window, void *dummy)
{
    T_MFW_HND win;

    TRACE_FUNCTION ("smscb_M_CB_TOP_CONS_start()");

    win = smscb_M_CB_TOP_CONS_create (parent_window);

    if (win NEQ NULL)
	{
	    SEND_EVENT (win, E_INIT, smscb_M_CB_TOP_CONS_ID, NULL);
	}
    return win;
}


/*******************************************************************************

 $Function:    	smscb_M_CB_TOP_CONS_create

 $Description:	

 $Returns:		

 $Arguments:	
 
*******************************************************************************/
static T_MFW_HND smscb_M_CB_TOP_CONS_create(MfwHnd parent_window)
{
    T_M_CB_TOP_CONS * data = (T_M_CB_TOP_CONS *)ALLOC_MEMORY (sizeof (T_M_CB_TOP_CONS));
    T_MFW_WIN  * win;

    TRACE_FUNCTION ("smscb_M_CB_TOP_CONS_create()");

    /*
     * Create window handler
     */
    data->win =win_create (parent_window, 0, 0, NULL);
    if (data->win EQ NULL)
	{
	    return NULL;
	}

    /* 
     * connect the dialog data to the MFW-window
     */

    data->mmi_control.dialog = (T_DIALOG_FUNC)smscb_M_CB_MID_DEL_exec_cb;
    data->mmi_control.data   = data;
    win                      = ((T_MFW_HDR *)data->win)->data;
    win->user                = (void *)data;
    data->parent             = parent_window;

    /*
     * return window handle
     */

    winShow(data->win);
    return data->win;
}

static void smscb_M_CB_TOP_CONS_destroy(MfwHnd own_window)
{
    T_MFW_WIN * win_data;
    T_M_CB_TOP_CONS     * data;

    TRACE_FUNCTION ("smscb_M_CB_TOP_CONS_destroy()");

    if (own_window)
	{
	    win_data = ((T_MFW_HDR *)own_window)->data;
	    data = (T_M_CB_TOP_CONS *)win_data->user;

	    if (data)
		{
		    /*
		     * Delete WIN handler
		     */ 
		    win_delete (data->win);

		    /*     
		     * Free Memory
		     */
		    if(data->list_menu_data.List)
		    FREE_MEMORY ((void *)data->list_menu_data.List, MAX_MIDS * sizeof (MfwMnuItem)); // c017 rsa
		    FREE_MEMORY ((void *)data, sizeof (T_M_CB_TOP_CONS));
		}
	    else
		{
		    TRACE_EVENT ("smscb_M_CB_TOP_CONS_destroy() called twice");
		}
	}
}

/*******************************************************************************

 $Function:    	smscb_M_CB_TOP_CONS_recreate

 $Description:	

 $Returns:		
 
 $Arguments:	
 
*******************************************************************************/
static void smscb_M_CB_TOP_CONS_recreate(T_M_CB_TOP_CONS *data)
{
    int i;
    int len;

#ifdef DEBUG_SMS_CB
    UBYTE ii;
    char buf[80];

	   TRACE_EVENT ("before create a list !"); 
	for (ii = 0; ii < MAX_MIDS; ii++)
	{
	   sprintf (buf, " 01 midlist:%d, i:%d", g_smscb_data.CBTopics.entry[ii].msg_id,ii);
	   TRACE_EVENT (buf); 
	}
	   sprintf (buf, "01 length:%d", g_smscb_data.CBTopics.length);
	   TRACE_EVENT (buf); 
#endif

    //on RECREATE|INIT: menucreate dynamiclist (List=CBTopics, SoftLeft=SELECT, SoftRight=BACK) /* show: 1. Name 2. MID */ 

    // c017 rsa
    /* allocate sufficient memory to hold the maximum list of menu items */
    data->list_menu_data.List = (MfwMnuItem *)ALLOC_MEMORY (MAX_MIDS * sizeof (MfwMnuItem));
    // end c017 rsa
	data->list_menu_data.ListMenuFlag=1;//for display the title
	data->list_menu_data.ListMenuHeadStr=(char *) MmiRsrcGetText(TxtCBChanel);
    for (i=0; i < g_smscb_data.CBTopics.length; i++) /* Fill Menu List */
	{
		/* 2004/06 sunsj modify for picture manage */
	    data->list_menu_data.List[i].icon = GET_ICONATTR(ICON_CB_CHANEL, 0);//cb_chanel_icon; //2004-04-20 Sunny modify old:0
	    data->list_menu_data.List[i].exta = 0;
	    data->list_menu_data.List[i].edit = 0;
	    data->list_menu_data.List[i].menu = 0;
	    data->list_menu_data.List[i].func = 0; 
	    /* get the correct entry in list of CBTopics */
	    data->list_menu_data.List[i].str = g_smscb_data.CBTopics.entry[i].name; 
	    data->list_menu_data.List[i].flagFunc = item_flag_none;
	}
	//actual number of entries in list menu
	data->list_menu_data.ListLength = g_smscb_data.CBTopics.length; 
	data->list_menu_data.autoDestroy    = TRUE;
    listDisplayListMenu(data->win, &data->list_menu_data, (ListCbFunc)smscb_M_CB_TOP_CONS_listmnu_cb,0);
}
/*******************************************************************************

 $Function:    	smscb_M_CB_MID_DEL_exec_cb

 $Description:	

 $Returns:		

 $Arguments:	
 
*******************************************************************************/
void smscb_M_CB_MID_DEL_exec_cb (T_MFW_HND win, USHORT event, SHORT value, void * dummy)
    /* callback handler for events sent to to trigger execution */
{
    //submenu "Consult": M_CB_TOP_CONS
    //on RECREATE|INIT: menucreate dynamiclist (List=CBTopics, SoftLeft=SELECT, SoftRight=BACK) /* show: 1. Name 2. MID */ 
    //on SELECT: menucreate M_CB_TOP_CONS_SEL(Index=selection)
    //on EXIT: RECREATE /* terminate EXIT from CB_EDT_MIDEDT|M_CB_TOP_CONS_SEL here */
    //on BACK: RECREATE
    //defaultreaction
    //end
    T_MFW_WIN         * win_data = ((T_MFW_HDR *) win)->data;
    T_M_CB_TOP_CONS   * data = (T_M_CB_TOP_CONS *)win_data->user;
    T_MFW_HND         * l_parent;
    SHORT               l_id;
    UBYTE i;

    TRACE_FUNCTION ("smscb_M_CB_MID_DEL_exec_cb()");

    switch (event)
	{
	case E_INIT:
	
#ifdef DEBUG_SMS_CB
    UBYTE ii;
    char buf[60];

	for (ii = 0; ii < MAX_MIDS; ii++)
	{
	   sprintf (buf, "g_smscb_data.CBTopics.entry[0].msg_id:%d, i:%d", g_smscb_data.CBTopics.entry[ii].msg_id,ii);
	   TRACE_EVENT (buf); 
	}
#endif

#if 1//disable followed code because now the data is read from flash
	if (!smsidle_get_ready_state())
	{
		l_parent = data->parent;
		info_screen(l_parent, TxtSimNot, TxtPleaseWait, NULL,REMIND_WAITTING); 
		smscb_M_CB_TOP_CONS_destroy(data->win);
	    //info_screen(data->win, TxtSimNot, TxtReady,(T_VOID_FUNC) ExeDelete_infoscreen_cb,REMIND_FAILURE); 
	    TRACE_EVENT ("SIM is not ready !!");
		return;
	}		
#endif
	if (g_smscb_data.CBTopics.length EQ 0) 
	{
		l_parent = data->parent;
	    info_screen(l_parent, TxtNull, TxtEmptyList,NULL,REMIND_NORECORD); 
		smscb_M_CB_TOP_CONS_destroy(data->win);
	    TRACE_EVENT ("CellBroadcast Topic-list is empty !!");
		return;
	}    /* initialization of administrative data */
	else
	{
	    data->id = value;
	    /* initialization of the dialog data */
	    data->list_menu_data.ListPosition   = 1;/* True cursor position in list menu. */
	    data->list_menu_data.CursorPosition = 1;/* Cursor position in list snapshot.  */
	    data->list_menu_data.SnapshotSize   = 8;/* Number of list entries to view.    */
	    data->list_menu_data.Font           = 0;
  	    if(smscb_win_stat==smscb_CH_DEL)
	  	  data->list_menu_data.LeftSoftKey    = TxtDelete;
  	    else if(smscb_win_stat==smscb_CH_MODIFY)
	    	  data->list_menu_data.LeftSoftKey    = TxtModify;
	    data->list_menu_data.RightSoftKey   = TxtSoftBack;
	    data->list_menu_data.KeyEvents      = KEY_CLEAR | KEY_RIGHT| KEY_HUP| KEY_LEFT | KEY_MNUUP | KEY_MNUDOWN;
	    data->list_menu_data.Reason         = 0;
	    data->list_menu_data.Strings        = TRUE; // c020 rsa
	    data->list_menu_data.Attr           = &smscb_M_CB_TOP_CONSAttrib;
     	    data->list_menu_data.autoDestroy    = TRUE;
	    smscb_M_CB_TOP_CONS_recreate(data);
	}
	break;

	    /* NOBREAK */
	case E_RETURN: 
	case E_EXIT: /* terminate EXIT from CB_EDT_MIDEDT|M_CB_TOP_CONS_SEL here */
	    /* (re)create the dialog handler */
        TRACE_FUNCTION ("EXIT in M_CB_TOP_CONS_exec_cb");	    

        //the special case : if the user deleted the last CB item
        //check if there is any more CB in the list when not
        //give a infoscreen and go back to the CB menu
		if (g_smscb_data.CBTopics.length EQ 0) 
 		{
		    info_screen(0, TxtNull, TxtEmptyList,NULL,REMIND_NORECORD); 
		    l_parent = data->parent;
		    l_id = data->id;
		    smscb_M_CB_TOP_CONS_destroy(data->win);
		    SEND_EVENT (l_parent, event, l_id, NULL); /* forward event to parent */

		}
		else
		{
		    l_parent = data->parent;
		    smscb_M_CB_TOP_CONS_destroy(data->win);
		    smscb_M_CB_TOP_CONS_start (l_parent, NULL);
		    //smscb_M_CB_TOP_CONS_recreate(data);

		    /* put the (new) dialog window on top of the window stack */
    	    /* winUpdate(data->win); c021 rsa */
    		}
	    break;
	case E_ABORT: /* defaultreaction */
	    TRACE_EVENT ("ABORT in M_CB_TOP_CONS_exec_cb");	    

	    l_parent = data->parent;
	    l_id = data->id;
	    smscb_M_CB_TOP_CONS_destroy(data->win);
	    SEND_EVENT (l_parent, event, l_id, NULL); /* forward event to parent */
	    break;
	default:
	    return;
	}
}

/*******************************************************************************

 $Function:    	smscb_M_CB_TOP_CONS_listmnu_cb

 $Description:	

 $Returns:		

 $Arguments:	
 
*******************************************************************************/
static void smscb_M_CB_TOP_CONS_listmnu_cb (T_MFW_HND * win, ListMenuData * ListData)
    /* smscb_M_CB_TOP_CONS menu event handler */
{
    T_MFW_WIN            * win_data = ((T_MFW_HDR *)win)->data;
    T_M_CB_TOP_CONS      * data = (T_M_CB_TOP_CONS *)win_data->user;
    T_MFW_HND            * l_parent;
    SHORT                  l_id;
    int index;
       char buf[80];
 TRACE_FUNCTION ("smscb_M_CB_TOP_CONS_listmnu_cb()");
    switch (ListData->Reason)
        {
	case LISTS_REASON_SELECT:
	case LISTS_REASON_OK: /*  */
	    //on SELECT: menucreate M_CB_TOP_CONS_SEL(Index=selection)
	    index = ListData->ListPosition;
	    data->CB_index_param.index = index;
	    sprintf (buf, "smscb_M_CB_TOP_CONS_listmnu_cb DELETE the index is %d", index);
  	    TRACE_EVENT (buf);
  	    if(smscb_win_stat==smscb_CH_DEL)
	    		smscb_M_CB_TOP_CONS_SEL_ExeDelete(win);
  	    else if(smscb_win_stat==smscb_CH_MODIFY)
			smscb_CB_EDT_MIDEDT_start(win, &data->CB_index_param);  	    	
	    break;
	    
	case LISTS_REASON_BACK: /* back to previous menu */
	    l_parent = data->parent;
	    l_id = data->id;
	    smscb_M_CB_TOP_CONS_destroy(data->win);
	    SEND_EVENT (data->parent, E_RETURN, data->id, NULL); // c007 rsa
	    break;
	default:
	    break;
        }
    return;
}



/*********************************************************************
**********************************************************************

			  M_CB_TOP_CONS_SEL   DYNAMIC MENU WINDOW. CONTENTS BEHAVIOUR

*********************************************************************
**********************************************************************/


/*******************************************************************************

 $Function:    	smscb_M_CB_TOP_CONS_SEL_ExeDelete

 $Description:	

 $Returns:		

 $Arguments:	
 
*******************************************************************************/
static int smscb_M_CB_TOP_CONS_SEL_ExeDelete(T_MFW_HND * win)
{
    T_MFW_WIN          * win_data = ((T_MFW_HDR *)win)->data;
    T_M_CB_TOP_CONS    * data = (T_M_CB_TOP_CONS *)win_data->user;
    T_MFW_HND          * l_parent;
    SHORT                l_id;
    int index;

    //index is the selected MID
    index = data->CB_index_param.index;
    //delete now the user selected Topic MID
    smscb_delete_topic((UBYTE) index);

    //show infoscreen ("Deleted"); 
    info_screen(win,  TxtNull, TxtDeleted,(T_VOID_FUNC) ExeDelete_infoscreen_cb,REMIND_SUCCEED); 
    //save the updated Topic list on the SIM
    //smscb_setCBsettings(0);
    CBSetting();
    //selfdestroy; sendevent EXIT


    return MFW_EVENT_CONSUMED;
}

void ExeDelete_infoscreen_cb (T_MFW_HND win , USHORT identifier, UBYTE reason)
{
    T_MFW_WIN          * win_data = ((T_MFW_HDR *)win)->data;
    T_M_CB_TOP_CONS    * data     = (T_M_CB_TOP_CONS *)win_data->user;
    SHORT                l_id;

    l_id     = data->id;

   //nm go back and recreate the CB list
   SEND_EVENT (win, E_EXIT, l_id, NULL);
   
}
#if 0
/*********************************************************************
**********************************************************************

			CB_EDT_TEXTEDT	 DYNAMIC MENU WINDOW. DECLARATION

*********************************************************************
**********************************************************************/

typedef struct
{
    /* administrative data */

    T_MMI_CONTROL   mmi_control;
    T_MFW_HND       win;
    SHORT           id;
    T_MFW_HND       parent;

    /* associated handlers */

    T_MFW_HND       editor;

    /* internal data */
    T_CB_index_param * param; /* pointer to parameter space in parent (index in list) */
    T_EDITOR_DATA   editor_info; /* storage for editor attributes */
    T_CB_EDT_MIDEDT_param CB_EDT_MIDEDT_param; /* parameter space for buffer for editing */
} T_CB_EDT_TEXTEDT;

static T_MFW_HND smscb_CB_EDT_TEXTEDT_create (T_MFW_HND parent);
static void smscb_CB_EDT_TEXTEDT_destroy (T_MFW_HND window);
static void smscb_CB_EDT_TEXTEDT_exec_cb (T_MFW_HND win, USHORT event, SHORT value, void * parameter);
static void smscb_CB_EDT_TEXTEDT_editor_cb (T_MFW_HND win, USHORT Identifier, USHORT event);

/*********************************************************************
**********************************************************************

			CB_EDT_TEXTEDT	DYNAMIC MENU WINDOW. IMPLEMENTATION

*********************************************************************
**********************************************************************/

T_MFW_HND smscb_CB_EDT_TEXTEDT_start(T_MFW_HND parent_window, T_CB_index_param * parameter)
{
    T_MFW_HND win;

    TRACE_FUNCTION ("smscb_CB_EDT_TEXTEDT_start()");

    win = smscb_CB_EDT_TEXTEDT_create (parent_window);

    if (win NEQ NULL)
	{
	    SEND_EVENT (win, E_INIT, smscb_CB_EDT_TEXTEDT_ID, (void *)parameter);
	}
    return win;
}


/*******************************************************************************

 $Function:    	smscb_CB_EDT_TEXTEDT_create

 $Description:	

 $Returns:		

 $Arguments:	
 
*******************************************************************************/
static T_MFW_HND smscb_CB_EDT_TEXTEDT_create(MfwHnd parent_window)
{
    T_CB_EDT_TEXTEDT      * data = (T_CB_EDT_TEXTEDT *)ALLOC_MEMORY (sizeof (T_CB_EDT_TEXTEDT));
    T_MFW_WIN             * win;

    TRACE_FUNCTION ("smscb_CB_EDT_TEXTEDT_create()");

    /*
     * Create window handler
     */

    data->win = 
	win_create (parent_window, 0, 0, NULL);
    if (data->win EQ NULL)
	{
	    return NULL;
	}

    /* 
     * connect the dialog data to the MFW-window
     */

    data->mmi_control.dialog = (T_DIALOG_FUNC)smscb_CB_EDT_TEXTEDT_exec_cb;
    data->mmi_control.data   = data;
    win                      = ((T_MFW_HDR *)data->win)->data;
    win->user                = (void *)data;
    data->parent             = parent_window;

    /*
     * return window handle
     */

    winShow(data->win);
    return data->win;
}


/*******************************************************************************

 $Function:    	smscb_CB_EDT_TEXTEDT_destroy

 $Description:	

 $Returns:		

 $Arguments:	
 
*******************************************************************************/
static void smscb_CB_EDT_TEXTEDT_destroy(MfwHnd own_window)
{
    T_MFW_WIN            * win_data;
    T_CB_EDT_TEXTEDT     * data;

    TRACE_FUNCTION ("smscb_CB_EDT_TEXTEDT_destroy()");

    if (own_window)
	{
	    win_data = ((T_MFW_HDR *)own_window)->data;
	    data     = (T_CB_EDT_TEXTEDT *)win_data->user;

	    if (data)
		{
		    /*
		     * Delete WIN handler
		     */ 
		    win_delete (data->win);

		    /*     
		     * Free Memory
		     */
		    FREE_MEMORY ((void *)data, sizeof (T_CB_EDT_TEXTEDT));
		}
	    else
		{
		    TRACE_EVENT ("smscb_CB_EDT_TEXTEDT_destroy() called twice");
		}
	}
}

/*******************************************************************************

 $Function:    	smscb_CB_EDT_TEXTEDT_exec_cb

 $Description:	

 $Returns:		

 $Arguments:	
 
*******************************************************************************/
void smscb_CB_EDT_TEXTEDT_exec_cb (T_MFW_HND win, USHORT event, SHORT value, void * parameter)
    /* callback handler for events sent to to trigger execution */
{
    //entry submenu "CB Topic Name": CB_EDT_TEXTEDT(List, Index)
    //defaultreaction
    //on INIT: exec "copy params to dynamic memory <data>"; RECREATE
    //on RECREATE: call editor (Type=alpha, Length=10, Prompt="Enter Name", Text=data->List.Text[data->ListIndex], SoftLeft=OK, SoftRight=BACK)
    //on OK: menucreate CB_EDT_MIDEDT(data)
    //on RETURN: RECREATE // recreate editor if back from number editor
    //end


    T_MFW_WIN            * win_data = ((T_MFW_HDR *) win)->data;
    T_CB_EDT_TEXTEDT     * data = (T_CB_EDT_TEXTEDT *)win_data->user;
    T_MFW_HND            * l_parent;
    SHORT                  l_id;
    int      
   	index;

#ifdef DEBUG_SMS_CB
    char buf[60];
#endif

    TRACE_FUNCTION ("smscb_CB_EDT_TEXTEDT_exec_cb()");

    switch (event)
	{
	case E_INIT:
	    /* initialization of administrative data */
	    data->id = value;
	    data->param = (T_CB_index_param *)parameter; /* store the pointer to the parameter for later use */
#ifdef DEBUG_SMS_CB
    sprintf (buf, "the index is %d", data->param->index);
    TRACE_EVENT (buf);
#endif

	    /* initialization of the dialog data */
	    index = data->param->index;
	    data->CB_EDT_MIDEDT_param.index = index; /* copy the index to parameter space of next menu */

//nm really need that ?
//nm	    strncpy (data->CB_EDT_MIDEDT_param.name, 
//nm		     g_smscb_data.CBTopics.entry[index].name, CB_TAG_LENGTH); /* edit in a copy */

//nm
	data->CB_EDT_MIDEDT_param.name[0] = '\0'; /* provide empty entry */

	    //on INIT: exec "copy params to dynamic memory <data>"; RECREATE
	    /* NOBREAK */
	case E_RETURN:    //on RETURN: RECREATE  // recreate editor if back from options
	    //on RECREATE: call editor (Type=alpha, Length=10, Prompt="Enter Name", 
	    //                          Text=data->List.Text[data->ListIndex], 
	    //                          SoftLeft=OK, SoftRight=BACK)
	    data->editor_info.editor_attr.win.px   = 6;
	    data->editor_info.editor_attr.win.py   = 12;
	    data->editor_info.editor_attr.win.sx   = 80;
//nm	    data->editor_info.editor_attr.win.sy   = 40;
	    data->editor_info.editor_attr.win.sy   = 20;
	    data->editor_info.editor_attr.fgColor  = 0;
	    data->editor_info.editor_attr.font     = 0;
	    data->editor_info.editor_attr.mode     = edtCurBar1;
	    data->editor_info.editor_attr.controls = 0;
	    data->editor_info.editor_attr.text     = data->CB_EDT_MIDEDT_param.name;  /* buffer to be edited */
	    data->editor_info.editor_attr.size     = CB_TAG_LENGTH;
							 						  
//nm	    data->editor_info.LeftSoftKey       = TxtSoftOptions;
	    data->editor_info.LeftSoftKey       = TxtSoftOK;	    
	    data->editor_info.RightSoftKey      = TxtDelete;
	    data->editor_info.TextId            = TxtEnterName;
	    data->editor_info.Identifier        = value;  /* optional */
	    data->editor_info.mode              = ALPHA_MODE;
	    data->editor_info.Callback          = (T_EDIT_CB)smscb_CB_EDT_TEXTEDT_editor_cb;
		data->editor_info.destroyEditor		= TRUE;
	    // c020 rsa
	    data->editor_info.hide                 = FALSE;
	    data->editor_info.AlternateLeftSoftKey = TxtNull;
	    data->editor_info.TextString           = NULL;
	    data->editor_info.min_enter            = 0;
	    data->editor_info.timeout              = TEN_SECS;
	    // end c020 rsa

	    /* create the dialog handler */
	    data->editor = editor_start(data->win,&data->editor_info);  /* start the editor */ 
	    break;
	case E_ABORT:
	case E_EXIT: /* defaultreaction */
	    l_parent = data->parent;
	    l_id = data->id;
	    smscb_CB_EDT_TEXTEDT_destroy(data->win);


	    SEND_EVENT (l_parent, event, l_id, NULL); /* forward event to parent */
	    break;
	default:
	    return;
	}
}

/*******************************************************************************

 $Function:    	smscb_CB_EDT_TEXTEDT_editor_cb

 $Description:	

 $Returns:		

 $Arguments:	
 
*******************************************************************************/
static void smscb_CB_EDT_TEXTEDT_editor_cb (T_MFW_HND win, USHORT Identifier, USHORT event)
    /* smscb_CB_EDT_TEXTEDT editor event handler */
{
    T_MFW_WIN    * win_data = ((T_MFW_HDR *)win)->data;
    T_CB_EDT_TEXTEDT * data = (T_CB_EDT_TEXTEDT *)win_data->user;
    T_MFW_HND    * l_parent;
    SHORT          l_id;

    TRACE_FUNCTION ("smscb_CB_EDT_TEXTEDT_editor_cb()");

    switch(event)
    	{
	case INFO_KCD_LEFT:
	    //on OK: menucreate CB_EDT_MIDEDT(data)
	   //WINDY smscb_CB_EDT_MIDEDT_start (win, &data->CB_EDT_MIDEDT_param);
	    break;
	case INFO_KCD_RIGHT:
	case INFO_KCD_HUP: /* defaultreaction */
	    l_parent = data->parent;
	    l_id = data->id;
	    smscb_CB_EDT_TEXTEDT_destroy(data->win);
	    SEND_EVENT (l_parent, E_RETURN, l_id, NULL); // c007 rsa
	    break;
	default:
	    break;
    	}
    return;
}
#endif
/*********************************************************************
**********************************************************************

			CB_EDT_MIDEDT	 DYNAMIC MENU WINDOW. DECLARATION

*********************************************************************
**********************************************************************/

typedef struct
{
    /* administrative data */

    T_MMI_CONTROL   mmi_control;
    T_MFW_HND       win;
    SHORT           id;
    T_MFW_HND       parent;

    /* associated handlers */

    T_MFW_HND       editor;

    /* internal data */
     T_CB_index_param * param; /* pointer to parameter space in parent (index in list) */
    T_CB_EDT_MIDEDT_param  MIDEDT_param; /* pointer to parameter space in parent (buffer for editing) */
    T_EDITOR_DATA   editor_info; /* storage for editor attributes */
} T_CB_EDT_MIDEDT;

static T_MFW_HND smscb_CB_EDT_MIDEDT_create (T_MFW_HND parent);
static void smscb_CB_EDT_MIDEDT_destroy (T_MFW_HND window);
static void smscb_CB_EDT_MIDEDT_exec_cb (T_MFW_HND win, USHORT event, SHORT value, void * parameter);
static void smscb_CB_EDT_MIDEDT_editor_cb (T_MFW_HND win, USHORT Identifier, USHORT event);
/*******************************************************************************

 $Function:    	smscb_M_CB_TOP_ExeAddNew

 $Description:	

 $Returns:		

 $Arguments:	
 
*******************************************************************************/
 int smscb_CB_EDT_ExeAdd(MfwMnu* m, MfwMnuItem* i)
{
    T_MFW_HND      win      = mfwParent((MfwHdr *)mfw_header());
    T_MFW_WIN    * win_data = ((T_MFW_HDR *)win)->data;
    T_M_CB_MID   * data     = (T_M_CB_MID *)win_data->user;
    int            index;

    smscb_win_stat=smscb_CH_ADD;
    //	selection "Add": call sms_cb_edit_topic(List=CBTopics, Index=ADD)
    index = g_smscb_data.CBTopics.length;/* get the index of the next free entry in list CBTopics */

    if (index >= MAXCB)
	{
	    /* all entries are already used */
	    info_screen (win, TxtNull, TxtChanelFull /* ??? */,NULL,REMIND_NORECORD);
	    return MFW_EVENT_CONSUMED;
	}
    else
	{
	    data->CB_index_param.index = index; 
	    return (int)smscb_CB_EDT_MIDEDT_start(win, &data->CB_index_param);
	}
}

/*********************************************************************
**********************************************************************

			CB_EDT_MIDEDT	DYNAMIC MENU WINDOW. IMPLEMENTATION

*********************************************************************
**********************************************************************/

T_MFW_HND smscb_CB_EDT_MIDEDT_start(T_MFW_HND parent_window, T_CB_index_param * parameter)
{
    T_MFW_HND win;

    TRACE_FUNCTION ("smscb_CB_EDT_MIDEDT_start()");

    win = smscb_CB_EDT_MIDEDT_create (parent_window);

    if (win NEQ NULL)
	{
	    SEND_EVENT (win, E_INIT, smscb_CB_EDT_MIDEDT_ID, (void *)parameter);
	}
    return win;
}

/*******************************************************************************

 $Function:    	smscb_CB_EDT_MIDEDT_create

 $Description:	

 $Returns:		

 $Arguments:	
 
*******************************************************************************/
static T_MFW_HND smscb_CB_EDT_MIDEDT_create(MfwHnd parent_window)
{
    T_CB_EDT_MIDEDT      * data = (T_CB_EDT_MIDEDT *)ALLOC_MEMORY (sizeof (T_CB_EDT_MIDEDT));
    T_MFW_WIN  * win;

    TRACE_FUNCTION ("smscb_CB_EDT_MIDEDT_create()");

    /*
     * Create window handler
     */

    data->win = win_create (parent_window, 0, 0, NULL);
    
    if (data->win EQ NULL)
	{
	    return NULL;
	}

    /* 
     * connect the dialog data to the MFW-window
     */

    data->mmi_control.dialog = (T_DIALOG_FUNC)smscb_CB_EDT_MIDEDT_exec_cb;
    data->mmi_control.data   = data;
    win                      = ((T_MFW_HDR *)data->win)->data;
    win->user                = (void *)data;
    data->parent             = parent_window;

    /*
     * return window handle
     */

    winShow(data->win);
    return data->win;
}


/*******************************************************************************

 $Function:    	smscb_CB_EDT_MIDEDT_destroy

 $Description:	

 $Returns:		

 $Arguments:	
 
*******************************************************************************/
static void smscb_CB_EDT_MIDEDT_destroy(MfwHnd own_window)
{
    T_MFW_WIN           * win_data;
    T_CB_EDT_MIDEDT     * data;


    TRACE_FUNCTION ("smscb_CB_EDT_MIDEDT_destroy()");

    if (own_window)
	{
	    win_data = ((T_MFW_HDR *)own_window)->data;
	    data     = (T_CB_EDT_MIDEDT *)win_data->user;

	    if (data)
		{
		    /*
		     * Delete WIN handler
		     */ 
		    if(data->editor!=0)
		    {
			editor_destroy(data->editor);
			data->editor = 0;
			}
		    win_delete (data->win);
		    /*     
		     * Free Memory
		     */
		    FREE_MEMORY ((void *)data, sizeof (T_CB_EDT_MIDEDT));
		}
	    else
		{
		    TRACE_EVENT ("smscb_CB_EDT_MIDEDT_destroy() called twice");
		}
	}
}


/*******************************************************************************

 $Function:    	smscb_CB_EDT_MIDEDT_exec_cb

 $Description:	
 
 $Returns:		

 $Arguments:	
 
*******************************************************************************/
void smscb_CB_EDT_MIDEDT_exec_cb (T_MFW_HND win, USHORT event, SHORT value, void * parameter)
    /* callback handler for events sent to to trigger execution */
{
    //submenu "SMS MID Number": CB_EDT_MIDEDT(data)
    //defaultreaction
    //on INIT: call editor (Type=numeric, Length=3, Prompt="Enter MID Number", Number=data->List.Number[data->ListIndex], SoftLeft=OK, SoftRight=BACK)
    //on OK: exec sms_cb_save_topic(); /* save in SIM if EF_CBMI available on SIM else save in phone memory */
    //      call infoscreen ("Saved"); selfdestroy; sendevent EXIT
    //end

    T_MFW_WIN          * win_data = ((T_MFW_HDR *) win)->data;
    T_CB_EDT_MIDEDT    * data     = (T_CB_EDT_MIDEDT *)win_data->user;
    T_MFW_HND          * l_parent;
    SHORT                l_id;
    int                  index;

    TRACE_FUNCTION ("smscb_CB_EDT_MIDEDT_exec_cb()");

    switch (event)
	{
	case E_INIT:
	    /* initialization of administrative data */
	    data->id = value;
	    data->param = (T_CB_index_param *)parameter; /* store the pointer to the parameter for later use */
	    /* initialization of the dialog data */
	    index = data->param->index;
	    data->MIDEDT_param.index = index; /* copy the index to parameter space of next menu */
	    data->MIDEDT_param.name[0] = '\0'; /* provide empty entry */

	    //windydata->param = (T_CB_EDT_MIDEDT_param *)parameter; /* store the pointer to the parameter for later use */

	    /* initialization of the dialog data */
	   //windy index = data->param->index;
	    if (g_smscb_data.CBTopics.entry[index].msg_id NEQ NO_MID)
		{
		    data->MIDEDT_param.number[0] = '\0'; /* provide empty entry */
		}
	    else
		{
		    data->MIDEDT_param.number[0] = '\0'; /* provide empty entry */
		}
	    //on INIT: call editor (Type=numeric, Length=3, Prompt="Enter MID Number", 
	    //                      Number=data->List.Number[data->ListIndex], SoftLeft=OK, SoftRight=BACK)
			sms_cb_loadEditDefault(&data->editor_info);
	    		memset(data->MIDEDT_param.number,0,sizeof(data->MIDEDT_param.number));
			editor_attr_init_fullscr(&data->editor_info.editor_attr,1,TxtCBChanel,(char *)(data->MIDEDT_param.number),
				CB_MID_LENGTH,edtCurBar1);
			data->editor_info.LeftSoftKey       = TxtSave;
			data->editor_info.RightSoftKey      = TxtDelete;
			data->editor_info.Identifier 	    = value;
			data->editor_info.Callback          = (T_EDIT_CB)smscb_CB_EDT_MIDEDT_editor_cb;
			data->editor_info.mode				= SHUZI_MODE;	/* 2004/04/16 sunsj modify */
			data->editor_info.destroyEditor     = FALSE ; 
			data->editor  = editor_start(data->win,&data->editor_info);  
	    break;
	case E_ABORT:
	case E_EXIT: /* defaultreaction */
	    l_parent = data->parent;
	    l_id = data->id;
	    smscb_CB_EDT_MIDEDT_destroy(data->win);
        SEND_EVENT (l_parent, event, l_id, NULL); /* forward event to parent */
	    break;
	default:
	    return;
	}
}


/*******************************************************************************

 $Function:    	smscb_CB_EDT_MIDEDT_editor_cb

 $Description:	

 $Returns:		

 $Arguments:	
 
*******************************************************************************/
static void smscb_CB_EDT_MIDEDT_editor_cb (T_MFW_HND win, USHORT Identifier, USHORT event)
    /* smscb_CB_EDT_MIDEDT editor event handler */
{
    T_MFW_WIN       * win_data = ((T_MFW_HDR *)win)->data;
    T_CB_EDT_MIDEDT * data = (T_CB_EDT_MIDEDT *)win_data->user;
    T_MFW_HND       * l_parent;
    SHORT             l_id;
    int               index;
    //char  num[5];
	char  num[20];/* 2003/10/17, wangyan, size of array num is too small.
				  If user input a string length more than 2, the array will overflow */

    char buf[80];

#ifdef DEBUG_SMS_CB
    char buf[80];
	UBYTE ii;
#endif

    TRACE_FUNCTION ("smscb_CB_EDT_MIDEDT_editor_cb()");

    l_parent = data->parent;
    l_id     = data->id;

    switch(event)
    	{
	case INFO_KCD_LEFT:
	    //on OK: exec sms_cb_save_topic(); /* save in SIM if EF_CBMI available on SIM else save in phone memory */
	    //      call infoscreen ("Saved"); selfdestroy; sendevent EXIT
	    
	    /* copy back from the temporary buffer */
	    index = data->param->index;

		//nm "-1"the zero is counted as well
	    if (index < MAXCB )
		{
		    /* add a new entry */
//		    index += 1;
//		    g_smscb_data.CBTopics.length = index;

		    //windystrncpy (g_smscb_data.CBTopics.entry[index].name, data->param->name, CB_TAG_LENGTH);
		    g_smscb_data.CBTopics.entry[index].msg_id = atoi(data->MIDEDT_param.number);
	  	    g_smscb_data.CBTopics.entry[index].name[0] = '\0'; 
		    /* ??? update here g_smscb_data.CBTopics.entry[i].name from PCM */
		    if (g_smscb_data.CBTopics.entry[index].name[0] EQ '\0')
			{
			    //memcpy(g_smscb_data.CBTopics.entry[index].name,(char *) MmiRsrcGetText(TxtList),5);
		    	sprintf (num,"CH%d,ID:%d", index,g_smscb_data.CBTopics.entry[index].msg_id);
		  	    wstrcat(g_smscb_data.CBTopics.entry[index].name,num);
			}
 		 sprintf (buf, "the index is %d", index);
  	  		TRACE_EVENT (buf);
		#ifdef DEBUG_SMS_CB
    			sprintf (buf, "the index is %d", index);
  		  TRACE_EVENT (buf);
		#endif

		    /* add a new entry */
  	    if(smscb_win_stat==smscb_CH_ADD)
  	    	{
		    index += 1;
		    g_smscb_data.CBTopics.length = index;
  	    	}
		    //WINDYsmscb_setCBsettings(SMSCB_SWITCH_ON); /* save in SIM */ 
		    CBSetting();/* save in SIM */ 
		    info_screen(l_parent, TxtNull,TxtSaved , NULL,REMIND_SUCCEED); 

		}
		else
		{
		    info_screen (l_parent, TxtNull, TxtFull, NULL,REMIND_NORECORD);
		}


#ifdef DEBUG_SMS_CB

	   TRACE_EVENT ("after add a new one !"); 
	for (ii = 0; ii < MAX_MIDS; ii++)
	{
	   sprintf (buf, " 04 midlist:%d, i:%d", g_smscb_data.CBTopics.entry[ii].msg_id,ii);
	   TRACE_EVENT (buf); 
	}
	   sprintf (buf, "04 midlist:%d", g_smscb_data.CBTopics.length);
	   TRACE_EVENT (buf); 
#endif

	    smscb_CB_EDT_MIDEDT_destroy(data->win);
	    SEND_EVENT (l_parent, E_EXIT, l_id, NULL);
	    break;
	case INFO_KCD_RIGHT:
	case INFO_KCD_HUP: /* defaultreaction */
	    //smscb_CB_EDT_MIDEDT_destroy(data->win);
	    //SEND_EVENT (l_parent, E_RETURN, l_id, NULL); // c007 rsa
		SEND_EVENT (win, E_EXIT, data->id, NULL);
	    break;
	default:
	    break;
    	}
    return;
}

/*********************************************************************
**********************************************************************

			cell broadcast routines IMPLEMENTATION

*********************************************************************
**********************************************************************/


/*******************************************************************************

 $Function:    	smscb_InitData

 $Description:	Initialises CB message storage

 $Returns:		none.

 $Arguments:	
 
*******************************************************************************/
void smscb_InitData (void)
{
    UBYTE i;

    g_smscb_data.CBTopics.length = 0;
    for (i = 0; i < MAX_MIDS; i++)
	{
	    g_smscb_data.CBTopics.entry[i].msg_id = NO_MID; /* Set MID list default values. */
	    g_smscb_data.CBTopics.entry[i].name[0] = '\0';
	    g_smscb_data.CBTopics.entry[i].dcs = 0;
	}

}

/*******************************************************************************

 $Function:    	smscb_delete_topic



 $Description: : This function is used to delete an MID from the
            current list <g_smscb_data.CBTopics>.


 $Returns:		

 $Arguments:	
 
*******************************************************************************/
static T_MFW smscb_delete_topic(UBYTE MidOffset)
{
    SHORT Mid;
    UBYTE MidCount;
    UBYTE i;
    char  num[25];//2004-04-20 Sunny modify. old: 5
#ifdef DEBUG_SMS_CB
    UBYTE ii;
    char buf[80];
#endif

    Mid = g_smscb_data.CBTopics.entry[MidOffset].msg_id;

    /* Find out how many messages are stored. 
     * set MidCount to first entry which contains NO_MID or 
     * set MidCount to MAX_MIDS if list is full
     */
	// 
    MidCount = g_smscb_data.CBTopics.length +1;//here need not +1 because it has done before

    /* Move the MID data up one. */
    while (MidOffset < (MidCount)) //c005 rsa avoid copying from behind the array
	{
	    g_smscb_data.CBTopics.entry[MidOffset] = g_smscb_data.CBTopics.entry[MidOffset + 1];  
	  	g_smscb_data.CBTopics.entry[MidOffset].name[0] = '\0'; 
		    /* ??? update here g_smscb_data.CBTopics.entry[i].name from PCM */
	  	//memcpy(g_smscb_data.CBTopics.entry[index].name,(char *) MmiRsrcGetText(TxtList),5);
		sprintf(num,"  CH%d,ID:%d", MidOffset,g_smscb_data.CBTopics.entry[MidOffset + 1].msg_id);
		wstrcat(g_smscb_data.CBTopics.entry[MidOffset].name,num);
	    MidOffset++;
	}

    //c005 rsa but assign NO_MID behind the last entry
    if (MidCount > 0)
        g_smscb_data.CBTopics.entry[MidCount].msg_id = NO_MID; 
    else
        g_smscb_data.CBTopics.entry[0].msg_id = NO_MID;
    //end c005 rsa
    
    g_smscb_data.CBTopics.length--;

    /* Delete any message in FIFO bearing the MID. */
    i = 0;
    while ((i < MAX_MIDS) && (g_smscb_data.smsCBList.entry[i].msg_id != Mid))
        i++;


#ifdef DEBUG_SMS_CB
    sprintf (buf, "i is %d", i);
    TRACE_EVENT (buf);
#endif


#ifdef DEBUG_SMS_CB

	   TRACE_EVENT ("after deleting !"); 
	for (ii = 0; ii < MAX_MIDS; ii++)
	{
	   sprintf (buf, " 03 midlist:%d, i:%d", g_smscb_data.CBTopics.entry[ii].msg_id,ii);
	   TRACE_EVENT (buf); 
	}
	   sprintf (buf, "03 midlist:%d", g_smscb_data.CBTopics.length);
	   TRACE_EVENT (buf); 
#endif



    return 1;
}



/*******************************************************************************

 $Function:    	smscb_getCBsettings

 $Description: This function allows the MS-User to examine the
            current list of MIDs being used to filter CB messages.


 $Returns:		

 $Arguments:	
 
*******************************************************************************/
T_MFW smscb_getCBsettings(void)
{
    T_MFW status;
    UBYTE i,d=0;

#ifdef DEBUG_SMS_CB
    char buf[80];
    TRACE_EVENT (" get the list from sim");
#endif

	//clear the Topic list 
	smscb_InitData();

	//get the Topic list from the SIM
    status = sms_get_cell_broadcast_info(&g_smscb_data.SmsCbInfo);    

	//set to default
	g_smscb_data.CBTopics.length = 0xFF;


#ifdef DEBUG_SMS_CB
//    for (i = 0; i <= 19; i++)
//	{
//	   sprintf (buf, "DATA from mfw %d i:%d", g_smscb_data.SmsCbInfo.msg_id[i],i);
//	   TRACE_EVENT (buf);
///	}
#endif


    for (i = 0; i < MAX_MIDS; i++)
	{
	    /* Update MID list from SIM data. */
	    g_smscb_data.CBTopics.entry[i].msg_id = g_smscb_data.SmsCbInfo.msg_id[d];
	    
	    g_smscb_data.CBTopics.entry[i].dcs = g_smscb_data.SmsCbInfo.dcs[d]; 

	    if (g_smscb_data.CBTopics.entry[i].msg_id EQ NO_MID)
		{
		    break;
		}
	    else
		{
		    g_smscb_data.CBTopics.length = i; 
		}

	    /* Set default tag string, until we get the saved name form PCM */
	    g_smscb_data.CBTopics.entry[i].name[0] = '\0'; 

	    /* ??? update here g_smscb_data.CBTopics.entry[i].name from PCM */
	    if (g_smscb_data.CBTopics.entry[i].name[0] EQ '\0')
		{
		    /* but if it's empty substitute it by the channel number */
		    sprintf (g_smscb_data.CBTopics.entry[i].name,"Ch%d,ID:%d",i,g_smscb_data.CBTopics.entry[i].msg_id);
		}

		d+=2;

#ifdef DEBUG_SMS_CB
//   sprintf (buf, "02 midlist:%d, i:%d", g_smscb_data.CBTopics.entry[i].msg_id,i);
//   TRACE_EVENT (buf);
#endif   

	}


	if (g_smscb_data.CBTopics.length EQ 0xFF)
	{
		//list is empty
		g_smscb_data.CBTopics.length = 0;
	}
	else
	{
		//+1 because the "0" was counted as well
		g_smscb_data.CBTopics.length +=1;	
	}

#ifdef DEBUG_SMS_CB
//   sprintf (buf, "g_smscb_data.CBTopics.length is %d", g_smscb_data.CBTopics.length);
//   TRACE_EVENT (buf);
#endif

	/* SPR759 - SH - switch on cell broadcast if set
	 * Commented out because always seems to switch it ON! */
	/*if (g_smscb_data.SmsCbInfo.mode EQ SMSCB_SWITCH_ON )
		sms_set_mt_ind(MT_CB_MSG);
	else
		sms_set_mt_ind(MT_CB_NO_IDX);*/

    return status;
}

/*******************************************************************************

 $Function:    	smscb_setCBsettings

 $Description: This function is used to activated/de-activate the CB
            feature and to submit MIDs for filter list control.

            When the CB feature is switched ON, the stored message
            index MIDs are set to invalid, i.e ready for incoming CB
            messages and the MID list is initialised. On switch OFF,
            the MID list parameters are refreshed before the switch
            OFF command is issued, to ensure that the most recent
            MID list is used. It is not necessary to handle the
            stored message index MIDs as they will be dealt with
            next time the feature is activated.
	    usage:
                 Status = smscb_setCBsettings(SMSCB_SWITCH_OFF);
                 Status = smscb_setCBsettings(SMSCB_SWITCH_ON);
 

 $Returns:		none.

 $Arguments:	
 
*******************************************************************************/
T_MFW smscb_setCBsettings(UBYTE State)
{
/* SPR759 - SH - removed 'static' so function can be called by Homezone */

    UBYTE i,d=0;
    T_MFW Status;


#ifdef DEBUG_SMS_CB
    char buf[60];
#endif

/* SH - moved up here otherwise does not execute*/
//nm 
	if (State EQ SMSCB_SWITCH_ON )
		sms_set_mt_ind(MT_CB_MSG);
	else
		sms_set_mt_ind(MT_CB_NO_IDX);
//nm

//    if (State == SMSCB_SWITCH_OFF)
//	{
//	    smscb_FlushMessageArrays();   /* Clear out the CB message storage area. */
//	}


    /* copy local list to SIM */
    g_smscb_data.SmsCbInfo.mode = State;


	for (i = 0; i < MAX_MIDS; i++)
	{
	    g_smscb_data.SmsCbInfo.msg_id[i] = NO_MID; /* Set MID list default values. */
	    g_smscb_data.SmsCbInfo.dcs[i]    = 0xFF;
	}

    
    for (i = 0; i < g_smscb_data.CBTopics.length; i++)
	{

	    g_smscb_data.SmsCbInfo.msg_id[d] = g_smscb_data.CBTopics.entry[i].msg_id;
	    /* ??? update here PCM with g_smscb_data.CBTopics.entry[i].name */
	    g_smscb_data.SmsCbInfo.dcs[d] = (char)g_smscb_data.CBTopics.entry[i].dcs;

		d++;
		
	    g_smscb_data.SmsCbInfo.msg_id[d] = g_smscb_data.CBTopics.entry[i].msg_id;
	    /* ??? update here PCM with g_smscb_data.CBTopics.entry[i].name */
	    g_smscb_data.SmsCbInfo.dcs[d] = (char)g_smscb_data.CBTopics.entry[i].dcs;

		d++;
	    
	}


#ifdef DEBUG_SMS_CB
    for (i = 0; i <= 25; i++)
	{
	   sprintf (buf, "SAVING..SmsCbInfo.msg_id[i] %d  i:%d", g_smscb_data.SmsCbInfo.msg_id[i],i);
	   TRACE_EVENT (buf);
	}
#endif

	Status = sms_set_cell_broadcast_info(State, &g_smscb_data.SmsCbInfo);

#ifdef DEBUG_SMS_CB
    sprintf (buf, "SAVING.Status is %d", Status);
    TRACE_EVENT (buf);
#endif


    return Status;
}


/*******************************************************************************

 $Function:    	check_mid_in_list

 $Description:	Check the "mid", does this Message ID exist in the CB-list

 $Returns:		

 $Arguments:	
 
*******************************************************************************/
//BOOL check_mid_in_list(USHORT mid)
//{

//	UBYTE i=0;	
 
//	while (i < MAX_MIDS)
//	{


//	    if (g_smscb_data.CBTopics.entry[i].msg_id EQ mid)
//		{
//		    return TRUE;
//		}
//	    i++;
//	}
//	return FALSE;
//}

/*******************************************************************************

 $Function:    	smscb_get_new_cb_messages_count

 $Description:	Calls from the MainMenu

 $Returns:		none.

 $Arguments:	
 
*******************************************************************************/
SHORT smscb_get_new_cb_messages_count(void)
{
    int j;
    int count;

    count = 0;
    for (j=0; j<g_smscb_data.smsCBList.length; j++)
	{
	    if (g_smscb_data.smsCBList.entry[j].status == UNREAD_MESSAGE)
		{
		    count++; /* found an unread message */
		}
	}
    return count;
}

/*******************************************************************************

 $Function:    	msgsExtractPhoneNumber

 $Description:	 This function extracts a phone number within quotes, which
            is embedded in a text message.

 $Returns:		
 $Arguments:	
 
*******************************************************************************/
void msgsExtractPhoneNumber(UBYTE *InputString, UBYTE *Buffer)
{
    UBYTE i;
    CHAR * FirstQuote;
    CHAR * SecondQuote;

    FirstQuote = strchr((const char *)InputString, (int)"\"");

    if (FirstQuote != '\0')
    {
        SecondQuote = strchr((const char *)FirstQuote++, (int)"\"");

        if (SecondQuote != '\0')
        {
            for (i = 0; FirstQuote != SecondQuote; i++)
                Buffer[i] = FirstQuote[i];
        }

        if (ss_check_ss_string((UBYTE *)Buffer) != MFW_SS_DIAL)
            Buffer[0] = '\0';
    }
}

/*******************************************************************************

 $Function:    	test_function_to_send_cb

 $Description:	This function is only for simulation of incoming cb messages

 $Returns:		
 $Arguments:	
 
*******************************************************************************/

#ifdef DEBUG_SMS_CB
void test_function_to_send_cb (void)
{
	T_MFW_SMS_CB Message;
	UBYTE i;

	
	DEBUG = FALSE;

// for testing only

TRACE_EVENT("Testcase");

Message.sn = 55;
Message.msg_id = 55;
Message.page = 1;
Message.pages = 1;
strcpy (Message.cb_msg, " 55 Toronto Blue Jays to a 4-0 victory Tuesday Multipage ");
sms_new_incoming_cb (&Message);


Message.sn = 56;
Message.msg_id = 56;
Message.page = 1;
Message.pages = 2;
strcpy (Message.cb_msg, "56 1/2  Toronto Blue Jays to a 4-0 victory Tuesday Multipage");
sms_new_incoming_cb (&Message);


Message.sn = 56;
Message.msg_id = 56;
Message.page = 2;
Message.pages = 2;
strcpy (Message.cb_msg, "56 2/2 Toronto Blue Jays to a 4-0 victory Tuesday Multipage");
sms_new_incoming_cb (&Message);

Message.sn = 58;
Message.msg_id = 58;
Message.page = 1;
Message.pages = 1;
strcpy (Message.cb_msg, "58 Toronto Blue Jays to a 4-0 victory Tuesday Multipage ");
sms_new_incoming_cb (&Message);


Message.sn = 16384;
Message.msg_id = 59;
Message.page = 1;
Message.pages = 3;
strcpy (Message.cb_msg, "59 1/3 Toronto Blue Jays to a 4-0 victory Tuesday Multipage 1/3");
sms_new_incoming_cb (&Message);

Message.sn = 16384;
Message.msg_id = 59;
Message.page = 2;
Message.pages = 3;
strcpy (Message.cb_msg, "59 2/3 Toronto Blue Jays to a 4-0 victory Tuesday Multipage 2/3");
sms_new_incoming_cb (&Message);

//Message.sn = 16384;
//Message.msg_id = 59;
//Message.page = 3;
//Message.pages = 3;
//strcpy (Message.cb_msg, "59 3/3 Toronto Blue Jays to a 4-0 victory Tuesday Multipage 3/3");
//sms_new_incoming_cb (&Message);


Message.sn = 60;
Message.msg_id = 60;
Message.page = 1;
Message.pages = 1;
strcpy (Message.cb_msg, "60 Toronto Blue Jays to a 4-0 victory Tuesday ");
sms_new_incoming_cb (&Message);


Message.sn = 61;
Message.msg_id = 61;
Message.page = 1;
Message.pages = 1;
strcpy (Message.cb_msg, "61 Toronto Blue Jays to a 4-0 victory Tuesday");
sms_new_incoming_cb (&Message);


Message.sn = 62;
Message.msg_id = 62;
Message.page = 1;
Message.pages = 1;
strcpy (Message.cb_msg, "62 Toronto Blue Jays to a 4-0 victory Tuesday");
sms_new_incoming_cb (&Message);


Message.sn = 63;
Message.msg_id = 63;
Message.page = 1;
Message.pages = 1;
strcpy (Message.cb_msg, "63 Toronto Blue Jays to a 4-0 victory Tuesday");
sms_new_incoming_cb (&Message);


Message.sn = 64;
Message.msg_id = 64;
Message.page = 1;
Message.pages = 1;
strcpy (Message.cb_msg, "64 Toronto Blue Jays to a 4-0 victory Tuesday");
sms_new_incoming_cb (&Message);

Message.sn = 65;
Message.msg_id = 65;
Message.page = 1;
Message.pages = 3;
strcpy (Message.cb_msg, "65 1/3 Toronto Blue Jays to a 4-0 victory Tuesday");
sms_new_incoming_cb (&Message);

Message.sn = 65;
Message.msg_id = 65;
Message.page = 2;
Message.pages = 3;
strcpy (Message.cb_msg, "65 2/3 Toronto Blue Jays to a 4-0 victory Tuesday 65");
sms_new_incoming_cb (&Message);

Message.sn = 65;
Message.msg_id = 65;
Message.page = 3;
Message.pages = 3;
strcpy (Message.cb_msg, "65 3/3 Toronto Blue Jays to a 4-0 victory Tuesday 65");
sms_new_incoming_cb (&Message);


Message.sn = 66;
Message.msg_id = 66;
Message.page = 1;
Message.pages = 1;
strcpy (Message.cb_msg, "66 Toronto Blue Jays to a 4-0 victory Tuesday");
sms_new_incoming_cb (&Message);

Message.sn = 67;
Message.msg_id = 67;
Message.page = 1;
Message.pages = 1;
strcpy (Message.cb_msg, " Toronto Blue Jays to a 4-0 victory Tuesday 67");
sms_new_incoming_cb (&Message);

Message.sn = 68;
Message.msg_id = 68;
Message.page = 1;
Message.pages = 1;
strcpy (Message.cb_msg, " Toronto Blue Jays to a 4-0 victory Tuesday 68 ");
sms_new_incoming_cb (&Message);


Message.sn = 72;
Message.msg_id = 72;
Message.page = 1;
Message.pages = 4;
strcpy (Message.cb_msg, " Toronto Blue Jays to a 4-0 victory Tuesday 72 ");
sms_new_incoming_cb (&Message);

Message.sn = 72;
Message.msg_id = 72;
Message.page = 2;
Message.pages = 4;
strcpy (Message.cb_msg, " Toronto Blue Jays to a 4-0 victory Tuesday 72 ");
sms_new_incoming_cb (&Message);

Message.sn = 72;
Message.msg_id = 72;
Message.page = 3;
Message.pages = 4;
strcpy (Message.cb_msg, " Toronto Blue Jays to a 4-0 victory Tuesday 72 ");
sms_new_incoming_cb (&Message);

Message.sn = 72;
Message.msg_id = 72;
Message.page =4;
Message.pages = 4;
strcpy (Message.cb_msg, " Toronto Blue Jays to a 4-0 victory Tuesday 72 ");
sms_new_incoming_cb (&Message);

Message.sn = 73;
Message.msg_id = 73;
Message.page = 1;
Message.pages = 1;
strcpy (Message.cb_msg, " Toronto Blue Jays to a 4-0 victory Tuesday 73 ");
sms_new_incoming_cb (&Message);


Message.sn = 74;
Message.msg_id = 74;
Message.page = 1;
Message.pages = 1;
strcpy (Message.cb_msg, " Toronto Blue Jays to a 4-0 victory Tuesday 74");
sms_new_incoming_cb (&Message);

Message.sn = 75;
Message.msg_id = 75;
Message.page = 1;
Message.pages = 1;
strcpy (Message.cb_msg, " Toronto Blue Jays to a 4-0 victory Tuesday 75 ");
sms_new_incoming_cb (&Message);

Message.sn = 77;
Message.msg_id = 77;
Message.page = 1;
Message.pages = 2;
strcpy (Message.cb_msg, "1/2 Toronto Blue Jays to a 4-0 victory Tuesday 77");
sms_new_incoming_cb (&Message);

Message.sn = 77;
Message.msg_id = 77;
Message.page = 2;
Message.pages = 2;
strcpy (Message.cb_msg, "2/2 Toronto Blue Jays to a 4-0 victory Tuesday 77 ");
sms_new_incoming_cb (&Message);
//
Message.sn = 78;
Message.msg_id = 78;
Message.page = 1;
Message.pages = 1;
strcpy (Message.cb_msg, " Toronto Blue Jays to a 4-0 victory Tuesday 78");
sms_new_incoming_cb (&Message);

Message.sn = 16386;
Message.msg_id = 79;
Message.page = 1;
Message.pages = 1;
strcpy (Message.cb_msg, " Toronto Blue Jays to a 4-0 victory Tuesday 79 ");
sms_new_incoming_cb (&Message);

Message.sn = 80;
Message.msg_id = 80;
Message.page = 1;
Message.pages = 3;
strcpy (Message.cb_msg, " Toronto Blue Jays to a 4-0 victory Tuesday 80");
sms_new_incoming_cb (&Message);

Message.sn = 80;
Message.msg_id = 80;
Message.page = 2;
Message.pages = 3;
strcpy (Message.cb_msg, "80 1/3 Toronto Blue Jays to Tuesday");
sms_new_incoming_cb (&Message);

Message.sn = 80;
Message.msg_id = 80;
Message.page = 3;
Message.pages = 3;
strcpy (Message.cb_msg, "80 2/3 Yoronto Blue Jays to a 4-0 Tuesday");
sms_new_incoming_cb (&Message);

DEBUG = TRUE;

Message.sn = 83;
Message.msg_id = 83;
Message.page = 1;
Message.pages = 1;
strcpy (Message.cb_msg, " Toronto Blue Jays to a 4-0 victory Tuesday 83");
sms_new_incoming_cb (&Message);


}

#endif


/*******************************************************************************

 $Function:    	sms_cb_init

 $Description:	This is initialize the CB fifo and the attributes after switch on the mobile

 $Returns:		
 $Arguments:	
 
*******************************************************************************/
void sms_cb_init (void)
{

//for tracing
#ifdef DEBUG_SMS_CB
	 char buf [100];
#endif 


	UBYTE i;

	
	//set to the default, "FF" -> no CB messages stored
	smscb_data.used_pointer = 0xFF;

	//set to the default, "0" -> free spaces on the first index in fifo
	smscb_data.free_pointer = 0;


	//set the fifo to the default
	for (i=0; i<MAX_STORED_PAGES -1 ; i++)	
	{
		smscb_data.cb_attributes[i].next_link = i + 1;
	}

	for (i=0; i<MAX_STORED_PAGES ; i++)	
	{
		smscb_data.cb_attributes[i].start_page = 0; 
	}



	//"FF" on the last entry means end of the free spaces list
	smscb_data.cb_attributes[MAX_STORED_PAGES-1].next_link  = 0xFF;

//for tracing
#ifdef DEBUG_SMS_CB

	test_function_to_send_cb ();

//	for (i=0; i<MAX_STORED_PAGES ; i++)	
//	{
//		sprintf(buf, "free_index: %d, i:%d", smscb_data.cb_attributes[i].next_link,i); 
//		TRACE_EVENT(buf);
//	}

//	sprintf(buf, "count of messages:%d", sms_cb_count_messages ()); 
//	TRACE_EVENT(buf);

//	for (i=0; i<MAX_STORED_PAGES ; i++)	
//	{
//		sprintf(buf, "start_page: %d, i:%d", smscb_data.cb_attributes[i].start_page,i); 
//		TRACE_EVENT(buf);
//	}

//	sprintf(buf, "smscb_data.used_pointer%d", smscb_data.used_pointer); 
//	TRACE_EVENT(buf);
//	sprintf(buf, "smscb_data.free_pointer%d", smscb_data.free_pointer); 
//	TRACE_EVENT(buf);



#endif //DEBUG_SMS_CB




}


/*******************************************************************************

 $Function:    	sms_new_incoming_cb

 $Description:	This function will be call after each NEW incoming cb messages.
				It checks for free spaces and organize to save the cb messages
				in the right order.
 $Returns:		

 $Arguments:	
 
*******************************************************************************/
void sms_new_incoming_cb (T_MFW_SMS_CB *Message)
{

//only for tracing
#ifdef DEBUG_SMS_CB
	char buf[100];
	UBYTE i, test;
#endif


	//set to default
	UBYTE result =0;
	smscb_data.found_mid = FALSE;
	smscb_data.clear_mid = FALSE;


//only for tracing
#ifdef DEBUG_SMS_CB
	TRACE_EVENT("--------------start------------");
//	sprintf(buf, "1 smscb_data.used_pointer %d ", smscb_data.used_pointer); 
//	TRACE_EVENT(buf);			
//	sprintf(buf, "1 smscb_data.free_pointer %d ", smscb_data.free_pointer); 
//	TRACE_EVENT(buf);			
//	sprintf(buf, "1 smscb_data.result_used %d ", smscb_data.result_used); 
//	TRACE_EVENT(buf);			
#endif //DEBUG_SMS_CB


	//search the new MID with the MID's in the fifo.
	//used_pointer EQ 0xFF means there is no message in the fifo
	if (smscb_data.used_pointer NEQ 0xFF)
	{
		result = sms_cb_search_mid (Message->msg_id, Message->sn);
	}


	//error: in searching mid
	if (result EQ 0xAA)
	{
		TRACE_EVENT("sms cb ERROR, in searching mid");
		return;
	}


	// does the message with the same MID already exist in the fifo ?
	if(smscb_data.found_mid)
	{
		TRACE_EVENT("MID already exist");

		sms_cb_delete_message (Message->msg_id);

		
		smscb_data.clear_mid = TRUE;
	}


	// does we have free space ?
	if (smscb_data.free_pointer EQ 0xFF)
	{
		//no free spaces -> means overwrite the oldest message
		sms_cb_overwrite_old_message (Message);
		
	}
	else // does we have free space ?
	{
		//still free spaces 
		sms_cb_store_message (Message);

	}



//only for tracing
#ifdef DEBUG_SMS_CB
	sprintf(buf, "5 smscb_data.used_pointer %d ", smscb_data.used_pointer); 
	TRACE_EVENT(buf);
	sprintf(buf, "5 smscb_data.free_pointer %d ", smscb_data.free_pointer); 
	TRACE_EVENT(buf);
//	sprintf(buf, "5 smscb_data.result_used %d ", smscb_data.result_used); 
//	TRACE_EVENT(buf);			


//	if (DEBUG)
//	{
//		for (i=0; i<MAX_STORED_PAGES ; i++)	
//		{
//			sprintf(buf, "link_index: %d, i:%d", smscb_data.cb_attributes[i].next_link,i); 
//			TRACE_EVENT(buf);
//		}
//		i = 0;
//	    test = smscb_data.used_pointer;
//		while (smscb_data.cb_attributes[test].next_link NEQ 0xFF)
//		{
//			sprintf(buf, "Buffer on %s,i:%d ", &smscb_data.cb_buffer[OFFSET(test)], i); 
//			TRACE_EVENT(buf);
			// going through the link list
//			test = smscb_data.cb_attributes[test].next_link;
			// only to be sure there is a ever ending loop
//			i++;
//			if (i > MAX_STORED_PAGES)	
//				return;
//		}
//		sprintf(buf, "Buffer on %s,i:%d ", &smscb_data.cb_buffer[OFFSET(test)], i); 
//		TRACE_EVENT(buf);
//
//		sprintf(buf, "count of messages:%d", sms_cb_count_messages ()); 
//		TRACE_EVENT(buf);

//		for (i=0; i<MAX_STORED_PAGES ; i++)	
//		{
//			sprintf(buf, "start_page: %d, i:%d", smscb_data.cb_attributes[i].start_page,i); 
//			TRACE_EVENT(buf);
//		}
//	}

	TRACE_EVENT("--------------finish------------");


#endif //DEBUG_SMS_CB

}



/*******************************************************************************

 $Function:    	sms_cb_store_message

 $Description:	This function will be call when we have still free space for saving
                 new incoming cb messages.
 $Returns:		

 $Arguments:	
 
*******************************************************************************/
UBYTE sms_cb_store_message (T_MFW_SMS_CB *Message)
{

	TRACE_FUNCTION("sms_cb_store_message");


	//it is only for the first time
	if (smscb_data.used_pointer EQ 0xFF)
	{
		// request for a free space
		smscb_data.used_pointer = smscb_data.free_pointer;
	}


	//save the new message in the fifo
	strcpy(&smscb_data.cb_buffer[OFFSET(smscb_data.free_pointer)], Message->cb_msg);

	//save the attributes of the new cb
	sms_cb_store_attributes (smscb_data.free_pointer, Message);

	//set the status of the message
	smscb_data.cb_attributes[smscb_data.free_pointer].status = UNREAD_MESSAGE;

	//was it the last free space ?
	if (smscb_data.cb_attributes[smscb_data.free_pointer].next_link EQ 0xFF)
	{
		//no more free space available

		UBYTE temp, result;
		

		//keep the link-value for later
		temp = smscb_data.used_pointer;

		//find the termination in the link list
		result = sms_cb_find_termination (&smscb_data.used_pointer);			
		
		//error: couldnt find the termination ?!
		if (result EQ 0xAA)
		{
			TRACE_EVENT("sms cb ERROR, couldnt find the termination");
			return;
		}


		//update the link-list
		smscb_data.cb_attributes[smscb_data.used_pointer].next_link = smscb_data.free_pointer;

		//no more free space
		smscb_data.free_pointer = 0xFF;

		//update the used pointer
		smscb_data.used_pointer = temp;

		//did it clear before ?
		if (smscb_data.clear_mid)
		{
			smscb_data.used_pointer = smscb_data.result_used;
		}


	}
	else //was it the last free space ?
	{
		//no, still free space available

		UBYTE temp, temp_2, result;

		//keep the usedpointer-value for later
		temp_2 = smscb_data.used_pointer;


		//keep the link-value for later
		temp = smscb_data.cb_attributes[smscb_data.free_pointer].next_link;


		//terminate the used list
		smscb_data.cb_attributes[smscb_data.free_pointer].next_link = 0xFF;


		//find the termination in the link list
		result = sms_cb_find_termination (&smscb_data.used_pointer);			

		//error: couldnt find the termination ?!
		if (result EQ 0xAA)
		{
			TRACE_EVENT("sms cb ERROR, couldnt find the termination");
			return;
		}


		//update the use pointer
		if (smscb_data.used_pointer NEQ smscb_data.free_pointer)
		{
			smscb_data.cb_attributes[smscb_data.used_pointer].next_link = smscb_data.free_pointer;
		}

	
		// update the free pointer as well
		smscb_data.free_pointer = temp;


		//did it clear before ?
		if (smscb_data.clear_mid)
		{
			smscb_data.used_pointer = smscb_data.result_used;
		}
		else
		{
			//update the used pointer
			 smscb_data.used_pointer = temp_2 ;
		}
	

	}
	


}

/*******************************************************************************

 $Function:    	sms_cb_overwrite_old_message

 $Description:	This function will be call when we don't have free space for saving
                 new incoming cb messages. The new cb just overwrite the 
                 oldest cb in the fifo.

 $Returns:		

 $Arguments:	
 
*******************************************************************************/
UBYTE sms_cb_overwrite_old_message (T_MFW_SMS_CB *Message)
{
	UBYTE temp, temp_2, result;

//only tracing
#ifdef DEBUG_SMS_CB
	char buf[100];

	sprintf(buf, "used_pointer= %d ", smscb_data.used_pointer); 
	TRACE_EVENT(buf);			
	sprintf(buf, "pages = %d, ", smscb_data.cb_attributes[smscb_data.used_pointer].pages); 
	TRACE_EVENT(buf);			
	sprintf(buf, "page = %d ", smscb_data.cb_attributes[smscb_data.used_pointer].page); 
	TRACE_EVENT(buf);			
	sprintf(buf, "mid = %d ", smscb_data.cb_attributes[smscb_data.used_pointer].mid); 
	TRACE_EVENT(buf);			
#endif //DEBUG_SMS_CB


	TRACE_EVENT("sms_cb_overwrite_old_message");

	//keep the value for later
//	temp = smscb_data.used_pointer;


	//find the page in the fifo with the same MID number & delete them
	sms_cb_delete_message (smscb_data.cb_attributes[smscb_data.used_pointer].mid);


	//now we have free space to store the new incoming cb message	
	sms_cb_store_message (Message);

}

/*******************************************************************************

 $Function:    	sms_cb_store_attributes

 $Description:	Store the attributes of the new incoming cb 

				Attributes are stored seperatly from the message text
 $Returns:		

 $Arguments:	
 
*******************************************************************************/

void sms_cb_store_attributes (UBYTE position, T_MFW_SMS_CB *Message)
{
	smscb_data.cb_attributes[position].sn	= Message->sn;
	smscb_data.cb_attributes[position].mid	= Message->msg_id;
	smscb_data.cb_attributes[position].dcs	= Message->dcs;
	smscb_data.cb_attributes[position].page	= Message->page;
	smscb_data.cb_attributes[position].pages      = Message->pages;
	smscb_data.cb_attributes[position].msg_len    = Message->msg_len;	

}


/*******************************************************************************

 $Function:    	sms_cb_search_mid

 $Description:	This function searchs the new incoming message in the 
                fifo for the already existing stored messages.
                If true, the stored messages in the fifo will be removed 

 $Returns:		

 $Arguments:	
 
*******************************************************************************/
UBYTE sms_cb_search_mid (USHORT mid ,SHORT sn)
{
	UBYTE temp, i=0;
	SHORT update_number_mask = 0x0F;

	temp = smscb_data.used_pointer;


	if (smscb_data.cb_attributes[temp].mid EQ mid)
	{
		//found the mid in the fifo

		if ((smscb_data.cb_attributes[temp].sn & update_number_mask) NEQ
			(sn & update_number_mask) )
		{
		   //the new cb is the update of the existing one 
			smscb_data.found_mid = TRUE;
   		   TRACE_EVENT("found = True, return");
			return temp;
		}
	}


	while (smscb_data.cb_attributes[temp].next_link NEQ 0xFF)
	{
		
		// going through the link list
		temp = smscb_data.cb_attributes[temp].next_link;


		if (smscb_data.cb_attributes[temp].mid EQ mid)
		{
			//found the mid in the fifo

			if ((smscb_data.cb_attributes[temp].sn & update_number_mask) NEQ
				(sn & update_number_mask) )
			{

			   //the new cb is the update of the existing one in the fifo
				smscb_data.found_mid = TRUE;
				return temp;
			}
		}
		

		// only to be sure there is a ever ending loop
		i++;
		if (i > MAX_STORED_PAGES)	
			return 0xAA;
	}
	
	return temp;
}



/*******************************************************************************

 $Function:    	sms_cb_delete_message

 $Description:	This function delete the messages in the fifo, even for multipages
           
 $Returns:		

 $Arguments:	
 
*******************************************************************************/
UBYTE sms_cb_delete_message (USHORT mid)
{

//only tracing
#ifdef DEBUG_SMS_CB
	char buf[100];
#endif 

	UBYTE temp, temp_2, result, i=0;

	TRACE_EVENT("sms_cb_delete_message");


	//keep the value for later
	temp_2 = smscb_data.used_pointer;


	// keep the value for later
	smscb_data.start_used = smscb_data.used_pointer;

	//search for the first page of the multipages
	smscb_data.start_used = sms_cb_find_startpoint (mid, smscb_data.used_pointer);

	//error: couldnt find the termination ?!
	if (smscb_data.start_used EQ 0xAA)
	{
		TRACE_EVENT("sms cb ERROR, couldnt find the termination");
		return;
	}


	//found immediately ?
	if (smscb_data.start_used EQ smscb_data.used_pointer)
	{
		//is in the OLDEST position in the fifo

		TRACE_EVENT("multipage is in the oldest position");		
		
		//keep the value for later
		temp = smscb_data.used_pointer;

		//search for the last page 
		smscb_data.end_used = sms_cb_find_endpoint (mid, smscb_data.used_pointer);


		//error: couldnt find the termination ?!
		if (smscb_data.end_used EQ 0xAA)
		{
			TRACE_EVENT("sms cb ERROR");
			return;
		}



		if ( (smscb_data.cb_attributes[smscb_data.used_pointer].next_link EQ 0xFF) &&
				(smscb_data.cb_attributes[smscb_data.used_pointer].mid EQ mid) ) 
		{
			//it is the oldest cb and also the only one in the fifo
			TRACE_EVENT("oldest position and the only one in the list");


			//deleted messages -> now more space availabe -> update the free pointer
			result = sms_cb_update_free_pointer (temp);

			//terminate the free pointer list 
			//smscb_data.cb_attributes[smscb_data.end_used].next_link = 0xFF;

			//error: couldnt find the termination ?!
			if (result EQ 0xAA)
			{
				TRACE_EVENT("sms cb ERROR, couldnt find the termination");
				return;
			}

			//no anymore cb message in the fifo
			//terminate the use list
			smscb_data.used_pointer = 0xFF;

			smscb_data.result_used = result;

		}
		else
		{
			//deleted messages -> now more space availabe -> update the free pointer
			result = sms_cb_update_free_pointer (temp);

			//terminate the free pointer list 
			smscb_data.cb_attributes[smscb_data.end_used].next_link = 0xFF;


			//error: couldnt find the termination ?!
			if (result EQ 0xAA)
			{
				TRACE_EVENT("sms cb ERROR, couldnt find the termination");
				return;	
			}

		}
	
	
	}
	else //found immediately ?
	{

		//keep the value for later
		temp = smscb_data.used_pointer;

		//search for the last page of the multipages
		smscb_data.end_used = sms_cb_find_endpoint (mid, smscb_data.used_pointer);

		if (smscb_data.start_used EQ 0xAA)
		{
			TRACE_EVENT("sms cb ERROR, couldnt find the termination");
			return;
		}


		if (smscb_data.cb_attributes[smscb_data.used_pointer].next_link EQ 0xFF &&
			smscb_data.cb_attributes[smscb_data.used_pointer].mid EQ mid)
		{
			//is in the the latest position in the fifo

			TRACE_EVENT("multipage is in the latest position");
			
			//terminate the position where the multipage began
			smscb_data.cb_attributes[smscb_data.start_used].next_link= 0xFF;


			//deleted messages -> now more space availabe -> update the free pointer
			result = sms_cb_update_free_pointer (temp);

			//terminate the free pointer list 
			smscb_data.cb_attributes[smscb_data.used_pointer].next_link = 0xFF;


			//error: couldnt find the termination ?!
			if (result EQ 0xAA)
			{
				TRACE_EVENT("sms cb ERROR, couldnt find the termination");
				return;
			}

			smscb_data.result_used = temp_2;
			smscb_data.used_pointer = temp_2;

			
		}
		else
		{
			//is in the middle position in the fifo

			TRACE_EVENT("multipage is in the middle position");
			
			//update the link list
			smscb_data.cb_attributes[smscb_data.start_used].next_link = smscb_data.used_pointer;		

			//deleted messages -> now more space availabe -> update the free pointer
			result = sms_cb_update_free_pointer (temp);

			//terminate the free pointer list 
			smscb_data.cb_attributes[smscb_data.end_used].next_link = 0xFF;



//only tracing
#ifdef DEBUG_SMS_CB
//			TRACE_EVENT("after free_pointer_update");	
//			sprintf(buf, "smscb_data.used_pointer  %d ", smscb_data.used_pointer); 
//			TRACE_EVENT(buf);	
//			sprintf(buf, "smscb_data.free_pointer  %d ", smscb_data.free_pointer); 
//			TRACE_EVENT(buf);	
//			sprintf(buf, "result of free pointer update %d ", result); 
//			TRACE_EVENT(buf);	
//			TRACE_EVENT("end from free_pointer_update");	
#endif //DEBUG_SMS_CB
		


			//error: couldnt find the termination ?!
			if (result EQ 0xAA)
			{
				TRACE_EVENT("sms cb ERROR, couldnt find the termination");
				return;
			}


			smscb_data.result_used = temp_2;		

		}
			

	}


}

/*******************************************************************************

 $Function:    	sms_cb_find_startpoint

 $Description:	find the startpoint of the multipage. It is also used for singlepage.

           
 $Returns:		

 $Arguments:	
 
*******************************************************************************/
UBYTE sms_cb_find_startpoint (USHORT mid, UBYTE start_used)
{
	UBYTE i = 0;

//only tracing
#ifdef DEBUG_SMS_CB
//	char buf[100];

	TRACE_EVENT("in sms_cb_find_startpoint ");	
//	sprintf(buf, "smscb_data.used_pointer:%d ", smscb_data.used_pointer); 
//	TRACE_EVENT(buf);	
//	sprintf(buf, "smscb_data.free_pointer:%d ", smscb_data.free_pointer); 
//	TRACE_EVENT(buf);	
//	sprintf(buf, "start_used:%d ", start_used); 
//	TRACE_EVENT(buf);	
#endif //DEBUG_SMS_CB


	//search for the first page of the multipages
	while (smscb_data.cb_attributes[smscb_data.used_pointer].mid NEQ mid)
	{
	
		start_used = smscb_data.used_pointer;

		// going through the link list
		smscb_data.used_pointer = smscb_data.cb_attributes[smscb_data.used_pointer].next_link;
	
			// only to be sure there is a ever ending loop
			i++;
			if (i > MAX_STORED_PAGES)	
				return 0xAA;
	}

//only tracing
#ifdef DEBUG_SMS_CB
//	TRACE_EVENT("after sms_cb_find_startpoint ");	
//	sprintf(buf, "smscb_data.used_pointer:%d ", smscb_data.used_pointer); 
//	TRACE_EVENT(buf);	
//	sprintf(buf, "smscb_data.free_pointer:%d ", smscb_data.free_pointer); 
//	TRACE_EVENT(buf);	
//	sprintf(buf, "start_used:%d ", start_used); 
//	TRACE_EVENT(buf);	
#endif //DEBUG_SMS_CB

	return start_used;
}

/*******************************************************************************

 $Function:    	sms_cb_find_startpoint

 $Description:	find the lastpage of the multipage.
 				It is also used for the singlepage but in this case the 
 				end_used will be equal to used_pointer
           
 $Returns:		

 $Arguments:	
 
*******************************************************************************/
UBYTE sms_cb_find_endpoint (USHORT mid, UBYTE end_used)
{
	UBYTE i = 0;

//only for tracing 
#ifdef DEBUG_SMS_CB
//	char buf[100];

	TRACE_EVENT("in sms_cb_find_endpoint ");	
//	sprintf(buf, "smscb_data.used_pointer:%d ", smscb_data.used_pointer); 
//	TRACE_EVENT(buf);	
//	sprintf(buf, "smscb_data.free_pointer:%d ", smscb_data.free_pointer); 
//	TRACE_EVENT(buf);	
//	sprintf(buf, "end_used:%d ", end_used); 
//	TRACE_EVENT(buf);	
#endif //DEBUG_SMS_CB


	//search for the last page of the multipages
	while (smscb_data.cb_attributes[smscb_data.used_pointer].mid EQ mid &&
			smscb_data.cb_attributes[smscb_data.used_pointer].next_link NEQ 0xFF )
	{
	
		end_used = smscb_data.used_pointer;

		// going through the link list
		smscb_data.used_pointer = smscb_data.cb_attributes[smscb_data.used_pointer].next_link;
	
		// only to be sure there is a ever ending loop
		i++;
		if (i > MAX_STORED_PAGES)	
			return 0xAA;
	}



//only for tracing 
#ifdef DEBUG_SMS_CB
//	TRACE_EVENT("after sms_cb_find_endpoint ");	
//	sprintf(buf, "smscb_data.used_pointer:%d ", smscb_data.used_pointer); 
//	TRACE_EVENT(buf);	
//	sprintf(buf, "smscb_data.free_pointer:%d ", smscb_data.free_pointer); 
//	TRACE_EVENT(buf);	
//	sprintf(buf, "end_used:%d ", end_used); 
//	TRACE_EVENT(buf);	
#endif //DEBUG_SMS_CB

	return end_used;
}

/*******************************************************************************

 $Function:    	sms_cb_update_free_pointer

 $Description:	This function is updating the free pointer after deleting 
		 		cb messages in the fifo (space available !)

           
 $Returns:		

 $Arguments:	
 
*******************************************************************************/
UBYTE sms_cb_update_free_pointer (UBYTE temp)
{
	UBYTE update, i=0;

//only for tracing 
#ifdef DEBUG_SMS_CB
//	char buf[100];	

//	TRACE_EVENT("in sms_cb_update_free_pointer ");	
//	sprintf(buf, "smscb_data.used_pointer:%d ", smscb_data.used_pointer); 
//	TRACE_EVENT(buf);	
//	sprintf(buf, "smscb_data.free_pointer:%d ", smscb_data.free_pointer); 
//	TRACE_EVENT(buf);	
//	sprintf(buf, "temp:%d ", temp); 
//	TRACE_EVENT(buf);	
#endif //DEBUG_SMS_CB


	TRACE_EVENT("sms_cb_update_free_pointer");

	if (smscb_data.free_pointer EQ 0xFF)
	{
		//take the old used_pointer (before of deleting the messages)
		smscb_data.free_pointer = temp;

	}
	else
	{
		//
		update = smscb_data.free_pointer;

		//
		while (smscb_data.cb_attributes[smscb_data.free_pointer].next_link NEQ 0xFF)
		{

			// going through the link list
			smscb_data.free_pointer = smscb_data.cb_attributes[smscb_data.free_pointer].next_link;


			// only to be sure there is a ever ending loop
			i++;
			if (i > MAX_STORED_PAGES)	
				return 0xAA;
		}


		//continue the free pointer link list to the new deleted multipage
		smscb_data.cb_attributes[smscb_data.free_pointer].next_link = temp;


		//show on the first deleted multipage 
		smscb_data.free_pointer = update;

		
	}


	//indicate that we deleted cb messages
	smscb_data.clear_mid = TRUE;


	//need for later
	smscb_data.result_used = smscb_data.used_pointer;


//only for tracing 
#ifdef DEBUG_SMS_CB
//	TRACE_EVENT("after update the free pointer");	
//	sprintf(buf, "smscb_data.result_used  %d ", smscb_data.result_used); 
//	TRACE_EVENT(buf);	
//	sprintf(buf, "smscb_data.used_pointer %d ", smscb_data.used_pointer); 
//	TRACE_EVENT(buf);	
//	sprintf(buf, "smscb_data.free_pointer %d ", smscb_data.free_pointer); 
//	TRACE_EVENT(buf);	
//	sprintf(buf, "update of temp2 is %d ", update); 
//	TRACE_EVENT(buf);	
#endif //DEBUG_SMS_CB


	return update;


}


/*******************************************************************************

 $Function:    	sms_cb_find_termination

 $Description:	This function search for the termination in the link list.
 				It jumps from one link to the another link until he founds 
 				the termination "0xFF"
           
 $Returns:		

 $Arguments:	
 
*******************************************************************************/
UBYTE sms_cb_find_termination (UBYTE *used)
{
	UBYTE i=0;
	
	while (smscb_data.cb_attributes[*used].next_link NEQ 0xFF)
	{
		// going through the link list
		*used = smscb_data.cb_attributes[*used].next_link;
	
		// only to be sure there is a ever ending loop
		i++;
		if (i > MAX_STORED_PAGES)	
			return 0xAA;

	}

	return *used;
}


/*******************************************************************************

 $Function:    	sms_cb_count_messages

 $Description:	This function counts messages in the fifo.
 				! Multipage are counted as one page !
           
 $Returns:		

 $Arguments:	
 
*******************************************************************************/
UBYTE sms_cb_count_messages ()
{
	UBYTE i=0, count=0, used, page, pages, mid, start_flag = 1; //, temp_used;

	char buf[70];


	for (i=0; i<MAX_STORED_PAGES ; i++)	
	{
		smscb_data.cb_attributes[i].start_page = 0; 
	}
	i=0;


	if (smscb_data.used_pointer EQ 0xFF)
	{
		//there are no cb messages in the fifo
		return count;
	}
		
	used =	smscb_data.used_pointer;

	while ( i < MAX_STORED_PAGES)
	{

		//single or multipage ?
		if(smscb_data.cb_attributes[used].pages EQ 0x01)
		{
			//Singlepage

			start_flag = 1;

			sprintf(buf, "11 : used%d, count%d", used,count); 
			TRACE_EVENT(buf);


			//indicate the first page, even for singlepage
			smscb_data.cb_attributes[used].start_page = 0x55;
			
			sprintf(buf, "1 set Flag at %d", used); 
			TRACE_EVENT(buf);

			//check, is it the last message in the link list
			//if yes, not necessary to go through the link list
			if (smscb_data.cb_attributes[used].next_link NEQ 0xFF)
			{
				count ++;

				// going through the link list
				used = smscb_data.cb_attributes[used].next_link;		
			}

			//is it the last message in the fifo ?
			if (smscb_data.cb_attributes[used].next_link EQ 0xFF)
			{

				sprintf(buf, "1 : used%d,count%d", used,count); 
				TRACE_EVENT(buf);


				//indicate the first page, even for singlepage
				smscb_data.cb_attributes[used].start_page = 0x55;
					
				sprintf(buf, "2 set Flag at %d", used); 
				TRACE_EVENT(buf);

				count ++;	
				return count;
			}

			
		}
		else //single or multipage ?
		{

			//Multipage
			if (start_flag)
			{
				//indicate the first page, even for singlepage
				smscb_data.cb_attributes[used].start_page = 0x55;
				sprintf(buf, "3 set Flag at %d", used); 
				TRACE_EVENT(buf);
			
			}
				
			
			page = 	smscb_data.cb_attributes[used].page;
			pages = smscb_data.cb_attributes[used].pages;
			mid = smscb_data.cb_attributes[used].mid;

			//check, is it the last message in the link list
			//if yes, not necessary to go through the link list
			if (smscb_data.cb_attributes[used].next_link NEQ 0xFF)
			{
				// going through the link list
				used = smscb_data.cb_attributes[used].next_link;		
			}

			
			//is it the last message in the fifo ?
			if (smscb_data.cb_attributes[used].next_link EQ 0xFF)
			{

				//The last page, does it still continue or does it complete 
				//the multipage ?
				if( smscb_data.cb_attributes[used].pages EQ pages   &&  
				   smscb_data.cb_attributes[used].page EQ (page+1) &&
				   smscb_data.cb_attributes[used].page EQ smscb_data.cb_attributes[used].pages)
				{
					//this one was the last page of the multipage and 
					//the last one in the fifo
					sprintf(buf, "5 : used%d,count%d", used,count); 
					TRACE_EVENT(buf);

					start_flag = 0;

					count ++;	
					return count;
				}
				else
				{
					//the multipage is not complete yet

					if (smscb_data.cb_attributes[used].mid EQ mid)
					{

						start_flag = 0;
								
						sprintf(buf, "6 : used%d,count%d", used,count); 
						TRACE_EVENT(buf);

						count ++;
						
						return count;
					}
					else
					{

						//this is a different page , it means the multipage before
						//is uncomplete !
						
						start_flag = 0;


						//indicate the first page, even for singlepage
						smscb_data.cb_attributes[used].start_page = 0x55;
							
						sprintf(buf, "4 set Flag at %d", used); 
						TRACE_EVENT(buf);


						//the page before is a uncomplete multipage
						//and the last message is a single page
						// ->count twice 
						count+=2;

						sprintf(buf, "7 : used%d,count%d", used,count); 
						TRACE_EVENT(buf);
						return count;

					}
				}
		
			}
			else
			{
				if( smscb_data.cb_attributes[used].pages EQ pages &&  
				   smscb_data.cb_attributes[used].page EQ (page+1))
				{
					//still continue

					sprintf(buf, "3 : used%d,count%d", used,count); 
					TRACE_EVENT(buf);

					start_flag = 0;
					
				}
				else
				{
					//there is a new page starting

					
					start_flag = 1;

					count ++;	

					sprintf(buf, "4 : used%d,count%d", used,count); 
					TRACE_EVENT(buf);
				}
			}
			
		}


		// only to be sure there is a ever ending loop
		i++;
	}

}

/*******************************************************************************

 $Function:    	sms_cb_count_unread_messages

 $Description:	This function counts the unread messages in the fifo
           
 $Returns:		

 $Arguments:	
 
*******************************************************************************/
UBYTE sms_cb_count_unread_messages (void)
{
	UBYTE i=0, m=0, used;

#ifdef DEBUG_SMS_CB
	char buf[40];
#endif

	used = smscb_data.used_pointer;
	
	while (smscb_data.cb_attributes[used].next_link NEQ 0xFF)
	{
		// count only the unread messages
		if (smscb_data.cb_attributes[used].status EQ UNREAD_MESSAGE)
			m ++;
		
		// going through the link list
		used = smscb_data.cb_attributes[used].next_link;
	
		// only to be sure there is a ever ending loop
		i++;
		if (i > MAX_STORED_PAGES)	
			return 0xAA;

	}


	// have a check of the last one
	if (smscb_data.cb_attributes[used].status EQ UNREAD_MESSAGE)
		m ++;


#ifdef DEBUG_SMS_CB
	sprintf(buf, "count unread %d ", m); 
	TRACE_EVENT(buf);
#endif

	return m;
}


/*******************************************************************************

 $Function:    	sms_cb_give_position_of_msg

 $Description:	You give the position (order of the msg from oldest to the latest)
 				and he gives you the real positon of the link list
           
 $Returns:		

 $Arguments:	used : current used_pointer
 
*******************************************************************************/
UBYTE sms_cb_give_position_of_msg (UBYTE used, UBYTE positon)
{
	UBYTE i=0, m=0;
	
	while ( (smscb_data.cb_attributes[used].next_link NEQ 0xFF)  &&
		   (i NEQ positon) )
	{

		// going through the link list
		used= smscb_data.cb_attributes[used].next_link;

		// only to be sure there is a ever ending loop
		if (smscb_data.cb_attributes[used].start_page EQ 0x55)
		{
			i++;
		}


		
		if (m > MAX_STORED_PAGES)	
			return 0xAA;

		m++;

	}

	//return the positon in the link list 
	return  used;
}

/*******************************************************************************

 $Function:    	sms_cb_select_read

 $Description:	called by pressing the read item in SMS-Broadcast
 
 $Returns:		

 $Arguments:	


 *******************************************************************************/
static int sms_cb_select_read (MfwMnu* m, MfwMnuItem* i)
{

	sms_cb_read_msg (ALL_MESSAGE);
    return 1;
}

/*******************************************************************************

 $Function:    	sms_cb_read_msg

 $Description:	This function prepare to read the cb messages
 				It could be call from everywhere.
 
 $Returns:		

 $Arguments:	


 *******************************************************************************/
int sms_cb_read_msg (T_SMSCB_STATUS status)
{
	UBYTE result, i;

	char buf[40];

    T_MFW_HND win = mfwParent(mfw_header());

	TRACE_FUNCTION("sms_cb_read_msg");



	//count the number of the stored messages in the fifo

	result = sms_cb_count_messages ();

	if (result EQ 0x00)
	{
		TRACE_EVENT("sms cb ERROR, there are 0 cb messages");
	}


	if((smscb_data.used_pointer EQ 0xFF) || (result EQ 0x00))
	{
		TRACE_EVENT("There is no cb message stored");

		info_screen(0, TxtNull,TxtEmptyList , NULL,REMIND_NORECORD); 
	}
	else
	{


		sprintf(buf, "Total msg%d ", result); 
		TRACE_EVENT(buf);			

		if (result EQ 0xAA)
		{
			TRACE_EVENT("sms cb ERROR, couldnt find the termination");
			return;
		}


		//keep this value for the callback function from menu-list
		smscb_data.total_number = result;

		//set the reason of reading
		//read "all_message" , "unread_message", ..........
		smscb_data.current_status = status;

		//fill up the menu list attributes
		smscb_data.cb_list_attributes = sms_cb_create_list (smscb_data.total_number);


		//show the menu list on the screen now
	    listDisplayListMenu (win, smscb_data.cb_list_attributes ,(ListCbFunc)sms_cb_list_cb,0);

	}
	
    return 1;
}


/*******************************************************************************

 $Function:    	sms_cb_create_list

 $Description:	Prepare the items for the menu list
    	     
 $Returns:		

 $Arguments:	
 
*******************************************************************************/
static ListMenuData * sms_cb_create_list (UBYTE number)
{
  UBYTE i = 0, used, next_used, page, pages, number_multipg =1, control_bit=0;


  ListMenuData *list_menu_data = (ListMenuData *)ALLOC_MEMORY (sizeof(ListMenuData));

  if (list_menu_data EQ 0)
  {
	TRACE_EVENT ("failed to create the list for cb");
    return NULL;
  }


  list_menu_data->List = (T_MFW_MNU_ITEM *) ALLOC_MEMORY (number * sizeof (T_MFW_MNU_ITEM));


  if (list_menu_data->List EQ 0)
  {
	TRACE_EVENT ("failed to create the list for cb");
    return NULL;
  }

	TRACE_EVENT ("sms_cb_create_list ()");  


	//take the current position in the fifo
	used = smscb_data.used_pointer;


  /*
   * Fill Menu List
   */


  	//until to reach the end of the fifo
	while ( smscb_data.cb_attributes[used].next_link NEQ 0xFF )
	{

		if (smscb_data.cb_attributes[used].start_page EQ 0x55)
		{
		    list_menu_data->List[i].icon = 0;
		    list_menu_data->List[i].exta = 0;
		    list_menu_data->List[i].edit = 0;
		    list_menu_data->List[i].menu = 0;
		    list_menu_data->List[i].func = 0;

		   	next_used = used;

			// going through the link list 
			next_used = smscb_data.cb_attributes[next_used].next_link;



			//before  we put the item for a multipage, we have to
			//know how long it is.
			//This is counting the number of pages belongs to a multipage
			//Every multipage &singlepage starts with the indication 0x55
			while(smscb_data.cb_attributes[next_used].start_page NEQ 0x55)
			{
				//sprintf(buf, "number_multipg %d in while", number_multipg); 
				//TRACE_EVENT(buf);			
				
				number_multipg++;

				//go out when we reach the end of the link list
				if (smscb_data.cb_attributes[next_used].next_link EQ 0xFF)
					break;

				// going through the link list & take the next cb message
				next_used = smscb_data.cb_attributes[next_used].next_link;
			}



			//fill up the header 
			//"MID" + "available Page" + "Total page"
			//the user release the uncomplete page about the available page.
	    	sprintf(smscb_data.cb_attributes[used].header,"MID%d,%d/%d", smscb_data.cb_attributes[used].mid, 
   								number_multipg, smscb_data.cb_attributes[used].pages);
		
	    	list_menu_data->List[i].str =(char *) smscb_data.cb_attributes[used].header;

			//set item to visible
			if (smscb_data.current_status EQ ALL_MESSAGE)
			{
				//show all the cb messages
		        list_menu_data->List[i].flagFunc = item_flag_none;
			}
			else if (smscb_data.current_status EQ UNREAD_MESSAGE)
			{
				if (smscb_data.cb_attributes[used].status EQ smscb_data.current_status)
					list_menu_data->List[i].flagFunc = item_flag_none;
				else
					list_menu_data->List[i].flagFunc = item_flag_hide;				
			}

			i++;
			number_multipg = 1;

			
		}

		// going through the link list & take the next cb message
		used = smscb_data.cb_attributes[used].next_link;

		// only to be sure there is a ever ending loop
		if (i > MAX_STORED_PAGES)	
			return NULL;
	}


		number_multipg = 1;

		//fill up the last message of the fifo !!
		if (smscb_data.cb_attributes[used].start_page EQ 0x55)
		{
			list_menu_data->List[i].icon = 0;
		    list_menu_data->List[i].exta = 0;
		    list_menu_data->List[i].edit = 0;
		    list_menu_data->List[i].menu = 0;
		    list_menu_data->List[i].func = 0;

			//fill up the header 
			//"MID" + "available Page" + "Total page"
			//the user release the uncomplete page about the available page.
	    	sprintf(smscb_data.cb_attributes[used].header,"MID%d,%d/%d", smscb_data.cb_attributes[used].mid, 
   								number_multipg, smscb_data.cb_attributes[used].pages);

		
	    	list_menu_data->List[i].str =(char *) smscb_data.cb_attributes[used].header;

			//set item to visible
			if (smscb_data.current_status EQ ALL_MESSAGE)
			{
				//show all the cb messages
		        list_menu_data->List[i].flagFunc = item_flag_none;
			}
			else if (smscb_data.current_status EQ UNREAD_MESSAGE)
			{
				if (smscb_data.cb_attributes[used].status EQ smscb_data.current_status)
					list_menu_data->List[i].flagFunc = item_flag_none;
				else
					list_menu_data->List[i].flagFunc = item_flag_hide;				
			}
		}





  /*
   * Fill common parameter for list handling
   */


  list_menu_data->ListLength     = number;
  list_menu_data->ListPosition     = 1;
  list_menu_data->CursorPosition  = 1;
  list_menu_data->SnapshotSize   = number;
  list_menu_data->Font           = 0;
  list_menu_data->LeftSoftKey     = TxtSoftSelect;
  list_menu_data->RightSoftKey    = TxtSoftBack;
  list_menu_data->KeyEvents      = KEY_CLEAR | KEY_RIGHT | KEY_LEFT | KEY_MNUUP | KEY_MNUDOWN;
  list_menu_data->Reason         = 0;
  list_menu_data->Strings        = TRUE;
  list_menu_data->Attr           = &sms_cb_menuAttrib;
  list_menu_data->autoDestroy    = TRUE; 

  return list_menu_data;
}

/*******************************************************************************

 $Function:    	sms_cb_list_cb

 $Description:	callback function of the menu list 
    	     
 $Returns:		

 $Arguments:	
 
*******************************************************************************/

static void sms_cb_list_cb(T_MFW_HND win, ListMenuData * ListData)
{
	UBYTE number, selected_postion;	


	TRACE_FUNCTION ("sms_cb_list_cb()");

    switch (ListData->Reason)
    {
		case LISTS_REASON_SELECT: /*  */
		case LISTS_REASON_OK: /*  */
			
			//
			selected_postion = sms_cb_give_position_of_msg (smscb_data.used_pointer, ListData->ListPosition);

			
			//display the selected message on the scree		
			sms_cb_show_cb_message (selected_postion);
			

			//destroy the list menu 
			listsDestroy(ListData->win);

			//counted the total number of cb msg before created the menu list
			number = smscb_data.total_number;

			
			FREE_MEMORY ((UBYTE *)smscb_data.cb_list_attributes->List, number * sizeof (T_MFW_MNU_ITEM));
			FREE_MEMORY ((UBYTE *)smscb_data.cb_list_attributes, sizeof (ListMenuData));
	    break;

		case LISTS_REASON_BACK: /* back to previous menu */

			TRACE_EVENT("pressed Back-key");

			//destroy the list menu 
			listsDestroy(ListData->win);

			//counted the total number of cb msg before created the menu list
			number = smscb_data.total_number;

			FREE_MEMORY ((UBYTE *)smscb_data.cb_list_attributes->List, number * sizeof (T_MFW_MNU_ITEM));
			FREE_MEMORY ((UBYTE *)smscb_data.cb_list_attributes, sizeof (ListMenuData));
	    break;
		default:
	    return;
    }



}


/*******************************************************************************

 $Function:    	sms_cb_list_cb

 $Description:	open a editor to show the cb message
    	     
 $Returns:		

 $Arguments:	
 
*******************************************************************************/
void sms_cb_show_cb_message(UBYTE link_list_position)
{

    T_MFW_HND win = mfwParent(mfw_header());

	T_EDITOR_DATA editor_data;

	UBYTE number_multipg =1, temp, next_used; 

	//char buf[60];
		
   	next_used = link_list_position;

	//check : is it the end of the link list
	//if yes, not necassary to go through the link list
	if (smscb_data.cb_attributes[next_used].next_link NEQ 0xFF)
	{
		// going through the link list 
		next_used = smscb_data.cb_attributes[next_used].next_link;
	}

	//if the multipage contain more than one page, find out
	//how many pages !
	//This is counting the number of pages belongs to a multipage
	//Every multipage & singlepge starts with the indication 0x55
	while(smscb_data.cb_attributes[next_used].start_page NEQ 0x55)
	{
		//sprintf(buf, "number_multipg %d", number_multipg); 
		//TRACE_EVENT(buf);			

		number_multipg++;

		if (smscb_data.cb_attributes[next_used].next_link EQ 0xFF)
			break;

		// going through the link list & take the next cb message
		next_used = smscb_data.cb_attributes[next_used].next_link;
	}

	//sprintf(buf, "number_multipg %d after..", number_multipg); 
	//TRACE_EVENT(buf);			

	//keep the value for later
	smscb_data.multipage_number = number_multipg;

	sms_cb_loadEditDefault(&editor_data);

	//buffer for the multipage
	smscb_data.multipage = (UBYTE *) ALLOC_MEMORY (OFFSET(smscb_data.multipage_number) * sizeof (UBYTE));

   	next_used = link_list_position;

	//fill up the new buffer with the first page
	strcat((char *)smscb_data.multipage, &smscb_data.cb_buffer[OFFSET(next_used)]);

	//check : is it the end of the link list
	//if yes, not necassary to go through the link list
	if (smscb_data.cb_attributes[next_used].next_link NEQ 0xFF)
	{
		// going through the link list 
		next_used = smscb_data.cb_attributes[next_used].next_link;
	}

	temp = smscb_data.multipage_number;

	//if the multipage contain more than one page, fill up the data
	//to the new buffer
	while(temp > 1)
	{
		strcat((char *)smscb_data.multipage, &smscb_data.cb_buffer[OFFSET(next_used)]);

		// going through the link list & take the next cb message
		next_used = smscb_data.cb_attributes[next_used].next_link;

		temp --;
	}

	//set the mark
	smscb_data.cb_attributes[link_list_position].status = READ_MESSAGE;

	editor_data.editor_attr.text =(char *) smscb_data.multipage;

	editor_data.Identifier            = link_list_position ; //keep the value (selected cb message)
	editor_data.editor_attr.size     = OFFSET(smscb_data.multipage_number); 
	editor_data.min_enter		 = 0; 
	editor_data.LeftSoftKey          = TxtDelete;
	editor_data.AlternateLeftSoftKey  = TxtSoftOptions;
	editor_data.RightSoftKey         = TxtSoftBack;
	editor_data.Callback            = (T_EDIT_CB)sms_cb_editor_cb;
	editor_data.mode           	 = READ_ONLY_MODE;

	editor_start(win, &editor_data);  /* start the editor */


}

/*******************************************************************************

 $Function:    	sms_cb_loadEditDefault

 $Description:	Configure T_EDITOR_DATA structure with default editor settings.

 $Returns:		none
 
 $Arguments:	editor_data - editor data.

*******************************************************************************/

void sms_cb_loadEditDefault (T_EDITOR_DATA *editor_data)
{
	    TRACE_FUNCTION ("sms_cb_loadEditDefault()");

		editor_attr_init(&editor_data->editor_attr, NULL, edtCurNone, 0, 0, 0);
		editor_data->hide				= FALSE;
		editor_data->min_enter			= 1;  
		editor_data->LeftSoftKey		= TxtSoftSelect;
		editor_data->RightSoftKey		= TxtSoftBack;
		editor_data->AlternateLeftSoftKey   = TxtNull;
		editor_data->TextId				= '\0';  
		editor_data->Identifier			=  0; 
		editor_data->TextString			= NULL;  
		editor_data->timeout			= FOREVER; 
		editor_data->mode                = ALPHA_MODE;
		editor_data->Callback			= NULL;
		editor_data->destroyEditor		= TRUE ; 
}

/*******************************************************************************

 $Function:    	sms_cb_editor_cb

 $Description:	callback function of the editor 

 $Returns:		none
 
 $Arguments:	

*******************************************************************************/
static void sms_cb_editor_cb (T_MFW_HND win, USHORT Identifier, SHORT reason)
{
	UBYTE result, test, i;

//only for tracing 
#ifdef DEBUG_SMS_CB
	char buf[100];
	UBYTE w;
#endif

	TRACE_FUNCTION ("sms_cb_editor_cb()");

	FREE_MEMORY ((UBYTE *)smscb_data.multipage, OFFSET(smscb_data.multipage_number) * sizeof (UBYTE));
	
	switch (reason)
	{
		case INFO_KCD_LEFT:


			//user selected to delete the message
			sms_cb_delete_message (smscb_data.cb_attributes[Identifier].mid);

			//update the use_pointer  after deleting
			smscb_data.used_pointer = smscb_data.result_used;

			if (smscb_data.used_pointer EQ smscb_data.free_pointer)
			{
				//no anymore messages stored
				smscb_data.used_pointer = 0xFF;
			}


			if (smscb_data.current_status EQ ALL_MESSAGE)
			{
				//go back to the cb list 
				sms_cb_read_msg (smscb_data.current_status);
			}
			else if (smscb_data.current_status EQ UNREAD_MESSAGE)
			{
//				if (sms_cb_count_unread_messages ())
//					sms_cb_read_msg (smscb_data.current_status);
		
			}


			
			break;
          
		case INFO_KCD_RIGHT:
		case INFO_KCD_HUP:


//only for tracing 
#ifdef DEBUG_SMS_CB
		sprintf(buf, ".. used_pointer %d ", smscb_data.used_pointer); 
		TRACE_EVENT(buf);
		sprintf(buf, "..free_pointer %d ", smscb_data.free_pointer); 
		TRACE_EVENT(buf);
		for (i=0; i<MAX_STORED_PAGES ; i++)	
		{
			sprintf(buf, "link_index: %d, i:%d", smscb_data.cb_attributes[i].next_link,i); 
			TRACE_EVENT(buf);
		}
	    test = smscb_data.used_pointer;
		while (smscb_data.cb_attributes[test].next_link NEQ 0xFF)
		{
			sprintf(buf, "Buffer on %s", &smscb_data.cb_buffer[OFFSET(test)]); 
			TRACE_EVENT(buf);
			test = smscb_data.cb_attributes[test].next_link;
		}
		sprintf(buf, "Buffer on %s,i:%d ", &smscb_data.cb_buffer[OFFSET(test)], i); 
		TRACE_EVENT(buf);
#endif //DEBUG_SMS_CB



			if (smscb_data.current_status EQ ALL_MESSAGE)
			{
				//go back to the cb list 
				sms_cb_read_msg (smscb_data.current_status);
			}
			else if (smscb_data.current_status EQ UNREAD_MESSAGE)
			{
//				if (sms_cb_count_unread_messages ())
//					sms_cb_read_msg (smscb_data.current_status);
		
			}
			break;

		default:
			TRACE_EVENT("Err: Default");
			break;
	}
}

// --------------------------------------------------------------------
// Function		:  SetLangAll
// Description	: 
// Input		: 
// Output		: 
// Notes		: 
// Last Modified: 
// --------------------------------------------------------------------
 int      SetLangAll(MfwMnu* m, MfwMnuItem* i)
{

 FFS_flashData.CBLangIdAll=0xFFFF;
 FFS_flashData.CBLanguage=AllLanguage;
 CBSetting();
    mnuUpdate(m);    //xsf add 2002.08.16
}//SetLangAll

// --------------------------------------------------------------------
// Function		:  SetLangNone
// Description	: 
// Input		: 
// Output		: 
// Notes		: 
// Last Modified: 
// --------------------------------------------------------------------
int      SetLangNone(MfwMnu* m, MfwMnuItem* i)
{
 FFS_flashData.CBLangIdAll=0;
 FFS_flashData.CBLanguage=NoLanguage;
 CBSetting();
    mnuUpdate(m);    //xsf add 2002.08.16
}//SetLangNone

// --------------------------------------------------------------------
// Function		:  SetLangGer
// Description	: 
// Input		: 
// Output		: 
// Notes		: 
// Last Modified: 
// --------------------------------------------------------------------

int      SetLangGer(MfwMnu* m, MfwMnuItem* i)
{
 if(FFS_flashData.CBLanguage!=ListLanguage)//Last time what we select is not ListLanguage
 {
  FFS_flashData.CBLangIdAll=0;
  FFS_flashData.CBLanguage=ListLanguage;
 }
 
 FFS_flashData.CBLangIdAll |=0x0001<<0;//According to GSM03.38
 CBSetting();
    mnuUpdate(m);    //xsf add 2002.08.16
}//SetLangGer

// --------------------------------------------------------------------
// Function		:  SetLangEng
// Description	: 
// Input		: 
// Output		: 
// Notes		: 
// Last Modified: 
// --------------------------------------------------------------------

 int      SetLangEng(MfwMnu* m, MfwMnuItem* i)
{
 if(FFS_flashData.CBLanguage!=ListLanguage)//Last time what we select is not ListLanguage
 {
  FFS_flashData.CBLangIdAll=0;
  FFS_flashData.CBLanguage=ListLanguage;
 }
 
 FFS_flashData.CBLangIdAll |=0x0001<<1;//According to GSM03.38
 CBSetting();
    mnuUpdate(m);    //xsf add 2002.08.16
}//SetLangEng

// --------------------------------------------------------------------
// Function		:  SetLangItal
// Description	: 
// Input		: 
// Output		: 
// Notes		: 
// Last Modified: 
// --------------------------------------------------------------------

int      SetLangItal(MfwMnu* m, MfwMnuItem* i)
{
 if(FFS_flashData.CBLanguage!=ListLanguage)//Last time what we select is not ListLanguage
 {
  FFS_flashData.CBLangIdAll=0;
  FFS_flashData.CBLanguage=ListLanguage;
 }
 
 FFS_flashData.CBLangIdAll |=0x0001<<2;//According to GSM03.38
 CBSetting();
    mnuUpdate(m);    //xsf add 2002.08.16
}//SetLangItal

// --------------------------------------------------------------------
// Function		:  SetLangFren
// Description	: 
// Input		: 
// Output		: 
// Notes		: 
// Last Modified: 
// --------------------------------------------------------------------

 int      SetLangFren(MfwMnu* m, MfwMnuItem* i)
{
 if(FFS_flashData.CBLanguage!=ListLanguage)//Last time what we select is not ListLanguage
 {
  FFS_flashData.CBLangIdAll=0;
  FFS_flashData.CBLanguage=ListLanguage;
 }
 
 FFS_flashData.CBLangIdAll |=0x0001<<3;//According to GSM03.38
 CBSetting();
    mnuUpdate(m);    //xsf add 2002.08.16
}//SetLangFren

// --------------------------------------------------------------------
// Function		:  SetLangSpan
// Description	: 
// Input		: 
// Output		: 
// Notes		: 
// Last Modified: 
// --------------------------------------------------------------------

 int     SetLangSpan(MfwMnu* m, MfwMnuItem* i)
{

 if(FFS_flashData.CBLanguage!=ListLanguage)//Last time what we select is not ListLanguage
 {
  FFS_flashData.CBLangIdAll=0;
  FFS_flashData.CBLanguage=ListLanguage;
 }
 
 FFS_flashData.CBLangIdAll |=0x0001<<4;//According to GSM03.38
 CBSetting();
    mnuUpdate(m);    //xsf add 2002.08.16
}//SetLangSpan

// --------------------------------------------------------------------
// Function		:   SetLangDutch
// Description	: 
// Input		: 
// Output		: 
// Notes		: 
// Last Modified: 
// --------------------------------------------------------------------

 int       SetLangDutch(MfwMnu* m, MfwMnuItem* i)
{
 if(FFS_flashData.CBLanguage!=ListLanguage)//Last time what we select is not ListLanguage
 {
  FFS_flashData.CBLangIdAll=0;
  FFS_flashData.CBLanguage=ListLanguage;
 }
 
 FFS_flashData.CBLangIdAll |=0x0001<<5;//According to GSM03.38
 CBSetting();
    mnuUpdate(m);    //xsf add 2002.08.16
}// SetLangDutch


// --------------------------------------------------------------------
// Function		:   SetLangSwed
// Description	: 
// Input		: 
// Output		: 
// Notes		: 
// Last Modified: 
// --------------------------------------------------------------------

 int       SetLangSwed(MfwMnu* m, MfwMnuItem* i)
{
 if(FFS_flashData.CBLanguage!=ListLanguage)//Last time what we select is not ListLanguage
 {
  FFS_flashData.CBLangIdAll=0;
  FFS_flashData.CBLanguage=ListLanguage;
 }
 
 FFS_flashData.CBLangIdAll |=0x0001<<6;//According to GSM03.38
 CBSetting();
    mnuUpdate(m);    //xsf add 2002.08.16
}// SetLangSwed


// --------------------------------------------------------------------
// Function		:   SetLangDanish
// Description	: 
// Input		: 
// Output		: 
// Notes		: 
// Last Modified: 
// --------------------------------------------------------------------

 int       SetLangDanish(MfwMnu* m, MfwMnuItem* i)
{
 if(FFS_flashData.CBLanguage!=ListLanguage)//Last time what we select is not ListLanguage
 {
  FFS_flashData.CBLangIdAll=0;
  FFS_flashData.CBLanguage=ListLanguage;
 }
 
 FFS_flashData.CBLangIdAll |=0x0001<<7;//According to GSM03.38
 CBSetting();
    mnuUpdate(m);    //xsf add 2002.08.16
}// SetLangDanish

// --------------------------------------------------------------------
// Function		:    SetLangPort
// Description	: 
// Input		: 
// Output		: 
// Notes		: 
// Last Modified: 
// --------------------------------------------------------------------

 int        SetLangPort(MfwMnu* m, MfwMnuItem* i)
{

 if(FFS_flashData.CBLanguage!=ListLanguage)//Last time what we select is not ListLanguage
 {
  FFS_flashData.CBLangIdAll=0;
  FFS_flashData.CBLanguage=ListLanguage;
 }
 
 FFS_flashData.CBLangIdAll |=0x0001<<8;//According to GSM03.38
 CBSetting();
    mnuUpdate(m);    //xsf add 2002.08.16
}//  SetLangPort


// --------------------------------------------------------------------
// Function		:     SetLangFinn
// Description	: 
// Input		: 
// Output		: 
// Notes		: 
// Last Modified: 
// --------------------------------------------------------------------

 int         SetLangFinn(MfwMnu* m, MfwMnuItem* i)
{

 if(FFS_flashData.CBLanguage!=ListLanguage)//Last time what we select is not ListLanguage
 {
  FFS_flashData.CBLangIdAll=0;
  FFS_flashData.CBLanguage=ListLanguage;
 }
 
 FFS_flashData.CBLangIdAll |=0x0001<<9;//According to GSM03.38
 CBSetting();
    mnuUpdate(m);    //xsf add 2002.08.16
}//  SetLangPort

// --------------------------------------------------------------------
// Function		:     SetLangNorw
// Description	: 
// Input		: 
// Output		: 
// Notes		: 
// Last Modified: 
// --------------------------------------------------------------------

 int SetLangNorw(MfwMnu* m, MfwMnuItem* i)
{
 if(FFS_flashData.CBLanguage!=ListLanguage)//Last time what we select is not ListLanguage
 {
  FFS_flashData.CBLangIdAll=0;
  FFS_flashData.CBLanguage=ListLanguage;
 }
 
 FFS_flashData.CBLangIdAll |=0x0001<<10;//According to GSM03.38
 CBSetting();
    mnuUpdate(m);    //xsf add 2002.08.16
}//  SetLangNorw

// --------------------------------------------------------------------
// Function		:     SetLangGreek
// Description	: 
// Input		: 
// Output		: 
// Notes		: 
// Last Modified: 
// --------------------------------------------------------------------

 int         SetLangGreek(MfwMnu* m, MfwMnuItem* i)
{
 if(FFS_flashData.CBLanguage!=ListLanguage)//Last time what we select is not ListLanguage
 {
  FFS_flashData.CBLangIdAll=0;
  FFS_flashData.CBLanguage=ListLanguage;
 }
 
 FFS_flashData.CBLangIdAll |=0x0001<<11;//According to GSM03.38
 CBSetting();
    mnuUpdate(m);    //xsf add 2002.08.16
}//  SetLangGreek

// --------------------------------------------------------------------
// Function		:     SetLangTurk
// Description	: 
// Input		: 
// Output		: 
// Notes		: 
// Last Modified: 
// --------------------------------------------------------------------

 int         SetLangTurk(MfwMnu* m, MfwMnuItem* i)
{

 if(FFS_flashData.CBLanguage!=ListLanguage)//Last time what we select is not ListLanguage
 {
  FFS_flashData.CBLangIdAll=0;
  FFS_flashData.CBLanguage=ListLanguage;
 }
 
 FFS_flashData.CBLangIdAll |=0x0001<<12;//According to GSM03.38
 CBSetting();
    mnuUpdate(m);    //xsf add 2002.08.16
}//  SetLangTurk

// --------------------------------------------------------------------
// Function		:      SetLangHung
// Description	: 
// Input		: 
// Output		: 
// Notes		: 
// Last Modified: 
// --------------------------------------------------------------------

 int SetLangHung(MfwMnu* m, MfwMnuItem* i)
{
 if(FFS_flashData.CBLanguage!=ListLanguage)//Last time what we select is not ListLanguage
 {
  FFS_flashData.CBLangIdAll=0;
  FFS_flashData.CBLanguage=ListLanguage;
 }
 
 FFS_flashData.CBLangIdAll |=0x0001<<13;//According to GSM03.38
 CBSetting();
    mnuUpdate(m);    //xsf add 2002.08.16
}//   SetLangHung

// --------------------------------------------------------------------
// Function		:      SetLangPolish
// Description	: 
// Input		: 
// Output		: 
// Notes		: 
// Last Modified: 
// --------------------------------------------------------------------

 int SetLangPolish(MfwMnu* m, MfwMnuItem* i)
{
 if(FFS_flashData.CBLanguage!=ListLanguage)//Last time what we select is not ListLanguage
 {
  FFS_flashData.CBLangIdAll=0;
  FFS_flashData.CBLanguage=ListLanguage;
 }
 
 FFS_flashData.CBLangIdAll |=0x0001<<14;//According to GSM03.38
 CBSetting();
 mnuUpdate(m);    //xsf add 2002.08.16

}//   SetLangPolish

// --------------------------------------------------------------------
// Function		:  CBSetting
// Description	: 
// Input		: 
// Output		: 
// Notes		: 
// Last Modified: 
// --------------------------------------------------------------------
static int CBSetting(void)
{
    UBYTE i=0;
    T_MFW_SMS_CB_INFO   CBInfo;
 
   ClearCBDcs();
	for (i = 0; i < MAXCB; i++)
	{
	    FFS_flashData.msg_id[i] = NO_MID; /* Set MID list default values. */
	}

  switch(FFS_flashData.CBLanguage)
  {
   case ListLanguage:
    GetCBDcs();
    break;
   case AllLanguage:
    FFS_flashData.CBdcs[0]=0;
    FFS_flashData.CBdcs[1]=0xFE;
     break;
   default:
    FFS_flashData.CBdcs[0]=0x0F;
    FFS_flashData.CBdcs[1]=0x0F;
     break;
  }
	 CBInfo.mode=CSCB_MOD_Accept;//accept
    for (i = 0; i < g_smscb_data.CBTopics.length; i++)
	{
	    FFS_flashData.msg_id[i] = g_smscb_data.CBTopics.entry[i].msg_id;
	    /* ??? update here PCM with g_smscb_data.CBTopics.entry[i].name */
	}
 flash_write();
 memcpy(CBInfo.dcs,FFS_flashData.CBdcs,MAXCB );
 memcpy(CBInfo.msg_id,FFS_flashData.msg_id,MAXCB*sizeof(USHORT) );
  sms_set_cell_broadcast_info(CSCB_MOD_Accept, &CBInfo);
 //Trace 
}


// --------------------------------------------------------------------
// Function		:  ClearCBDcs
// Description	: 
// Input		: 
// Output		: 
// Notes		: 
// Last Modified: 
// --------------------------------------------------------------------
static void  ClearCBDcs(void)
{
 memset(FFS_flashData.CBdcs,0x0F,MAXCB );
}//ClearCBDcs

// --------------------------------------------------------------------
// Function		: GetCBDcs
// Description	: 1      1........ 1        1 
//                15     14        1        0
//                      Polish.... English  German
// Input		: 
// Output		: 
// Notes		: 
// Last Modified: 
// --------------------------------------------------------------------
static void  GetCBDcs(void)
{
 UINT8  i,j=0,LangSetTime=0;


 for(i=0;i<14;i++)
 {
  if(LangSetTime==0)
  {
   if(FFS_flashData.CBLangIdAll &(1<<i))
   {
    FFS_flashData.CBdcs[j++]=i;
    LangSetTime=1;
   } //if for CBLangIdAll
  
  }//if for Langset
  else//Find the first
  {
   if(!(FFS_flashData.CBLangIdAll &(1<<i)))
   {
    FFS_flashData.CBdcs[j++]=i-1;
    LangSetTime=0;
   }//if
  }//else
 }
} 
// --------------------------------------------------------------------
// Function		:  CBInit
// Description	: 
// Input		: 
// Output		: 
// Notes		: 
// Last Modified: 
// --------------------------------------------------------------------
int CBInit(void)
{
	 T_MFW_SMS_CB_INFO   CBInfo;

 	//copy the data from the ALConfig to CBInfo for CBSetting
	 CBInfo.mode=CSCB_MOD_Accept;//accept
 	memcpy(CBInfo.msg_id,FFS_flashData.msg_id,MAXCB *sizeof(USHORT));
 	memcpy(CBInfo.dcs,FFS_flashData.CBdcs,MAXCB );
 	sms_set_cell_broadcast_info(CSCB_MOD_Accept, &CBInfo);

}
// --------------------------------------------------------------------
// Function		: Get_CB_Info
// Description	: 
// Input		: 
// Output		: 
// Notes		: 
// Last Modified: 
// --------------------------------------------------------------------
int Get_CB_Info(void)
{
    	UBYTE i,d=0;
	char buf[100];

    	//CBInit();
    	//get the Topic list from the SIM
    	//sms_get_cell_broadcast_info(&g_smscb_data.SmsCbInfo);    
	//FlashDataInitialise();  //added by windy according hxl's advice
       //flash_write(); //added by windy according hxl's advice
	g_smscb_data.SmsCbInfo.mode=CSCB_MOD_Accept;//accept
 	memcpy(g_smscb_data.SmsCbInfo.msg_id,FFS_flashData.msg_id,MAXCB *sizeof(USHORT));
	/*for (i = 0; i < MAXCB; i++)
	{
	   sprintf (buf, "g_smscb_data.SmsCbInfo.msg_id[i]:%d, i:%d", g_smscb_data.SmsCbInfo.msg_id[i],i);
	   TRACE_EVENT (buf); 
	}*/
	memcpy(g_smscb_data.SmsCbInfo.dcs,FFS_flashData.CBdcs,MAXCB );

    	g_smscb_data.CBTopics.length = 0xFF;

  	for (i = 0; i < MAXCB; i++)
	{
	    /* Update MID list from SIM data. */
	    g_smscb_data.CBTopics.entry[i].msg_id =  g_smscb_data.SmsCbInfo.msg_id[d];
	    
	    g_smscb_data.CBTopics.entry[i].dcs =  g_smscb_data.SmsCbInfo.dcs[d]; 

	    if (g_smscb_data.CBTopics.entry[i].msg_id EQ NO_MID)
		{
		    break;
		}
	    else
		{
		    g_smscb_data.CBTopics.length = i; 
		}

	    /* Set default tag string, until we get the saved name form PCM */
	    g_smscb_data.CBTopics.entry[i].name[0] = '\0'; 

	    /* ??? update here g_smscb_data.CBTopics.entry[i].name from PCM */
	    if (g_smscb_data.CBTopics.entry[i].name[0] EQ '\0')
		{
		    /* but if it's empty substitute it by the channel number */
		   // memcpy(g_smscb_data.CBTopics.entry[i].name,(char *) MmiRsrcGetText(TxtList),CB_TAG_LENGTH);
		    sprintf (g_smscb_data.CBTopics.entry[i].name,"Ch%d,ID:%d",i,g_smscb_data.CBTopics.entry[i].msg_id);
		}

		d+=1;
	}


	if (g_smscb_data.CBTopics.length EQ 0xFF)
	{
		//list is empty
		g_smscb_data.CBTopics.length = 0;
	}
	else
	{
		//+1 because the "0" was counted as well
		g_smscb_data.CBTopics.length +=1;	
	}
  	if(FFS_flashData.CBONOFF==1)
		sms_set_mt_ind(MT_CB_MSG);
	else
		sms_set_mt_ind(MT_CB_NO_IDX);
	}

/******************************************************/
/*                                                    */
/*Function Name: CBLanguage_check                       */
/*Purpose:      CBLanguage_check items check                      */
/*Prarmeter:   struct MfwMnuTag *m, struct MfwMnuAttrTag *ma, struct MfwMnuItemTag *mi */
/*return   :     USHORT                                 */
/*                                                    */
/*Author   :     windy                              */
/*Date     :    Aug.16th,2002                             */
/******************************************************/
USHORT CBLanguage_check (struct MfwMnuTag *m, struct MfwMnuAttrTag *ma, struct MfwMnuItemTag *mi)
{
    if(((int)mi->str>=T_CBGerman)&&((int)mi->str<=T_CBPolish))
    {
      switch(FFS_flashData.CBLanguage)
      {
	      case ListLanguage:
	          if(FFS_flashData.CBLangIdAll & (1<<((int)mi->str-T_CBGerman)))
			      return FLAGFUNC_TRUE_ICON; 
	          else
	    		  return FLAGFUNC_FALSE_ICON;
	           
	       case NoLanguage:
		    	return FLAGFUNC_FALSE_ICON;
		    
	       case AllLanguage:
			    return FLAGFUNC_FALSE_ICON;
			      
	       default: 
	       	  	break;
      }
    }
    else if((int)mi->str==T_CBLanguageAll)
    {
      	if(FFS_flashData.CBLanguage==AllLanguage)
		    return FLAGFUNC_TRUE_ICON; 
       	 else     
		    return FLAGFUNC_FALSE_ICON;
    }
    else if((int)mi->str==T_CBLanguageNone)
    {
      	if(FFS_flashData.CBLanguage==NoLanguage)
		    return FLAGFUNC_TRUE_ICON; 
       	else     
		    return FLAGFUNC_FALSE_ICON;
    }
 return;  
}
