/*******************************************************************************

					CONDAT (UK)

********************************************************************************                                                                              

 This software product is the property of Condat (UK) Ltd and may not be
 disclosed to any third party without the express permission of the owner.                                 
                                                                              
********************************************************************************

 $Project name:	Basic MMI                                                      
 $Project code:	BMI (6349)                                                           
 $Module:		Softkeys
 $File:		    MmiSoftkeys.c
 $Revision:		1.0                                                       
                                                                              
 $Author:		Condat(UK)                                                         
 $Date:		    22/02/01                                                      
                                                                               
********************************************************************************
                                                                              
 Description:
 
  

********************************************************************************

 $History:
 
	25/10/00			Original Condat(UK) BMI version.	
	   
 $End

*******************************************************************************/


/*******************************************************************************
                                                                              
                                Include Files
                                                                              
*******************************************************************************/

#define ENTITY_MFW

/* includes */
#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#if defined (NEW_FRAME)

#include "typedefs.h"
#include "vsi.h"
#include "pei.h"
#include "custom.h"
#include "gsm.h"

#else

#include "stddefs.h"
#include "custom.h"
#include "gsm.h"
#include "vsi.h"

#endif
#include "mfw_sys.h"
#include "p_sim.h"

//#include <stdio.h>
//#include <string.h>

//#include "mfw_sys.h"

//#include "stddefs.h"    // needed for mfw_mfw.h
#include "mfw_mfw.h"
#include "mfw_win.h"
#include "mfw_kbd.h"
#include "mfw_lng.h"
#include "mfw_phb.h"
#include "mfw_nm.h"
#include "dspl.h"

#include "ksd.h"
#include "psa.h"


#include "MmiMmi.h"
#include "MmiMain.h"

#include "MmiUserData.h"
#include "MmiSoftKeys.h"
#include "MmiResources.h"
#include "Mmi3dframe.h"
#include "mmiiconresource.h"
typedef struct
{
    MfwHnd leftKeyHandler;
    MfwHnd rightKeyHandler;

    LangTxt leftKeyLabel;
    LangTxt rightKeyLabel;
} SoftKeysData, *SoftKeys;


#define HorizontalOffset 0 //zy 09/10
#define VerticalOffset   1
#define LeftMargin       HorizontalOffset
#define RightMargin      (mmiScrX - HorizontalOffset)
#define TopMargin        (SCREEN_SIZE_Y)
#define FIFTH_LINE		(TopMargin-LINE_HEIGHT)

#define ALLOC_MEMORY mfwAlloc
#define FREE_MEMORY  mfwFree

static U8    softKeyFont              = 0;
static UBYTE softKeysDisplayAttibutes = DSPL_TXTATTR_NORMAL;


/*
 * Change the <font> and <displayAttibutes> for _all_ softkeys.
 * All consecutive calls of softKeysUpdate() will reflect the change.
 * Returns SOFTKEYS_CHANGED when everything went fine,
 * or SOFTKEYS_FAILURE on failure.
 */
SoftKeysResult softKeysAttributes( U8 font, UBYTE displayAttibutes)
{
    if( font == (U8)-1 )
    {
        return SOFTKEYS_FAILURE;
    }

    softKeyFont              = font;
    softKeysDisplayAttibutes = displayAttibutes;

    return SOFTKEYS_CHANGED;
}


/*
 * Creates the softkeys for <window>, which will display the
 * <leftKeyLabel> and <rightKeyLabel> in the softkey area
 * on the screen, and invoke <leftKeyCallBack> or <rightKeyCallBack>
 * when the approrpiate key is pressed. Returns SOFTKEYS_CREATED when
 * everything went fine, or SOFTKEYS_FAILURE on failure.
 */
SoftKeysResult softKeysHndCreate( MfwHnd window, SoftKeysSetup *setup)
{
	if( window == NULL || setup == NULL )
    {
        return SOFTKEYS_FAILURE;
    }

	return softKeysWinCreate( (MfwWin *)((MfwHdr *)window)->data, setup);
}


SoftKeysResult softKeysWinCreate( MfwWin *window, SoftKeysSetup *setup)
{
    SoftKeys newKeys;

    if( window == NULL || setup == NULL )
    {
        return SOFTKEYS_FAILURE;
    }

    newKeys = (SoftKeys)ALLOC_MEMORY(sizeof(SoftKeysData));

    if( newKeys == NULL )
    {
        return SOFTKEYS_FAILURE;
    }

    if( userDataWinSet( window, UD_SOFTKEYS, (void *)newKeys) == NULL )
    {
        FREE_MEMORY( (void *)newKeys, sizeof(SoftKeysData));

        return SOFTKEYS_FAILURE;
    }

    newKeys->leftKeyHandler  = kbdCreate( NULL, KEY_LEFT,
                                          setup->leftKeyCallBack);
    newKeys->rightKeyHandler = kbdCreate( NULL, KEY_RIGHT,
                                          setup->rightKeyCallBack);

    newKeys->leftKeyLabel    = setup->leftKeyLabel;
    newKeys->rightKeyLabel   = setup->rightKeyLabel;

    if( newKeys->leftKeyHandler == NULL || newKeys->rightKeyHandler == NULL )
    {
        kbdDelete(newKeys->leftKeyHandler);
        kbdDelete(newKeys->rightKeyHandler);

        userDataWinDelete( window, UD_SOFTKEYS);

        FREE_MEMORY( (void *)newKeys, sizeof(SoftKeysData));

        return SOFTKEYS_FAILURE;
    }

    return SOFTKEYS_CREATED;
}


/*
 * Change the setup of the softkeys for <window>. Use TxtNull if you
 * don't want to change a ...Label, and NULL if you don't want to change
 * a ...CallBack. Returns SOFTKEYS_CHANGED when everything went fine,
 * or SOFTKEYS_FAILURE on failure.
 */
SoftKeysResult softKeysHndSet( MfwHnd window, SoftKeysSetup *changes)
{
	if( window == NULL || changes == NULL )
    {
        return SOFTKEYS_FAILURE;
    }

	return softKeysWinSet( (MfwWin *)((MfwHdr *)window)->data, changes);
}


SoftKeysResult softKeysWinSet( MfwWin *window, SoftKeysSetup *changes)
{
    SoftKeys softKeys = (SoftKeys)userDataWinGet( window, UD_SOFTKEYS);

    if( window == NULL || softKeys == NULL || changes == NULL )
    {
        return SOFTKEYS_FAILURE;
    }

    if( changes->leftKeyCallBack != NULL )
    {
        MfwHnd NewKeyHandler = kbdCreate( NULL, KEY_LEFT,
                                          changes->leftKeyCallBack);

        if( NewKeyHandler == NULL )
        {
            return SOFTKEYS_FAILURE;
        }
        else
        {
            kbdDelete(softKeys->leftKeyHandler);

            softKeys->leftKeyHandler = NewKeyHandler;
        }
    }

    if( changes->rightKeyCallBack != NULL )
    {
        MfwHnd NewKeyHandler = kbdCreate( NULL, KEY_LEFT,
                                          changes->rightKeyCallBack);

        if( NewKeyHandler == NULL )
        {
            return SOFTKEYS_FAILURE;
        }
        else
        {
            kbdDelete(softKeys->rightKeyHandler);

            softKeys->rightKeyHandler = NewKeyHandler;
        }
    }

    if( changes->leftKeyLabel != TxtNull )
    {
        softKeys->leftKeyLabel = changes->leftKeyLabel;
    }

    if( changes->rightKeyLabel != TxtNull )
    {
        softKeys->rightKeyLabel = changes->rightKeyLabel;
    }

    return SOFTKEYS_CHANGED;
}


/*
 * You will need to call this whenever <window> is updated.
 */
void softKeysHndUpdate(MfwHnd window)
{
	if( window == NULL )
    {
        return;
    }

	softKeysWinUpdate((MfwWin *)((MfwHdr *)window)->data);
}


void softKeysWinUpdate(MfwWin *window)
{
    SoftKeys    softKeys     = (SoftKeys)userDataWinGet( window, UD_SOFTKEYS);
    U8          oldFont      = dspl_SelectFontbyID(softKeyFont);
    USHORT      textTop      = (USHORT)(TopMargin - dspl_GetFontHeight());

    if( window == NULL || softKeys == NULL )
    {
        return;
    }

    PROMPT( LeftMargin,
                  textTop, 0,
                  softKeys->leftKeyLabel);


    dspl_SelectFontbyID(oldFont);
}


/*
 * Deletes the softkeys for <window>.
 */
void softkeysHndDelete(MfwHnd window)
{
	if( window == NULL )
    {
        return;
    }

	softkeysWinDelete((MfwWin *)((MfwHdr *)window)->data);
}


void softkeysWinDelete(MfwWin *window)
{
    SoftKeys softKeys = (SoftKeys)userDataWinDelete( window, UD_SOFTKEYS);

    if( window == NULL || softKeys == NULL )
    {
        return;
    }

    kbdDelete(softKeys->leftKeyHandler);
    kbdDelete(softKeys->rightKeyHandler);

    FREE_MEMORY( (void *)softKeys, sizeof(SoftKeysData));
}

void displaySoftKeys(int leftSoftKey,int rightSoftKey)
{
    char *l, *r;
    UBYTE len;
    if((leftSoftKey == 0) && (rightSoftKey == 0))  //xsf add for no softkey 12.20
      return;
    l = (char*)MmiRsrcGetText(leftSoftKey);
    r = (char*)MmiRsrcGetText(rightSoftKey);
    len = dspl_str_length(r);
	/* 2004/01/13 sunsj modify */
    dspl_Clear(0, SCREEN_SIZE_Y-dspl_GetFontHeight()-2, RightMargin, SCREEN_SIZE_Y);//zy 09/11
    //begin of xsf add for draw menudown bmp 12.11
//    drawDownbmp( );
draw_softkeybar_background();
    //dspl_BitBlt(0, 142, menudown[FlashSettingData.theme].area.sx, menudown[FlashSettingData.theme].area.sy, 0, (void*)menudown[FlashSettingData.theme].icons, 0);
	/* 2004/02/11 sunsj modify softkey color */
    drawcolorclarity(LeftMargin+1,TopMargin-dspl_GetFontHeight()-1,l, 0x000000);
	//2004-03-29 Sunny modify
    /*drawcolorclarity(RightMargin-dspl_GetTextExtent((char*)MmiRsrcGetText(rightSoftKey),
		(USHORT)len)-1,TopMargin-dspl_GetFontHeight()-1,r, 0x000000);*/

	drawcolorclarity(RightMargin- get_StringWidth(r)-1,TopMargin-dspl_GetFontHeight()-1,r, 0x000000);//shiheng 0811

    //end of xsf add for draw menudown bmp 12.11
    
	/*dspl_TextOut(LeftMargin,TopMargin-dspl_GetFontHeight(),
                        DSPL_TXTATTR_CURRENT_MODE,l);
    dspl_TextOut(RightMargin-dspl_GetTextExtent((char*)MmiRsrcGetText(rightSoftKey),
		(USHORT)len),
				 TopMargin-dspl_GetFontHeight(),
		       DSPL_TXTATTR_CURRENT_MODE,r);*/

}

void displayColorSoftKeys(int leftSoftKey,int rightSoftKey,int color)
{
    char *l, *r;
    UBYTE len;
    if((leftSoftKey == 0) && (rightSoftKey == 0))  //xsf add for no softkey 12.20
      return;
    l = (char*)MmiRsrcGetText(leftSoftKey);
    r = (char*)MmiRsrcGetText(rightSoftKey);
    len = dspl_str_length(r);
    dspl_Clear(0, 144, RightMargin, 160);//zy 09/11
	dspl_colorTextOut(LeftMargin+1,TopMargin-dspl_GetFontHeight()-1,
                        DSPL_TXTATTR_CURRENT_MODE,l,color);
    //2004-03-29 Sunny modify
    /*drawcolorclarity(RightMargin-dspl_GetTextExtent((char*)MmiRsrcGetText(rightSoftKey),
		(USHORT)len)-1,TopMargin-dspl_GetFontHeight()-1,r, 0x000000);*/

	drawcolorclarity(RightMargin- get_StringWidth(r) -1,TopMargin-dspl_GetFontHeight()-1,r, 0x000000);

}

void displayPhbkEditSoftKeys(int leftSoftKey,int rightSoftKey)
{
    char *l, *r;
    UBYTE len;

    l = (char*)MmiRsrcGetText(leftSoftKey);
    r = (char*)MmiRsrcGetText(rightSoftKey);
    len = dspl_str_length(r);  
    drawcolorclarity(LeftMargin,TopMargin-dspl_GetFontHeight()-1,l, 0xffffff);

	//2004-03-29 Sunny modify
    /*drawcolorclarity(RightMargin-dspl_GetTextExtent((char*)MmiRsrcGetText(rightSoftKey),
		(USHORT)len),TopMargin-dspl_GetFontHeight()-1,r, 0xffffff);*/
	drawcolorclarity(RightMargin - get_StringWidth(r) - 1,TopMargin-dspl_GetFontHeight()-1,r, 0xffffff);
 
    /*
	dspl_TextOut(LeftMargin,TopMargin-dspl_GetFontHeight(),
                        DSPL_TXTATTR_CURRENT_MODE,l);
    dspl_TextOut(RightMargin-dspl_GetTextExtent((char*)MmiRsrcGetText(rightSoftKey),
		(USHORT)len),
				 TopMargin-dspl_GetFontHeight(),
		       DSPL_TXTATTR_CURRENT_MODE,r);
	*/

}

//hxl add for qq 2003-3-2
#ifdef QQFUNC//hxl add for qq func switch,2003-3-24
void display3SoftKeys(int midSoftKey)//(int leftSoftKey,int midSoftKey,int rightSoftKey)
{
    char  *m;
    UBYTE lenm;

    m = (char*)MmiRsrcGetText(midSoftKey);
    lenm = dspl_str_length(m);  

	//2004-03-29 Sunny modify
    /*drawcolorclarity((RightMargin/2)-((dspl_GetTextExtent((char*)MmiRsrcGetText(midSoftKey),
		(USHORT)lenm))/2),TopMargin-dspl_GetFontHeight()-1,m, 0xffffff);*/
	drawcolorclarity((RightMargin - get_StringWidth(m) /2)-,TopMargin-dspl_GetFontHeight()-1,m, 0xffffff);
	

}
#endif
void displaySoftKeys_edition(int leftSoftKey,int index, int rightSoftKey)
{


	dspl_TextOut(LeftMargin,
		Mmi_layout_fifth_line()-dspl_GetFontHeight(), 0,
                       (char*)MmiRsrcGetText(leftSoftKey));

	//2004-03-29 Sunny modify 
	/*dspl_TextOut( 5 + dspl_GetTextExtent((char*)MmiRsrcGetText(leftSoftKey), (USHORT)strlen((char*)MmiRsrcGetText(leftSoftKey))) ,
		Mmi_layout_fifth_line() -dspl_GetFontHeight(), 0,
                       (char*)MmiRsrcGetText(index));*/
	dspl_TextOut( 5 + get_StringWidth((char*)MmiRsrcGetText(leftSoftKey)) ,
		Mmi_layout_fifth_line() -dspl_GetFontHeight(), 0,
		(char*)MmiRsrcGetText(index));

	//2004-03-29 Sunny modify
    /*dspl_TextOut(RightMargin-dspl_GetTextExtent((char*)MmiRsrcGetText(rightSoftKey), (USHORT)strlen((char*)MmiRsrcGetText(rightSoftKey))),
				 Mmi_layout_fifth_line()-dspl_GetFontHeight(), 0,
		       (char*)MmiRsrcGetText(rightSoftKey));*/
	dspl_TextOut(RightMargin-get_StringWidth((char*)MmiRsrcGetText(rightSoftKey)),
		Mmi_layout_fifth_line()-dspl_GetFontHeight(), 0,
		(char*)MmiRsrcGetText(rightSoftKey));

}


// SH - 25/5/01
// This function provided for WAP, to allow the printing of custom softkeys
// from strings provided.

void displayCustSoftKeys(char *LeftSoftKey, char *RightSoftKey)
{
	dspl_TextOut(LeftMargin,TopMargin-dspl_GetFontHeight(), DSPL_TXTATTR_CURRENT_MODE, LeftSoftKey);

	//2004-03-29 Sunny modify
    /*dspl_TextOut(RightMargin-dspl_GetTextExtent(RightSoftKey,
		(USHORT)dspl_str_length(RightSoftKey)),
				 TopMargin-dspl_GetFontHeight(),
		       DSPL_TXTATTR_CURRENT_MODE,RightSoftKey);*/
	dspl_TextOut(RightMargin-get_StringWidth(RightSoftKey),
		TopMargin-dspl_GetFontHeight(),
		DSPL_TXTATTR_CURRENT_MODE,RightSoftKey);
	
    return;
}

/*2003/11/13,talcon add*/
void displayColorSoftKeys2(int leftSoftKey,int rightSoftKey,int color,int bkgcolor)
{
    char *l, *r;
    UBYTE len;
	UINT oldColor =0;
    if((leftSoftKey == 0) && (rightSoftKey == 0))  //xsf add for no softkey 12.20
		return;
    l = (char*)MmiRsrcGetText(leftSoftKey);
    r = (char*)MmiRsrcGetText(rightSoftKey);
    len = dspl_str_length(r);
	oldColor = dspl_SetBkgColor(bkgcolor);
	/* 2004/02/11 sunsj modify for idle softkey icon */
    //dspl_Clear(0, 144, RightMargin, 160);
	draw_softkeybar_background();
	
	dspl_colorTextOut(LeftMargin+1,TopMargin-dspl_GetFontHeight()-1,
		DSPL_TXTATTR_TRANSPARENT,l,color);

	//2004-03-29 Sunny modify 
    /*dspl_colorTextOut(RightMargin-dspl_GetTextExtent((char*)MmiRsrcGetText(rightSoftKey),
		(USHORT)len)-1,
		TopMargin-dspl_GetFontHeight()-1,
		DSPL_TXTATTR_TRANSPARENT,r,color);*/
	dspl_colorTextOut(RightMargin-get_StringWidth((char*)MmiRsrcGetText(rightSoftKey))+3,//shiheng 0811
		TopMargin-dspl_GetFontHeight()-1,
		DSPL_TXTATTR_TRANSPARENT,r,color);
	dspl_SetBkgColor(oldColor);
}

#if(MAIN_LCD_SIZE==3)
void draw_softkeybar_leftright_arrow(void)
{
	//dspl_BitBlt(54, 117, 20, 6, 0, (void * )leftright_arrow, 0);
	DRAW_ICON(ICON_LEFTRIGHT_ARROW);
}//jhxu720 add
void draw_softkeybar_updown_arrow(void)
{
	//dspl_BitBlt(60, 115, 8, 10, 0, (void * )updown_arrow, 0);
	DRAW_ICON(ICON_UPDOWN_ARROW);
}//jhxu720 add
void draw_softkeybar_fourdirection_arrow(void)
{
	//	dspl_BitBlt(54, 115, 20, 10, 0, (void * )fourdirection_arrow, 0);
    DRAW_ICON(ICON_FOURDIRECTION_ARROW);
}//jhxu720 add
#endif