/*******************************************************************************

          CONDAT (UK)

********************************************************************************

 This software product is the property of Condat (UK) Ltd and may not be
 disclosed to any third party without the express permission of the owner.

********************************************************************************

 $Project name: Basic MMI
 $Project code: BMI (6349)
 $Module:   Sounds
 $File:       MmiSounds.c
 $Revision:   1.0

 $Author:   Condat(UK)
 $Date:       25/10/00

********************************************************************************

 Description:

    This provides the implementation of the sounds for the basic MMI

    1. Playing various system sounds such as the paging ring,
       new sms message etc. 
    2. The menus used to enable or disable the system sounds
    3. The menu used to select the paging ring tone
    4. The menu used to select the volume.
    5. The menu used to select the key press tone or dtmf

  The mmi should play system sounds by calling soundExec with the appropriate
  SoundsXXXX message it is possible to go directly to the audio driver to play
  a sound but this will bypass any user settings to turn sounds on or off.

  The MelodySelect and SettingXXXX messages are used to control the key events
  during menu handling, entry to the sounds menus is through the soundsXXXX
  functions. These display the appropriate menu and handle the keypad events.

  The settings are read from PCM on startup and saved whenever a setting menu
  is selected.

********************************************************************************
 $History: MmiSounds.c

  25/10/00      Original Condat(UK) BMI version.

 $End

*******************************************************************************/
/********************************************************************************
 $History: MmiSounds.c

 04/08/02     zym   added audio funcion 
  22/07/02      zy Mobile Innovation make the sounds dialog has icon
  12/08/02    zym add 12/08/02 for control voice ampifier
  15/08/02    zy add hallonoff and speak volume
  29/08/02    zy change speak volume
 $End

*******************************************************************************/

/*******************************************************************************

                                Include Files

*******************************************************************************/

#define ENTITY_MFW

#define NEWAUDIO

/* includes */
#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#if defined (NEW_FRAME)

    #include "typedefs.h"
    #include "vsi.h"
    #include "pei.h"
    #include "custom.h"
    #include "gsm.h"

///added by dx
    #include "rv_general.h"
    #include "Audio_api.h"


#else

    #include "stddefs.h"
    #include "custom.h"
    #include "gsm.h"
    #include "vsi.h"

#endif
#include "mfw_sys.h"

#include "mfw_mfw.h"
#include "mfw_kbd.h"
#include "mfw_tim.h"
#include "mfw_lng.h"
#include "mfw_win.h"
#include "mfw_icn.h"
#include "mfw_sim.h"
#include "mfw_nm.h"
#include "mfw_edt.h"
#include "mfw_mnu.h"
#include "mfw_phb.h"
#include "mfw_sms.h"

#include "dspl.h"


#include "p_mmi.h"
#include "aci_cmh.h"
#include "mfw_mme.h"

#include "MmiMmi.h"

#include "MmiIcons.h"
#include "MmiPins.h"
#include "MmiMain.h"
#include "MmiDialogs.h"
#include "MmiIdle.h"
#include "MmiStart.h"
#include "MmiCall.h"
#include "MmiSimToolkit.h"
#include "gdi.h"
#include "Audio.h"
#include "MmiLists.h"
#include "MmiSounds.h"
#include "MmiResources.h"
#include "MmiTimeDate.h"

#include "cus_aci.h"
#include "p_sim.h"
#include "pcm.h"
#include "MmiTimers.h"
#include "pwr.h"

#include "mmiutilapi.h"

#include "typedefs.h"

#include "melody_struct_i.h"
#include "melody_api.h"
#include "melody_env.h"

//#include "spi_drv.h"	/* Robert.Chen added, 2004-06-08, for access ABB API */

extern const int MelodyTable[14];

extern T_call call_data;
extern driver_Parameter drvpara; // added 09/10/02
extern uint8 SpeakerVols;
extern BOOL call_win_flag;// added 02/12/27
BOOL IncallLedisRun=FALSE;/*add 01-23-2003 for led incomingcall*/

/*glowing, 2003-12-20, comment the following statement */
//UINT8 g_HandsetPcsycMode = 0; 
/*******************************************************************************

                                Constant Definitions

*******************************************************************************/


/* Control FLUSH mechanism to PCM
*/
#define FLUSH 1
#define NO_FLUSH 0


/* Timer constants, pretty self explanatory
*/
#define TIMHALFSECOND 300 // 500
#define TIMSECOND   1000
#define TIM4SECOND      4000

#define VIBRATOR_TIME 2000  /* two second timer on the vibrator


/* Define a generic trace mechanism
*/
//#define PTRACE(prn) { char buf[64]; ## prn ##   TRACE_FUNCTION(buf); }
#define PTRACE(prn) { char buf[200]; ## prn ##   MMI_TRACE_EVENT(buf); }



/* Define maximum counter for vibrator
*/
#define VIBRATOR_COUNTER_MAX  4
#define NOT_VIBRATING     0


/* define an invalid tone marker
*/
#define INVALID_TONE  -1


//zym for vibrator

#define SET_BIT(REG,BIT)   (*((volatile unsigned short *) (REG)) |= (1 << (BIT)))
#define RESET_BIT(REG,BIT) (*((volatile unsigned short *) (REG)) &= ~(1 << (BIT)))
#define io_conf_reg_g       (0x0fffef00a)
#define io_cntl_reg_g        (0x0fffe4804)
#define armio_latch_out_g   (0x0fffe4802)
#define armio_latch_out_g   (0x0fffe4802)


UINT8 g_pcsyncstatus = 0;
UINT32 driverFlag=0x00000000;//defined for global Flag for driver 02/12/28
static char gChMelody_Name[20];
static unsigned char gChMelody_Looping;
static unsigned char waitToPlay=0;  
static unsigned char StopMelodyAgain=1;  //add for stop playing melody again when MelodyStop_cb() called 2003.03.03
//static UINT8 gCjMeloList2Free=0;//chenjun defined to remember hoe many list are alloced, we should free,2003-01-14
T_USER_RING tUserRing;

#ifndef _M188_
/* glowing,2004-06-16, this enum defines in mfw_ffs.h in M188, 
 * but in M288, these codes has been removed in mfw_ffs.h, 
 * so we move them to here
 */

/*
 Define the types of volume we cater for
*/
typedef enum
{
    silent = (UBYTE)0,
    volLow,
    volMedium,
    volHigh,
    increasing
}VOL_ENUM;
/* glowing,2004-06-12, end of import */
#endif

/*******************************************************************************

                            Local Function prototypes

*******************************************************************************/

static int winEvent( MfwEvt e, MfwWin *w );
static int keyEvent( MfwEvt e, MfwKbd *kc );
static int keyEventLong( MfwEvt e, MfwKbd *kc );

static void ( *show )( void );

static int playCurrentSound( MfwEvt e, MfwTim *tc );
static int getCallingMenu( MfwEvt e, MfwTim *tc );
static int setSelection( MfwEvt e, MfwWin *w );

static void delayThenPlayMelody( void );
static void stopPlayingMelody( void );

static void readSettingsFromPCM( void );
static void writeSettingsToPCM( UBYTE flush );

static void soundsCreate( MfwHnd parent );
static int VibratorTimerEvent( MfwEvt e, MfwTim *t );

void SetMelodyMode_aloud(void);
void SetHandsetMode(void);
void AudioFullAcessWrite(UINT8 varid, void* ptr);
void sounds (T_MFW_HND win, USHORT event, SHORT value, void * parameter);
void sounds_dialog_cb(T_MFW_HND win, UBYTE identifier, UBYTE reason);
static void sounds_dialog(void);
static int sounds_win_cb (T_MFW_EVENT event, T_MFW_WIN * win);
static int melody_win_cb (MfwEvt e, MfwWin *w);
void melody_exec_cb (T_MFW_HND win, USHORT event, SHORT value, void * parameter);
void sounds_destroy (T_MFW_HND own_window);
T_MFW_HND sounds_create (T_MFW_HND parent_window);
void sounds_Exit (T_MFW_HND own_window);
T_MFW_HND sounds_Init (T_MFW_HND parent_window);
void MelodyList_cb(T_MFW_HND * Parent, ListMenuData * ListData);
static void melody_destroy(MfwHnd own_window);
void SetMVol_DownLinkPGA(UINT16 Amp);
void SetASpk_DownLinkPGA(UINT16 Amp);
void getVolSetting(UINT8 * amp);	/* Robert.Chen added, 2004-06-08 */

void SRReco_cb (void* parameter);
void SREnroll_cb (void* parameter);
void SRUpdate_cb (void* parameter);


#define SHOW_CONFIRM          3
#define ACTIVATE_CONFIRM       4
#define DEACTIVATE_CONFIRM     5


/* glowing: add it for new melody api */
UINT8 curMelodyId;  //for replacing the var currentringstr
int gPlayKeytoneFlag = 1;




/*******************************************************************************

                                Local typedefs

*******************************************************************************/

/* And the key tones
*/
typedef enum
{
    KEYTONE_NONE,
    KEYTONE_BEEP,
    KEYTONE_DTMF
} SoundsKeyTones;


/*  ------------- Variables ---------------   */

static SoundSettingState soundReason=SoundsNone;
// Change behavior of menu scroll up/down, meni item selected on scrolling.
static BOOL scrollSelectMenuItem= FALSE;
static BOOL volumeSetting2Pcm =FALSE;
static RINGER_STRUCT_BROWSER current =
{ // default settings.
    AUDIO_MEL_TUNES,  // ringer
    0,          // vibrator - off by default
    volMedium,      // volumeSetting
    2,          // keypadOn dtmf
    1,          // AlarmOn
    1,          // OrganiserAlert
    1,          // CreditLow
    1,          // SMSTone
    1,          // SMSBroadcast
    1,          // battLow
    3,          // earpiece
    1,          // language
    1         // Equalizer
};

const char UserRingIdStr[]="/meloe2/user/userid";//chenjun add for user ring index file 2003-01-15

typedef enum
{
    E_INIT = 0x0801,
    E_BACK,
    E_ABORT,
    E_EXIT,
    E_RETURN,
    E_OK
} e_M_events;

T_MELODY_INFO sounds_data;


/*******************************************************************************

                                Module local variables

*******************************************************************************/

static UBYTE oldVolumeSetting;     /* Previous setting of the volume      */
static MfwHnd win;                     /* our window              */
static MfwHnd kbd;                     /* our keyboard handler          */
static MfwHnd kbdLong;                 /* our keyboard handler for Long pressed    */
static MfwHnd hRingerStartTimer;         /* timeout till start of melody    */
static MfwHnd hRingerVolSettingTimeOut;   /* 4 second timeout for volume setting menu */

static MfwHnd hWinDispTimer;     /* timer to show result windows      */

/* Maintain vibrator state information here
*/
static MfwHnd VibratorTimer;
static UBYTE  Vibrating = NOT_VIBRATING;
static UBYTE  vibrator_counter=0;

/* General state information
*/
static MmiState nextState;
static MfwHnd lastFocus;

/* Tune details
*/
static int CurrentTune = 0;
static int LastTune = 0;

/*Globle varaible add by xsf  2002.06.26*/
UINT8    Gu8_Enviro_vol=0;//in enviroment setting adjust vol,1:in other place adjust vol
extern UINT8    Gu8_Envio_num;//enter which enviroment flag  
//extern UINT16  l_playing; 
//extern UINT16  vib_playing;  //xsf add 2002.08.12
//extern UINT16  ton_playing;
extern UINT8   melody_status;
extern UINT8   tone_status;
extern UINT8   vibra_status;
extern int     CurrentMode;  //xsf add 12.30
//UINT8 currentring=0; 
char currentringstr[20];
char oldringstr[20];   //xsf add 2002.09.06
INT8 e2_status;
//yq added for voicedial
static char  current_vd_file[20];
static char enrollname[10];
static unsigned char startenroll=0;
static unsigned char startupdate=0;
static unsigned char startdial=0;
//end yq added 
//Bob added 01/21
MfwHnd ledblink=0;
UBYTE ledblinkflag=0;
static UBYTE ledblinkison=1;
//Bob added to indicate Hall state 01/23
extern driver_Parameter drvpara;
//Bob added to indicate if incoming call or not 01/23
extern BOOL IncallLedisRun;
extern pwr_Status_Type    pwr_Status;
/* Menu definitions
*/
//static ST_MENU on_off_menu ;
//static ST_MENU dynamenu;
//static ST_MENU volume_menu;

//static ST_MENU_ITEM ONOFF_MENU_ITEMS[] = {
//  { 0, ST_PROMPT_MENU, TxtOff  },
//  { 1, ST_PROMPT_MENU, TxtOn   },
//  { 2, ST_PROMPT_MENU, TxtNull },
//};

//nm static ST_MENU_ITEM VOLUME_MENU_ITEMS[] = {
//nm  { silent,   ST_PICTURE_MENU, VOLUME_ICON_SILENT   },
//nm  { volLow,   ST_PICTURE_MENU, VOLUME_ICON_LOW    },
//nm  { volMedium,  ST_PICTURE_MENU, VOLUME_ICON_MEDIUM   },
//nm  { volHigh,    ST_PICTURE_MENU, VOLUME_ICON_HIGH   },
//nm  { increasing, ST_PICTURE_MENU, VOLUME_ICON_INCREASING }
//nm};


/* Provide a mapping between keys and their associated DTMF tones
*/
//nmstatic const keytonemap[][2] = {
//nm  { KEY_0,       TONES_DTMF_0    },
//nm  { KEY_1,       TONES_DTMF_1    },
//nm  { KEY_2,       TONES_DTMF_2    },
//nm  { KEY_3,       TONES_DTMF_3    },
//nm  { KEY_4,       TONES_DTMF_4    },
//nm  { KEY_5,       TONES_DTMF_5    },
//nm  { KEY_6,       TONES_DTMF_6    },
//nm  { KEY_7,       TONES_DTMF_7    },
//nm  { KEY_8,       TONES_DTMF_8    },
//nm  { KEY_9,       TONES_DTMF_9    },
//nm  { KEY_STAR,    TONES_DTMF_STAR },
//nm  { KEY_HASH,    TONES_DTMF_HASH },
//nm  { KEY_MNUUP,   TONES_DTMF_A    },
//nm  { KEY_MNUDOWN, TONES_DTMF_A    },
//nm  { KEY_LEFT,    TONES_DTMF_B    },
//nm  { KEY_RIGHT,   TONES_DTMF_C    },
//nm  { KEY_CLEAR,   TONES_DTMF_D    },
//nm  { 0,           INVALID_TONE    }
//nm};
//nm
static MfwMnuAttr melody_menuAttrib =
{
    0,
    &melody_menuArea,
    MNU_LEFT | MNU_LIST | MNU_CUR_LINE, /* centered page menu       */
    -1,                                 /* use default font         */
    NULL,                               /* with these items         */
    0                                   /* number of items     */
};
//yq added for voicedial
extern T_MFW_HND voice_win;
extern UBYTE recognize_true;
static void VdPlayCB (void* parameter);
static void VDEnroll_cb (void* parameter);
static void VDUpdate_cb (void* parameter);
static void VDReco_cb (void* parameter);
void PlayVmRecord_cb (void* parameter);
//end yq added 
//Bob added for led blink 01/23
void LedBlinkEvent()
{
    MmiTrace("Bob test LED blink");
    if( FlashSettingData.ledon==0 )
    {
        return;
    }
    //when ledblink is close or Hall is on or incoming call or mobile is charging, then stop blinking
    if( (drvpara.hallstatusflag == 1)||(pwr_Status.charging_state!=0)||(IncallLedisRun)||(ledblinkflag==0) )
    {
        //  if(IncallLedisRun)
        //   {
        //ledblinkflag=0;
        //    }
        timStop(ledblink);
        MmiTraceInt(ledblinkflag);
        MmiTraceInt(drvpara.hallstatusflag);
        MmiTraceInt(IncallLedisRun);
        MmiTraceInt(pwr_Status.charging_state);
        return;
    }
    //MmiTrace("Bob test LED blink with entering");
    if( ledblinkison==1 )
    {
        //reset the led blink timer     	
        timSetTime(ledblink,500);
        //the network is full service
        if( ledblinkflag==1 )
        {
			/* 2004/03/15 sunsj remove */
            //Green_On();
            //if red led is on, close it   Bob 02/18
			/* 2004/03/15 sunsj remove */
            //if( driverFlag&=0x0040 )
                //Red_Off();
        }
        //the network is no service
        else if( ledblinkflag==2 )
        {
			/* 2004/03/15 sunsj remove */
            //Red_On();
            //if green led is on, close it Bob 02/18
			/* 2004/03/15 sunsj remove */
            //if( driverFlag&=0x0020 )
                //Green_Off();
        }
        ledblinkison=0;
    }
    else
    {
        //reset the led blink timer
        timSetTime(ledblink,10000);
        //the network is full service
        if( ledblinkflag==1 )
            Green_Off();
        //the network is no service
		/* 2004/03/15 sunsj remove */
        //else if( ledblinkflag==2 )
        //    Red_Off();
        ledblinkison=1;
    }
    timStart(ledblink);
}
//Bob added for start ledblink 01/23
void ledblink_begin()
{
    ledblinkflag=0;
    ledblink=timCreate(0,2000,(MfwCb)LedBlinkEvent);
}
/*******************************************************************************

                                Private Methods

*******************************************************************************/
USHORT getSoundsReason(void)
{
    return soundReason;
}
/*******************************************************************************

 $Function:     setSoundsReason

 $Description:  Set the reason why the volume settings was invoked. Called to indicated
        that the volume settings is being invoked from Idle or from the incoming
        call screens.

 $Returns:    none

 $Arguments:  none

*******************************************************************************/
void setSoundsReason(USHORT reason)
{

    switch( reason )
    {
    case SettingVolume:
        soundReason = SettingVolume;
        break;
    case SoundsNone:
    default:
        soundReason = SoundsNone;
        break;
    }
}
/*******************************************************************************

 $Function:     volMenuItemSelected

 $Description:  A volume menu item has been selected, store the setting into PCM.

 $Returns:    none

 $Arguments:  none

*******************************************************************************/
void volMenuItemSelected(void)
{
    volumeSetting2Pcm =TRUE;
}
/*******************************************************************************

 $Function:     getScrollSelectMenuItem

 $Description:  get the scroll/select menu status.

 $Returns:    Status

 $Arguments:  none

*******************************************************************************/
BOOL getScrollSelectMenuItem(void)
{
    if( scrollSelectMenuItem == TRUE )
        return TRUE;
    else
        return FALSE;
}
/*******************************************************************************

 $Function:     clearScrollSelectMenuItem

 $Description:  Reset the scroll/select menu behavior.

 $Returns:    none

 $Arguments:  none

*******************************************************************************/
void clearScrollSelectMenuItem(void)
{
    scrollSelectMenuItem=FALSE;
    // soundReason = SoundsNone; MZ this flag cleared when user exits the melody/Volume menu.
    stopPlayingMelody();
}

/*******************************************************************************

 $Function:     soundsSetVolume

 $Description:  Function determines if user is in the Melody/SMS/Alarm ringer menu
        and has selected the vloume settings option.

 $Returns:    none

 $Arguments:  none

*******************************************************************************/
void soundsSetVolume(void)
{
    if( soundReason == MelodySelect || soundReason == SettingSMSTone ||
        soundReason == SettingAlarm || soundReason == SettingVolume )
    {
        // Menu items will be selected on scrolling.
        scrollSelectMenuItem = TRUE;
        // store the setting to PCM if volume setting is selected.
        volumeSetting2Pcm=FALSE;
    }
    else
    {
        scrollSelectMenuItem = FALSE;
    }

}
/*******************************************************************************

 $Function:     setSilentModeVolumeOn

 $Description:

 $Returns:    none

 $Arguments:  none

*******************************************************************************/
void setSilentModeVolume(void)
{
    stopPlayingMelody();
    oldVolumeSetting = current.volumeSetting;
    current.volumeSetting =0;
    audio_SetAmplf (AUDIO_BUZZER, current.volumeSetting);
}

/*******************************************************************************

 $Function:     setSilentModeVolumeOn

 $Description:

 $Returns:    none

 $Arguments:  none

*******************************************************************************/
void restoreSilentModeVolume(void)
{
    stopPlayingMelody();
    current.volumeSetting =oldVolumeSetting;
    audio_SetAmplf (AUDIO_BUZZER, current.volumeSetting);

}

/*******************************************************************************

 $Function:     soundReasonRinger

 $Description:  setup the soundReason to be Ringer Melody settings.

 $Returns:    none

 $Arguments:  none

*******************************************************************************/
void soundsVolSilent(void)
{

    FFS_flashData.settings_status |= SettingsSilentMode;
    flash_write();

    stopPlayingMelody();

    if( volumeSetting2Pcm != TRUE )
    {
        current.volumeSetting =0;
        audio_SetAmplf (AUDIO_BUZZER, current.volumeSetting);
        audio_PlaySoundID(AUDIO_BUZZER, current.ringer, 0, AUDIO_PLAY_INFINITE);
    }
    else
    {
        //store setting into PCM
        volumeSetting2Pcm=FALSE;
    }
}
/*******************************************************************************

 $Function:     soundsVolLow

 $Description:  set voulme level to low.

 $Returns:    none

 $Arguments:  none

*******************************************************************************/
void soundsVolLow(void)
{
    FFS_flashData.settings_status &= ~SettingsSilentMode;
    flash_write();
    stopPlayingMelody();
    if( volumeSetting2Pcm != TRUE )
    {
        current.volumeSetting =5;
        audio_SetAmplf (AUDIO_BUZZER, current.volumeSetting);
        audio_PlaySoundID(AUDIO_BUZZER, current.ringer, 0, AUDIO_PLAY_INFINITE);
    }
    else
    {
        //store setting into PCM
        volumeSetting2Pcm=FALSE;
    }
}
/*******************************************************************************

 $Function:     soundsVolMedium

 $Description:  set voulme level to Medium

 $Returns:    none

 $Arguments:  none

*******************************************************************************/
void soundsVolMedium(void)
{
    FFS_flashData.settings_status &= ~SettingsSilentMode;
    flash_write();
    stopPlayingMelody();
    if( volumeSetting2Pcm != TRUE )
    {
        current.volumeSetting =25;
        audio_SetAmplf (AUDIO_BUZZER, current.volumeSetting);
        audio_PlaySoundID(AUDIO_BUZZER, current.ringer, 0, AUDIO_PLAY_INFINITE);
    }
    else
    {
        //store setting into PCM
        volumeSetting2Pcm=FALSE;
    }

}
/*******************************************************************************

 $Function:     soundsVolHigh

 $Description:  set voulme level to High

 $Returns:    none

 $Arguments:  none

*******************************************************************************/
void soundsVolHigh(void)
{
    FFS_flashData.settings_status &= ~SettingsSilentMode;
    flash_write();
    stopPlayingMelody();
    if( volumeSetting2Pcm != TRUE )
    {
        current.volumeSetting =175;
        audio_SetAmplf (AUDIO_BUZZER, current.volumeSetting);
        audio_PlaySoundID(AUDIO_BUZZER, current.ringer, 0, AUDIO_PLAY_INFINITE);
    }
    else
    {
        //store setting into PCM
        volumeSetting2Pcm=FALSE;
    }

}
/*******************************************************************************

 $Function:     soundsVolInc

 $Description:  set voulme level to increasing

 $Returns:    none

 $Arguments:  none

*******************************************************************************/
void soundsVolInc(void)
{
    FFS_flashData.settings_status &= ~SettingsSilentMode;
    flash_write();
    stopPlayingMelody();
    if( volumeSetting2Pcm != TRUE )
    {
        current.volumeSetting = 5;
        audio_SetAmplf (AUDIO_BUZZER, current.volumeSetting);
        audio_PlaySoundID(AUDIO_BUZZER, current.ringer, 0, AUDIO_PLAY_INFINITE);
    }
    else
    {
        //store setting into PCM
        volumeSetting2Pcm=FALSE;

    }


}
/*******************************************************************************

 $Function:     soundReasonRinger

 $Description:  setup the soundReason to be Ringer Melody settings.

 $Returns:    none

 $Arguments:  none

*******************************************************************************/
USHORT soundReasonRinger( struct MfwMnuTag *m, struct MfwMnuAttrTag *ma, struct MfwMnuItemTag *mi )
{
    soundReason = MelodySelect;
    return 0;
}
/*******************************************************************************

 $Function:     soundReasonSmsTone

 $Description:  Setup the soundReason to be SMS tones settings.

 $Returns:    none

 $Arguments:  none

*******************************************************************************/
USHORT soundReasonSmsTone( struct MfwMnuTag *m, struct MfwMnuAttrTag *ma, struct MfwMnuItemTag *mi )
{ 
    //soundReason = SettingSMSTone;  xsf remarked it 2002.09.28
    soundReason = SoundsNone; 
    driverFlag&=0xFFFFFFFB;  //xsf add for KCD_LEFT 2003.01.14
    return 0;
}
/*******************************************************************************

 $Function:     soundReasonAlarm

 $Description:  Setup the soundReason to be Alarm settings.

 $Returns:    none

 $Arguments:  none

*******************************************************************************/
USHORT soundReasonAlarm( struct MfwMnuTag *m, struct MfwMnuAttrTag *ma, struct MfwMnuItemTag *mi )
{
    if( (((int)mi->str==TxtRingPrompt) ||((int)mi->str==TxtMelody)) && ( m->lCursor[m->level] == 1) )  //xsf add for KCD_LEFT 2003.01.14
        driverFlag|=0x04;
    else
        if( ((int)mi->str==TxtMelody) && ( m->lCursor[m->level] == 2) )
    {
        soundReason = SettingAlarm;
        driverFlag&=0xFFFFFFFB;  //xsf add for KCD_LEFT 2003.01.14
    }
    else
        driverFlag&=0xFFFFFFFB;

    if( ((int)mi->str==TxtRingPrompt) && (FlashEnvironmentData.EnviroProfile[Gu8_Envio_num].alarmprompt==RING) )
        return FLAGFUNC_TRUE_ICON;
    return 0;
}

USHORT soundReasonPhbk( struct MfwMnuTag *m, struct MfwMnuAttrTag *ma, struct MfwMnuItemTag *mi )
{
    soundReason = SettingPHBKGrouping;
    return 0;
}
/*******************************************************************************

 $Function:     getcurrentSMSTone

 $Description:  Access function for the current SMS Tone.
 $Returns:    Tone Id

 $Arguments:  None

*******************************************************************************/
UBYTE getcurrentSMSTone(void)
{
    return current.SMSTone;
}/*******************************************************************************

 $Function:     getcurrentSMSTone

 $Description:  Access function for the current SMS Tone.
 $Returns:    Tone Id

 $Arguments:  None

*******************************************************************************/
UBYTE getcurrentAlarmTone(void)
{
    //return current.AlarmOn;  //xsf changed 2003.01.29
    return FlashEnvironmentData.EnviroProfile[FlashEnvironmentData.CurrentEnv].alarmtype;
}
/*******************************************************************************

 $Function:     getCurrentRingerSettings

 $Description:  Access function for the current ringer settings data.
 $Returns:    Ringer Id.

 $Arguments:  None

*******************************************************************************/
UBYTE getCurrentRingerSettings(void)
{
    //return current.ringer;  xsf 2002.07.09
    return FlashEnvironmentData.EnviroProfile[FlashEnvironmentData.CurrentEnv].ringtype;
}
/*******************************************************************************

 $Function:     getCurrentVoulmeSettings

 $Description:  Access function for the current ringer volume settings data.
 $Returns:    volume

 $Arguments:  None

*******************************************************************************/
UBYTE getCurrentVoulmeSettings(void)
{
    return current.volumeSetting;
}
/*******************************************************************************

 $Function:     getMelodyListStatus

 $Description:  Status flag indicating if a melody selection menu is active.
 $Returns:    status

 $Arguments:  None

*******************************************************************************/
BOOL getMelodyListStatus(void)
{
    if( soundReason == MelodySelect ||
        soundReason == SettingSMSTone ||
        soundReason == SettingAlarm ||
        soundReason == SettingPHBKGrouping )
        return TRUE;
    else
        return FALSE;
}


/*******************************************************************************

 $Function:     stopPlayingMelody

 $Description:  stop currently played ringer melody

 $Returns:    None

 $Arguments:  None

*******************************************************************************/

static void stopPlayingMelody( void )
{
    TRACE_FUNCTION( "stopPlayingMelody" );

    /* If a timer event is running we want to stop that as well,
       since we don't really want to turn the tune off and have
       it start again when the timer expires
    */
    if( hRingerStartTimer != NULL )
        timStop( hRingerStartTimer );

    /* Are we playing a tune
    */
    if( LastTune )
        audio_StopSoundbyID( AUDIO_BUZZER, LastTune );

    /* Need to wait for the tune to stop before trying to
       set up the next tune, so wait always
    */
    vsi_t_sleep( 0, 5 );
}

/*******************************************************************************

 $Function:     startPlayingMelody

 $Description:  Start played the ringer melody

 $Returns:    None

 $Arguments:  Melody_id

*******************************************************************************/

static void startPlayingMelody(UBYTE melody_id)
{

    if( hRingerStartTimer != NULL )
        timStop( hRingerStartTimer );

    audio_PlaySoundID(AUDIO_BUZZER, melody_id, (BYTE)current.volumeSetting, AUDIO_PLAY_INFINITE);

}


/*******************************************************************************

 $Function:     soundsPlayRinger

 $Description:  Play the tune selected by melody_id, stopping any currently
        playing tunes if necessary. Remember to take into account the
        volume setting and provide a crescendo if requested

 $Returns:    None

 $Arguments:  melody_id, selects the melody to be played

*******************************************************************************/

static void soundsPlayRinger( int melody_id )
{
    CurrentTune = melody_id;
    if( LastTune )
        audio_StopSoundbyID (AUDIO_BUZZER,LastTune );
    vsi_t_sleep(0, 5);

    if( current.volumeSetting == increasing )
    {
        audio_PlaySoundID ( AUDIO_BUZZER,CurrentTune, 0 , AUDIO_PLAY_CRESCENDO );
    }
    else
    {
        audio_PlaySoundID ( AUDIO_BUZZER,CurrentTune, 0, AUDIO_PLAY_INFINITE );
    }
    LastTune = CurrentTune;
}


/*******************************************************************************

 $Function:     destroyDynaMenu

 $Description:  frees the memory allocated to a dynamenu (Refer to the
        buildMelodyMenu routine below

 $Returns:    None.

 $Arguments:  None.

*******************************************************************************/

static void destroyDynaMenu( void )
{

}


/*******************************************************************************

 $Function:     buildMelodyMenu

 $Description:  Builds the melody menu dynamenu from the melodies supplied
        by the resource manager

 $Returns:    None

 $Arguments:  None

*******************************************************************************/
static T_MFW_HND  buildMelodyMenu( MfwHnd parent_window)
{
    T_MELODY_INFO *  data = (T_MELODY_INFO *)ALLOC_MEMORY (sizeof (T_MELODY_INFO));
    T_MFW_WIN  * win;

    //TRACE_EVENT (">>>> buildMelodyMenu()");

    /*
     * Create window handler
     */

    data->sounds_win = win_create (parent_window, 0, E_WIN_VISIBLE, (T_MFW_CB)melody_win_cb);
    if( data->sounds_win EQ NULL )
    {
        return NULL;
    }

    //TRACE_EVENT(">>>> Melody window created: " );
    /*
     * connect the dialog data to the MFW-window
     */

    data->mmi_control.dialog = (T_DIALOG_FUNC)melody_exec_cb;
    data->mmi_control.data   = data;
    win                      = ((T_MFW_HDR *)data->sounds_win)->data;
    win->user                = (void *)data;
    data->parent             = parent_window;
    winShow(data->sounds_win);
    /*
     * return window handle
     */

    return data->sounds_win;
}

USHORT MelodymenuItemCheck(struct MfwMnuTag *m, struct MfwMnuAttrTag *ma, struct MfwMnuItemTag *mi)
{ 
    //TRACE_EVENT("MelodymenuItemCheck");
    char temp[100];
    if( soundReason == MelodySelect )   //xsf add for MelodySelect and SettingAlarm 2002.09.24
    {
        // if((int)mi->str==MelodyTable[FlashEnvironmentData.EnviroProfile[Gu8_Envio_num].ringtype])

        if( (int)mi->Dynamicindex==FlashEnvironmentData.EnviroProfile[Gu8_Envio_num].ringtype )
            //chenjun changed for user ring 2003-01-10   
            return FLAGFUNC_TRUE_ICON;
        else
            return FLAGFUNC_FALSE_ICON;    
    }
    else if( soundReason==SettingPHBKGrouping )
    {
        //    if ((int)mi->str==MelodyTable[FFS_PBOtherData.GroupingItem[RingSelect].GroupingRing])
        if( (int)mi->Dynamicindex==FFS_PBOtherData.GroupingItem[RingSelect].GroupingRing )
            //chenjun changed for user ring 2003-01-10   
            return FLAGFUNC_TRUE_ICON;
        else
            return FLAGFUNC_FALSE_ICON;     
    }
    else
    {
        // if((int)mi->str==MelodyTable[FlashEnvironmentData.EnviroProfile[Gu8_Envio_num].alarmtype])     
        if( (int)mi->Dynamicindex==FlashEnvironmentData.EnviroProfile[Gu8_Envio_num].alarmtype )
            //chenjun changed for user ring 2003-01-10
            return FLAGFUNC_TRUE_ICON;
        else
            return FLAGFUNC_FALSE_ICON;    
    }
}


/*******************************************************************************

 $Function:     melody_exec_cb

 $Description:  Exec callback function of the SMS read window (dynamic list of
        all the SMS)

 $Returns:    none

 $Arguments:  win - window handler
        event - mfw event
        parameter - optional data.
      
*******************************************************************************/
void melody_exec_cb (T_MFW_HND win, USHORT event, SHORT value, void * parameter)
{
    T_MFW_WIN      * win_data = ((T_MFW_HDR *) win)->data;
    T_MELODY_INFO   * data = (T_MELODY_INFO *)win_data->user;
    UBYTE i;
    UBYTE numberOfMelodies;
    UBYTE numberOfUserMelodies;// user ring
    //UBYTE j,k;

    ListWinInfo      * mnu_data = (ListWinInfo *)parameter;

    ReadUserRingInfo(&tUserRing); 
    numberOfUserMelodies=tUserRing.number;

    //MmiTrace("chenjun ,number of user melody is:");
    //MmiTraceInt(numberOfUserMelodies);
    //for(i=0;i<5;i++)
    //{
    //	MmiTrace(tUserRing.filename[i]);
    //	MmiTraceInt(strlen(tUserRing.filename[i]));
    //	if(strlen(tUserRing.filename[i])==0)
    //		MmiTrace("kong");  	
    //}

    MMI_TRACE_EVENT( ("glowing: melody_exec_cb"));

    switch( event )
    {
    case E_INIT:

        MMI_TRACE_EVENT(("melody_exec_cb -- Event:E_INIT"));
        gPlayKeytoneFlag = 0;

        /* initialization of administrative data */
        data->menu_list_data = (ListMenuData *)ALLOC_MEMORY(sizeof(ListMenuData));

        if( data->menu_list_data == 0 )
        {
            MMI_TRACE_EVENT(("Failed memory alloc 1 "));
            return;
        }

        numberOfMelodies = resource_GetListCount(RES_MELODY);
        //sprintf(temp,"numberofmelodies:%d",numberOfMelodies);
        //TRACE_EVENT(temp);

        //gCjMeloList2Free=0;//set it to 0, ++ when alloced one  ,for user ring

        //sprintf(temp,"numberofmelodies:%d,user:%d",numberOfMelodies,numberOfUserMelodies);
        //MmiTrace(temp);

        //ڷ -- һ˵""
        if( soundReason==SettingPHBKGrouping )
        {
            data->menu_list_data->List = (T_MFW_MNU_ITEM *)ALLOC_MEMORY( (numberOfMelodies+numberOfUserMelodies) * sizeof(T_MFW_MNU_ITEM) );//chenjun 2003-01-09 user ring

            if( data->menu_list_data->List == 0 )
            {
                MMI_TRACE_EVENT(("Failed memory alloc 2"));
                if( data->menu_list_data NEQ 0 )
                    FREE_MEMORY ((void *)data->menu_list_data, sizeof (ListMenuData));
                return;
            }

            for( i = 0; i < numberOfMelodies-1 ; i++ )
            {
				/* 2004/06 sunsj modify for picture manage */
                data->menu_list_data->List[i].icon = GET_ICONATTR(ICON_SITUATION, 0);//&situationIcon;
                data->menu_list_data->List[i].exta = 0;
                data->menu_list_data->List[i].edit = 0;
                data->menu_list_data->List[i].menu = 0;
                data->menu_list_data->List[i].func = NULL;


                data->menu_list_data->List[i].str=(char*)MmiRsrcGetText(MelodyTable[i]);
                data->menu_list_data->List[i].Dynamicindex=i;//chenjun add for MelodymenuItemCheck() use. user ring ,2003-01-10
                data->menu_list_data->List[i].flagFunc = MelodymenuItemCheck;
            }

            for( i=numberOfMelodies-1;i<numberOfMelodies+numberOfUserMelodies-1;i++ )
            {
				/* 2004/06 sunsj modify for picture manage */
                data->menu_list_data->List[i].icon = GET_ICONATTR(ICON_SITUATION, 0);//&situationIcon;
                data->menu_list_data->List[i].exta = 0;
                data->menu_list_data->List[i].edit = 0;
                data->menu_list_data->List[i].menu = 0;
                data->menu_list_data->List[i].func = NULL;

                data->menu_list_data->List[i].str=(char*)tUserRing.filename[i-numberOfMelodies+1];

                data->menu_list_data->List[i].Dynamicindex=i;//chenjun add for MelodymenuItemCheck() use. user ring ,2003-01-10
                data->menu_list_data->List[i].flagFunc = MelodymenuItemCheck;
            }// 2003-01-09 user ring,add

            //changed former to following  grouping ring ==none, depart  from the round
            //for (i = 0; i < numberOfMelodies; i++),  now is for (i = 0; i < numberOfMelodies-1 ; i++) + this
            {
                i=numberOfMelodies+numberOfUserMelodies-1;
				/* 2004/06 sunsj modify for picture manage */
                data->menu_list_data->List[i].icon = GET_ICONATTR(ICON_SITUATION, 0);//&situationIcon;
                data->menu_list_data->List[i].exta = 0;
                data->menu_list_data->List[i].edit = 0;
                data->menu_list_data->List[i].menu = 0;
                data->menu_list_data->List[i].func = NULL;

                data->menu_list_data->List[i].str=(char*)MmiRsrcGetText(TxtmelodyNone/*MelodyTable[14]*/); //TxtmelodyNone, 
                data->menu_list_data->List[i].Dynamicindex=i;//add for MelodymenuItemCheck() use. user ring ,2003-01-10
                data->menu_list_data->List[i].flagFunc = MelodymenuItemCheck;
            }

            data->menu_list_data->ListLength =numberOfMelodies+numberOfUserMelodies;  //2003-01-09 user ring,add  
            data->menu_list_data->ListPosition = numberOfMelodies+numberOfUserMelodies-2;   //2003-01-09 user ring,add
            data->menu_list_data->CursorPosition = numberOfMelodies+numberOfUserMelodies-2;  //2003-01-09 user ring,add 
            data->menu_list_data->SnapshotSize = numberOfMelodies+numberOfUserMelodies;  //2003-01-09 user ring,add
            data->menu_list_data->Font = 0;
            data->menu_list_data->LeftSoftKey = TxtSoftSelect;
            data->menu_list_data->RightSoftKey = TxtSoftBack;
            data->menu_list_data->KeyEvents = KEY_ALL;
            data->menu_list_data->Reason = 0;
            data->menu_list_data->Strings = TRUE;//FALSE; 
            data->menu_list_data->Attr   = &melody_menuAttrib;
            data->menu_list_data->autoDestroy    = FALSE;

            data->menu_list_data->ListMenuFlag =1;
            data->menu_list_data->ListMenuHeadStr = (char *) MmiRsrcGetText(TxtMelody);


        }
        else
        {
            data->menu_list_data->List = (T_MFW_MNU_ITEM *)ALLOC_MEMORY( (numberOfMelodies+numberOfUserMelodies-1) * sizeof(T_MFW_MNU_ITEM) );

            if( data->menu_list_data->List == 0 )
            {
                MMI_TRACE_EVENT(("Failed memory alloc 2"));
                if( data->menu_list_data NEQ 0 )
                    FREE_MEMORY ((void *)data->menu_list_data, sizeof (ListMenuData));
                return;
            }

            for( i = 0; i < (numberOfMelodies-1) ; i++ )
            {
				/* 2004/06 sunsj modify for picture manage */
                data->menu_list_data->List[i].icon = GET_ICONATTR(ICON_SITUATION, 0);//&situationIcon;//changed 02/12/29
                data->menu_list_data->List[i].exta = 0;
                data->menu_list_data->List[i].edit = 0;
                data->menu_list_data->List[i].menu = 0;
                data->menu_list_data->List[i].func = NULL;
                //(data->menu_list_data->List[i].str ) =(char *) MelodyTable[i]; 


                data->menu_list_data->List[i].str=(char*)MmiRsrcGetText(MelodyTable[i]);
                data->menu_list_data->List[i].Dynamicindex=i;//chenjun add for MelodymenuItemCheck() use. user ring ,2003-01-10
                data->menu_list_data->List[i].flagFunc = MelodymenuItemCheck;
            }
            for( i=numberOfMelodies-1;i<numberOfMelodies+numberOfUserMelodies-1;i++ )
            {
				/* 2004/06 sunsj modify for picture manage */
                data->menu_list_data->List[i].icon = GET_ICONATTR(ICON_SITUATION, 0);//&situationIcon;
                data->menu_list_data->List[i].exta = 0;
                data->menu_list_data->List[i].edit = 0;
                data->menu_list_data->List[i].menu = 0;
                data->menu_list_data->List[i].func = NULL;

                data->menu_list_data->List[i].str=(char*)tUserRing.filename[i-numberOfMelodies+1];

                data->menu_list_data->List[i].Dynamicindex=i;//chenjun add for MelodymenuItemCheck() use. user ring ,2003-01-10
                data->menu_list_data->List[i].flagFunc = MelodymenuItemCheck;
            }//chenjun 2003-01-09 user ring,add
            data->menu_list_data->ListLength =numberOfMelodies+numberOfUserMelodies-1;//chenjun 2003-01-09 user ring
            data->menu_list_data->ListPosition = 1;
            data->menu_list_data->CursorPosition = 1;
            data->menu_list_data->SnapshotSize = numberOfMelodies+numberOfUserMelodies-1;//chenjun 2003-01-09 user ring
            data->menu_list_data->Font = 0;
            data->menu_list_data->LeftSoftKey = TxtSoftSelect;
            data->menu_list_data->RightSoftKey = TxtSoftBack;
            data->menu_list_data->KeyEvents = KEY_ALL;
            data->menu_list_data->Reason = 0;
            data->menu_list_data->Strings = TRUE;//FALSE;//chenjun
            data->menu_list_data->Attr   = &melody_menuAttrib;
            data->menu_list_data->autoDestroy    = FALSE;    
            data->menu_list_data->ListMenuFlag =1;
            data->menu_list_data->ListMenuHeadStr = (char *) MmiRsrcGetText(TxtMelody);
        }

        listDisplayListMenu(win, data->menu_list_data,(ListCbFunc)MelodyList_cb,0);
        break;  
    case E_START_MELODY:
        // User has scrolled the Melody list, stop old melody and start playing new melody.
        MMI_TRACE_EVENT(("glowing: melody_exec_cb-E_START_MELODY"));
        //if (melody_status==MELODY_BUSY)
        StopPlayingMelody2( );  //glowing: modify it as the new api
        //StopMelody2(oldringstr);

        //memset(currentringstr, '\0', 20);
        numberOfMelodies=resource_GetListCount(RES_MELODY);
        //PTRACE( sprintf( buf, "glowing: MenuData.ListPosition: %d", mnu_data->MenuData.ListPosition ); )

        if( !strcmp(mnu_data->MenuData.List[mnu_data->MenuData.ListPosition].str,(char*)MmiRsrcGetText(TxtmelodyNone)) )
            break;     //add for user ring, this deal none in grouping ring 2003-01-11

        //{
        //   strncpy(currentringstr,MelodyTablestr[/*j*/mnu_data->MenuData.ListPosition],strlen(MelodyTablestr[/*j*/mnu_data->MenuData.ListPosition]));
        //}
        curMelodyId = mnu_data->MenuData.ListPosition; //glowing: replace it with currentringstr

        //MmiTrace("chenjun currentringstr in exec_cb:"); 
        //MmiTrace(currentringstr);
        timStart(hRingerStartTimer);
        break;

    case E_RETURN:
        //stop playing the melody stopPlayingMelody();
        //if (melody_status==MELODY_BUSY)
        StopPlayingMelody2( );
        //StopMelody2(oldringstr);

        timStop(hRingerStartTimer);
        gPlayKeytoneFlag  = 1;
        break;
    default:
        break;
    }
}
/*******************************************************************************

 $Function:     MelodyList_cb

 $Description:  Callback function for the melody list.

 $Returns:    none

 $Arguments:  Parent - parent window.
        ListData - Menu item list
*******************************************************************************/

void MelodyList_cb(T_MFW_HND * Parent, ListMenuData * ListData)
{
    T_MFW_WIN       * win_data = ((T_MFW_HDR *)Parent)->data;
    T_MELODY_INFO * data     = (T_MELODY_INFO *)win_data->user;
    //T_MFW_HND      * l_parent;

    MMI_TRACE_EVENT(("glowing: MelodyList_cb-enter"));
    if( (ListData->Reason EQ LISTS_REASON_BACK) || (ListData->Reason EQ LISTS_REASON_CLEAR)
        || (ListData->Reason EQ LISTS_REASON_HANGUP) )   // added hangup key
    {
        soundReason=SoundsNone; //add 2002.09.28
        //if (melody_status==MELODY_BUSY)
        //{
        //StopMelody2(oldringstr); 
        StopPlayingMelody2( ); //glowing: use the new api 2003-12-01
        //}

        if( hRingerStartTimer != NULL )
            timDelete(hRingerStartTimer);  //add 2003.01.22
        driverFlag&=0xFFFFFFFB;  //add for KCD_LEFT 2003.01.14
        listsDestroy(ListData->win);   //moved from the end of this function  for LISTS_REASON_UPDATE 2002.09.04		   
        melody_destroy(data->sounds_win);
    }

    else if( (ListData->Reason EQ LISTS_REASON_SELECT) || (ListData->Reason EQ LISTS_REASON_OK) )     //xsf add for list menu KCD_OK=KCD_LEFT 2003.02.18
    {
        switch( soundReason )
        {
        case MelodySelect:
            if( melody_status==MELODY_IDLE )  //add 2003.01.22
                timStart(hRingerStartTimer);
            FlashEnvironmentData.EnviroProfile[Gu8_Envio_num].ringtype=ListData->ListPosition;

            flash_environmentwrite();
            break;
        case SettingSMSTone:
            StopPlayingMelody2( ); //glowing: use the new api 2003-12-01
            //stopPlayingMelody();
            break;
        case SettingAlarm:
            if( melody_status==MELODY_IDLE )  //add 2003.01.22
                timStart(hRingerStartTimer);
            FlashEnvironmentData.EnviroProfile[Gu8_Envio_num].alarmtype=ListData->ListPosition;
            flash_environmentwrite();
            break;
        case SettingPHBKGrouping:
            {
                FFS_PBOtherData.GroupingItem[RingSelect].GroupingRing=ListData->ListPosition;
                flash_PBOtherWrite("PBOption"); 
            }  
            break;         
        default:
            break;
        }
        winShow(ListData->win);   //xsf add 2002.09.05
        // Store selected melody in PCM storage.
    }
}

/*******************************************************************************

 $Function:     melody_destroy

 $Description:  Destroy the melody window.

 $Returns:    none

 $Arguments:  own_window- current window
*******************************************************************************/

static void melody_destroy(MfwHnd own_window)
{
    T_MFW_WIN * win_data;
    T_MELODY_INFO     * data;
    UBYTE numberOfMelodies;
    UBYTE i;

    //TRACE_EVENT ("melody_destroy()");

    if( own_window )
    {
        win_data = ((T_MFW_HDR *)own_window)->data;
        data = (T_MELODY_INFO *)win_data->user;

        if( data )
        {
            /*
             * Delete WIN handler
             */
            win_delete (data->sounds_win);

            gPlayKeytoneFlag = 1; //glowing,2003-12-15, enable Play Keytone

            /*
             * Free Memory
             */

            /*     //chenjun add to destroy alloced for dynamic menu list, user ring 2003-01-14 
                for(i=0;i<gCjMeloList2Free;i++)
                    FREE_MEMORY ((void *)data->menu_list_data->List[i].str, MAX_RING_NAME_LEN);
                //chenjun add over for user ring 2003-01-14
              */  

            numberOfMelodies = resource_GetListCount(RES_MELODY);
            numberOfMelodies+= GetUserRingNum();//chenjun add to destroy menu alloced for user ring 2003-01-14
            if( data->menu_list_data != NULL )
            {
                if( soundReason==SettingPHBKGrouping )  //xsf add for distingish from group melody 2002.09.27
                    FREE_MEMORY ((void *)data->menu_list_data->List, numberOfMelodies * sizeof (T_MFW_MNU_ITEM));
                else
                    FREE_MEMORY ((void *)data->menu_list_data->List, (numberOfMelodies-1) * sizeof (T_MFW_MNU_ITEM));
                FREE_MEMORY ((void *)data->menu_list_data, sizeof (ListMenuData));
            }
            FREE_MEMORY ((void *)data, sizeof (T_MELODY_INFO));

        }
        else
        {
            TRACE_EVENT ("melody_destroy() called twice");
        }
    }
}

/*******************************************************************************

 $Function:     melody_win_cb

 $Description:  Window callback function for the melody window.

 $Returns:    none

 $Arguments:  w - mfw window handler
        e - mfw event

*******************************************************************************/

static int melody_win_cb (MfwEvt e, MfwWin *w)    /* yyy window event handler */
{
    //TRACE_EVENT ("melody_win_cb()");
    switch( e )
    {
    case MfwWinVisible:  /* window is visible  */
        break;
    case MfwWinFocussed: /* input focus / selected   */
    case MfwWinDelete:   /* window will be deleted   */

    default:
        return MFW_EVENT_REJECTED;
    }
    return MFW_EVENT_CONSUMED;
}

/*******************************************************************************

 $Function:     melody_menu_cb

 $Description:  Call back function for the melody menu.

 $Returns:

 $Arguments:
*******************************************************************************/

static int melody_menu_cb (MfwEvt e, MfwMnu *m){

    T_MFW_HND       win  = mfwParent(mfw_header());
    T_MFW_WIN     * win_data = ((T_MFW_HDR *)win)->data;
    //    T_call_menu * data = (T_call_menu *)win_data->user;

    switch( e )
    {
    case E_MNU_ESCAPE:
        //call_menu_destroy(win);
        //call_data.win_menu = 0;
        break;
    default:
        return 0;
    }
    return 1;
}

/*******************************************************************************

 $Function:     resource_GetListCount

 $Description:  Returns the number of entries in resource table.

 $Returns:    Number of entries in Melody table.

 $Arguments:  res - Resource Id.

*******************************************************************************/
int resource_GetListCount(res_ResourceID_type res)
// Returns the number of the resource type available in the phone.
{
    int index=0;
    switch( res )
    {
    case RES_MELODY:
        //while (MelodyTable[index] !=0)  //glowing: comment it: 2003-12-09
        //   index++;
        //return index;
        return(LAST_RING_MELODY - FIRST_RING_MELODY + 1); //glowing,2003-12-15
        break;
    case RES_MELODY_STR:  //add for melodytablestr[] 12.26
        //while (MelodyTablestr[index] !=0) //glowing: comment it: 2003-12-09
        //   index++;
        //return index;
        return(LAST_RING_MELODY - FIRST_RING_MELODY + 1); //glowing,2003-12-15
        break;
    case RES_MELODY_SMS:
        //while (SmsMelodyTablestr[index] !=0) //glowing: comment it: 2003-12-09
        //   index++;
        //return index;
        return( LAST_SMS_MELODY - FIRST_SMS_MELODY +1 );
        break;
    case RES_LANGUAGE:

        break;
    case RES_STRING:

        break;
    default:

        break;

    }
    return index;
}
// 2004.4.9 wyj modify 
#if 0
/*******************************************************************************

 $Function:     buildOnOffMenu

 $Description:  builds an on/off menu and associates it with a context

 $Returns:    None

 $Arguments:  count, number of elements in menu
        id3, prompt string
        active, context into which the menu should be associated

*******************************************************************************/

static void buildOnOffMenu( int count, int id3, int active )
{
}

/*******************************************************************************

 $Function:     buildVolumeMenu

 $Description:  builds the volume menu and associates it with the volume
        context

 $Returns:    None

 $Arguments:  volume, the context to be associated with

*******************************************************************************/

static void buildVolumeMenu( VOL_ENUM volume )
{
}

/*******************************************************************************

 $Function:     displayDynaMenu

 $Description:  show one page of the ringer list, determined by item.

 $Returns:    None

 $Arguments:  None

*******************************************************************************/

static void displayDynamenu( void )
{

}

/*******************************************************************************

 $Function:     displayRingerVolume

 $Description:  shows the volume setting determined by cVolume

 $Returns:    None

 $Arguments:  None

*******************************************************************************/

static void displayRingerVolume( void )
{
}

/*******************************************************************************

 $Function:     displayOnOffList

 $Description:  show the On Off (3rd) list, determined by item.

 $Returns:    None

 $Arguments:  None

*******************************************************************************/

static void displayOnOffList( void )
{
}
/*******************************************************************************

 $Function:     buildLanguageMenu

 $Description:  builds the dynamenu for the RES_LANGUAGE resource type

 $Returns:    None

 $Arguments:  None

*******************************************************************************/

static void buildLanguageMenu(void)
{

}

/*******************************************************************************

 $Function:     delayThenPlayMelody

 $Description:  play selected ringer melody after short delay

 $Returns:    None

 $Arguments:  None

*******************************************************************************/

static void delayThenPlayMelody( void )
{

    MMI_TRACE_EVENT(("glowing: delayThenPlayMelody - enter")); 

    //if (melody_status==MELODY_BUSY)
    StopPlayingMelody2( );
    //StopMelody2(oldringstr);      

    curMelodyId = FIRST_RING_MELODY;
    //memset(currentringstr, '\0', 20);
    //strncpy(currentringstr,MelodyTablestr[0],strlen(MelodyTablestr[0]));

    // start timer for half a second and on timeout start everlasting play  
    timStart( hRingerStartTimer );
}

/*******************************************************************************

 $Function:     playCurrentSound

 $Description:  play tune which is temporarily selected, this event handler
        is invoked when the hRingerStartTimer event completes

 $Returns:    MFW_EVENT_CONSUMED always

 $Arguments:  e, event, tc timer context

*******************************************************************************/

static int playCurrentSound( MfwEvt e, MfwTim *tc )
{
    uint8 volume;

    if( soundReason == SettingAlarm )  //xsf add 2003.04.14
        volume = FlashEnvironmentData.EnviroProfile[Gu8_Envio_num].alarmvolume;
    else
        volume = FlashEnvironmentData.EnviroProfile[Gu8_Envio_num].ringvolume;

    SetMVol_DownLinkPGA(volume);


    //if(!strcmp(currentringstr, MelodyTablestr[0]))
    //	StartMelody2_aloud(currentringstr,0);  
    //else
    // StartMelody2(currentringstr,0);  
    StartPlayingMelody2(curMelodyId, 0);



    // stop running timer (One shot mode only)
    if( hRingerStartTimer != NULL )
    {
        timStop(hRingerStartTimer);
    }

    return MFW_EVENT_CONSUMED;
}
#endif

/*******************************************************************************

 $Function:     setSelection

 $Description:  reactivates calling menu after confirming setting

 $Returns:    MFW_EVENT_PASSED always

 $Arguments:  e, event, m, window handle

*******************************************************************************/

static int setSelection( MfwEvt e, MfwWin *m )
{
    int UpdatePCM = 0;
    int SoundToPlay = -1;

    TRACE_FUNCTION("setSelection");

    /* This implements a state machine, the next setting depends
       on the reason we have been invoked
  
       Note the following is still pending implementation
       USSD TONE, SVC TONE, MANPLMN TONE
    */
    switch( soundReason )
    {
    case MelodySelect:
        {
            //      stopPlayingMelody();
            //      current.ringer = menu_Select( &dynamenu );
            //      UpdatePCM = 1;
        }
        break;

    case SettingVolume:
        {
            //      stopPlayingMelody();
            //      current.volumeSetting = menu_Select( &volume_menu );
            //      UpdatePCM = 1;
        }
        break;

    default:
        {
            /* No action required
            */
        }
        break;
    }

    /* Update the PCM if we need to
    */
    //nm  if ( UpdatePCM )
    //nm    writeSettingsToPCM( NO_FLUSH );

    /* And play the requested sound if we need to
    */
    //nm  if ( SoundToPlay != -1 )
    //nm    audio_PlaySoundID ( AUDIO_BUZZER, SoundToPlay, 0, AUDIO_PLAY_ONCE );

    /* redisplay sub menu showing changed position of active marker.
    */
    winShow( win );
    //JVJE  soundExec( SoundsDone, 0 );

    return MFW_EVENT_PASSED;
}







/*******************************************************************************

 $Function:     writeSettingsToPCM

 $Description:  writes contents of global vars of sounds to PCM or
          sets default values

 $Returns:    None

 $Arguments:  flush, dictates whether or not flushing should be
        performed on the write operation

*******************************************************************************/
static void writeSettingsToPCM( U8 flush )
{
}
/*******************************************************************************

 $Function:     readSettingsFromPCM

 $Description:  inits global vars of sounds from PCM or
          sets default values

 $Returns:    None

 $Arguments:  None

*******************************************************************************/

static void readSettingsFromPCM(void)
{
}

/*******************************************************************************

 $Function:     soundsCreate

 $Description:  Create a sounds context, with a window, keyboards, timers
        etc.

 $Returns:    None

 $Arguments:  parent, handle of the parent window

*******************************************************************************/

static void soundsCreate(MfwHnd parent)
{

}


/*******************************************************************************

 $Function:     soundsDestroy

 $Description:  cleans up a context

 $Returns:    None

 $Arguments:  None

*******************************************************************************/

static void soundsDestroy(void)
{

}


/*******************************************************************************

 $Function:     getCallingMenu

 $Description:  handles transition to calling menu
          a) on timeout,
          b) during running timer when Clear, SoftKeyLeft
             or SoftKeyRight is pressed

 $Returns:    MFW_EVENT_PASSED

 $Arguments:  e, event, tc, timer context

*******************************************************************************/

static int getCallingMenu(MfwEvt e, MfwTim *tc)
{
    TRACE_FUNCTION( "BACK to invocating Menu" );

    return 0;
}

/*******************************************************************************

 $Function:     winEvent

 $Description:  window event handler, only deals with the win visible
        event, all others are ignored

 $Returns:    MFW_EVENT_PASSED or MFW_EVENT_CONSUMED depending on the
        event

 $Arguments:  e, event, w, window handle

*******************************************************************************/

static int winEvent (MfwEvt e, MfwWin *w)
{
    return MFW_EVENT_CONSUMED;
}


/*******************************************************************************

 $Function:     keyEventLong

 $Description:  keyboard event handler Long press

 $Returns:    MFW_EVENT_PASSED or MFW_EVENT_CONSUMED depending on the
        event

 $Arguments:  e, event, k, keyboard handle

*******************************************************************************/

static int keyEventLong (MfwEvt e, MfwKbd *k)
{
    //   PTRACE( sprintf( buf, "keycode in SOUND keyEvtLong %d", (int) k->code ); )
    //   PTRACE( sprintf( buf, "mfwEvt in SOUND Long 0x%10x", e ); )


    return MFW_EVENT_PASSED; /* give other handlers a chance */
}

/*******************************************************************************

 $Function:     keyEvent

 $Description:  keyboard event handler

 $Returns:    MFW_EVENT_PASSED or MFW_EVENT_CONSUMED depending on the
        event

 $Arguments:  e, event, k, keyboard handle

*******************************************************************************/

static int keyEvent (MfwEvt e, MfwKbd *k)
{

    //   PTRACE( sprintf( buf, "keycode in SOUND keyEvt %d", (int) k->code ); )
    //   PTRACE( sprintf( buf, "mfwEvt in SOUND 0x%10x", e ); )

    return MFW_EVENT_CONSUMED;
}

/*******************************************************************************

 $Function:     stepVibrator

 $Description:  Changes the state of the vibrator from vibrating to
                not vibrating and then restarts the vibration timer

 $Returns:    None

 $Arguments:  None

*******************************************************************************/

static void stepVibrator( void )
{
    //TRACE_FUNCTION("Vibrator Ringing");

}

/*******************************************************************************

 $Function:     ringVibrator

 $Description:  generates the ringing vibrations as configured

 $Returns:    New setting of the vibrator counter

 $Arguments:  c, pointer to a vibrator counter

*******************************************************************************/

static int ringVibrator( UBYTE *c )
{
    return 0;
}


/*******************************************************************************

 $Function:     VibratorTimerEvent

 $Description:  Vibrator timer event

 $Returns:    Event consumed, always

 $Arguments:  e, event, t, timer

*******************************************************************************/

static int VibratorTimerEvent ( MfwEvt e, MfwTim *t )
{

    return MFW_EVENT_CONSUMED;
}

/*******************************************************************************

 $Function:     volumeSettingTimeOut

 $Description:  The Ringer volume settings time out has occured and used has not selected
        a volume setting.

 $Returns:    MFW_EVENT_CONSUMED always

 $Arguments:  e, event, tc timer context

*******************************************************************************/
static int volumeSettingTimeOut( MfwEvt e, MfwTim *tc )
{
    T_MFW_HND win = mfw_parent(mfw_header());
    T_MFW_WIN * win_data = ((T_MFW_HDR *)win)->data;
    T_idle * idle_data = (T_idle *)win_data->user;
    //T_call * call_data = (T_call*)win_data->user;

    TRACE_EVENT(">>>> volumeSettingTimeOut(), Vol Setiing Time out. ");

    // stop running timer.
    if( hRingerVolSettingTimeOut != NULL )
    {
        timStop(hRingerVolSettingTimeOut);
    }

    if( call_data.win_menu !=NULL && (call_data.call_direction == INCOMING) )
    {
        bookMenuDestroy(call_data.win_menu);
        /*
          If volume settings dialog timed out while in the incoming call, allow the
          ringer to continue ringing. MZ 9/4/01
        */
        startPlayingMelody(getCurrentRingerSettings());

    }
    else if( idle_data->info_win != NULL )
    {
        bookMenuDestroy(idle_data->info_win);
    }

    return MFW_EVENT_CONSUMED;
}

/*******************************************************************************

                                Public Methods

*******************************************************************************/
/*******************************************************************************

 $Function:     restartVolSettingInactivityTimer

 $Description:  Restart the Volume settings timer.

 $Returns:    none

 $Arguments:  none

*******************************************************************************/
void restartVolSettingInactivityTimer(void)
{
    if( hRingerVolSettingTimeOut != NULL )
        timStop( hRingerVolSettingTimeOut );

    timStart(hRingerVolSettingTimeOut);

}
/*******************************************************************************

 $Function:     stopRingerVolTimeOut

 $Description:  Stop the ringer volume settings timer.

 $Returns:    none

 $Arguments:  none

*******************************************************************************/
void stopRingerVolSettingInactivityTimer(void)
{

    if( hRingerVolSettingTimeOut != NULL )
        timStop( hRingerVolSettingTimeOut );

}
/*******************************************************************************

 $Function:     startRingerVolumeSettingTimer

 $Description:  Create and start the ringer voulme inactivity timer.

 $Returns:    none

 $Arguments:  none

*******************************************************************************/
void startRingerVolSettingInactivityTimer(T_MFW_HND parent_win)
{
    T_MFW_HND win = mfw_parent(mfw_header());
    T_MFW_WIN * win_data = ((T_MFW_HDR *)win)->data;
    T_idle * idle_data = (T_idle *)win_data->user;
    //T_call * call_data = (T_call*)win_data->user;

    TRACE_EVENT(">>>> startRingerVolSettingInactivityTime, Create and Start Vol timer ");
    //Create 4 second ringer delay timer.
    if( hRingerVolSettingTimeOut == NULL )
    {
        if( call_data.call_direction == INCOMING )
        {
            hRingerVolSettingTimeOut = timCreate(call_data.win, TIM4SECOND, (MfwCb)volumeSettingTimeOut);
        }
        else
        {
            hRingerVolSettingTimeOut = timCreate(idle_data->win, TIM4SECOND, (MfwCb)volumeSettingTimeOut);
        }
    }

    if( hRingerVolSettingTimeOut != NULL )
        timStop( hRingerVolSettingTimeOut );

    timStart(hRingerVolSettingTimeOut);


}
// 2004.4.9 WYJ modify
#if 0
/*******************************************************************************

 $Function:     soundsRinger

 $Description:  prepares Ringer setting screen

 $Returns:    MFW_EVENT_CONSUMED always

 $Arguments:  m, menu, i, item

*******************************************************************************/

int soundsRinger( MfwMnu *m, MfwMnuItem *i )
{
    T_MFW_HND win;
    T_MFW_HND parent_win = mfw_parent(mfw_header());

    //recources_Init (win);

    //Create 0.5 second ringer delay timer.
    hRingerStartTimer = timCreate( 0, TIMHALFSECOND, (MfwCb)playCurrentSound);

    win = buildMelodyMenu(parent_win);

    if( win NEQ NULL )
    {
        SEND_EVENT (win, E_INIT, 0, 0);
    }

    // Start the 0.5 second ringer delay timer.
    if( soundReason!=SettingPHBKGrouping )
        delayThenPlayMelody();
    //  soundExec();
    driverFlag |= 0x04 ; //xsf add for KCD_LEFT 2003.01.14
    return MFW_EVENT_CONSUMED;
}


/*******************************************************************************

 $Function:     soundsVolume

 $Description:  prepares Volume setting screen

 $Returns:    MFW_EVENT_CONSUMED always

 $Arguments:  m, menu, i, item

*******************************************************************************/

int soundsVolume( MfwMnu *m, MfwMnuItem *i )
{

    return MFW_EVENT_CONSUMED;
}


/*******************************************************************************

 $Function:     settingsVibrator

 $Description:  prepares Vibrator settings screen

 $Returns:    MFW_EVENT_CONSUMED always

 $Arguments:  m, menu, i, item

*******************************************************************************/

int settingsVibrator( MfwMnu *m, MfwMnuItem *i )
{

    return MFW_EVENT_CONSUMED;
}
/*******************************************************************************

 $Function:     settingsLanguages

 $Description:  prepares languages screen

 $Returns:    MFW_EVENT_CONSUMED always

 $Arguments:  m, menu, i, item

*******************************************************************************/

int settingsLanguages( MfwMnu *m, MfwMnuItem *i )
{
    return MFW_EVENT_CONSUMED;
}

/*******************************************************************************

 $Function:     soundsNewMessage

 $Description:  prepares NewMessage setting screen

 $Returns:    MFW_EVENT_CONSUMED always

 $Arguments:  m, menu, i, item

*******************************************************************************/

int soundsNewMessage( MfwMnu *m, MfwMnuItem *i )
{

    return MFW_EVENT_CONSUMED;
}

/*******************************************************************************

 $Function:     soundsNewBroadcast

 $Description:  prepares Broadcast setting screen

 $Returns:    MFW_EVENT_CONSUMED always

 $Arguments:  m, menu, i, item

*******************************************************************************/

int soundsNewBroadcast( MfwMnu *m, MfwMnuItem *i )
{

    return MFW_EVENT_CONSUMED;
}

/*******************************************************************************

 $Function:     soundsBatteryLow

 $Description:  prepares BatteryLow setting screen

 $Returns:    MFW_EVENT_CONSUMED always

 $Arguments:  m, menu, i, item

*******************************************************************************/

int soundsBatteryLow( MfwMnu *m, MfwMnuItem *i )
{

    return MFW_EVENT_CONSUMED;
}


/*******************************************************************************

 $Function:     soundsCreditLow

 $Description:  prepares Credit setting screen

 $Returns:    MFW_EVENT_CONSUMED always

 $Arguments:  m, menu, i, item

*******************************************************************************/

int soundsCreditLow( MfwMnu *m, MfwMnuItem *i )
{

    return MFW_EVENT_CONSUMED;
}
/*******************************************************************************

 $Function:     soundsOrganiser

 $Description:  prepares Organiser setting screen

 $Returns:    MFW_EVENT_CONSUMED always

 $Arguments:  m, menu, i, item

*******************************************************************************/

int soundsOrganiser( MfwMnu *m, MfwMnuItem *i )
{

    return MFW_EVENT_CONSUMED;
}

/*******************************************************************************

 $Function:     soundsAlarm

 $Description:  prepares Alarm setting screen

 $Returns:    MFW_EVENT_CONSUMED always

 $Arguments:  m, menu, i, item

*******************************************************************************/

int soundsAlarm( MfwMnu *m, MfwMnuItem *i )
{

    return MFW_EVENT_CONSUMED;
}


/*******************************************************************************

 $Function:     soundInit

 $Description:  init sounds handling

 $Returns:    None

 $Arguments:  parent, handle of parent window

*******************************************************************************/

void soundInit( MfwHnd parent )
{


}


/*******************************************************************************

 $Function:     soundExit

 $Description:  exit sounds handling

 $Returns:    None

 $Arguments:  None

*******************************************************************************/

void soundExit (void)
{

}


/*******************************************************************************

 $Function:     soundExec

 $Description:  sends messages to soundExec to get the various melodies to play

 $Returns:    None

 $Arguments:  reason, for invocation, next, state

*******************************************************************************/

void soundExec (int reason, MmiState next)

{

}


/*******************************************************************************

 $Function:     soundsKeypadSilent

 $Description:  prepares KeypadTones setting screen

 $Returns:    MFW_EVENT_CONSUMED always

 $Arguments:  m, menu, i, item

*******************************************************************************/

int soundsKeypadSilent(MfwMnu* m, MfwMnuItem* i)
{

    T_MFW_HND win = mfw_parent(mfw_header());

    sounds_Init(win);

    if( FFS_flashData.settings_status & SettingsKeypadClick )
    {
        FFS_flashData.settings_status &= ~SettingsKeypadClick;
    }
    else if( FFS_flashData.settings_status & SettingsKeypadDTMF )
    {
        FFS_flashData.settings_status &= ~SettingsKeypadDTMF;
    }
    flash_write();
    sounds_data.local_status = ACTIVATE_CONFIRM;

    SEND_EVENT (sounds_data.sounds_win, SHOW_CONFIRM, 0, 0);

    return 0;
}
/*******************************************************************************

 $Function:     soundsKeypadClick

 $Description:  prepares KeypadTones setting screen

 $Returns:    MFW_EVENT_CONSUMED always

 $Arguments:  m, menu, i, item

*******************************************************************************/
int soundsKeypadClick(MfwMnu* m, MfwMnuItem* i)
{

    T_MFW_HND win = mfw_parent(mfw_header());

    sounds_Init(win);



    if( FFS_flashData.settings_status & SettingsKeypadDTMF )
    {
        FFS_flashData.settings_status &= ~SettingsKeypadDTMF;
    }

    FFS_flashData.settings_status |= SettingsKeypadClick;

    //play KeypadClick
    audio_PlaySoundID(0, TONES_KEYBEEP, 5, 0 );

    sounds_data.local_status = ACTIVATE_CONFIRM;

    SEND_EVENT (sounds_data.sounds_win, SHOW_CONFIRM, 0, 0);
    flash_write();
    return 0;
}

/*******************************************************************************

 $Function:     soundsKeypadDTMF

 $Description:  prepares KeypadTones setting screen

 $Returns:    MFW_EVENT_CONSUMED always

 $Arguments:  m, menu, i, item

*******************************************************************************/
int soundsKeypadDTMF(MfwMnu* m, MfwMnuItem* i)
{

    T_MFW_HND win = mfw_parent(mfw_header());

    sounds_Init(win);

    if( FFS_flashData.settings_status & SettingsKeypadClick )
    {
        FFS_flashData.settings_status &= ~SettingsKeypadClick;
    }

    // test
    //audio_SetAmplf (AUDIO_SPEAKER, 240);
    // test


    FFS_flashData.settings_status |= SettingsKeypadDTMF;
    flash_write();

    sounds_data.local_status = ACTIVATE_CONFIRM;

    SEND_EVENT (sounds_data.sounds_win, SHOW_CONFIRM, 0, 0);

    return 0;
}



/*******************************************************************************

 $Function:     sounds_Init

 $Description:

 $Returns:

 $Arguments:

*******************************************************************************/
T_MFW_HND sounds_Init (T_MFW_HND parent_window)
{
    return(sounds_create (parent_window));
}
/*******************************************************************************

 $Function:     confirm_Exit

 $Description:

 $Returns:

 $Arguments:

*******************************************************************************/
void sounds_Exit (T_MFW_HND own_window)
{

    T_MFW_WIN   * win_data = ((T_MFW_HDR *)own_window)->data;
    T_MELODY_INFO * data     = (T_MELODY_INFO *)win_data->user;

    sounds_destroy (data->sounds_win);
}

/*******************************************************************************

 $Function:     sounds_create

 $Description:

 $Returns:

 $Arguments:

*******************************************************************************/
T_MFW_HND sounds_create (T_MFW_HND parent_window)
{
    T_MELODY_INFO * data = &sounds_data;
    T_MFW_WIN   * win;

    TRACE_FUNCTION ("sounds_create()");

    data->sounds_win = win_create (parent_window, 0, 0, (T_MFW_CB)sounds_win_cb);

    if( data->sounds_win EQ 0 )
        return 0;

    /*
     * Create window handler
     */
    data->mmi_control.dialog = (T_DIALOG_FUNC)sounds;
    data->mmi_control.data   = data;
    win                      = ((T_MFW_HDR *)data->sounds_win)->data;
    win->user                = (void *) data;


    winShow(data->sounds_win);
    /*
     * return window handle
     */
    return data->sounds_win;
}

/*******************************************************************************

 $Function:     sounds_destroy

 $Description:

 $Returns:

 $Arguments:

*******************************************************************************/
void sounds_destroy (T_MFW_HND own_window)
{
    T_MFW_WIN   * win;
    T_MELODY_INFO * data;

    TRACE_FUNCTION ("sounds_destroy()");

    if( own_window )
    {
        win  = ((T_MFW_HDR *)own_window)->data;
        data = (T_MELODY_INFO *)win->user;

        if( data )
        {

            /*
             * Delete WIN handler
             */
            win_delete (data->sounds_win);
            data->sounds_win = 0;
        }
    }
}
/*******************************************************************************

 $Function:     sounds_win_cb

 $Description:

 $Returns:

 $Arguments:

*******************************************************************************/
static int sounds_win_cb (T_MFW_EVENT event, T_MFW_WIN * win)
{
    /*
     * Top Window has no output
     */
    return 1;
}


/*******************************************************************************

 $Function:     sounds_dialog

 $Description:

 $Returns:

 $Arguments:

*******************************************************************************/
static void sounds_dialog(void)
{
    T_DISPLAY_DATA display_info;

    TRACE_FUNCTION ("sounds_dialog()");

    display_info.KeyEvents    = KEY_CLEAR | KEY_RIGHT;

    /*************************zy change for show dialog with icon*********************/
    if( sounds_data.local_status EQ ACTIVATE_CONFIRM )
    {
        display_info.TextId   = TxtActivated;

    }
    else
    {
        display_info.TextId    = TxtDeActivated;
    }


    display_info.Callback     = (T_VOID_FUNC)sounds_dialog_cb;
    /*
     * Call Info Screen
     */
    bookShowInformationWithIcon( sounds_data.sounds_win, display_info.TextId, &Confirm, 0, display_info.Callback ,0);
    /*************************end of zy change for show dialog with icon*********************/
}


/*******************************************************************************

 $Function:     confirm_dialog_cb

 $Description:

 $Returns:

 $Arguments:

*******************************************************************************/
void sounds_dialog_cb(T_MFW_HND win, UBYTE identifier, UBYTE reason)
{
    TRACE_FUNCTION ("sounds_dialog_cb()");

    switch( reason )
    {
    case INFO_TIMEOUT:
        /* no break; */
    case INFO_KCD_HUP:
        /* no break; */
    case INFO_KCD_LEFT:
        /* no break; */
    case INFO_KCD_RIGHT:
        /* no break; */
    case INFO_KCD_CLEAR:
        sounds_Exit(win);
        break;
    }
}

/*******************************************************************************

 $Function:     sounds

 $Description:

 $Returns:

 $Arguments:

*******************************************************************************/
void sounds (T_MFW_HND win, USHORT event, SHORT value, void * parameter)
{
    T_MFW_WIN         * win_data = ((T_MFW_HDR *) win)->data;
    T_MELODY_INFO      * data     = (T_MELODY_INFO *)win_data->user;

    TRACE_FUNCTION("sounds()");

    switch( event )
    {
    case SHOW_CONFIRM:
        sounds_dialog();
        break;
    }

}
#endif
/*****************************************zym added*******************************************/
///zym for vibrator

/*******************************************************************************

 $Function:     StartVibrator

 $Description: start Vibrator

 $Returns:

 $Arguments:
*******************************************************************************/
#ifdef _INNOVATION_EMULATOR_
void StartVibrator()
{
}
#else
void StartVibrator(){
    if( FlashEnvironmentData.CurrentEnv == MUTE_ENV )  //xsf add 2003.01.13
        return;
#if 0
    SET_BIT(io_conf_reg_g,2);
    RESET_BIT(io_cntl_reg_g,6);
    SET_BIT(armio_latch_out_g,6);
#endif
    vibra_status = VIBRA_BUSY;
    MI_Vibrator_ForcedCtrl( MI_VBR_FORCED_ON );
}
#endif 

/*******************************************************************************

 $Function:     StopVibrator

 $Description: stop  Vibrator

 $Returns:

 $Arguments:
*******************************************************************************/
#ifdef _INNOVATION_EMULATOR_
void StopVibrator()
{  
}
#else
void StopVibrator(){
    //RESET_BIT(armio_latch_out_g,6);
    MI_Vibrator_ForcedCtrl( MI_VBR_FORCED_OFF );
    vibra_status = VIBRA_IDLE;
}
#endif


void KeepBeep_cb (void* parameter)
{

    //rvf_free_buf (parameter); 
    return;
}



///for keybeep
// 2004.4.9 wyj modify
#if 0
void StartKeyBeep(BOOLEANS keycode){  
    if( (FlashEnvironmentData.CurrentEnv == MUTE_ENV) || call_win_flag )  //wms added call_win_flag and driverFlag
        return;
    //add KCD_HUP/KCD_RIGHT for adjust volume window to play keytone 2003.02.20
    else if( ( (driverFlag&0x00000001)==0x00000001) && (keycode !=KCD_RIGHT) && (keycode !=KCD_HUP) )
        return;
    else if( ((keycode == KCD_MNUUP)||(keycode == KCD_MNUDOWN)) && (driverFlag&0x04) && (soundReason != SoundsNone) )  //xsf 03.07
        return;
    else
    {
        //if((driverFlag & 0x00000001) !=0x00000001){
#if 0  //glowing: comment the following block
        T_AUDIO_KEYBEEP_PARAMETER parameter;
        T_RV_RETURN return_path;
        //return;//ganchh 2002/8/27

        parameter.amplitude_beep[0] = -2;  //xsf change 0 to -2
        parameter.amplitude_beep[1] = -2;   //xsf change 0 to -2

        parameter.frequency_beep[0] = 1750;  //wms changed to 1750
        parameter.frequency_beep[1] = 1750; //wms changed to 1750

        parameter.duration = 110;//wms changed to 110

        return_path.callback_func = KeepBeep_cb;
        return_path.addr_id = 0;   
#endif

        if( (driverFlag & 0x00000008) ==0x00000000 ) //wms added the judgement to make sure of Earmode 03-01-20
            SetEarMode();
        /*if( (driverFlag | 0xFFFFFFEF) ==0xFFFFFFEF) {//wms judge the keybeep initial status back 5th bit
           SetASpk_DownLinkPGA(6);
           driverFlag |=0x00000010; //wms set keybeep intial  status 
        }*/

        //audio_keybeep_start (parameter, return_path);
        StartPlayingMelody2( KEYTONE_SINGLE, 1); //glowing: use the new api
    }
}
#endif

void StartKeyVoice(unsigned char keytone_type, unsigned char keycode) // glowing, 2003-12-11, play key voice
{
    //add KCD_HUP/KCD_RIGHT for adjust volume window to play keytone 2003.02.20
    if( ( (driverFlag&0x00000001)==0x00000001) && (keycode !=KCD_RIGHT) && (keycode !=KCD_HUP) )
        return;
    else if( ((keycode == KCD_MNUUP)||(keycode == KCD_MNUDOWN)) && (driverFlag&0x04) && (soundReason != SoundsNone) )  //xsf 03.07
        return;
    else
    {
        if( keytone_type==1 || (Mmi_getCurrentLanguage()==ENGLISH_LANGUAGE))	// 2004.4.13 WYJ not play voice in English
            StartPlayingMelody2( KEYTONE_SINGLE, 1); //glowing: use the new api
        else
        {
            switch( keycode )
            {
            case KCD_0:
            case KCD_1:
            case KCD_2:
            case KCD_3:
            case KCD_4:
            case KCD_5:
            case KCD_6:
            case KCD_7:
            case KCD_8:
            case KCD_9:
            case KCD_STAR:
            case KCD_HASH:
                StartPlayingMelody2( KEYTONE_VOICE0+keycode, 1); //glowing: use the new api
                break;
            default:
                StartPlayingMelody2( KEYTONE_SINGLE, 1); //glowing: use the new api
                break;
            }
        }
    }

}


void StopKeyBeep(void){
#if 0
    T_RV_RETURN return_path;
    return_path.callback_func = KeepBeep_cb;
    return_path.addr_id = 0;   

    audio_keybeep_stop (return_path);
#endif

    StopPlayingMelody2( ); //glowing: use the new api

}
void Tone_cb (void* parameter)
{

    //rvf_free_buf (parameter); 
    //tone_status = TONE_IDLE;
    return;
}
//wms moved this function from line 3538 to 2737
void StartTone_cb(void* parameter)
{
    TRACE_EVENT("StartTone_cb");
    if( (driverFlag & 0x00000008) ==0x00000000 )
    {//if current mode is MelodyMode ,wms added  03-01-22

        if( IsHandsetPlugin()&&call_data.win )    ///zhanym modied to fix the issue that mobile cann't warn in handset mode
            SetHandsetMode();
        else
            SetEarMode();

        //SetEarMode();//wms added for coming back to earmode
        //driverFlag&=0xFFFFFFEF;//wms set the keybeep initial status 03-01-22
    }
    CurrentMode = CurrentMode & (~0x020);      //ModeSms =0x020

    //rvf_free_buf (parameter); 
    tone_status = TONE_IDLE;
}

void StartStdTone(UBYTE toneValue){
    T_AUDIO_TONES_PARAMETER parameter;
    T_RV_RETURN return_path;
    if( FlashEnvironmentData.CurrentEnv == MUTE_ENV )  //xsf add 2003.01.13
        return;

#if 0
    SetMelodyMode(); 

    switch( toneValue )
    {
    case 1:{
            parameter.tones[0].start_tone    = 0;
            parameter.tones[0].stop_tone        = 160;
            parameter.tones[0].frequency_tone   = 500;      // Hz
            parameter.tones[0].amplitude_tone   = 0;     // dB
            parameter.tones[1].start_tone    = 160;
            parameter.tones[1].stop_tone     = 320;
            parameter.tones[1].frequency_tone   = 900;      // Hz
            parameter.tones[1].amplitude_tone   = 0;     // dB
            parameter.tones[2].start_tone    = 320;
            parameter.tones[2].stop_tone        =490;
            parameter.tones[2].frequency_tone   = 700;      // Hz
            parameter.tones[2].amplitude_tone   = 0   ;  // dB
            parameter.frame_duration            = 800;
            parameter.sequence_duration         = 800;
            parameter.period_duration        =800;
            parameter.repetition             = 3; //200;	
            break;
        }
    case 2:{
            parameter.tones[0].start_tone    = 0;
            parameter.tones[0].stop_tone        = 160;
            parameter.tones[0].frequency_tone   = 700;      // Hz
            parameter.tones[0].amplitude_tone   = 0;     // dB
            parameter.tones[1].start_tone    = 160;
            parameter.tones[1].stop_tone     = 320;
            parameter.tones[1].frequency_tone   = 1100;     // Hz
            parameter.tones[1].amplitude_tone   = 0;     // dB
            parameter.tones[2].start_tone    = 320;
            parameter.tones[2].stop_tone        =490;
            parameter.tones[2].frequency_tone   = 900;      // Hz
            parameter.tones[2].amplitude_tone   = 0   ;  // dB
            parameter.frame_duration            = 800;
            parameter.sequence_duration         = 800;
            parameter.period_duration        =800;
            parameter.repetition             = 3;  //200;	
        }
        break;
    case 3:{
            parameter.tones[0].start_tone    = 0;
            parameter.tones[0].stop_tone        = 110;
            parameter.tones[0].frequency_tone   = 750;      // Hz
            parameter.tones[0].amplitude_tone   = 0;     // dB
            parameter.tones[1].start_tone    = 110;
            parameter.tones[1].stop_tone     = 220;
            parameter.tones[1].frequency_tone   = 1000;     // Hz
            parameter.tones[1].amplitude_tone   = 0;     // dB
            parameter.tones[2].start_tone    = 220;
            parameter.tones[2].stop_tone        =330;
            parameter.tones[2].frequency_tone   =850;    // Hz
            parameter.tones[2].amplitude_tone   = 0   ;  // dB
            parameter.frame_duration            = 500;
            parameter.sequence_duration         = 500;
            parameter.period_duration        =500;
            parameter.repetition             = 3; //200;	
        }
        break;
    default:break;
    }

    return_path.callback_func = StartTone_cb; //wms modified
    return_path.addr_id = 0;   

    audio_tones_start (&parameter, return_path);
    //ton_playing = 1;
    tone_status = TONE_BUSY;
#endif
    StartPlayingMelody2(FULLHOUR_TONE,3); // glowing: 
}

void StartTone(void){
    T_AUDIO_TONES_PARAMETER parameter;
    T_RV_RETURN return_path;
    if( FlashEnvironmentData.CurrentEnv == MUTE_ENV )  //xsf add 2003.01.13
        return;
    parameter.tones[0].start_tone    = 0;
    parameter.tones[0].stop_tone        = 60;
    parameter.tones[0].frequency_tone   = 520;      // Hz
    parameter.tones[0].amplitude_tone   = 0;     // dB
    parameter.tones[1].start_tone    = 20;
    parameter.tones[1].stop_tone        = 80;
    parameter.tones[1].frequency_tone   = 775;      // Hz
    parameter.tones[1].amplitude_tone   = 0;     // dB
    parameter.tones[2].start_tone    = 100;
    parameter.tones[2].stop_tone        = 140;
    parameter.tones[2].frequency_tone   = 643;      // Hz
    parameter.tones[2].amplitude_tone   = 0   ;  // dB
    parameter.frame_duration            = 160;
    parameter.sequence_duration         = 320;
    parameter.period_duration        = 320;
    parameter.repetition             = TONE_INFINITE;  // infinite tones

    return_path.callback_func = StartTone_cb;
    return_path.addr_id = 0;   

    audio_tones_start (&parameter, return_path);
    //ton_playing = 1;
    tone_status = TONE_BUSY;
}


void StopTone(void){

    //   T_RV_RETURN return_path;
    //   return_path.callback_func = Tone_cb;
    //   return_path.addr_id = 0;   

    //audio_tones_stop (return_path);
    StopPlayingMelody2( );  //glowing: use the new api

    if( (driverFlag & 0x00000008) ==0x00000000 )
        SetEarMode();
    //driverFlag&=0xFFFFFFEF;//wms set the keybeep initial status
    tone_status = TONE_IDLE;
}

//***********************************Voice Memo************************************//
void StartVmRecord(callback_func cb_function,char * filename)
{
    T_AUDIO_VM_RECORD_PARAMETER record_parameter;
    T_AUDIO_TONES_PARAMETER tones_parameter;
    T_RV_RETURN return_path;


    strcpy(record_parameter.memo_name,filename);
    record_parameter.compression_mode = AUDIO_VM_COMPRESSION_MODE;
    record_parameter.memo_duration = 250;
    record_parameter.microphone_gain = 0x100;
    record_parameter.network_gain = 0x100;

    tones_parameter.tones[0].start_tone      = 0;
    tones_parameter.tones[0].stop_tone        = 60;
    tones_parameter.tones[0].frequency_tone   = 520;      // Hz
    tones_parameter.tones[0].amplitude_tone   = -24;      // dB
    tones_parameter.tones[1].start_tone    = 20;
    tones_parameter.tones[1].stop_tone        = 80;
    tones_parameter.tones[1].frequency_tone   = 775;      // Hz
    tones_parameter.tones[1].amplitude_tone   = -15;      // dB
    tones_parameter.tones[2].start_tone    = 100;
    tones_parameter.tones[2].stop_tone        = 140;
    tones_parameter.tones[2].frequency_tone   = 643;      // Hz
    tones_parameter.tones[2].amplitude_tone   = -6  ;  // dB
    tones_parameter.frame_duration            = 160;
    tones_parameter.sequence_duration         = 320;
    tones_parameter.period_duration        = 320;
    tones_parameter.repetition             = 1;  // only once

    return_path.callback_func = cb_function;
    return_path.addr_id = 0;   

    audio_vm_record_start (&record_parameter, &tones_parameter,return_path);
}


void StopVmRecord(callback_func cb_function)
{

    T_RV_RETURN return_path;
    return_path.callback_func = cb_function;
    return_path.addr_id = 0;   

    audio_vm_record_stop (return_path);
}

void StartPlayVmRecord(callback_func cb_founction,char * filename)
{
    T_AUDIO_VM_PLAY_PARAMETER parameter;
    T_RV_RETURN return_path;

    return_path.callback_func = cb_founction;
    return_path.addr_id = 0;   

    strcpy(parameter.memo_name,filename);  //test recognition  "/speech/srEnroll");       
    audio_vm_play_start (&parameter,return_path);
}

void StopPlayVmRecord(callback_func cb_founction)
{
    T_RV_RETURN return_path;
    return_path.callback_func = cb_founction;
    return_path.addr_id = 0;   

    audio_vm_play_stop (return_path);
}
//*******************************************************************************//

void StartSREnroll(void){
    T_AUDIO_SR_ENROLL_PARAMETER parameter;
    T_RV_RETURN return_path;

    strcpy(parameter.enroll_name,"/speech/srEnroll");
    parameter.record_speech = AUDIO_SR_RECORD_SPEECH;

    return_path.callback_func = SREnroll_cb;  
    return_path.addr_id = 0; 

    audio_sr_enroll_start( &parameter, return_path);

}

void StopSREnroll(void){
    T_RV_RETURN return_path;
    return_path.callback_func = SREnroll_cb;
    return_path.addr_id = 0; 

    audio_sr_enroll_stop (return_path);
}

void StartSRUpdate(void){

    T_AUDIO_SR_UPDATE_PARAMETER parameter;
    T_RV_RETURN return_path;

    strcpy(parameter.update_name,"/speech/srEnroll");
    return_path.callback_func = SRUpdate_cb;
    return_path.addr_id = 0; 

    parameter.record_speech = AUDIO_SR_RECORD_SPEECH;


    audio_sr_update_start (&parameter,return_path);
}

void StopSRUpdate(void){
    T_RV_RETURN return_path;


    return_path.callback_func = SRUpdate_cb;
    return_path.addr_id = 0; 

    audio_sr_update_stop (return_path);  
}

void StartSRReco(void)
{
    T_RV_RETURN return_path;
    T_AUDIO_SR_RECO_PARAMETER parameter;

    return_path.callback_func = SRReco_cb;
    return_path.addr_id = 0; 

    strcpy(parameter.database_directory,"/speech");

    audio_sr_reco_start (&parameter,return_path);  
}


void StopSRReco(void){
    T_RV_RETURN return_path;

    return_path.callback_func = SRReco_cb;
    return_path.addr_id = 0; 

    audio_sr_reco_stop (return_path);
}



void SREnroll_cb (void* parameter)
{

    T_AUDIO_SR_ENROLL_STATUS *pRslt;


    int i;

    char *pchar;

    char  str[AUDIO_PATH_NAME_MAX_SIZE];


    pRslt = (T_AUDIO_SR_ENROLL_STATUS*)parameter;

    MmiTrace("zym:SREnroll_cb");


    switch( pRslt->status )
    {
    case AUDIO_OK:
        {

            MmiTrace("zym:= AUDIO_OK");

            StartSRUpdate();

            break;
        }
    case -1:
        {
            MmiTrace("zym:= AUDIO_ERROR");
            StartSREnroll();
            break;
        }
    case -3:
        {
            MmiTrace("zym:= AUDIO_BAD_RECOGNITION");
            StartSREnroll();

            break;
        }
    case -4:
        {
            MmiTrace("zym:= AUDIO_BAD_RECOGNITION");
            StartSREnroll();
            break;
        }
    default:
        {
            break;
        }  
    }

    //rvf_free_buf (parameter); 
    return;
}



void SRUpdate_cb (void* parameter)
{

    T_AUDIO_SR_UPDATE_STATUS *pRslt;

    int i;

    char *pchar;

    char  str[AUDIO_PATH_NAME_MAX_SIZE];


    pRslt = (T_AUDIO_SR_UPDATE_STATUS*)parameter;

    switch( pRslt->status )
    {
    case AUDIO_OK:
        {
            break;
        }
    case -1:
        {
            StartSRUpdate();
            break;
        }
    case -3:
        {
            StartSRUpdate();

            break;
        }
    case -4:
        {
            StartSRUpdate();
            break;
        }
    case -5:
        {
            StartSRUpdate();
            break;
        }
    case -7:
        {
            StartSRUpdate();
            break;
        }
    case -8:
        {
            StartSRUpdate();
            break;
        }
    default:
        {
            break;
        }  
    }

    //rvf_free_buf (parameter); 
    return;
}

void PlayVmRecord_cb (void* parameter)
{

    //rvf_free_buf (parameter); 
    return;
}

void SRReco_cb (void* parameter)
{

    T_AUDIO_SR_RECO_STATUS *pRslt;

    T_AUDIO_SR_RECO_STATUS*    reco_status;
    T_AUDIO_VM_PLAY_PARAMETER vm_parameter;
    T_RV_RETURN return_path;

    int i;

    char *pchar;

    char  str[AUDIO_PATH_NAME_MAX_SIZE];

    return_path.callback_func = PlayVmRecord_cb;
    return_path.addr_id = 0;

    pRslt = (T_AUDIO_SR_RECO_STATUS*)parameter;

    //	MmiTrace("zym:SRReco_cb");


    switch( pRslt->status )
    {
    case AUDIO_OK:
        {


            pchar = (char*)(pRslt->word_recognized);

            strcpy(vm_parameter.memo_name,pchar);

            audio_vm_play_start (&vm_parameter,return_path);

            break;
        }
    default:
        {
            StartSRReco();
            break;
        } 
    }

    //rvf_free_buf (parameter); 
    return;

}

//added for adjust Melody Volume
void SetMVol_DownLinkPGA(UINT16 Amp)  //set the Music Volum downlink programmable gain amplifier
{
#if 0
    UINT16 status;
    status=SPIRead_ABB_Register(1,12)&0xFF00;// zy change 1030 must keep the Spk_V  be larger
    status=status|Amp;
    status=status|0x0020;//zy 11/01 make sound larger
    SPIWrite_ABB_Register(1,12,status);
#endif
    //AdjustSpeakerVolume(2, Amp%2);
#if 0
    INT8 gainValue;
    INT8 levelValue;
    INT8 filter; //wms added 02/12/24
    T_AUDIO_SPEAKER_LEVEL audioSpeakerLevel;


    switch( Amp )
    {
    /*
    case 0:
       gainValue=-6;
       levelValue=AUDIO_SPEAKER_VOLUME_MUTE;
       break;
    case 1:
       gainValue=-3;
       levelValue=AUDIO_SPEAKER_VOLUME_24dB ;
       break;
    case 2:
       gainValue=-5;
       levelValue=AUDIO_SPEAKER_VOLUME_18dB ;
       break;
    case 3:
       gainValue=-2;
       levelValue=AUDIO_SPEAKER_VOLUME_18dB ;
       break;
    case 4:
       gainValue=1;
       levelValue=AUDIO_SPEAKER_VOLUME_18dB ;
       break;
    case 5:
       gainValue=-3;
       levelValue=AUDIO_SPEAKER_VOLUME_12dB ;
       break;
    case 6:
       gainValue=0;
       levelValue=AUDIO_SPEAKER_VOLUME_12dB ;
       break;
    case 7:
       gainValue=4;
       levelValue=AUDIO_SPEAKER_VOLUME_12dB;
       break;
    case 8:
       gainValue=6;
       levelValue=AUDIO_SPEAKER_VOLUME_12dB;
       break;
    case 9:
       gainValue=3;
       levelValue=AUDIO_SPEAKER_VOLUME_6dB;
       break;
    case 10:
       gainValue=1;
       levelValue=AUDIO_SPEAKER_VOLUME_0dB;
       break;
    case 11:
       gainValue=4;
       levelValue=AUDIO_SPEAKER_VOLUME_0dB;
       break;
    case 12:
       gainValue=6;
       levelValue=AUDIO_SPEAKER_VOLUME_0dB ;
       break;
       */
    
    case 0:
        gainValue=-6;
        levelValue=AUDIO_SPEAKER_VOLUME_MUTE;
        break;

    case 1:
        gainValue=-5;
        levelValue=AUDIO_SPEAKER_VOLUME_18dB ;
        break;

    case 2:
        gainValue=1;
        levelValue=AUDIO_SPEAKER_VOLUME_18dB ;
        break;

    case 3:
        gainValue=0;
        levelValue=AUDIO_SPEAKER_VOLUME_12dB ;
        break;

    case 4:
        gainValue=6;
        levelValue=AUDIO_SPEAKER_VOLUME_12dB;
        break;

    case 5:
        gainValue=1;
        levelValue=AUDIO_SPEAKER_VOLUME_0dB;
        break;

    case 6:
        gainValue=6;
        levelValue=AUDIO_SPEAKER_VOLUME_0dB ;
        break;

    }
    audioSpeakerLevel.audio_speaker_level=levelValue;
    AudioFullAcessWrite(AUDIO_SPEAKER_GAIN,(void *)(&gainValue));
    AudioFullAcessWrite(AUDIO_SPEAKER_VOLUME_LEVEL ,(void *)(&audioSpeakerLevel));
    //AudioFullAcessWrite(AUDIO_SPEAKER_FILTER,(void*)(&filter));//wms added 02/12/24
    /*wms modified*/

#endif
    int result;
    UINT8 amptmp =(UINT8)Amp;

    //result = MI_Volume_Set( (UINT8)Amp );
    result = MI_Volume_Set( amptmp );   //2004.1.6 WYJ

    if( result != MI_MELODY_OK )
    {
        MMI_TRACE_EVENT(("glowing: SetMVol_DownLinkPGA-result:%d", result));
    }
    else
    {
        MMI_TRACE_EVENT(("glowing:SetMVol_DownLinkPGA-success! amptmp:%d", amptmp));
    }   


}  

void SetASpk_DownLinkPGA(UINT16 Amp)  //set the voice downlink programmable gain amplifier
{
#if 0
    UINT16 status;
    status=SPIRead_ABB_Register(1,12)&0xFF00;// zy change 1030 must keep the Spk_V  be larger
    status=status|Amp;
    status=status|0x0020;//zy 11/01 make sound larger
    SPIWrite_ABB_Register(1,12,status);
#endif
    //AdjustSpeakerVolume(2, Amp%2);
    INT8 gainValue;
    INT8 levelValue;
    T_AUDIO_SPEAKER_LEVEL audioSpeakerLevel;

    TRACE_FUNCTION("zhqdebug: SetASpk_DownLinkPGA");
    switch( Amp )
    {
    /*
    case 0:
       gainValue=-6;
       levelValue=AUDIO_SPEAKER_VOLUME_24dB ;
       break;
    case 1:
       gainValue=-4;
       levelValue=AUDIO_SPEAKER_VOLUME_24dB ;
       break;
    case 2:
       gainValue=-2;    //-26dbm
       levelValue=AUDIO_SPEAKER_VOLUME_24dB ;
       break;
    case 3:
       gainValue=-5;    //-23dbm
       levelValue=AUDIO_SPEAKER_VOLUME_18dB ;
       break;
    case 4:
       gainValue=-2;   //-20dbm
       levelValue=AUDIO_SPEAKER_VOLUME_18dB ;
       break;
    case 5:         
       gainValue=-5;   //-17dbm
       levelValue=AUDIO_SPEAKER_VOLUME_12dB ;
       break;
    case 6:         //-15dbm
       gainValue=-3;
       levelValue=AUDIO_SPEAKER_VOLUME_12dB ;
       //gainValue=0;
       //levelValue=AUDIO_SPEAKER_VOLUME_12dB ;
       break;
    case 7:      //-13dbm
       gainValue=-1;
       levelValue=AUDIO_SPEAKER_VOLUME_12dB;
       break;
    case 8:     //-11dbm
       gainValue=1;
       levelValue=AUDIO_SPEAKER_VOLUME_12dB;
       break;
    case 9:
       gainValue=-2;    //-8dbm
       levelValue=AUDIO_SPEAKER_VOLUME_6dB;
       break;
    case 10:         //-5dbm
       gainValue=1;
       levelValue=AUDIO_SPEAKER_VOLUME_6dB;
       break;
    case 11:         //-3dbm
       gainValue=3;
       levelValue=AUDIO_SPEAKER_VOLUME_6dB;
       break;
    case 12:   //0dbm
       gainValue=0;
       levelValue=AUDIO_SPEAKER_VOLUME_0dB ;
       break;
       */
    
    case 0://-16dbm
        gainValue=2;
        levelValue=AUDIO_SPEAKER_VOLUME_18dB ;
        break;

    case 1:
        gainValue=0;    //-12dbm
        levelValue=AUDIO_SPEAKER_VOLUME_12dB ;
        break;

    case 2:
        gainValue=4;   //-8dbm
        levelValue=AUDIO_SPEAKER_VOLUME_12dB ;
        break;

    case 3:         //-4dbm
        gainValue=-4;
        levelValue=AUDIO_SPEAKER_VOLUME_0dB ;
        //gainValue=0;
        //levelValue=AUDIO_SPEAKER_VOLUME_12dB ;
        break;

    case 4:     // 0dbm
        gainValue=0;
        levelValue=AUDIO_SPEAKER_VOLUME_0dB;
        break;

    case 5:         // 3dbm
        gainValue=3;
        levelValue=AUDIO_SPEAKER_VOLUME_0dB;
        break;

    case 6:   // 5dbm
        gainValue=5;
        levelValue=AUDIO_SPEAKER_VOLUME_0dB ;
        break;


    }
    audioSpeakerLevel.audio_speaker_level=levelValue;
    AudioFullAcessWrite(AUDIO_SPEAKER_GAIN,(void *)(&gainValue));
    AudioFullAcessWrite(AUDIO_SPEAKER_VOLUME_LEVEL ,(void *)(&audioSpeakerLevel));
    //AudioFullAcessWrite(AUDIO_SPEAKER_FILTER,(void*)(&filter));//wms added 02/12/24
    /*wms modified*/
}  

#ifdef _M188_
/* Robert.Chen added, 2004-06-08 */
void getVolSetting(UINT8* amp)
{
	UINT8	i;
	UINT16	vbdctrl;
	/* NOTE: to get correct result, this table must match to the SetASpk_DownLinkPGA() func's methonology */
	UINT8	value2lel[7] = {72/*-16*/, 6/*-12*/, 10/*-8*/, 34/*-4*/, 38/*0*/, 41/*3*/, 43/*5*/};
	
#ifndef _INNOVATION_EMULATOR_
	vbdctrl	= (UCHAR)SPIRead_ABB_Register(PAGE0, VBDR);
#endif
	vbdctrl	= (UBYTE)(vbdctrl & 0x7f);

	for(i = 0; i < 7; i ++)
	{
		if(value2lel[i] == (UINT8)vbdctrl)
			break;
	}

	if(i < 7)
		*amp = i;
	else
		*amp = 1;	/* default */
}
/* add end */
#endif

void SetASpk_UpLinkPGA(UINT16 Amp){
#if 0
    UINT16 status;
    status=SPIRead_ABB_Register(2,14)&0xFFF0;
    status=status|Amp;
    SPIWrite_ABB_Register(2,14,status);
#endif
    //wms modified
    INT8 microPhoneValue;
    microPhoneValue=Amp;
    AudioFullAcessWrite(AUDIO_MICROPHONE_GAIN,(void *)(&microPhoneValue));

}

//added  StartConnectedTone for the connected sound
//changed the name and  added new of the function for special tone during phone call 02/12/29
void StartSpecialTone(UBYTE toneValue){ 
    T_AUDIO_TONES_PARAMETER parameter;
    T_RV_RETURN return_path;
    if( FlashEnvironmentData.CurrentEnv == MUTE_ENV )  //xsf add 2003.01.13
        return;

    //glowing: use the new api 2003-12-01
    tone_status = TONE_IDLE;
    StartPlayingMelody2(SPECIAL_MELODY, 1);
}

//for sms tone,wms modified the perioid 2002,12,12
void StartTonebypara(UINT16 repeat){

    if( !call_win_flag )
    {//wms added the judegment to avoid tone during phone call
        T_AUDIO_TONES_PARAMETER parameter;
        T_RV_RETURN return_path;
        if( FlashEnvironmentData.CurrentEnv == MUTE_ENV )  //xsf add 2003.01.13
            return;

        tone_status = TONE_IDLE;  //glowing: add the flag
        StartPlayingMelody2( SMS_MELODY_NORMAL, 1);
    }

}

void AdjustSpk_VolUp(void)
{
    uint8* spkVol;
    spkVol=&(FlashEnvironmentData.EnviroProfile[FlashEnvironmentData.CurrentEnv].speakervolume);
    if( *spkVol<6 )
        *spkVol+=1;
    else
        *spkVol=6;

    // MmiTrace("WMS trace speakerVolum");
    // MmiTraceInt((uint8)(FlashEnvironmentData.EnviroProfile[FlashEnvironmentData.CurrentEnv].speakervolume));
    SetASpk_DownLinkPGA(*spkVol);
    AdjustEarVolumeCall ();
    /*wms modified*/
}
void AdjustSpkHandsetHallOff_VolUp(void)
{
    uint8* spkVol;
    spkVol=&(FlashEnvironmentData.EnviroProfile[FlashEnvironmentData.CurrentEnv].speakervolume);
    if( *spkVol<6 )
        *spkVol+=1;
    else
        *spkVol=6;
    SetASpk_DownLinkPGA(*spkVol);
    FlashEnvironmentData.EnviroProfile[FlashEnvironmentData.CurrentEnv].speakervolume=*spkVol;
    flash_environmentwrite();


}

void AdjustSpk_VolDown(void)
{
#if 0
    int CurrentVol;
    UINT16 status;
    status=SPIRead_ABB_Register(1,12)&0x000F;
    CurrentVol=(int)status;
    TraceInt(CurrentVol);
    if( CurrentVol>0 )
        CurrentVol-=2;
    else
        CurrentVol=0;
    SpeakerVols=CurrentVol;
    SetASpk_DownLinkPGA(CurrentVol);
    AdjustEarVolumeCall ();
    //xsf add 2002.09.04  2 lines
    // FlashEnvironmentData.EnviroProfile[FlashEnvironmentData.CurrentEnv].speakervolume = CurrentVol;
    // flash_environmentwrite();
#endif

    uint8* spkVol;
    spkVol=&(FlashEnvironmentData.EnviroProfile[FlashEnvironmentData.CurrentEnv].speakervolume);
    if( *spkVol>0 )
        *spkVol-=1;
    else
        *spkVol=0;
    //MmiTrace("WMS trace speakerVolum");
    //MmiTraceInt((uint8)(FlashEnvironmentData.EnviroProfile[FlashEnvironmentData.CurrentEnv].speakervolume));
    SetASpk_DownLinkPGA(*spkVol);
    AdjustEarVolumeCall ();

    /*wms modified*/
}
void AdjustSpkHandsetHallOff_VolDown(void)
{
    uint8* spkVol;
    spkVol=&(FlashEnvironmentData.EnviroProfile[FlashEnvironmentData.CurrentEnv].speakervolume);
    if( *spkVol>0 )
        *spkVol-=1;
    else
        *spkVol=0;
    MmiTrace("WMS trace speakerVolum");
    MmiTraceInt((uint8)(FlashEnvironmentData.EnviroProfile[FlashEnvironmentData.CurrentEnv].speakervolume));
    SetASpk_DownLinkPGA(*spkVol);
    FlashEnvironmentData.EnviroProfile[FlashEnvironmentData.CurrentEnv].speakervolume=*spkVol;
    flash_environmentwrite();
}


void SetMelodyMode(void){
    INT8 value;
    INT8 filter;//wms added 02/12/29
    value = AUDIO_SPEAKER_HANDHELD;//wms use macro define instead of 0
    filter=AUDIO_SPEAKER_FILTER_ON;
    driverFlag &=0xFFFFFFF7;//wms used driverflag 03-01-20
    //AudioFullAcessRead(6);
    //Cust_set_melodyfir_music();//wms modified the function
    AudioFullAcessWrite(AUDIO_SPEAKER_MODE ,(void *)(&value));//wms modified by using macro define
    AudioFullAcessWrite(AUDIO_SPEAKER_FILTER,(void*)(&filter));//wms added 02/12/24
    //LoadAudioMode("Melody");

}

void SetEarMode(void){
#if 0 
    UINT16 status;
    MmiTrace("SetMelodyMode");
    //Cust_set_melodyfir();
    status=SPIRead_ABB_Register(2,16)&0xfffc;//(PAGE1,VBCR)&0xfffd;
    MmiTraceInt(status);
    //status=(status|0x03);
    status=status|0x02; 
    //status=(status|0x02); XSF change 0x02 to 0x01 2002.10.28
    SPIWrite_ABB_Register(2,16,status);//(PAGE1,VBCR,status);
    SetASpk_DownLinkPGA(FlashEnvironmentData.EnviroProfile[FlashEnvironmentData.CurrentEnv].ringvolume);
#endif
    INT8 value;

	SetSpeakerMode(0); //Glowing,2004,08.11 Set Speaker work at receiver mode
	
    //value = AUDIO_SPEAKER_HANDFREE;//wms use macro define instead of 1
    value = AUDIO_SPEAKER_HANDHELD;
    driverFlag |=0x00000008;//wms used driverFlag 03-01-20
    //AudioFullAcessRead(6);
    Cust_set_voicefir();	
    AudioFullAcessWrite(AUDIO_SPEAKER_MODE,(void *)(&value));//wms modified by using macro define
    //LoadAudioMode("calling");
    value = AUDIO_MICROPHONE_HANDHELD;//wms use macro define instead of 0
    AudioFullAcessWrite(AUDIO_MICROPHONE_MODE,(void *)(&value));//wms modified by using macro define
    //SetASpk_DownLinkPGA(FlashEnvironmentData.EnviroProfile[FlashEnvironmentData.CurrentEnv].speakervolume);  //xsf add 2003.01.21
    SetASpk_DownLinkPGA(6);//wms added 03-01-22
}

//wms changed numbers into macro define
void SetHandsetMode(void){   
    INT8 value;
    value = AUDIO_SPEAKER_HEADSET;

    AudioFullAcessWrite(AUDIO_MICROPHONE_MODE,(void *)(&value));
    AudioFullAcessWrite(AUDIO_SPEAKER_MODE,(void *)(&value));
}

/* Robert.Chen added, 2004-06-08 */
void getAudioHrdMode(UINT8 *mode)
{
	T_AUDIO_FULL_ACCESS_READ para;

	para.data					= (void*)mode;
	para.variable_indentifier	= AUDIO_MICROPHONE_MODE;

	audio_full_access_read(&para);
}
/* end */

void  AudioFullAcessRead(int value){
    int i,  j ;
    T_AUDIO_FULL_ACCESS_READ para;
    T_AUDIO_VOICE_PATH_SETTING voicepath;


    para.variable_indentifier = value;

    switch( para.variable_indentifier )
    {
    case AUDIO_PATH_USED:
        {
            audio_full_access_read(&para);
            MmiTrace("wms test AUDIO_PATH_USED" );
            MmiTraceInt(*(T_AUDIO_VOICE_PATH_SETTING *)para.data);
            break;
        }
    case AUDIO_MICROPHONE_MODE:
        {
            audio_full_access_read(&para);
            MmiTrace("wms test AUDIO_MICROPHONE_MODE" );
            MmiTraceInt(*(INT8*)para.data);
            break;
        }
    case AUDIO_MICROPHONE_GAIN:
        {
            audio_full_access_read(&para);
            MmiTrace("wms test AUDIO_MICROPHONE_GAIN" );
            MmiTraceInt(*(INT8*)para.data);
            break;
        }
    case AUDIO_MICROPHONE_EXTRA_GAIN:
        {
            audio_full_access_read(&para);
            MmiTrace("wms test AUDIO_MICROPHONE_EXTRA_GAIN" );
            MmiTraceInt(*(INT8*)para.data);
            break;
        }
    case AUDIO_MICROPHONE_OUTPUT_BIAS:
        {
            audio_full_access_read(&para);
            MmiTrace("wms test AUDIO_MICROPHONE_OUTPUT_BIAS" );
            MmiTraceInt(*(INT8*)para.data);
            break;
        }
    case AUDIO_MICROPHONE_FIR:
        {
            audio_full_access_read(&para);
            MmiTrace("wms test AUDIO_MICROPHONE_FIR" );
            for( j = 0; j < 31 ; j ++ )
                MmiTraceInt((UINT16)(((T_AUDIO_FIR_COEF*)para.data)->coefficient[j]));

            break;
        }

    case AUDIO_SPEAKER_MODE:
        {
            audio_full_access_read(&para);
            MmiTrace("wms test  AUDIO_SPEAKER_MODE");
            MmiTraceInt(*(INT8* )(para.data));
            break;
        }

    case AUDIO_SPEAKER_GAIN:
        {
            audio_full_access_read(&para);
            MmiTrace("wms test AUDIO_SPEAKER_GAIN" );
            MmiTraceInt(*(INT8* )para.data);
            break;
        }
    case AUDIO_SPEAKER_FILTER:
        {
            audio_full_access_read(&para);
            MmiTrace("wms test AUDIO_SPEAKER_FILTER" );
            MmiTraceInt(*(INT8* )para.data);
            break;
        }
    case AUDIO_SPEAKER_FIR:
        {
            audio_full_access_read(&para);
            MmiTrace("wms test AUDIO_SPEAKER_FIR" );
            for( j = 0; j < 31 ; j ++ )
                MmiTraceInt((UINT16)(((T_AUDIO_FIR_COEF*)para.data)->coefficient[j]));
            break;
        }
    case AUDIO_SPEAKER_BUZZER_STATE:
        {
            audio_full_access_read(&para);
            MmiTrace("wms test AUDIO_SPEAKER_BUZZER_STATE" );
            MmiTraceInt(*(INT8* )para.data);
            break;
        }
    case AUDIO_MICROPHONE_SPEAKER_LOOP_SIDETONE:
        {
            audio_full_access_read(&para);
            MmiTrace("wms test AUDIO_MICROPHONE_SPEAKER_LOOP_SIDETONE" );
            MmiTraceInt(*(INT8* )para.data);
            break;
        }
    case AUDIO_MICROPHONE_SPEAKER_LOOP_AEC:
        {
            audio_full_access_read(&para);
            MmiTrace("wms test AUDIO_MICROPHONE_SPEAKER_LOOP_AEC" );
            MmiTraceInt((UINT16)(((T_AUDIO_AEC_CFG*)para.data)->aec_enable));
            MmiTraceInt((UINT16)(((T_AUDIO_AEC_CFG*)para.data)->aec_mode));
            MmiTraceInt((UINT16)(((T_AUDIO_AEC_CFG*)para.data)->echo_suppression_level));
            MmiTraceInt((UINT16)(((T_AUDIO_AEC_CFG*)para.data)->noise_suppression_enable));
            MmiTraceInt((UINT16)(((T_AUDIO_AEC_CFG*)para.data)->noise_suppression_level));
            break;
        }
    case AUDIO_SPEAKER_VOLUME_LEVEL:
        {
            audio_full_access_read(&para);
            MmiTrace("wms test AUDIO_SPEAKER_VOLUME_LEVEL" );
            MmiTraceInt((UINT8)(((T_AUDIO_SPEAKER_LEVEL*)para.data)->audio_speaker_level));
            break;
        }
    }


}


void AudioFullAcessWrite(UINT8 varid, void* ptr){
    T_AUDIO_FULL_ACCESS_WRITE  parameter;
    T_RV_RETURN                return_path;
    T_AUDIO_RET        tunevolumeresult ;
    TRACE_FUNCTION("zhqdebug: AudioFullAcessWrite");
    return_path.callback_func = NULL;
    return_path.addr_id = 0; 

    parameter.variable_indentifier = varid;
    parameter.data = ptr;
    // 2003/10/14, zhq, added return value to test whether audio_full_access_write() is executed correctly.
    tunevolumeresult = AUDIO_OK;
    tunevolumeresult = audio_full_access_write (&parameter,return_path);
    TRACE_EVENT_P1( "Zhq debug: Audio adjust return value: %d", tunevolumeresult);
}

void ledblinkEvent( )//UBYTE redflag, UBYTE greenflag)
{
    if( ledblinkflag==1 )
    {
        ledblinkflag=0;
        //if(redflag==1)
        //close green led
        Green_Off();
        //if(greenflag==1)
        //open red led
		/* 2004/03/15 sunsj remove */
        //Red_On();
    }
    else
    {
        ledblinkflag=1;
        //if(redflag==1)
        //open green led
        Green_On();
        //if(greenflag==1)
        //close red led
		/* 2004/03/15 sunsj remove */
        //Red_Off();
    }

    timStart(ledblink);
}
void ledblink_start(int nms)//UBYTE redflag, UBYTE greenflag, int nms)
{
    ledblink=timCreate(0,nms,(MfwCb)ledblinkEvent);    
    timStart(ledblink);
}
void ledblink_stop(void)
{
    timStop(ledblink);
    timDelete(ledblink);
}

#if 0
/*beging add 01-23-2003 for led incomingcall*/
void StartIncallLed()//begin incall led working mode
{
    //added 02/18
    if( pwr_Status.charging_state!=0 )
        Green_Off_incall();
    IncallLedisRun=TRUE;
    timStop(ledblink);
}
#endif

void IncallLed(BOOL IncallLedControl)//control led on or off
{
    if( IncallLedControl )
    {
        Green_On();
    }
    else
    {
        Green_Off();
    }
}
void stopIncallLed()//end incall led control mode
{
    IncallLedisRun=FALSE;
    Green_Off();
    timStart(ledblink);
}
/*end ofzy add 01-23 -2003 for led incomingcall*/
/* -----------chenjun add for user ring begin 2003-01-09------------------------*/
BOOL ReadUserRingInfo(T_USER_RING* Ringinfo)
{
    int res;   
    UBYTE i;
    Ringinfo->number=0;
    for( i=0;i<MAX_USER_RING_NUM;i++ )
    {
        memset(Ringinfo->filename[i],'\0' ,MAX_RING_NAME_LEN);
    }
    if( flash_CheckStat(UserRingIdStr)<0 )
    {
        TRACE_EVENT("chenjun:read userid failed");
        return FALSE;
    }
    else
    {
        res=ffs_fread(UserRingIdStr, Ringinfo, sizeof(T_USER_RING));    
        if( res<=0 )   //chenjun: ffs_fread error, read flash file failed,return
        {
            TRACE_EVENT("chenjun:read ring file string failed");
            return FALSE;
        }
        for( i=0;i<5;i++ )
            MmiTrace(Ringinfo->filename[i]);
    }
    return TRUE;
}

BOOL GetUserRingFileName(int index,char *filename) //chenjun 2003-01-09 user ring
{
    T_USER_RING *tUserRing=NULL;
    int res;
    if( flash_CheckStat(UserRingIdStr)<0 )
    {
        return FALSE;
    }
    else //chenjun,there has a index file of user ring in ffs
    {
        tUserRing=(T_USER_RING*) ALLOC_MEMORY(sizeof(T_USER_RING));
        res=ffs_fread(UserRingIdStr, tUserRing, sizeof(T_USER_RING));    
        if( res<=0 )   //chenjun: ffs_fread error, read flash file failed,return
        {

            FREE_MEMORY((void*)tUserRing,sizeof(T_USER_RING));
            return FALSE;
        }

        memcpy(filename,tUserRing->filename[index],MAX_RING_NAME_LEN);          
        FREE_MEMORY((void*)tUserRing,sizeof(T_USER_RING));
        TRACE_EVENT(filename);
        return TRUE;
    }
}
UBYTE GetUserRingNum()//chenjun 2003-01-09 user ring
{
    T_USER_RING *tUserRing=NULL;
    UBYTE number=0;
    int res;
    if( flash_CheckStat(UserRingIdStr)<0 )
    {
        return 0;
    }
    else  //chenjun,there has a index file of user ring in ffs
    {
        tUserRing=(T_USER_RING*) ALLOC_MEMORY(sizeof(T_USER_RING));
        res=ffs_fread(UserRingIdStr, tUserRing, sizeof(T_USER_RING));   
        if( res<=0 )   //chenjun: ffs_fread error, read flash file failed,return
        {
            FREE_MEMORY((void*)tUserRing,sizeof(T_USER_RING));
            return 0;

        }

        number=(UBYTE)(tUserRing->number);  
        FREE_MEMORY((void*)tUserRing,sizeof(T_USER_RING));
        return number;
    }
}

/******************************************
yq add for voice dial
******************************************/
void StartVDEnroll(char * source)
{
    T_AUDIO_SR_ENROLL_PARAMETER parameter;
    T_RV_RETURN return_path;

    strcpy(parameter.enroll_name,"/voiceD/");
    strcat(parameter.enroll_name,source);
    //memset(enrollname,0,sizeof(enrollname));
    if( strcmp(enrollname,source)!=0 )
        strcpy(enrollname,source);
    memset(current_vd_file,0,sizeof(current_vd_file));
    strcpy(current_vd_file,parameter.enroll_name);
    parameter.record_speech = AUDIO_SR_RECORD_SPEECH;

    return_path.callback_func = VDEnroll_cb;  
    return_path.addr_id = 0; 
    //YqTrace("audio_sr_enroll_start");
    //YqTrace((char *)source);
    audio_sr_enroll_start( &parameter, return_path);
}

static void VDEnroll_cb (void* parameter)
{

    T_AUDIO_SR_ENROLL_STATUS *pRslt;


    static int i=5;

    char *pchar;

    char  str[AUDIO_PATH_NAME_MAX_SIZE];


    pRslt = (T_AUDIO_SR_ENROLL_STATUS*)parameter;

    MmiTrace("VDEnroll_cb");
    switch( pRslt->status )
    {
    case AUDIO_OK:
        MmiTrace("OK");
        {
            i=5;
            if( winIsFocussed(voice_win) )
            {

                SEND_EVENT(voice_win,record_over,0,0);
                StartPlayVmRecord((callback_func)VdRecordPlayCB,(char *)current_vd_file);

            }
            else
            {
                if( voice_win )
                    SetCurrentMode(updateready);
            }

            break;
        }
    case -1:
        MmiTrace("error-1");
        {
            if( winIsFocussed(voice_win) )//current focus is voice_win
            {
                i--;
                if( i>0 )
                    StartVDEnroll(enrollname);
                else
                {
                    i=5;
                    SEND_EVENT(voice_win,record_error,0,0);
                }
            }
            else
            {
                i=5;
                if( voice_win );
                SetCurrentMode(recorderror);
            }
            break;
        }
    case -3:
    case -4:
    default:
        MmiTrace("error other");
        {
            i=5;
            if( winIsFocussed(voice_win) )//current focus is voice_win
            {
                SEND_EVENT(voice_win,record_error,0,0);
            }
            else
            {
                if( voice_win );
                SetCurrentMode(recorderror);
            }
            break;
        }  
    }

    rvf_free_buf (parameter); 
    return;
}


void StopVDEnroll(void)
{
    T_RV_RETURN return_path;
    return_path.callback_func = VDEnroll_cb;
    return_path.addr_id = 0; 

    audio_sr_enroll_stop (return_path);
}

void StartVDUpdate(){

    T_AUDIO_SR_UPDATE_PARAMETER parameter;
    T_RV_RETURN return_path;

    strcpy(parameter.update_name,current_vd_file);




    return_path.callback_func = VDUpdate_cb;
    return_path.addr_id = 0; 

    parameter.record_speech = AUDIO_SR_RECORD_SPEECH;
    audio_sr_update_start (&parameter,return_path);
}

void StopVDUpdate(void){
    T_RV_RETURN return_path;


    return_path.callback_func = VDUpdate_cb;
    return_path.addr_id = 0; 

    audio_sr_update_stop (return_path);  
}

static void VDUpdate_cb (void* parameter)
{

    T_AUDIO_SR_UPDATE_STATUS *pRslt;

    static int i=6;

    char *pchar;

    char  str[AUDIO_PATH_NAME_MAX_SIZE];


    pRslt = (T_AUDIO_SR_UPDATE_STATUS*)parameter;

    //YqTrace("zym:VDUpdate_cb");

    switch( pRslt->status )
    {
    case AUDIO_OK:
        {
            i=6;
            if( winIsFocussed(voice_win) )//current focus is voice_win
            {
                SEND_EVENT(voice_win,record_update_over,0,0);
                StartPlayVmRecord((callback_func)VdUpdatePlayCB,(char *)current_vd_file);
            }
            else
            {
                if( voice_win )
                    SetCurrentMode(recordsucess);
            }
            break;
        }
    case -1:
        {
            if( winIsFocussed(voice_win) )//current focus is voice_win
            {
                i--;
                if( i>0 )
                {
                    StartVDUpdate();
                }
                else
                {
                    i=6;
                    SEND_EVENT(voice_win,record_update_error,0,0);
                }
            }
            else
            {
                i=6;
                if( voice_win );
                SetCurrentMode(updateerror);
            }
            break;
        }
    case -3:
    case -4:
    case -5:
    case -7:
    case -8:
    default:
        {
            i=6;
            if( winIsFocussed(voice_win) )//current focus is voice_win
            {
                SEND_EVENT(voice_win,record_update_error,0,0);
            }
            else
            {
                if( voice_win )
                    SetCurrentMode(updateerror);
            }
            break;
        }  
    }

    rvf_free_buf (parameter); 
    return;
}


int StartVDReco(void)
{
    T_RV_RETURN return_path;
    T_AUDIO_SR_RECO_PARAMETER parameter;
    //YqTrace("startvdreco");
    return_path.callback_func = VDReco_cb;
    return_path.addr_id = 0; 

    strcpy(parameter.database_directory,"/voiceD");
    return(int)(audio_sr_reco_start (&parameter,return_path));  
}

void StopVDReco(void){
    T_RV_RETURN return_path;

    return_path.callback_func = VDReco_cb;
    return_path.addr_id = 0; 

    audio_sr_reco_stop (return_path);
}


static void VDReco_cb (void* parameter)
{

    T_AUDIO_SR_RECO_STATUS *pRslt;

    T_AUDIO_SR_RECO_STATUS*    reco_status;
    T_AUDIO_VM_PLAY_PARAMETER vm_parameter;
    T_RV_RETURN return_path;

    int i;

    char pchar[20];

    char  str[AUDIO_PATH_NAME_MAX_SIZE];

    return_path.callback_func = PlayVmRecord_cb;
    return_path.addr_id = 0;

    pRslt = (T_AUDIO_SR_RECO_STATUS*)parameter;

    //YqTrace("yq:VDReco_cb");
    rvf_free_buf (parameter);
    switch( pRslt->status )
    {
    case AUDIO_OK:
        {


            memcpy(pchar,(char *)(pRslt->word_recognized), 20);
            if( voice_win )
            {

                SEND_EVENT(voice_win,dial_ok,0,pchar);
            }
        }break;

    default:
        {

            if( voice_win )
                SEND_EVENT(voice_win,dial_error,0,0);

        } break;
    }



    return;

}

//the function for set enrollflag
void SetEnrollflag(unsigned char flag)
{
    startenroll=flag;
    return;
}

//the function for set updateflag
void SetUpdateflag(unsigned char flag)
{
    startupdate=flag;
    return;
}

//the fuction for set dialflag
void SetDialflag(unsigned char flag)
{
    startdial=flag;
    return;
}

void SetEnrollName(char * name)
{
    strcpy(enrollname,name);
    return;
}
/*******************************************
   end added
********************************************/

void StartPlayingMelody2( UBYTE melody_id, UBYTE count )
{
    int result;
    char temp[120];

    MMI_TRACE_EVENT(("glowing: StartPlayingMelody2 - enter melody_id:%d",melody_id ));

	SetSpeakerMode(1); //Glowing,2004,08.11 Set Speaker work at speaker mode


	if(melody_id >= MELODY_SUM )			//2004.3.29 WYJ add
	{
		MMI_TRACE_EVENT(("wyj: StartPlayingMelody2 - melody_id err"));
		return ;
	}

    result = MI_PlayMusic_Start(melody_id, count);

    if( result != MI_MELODY_OK )
    {
        MMI_TRACE_EVENT(("glowing: StartPlayingMelody2-result:%d", result));
		
		//2004.3.29 WYJ modify
		if( (result = MI_Melody_Initialize(MelInitForced)) != MI_MELODY_OK)
        	MMI_TRACE_EVENT( ("wyj: MI_Melody_Initialize(MelInitForced) - result=%d!!!!!!", result ));
		else	
		    MMI_TRACE_EVENT(("wyj: MI_Melody_Initialize(MelInitForced) OK"));
	}
    else
    {
         MMI_TRACE_EVENT(("glowing:StartPlayingMelody2-success! melody_id:%d",melody_id));
    }   
}

void StopPlayingMelody2( void )
{   
    int result;
    result = MI_PlayMusic_Stop( );

    MMI_TRACE_EVENT(("glowing: StopPlayingMelody2 - enter"));

    if( result != MI_MELODY_OK )
    {
        MMI_TRACE_EVENT(("glowing: StopPlayingMelody2-result:%d", result));
		
		//2004.3.29 WYJ modify
		if( (result = MI_Melody_Initialize(MelInitForced)) != MI_MELODY_OK)
        	MMI_TRACE_EVENT( ("wyj: MI_Melody_Initialize(MelInitForced) - result=%d!!!!!!", result ));
		else	
		    MMI_TRACE_EVENT(("wyj: MI_Melody_Initialize(MelInitForced) OK"));
	}
    else
    {
        MMI_TRACE_EVENT(("glowing:StopPlayingMelody2-success!"));
    }   
}

void Melody_Dev_Init( )
{
    int result;

    if( (result = MI_Melody_Initialize(MelInitNormal)) != MI_MELODY_OK )
        MMI_TRACE_EVENT( ("glowing: Melody_Dev_Init - result=%d!!!!!!", result ));
    else
        MMI_TRACE_EVENT(("glowing. Melody_Dev_Init OK"));

    if( (result = MI_Vibrator_SetMode(MI_VBR_SRC_FORCED, MI_VBR_BLK_DEFAULT ) )!= MI_MELODY_OK )
        MMI_TRACE_EVENT( ("glowing: Melody_Dev_Init - MI_Vibrator_SetMode result=%d!!!!!", result ));
    else
        MMI_TRACE_EVENT(("glowing: Melody_Dev_Init- MI_Vibrator_SetMode success"));

}

/* 2004/06/10 sunsj modify */
#ifdef SEVEN_COLORS_LED
extern MfwHnd ColorLedBlinkTimer;
extern MfwHnd ServiceLedTimer;
static int iColorLedBlinkLoop = 0;
/* 2004/06/09 sunsj modify */
void ColorLedBlinkEvent( MfwEvt e, MfwTim *tc )
{
	//static int iloop=0;
	timStop(ColorLedBlinkTimer);
	
	if( iColorLedBlinkLoop >= 100 )
	{
		switch( iColorLedBlinkLoop % 7 ){
		case 0: //RGB -- 010     
			Red_Off();
			Green2_On();      
			Blue_Off();
			break;
		case 1: //RGB-- 110
			Red_On();          
			break;
		case 2: //RGB -- 111
			Blue_On();           
			break;
		case 3: //RGB-- 011
			Red_Off();           
			break;
		case 4: //RGB -001
			Green2_Off();
			break;
		case 5: //RGB -101
			Red_On();
			break;
		case 6://RGB -100
			Blue_Off();
			break;
		}
		iColorLedBlinkLoop--;
		timStart( ColorLedBlinkTimer );
	}
	else
	{
    switch( iColorLedBlinkLoop % 7 ){
		case 0: //RGB -- 010     
			Red_Off();
			Green2_On();      
			Blue_Off();
			break;
		case 1: //RGB-- 110
			Red_On();          
			break;
		case 2: //RGB -- 111
			Blue_On();           
			break;
		case 3: //RGB-- 011
			Red_Off();           
			break;
		case 4: //RGB -001
			Green2_Off();
			break;
		case 5: //RGB -101
			Red_On();
			break;
		case 6://RGB -100
			Blue_Off();
			break;
    }
			
	iColorLedBlinkLoop++;
	if( iColorLedBlinkLoop < 7 )
	{
	    timStart(ColorLedBlinkTimer );
	}
	else
	{
		iColorLedBlinkLoop = 0;
		Red_Off();
		Green2_Off();
		Blue_Off();
		timStop(ColorLedBlinkTimer );
		if(ServiceLedTimer)
		{
			timStart(ServiceLedTimer);
		}
	}
}
}

/* 2004/06/10 sunsj modify */
extern MfwHnd ServiceLedDelay;
void ServiceLedDelayEvent( MfwEvt e, MfwTim *tc )
{
	timStop(ServiceLedDelay);
	Red_Off();
	Green2_Off();
	Blue_Off();
}

/* 2004/06/10 sunsj modify */
void ServiceLedEvent( MfwEvt e, MfwTim *tc )
{
	timStop(ServiceLedTimer);
	if(FlashSettingData.LedSwitch == 0)
	{
		return;		
	}
	
	if(g_nmService == NM_FULL_SERVICE)
	{
		SinglLedLight(BLUELED);
	}
	else if(g_nmService == E_NM_LIMITED_SERVICE || g_nmService == E_NM_NO_SERVICE)
	{
		SinglLedLight(REDLED);
	}
	timStart(ServiceLedDelay);
	timStart(ServiceLedTimer);
}
#endif


//glowing, 2004-05-22, import from mmisettings.c
/* 2004/06/09 sunsj modify */
void LEDStart(int loopflag)
{
#ifndef SEVEN_COLORS_LED
	Green_Off();
	Green_On();
#else
/* zhq, 2004/07/08, modified for we use MA5 to drive LED. */
#ifndef SEVEN_COLORS_LED_YAMAHA
    Blue_On();
    if(ServiceLedTimer)
    {
        timStop(ServiceLedTimer);
    }
	
    if(!loopflag)
    {
        iColorLedBlinkLoop = 0;
    }
    else
    {
        iColorLedBlinkLoop = 1000;
    }
    timStart(ColorLedBlinkTimer );
#endif //SEVEN_COLORS_LED_YAMAHA
#endif
}

void LEDOff()
{
#ifndef SEVEN_COLORS_LED
	Green_Off();
#else
/* zhq, 2004/07/08, modified for we use MA5 to drive LED. */
#ifndef SEVEN_COLORS_LED_YAMAHA
	if( ColorLedBlinkTimer ){
		timStop( ColorLedBlinkTimer );
		Red_Off();
		Green2_Off();
		Blue_Off();
	}
#endif //SEVEN_COLORS_LED_YAMAHA
#endif
}

/* 2004/06/10 sunsj modify */
#ifdef SEVEN_COLORS_LED
void SinglLedLight(int LedType)
{
	switch(LedType) 
{
	case REDLED:
		Red_On();
		break;
	case BLUELED:
		Blue_On();
		break;
	case GREENLED:
		Green2_On();
		break;
	default:
		break;
	}
}
#endif

/*******************************************************************************

                                End Of File

*******************************************************************************/

#if 0

void SetMelodyMode_aloud(void){//wms added the new function
#if 0
    UINT16 status;
    MmiTrace("SetEarMode");
    //Cust_set_voicefir();
    status=SPIRead_ABB_Register(2,16)&0xfffc;//(PAGE1,VBCR)&0xfffd;
    MmiTraceInt(status);
    status=status|0x02;
    //status=status|0x01;  XSF change 0x01 to 0x02
    SPIWrite_ABB_Register(2,16,status);//(PAGE1,VBCR,status);
    // SetASpk_DownLinkPGA(FlashEnvironmentData.EnviroProfile[FlashEnvironmentData.CurrentEnv].speakervolume);
#endif
    INT8 value;
    INT8 filter;//wms added 02/12/29
    value = AUDIO_SPEAKER_HANDHELD;//wms use macro define instead of 0
    filter=AUDIO_SPEAKER_FILTER_ON;
    driverFlag &=0xFFFFFFF7;//wms used driverflag 03-01-20
    //AudioFullAcessRead(6);
    //Cust_set_melodyfir_aloud();//wms modified the function
    Cust_set_melodyfir_music();
    //driverFlag |=0x00000004;//wms use the count backwoead 3rd bit to judge the mode, 1 for melodymode 03/01/14
    AudioFullAcessWrite(AUDIO_SPEAKER_MODE ,(void *)(&value));//wms modified by using macro define
    AudioFullAcessWrite(AUDIO_SPEAKER_FILTER,(void*)(&filter));//wms added 02/12/24
    //LoadAudioMode("Melody");

}


void MelodyStop_cb (void* parameter)
{
    //    TraceMMI("XSF0109:MelodyStop_cb()");
    melody_status = MELODY_BUSY; 
    if( StopMelodyAgain EQ 1 )  //xsf add for stop playing melody again when MelodyStop_cb() called 2003.03.03
    {
        StopMelodyAgain = 0;
        //StopMelody2(oldringstr);
        StopPlayingMelody2( ); //glowing: use the yamaha api
    }
    else
    {
        //melody_status = MELODY_IDLE; 
        StopMelodyAgain = 1;
    }
    //rvf_free_buf (parameter); 

    return;
}

void MelodyOld_cb (void* parameter)
{
    //    TraceMMI("XSF0305:MelodyOld_cb()");
    melody_status = MELODY_IDLE; 

    return;
}

void Melody_cb (void* parameter)
{
    T_AUDIO_MELODY_E2_PARAMETER temp;//chenjun add   
    T_RV_RETURN return_path;
    T_AUDIO_MELODY_E2_LOAD_FILE_INSTR_PARAMETER parameterNew;
    T_AUDIO_RET audio_ret;   //xsf add temporarily 2002.11.05

    //   TraceMMI("XSF0109:Melody_cb()");
    //rvf_free_buf (parameter); 
    if( waitToPlay )    //xsf add for melody wait to play 2003.02.27
    {
        //	TraceMMI("XSF0227:waitToPlay==1");
        if( findstr(gChMelody_Name) && (gChMelody_Name != 0) && (flash_CheckStat(gChMelody_Name)>=0) )
        {
            //   	TraceMMI("XSF0305:Melody_cb() PLAY");
            memset(oldringstr, '\0', 20); //xsf add 2002.09.06
            strcpy(oldringstr,gChMelody_Name);   
            strcpy(temp.melody_E2_name , gChMelody_Name); 
            temp.E2_loopback = 0;   //xsf  2002.09.06 change 1 to looping
            temp.melody_E2_mode = 1;
            return_path.callback_func = Melody_cb;  //call back for stop success or play one loop 
            return_path.addr_id = 0;
            strcpy(parameterNew.melody_E2_file_name,"/default");
            audio_ret = audio_melody_E2_load_file_instruments ( &parameterNew);
            if( audio_ret == AUDIO_OK )
            {
                if( waitToPlay==2 )
                    SetMelodyMode_aloud();   //aloud melody mode
                else
                    SetMelodyMode();     //normal melody mode
                SetMVol_DownLinkPGA(FlashEnvironmentData.EnviroProfile[FlashEnvironmentData.CurrentEnv].ringvolume);//wms added 03-04-07
                audio_melody_E2_start (&temp,return_path);  
                melody_status = MELODY_BUSY;
                waitToPlay = 0;
            }
        }
        else
            SetASpk_DownLinkPGA(6);   //xsf add for keybeep volume 2003.03.02
    }
    else   // no melody wait to play 
    {
        melody_status = MELODY_IDLE;
        if( gChMelody_Looping )  // played one loop,continue to play
        {
            //	   TraceMMI("XSF0109:gChMelody_Looping");
            strcpy(temp.melody_E2_name , gChMelody_Name); 
            //temp.E2_loopback = looping;   //xsf  2002.09.06 change 1 to looping
            temp.E2_loopback = 0;   //xsf  2002.09.06 change 1 to looping
            temp.melody_E2_mode = 1;
            return_path.callback_func = Melody_cb;   
            return_path.addr_id = 0;
            strcpy(parameterNew.melody_E2_file_name,"/default");
            //SetMelodyMode(); 
            audio_ret = audio_melody_E2_load_file_instruments ( &parameterNew);
            if( audio_ret == AUDIO_OK )
            {
                audio_melody_E2_start (&temp,return_path);  
                melody_status = MELODY_BUSY;
            }
        }
        else   //xsf add 2003.01.21
        {
            if( call_data.isHandSetMode==0 )//zy 12/12 judge to earmode or handsetmode if need audioplay in call win
            {
                SetEarMode();
                call_data.handSetOldStatus=0;
            }
            else
            {
                SetHandsetMode();
                call_data.handSetOldStatus=1;
            }
            CurrentMode = CurrentMode & (~0x020);    //ModeSms =0x020
        }
    }
    //driverFlag&=0xFFFFFFEF;//wms set the keybeep initial status
    return;
}


static int findstr(char * melody_name)
{
    int i;
    //chenjun add //chenjun add for user ring 2003-01-10 begin
    /*  char *tempName=(char*)ALLOC_MEMORY(20 );
       if(tempName==NULL)  
          {
          TRACE_EVENT("chenjun,tempName is NULL");
          return 0;
             }
    */  //chenjun 030214  revise memory leak
    char tempName[20];   //chenjun 030214  revise memory leak
    memset(tempName,0,strlen(melody_name));
    ////chenjun add for user ring 2003-01-10 over
    for( i=0;i<resource_GetListCount(RES_MELODY_STR);i++ )
    {
        if( !strcmp(melody_name, MelodyTablestr[i]) )
            return 1;
    }
    for( i=0;i<resource_GetListCount(RES_MELODY_SMS);i++ )
    {
        if( !strcmp(melody_name, SmsMelodyTablestr[i]) )
            return 1;
    }
    //following chenjun add for user ring

    for( i=0;i<GetUserRingNum();i++ )
    {
        GetUserRingFileName(i, tempName);
        if( !strcmp(melody_name, tempName) )
        {
            // 	  FREE_MEMORY((void*)tempName,20);//chenjun 030214  revise memory leak
            return 1;
        }
        //return 1;
    }
    //  FREE_MEMORY((void*)tempName,20); //chenjun 030214  revise memory leak
    return 0;
}//changed for user ring 2003-01-10


/*******************************************************************************

 $Function:     StartMelody2

 $Description:

 $Returns:

 $Arguments:

*******************************************************************************/
void StartMelody2(char * melody_name, BOOLEANS looping){
    T_AUDIO_MELODY_E2_PARAMETER temp;//chenjun add   
    T_RV_RETURN return_path;
    T_AUDIO_MELODY_E2_LOAD_FILE_INSTR_PARAMETER parameter;
    T_AUDIO_RET audio_ret;   //xsf add temporarily 2002.11.05
    int errorNum;
    //TRACE_EVENT(melody_name);
    if( FlashEnvironmentData.CurrentEnv == MUTE_ENV )  //xsf add 2003.01.13
        return;
    //   TraceMMI("XSF0305:StartMelody2()");
    gChMelody_Looping=looping;
    //if(looping)
    memset(gChMelody_Name, '\0', 20);
    strcpy(gChMelody_Name,melody_name);   //record the melody name to play
    if( (melody_status == MELODY_BUSY)||(melody_status == MELODY_STOPPING) )  //xsf add 2003.02.27
    {
        //   	 TraceMMI("XSF0305:StartMelody2() WAIT");
        if( melody_status == MELODY_BUSY )
            StopMelody2(oldringstr);
        waitToPlay = 1;
    }
    else
    {
        errorNum = flash_CheckStat(melody_name);
        if( findstr(melody_name) && (melody_name != 0) && (flash_CheckStat(melody_name)>=0) )
        {
            //   	TraceMMI("XSF0305:StartMelody2() PLAY");
            TRACE_EVENT("xsf:StartMelody2()");
            memset(oldringstr, '\0', 20); //xsf add 2002.09.06
            strncpy(oldringstr,melody_name,strlen(melody_name));   
            //Initial_AuxSpeaker();
            strcpy(temp.melody_E2_name , melody_name); 
            //temp.E2_loopback = looping;   //xsf  2002.09.06 change 1 to looping
            temp.E2_loopback = 0;   //xsf  2002.09.06 change 1 to looping
            temp.melody_E2_mode = 1;
            return_path.callback_func = Melody_cb;   
            return_path.addr_id = 0;

            strcpy(parameter.melody_E2_file_name,"/default");
            //SetMelodyMode(); //wms moved the function down 03-01-20
            audio_ret = audio_melody_E2_load_file_instruments ( &parameter);
            if( audio_ret == AUDIO_OK )
            {
                SetMelodyMode(); 
                audio_melody_E2_start (&temp,return_path);  
                melody_status = MELODY_BUSY;
            }
        }
        else
        {
            TRACE_EVENT("StartMelody2():Can't find melody name");
            //   	  TraceMMIValue_EAST("StartMelody2():Can't",errorNum);
            SetASpk_DownLinkPGA(6);   //xsf add for keybeep volume
        }
    }
}
/*******************************************************************************

 $Function:     StartMelody2_aloud

 $Description:  special for play loud melody

 $Returns:

 $Arguments:

*******************************************************************************/
void StartMelody2_aloud(char * melody_name, BOOLEANS looping){
    T_AUDIO_MELODY_E2_PARAMETER temp;//chenjun add   
    T_RV_RETURN return_path;
    T_AUDIO_MELODY_E2_LOAD_FILE_INSTR_PARAMETER parameter;
    T_AUDIO_RET audio_ret;   //xsf add temporarily 2002.11.05

    if( FlashEnvironmentData.CurrentEnv == MUTE_ENV )  //xsf add 2003.01.13
        return;
    //TraceMMI("XSF0305:StartMelody2_aloud()");
    gChMelody_Looping=looping;
    memset(gChMelody_Name, '\0', 20);
    strcpy(gChMelody_Name,melody_name);
    if( (melody_status == MELODY_BUSY)||(melody_status == MELODY_STOPPING) )  //xsf add 2003.02.27
    {
        //TraceMMI("XSF0305:StartMelody2_aloud() WAIT");
        if( melody_status == MELODY_BUSY )
            StopMelody2(oldringstr);
        waitToPlay = 2;  //aloud melody
    }
    else
    {
        if( findstr(melody_name) && (melody_name != 0) && (flash_CheckStat(melody_name)>=0) )
        {
            //TraceMMI("XSF0305:StartMelody2_aloud() PLAY");
            TRACE_EVENT("xsf:StartMelody2_aloud()");
            memset(oldringstr, '\0', 20); //xsf add 2002.09.06
            strncpy(oldringstr,melody_name,strlen(melody_name));   
            strcpy(temp.melody_E2_name , melody_name); 
            //temp.E2_loopback = looping;   //xsf  2002.09.06 change 1 to looping
            temp.E2_loopback = 0;   //xsf  2002.09.06 change 1 to looping
            temp.melody_E2_mode = 1;
            return_path.callback_func = Melody_cb;   
            return_path.addr_id = 0;

            strcpy(parameter.melody_E2_file_name,"/default");
            // SetMelodyMode_aloud();  wms moved the function down 03-01-20
            audio_ret = audio_melody_E2_load_file_instruments ( &parameter);
            if( audio_ret == AUDIO_OK )
            {
                SetMelodyMode_aloud(); 
                audio_melody_E2_start (&temp,return_path);  
                melody_status = MELODY_BUSY;
            }
        }
        else
        {
            TRACE_EVENT("StartMelody2_aloud():Can't find melody name");
            SetASpk_DownLinkPGA(6);   //xsf add for keybeep volume
        }
    }
}


void StopMelody2(char * melody_name){
    T_AUDIO_MELODY_E2_STOP_PARAMETER parameter;
    T_RV_RETURN return_path;

    TRACE_EVENT(melody_name);
    //TraceMMI("XSF0305:StopMelody2()");
    gChMelody_Looping=0;
    if( waitToPlay )  //now is stopping melody
        waitToPlay = 0;
    //else
    //{
    if( findstr(melody_name) && (melody_name != 0) && (flash_CheckStat(melody_name)>=0) )
    {
        TRACE_EVENT("xsf0305:StopMelody2() EXE");
        strcpy(parameter.melody_E2_name , melody_name);  //xsf change 2002.09.05
        return_path.callback_func = MelodyStop_cb;  //xsf call back for stop error  /stop success will call Melody_cb() 
        return_path.addr_id = 0;
        melody_status = MELODY_STOPPING;  //melody is stopping 
        audio_melody_E2_stop (&parameter, return_path);
        //melody_status = MELODY_IDLE;
        //driverFlag&=0xFFFFFFEF;//wms set the keybeep initial status
        //SetMVol_DownLinkPGA(0);//wms addded to make melody slient when stopping 03-04-07
        if( call_data.isHandSetMode==0 )//wms added back to avoid a delay in stop 03-04-04
            SetEarMode();
        else
            SetHandsetMode();
        SetMVol_DownLinkPGA(0);
    }
    else
        TRACE_EVENT("StopMelody2():Can't find melody name");
    //}
}

///for melody2

void StartMelody(BOOLEANS looping)
{
    T_AUDIO_MELODY_E2_PARAMETER temp;//chenjun add   
    T_RV_RETURN return_path;
    T_AUDIO_MELODY_E2_LOAD_FILE_INSTR_PARAMETER parameter;

    //Initial_AuxSpeaker();

    strcpy(temp.melody_E2_name , "/meloe2/e2/music1"); //"/bach");
    temp.E2_loopback = looping;  //xsf  2002.08.14 change 1 to loop
    temp.melody_E2_mode = 1;
    return_path.callback_func = MelodyOld_cb;   
    return_path.addr_id = 0;

    strcpy(parameter.melody_E2_file_name,"/default");

    audio_melody_E2_load_file_instruments ( &parameter);
    SetMelodyMode(); 
    audio_melody_E2_start (& temp,return_path);   
    melody_status = MELODY_BUSY;
}


void StopMelody(void){
    T_AUDIO_MELODY_E2_STOP_PARAMETER parameter;
    T_RV_RETURN return_path;
    return_path.callback_func = MelodyOld_cb;   
    return_path.addr_id = 0;

    strcpy(parameter.melody_E2_name , "/meloe2/e2/music1"); //"/bach");

    audio_melody_E2_stop (&parameter, return_path);
    //l_playing = 0;
    SetEarMode();
    melody_status = MELODY_IDLE;
}


void SetSpk_UplinkSLT(UINT16 Amp){//for echo, use function full_access no use 
#if 0
    UINT16 status;
    status=SPIRead_ABB_Register(2,14)&0xFE1F;
    status=status|(Amp<<5);
    SPIWrite_ABB_Register(2,14,status);
#endif
}

void LoadAudioMode(char* str){
    T_RV_RETURN return_path;
    T_AUDIO_MODE_LOAD  para;

    MmiTrace("LoadAudioMode");


    return_path.callback_func = NULL;
    return_path.addr_id = 0; 

    strcpy(para.audio_mode_filename,str);

    audio_mode_load(&para,return_path);
}

void SaveAudioMode(char* str){
    T_RV_RETURN return_path;
    T_AUDIO_MODE_SAVE  para;

    MmiTrace("SaveAudioMode");

    return_path.callback_func = NULL;
    return_path.addr_id = 0; 

    strcpy(para.audio_mode_filename,str);

    audio_mode_save(&para, return_path);

}


#endif

