/*******************************************************************************

          CONDAT (UK)

********************************************************************************

 This software product is the property of Condat (UK) Ltd and may not be
 disclosed to any third party without the express permission of the owner.

********************************************************************************

 $Project name: Basic MMI
 $Project code: BMI (6349)
 $Module:   Sounds
 $File:       MmiSounds.c
 $Revision:   1.0

 $Author:   Condat(UK)
 $Date:       25/10/00

********************************************************************************

 Description:

    This provides the implementation of the sounds for the basic MMI

    1. Playing various system sounds such as the paging ring,
       new sms message etc.
    2. The menus used to enable or disable the system sounds
    3. The menu used to select the paging ring tone
    4. The menu used to select the volume.
    5. The menu used to select the key press tone or dtmf

  The mmi should play system sounds by calling soundExec with the appropriate
  SoundsXXXX message it is possible to go directly to the audio driver to play
  a sound but this will bypass any user settings to turn sounds on or off.

  The MelodySelect and SettingXXXX messages are used to control the key events
  during menu handling, entry to the sounds menus is through the soundsXXXX
  functions. These display the appropriate menu and handle the keypad events.

  The settings are read from PCM on startup and saved whenever a setting menu
  is selected.

********************************************************************************
 $History: MmiSounds.c

  25/10/00      Original Condat(UK) BMI version.

 $End

*******************************************************************************/
/********************************************************************************
 $History: MmiSounds.c

 04/08/02     zym   added audio funcion 
  22/07/02      zy Mobile Innovation make the sounds dialog has icon
  12/08/02    zym add 12/08/02 for control voice ampifier
  15/08/02    zy add hallonoff and speak volume
 $End

*******************************************************************************/

/*******************************************************************************

                                Include Files

*******************************************************************************/

#define ENTITY_MFW

/* includes */
#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#if defined (NEW_FRAME)

#include "typedefs.h"
#include "vsi.h"
#include "pei.h"
#include "custom.h"
#include "gsm.h"
///added by zym
#include "rv_general.h"
#include "Audio_api.h"


#else

#include "stddefs.h"
#include "custom.h"
#include "gsm.h"
#include "vsi.h"

#endif
#include "mfw_sys.h"

#include "mfw_mfw.h"
#include "mfw_kbd.h"
#include "mfw_tim.h"
#include "mfw_lng.h"
#include "mfw_win.h"
#include "mfw_icn.h"
#include "mfw_sim.h"
#include "mfw_nm.h"
#include "mfw_edt.h"
#include "mfw_mnu.h"
#include "mfw_phb.h"
#include "mfw_sms.h"

#include "dspl.h"


#include "p_mmi.h"
#include "aci_cmh.h"
#include "mfw_mme.h"

#include "MmiMmi.h"

#include "MmiIcons.h"
#include "MmiPins.h"
#include "MmiMain.h"
#include "MmiDialogs.h"
#include "MmiIdle.h"
#include "MmiStart.h"
#include "MmiCall.h"
#include "MmiSimToolkit.h"
#include "gdi.h"
#include "Audio.h"
#include "MmiLists.h"
#include "MmiSounds.h"
#include "MmiResources.h"
#include "MmiTimeDate.h"

#include "cus_aci.h"
#include "p_sim.h"
#include "pcm.h"
#include "MmiTimers.h"



extern char* MelodyTable[];
extern T_call call_data;
/*******************************************************************************

                                Constant Definitions

*******************************************************************************/


/* Control FLUSH mechanism to PCM
*/
#define FLUSH 1
#define NO_FLUSH 0


/* Timer constants, pretty self explanatory
*/
#define TIMHALFSECOND 500
#define TIMSECOND   1000
#define TIM4SECOND      4000

#define VIBRATOR_TIME 2000  /* two second timer on the vibrator


/* Define a generic trace mechanism
*/
#define PTRACE(prn) { char buf[64]; ## prn ##   TRACE_EVENT(buf); }


/* Define maximum counter for vibrator
*/
#define VIBRATOR_COUNTER_MAX  4
#define NOT_VIBRATING     0


/* define an invalid tone marker
*/
#define INVALID_TONE  -1
//zym for vibrator

#define SET_BIT(REG,BIT)   (*((volatile unsigned short *) (REG)) |= (1 << (BIT)))
#define RESET_BIT(REG,BIT) (*((volatile unsigned short *) (REG)) &= ~(1 << (BIT)))
#define io_conf_reg_g       (0x0fffef00a)
#define io_cntl_reg_g        (0x0fffe4804)
#define armio_latch_out_g   (0x0fffe4802)
#define armio_latch_out_g   (0x0fffe4802)




/*******************************************************************************

                            Local Function prototypes

*******************************************************************************/

static int winEvent( MfwEvt e, MfwWin *w );
static int keyEvent( MfwEvt e, MfwKbd *kc );
static int keyEventLong( MfwEvt e, MfwKbd *kc );

static void ( *show )( void );

static int playCurrentSound( MfwEvt e, MfwTim *tc );
static int getCallingMenu( MfwEvt e, MfwTim *tc );
static int setSelection( MfwEvt e, MfwWin *w );

static void delayThenPlayMelody( void );
static void stopPlayingMelody( void );

static void readSettingsFromPCM( void );
static void writeSettingsToPCM( UBYTE flush );

static void soundsCreate( MfwHnd parent );
static int VibratorTimerEvent( MfwEvt e, MfwTim *t );


void sounds (T_MFW_HND win, USHORT event, SHORT value, void * parameter);
void sounds_dialog_cb(T_MFW_HND win, UBYTE identifier, UBYTE reason);
static void sounds_dialog(void);
static int sounds_win_cb (T_MFW_EVENT event, T_MFW_WIN * win);
static int melody_win_cb (MfwEvt e, MfwWin *w);
void melody_exec_cb (T_MFW_HND win, USHORT event, SHORT value, void * parameter);
void sounds_destroy (T_MFW_HND own_window);
T_MFW_HND sounds_create (T_MFW_HND parent_window);
void sounds_Exit (T_MFW_HND own_window);
T_MFW_HND sounds_Init (T_MFW_HND parent_window);
void MelodyList_cb(T_MFW_HND * Parent, ListMenuData * ListData);
static void melody_destroy(MfwHnd own_window);

///zym
void Initial_AuxSpeaker(void);

void SRReco_cb (void* parameter);
void SREnroll_cb (void* parameter);
void SRUpdate_cb (void* parameter);
#define SHOW_CONFIRM          3
#define ACTIVATE_CONFIRM       4
#define DEACTIVATE_CONFIRM     5
///zym
#define zymTOGBR1 (0x04)
#define zymTOGBR2 (0x05)
#define zymVBCR    (0x08)
#define zymVBUR    (0x07)
#define zymVBDR    (0x06)


/*******************************************************************************

                                Local typedefs

*******************************************************************************/

/* Define the types of volume we cater for
*/
typedef enum {
   silent = (UBYTE)0,
   volLow,
   volMedium,
   volHigh,
   increasing
} VOL_ENUM;


/* And the key tones
*/
typedef enum
{
  KEYTONE_NONE,
  KEYTONE_BEEP,
  KEYTONE_DTMF
} SoundsKeyTones;


/* The following definition provides a structure to control the
   ringing operation
*/

typedef struct MMI_CURRENT_RINGER_STRUCT {
                      /* variable to browse a list */
  UBYTE ringer;             // index of permanent active tune in soundlist and ringerItem
  UBYTE vibrator;             // vibrator
  UBYTE volumeSetting;          // values (0 = silent, 4= loud, 5 = increasing)
  UBYTE keypadOn;             // on/off values
  UBYTE AlarmOn;
  UBYTE OrganiserAlert;
  UBYTE CreditLow;
  UBYTE SMSTone;
  UBYTE SMSBroadcast;
  UBYTE battLow;
  UBYTE earpiece;
  UBYTE language;
  UBYTE Equalizer;
} RINGER_STRUCT_BROWSER;

/*  ------------- Variables ---------------   */

static SoundSettingState soundReason=SoundsNone;
// Change behavior of menu scroll up/down, meni item selected on scrolling.
static BOOL scrollSelectMenuItem= FALSE;
static BOOL volumeSetting2Pcm =FALSE;
static RINGER_STRUCT_BROWSER current =
{ // default settings.
  AUDIO_MEL_TUNES,  // ringer
  0,          // vibrator - off by default
  volMedium,      // volumeSetting
  2,          // keypadOn dtmf
  1,          // AlarmOn
  1,          // OrganiserAlert
  1,          // CreditLow
  1,          // SMSTone
  1,          // SMSBroadcast
  1,          // battLow
  3,          // earpiece
  1,          // language
  1         // Equalizer
};

typedef enum
{
    E_INIT = 0x0801,
    E_BACK,
    E_ABORT,
    E_EXIT,
  E_RETURN,
  E_OK
} e_M_events;

T_MELODY_INFO sounds_data;



/*******************************************************************************

                                Module local variables

*******************************************************************************/

static UBYTE oldVolumeSetting;     /* Previous setting of the volume      */
static MfwHnd win;                     /* our window              */
static MfwHnd kbd;                     /* our keyboard handler          */
static MfwHnd kbdLong;                 /* our keyboard handler for Long pressed    */
static MfwHnd hRingerStartTimer;         /* timeout till start of melody    */
static MfwHnd hRingerVolSettingTimeOut;   /* 4 second timeout for volume setting menu */

static MfwHnd hWinDispTimer;     /* timer to show result windows      */

/* Maintain vibrator state information here
*/
static MfwHnd VibratorTimer;
static UBYTE  Vibrating = NOT_VIBRATING;
static UBYTE  vibrator_counter=0;

/* General state information
*/
static MmiState nextState;
static MfwHnd lastFocus;

/* Define a default state for the ringer, this will control the
   power up state for the ringer and will be modified as required
   during normal operation
*/
//static RINGER_STRUCT_BROWSER current =
//{
//nm  AUDIO_MEL_TUNES,  // ringer
//  0,          // vibrator
//nm  volMedium,      // volumeSetting
//  2,          // keypadOn dtmf
//  1,          // AlarmOn
//  1,          // OrganiserAlert
//  1,          // CreditLow
//  1,          // SMSTone
//  1,          // SMSBroadcast
//  1,          // battLow
//  3         // earpiece
//};


/* PCM Related control structures
*/
//static drv_Return_Type pcmResult;
//static pcm_FileInfo_Type pcmFileInfo;


/* Tune details
*/
static int CurrentTune = 0;
static int LastTune = 0;


/* Menu definitions
*/
//static ST_MENU on_off_menu ;
//static ST_MENU dynamenu;
//static ST_MENU volume_menu;

//static ST_MENU_ITEM ONOFF_MENU_ITEMS[] = {
//  { 0, ST_PROMPT_MENU, TxtOff  },
//  { 1, ST_PROMPT_MENU, TxtOn   },
//  { 2, ST_PROMPT_MENU, TxtNull },
//};

//nm static ST_MENU_ITEM VOLUME_MENU_ITEMS[] = {
//nm  { silent,   ST_PICTURE_MENU, VOLUME_ICON_SILENT   },
//nm  { volLow,   ST_PICTURE_MENU, VOLUME_ICON_LOW    },
//nm  { volMedium,  ST_PICTURE_MENU, VOLUME_ICON_MEDIUM   },
//nm  { volHigh,    ST_PICTURE_MENU, VOLUME_ICON_HIGH   },
//nm  { increasing, ST_PICTURE_MENU, VOLUME_ICON_INCREASING }
//nm};


/* Provide a mapping between keys and their associated DTMF tones
*/
//nmstatic const keytonemap[][2] = {
//nm  { KEY_0,       TONES_DTMF_0    },
//nm  { KEY_1,       TONES_DTMF_1    },
//nm  { KEY_2,       TONES_DTMF_2    },
//nm  { KEY_3,       TONES_DTMF_3    },
//nm  { KEY_4,       TONES_DTMF_4    },
//nm  { KEY_5,       TONES_DTMF_5    },
//nm  { KEY_6,       TONES_DTMF_6    },
//nm  { KEY_7,       TONES_DTMF_7    },
//nm  { KEY_8,       TONES_DTMF_8    },
//nm  { KEY_9,       TONES_DTMF_9    },
//nm  { KEY_STAR,    TONES_DTMF_STAR },
//nm  { KEY_HASH,    TONES_DTMF_HASH },
//nm  { KEY_MNUUP,   TONES_DTMF_A    },
//nm  { KEY_MNUDOWN, TONES_DTMF_A    },
//nm  { KEY_LEFT,    TONES_DTMF_B    },
//nm  { KEY_RIGHT,   TONES_DTMF_C    },
//nm  { KEY_CLEAR,   TONES_DTMF_D    },
//nm  { 0,           INVALID_TONE    }
//nm};
//nm
static MfwMnuAttr melody_menuAttrib =
{
    &melody_menuArea,
    MNU_LEFT | MNU_LIST | MNU_CUR_LINE, /* centered page menu       */
    -1,                                 /* use default font         */
    NULL,                               /* with these items         */
    0                                   /* number of items     */
};


/*******************************************************************************

                                Private Methods

*******************************************************************************/
USHORT getSoundsReason(void)
{
  return soundReason;
}
/*******************************************************************************

 $Function:     setSoundsReason

 $Description:  Set the reason why the volume settings was invoked. Called to indicated
        that the volume settings is being invoked from Idle or from the incoming
        call screens.

 $Returns:    none

 $Arguments:  none

*******************************************************************************/
void setSoundsReason(USHORT reason)
{

  switch(reason)
  {
   case SettingVolume:
    soundReason = SettingVolume;
    break;
   case SoundsNone:
   default:
    soundReason = SoundsNone;
    break;
  }
}
/*******************************************************************************

 $Function:     volMenuItemSelected

 $Description:  A volume menu item has been selected, store the setting into PCM.

 $Returns:    none

 $Arguments:  none

*******************************************************************************/
void volMenuItemSelected(void)
{
  volumeSetting2Pcm =TRUE;
}
/*******************************************************************************

 $Function:     getScrollSelectMenuItem

 $Description:  get the scroll/select menu status.

 $Returns:    Status

 $Arguments:  none

*******************************************************************************/
BOOL getScrollSelectMenuItem(void)
{
  if (scrollSelectMenuItem == TRUE)
    return TRUE;
  else
    return FALSE;
}
/*******************************************************************************

 $Function:     clearScrollSelectMenuItem

 $Description:  Reset the scroll/select menu behavior.

 $Returns:    none

 $Arguments:  none

*******************************************************************************/
void clearScrollSelectMenuItem(void)
{
  scrollSelectMenuItem=FALSE;
  // soundReason = SoundsNone; MZ this flag cleared when user exits the melody/Volume menu.
  stopPlayingMelody();
}

/*******************************************************************************

 $Function:     soundsSetVolume

 $Description:  Function determines if user is in the Melody/SMS/Alarm ringer menu
        and has selected the vloume settings option.

 $Returns:    none

 $Arguments:  none

*******************************************************************************/
void soundsSetVolume(void)
{
  if(soundReason == MelodySelect || soundReason == SettingSMSTone ||
    soundReason == SettingAlarm || soundReason == SettingVolume)
  {
    // Menu items will be selected on scrolling.
    scrollSelectMenuItem = TRUE;
    // store the setting to PCM if volume setting is selected.
    volumeSetting2Pcm=FALSE;
  }
  else
  {
     scrollSelectMenuItem = FALSE;
  }

}
/*******************************************************************************

 $Function:     setSilentModeVolumeOn

 $Description:

 $Returns:    none

 $Arguments:  none

*******************************************************************************/
void setSilentModeVolume(void)
{
  stopPlayingMelody();
  oldVolumeSetting = current.volumeSetting;
  current.volumeSetting =0;
  audio_SetAmplf (AUDIO_BUZZER, current.volumeSetting);
}

/*******************************************************************************

 $Function:     setSilentModeVolumeOn

 $Description:

 $Returns:    none

 $Arguments:  none

*******************************************************************************/
void restoreSilentModeVolume(void)
{
  stopPlayingMelody();
  current.volumeSetting =oldVolumeSetting;
  audio_SetAmplf (AUDIO_BUZZER, current.volumeSetting);

}

/*******************************************************************************

 $Function:     soundReasonRinger

 $Description:  setup the soundReason to be Ringer Melody settings.

 $Returns:    none

 $Arguments:  none

*******************************************************************************/
void soundsVolSilent(void)
{

  FFS_flashData.settings_status |= SettingsSilentMode;
  flash_write();

  stopPlayingMelody();

  if(volumeSetting2Pcm != TRUE)
  {
    current.volumeSetting =0;
    audio_SetAmplf (AUDIO_BUZZER, current.volumeSetting);
    audio_PlaySoundID(AUDIO_BUZZER, current.ringer, 0, AUDIO_PLAY_INFINITE);
  }
  else
  {
    //store setting into PCM
    volumeSetting2Pcm=FALSE;
  }
}
/*******************************************************************************

 $Function:     soundsVolLow

 $Description:  set voulme level to low.

 $Returns:    none

 $Arguments:  none

*******************************************************************************/
void soundsVolLow(void)
{
  FFS_flashData.settings_status &= ~SettingsSilentMode;
  flash_write();
  stopPlayingMelody();
  if(volumeSetting2Pcm != TRUE)
  {
    current.volumeSetting =5;
    audio_SetAmplf (AUDIO_BUZZER, current.volumeSetting);
    audio_PlaySoundID(AUDIO_BUZZER, current.ringer, 0, AUDIO_PLAY_INFINITE);
  }
  else
  {
    //store setting into PCM
    volumeSetting2Pcm=FALSE;
  }
}
/*******************************************************************************

 $Function:     soundsVolMedium

 $Description:  set voulme level to Medium

 $Returns:    none

 $Arguments:  none

*******************************************************************************/
void soundsVolMedium(void)
{
  FFS_flashData.settings_status &= ~SettingsSilentMode;
  flash_write();
  stopPlayingMelody();
  if(volumeSetting2Pcm != TRUE)
  {
    current.volumeSetting =25;
    audio_SetAmplf (AUDIO_BUZZER, current.volumeSetting);
    audio_PlaySoundID(AUDIO_BUZZER, current.ringer, 0, AUDIO_PLAY_INFINITE);
  }
  else
  {
    //store setting into PCM
    volumeSetting2Pcm=FALSE;
  }

}
/*******************************************************************************

 $Function:     soundsVolHigh

 $Description:  set voulme level to High

 $Returns:    none

 $Arguments:  none

*******************************************************************************/
void soundsVolHigh(void)
{
  FFS_flashData.settings_status &= ~SettingsSilentMode;
  flash_write();
  stopPlayingMelody();
  if(volumeSetting2Pcm != TRUE)
  {
    current.volumeSetting =175;
    audio_SetAmplf (AUDIO_BUZZER, current.volumeSetting);
    audio_PlaySoundID(AUDIO_BUZZER, current.ringer, 0, AUDIO_PLAY_INFINITE);
  }
  else
  {
    //store setting into PCM
    volumeSetting2Pcm=FALSE;
  }

}
/*******************************************************************************

 $Function:     soundsVolInc

 $Description:  set voulme level to increasing

 $Returns:    none

 $Arguments:  none

*******************************************************************************/
void soundsVolInc(void)
{
  FFS_flashData.settings_status &= ~SettingsSilentMode;
  flash_write();
  stopPlayingMelody();
  if(volumeSetting2Pcm != TRUE)
  {
    current.volumeSetting = 5;
    audio_SetAmplf (AUDIO_BUZZER, current.volumeSetting);
    audio_PlaySoundID(AUDIO_BUZZER, current.ringer, 0, AUDIO_PLAY_INFINITE);
  }
  else
  {
    //store setting into PCM
    volumeSetting2Pcm=FALSE;

  }


}
/*******************************************************************************

 $Function:     soundReasonRinger

 $Description:  setup the soundReason to be Ringer Melody settings.

 $Returns:    none

 $Arguments:  none

*******************************************************************************/
void soundReasonRinger(void)
{
  soundReason = MelodySelect;
}
/*******************************************************************************

 $Function:     soundReasonSmsTone

 $Description:  Setup the soundReason to be SMS tones settings.

 $Returns:    none

 $Arguments:  none

*******************************************************************************/
void soundReasonSmsTone(void)
{
  soundReason = SettingSMSTone;
}
/*******************************************************************************

 $Function:     soundReasonAlarm

 $Description:  Setup the soundReason to be Alarm settings.

 $Returns:    none

 $Arguments:  none

*******************************************************************************/
void soundReasonAlarm(void)
{
  soundReason = SettingAlarm;
}
/*******************************************************************************

 $Function:     getcurrentSMSTone

 $Description:  Access function for the current SMS Tone.
 $Returns:    Tone Id

 $Arguments:  None

*******************************************************************************/
UBYTE getcurrentSMSTone(void)
{
  return current.SMSTone;
}/*******************************************************************************

 $Function:     getcurrentSMSTone

 $Description:  Access function for the current SMS Tone.
 $Returns:    Tone Id

 $Arguments:  None

*******************************************************************************/
UBYTE getcurrentAlarmTone(void)
{
  return current.AlarmOn;
}
/*******************************************************************************

 $Function:     getCurrentRingerSettings

 $Description:  Access function for the current ringer settings data.
 $Returns:    Ringer Id.

 $Arguments:  None

*******************************************************************************/
UBYTE getCurrentRingerSettings(void)
{
  return current.ringer;
}
/*******************************************************************************

 $Function:     getCurrentVoulmeSettings

 $Description:  Access function for the current ringer volume settings data.
 $Returns:    volume

 $Arguments:  None

*******************************************************************************/
UBYTE getCurrentVoulmeSettings(void)
{
  return current.volumeSetting;
}
/*******************************************************************************

 $Function:     getMelodyListStatus

 $Description:  Status flag indicating if a melody selection menu is active.
 $Returns:    status

 $Arguments:  None

*******************************************************************************/
BOOL getMelodyListStatus(void)
{
  if(soundReason == MelodySelect ||
    soundReason == SettingSMSTone ||
    soundReason == SettingAlarm )
     return TRUE;
  else
    return FALSE;
}


/*******************************************************************************

 $Function:     stopPlayingMelody

 $Description:  stop currently played ringer melody

 $Returns:    None

 $Arguments:  None

*******************************************************************************/

static void stopPlayingMelody( void )
{
  TRACE_FUNCTION( "stopPlayingMelody" );

  /* If a timer event is running we want to stop that as well,
     since we don't really want to turn the tune off and have
     it start again when the timer expires
  */
  if ( hRingerStartTimer != NULL )
    timStop( hRingerStartTimer );

  /* Are we playing a tune
  */
  if ( LastTune )
    audio_StopSoundbyID( AUDIO_BUZZER, LastTune );

  /* Need to wait for the tune to stop before trying to
     set up the next tune, so wait always
  */
  vsi_t_sleep( 0, 5 );
}

/*******************************************************************************

 $Function:     startPlayingMelody

 $Description:  Start played the ringer melody

 $Returns:    None

 $Arguments:  Melody_id

*******************************************************************************/

static void startPlayingMelody(UBYTE melody_id)
{

  if ( hRingerStartTimer != NULL )
    timStop( hRingerStartTimer );

  audio_PlaySoundID(AUDIO_BUZZER, melody_id, (BYTE)current.volumeSetting, AUDIO_PLAY_INFINITE);

}


/*******************************************************************************

 $Function:     soundsPlayRinger

 $Description:  Play the tune selected by melody_id, stopping any currently
        playing tunes if necessary. Remember to take into account the
        volume setting and provide a crescendo if requested

 $Returns:    None

 $Arguments:  melody_id, selects the melody to be played

*******************************************************************************/

static void soundsPlayRinger( int melody_id )
{
  CurrentTune = melody_id;
  if (LastTune)
    audio_StopSoundbyID (AUDIO_BUZZER,LastTune );
  vsi_t_sleep(0, 5);

  if (current.volumeSetting == increasing)
  {
    audio_PlaySoundID ( AUDIO_BUZZER,CurrentTune, 0 , AUDIO_PLAY_CRESCENDO );
  }
  else
  {
    audio_PlaySoundID ( AUDIO_BUZZER,CurrentTune, 0, AUDIO_PLAY_INFINITE );
  }
  LastTune = CurrentTune;
}


/*******************************************************************************

 $Function:     destroyDynaMenu

 $Description:  frees the memory allocated to a dynamenu (Refer to the
        buildMelodyMenu routine below

 $Returns:    None.

 $Arguments:  None.

*******************************************************************************/

static void destroyDynaMenu( void )
{

}


/*******************************************************************************

 $Function:     buildMelodyMenu

 $Description:  Builds the melody menu dynamenu from the melodies supplied
        by the resource manager

 $Returns:    None

 $Arguments:  None

*******************************************************************************/
static T_MFW_HND  buildMelodyMenu( MfwHnd parent_window)
{
  T_MELODY_INFO *  data = (T_MELODY_INFO *)ALLOC_MEMORY (sizeof (T_MELODY_INFO));
  T_MFW_WIN  * win;

    TRACE_FUNCTION (">>>> buildMelodyMenu()");

    /*
     * Create window handler
     */

    data->sounds_win = win_create (parent_window, 0, E_WIN_VISIBLE, (T_MFW_CB)melody_win_cb);
    if (data->sounds_win EQ NULL)
  {
     return NULL;
  }

  TRACE_EVENT(">>>> Melody window created: " );
    /*
     * connect the dialog data to the MFW-window
     */

    data->mmi_control.dialog = (T_DIALOG_FUNC)melody_exec_cb;
    data->mmi_control.data   = data;
    win                      = ((T_MFW_HDR *)data->sounds_win)->data;
    win->user                = (void *)data;
    data->parent             = parent_window;
  winShow(data->sounds_win);
    /*
     * return window handle
     */

     return data->sounds_win;
}

/*******************************************************************************

 $Function:     melody_exec_cb

 $Description:  Exec callback function of the SMS read window (dynamic list of
        all the SMS)

 $Returns:    none

 $Arguments:  win - window handler
        event - mfw event
        parameter - optional data.

*******************************************************************************/
void melody_exec_cb (T_MFW_HND win, USHORT event, SHORT value, void * parameter)
{
    T_MFW_WIN      * win_data = ((T_MFW_HDR *) win)->data;
    T_MELODY_INFO   * data = (T_MELODY_INFO *)win_data->user;
  UBYTE i;
  UBYTE numberOfMelodies;
  ListWinInfo      * mnu_data = (ListWinInfo *)parameter;

    TRACE_FUNCTION (">>>>> melody_exec_cb()");

    switch (event)
  {
  case E_INIT:

    TRACE_EVENT(">>>> melody_exec_cb() Event:E_INIT");

     /* initialization of administrative data */


    data->menu_list_data = (ListMenuData *)ALLOC_MEMORY(sizeof(ListMenuData));

    if(data->menu_list_data == 0)
    {
      TRACE_EVENT("Failed memory alloc 1 ");
      return;
    }

    numberOfMelodies = resource_GetListCount(RES_MELODY);

    data->menu_list_data->List = (T_MFW_MNU_ITEM *)ALLOC_MEMORY( numberOfMelodies * sizeof(T_MFW_MNU_ITEM) );

    if(data->menu_list_data->List == 0)
    {
      TRACE_EVENT("Failed memory alloc 2");
      return;
    }

    for (i = 0; i < numberOfMelodies ; i++)
    {
      data->menu_list_data->List[i].icon = 0;
      data->menu_list_data->List[i].exta = 0;
      data->menu_list_data->List[i].edit = 0;
      data->menu_list_data->List[i].menu = 0;
      data->menu_list_data->List[i].func = NULL;
      data->menu_list_data->List[i].str  = (char *)MelodyTable[i];
      data->menu_list_data->List[i].flagFunc = item_flag_none;
    }

      data->menu_list_data->ListLength =numberOfMelodies;
    data->menu_list_data->ListPosition = 1;
    data->menu_list_data->CursorPosition = 1;
    data->menu_list_data->SnapshotSize = numberOfMelodies;
    data->menu_list_data->Font = 0;
    data->menu_list_data->LeftSoftKey = TxtSoftSelect;
    data->menu_list_data->RightSoftKey = TxtSoftBack;
    data->menu_list_data->KeyEvents = KEY_ALL;
    data->menu_list_data->Reason = 0;
    data->menu_list_data->Strings = TRUE;
    data->menu_list_data->Attr   = &melody_menuAttrib;
    data->menu_list_data->autoDestroy    = FALSE;
  	    listDisplayListMenu(win, data->menu_list_data,(ListCbFunc)MelodyList_cb,0);
  case E_START_MELODY:
      // User has scrolled the Melody list, stop old melody and start playing new melody.
      // stopPlayingMelody();

      //delayThenPlayMelody();

    //  startPlayingMelody(mnu_data->MenuData.ListPosition);
    startPlayingMelody(mnu_data->MenuData.ListPosition);

    break;

  case E_RETURN:
    //stop playing the melody stopPlayingMelody();

    break;
  default:
      break;
  }
}
/*******************************************************************************

 $Function:     MelodyList_cb

 $Description:  Callback function for the melody list.

 $Returns:    none

 $Arguments:  Parent - parent window.
        ListData - Menu item list
*******************************************************************************/

void MelodyList_cb(T_MFW_HND * Parent, ListMenuData * ListData)
{
    T_MFW_WIN       * win_data = ((T_MFW_HDR *)Parent)->data;
    T_MELODY_INFO * data     = (T_MELODY_INFO *)win_data->user;
  T_MFW_HND      * l_parent;

  if ((ListData->Reason EQ LISTS_REASON_BACK) || (ListData->Reason EQ LISTS_REASON_CLEAR)
  	|| (ListData->Reason EQ LISTS_REASON_HANGUP))	// sbh - added hangup key
  {
      l_parent = data->parent;

  }
  else if(ListData->Reason EQ LISTS_REASON_SELECT)
  {
//      current.ringer = menu_Select( &dynamenu );

    switch(soundReason)
      {
      case MelodySelect:
        stopPlayingMelody();
        current.ringer = ListData->ListPosition;
        break;
      case SettingSMSTone:
        stopPlayingMelody();
        current.SMSTone = ListData->ListPosition;
        break;
      case SettingAlarm:
        // need to check alarm status MZ.
        current.AlarmOn = ListData->ListPosition;
      default:
        break;
      }
    // Store selected melody in PCM storage.


  }
  stopPlayingMelody();
  listsDestroy(ListData->win);
  melody_destroy(data->sounds_win);

}

/*******************************************************************************

 $Function:     melody_destroy

 $Description:  Destroy the melody window.

 $Returns:    none

 $Arguments:  own_window- current window
*******************************************************************************/

static void melody_destroy(MfwHnd own_window)
{
    T_MFW_WIN * win_data;
    T_MELODY_INFO     * data;
  UBYTE numberOfMelodies;

    TRACE_FUNCTION ("melody_destroy()");

    if (own_window)
  {
      win_data = ((T_MFW_HDR *)own_window)->data;
      data = (T_MELODY_INFO *)win_data->user;

      if (data)
    {
        /*
         * Delete WIN handler
         */
        win_delete (data->sounds_win);

        /*
         * Free Memory
         */

        numberOfMelodies = resource_GetListCount(RES_MELODY);

      if (data->menu_list_data != NULL)
      {
        FREE_MEMORY ((void *)data->menu_list_data->List, numberOfMelodies * sizeof (T_MFW_MNU_ITEM));
          FREE_MEMORY ((void *)data->menu_list_data, sizeof (ListMenuData));
      }
      FREE_MEMORY ((void *)data, sizeof (T_MELODY_INFO));

    }
      else
    {
        TRACE_EVENT ("melody_destroy() called twice");
    }
  }
}

/*******************************************************************************

 $Function:     melody_win_cb

 $Description:  Window callback function for the melody window.

 $Returns:    none

 $Arguments:  w - mfw window handler
        e - mfw event

*******************************************************************************/

static int melody_win_cb (MfwEvt e, MfwWin *w)    /* yyy window event handler */
{
  TRACE_FUNCTION ("melody_win_cb()");
  switch (e)
  {
  case MfwWinVisible:  /* window is visible  */
    break;
  case MfwWinFocussed: /* input focus / selected   */
  case MfwWinDelete:   /* window will be deleted   */

  default:
    return MFW_EVENT_REJECTED;
  }
  return MFW_EVENT_CONSUMED;
}

/*******************************************************************************

 $Function:     melody_menu_cb

 $Description:  Call back function for the melody menu.

 $Returns:

 $Arguments:
*******************************************************************************/

static int melody_menu_cb (MfwEvt e, MfwMnu *m){

    T_MFW_HND       win  = mfwParent(mfw_header());
    T_MFW_WIN     * win_data = ((T_MFW_HDR *)win)->data;
//    T_call_menu * data = (T_call_menu *)win_data->user;

  switch (e)
        {
            case E_MNU_ESCAPE:
        //call_menu_destroy(win);
        //call_data.win_menu = 0;
      break;
            default:
                return 0;
        }
  return 1;
}

/*******************************************************************************

 $Function:     resource_GetListCount

 $Description:  Returns the number of entries in resource table.

 $Returns:    Number of entries in Melody table.

 $Arguments:  res - Resource Id.

*******************************************************************************/
int resource_GetListCount(res_ResourceID_type res)
  // Returns the number of the resource type available in the phone.
{
  int index=0;
  switch (res)
  {
    case RES_MELODY:
    //  if (this_res.melody_table)
    //    result = this_res.melody_table->count;
      while(MelodyTable[index] !=0)
        index++;
      return index;
      break;

    case RES_LANGUAGE:

      break;
    case RES_STRING:

      break;
    default:

      break;

  }
  return index;
}

/*******************************************************************************

 $Function:     buildOnOffMenu

 $Description:  builds an on/off menu and associates it with a context

 $Returns:    None

 $Arguments:  count, number of elements in menu
        id3, prompt string
        active, context into which the menu should be associated

*******************************************************************************/

static void buildOnOffMenu( int count, int id3, int active )
{
}

/*******************************************************************************

 $Function:     buildVolumeMenu

 $Description:  builds the volume menu and associates it with the volume
        context

 $Returns:    None

 $Arguments:  volume, the context to be associated with

*******************************************************************************/

static void buildVolumeMenu( VOL_ENUM volume )
{
}

/*******************************************************************************

 $Function:     displayDynaMenu

 $Description:  show one page of the ringer list, determined by item.

 $Returns:    None

 $Arguments:  None

*******************************************************************************/

static void displayDynamenu( void )
{

}

/*******************************************************************************

 $Function:     displayRingerVolume

 $Description:  shows the volume setting determined by cVolume

 $Returns:    None

 $Arguments:  None

*******************************************************************************/

static void displayRingerVolume( void )
{
}

/*******************************************************************************

 $Function:     displayOnOffList

 $Description:  show the On Off (3rd) list, determined by item.

 $Returns:    None

 $Arguments:  None

*******************************************************************************/

static void displayOnOffList( void )
{
}
/*******************************************************************************

 $Function:     buildLanguageMenu

 $Description:  builds the dynamenu for the RES_LANGUAGE resource type

 $Returns:    None

 $Arguments:  None

*******************************************************************************/

static void buildLanguageMenu(void)
{

}

/*******************************************************************************

 $Function:     delayThenPlayMelody

 $Description:  play selected ringer melody after short delay

 $Returns:    None

 $Arguments:  None

*******************************************************************************/

static void delayThenPlayMelody( void )
{

  /* start timer for half a second and on timeout start everlasting play
  */
  timStart( hRingerStartTimer );
}

/*******************************************************************************

 $Function:     playCurrentSound

 $Description:  play tune which is temporarily selected, this event handler
        is invoked when the hRingerStartTimer event completes

 $Returns:    MFW_EVENT_CONSUMED always

 $Arguments:  e, event, tc timer context

*******************************************************************************/

static int playCurrentSound( MfwEvt e, MfwTim *tc )
{
  soundsPlayRinger(current.ringer);

  // stop running timer (One shot mode only)
  if (hRingerStartTimer != NULL ) {
    timStop(hRingerStartTimer);
  }

    return MFW_EVENT_CONSUMED;
}


/*******************************************************************************

 $Function:     setSelection

 $Description:  reactivates calling menu after confirming setting

 $Returns:    MFW_EVENT_PASSED always

 $Arguments:  e, event, m, window handle

*******************************************************************************/

static int setSelection( MfwEvt e, MfwWin *m )
{
  int UpdatePCM = 0;
  int SoundToPlay = -1;

  TRACE_FUNCTION("setSelection");

  /* This implements a state machine, the next setting depends
     on the reason we have been invoked

     Note the following is still pending implementation
     USSD TONE, SVC TONE, MANPLMN TONE
  */
  switch (soundReason)
  {
    case MelodySelect:
    {
//      stopPlayingMelody();
//      current.ringer = menu_Select( &dynamenu );
//      UpdatePCM = 1;
    }
    break;

    case SettingVolume:
    {
//      stopPlayingMelody();
//      current.volumeSetting = menu_Select( &volume_menu );
//      UpdatePCM = 1;
    }
    break;

    default:
    {
      /* No action required
      */
    }
    break;
  }

  /* Update the PCM if we need to
  */
//nm  if ( UpdatePCM )
//nm    writeSettingsToPCM( NO_FLUSH );

  /* And play the requested sound if we need to
  */
//nm  if ( SoundToPlay != -1 )
//nm    audio_PlaySoundID ( AUDIO_BUZZER, SoundToPlay, 0, AUDIO_PLAY_ONCE );

  /* redisplay sub menu showing changed position of active marker.
  */
  winShow( win );
//JVJE  soundExec( SoundsDone, 0 );

  return MFW_EVENT_PASSED;
}







/*******************************************************************************

 $Function:     writeSettingsToPCM

 $Description:  writes contents of global vars of sounds to PCM or
          sets default values

 $Returns:    None

 $Arguments:  flush, dictates whether or not flushing should be
        performed on the write operation

*******************************************************************************/
static void writeSettingsToPCM( U8 flush )
{
}
/*******************************************************************************

 $Function:     readSettingsFromPCM

 $Description:  inits global vars of sounds from PCM or
          sets default values

 $Returns:    None

 $Arguments:  None

*******************************************************************************/

static void readSettingsFromPCM(void)
{
}

/*******************************************************************************

 $Function:     soundsCreate

 $Description:  Create a sounds context, with a window, keyboards, timers
        etc.

 $Returns:    None

 $Arguments:  parent, handle of the parent window

*******************************************************************************/

static void soundsCreate(MfwHnd parent)
{

}


/*******************************************************************************

 $Function:     soundsDestroy

 $Description:  cleans up a context

 $Returns:    None

 $Arguments:  None

*******************************************************************************/

static void soundsDestroy(void)
{

}


/*******************************************************************************

 $Function:     getCallingMenu

 $Description:  handles transition to calling menu
          a) on timeout,
          b) during running timer when Clear, SoftKeyLeft
             or SoftKeyRight is pressed

 $Returns:    MFW_EVENT_PASSED

 $Arguments:  e, event, tc, timer context

*******************************************************************************/

static int getCallingMenu(MfwEvt e, MfwTim *tc)
{
  TRACE_FUNCTION( "BACK to invocating Menu" );

  return 0;
}

/*******************************************************************************

 $Function:     winEvent

 $Description:  window event handler, only deals with the win visible
        event, all others are ignored

 $Returns:    MFW_EVENT_PASSED or MFW_EVENT_CONSUMED depending on the
        event

 $Arguments:  e, event, w, window handle

*******************************************************************************/

static int winEvent (MfwEvt e, MfwWin *w)
{
    return MFW_EVENT_CONSUMED;
}


/*******************************************************************************

 $Function:     keyEventLong

 $Description:  keyboard event handler Long press

 $Returns:    MFW_EVENT_PASSED or MFW_EVENT_CONSUMED depending on the
        event

 $Arguments:  e, event, k, keyboard handle

*******************************************************************************/

static int keyEventLong (MfwEvt e, MfwKbd *k)
{
    PTRACE( sprintf( buf, "keycode in SOUND keyEvtLong %d", (int) k->code ); )
  PTRACE( sprintf( buf, "mfwEvt in SOUND Long 0x%10x", e ); )


  return MFW_EVENT_PASSED; /* give other handlers a chance */
}

/*******************************************************************************

 $Function:     keyEvent

 $Description:  keyboard event handler

 $Returns:    MFW_EVENT_PASSED or MFW_EVENT_CONSUMED depending on the
        event

 $Arguments:  e, event, k, keyboard handle

*******************************************************************************/

static int keyEvent (MfwEvt e, MfwKbd *k)
{

    PTRACE( sprintf( buf, "keycode in SOUND keyEvt %d", (int) k->code ); )
  PTRACE( sprintf( buf, "mfwEvt in SOUND 0x%10x", e ); )

  return MFW_EVENT_CONSUMED;
}

/*******************************************************************************

 $Function:     stepVibrator

 $Description:  Changes the state of the vibrator from vibrating to
                not vibrating and then restarts the vibration timer

 $Returns:    None

 $Arguments:  None

*******************************************************************************/

static void stepVibrator( void )
{
  TRACE_FUNCTION("Vibrator Ringing");

}

/*******************************************************************************

 $Function:     ringVibrator

 $Description:  generates the ringing vibrations as configured

 $Returns:    New setting of the vibrator counter

 $Arguments:  c, pointer to a vibrator counter

*******************************************************************************/

static int ringVibrator( UBYTE *c )
{
  return 0;
}


/*******************************************************************************

 $Function:     VibratorTimerEvent

 $Description:  Vibrator timer event

 $Returns:    Event consumed, always

 $Arguments:  e, event, t, timer

*******************************************************************************/

static int VibratorTimerEvent ( MfwEvt e, MfwTim *t )
{

  return MFW_EVENT_CONSUMED;
}

/*******************************************************************************

 $Function:     volumeSettingTimeOut

 $Description:  The Ringer volume settings time out has occured and used has not selected
        a volume setting.

 $Returns:    MFW_EVENT_CONSUMED always

 $Arguments:  e, event, tc timer context

*******************************************************************************/
static int volumeSettingTimeOut( MfwEvt e, MfwTim *tc )
{
  T_MFW_HND win = mfw_parent(mfw_header());
  T_MFW_WIN * win_data = ((T_MFW_HDR *)win)->data;
  T_idle * idle_data = (T_idle *)win_data->user;
  //T_call * call_data = (T_call*)win_data->user;

  TRACE_EVENT(">>>> volumeSettingTimeOut(), Vol Setiing Time out. ");

  // stop running timer.
  if (hRingerVolSettingTimeOut != NULL ) {
    timStop(hRingerVolSettingTimeOut);
  }

    if(call_data.win_menu !=NULL && (call_data.call_direction == INCOMING))
  {
    bookMenuDestroy(call_data.win_menu);
    /*
      If volume settings dialog timed out while in the incoming call, allow the
      ringer to continue ringing. MZ 9/4/01
    */
    startPlayingMelody(getCurrentRingerSettings());

  }
  else if (idle_data->info_win != NULL )
  {
    bookMenuDestroy(idle_data->info_win);
  }

    return MFW_EVENT_CONSUMED;
}

/*******************************************************************************

                                Public Methods

*******************************************************************************/
/*******************************************************************************

 $Function:     restartVolSettingInactivityTimer

 $Description:  Restart the Volume settings timer.

 $Returns:    none

 $Arguments:  none

*******************************************************************************/
void restartVolSettingInactivityTimer(void)
{
  if ( hRingerVolSettingTimeOut != NULL )
    timStop( hRingerVolSettingTimeOut );

  timStart(hRingerVolSettingTimeOut);

}
/*******************************************************************************

 $Function:     stopRingerVolTimeOut

 $Description:  Stop the ringer volume settings timer.

 $Returns:    none

 $Arguments:  none

*******************************************************************************/
void stopRingerVolSettingInactivityTimer(void)
{

  if ( hRingerVolSettingTimeOut != NULL )
    timStop( hRingerVolSettingTimeOut );

}
/*******************************************************************************

 $Function:     startRingerVolumeSettingTimer

 $Description:  Create and start the ringer voulme inactivity timer.

 $Returns:    none

 $Arguments:  none

*******************************************************************************/
void startRingerVolSettingInactivityTimer(T_MFW_HND parent_win)
{
  T_MFW_HND win = mfw_parent(mfw_header());
  T_MFW_WIN * win_data = ((T_MFW_HDR *)win)->data;
  T_idle * idle_data = (T_idle *)win_data->user;
  //T_call * call_data = (T_call*)win_data->user;

  TRACE_FUNCTION(">>>> startRingerVolSettingInactivityTime, Create and Start Vol timer ");
  //Create 4 second ringer delay timer.
  if(hRingerVolSettingTimeOut == NULL )
  {
    if(call_data.call_direction == INCOMING)
    {
      hRingerVolSettingTimeOut = timCreate(call_data.win, TIM4SECOND, (MfwCb)volumeSettingTimeOut);
    }
    else
    {
      hRingerVolSettingTimeOut = timCreate(idle_data->win, TIM4SECOND, (MfwCb)volumeSettingTimeOut);
    }
  }

  if ( hRingerVolSettingTimeOut != NULL )
    timStop( hRingerVolSettingTimeOut );

  timStart(hRingerVolSettingTimeOut);


}
/*******************************************************************************

 $Function:     soundsRinger

 $Description:  prepares Ringer setting screen

 $Returns:    MFW_EVENT_CONSUMED always

 $Arguments:  m, menu, i, item

*******************************************************************************/

int soundsRinger( MfwMnu *m, MfwMnuItem *i )
{
  T_MFW_HND win;
    T_MFW_HND parent_win = mfw_parent(mfw_header());

  //recources_Init (win);

  //Create 0.5 second ringer delay timer.
  hRingerStartTimer = timCreate( 0, TIMHALFSECOND, (MfwCb)playCurrentSound);

  win = buildMelodyMenu(parent_win);

   if (win NEQ NULL)
   {
      SEND_EVENT (win, E_INIT, 0, 0);
   }

   // Start the 0.5 second ringer delay timer.
     delayThenPlayMelody();
  //  soundExec();

  return MFW_EVENT_CONSUMED;
}


/*******************************************************************************

 $Function:     soundsVolume

 $Description:  prepares Volume setting screen

 $Returns:    MFW_EVENT_CONSUMED always

 $Arguments:  m, menu, i, item

*******************************************************************************/

int soundsVolume( MfwMnu *m, MfwMnuItem *i )
{

  return MFW_EVENT_CONSUMED;
}


/*******************************************************************************

 $Function:     settingsVibrator

 $Description:  prepares Vibrator settings screen

 $Returns:    MFW_EVENT_CONSUMED always

 $Arguments:  m, menu, i, item

*******************************************************************************/

int settingsVibrator( MfwMnu *m, MfwMnuItem *i )
{

  return MFW_EVENT_CONSUMED;
}
/*******************************************************************************

 $Function:     settingsLanguages

 $Description:  prepares languages screen

 $Returns:    MFW_EVENT_CONSUMED always

 $Arguments:  m, menu, i, item

*******************************************************************************/

int settingsLanguages( MfwMnu *m, MfwMnuItem *i )
{
  return MFW_EVENT_CONSUMED;
}

/*******************************************************************************

 $Function:     soundsNewMessage

 $Description:  prepares NewMessage setting screen

 $Returns:    MFW_EVENT_CONSUMED always

 $Arguments:  m, menu, i, item

*******************************************************************************/

int soundsNewMessage( MfwMnu *m, MfwMnuItem *i )
{

  return MFW_EVENT_CONSUMED;
}

/*******************************************************************************

 $Function:     soundsNewBroadcast

 $Description:  prepares Broadcast setting screen

 $Returns:    MFW_EVENT_CONSUMED always

 $Arguments:  m, menu, i, item

*******************************************************************************/

int soundsNewBroadcast( MfwMnu *m, MfwMnuItem *i )
{

  return MFW_EVENT_CONSUMED;
}

/*******************************************************************************

 $Function:     soundsBatteryLow

 $Description:  prepares BatteryLow setting screen

 $Returns:    MFW_EVENT_CONSUMED always

 $Arguments:  m, menu, i, item

*******************************************************************************/

int soundsBatteryLow( MfwMnu *m, MfwMnuItem *i )
{

  return MFW_EVENT_CONSUMED;
}


/*******************************************************************************

 $Function:     soundsCreditLow

 $Description:  prepares Credit setting screen

 $Returns:    MFW_EVENT_CONSUMED always

 $Arguments:  m, menu, i, item

*******************************************************************************/

int soundsCreditLow( MfwMnu *m, MfwMnuItem *i )
{

  return MFW_EVENT_CONSUMED;
}
/*******************************************************************************

 $Function:     soundsOrganiser

 $Description:  prepares Organiser setting screen

 $Returns:    MFW_EVENT_CONSUMED always

 $Arguments:  m, menu, i, item

*******************************************************************************/

int soundsOrganiser( MfwMnu *m, MfwMnuItem *i )
{

  return MFW_EVENT_CONSUMED;
}

/*******************************************************************************

 $Function:     soundsAlarm

 $Description:  prepares Alarm setting screen

 $Returns:    MFW_EVENT_CONSUMED always

 $Arguments:  m, menu, i, item

*******************************************************************************/

int soundsAlarm( MfwMnu *m, MfwMnuItem *i )
{

  return MFW_EVENT_CONSUMED;
}


/*******************************************************************************

 $Function:     soundInit

 $Description:  init sounds handling

 $Returns:    None

 $Arguments:  parent, handle of parent window

*******************************************************************************/

void soundInit( MfwHnd parent )
{


}


/*******************************************************************************

 $Function:     soundExit

 $Description:  exit sounds handling

 $Returns:    None

 $Arguments:  None

*******************************************************************************/

void soundExit (void)
{

}


/*******************************************************************************

 $Function:     soundExec

 $Description:  sends messages to soundExec to get the various melodies to play

 $Returns:    None

 $Arguments:  reason, for invocation, next, state

*******************************************************************************/

void soundExec (int reason, MmiState next)

{

}


/*******************************************************************************

 $Function:     soundsKeypadSilent

 $Description:  prepares KeypadTones setting screen

 $Returns:    MFW_EVENT_CONSUMED always

 $Arguments:  m, menu, i, item

*******************************************************************************/

int soundsKeypadSilent(MfwMnu* m, MfwMnuItem* i)
{

   T_MFW_HND win = mfw_parent(mfw_header());

  sounds_Init(win);

  if (FFS_flashData.settings_status & SettingsKeypadClick)
  {
    FFS_flashData.settings_status &= ~SettingsKeypadClick;
  }
  else if (FFS_flashData.settings_status & SettingsKeypadDTMF)
  {
    FFS_flashData.settings_status &= ~SettingsKeypadDTMF;
  }
  flash_write();
    sounds_data.local_status = ACTIVATE_CONFIRM;

  SEND_EVENT (sounds_data.sounds_win, SHOW_CONFIRM, 0, 0);

  return 0;
}
/*******************************************************************************

 $Function:     soundsKeypadClick

 $Description:  prepares KeypadTones setting screen

 $Returns:    MFW_EVENT_CONSUMED always

 $Arguments:  m, menu, i, item

*******************************************************************************/
int soundsKeypadClick(MfwMnu* m, MfwMnuItem* i)
{

   T_MFW_HND win = mfw_parent(mfw_header());

  sounds_Init(win);



  if (FFS_flashData.settings_status & SettingsKeypadDTMF)
  {
    FFS_flashData.settings_status &= ~SettingsKeypadDTMF;
  }

  FFS_flashData.settings_status |= SettingsKeypadClick;

  //play KeypadClick
  audio_PlaySoundID(0, TONES_KEYBEEP, 5, 0 );

    sounds_data.local_status = ACTIVATE_CONFIRM;

  SEND_EVENT (sounds_data.sounds_win, SHOW_CONFIRM, 0, 0);
  flash_write();
  return 0;
}

/*******************************************************************************

 $Function:     soundsKeypadDTMF

 $Description:  prepares KeypadTones setting screen

 $Returns:    MFW_EVENT_CONSUMED always

 $Arguments:  m, menu, i, item

*******************************************************************************/
int soundsKeypadDTMF(MfwMnu* m, MfwMnuItem* i)
{

   T_MFW_HND win = mfw_parent(mfw_header());

  sounds_Init(win);

  if (FFS_flashData.settings_status & SettingsKeypadClick)
  {
    FFS_flashData.settings_status &= ~SettingsKeypadClick;
  }

  // test
  //audio_SetAmplf (AUDIO_SPEAKER, 240);
  // test


  FFS_flashData.settings_status |= SettingsKeypadDTMF;
  flash_write();

    sounds_data.local_status = ACTIVATE_CONFIRM;

  SEND_EVENT (sounds_data.sounds_win, SHOW_CONFIRM, 0, 0);

  return 0;
}



/*******************************************************************************

 $Function:     sounds_Init

 $Description:

 $Returns:

 $Arguments:

*******************************************************************************/
T_MFW_HND sounds_Init (T_MFW_HND parent_window)
{
  return (sounds_create (parent_window));
}
/*******************************************************************************

 $Function:     confirm_Exit

 $Description:

 $Returns:

 $Arguments:

*******************************************************************************/
void sounds_Exit (T_MFW_HND own_window)
{

    T_MFW_WIN   * win_data = ((T_MFW_HDR *)own_window)->data;
    T_MELODY_INFO * data     = (T_MELODY_INFO *)win_data->user;

  sounds_destroy (data->sounds_win);
}

/*******************************************************************************

 $Function:     sounds_create

 $Description:

 $Returns:

 $Arguments:

*******************************************************************************/
T_MFW_HND sounds_create (T_MFW_HND parent_window)
{
  T_MELODY_INFO * data = &sounds_data;
  T_MFW_WIN   * win;

  TRACE_FUNCTION ("sounds_create()");

  data->sounds_win = win_create (parent_window, 0, 0, (T_MFW_CB)sounds_win_cb);

  if (data->sounds_win EQ 0)
    return 0;

  /*
   * Create window handler
   */
  data->mmi_control.dialog = (T_DIALOG_FUNC)sounds;
  data->mmi_control.data   = data;
  win                      = ((T_MFW_HDR *)data->sounds_win)->data;
  win->user                = (void *) data;


  winShow(data->sounds_win);
  /*
   * return window handle
   */
  return data->sounds_win;
}

/*******************************************************************************

 $Function:     sounds_destroy

 $Description:

 $Returns:

 $Arguments:

*******************************************************************************/
void sounds_destroy (T_MFW_HND own_window)
{
  T_MFW_WIN   * win;
  T_MELODY_INFO * data;

  TRACE_FUNCTION ("sounds_destroy()");

  if (own_window)
  {
    win  = ((T_MFW_HDR *)own_window)->data;
    data = (T_MELODY_INFO *)win->user;

    if (data)
    {

      /*
       * Delete WIN handler
       */
      win_delete (data->sounds_win);
      data->sounds_win = 0;
    }
  }
}
/*******************************************************************************

 $Function:     sounds_win_cb

 $Description:

 $Returns:

 $Arguments:

*******************************************************************************/
static int sounds_win_cb (T_MFW_EVENT event, T_MFW_WIN * win)
{
  /*
   * Top Window has no output
   */
  return 1;
}


/*******************************************************************************

 $Function:     sounds_dialog

 $Description:

 $Returns:

 $Arguments:

*******************************************************************************/
static void sounds_dialog(void)
{
  T_DISPLAY_DATA display_info;

  TRACE_FUNCTION ("sounds_dialog()");

  dialog_info_init(&display_info);//gdy add
  display_info.KeyEvents    = KEY_CLEAR | KEY_RIGHT;

  if (sounds_data.local_status EQ ACTIVATE_CONFIRM)
  {
    display_info.TextId   = TxtActivated;
  }
  else
  {
     display_info.TextId    = TxtDeActivated;
  }

  display_info.TextId2      = 0;
  display_info.TextString   = '\0';
  display_info.TextString2  = '\0';
  display_info.LeftSoftKey  = '\0';
  display_info.RightSoftKey = '\0';
  display_info.Time         = THREE_SECS;
  display_info.Callback     = (T_VOID_FUNC)sounds_dialog_cb;
  /*
   * Call Info Screen
   */
  info_dialog (sounds_data.sounds_win, &display_info);

}
/*******************************************************************************

 $Function:     confirm_dialog_cb

 $Description:

 $Returns:

 $Arguments:

*******************************************************************************/
void sounds_dialog_cb(T_MFW_HND win, UBYTE identifier, UBYTE reason)
{
  TRACE_FUNCTION ("sounds_dialog_cb()");

      switch (reason)
      {
        case INFO_TIMEOUT:
          /* no break; */
        case INFO_KCD_HUP:
          /* no break; */
        case INFO_KCD_LEFT:
          /* no break; */
        case INFO_KCD_RIGHT:
          /* no break; */
        case INFO_KCD_CLEAR:
            sounds_Exit(win);
          break;
      }
}

/*******************************************************************************

 $Function:     sounds

 $Description:

 $Returns:

 $Arguments:

*******************************************************************************/
void sounds (T_MFW_HND win, USHORT event, SHORT value, void * parameter)
{
  T_MFW_WIN         * win_data = ((T_MFW_HDR *) win)->data;
  T_MELODY_INFO      * data     = (T_MELODY_INFO *)win_data->user;

 TRACE_FUNCTION("sounds()");

 switch (event)
  {
    case SHOW_CONFIRM:
    sounds_dialog();
    break;
  }

}


///zym for vibrator

/*******************************************************************************

 $Function:     StartVibrator

 $Description: start Vibrator

 $Returns:

 $Arguments:
*******************************************************************************/

void StartVibrator(){
      SET_BIT(io_conf_reg_g,2);
      RESET_BIT(io_cntl_reg_g,6);
      SET_BIT(armio_latch_out_g,6);
 }

/*******************************************************************************

 $Function:     StopVibrator

 $Description: stop  Vibrator

 $Returns:

 $Arguments:
*******************************************************************************/

void StopVibrator(){
      RESET_BIT(armio_latch_out_g,6);
}


void Initial_AuxSpeaker(void)
{
    RESET_BIT(0xfffef008,1);//liujb test-------select GPIO
    RESET_BIT(0xfffe4804,1);//liujb test----------select GPIO1
    SET_BIT(0xfffe4802,1);//liujb test-----------GPIO1=1
    SPIWrite_ABB_Register(0,zymTOGBR1,0x02AA);
    SPIWrite_ABB_Register(0,zymTOGBR2,0x007A);
    SPIWrite_ABB_Register(1,zymVBCR,0x0009);     //A);era
    SPIWrite_ABB_Register(1,zymVBUR,0x01B0);
    SPIWrite_ABB_Register(0,zymVBDR,0x0026);
      
}  

void KeepBeep_cb (void* parameter)
{

	rvf_free_buf (parameter); 
	return;
}



///for keybeep

void StartKeyBeep(void){  
    T_AUDIO_KEYBEEP_PARAMETER parameter;
    T_RV_RETURN return_path;


   parameter.amplitude_beep[0] = 0;
   parameter.amplitude_beep[1] = 0;   

   parameter.frequency_beep[0] = 1000;
   parameter.frequency_beep[1] = 1000;

   parameter.duration = 21;

  return_path.callback_func = KeepBeep_cb;
  return_path.addr_id = 0; 	
    
  audio_keybeep_start (parameter, return_path);
    

}

void StopKeyBeep(void){
	T_RV_RETURN return_path;
	return_path.callback_func = KeepBeep_cb;
	return_path.addr_id = 0; 	

	audio_keybeep_stop (return_path);
}


void Tone_cb (void* parameter)
{

	rvf_free_buf (parameter); 
	return;
}


void StartTone(void){
	T_AUDIO_TONES_PARAMETER parameter;
	T_RV_RETURN return_path;

	parameter.tones[0].start_tone		= 0;
	parameter.tones[0].stop_tone			= 60;
	parameter.tones[0].frequency_tone	= 520;		// Hz
	parameter.tones[0].amplitude_tone	= 0;		// dB
	parameter.tones[1].start_tone		= 20;
	parameter.tones[1].stop_tone			= 80;
	parameter.tones[1].frequency_tone	= 775;		// Hz
	parameter.tones[1].amplitude_tone	= 0;		// dB
	parameter.tones[2].start_tone		= 100;
	parameter.tones[2].stop_tone			= 140;
	parameter.tones[2].frequency_tone	= 643;		// Hz
	parameter.tones[2].amplitude_tone	= 0	;	// dB
	parameter.frame_duration				= 160;
	parameter.sequence_duration			= 320;
	parameter.period_duration			= 320;
	parameter.repetition					= TONE_INFINITE;	// infinite tones

	return_path.callback_func = Tone_cb;
	return_path.addr_id = 0; 	

	audio_tones_start (&parameter, return_path);
}


void StopTone(void){
	T_RV_RETURN return_path;
	return_path.callback_func = Tone_cb;
	return_path.addr_id = 0; 	

	audio_tones_stop (return_path);
}


void Melody_cb (void* parameter)
{

	rvf_free_buf (parameter); 
	return;
}



///for melody2

void StartMelody2(char * melody_name){
       T_AUDIO_MELODY_E2_PARAMETER temp;//chenjun add   
       T_RV_RETURN return_path;
       T_AUDIO_MELODY_E2_LOAD_FILE_INSTR_PARAMETER parameter;

	//Initial_AuxSpeaker();
	strncpy(temp.melody_E2_name , melody_name,strlen(melody_name)); 
	//strcpy(temp.melody_E2_name , "/meloe2/e2/nonreg14"); //"/bach");
	temp.E2_loopback = 1;
	temp.melody_E2_mode = 1;
	return_path.callback_func = Melody_cb;   
	return_path.addr_id = 0;

	strcpy(parameter.melody_E2_file_name,"/default");

	audio_melody_E2_load_file_instruments ( &parameter);
	
	audio_melody_E2_start (& temp,return_path);   
}


void StopMelody2(char * melody_name){
	T_AUDIO_MELODY_E2_STOP_PARAMETER parameter;
	T_RV_RETURN return_path;
	return_path.callback_func = Melody_cb;   
	return_path.addr_id = 0;

    strncpy(parameter.melody_E2_name , melody_name,strlen(melody_name)); 
	//strcpy(parameter.melody_E2_name , "/meloe2/e2/nonreg14"); //"/bach");
	audio_melody_E2_stop (&parameter, return_path);

}

///for melody2

void StartMelody(int looping)
{
       T_AUDIO_MELODY_E2_PARAMETER temp;//chenjun add   
       T_RV_RETURN return_path;
       T_AUDIO_MELODY_E2_LOAD_FILE_INSTR_PARAMETER parameter;

	//Initial_AuxSpeaker();
	
      strcpy(temp.melody_E2_name , "/meloe2/e2/bach"); //"/bach");
	temp.E2_loopback = 1;  //xsf  2002.08.14 change 1 to loop
	temp.melody_E2_mode = 1;
	return_path.callback_func = Melody_cb;   
	return_path.addr_id = 0;

	strcpy(parameter.melody_E2_file_name,"/default");

	audio_melody_E2_load_file_instruments ( &parameter);
	
	audio_melody_E2_start (& temp,return_path);   
}


void StopMelody(void){
	T_AUDIO_MELODY_E2_STOP_PARAMETER parameter;
	T_RV_RETURN return_path;
	return_path.callback_func = Melody_cb;   
	return_path.addr_id = 0;

	strcpy(parameter.melody_E2_name , "/meloe2/e2/bach"); //"/bach");
	
	audio_melody_E2_stop (&parameter, return_path);
}

void VmRecord_cb (void* parameter)
{

	rvf_free_buf (parameter); 
	return;
}


void StartVmRecord(void){
	T_AUDIO_VM_RECORD_PARAMETER record_parameter;
	T_AUDIO_TONES_PARAMETER tones_parameter;
	T_RV_RETURN return_path;


	strcpy(record_parameter.memo_name,"/vmrecord");
	record_parameter.compression_mode = AUDIO_VM_COMPRESSION_MODE;
	record_parameter.memo_duration = 250;
	record_parameter.microphone_gain = 0x100;
	record_parameter.network_gain = 0x100;

       tones_parameter.tones[0].start_tone		= 0;
	tones_parameter.tones[0].stop_tone			= 60;
	tones_parameter.tones[0].frequency_tone	= 520;		// Hz
	tones_parameter.tones[0].amplitude_tone	= -24;		// dB
	tones_parameter.tones[1].start_tone		= 20;
	tones_parameter.tones[1].stop_tone			= 80;
	tones_parameter.tones[1].frequency_tone	= 775;		// Hz
	tones_parameter.tones[1].amplitude_tone	= -15;		// dB
	tones_parameter.tones[2].start_tone		= 100;
	tones_parameter.tones[2].stop_tone			= 140;
	tones_parameter.tones[2].frequency_tone	= 643;		// Hz
	tones_parameter.tones[2].amplitude_tone	= -6	;	// dB
	tones_parameter.frame_duration				= 160;
	tones_parameter.sequence_duration			= 320;
	tones_parameter.period_duration			= 320;
	tones_parameter.repetition					= TONE_INFINITE;	// infinite tones

	return_path.callback_func = VmRecord_cb;
	return_path.addr_id = 0; 	
	

	audio_vm_record_start (&record_parameter, &tones_parameter,return_path);

}


void StopVmRecord(void){

	T_RV_RETURN return_path;
	return_path.callback_func = VmRecord_cb;
	return_path.addr_id = 0; 	

	audio_vm_record_stop (return_path);

}


void PlayVmRecord_cb (void* parameter)
{

	rvf_free_buf (parameter); 
	return;
}



void StartPlayVmRecord(void){

	T_AUDIO_VM_PLAY_PARAMETER parameter;
	T_RV_RETURN return_path;
	return_path.callback_func = PlayVmRecord_cb;
	return_path.addr_id = 0; 	

       strcpy(parameter.memo_name,"/vmrecord");  //test recognition  "/speech/srEnroll");
       

	audio_vm_play_start (&parameter,return_path);

}

void StopPlayVmRecord(void){

	T_RV_RETURN return_path;
	return_path.callback_func = PlayVmRecord_cb;
	return_path.addr_id = 0; 	

       audio_vm_record_stop (return_path);
}


void StartSREnroll(void){
	T_AUDIO_SR_ENROLL_PARAMETER parameter;
	T_RV_RETURN return_path;

	strcpy(parameter.enroll_name,"/speech/srEnroll");
	parameter.record_speech = AUDIO_SR_RECORD_SPEECH;

	return_path.callback_func = SREnroll_cb;  
	return_path.addr_id = 0; 

	audio_sr_enroll_start( &parameter, return_path);
	
}

void StopSREnroll(void){
	T_RV_RETURN return_path;
	return_path.callback_func = SREnroll_cb;
	return_path.addr_id = 0; 

	audio_sr_enroll_stop (return_path);
}

void StartSRUpdate(void){

	T_AUDIO_SR_UPDATE_PARAMETER parameter;
	T_RV_RETURN return_path;

	strcpy(parameter.update_name,"/speech/srEnroll");
	return_path.callback_func = SRUpdate_cb;
	return_path.addr_id = 0; 

	parameter.record_speech = AUDIO_SR_RECORD_SPEECH;


	audio_sr_update_start (&parameter,return_path);
}

void StopSRUpdate(void){
	T_RV_RETURN return_path;


	return_path.callback_func = SRUpdate_cb;
	return_path.addr_id = 0; 

       audio_sr_update_stop (return_path);  
}

void StartSRReco(void){
	T_RV_RETURN return_path;
       T_AUDIO_SR_RECO_PARAMETER parameter;

	return_path.callback_func = SRReco_cb;
	return_path.addr_id = 0; 

	strcpy(parameter.database_directory,"/speech");

       audio_sr_reco_start (&parameter,return_path);  
}


void StopSRReco(void){
	T_RV_RETURN return_path;

	return_path.callback_func = SRReco_cb;
	return_path.addr_id = 0; 

       audio_sr_reco_stop (return_path);
}



void SREnroll_cb (void* parameter)
{

	T_AUDIO_SR_ENROLL_STATUS *pRslt;


	int i;

	char *pchar;

	char  str[AUDIO_PATH_NAME_MAX_SIZE];


	pRslt = (T_AUDIO_SR_ENROLL_STATUS*)parameter;
	
	MmiTrace("zym:SREnroll_cb");


    switch (pRslt->status)
    {
        case AUDIO_OK:
        {

            MmiTrace("zym:= AUDIO_OK");

            StartSRUpdate();

            break;
        }
        case -1:
        {
		MmiTrace("zym:= AUDIO_ERROR");
		StartSREnroll();
		break;
			}
        case -3:
        {
		MmiTrace("zym:= AUDIO_BAD_RECOGNITION");
		StartSREnroll();
		
		break;
        }
        case -4:
        {
		MmiTrace("zym:= AUDIO_BAD_RECOGNITION");
		StartSREnroll();
		break;
        }
        default:
        {
            break;
        }  
    }

	rvf_free_buf (parameter); 
	return;
}



void SRUpdate_cb (void* parameter)
{

	T_AUDIO_SR_UPDATE_STATUS *pRslt;

	int i;

	char *pchar;

	char  str[AUDIO_PATH_NAME_MAX_SIZE];


	pRslt = (T_AUDIO_SR_UPDATE_STATUS*)parameter;

	MmiTrace("zym:SRUpdate_cb");

 
    switch (pRslt->status)
    {
        case AUDIO_OK:
        {

            MmiTrace("zym:= AUDIO_OK");

            break;
        }
        case -1:
        {
		MmiTrace("zym:= AUDIO_ERROR");
		StartSRUpdate();
		break;
        }
        case -3:
        {
		MmiTrace("zym:= AUDIO_BAD_ACQUISITION");
		StartSRUpdate();

		break;
        }
        case -4:
        {
		MmiTrace("zym:= AUDIO_TIME_OUT");
		StartSRUpdate();
		break;
        }
          case -5:
        {
		MmiTrace("zym:= AUDIO_BAD_UPDATE");
		StartSRUpdate();
		break;
        }
        case -7:
        {
		MmiTrace("zym:= AUDIO_CTO_WORD");
		StartSRUpdate();
		break;
        }
        case -8:
        {
		MmiTrace("zym:= AUDIO_CHECK_ERROR");
		StartSRUpdate();
		break;
        }
     default:
         {
            break;
        }  
    }
    
    rvf_free_buf (parameter); 
    return;
}




void SRReco_cb (void* parameter)
{

	T_AUDIO_SR_RECO_STATUS *pRslt;

	T_AUDIO_SR_RECO_STATUS*    reco_status;
	T_AUDIO_VM_PLAY_PARAMETER vm_parameter;
	T_RV_RETURN return_path;

	int i;

	char *pchar;

	char  str[AUDIO_PATH_NAME_MAX_SIZE];

	return_path.callback_func = PlayVmRecord_cb;
	return_path.addr_id = 0;

	pRslt = (T_AUDIO_SR_RECO_STATUS*)parameter;

//	MmiTrace("zym:SRReco_cb");

 
    switch (pRslt->status)
    {
        case AUDIO_OK:
        {

      //      MmiTrace("zym:= AUDIO_OK");

            pchar = (char*)(pRslt->word_recognized);

            //str = (char)pchar;

            /*

		for(i =0;i <AUDIO_PATH_NAME_MAX_SIZE;i ++){

			str[i] = * (pRslt->word_recognized[i]);
			}
*/
	//	MmiTrace(pchar);
		
		strcpy(vm_parameter.memo_name,pchar);
		
		audio_vm_play_start (&vm_parameter,return_path);

            break;
        }
/*        case -1:
        {
		MmiTrace("zym:= e1!");
		StartSRReco();
		break;
        }
        case -3:
        {
		MmiTrace("zym:= e2!");
		StartSRReco();

		break;
        }
        case -4:
        {
		MmiTrace("zym:= e3!");
		StartSRReco();

		break;
        }
        case -6:
        {
		MmiTrace("zym:= e4!");
		StartSRReco();

		break;
        } */
        default:
        {
      //      MmiTrace("zym:= err default!");           
		StartSRReco();
            break;
        } 
    }
    
    rvf_free_buf (parameter); 
    return;

}
int HallOnOff(void){
	
	*((volatile UINT16 *) 0xfffef006) &= 0XFFBF;
	*((volatile UINT16 *) 0xfffe4804) |= 0X04;
	if((*((volatile UINT16 *) (0xfffe4800)) & 0x04)!=0)
	{
	       MmiTrace("HallOn");
	       ColorLCD_Init(0);
	       r2d_refresh();
	       LCD_ON();	       
              return 1;
	}
	else
	{
	       Sleep_Or_Quality(1);
	       LCD_OFF();
	       MmiTrace("HallOff");
              return 0;
	}


}

/*******************************************************************************

                                End Of File

*******************************************************************************/
