/*
 * Description:
 *		This module contains utilities API for MMI development.
 *
 * Histroy:
 *		2003/11/20	Talcon created.
 *		2004/07/06	robert.chen 
 *			add dynamic memory manager API for MMI
 *			add trace API to log message in internal RAM
 */ 
#ifndef _MMIUTILAPI_C_
#define _MMIUTILAPI_C_

#include "nucleus.h"

#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include "mmiutilapi.h"
#include "rvf_api.h"

//#define TDTRACE 1
//#if TDTRACE
#define _TARGET
//#define RT_TRACE

#ifndef _INNOVATION_EMULATOR_
#pragma DATA_SECTION (mmi_dbgBuf, ".MMITR_BUF");
#endif
static char mmi_dbgBuf[MMI_DBGBUF_SZ];

#ifndef _INNOVATION_EMULATOR_
	static NU_SEMAPHORE mmimemSem;
	static NU_MEMORY_POOL mmimemPool;
#endif

#if (MMIMEM_INFORMATION == 1)
	static T_MMIMEM_INFO mmi_memInfo;
	static unsigned int	totalAlloc = 0;
#endif

#if (MMIMEM_TRACE == 1) 
	#define MMIMEMTRACE_MSG(msg) MI_dbgPrintf msg
#else
	#define MMIMEMTRACE_MSG(msg)
#endif

/**
 *	Output printf-like formated string to rivieratracer.
 */
#ifndef RT_TRACE 
void MI_dbgPrintf(const char* const format, ...)
{

	va_list pArgs;
	if(format == NULL) //modified by xie 04-07-22 for avoiding print NULL string.
	{
	#ifdef _TARGET
	    rvf_send_trace("occur NULL string in Trace!!!!",strlen("occur NULL string in Trace!!!!"), NULL_PARAM, RV_TRACE_LEVEL_DEBUG_HIGH, 0);
	#endif
		return;
	}
	//return;/*2004/04/01,talcon add for disable trace*/
	va_start(pArgs, format);
	vsprintf(mmi_dbgBuf, format, pArgs);
#ifdef _TARGET
	rvf_send_trace(mmi_dbgBuf, strlen(mmi_dbgBuf), NULL_PARAM, RV_TRACE_LEVEL_DEBUG_HIGH, 0);
#else
	printf(mmi_dbgBuf);
#endif
	va_end(pArgs);
}
#else
#define MEM_UART_MODEM  (0xFFFF5800)
#define MEM_UART_IRDA   (0xFFFF5000)
/*
 * 16750 addresses. Registers accessed when LCR[7] = 0.
 */

#define RHR (0x00) /* Rx buffer register - Read access   */
#define THR (0x00) /* Tx holding register - Write access */
#define IER (0x01) /* Interrupt enable register          */



/*
 * 16750 addresses. Bit 5 of the FCR register is accessed when LCR[7] = 1.
 */

#define IIR  (0x02) /* Interrupt ident. register - Read only */
#define FCR  (0x02) /* FIFO control register - Write only    */
#define LCR  (0x03) /* Line control register                 */
#define MCR  (0x04) /* Modem control register                */
#define LSR  (0x05) /* Line status register                  */
#define MSR  (0x06) /* Modem status register                 */
#define TCR  (0x06) /* Transmission control register         */
#define TLR  (0x07) /* Trigger level register                */
#define MDR1 (0x08) /* Mode definition register 1            */
#define SCR  (0x10) /* Supplementary Control register        */
#define SSR  (0x11) /* Supplementary Status register         */

/*
 * Line status register.
 */

#define DR   (0x01) /* Data ready                                  */
#define OE   (0x02) /* Overrun error                               */
#define PE   (0x04) /* Parity error                                */
#define FE   (0x08) /* Framing error                               */
#define BI   (0x10) /* Break interrupt                             */
#define THRE (0x20) /* Transmitter holding register (FIFO empty)   */
#define TEMT (0x40) /* Transmitter empty (FIFO and TSR both empty) */


/*
 * Mode definition register 1.
 */

#define UART_MODE             (0x00)
#define SIR_MODE              (0x01)
#define UART_MODE_AUTOBAUDING (0x02) /* Reserved in UART/IrDA. */
#define RESET_DEFAULT_STATE   (0x07)
#define IR_SLEEP_DISABLED     (0x00)
#define IR_SLEEP_ENABLED      (0x08)
#define SIR_TX_WITHOUT_ACREG2 (0x00) /* Reserved in UART/modem. */
#define SIR_TX_WITH_ACREG2    (0x20) /* Reserved in UART/modem. */
#define FRAME_LENGTH_METHOD   (0x00) /* Reserved in UART/modem. */
#define EOT_BIT_METHOD        (0x80) /* Reserved in UART/modem. */


/*
 * EFR is accessed when LCR[7:0] = 0xBF.
 */

#define EFR (0x02) /* Enhanced feature register */


/*
 * Enhanced feature register.
 */
 
#define ENHANCED_FEATURE_BIT (4) /* Use RESET_BIT and SET_BIT macros. */


/*
 * FIFO control register.
 */

#define FIFO_ENABLE   (0x01)
#define RX_FIFO_RESET (0x02)
#define TX_FIFO_RESET (0x04)


/*
 * 16750 addresses. Registers accessed when LCR[7] = 1.
 */

#define DLL (0x00) /* Divisor latch (LSB) */
#define DLM (0x01) /* Divisor latch (MSB) */

/*
 * Line control register.
 */

#define WLS_5         (0x00) /* Word length: 5 bits                    */
#define WLS_6         (0x01) /* Word length: 6 bits                    */
#define WLS_7         (0x02) /* Word length: 7 bits                    */
#define WLS_8         (0x03) /* Word length: 8 bits                    */
#define STB           (0x04) /* Number of stop bits: 0: 1, 1: 1,5 or 2 */
#define PEN           (0x08) /* Parity enable                          */
#define EPS           (0x10) /* Even parity select                     */
#define BREAK_CONTROL (0x40) /* Enable a break condition               */
#define DLAB          (0x80) /* Divisor latch access bit               */
#define DIV_EN_BIT    (7)

/*
 * Modem control register.
 */

#define MDTR (0x01) /* Data terminal ready. */
#define MRTS (0x02) /* Request to send.     */
#define TCR_TLR_BIT (6)

/*
 * TLR is used to program the RX FIFO trigger levels. FCR[7:4] are  not used.
 */
 
#define RX_FIFO_TRIGGER_LEVEL (12 << 4)


#define UART_UIR	(0xFFFF6000)	   /* UART Interface Register */


/*
 * UART_UIR bit definitions.
 */
#define UART_ACCESS		(0)
#define UART_MASK_IT	(1)


#define IER_SLEEP (0x10)  



#define WRITE_UART_REGISTER(REG,VALUE) \
            *((volatile unsigned char *) (MEM_UART_IRDA + (REG))) = (VALUE)

#define READ_UART_REGISTER(REG) \
            *((volatile unsigned char *) (MEM_UART_IRDA + (REG)))

#define WRITE_UART1_REGISTER(REG,VALUE) \
            *((volatile unsigned char *) (MEM_UART_MODEM + (REG))) = (VALUE)

#define READ_UART1_REGISTER(REG) \
            *((volatile unsigned char *) (MEM_UART_MODEM + (REG)))


#define RESET_BIT(REG,BIT)    \
			(WRITE_UART_REGISTER ( \
			      REG, READ_UART_REGISTER ( REG) & ~(1 << (BIT))))

#define SET_BIT(REG,BIT)      \
			(WRITE_UART_REGISTER ( \
			      REG, READ_UART_REGISTER ( REG) | (1 << (BIT))))




char head[] = "\x02\x11\x00\x00\x00\x00\x03";
void Uart_Send_String(char *p)
{
int	i;
    WRITE_UART_REGISTER (IER, 0x00);
	
	while(!(READ_UART_REGISTER ( LSR) & THRE));
	
	for(i=0; i<7; i++)
		WRITE_UART_REGISTER ( THR, head[i]);

	while(*(p)) 
	{
		while(!(READ_UART_REGISTER ( LSR) & THRE));

		if (READ_UART_REGISTER ( LSR) & THRE) 
		{
	       	WRITE_UART_REGISTER ( THR, *p++);
	    }
	}

	while(!(READ_UART_REGISTER ( LSR) & THRE));
	WRITE_UART_REGISTER ( THR, 0x02);
}

void MI_dbgPrintf(const char* const format, ...)
{

	va_list pArgs;
	//return;/*2004/04/01,talcon add for disable trace*/
	va_start(pArgs, format);
	vsprintf(mmi_dbgBuf, format, pArgs);
#ifndef _INNOVATION_EMULATOR_
	Uart_Send_String(mmi_dbgBuf);
#else
	printf(mmi_dbgBuf);
#endif
	va_end(pArgs);
}
#endif
/***
 *	Initialize MMI memory manager.
 */
int MI_mmimem_Init(void)
{
	int i, j;
	T_MMIMEM_PCB *pPCB;
	T_MMIMEM_PMHDR	*pPMH;

#ifndef _INNOVATION_EMULATOR_
	i = NU_Create_Semaphore(&mmimemSem, "s", 1, NU_FIFO);
	MMIMEMTRACE_MSG(("MI_mmimem_Init():Create Sema, ret=%d", i));
	if(i != NU_SUCCESS)
		return EC_MMIMEM_ERROR;	

	/* init dynamic memory management */
	i = NU_Create_Memory_Pool(&mmimemPool, "p", mmiDynMemCB.pStartAddr, mmiDynMemCB.poolSize, MMI_DYNMEM_BLKSZ, NU_FIFO);
	MMIMEMTRACE_MSG(("MI_mmimem_Init():Create dyn mem, ret=%d", i));
	if(i != NU_SUCCESS)
		return EC_MMIMEM_ERROR;	
#endif
	

	/* init partition memory management */	
	for(i = 0; i < MMI_PARTMEM_POOL_NUM; i ++)
	{	
		pPCB = &mmiPCBTbl[i];
		j	 = 1;		
		pPMH = (T_MMIMEM_PMHDR*)pPCB->pStartAddr;
		pPCB->pFree = pPMH;
		
		while(j <= pPCB->totalNum)
		{
			pPMH->pPCB = pPCB;			
			pPMH->next = (T_MMIMEM_PMHDR*)((unsigned char*)pPCB->pStartAddr + (j * (PM_HEADER_LEN + pPCB->partSize)));
			
		#if (MMIMEM_MONITOR == 1)
			memset((void*)pPMH->caller, 0, MMIMEM_CALLER_LEN);
			pPMH->line = 0;	
		#endif
			
			if(j == pPCB->totalNum)
				pPMH->next = NULL;
			else
				pPMH = pPMH->next;

			j ++;	
		}		
	}

#if (MMIMEM_INFORMATION == 1)
	memset((void*)&mmi_memInfo, 0, sizeof(T_MMIMEM_INFO));
	mmi_memInfo.min_reqsz = 0xffffffff;
#endif	

	return EC_MMIMEM_OK;
}


/***
 * Allocate a memory block, if success, return EC_MMIMEM_OK,
 * otherwise return a negative number indicating the reason.
 */
#if (MMIMEM_MONITOR == 1)
EXTERN_LCL int MI_mmimem_Alloc(void **buf, unsigned int size, char *name, int line)
#else
EXTERN_LCL int MI_mmimem_Alloc(void **buf, unsigned int size)
#endif
{
	int i;
	STATUS ret, error = EC_MMIMEM_ERROR;	
	T_MMIMEM_PMHDR *pPMH;
	T_MMIMEM_DMHDR *pDMH;
#if (MMIMEM_MONITOR == 1)
	char *file;
#endif
#if (MMIMEM_UNBLKFREE == 1)
	T_MMIMEM_PMHDR *pPrePMH;
	T_MMIMEM_DMHDR *pPreDMH;
#endif

#if (MMIMEM_MONITOR == 1)
  #ifndef _INNOVATION_EMULATOR_
	file = strrchr(name, '/');
  #else
  	file = strrchr(name, '\\');
  #endif
	if(file)
		file++;		
	MMIMEMTRACE_MSG(("MI_mmimem_Alloc():%d, %d, %s, %d", (UINT32)buf, size, file, line));
#else
	MMIMEMTRACE_MSG(("MI_mmimem_Alloc():%d, %d", (UINT32)buf, size));;
#endif

	if(buf == NULL || size == 0)
	{
		MMIMEMTRACE_MSG(("MI_mmimem_Alloc():para err!"));		
		return error;
	}
	*buf = NULL;
	
#ifndef _INNOVATION_EMULATOR_
	ret = NU_Obtain_Semaphore(&mmimemSem, 20);
	if(ret != NU_SUCCESS)
	{
		MMIMEMTRACE_MSG(("MI_mmimem_Alloc():obtain sema err,ret=%d", ret));
		return error;
	}
#endif

#if (MMIMEM_INFORMATION == 1)
	if(mmi_memInfo.min_reqsz > size)
		mmi_memInfo.min_reqsz = size;
	else if(mmi_memInfo.max_reqsz < size)
		mmi_memInfo.max_reqsz = size;
#endif

	/* allocate from memory pool if possible */
	for(i = 0; i < MMI_PARTMEM_POOL_NUM; i ++)
	{
		if(size <= mmiPCBTbl[i].partSize)
		{		
		#if (MMIMEM_UNBLKFREE == 1)
		 	pPMH	= mmiPCBTbl[i].pAllocated;
		 	pPrePMH = NULL;
		 	
		 	while(pPMH)
		 	{
		 		if(pPMH->inuse == MMI_MEMINUSE_FLAG)
		 		{
		 			pPrePMH = pPMH;
		 			pPMH 	= pPMH->next;
		 		}
		 		else
		 		{
		 			break;
		 		}
		 	}

			if(pPMH != NULL)
			{
				if(pPrePMH == NULL)
				{
					mmiPCBTbl[i].pAllocated = pPMH->next;					
				}
				else
				{
					pPrePMH->next = pPMH->next;
				}

				pPMH->next 	= NULL;
				pPMH->inuse	= MMI_MEMFREE_FLAG;

				pPMH->next 	= mmiPCBTbl[i].pFree;
				mmiPCBTbl[i].pFree	= pPMH;	
				mmiPCBTbl[i].aviableNum++;	
			}			 
		#endif //MMIMEM_UNBLKFREE		
		  
			if(mmiPCBTbl[i].aviableNum > 0)
			{
				*buf = (void*)((unsigned char*)mmiPCBTbl[i].pFree + PM_HEADER_LEN);
				pPMH = mmiPCBTbl[i].pFree;

			#if (MMIMEM_INFORMATION == 1)
				mmi_memInfo.cur_allocpart[i]++;
				if(mmi_memInfo.max_allocpart[i] < mmi_memInfo.cur_allocpart[i])
					mmi_memInfo.max_allocpart[i] = mmi_memInfo.cur_allocpart[i];
			#endif
			
			#if (MMIMEM_UNBLKFREE == 1)
				pPMH->inuse	= MMI_MEMINUSE_FLAG;
			#endif
			#if (MMIMEM_MONITOR == 1)
				pPMH->line	= line;
				memset(pPMH->caller, 0, MMIMEM_CALLER_LEN);
				ret = strlen(name);
				if(ret > (MMIMEM_CALLER_LEN - 1))				
					name = name + (ret - MMIMEM_CALLER_LEN +1);
				strcpy(pPMH->caller, name);
			#endif
				
				mmiPCBTbl[i].pFree 		= pPMH->next;
				pPMH->next				= mmiPCBTbl[i].pAllocated;
				mmiPCBTbl[i].pAllocated = pPMH;				
				mmiPCBTbl[i].aviableNum --;	
				error = EC_MMIMEM_OK;
				MMIMEMTRACE_MSG(("MI_mmimem_Alloc():find one in pool%d,add=%d", i, (UINT32)*buf));
				break;
			}			
			else
			/* the most fitable partition pool no free space, try the bigger partition */
			{
				MMIMEMTRACE_MSG(("MI_mmimem_Alloc():pool%d full, try pool%d", i, i+1));
				continue;
			}
		}
	}

	
	if(error != EC_MMIMEM_OK)	/* can't allocate from partition pools */
	{
		MMIMEMTRACE_MSG(("MI_mmimem_Alloc():all pool no parts, try dynmem,sz=%d", size));
		size = (size + sizeof(int) - 1) & (~(sizeof(int) - 1));		
		size += DM_HEADER_LEN;

	#if (MMIMEM_UNBLKFREE == 1)
		pDMH 	= mmiDynMemCB.pAllocated;
		pPreDMH = NULL;

		while(pDMH)
		{
			while(pDMH)
			{
				if(pDMH->inuse == MMI_MEMINUSE_FLAG)
				{
					pPreDMH	= pDMH;
					pDMH 	= pDMH->next;
				}
				else
				{
					break;
				}
			}

			if(pDMH != NULL)  //find a free block
			{
				if(pPreDMH == NULL)
				{
					mmiDynMemCB.pAllocated = pDMH->next;						
				}
				else
				{
					pPreDMH->next = pDMH->next;
				}			

			#ifndef _INNOVATION_EMULATOR_
				ret = NU_Deallocate_Memory((void*)pDMH);
				if(ret != NU_SUCCESS)
				{
					MMIMEMTRACE_MSG(("MI_mmimem_Free():dealloc to dynmem err,ret=%d",ret));
				}
			#else
				free((void*)pDMH);
			#endif

				//test next memory block
				if(pPreDMH == NULL)
				{
					pDMH = mmiDynMemCB.pAllocated;	
				}
				else
				{
					pDMH = pPreDMH->next;	
				}
			}			
		}
	#endif
			
	#ifndef _INNOVATION_EMULATOR_	
		ret = NU_Allocate_Memory(&mmimemPool, buf, size, 20);
		if(ret != NU_SUCCESS)
		{
			error = EC_MMIMEM_ERROR;
			MMIMEMTRACE_MSG(("MI_mmimem_Alloc():alloc dynmem failed,ret=%d", ret));
		}
		else
			error = EC_MMIMEM_OK;
	#else
		*buf = (void*)malloc(size);
		if(*buf == NULL)
			error = EC_MMIMEM_ERROR;
		else
			error = EC_MMIMEM_OK;
	#endif

		if(error == EC_MMIMEM_OK)
		{
			pDMH = (T_MMIMEM_DMHDR*)*buf;
			*buf = (unsigned char*)pDMH + DM_HEADER_LEN;
			pDMH->pDCB	= &mmiDynMemCB;
			pDMH->next	= mmiDynMemCB.pAllocated;			
			mmiDynMemCB.pAllocated = pDMH;
		#if (MMIMEM_UNBLKFREE == 1)
			pDMH->inuse = MMI_MEMINUSE_FLAG;
		#endif	

			pDMH->size	= size;

		#if (MMIMEM_INFORMATION == 1)
			totalAlloc += (pDMH->size / MMI_DYNMEM_BLKSZ + 1) * MMI_DYNMEM_BLKSZ;
		#endif
		
		#if (MMIMEM_MONITOR == 1)			
			pDMH->line	= line;
			ret = strlen(name);
			if(ret > (MMIMEM_CALLER_LEN - 1))				
				name = name + (ret - MMIMEM_CALLER_LEN +1);
			strcpy(pDMH->caller, name);
		#endif	
		}

	#if (MMIMEM_INFORMATION == 1)
		mmi_memInfo.cur_dyn_used = (totalAlloc * 100) / MMI_DYNMEM_POOL_SIZE;
		if(mmi_memInfo.cur_dyn_used > mmi_memInfo.max_dyn_used)
			mmi_memInfo.max_dyn_used = mmi_memInfo.cur_dyn_used;
	#endif
	}

#ifndef _INNOVATION_EMULATOR_
	ret = NU_Release_Semaphore(&mmimemSem);
	if(ret != NU_SUCCESS)
	{
		MMIMEMTRACE_MSG(("MI_mmimem_Alloc():relese sema err,ret=%d", ret));
	}	
#endif

	if(error != EC_MMIMEM_OK)
	{
		MI_mmimem_Dump();
	}
#if (MMIMEM_INFORMATION == 1)
	else
	{
		if(size > mmi_memInfo.max_allocsz)
			mmi_memInfo.max_allocsz = size;
	}
#endif	
	
	MMIMEMTRACE_MSG(("MI_mmimem_Alloc():allocated buf=%d", *buf));
	return error;
}

/***
 * Free a previously allocated memory block, if success return 0, 
 * otherwise return a negative number indicating the reason.
 */
#if (MMIMEM_MONITOR == 1)
#if (MMIMEM_UNBLKFREE == 1)
EXTERN_LCL int MI_mmimem_Free(void *buf, char* name, int line)
#else //MMIMEM_UNBLKFREE
EXTERN_LCL int MI_mmimem_Free(void *buf, unsigned int suspend, char* name, int line)
#endif 
#else  //MMIMEM_MONITOR
#if (MMIMEM_UNBLKFREE == 1)
EXTERN_LCL int MI_mmimem_Free(void *buf)
#else //MMIMEM_UNBLKFREE
EXTERN_LCL int MI_mmimem_Free(void *buf,  unsigned int suspend)
#endif
#endif 
{
	STATUS ret, error = EC_MMIMEM_ERROR;
	T_MMIMEM_PMHDR *pPMH, *pPrePMH;
	T_MMIMEM_PCB   *pPCB;
	T_MMIMEM_DMHDR *pDMH, *pPreDMH;
	T_MMIMEM_DCB   *pDCB;
#if (MMIMEM_MONITOR == 1)
	char *file;
#endif
	
#if (MMIMEM_MONITOR == 1)
  #ifndef _INNOVATION_EMULATOR_
	file = strrchr(name, '/');
  #else
  	file = strrchr(name, '\\');
  #endif

	if(file)
		file++;	
	MMIMEMTRACE_MSG(("MI_mmimem_Free():%d, %s, %d", (UINT32)buf, file, line));
#else
	MMIMEMTRACE_MSG(("MI_mmimem_Free():%d", (UINT32)buf));
#endif

	if(buf == NULL || (UINT32)buf%4 > 0)
	{
		MMIMEMTRACE_MSG(("MI_mmimem_Free():para err!"));
		return EC_MMIMEM_PTRERROR;
	}

#ifndef _INNOVATION_EMULATOR_
  #if (MMIMEM_UNBLKFREE == 0)
	ret = NU_Obtain_Semaphore(&mmimemSem, suspend);
	if(ret != NU_SUCCESS)
	{
		MMIMEMTRACE_MSG(("MI_mmimem_Free():obtain sema err,ret=%d", ret));
		return EC_MMIMEM_ERROR;
	}
  #endif	
#endif

	if((unsigned char*)buf > (unsigned char*)&mmiPartMem[0] && 
	   (unsigned char*)buf <  (unsigned char*)(&mmiPartMem[0] + MMI_PARTMEM_POOL_SIZE))
	{
		pPMH = (T_MMIMEM_PMHDR*)((unsigned char*)buf - PM_HEADER_LEN);
		pPCB = pPMH->pPCB;
		if(pPCB != NULL && pPCB->magicWord == MMI_PARTMEM_MAGICWORD 
		   && pPCB->totalNum != pPCB->aviableNum)
		{		
			if((void*)pPMH >= pPCB->pStartAddr && (unsigned char*)pPMH < ((unsigned char*)pPCB->pStartAddr + pPCB->poolSize))
			{
				error = EC_MMIMEM_OK;
				
			  #if (MMIMEM_UNBLKFREE == 0)
				if(pPCB->pAllocated == pPMH)
				{
					pPCB->pAllocated = pPMH->next;				
				}
				else
				{
					pPrePMH = pPCB->pAllocated;
					while(pPrePMH)
					{
						if(pPrePMH->next != pPMH)
							pPrePMH = pPrePMH->next;
						else
							break;
					}

					if(pPrePMH != NULL)
					{
						pPrePMH->next = pPMH->next;						
					}
					else
						error = EC_MMIMEM_ERROR;
				}

				if(error == EC_MMIMEM_OK)
				{
					pPMH->next 	= pPCB->pFree;
					pPCB->pFree	= pPMH;	
					pPCB->aviableNum++;
				}
			  #else	//MMIMEM_UNBLKFREE
				pPMH->inuse = MMI_MEMFREE_FLAG;
				
				#if (MMIMEM_INFORMATION == 1)
				mmi_memInfo.cur_allocpart[pPCB->idx]--;
				#endif
				
			  #endif //MMIMEM_UNBLKFREE
			}
		}
	}
#ifndef _INNOVATION_EMULATOR_
	else if((unsigned char*)buf >= (unsigned char*)&mmiDynMem[0] && 
	   		(unsigned char*)buf <  (unsigned char*)&mmiDynMem[0] + MMI_DYNMEM_POOL_SIZE)
#else
	else /* try dynamic memory space */
#endif	
	{
		pDMH = (T_MMIMEM_DMHDR*)((unsigned char*)buf - DM_HEADER_LEN);
		pDCB = pDMH->pDCB;
		if(pDCB != NULL && pDCB->magicWord == MMI_DYNMEM_MAGICWORD
		   && pDCB->pAllocated != NULL)
		{
			error = EC_MMIMEM_OK;
		  #if (MMIMEM_UNBLKFREE == 0)
			if(pDCB->pAllocated == pDMH)
			{
				pDCB->pAllocated = pDMH->next;				
			}
			else
			{
				pPreDMH = pDCB->pAllocated;
				while(pPreDMH)
				{
					if(pPreDMH->next != pDMH)
						pPreDMH = pPreDMH->next;
					else
						break;
				}

				if(pPreDMH != NULL)
				{
					pPreDMH->next = pDMH->next;						
				}
				else
					error = EC_MMIMEM_ERROR;					
			}

			if(error == EC_MMIMEM_OK)
			{
			#ifndef _INNOVATION_EMULATOR_
				ret = NU_Deallocate_Memory((void*)pDMH);
				if(ret != NU_SUCCESS)
				{
					MMIMEMTRACE_MSG(("MI_mmimem_Free():dealloc to dynmem err,ret=%d",ret));
				}
			#else
				free((void*)pDMH);
			#endif
			}
		  #else
			pDMH->inuse = MMI_MEMFREE_FLAG;
			#if (MMIMEM_INFORMATION == 1)
				totalAlloc -= (pDMH->size / MMI_DYNMEM_BLKSZ + 1) * MMI_DYNMEM_BLKSZ;				

				mmi_memInfo.cur_dyn_used = (totalAlloc * 100) / MMI_DYNMEM_POOL_SIZE;
				if(mmi_memInfo.cur_dyn_used > mmi_memInfo.max_dyn_used)
					mmi_memInfo.max_dyn_used = mmi_memInfo.cur_dyn_used;
			#endif

		  #endif //MMIMEM_UNBLKFREE
		}   
	}
#ifndef _INNOVATION_EMULATOR_
	else
	{
		error = EC_MMIMEM_NOTALLOC;
	}
#endif

#ifndef _INNOVATION_EMULATOR_
  #if (MMIMEM_UNBLKFREE == 0)
	ret = NU_Release_Semaphore(&mmimemSem);
	if(ret != NU_SUCCESS)
	{
		MMIMEMTRACE_MSG(("MI_mmimem_Alloc():relese sema err,ret=%d", ret));
	}
  #endif
#endif

	return  error;	
}	

#if (MMIMEM_INFORMATION == 1)
/* Get mmi memory usage information */
EXTERN_LCL void MI_mmimem_Info(T_MMIMEM_INFO *pInfo)
{
	memcpy((void*)pInfo, (void*)&mmi_memInfo, sizeof(T_MMIMEM_INFO));
}


void MI_Dump_mmimem_Info()
{
	int i = 0;
	MMIMEMTRACE_MSG(("****begin dump mmimem info"));
	MMIMEMTRACE_MSG(("min_reqsz:  		%d",mmi_memInfo.min_reqsz));
	MMIMEMTRACE_MSG(("max_reqsz: 		%d",mmi_memInfo.max_reqsz));
	MMIMEMTRACE_MSG(("max_allocsz: 	%d",mmi_memInfo.max_allocsz));
	for(i = 0; i< MMI_PARTMEM_POOL_NUM;i++)
		MMIMEMTRACE_MSG(("max_allocpart:%d, cur_allocpart:%d",mmi_memInfo.max_allocpart[i],mmi_memInfo.cur_allocpart[i]));	
	MMIMEMTRACE_MSG(("max_dyn_used: 	%d",mmi_memInfo.max_dyn_used));
	MMIMEMTRACE_MSG(("cur_dyn_used: 	%d",mmi_memInfo.cur_dyn_used));
	
	MMIMEMTRACE_MSG(("*****end dump mmimem info"));
}

#endif

void MI_mmimem_Dump(void)
{
	
}


#endif

