/*
+--------------------------------------------------------------------+
| PROJECT: MMI-Framework (8417)         $Workfile:: mfw_cnvt.c      $|
| $Author: zhenggl $ CONDAT GmbH           $Revision: 1.2 $|
| CREATED: 03.08.00                     $Modtime:: 03.08.00 18:57   $|
| STATE  : code                                                      |
+--------------------------------------------------------------------+

   MODULE  : MFW_CNVT

   PURPOSE : string convertion

   EXPORT  :

   TO DO   :

   $History:: mfw_cnvt.c                                             $
*/

#define ENTITY_MFW

#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#if defined (NEW_FRAME)

#include "typedefs.h"
#include "vsi.h"
#include "pei.h"
#include "custom.h"
#include "gsm.h"

#else

#include "stddefs.h"
#include "custom.h"
#include "gsm.h"
#include "vsi.h"

#endif

#include "mfw_mfw.h"
#include "mfw_phb.h"
#include "mfw_phbi.h"
#include "mfw_cm.h"
#include "mfw_cmi.h"
#include "mfw_nm.h"
#include "mfw_sim.h"
#include "mfw_sima.h"
#include "mfw_nmi.h"
#include "mfw_simi.h"
#include "mfw_win.h"

#include "mfw_tim.h"

#include "ksd.h"
#include "psa.h"

#if defined (FAX_AND_DATA)
#include "aci_fd.h"
#endif

#include "cmh.h"
#include "phb.h"
#include "cmh_phb.h"

#include "mfw_ss.h"
#include "mfw_ssi.h"

#define PASSED 0
#define FAILED 1
#define LEN_OFFSET 1

extern char ATB_char_GSM(char ascii_character);

/*
+---------------------------------------------------------------+
| PROJECT: MMI-Framework (8417)         MODULE: MFW_CNVT        |
| STATE  : code                         ROUTINE: mfw_getStrMode |
+---------------------------------------------------------------+

   PURPOSE : This function is used to request the coding format
             of a given string. The return value is 
             "MFW_DCS_7bits" or " MFW_DCS_UCS2".

*/

T_MFW_DCS mfw_getStrMode( UBYTE *str, // alpha string in MFW
                          UBYTE len)  // length of alpha string
                         
{
  
  if( str[0] == 0x80)
    return MFW_DCS_UCS2;
  else
    return MFW_DCS_7bits;
}

/*
+---------------------------------------------------------------+
| PROJECT: MMI-Framework (8417)         MODULE: MFW_CNVT        |
| STATE  : code                         ROUTINE: mfw_SIM2GsmStr |
+---------------------------------------------------------------+

   PURPOSE : This function is used to convert the MFW alpha
             formatted string (consisting of alpha name and
             length of name in bytes) to a GSM default string
             (including zero-termination) and to additionally
             set the @ code to 0x80. 
   
*/

T_MFW mfw_SIM2GsmStr( UBYTE inLen, 
                      UBYTE *inMfwStr, 
                      UBYTE maxOutSize,   /* in byte */
                      UBYTE *outGsmStr )
                     
{
  int i;
  UBYTE len;
  
  if (!inLen OR !maxOutSize)
    return FAILED;        /* string is zero length */
  
  if (*inMfwStr EQ 0x80)  /* input UCS, output GSM default */
  {
    len = inLen / 2;      /* GSM string is half length of unicode string */
    for( i=0; i<MINIMUM(len,maxOutSize-1); i++ )
    {
      if (inMfwStr[(i*2)+2] == 0x00)
        * (outGsmStr + i) = 0x80;
      else
        * (outGsmStr + i) = inMfwStr[(i*2)+2]; /* copy every 2nd byte */
    }
    outGsmStr[i] = '\0';
    return PASSED;
  }
  else                   /* input GSM default, output GSM default */
  {
    for( i=0; i<MINIMUM(inLen,maxOutSize-1); i++ )
    {
      if (inMfwStr[i] EQ 0x00)
        * (outGsmStr + i) = 0x80;
      else
        * (outGsmStr + i) = inMfwStr[i];
    }
    outGsmStr[i] = '\0';
    return PASSED;
  }            

}

/*
+---------------------------------------------------------------+
| PROJECT: MMI-Framework (8417)         MODULE: MFW_CNVT        |
| STATE  : code                         ROUTINE: mfw_SIM2UcsStr |
+---------------------------------------------------------------+

   PURPOSE : This function is used to convert the MFW alpha
             formatted string to a unicode string. The first
             two bytes of this unicode string indicates the
             number of characters in the string.
   
*/

T_MFW mfw_SIM2UcsStr( UBYTE   inLen, 
                      UBYTE*  inMfwStr, 
                      UBYTE   maxOutSize,    /* in short */
                      U16*    outUcsStr)
                     
{
  int i;
  short cnt;
  
  if (!inLen OR !maxOutSize)
    return FAILED;
  
  if (inMfwStr[0] EQ 0x80)  /* input UCS, output UCS */
  {
  TRACE_FUNCTION("First byte is indeed 0x80");
    //inMfwStr++;
    inLen--;
    
    // NOTE: do not add '\0' termination!
    // len should not include termination
    cnt = 1;
    for( i=0; i<MINIMUM(inLen/2,maxOutSize-1); i++ )
    {
      outUcsStr[ i + LEN_OFFSET] = ((U16)inMfwStr[cnt] << 8)&0xFF00 + ((U16)inMfwStr[cnt + 1])&0xFF;
      cnt += 2;
    } 
    outUcsStr[0] = i;
    return PASSED;
  }
  else                    /* input GSM default, output UCS */
  {
    for( i=0; i<MINIMUM(inLen,maxOutSize-1); i++ )
      * (outUcsStr + i + LEN_OFFSET) = (U16) inMfwStr[i];
    
    outUcsStr[0] = i;     /* length of unicode string */
    return PASSED;
  }
}

/*
+---------------------------------------------------------------+
| PROJECT: MMI-Framework (8417)         MODULE: MFW_CNVT        |
| STATE  : code                         ROUTINE: mfw_Gsm2SIMStr |
+---------------------------------------------------------------+

   PURPOSE : This function is used to convert a zero-terminated
             GSM default string ( @ code is 0x80) to a MFW 
             alpha formatted string (consisting of alpha name
             and length of name in bytes and @ code is 0x00). 
             The MFW alpha format is dependent on the parameter "outMode" 
             "outMode" is set to either " MFW_DCS_7bits" or " MFW_DCS_UCS2" 

GW-SPR#762- Increased array size - required for larger phonebook entries (>40 chars). 
*/

#define MAX_CONVERT_LEN 255
T_MFW mfw_Gsm2SIMStr( T_MFW_DCS outMode, 
                      UBYTE *inGsmStr, 
                      UBYTE maxOutSize,
                      UBYTE *outMfwStr,
                      UBYTE *outLen )
{
  
  UBYTE temp_str[MAX_CONVERT_LEN];
  int i;
  UBYTE len = (UBYTE) strlen( (char *)inGsmStr );
  char GSM_char;/*SPR2175*/
  
  /* copy string to local storage */
  if (len > MAX_CONVERT_LEN)
  	len = MAX_CONVERT_LEN;
  memcpy( temp_str,inGsmStr,len );
  
  if( outMode EQ MFW_DCS_7bits )
  {
    /* write to outMfwStr */
    for( i=0; i<MINIMUM(len,maxOutSize); i++ )
    {	/*SPR2175, convert to GSM encoding*/
       GSM_char= ATB_char_GSM( temp_str[i] );
       temp_str[i]= GSM_char;
      * (outMfwStr + i) = temp_str[i];
    }
    /* write length of name to outLen */
    *outLen = len;
    return PASSED;
  }
  else if( outMode EQ MFW_DCS_UCS2 )
  {
    /* write to outMfwStr */
    outMfwStr[0] = 0x80;
    for( i=0; i<MINIMUM(len,maxOutSize-1); i++ )
    {
      if( temp_str[i] EQ 0x80 ) /* check for occurance of @ */
        temp_str[i] = 0x00;
      * (outMfwStr + (i*2) + 1) = 0x00;
      * (outMfwStr + (i*2) + 2) = temp_str[i];
    }
    /* write length of name to outLen */
    *outLen = len*2 + 1;
    return PASSED;
  }
  else
  {
    TRACE_EVENT ("mfw_Gsm2SIMStr() - outMode is not valid");
    return FAILED;
  }
  
  return PASSED;
}

/*
+---------------------------------------------------------------+
| PROJECT: MMI-Framework (8417)         MODULE: MFW_CNVT        |
| STATE  : code                         ROUTINE: mfw_Ucs2SIMStr |
+---------------------------------------------------------------+

   PURPOSE : This function is used to convert a unicode string
             to MFW alpha format. The MFW alpha format is 
             dependent on the parameter "outMode". "outMode" 
             is " MFW_DCS_7bits" or " MFW_DCS_UCS2".
   
*/

T_MFW mfw_Ucs2SIMStr( T_MFW_DCS outMode, 
                      U16 *inDcsStr, 
                      UBYTE maxOutSize,
                      UBYTE *outMfwStr,
                      UBYTE *outLen )
{
  int i;
  U16 len;
  
  len = inDcsStr[0];
  
  if( outMode EQ MFW_DCS_7bits )
  {
    /* write to outMfwStr */
    for( i=0; i<MINIMUM(len,maxOutSize); i++ )
    {
      * (outMfwStr + (i*2) + 1) = (UBYTE) (inDcsStr[i+1] & 0xff);
      * (outMfwStr + (i*2)) = (UBYTE) ((inDcsStr[i+1] & 0xff00) >> 8);
    }
    /* write length of name to outLen */
    *outLen = len * 2;
    
    return PASSED;
  }
  else if( outMode EQ MFW_DCS_UCS2 )
  {
    /* write to outMfwStr */
    outMfwStr[0] = 0x80;
    for( i=0; i<len; i++)
    {/*MC, SPR 1752, bytes are no longer swapped*/
      * (outMfwStr + (i*2) + 1) = (UBYTE) (inDcsStr[i+1] & 0xff);
      * (outMfwStr + (i*2) + 2) = (UBYTE) ((inDcsStr[i+1] & 0xff00) >> 8);
    }
    /* write length to outLen */
    *outLen = (len*2) + 1;
    
    return PASSED;
    
  }
  else	/* string does not require conversion to unicode */
  {
    TRACE_EVENT ("mfw_Ucs2SIMStr() - outMode is not valid");
    return FAILED;
  }
  
  return PASSED;
}

/*CONQUEST 5992(related) MC added function to convert GSM string to ASCII*/
static const char gsm_2_ascii_table[128] = { 64, 156, 36, 157, 138, 130, 151, 141, 149, 128,
													 10,   2 ,  7,  32, 143, 134,   4,  95, 232, 226, 239, 234, 227,   5, 228,
											 		233, 240,  32, 146, 145, 225, 144,  32,  33,  34,  35,   1,  37,  38,  39,
											 		 40,  41,  42,  43,  44, 45,  46,  47,  48,  49,  50,  51,  52,  53,  54,
											 		 55,  56,  57,  58,  59,  60,  61,  62,  63, 173,  65,  66,  67,  68,  69,
											 		 70,  71,  72,  73,  74,  75,  76,  77,  78,  79,  80,  81,  82,  83,  84,
											 		 85,  86,  87,  88,  89,  90, 142, 153, 165, 154,  06, 168,  97,  98,  99,
											 		100, 101, 102, 103, 104, 105, 106, 107, 108, 109, 110, 111, 112, 113, 114,
											 		115, 116, 117, 118, 119, 120, 121, 122, 132, 148, 164, 129, 133};
/* PATCH JPS 29.09 END */

/*
+---------------------------------------------------------------------------+
| PROJECT : MMI-Framework (8417)         MODULE  : MFW_SMS                  |
| STATE   : code                         ROUTINE :   mfw_GSM_2_ASCII      |
+---------------------------------------------------------------------------+

  PURPOSE : copy the index and status of a short message

*/
void mfw_GSM_2_ASCII(UBYTE* gsm_string, UBYTE* ascii_string, SHORT length)
{	int i;

	for(i=0;i<length;i++)
	{
		ascii_string[i] = gsm_2_ascii_table[gsm_string[i]];
	}
}

//It's only 0x00~0x1f codes need to convert.
static const unsigned short ascExToUni[]={
0x0000,0x00a3,0x0024,0x00a5,0x00e8,0x00e9,0x00f9,0x00ec,
0x00f2,0x00c7,0x000a,0x00d8,0x00f8,0x000d,0x00c5,0x00e5,
0x0394,0x005f,0x03a6,0x0393,0x0396,0x03a9,0x03a0,0x03a8,
0x03a3,0x0398,0x039e,0x001b,0x00c6,0x00e6,0x00df,0x00c9
};

unsigned short ConvertToUni(char code)
{
    unsigned short code_uni;
    char ch=code;
    if(ch<=0x1f&&ch>=0x00)
        code_uni=ascExToUni[ch];
    else
        code_uni=(unsigned short)ch;

    return code_uni;
}


void AscEx_To_Unicode(char* ascEx_txt, char* uni_txt)
{
    int i=0;
    unsigned short code;
    unsigned char *ss=(unsigned char*)ascEx_txt;

    if(wstrlen((char*)ss)<=0)
    {
        uni_txt[0]=0x80;
        uni_txt[1]=0;
        uni_txt[2]=0;
        return;//ascEx_txt is NULL
    }

    if(*ss==0x80)
    {//if the txt is unicode,copy it to uni_txt
        memcpy(uni_txt,ss,wstrlen((char*)ss)+2);
        return;
    }
    else
    {
        i=0;
        uni_txt[0]=0x80;//set the first code to 0x80
        while(*ss!=0)
        {
            code =ConvertToUni((char)(*ss));
            uni_txt[2*i+1]=(char)((code>>8)&0xff);
            uni_txt[2*i+2]=(char)(code&0xff);
            i++;
            ss++;
        }
        uni_txt[2*i+1]=0x00;
        uni_txt[2*i+2]=0x00;
    }
    return;//convert success
}


/* glowing,2004-06-10, the following is imported from M188A,
   in TCS2112 ATBCommon.c define some code convert function,
   they can replace the following function.
 */

typedef  struct {
	unsigned char sms_code;
	unsigned char ascEx_code;
} _Sms_AscEx_;

//It's only 13 codes need to change form Sms to UniEx
static const _Sms_AscEx_ SmsToAscEx[]={
{0x24,0xa4},{0x40,0xa1},{0x5b,0xc4},{0x5c,0xd6},
{0x5d,0xd1},{0x5e,0xdc},{0x5f,0xa7},{0x60,0xbf},
{0x7b,0xe4},{0x7c,0xf6},{0x7d,0xf1},{0x7e,0xfc},
{0x7f,0xe0},{0x00,0x40}
};

/*for sms_to_asc, it defined in mfw_mfw.h*/
char ConvertToAscEx(char code)
{
    int i=0;
    while(i<14)//14 is SmsToAscEx[] lenth
    {
        if(SmsToAscEx[i].sms_code==(unsigned char)code)
            return(char)(SmsToAscEx[i].ascEx_code);
        i++;
    }
    return code;
}

char ConvertToSms(char code)
{
    int i=0;
    while(i<14)//14 if SmsToAscEx[] lenth
    {
        if(SmsToAscEx[i].ascEx_code==(unsigned char)code)
            return(char)(SmsToAscEx[i].sms_code);
        i++;
    }
    return code;
}


/*First we must check the code if can convert to asciiEx,if not return 0x00(blank).*/
/*Note:the return val is char*/
char ConvertToAscllEx(unsigned short code)
{
    int i=0,found=0;
    if(code<=0x7f) //if the code <=0x7f, direct return code as char
        return(char)code;

    while(i<0x20&&found!=0)
    {
        if(code==ascExToUni[i])
            found=i;
        else
            found=0;
        i++;
    }
    if(found)
        return(char)found;
    else
        return 0x00;
}

/*This function judge the txt if all is ascex code,
if not or txt is none, return 0;
else return 1;
*/
int All_Is_AscEx(char * txt)
{
    int i=0,is_ascex = 1;
    unsigned char * ss=(unsigned char*)txt;

    if((wstrlen((char*)ss)==0)||(*ss!=0x80))//not unicode txt or lenth is 0;
        return 1;

    ss++;//the first code is 0x80, so ss++;
    while(((*ss<<8)+*(ss+1)!=0) && (is_ascex!=0))
    {
        if(*ss!=0)
            is_ascex=0;//it isn't ascii
        else
            is_ascex=1;
        ss+=2;
    }

    return is_ascex;
}
/*
This function convert the sms code to ascEx code;
if sms_txt is null, or sms_txt is unicode (sms_txt[0]==0x80), or convert fault return 0;
else return success, and convert to ascex.
*/
void Sms_To_AscEx(char* sms_txt,char* ascEx_txt,int len)
{
    int i=0;
    unsigned char *ss=(unsigned char*)sms_txt;
    //if(*ss==0x80||len==0) return;//cannot use wstrlen(ss),because sms char maybe 0x00,zhonghz,2002/9/3

    while(i<len)
    {
        ascEx_txt[i++]=ConvertToAscEx((char)(*ss++));
    }
    ascEx_txt[i]=0x00;
    return;//convert success
}

/*
This function convert the ascEx code to sms code;
if ascEx_txt is null, or ascEx_txt is unicode (ascEx_txt[0]==0x80), or convert fault return 0;
else return success, and convert to sms code.
*/
int AscEx_To_Sms(char* ascEx_txt, char* sms_txt)
{
    int i=0;
    unsigned char *ss=(unsigned char*)ascEx_txt;
    if(wstrlen((char*)ss)<=0||*ss==0x80)
    {
        sms_txt="\0";
        return 0;//ascEx_txt is NULL
    }

    while(*ss!=0)
    {
        sms_txt[i++]=ConvertToSms((char)(*ss++));
    }
    sms_txt[i]=0x00;
    return i;//convert success
}

//len is the quantity of 7bits-char in txt_7bits,
//lenth is the lenth of txt_7bits
void Sim_7bits_8bits(char* txt_7bits, int len,char* txt_8bits)
{
    int i=1,j=1,n=0,lenth;

    if(len<=0) return;//txt_7bits is NULL
    lenth=(len+1)*7/8;
    txt_8bits[0]=(txt_7bits[0]>>1)&0x7f;
    while(i<lenth&&j<len)
    {
        n=i%7;
        if(n==0)
        {
            txt_8bits[j++]=(txt_7bits[i++]>>1)&0x7f;
            continue;
        }
        if(j==len-1)
        {
            txt_8bits[j++]=(txt_7bits[i-1])&(0x7f>>(7-n));
            continue;
        }
        txt_8bits[j++]=((txt_7bits[i-1]<<(7-n))+((txt_7bits[i]>>(n+1))&(0x7f>>n)))&0x7f;
        if(n==6)
            txt_8bits[j++]=txt_7bits[i]&0x7f;
        i++;
    }
    txt_8bits[j]=0x00;
    return;//convert success
}


//lenth is the lenth of txt_7bits
int Sim_8bits_7bits(char* txt_8bits, int lenth,char* txt_7bits)
{
    int i=0,n=0;
    unsigned char *ss=(unsigned char*)txt_8bits;
    if(lenth<=0) return 0;
    while(*ss!=0&&n<lenth)
    {
        txt_7bits[i]=((*ss&0x7f)<<(i%7+1))+((*(ss+1)&0x7f)>>(6-i%7));
        i++;
        ss++;
        n++;
        if(i%7==0)
        {
            ss++;
            n++;
        }
    }
    return lenth;
}
/*
This function convert the unicode to ascEx code;
the parameter "lenth" is lenth of unicode_txt.
Note:the lenth must count by every 8bits.
*/
void Unicode_To_AscEx(char* uni_txt, char* ascEx_txt)
{
    int i=0;
    unsigned char ascchar;
    unsigned short tchar;
    unsigned char * ss=(unsigned char*)uni_txt;

    if(wstrlen((char *)ss)==0||*ss!=0x80)
    {//uni_txt is NULL, or it's not unicode
        ascEx_txt="\0";
        return;
    }

    ss++;
    tchar=(*ss<<8)+*(ss+1);
    i=0;
    while(tchar!=0)
    {
        if(tchar>0xff&&(tchar>0x03a9||tchar<0x0393))
            ascEx_txt[i]=0x20;//it's out off ascEx range,set it 0x00;
        else
        {
            if((ascchar=(unsigned char)ConvertToAscllEx(tchar))!=0x00)//if find the code in ascExToUni table
                ascEx_txt[i]=(char)ascchar;
            else if(tchar>0xff)//if tchar >0xff, set it 0x00;
                ascEx_txt[i]=0x20;
            else
                ascEx_txt[i]=(char)tchar&0xff;
        }
        ss+=2;
        i++;
        tchar=(*ss<<8)+*(ss+1);
    }
    ascEx_txt[i]=0x00;
    return;//convert success
}
/*
*/
char * mmi_Sim_Sys(char* intxt,int lenth,T_MFW_DCS in_dcs, int outlen)
{
    int off_set=0;
    int inlen=lenth;
    T_MFW_DCS indcs=in_dcs;
    unsigned char * ss=(unsigned char*)intxt;
    char * tmptr=(char*)malloc(inlen+3);
    static char * tr;
    free(tr);//free it before allot,zhonghz
    tr=(char *)malloc(inlen+3);

    if(indcs==MFW_MAX_DCS)
    {//for phonebook
        if(*ss==0x80)
        {
            indcs=MFW_DCS_UCS2;
            ss++;
        }
        else
            indcs=MFW_DCS_8bits;
    }

    if(inlen==0||(indcs>MFW_ASCII))
    {//if txt is null,return 0;
        memset(tr,0,3);
        free(tmptr);
        return tr;
    }

    if(indcs==MFW_DCS_UCS2)
    {
        //*if(*ss!=0x80){//if the first code is not 0x80,set it to 0x80;
        tr[0]=0x80;
        off_set=1;
        //}
        memcpy(&tr[off_set],ss,inlen);
        if(inlen>outlen)
            inlen=outlen;
        tr[inlen+off_set]=0;
        tr[inlen+off_set+1]=0;
        tr[inlen+off_set+2]=0;//to avoid crash
    }
    else if(indcs==MFW_DCS_7bits)
    {
        memcpy(tr,ss,inlen);
        Sim_7bits_8bits(tr, inlen, tmptr);
        Sms_To_AscEx(tmptr, tr, inlen);
        if(inlen>outlen)
            inlen=outlen;
        tr[inlen]=0;
        tr[inlen+1]=0;//to avoid crash
    }
    else
    {
        memcpy(tmptr,ss,inlen);
        Sms_To_AscEx(tmptr, tr, inlen);
        if(inlen>outlen)
            inlen=outlen;
        tr[inlen]=0;
        tr[inlen+1]=0;//to avoid crash
    }
    free(tmptr);
    return(tr);
}

UBYTE* mmi_Sys_Sim(char* intxt, USHORT * len,SHORT *dcs)
{
    int lenth=wstrlen(intxt);
    static UBYTE * tr;
    UBYTE * tmptr;
    UBYTE * ss=(UBYTE*)intxt;
    free(tr);
    tr=(UBYTE*)malloc(lenth+3);
    tmptr=(UBYTE*)malloc(lenth+3);
    if((UBYTE)*ss==0x80)
    {
        if(All_Is_AscEx((char*)ss))
        {//if all is ascii, convert txt to asciiEx
            Unicode_To_AscEx((char*)ss, (char*)tmptr);
            AscEx_To_Sms((char*)tmptr, (char*)tr);
            *dcs=MFW_DCS_7bits;
            *len=lenth/2;
        }
        else
        {//else it is unicode
            memcpy(tr,&ss[1],lenth+1);//lenth+1 for two 0 in txt end; remove 0x80
            *dcs=MFW_DCS_UCS2;
            *len=lenth-1;
        }
    }
    else
    {
        AscEx_To_Sms((char*)ss,(char*)tr);
        *dcs=MFW_DCS_7bits;
        *len=lenth;
    }
    free(tmptr);
    return(tr);
}

/*for phonebook */
UBYTE* mmi_Sys_Sim_phb(char* intxt, int * outlen)
{
    int lenth=wstrlen(intxt);
    static UBYTE * tr;
    UBYTE * tmptr;
    UBYTE * ss=(UBYTE*)intxt;
    free(tr);
    tr=(UBYTE*)malloc(lenth+3);
    tmptr=(UBYTE*)malloc(lenth+3);
    if((UBYTE)*ss==0x80)
    {
        if(All_Is_AscEx((char*)ss))
        {//if all is ascii, convert txt to asciiEx
            Unicode_To_AscEx((char*)ss, (char*)tmptr);
            AscEx_To_Sms((char*)tmptr, (char*)tr);
            *outlen=lenth/2;
        }
        else
        {//else it is unicode
            memcpy(tr,ss,lenth+2);//lenth+1 for two 0 in txt end; remove 0x80
            *outlen=lenth;
        }
    }
    else
    {
        AscEx_To_Sms((char*)ss,(char*)tr);
        *outlen=lenth;
    }
    free(tmptr);
    return(tr);
}
/* glowing,2004-06-10, end of import */

