/* 
+----------------------------------------------------------------------------- 
|  Project :  GSM-PS (6147)
|  Modul   :  CUS_SMI
+----------------------------------------------------------------------------- 
|  Copyright 2002 Texas Instruments Berlin, AG 
|                 All rights reserved. 
| 
|                 This file is confidential and a trade secret of Texas 
|                 Instruments Berlin, AG 
|                 The receipt of or possession of this file does not convey 
|                 any rights to reproduce or disclose its contents or to 
|                 manufacture, use, or sell anything it may describe, in 
|                 whole, or in part, without the specific written consent of 
|                 Texas Instruments Berlin, AG. 
+----------------------------------------------------------------------------- 
|  Purpose :  Custom dependent definitions for the SMI of the
|             mobile station
|             
|             Use this header for definitions to integrate the
|             protocol stack entity SMI in your target system !
+----------------------------------------------------------------------------- 
*/ 

#ifndef CUS_SMI_H
#define CUS_SMI_H

/*==== CONSTANTS ==================================================*/
/*
 * TIMER_VALUES
 *
 * Description :  The constants define the timer values
 *                for the SMI timer TXXX depending on the various
 *                channel types and service access point identifiers.
 *                If your target system uses other units please
 *                change the values.
 */

/*
 * Unit is 1 ms
 */
#define TSLIDER_VALUE              1000
#define TBACKLIGHT_VALUE           7500
#define TSHIFTKEY_VALUE            3750
#define TWELCOME_VALUE             2000
#ifdef BT_ADAPTER
#define TBT_VALUE                  4000
#endif /* BT_ADAPTER */
#define TBOOTING_VALUE             2000
#define TNIBT_VALUE                20000
#define TLCD_TEST_VALUE            4000
#define TICS_PER_DECIHOURS      (20*60*6*50)


/*
 * MAX_SMI_TIMER
 *
 * Description :  The constant defines the number of timer
 *                available in the timer pool.
 */

#define MAX_SMI_TIMER 4

/*
 * language table
 */

/*
 * constants for the lng_getStatTxt function call
 */
#define STAT_TXT_WAIT                     0
#define STAT_TXT_HELD                     1
#define STAT_TXT_ACTIVE                   2
#define STAT_TXT_DIAL                     3
#define NUM_OF_STAT_TXT_STRINGS           4

/*
 * constants for the lng_getTxt function call
 */
#define TXT_DIALING                       0
#define TXT_ALERT                         1
#define TXT_CONNECTED                     2
#define TXT_NORMAL_SERVICE                3
#define TXT_LIMITD_SERVICE                4
#define TXT_MEN_LANGUAGE                  5
#define TXT_MEN_SEND_DTMF                 6
#define TXT_MEN_KEYBEEP                   7
#define TXT_MEN_START                     8
#define TXT_MEN_STOP                      9
#define TXT_MEN_ON                       10
#define TXT_MEN_OFF                      11
#define TXT_OPTIONS                      12
#define TXT_MEN_ENGLISH                  13
#define TXT_MEN_FRENCH                   14
#define TXT_MEN_GERMAN                   15
#define TXT_MEN_ITALIAN                  16
#define TXT_MEN_SPANISH                  17
#define TXT_NO_SERVICE                   18
#define TXT_MEN_SELECT_PLMN              19
#define TXT_WAITING_CALL                 20
#define TXT_SIM_NOT_INSERT               21
#define TXT_SIM_WRONG                    22
#define TXT_MEN_CALL_TABLE               23
#define TXT_DISCONNECTING                24
#define TXT_CONNECTING                   25
#define TXT_REGISTERING                  26
#define TXT_DEREGISTERING                27
#define TXT_MEN_SMS_LIST                 28
#define TXT_MEN_MSG_VIEW                 29
#define TXT_EMPTY                        30
#define TXT_USER_BUSY                    31
#define TXT_NO_ANSWER                    32
#define NUM_OF_TXT_STRINGS               33  /* must be the last in the list */

/*
 * supported languages
 */
#define LNG_ENGLISH                       0
#define LNG_FRENCH                        1
#define LNG_GERMAN                        2
#define LNG_ITALIEN                       3
#define LNG_SPANISH                       4
#define NUM_OF_LANGUAGES                  5  /* must be the last in the list */

/*
 * for the EVABOARD
 */
#ifdef _TMS470
#define KP_OFF_HOOK 0x61
#define KP_ON_HOOK  0x56
#define KP_UP       0x2D
#define KP_DOWN     0x62
#define KP_SHIFT    0x5E
#define KP_CLEAR    0x43
#define KP_HASH     0x23
#define KP_STAR     0x2A
#define KP_POWER    0x45
#endif
/*
 * for WIN95/NT
 */
#ifdef WIN32
#define KP_OFF_HOOK 0x61
#define KP_ON_HOOK  0x56
#define KP_UP       0x2D
#define KP_DOWN     0x62
#define KP_SHIFT    0x5E
#define KP_CLEAR    0x43
#define KP_HASH     0x23
#define KP_STAR     0x2A
#define KP_POWER    0x45
#endif

#endif