/* 
+----------------------------------------------------------------------------- 
|  Project :  GSM-PS (6147)
|  Modul   :  SMI
+----------------------------------------------------------------------------- 
|  Copyright 2002 Texas Instruments Berlin, AG 
|                 All rights reserved. 
| 
|                 This file is confidential and a trade secret of Texas 
|                 Instruments Berlin, AG 
|                 The receipt of or possession of this file does not convey 
|                 any rights to reproduce or disclose its contents or to 
|                 manufacture, use, or sell anything it may describe, in 
|                 whole, or in part, without the specific written consent of 
|                 Texas Instruments Berlin, AG. 
+----------------------------------------------------------------------------- 
|  Purpose :  Definitions for the Protocol Stack Entity
|             Man Machine Interface
+----------------------------------------------------------------------------- 
*/ 

#ifndef SMI_H
#define SMI_H

#include "gdi.h"

#ifdef BT_ADAPTER
#include "p_btp.h"
#endif /* BT_ADAPTER */

/*==== CONSTANTS ==================================================*/

#define ENCODE_OFFSET 0

/*
 * Dynamic Configuration Numbers
 */
#define RESET                     0
#define KEY_SEQUENCE              6
#define EXT_DISPLAY               7
#define START_SS                  8
#ifdef BT_ADAPTER
#define BT_PRIM                   9
#endif /* BT_ADAPTER */
#define EXT_TEST_ENABLE           10

/*
 * Timer Names
 */
#define TBACKLIGHT                "TBACK"
#define TSLIDER                   "TSLID"
#define TSHIFTKEY                 "TSHIF"
#define TUNIVERSAL                "TUNIV"


#ifdef OPTION_TIMER
#define SMI_TSTART(i,h,v) _tim_start_timer(i,h,v)
#else
#define SMI_TSTART(i,h,v) vsi_t_start(VSI_CALLER h,v)
#endif

/*
 * instance management
 */
#ifdef OPTION_MULTI_INSTANCE
#define GET_INSTANCE(p)       &mmi_data_base[p->custom.route.inst_no]
#define SMI_INSTANCES         MAX_INSTANCES
#else
#define GET_INSTANCE(p)       &mmi_data_base[0]
#define SMI_INSTANCES         1
#endif

#define CR        0x0D /* carriage return    */
#define LF        0x0A /* line feed          */
#define SPACE     ' '  /* space              */
#define NULL_TERM '\0' /* string termination */

/*
 * definitions of the keycodes
 */
#define NONE_KEY      0xff
#define OK_KEY        'a'
#define CLEAR_KEY     'b'
#define ON_HOOK_KEY   'c'
#define OFF_HOOK_KEY  'd'
#define UP_KEY        'e'
#define DOWN_KEY      'f'
#define SHIFT_KEY     'g'
#define F1_KEY        'h'
#define F2_KEY        'i'
#define POWER_KEY     'j'
#define INC_KEY       'k'
#define DEC_KEY       'l'
#define EMERGENCY_KEY '\177'

/*
 * numer for emergency call
 */
#define EMERGENCY_SEQUENCE "112"

/*
 * types of input
 */
#define IT_NUMERIC     1
#define IT_ALPHA       2
#define IT_ECHO        4
#define IT_HIDDEN      8

/*
 * states of an area
 */
#define AREA_HIDDEN    0
#define AREA_DISPLAYED 1

/*
 * functions of an area
 */
#define FN_NONE        0
#define FN_INPUT       1
#define FN_MENU        2
#define FN_SLIDER      3

/*
 * dimensions of the used LCD
 */
#define SCREEN_DX  20
#define SCREEN_DY  4

#define MAX_INPUT_SIZE  (MAX_SM_LEN + 10)
#define MAX_OUTPUT_SIZE SCREEN_DX

/*
 * security related constants
 */
/*#define MAX_PIN_LEN 8 /* maximum length of PIN (defined in SAP SIM) */
#define MAX_PUK_LEN (MAX_PIN_LEN) /* maximum length of PUK */

/*==== TYPES ======================================================*/

typedef struct
{
  T_VERSION * version;
} T_MONITOR;

typedef struct
{
  UBYTE                 t_mode;
  ULONG                 t_val;
} T_TIMER_CONFIG;

typedef struct
{
  BOOL    free;
  UBYTE   state;
  UBYTE   funcType;
  UBYTE   func;
  USHORT  x, y;
  USHORT  dx, dy;
  USHORT  cursx, cursy;
  BOOL    cursActive;
  CHAR    screen[SCREEN_DX*SCREEN_DY];
}
T_AREA;

typedef struct
{
  BOOL    free;
  USHORT  aktInputIdx;
  USHORT  maxInputLen;
  USHORT  aktInputRow;
  USHORT  aktInputCol;
  UBYTE   aktInputType;
  CHAR    aktInput[MAX_INPUT_SIZE+1];
}
T_INPUT;

typedef struct
{
  BOOL    free;
  CHAR    title[MAX_OUTPUT_SIZE+1];
  USHORT  aktMenuIdx;
  USHORT  numEntries;
  CHAR  **menEntries;
  USHORT  numVisible;
}
T_MENU;

typedef struct
{
  BOOL    free;
  USHORT  minValue;
  USHORT  maxValue;
  USHORT  aktValue;
  USHORT  stepValue;
  USHORT  sliderPos;
}
T_SLID;

typedef struct
{
  SHORT           index;
  T_ACI_CAL_STAT  status;
}
T_CTB_INFO;

/*
 * NUM_OF_VTS / T_volType
 *
 * Description :  Defines the number of different volumes and an
 *                enumeration type to select the volume type for
 *                calling the csf_set/getVolume functions.
 */
#define NUM_OF_VTS 5

typedef enum
{
  VT_AUDIO_IN  = 0,
  VT_AUDIO_OUT = 1,
  VT_KEY_BEEP  = 2,
  VT_DTMF_TONE = 3,
  VT_RINGER    = 4
}
T_volType;

#if defined (NEW_FRAME)
#define TIMERSTART(v,h) vsi_t_start (VSI_CALLER h, v);
#define TIMERSTOP(h) vsi_t_stop(VSI_CALLER h);
#else
#define TIMERSTART(v,h) vsi_t_start (VSI_CALLER h, v);
#define TIMERSTOP(h) vsi_t_stop(VSI_CALLER h); aci_flush_fifo (h);
#endif

/*==== EXPORT =====================================================*/
/*
 * only necessary for external display
 */
#ifdef OPTION_MULTITHREAD
  #define hCommPL         _ENTITY_PREFIXED(hCommPL)
  #define hCommACI        _ENTITY_PREFIXED(hCommACI)
  #define hCommSS         _ENTITY_PREFIXED(hCommSS)
#endif

#if defined (NEW_FRAME)
EXTERN T_HANDLE  hCommPL;         /* PL   Communication       */
EXTERN T_HANDLE  hCommACI;        /* MMI  Communication       */
EXTERN T_HANDLE  hCommSS;         /* SS   Communication       */
#else
EXTERN T_VSI_CHANDLE  hCommPL;         /* PL   Communication       */
EXTERN T_VSI_CHANDLE  hCommACI;        /* MMI  Communication       */
EXTERN T_VSI_CHANDLE  hCommSS;         /* SS   Communication       */
#endif
/*
 * MMI global data declarations
 */
EXTERN USHORT screensize_x, screensize_y;
#if defined (NEW_FRAME)
EXTERN T_HANDLE t_slider_running,
                t_backlight_running,
                t_shiftkey_running,
                t_universal_running;
#else
EXTERN T_VSI_THANDLE t_slider_handle,
                     t_backlight_handle,
                     t_shiftkey_handle,
                     t_universal_handle;
#endif
/*
 * no in-band tones
 */
#define NIBT_OFF   (UBYTE)0x00
#define NIBT_ALERT (UBYTE)0x01
#define NIBT_DISC  (UBYTE)0x02
#define NIBT_BUSY  (UBYTE)0x03

/*
 * signals send from ACA->UI
 */
#define UI_SIG_ENTER_PIN                0
#define UI_SIG_ENTER_PUK                1
#define UI_SIG_SIM_NOT_INSERT           2
#define UI_SIG_SIM_WRONG                3
#define UI_SIG_POWER_DOWN               4
#define UI_SIG_POWER_UP                 5
#define UI_SIG_PLMN                     6
#define UI_SIG_SELECT_PLMN              7
#define UI_SIG_CONNECTED                8
#define UI_SIG_BUSY                     9
#define UI_SIG_DISCONNECT               10
#define UI_SIG_ILLEGAL_MS               11
#define UI_SIG_LIMITED_SERVICE          12
#define UI_SIG_NO_CELL                  13
#define UI_SIG_CALL_HELD                14
#define UI_SIG_WAITING_CALL             15
#define UI_SIG_INCOMMING_CALL           16
#define UI_SIG_OUTGOING_CALL            17
#define UI_SIG_RXLEV                    18
#define UI_SIG_BATLEV                   19
#define UI_SIG_SS                       20
#define UI_SIG_CLIP                     21
#define UI_SIG_COLP                     22
#define UI_SIG_UNHND_CME_ERR            23
#define UI_SIG_UNHND_CMS_ERR            24
#define UI_SIG_SHOW_CALL_TABLE          25
#define UI_SIG_DISCONNECTING            26
#define UI_SIG_CONNECTING               27
#define UI_SIG_SSWC                     28
#define UI_SIG_REGISTERING              29
#define UI_SIG_DEREGISTERING            30
#define UI_SIG_REGISTERED               31
#define UI_SIG_DATA_CONNECTED           32
#define UI_SIG_SEC                      33
#define UI_SIG_NO_ANSWER                34
#define UI_SIG_RING_OFF                 35
#define UI_SIG_NIBT                     36

/*
 * SMS signals send from ACA->UI
 */
#define UI_SIG_SMS_REPORT               50
#define UI_SIG_SMS_ALERT                51
#define UI_SIG_SMS_MO                   52
#define UI_SIG_SMS_MT                   53
#define UI_SIG_SMS_CB                   54
#define UI_SIG_SMS_PROC_SUBMIT          100
#define UI_SIG_SMS_PROC_SUBMIT_FROM_MEM 101
#define UI_SIG_SMS_PROC_WRITE           102
#define UI_SIG_SMS_PROC_DELETE          103
#define UI_SIG_SMS_PROC_READ            104
#define UI_SIG_SMS_PROC_LIST            105
#define UI_SIG_SMS_PROC_READ_SINGLE     106
#define UI_SIG_SMS_SHOW_LIST            107
#define UI_SIG_SMS_SHOW_MSG             108
#define UI_SIG_SMS_DELETE               109
#define UI_SIG_SMS_SUBMIT               110
#define UI_SIG_SMS_WRITE                111
#define UI_SIG_SMS_MT_IND               112


/*
 * Bluetooth display message id
 */

#ifdef BT_ADAPTER
typedef enum{
  UI_BT_HS_INIT_SUCCES =0x01,
  UI_BT_HS_INIT_ERROR,
  UI_BT_DU_INIT_SUCCES,
  UI_BT_DU_INIT_ERROR,
  UI_BT_UNKNOWN_DEVICE,
  UI_BT_HS_DEINIT_SUCCES,
  UI_BT_HS_DEINIT_ERROR,
  UI_BT_DU_DEINIT_SUCCES,
  UI_BT_DU_DEINIT_ERROR,
  UI_BT_HS_SEARCH_FINISHED,
  UI_BT_DU_SEARCH_FINISHED,
  UI_BT_HS_FOUND,
  UI_BT_DU_FOUND,
  UI_BT_HS_CONNECTED,
  UI_BT_HS_CONNECT_ERROR,
  UI_BT_DU_CONNECTED,
  UI_BT_DU_CONNECT_ERROR,
  UI_BT_HS_DISCONNECTED,
  UI_BT_DU_DISCONNECTED,
  UI_BT_HS_INIT_START,
  UI_BT_DU_INIT_START,
  UI_BT_HS_SEARCH_START,
  UI_BT_DU_SEARCH_START,
  UI_BT_HS_CONNECTING,
  UI_BT_DU_CONNECTING,
  UI_BT_HS_DISCONNECTING,
  UI_BT_DU_DISCONNECTING,
  UI_BT_HS_DEINIT_START,
  UI_BT_DU_DEINIT_START
}T_UI_BT_MES;

#endif /* BT_ADAPTER */


/*
 * function prototypes
 */

#ifdef OPTION_TIMER
  /*
   * If all entities are linked into one module this definitions
   * prefixes the global data with the entity name
   */
  #ifdef OPTION_MULTITHREAD
    #define _partab    _ENTITY_PREFIXED(_partab)
  #endif

  EXTERN KW_DATA       _partab[];
#endif

/*==== FUNCTION PROTOTYPES CSF ====================================*/

EXTERN void   smi_init_timer (void);
EXTERN void   smi_exit_timer (void);

EXTERN UBYTE  csf_getVolume    (T_volType volType);

EXTERN void   csf_setVolume    (T_volType volType,
                                UBYTE     volume);

EXTERN USHORT csf_getLanguage  (void);

EXTERN void   csf_setLanguage  (USHORT lng);

EXTERN void   csf_getPanelInfo (USHORT* LCD_DX,
                                USHORT* LCD_DY,
                                USHORT* KeyboardType);

/*==== FUNCTION PROTOTYPES UI =====================================*/

EXTERN void       ui_init             (void);

#ifdef BT_ADAPTER
EXTERN void       ui_bt               (T_UI_BT_MES id, UBYTE data[6]);
#endif /* BT_ADAPTER */

EXTERN void       ui_signal           (UBYTE sigCode,
                                       ...);

EXTERN void       ui_sms_signal       (UBYTE sigCode,
                                       ...);

EXTERN void       ui_timeoutSlider    (void);

EXTERN void       ui_timeoutBacklight (void);

EXTERN void       ui_timeoutShiftKey  (void);

EXTERN void       ui_timeoutUniversal (void);

EXTERN const void ui_keypadInd        (UBYTE key_code,
                                       UBYTE key_stat);

EXTERN void       ui_startLcdTest     (void);

EXTERN BOOL       ui_isScrnHeightMin  (void);

EXTERN BOOL       ui_isScrnWidthMin   (void);

/*==== FUNCTION PROTOTYPES WINDOWMANAGER ==========================*/

EXTERN void    wm_init        (void);

EXTERN void    wm_areaOutput  (T_AREA* a,
                               int     row,
                               int     col,
                               CHAR*   s);

EXTERN void    wm_setCursor   (T_AREA* a,
                               BOOL    visible,
                               USHORT  y,
                               USHORT  x);

EXTERN void    wm_areaRefresh (T_AREA* a);

EXTERN USHORT  wm_createArea  (USHORT x,
                               USHORT y,
                               USHORT dx,
                               USHORT dy);

EXTERN void    wm_showArea    (USHORT aHandle);

EXTERN void    wm_hideArea    (USHORT aHandle);

EXTERN void    wm_topArea     (USHORT aHandle);

EXTERN void    wm_clearArea   (USHORT aHandle);

EXTERN void    wm_areaWrite   (USHORT aHandle,
                               UBYTE  row,
                               UBYTE  col,
                               CHAR*  text);

EXTERN T_AREA* wm_getArea     (USHORT aHandle);

/*==== FUNCTION PROTOTYPES INPUT ==================================*/

EXTERN void  inp_init        (void);

EXTERN void  inp_startInput  (USHORT aHandle,
                              UBYTE  row,
                              UBYTE  col,
                              CHAR*  prompt,
                              UBYTE  length,
                              UBYTE  type);

EXTERN void  inp_stopInput   (USHORT aHandle);

EXTERN void  inp_charToInput (USHORT aHandle,
                              CHAR   c);

EXTERN CHAR* inp_getInput    (USHORT aHandle);

EXTERN void  inp_setInput    (USHORT aHandle,
                              CHAR*  inpString);

/*==== FUNCTION PROTOTYPES MENU ===================================*/

EXTERN void   men_init       (void);

EXTERN void   men_startMenu  (USHORT aHandle,
                              CHAR*  title,
                              CHAR*  menu[]);

EXTERN void   men_stopMenu   (USHORT aHandle);

EXTERN void   men_upMenu     (USHORT aHandle);

EXTERN void   men_downMenu   (USHORT aHandle);

EXTERN USHORT men_getMenuIdx (USHORT aHandle);

/*==== FUNCTION PROTOTYPES SLIDER =================================*/

EXTERN void   sld_init           (void);

EXTERN void   sld_startSlider    (USHORT aHandle,
                                  USHORT minValue,
                                  USHORT maxValue,
                                  USHORT aktValue);

EXTERN void   sld_stopSlider     (USHORT aHandle);

EXTERN void   sld_upSlider       (USHORT aHandle);

EXTERN void   sld_downSlider     (USHORT aHandle);

EXTERN USHORT sld_getSliderValue (USHORT aHandle);

/*==== FUNCTION PROTOTYPES LANGUAGE ===============================*/

EXTERN void  lng_init        (void);

EXTERN CHAR* lng_getText     (USHORT txtNum);

EXTERN CHAR* lng_getStatText (USHORT txtNum);

EXTERN void  lng_setLanguage (USHORT lngNum);

/*==== FUNCTION PROTOTYPES TIMER ==================================*/

#if defined (NEW_FRAME)
EXTERN void aci_flush_fifo       (T_HANDLE handle);
#else
EXTERN void aci_flush_fifo       (T_VSI_THANDLE handle);
#endif

#if defined (NEW_FRAME)
/*
 * to achieve backward compatibility with older definitions
 */
#define drv_SignalCB_Type           T_DRV_CB_FUNC
#define drv_SignalID_Type           T_DRV_SIGNAL
#define T_VSI_THANDLE               USHORT
#endif

/*
 * Callback prototypes
 */
EXTERN void kbd_callback (drv_SignalID_Type * signal_params);
EXTERN void pwr_callback (drv_SignalID_Type * signal_params);
EXTERN void rx_callback  (drv_SignalID_Type * signal_params);
EXTERN void em_callback  (drv_SignalID_Type * signal_params);


/*==== FUNCTION PROTOTYPES BT_ADAPTER INTERFACE PRIMITIVE PROCESSING ================*/

#ifdef BT_ADAPTER
EXTERN const void bt_init_profile_cnf       (T_BTP_INIT_PROFILE_CNF   *btp_init_profile_cnf);
EXTERN const void bt_deinit_profile_cnf       (T_BTP_DEINIT_PROFILE_CNF *btp_deinit_profile_cnf);
EXTERN const void bt_device_search_cnf        (T_BTP_DEVICE_SEARCH_CNF    *btp_device_search_cnf);
EXTERN const void bt_device_found_ind       (T_BTP_DEVICE_FOUND_IND   *btp_device_found_ind);
EXTERN const void bt_connect_device_cnf       (T_BTP_CONNECT_DEVICE_CNF *btp_connect_device_cnf);
EXTERN const void bt_connect_device_ind       (T_BTP_CONNECT_DEVICE_IND *btp_connect_device_ind);
EXTERN const void bt_disconnect_device_cnf      (T_BTP_DISCONNECT_DEVICE_CNF *btp_disconnect_device_cnf);
EXTERN const void bt_disconnect_device_ind      (T_BTP_DISCONNECT_DEVICE_IND *btp_disconnect_device_ind);
EXTERN const void bt_audio_connection_ind     (T_BTP_AUDIO_CONNECTION_IND *btp_audio_connection_ind);
EXTERN const void bt_transfer_audio_in_cnf      (T_BTP_TRANSFER_AUDIO_IN_CNF *btp_transfer_audio_in_cnf);
EXTERN const void bt_transfer_audio_out_cnf     (T_BTP_TRANSFER_AUDIO_OUT_CNF *btp_transfer_audio_out_cnf);
EXTERN const void bt_pin_ind            (T_BTP_PIN_IND         *btp_pin_ind);
EXTERN const void bt_reconfig_profile_cnf     (T_BTP_RECONFIG_PROFILE_CNF *btp_reconfig_profile_cnf);
#endif /* BT_ADAPTER */


/*==== EXPORT =====================================================*/

#endif

