/* 
+----------------------------------------------------------------------------- 
|  Project :  GSM-F&D (8411)
|  Modul   :  SMI_ACA
+----------------------------------------------------------------------------- 
|  Copyright 2002 Texas Instruments Berlin, AG 
|                 All rights reserved. 
| 
|                 This file is confidential and a trade secret of Texas 
|                 Instruments Berlin, AG 
|                 The receipt of or possession of this file does not convey 
|                 any rights to reproduce or disclose its contents or to 
|                 manufacture, use, or sell anything it may describe, in 
|                 whole, or in part, without the specific written consent of 
|                 Texas Instruments Berlin, AG. 
+----------------------------------------------------------------------------- 
|  Purpose :  This module defines the interfacing with
|             the ACI on the base of function calls
+----------------------------------------------------------------------------- 
*/ 

#ifndef SMI_ACA_C
#define SMI_ACA_C
#endif

#define ENTITY_SMI

/*==== INCLUDES ===================================================*/
#include <string.h>
#include <stdarg.h>
#include <stdio.h>
#include "typedefs.h"
#include "vsi.h"
#include "custom.h"
#include "gsm.h"
#include "prim.h"
#include "tok.h"
#include "message.h"

#include "aci_cmh.h"
#include "ksd.h"
#include "aca.h"
#include "smi.h"

#include "gdi.h"
#include "audio.h"

#ifdef BT_ADAPTER

#include "dti.h"
#include "bti.h"
#endif
#ifdef BT_INTERFACE
#include "bti_btp.h"
#endif /* BT_ADAPTER */

/*==== CONSTANTS ==================================================*/

/*==== EXPORT =====================================================*/

/*==== VARIABLES ==================================================*/

LOCAL T_ACI_AT_CMD currAbrtCmd  = AT_CMD_NONE;
LOCAL CHAR*        atRedirect[] = { "D", NULL };

#ifdef BT_ADAPTER
static UBYTE dummy[BTI_BD_ADDR_LENGTH];
static UBYTE hs_addr[BTI_BD_ADDR_LENGTH];
static UBYTE du_addr[BTI_BD_ADDR_LENGTH];
#endif /* BT_ADAPTER */

/*==== FUNCTIONS ==================================================*/

EXTERN T_ACI_RETURN cmhSS_ksdIMEI  ( T_ACI_CMD_SRC srcId );

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_ACA                  |
| STATE   : code                  ROUTINE : aca_init                 |
+--------------------------------------------------------------------+

  PURPOSE : This function initializes the ACI adapter parameters.
*/
GLOBAL void aca_init (BOOL isStartup)
{
  /*
   *-----------------------------------------------------------------
   * redirect ATD commands from the AT command interpreter to the
   * extension mechanism if necessary
   *-----------------------------------------------------------------
   */
//  sAT_CfgEXT ( atRedirect );

  /*
   *-----------------------------------------------------------------
   * initialize general parameter
   *-----------------------------------------------------------------
   */
  mmi_data->state       = MMI_OFF;
  mmi_data->serviceType = SRV_NONE;

  /*
   *-----------------------------------------------------------------
   * initialize the modul SMI_REG
   *-----------------------------------------------------------------
   */
  reg_initialize(isStartup);

  /*
   *-----------------------------------------------------------------
   * initialize the modul SMI_SEC
   *-----------------------------------------------------------------
   */
  sec_initialize(isStartup);

  /*
   *-----------------------------------------------------------------
   * initialize the modul SMI_CAL
   *-----------------------------------------------------------------
   */
  cal_initialize(isStartup);

  /*
   *-----------------------------------------------------------------
   * initialize the modul SMI_SS
   *-----------------------------------------------------------------
   */
  ss_initialize(isStartup);

  /*
   *-----------------------------------------------------------------
   * initialize the modul SMI_SMS
   *-----------------------------------------------------------------
   */
  sms_initialize(isStartup);
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_ACA                  |
| STATE   : code                  ROUTINE : aca_setupVolume          |
+--------------------------------------------------------------------+

  PURPOSE : This function sets the volume for the audio input and
            audio output path.

            <audioIn>:  input volume
            <audioOut>: output volume

            returns:    status information concerning the setting
                        progress
*/
GLOBAL T_ACI_RETURN aca_setupVolume (UBYTE audioIn, UBYTE audioOut)
{
  TRACE_FUNCTION ("aca_setupVolume ()");

  audio_SetAmplf (AUDIO_MICROPHONE, audioIn);
  audio_SetAmplf (AUDIO_SPEAKER,    audioOut);

  return (AT_CMPL);
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_ACA                  |
| STATE   : code                  ROUTINE : rAT_PercentSQ            |
+--------------------------------------------------------------------+

  PURPOSE : This function is called when the RX level changes with
            respect to the further set profile.

            <segm>: number of RX level segment
*/
GLOBAL void rAT_PercentSQ (BYTE segm)
{
  TRACE_FUNCTION ("rAT_PercentSQ ()");

  ui_signal (UI_SIG_RXLEV, segm);
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_ACA                  |
| STATE   : code                  ROUTINE : rAT_PercentBC            |
+--------------------------------------------------------------------+

  PURPOSE : This function is called when battery charge changes with
            respect to the further set profile.

            <segm>: number of battery charge segment
*/
GLOBAL void rAT_PercentBC (BYTE segm)
{
  TRACE_FUNCTION ("rAT_PercentBC ()");

  ui_signal (UI_SIG_BATLEV, segm);
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_ACA                  |
| STATE   : code                  ROUTINE : aca_signal               |
+--------------------------------------------------------------------+

  PURPOSE : This function processs the ACA_SIG_XXX signals that are
            send by the MMI_UI logic.
*/
static USHORT aca_siglevel = 0; /* used to indicate a recursion */

GLOBAL void aca_signal (UBYTE sigCode, ...)
{
  va_list varpars;
  CHAR    buf[30];

  sprintf (buf, "[%d], ACA-Signal: %d", aca_siglevel++, sigCode);
  TRACE_EVENT (buf);

  switch (sigCode)
  {
    /*
     *---------------------------------------------------------------
     * handling of signal ACA_SIG_KEYSEQUENCE
     *---------------------------------------------------------------
     */
    case ACA_SIG_KEYSEQUENCE:
    {
      static T_KEYSEQ_HND  hnd;   /* handle for key sequence */
                                  /* decoding                */
      CHAR                *input; /* keystroke sequence      */

      va_start (varpars, sigCode);
      input = va_arg (varpars, CHAR*);
      va_end (varpars);

      aca_keySeqHndInit (&hnd);
      aca_keySeqInd (input, &hnd);

      break;
    }

    /*
     *---------------------------------------------------------------
     * handling of signal ACA_SIG_PININPUT
     *---------------------------------------------------------------
     */
    case ACA_SIG_PININPUT:
    {
      CHAR *input; /* entered PIN */

      va_start (varpars, sigCode);
      input = va_arg (varpars, CHAR*);
      va_end (varpars);

/*
      if (mmi_data->state EQ MMI_WAIT_FOR_PIN)
*/
          sec_pinEntered (input);

      break;
    }

    /*
     *---------------------------------------------------------------
     * handling of signal ACA_SIG_PUKINPUT
     *---------------------------------------------------------------
     */
    case ACA_SIG_PUKINPUT:
    {
      CHAR *puk;   /* entered PUK                     */
      CHAR *npin1; /* entered new pin the first time  */
      CHAR *npin2; /* entered new pin the second time */

      va_start (varpars, sigCode);
      puk   = va_arg (varpars, CHAR*);
      npin1 = va_arg (varpars, CHAR*);
      npin2 = va_arg (varpars, CHAR*);
      va_end (varpars);

      if (mmi_data->state EQ MMI_WAIT_FOR_PUK)
          sec_pukEntered (puk, npin1, npin2);

      break;
    }

    /*
     *---------------------------------------------------------------
     * handling of signal ACA_SIG_POWER_ON
     *---------------------------------------------------------------
     */
    case ACA_SIG_POWER_ON:
      reg_powerOn ();
      break;

    /*
     *---------------------------------------------------------------
     * handling of signal ACA_SIG_POWER_OFF
     *---------------------------------------------------------------
     */
    case ACA_SIG_POWER_OFF:
      reg_powerOff ();
      break;

    /*
     *---------------------------------------------------------------
     * handling of signal ACA_SIG_RELEASE
     *---------------------------------------------------------------
     */
    case ACA_SIG_RELEASE:
      cal_releaseCall ();
      break;

    /*
     *---------------------------------------------------------------
     * handling of signal ACA_SIG_ACCEPT
     *---------------------------------------------------------------
     */
    case ACA_SIG_ACCEPT:
      cal_acceptCall ();
      break;

    /*
     *---------------------------------------------------------------
     * handling of signal ACA_SIG_SEND_DTMF
     *---------------------------------------------------------------
     */
    case ACA_SIG_SEND_DTMF:
    {
      CHAR key; /* DTMF key */
      UBYTE stat;

      va_start (varpars, sigCode);
      key  = va_arg (varpars, CHAR);
      stat = va_arg (varpars, UBYTE);
      va_end (varpars);

      cal_sendDtmf (key, stat EQ KEY_STAT_PRS);
      break;
    }

    /*
     *---------------------------------------------------------------
     * handling of signal ACA_SIG_REGISTER
     *---------------------------------------------------------------
     */
    case ACA_SIG_REGISTER:
    {
      va_start (varpars, sigCode);
      reg_setOperName (va_arg (varpars, CHAR*));
      va_end (varpars);

      reg_startReg ();

      break;
    }

    /*
     *---------------------------------------------------------------
     * handling of signal ACA_SIG_VOLUME
     *---------------------------------------------------------------
     */
    case ACA_SIG_VOLUME:
    {
      UBYTE volume; /* level of volume */

      va_start (varpars, sigCode);
      volume = va_arg (varpars, UBYTE);
      va_end (varpars);

      aca_setupVolume (volume, volume);
      break;
    }
  }
  aca_siglevel--;
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_ACA                  |
| STATE   : code                  ROUTINE : aca_keySeqHndInit        |
+--------------------------------------------------------------------+

  PURPOSE : This function initializes the structure for handling
            of keystroke sequences.

            <hnd>: keystroke sequence decoder handle
*/
GLOBAL void aca_keySeqHndInit (T_KEYSEQ_HND* hnd)
{
  *hnd->keySeq  = NULL_TERM;
  hnd->nextSeq  = hnd->keySeq;
  hnd->restSeq  = hnd->nextSeq;
  hnd->grp      = SEQGRP_UNKNOWN;
  hnd->atReturn = AT_FAIL;
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_ACA                  |
| STATE   : code                  ROUTINE : aca_keySeqInd            |
+--------------------------------------------------------------------+

  PURPOSE : This function is used to decode a keystroke sequence and
            to initiate an appropriate action.

            <inSeq>: incoming keystroke sequence
            <hnd>:   keystroke sequence decoder handle
*/
GLOBAL void aca_keySeqInd (CHAR* inSeq, T_KEYSEQ_HND* hnd)
{

  TRACE_FUNCTION ("aca_keySeqInd ()");

  if (strlen (hnd->nextSeq) NEQ 0 OR
      strlen (inSeq)         EQ 0)
    return;

  /*
   *-----------------------------------------------------------------
   * make a copy of the key sequence
   *-----------------------------------------------------------------
   */
  strncpy (hnd->keySeq, inSeq, MAX_KEYSEQ_SIZE);
  hnd->keySeq[MAX_KEYSEQ_SIZE] = NULL_TERM;

  hnd->nextSeq  = hnd->keySeq;
  hnd->restSeq  = hnd->nextSeq;
  hnd->grp      = SEQGRP_UNKNOWN;
  hnd->atReturn = AT_FAIL;

  do
  {
    if (ksd_decode (hnd->nextSeq,
                    cal_isWithinCall (),
                    &hnd->grp,
                    &hnd->restSeq,
                    &hnd->param)         EQ TRUE)
    {
      switch (hnd->grp)
      {
        /*
         *---------------------------------------------------------
         * handling of sequence group SEQGRP_MS_OFF
         *---------------------------------------------------------
         */
        case (SEQGRP_MS_OFF):
          hnd->atReturn = reg_powerOff ();
          break;

        /*
         *---------------------------------------------------------
         * handling of sequence group SEQGRP_MS_ON
         *---------------------------------------------------------
         */
        case (SEQGRP_MS_ON):
          hnd->atReturn = reg_powerOn ();
          break;

        /*
         *---------------------------------------------------------
         * handling of sequence group SEQGRP_HOOK_OFF
         *---------------------------------------------------------
         */
        case (SEQGRP_HOOK_OFF):
          hnd->atReturn = cal_acceptCall ();
          break;

        /*
         *---------------------------------------------------------
         * handling of sequence group SEQGRP_HOOK_ON
         *---------------------------------------------------------
         */
        case (SEQGRP_HOOK_ON):
          hnd->atReturn = cal_releaseCall ();
          break;

        /*
         *---------------------------------------------------------
         * handling of sequence group SEQGRP_DTMF
         *---------------------------------------------------------
         */
        case (SEQGRP_DTMF):
          hnd->atReturn = cal_sendDtmf (hnd->param.dtmf, TRUE);
          hnd->atReturn = cal_sendDtmf (hnd->param.dtmf, FALSE);
          break;

        /*
         *---------------------------------------------------------
         * handling of sequence group SEQGRP_KEYPAD_IND
         *---------------------------------------------------------
         */
        case (SEQGRP_KEYPAD_IND):
          ui_keypadInd (hnd->param.keypad.keyCode, KEY_STAT_PRS);
          ui_keypadInd (hnd->param.keypad.keyCode, KEY_STAT_REL);

          hnd->atReturn = AT_CMPL;
          break;

        /*
         *---------------------------------------------------------
         * handling of sequence group SEQGRP_LCD_TEST
         *---------------------------------------------------------
         */
        case (SEQGRP_LCD_TEST):
          ui_startLcdTest ();

          hnd->atReturn = AT_CMPL;
          break;

        /*
         *---------------------------------------------------------
         * handling of sequence group SEQGRP_SHOW_CALL_TABLE
         *---------------------------------------------------------
         */
        case (SEQGRP_SHOW_CALL_TABLE):
          hnd->atReturn = cal_showCallTable ();
          break;

        /*
         *---------------------------------------------------------
         * handling of sequence group SEQGRP_SET_ABBR_DIAL
         *---------------------------------------------------------
         */
        case (SEQGRP_SET_ABBR_DIAL):
          cal_setAbbrDial (hnd->param.abbrDial.index,
                           hnd->param.abbrDial.number);
          hnd->atReturn = AT_CMPL;
          break;

        /*
         *---------------------------------------------------------
         * handling of sequence group SEQGRP_CHANGE_REGISTER
         *---------------------------------------------------------
         */
        case (SEQGRP_CHANGE_REGISTER):
          hnd->atReturn = reg_setReg (&hnd->param.cops);

          if (hnd->atReturn EQ AT_CMPL)
            hnd->atReturn = reg_changeReg ();
          break;

        /*
         *---------------------------------------------------------
         * handling of sequence group SEQGRP_SET_REGISTER
         *---------------------------------------------------------
         */
        case (SEQGRP_SET_REGISTER):
          hnd->atReturn = reg_setReg (&hnd->param.cops);
          break;

        /*
         *---------------------------------------------------------
         * handling of sequence group SEQGRP_START_REGISTER
         *---------------------------------------------------------
         */
        case (SEQGRP_START_REGISTER):
          hnd->atReturn = reg_setReg (&hnd->param.cops);

          if (hnd->atReturn EQ AT_CMPL)
            hnd->atReturn = reg_startReg ();
          break;

        /*
         *---------------------------------------------------------
         * handling of sequence group SEQGRP_SMS_SEND
         *---------------------------------------------------------
         */
        case (SEQGRP_SMS_SEND):
          hnd->atReturn = sms_startInputProc (SMS_PROC_SEND);
          break;

        /*
         *---------------------------------------------------------
         * handling of sequence group SEQGRP_SMS_SEND_FROM_MEM
         *---------------------------------------------------------
         */
        case (SEQGRP_SMS_SEND_FROM_MEM):
          hnd->atReturn = sms_startInputProc (SMS_PROC_SEND_FROM_MEM);
          break;

        /*
         *---------------------------------------------------------
         * handling of sequence group SEQGRP_SMS_WRITE
         *---------------------------------------------------------
         */
        case (SEQGRP_SMS_WRITE):
          hnd->atReturn = sms_startInputProc (SMS_PROC_WRITE);
          break;

        /*
         *---------------------------------------------------------
         * handling of sequence group SEQGRP_SMS_DELETE
         *---------------------------------------------------------
         */
        case (SEQGRP_SMS_DELETE):
          hnd->atReturn = sms_startInputProc (SMS_PROC_DELETE);
          break;

        /*
         *---------------------------------------------------------
         * handling of sequence group SEQGRP_SMS_READ
         *---------------------------------------------------------
         */
        case (SEQGRP_SMS_READ):
          hnd->atReturn = sms_startInputProc (SMS_PROC_READ);
          break;

        /*
         *---------------------------------------------------------
         * handling of sequence group SEQGRP_SMS_LIST
         *---------------------------------------------------------
         */
        case (SEQGRP_SMS_LIST):
          hnd->atReturn = sms_startInputProc (SMS_PROC_LIST);
          break;

        /*
         *---------------------------------------------------------
         * handling of sequence group SEQGRP_SMS_READ_SINGLE
         *---------------------------------------------------------
         */
        case (SEQGRP_SMS_READ_SINGLE):
          hnd->atReturn = sms_startInputProc (SMS_PROC_READ_SINGLE);
          break;

        /*
         *---------------------------------------------------------
         * handling of sequence group SEQGRP_DIAL
         *---------------------------------------------------------
         */
        case (SEQGRP_DIAL):
          hnd->atReturn = cal_setupCall (&hnd->param.dial);
          break;

        /*
         *---------------------------------------------------------
         * handling of sequence groups which are handled by the
         * ACI dirctly by using the sAT_Dn function
         *---------------------------------------------------------
         */
        case (SEQGRP_CB):
          hnd->atReturn = ss_changeCb (inSeq, &hnd->param.cb);
          break;

        /*
         *---------------------------------------------------------
         * handling of sequence groups which are handled by the
         * ACI dirctly by using the sAT_Dn function
         *---------------------------------------------------------
         */
        case (SEQGRP_CF):
          hnd->atReturn = ss_changeCf (inSeq, &hnd->param.cf);
          break;

        /*
         *---------------------------------------------------------
         * handling of sequence groups which are handled by the
         * ACI dirctly by using the sAT_Dn function
         *---------------------------------------------------------
         */
        case (SEQGRP_CL):
          switch (hnd->param.cl.ssCd)
          {
            case (KSD_SS_CLIP):
              hnd->atReturn = ss_intrgtClip ();
              break;

            case (KSD_SS_CLIR):
              hnd->atReturn = ss_intrgtClir ();
              break;

            case (KSD_SS_COLP):
              hnd->atReturn = ss_intrgtColp ();
              break;

            case (KSD_SS_COLR):
              break;
          }
          break;

        /*
         *---------------------------------------------------------
         * handling of sequence groups which are handled by the
         * ACI dirctly by using the sAT_Dn function
         *---------------------------------------------------------
         */
        case (SEQGRP_CW):
          hnd->atReturn = ss_changeCw (inSeq, &hnd->param.cw);
          break;

        /*
         *---------------------------------------------------------
         * handling of sequence groups which are handled by the
         * ACI dirctly by using the sAT_Dn function
         *---------------------------------------------------------
         */
        case (SEQGRP_PWD):
          hnd->atReturn = sec_regPasswd (inSeq, &hnd->param.pwd);
          break;

        /*
         *---------------------------------------------------------
         * handling of sequence groups which are handled by the
         * ACI dirctly by using the sAT_Dn function
         *---------------------------------------------------------
         */
        case (SEQGRP_UBLK):
          break;

        /*
         *---------------------------------------------------------
         * handling of sequence groups which are handled by the
         * ACI dirctly by using the sAT_Dn function
         *---------------------------------------------------------
         */
        case (SEQGRP_USSD):
          break;

        /*
         *---------------------------------------------------------
         * handling of sequence groups which results in a change
         * of CLIR parameter
         * in temporary mode a call is set up
         *---------------------------------------------------------
         */
        case (SEQGRP_SUP_CLIR):
        case (SEQGRP_INV_CLIR):
          if (strlen (hnd->restSeq) EQ 0)
            hnd->atReturn = ss_changeClir (&hnd->param.Clir);
          else
          {
            /*
             *-----------------------------------------------------
             * in temporary mode a call is set up
             *-----------------------------------------------------
             */
            T_KSD_CLIR   Clir;
            T_KSD_SEQGRP nextGrp  = SEQGRP_UNKNOWN;
            CHAR*        remSeq   = NULL;

            Clir.mode = hnd->param.Clir.mode;

            if (ksd_decode (hnd->restSeq, FALSE,
                            &nextGrp,     &remSeq,
                            &hnd->param)           EQ TRUE
                AND
                nextGrp EQ SEQGRP_DIAL)
            {
              switch (Clir.mode)
              {
                case (CLIR_MOD_Invoc):
                  hnd->param.dial.clirOvrd = D_CLIR_OVRD_Invoc;
                  break;

                case (CLIR_MOD_Supp):
                  hnd->param.dial.clirOvrd = D_CLIR_OVRD_Supp;
                  break;

                default:
                  hnd->param.dial.clirOvrd = D_CLIR_OVRD_Default;
                  break;
              }

              hnd->atReturn = cal_setupCall (&hnd->param.dial);
            }
          }
          break;

        /*
         *---------------------------------------------------------
         * handling of sequence groups which results in a change
         * of CLIP parameter
         *---------------------------------------------------------
         */
        case (SEQGRP_SUP_CLIP):
        case (SEQGRP_INV_CLIP):
          hnd->atReturn = ss_changeClip (&hnd->param.Clip);
          break;

        /*
         *---------------------------------------------------------
         * handling of sequence groups which results in a change
         * of COLR parameter
         *---------------------------------------------------------
         */
        case (SEQGRP_SUP_COLR):
        case (SEQGRP_INV_COLR):
          break;

        /*
         *---------------------------------------------------------
         * handling of sequence groups which results in a change
         * of COLP parameter
         *---------------------------------------------------------
         */
        case (SEQGRP_SUP_COLP):
        case (SEQGRP_INV_COLP):
          hnd->atReturn = ss_changeColp (&hnd->param.Colp);
          break;

        /*
         *---------------------------------------------------------
         * handling of sequence group SEQGRP_PRSNT_IMEI
         *---------------------------------------------------------
         */
        case (SEQGRP_PRSNT_IMEI):
          hnd->atReturn = cmhSS_ksdIMEI  ( CMD_SRC_LCL );

          break;

        /*
         *---------------------------------------------------------
         * handling of sequence group SEQGRP_ACT_SIM_LOCK
         *---------------------------------------------------------
         */
        case (SEQGRP_ACT_SIM_LOCK):
          break;

        /*
         *---------------------------------------------------------
         * handling of sequence group SEQGRP_DEACT_SIM_LOCK
         *---------------------------------------------------------
         */
        case (SEQGRP_DEACT_SIM_LOCK):
          break;

        /*
         *---------------------------------------------------------
         * handling of sequence group SEQGRP_INTRGT_SIM_LOCK
         *---------------------------------------------------------
         */
        case (SEQGRP_INTRGT_SIM_LOCK):
          break;

        /*
         *---------------------------------------------------------
         * handling of sequence group SEQGRP_SND_CHLD
         *---------------------------------------------------------
         */
        case (SEQGRP_CHLD):
          hnd->atReturn = cal_changeCall (&hnd->param.chld);
          break;

        /*
         *---------------------------------------------------------
         * handling of sequence group SEQGRP_SND_CTFR
         *---------------------------------------------------------
         */
        case (SEQGRP_CTFR):
          hnd->atReturn = cal_invokeCTFR(&hnd->param.ctfr);
          break;

        /*
         *---------------------------------------------------------
         * handling of sequence group SEQGRP_SND_CBST
         *---------------------------------------------------------
         */
        case (SEQGRP_SND_CBST):
          hnd->atReturn = cal_setCbst (&hnd->param.cbst);
          break;

        /*
         *---------------------------------------------------------
         * handling of sequence group SEQGRP_SND_CRLP
         *---------------------------------------------------------
         */
        case (SEQGRP_SND_CRLP):
          hnd->atReturn = cal_setCrlp (&hnd->param.crlp);
          break;

        /*
         *---------------------------------------------------------
         * handling of sequence group SEQGRP_SND_DS
         *---------------------------------------------------------
         */
        case (SEQGRP_SND_DS):
          hnd->atReturn = cal_setDs (&hnd->param.ds);
          break;

        /*
         *---------------------------------------------------------
         * handling of sequence group SEQGRP_SPEECH_DATA
         *---------------------------------------------------------
         */
        case (SEQGRP_SPEECH_DATA):
          break;

        /*
         *---------------------------------------------------------
         * handling of sequence group SEQGRP_DATA_SPEECH
         *---------------------------------------------------------
         */
        case (SEQGRP_DATA_SPEECH):
          break;

        /*
         *---------------------------------------------------------
         * handling of sequence group SEQGRP_EM_MODE
         *---------------------------------------------------------
         */
#if defined (EM_MODE)
        case (SEQGRP_EM_MODE):
          if (!strcmp (hnd->restSeq, "*0"))
          {
            // enable recording
            em_Enable_Post_Mortem ();
          }
          if (!strcmp (hnd->restSeq, "*1"))
          {
            // disable recording
            em_Disable_Post_Mortem ();
          }
          if (!strcmp (hnd->restSeq, "*2"))
          {
            UBYTE dump = TRUE;
            em_data_type em_data;
            UBYTE i, max_i;
            char buf[60];

            while (dump)
            {
              switch (em_Read_Post_Mortem (&em_data))
              {
                case DRV_OK:
                  sprintf (buf, ">PM %x %x %x %x",
                           em_data.em_class,
                           em_data.em_subclass,
                           em_data.em_type,
                           em_data.em_length);
                  if (em_data.em_length > 10)
                    max_i = 10;
                  else
                    max_i = i;
                  for (i=0;i<max_i;i++)
                    sprintf (buf+strlen (buf)," %x", em_data.em_parameter[i]);
                  TRACE_EVENT (buf);
                  break;
                case EM_NO_MORE_DATA:
                case EM_INVALID_ACCESS:
                  dump = FALSE;
                  break;
              }
            }
          }
          break;
#endif
        /*
         *---------------------------------------------------------
         * handling of sequence group SEQGRP_UNKNOWN
         *---------------------------------------------------------
         */
        case (SEQGRP_UNKNOWN):
          TRACE_EVENT ("smi_aca.c: aca_keySeqInd: sequence group unknown");
          ui_signal (UI_SIG_UNHND_CME_ERR, CME_ERR_Unknown);
          aca_keySeqHndInit (hnd);
          break;


}

      if (hnd->atReturn EQ AT_FAIL)
      {
        aca_keySeqHndInit (hnd);
        return;
      }
      else
      {
        hnd->nextSeq = hnd->restSeq;
      }
    }
    else
    {
      TRACE_EVENT ("smi_aca.c: aca_keySeqInd: ksd_decode failed");
      ui_signal (UI_SIG_UNHND_CME_ERR, CME_ERR_Unknown);
      aca_keySeqHndInit (hnd);
      return;
    }
  }
  while (hnd->atReturn EQ AT_CMPL AND strlen (hnd->nextSeq) NEQ 0);
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_ACA                  |
| STATE   : code                  ROUTINE : rAT_PlusCME              |
+--------------------------------------------------------------------+

  PURPOSE : This function will be called by the ACI when an error
            inside the ME occured.

            <cmdId>: command identity
            <err>:   error code
*/
GLOBAL void rAT_PlusCME (T_ACI_AT_CMD cmdId, T_ACI_CME_ERR err)
{
  CHAR buf[30];

  sprintf (buf, "rAT_PlusCME (%d, %d)", cmdId, err);
  TRACE_FUNCTION (buf);

  switch (cmdId)
  {
    case (AT_CMD_ABRT): rAT_PlusCME    (currAbrtCmd, err);
                        currAbrtCmd = AT_CMD_NONE;         break;
    case (AT_CMD_CFUN): reg_CfunRej          (cmdId, err); break;
    case (AT_CMD_CPIN): sec_CpinRej          (cmdId, err); break;
    case (AT_CMD_COPS): reg_CopsRej          (cmdId, err); break;
    case (KSD_CMD_CF) : ss_CcfcRej           (cmdId, err); break;
    case (KSD_CMD_CB) : ss_ClckRej           (cmdId, err); break;
    case (KSD_CMD_CW) : ss_CcwaRej           (cmdId, err); break;
    case (KSD_CMD_PWD): sec_CpwdRej          (cmdId, err); break;
    case (AT_CMD_D)   : cal_DRejWhilePlusCME (cmdId, err); break;
    case (AT_CMD_H)   : cal_HRejWhilePlusCME (cmdId, err); break;
    case (AT_CMD_Z)   : cal_HRejWhilePlusCME (cmdId, err); break;
    case (AT_CMD_A)   : cal_ARejWhilePlusCME (cmdId, err); break;
    case (AT_CMD_CHLD): cal_ChldCtfrRej      (cmdId, err); break;
    case (AT_CMD_VTS) : cal_VtsRej           (cmdId, err); break;
    case (KSD_CMD_CL)  :                                   break;
    case (KSD_CMD_UBLK):                                   break;
    case (KSD_CMD_USSD):                                   break;

    default:
      ui_signal (UI_SIG_UNHND_CME_ERR, err);
      break;
  }
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_ACA                  |
| STATE   : code                  ROUTINE : rAT_OK                   |
+--------------------------------------------------------------------+

  PURPOSE : This function will be called by the ACI to signal the
            correct processing of a previous stimulated ACI
            transaction.

            <cmdId>: command identity
*/
#ifdef AT_INTERPRETER
EXTERN T_ACI_AT_CMD currExtCmd;
EXTERN void ext_OK ( T_ACI_AT_CMD cmdId );
#endif

GLOBAL void rAT_OK (T_ACI_AT_CMD cmdId)
{
  CHAR buf[20];

  sprintf (buf, "rAT_OK (%d)", cmdId);
  TRACE_FUNCTION (buf);

#ifdef AT_INTERPRETER
  if (currExtCmd NEQ AT_CMD_NONE)
  {
    ext_OK (cmdId);
  }
  else
#endif
  {
    switch (cmdId)
    {
      case (AT_CMD_ABRT) : rAT_OK (currAbrtCmd);
                           currAbrtCmd = AT_CMD_NONE; break;
      case (AT_CMD_CFUN) : reg_CfunCnf  (cmdId);      break;
      case (AT_CMD_CPIN) : sec_CpinCnf  (cmdId);      break;
      case (AT_CMD_COPS) : reg_CopsCnf  (cmdId);      break;
      case (AT_CMD_D)    : cal_DCnf     (cmdId);      break;
      case (AT_CMD_A)    : cal_ACnf     (cmdId);      break;
      case (AT_CMD_H)    : cal_HCnf     (cmdId);      break;
      case (AT_CMD_Z)    : cal_HCnf     (cmdId);      break;
      case (KSD_CMD_CF)  : ss_CcfcCnf   (cmdId);      break;
      case (KSD_CMD_CB)  : ss_ClckCnf   (cmdId);      break;
      case (KSD_CMD_CW)  : ss_CcwaCnf   (cmdId);      break;
      case (KSD_CMD_PWD) : sec_CpwdCnf  (cmdId);      break;
      case (AT_CMD_CHLD) :
      case (AT_CMD_CTFR) : cal_ChldCtfrCnf (cmdId);   break;
      case (AT_CMD_CMGD) : sms_CmgdCnf  (cmdId);      break;
      case (AT_CMD_VTS)  : cal_VtsCnf   (cmdId);      break;
      case (KSD_CMD_CL)  :                            break;
      case (KSD_CMD_UBLK):                            break;
      case (KSD_CMD_USSD):                            break;

      default: break;
    }
  }
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_ACA                  |
| STATE   : code                  ROUTINE : rAT_PercentDRV           |
+--------------------------------------------------------------------+

  PURPOSE : This function is used ...

            <device>:   device identity
            <function>: function corresponding to devide
            <val1>:     first value of corresponding function
            <val2>:     second value of corresponding function
*/
GLOBAL void rAT_PercentDRV(T_ACI_DRV_DEV device,
                           T_ACI_DRV_FCT function,
                           UBYTE         val1,
                           UBYTE         val2)
{
  TRACE_FUNCTION ("rAT_PercentDRV");

  switch (device)
  {
    case (DRV_DEV_Keypad):
      switch (function)
      {
        case (DRV_FCT_KeypadInd):
          ui_keypadInd (val1, val2);
          break;

         default:
          break;
      }
      break;

    default:
      break;
  }
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_ACA                  |
| STATE   : code                  ROUTINE : rAT_PercentCLOG          |
+--------------------------------------------------------------------+

  PURPOSE : This function is used ...

*/
GLOBAL void rAT_PercentCLOG( T_ACI_CLOG * cmd )
{
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_ACA                  |
| STATE   : code                  ROUTINE : rAT_PercentRLOG          |
+--------------------------------------------------------------------+

  PURPOSE : This function is used ...

*/
GLOBAL void rAT_PercentRLOG( T_ACI_RLOG * rslt )
{
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_ACA                  |
| STATE   : code                  ROUTINE : aca_cvtNumber            |
+--------------------------------------------------------------------+

  PURPOSE : This function is used to convert a phone number (either 
            SMS service centre address or destination address) as a
            string into a phone number in the format used by the ACI.

*/
GLOBAL void aca_cvtNumber (CHAR* inNumber, T_ACA_NUMBER* outNumber)
{
  BOOL  international = FALSE; /* holds whether number is          */
                               /* international                    */
  CHAR* restNumber;            /* points to the phone number       */
                               /* without international indication */

  if (inNumber[0] EQ '+')
  {
    international = TRUE;
    restNumber    = inNumber + 1;
  }
  else if (inNumber[0] EQ '0' AND inNumber[1] EQ '0')
  {
    international = TRUE;
    restNumber    = inNumber + 2;
  }
  else
    restNumber    = inNumber;

  strncpy (outNumber->number, restNumber, MAX_SMS_NUM_LEN - 1);
  outNumber->number[MAX_SMS_NUM_LEN - 1] = NULL_TERM;

  outNumber->type.ton = (international ? TON_International :
                                         TON_Unknown);

  outNumber->type.npi = NPI_IsdnTelephony;
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_ACA                  |
| STATE   : code                  ROUTINE : aca_recvtNumber          |
+--------------------------------------------------------------------+

  PURPOSE : This function is used to convert a phone number (either 
            SMS service centre address or destination address) in the
            format used by the ACI into a phone number as a string.

*/
GLOBAL void aca_recvtNumber (T_ACA_NUMBER* inNumber, CHAR* outNumber)
{
  UBYTE offset = 0;

  if (inNumber->type.ton EQ TON_International)
  {
    outNumber[0] = '+';
    offset       = 1;
  }

  strncpy (outNumber + offset,
           inNumber->number,
           MAX_SMS_NUM_LEN - 1 - offset);

  outNumber[MAX_SMS_NUM_LEN - 1] = NULL_TERM;
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_ACA                  |
| STATE   : code                  ROUTINE : aca_setAbrtCmd           |
+--------------------------------------------------------------------+

  PURPOSE : This function is used to set the command which should be
            aborted.
*/
GLOBAL BOOL aca_setAbrtCmd (T_ACI_AT_CMD cmd)
{
  BOOL ret = FALSE;

  if ( currAbrtCmd EQ  AT_CMD_NONE  AND
       cmd         NEQ AT_CMD_ABRT      )
  {
    currAbrtCmd = cmd;
    ret         = TRUE;
  }

  return ret;
}
