/* 
+----------------------------------------------------------------------------- 
|  Project :  GSM-F&D (8411)
|  Modul   :  SMI_CAL
+----------------------------------------------------------------------------- 
|  Copyright 2002 Texas Instruments Berlin, AG 
|                 All rights reserved. 
| 
|                 This file is confidential and a trade secret of Texas 
|                 Instruments Berlin, AG 
|                 The receipt of or possession of this file does not convey 
|                 any rights to reproduce or disclose its contents or to 
|                 manufacture, use, or sell anything it may describe, in 
|                 whole, or in part, without the specific written consent of 
|                 Texas Instruments Berlin, AG. 
+----------------------------------------------------------------------------- 
|  Purpose :  This module is used to handle all call related aspects
|             of the SMI.
+----------------------------------------------------------------------------- 
*/ 

#ifndef SMI_CAL_C
#define SMI_CAL_C
#endif

#define ENTITY_SMI

/*==== INCLUDES ===================================================*/
#include <string.h>
#include <stdio.h>
#include "typedefs.h"
#include "vsi.h"
#include "custom.h"
#include "gsm.h"
#include "prim.h"
#include "tok.h"
#include "message.h"

#include "aci_cmh.h"
#include "ksd.h"
#include "aca.h"
#include "smi.h"

#include "gdi.h"
#include "audio.h"

/*==== CONSTANTS ==================================================*/

#define MAX_ABBR_DIAL_NUM 10     /* maximum number of phone        */
                                 /* numbers used for abbreviated   */
                                 /* dialing                        */

/*==== TYPES ======================================================*/

#ifdef AT_INTERPRETER
EXTERN SHORT extCallId;
#endif

/*==== EXPORT =====================================================*/

/*==== VARIABLES ==================================================*/

T_ACI_CPI_MSG currMsgType      = CPI_MSG_NotPresent;
T_ACI_CPI_IBT currIbt          = CPI_IBT_NotPresent;
T_ACI_CPI_TCH currTch          = CPI_TCH_NotPresent;

LOCAL T_ACI_AT_CMD currCalCmd  = AT_CMD_NONE;
LOCAL T_ACI_AT_CMD currDTMFCmd = AT_CMD_NONE;
LOCAL T_ACI_AT_CMD currAbrtCmd = AT_CMD_NONE;

LOCAL SHORT CHLDmode; /* holds chld mode                          */

                      /* holds the phone numbers for abbreviated  */
LOCAL CHAR abbrDial[MAX_ABBR_DIAL_NUM][MAX_PHB_NUM_LEN]; /* dialing */

/*==== FUNCTIONS ==================================================*/
/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_CAL                  |
| STATE   : code                  ROUTINE : cal_initialize           |
+--------------------------------------------------------------------+

  PURPOSE : This function is used to initialize this modul.
*/
GLOBAL void cal_initialize (BOOL isStartup)
{
  USHORT i;

  TRACE_FUNCTION ("cal_initialize()");
  
  currCalCmd  = AT_CMD_NONE;
  currDTMFCmd = AT_CMD_NONE;
  currAbrtCmd = AT_CMD_NONE;

  /*
   *-----------------------------------------------------------------
   * initialize abbreviated dialing numbers
   *-----------------------------------------------------------------
   */
  for (i = 0; i < MAX_ABBR_DIAL_NUM; i++)
    abbrDial[i][0] = NULL_TERM;

  /*
   *-----------------------------------------------------------------
   * initialize CHLD parameter
   *-----------------------------------------------------------------
   */
  CHLDmode = CHLD_MOD_NotPresent;

  currMsgType = CPI_MSG_NotPresent;
  currIbt     = CPI_IBT_NotPresent;
  currTch     = CPI_TCH_NotPresent;
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_CAL                  |
| STATE   : code                  ROUTINE : cal_getCause             |
+--------------------------------------------------------------------+

  PURPOSE : This function returns the error causes after call is
            disconnected or call is  rejected or during in-call 
            modification.
*/
LOCAL USHORT cal_getCause (void)
{
  USHORT cause = NOT_PRESENT_16BIT;
  
  TRACE_FUNCTION ("cal_getCause ()");

  qAT_PlusCEER (CMD_SRC_LCL, &cause);

  return cause;
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_CAL                  |
| STATE   : code                  ROUTINE : cal_setupCall            |
+--------------------------------------------------------------------+

  PURPOSE : This function is called to stimulate a call setup by the
            ACI.

            <dial>:  holds all call related information

            returns: status information concerning the setup progress
*/
GLOBAL T_ACI_RETURN cal_setupCall (T_KSD_DIAL* dial)
{
  T_ACI_RETURN ret = AT_FAIL; /* holds the status information */

  TRACE_FUNCTION ("cal_setupCall ()");
  
  /*
   *-----------------------------------------------------------------
   * call to ACI function when no answer on a further ACI function
   * call is expected
   *-----------------------------------------------------------------
   */
  if (currCalCmd EQ AT_CMD_NONE)
  {
    ret = sAT_Dn (CMD_SRC_LCL,
                  dial->number,  dial->clirOvrd,
                  dial->cugCtrl, dial->callType);

    currCalCmd = AT_CMD_D;

    /*
     *---------------------------------------------------------------
     * process result code of ACI function call
     *---------------------------------------------------------------
     */   
    switch (ret)
    {
      case (AT_CMPL):
        ui_signal (UI_SIG_OUTGOING_CALL, dial->number);
        rAT_OK (currCalCmd);      
        break;

      case (AT_EXCT):
        ui_signal (UI_SIG_OUTGOING_CALL, dial->number);
        break;

      default:
        rAT_PlusCME (currCalCmd, CME_ERR_Unknown);
        break;
    }
  }
  else
  {
    /*
     *---------------------------------------------------------------
     * error handling while waiting for answer on further ACI
     * function call
     *---------------------------------------------------------------
     */
    ui_signal (UI_SIG_UNHND_CME_ERR, CME_ERR_Unknown);

    ret = AT_CMPL;
  }

  return ret;
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_CAL                  |
| STATE   : code                  ROUTINE : cal_DRejWhilePlusCME     |
+--------------------------------------------------------------------+

  PURPOSE : This function is called when an error occured during call
            setup. It handles the errors signaled by the ACI callback
            function rAT_PlusCME.

            <err>: error code
*/
GLOBAL void cal_DRejWhilePlusCME (T_ACI_AT_CMD  cmdId, 
                                  T_ACI_CME_ERR err)
{
  TRACE_FUNCTION ("cal_DRejWhilePlusCME ()");

  if (cmdId EQ currCalCmd)
    currCalCmd = AT_CMD_NONE;

  if (currAbrtCmd EQ AT_CMD_ABRT)
  {
    currAbrtCmd = AT_CMD_NONE;
    
    ui_signal (UI_SIG_UNHND_CME_ERR, CME_ERR_FailedToAbort);
  }
  else
  {
    ui_signal (UI_SIG_UNHND_CME_ERR, err);

    if (currIbt NEQ CPI_IBT_True OR currTch NEQ CPI_TCH_True)
      ui_signal (UI_SIG_NIBT, NIBT_DISC);
  }
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_CAL                  |
| STATE   : code                  ROUTINE : cal_DRej                 |
+--------------------------------------------------------------------+

  PURPOSE : This function is called when an error occured during call
            setup or while call is in progress. It handles the errors
            signaled by the ACI callback functions rAT_NO_CARRIER,
            rAT_BUSY and rAT_NO_ANSWER.

            <cbId>: identity of the related ACI callback function
*/
GLOBAL void cal_DRej (T_MMI_RCB_ID cbId)
{
  TRACE_FUNCTION ("cal_DRej ()");

  switch (cbId)
  {
    case (NO_CARRIER):
      ui_signal (UI_SIG_DISCONNECT, cal_getCause ());

      if (currIbt NEQ CPI_IBT_True OR currTch NEQ CPI_TCH_True)
        ui_signal (UI_SIG_NIBT, NIBT_DISC);
      break;

    case (BUSY):
      ui_signal (UI_SIG_BUSY);
      break;

    case (NO_ANSWER):
      ui_signal (UI_SIG_NO_ANSWER);

      if (currIbt NEQ CPI_IBT_True OR currTch NEQ CPI_TCH_True)
        ui_signal (UI_SIG_NIBT, NIBT_DISC);
      break;

    default:
      ui_signal (UI_SIG_UNHND_CME_ERR, CME_ERR_Unknown);

      if (currIbt NEQ CPI_IBT_True OR currTch NEQ CPI_TCH_True)
        ui_signal (UI_SIG_NIBT, NIBT_DISC);
      break;
  }
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_CAL                  |
| STATE   : code                  ROUTINE : cal_DCnf                 |
+--------------------------------------------------------------------+

  PURPOSE : This function is called when call setup is successfull.
*/
GLOBAL void cal_DCnf (T_ACI_AT_CMD cmdId)
{
  TRACE_FUNCTION ("cal_DCnf ()");

  if (cmdId EQ currCalCmd)
  {
    currCalCmd = AT_CMD_NONE;

    if ( currAbrtCmd EQ AT_CMD_ABRT )
    {
      currAbrtCmd = AT_CMD_NONE;
 
      ui_signal (UI_SIG_DISCONNECT, cal_getCause ());

      if (currIbt NEQ CPI_IBT_True OR currTch NEQ CPI_TCH_True)
        ui_signal (UI_SIG_NIBT, NIBT_DISC);
    }
    else
    {
      ui_signal (UI_SIG_CONNECTED);

      if (currIbt NEQ CPI_IBT_True OR currTch NEQ CPI_TCH_True)
        ui_signal (UI_SIG_NIBT, NIBT_OFF);
    }
  }
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_CAL                  |
| STATE   : code                  ROUTINE : cal_releaseCall          |
+--------------------------------------------------------------------+

  PURPOSE : This function is called to stimulate a call release by
            the ACI.

            returns: status information concerning the release
                     progress
*/
GLOBAL T_ACI_RETURN cal_releaseCall (void)
{
  T_ACI_RETURN ret = AT_FAIL; /* holds the status information */
  
  TRACE_FUNCTION ("cal_releaseCall ()");

  /*
   *-----------------------------------------------------------------
   * call to ACI function when no answer on a further ACI function
   * call is expected
   *-----------------------------------------------------------------
   */
  if (currCalCmd EQ AT_CMD_NONE)
  {
    ret = sAT_H(CMD_SRC_LCL);

    /*
     *---------------------------------------------------------------
     * process result code of ACI function call
     *---------------------------------------------------------------
     */   
    switch (ret)
    {
      case (AT_CMPL):
        /* Condat RT sends hook on command twice */
        
        if (currIbt NEQ CPI_IBT_True OR currTch NEQ CPI_TCH_True)
          ui_signal (UI_SIG_NIBT, NIBT_OFF);
        
        break;

      case (AT_EXCT):
        currCalCmd = AT_CMD_H;
        ui_signal (UI_SIG_DISCONNECTING);
        break;

      case (AT_FAIL):
        /* Condat RT sends hook on command twice */
        
        if (currIbt NEQ CPI_IBT_True OR currTch NEQ CPI_TCH_True)
          ui_signal (UI_SIG_NIBT, NIBT_OFF);
        
        break;

      default:
        currCalCmd = AT_CMD_H;
        rAT_PlusCME (currCalCmd, CME_ERR_Unknown);
        break;
    }
  }
  else
  {
    switch (currCalCmd)
    {
      case (AT_CMD_D):
      case (AT_CMD_A):
        if (currAbrtCmd EQ AT_CMD_ABRT)
        {
          ret = AT_CMPL;
        }
        else
        {
          if (aca_setAbrtCmd (currCalCmd))
          {
            ret = sAT_Abort (CMD_SRC_LCL, currCalCmd);

            currAbrtCmd = AT_CMD_ABRT;
  
            switch (ret)
            {
              case (AT_EXCT):
                break;

              case (AT_CMPL):
                rAT_OK (currAbrtCmd);
                break;

              default:
                rAT_PlusCME (currAbrtCmd, CME_ERR_FailedToAbort);
              break;
            }
          }
          else
          {
            ui_signal (UI_SIG_UNHND_CME_ERR, CME_ERR_FailedToAbort);
            ret = AT_FAIL;
          }
        }
        break;

      default:
        ui_signal (UI_SIG_UNHND_CME_ERR, CME_ERR_Unknown);
        ret = AT_CMPL;
        break;
    }
  }

  return ret;
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_CAL                  |
| STATE   : code                  ROUTINE : cal_HRejWhilePlusCME     |
+--------------------------------------------------------------------+

  PURPOSE : This function is called when an error occured during call
            release. It handles the errors signaled by the ACI
            callback function rAT_PlusCME.

            <err>: error code
*/
GLOBAL void cal_HRejWhilePlusCME (T_ACI_AT_CMD  cmdId, 
                                  T_ACI_CME_ERR err)
{
  TRACE_FUNCTION ("cal_HRejWhilePlusCME ()");

  if (cmdId EQ currCalCmd)
    currCalCmd = AT_CMD_NONE;

  ui_signal (UI_SIG_UNHND_CME_ERR, err);
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_CAL                  |
| STATE   : code                  ROUTINE : cal_HRej                 |
+--------------------------------------------------------------------+

  PURPOSE : This function is called when an error occured during call
            release. It handles the errors signaled by the ACI
            callback functions rAT_NO_CARRIER, rAT_BUSY and
            rAT_NO_ANSWER.

            <cbId>: identity of the related ACI callback function
*/
GLOBAL void cal_HRej (T_MMI_RCB_ID cbId)
{
  TRACE_FUNCTION ("cal_HRej ()");

  ui_signal (UI_SIG_UNHND_CME_ERR, CME_ERR_Unknown);
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_CAL                  |
| STATE   : code                  ROUTINE : cal_HCnf                 |
+--------------------------------------------------------------------+

  PURPOSE : This function is called when call release is successfull.
*/
GLOBAL void cal_HCnf (T_ACI_AT_CMD cmdId)
{
  TRACE_FUNCTION ("cal_HCnf ()");

  if (cmdId EQ currCalCmd)
  {
    currCalCmd = AT_CMD_NONE;
    ui_signal (UI_SIG_DISCONNECT, cal_getCause ());

    if (currIbt NEQ CPI_IBT_True OR currTch NEQ CPI_TCH_True)
      ui_signal (UI_SIG_NIBT, NIBT_DISC);
  }
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_CAL                  |
| STATE   : code                  ROUTINE : cal_acceptCall           |
+--------------------------------------------------------------------+

  PURPOSE : This function is called to stimulate an acception of an
            incoming call by the ACI.

            returns: status information concerning the acception
                     progress
*/
GLOBAL T_ACI_RETURN cal_acceptCall ()
{
  T_ACI_RETURN ret = AT_FAIL; /* holds the status information */
  
  TRACE_FUNCTION ("cal_acceptCall ()");

  /*
   *-----------------------------------------------------------------
   * call to ACI function when no answer on a further ACI function
   * call is expected
   *-----------------------------------------------------------------
   */
  if (currCalCmd EQ AT_CMD_NONE)
  {  
    ret = sAT_A(CMD_SRC_LCL);

    currCalCmd = AT_CMD_A;

    /*
     *---------------------------------------------------------------
     * process result code of ACI function call
     *---------------------------------------------------------------
     */   
    switch (ret)
    {
      case (AT_CMPL):
        ui_signal (UI_SIG_CONNECTING);
        rAT_OK (currCalCmd);
        break;

      case (AT_EXCT):
        ui_signal (UI_SIG_CONNECTING);
        break;

      default:
        rAT_PlusCME (currCalCmd, CME_ERR_Unknown);
        break;
    }
  }
  else
  {
    /*
     *---------------------------------------------------------------
     * error handling while waiting for answer on further ACI
     * function call
     *---------------------------------------------------------------
     */
    ui_signal (UI_SIG_UNHND_CME_ERR, CME_ERR_Unknown);

    ret = AT_CMPL;
  }

  return ret;
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_CAL                  |
| STATE   : code                  ROUTINE : cal_ARejWhilePlusCME     |
+--------------------------------------------------------------------+

  PURPOSE : This function is called when an error occured during call
            acception. It handles the errors signaled by the ACI
            callback function rAT_PlusCME.

            <err>: error code
*/
GLOBAL void cal_ARejWhilePlusCME (T_ACI_AT_CMD  cmdId, 
                                  T_ACI_CME_ERR err)
{
  TRACE_FUNCTION ("cal_ARejWhilePlusCME ()");

  if (cmdId EQ currCalCmd)
    currCalCmd = AT_CMD_NONE;

  if (currAbrtCmd EQ AT_CMD_ABRT)
  {
    currAbrtCmd = AT_CMD_NONE;
    
    ui_signal (UI_SIG_UNHND_CME_ERR, CME_ERR_FailedToAbort);
  }
  else
    ui_signal (UI_SIG_UNHND_CME_ERR, err);
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_CAL                  |
| STATE   : code                  ROUTINE : cal_ARej                 |
+--------------------------------------------------------------------+

  PURPOSE : This function is called when an error occured during call
            acception. It handles the errors signaled by the ACI
            callback functions rAT_NO_CARRIER, rAT_BUSY and
            rAT_NO_ANSWER.

            <cbId>: identity of the related ACI callback function
*/
GLOBAL void cal_ARej (T_MMI_RCB_ID cbId)
{
  TRACE_FUNCTION ("cal_ARej ()");

  ui_signal (UI_SIG_DISCONNECT, cal_getCause ());
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_CAL                  |
| STATE   : code                  ROUTINE : cal_ACnf                 |
+--------------------------------------------------------------------+

  PURPOSE : This function is called when call acception is
            successfull.
*/
GLOBAL void cal_ACnf (T_ACI_AT_CMD cmdId)
{
  TRACE_FUNCTION ("cal_ACnf ()");

  if (cmdId EQ currCalCmd)
  {
    currCalCmd = AT_CMD_NONE;

    if ( currAbrtCmd EQ AT_CMD_ABRT )
    {
      currAbrtCmd = AT_CMD_NONE;
 
      ui_signal (UI_SIG_DISCONNECT, cal_getCause ());
    }
    else
      ui_signal (UI_SIG_CONNECTED);
  }
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_CAL                  |
| STATE   : code                  ROUTINE : rAT_PlusCRING            |
+--------------------------------------------------------------------+

  PURPOSE : This function is called to signal an incoming call.

            <mode>:  mode of call sequence
            <type1>: type of call 1
            <type2>: type of call 2
*/
GLOBAL void rAT_PlusCRING (T_ACI_CRING_MOD mode,
                           T_ACI_CRING_TYP type1,
                           T_ACI_CRING_TYP type2)
{
  TRACE_FUNCTION ("rAT_PlusCRING ()");

  /*
   *-----------------------------------------------------------------
   * process parameter mode
   *-----------------------------------------------------------------
   */
  switch (mode)
  {
    case (CRING_MOD_Direct):

      /*
       *-------------------------------------------------------------
       * process parameter type1 when mode = CRING_MOD_Direct
       *-------------------------------------------------------------
       */
      switch (type1)
      {
        case (CRING_TYP_Async):
        case (CRING_TYP_Sync):
        case (CRING_TYP_RelAsync):
        case (CRING_TYP_RelSync):
        case (CRING_TYP_Fax):
          break;

        case (CRING_TYP_Voice):
          ui_signal (UI_SIG_INCOMMING_CALL);
          break;
      }
      break;

    case (CRING_MOD_Alternate):
      break;

    case (CRING_MOD_NotPresent):
      break;
  }
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_CAL                  |
| STATE   : code                  ROUTINE : rAT_PlusCRING_OFF        |
+--------------------------------------------------------------------+

  PURPOSE : This function is called to indicate that an incoming call
            is no longer waiting.
*/
GLOBAL void rAT_PlusCRING_OFF ( SHORT cId )
{
  TRACE_FUNCTION ("rAT_PlusCRING_OFF ()");

  ui_signal (UI_SIG_RING_OFF);
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_CAL                  |
| STATE   : code                  ROUTINE : rAT_PercentSIMREM        |
+--------------------------------------------------------------------+

  PURPOSE : This function is called to indicate that the SIM card was
            removed.
*/
GLOBAL void rAT_PercentSIMREM (T_ACI_SIMREM_TYPE srType)
{
  TRACE_FUNCTION ("rAT_PercentSIMREM ()");

  ui_signal (UI_SIG_SIM_NOT_INSERT);
  if (mmi_data->serviceType NEQ SRV_NONE)
  {
    mmi_data->serviceType = SRV_LIMITED;
    ui_signal (UI_SIG_LIMITED_SERVICE);
  }
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_CAL                  |
| STATE   : code                  ROUTINE : cal_setCbst              |
+--------------------------------------------------------------------+

  PURPOSE : This function is used to select the bearer service type.

            <cbst>:  holds all bearer service related information

            returns: status information concerning the selection
                     progress
*/
GLOBAL T_ACI_RETURN cal_setCbst (T_KSD_CBST* cbst)
{
  TRACE_FUNCTION ("cal_setCbst ()");

  return sAT_PlusCBST (CMD_SRC_LCL,
                       cbst->speed, cbst->name, cbst->ce);
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_CAL                  |
| STATE   : code                  ROUTINE : cal_setCrlp              |
+--------------------------------------------------------------------+

  PURPOSE : This function is used to set the parameter for the radio
            link protocol.

            <crlp>:  holds all radio link protocol related information

            returns: status information concerning the setting
                     progress
*/
GLOBAL T_ACI_RETURN cal_setCrlp (T_KSD_CRLP* crlp)
{
  TRACE_FUNCTION ("cal_setCrlp ()");

#ifdef FAX_AND_DATA
  return sAT_PlusCRLP (CMD_SRC_LCL,
                       crlp->iws, crlp->mws, crlp->t1, crlp->n2);
#else
  return AT_FAIL;
#endif
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_CAL                  |
| STATE   : code                  ROUTINE : cal_setDs                |
+--------------------------------------------------------------------+

  PURPOSE : This function is used to set the parameter for the data
            compression.

            <cbst>:  holds all data compression related information

            returns: status information concerning the setting
                     progress
*/
GLOBAL T_ACI_RETURN cal_setDs (T_KSD_DS* ds)
{
  TRACE_FUNCTION ("cal_setDs ()");

#ifdef FAX_AND_DATA
  return sAT_PlusDS (CMD_SRC_LCL,
                     ds->dir, ds->comp, ds->maxDict, ds->maxStr);
#else
  return AT_FAIL;
#endif
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_CAL                  |
| STATE   : code                  ROUTINE : rAT_CONNECT              |
+--------------------------------------------------------------------+

  PURPOSE : This function is used to indicate that a MO data call is
            successfully established.
*/
GLOBAL void rAT_CONNECT (T_ACI_AT_CMD cmdId, T_ACI_BS_SPEED speed,
                         SHORT cId )
{
  /* used for transfer of speed enumerator into numerical value */
  static const struct T_CAL_SPEED_TAB
  {                                  
    const T_ACI_BS_SPEED  enumSpeed; /* speed defined as */
                                     /* enumerator       */
    const USHORT          valSpeed;  /* speed defined as */
                                     /* numerical value  */
  }
    speedTab[] = 
  {
    BS_SPEED_300_V21,       300,
    BS_SPEED_1200_V22,     1200,
    BS_SPEED_1200_75_V23,  1200,
    BS_SPEED_2400_V22bis,  2400,
    BS_SPEED_2400_V26ter,  2400,
    BS_SPEED_4800_V32,     4800,
    BS_SPEED_9600_V32,     9600,
    BS_SPEED_1200_V120,    1200,
    BS_SPEED_2400_V120,    2400,
    BS_SPEED_4800_V120,    4800,
    BS_SPEED_9600_V120,    9600,
    BS_SPEED_14400_V120,  14400,
    BS_SPEED_300_V110,      300,
    BS_SPEED_1200_V110,    1200,
    BS_SPEED_2400_V110,    2400,
    BS_SPEED_4800_V110,    4800,
    BS_SPEED_9600_V110,    9600,
    BS_SPEED_14400_V110,  14400,
    BS_SPEED_NotPresent,     -1
  };

  CHAR buf[20];

  sprintf (buf, "rAT_CONNECT (%d)", cmdId);
  TRACE_FUNCTION (buf);
  
  if (cmdId NEQ currCalCmd)
    return;

  currCalCmd = AT_CMD_NONE;

  if (cmdId EQ AT_CMD_D)
  {
    USHORT i = 0;

    /*
     *---------------------------------------------------------------
     * searching for numerical speed value
     *---------------------------------------------------------------
     */
    while (speedTab[i].enumSpeed NEQ BS_SPEED_NotPresent AND
           speedTab[i].enumSpeed NEQ speed)
      i++;

    ui_signal (UI_SIG_DATA_CONNECTED, speedTab[i].valSpeed);

    if (currIbt NEQ CPI_IBT_True OR currTch NEQ CPI_TCH_True)
      ui_signal (UI_SIG_NIBT, NIBT_OFF);
  }
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_CAL                  |
| STATE   : code                  ROUTINE : cal_sigChldCtfr          |
+--------------------------------------------------------------------+

  PURPOSE : This function is called to signal the result of the
            handling of supplementary services within a call according
            to GSM 02.30.

            <rslt>: result code
*/
LOCAL void cal_sigChldCtfr (T_SS_ACA_RSLT rslt)
{
  TRACE_FUNCTION ("cal_sigChldCtfr ()");

  ui_signal (UI_SIG_SSWC, rslt);
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_CAL                  |
| STATE   : code                  ROUTINE : cal_changeCall           |
+--------------------------------------------------------------------+

  PURPOSE : This function is used for handling of supplementary 
            services within a call according to GSM 02.30.

            <chld>: holds all information concerning SS within a call

            returns: status information concerning the changing
                     progress
*/
GLOBAL T_ACI_RETURN cal_changeCall (T_KSD_CHLD* chld)
{
  T_ACI_RETURN ret = AT_FAIL; /* holds the status information */

  TRACE_FUNCTION ("cal_changeCall ()");

  /*
   *-----------------------------------------------------------------
   * call to ACI function when no answer on a further ACI function
   * call is expected
   *-----------------------------------------------------------------
   */
  if (currCalCmd EQ AT_CMD_NONE)
  {  
    ret = sAT_PlusCHLD (CMD_SRC_LCL, chld->mode, chld->call);

    CHLDmode   = chld->mode;
    currCalCmd = AT_CMD_CHLD;

    /*
     *---------------------------------------------------------------
     * process result code of ACI function call
     *---------------------------------------------------------------
     */   
    switch (ret)
    {
      case (AT_CMPL):
        rAT_OK (currCalCmd);
        break;

      case (AT_EXCT):
        break;

      default:
        rAT_PlusCME (currCalCmd, CME_ERR_Unknown);
        break;
    }
  }
  else
  {
    /*
     *---------------------------------------------------------------
     * error handling while waiting for answer on further ACI
     * function call
     *---------------------------------------------------------------
     */
    ui_signal (UI_SIG_UNHND_CME_ERR, CME_ERR_Unknown);

    ret = AT_CMPL;
  }

  return ret;
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_CAL                  |
| STATE   : code                  ROUTINE : cal_invokeCTFR           |
+--------------------------------------------------------------------+

  PURPOSE : This function is used for handling call deflection.

            returns: status information concerning the changing
                     progress
*/
GLOBAL T_ACI_RETURN cal_invokeCTFR (T_KSD_CTFR *ctfr)
{
  T_ACI_RETURN ret = AT_FAIL; /* holds the status information */

  TRACE_FUNCTION ("cal_invokeCTFR ()");

  /*
   *-----------------------------------------------------------------
   * call to ACI function when no answer on a further ACI function
   * call is expected
   *-----------------------------------------------------------------
   */
  if (currCalCmd EQ AT_CMD_NONE)
  {  
    ret = sAT_PlusCTFR (CMD_SRC_LCL,
                        ctfr->number, &ctfr->type,
                        ctfr->subaddr, &ctfr->satype);

    currCalCmd = AT_CMD_CTFR;

    /*
     *---------------------------------------------------------------
     * process result code of ACI function call
     *---------------------------------------------------------------
     */   
    switch (ret)
    {
      case (AT_CMPL):
        rAT_OK (currCalCmd);  /* Not expected here */
        break;

      case (AT_EXCT):
        break;

      default:
        rAT_PlusCME (currCalCmd, CME_ERR_Unknown);
        break;
    }
  }
  else
  {
    /*
     *---------------------------------------------------------------
     * error handling while waiting for answer on further ACI
     * function call
     *---------------------------------------------------------------
     */
    ui_signal (UI_SIG_UNHND_CME_ERR, CME_ERR_Unknown);

    ret = AT_CMPL;
  }

  return ret;
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_CAL                  |
| STATE   : code                  ROUTINE : cal_ChldCtfrCnf          |
+--------------------------------------------------------------------+

  PURPOSE : This function is called when handling of supplementary 
            services via sAT_PlusCHLD() or sAT_PlusCTFR within a call 
            is successfull.
*/
GLOBAL void cal_ChldCtfrCnf (T_ACI_AT_CMD cmdId)
{
  TRACE_FUNCTION ("cal_ChldCtfrCnf ()");

  if (cmdId EQ currCalCmd)
  {
    currCalCmd = AT_CMD_NONE;
    cal_sigChldCtfr (SS_RSLT_OK);
  }
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_CAL                  |
| STATE   : code                  ROUTINE : cal_ChldCtfrRej          |
+--------------------------------------------------------------------+

  PURPOSE : This function is called when handling of supplementary
            services via sAT_PlusCHLD() or sAT_PlusCTFR within a call 
            failed.
            <err>: error code
*/
GLOBAL void cal_ChldCtfrRej (T_ACI_AT_CMD cmdId,
                             T_ACI_CME_ERR err)
{
  TRACE_FUNCTION ("cal_ChldCtfrRej ()");

  if (cmdId EQ currCalCmd)
    currCalCmd = AT_CMD_NONE;

  cal_sigChldCtfr (SS_RSLT_FAIL);  
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_CAL                  |
| STATE   : code                  ROUTINE : cal_showCallTable        |
+--------------------------------------------------------------------+

  PURPOSE : This function is used to stimulate the display of
            the call table.

            returns: status information of call table query
*/
EXTERN T_ACI_RETURN cal_showCallTable (void)
{
  T_ACI_RETURN  ret = AT_FAIL; /* holds the status information */
  T_ACI_CAL_LST callTable;     /* holds call table             */
  T_CTB_INFO    lstItem[MAX_CALL_NR + 1]; 
                               /* holds call table information */
                               /* relevant for display         */
  USHORT        i   = 0;       /* used as counter              */

  TRACE_FUNCTION ("cal_showCallTable ()");

  ret = qAT_PercentCAL (CMD_SRC_LCL, &callTable[0]);

  if (ret EQ AT_CMPL)
  {
    /*
     *---------------------------------------------------------------
     * extract all relevant information out of the call table that
     * should be displayed to the user
     *---------------------------------------------------------------
     */
    while (i < MAX_CALL_NR AND
           callTable[i].index NEQ -1)
    {        
      lstItem[i].index  = callTable[i].index;
      lstItem[i].status = callTable[i].status;

      i++;
    }

    /*
     *---------------------------------------------------------------
     * terinate the list
     *---------------------------------------------------------------
     */
    lstItem[i].index  = -1;
    lstItem[i].status = CAL_STAT_NotPresent;

    ui_signal (UI_SIG_SHOW_CALL_TABLE, &lstItem[0]);
  }
  else
    ui_signal (UI_SIG_UNHND_CME_ERR, CME_ERR_Unknown);

  return ret;
}

#if 0 /* Disable code as it seems not to be needed anymore */
/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_CAL                  |
| STATE   : code                  ROUTINE : cal_isCallActive         |
+--------------------------------------------------------------------+

  PURPOSE : This function is used to request whether at least one
            call is active.

            returns: TRUE if at least one call is active, otherwise
                     FALSE
*/
GLOBAL BOOL cal_isCallActive (void)
{
  T_ACI_RETURN  ret          = AT_FAIL; /* holds status information */
  T_ACI_CAL_LST callTable;              /* holds call table         */
  USHORT        i            = 0;       /* used as counter          */
  BOOL          isCallActive = FALSE;   /* holds result of request  */

  TRACE_FUNCTION ("cal_isCallActive ()");

  ret = qAT_PercentCAL (CMD_SRC_LCL, &callTable[0]);

  if (ret EQ AT_CMPL)
  {
    /*
     *---------------------------------------------------------------
     * searching for active calls
     *---------------------------------------------------------------
     */
    // I have my doubts whether the expression (callTable[i].index NEQ -1)
    // should be in the while loop and not in the if clause, but I'm
    // not expirienced enough in the SMI code to change that now... (HM)
    while (i < MAX_CALL_NR AND
           callTable[i].index NEQ -1)
    {
      if (callTable[i].status EQ CAL_STAT_Active)
        isCallActive = TRUE;

      i++;
    }
  }

  return isCallActive;
}
#endif

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_CAL                  |
| STATE   : code                  ROUTINE : cal_isDTMFPossible       |
+--------------------------------------------------------------------+

  PURPOSE : This function is used to request whether in the current 
            situation DTMF maybe possible. The final decision whether 
            DTMF is possible is met by the call control entitity, the 
            attempt maybe rejected.

            returns: TRUE if DTMF maybe possible, otherwise FALSE.

*/
GLOBAL BOOL cal_isDTMFPossible (void)
{
  T_ACI_RETURN ret;
  T_ACI_CAL_LST callTable;
  USHORT i;

  TRACE_FUNCTION ("cal_isDTMFPossible()");
  
  ret = qAT_PercentCAL (CMD_SRC_LCL, &callTable[0]);
  
  assert (ret EQ AT_CMPL); /* All other return values are really unexpected */
  
  /*
   * Search for calls which are in a state which may allow DTMF 
   */
  for (i = 0; i < MAX_CALL_NR; i++)
  {
    if ((callTable[i].index NEQ -1) AND
        ((callTable[i].status EQ CAL_STAT_Active) OR
         (callTable[i].status EQ CAL_STAT_Dial) OR
         (callTable[i].status EQ CAL_STAT_DeactiveReq)OR
         (callTable[i].status EQ CAL_STAT_Incomming)))
      return TRUE;
  }

  /* No call present which may be able to do DTMF */
  return FALSE;
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_CAL                  |
| STATE   : code                  ROUTINE : cal_isCallWaiting        |
+--------------------------------------------------------------------+

  PURPOSE : This function is used to request whether at least one
            call is waiting.

            returns: TRUE if at least one call is waiting, otherwise
                     FALSE
*/
GLOBAL BOOL cal_isCallWaiting (void)
{
  T_ACI_RETURN  ret           = AT_FAIL; /* holds status information */
  T_ACI_CAL_LST callTable;               /* holds call table         */
  USHORT        i             = 0;       /* used as counter          */
  BOOL          isCallWaiting = FALSE;   /* holds result of request  */

  TRACE_FUNCTION ("cal_isCallWaiting ()");

  ret = qAT_PercentCAL (CMD_SRC_LCL, &callTable[0]);

  if (ret EQ AT_CMPL)
  {
    /*
     *---------------------------------------------------------------
     * searching for waiting calls
     *---------------------------------------------------------------
     */
    while (i < MAX_CALL_NR AND
           callTable[i].index NEQ -1)
    {
      if (callTable[i].status EQ CAL_STAT_Wait)
        isCallWaiting = TRUE;

      i++;
    }
  }

  return isCallWaiting;
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_CAL                  |
| STATE   : code                  ROUTINE : cal_isFirstCallWait      |
+--------------------------------------------------------------------+

  PURPOSE : This function is used to request whether there is only
            one call in the call table and whether this call is
            waiting.

            returns: TRUE if the above condition is correct, otherwise
                     FALSE
*/
GLOBAL BOOL cal_isFirstCallWait (void)
{
  T_ACI_RETURN  ret        = AT_FAIL; /* holds status information  */
  T_ACI_CAL_LST callTable;            /* holds call table          */
  USHORT        i          = 0;       /* used as counter           */
  BOOL          isCallWait = FALSE;   /* holds whether calls are   */
                                      /* waiting                   */ 

  TRACE_FUNCTION ("cal_isFirstCallWait ()");

  ret = qAT_PercentCAL (CMD_SRC_LCL, &callTable[0]);

  if (ret EQ AT_CMPL)
  {
    /*
     *---------------------------------------------------------------
     * searching for waiting calls
     *---------------------------------------------------------------
     */
    while (i < MAX_CALL_NR AND
           callTable[i].index NEQ -1)
    {
      if (callTable[i].status EQ CAL_STAT_Wait)
        isCallWait = TRUE;

      i++;
    }
  }

  /*
   *-----------------------------------------------------------------
   * calculating the result
   *-----------------------------------------------------------------
   */
  return (i EQ 1 AND isCallWait);
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_CAL                  |
| STATE   : code                  ROUTINE : cal_isWithinCall         |
+--------------------------------------------------------------------+

  PURPOSE : This function is used to request whether at least one call
            is present (any non-idle state).

  Definition from GSM 02.30 subclause 4.5.3.2:
    
      A MS is "in a call" from the time that signalling related to the 
      establishment or attempted establishment of a MO or MT call 
      commences and before the call or call attempt ends, 
      and (if applicable) the ME has stopped generating tones related 
      to this call to the user.

  returns: TRUE if the above condition is correct, otherwise FALSE.

*/
GLOBAL BOOL cal_isWithinCall (void)
{
  T_ACI_RETURN  ret;                    /* holds status information */
  T_ACI_CAL_LST callTable;              /* holds call table         */
  USHORT        i;                      /* Counter variable         */

  TRACE_FUNCTION ("cal_isWithinCall ()");

  ret = qAT_PercentCAL (CMD_SRC_LCL, &callTable[0]);

  if (ret EQ AT_CMPL)
  {
    /*
     * searching for any non-idle call.
     * Note1: This code could be simplified, the switch() maybe superflous.
     * Note2: The check for having stopped tone generation maybe weak here. 
     */
    for (i = 0; (i < MAX_CALL_NR) AND (callTable[i].index NEQ -1); i++)
    {
      switch (callTable[i].status)
      {
        case CAL_STAT_Held:
        case CAL_STAT_Active:
        case CAL_STAT_Wait:
        case CAL_STAT_Dial:
        case CAL_STAT_DeactiveReq:
        case CAL_STAT_Incomming:
        case CAL_STAT_Alerting:
          return TRUE; /* One call in appropriate state found */
        default: /* Check next call table entry */
          break;
      }
    }
  }
  return FALSE;
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_CAL                  |
| STATE   : code                  ROUTINE : cal_isNoCall             |
+--------------------------------------------------------------------+

  PURPOSE : This function is used to request whether no call is 
            listed in the call table.

            returns: TRUE if the above condition is correct, otherwise
                     FALSE
*/
GLOBAL BOOL cal_isNoCall (void)
{
  T_ACI_RETURN  ret        = AT_FAIL; /* holds status information   */
  T_ACI_CAL_LST callTable;            /* holds call table           */
  SHORT         index      = -1;      /* first index in call table  */

  TRACE_FUNCTION ("cal_isNoCall ()");

  ret = qAT_PercentCAL (CMD_SRC_LCL, &callTable[0]);

  if (ret EQ AT_CMPL)
    index = callTable[0].index;

  return (index EQ -1);
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_CAL                  |
| STATE   : code                  ROUTINE : cal_rejectCallbackInd    |
+--------------------------------------------------------------------+

  PURPOSE : This function is called to concentrate all reject 
            indications delivered by rAT_NO_CARRIER, rAT_BUSY and
            rAT_NO_ANSWER.

            <cmdId>: command indentity
            <cId>  : call id
            <cbId> : callback indentity
*/
LOCAL void cal_rejectCallbackInd (T_ACI_AT_CMD cmdId,
                                  SHORT        cId,
                                  T_MMI_RCB_ID cbId)
{
#ifdef AT_INTERPRETER
  if ( cId EQ extCallId )
  {

    switch ( cbId )
    {
      case ( NO_CARRIER ):
        rCI_NO_CARRIER ( cmdId, cId );
        break;

      case ( BUSY ):
        rCI_BUSY ( cmdId, cId );
        break;

      case ( NO_ANSWER ):
        rCI_NO_ANSWER ( cmdId, cId );
        break;
    }
    extCallId = ACI_NumParmNotPresent;
  }
  else
#endif
  {
    if (cmdId EQ currCalCmd)
      currCalCmd = AT_CMD_NONE;

    switch (cmdId)
    {
      case (AT_CMD_D): cal_DRej (cbId); break;
      case (AT_CMD_A): cal_ARej (cbId); break;
      case (AT_CMD_H): cal_HRej (cbId); break;
      case (AT_CMD_Z): cal_HRej (cbId); break;
 
      case (AT_CMD_NONE):
        ui_signal (UI_SIG_DISCONNECT, cal_getCause ());
        break;

      default:
        break;
    }
  }
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_CAL                  |
| STATE   : code                  ROUTINE : rAT_NO_CARRIER           |
+--------------------------------------------------------------------+

  PURPOSE : This function is called to indicated that no carrier
            is present or carrier is lost.

            <cmdId>: command identity
*/
GLOBAL void rAT_NO_CARRIER (T_ACI_AT_CMD cmdId, SHORT cId)
{
  TRACE_FUNCTION ("rAT_NO_CARRIER");

  cal_rejectCallbackInd (cmdId, cId, NO_CARRIER);
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_CAL                  |
| STATE   : code                  ROUTINE : rAT_BUSY                 |
+--------------------------------------------------------------------+

  PURPOSE : This function is called to indicate that line is busy.
            
            <cmdId>: command identity
*/
GLOBAL void rAT_BUSY (T_ACI_AT_CMD cmdId, SHORT cId )
{
  TRACE_FUNCTION ("rAT_BUSY");

  cal_rejectCallbackInd (cmdId, cId, BUSY);
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_CAL                  |
| STATE   : code                  ROUTINE : rAT_NO_ANSWER            |
+--------------------------------------------------------------------+

  PURPOSE : This function is called to indicate ...

            <cmdId>: command identity
*/
GLOBAL void rAT_NO_ANSWER (T_ACI_AT_CMD cmdId, SHORT cId)
{
  TRACE_FUNCTION ("rAT_NO_ANSWER");

  cal_rejectCallbackInd (cmdId, cId, NO_ANSWER);
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_CAL                  |
| STATE   : code                  ROUTINE : cal_sendAudioToneReq     |
+--------------------------------------------------------------------+

  PURPOSE : This functions sends a MMI_AUDIO_TONE_REQ primitive to PL
            to switch on/off a specified tone.

            <callTone>: call tone
            <volume>:   volume
            <toneStat>: tone status
*/
GLOBAL void cal_sendAudioToneReq (UBYTE callTone,
                                  UBYTE volume,
                                  UBYTE toneStat)
{
  if (toneStat EQ TONE_SWT_OFF)
    audio_StopSoundbyID (AUDIO_SPEAKER,
                         callTone);
  else
    audio_PlaySoundID (AUDIO_SPEAKER,
                       callTone,
                       volume,
                       toneStat);
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_CAL                  |
| STATE   : code                  ROUTINE : cal_getAbbrDial          |
+--------------------------------------------------------------------+

  PURPOSE : This functions is used to get the phone number used in
            the context of abbreviated dialing.

            <index>: abbreviated dialing index

            returns: exact phone number
*/
GLOBAL CHAR* cal_getAbbrDial (UBYTE index)
{
  if (index >= 0 AND index < MAX_ABBR_DIAL_NUM)
    return abbrDial[index];

  return "";
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_CAL                  |
| STATE   : code                  ROUTINE : cal_setAbbrDial          |
+--------------------------------------------------------------------+

  PURPOSE : This functions is used to set the phone number used in
            the context of abbreviated dialing.

            <index>:  abbreviated dialing index
            <number>: exact phone number
*/
GLOBAL void cal_setAbbrDial (BYTE index, CHAR* number)
{
  if (index >= 0 AND index < MAX_ABBR_DIAL_NUM AND number NEQ NULL)
  {
    strncpy (abbrDial[index], number, MAX_PHB_NUM_LEN - 1);
    abbrDial[index][MAX_PHB_NUM_LEN - 1] = NULL_TERM;
  }
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_CAL                  |
| STATE   : code                  ROUTINE : cal_sendDmtf             |
+--------------------------------------------------------------------+

  PURPOSE : This function is used to send a DMTF tone.

            <dtmf>:  DTMF tone

            returns: status information concerning the sending
                     progress
*/
GLOBAL T_ACI_RETURN cal_sendDtmf (CHAR dtmf, BOOL pressed)
{
  T_ACI_RETURN ret = AT_FAIL; /* holds the status information */

  TRACE_FUNCTION ("cal_sendDtmf ()");
  
  /*
   *-----------------------------------------------------------------
   * call to ACI function when no answer on a further ACI function
   * call is expected
   *-----------------------------------------------------------------
   */
  if (currDTMFCmd EQ AT_CMD_NONE)
  {
    ret = sAT_PlusVTS (CMD_SRC_LCL, dtmf, ((pressed)?
                                            VTS_MOD_ManStart:
                                            VTS_MOD_ManStop));
    currDTMFCmd = AT_CMD_VTS;

    /*
     *---------------------------------------------------------------
     * process result code of ACI function call
     *---------------------------------------------------------------
     */   
    switch (ret)
    {
      case (AT_CMPL):
        rAT_OK (AT_CMD_VTS);
        break;

      case (AT_EXCT):
        break;

      default:
        rAT_PlusCME (AT_CMD_VTS, CME_ERR_Unknown);
        break;
    }
  }
  else if( currDTMFCmd EQ AT_CMD_VTS )
  {
    /*
     *---------------------------------------------------------------
     * abort current +VTS command
     *---------------------------------------------------------------
     */
    ret = sAT_Abort (CMD_SRC_LCL, AT_CMD_VTS);

    currDTMFCmd = AT_CMD_NONE;

    /*
     *---------------------------------------------------------------
     * process result code of ACI function call
     *---------------------------------------------------------------
     */   
    switch (ret)
    {
      case (AT_CMPL):
        rAT_OK (AT_CMD_NONE); // Is this correct ...?
        break;

      default:
        rAT_PlusCME (AT_CMD_NONE, CME_ERR_Unknown); // Is this correct ...?
        break;
    }
  }
  else
  {
    /*
     *---------------------------------------------------------------
     * error handling while waiting for answer on further ACI
     * function call
     *---------------------------------------------------------------
     */
    ui_signal (UI_SIG_UNHND_CME_ERR, CME_ERR_Unknown);

    ret = AT_CMPL;
  }

  return ret;
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_CAL                  |
| STATE   : code                  ROUTINE : cal_VtsCnf               |
+--------------------------------------------------------------------+

  PURPOSE : This function is called when sending of a DTMF tone is
            successfull
*/
GLOBAL void cal_VtsCnf (T_ACI_AT_CMD cmdId)
{
  TRACE_FUNCTION ("cal_VtsCnf ()");

  currDTMFCmd = AT_CMD_NONE;
}
                                        
/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_CAL                  |
| STATE   : code                  ROUTINE : cal_VtsRej               |
+--------------------------------------------------------------------+

  PURPOSE : This function is called when sending of a DTMF tone
            failed.

            <err>: error code
*/
GLOBAL void cal_VtsRej (T_ACI_AT_CMD  cmdId, 
                        T_ACI_CME_ERR err)
{
  TRACE_FUNCTION ("cal_VtsRej ()");

  currDTMFCmd = AT_CMD_NONE;
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_CAL                  |
| STATE   : code                  ROUTINE : rAT_PercentCPI           |
+--------------------------------------------------------------------+

  PURPOSE : This function is used to indicate additional information
            about call progress.

            <cId>:     call identity
            <msgType>: source of information
            <ibt>:     in-band tones indication
            <tch>:     traffic channel indication 
*/

GLOBAL void rAT_PercentCPI ( SHORT         cId, 
                             T_ACI_CPI_MSG msgType,
                             T_ACI_CPI_IBT ibt,
                             T_ACI_CPI_TCH tch )
{
  TRACE_FUNCTION ("rAT_PercentCPI()");

  if (msgType NEQ CPI_MSG_NotPresent)
    currMsgType = msgType;

  if (ibt NEQ CPI_IBT_NotPresent)
    currIbt = ibt;

  if (tch NEQ CPI_TCH_NotPresent)
    currTch = tch;

  if (ibt EQ CPI_IBT_True AND
      tch EQ CPI_TCH_True     )
  {
    ui_signal (UI_SIG_NIBT, NIBT_OFF);
  }
  else
  {
    if (msgType EQ CPI_MSG_Disc)
    {
      ui_signal (UI_SIG_NIBT, NIBT_DISC);
    }
    else if (msgType EQ CPI_MSG_Alert)
    {
      ui_signal (UI_SIG_NIBT, NIBT_ALERT);
    }
  }
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_CAL                  |
| STATE   : code                  ROUTINE : rAT_PercentCTYI          |
+--------------------------------------------------------------------+

  PURPOSE : Indication about TTY service
*/

GLOBAL void rAT_PercentCTYI (T_ACI_CTTY_NEG neg,
                             T_ACI_CTTY_TRX trx)
{
  TRACE_FUNCTION ("rAT_PercentCTYI()");

//  if (cmhPrm[CMD_SRC_LCL].ccCmdPrm.CTTYmode EQ CTTY_MOD_ENABLE)
    TRACE_EVENT_P2 ("[0]CTYI: %d, %d", neg, trx);
}
