/* 
+----------------------------------------------------------------------------- 
|  Project :  GSM-F&D (8411)
|  Modul   :  SMI_DMY
+----------------------------------------------------------------------------- 
|  Copyright 2002 Texas Instruments Berlin, AG 
|                 All rights reserved. 
| 
|                 This file is confidential and a trade secret of Texas 
|                 Instruments Berlin, AG 
|                 The receipt of or possession of this file does not convey 
|                 any rights to reproduce or disclose its contents or to 
|                 manufacture, use, or sell anything it may describe, in 
|                 whole, or in part, without the specific written consent of 
|                 Texas Instruments Berlin, AG. 
+----------------------------------------------------------------------------- 
|  Purpose :  This module is used to implement all callback functions
|             which are not yet supported by the SMI.
+----------------------------------------------------------------------------- 
*/ 

#ifndef SMI_DMY_C
#define SMI_DMY_C
#endif

#define ENTITY_SMI

/*==== INCLUDES ===================================================*/
#include <stdio.h>
#include <string.h>
#include "typedefs.h"
#include "vsi.h"
#include "custom.h"
#include "gsm.h"
#include "prim.h"
#include "tok.h"
#include "message.h"

#include "aci_cmh.h"
#include "ksd.h"
#include "aca.h"
#include "smi.h"

/*==== CONSTANTS ==================================================*/
#undef FAC_TRACE   /* Facility Trace */

/*==== TYPES ======================================================*/

/*==== EXPORT =====================================================*/

/*==== VARIABLES ==================================================*/

/*==== FUNCTIONS ==================================================*/

#ifdef FAC_TRACE
LOCAL void smi_trace_ss_opcode  (UBYTE opcode);
LOCAL void smi_trace_ss_codes   (UBYTE ss_code);
LOCAL void smi_trace_ss_error   (UBYTE ss_error);
LOCAL void smi_trace_ss_status  (UBYTE ss_status);
LOCAL void smi_trace_clir_opt   (UBYTE clir_opt);
LOCAL void smi_trace_ovr_ctg    (UBYTE ovr_ctg);
LOCAL void smi_trace_cf_feature (T_CF_FEAT * feature);
LOCAL void smi_trace_cw_feature (T_Cx_BSG  * feature);
LOCAL void smi_trace_cb_feature (T_CB_INFO * feature);
LOCAL void smi_trace_bs_ts      (UBYTE bsTp, UBYTE bsCd);
LOCAL void smi_trace_bs         (UBYTE bsCd);
LOCAL void smi_trace_ts         (UBYTE bsCd);
LOCAL void smi_trace_ton        (UBYTE ton);
LOCAL void smi_trace_npi        (UBYTE npi);
LOCAL void smi_trace_tos        (UBYTE tos);
#endif /* FAC_TRACE */
/*
+--------------------------------------------------------------------+
| PROJECT : GSM-F&D (8411)              MODULE  : ACI_RET            |
| STATE   : code                        ROUTINE : smi_trace_ss_opcode|
+--------------------------------------------------------------------+

  PURPOSE : Traces the SS opcode

*/
#ifdef FAC_TRACE
LOCAL void smi_trace_ss_opcode (UBYTE opcode)
{

  switch (opcode)
  {
    case KSD_OP_IRGT:
      TRACE_EVENT ("Operation: Interrogate SS");
      break;
    case KSD_OP_REG:
      TRACE_EVENT ("Operation: Register SS");
      break;
    case KSD_OP_ERS:
      TRACE_EVENT ("Operation: Erase SS");
      break;
    case KSD_OP_ACT:
      TRACE_EVENT ("Operation: Activate SS");
      break;
    case KSD_OP_DEACT:
      TRACE_EVENT ("Operation: Deactivate SS");
      break;
  }

}
#endif /* FAC_TRACE */

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-F&D (8411)              MODULE  : ACI_RET            |
| STATE   : code                        ROUTINE : smi_trace_clir_opt |
+--------------------------------------------------------------------+

  PURPOSE : Traces the Clir Options

*/
#ifdef FAC_TRACE
LOCAL void smi_trace_clir_opt (UBYTE clir_opt)
{
  switch (clir_opt)
  {
    case KSD_CO_NOT_VALID:
      TRACE_EVENT ("CLIR opt : Not available");
      break;
    case KSD_CO_PERMANENT:
      TRACE_EVENT ("CLIR opt : Permanent");
      break;
    case KSD_CO_TEMPORARY:
      TRACE_EVENT ("CLIR opt : Temporary Default");
      break;
    case KSD_CO_ALLOWED:
      TRACE_EVENT ("CLIR opt : Allowed Temporary Default");
      break;
  }
}
#endif /* FAC_TRACE */

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-F&D (8411)              MODULE  : ACI_RET            |
| STATE   : code                        ROUTINE : smi_trace_bs       |
+--------------------------------------------------------------------+

  PURPOSE : Trace Bearer Service

*/
#ifdef FAC_TRACE
LOCAL void smi_trace_bs (UBYTE bsCd)
{
  switch (bsCd)
  {
    case KSD_BS_ALL_BS:
      TRACE_EVENT ("Bearer   : all bearer services");
      break;
    case KSD_BS_ALL_DATA_CDA:
      TRACE_EVENT ("Bearer   : all data CDA services");
      break;
    case KSD_BS_CDA_300:
      TRACE_EVENT ("Bearer   : data CDA 300 bps");
      break;
    case KSD_BS_CDA_1200:
      TRACE_EVENT ("Bearer   : data CDA 1200 bps");
      break;
    case KSD_BS_CDA_1200_75:
      TRACE_EVENT ("Bearer   : data CDA 1200/75 bps");
      break;
    case KSD_BS_CDA_2400:
      TRACE_EVENT ("Bearer   : data CDA 2400 bps");
      break;
    case KSD_BS_CDA_4800:
      TRACE_EVENT ("Bearer   : data CDA 4800 bps");
      break;
    case KSD_BS_CDA_9600:
      TRACE_EVENT ("Bearer   : data CDA 9600 bps");
      break;
    case KSD_BS_ALL_DATA_CDS:
      TRACE_EVENT ("Bearer   : all data CDS services");
      break;
    case KSD_BS_CDS_1200:
      TRACE_EVENT ("Bearer   : data CDS 1200 bps");
      break;
  }

  switch (bsCd)
  {
    /*
     * this construct needed due to a bug in
     * the target compiler (dont use switch
     * with too many cases, the assembler produces
     * failures)
     */
    case KSD_BS_CDS_2400:
      TRACE_EVENT ("Bearer   : data CDS 2400 bps");
      break;
    case KSD_BS_CDS_4800:
      TRACE_EVENT ("Bearer   : data CDS 4800 bps");
      break;
    case KSD_BS_CDS_9600:
      TRACE_EVENT ("Bearer   : data CDS 9600 bps");
      break;
    case KSD_BS_ALL_DATA_PAD:
      TRACE_EVENT ("Bearer   : all data PAD services");
      break;
    case KSD_BS_PAD_300:
      TRACE_EVENT ("Bearer   : data PAD 300 bps");
      break;
    case KSD_BS_PAD_1200:
      TRACE_EVENT ("Bearer   : data PAD 1200 bps");
      break;
    case KSD_BS_PAD_1200_75:
      TRACE_EVENT ("Bearer   : data PAD 1200/75 bps");
      break;
    case KSD_BS_PAD_2400:
      TRACE_EVENT ("Bearer   : data PAD 2400 bps");
      break;
    case KSD_BS_PAD_4800:
      TRACE_EVENT ("Bearer   : data PAD 4800 bps");
      break;
    case KSD_BS_PAD_9600:
      TRACE_EVENT ("Bearer   : data PAD 9600 bps");
      break;
  }

  switch (bsCd)
  {
    /*
     * this construct needed due to a bug in
     * the target compiler (dont use switch
     * with too many cases, the assembler produces
     * failures)
     */
    case KSD_BS_ALL_DATA_PDS:
      TRACE_EVENT ("Bearer   : all data PDS services");
      break;
    case KSD_BS_PDS_2400:
      TRACE_EVENT ("Bearer   : data PDS 2400 bps");
      break;
    case KSD_BS_PDS_4800:
      TRACE_EVENT ("Bearer   : data PDS 4800 bps");
      break;
    case KSD_BS_PDS_9600:
      TRACE_EVENT ("Bearer   : data PDS 9600 bps");
      break;
    case KSD_BS_SPCH_ALT_CDA:
      TRACE_EVENT ("Bearer   : all data CDA alternating speech");
      break;
    case KSD_BS_SPCH_ALT_CDS:
      TRACE_EVENT ("Bearer   : all data CDS alternating speech");
      break;
    case KSD_BS_SPCH_FLD_CDA:
      TRACE_EVENT ("Bearer   : all data speech followed by CDA");
      break;
    case KSD_BS_SPCH_FLD_CDS:
      TRACE_EVENT ("Bearer   : all data speech followed by CDS");
      break;
    case KSD_BS_ALL_DC_ASYN:
      TRACE_EVENT ("Bearer   : all data circuit asynchronous");
      break;
    case KSD_BS_ALL_ASYN:
      TRACE_EVENT ("Bearer   : all asynchronous");
      break;
  }

  switch (bsCd)
  {
    /*
     * this construct needed due to a bug in
     * the target compiler (dont use switch
     * with too many cases, the assembler produces
     * failures)
     */
    case KSD_BS_ALL_DC_SYN:
      TRACE_EVENT ("Bearer   : all data circuit synchronous");
      break;
    case KSD_BS_ALL_SYN:
      TRACE_EVENT ("Bearer   : all synchronous");
      break;
    case KSD_BS_ALL_PSSS:
      TRACE_EVENT ("Bearer   : all PLMN specific bearer service");
      break;
    case KSD_BS_PLMN1:
      TRACE_EVENT ("Bearer   : PLMN specific bearer service 1");
      break;
    case KSD_BS_PLMN2:
      TRACE_EVENT ("Bearer   : PLMN specific bearer service 2");
      break;
    case KSD_BS_PLMN3:
      TRACE_EVENT ("Bearer   : PLMN specific bearer service 3");
      break;
    case KSD_BS_PLMN4:
      TRACE_EVENT ("Bearer   : PLMN specific bearer service 4");
      break;
    case KSD_BS_PLMN5:
      TRACE_EVENT ("Bearer   : PLMN specific bearer service 5");
      break;
    case KSD_BS_PLMN6:
      TRACE_EVENT ("Bearer   : PLMN specific bearer service 6");
      break;
    case KSD_BS_PLMN7:
      TRACE_EVENT ("Bearer   : PLMN specific bearer service 7");
      break;
  }

  switch (bsCd)
  {
    /*
     * this construct needed due to a bug in
     * the target compiler (dont use switch
     * with too many cases, the assembler produces
     * failures)
     */
    case KSD_BS_PLMN8:
      TRACE_EVENT ("Bearer   : PLMN specific bearer service 8");
      break;
    case KSD_BS_PLMN9:
      TRACE_EVENT ("Bearer   : PLMN specific bearer service 9");
      break;
    case KSD_BS_PLMNA:
      TRACE_EVENT ("Bearer   : PLMN specific bearer service 10");
      break;
    case KSD_BS_PLMNB:
      TRACE_EVENT ("Bearer   : PLMN specific bearer service 11");
      break;
    case KSD_BS_PLMNC:
      TRACE_EVENT ("Bearer   : PLMN specific bearer service 12");
      break;
    case KSD_BS_PLMND:
      TRACE_EVENT ("Bearer   : PLMN specific bearer service 13");
      break;
    case KSD_BS_PLMNE:
      TRACE_EVENT ("Bearer   : PLMN specific bearer service 14");
      break;
    case KSD_BS_PLMNF:
      TRACE_EVENT ("Bearer   : PLMN specific bearer service 15");
      break;
  }
}
#endif /* FAC_TRACE */


/*
+--------------------------------------------------------------------+
| PROJECT : GSM-F&D (8411)              MODULE  : ACI_RET            |
| STATE   : code                        ROUTINE : smi_trace_ts       |
+--------------------------------------------------------------------+

  PURPOSE : Trace Tele Service

*/
#ifdef FAC_TRACE
LOCAL void smi_trace_ts (UBYTE bsCd)
{
  switch (bsCd)
  {
    case KSD_TS_ALL_TS:
      TRACE_EVENT ("Tele     : all tele services");
      break;
    case KSD_TS_ALL_SPCH:
      TRACE_EVENT ("Tele     : all speech transmission services");
      break;
    case KSD_TS_TLPHNY:
      TRACE_EVENT ("Tele     : telephony");
      break;
    case KSD_TS_EMRGNCY:
      TRACE_EVENT ("Tele     : emergency calls");
      break;
    case KSD_TS_ALL_SMS:
      TRACE_EVENT ("Tele     : all SMS services");
      break;
    case KSD_TS_SMS_MT:
      TRACE_EVENT ("Tele     : SMS MT PP");
      break;
    case KSD_TS_SMS_MO:
      TRACE_EVENT ("Tele     : SMS MO PP");
      break;
    case KSD_TS_ALL_FAX:
      TRACE_EVENT ("Tele     : all FAX services");
      break;
    case KSD_TS_FAX3_ALT_SPCH:
      TRACE_EVENT ("Tele     : FAX group 3 alternating speech");
      break;
    case KSD_TS_FAX3_AUTO:
      TRACE_EVENT ("Tele     : FAX group 3 automatic");
      break;
  }

  switch (bsCd)
  {
    /*
     * this construct needed due to a bug in
     * the target compiler (dont use switch
     * with too many cases, the assembler produces
     * failures)
     */
    case KSD_TS_FAX4:
      TRACE_EVENT ("Tele     : FAX group 4");
      break;
    case KSD_TS_ALL_DATA:
      TRACE_EVENT ("Tele     : all FAX and SMS services");
      break;
    case KSD_TS_ALL_XCPT_SMS:
      TRACE_EVENT ("Tele     : all FAX and Speech services");
      break;
    case KSD_TS_ALL_PSSS:
      TRACE_EVENT ("Tele     : all PLMN specific tele services");
      break;
    case KSD_TS_PLMN1:
      TRACE_EVENT ("Tele     : PLMN specifc tele service 1");
      break;
    case KSD_TS_PLMN2:
      TRACE_EVENT ("Tele     : PLMN specifc tele service 2");
      break;
    case KSD_TS_PLMN3:
      TRACE_EVENT ("Tele     : PLMN specifc tele service 3");
      break;
    case KSD_TS_PLMN4:
      TRACE_EVENT ("Tele     : PLMN specifc tele service 4");
      break;
    case KSD_TS_PLMN5:
      TRACE_EVENT ("Tele     : PLMN specifc tele service 5");
      break;
    case KSD_TS_PLMN6:
      TRACE_EVENT ("Tele     : PLMN specifc tele service 6");
      break;
  }

  switch (bsCd)
  {
    /*
     * this construct needed due to a bug in
     * the target compiler (dont use switch
     * with too many cases, the assembler produces
     * failures)
     */
    case KSD_TS_PLMN7:
      TRACE_EVENT ("Tele     : PLMN specifc tele service 7");
      break;
    case KSD_TS_PLMN8:
      TRACE_EVENT ("Tele     : PLMN specifc tele service 8");
      break;
    case KSD_TS_PLMN9:
      TRACE_EVENT ("Tele     : PLMN specifc tele service 9");
      break;
    case KSD_TS_PLMNA:
      TRACE_EVENT ("Tele     : PLMN specifc tele service 10");
      break;
    case KSD_TS_PLMNB:
      TRACE_EVENT ("Tele     : PLMN specifc tele service 11");
      break;
    case KSD_TS_PLMNC:
      TRACE_EVENT ("Tele     : PLMN specifc tele service 12");
      break;
    case KSD_TS_PLMND:
      TRACE_EVENT ("Tele     : PLMN specifc tele service 13");
      break;
    case KSD_TS_PLMNE:
      TRACE_EVENT ("Tele     : PLMN specifc tele service 14");
      break;
    case KSD_TS_PLMNF: 
      TRACE_EVENT ("Bearer   : PLMN specifc tele service 15");
      break;
  }
}
#endif /* FAC_TRACE */
 

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-F&D (8411)              MODULE  : ACI_RET            |
| STATE   : code                        ROUTINE : smi_trace_bs_ts    |
+--------------------------------------------------------------------+

  PURPOSE : Traces Bearer or Teleservice

*/
#ifdef FAC_TRACE
LOCAL void smi_trace_bs_ts (UBYTE bsTp, UBYTE bsCd)
{
  switch (bsTp)
  {
    case KSD_BS_TP_None:
      TRACE_EVENT ("BS-TS Typ: Not available");
      break;
    case KSD_BS_TP_Tele:
      TRACE_EVENT ("BS-TS Typ: Tele Service");
      smi_trace_ts (bsCd);
      break;
    case KSD_BS_TP_Bearer:
      TRACE_EVENT ("BS-TS Typ: Bearer Service");
      smi_trace_bs (bsCd);
      break;
  }
}
#endif /* FAC_TRACE */

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-F&D (8411)              MODULE  : ACI_RET            |
| STATE   : code                        ROUTINE : smi_trace_ton      |
+--------------------------------------------------------------------+

  PURPOSE : Trace type of number

*/
#ifdef FAC_TRACE
LOCAL void smi_trace_ton (UBYTE ton)
{
  switch (ton)
  {
    case TON_NotPresent:
      TRACE_EVENT ("TON      : Not available");
      break;
    case TON_Unknown:
      TRACE_EVENT ("TON      : unknown");
      break;
    case TON_International:
      TRACE_EVENT ("TON      : international");
      break;
    case TON_National:
      TRACE_EVENT ("TON      : national");
      break;
    case TON_NetSpecific:
      TRACE_EVENT ("TON      : Network specific");
      break;
    case TON_DedAccess:
      TRACE_EVENT ("TON      : Dedicated Access");
      break;
  }
}
#endif /* FAC_TRACE */

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-F&D (8411)              MODULE  : ACI_RET            |
| STATE   : code                        ROUTINE : smi_trace_npi      |
+--------------------------------------------------------------------+

  PURPOSE : Trace Numbering plan identifier

*/
#ifdef FAC_TRACE
LOCAL void smi_trace_npi (UBYTE npi)
{
  switch (npi)
  {
    case NPI_NotPresent:
      TRACE_EVENT ("NPI      : Not available");
      break;
    case NPI_Unknown:
      TRACE_EVENT ("NPI      : unknown");
      break;
    case NPI_IsdnTelephony:
      TRACE_EVENT ("NPI      : ISDN Telephony");
      break;
    case NPI_Data:
      TRACE_EVENT ("NPI      : Data");
      break;
    case NPI_Telex:
      TRACE_EVENT ("NPI      : Telex");
      break;
    case NPI_National:
      TRACE_EVENT ("NPI      : National");
      break;
    case NPI_Private:
      TRACE_EVENT ("NPI      : Private");
      break;
  }
}
#endif /* FAC_TRACE */

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-F&D (8411)              MODULE  : ACI_RET            |
| STATE   : code                        ROUTINE : smi_trace_tos      |
+--------------------------------------------------------------------+

  PURPOSE : Trace Type of Subaddress

*/
#ifdef FAC_TRACE
LOCAL void smi_trace_tos (UBYTE tos)
{
  switch (tos)
  {
    case TOS_NotPresent:
      TRACE_EVENT ("TOS      : Not available");
      break;
    case TOS_Nsap:
      TRACE_EVENT ("TOS      : NSAP");
      break;
    case TOS_User:
      TRACE_EVENT ("TOS      : User defined");
      break;
  }
}
#endif /* FAC_TRACE */


/*
+--------------------------------------------------------------------+
| PROJECT : GSM-F&D (8411)            MODULE  : ACI_RET              |
| STATE   : code                      ROUTINE : smi_trace_cf_feature |
+--------------------------------------------------------------------+

  PURPOSE : Traces the Call Forwarding Feature

*/
#ifdef FAC_TRACE
LOCAL void smi_trace_cf_feature (T_CF_FEAT * feature)
{
  char buf[60];
  TRACE_EVENT ("----------------------------------------");

  smi_trace_bs_ts (feature->bsTp, feature->bsCd);
  smi_trace_ss_status (feature->ssSt);
  if (strlen ((CHAR*)feature->num) NEQ 0)
  {
    sprintf (buf, "Number   : %s", feature->num);
    TRACE_EVENT (buf);
    smi_trace_ton (feature->ton);
    smi_trace_npi (feature->npi);
  }
  if (strlen ((CHAR*)feature->sub) NEQ 0)
  {
    sprintf (buf, "Subaddr  : %s", feature->sub);
    TRACE_EVENT (buf);
    smi_trace_tos (feature->tos);
  }
}
#endif /* FAC_TRACE */

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-F&D (8411)            MODULE  : ACI_RET              |
| STATE   : code                      ROUTINE : smi_trace_cw_feature |
+--------------------------------------------------------------------+

  PURPOSE : Traces the Call Waiting Feature

*/
#ifdef FAC_TRACE
LOCAL void smi_trace_cw_feature (T_Cx_BSG * feature)
{
  TRACE_EVENT ("----------------------------------------");

  smi_trace_bs_ts (feature->bsTp, feature->bsCd);
}
#endif /* FAC_TRACE */

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-F&D (8411)            MODULE  : ACI_RET              |
| STATE   : code                      ROUTINE : smi_trace_cb_feature |
+--------------------------------------------------------------------+

  PURPOSE : Traces the Call Barring Feature

*/
#ifdef FAC_TRACE
LOCAL void smi_trace_cb_feature (T_CB_INFO * feature)
{
  TRACE_EVENT ("----------------------------------------");

  smi_trace_bs_ts (feature->bsTp, feature->bsCd);
  smi_trace_ss_status (feature->ssSt);
}
#endif /* FAC_TRACE */

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-F&D (8411)              MODULE  : ACI_RET            |
| STATE   : code                        ROUTINE : smi_trace_ovr_ctg  |
+--------------------------------------------------------------------+

  PURPOSE : Traces the Override Category

*/
#ifdef FAC_TRACE
LOCAL void smi_trace_ovr_ctg (UBYTE ovr_ctg)
{
  switch (ovr_ctg)
  {
    case KSD_OVR_CAT_NOT_VALID:
      TRACE_EVENT ("Ovr cat  : Not available");
      break;
    case KSD_OVR_CAT_ENABLED:
      TRACE_EVENT ("Ovr cat  : Enabled");
      break;
    case KSD_OVR_CAT_DISABLED:
      TRACE_EVENT ("Ovr cat  : Disabled");
      break;
  }
}
#endif /* FAC_TRACE */

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-F&D (8411)              MODULE  : ACI_RET            |
| STATE   : code                        ROUTINE : smi_trace_ss_codes |
+--------------------------------------------------------------------+

  PURPOSE : Traces the SS code

*/
#ifdef FAC_TRACE
LOCAL void smi_trace_ss_codes (UBYTE ss_code)
{
  switch (ss_code)
  {
    case KSD_SS_NONE:
      TRACE_EVENT ("SS Code  : not present");
      break;
    case KSD_SS_CLIP:
      TRACE_EVENT ("SS Code  : CLIP");
      break;
    case KSD_SS_CLIR:
      TRACE_EVENT ("SS Code  : CLIR");
      break;
    case KSD_SS_COLP:
      TRACE_EVENT ("SS Code  : COLP");
      break;
    case KSD_SS_COLR:
      TRACE_EVENT ("SS Code  : COLR");
      break;
    case KSD_SS_ALL_FWSS:
      TRACE_EVENT ("SS Code  : All Forwarding SS");
      break;
    case KSD_SS_CFU:
      TRACE_EVENT ("SS Code  : CF unconditional");
      break;
    case KSD_SS_ALL_CFWSS:
      TRACE_EVENT ("SS Code  : All conditional CF SS");
      break;
    case KSD_SS_CFB:
      TRACE_EVENT ("SS Code  : CF on MS busy");
      break;
    case KSD_SS_CFNRY:
      TRACE_EVENT ("SS Code  : CF on no reply");
      break;
  }

  switch (ss_code)
  {
    /*
     * this construct needed due to a bug in
     * the target compiler (dont use switch
     * with too many cases, the assembler produces
     * failures)
     */
    case KSD_SS_CFNRC:
      TRACE_EVENT ("SS Code  : CF on MS not reachable");
      break;
    case KSD_SS_CW:
      TRACE_EVENT ("SS Code  : Call Waiting");
      break;
    case KSD_SS_CCBS:
      TRACE_EVENT ("SS Code  : Completion of call to busy MS");
      break;
    case KSD_SS_ALL_CBSS:
      TRACE_EVENT ("SS Code  : All barring SS");
      break;
    case KSD_SS_BOC:
      TRACE_EVENT ("SS Code  : barring of outgoing calls");
      break;
    case KSD_SS_BAOC:
      TRACE_EVENT ("SS Code  : barring of all outgoing calls");
      break;
    case KSD_SS_BOIC:
      TRACE_EVENT ("SS Code  : barring of outging international calls");
      break;
    case KSD_SS_BOICXH:
      TRACE_EVENT ("SS Code  : barring of outgoing int. calls ex. HPLMN");
      break;
    case KSD_SS_BIC:
      TRACE_EVENT ("SS Code  : barring of incoming calls");
      break;
    case KSD_SS_BAIC:
      TRACE_EVENT ("SS Code  : barring of all incoming calls");
      break;
  }

  switch (ss_code)
  {
    /*
     * this construct needed due to a bug in
     * the target compiler (dont use switch
     * with too many cases, the assembler produces
     * failures)
     */
    case KSD_SS_BICRM:
      TRACE_EVENT ("SS Code  : barring of incoming calls when roaming");
      break;
    case KSD_SS_PIN1:
      TRACE_EVENT ("SS Code  : PIN 1");
      break;
    case KSD_SS_PIN2:
      TRACE_EVENT ("SS Code  : PIN 2");
      break;
  }
}
#endif /* FAC_TRACE */

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-F&D (8411)              MODULE  : ACI_RET            |
| STATE   : code                        ROUTINE : smi_trace_ss_error |
+--------------------------------------------------------------------+

  PURPOSE : Traces the SS error

*/
#ifdef FAC_TRACE
LOCAL void smi_trace_ss_error (UBYTE ss_error)
{
  switch (ss_error)
  {
    case KSD_NO_ERROR:
      TRACE_EVENT ("SS Error : No Error");
      break;
    case KSD_ERR_UNKNOWN_SUBSCRIBER:
      TRACE_EVENT ("SS Error : Unknown Subscriber");
      break;
    case KSD_ERR_ILLEGAL_SUBSCRIBER:
      TRACE_EVENT ("SS Error : Illegal Subscriber");
      break;
    case KSD_ERR_BEARER_SVC_NOT_PROV:
      TRACE_EVENT ("SS Error : Bearer Service not provisioned");
      break;
    case KSD_ERR_TELE_SVC_NOT_PROV:
      TRACE_EVENT ("SS Error : Tele Service not provisioned");
      break;
    case KSD_ERR_ILLEGAL_EQUIPMENT:
      TRACE_EVENT ("SS Error : Illegal Equipment");
      break;
    case KSD_ERR_CALL_BARRED:
      TRACE_EVENT ("SS Error : Call Barred");
      break;
    case KSD_ERR_ILLEGAL_SS_OPERATION:
      TRACE_EVENT ("SS Error : Illegal SS Operation");
      break;
    case KSD_ERR_SS_ERR_STATUS:
      TRACE_EVENT ("SS Error : SS Error Status");
      break;
    case KSD_ERR_SS_NOT_AVAIL:
      TRACE_EVENT ("SS Error : SS not available");
      break;
  }

  switch (ss_error)
  {
    /*
     * this construct needed due to a bug in
     * the target compiler (dont use switch
     * with too many cases, the assembler produces
     * failures)
     */
    case KSD_ERR_SS_SUBS_VIOLATION:
      TRACE_EVENT ("SS Error : SS Subscriber Violation");
      break;
    case KSD_ERR_SS_INCOMP:
      TRACE_EVENT ("SS Error : SS Incompatibility");
      break;
    case KSD_ERR_FAC_NOT_SUPPORTED:
      TRACE_EVENT ("SS Error : Facility not supported");
      break;
    case KSD_ERR_ABSENT_SUBS:
      TRACE_EVENT ("SS Error : Absent Subscriber");
      break;
    case KSD_ERR_SYSTEM_FAIL:
      TRACE_EVENT ("SS Error : System Failure");
      break;
    case KSD_ERR_DATA_MISSING:
      TRACE_EVENT ("SS Error : Data Missing");
      break;
    case KSD_ERR_UNEXPECT_DATA:
      TRACE_EVENT ("SS Error : Unexpected Data");
      break;
    case KSD_ERR_PWD_REG_FAIL:
      TRACE_EVENT ("SS Error : Password Registration Failure");
      break;
    case KSD_ERR_NEG_PWD_CHECK:
      TRACE_EVENT ("SS Error : Negative Password Check");
      break;
    case KSD_ERR_NUM_PWD_VIOLATION:
      TRACE_EVENT ("SS Error : Number Password Violation");
      break;
  }

  switch (ss_error)
  {
    /*
     * this construct needed due to a bug in
     * the target compiler (dont use switch
     * with too many cases, the assembler produces
     * failures)
     */
    case KSD_ERR_UNKNOWN_ALPHA:
      TRACE_EVENT ("SS Error : Unknown Alpha");
      break;
    case KSD_ERR_USSD_BUSY:
      TRACE_EVENT ("SS Error : Unstructured SS busy");
      break;
    case KSD_ERR_MAX_NUM_MPTY_EXCEED:
      TRACE_EVENT ("SS Error : Too many members in multiparty");
      break;
    case KSD_ERR_RESOURCE_NOT_AVAIL:
      TRACE_EVENT ("SS Error : Resource not available");
      break;
    case KSD_GEN_PROB_UNRECOG_CMP:
      TRACE_EVENT ("SS Error : Unrecognized Component");
      break;
    case KSD_GEN_PROB_MISTYPED_CMP:
      TRACE_EVENT ("SS Error : Mistyped Component");
      break;
    case KSD_GEN_PROB_BAD_STRUCT_CMP:
      TRACE_EVENT ("SS Error : bad structured Component");
      break;
    case KSD_INV_PROB_DUPL_INV_ID:
      TRACE_EVENT ("SS Error : Duplicated Invoke ID");
      break;
    case KSD_INV_PROB_UNRECOG_OP:
      TRACE_EVENT ("SS Error : Unrecognized Opcode");
      break;
    case KSD_INV_PROB_MISTYPED_PAR:
      TRACE_EVENT ("SS Error : Mistyped Parameter");
      break;
  }

  switch (ss_error)
  {
    /*
     * this construct needed due to a bug in
     * the target compiler (dont use switch
     * with too many cases, the assembler produces
     * failures)
     */
    case KSD_INV_PROB_RESOURCE_LIM:
      TRACE_EVENT ("SS Error : Resource Limitation");
      break;
    case KSD_INV_PROB_INIT_RELEASE:
      TRACE_EVENT ("SS Error : Init Release Problem");
      break;
    case KSD_INV_PROB_UNRECOG_LNK_ID:
      TRACE_EVENT ("SS Error : unrecognized Link ID");
      break;
    case KSD_INV_PROB_LNK_RES_UNEXP:
      TRACE_EVENT ("SS Error : unexpected link result");
      break;
    case KSD_INV_PROB_UNEXP_LNK_OP:
      TRACE_EVENT ("SS Error : unrecognized Link Operation");
      break;
    case KSD_RES_PROB_UNRECOG_INV_ID:
      TRACE_EVENT ("SS Error : unrecognized Invoke ID");
      break;
    case KSD_RES_PROB_RET_RES_UNEXP:
      TRACE_EVENT ("SS Error : unexpected return result");
      break;
    case KSD_RES_PROB_MISTYPED_PAR:
      TRACE_EVENT ("SS Error : mistyped parameter in return result");
      break;
    case KSD_ERR_PROB_UNRECOG_INV_ID:
      TRACE_EVENT ("SS Error : unrecognized Invoke ID");
      break;
    case KSD_ERR_PROB_RET_ERR_UNEXP:
      TRACE_EVENT ("SS Error : return error unexpected");
      break;
  }

  switch (ss_error)
  {
    /*
     * this construct needed due to a bug in
     * the target compiler (dont use switch
     * with too many cases, the assembler produces
     * failures)
     */
    case KSD_ERR_PROB_UNRECOG_ERR:
      TRACE_EVENT ("SS Error : unrecognized error");
      break;
    case KSD_ERR_PROB_UNEXP_ERR:
      TRACE_EVENT ("SS Error : unexpected error");
      break;
    case KSD_ERR_PROB_MISTYPED_PAR:
      TRACE_EVENT ("SS Error : mistyped error");
      break;
    case KSD_ERR_FATAL_INV_RESULT:
      TRACE_EVENT ("SS Error : fatal error invoke result");
      break;
    case KSD_ERR_FATAL_CCD_DEC:
      TRACE_EVENT ("SS Error : fatal error CCD decode");
      break;
    case KSD_ERR_FATAL_SS_ENT:
      TRACE_EVENT ("SS Error : fatal error SS entity");
      break;
  }
}
#endif /* FAC_TRACE */

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-F&D (8411)              MODULE  : ACI_RET            |
| STATE   : code                        ROUTINE : smi_trace_ss_status|
+--------------------------------------------------------------------+

  PURPOSE : Traces the SS Status

*/
#ifdef FAC_TRACE
LOCAL void smi_trace_ss_status (UBYTE ss_status)
{
  switch (ss_status)
  {
    case KSD_ST_NOT_VALID:
      TRACE_EVENT ("SS Status: Not available");
      break;
    case KSD_ST_NONE:
      TRACE_EVENT ("SS Status: status none");
      break;
    case KSD_ST_A:
      TRACE_EVENT ("SS Status: Active");
      break;
    case KSD_ST_R:
      TRACE_EVENT ("SS Status: Registered");
      break;
    case KSD_ST_RA:
      TRACE_EVENT ("SS Status: Registered, Active");
      break;
    case KSD_ST_P:
      TRACE_EVENT ("SS Status: Provisioned");
      break;
    case KSD_ST_PA:
      TRACE_EVENT ("SS Status: Provisioned, Active");
      break;
    case KSD_ST_PR:
      TRACE_EVENT ("SS Status: Provisioned, Registered");
      break;
    case KSD_ST_PRA:
      TRACE_EVENT ("SS Status: Provisioned, Registered, Active");
      break;
  }

  switch (ss_status)
  {
    /*
     * this construct needed due to a bug in
     * the target compiler (dont use switch
     * with too many cases, the assembler produces
     * failures)
     */
    case KSD_ST_Q:
      TRACE_EVENT ("SS Status: Quiescent");
      break;
    case KSD_ST_QA:
      TRACE_EVENT ("SS Status: Quiescent, Active");
      break;
    case KSD_ST_QR:
      TRACE_EVENT ("SS Status: Quiescent, Registered");
      break;
    case KSD_ST_QRA:
      TRACE_EVENT ("SS Status: Quiescent, Registered, Active");
      break;
    case KSD_ST_QP:
      TRACE_EVENT ("SS Status: Quiescent, Provisioned");
      break;
    case KSD_ST_QPA:
      TRACE_EVENT ("SS Status: Quiescent, Provisioned, Active");
      break;
    case KSD_ST_QPR:
      TRACE_EVENT ("SS Status: Quiescent, Provisioned, Registered");
      break;
    case KSD_ST_QPRA:
      TRACE_EVENT ("SS Status: Quiescent, Provisioned, Registered, Active");
      break;
  }
}
#endif /* FAC_TRACE */

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_DMY                  |
| STATE   : code                  ROUTINE : rAT_PlusDR               |
+--------------------------------------------------------------------+

  PURPOSE : 
*/
GLOBAL void rAT_PlusDR (T_ACI_DR_TYP type)
{
  TRACE_FUNCTION ("rAT_PlusDR ()");
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_DMY                  |
| STATE   : code                  ROUTINE : rAT_PlusCR               |
+--------------------------------------------------------------------+

  PURPOSE : 
*/
GLOBAL void rAT_PlusCR (T_ACI_CR_SERV service)
{
  TRACE_FUNCTION ("rAT_PlusCR ()");
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_DMY                  |
| STATE   : code                  ROUTINE : rAT_PlusCPIN             |
+--------------------------------------------------------------------+

  PURPOSE : 
*/
GLOBAL void rAT_PlusCPIN (T_ACI_CPIN_RSLT rslt)
{
  TRACE_FUNCTION ("rAT_PlusCPIN");
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_DMY                  |
| STATE   : code                  ROUTINE : rAT_PlusCIMI             |
+--------------------------------------------------------------------+

  PURPOSE : 
*/
GLOBAL void rAT_PlusCIMI (CHAR * imsi)
{
  TRACE_FUNCTION ("rAT_PlusCIMI");
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_DMY                  |
| STATE   : code                  ROUTINE : rAT_PlusCSMS             |
+--------------------------------------------------------------------+

  PURPOSE : 
*/
GLOBAL void rAT_PlusCSMS (T_ACI_CSMS_SERV service,
                          T_ACI_CSMS_SUPP mt,
                          T_ACI_CSMS_SUPP mo,
                          T_ACI_CSMS_SUPP bm)
{
  TRACE_FUNCTION ("rAT_PlusCSMS");
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_DMY                  |
| STATE   : code                  ROUTINE : rAT_PlusCMT              |
+--------------------------------------------------------------------+

  PURPOSE : 
*/
EXTERN void rAT_PlusCMT (T_ACI_CMGL_SM*  sm)
{
  TRACE_FUNCTION ("rAT_PlusCMT");
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_DMY                  |
| STATE   : code                  ROUTINE : rAT_PlusCDS              |
+--------------------------------------------------------------------+

  PURPOSE : 
*/
GLOBAL void rAT_PlusCDS (T_ACI_CDS_SM *st)
{
  TRACE_FUNCTION ("rAT_PlusCDS");
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_DMY                  |
| STATE   : code                  ROUTINE : rAT_PlusCMGC             |
+--------------------------------------------------------------------+

  PURPOSE : 
*/
GLOBAL void rAT_PlusCMGC (UBYTE mr)
{
  TRACE_FUNCTION ("rAT_PlusCMGC");
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_DMY                  |
| STATE   : code                  ROUTINE : rAT_PlusCPMS             |
+--------------------------------------------------------------------+

  PURPOSE : 
*/
GLOBAL void rAT_PlusCPMS (T_ACI_SMS_STOR_OCC * mem1,
                          T_ACI_SMS_STOR_OCC * mem2,
                          T_ACI_SMS_STOR_OCC * mem3)
{
  TRACE_FUNCTION ("rAT_PlusCPMS");
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_DMY                  |
| STATE   : code                  ROUTINE : rAT_PlusCMGL             |
+--------------------------------------------------------------------+

  PURPOSE : 
*/
GLOBAL void rAT_PlusCMGL ( T_ACI_CMGL_SM * smLst )
{
  TRACE_FUNCTION ("rAT_PlusCMGL");
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-F&D (8411)              MODULE  : ACI_RET            |
| STATE   : code                        ROUTINE : rAT_PercentSATI    |
+--------------------------------------------------------------------+

  PURPOSE : handles AT_PercentSATI call back

*/
#ifdef SIM_TOOLKIT
GLOBAL void rAT_PercentSATI  ( SHORT           len,
                               UBYTE          *satCmd)
{
  TRACE_FUNCTION("rAT_PercentSATI()");
}
#endif

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-F&D (8411)              MODULE  : ACI_RET            |
| STATE   : code                        ROUTINE : rAT_PercentSATE    |
+--------------------------------------------------------------------+

  PURPOSE : handles AT_PercentSATE call back

*/
#ifdef SIM_TOOLKIT
GLOBAL void rAT_PercentSATE  ( SHORT           len,
                               UBYTE          *satCmd)
{
  TRACE_FUNCTION("rAT_PercentSATE()");
}
#endif

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-F&D (8411)              MODULE  : ACI_RET            |
| STATE   : code                        ROUTINE : rAT_PercentSATN    |
+--------------------------------------------------------------------+

  PURPOSE : handles AT_PercentSATN call back

*/
#ifdef SIM_TOOLKIT
GLOBAL void rAT_PercentSATN  ( SHORT           len,
                               UBYTE          *satCmd)
{
  TRACE_FUNCTION("rAT_PercentSATN()");
}
#endif

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-F&D (8411)              MODULE  : ACI_RET            |
| STATE   : code                        ROUTINE : rAT_PercentSATA    |
+--------------------------------------------------------------------+

  PURPOSE : handles AT_PercentSATA call back

*/
#ifdef SIM_TOOLKIT
GLOBAL void rAT_PercentSATA  ( SHORT cId, LONG rdlTimeout_ms,
                               T_ACI_SATA_ADD *addParm )
{
  TRACE_FUNCTION("rAT_PercentSATA()");
}
#endif


/*
+--------------------------------------------------------------------+
| PROJECT : GSM-F&D (8411)              MODULE  : ACI_RET            |
| STATE   : code                        ROUTINE : rAT_PercentKSIR    |
+--------------------------------------------------------------------+

  PURPOSE : handles AT_PercentKSIR call back

*/

GLOBAL void rAT_PercentKSIR  ( T_ACI_KSIR *ksStat )
{
#ifdef FAC_TRACE
  char buf[60];
#endif

  TRACE_FUNCTION("rAT_PercentKSIR()");

#ifdef FAC_TRACE
  TRACE_EVENT ("########################################");
  switch (ksStat->ksdCmd)
  {
    case KSD_CMD_CB:
      /*
       * call barring
       */
      TRACE_EVENT ("Call Barring Result");
      smi_trace_ss_opcode (ksStat->ir.rKSCB.opCd);
      smi_trace_ss_codes (ksStat->ir.rKSCB.ssCd);      
      smi_trace_ss_error (ksStat->ir.rKSCB.ssErr);      
      if (ksStat->ir.rKSCB.ssErr EQ KSD_NO_ERROR)
      {
        if (ksStat->ir.rKSCB.c_cbInfoLst EQ 0)
        {
          TRACE_EVENT ("Call Barring Info List not available");
        }
        else
        {
          UBYTE i;
          T_CB_INFO * feature;

          TRACE_EVENT ("Call Barring Info List available");
          feature = ksStat->ir.rKSCB.cbInfoLst;
          for (i=0;i<ksStat->ir.rKSCB.c_cbInfoLst;i++,feature++)
            smi_trace_cb_feature (feature);
        }
      }
      break;
    case KSD_CMD_CF:
      /*
       * call forwarding
       */
      TRACE_EVENT ("Call Forwarding Result");
      smi_trace_ss_opcode (ksStat->ir.rKSCF.opCd);
      smi_trace_ss_codes (ksStat->ir.rKSCF.ssCd);      
      smi_trace_ss_error (ksStat->ir.rKSCF.ssErr);      
      if (ksStat->ir.rKSCF.ssErr EQ KSD_NO_ERROR)
      {
        if (ksStat->ir.rKSCF.c_cfFeatLst EQ 0)
        {
          TRACE_EVENT ("Call Forwarding Feature List not available");
        }
        else
        {
          UBYTE i;
          T_CF_FEAT * feature;

          TRACE_EVENT ("Call Forwarding Feature List available");
          feature = ksStat->ir.rKSCF.cfFeatLst;
          for (i=0;i<ksStat->ir.rKSCF.c_cfFeatLst;i++,feature++)
            smi_trace_cf_feature (feature);
        }
      }
      break;
    case KSD_CMD_CL:
      /*
       * calling line
       */
      TRACE_EVENT ("Calling Line Result");
      smi_trace_ss_opcode (ksStat->ir.rKSCL.opCd);
      smi_trace_ss_codes (ksStat->ir.rKSCL.ssCd);      
      smi_trace_ss_error (ksStat->ir.rKSCL.ssErr);      
      if (ksStat->ir.rKSCL.ssErr EQ KSD_NO_ERROR)
      {
        smi_trace_ss_status (ksStat->ir.rKSCL.ssSt);
        smi_trace_clir_opt  (ksStat->ir.rKSCL.clirOpt);
        smi_trace_ovr_ctg   (ksStat->ir.rKSCL.ovrdCtg);
      }
      break;
    case KSD_CMD_CW:
      /*
       * call waiting
       */
      TRACE_EVENT ("Call Waiting Result");
      smi_trace_ss_opcode (ksStat->ir.rKSCW.opCd);
      smi_trace_ss_codes (ksStat->ir.rKSCW.ssCd);      
      smi_trace_ss_error (ksStat->ir.rKSCW.ssErr);      
      if (ksStat->ir.rKSCW.ssErr EQ KSD_NO_ERROR)
      {
        smi_trace_ss_status (ksStat->ir.rKSCW.ssSt);
        if (ksStat->ir.rKSCW.c_cwBSGLst EQ 0)
        {
          TRACE_EVENT ("Call Waiting Basic Service Group List not available");
        }
        else
        {
          UBYTE i;
          T_Cx_BSG * feature;

          TRACE_EVENT ("Call Waiting Basic Service Group List available");
          feature = ksStat->ir.rKSCW.cwBSGLst;
          for (i=0;i<ksStat->ir.rKSCW.c_cwBSGLst;i++,feature++)
            smi_trace_cw_feature (feature);
        }
      }
      break;
    case KSD_CMD_PWD:
      /*
       * password
       */
      TRACE_EVENT ("Password Result");
      smi_trace_ss_opcode (ksStat->ir.rKSPW.opCd);
      smi_trace_ss_codes (ksStat->ir.rKSPW.ssCd);      
      smi_trace_ss_error (ksStat->ir.rKSPW.ssErr);      
      sprintf (buf, "New PW   : %s", ksStat->ir.rKSPW.newPwd);
      TRACE_EVENT (buf);
      break;
    case KSD_CMD_UBLK:
      /*
       * unblock PIN
       */
      TRACE_EVENT ("Unblock PIN Result");
      break;
    case KSD_CMD_USSD:
      /*
       * unstructured SS result
       */
#if defined (WIN32)
      /*
       * Problems with target compilation, use lint to check
       */
      if (strlen (ksStat->ir.rKSUS.ussd) > 30)
        ksStat->ir.rKSUS.ussd[30] = 0;
      sprintf (buf, "%s", ksStat->ir.rKSUS.ussd);
      TRACE_EVENT (buf);
#endif
      break;
    case KSD_CMD_IMEI:
      /*
       * get IMEI
       */
      TRACE_EVENT ("Get IMEI Result");
      sprintf(buf, "TAC  [0] : %2x", ksStat->ir.rKSIMEI.tac1);
      TRACE_EVENT (buf);
      sprintf(buf, "TAC  [1] : %2x", ksStat->ir.rKSIMEI.tac2);
      TRACE_EVENT (buf);
      sprintf(buf, "TAC  [2] : %2x", ksStat->ir.rKSIMEI.tac3);
      TRACE_EVENT (buf);
      sprintf(buf, "FAC      : %2x", ksStat->ir.rKSIMEI.fac);
      TRACE_EVENT (buf);
      sprintf(buf, "SNR  [1] : %2x", ksStat->ir.rKSIMEI.snr1);
      TRACE_EVENT (buf);
      sprintf(buf, "SNR  [2] : %2x", ksStat->ir.rKSIMEI.snr2);
      TRACE_EVENT (buf);
      sprintf(buf, "SNR  [3] : %2x", ksStat->ir.rKSIMEI.snr3);
      TRACE_EVENT (buf);
      sprintf(buf, "SVN      : %2x", ksStat->ir.rKSIMEI.svn);
      TRACE_EVENT (buf);
      break;
  }
  TRACE_EVENT ("########################################");
#endif /* FAC_TRACE */
}


/*
+--------------------------------------------------------------------+
| PROJECT : GSM-F&D (8411)              MODULE  : ACI_RET            |
| STATE   : code                        ROUTINE : rAT_PlusCNUM       |
+--------------------------------------------------------------------+

  PURPOSE : handles AT_PlusCNUM call back

*/

GLOBAL void rAT_PlusCNUM ( T_ACI_CNUM_MSISDN *msisdn,
                           UBYTE             num )
{
  TRACE_FUNCTION("rAT_PlusCNUM()");
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-F&D (8411)              MODULE  : ACI_RET            |
| STATE   : code                        ROUTINE : rAT_PlusCCCM       |
+--------------------------------------------------------------------+

  PURPOSE : handles AT_PlusCCCM call back

*/

GLOBAL void rAT_PlusCCCM  ( LONG * ccm)
{
  TRACE_FUNCTION("rCI_PlusCCCM()");
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-F&D (8411)              MODULE  : ACI_RET            |
| STATE   : code                        ROUTINE : rAT_PercentCTV     |
+--------------------------------------------------------------------+

  PURPOSE : handles AT_PercentCTV call back

*/

GLOBAL void rAT_PercentCTV   ( void )
{
/*  TRACE_FUNCTION("rAT_PercentCTV()"); */
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-F&D (8411)              MODULE  : ACI_RET            |
| STATE   : code                        ROUTINE : rAT_PlusCPOL       |
+--------------------------------------------------------------------+

  PURPOSE : handles AT_PlusCPOL call back

*/

EXTERN void rAT_PlusCPOL  ( SHORT              startIdx,
                            SHORT              lastIdx,
                            T_ACI_CPOL_OPDESC *operLst,
                            SHORT              usdNtry )
{
  TRACE_FUNCTION("rAT_PercentCPOL()");
}
/*
+--------------------------------------------------------------------+
| PROJECT : GSM-F&D (8411)              MODULE  : ACI_RET            |
| STATE   : code                        ROUTINE : rAT_PlusCLAN       |
+--------------------------------------------------------------------+

  PURPOSE : handles AT_PlusCLAN call back

*/

EXTERN void rAT_PlusCLAN  (T_ACI_LAN_SUP  *CLang )
{
  TRACE_FUNCTION("rAT_PlusCLAN()");
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-F&D (8411)              MODULE  : ACI_RET            |
| STATE   : code                        ROUTINE : rAT_PlusCLAE       |
+--------------------------------------------------------------------+

  PURPOSE : handles AT_PlusCLAE call back

*/

EXTERN void rAT_PlusCLAE  (T_ACI_LAN_SUP  *CLang )
{
  TRACE_FUNCTION("rAT_PlusCLAE()");
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-F&D (8411)              MODULE  : ACI_RET            |
| STATE   : code                        ROUTINE : rAT_PlusCRSM       |
+--------------------------------------------------------------------+

  PURPOSE : handles AT_PlusCRSM call back

*/

GLOBAL void rAT_PlusCRSM  ( SHORT           sw1, 
                            SHORT           sw2, 
                            SHORT           rspLen, 
                            UBYTE          *rsp    )
{
  TRACE_FUNCTION("rAT_PlusCRSM()");
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_CAL                  |
| STATE   : code                  ROUTINE : rAT_PercentSIMINS        |
+--------------------------------------------------------------------+

  PURPOSE : This function is called to indicate that the SIM card was
            inserted.
*/
GLOBAL void rAT_PercentSIMINS (T_ACI_CME_ERR err)
{
  TRACE_FUNCTION ("rAT_PercentSIMINS ()");
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_CAL                  |
| STATE   : code                  ROUTINE : rAT_PercentCOLR          |
+--------------------------------------------------------------------+

  PURPOSE : This function is called to indicate that the status of COLR.
*/
GLOBAL void rAT_PercentCOLR  ( T_ACI_COLR_STAT stat)
{
  TRACE_FUNCTION ("rAT_PercentCOLR ()");
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-F&D (8411)              MODULE  : SMI_DMY            |
| STATE   : code                        ROUTINE : rAT_phb_status     |
+--------------------------------------------------------------------+

  PURPOSE : handles AT_phb_status call back

*/

GLOBAL void rAT_phb_status ( T_ACI_PB_STAT status )
{
  TRACE_FUNCTION("rAT_phb_status()");
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-F&D (8411)              MODULE  : SMI_DMY            |
| STATE   : code                        ROUTINE : rAT_PercentCCBS    |
+--------------------------------------------------------------------+

  PURPOSE : handles rAT_PercentCCBS call back

*/

GLOBAL void rAT_PercentCCBS  ( T_ACI_CCBS_IND  ind,
                               T_ACI_CCBS_STAT status,
                               T_ACI_CCBS_SET *setting )
{
  TRACE_FUNCTION("rAT_PercentCCBS()");
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-F&D (8411)              MODULE  : SMI_DMY            |
| STATE   : code                        ROUTINE : rAT_PlusCCWV       |
+--------------------------------------------------------------------+

  PURPOSE : handles rAT_PlusCCWV call back

*/

GLOBAL void rAT_PlusCCWV ( T_ACI_CCWV_CHRG charge )
{
  TRACE_FUNCTION("rAT_PlusCCWV()");
}


/*
+--------------------------------------------------------------------+
| PROJECT : GSM-F&D (8411)              MODULE  : SMI_DMY            |
| STATE   : code                        ROUTINE : rAT_PercentCNAP    |
+--------------------------------------------------------------------+

  PURPOSE : handles rAT_PercentCNAP call back

*/

GLOBAL void rAT_PercentCNAP   ( T_callingName *NameId, T_ACI_CNAP_STATUS status )
{
  TRACE_FUNCTION("rAT_PercentCNAP()");
}


/*
+--------------------------------------------------------------------+
| PROJECT : GSM-F&D (8411)              MODULE  : SMI_DMY            |
| STATE   : code                        ROUTINE : rAT_SignalSMS      |
+--------------------------------------------------------------------+

  PURPOSE : handles rAT_SignalSMS call back

*/
GLOBAL void rAT_SignalSMS ( UBYTE state )
{
  TRACE_FUNCTION("rAT_SignalSMS()");
}


/*
+--------------------------------------------------------------------+
| PROJECT : GSM-F&D (8411)              MODULE  : SMI_DMY            |
| STATE   : code                        ROUTINE : rAT_PercentCPRI    |
+--------------------------------------------------------------------+

  PURPOSE : handles rAT_PercentCPRI call back

*/

EXTERN void rAT_PercentCPRI   ( UBYTE gsm_ciph, UBYTE gprs_ciph )
{
  TRACE_FUNCTION("rAT_PercentCPRI()");
}


/*
+--------------------------------------------------------------------+
| PROJECT : ACI/MMI             MODULE  : ACI_RET            |
| STATE   : code                        ROUTINE : rAT_PercentCSQ       |
+--------------------------------------------------------------------+

  PURPOSE : handles rAT_PercentCSQ call back

  Shen,Chao

*/

EXTERN void rAT_PercentCSQ(UBYTE rssi, UBYTE ber, UBYTE actlevel)
{
  TRACE_FUNCTION("rAT_PercentCSQ()");
}

/*
+--------------------------------------------------------------------+
| PROJECT : ACI/MMI             MODULE  : ACI_RET                    |
| STATE   : code                ROUTINE : rAT_PercentALS             |
+--------------------------------------------------------------------+

  PURPOSE : handles rAT_PercentALS call back
*/

GLOBAL void rAT_PercentALS(T_ACI_ALS_MOD ALSmode)
{
  TRACE_FUNCTION("rAT_PercentALS()");
}