/* 
+----------------------------------------------------------------------------- 
|  Project :  MMI-Framework (8417)
|  Modul   :  SMI_LCD
+----------------------------------------------------------------------------- 
|  Copyright 2002 Texas Instruments Berlin, AG 
|                 All rights reserved. 
| 
|                 This file is confidential and a trade secret of Texas 
|                 Instruments Berlin, AG 
|                 The receipt of or possession of this file does not convey 
|                 any rights to reproduce or disclose its contents or to 
|                 manufacture, use, or sell anything it may describe, in 
|                 whole, or in part, without the specific written consent of 
|                 Texas Instruments Berlin, AG. 
+----------------------------------------------------------------------------- 
|  Purpose :  screen driver interface (TI1 VERSION)
+----------------------------------------------------------------------------- 
*/ 

#ifndef SMI_LCD_C
#define SMI_LCD_C
#endif

#define ENTITY_SMI

#include <stdio.h>
#include <string.h>

#include "smi_lcd.h"

#define LCD_COL 100                     /* display size             */
#define LCD_COL_B ((LCD_COL+7)/8)       /* row size in byte         */
#define LCD_ROW 32

#if defined (_TMS470)
extern void LCD_SetPower (void);        /* TI driver functions      */
extern void LCD_Reset (void);
extern void LCD_Enable (int);
extern void LCD_Clear (void);
extern void LCD_WriteString (int y, int x, char *s);
extern void LCD_PixBlt (int x, int y, int w, int h, char *bmp);
extern void LCD_DrawIcon (int x, int y, char id);
extern void LCD_TranslateBitmap (int w, int h, char *bo, char *bt);
#else
#define LCD_SetPower();
#define LCD_Reset();
#define LCD_Enable(e);
#define LCD_Clear();
#define LCD_WriteString(y,x,s);
#define LCD_PixBlt(x,y,w,h,b);
#define LCD_DrawIcon(x,y,i);
#define LCD_TranslateBitmap(w,h,o,t);
#endif

#define scrHlen 18
static const char *scrHeader = "CONFIG MFWMOB SCR ";

static int scrX, scrY;                  /* display size             */
static int mobEnabled = 0;              /* mobile simulation ?      */
static int internal = 0;                /* internal use only        */
static char scratch [512];              /* command buffer           */
static U8 display [LCD_ROW][LCD_COL_B]; /* display buffer           */
static int xIndex;                      /* data transfer index      */

static void scrPoint (int px, int py, int col);

static const char char_font[256][6] =
{
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /*   0 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /*   1 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /*   2 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /*   3 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /*   4 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /*   5 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /*   6 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /*   7 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /*   8 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /*   9 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /*  10 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /*  11 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /*  12 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /*  13 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /*  14 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /*  15 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /*  16 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /*  17 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /*  18 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /*  19 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /*  20 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /*  21 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /*  22 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /*  23 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /*  24 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /*  25 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /*  26 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /*  27 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /*  28 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /*  29 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /*  30 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /*  31 */
    { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, } , /*  32 */
    { 0x00, 0x00, 0x4F, 0x00, 0x00, 0x00, } , /*  33 */
    { 0x00, 0x07, 0x00, 0x07, 0x00, 0x00, } , /*  34 */
    { 0x14, 0x7F, 0x14, 0x7F, 0x14, 0x00, } , /*  35 */
    { 0x24, 0x2A, 0x7F, 0x2A, 0x12, 0x00, } , /*  36 */
    { 0x23, 0x13, 0x08, 0x64, 0x62, 0x00, } , /*  37 */
    { 0x36, 0x49, 0x55, 0x22, 0x50, 0x00, } , /*  38 */
    { 0x00, 0x05, 0x03, 0x00, 0x00, 0x00, } , /*  39 */
    { 0x00, 0x1C, 0x22, 0x41, 0x00, 0x00, } , /*  40 */
    { 0x00, 0x41, 0x22, 0x1C, 0x00, 0x00, } , /*  41 */
    { 0x14, 0x08, 0x3E, 0x08, 0x14, 0x00, } , /*  42 */
    { 0x08, 0x08, 0x3E, 0x08, 0x08, 0x00, } , /*  43 */
    { 0x00, 0x50, 0x30, 0x00, 0x00, 0x00, } , /*  44 */
    { 0x08, 0x08, 0x08, 0x08, 0x08, 0x00, } , /*  45 */
    { 0x00, 0x60, 0x60, 0x00, 0x00, 0x00, } , /*  46 */
    { 0x20, 0x10, 0x08, 0x04, 0x02, 0x00, } , /*  47 */
    { 0x3E, 0x51, 0x49, 0x45, 0x3E, 0x00, } , /*  48 */
    { 0x00, 0x42, 0x7F, 0x40, 0x00, 0x00, } , /*  49 */
    { 0x42, 0x61, 0x51, 0x49, 0x46, 0x00, } , /*  50 */
    { 0x21, 0x41, 0x45, 0x4B, 0x31, 0x00, } , /*  51 */
    { 0x18, 0x14, 0x12, 0x7F, 0x10, 0x00, } , /*  52 */
    { 0x27, 0x45, 0x45, 0x45, 0x39, 0x00, } , /*  53 */
    { 0x3C, 0x4A, 0x49, 0x49, 0x30, 0x00, } , /*  54 */
    { 0x01, 0x71, 0x09, 0x05, 0x03, 0x00, } , /*  55 */
    { 0x36, 0x49, 0x49, 0x49, 0x36, 0x00, } , /*  56 */
    { 0x06, 0x49, 0x49, 0x29, 0x1E, 0x00, } , /*  57 */
    { 0x00, 0x36, 0x36, 0x00, 0x00, 0x00, } , /*  58 */
    { 0x00, 0x56, 0x36, 0x00, 0x00, 0x00, } , /*  59 */
    { 0x08, 0x14, 0x22, 0x41, 0x00, 0x00, } , /*  60 */
    { 0x14, 0x14, 0x14, 0x14, 0x14, 0x00, } , /*  61 */
    { 0x00, 0x41, 0x22, 0x14, 0x08, 0x00, } , /*  62 */
    { 0x02, 0x01, 0x51, 0x09, 0x06, 0x00, } , /*  63 */
    { 0x3E, 0x41, 0x5D, 0x55, 0x1E, 0x00, } , /*  64 */
    { 0x7E, 0x11, 0x11, 0x11, 0x7E, 0x00, } , /*  65 */
    { 0x7F, 0x49, 0x49, 0x49, 0x36, 0x00, } , /*  66 */
    { 0x3E, 0x41, 0x41, 0x41, 0x22, 0x00, } , /*  67 */
    { 0x7F, 0x41, 0x41, 0x22, 0x1C, 0x00, } , /*  68 */
    { 0x7F, 0x49, 0x49, 0x49, 0x41, 0x00, } , /*  69 */
    { 0x7F, 0x09, 0x09, 0x09, 0x01, 0x00, } , /*  70 */
    { 0x3E, 0x41, 0x49, 0x49, 0x3A, 0x00, } , /*  71 */
    { 0x7F, 0x08, 0x08, 0x08, 0x7F, 0x00, } , /*  72 */
    { 0x00, 0x41, 0x7F, 0x41, 0x00, 0x00, } , /*  73 */
    { 0x20, 0x40, 0x41, 0x3F, 0x01, 0x00, } , /*  74 */
    { 0x7F, 0x08, 0x14, 0x22, 0x41, 0x00, } , /*  75 */
    { 0x7F, 0x40, 0x40, 0x40, 0x40, 0x00, } , /*  76 */
    { 0x7F, 0x02, 0x04, 0x02, 0x7F, 0x00, } , /*  77 */
    { 0x7F, 0x04, 0x08, 0x10, 0x7F, 0x00, } , /*  78 */
    { 0x3E, 0x41, 0x41, 0x41, 0x3E, 0x00, } , /*  79 */
    { 0x7F, 0x09, 0x09, 0x09, 0x06, 0x00, } , /*  80 */
    { 0x3E, 0x41, 0x51, 0x21, 0x5E, 0x00, } , /*  81 */
    { 0x7F, 0x09, 0x19, 0x29, 0x46, 0x00, } , /*  82 */
    { 0x26, 0x49, 0x49, 0x49, 0x32, 0x00, } , /*  83 */
    { 0x01, 0x01, 0x7F, 0x01, 0x01, 0x00, } , /*  84 */
    { 0x3F, 0x40, 0x40, 0x40, 0x3F, 0x00, } , /*  85 */
    { 0x1F, 0x20, 0x40, 0x20, 0x1F, 0x00, } , /*  86 */
    { 0x3F, 0x40, 0x38, 0x40, 0x3F, 0x00, } , /*  87 */
    { 0x63, 0x14, 0x08, 0x14, 0x63, 0x00, } , /*  88 */
    { 0x07, 0x08, 0x70, 0x08, 0x07, 0x00, } , /*  89 */
    { 0x61, 0x51, 0x49, 0x45, 0x43, 0x00, } , /*  90 */
    { 0x00, 0x7F, 0x41, 0x41, 0x00, 0x00, } , /*  91 */
    { 0x02, 0x04, 0x08, 0x10, 0x20, 0x00, } , /*  92 */
    { 0x00, 0x41, 0x41, 0x7F, 0x00, 0x00, } , /*  93 */
    { 0x04, 0x02, 0x01, 0x02, 0x04, 0x00, } , /*  94 */
    { 0x40, 0x40, 0x40, 0x40, 0x40, 0x00, } , /*  95 */
    { 0x00, 0x00, 0x03, 0x05, 0x00, 0x00, } , /*  96 */
    { 0x20, 0x54, 0x54, 0x54, 0x78, 0x00, } , /*  97 */
    { 0x7F, 0x44, 0x44, 0x44, 0x38, 0x00, } , /*  98 */
    { 0x38, 0x44, 0x44, 0x44, 0x44, 0x00, } , /*  99 */
    { 0x38, 0x44, 0x44, 0x44, 0x7F, 0x00, } , /* 100 */
    { 0x38, 0x54, 0x54, 0x54, 0x18, 0x00, } , /* 101 */
    { 0x08, 0x7E, 0x09, 0x09, 0x00, 0x00, } , /* 102 */
    { 0x08, 0x54, 0x54, 0x54, 0x3C, 0x00, } , /* 103 */
    { 0x7F, 0x08, 0x04, 0x04, 0x78, 0x00, } , /* 104 */
    { 0x00, 0x44, 0x7D, 0x40, 0x00, 0x00, } , /* 105 */
    { 0x20, 0x40, 0x44, 0x3D, 0x00, 0x00, } , /* 106 */
    { 0x7F, 0x10, 0x28, 0x44, 0x00, 0x00, } , /* 107 */
    { 0x00, 0x41, 0x7F, 0x40, 0x00, 0x00, } , /* 108 */
    { 0x7C, 0x04, 0x78, 0x04, 0x78, 0x00, } , /* 109 */
    { 0x7C, 0x08, 0x04, 0x04, 0x78, 0x00, } , /* 110 */
    { 0x38, 0x44, 0x44, 0x44, 0x38, 0x00, } , /* 111 */
    { 0x7C, 0x14, 0x14, 0x14, 0x08, 0x00, } , /* 112 */
    { 0x08, 0x14, 0x14, 0x14, 0x7C, 0x00, } , /* 113 */
    { 0x7C, 0x08, 0x04, 0x04, 0x00, 0x00, } , /* 114 */
    { 0x48, 0x54, 0x54, 0x54, 0x24, 0x00, } , /* 115 */
    { 0x04, 0x3F, 0x44, 0x44, 0x00, 0x00, } , /* 116 */
    { 0x3C, 0x40, 0x40, 0x20, 0x7C, 0x00, } , /* 117 */
    { 0x1C, 0x20, 0x40, 0x20, 0x1C, 0x00, } , /* 118 */
    { 0x3C, 0x40, 0x30, 0x40, 0x3C, 0x00, } , /* 119 */
    { 0x44, 0x28, 0x10, 0x28, 0x44, 0x00, } , /* 120 */
    { 0x0C, 0x50, 0x50, 0x50, 0x3C, 0x00, } , /* 121 */
    { 0x44, 0x64, 0x54, 0x4C, 0x44, 0x00, } , /* 122 */
    { 0x08, 0x36, 0x41, 0x41, 0x00, 0x00, } , /* 123 */
    { 0x00, 0x00, 0x77, 0x00, 0x00, 0x00, } , /* 124 */
    { 0x00, 0x41, 0x41, 0x36, 0x08, 0x00, } , /* 125 */
    { 0x02, 0x01, 0x02, 0x04, 0x02, 0x00, } , /* 126 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 127 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 128 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 129 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 130 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 131 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 132 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 133 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 134 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 135 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 136 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 137 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 138 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 139 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 140 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 141 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 142 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 143 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 144 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 145 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 146 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 147 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 148 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 149 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 150 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 151 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 152 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 153 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 154 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 155 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 156 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 157 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 158 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 159 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 160 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 161 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 162 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 163 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 164 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 165 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 166 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 167 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 168 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 169 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 170 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 171 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 172 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 173 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 174 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 175 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 176 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 177 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 178 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 179 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 180 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 181 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 182 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 183 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 184 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 185 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 186 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 187 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 188 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 189 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 190 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 191 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 192 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 193 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 194 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 195 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 196 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 197 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 198 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 199 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 200 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 201 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 202 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 203 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 204 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 205 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 206 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 207 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 208 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 209 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 210 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 211 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 212 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 213 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 214 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 215 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 216 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 217 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 218 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 219 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 220 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 221 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 222 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 223 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 224 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 225 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 226 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 227 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 228 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 229 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 230 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 231 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 232 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 233 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 234 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 235 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 236 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 237 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 238 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 239 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 240 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 241 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 242 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 243 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 244 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 245 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 246 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 247 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 248 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 249 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 250 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 251 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 252 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 253 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 254 */
    { 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x00, } , /* 255 */
};

#if 0
static char lng1 [] [32] =
{
    "asdjhsdjkh",
    "jhjhjhjkh"
};

static char *cl = lng1[0];
#endif

/*
+--------------------------------------------------------------------+
| PROJECT : MMI-Framework (8417)        MODULE  : ti1_scr            |
| STATE   : code                        ROUTINE : scrMobUpdate       |
+--------------------------------------------------------------------+

  PURPOSE : Shadow display to mobile simulation

*/

void scrMobUpdate (void)
{
    if (mobEnabled)
    {
        sprintf(scratch,"%s%s %d",scrHeader,"disp",LCD_COL_B);
        xIndex = 0;
/*
        mfwExtOut(scratch);
*/
    }
}


/*
+--------------------------------------------------------------------+
| PROJECT : MMI-Framework (8417)        MODULE  : ti1_scr            |
| STATE   : code                        ROUTINE : scrLCDUpdate       |
+--------------------------------------------------------------------+

  PURPOSE : Shadow display to LCD

*/

void scrLCDUpdate (void)
{
    int i;

    /* work around limitations of TI driver !                       */
    for (i = 0; i < LCD_ROW; i++)
    {
        display[i][LCD_COL_B-1] &= 0xF0;
        display[i][LCD_COL_B-1] |= (display[i][6] >> 2) & 0x0F;
    }
    LCD_PixBlt(0,0,LCD_COL_B*8,8,(char*)display);
    LCD_PixBlt(0,8,LCD_COL_B*8,8,(char*)(&(display[8][0])));
    LCD_PixBlt(0,16,LCD_COL_B*8,8,(char*)(&(display[16][0])));
    LCD_PixBlt(0,24,LCD_COL_B*8,8,(char*)(&(display[24][0])));
}


/*
+--------------------------------------------------------------------+
| PROJECT : MMI-Framework (8417)        MODULE  : ti1_scr            |
| STATE   : code                        ROUTINE : scrMobCommand      |
+--------------------------------------------------------------------+

  PURPOSE : Command string from mobile simulation

*/

void scrMobCommand (char *s)
{
    char *p;
    unsigned char *q;
    int i;

    if (strstr(s,"ena") == s)
    {
        mobEnabled = 1;
        return;
    }
    if (strstr(s,"dis") == s)
    {
        mobEnabled = 0;
        return;
    }
    if (strstr(s,"ok") == s)
    {
        if (xIndex < 0)
            return;
        strcpy(scratch,scrHeader);
        if (xIndex >= sizeof(display))
        {
            strcat(scratch,"done");
/*
            mfwExtOut(scratch);
*/
            return;
        }
        strcat(scratch,"data ");
        p = scratch + strlen(scratch);
        sprintf(p,"%04d %04d ",xIndex,LCD_COL_B*LCD_ROW/2);
        p = scratch + strlen(scratch);
        q = (U8 *) display;
        for (i = 0; i < LCD_COL_B * LCD_ROW / 2; i++)
        {
            sprintf(p,"%02x",q[xIndex++]);
            p += 2;
        }
        *p = '\0';
/*
        mfwExtOut(scratch);
*/
        return;
    }
}


/*
+--------------------------------------------------------------------+
| PROJECT : MMI-Framework (8417)        MODULE  : ti1_scr            |
| STATE   : code                        ROUTINE : scrInit            |
+--------------------------------------------------------------------+

  PURPOSE : initialize screen driver

*/

void scrInit (void)
{
    LCD_SetPower ();
    LCD_Reset ();
    LCD_Enable(1);
    LCD_Clear ();

    memset(display,0,sizeof(display));
    mobEnabled = 0;
    internal = 0;
    scrX = LCD_COL;
    scrY = LCD_ROW;
    xIndex = -1;
}


/*
+--------------------------------------------------------------------+
| PROJECT : MMI-Framework (8417)        MODULE  : ti1_scr            |
| STATE   : code                        ROUTINE : scrExit            |
+--------------------------------------------------------------------+

  PURPOSE : finalize screen driver

*/

void scrExit (void)
{
    LCD_Enable(0);
    mobEnabled = 0;
    xIndex = -1;
}


/*
+--------------------------------------------------------------------+
| PROJECT : MMI-Framework (8417)        MODULE  : ti1_scr            |
| STATE   : code                        ROUTINE : scrSize            |
+--------------------------------------------------------------------+

  PURPOSE : retrieve display size

*/

void scrSize (int *x, int *y)
{
    *x = scrX;
    *y = scrY;
}


/*
+--------------------------------------------------------------------+
| PROJECT : MMI-Framework (8417)        MODULE  : ti1_scr            |
| STATE   : code                        ROUTINE : scrClear           |
+--------------------------------------------------------------------+

  PURPOSE : clear display

*/

void scrClear (void)
{
    memset(display,0,sizeof(display));  /* clear shadow display     */
    LCD_Clear();                        /* TI driver function       */
    scrMobUpdate();                     /* update mobile simulation */
}


/*
+--------------------------------------------------------------------+
| PROJECT : MMI-Framework (8417)        MODULE  : ti1_scr            |
| STATE   : code                        ROUTINE : scrClearRect       |
+--------------------------------------------------------------------+

  PURPOSE : clear display area

*/

void scrClearRect (int px, int py, int sx, int sy)
{
    int x1, x2, i, j;
    U8 m1, m2;

    if (px < 0 || py < 0 || px >= scrX || py >= scrY
                                       || sx < 0 || sy < 0)
        return;
    if (px + sx > scrX)
        sx = scrX - px;
    if (py + sy > scrY)
        sy = scrY - py;

    x1 = px / 8;                        /* first x offset           */
    x2 = (px + sx) / 8;                 /* last x offset            */
    m1 = m2 = 0xff;
    m1 = (U8) ~(m1 >> (px & 7));        /* left mask                */
    m2 = (U8) (m2 >> ((px + sx) & 7));  /* right mask               */
    if (x1 == x2)
        m1 = m2 = m1 | m2;              /* combined masks           */
    for (j = py; j < py + sy; j++)
    {
        display[j][x1] &= m1;
        for (i = x1 + 1; i < x2; i++)
            display[j][i] = '\0';
        display[j][x2] &= m2;
    }

    scrLCDUpdate();                     /* update LCD               */
    scrMobUpdate();                     /* update mobile simulation */
}


/*
+--------------------------------------------------------------------+
| PROJECT : MMI-Framework (8417)        MODULE  : ti1_scr            |
| STATE   : code                        ROUTINE : scrText            |
+--------------------------------------------------------------------+

  PURPOSE : draw text

*/

void scrText (int x, int y, char *txt)
{
    int i, j;

    if (x < 0 || y < 0 || y + 8 > scrY)
        return;

    while (*txt && x + 6 <= scrX)
    {
        for (j = 0; j < 8; j++)
            for (i = 0; i < 6; i++)
                scrPoint(x+i,y+j,char_font[*txt][i]&(1<<j));
        x += 6;
        txt++;
    }
                                        /* LCD_WriteString(y,x,txt);*/
    scrLCDUpdate();                     /* update LCD               */
    scrMobUpdate();                     /* update mobile simulation */
}


/*
+--------------------------------------------------------------------+
| PROJECT : MMI-Framework (8417)        MODULE  : ti1_scr            |
| STATE   : code                        ROUTINE : scrLine            |
+--------------------------------------------------------------------+

  PURPOSE : draw line

*/

void scrLine (int x1, int y1, int x2, int y2)
{
    int x, y, dx, dy, c1, c2, err, swap = 0;

    if (x1 < 0 || x1 >= scrX || x2 < 0 || x2 >= scrX ||
        y1 < 0 || y1 >= scrY || y2 < 0 || y2 >= scrY)
        return;

    dx = x2 - x1; dy = y2 - y1;
    if (((dx < 0) ? -dx : dx) < ((dy < 0) ? -dy : dy))
    {
        swap = 1;                       /* take the long way        */
        x = x1; x1 = y1; y1 = x;
        x = x2; x2 = y2; y2 = x;
    }
    if (x1 > x2)
    {
        x = x1; x1 = x2; x2 = x;        /* always move to the right */
        y = y1; y1 = y2; y2 = y;
    }

    dx = x2 - x1; dy = y2 - y1;
    c1 = dy * 2; dy = 1;
    if (c1 < 0)
    {
        c1 = -c1;
        dy = -1;
    }
    err = c1 - dx; c2 = err - dx;
    x = x1; y = y1;
    while (x <= x2)
    {
        scrPoint(swap?y:x,swap?x:y,1);
        x++;
        if (err < 0)
            err += c1;
        else
        {
            y += dy;
            err += c2;
        }
    }

    if (internal)
        return;

    scrLCDUpdate();                     /* update LCD               */
    scrMobUpdate();                     /* update mobile simulation */
}


/*
+--------------------------------------------------------------------+
| PROJECT : MMI-Framework (8417)        MODULE  : ti1_scr            |
| STATE   : code                        ROUTINE : scrRect            |
+--------------------------------------------------------------------+

  PURPOSE : draw rectangle

*/

void scrRect (int px, int py, int sx, int sy)
{
    internal = 1;
    scrLine(px,py,px+sx,py);
    scrLine(px,py+sy,px+sx,py+sy);
    scrLine(px,py,px,py+sy);
    internal = 0;
    scrLine(px+sx,py,px+sx,py+sy);
}


/*
+--------------------------------------------------------------------+
| PROJECT : MMI-Framework (8417)        MODULE  : ti1_scr            |
| STATE   : code                        ROUTINE : scrFntSelect       |
+--------------------------------------------------------------------+

  PURPOSE : select the current font

*/

int scrFntSelect (int font)
{
    return 1;                           /* always OK                */
}


/*
+--------------------------------------------------------------------+
| PROJECT : MMI-Framework (8417)        MODULE  : ti1_scr            |
| STATE   : code                        ROUTINE : scrFntHeight       |
+--------------------------------------------------------------------+

  PURPOSE : get height of current font

*/

int scrFntHeight (void)
{
    return 8;
}


/*
+--------------------------------------------------------------------+
| PROJECT : MMI-Framework (8417)        MODULE  : ti1_scr            |
| STATE   : code                        ROUTINE : scrFntGetFit       |
+--------------------------------------------------------------------+

  PURPOSE : number of string chars that fit into the line

*/

int scrFntGetFit (char *tp, int lineWidth)
{
    return (lineWidth / 6);
}


/*
+--------------------------------------------------------------------+
| PROJECT : MMI-Framework (8417)        MODULE  : ti1_scr            |
| STATE   : code                        ROUTINE : scrFntGetLen       |
+--------------------------------------------------------------------+

  PURPOSE : number of pixel used by first n string chars

*/

int scrFntGetLen(char *tp, int nChars)
{
    return (nChars * 6);
}


/*
+--------------------------------------------------------------------+
| PROJECT : MMI-Framework (8417)        MODULE  : ti1_scr            |
| STATE   : code                        ROUTINE : scrBmpDraw         |
+--------------------------------------------------------------------+

  PURPOSE : draw bitmap

*/

int scrBmpDraw (int px, int py, int sx, int sy,
                    int ix, char *bm, int mode)
{
    int i, j;
    unsigned char m;

    if (px < 0 || px >= scrX || py < 0 || py >= scrY
                                       || sx < 0 || sy < 0)
        return 0;
    if (px + sx > scrX)
        sx = scrX - px;
    if (py + sy > scrY)
        sy = scrY - py;

    m = 0x80;
    for (j = 0; j < sy; j++)
    {
        for (i = 0; i < sx; i++)
        {
            scrPoint(px+i,py+j,*bm & m);
            m >>= 1;
            if (!m)
            {
                m = 0x80;
                bm++;
            }
        }
        if (m != 0x80)
        {
            m = 0x80;
            bm++;
        }
    }

    scrLCDUpdate();                     /* update LCD               */
    scrMobUpdate();                     /* update mobile simulation */

    return 0;
}


/*
+--------------------------------------------------------------------+
| PROJECT : MMI-Framework (8417)        MODULE  : ti1_scr            |
| STATE   : code                        ROUTINE : scrCurType         |
+--------------------------------------------------------------------+

  PURPOSE : set cursor type (0 = no cursor)

*/

void scrCurType (int type)
{
}


/*
+--------------------------------------------------------------------+
| PROJECT : MMI-Framework (8417)        MODULE  : ti1_scr            |
| STATE   : code                        ROUTINE : scrCursor          |
+--------------------------------------------------------------------+

  PURPOSE : set cursor at specified position

*/

void scrCursor (int x, int y)
{
    scrLine(x,y+7,x+5,y+7);
}


/*
+--------------------------------------------------------------------+
| PROJECT : MMI-Framework (8417)        MODULE  : ti1_scr            |
| STATE   : code                        ROUTINE : scrPoint           |
+--------------------------------------------------------------------+

  PURPOSE : draw point

*/

static void scrPoint (int px, int py, int col)
{
    unsigned char m = 0x80;

    if (px < 0 || py < 0 || px >= scrX || py >= scrY)
        return;

    m >>= px & 7;                       /* mask                     */
    px >>= 3;                           /* x offset                 */
    if (col)
        display[py][px] |= m;
    else
        display[py][px] &= (U8) ~m;
}
