/* 
+----------------------------------------------------------------------------- 
|  Project :  GSM-F&D (8411)
|  Modul   :  SMI_LNG
+----------------------------------------------------------------------------- 
|  Copyright 2002 Texas Instruments Berlin, AG 
|                 All rights reserved. 
| 
|                 This file is confidential and a trade secret of Texas 
|                 Instruments Berlin, AG 
|                 The receipt of or possession of this file does not convey 
|                 any rights to reproduce or disclose its contents or to 
|                 manufacture, use, or sell anything it may describe, in 
|                 whole, or in part, without the specific written consent of 
|                 Texas Instruments Berlin, AG. 
+----------------------------------------------------------------------------- 
|  Purpose :  This Modul defines functions for language support.
+----------------------------------------------------------------------------- 
*/ 

#ifndef SMI_LNG_C
#define SMI_LNG_C
#endif

#define ENTITY_SMI

/*==== INCLUDES ===================================================*/
#include "typedefs.h"
#include "vsi.h"
#include "custom.h"
#include "gsm.h"
#include "prim.h"
#include "tok.h"
#include "message.h"

#include "aci_cmh.h"
#include "ksd.h"
#include "aca.h"
#include "smi.h"

#include "cus_smi.h"

/*==== CONSTANTS ==================================================*/

/*==== TYPES ======================================================*/

/*==== EXPORT =====================================================*/

/*==== VARIABLES ==================================================*/

GLOBAL const CHAR *statText[NUM_OF_LANGUAGES][NUM_OF_STAT_TXT_STRINGS] =
{
  /*
   * english
   */
  {
    "WAITING",
    "HOLD",
    "ACTIVE",
    "DIAL"
  },
  /*
   * french
   */
  {
    "WAITING", /* ->not translated */
    "HOLD",    /* ->not translated */
    "ACTIVE",  /* ->not translated */
    "DIAL"     /* ->not translated */
    },
  /*
   * german
   */
  {
    "ANKLOPFEN",
    "GEHALTEN",
    "AKTIV",
    "WHLEN"
    },
  /*
   * italien
   */
  {
    "WAITING", /* ->not translated */
    "HOLD",    /* ->not translated */
    "ACTIVE",  /* ->not translated */
    "DIAL"     /* ->not translated */
    },
  /*
   * spanish
   */
  {
    "WAITING", /* ->not translated */
    "HOLD",    /* ->not translated */
    "ACTIVE",  /* ->not translated */
    "DIAL"     /* ->not translated */
    },
};

GLOBAL const CHAR *text[NUM_OF_LANGUAGES][NUM_OF_TXT_STRINGS] =
{
  /*
   * english
   */
  {
    "DIALING",
    "ALERT",
    "CONNECT",
    "SERVICE",
    "LIMITED",
    "Language",
    "Send DTMF",
    "Key click",
    "Start",
    "Stop",
    "On",
    "Off",
    "Options",
    "English",
    "French",
    "German",
    "Italien",
    "Spanish",
    "NO SERV",
    "Select PLMN",
    "WAITING",
    "SIM REQ",
    "SIM WRNG",
    "Call table",
    "DISC REQ",
    "CNCT REQ",
    "REG REQ",
    "DREG REQ",
    "SMS List",
    "SM / CBM",
    "Empty",
    "BUSY",
    "NO ANSWR"
    },
  /*
   * french
   */
  {
    "FAIR NUMERO",
    "APPEL",
    "CONNECTION",
    "SERVICE",
    "LIMITE",
    "Langue",
    "Envoi DTMF",
    "Key click",
    "Envoi",
    "Stop",
    "Allume",
    "Eteint",
    "Options",
    "Anglais",
    "Francais",
    "Allemand",
    "Italien",
    "Espagnol",
    "NO SERV",     /* ->not translated */
    "Select PLMN", /* ->not translated */
    "WAITING",     /* ->not translated */
    "SIM REQ",     /* ->not translated */
    "SIM WRNG",    /* ->not translated */
    "Call table",  /* ->not translated */
    "DISC REQ",    /* ->not translated */
    "CNCT REQ",    /* ->not translated */
    "REG REQ",     /* ->not translated */
    "DREG REQ",    /* ->not translated */
    "SMS List",    /* ->not translated */
    "SM / CBM",    /* ->not translated */
    "Empty",       /* ->not translated */
    "BUSY",        /* ->not translated */    
    "NO ANSWR"     /* ->not translated */    
    },
  /*
   * german
   */
  {
    "WAHL",
    "EINGEHENDER RUF",
    "VERBUNDEN",
    "SERVICE",
    "NUR NOTR.",
    "Sprache",
    "Sende DTMF",
    "Tasten klick",
    "Start",
    "Stop",
    "An",
    "Aus",
    "Optionen",
    "Englisch",
    "Franzsisch",
    "Deutsch",
    "Italienisch",
    "Spanisch",
    "NO SERV",       /* ->not translated */
    "Auswahl PLMN",
    "ANKLOPFEN",
    "SIM REQ",       /* ->not translated */
    "SIM WRNG",      /* ->not translated */
    "Rufbersicht",
    "DISC REQ",      /* ->not translated */
    "CNCT REQ",      /* ->not translated */
    "REG REQ",       /* ->not translated */
    "DREG REQ",      /* ->not translated */
    "SMS Liste",
    "SM / CBM",      /* ->not translated */
    "Leer",
    "BESETZT",       /* ->not translated */    
    "NO ANSWR"    /* ->not translated */    
    },
  /*
   * italien
   */
  {
    "INVIO CH.",
    "CHIAMATA",
    "CONNESSIONE",
    "SERVICE",
    "LIMITADO",
    "Lingua",
    "Invio DTMF",
    "Toni tastiera",
    "Start",
    "Stop",
    "Alleso",
    "Spento",
    "Optioni",
    "Inglese",
    "Francese",
    "Tedesco",
    "Italiano",
    "Spagnolo",
    "NO SERV",     /* ->not translated */
    "Select PLMN", /* ->not translated */
    "WAITING",     /* ->not translated */
    "SIM REQ",     /* ->not translated */
    "SIM WRNG",    /* ->not translated */
    "Call table",  /* ->not translated */
    "DISC REQ",    /* ->not translated */
    "CNCT REQ",    /* ->not translated */
    "REG REQ",     /* ->not translated */
    "DREG REQ",    /* ->not translated */
    "SMS List",    /* ->not translated */
    "SM / CBM",    /* ->not translated */
    "Empty",       /* ->not translated */
    "BUSY",        /* ->not translated */
    "NO ANSWR"     /* ->not translated */
    },
  /*
   * spanish
   */
  {
    "MARCAR",
    "SONAR",
    "CONNEXION",
    "SERV",
    "LIMTD",
    "Idioma",
    "Enviar DTMF",
    "Tono de tecla",
    "Inicio",
    "Termino",
    "Encendido",
    "Apagado",
    "Opciones",
    "Ingles",
    "Frances",
    "Aleman",
    "Italiano",
    "Espanol",
    "NO SERV",     /* ->not translated */
    "Select PLMN", /* ->not translated */
    "WAITING",     /* ->not translated */
    "SIM REQ",     /* ->not translated */
    "SIM WRNG",    /* ->not translated */
    "Call table",  /* ->not translated */
    "DISC REQ",    /* ->not translated */
    "CNCT REQ",    /* ->not translated */
    "REG REQ",     /* ->not translated */
    "DREG REQ",    /* ->not translated */
    "SMS List",    /* ->not translated */
    "SM / CBM",    /* ->not translated */
    "Empty",       /* ->not translated */
    "BUSY",        /* ->not translated */
    "NO ANSWR"     /* ->not translated */
    },
};

/*==== FUNCTIONS ==================================================*/

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_LNG                  |
| STATE   : code                  ROUTINE : lng_init                 |
+--------------------------------------------------------------------+

  PURPOSE : Initialize the language module.

*/

GLOBAL void lng_init (void)
{
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_LNG                  |
| STATE   : code                  ROUTINE : lng_getTxt               |
+--------------------------------------------------------------------+

  PURPOSE : returns the text to the given TXT_XXX constant defined
            in cus_smi.h. The language table is selected with
            csf_getLanguage ().

*/

GLOBAL CHAR *lng_getText (USHORT txtNum)
{
  return (CHAR *) text[csf_getLanguage ()][txtNum];
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_LNG                  |
| STATE   : code                  ROUTINE : lng_getStatTxt           |
+--------------------------------------------------------------------+

  PURPOSE : returns the text to the given STAT_TXT_XXX constant
            defined in cus_smi.h. The language table is selected with
            csf_getLanguage ().

*/

GLOBAL CHAR *lng_getStatText (USHORT txtNum)
{
  return (CHAR *) statText[csf_getLanguage ()][txtNum];
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_LNG                  |
| STATE   : code                  ROUTINE : lng_getLanguage          |
+--------------------------------------------------------------------+

  PURPOSE : Set the used language to langNum.

*/

GLOBAL void lng_setLanguage (USHORT lngNum)
{
  csf_setLanguage (lngNum);
}

