/* 
+----------------------------------------------------------------------------- 
|  Project :  GSM-F&D (8411)
|  Modul   :  SMI_REG
+----------------------------------------------------------------------------- 
|  Copyright 2002 Texas Instruments Berlin, AG 
|                 All rights reserved. 
| 
|                 This file is confidential and a trade secret of Texas 
|                 Instruments Berlin, AG 
|                 The receipt of or possession of this file does not convey 
|                 any rights to reproduce or disclose its contents or to 
|                 manufacture, use, or sell anything it may describe, in 
|                 whole, or in part, without the specific written consent of 
|                 Texas Instruments Berlin, AG. 
+----------------------------------------------------------------------------- 
|  Purpose :  This Modul defines the registration/deregestration
|             handling for the component SMI of the mobile station
+----------------------------------------------------------------------------- 
*/ 

#ifndef SMI_REG_C
#define SMI_REG_C
#endif

#define ENTITY_SMI

/*==== INCLUDES ===================================================*/

#include <string.h>
#include "typedefs.h"
#include "vsi.h"
#include "custom.h"
#include "gsm.h"
#include "prim.h"
#include "tok.h"
#include "message.h"

#include "aci_cmh.h"
#include "ksd.h"
#include "aca.h"
#include "smi.h"

/*==== CONSTANTS ==================================================*/

/*==== EXPORT =====================================================*/
/*EXTERN GLOBAL void reg_CfunCnf (T_ACI_AT_CMD cmdId);*/
/*==== VARIABLES ==================================================*/

LOCAL T_ACI_AT_CMD currRegCmd   = AT_CMD_NONE;
LOCAL T_ACI_AT_CMD currAbrtCmd  = AT_CMD_NONE;

LOCAL BOOL pinCnf  = FALSE;
LOCAL BOOL pinNeed = FALSE;

LOCAL T_ACI_COPS_MOD  COPSmode;     /* holds cops mode             */
LOCAL T_ACI_COPS_FRMT COPSfrmt;     /* holds cops format           */
LOCAL CHAR*           COPSoper;     /* holds cops operator         */
LOCAL SHORT           COPSstartidx; /* start index of operator list*/
LOCAL SHORT           COPSlastidx;  /* last index of operator list */
LOCAL T_ACI_COPS_LST  COPSlst;      /* operator list               */
LOCAL T_ACA_RSN       COPSqryrsn;   /* reason for query cops param.*/

LOCAL T_ACI_CREG_STAT CREGstat;     /* registration status         */

/*==== FUNCTIONS ==================================================*/

GLOBAL T_ACI_RETURN reg_powerOff (void);

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_REG                  |
| STATE   : code                  ROUTINE : reg_initialize           |
+--------------------------------------------------------------------+

  PURPOSE : This function is used to initialize this modul.
*/
GLOBAL void reg_initialize (BOOL isStartup)
{
  currRegCmd  = AT_CMD_NONE;
  currAbrtCmd = AT_CMD_NONE;

  if (isStartup)
    COPSmode   = COPS_MOD_Auto;

  COPSoper     = NULL;
  COPSstartidx = 0;
  COPSlastidx  = 0;

  {
    USHORT i;

    for (i = 0; i < MAX_OPER; i++)
    {
      COPSlst[i].status     = COPS_STAT_NotPresent;
      COPSlst[i].longOper   = NULL;
      COPSlst[i].shortOper  = NULL;
      COPSlst[i].numOper[0] = NULL_TERM;
    }

    COPSqryrsn = RSN_NONE;
  
    if (ui_isScrnWidthMin ())
      COPSfrmt = COPS_FRMT_Short;
    else
      COPSfrmt = COPS_FRMT_Long;
  }

  CREGstat = CREG_STAT_Unknown;

  pinCnf   = FALSE;
  pinNeed  = FALSE;
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_REG                  |
| STATE   : code                  ROUTINE : reg_powerOn              |
+--------------------------------------------------------------------+

  PURPOSE : Function for handling of the power on signal. It is
            called after initialization of the PS or from the
            keystroke sequence interpreter.
*/
GLOBAL T_ACI_RETURN reg_powerOn ()
{
  T_ACI_RETURN ret = AT_CMPL; /* holds status information */

  TRACE_FUNCTION ("reg_power_on ()");

  if (mmi_data->state EQ MMI_OFF AND
      currRegCmd      EQ AT_CMD_NONE)
  {
    aca_init (FALSE);

    currRegCmd = AT_CMD_CFUN;

    /*
     *---------------------------------------------------------------
     * check if the remote communication partner has switch on the
     * mobile
     *---------------------------------------------------------------
     */
#if 0
    {
      T_ACI_CFUN_FUN  fun;
      T_ACI_CPIN_RSLT code;
      T_ACI_CME_ERR   error;

      qAT_PlusCFUN (CMD_SRC_LCL, &fun);
      qAT_PlusCPIN (CMD_SRC_LCL, &code);

      if (fun EQ CFUN_FUN_Full)
      {
        switch (code)
        {
          case (CPIN_RSLT_SimReady):
            rAT_OK (currRegCmd);
            break;

          case (CPIN_RSLT_NotPresent):
            error = CME_ERR_SimNotIns;
            break;

          case (CPIN_RSLT_SimPinReq):
            error = CME_ERR_SimPinReq;
            break;

          case (CPIN_RSLT_SimPukReq):
            error = CME_ERR_SimPukReq;
            break;

          case (CPIN_RSLT_PhSimPinReq):
            error = CME_ERR_PhSimPinReq;
            break;
        }

        if (code NEQ CPIN_RSLT_SimReady)
          rAT_PlusCME (currRegCmd, code);

        return ret;
      }
    }
#endif

    /*
     *---------------------------------------------------------------
     * the remote communication partner has not switched on the
     * mobile, perform normal processing
     *---------------------------------------------------------------
     */
    ret = sAT_PlusCFUN (CMD_SRC_LCL,
                        CFUN_FUN_Full,
                        CFUN_FUN_NotPresent);

    /*
     *---------------------------------------------------------------
     * process result code of ACI function call
     *---------------------------------------------------------------
     */
    switch (ret)
    {
      case (AT_CMPL):
        rAT_OK (currRegCmd);
        break;

      case (AT_EXCT):
        break;

      default:
        rAT_PlusCME (currRegCmd, CME_ERR_Unknown);
        break;
    }
  }

  return ret;
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_REG                  |
| STATE   : code                  ROUTINE : reg_powerOff             |
+--------------------------------------------------------------------+

  PURPOSE : Function for handling the power off signal. It is called
            after the raise of specific conditions in the keystroke
            sequence interpreter or in the power handling circuit.
*/
GLOBAL T_ACI_RETURN reg_powerOff (void)
{
  T_ACI_RETURN ret = AT_CMPL; /* holds status information */

  TRACE_FUNCTION ("reg_powerOff ()");

  if (mmi_data->state NEQ MMI_OFF AND
      currRegCmd      EQ  AT_CMD_NONE)
  {
    ret = sAT_PlusCFUN (CMD_SRC_LCL,
                        CFUN_FUN_Minimum,
                        CFUN_FUN_NotPresent);

    currRegCmd = AT_CMD_CFUN;

    /*
     *---------------------------------------------------------------
     * process result code of ACI function call
     *---------------------------------------------------------------
     */
    switch (ret)
    {
      case (AT_CMPL):
        rAT_OK (currRegCmd);
        break;

      case (AT_EXCT):
        break;

      default:
        rAT_PlusCME (currRegCmd, CME_ERR_Unknown);
        break;
    }
  }
  else
  {
    switch (currRegCmd)
    {
      case (AT_CMD_COPS):
      case (AT_CMD_CFUN):
        if (currAbrtCmd EQ AT_CMD_ABRT)
        {
          ret = AT_CMPL;
        }
        else
        {
          if (aca_setAbrtCmd (currRegCmd))
          {
            ret = sAT_Abort (CMD_SRC_LCL, currRegCmd);

            currAbrtCmd = AT_CMD_ABRT;

            switch (ret)
            {
              case (AT_EXCT):
                break;

              case (AT_CMPL):
                rAT_OK (currAbrtCmd);
                break;

              default:
                rAT_PlusCME (currAbrtCmd, CME_ERR_FailedToAbort);
                break;
            }
          }
          else
          {
            ui_signal (UI_SIG_UNHND_CME_ERR, CME_ERR_FailedToAbort);
            ret = AT_CMPL;
          }
        }
        break;

      default:
        ui_signal (UI_SIG_UNHND_CME_ERR, CME_ERR_FailedToAbort);
        ret = AT_CMPL;
        break;
    }
  }

  return ret;
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_REG                  |
| STATE   : code                  ROUTINE : reg_CfunCnf              |
+--------------------------------------------------------------------+

  PURPOSE : This function is called to indicate the correct processing
            of the phone functionality request. Network registration
            is started if necessary.
*/
GLOBAL void reg_CfunCnf (T_ACI_AT_CMD cmdId)
{
  TRACE_FUNCTION ("reg_CfunCnf ()");

  if (cmdId EQ currRegCmd)
    currRegCmd = AT_CMD_NONE;
  else
    return;

  if (currAbrtCmd EQ AT_CMD_ABRT)
  {
    currAbrtCmd = AT_CMD_NONE;

    reg_powerOff();
    return;
  }

  if (mmi_data->state EQ MMI_OFF)
  {
    aca_setupVolume (csf_getVolume (VT_AUDIO_IN),
                     csf_getVolume (VT_AUDIO_OUT));

    ui_signal (UI_SIG_POWER_UP);

    /*
     *---------------------------------------------------------------
     * check if the remote communication partner has already
     * registered the mobile
     *---------------------------------------------------------------
     */   
#if 0
    {
      T_ACI_CREG_STAT stat;

      qAT_PlusCREG (CMD_SRC_LCL, &stat);

      if (stat EQ CREG_STAT_Reg OR
          stat EQ CREG_STAT_Roam  )
      {
        if (ui_isScrnWidthMin ())
          COPSfrmt = COPS_FRMT_Short;
        else
          COPSfrmt = COPS_FRMT_Long;

        COPSoper        = NULL;
        currRegCmd      = AT_CMD_COPS;
        mmi_data->state = MMI_WAIT_FOR_REG;

        rAT_OK (currRegCmd);

        return;
      }
    }
#endif
    /*
     *---------------------------------------------------------------
     * the remote communication partner has not yet registered the
     * mobile, perform normal processing
     *---------------------------------------------------------------
     */   
    /*
     *---------------------------------------------------------------
     * manual registration is started via user interface or
     * via keystroke sequences entered using pei_config
     *---------------------------------------------------------------
     */
    /*if (COPSmode EQ COPS_MOD_Auto)*/
      reg_startReg ();
  }
  else
  {
    COPSmode   = COPS_MOD_Auto;
    ui_signal (UI_SIG_POWER_DOWN);
    mmi_data->state = MMI_OFF;
  }
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_REG                  |
| STATE   : code                  ROUTINE : reg_CfunRej              |
+--------------------------------------------------------------------+

  PURPOSE : This function is called to indicate the incorrect
            processing of the phone functionality request. The
            behaviour of the SMI depends on the transfered error
            code.

            <err>: error code
*/
GLOBAL void reg_CfunRej (T_ACI_AT_CMD cmdId, T_ACI_CME_ERR err)
{
  TRACE_FUNCTION ("reg_CfunRej ()");

  if (cmdId EQ currRegCmd)
    currRegCmd = AT_CMD_NONE;

  if (currAbrtCmd EQ AT_CMD_ABRT)
  {
    currAbrtCmd = AT_CMD_NONE;
    
    ui_signal (UI_SIG_UNHND_CME_ERR, CME_ERR_FailedToAbort);
    return;
  }

  if (mmi_data->state EQ MMI_OFF)
  {
    aca_setupVolume (csf_getVolume (VT_AUDIO_IN),
                     csf_getVolume (VT_AUDIO_OUT));

    ui_signal (UI_SIG_POWER_UP);

    switch (err) 
    {
      case (CME_ERR_SimPinReq):
        ui_signal (UI_SIG_ENTER_PIN);
        pinNeed         = TRUE;
        mmi_data->state = MMI_WAIT_FOR_PIN;
        break;

      case (CME_ERR_SimPukReq):
        ui_signal (UI_SIG_ENTER_PUK);
        pinNeed         = TRUE;
        mmi_data->state = MMI_WAIT_FOR_PUK;
        break;
    
      case (CME_ERR_SimFail):
      case (CME_ERR_SimWrong):
        ui_signal (UI_SIG_SIM_WRONG);
        mmi_data->state = MMI_ON;
        break;

      case (CME_ERR_SimNotIns):
        ui_signal (UI_SIG_SIM_NOT_INSERT);
        mmi_data->state = MMI_ON;
        break;

      default:
        ui_signal (UI_SIG_UNHND_CME_ERR, err);
        mmi_data->state = MMI_ON;
        break;
    }
  
    reg_startReg ();
  }
  else
    ui_signal (UI_SIG_UNHND_CME_ERR, err);
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_REG                  |
| STATE   : code                  ROUTINE : reg_startManReg          |
+--------------------------------------------------------------------+

  PURPOSE : This function is used to start a manual registration.

            returns: status information concerning the registration
                     progress
*/
LOCAL T_ACI_RETURN reg_startManReg (void)
{
  T_ACI_RETURN ret = AT_FAIL; /* holds the status information */

  /*
   *-----------------------------------------------------------------
   * call to ACI function when no answer on a further ACI function
   * call is expected
   *-----------------------------------------------------------------
   */
  if (currRegCmd EQ AT_CMD_NONE)
  {
    TRACE_FUNCTION ("reg_startManReg ()");

    if (COPSmode EQ  COPS_MOD_Man AND
        COPSoper NEQ NULL            )
    {
      /*
       *-------------------------------------------------------------
       * start registration with predefined registration mode
       *-------------------------------------------------------------
       */
      ret = sAT_PlusCOPS (CMD_SRC_LCL,
                          COPSmode,
                          COPSfrmt,
                          COPSoper);

      if (ui_isScrnWidthMin ())
        COPSfrmt = COPS_FRMT_Short;
      else
        COPSfrmt = COPS_FRMT_Long;

      COPSoper        = NULL;
      currRegCmd      = AT_CMD_COPS;
      mmi_data->state = MMI_WAIT_FOR_REG;

      /*
       *---------------------------------------------------------
       * process result code of ACI function call
       *---------------------------------------------------------
       */   
      switch (ret)
      {
        case (AT_CMPL):
          ui_signal (UI_SIG_REGISTERING);
          rAT_OK (currRegCmd);
          break;

        case (AT_EXCT):
          ui_signal (UI_SIG_REGISTERING);
          break;

        default:
          rAT_PlusCME (currRegCmd, CME_ERR_Unknown);
          break;
      }
    }
    else
    {
      switch (CREGstat)
      {
        case (CREG_STAT_NoSearch):
        case (CREG_STAT_Denied):
        case (CREG_STAT_Unknown):
          /*
           *---------------------------------------------------------
           * operator list is requested
           *---------------------------------------------------------
           */   
          ret = tAT_PlusCOPS (CMD_SRC_LCL, 
                              COPSstartidx,
                              &COPSlastidx,
                              &COPSlst[0]);

          /*
           *---------------------------------------------------------
           * process result code of ACI function call
           *---------------------------------------------------------
           */   
          switch (ret)
          {
            case (AT_CMPL):
              currRegCmd = AT_CMD_COPS;
              rAT_PlusCOPS (COPSlastidx, &COPSlst[0]);
              break;
        
            case (AT_EXCT):
              currRegCmd = AT_CMD_COPS;
              break;

            default:
              ui_signal (UI_SIG_UNHND_CME_ERR, CME_ERR_Unknown);
              break;
          }

        default:
          /*
           *---------------------------------------------------------
           * mobile equipment must be deregistered before trying to
           * register again
           *---------------------------------------------------------
           */
          ui_signal (UI_SIG_UNHND_CME_ERR, CME_ERR_Unknown);
          break;
      }
    }
  }
  else
  {
    /*
     *---------------------------------------------------------------
     * error handling while waiting for answer on further ACI
     * function call
     *---------------------------------------------------------------
     */
    ui_signal (UI_SIG_UNHND_CME_ERR, CME_ERR_Unknown);
  }

  return ret;
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_REG                  |
| STATE   : code                  ROUTINE : reg_CopsCnf              |
+--------------------------------------------------------------------+

  PURPOSE : This function is called to indicate that a former call to
            function sAT_PlusCOPS results in a success.
*/
GLOBAL void reg_CopsCnf (T_ACI_AT_CMD cmdId)
{
  CHAR            oper[MAX_ALPHA_OPER_LEN]; /* operator name */
  T_ACI_COPS_MOD  mode;                     /* COPS mode     */
  T_ACI_COPS_FRMT frmt;                     /* COPS format   */
  T_ACI_NRG_RGMD  regMode;
  T_ACI_NRG_SVMD  srvMode;
  T_ACI_NRG_FRMT  oprFrmt;
  T_ACI_NRG_SVMD  srvStat;

  TRACE_FUNCTION ("reg_CopsCnf ()");

  if (cmdId EQ currRegCmd)
    currRegCmd = AT_CMD_NONE;
  else 
    return;

  if (currAbrtCmd EQ AT_CMD_ABRT)
  {
    currAbrtCmd = AT_CMD_NONE;
    
    reg_powerOff();
    return;
  }

  if (mmi_data->state EQ MMI_WAIT_FOR_REG)
  {
    qAT_PercentNRG( CMD_SRC_LCL,
                    &regMode,
                    &srvMode,
                    &oprFrmt,
                    &srvStat,
                    oper);
    
    COPSqryrsn = RSN_NONE;

    if (srvStat EQ NRG_SVMD_Limited)
    {
      mmi_data->serviceType = SRV_LIMITED;
      ui_signal (UI_SIG_LIMITED_SERVICE);
      ui_signal (UI_SIG_REGISTERED);

      if (pinNeed AND pinCnf)
      {
        pinNeed = FALSE;
        reg_startReg ();
      }
    }
    else if (srvStat EQ NRG_SVMD_NoSrv)
    {
      mmi_data->serviceType = SRV_NONE;
      ui_signal (UI_SIG_NO_CELL);
      ui_signal (UI_SIG_REGISTERED);

      if (pinNeed AND pinCnf)
      {
        pinNeed = FALSE;
        reg_startReg ();
      }
    }
    else
    {
      /*
       *---------------------------------------------------------------
       * a registration request was successfull
       *---------------------------------------------------------------
       */
      sAT_PlusCOPS (CMD_SRC_LCL, 
                    COPS_MOD_SetOnly,
                    COPSfrmt,
                    NULL);

      /*
       *---------------------------------------------------------------
       * get the operator name and inform the user interface
       *---------------------------------------------------------------
       */
      if (qAT_PlusCOPS (CMD_SRC_LCL, &mode, &frmt, oper) NEQ AT_CMPL)
        oper[0] = NULL_TERM; 

      /*
       *---------------------------------------------------------------
       * if operator unknown show MCC MNC
       *---------------------------------------------------------------
       */
      if (strlen(oper) EQ 0)
      {
        sAT_PlusCOPS (CMD_SRC_LCL, 
                      COPS_MOD_SetOnly,
                      COPS_FRMT_Numeric,
                      NULL);

        if (qAT_PlusCOPS (CMD_SRC_LCL, &mode, &frmt, oper) NEQ AT_CMPL)
          oper[0] = NULL_TERM; 
      }

      mmi_data->serviceType = SRV_NORMAL;
      ui_signal (UI_SIG_PLMN, &oper[0]);
      ui_signal (UI_SIG_REGISTERED);

      mmi_data->state = MMI_ON;
    }
  }
  else if (mmi_data->state EQ MMI_WAIT_FOR_DEREG)
  {
    /*
     *---------------------------------------------------------------
     * a deregistration request was successfull
     *---------------------------------------------------------------
     */
    qAT_PercentNRG( CMD_SRC_LCL,
                    &regMode,
                    &srvMode,
                    &oprFrmt,
                    &srvStat,
                    oper);
    
    if (srvStat EQ NRG_SVMD_Limited)
    {
      mmi_data->serviceType = SRV_LIMITED;
      ui_signal (UI_SIG_LIMITED_SERVICE);
      ui_signal (UI_SIG_REGISTERED);
    }
    else if (srvStat EQ NRG_SVMD_NoSrv)
    {
      mmi_data->serviceType = SRV_NONE;
      ui_signal (UI_SIG_NO_CELL);
      ui_signal (UI_SIG_REGISTERED);
    }
    else
    {
      if (COPSqryrsn EQ RSN_CHANGE AND
          COPSmode   EQ COPS_MOD_Auto)
      {
        /*
         *-----------------------------------------------------------
         * start an automatic registration
         *-----------------------------------------------------------
         */
        COPSqryrsn      = RSN_NONE;
        mmi_data->state = MMI_ON;

        reg_startReg ();
      }
      else if (COPSqryrsn EQ RSN_CHANGE   AND
              (COPSmode   EQ COPS_MOD_Man OR
               COPSmode   EQ COPS_MOD_Both))
      {
        /*
         *-----------------------------------------------------------
         * start a manual registration
         *-----------------------------------------------------------
         */
        COPSqryrsn      = RSN_NONE; 
        mmi_data->state = MMI_ON;

        reg_startManReg ();
      }
      else
      {
        /*
         *-----------------------------------------------------------
         * start no registration once again
         *-----------------------------------------------------------
         */
        COPSqryrsn      = RSN_NONE;
        mmi_data->state = MMI_ON;
      }
    }
  } 
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_REG                  |
| STATE   : code                  ROUTINE : reg_CopsRej              |
+--------------------------------------------------------------------+

  PURPOSE : This function is called when an registration or
            deregistration attempt failed.

            <err>: error code
*/
GLOBAL void reg_CopsRej (T_ACI_AT_CMD cmdId, T_ACI_CME_ERR err)
{
  CHAR            oper[MAX_ALPHA_OPER_LEN]; /* operator name */
  T_ACI_NRG_RGMD  regMode;
  T_ACI_NRG_SVMD  srvMode;
  T_ACI_NRG_FRMT  oprFrmt;
  T_ACI_NRG_SVMD  srvStat;

  TRACE_FUNCTION ("reg_CopsRej ()");

  if (cmdId EQ currRegCmd)
    currRegCmd = AT_CMD_NONE;

  if (currAbrtCmd EQ AT_CMD_ABRT)
  {
    currAbrtCmd = AT_CMD_NONE;
    
    ui_signal (UI_SIG_UNHND_CME_ERR, CME_ERR_FailedToAbort);
    return;
  }

  if (mmi_data->state EQ MMI_WAIT_FOR_REG)
  {
    /*
     *---------------------------------------------------------------
     * a registration request was unsuccessfull
     *---------------------------------------------------------------
     */
    qAT_PercentNRG( CMD_SRC_LCL,
                    &regMode,
                    &srvMode,
                    &oprFrmt,
                    &srvStat,
                    oper);
    
    if (srvStat EQ NRG_SVMD_Limited)
    {
      mmi_data->serviceType = SRV_LIMITED;
      ui_signal (UI_SIG_LIMITED_SERVICE);
      ui_signal (UI_SIG_REGISTERED);

      if (pinNeed AND pinCnf)
      {
        pinNeed = FALSE;
        reg_startReg ();
      }
    }
    else if (srvStat EQ NRG_SVMD_NoSrv)
    {
      mmi_data->serviceType = SRV_NONE;
      ui_signal (UI_SIG_NO_CELL);
      ui_signal (UI_SIG_REGISTERED);

      if (pinNeed AND pinCnf)
      {
        pinNeed = FALSE;
        reg_startReg ();
      }
    }
  }
  else if (mmi_data->state EQ MMI_WAIT_FOR_DEREG)
  {
    /*
     *---------------------------------------------------------------
     * a deregistration request was unsuccessfull
     *---------------------------------------------------------------
     */
    ui_signal (UI_SIG_UNHND_CME_ERR, err);
    mmi_data->state = MMI_ON;
  }

  COPSqryrsn = RSN_NONE;
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_REG                  |
| STATE   : code                  ROUTINE : reg_changeReg            |
+--------------------------------------------------------------------+

  PURPOSE : This function is called when a deregistration followed by
            a registration once again should be performed.

            returns: status information concerning the deregistration
                     progress
*/
GLOBAL T_ACI_RETURN reg_changeReg (void)
{
  T_ACI_RETURN ret = AT_FAIL; /* holds status information */
  
  TRACE_FUNCTION ("reg_changeReg ()");

  switch (COPSmode)
  {
    case (COPS_MOD_Auto):
    case (COPS_MOD_Man):
    case (COPS_MOD_Both):
      /*
       *-------------------------------------------------------------
       * call to ACI function when no answer on a further ACI
       * function call is expected
       *-------------------------------------------------------------
       */
      if (currRegCmd EQ AT_CMD_NONE)
      {  
        switch (CREGstat)
        {
          /*
           *---------------------------------------------------------
           * no deregistration necessary
           *---------------------------------------------------------
           */   
          case (CREG_STAT_NoSearch):
          case (CREG_STAT_Denied):
            COPSqryrsn      = RSN_CHANGE;
            currRegCmd      = AT_CMD_COPS;
            mmi_data->state = MMI_WAIT_FOR_DEREG;
            rAT_OK (currRegCmd);
            break;

          /*
           *---------------------------------------------------------
           * deregistration necessary
           *---------------------------------------------------------
           */   
          case (CREG_STAT_Reg):
          case (CREG_STAT_Roam):
            ret = sAT_PlusCOPS (CMD_SRC_LCL,
                                COPS_MOD_Dereg,
                                COPS_FRMT_NotPresent,
                                NULL);

            COPSqryrsn      = RSN_CHANGE;
            currRegCmd      = AT_CMD_COPS;
            mmi_data->state = MMI_WAIT_FOR_DEREG;

            /*
             *-------------------------------------------------------
             * process result code of ACI function call
             *-------------------------------------------------------
             */   
            switch (ret)
            {
              case (AT_CMPL):
                ui_signal (UI_SIG_DEREGISTERING);
                rAT_OK (currRegCmd);
                break;

              case (AT_EXCT):
                ui_signal (UI_SIG_DEREGISTERING);
                break;

              default:
                rAT_PlusCME (currRegCmd, CME_ERR_Unknown);
                break;
            }
            break;
        
          /*
           *---------------------------------------------------------
           * unsufficient registration status
           *---------------------------------------------------------
           */   
          default:
            ui_signal (UI_SIG_UNHND_CME_ERR, CME_ERR_Unknown);
            break;
        }
      }
      else
      {
        /*
         *-----------------------------------------------------------
         * error handling while waiting for answer on further ACI
         * function call
         *-----------------------------------------------------------
         */
        ui_signal (UI_SIG_UNHND_CME_ERR, CME_ERR_Unknown);
      }
      break;
    
    default:
      ui_signal (UI_SIG_UNHND_CME_ERR, CME_ERR_Unknown);
      break;
  }
  
  return ret;
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_REG                  |
| STATE   : code                  ROUTINE : rAT_PlusCOPS             |
+--------------------------------------------------------------------+

  PURPOSE : This function is called when the operator list calculated
            by the ACI is available. The operator list is only 
            requested, when an manual registration should be
            performed.

            <lastIdx>: index of last operator in list
            <operLst>: operator list
*/
GLOBAL void rAT_PlusCOPS  (SHORT lastIdx, T_ACI_COPS_OPDESC* operLst)
{
  CHAR*  lstItem[MAX_OPER + 1]; /* holds the operator list */
  USHORT i = 0;                 /* used for counting       */

  TRACE_FUNCTION ("rAT_PlusCOPS ()");

  if (lastIdx EQ ACI_NumParmNotPresent OR
      operLst EQ NULL)
    return;

  /*
   *-----------------------------------------------------------------
   * separate the operator names of the available operators and
   * inform the user interface
   *-----------------------------------------------------------------
   */
  while (i < MAX_OPER                  AND
         operLst[i].longOper  NEQ NULL AND
         operLst[i].shortOper NEQ NULL    )
  {
    if (operLst[i].status EQ COPS_STAT_Available)
    {
      if (COPSfrmt EQ COPS_FRMT_Long)
        lstItem[i] = operLst[i].longOper;
      else
        lstItem[i] = operLst[i].shortOper;
    }

    i++;
  }

  lstItem[i] = NULL;

  ui_signal (UI_SIG_SELECT_PLMN, &lstItem[0]);
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_REG                  |
| STATE   : code                  ROUTINE : reg_startReg             |
+--------------------------------------------------------------------+

  PURPOSE : This function is used to start a registration.

            returns: status information concerning the registration
                     progress
*/
GLOBAL T_ACI_RETURN reg_startReg (void)
{
  T_ACI_RETURN ret = AT_FAIL; /* holds status information */

  TRACE_FUNCTION ("reg_startReg ()");

  if (COPSmode EQ COPS_MOD_Man OR
      COPSmode EQ COPS_MOD_Both  )
  {
    /*
     *---------------------------------------------------------------
     * start manual registration 
     *---------------------------------------------------------------
     */
    ret = reg_startManReg ();
  }
  else
  {
    /*
     *---------------------------------------------------------------
     * call to ACI function when no answer on a further ACI function
     * call is expected
     *---------------------------------------------------------------
     */
    if (currRegCmd EQ AT_CMD_NONE)
    {  
      switch (CREGstat)
      {
        case (CREG_STAT_NoSearch):
        case (CREG_STAT_Denied):
        case (CREG_STAT_Unknown):
          /*
           *---------------------------------------------------------
           * start registration with predefined registration mode
           *---------------------------------------------------------
           */
          ret = sAT_PlusCOPS (CMD_SRC_LCL,
                              COPSmode,
                              COPSfrmt,
                              COPSoper);

          if (ui_isScrnWidthMin ())
            COPSfrmt = COPS_FRMT_Short;
          else
            COPSfrmt = COPS_FRMT_Long;

          COPSoper        = NULL;
          currRegCmd      = AT_CMD_COPS;
          mmi_data->state = MMI_WAIT_FOR_REG;

          /*
           *---------------------------------------------------------
           * process result code of ACI function call
           *---------------------------------------------------------
           */   
          switch (ret)
          {
            case (AT_CMPL):
              ui_signal (UI_SIG_REGISTERING);
              rAT_OK (currRegCmd);
              break;

            case (AT_EXCT):
              ui_signal (UI_SIG_REGISTERING);
              break;

            default:
              rAT_PlusCME (currRegCmd, CME_ERR_Unknown);
              break;
          }
          break;

        default:
          /*
           *---------------------------------------------------------
           * mobile equipment must be deregistered before trying to
           * register again
           *---------------------------------------------------------
           */
          ui_signal (UI_SIG_UNHND_CME_ERR, CME_ERR_Unknown);
          break;
      }

    }
    else
    {
      if (currRegCmd NEQ AT_CMD_COPS)
      {
        /*
         *-----------------------------------------------------------
         * error handling while waiting for answer on further ACI
         * function call
         *-----------------------------------------------------------
         */
        ui_signal (UI_SIG_UNHND_CME_ERR, CME_ERR_Unknown);
      }
    }
  }

  return ret;
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_REG                  |
| STATE   : code                  ROUTINE : reg_setReg               |
+--------------------------------------------------------------------+

  PURPOSE : This function is used to set registration parameter.

            <cops>: holds registration parameter
            
            returns: status information concerning the setting
                     progress
*/
GLOBAL T_ACI_RETURN reg_setReg (T_KSD_COPS *cops)
{
  T_ACI_RETURN ret = AT_FAIL; /* holds status information */

  TRACE_FUNCTION ("reg_setReg ()");

  /*
   *-----------------------------------------------------------------
   * do not disturb any registration / deregistration
   *-----------------------------------------------------------------
   */
  if (currRegCmd EQ AT_CMD_COPS)
  {
    ui_signal (UI_SIG_UNHND_CME_ERR, CME_ERR_Unknown);
  }
  else
  {
    if (cops->mode NEQ COPS_MOD_NotPresent)
      COPSmode = cops->mode;

    if (cops->frmt NEQ COPS_FRMT_NotPresent)
      COPSfrmt = cops->frmt;

    if (cops->oper NEQ NULL)
      COPSoper = cops->oper;

    ret = AT_CMPL;
  }

  return ret;
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_REG                  |
| STATE   : code                  ROUTINE : reg_setOperName          |
+--------------------------------------------------------------------+

  PURPOSE : This function is used to set the operator name used for
            the next manual registration.

            <oper>: operator name
*/
GLOBAL void reg_setOperName (CHAR* oper)
{
  COPSoper = oper;
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_REG                  |
| STATE   : code                  ROUTINE : rAT_PlusCREG             |
+--------------------------------------------------------------------+

  PURPOSE : This function is used to indicate the registration of the 
            mobile equipment.

            <stat>: registration status
*/
GLOBAL void rAT_PlusCREG (T_ACI_CREG_STAT stat,
                            USHORT          lac,
                            USHORT          cid)
{
  CHAR            oper[MAX_ALPHA_OPER_LEN]; /* operator name */
  T_ACI_COPS_MOD  mode;                     /* COPS mode     */
  T_ACI_COPS_FRMT frmt;                     /* COPS format   */
  T_ACI_NRG_RGMD  regMode;
  T_ACI_NRG_SVMD  srvMode;
  T_ACI_NRG_FRMT  oprFrmt;
  T_ACI_NRG_SVMD  srvStat;

  TRACE_FUNCTION ("rAT_PlusCREG ()");

  if (stat NEQ CREG_STAT_NotPresent)
    CREGstat = stat;

  qAT_PercentNRG( CMD_SRC_LCL,
                  &regMode,
                  &srvMode,
                  &oprFrmt,
                  &srvStat,
                  oper);

  switch (srvStat)
  {
    case (NRG_SVMD_Full):
      sAT_PlusCOPS (CMD_SRC_LCL, 
                    COPS_MOD_SetOnly,
                    COPSfrmt,
                    NULL);

      /*
       *-------------------------------------------------------------
       * get the operator name and inform the user interface
       *-------------------------------------------------------------
       */
      if (qAT_PlusCOPS (CMD_SRC_LCL, &mode, &frmt, oper) NEQ AT_CMPL)
        oper[0] = NULL_TERM; 

      /*
       *---------------------------------------------------------------
       * if operator unknown show MCC MNC
       *---------------------------------------------------------------
       */
      if (strlen(oper) EQ 0)
      {
        sAT_PlusCOPS (CMD_SRC_LCL, 
                      COPS_MOD_SetOnly,
                      COPS_FRMT_Numeric,
                      NULL);

        if (qAT_PlusCOPS (CMD_SRC_LCL, &mode, &frmt, oper) NEQ AT_CMPL)
          oper[0] = NULL_TERM; 
      }

      mmi_data->serviceType = SRV_NORMAL;
      ui_signal (UI_SIG_PLMN, &oper[0]);
      break;

    case (NRG_SVMD_Limited):
      mmi_data->serviceType = SRV_LIMITED;
      ui_signal (UI_SIG_LIMITED_SERVICE);
      break;

    case (NRG_SVMD_NoSrv):
      mmi_data->serviceType = SRV_NONE;
      ui_signal (UI_SIG_NO_CELL);
      break;

    default:
      break;
  }

  switch (stat)
  {
    case (CREG_STAT_Search):
      ui_signal (UI_SIG_REGISTERING);
      break;

    default:
      break;
  }
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_REG                  |
| STATE   : code                  ROUTINE : reg_pinState             |
+--------------------------------------------------------------------+

  PURPOSE : This function sets whether PIN or PUK is entered
            successfully.
*/
GLOBAL void reg_pinState (BOOL cnf, BOOL need)
{
  pinCnf  = cnf;
  pinNeed = need;
}
