/* 
+----------------------------------------------------------------------------- 
|  Project :  GSM-F&D (8411)
|  Modul   :  SMI_SMS
+----------------------------------------------------------------------------- 
|  Copyright 2002 Texas Instruments Berlin, AG 
|                 All rights reserved. 
| 
|                 This file is confidential and a trade secret of Texas 
|                 Instruments Berlin, AG 
|                 The receipt of or possession of this file does not convey 
|                 any rights to reproduce or disclose its contents or to 
|                 manufacture, use, or sell anything it may describe, in 
|                 whole, or in part, without the specific written consent of 
|                 Texas Instruments Berlin, AG. 
+----------------------------------------------------------------------------- 
|  Purpose :  This Modul defines the short message service functions
|             for the Man Machine Interface.
+----------------------------------------------------------------------------- 
*/ 

#ifndef SMI_SMS_C
#define SMI_SMS_C
#endif

#define ENTITY_SMI

/*==== INCLUDES ===================================================*/

#include <stdio.h>
#include <string.h>
#include <stdarg.h>
#include "typedefs.h"
#include "vsi.h"
#include "custom.h"
#include "gsm.h"
#include "prim.h"
#include "tok.h"
#include "message.h"

#include "aci_cmh.h"
#include "ksd.h"
#include "aca.h"
#include "smi.h"

/*==== CONSTANTS ==================================================*/

/*==== TYPES ======================================================*/

typedef enum
{
  SMS_RSLT_FAIL   = 0, /* SMS request failed with unspecific error */
  SMS_RSLT_DELETE = 1  /* SMS is deleted from memory               */
}
T_SMS_ACA_RSLT;

/*==== EXPORT =====================================================*/

/*==== VARIABLES ==================================================*/

LOCAL T_ACI_AT_CMD currSmsCmd = AT_CMD_NONE;

LOCAL T_ACA_NUMBER sca;           /* service center address        */
LOCAL T_ACA_NUMBER da;            /* destination address           */

LOCAL T_ACI_SM_DATA msg;          /* short message                 */
LOCAL SHORT vp;                   /* validity period               */
LOCAL UBYTE curIdx;               /* current index of SMS list     */
                                  /* held by ACI                   */

/* t.b.d.: smLst and cbmLst used for call to function +CMGL        */
/* t.b.d.: lastIdx and startIdx for call to function +CMGL         */

/*==== PROTOTYPES==================================================*/

LOCAL void sms_CmgdRej (T_ACI_CMS_ERR err);
LOCAL void sms_CmgwRej (T_ACI_CMS_ERR err);
LOCAL void sms_CmgsRej (T_ACI_CMS_ERR err);
LOCAL void sms_CmssRej (T_ACI_CMS_ERR err);

/*==== FUNCTIONS ==================================================*/
/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_SMS                  |
| STATE   : code                  ROUTINE : sms_initialize           |
+--------------------------------------------------------------------+

  PURPOSE : This function is used to initialize this modul.
*/
GLOBAL void sms_initialize (BOOL isStartup)
{
  currSmsCmd = AT_CMD_NONE;

  sca.number[0] = NULL_TERM;
  sca.type.ton  = TON_Unknown;
  sca.type.npi  = NPI_IsdnTelephony;

  da.number [0] = NULL_TERM;
  da.type.ton   = TON_Unknown;
  da.type.npi   = NPI_IsdnTelephony;

  msg.len       = 0;
  vp            = ACI_NumParmNotPresent;                 
  curIdx        = -1;
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_SMS                  |
| STATE   : code                  ROUTINE : rAT_sms_ready            |
+--------------------------------------------------------------------+

  PURPOSE : Indication by phonebook that sms is ready
*/
GLOBAL void rAT_sms_ready (void)
{
  TRACE_FUNCTION ("rAT_sms_ready ()");

  /* t.b.d.: initializing of smLst and cbmLst used for call to */
  /*         function +CMGL                                    */
  /* t.b.d.: initializing lastIdx, startIdx, curIdx for call   */
  /*         to function +CMGL                                 */
/*  Does not work properly and disturbes testing
  sAT_PlusCSMS (CMD_SRC_LCL, CSMS_SERV_GsmPh2);
  sAT_PlusCPMS (CMD_SRC_LCL, SMS_STOR_Sm, SMS_STOR_Sm, SMS_STOR_Sm);
  sAT_PlusCMGF (CMD_SRC_LCL, CMGF_MOD_Txt);
  sAT_PlusCNMI (CMD_SRC_LCL,
                CNMI_MT_NoSmsDeliverInd,
                CNMI_BM_NoCbmInd, 
                CNMI_DS_NoSmsStatRpt);
*/
/*
  sAT_PlusCNMI (CMD_SRC_LCL,
                CNMI_MT_SmsDeliverInd,
                CNMI_BM_Cbm, 
                CNMI_DS_NoSmsStatRpt);
*/
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_SMS                  |
| STATE   : code                  ROUTINE : sms_CmgrRej              |
+--------------------------------------------------------------------+

  PURPOSE : This function is called to indicate that a message is not
            read successfully from the preferred message storage.

            <err>: error code
*/
LOCAL void sms_CmgrRej (T_ACI_CMS_ERR err)
{
  TRACE_FUNCTION ("sms_CmgrRej ()");

  ui_signal (UI_SIG_UNHND_CMS_ERR, err);
}
                
/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_SMS                  |
| STATE   : code                  ROUTINE : sms_startInputProc       |
+--------------------------------------------------------------------+

  PURPOSE : This function is called to start an input procedure
            leading in execution of a short message transaction.

            <proc>: specifies the transaction

            returns: status information concerning the input
                     procedure progress
*/
GLOBAL T_ACI_RETURN sms_startInputProc (T_ACA_SMS_PROC proc)
{
  UBYTE        signal;
  T_ACI_RETURN ret = AT_FAIL;

  TRACE_FUNCTION ("sms_startInputProc ()");

  if (mmi_data->serviceType EQ SRV_NORMAL)
  {
    switch (proc)
    {
      case (SMS_PROC_SEND):
        signal = UI_SIG_SMS_PROC_SUBMIT;
        break;

      case (SMS_PROC_SEND_FROM_MEM):
        signal = UI_SIG_SMS_PROC_SUBMIT_FROM_MEM;
        break;

      case (SMS_PROC_WRITE):
        signal = UI_SIG_SMS_PROC_WRITE;
        break;

      case (SMS_PROC_READ):
        signal = UI_SIG_SMS_PROC_READ;
        break;

      case (SMS_PROC_DELETE):
        signal = UI_SIG_SMS_PROC_DELETE;
        break;

      case (SMS_PROC_LIST):
        signal = UI_SIG_SMS_PROC_LIST;
        break;

      case (SMS_PROC_READ_SINGLE):
        signal = UI_SIG_SMS_PROC_READ_SINGLE;
        break;
    }

    ui_sms_signal (signal);

    ret = AT_EXCT;
  }

  return ret;
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_SMS                  |
| STATE   : code                  ROUTINE : sms_signal               |
+--------------------------------------------------------------------+

  PURPOSE : This function processs the SMS_SIG_XXX signals that are
            send by the MMI_UI logic.
*/
static USHORT sms_siglevel = 0; /* used to indicate a recursion */

GLOBAL void sms_signal (UBYTE sigCode, ...)
{
  va_list varpars;
  CHAR    buf[30];

  sprintf (buf, "[%d], SMS-Signal: %d", sms_siglevel++, sigCode);
  TRACE_EVENT (buf); 
   
  switch (sigCode)
  {
    /*
     *---------------------------------------------------------------
     * process the signal SMS_SIG_START_LIST_VIEW
     *---------------------------------------------------------------
     */
    case (SMS_SIG_START_LIST_VIEW):
    {
      curIdx = -1;
      
      ui_sms_signal (UI_SIG_SMS_SHOW_LIST, 0, 1, "1234567890",
                     "First short message used for testing purposes");
    }
    break;

    /*
     *---------------------------------------------------------------
     * process the signal SMS_SIG_NEXT_LIST_VIEW
     *---------------------------------------------------------------
     */
    case (SMS_SIG_NEXT_LIST_VIEW):
    {
      ui_sms_signal (UI_SIG_SMS_SHOW_LIST, 0, 2, "2345678901",
                     "Second short message used for testing purposes");
    }
    break;

    /*
     *---------------------------------------------------------------
     * process the signal SMS_SIG_PREV_LIST_VIEW
     *---------------------------------------------------------------
     */
    case (SMS_SIG_PREV_LIST_VIEW):
    {
      ui_sms_signal (UI_SIG_SMS_SHOW_LIST, 1, 3, 
                     "First cell broadcast message used for testing purposes");
    }
    break;

    /*
     *---------------------------------------------------------------
     * process the signal SMS_SIG_START_MSG_VIEW
     *---------------------------------------------------------------
     */
    case (SMS_SIG_START_MSG_VIEW):
    {
      curIdx = -1;
      
      ui_sms_signal (UI_SIG_SMS_SHOW_MSG,
                     "This is a testing short message which will be deleted unless the SMS functionalities are supported by ACI. \nThe quick brown fox jumps over the lazy dog. \nEnd");
    }
    break;

    /*
     *---------------------------------------------------------------
     * process the signal SMS_SIG_DELETE
     *---------------------------------------------------------------
     */
    case (SMS_SIG_DELETE):
    {
      if (curIdx > -1)
      {
        /*
         *-----------------------------------------------------------
         * call to ACI function when no answer on a further ACI
         * function call is expected
         *-----------------------------------------------------------
         */
        if (currSmsCmd EQ AT_CMD_NONE)
        {
          T_ACI_RETURN ret;
      
          ret = sAT_PlusCMGD (CMD_SRC_LCL, curIdx);
  
          currSmsCmd = AT_CMD_CMGD;

          /*
           *---------------------------------------------------------
           * process result code of ACI function call
           *---------------------------------------------------------
           */   
          switch (ret)
          {
            case (AT_CMPL):
              rAT_OK (currSmsCmd);
              break;

            case (AT_EXCT):
              break;

            default:
              rAT_PlusCMS (currSmsCmd, CMS_ERR_UnknownErr, NULL);
              break;
          }
        }
        else
          /*
           *---------------------------------------------------------
           * error handling while waiting for answer on further ACI
           * function call
           *---------------------------------------------------------
           */
          sms_CmgdRej (CMS_ERR_UnknownErr);
      }
    }
    break;

    /*
     *---------------------------------------------------------------
     * process the signal SMS_SIG_READ_SINGLE
     *---------------------------------------------------------------
     */
    case (SMS_SIG_READ_SINGLE):
    {
      /*
       *-------------------------------------------------------------
       * call to ACI function when no answer on a further ACI
       * function call is expected
       *-------------------------------------------------------------
       */
      if (currSmsCmd EQ AT_CMD_NONE)
      {
        T_ACI_RETURN   ret;
        T_ACI_SMS_STOR mem;
        UBYTE          index;
      
        va_start (varpars, sigCode);
        mem   = va_arg (varpars, T_ACI_SMS_STOR);
        index = va_arg (varpars, UBYTE);
        va_end (varpars);

        if (sAT_PlusCPMS (CMD_SRC_LCL,
                          mem,
                          SMS_STOR_NotPresent, 
                          SMS_STOR_NotPresent ) NEQ AT_CMPL)
          sms_CmgrRej (CMS_ERR_UnknownErr);

        ret = sAT_PlusCMGR (CMD_SRC_LCL, index, SMS_READ_Normal);
  
        currSmsCmd = AT_CMD_CMGR;

        /*
         *-----------------------------------------------------------
         * process result code of ACI function call
         *-----------------------------------------------------------
         */   
        switch (ret)
        {
          case (AT_EXCT):
            break;

          default:
            rAT_PlusCMS (currSmsCmd, CMS_ERR_UnknownErr, NULL);
            break;
        }
      }
      else
        /*
         *-----------------------------------------------------------
         * error handling while waiting for answer on further ACI
         * function call
         *-----------------------------------------------------------
         */
        sms_CmgrRej (CMS_ERR_UnknownErr);
    }
    break;

    /*
     *---------------------------------------------------------------
     * process the signal SMS_SIG_WRITE
     *---------------------------------------------------------------
     */
    case (SMS_SIG_WRITE):
    {
      /*
       *-------------------------------------------------------------
       * call to ACI function when no answer on a further ACI
       * function call is expected
       *-------------------------------------------------------------
       */
      if (currSmsCmd EQ AT_CMD_NONE)
      {
        T_ACI_RETURN ret;
      
        ret = sAT_PlusCMGW_Old (CMD_SRC_LCL,
                            ACI_NumParmNotPresent,
                            &da.number[0],
                            &da.type,
                            SMS_STAT_NotPresent,
                            NOT_PRESENT_8BIT,
                            &msg, NULL, NULL, FALSE);
  
        currSmsCmd = AT_CMD_CMGW;

        /*
         *-----------------------------------------------------------
         * process result code of ACI function call
         *-----------------------------------------------------------
         */   
        switch (ret)
        {
          case (AT_EXCT):
            break;

          default:
            rAT_PlusCMS (currSmsCmd, CMS_ERR_UnknownErr, NULL);
            break;
        }
      }
      else
        /*
         *-----------------------------------------------------------
         * error handling while waiting for answer on further ACI
         * function call
         *-----------------------------------------------------------
         */
        sms_CmgwRej (CMS_ERR_UnknownErr);
    }
    break;

    /*
     *---------------------------------------------------------------
     * process the signals SMS_SIG_SUBMIT and SMS_SIG_SUBMIT_FROM_MEM
     *---------------------------------------------------------------
     */
    case (SMS_SIG_SUBMIT):
    case (SMS_SIG_SUBMIT_FROM_MEM):
    {
      SHORT pid = 0;
      SHORT dcs = 0;
      SHORT fo  = TP_MTI_SMS_SUBMIT | TP_SRR_REQUEST;

      fo |= (vp < 0 ? TP_VPF_NOT_PRESENT : TP_VPF_RELATIVE);

      if (!(sigCode EQ SMS_SIG_SUBMIT_FROM_MEM AND
            curIdx <= -1))
      {
        /*
         *-----------------------------------------------------------
         * setting of text mode parameter and service center address 
         *-----------------------------------------------------------
         * call to ACI function when no answer on a further ACI
         * function call is expected
         *-----------------------------------------------------------
        */
        if (sAT_PlusCSMP (CMD_SRC_LCL, fo, vp, NULL, NULL, pid, dcs)
                                                           EQ AT_CMPL
            AND

            sAT_PlusCSCA (CMD_SRC_LCL,
                          &sca.number[0],
                          &sca.type       ) EQ AT_CMPL

            AND

            currSmsCmd EQ AT_CMD_NONE)
        {
          T_ACI_RETURN ret;

          if (sigCode EQ SMS_SIG_SUBMIT)
          {
            /*
             *-------------------------------------------------------
             * submit the short message
             *-------------------------------------------------------
             */
            ret = sAT_PlusCMGS_Old (CMD_SRC_LCL,
                                &da.number[0],
                                &da.type,
                                &msg, NULL, NULL, FALSE);
  
            currSmsCmd = AT_CMD_CMGS;
          }
          else
          {
            /*
             *-------------------------------------------------------
             * submit the short message from memory
             *-------------------------------------------------------
             */
            ret = sAT_PlusCMSS (CMD_SRC_LCL,
                                curIdx,
                                &da.number[0],
                                &da.type);

            currSmsCmd = AT_CMD_CMSS;
          }

          /*
           *---------------------------------------------------------
           * process result code of ACI function call
           *---------------------------------------------------------
           */   
          switch (ret)
          {
            case (AT_EXCT):
              break;

            default:
              rAT_PlusCMS (currSmsCmd, CMS_ERR_UnknownErr, NULL);
              break;
          }
        }
        else
        {
          /*
           *-------------------------------------------------------
           * error handling while waiting for answer on further ACI
           * function call
           *-------------------------------------------------------
           */
           if (sigCode EQ SMS_SIG_SUBMIT)
             sms_CmgsRej (CMS_ERR_UnknownErr);
           else
             sms_CmssRej (CMS_ERR_UnknownErr);
        }
      }
    }
    break;
  }
  sms_siglevel--;
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_SMS                  |
| STATE   : code                  ROUTINE : sms_sigCmss              |
+--------------------------------------------------------------------+

  PURPOSE : This function is called to signal that a message is
            send successfully to the service center.

            <mr>: message reference
*/
LOCAL void sms_sigCmss (UBYTE mr)
{
  TRACE_FUNCTION ("sms_sigCmss ()");

  ui_signal (UI_SIG_SMS_SUBMIT, mr);
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_SMS                  |
| STATE   : code                  ROUTINE : rAT_PlusCMSS             |
+--------------------------------------------------------------------+

  PURPOSE : This function is called to indicate that a message is
            send successfully to the service center.

            <mr>: message reference
*/
GLOBAL void rAT_PlusCMSS (UBYTE mr, UBYTE numSeg)
{
  TRACE_FUNCTION ("rAT_PlusCMSS ()");

  if (currSmsCmd EQ AT_CMD_CMSS)
  {
    currSmsCmd = AT_CMD_NONE;
    sms_sigCmss (mr);  
  }
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_SMS                  |
| STATE   : code                  ROUTINE : sms_CmssRej              |
+--------------------------------------------------------------------+

  PURPOSE : This function is called to indicate that a previous 
            submitted short message could not be sent to the
            service center.

            <err>: error code
*/
LOCAL void sms_CmssRej (T_ACI_CMS_ERR err)
{
  TRACE_FUNCTION ("sms_CmssRej ()");

  ui_signal (UI_SIG_UNHND_CMS_ERR, err);
}
                
/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_SMS                  |
| STATE   : code                  ROUTINE : sms_sigCmgs              |
+--------------------------------------------------------------------+

  PURPOSE : This function is called to signal that a message is
            send successfully to the service center.

            <mr>: message reference
*/
LOCAL void sms_sigCmgs (UBYTE mr)
{
  TRACE_FUNCTION ("sms_sigCmgs ()");

  ui_signal (UI_SIG_SMS_SUBMIT, mr);
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_SMS                  |
| STATE   : code                  ROUTINE : rAT_PlusCMGS             |
+--------------------------------------------------------------------+

  PURPOSE : This function is called to indicate that a message is
            send successfully to the service center.

            <mr>: message reference
*/
GLOBAL void rAT_PlusCMGS (UBYTE mr, UBYTE numSeg)
{
  TRACE_FUNCTION ("rAT_PlusCMGS ()");

  if (currSmsCmd EQ AT_CMD_CMGS)
  {
    currSmsCmd = AT_CMD_NONE;
    sms_sigCmgs (mr);  
  }
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_SMS                  |
| STATE   : code                  ROUTINE : sms_CmgsRej              |
+--------------------------------------------------------------------+

  PURPOSE : This function is called to indicate that a previous 
            submitted short message could not be sent to the
            service center.

            <err>: error code
*/
LOCAL void sms_CmgsRej (T_ACI_CMS_ERR err)
{
  TRACE_FUNCTION ("sms_CmgsRej ()");

  ui_signal (UI_SIG_UNHND_CMS_ERR, err);
}
                
/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_SMS                  |
| STATE   : code                  ROUTINE : sms_sigCmgd              |
+--------------------------------------------------------------------+

  PURPOSE : This function is called to signal the result of a former
            requested deletion of a message from preferred message
            storage.

            <rslt>: result code
*/
LOCAL void sms_sigCmgd (T_SMS_ACA_RSLT rslt)
{
  TRACE_FUNCTION ("sms_sigCmgd ()");

  ui_signal (UI_SIG_SMS_DELETE, rslt);
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_SMS                  |
| STATE   : code                  ROUTINE : sms_CmgdRej              |
+--------------------------------------------------------------------+

  PURPOSE : This function is called to indicate that a message was not
            deleted successfully from preferred message storage.

            <err>: error code
*/
LOCAL void sms_CmgdRej (T_ACI_CMS_ERR err)
{
  TRACE_FUNCTION ("sms_CmgdRej ()");

  ui_signal (UI_SIG_UNHND_CMS_ERR, err);
}
                
/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_SMS                  |
| STATE   : code                  ROUTINE : sms_CmgdCnf              |
+--------------------------------------------------------------------+

  PURPOSE : This function is called to indicate that a message was 
            deleted successfully from preferred message storage.
*/
GLOBAL void sms_CmgdCnf (T_ACI_AT_CMD cmdId)
{
  TRACE_FUNCTION ("sms_CmgdCnf ()");

  if (cmdId EQ currSmsCmd)
  {
    currSmsCmd = AT_CMD_NONE;
    sms_sigCmgd (SMS_RSLT_DELETE);  
  }
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_SMS                  |
| STATE   : code                  ROUTINE : sms_sigCmgw              |
+--------------------------------------------------------------------+

  PURPOSE : This function is called to signal that a message was 
            written successfully to the preferred message storage.

            <index>: location area index
*/
LOCAL void sms_sigCmgw (SHORT index)
{
  TRACE_FUNCTION ("sms_sigCmgw ()");

  ui_signal (UI_SIG_SMS_WRITE, index);
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_SMS                  |
| STATE   : code                  ROUTINE : rAT_PlusCMGW             |
+--------------------------------------------------------------------+

  PURPOSE : This function is called to indicate that a message is
            written successfully to the preferred message storage.

            <index>: location area index
*/
GLOBAL void rAT_PlusCMGW  (UBYTE index, UBYTE numSeg)
{
  TRACE_FUNCTION ("rAT_PlusCMGW ()");

  if (currSmsCmd EQ AT_CMD_CMGW)
  {
    currSmsCmd = AT_CMD_NONE;
    sms_sigCmgw (index);  
  }
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_SMS                  |
| STATE   : code                  ROUTINE : sms_CmgwRej              |
+--------------------------------------------------------------------+

  PURPOSE : This function is called to indicate that a message is not
            stored successfully to the preferred message storage.

            <err>: error code
*/
LOCAL void sms_CmgwRej (T_ACI_CMS_ERR err)
{
  TRACE_FUNCTION ("sms_CmgwRej ()");

  ui_signal (UI_SIG_UNHND_CMS_ERR, err);
}
                
/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_SMS                  |
| STATE   : code                  ROUTINE : rAT_PlusCMS              |
+--------------------------------------------------------------------+

  PURPOSE : This function will be called by the ACI when an error
            inside the ME occured.

            <cmdId>: command identity
            <err>:   error code
*/
GLOBAL void rAT_PlusCMS (T_ACI_AT_CMD cmdId, T_ACI_CMS_ERR err, 
                         T_EXT_CMS_ERROR *conc_error)
{
  CHAR buf[30];

  sprintf (buf, "rAT_PlusCMS (%d, %d)", cmdId, err);
  TRACE_FUNCTION (buf);

  if (cmdId EQ currSmsCmd)
    currSmsCmd = AT_CMD_NONE;

  switch (cmdId)
  {
    case (AT_CMD_CMGR): sms_CmgrRej (err); break;
    case (AT_CMD_CMGD): sms_CmgdRej (err); break;
    case (AT_CMD_CMGW): sms_CmgwRej (err); break;
    case (AT_CMD_CMGS): sms_CmgsRej (err); break;
    case (AT_CMD_CMSS): sms_CmssRej (err); break;

    default: 
      ui_signal (UI_SIG_UNHND_CMS_ERR, err);
      break;
  }
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_SMS                  |
| STATE   : code                  ROUTINE : sms_setSca               |
+--------------------------------------------------------------------+

  PURPOSE : This function will be used to set the last edited service
            center address.

            <inSca>: service center address
*/
GLOBAL void sms_setSca (CHAR* inSca)
{
  if (inSca NEQ NULL)
    aca_cvtNumber (inSca, &sca);
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_SMS                  |
| STATE   : code                  ROUTINE : sms_setDa                |
+--------------------------------------------------------------------+

  PURPOSE : This function will be used to set the last edited 
            destination address.

            <inDa>: destination address
*/
GLOBAL void sms_setDa (CHAR* inDa)
{
  if (inDa NEQ NULL)
    aca_cvtNumber (inDa, &da);
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_SMS                  |
| STATE   : code                  ROUTINE : sms_setMsg               |
+--------------------------------------------------------------------+

  PURPOSE : This function will be used to set the last edited short
            message.

            <inMsg>: short message
*/
GLOBAL void sms_setMsg (CHAR* inMsg)
{
  if (inMsg NEQ NULL)
  {
    msg.len = MINIMUM (strlen(inMsg), MAX_SM_LEN);
    memcpy (msg.data, inMsg, msg.len);
  }
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_SMS                  |
| STATE   : code                  ROUTINE : sms_setVp                |
+--------------------------------------------------------------------+

  PURPOSE : This function will be used to set the last edited 
            validity period.

            <inVp>: validity period
*/
GLOBAL void sms_setVp (SHORT inVp)
{
  if ( inVp >= 0 AND inVp <= 255 )
    vp = inVp;
  else
    vp = -1;
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_SMS                  |
| STATE   : code                  ROUTINE : sms_getSca               |
+--------------------------------------------------------------------+

  PURPOSE : This function will be used to get the last edited service
            center address.

            returns: service center address
*/
GLOBAL void sms_getSca (CHAR* outSca)
{
  aca_recvtNumber (&sca, outSca);
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_SMS                  |
| STATE   : code                  ROUTINE : sms_getDa                |
+--------------------------------------------------------------------+

  PURPOSE : This function will be used to get the last edited 
            destination address.

            returns: destination address
*/
GLOBAL void sms_getDa (CHAR* outDa)
{
  aca_recvtNumber (&da, outDa);
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_SMS                  |
| STATE   : code                  ROUTINE : sms_getMsg               |
+--------------------------------------------------------------------+

  PURPOSE : This function will be used to get the last edited short
            message.

            returns: short message
*/
GLOBAL void sms_getMsg (CHAR* outMsg)
{
  if (outMsg NEQ NULL)
  {
    memcpy (outMsg, msg.data, msg.len);
    outMsg[msg.len] = NULL_TERM;
  }
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_SMS                  |
| STATE   : code                  ROUTINE : sms_getVp                |
+--------------------------------------------------------------------+

  PURPOSE : This function will be used to get the last edited 
            validity period.

            returns: validity period
*/
GLOBAL SHORT sms_getVp (void)
{
  return vp;
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_SMS                  |
| STATE   : code                  ROUTINE : rAT_PlusCMTI             |
+--------------------------------------------------------------------+

  PURPOSE : This function will be called when an incomming short
            message was stored in memory.

            <mem>:   type of memory
            <index>: memory location index
*/
GLOBAL void rAT_PlusCMTI (T_ACI_SMS_STOR mem,
                          UBYTE          index,
                          T_ACI_CMGL_SM* sm)
{
  TRACE_FUNCTION ("rAT_PlusCMTI");

  ui_sms_signal (UI_SIG_SMS_MT_IND, mem, index);
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_SMS                  |
| STATE   : code                  ROUTINE : rAT_PlusCMGR             |
+--------------------------------------------------------------------+

  PURPOSE : This function is used to deliver a short message or a
            cell braodcast message which was read from the memory 
            to the MMI.

            <sm>:  short message
            <cbm>: cell broadcast message
*/
GLOBAL void rAT_PlusCMGR (T_ACI_CMGL_SM*  sm,
                          T_ACI_CMGR_CBM* cbm)
{
  TRACE_FUNCTION ("rAT_PlusCMGR");

  if (currSmsCmd EQ AT_CMD_CMGR)
  {
    currSmsCmd = AT_CMD_NONE;

    if (sm NEQ NULL)
      ui_sms_signal (UI_SIG_SMS_SHOW_MSG, sm->data);
  }
}


/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_SMS                  |
| STATE   : code                  ROUTINE : rAT_PlusCMGD             |
+--------------------------------------------------------------------+

  PURPOSE :
*/
GLOBAL void rAT_PlusCMGD ()
{
  TRACE_FUNCTION ("rAT_PlusCMGD");
}


/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_SMS                  |
| STATE   : code                  ROUTINE : rAT_PlusCBM              |
+--------------------------------------------------------------------+

  PURPOSE : This function is called to indicate an incomming cell
            broadcast message.

            <sn>:    serial number
            <mid>:   message identifier
            <dcs>:   data coding scheme
            <page>:  page parameter bits 4-7
            <pages>: page parameter bits 0-3
            <data>:  message data
*/
GLOBAL void rAT_PlusCBM (SHORT           sn,
                         SHORT           mid, 
                         SHORT           dcs, 
                         UBYTE           page, 
                         UBYTE           pages, 
                         T_ACI_CBM_DATA* data)
{
  TRACE_FUNCTION ("rAT_PlusCBM");

/*
  if (data NEQ NULL)
    ui_sms_signal (UI_SIG_SMS_SHOW_MSG, data);
*/
}
