/* 
+----------------------------------------------------------------------------- 
|  Project :  GSM-F&D (8411)
|  Modul   :  SMI_SS
+----------------------------------------------------------------------------- 
|  Copyright 2002 Texas Instruments Berlin, AG 
|                 All rights reserved. 
| 
|                 This file is confidential and a trade secret of Texas 
|                 Instruments Berlin, AG 
|                 The receipt of or possession of this file does not convey 
|                 any rights to reproduce or disclose its contents or to 
|                 manufacture, use, or sell anything it may describe, in 
|                 whole, or in part, without the specific written consent of 
|                 Texas Instruments Berlin, AG. 
+----------------------------------------------------------------------------- 
|  Purpose :  This Modul defines the supplementary service functions
|             for the Man Machine Interface.
+----------------------------------------------------------------------------- 
*/ 

#ifndef SMI_SS_C
#define SMI_SS_C
#endif

#define ENTITY_SMI

/*==== INCLUDES ===================================================*/

#include "typedefs.h"
#include <string.h>
#include "vsi.h"
#include "custom.h"
#include "gsm.h"
#include "prim.h"
#include "tok.h"
#include "message.h"

#include "aci_cmh.h"
#include "ksd.h"
#include "aca.h"
#include "smi.h"

/*==== CONSTANTS ==================================================*/

/*==== TYPES ======================================================*/

/*==== EXPORT =====================================================*/

/*==== VARIABLES ==================================================*/

LOCAL T_ACI_AT_CMD currSsCmd = AT_CMD_NONE;

LOCAL T_ACA_CLIP_MOD CLIPmode;    /* holds clip mode                */
LOCAL T_ACA_CLIP_MOD CLIPreqmode; /* holds requested clip mode      */
LOCAL T_ACA_RSN      CLIPqryrsn;  /* reason for query clip parameter*/

LOCAL T_ACI_CLIR_MOD CLIRreqmode; /* holds requested clir support   */
LOCAL T_ACA_RSN      CLIRqryrsn;  /* reason for query clir parameter*/

LOCAL T_ACA_COLP_MOD COLPmode;    /* holds colp mode                */
LOCAL T_ACA_COLP_MOD COLPreqmode; /* holds requested colp mode      */
LOCAL T_ACA_RSN      COLPqryrsn;  /* reason for query colp parameter*/

LOCAL T_ACI_KSD_OP   CCFCmode;    /* holds ccfc mode                */
LOCAL T_ACA_RSN      CCFCqryrsn;  /* reason for query ccfc parameter*/

LOCAL T_ACI_KSD_OP   CLCKmode;    /* holds clck mode                */
LOCAL T_ACA_RSN      CLCKqryrsn;  /* reason for query clck parameter*/

LOCAL T_ACI_KSD_OP   CCWAmode;    /* holds ccwa mode                */
LOCAL T_ACA_RSN      CCWAqryrsn;  /* reason for query ccwa parameter*/

/*==== FUNCTIONS ==================================================*/
/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_SS                   |
| STATE   : code                  ROUTINE : ss_initialize            |
+--------------------------------------------------------------------+

  PURPOSE : This function is used to initialize this modul.
*/
GLOBAL void ss_initialize (BOOL isStartup)
{
  currSsCmd = AT_CMD_NONE;

  /*
   *-----------------------------------------------------------------
   * initialize CLIP parameter
   *-----------------------------------------------------------------
   */
  CLIPmode      = CLIP_MOD_NotPresent;
  CLIPreqmode   = CLIP_MOD_NotPresent;
  CLIPqryrsn    = RSN_NONE;
                          
  /*
   *-----------------------------------------------------------------
   * initialize CLIR parameter
   *-----------------------------------------------------------------
   */
  CLIRreqmode   = CLIR_MOD_NotPresent;
  CLIRqryrsn    = RSN_NONE;
                          
  /*
   *-----------------------------------------------------------------
   * initialize COLP parameter
   *-----------------------------------------------------------------
   */
  COLPmode      = COLP_MOD_NotPresent;
  COLPreqmode   = COLP_MOD_NotPresent;
  COLPqryrsn    = RSN_NONE;
                          
  /*
   *-----------------------------------------------------------------
   * initialize CCFC parameter
   *-----------------------------------------------------------------
   */
  CCFCmode      = KSD_OP_NONE;
  CCFCqryrsn    = RSN_NONE;
                          
  /*
   *-----------------------------------------------------------------
   * initialize CLCK parameter
   *-----------------------------------------------------------------
   */
  CLCKmode      = KSD_OP_NONE;
  CLCKqryrsn    = RSN_NONE;

  /*
   *-----------------------------------------------------------------
   * initialize CCWA parameter
   *-----------------------------------------------------------------
   */
  CCWAmode      = KSD_OP_NONE;
  CCWAqryrsn    = RSN_NONE;
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_SS                   |
| STATE   : code                  ROUTINE : ss_sigClip               |
+--------------------------------------------------------------------+

  PURPOSE : 
*/
LOCAL void ss_sigClip (T_SS_ACA_RSLT rslt)
{
  TRACE_FUNCTION ("ss_sigClip ()");

  ui_signal (UI_SIG_SS, rslt);
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_SS                   |
| STATE   : code                  ROUTINE : ss_changeClip            |
+--------------------------------------------------------------------+

  PURPOSE : 
*/
GLOBAL T_ACI_RETURN ss_changeClip (T_KSD_CLIP* Clip)
{
  T_ACI_RETURN ret = AT_FAIL;

  TRACE_FUNCTION ("ss_changeClip ()");

  switch (Clip->mode)
  {
    case (CLIP_MOD_NotPresent):
      ss_sigClip (SS_RSLT_FAIL);
      break;

    case (CLIP_MOD_Disable):
      CLIPmode = Clip->mode;
      ss_sigClip (SS_RSLT_SUPP);
      break;

    case (CLIP_MOD_Enable):
      {
        /*
         *-----------------------------------------------------------
         * call to ACI function when no answer on a further ACI
         * function call is expected
         *-----------------------------------------------------------
         */
        if (currSsCmd EQ AT_CMD_NONE)
        {
          T_ACI_CLIP_STAT clipStat;

          ret = qAT_PlusCLIP (CMD_SRC_LCL, &clipStat);
        
          switch (ret)
          {
            case (AT_CMPL):
              CLIPreqmode = Clip->mode;
              CLIPqryrsn  = RSN_CHANGE;
              currSsCmd   = AT_CMD_CLIP;
              rAT_PlusCLIP (clipStat, NULL, NULL, PRES_NOT_PRES, NULL, NULL, NULL);
              break;

            case (AT_EXCT):
              CLIPreqmode = Clip->mode;
              CLIPqryrsn  = RSN_CHANGE;
              currSsCmd   = AT_CMD_CLIP;
              break;

            default:
              ss_sigClip (SS_RSLT_FAIL);
              break;
          }
        }
        else
        {
          /*
           *---------------------------------------------------------
           * error handling while waiting for answer on further ACI
           * function call
           *---------------------------------------------------------
           */
          ss_sigClip (SS_RSLT_FAIL);

          ret = AT_CMPL;
        }
      }
      break;
  }

  return ret;  
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_SS                   |
| STATE   : code                  ROUTINE : ss_intrgtClip            |
+--------------------------------------------------------------------+

  PURPOSE : 
*/
GLOBAL T_ACI_RETURN ss_intrgtClip (void)
{
  TRACE_FUNCTION ("ss_intrgtClip ()");

  switch (CLIPmode)
  {
    case (CLIP_MOD_NotPresent):
      ss_sigClip (SS_RSLT_NET);
      break;

    case (CLIP_MOD_Disable):
      ss_sigClip (SS_RSLT_SUPP);
      break;

    case (CLIP_MOD_Enable):
      ss_sigClip (SS_RSLT_INVOC);
      break;

    default:
      ss_sigClip (SS_RSLT_FAIL);
      break;
  }

  return AT_CMPL;
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_SS                   |
| STATE   : code                  ROUTINE : rAT_PlusCLIP             |
+--------------------------------------------------------------------+

  PURPOSE : 
*/

#ifdef NO_ASCIIZ
GLOBAL void rAT_PlusCLIP (T_ACI_CLIP_STAT stat,
                          CHAR*           number,
                          T_ACI_TOA*      type,
                          U8              validity,
                          CHAR*           subaddr,
                          T_ACI_TOS*      satype,
                          T_ACI_PB_TEXT*  pbText)
#else  /* #ifdef NO_ASCIIZ */
GLOBAL void rAT_PlusCLIP (T_ACI_CLIP_STAT stat,
                          CHAR*           number,
                          T_ACI_TOA*      type,
                          U8              validity,
                          CHAR*           subaddr,
                          T_ACI_TOS*      satype,
                          CHAR*           alpha)
#endif /* #ifdef NO_ASCIIZ */
{
#ifdef NO_ASCIIZ
  char  alphaBuf[MAX_ALPHA_LEN], *alpha;

  if ( ( pbText NEQ NULL ) AND ( pbText->len NEQ 0 ) )
  {
    memcpy ( alphaBuf, pbText->data, pbText->len );
    alphaBuf[pbText->len] = '\0';
    alpha = alphaBuf;
  }
  else
  {
    alpha = NULL;
  }
#endif /* #ifdef NO_ASCIIZ */

  TRACE_FUNCTION ("rAT_PlusCLIP ()");

  if (stat EQ CLIP_STAT_NotPresent)
  {
    if (CLIPmode EQ CLIP_MOD_Enable OR
        CLIPmode EQ CLIP_MOD_NotPresent)
    {
      if (alpha NEQ NULL)
        ui_signal (UI_SIG_CLIP, alpha, "");
      else if (number NEQ NULL)
      {
        if (subaddr NEQ NULL)
          ui_signal (UI_SIG_CLIP, number, subaddr);
        else
          ui_signal (UI_SIG_CLIP, number , "");
      }
      else
        ui_signal (UI_SIG_CLIP, "", "");
    }
  }
  else
  {
    switch (CLIPqryrsn)
    {
      case (RSN_CHANGE):
        switch (stat)
        {
          case (CLIP_STAT_NotProv):
          case (CLIP_STAT_Unknown):
            switch (CLIPreqmode)
            {
              case (CLIP_MOD_Disable):
                CLIPmode = CLIPreqmode; 
                ss_sigClip (SS_RSLT_SUPP);
                break;

              case (CLIP_MOD_NotPresent):
              case (CLIP_MOD_Enable):
                ss_sigClip (SS_RSLT_FAIL);
                break;
            }
            break;

          case (CLIP_STAT_Prov):
            switch (CLIPreqmode)
            {
              case (CLIP_MOD_Disable):
                CLIPmode = CLIPreqmode; 
                ss_sigClip (SS_RSLT_SUPP);
                break;

              case (CLIP_MOD_Enable):
                CLIPmode = CLIPreqmode; 
                ss_sigClip (SS_RSLT_INVOC);
                break;

              case (CLIP_MOD_NotPresent):
                ss_sigClip (SS_RSLT_FAIL);
                break;
            }
            break;
        }
        break;
        
      default:
        break;
    }

    if (currSsCmd EQ AT_CMD_CLIP)
      currSsCmd = AT_CMD_NONE;

    CLIPqryrsn  = RSN_NONE;
    CLIPreqmode = CLIP_MOD_NotPresent;
  }
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_SS                   |
| STATE   : code                  ROUTINE : ss_sigClir               |
+--------------------------------------------------------------------+

  PURPOSE : 
*/
LOCAL void ss_sigClir (T_SS_ACA_RSLT rslt)
{
  TRACE_FUNCTION ("ss_sigClir ()");

  ui_signal (UI_SIG_SS, rslt);
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_SS                   |
| STATE   : code                  ROUTINE : ss_calcClirRslt          |
+--------------------------------------------------------------------+

  PURPOSE : 
*/
LOCAL T_SS_ACA_RSLT ss_calcClirRslt (T_ACI_CLIR_MOD  mode,
                                     T_ACI_CLIR_STAT state)
{
  /* used to calculate a final result code */
  /* out of the mode and state value       */
  static const struct T_SS_ACA_CLIR_RSLT_TAB
  {
    const T_ACI_CLIR_MOD  mode;
    const T_ACI_CLIR_STAT state;
    const T_SS_ACA_RSLT   rslt;
  }
    clirRslt[] = 
  {
    CLIR_MOD_Subscript,  CLIR_STAT_NotProv,      SS_RSLT_SUPP,
    CLIR_MOD_Subscript,  CLIR_STAT_Permanent,    SS_RSLT_INVOC,
    CLIR_MOD_Subscript,  CLIR_STAT_Unknown,      SS_RSLT_FAIL,
    CLIR_MOD_Subscript,  CLIR_STAT_RestrictTemp, SS_RSLT_INVOC,
    CLIR_MOD_Subscript,  CLIR_STAT_AllowTemp,    SS_RSLT_SUPP,


    CLIR_MOD_Invoc,      CLIR_STAT_NotProv,      SS_RSLT_FAIL,
    CLIR_MOD_Invoc,      CLIR_STAT_Permanent,    SS_RSLT_INVOC,
    CLIR_MOD_Invoc,      CLIR_STAT_Unknown,      SS_RSLT_FAIL,
    CLIR_MOD_Invoc,      CLIR_STAT_RestrictTemp, SS_RSLT_INVOC,
    CLIR_MOD_Invoc,      CLIR_STAT_AllowTemp,    SS_RSLT_INVOC,

    
    CLIR_MOD_Supp,       CLIR_STAT_NotProv,      SS_RSLT_SUPP,
    CLIR_MOD_Supp,       CLIR_STAT_Permanent,    SS_RSLT_INVOC,
    CLIR_MOD_Supp,       CLIR_STAT_Unknown,      SS_RSLT_FAIL,
    CLIR_MOD_Supp,       CLIR_STAT_RestrictTemp, SS_RSLT_SUPP,
    CLIR_MOD_Supp,       CLIR_STAT_AllowTemp,    SS_RSLT_SUPP,

    CLIR_MOD_NotPresent, CLIR_STAT_NotPresent,   SS_RSLT_FAIL
  };

  USHORT i = 0;

  /*
   *-----------------------------------------------------------------
   * searching for the final result code
   *-----------------------------------------------------------------
   */
  while (  clirRslt[i].mode  NEQ CLIR_MOD_NotPresent AND
         !(clirRslt[i].mode  EQ  mode                AND
           clirRslt[i].state EQ  state))
    i++;
  
  return clirRslt[i].rslt;      
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_SS                   |
| STATE   : code                  ROUTINE : ss_changeClirRslt        |
+--------------------------------------------------------------------+

  PURPOSE : 
*/
LOCAL void ss_changeClirRslt (T_ACI_CLIR_MOD  mode, 
                              T_ACI_CLIR_STAT state)
{
  T_SS_ACA_RSLT rslt = ss_calcClirRslt (mode, state);
      
  switch (rslt)
  {
    case (SS_RSLT_SUPP):
      if (sAT_PlusCLIR (CMD_SRC_LCL, mode) EQ AT_CMPL)
        ss_sigClir (SS_RSLT_SUPP);
        
      break;

    case (SS_RSLT_INVOC):
      if (sAT_PlusCLIR (CMD_SRC_LCL, mode) EQ AT_CMPL)
        ss_sigClir (SS_RSLT_INVOC);
         
      break;

    default:
      ss_sigClir (SS_RSLT_FAIL);
      break;
  }
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_SS                   |
| STATE   : code                  ROUTINE : ss_changeClir            |
+--------------------------------------------------------------------+

  PURPOSE : 
*/
GLOBAL T_ACI_RETURN ss_changeClir (T_KSD_CLIR* Clir)
{
  T_ACI_RETURN    ret    = AT_FAIL;
  T_ACI_CLIR_MOD  mode;
  T_ACI_CLIR_STAT state;

  TRACE_FUNCTION ("ss_changeClir ()");

  /*
   *-----------------------------------------------------------------
   * call to ACI function when no answer on a further ACI function
   * call is expected
   *-----------------------------------------------------------------
   */
  if (currSsCmd EQ AT_CMD_NONE)
  {
    ret = qAT_PlusCLIR  (CMD_SRC_LCL, &mode, &state);
  
    switch (ret)
    {
      case (AT_CMPL):
        CLIRreqmode = Clir->mode;
        CLIRqryrsn  = RSN_CHANGE;
        currSsCmd   = AT_CMD_CLIR;
        rAT_PlusCLIR (Clir->mode, state);
        break;

      case (AT_EXCT):
        CLIRreqmode = Clir->mode;
        CLIRqryrsn  = RSN_CHANGE;
        currSsCmd   = AT_CMD_CLIR;
        break;

      default:
        ss_sigClir (SS_RSLT_FAIL);
        break;
    }
  }
  else
  {
    /*
     *---------------------------------------------------------------
     * error handling while waiting for answer on further ACI
     * function call
     *---------------------------------------------------------------
     */
    ss_sigClir (SS_RSLT_FAIL);

    ret = AT_CMPL;
  }

  return ret;
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_SS                   |
| STATE   : code                  ROUTINE : ss_intrgtClir            |
+--------------------------------------------------------------------+

  PURPOSE : 
*/
GLOBAL T_ACI_RETURN ss_intrgtClir (void)
{
  T_ACI_RETURN    ret    = AT_FAIL;
  T_ACI_CLIR_MOD  mode;
  T_ACI_CLIR_STAT state;
  
  /*
   *-----------------------------------------------------------------
   * call to ACI function when no answer on a further ACI function
   * call is expected
   *-----------------------------------------------------------------
   */
  if (currSsCmd EQ AT_CMD_NONE)
  {
    TRACE_FUNCTION ("ss_intrgtClir ()");

    ret = qAT_PlusCLIR  (CMD_SRC_LCL, &mode, &state);

    switch (ret)
    {
      case (AT_CMPL):
        CLIRqryrsn = RSN_INTRGT;
        currSsCmd  = AT_CMD_CLIR;
        rAT_PlusCLIR (mode, state);
        break;

      case (AT_EXCT):
        CLIRqryrsn = RSN_INTRGT;
        currSsCmd  = AT_CMD_CLIR;
        break;

      default:
        ss_sigClir (SS_RSLT_FAIL);
        break;
    }
  }
  else
  {
    /*
     *---------------------------------------------------------------
     * error handling while waiting for answer on further ACI
     * function call
     *---------------------------------------------------------------
     */
    ss_sigClir (SS_RSLT_FAIL);

    ret = AT_CMPL;
  }

  return ret;
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_SS                   |
| STATE   : code                  ROUTINE : rAT_PlusCLIR             |
+--------------------------------------------------------------------+

  PURPOSE : 
*/
GLOBAL void rAT_PlusCLIR  (T_ACI_CLIR_MOD mode, T_ACI_CLIR_STAT state)
{
  TRACE_FUNCTION ("rAT_PlusCLIR ()");

  switch (CLIRqryrsn)
  {
    case (RSN_INTRGT):
      ss_sigClir (ss_calcClirRslt (mode, state));
      break;

    case (RSN_CHANGE):
      ss_changeClirRslt (CLIRreqmode, state);
      break;

    default:
      break;
  }

  if (currSsCmd EQ AT_CMD_CLIR)
    currSsCmd = AT_CMD_NONE;

  CLIRqryrsn  = RSN_NONE;
  CLIRreqmode = CLIR_MOD_NotPresent;
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_SS                   |
| STATE   : code                  ROUTINE : ss_sigColp               |
+--------------------------------------------------------------------+

  PURPOSE : 
*/
LOCAL void ss_sigColp (T_SS_ACA_RSLT rslt)
{
  TRACE_FUNCTION ("ss_sigColp ()");

  ui_signal (UI_SIG_SS, rslt);
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_SS                   |
| STATE   : code                  ROUTINE : ss_changeColpMod         |
+--------------------------------------------------------------------+

  PURPOSE : 
*/
GLOBAL T_ACI_RETURN ss_changeColp (T_KSD_COLP* Colp)
{
  T_ACI_RETURN ret = AT_FAIL;

  TRACE_FUNCTION ("ss_changeColp ()");

  switch (Colp->mode)
  {
    case (COLP_MOD_NotPresent):
      ss_sigColp (SS_RSLT_FAIL);
      break;

    case (COLP_MOD_Disable):
      COLPmode = Colp->mode;
      ss_sigColp (SS_RSLT_SUPP);
      break;

    case (COLP_MOD_Enable):
      {
        /*
         *-----------------------------------------------------------
         * call to ACI function when no answer on a further ACI
         * function call is expected
         *-----------------------------------------------------------
         */
        if (currSsCmd EQ AT_CMD_NONE)
        {
          T_ACI_COLP_STAT colpStat;

          ret = qAT_PlusCOLP (CMD_SRC_LCL, &colpStat);
        
          switch (ret)
          {
            case (AT_CMPL):
              COLPreqmode = Colp->mode;
              COLPqryrsn  = RSN_CHANGE;
              currSsCmd   = AT_CMD_COLP;
              rAT_PlusCOLP (colpStat, NULL, NULL, NULL, NULL, NULL);
              break;

            case (AT_EXCT):
              COLPreqmode = Colp->mode;
              COLPqryrsn  = RSN_CHANGE;
              currSsCmd   = AT_CMD_COLP;
              break;
      
            default:
              ss_sigColp (SS_RSLT_FAIL);
              break;
          }
        }
        else
        {
          /*
           *---------------------------------------------------------
           * error handling while waiting for answer on further ACI
           * function call
           *---------------------------------------------------------
           */
          ss_sigColp (SS_RSLT_FAIL);

          ret = AT_CMPL;
        }
      }
      break;
  }

  return ret;  
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_SS                   |
| STATE   : code                  ROUTINE : ss_intrgtColp            |
+--------------------------------------------------------------------+

  PURPOSE : 
*/
GLOBAL T_ACI_RETURN ss_intrgtColp (void)
{
  TRACE_FUNCTION ("ss_intrgtColp ()");

  switch (COLPmode)
  {
    case (COLP_MOD_NotPresent):
      ss_sigColp (SS_RSLT_NET);
      break;

    case (COLP_MOD_Disable):
      ss_sigColp (SS_RSLT_SUPP);
      break;

    case (COLP_MOD_Enable):
      ss_sigColp (SS_RSLT_INVOC);
      break;

    default:
      ss_sigColp (SS_RSLT_FAIL);
      break;
  }

  return AT_CMPL;
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_SS                   |
| STATE   : code                  ROUTINE : rAT_PlusCOLP             |
+--------------------------------------------------------------------+

  PURPOSE : 
*/
#ifdef NO_ASCIIZ
GLOBAL void rAT_PlusCOLP (T_ACI_COLP_STAT stat,
                          CHAR*           number,
                          T_ACI_TOA*      type,
                          CHAR*           subaddr,
                          T_ACI_TOS*      satype,
                          T_ACI_PB_TEXT*  pbText)
#else  /* #ifdef NO_ASCIIZ */
GLOBAL void rAT_PlusCOLP (T_ACI_COLP_STAT stat,
                          CHAR*           number,
                          T_ACI_TOA*      type,
                          CHAR*           subaddr,
                          T_ACI_TOS*      satype,
                          CHAR*           alpha)
#endif /* #ifdef NO_ASCIIZ */
{
#ifdef NO_ASCIIZ
  char  alphaBuf[MAX_ALPHA_LEN], *alpha;

  if ( ( pbText NEQ NULL ) AND ( pbText->len NEQ 0 ) )
  {
    memcpy ( alphaBuf, pbText->data, pbText->len );
    alphaBuf[pbText->len] = '\0';
    alpha = alphaBuf;
  }
  else
  {
    alpha = NULL;
  }
#endif /* #ifdef NO_ASCIIZ */

  TRACE_FUNCTION ("rAT_PlusCOLP ()");

  if (stat EQ COLP_STAT_NotPresent)
  {
    if (COLPmode EQ COLP_MOD_Enable OR
        COLPmode EQ COLP_MOD_NotPresent)
    {
      if (alpha NEQ NULL)
        ui_signal (UI_SIG_COLP, alpha, "");
      else if (number NEQ NULL)
      {
        if (subaddr NEQ NULL)
          ui_signal (UI_SIG_COLP, number, subaddr);
        else
          ui_signal (UI_SIG_COLP, number , "");
      }
      else
        ui_signal (UI_SIG_COLP, "", "");
    }
  }
  else
  {
    switch (COLPqryrsn)
    {
      case (RSN_CHANGE):
        switch (stat)
        {
          case (COLP_STAT_NotProv):
          case (COLP_STAT_Unknown):
            switch (COLPreqmode)
            {
              case (COLP_MOD_Disable):
                COLPmode = COLPreqmode; 
                ss_sigColp (SS_RSLT_SUPP);
                break;

              case (COLP_MOD_NotPresent):
              case (COLP_MOD_Enable):
                ss_sigColp (SS_RSLT_FAIL);
                break;
            }
            break;

          case (COLP_STAT_Prov):
            switch (COLPreqmode)
            {
              case (COLP_MOD_Disable):
                COLPmode = COLPreqmode; 
                ss_sigColp (SS_RSLT_SUPP);
                break;

              case (COLP_MOD_Enable):
                COLPmode = COLPreqmode; 
                ss_sigColp (SS_RSLT_INVOC);
                break;

              case (COLP_MOD_NotPresent):
                ss_sigColp (SS_RSLT_FAIL);
                break;
            }
            break;
        }
        break;
        
      default:
        break;
    }

    if (currSsCmd EQ AT_CMD_COLP)
      currSsCmd = AT_CMD_NONE;

    COLPqryrsn  = RSN_NONE;
    COLPreqmode = COLP_MOD_NotPresent;
  }
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_SS                   |
| STATE   : code                  ROUTINE : ss_sigCcfc               |
+--------------------------------------------------------------------+

  PURPOSE : 
*/
LOCAL void ss_sigCcfc (T_SS_ACA_RSLT rslt)
{
  TRACE_FUNCTION ("ss_sigCcfc ()");

  ui_signal (UI_SIG_SS, rslt);
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_SS                   |
| STATE   : code                  ROUTINE : ss_changeCf              |
+--------------------------------------------------------------------+

  PURPOSE : 
*/
GLOBAL T_ACI_RETURN ss_changeCf (CHAR* seq, T_ACI_KSD_CF_PRM* prm)
{
  T_ACI_RETURN ret;

  TRACE_FUNCTION ("ss_changeCf ()");

  /*
   *-----------------------------------------------------------------
   * call to ACI function when no answer on a further ACI function
   * call is expected
   *-----------------------------------------------------------------
   */
  if (currSsCmd EQ AT_CMD_NONE)
  {  
    ret = sAT_Dn (CMD_SRC_LCL,
                  seq,
                  D_CLIR_OVRD_Default,
                  D_CUG_CTRL_NotPresent,
                  D_TOC_Voice);

    CCFCmode   = prm->opCd;
    CCFCqryrsn = RSN_CHANGE;
    currSsCmd  = KSD_CMD_CF;

    switch (ret)
    {
      case (AT_CMPL):
        rAT_OK (currSsCmd);
        break;

      case (AT_EXCT):
        break;

      default:
        rAT_PlusCME (currSsCmd, CME_ERR_Unknown);
        break;
    }
  }
  else
  {
    /*
     *---------------------------------------------------------------
     * error handling while waiting for answer on further ACI
     * function call
     *---------------------------------------------------------------
     */
    ui_signal (UI_SIG_UNHND_CME_ERR, CME_ERR_Unknown);

    ret = AT_CMPL;
  }

  return ret;
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_SS                   |
| STATE   : code                  ROUTINE : ss_CcfcRej               |
+--------------------------------------------------------------------+

  PURPOSE : 
*/
GLOBAL void ss_CcfcRej (T_ACI_AT_CMD  cmdId, 
                        T_ACI_CME_ERR err)
{
  TRACE_FUNCTION ("ss_CcfcRej ()");

  if (cmdId EQ currSsCmd)
    currSsCmd = AT_CMD_NONE;

  ui_signal (UI_SIG_UNHND_CME_ERR, err);

  if (CCFCqryrsn EQ RSN_CHANGE)
  {
    CCFCmode   = KSD_OP_NONE;
    CCFCqryrsn = RSN_NONE;
  }
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_SS                   |
| STATE   : code                  ROUTINE : ss_CcfcCnf               |
+--------------------------------------------------------------------+

  PURPOSE : 
*/
GLOBAL void ss_CcfcCnf (T_ACI_AT_CMD cmdId)
{
  TRACE_FUNCTION ("ss_CcfcCnf ()");

  if (cmdId EQ currSsCmd)
    currSsCmd = AT_CMD_NONE;
  else 
    return;

  if (CCFCqryrsn EQ RSN_CHANGE)
  {
    switch (CCFCmode)
    {
      case (KSD_OP_NONE):
        ss_sigCcfc (SS_RSLT_FAIL);
        break;

      case (KSD_OP_DEACT):  
        ss_sigCcfc (SS_RSLT_DEACT);
        break;

      case (KSD_OP_ACT):
        ss_sigCcfc (SS_RSLT_ACT);
        break;

      case (KSD_OP_REG):
        ss_sigCcfc (SS_RSLT_REG);
        break;

      case (KSD_OP_ERS):
        ss_sigCcfc (SS_RSLT_ERASE);
        break;
    }

    CCFCmode   = KSD_OP_NONE;
    CCFCqryrsn = RSN_NONE;
  }
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_SS                   |
| STATE   : code                  ROUTINE : ss_sigClck               |
+--------------------------------------------------------------------+

  PURPOSE : 
*/
LOCAL void ss_sigClck (T_SS_ACA_RSLT rslt)
{
  TRACE_FUNCTION ("ss_sigClck ()");

  ui_signal (UI_SIG_SS, rslt);
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_SS                   |
| STATE   : code                  ROUTINE : ss_changeCb              |
+--------------------------------------------------------------------+

  PURPOSE : 
*/
GLOBAL T_ACI_RETURN ss_changeCb (CHAR* seq, T_ACI_KSD_CB_PRM* prm)
{
  T_ACI_RETURN ret;

  TRACE_FUNCTION ("ss_changeCb ()");

  /*
   *-----------------------------------------------------------------
   * call to ACI function when no answer on a further ACI function
   * call is expected
   *-----------------------------------------------------------------
   */
  if (currSsCmd EQ AT_CMD_NONE)
  {  
    ret = sAT_Dn (CMD_SRC_LCL,
                  seq,
                  D_CLIR_OVRD_Default,
                  D_CUG_CTRL_NotPresent,
                  D_TOC_Voice);

    CLCKmode   = prm->opCd;
    CLCKqryrsn = RSN_CHANGE;
    currSsCmd  = KSD_CMD_CB;

    switch (ret)
    {
      case (AT_CMPL):
        rAT_OK (currSsCmd);
        break;

      case (AT_EXCT):
        break;

      default:
        rAT_PlusCME (currSsCmd, CME_ERR_Unknown);
        break;
    }
  }
  else
  {
    /*
     *---------------------------------------------------------------
     * error handling while waiting for answer on further ACI
     * function call
     *---------------------------------------------------------------
     */
    ui_signal (UI_SIG_UNHND_CME_ERR, CME_ERR_Unknown);

    ret = AT_CMPL;
  }

  return ret;
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_SS                   |
| STATE   : code                  ROUTINE : ss_ClckRej               |
+--------------------------------------------------------------------+

  PURPOSE : 
*/
GLOBAL void ss_ClckRej (T_ACI_AT_CMD  cmdId,
                        T_ACI_CME_ERR err)
{
  TRACE_FUNCTION ("ss_ClckRej ()");

  if (cmdId EQ currSsCmd)
    currSsCmd = AT_CMD_NONE;

  ui_signal (UI_SIG_UNHND_CME_ERR, err);

  if (CLCKqryrsn EQ RSN_CHANGE)
  {
    CLCKmode   = KSD_OP_NONE;
    CLCKqryrsn = RSN_NONE;
  }
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_SS                   |
| STATE   : code                  ROUTINE : ss_ClckCnf               |
+--------------------------------------------------------------------+

  PURPOSE : 
*/
GLOBAL void ss_ClckCnf (T_ACI_AT_CMD cmdId)
{
  TRACE_FUNCTION ("ss_ClckCnf ()");

  if (cmdId EQ currSsCmd)
    currSsCmd = AT_CMD_NONE;
  else 
    return;

  if (CLCKqryrsn EQ RSN_CHANGE)
  {
    switch (CLCKmode)
    {
      case (KSD_OP_NONE):
        ss_sigClck (SS_RSLT_FAIL);
        break;

      case (KSD_OP_ACT):  
        ss_sigClck (SS_RSLT_LOCK);
        break;

      case (KSD_OP_DEACT):
        ss_sigClck (SS_RSLT_UNLOCK);
        break;
    }

    CLCKmode   = KSD_OP_NONE;
    CLCKqryrsn = RSN_NONE;
  }
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_SS                   |
| STATE   : code                  ROUTINE : ss_sigCcwa               |
+--------------------------------------------------------------------+

  PURPOSE : 
*/
LOCAL void ss_sigCcwa (T_SS_ACA_RSLT rslt)
{
  TRACE_FUNCTION ("ss_sigCcwa ()");

  ui_signal (UI_SIG_SS, rslt);
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_SS                   |
| STATE   : code                  ROUTINE : ss_changeCcwa            |
+--------------------------------------------------------------------+

  PURPOSE : 
*/
GLOBAL T_ACI_RETURN ss_changeCw (CHAR* seq, T_ACI_KSD_CW_PRM* prm)
{
  T_ACI_RETURN ret = AT_FAIL;

  TRACE_FUNCTION ("ss_changeCcwa ()");

  /*
   *-----------------------------------------------------------------
   * call to ACI function when no answer on a further ACI function
   * call is expected
   *-----------------------------------------------------------------
   */
  if (currSsCmd EQ AT_CMD_NONE)
  {  
    switch (prm->opCd)
    {
      case (KSD_OP_ACT):
      case (KSD_OP_DEACT):
        ret = sAT_Dn (CMD_SRC_LCL,
                      seq,
                      D_CLIR_OVRD_Default,
                      D_CUG_CTRL_NotPresent,
                      D_TOC_Voice);


        CCWAmode   = prm->opCd;
        CCWAqryrsn = RSN_CHANGE;
        currSsCmd  = KSD_CMD_CW;

        /*
         *-----------------------------------------------------------
         * process result code of ACI function call
         *-----------------------------------------------------------
         */   
        switch (ret)
        {
          case (AT_CMPL):
            rAT_OK (currSsCmd);
            break;

          case (AT_EXCT):
            break;

          default:
            rAT_PlusCME (currSsCmd, CME_ERR_Unknown);
            break;
        }
        break;

      default:
        ss_sigCcwa (SS_RSLT_FAIL);
        break;
    }
  }
  else
  {
    /*
     *---------------------------------------------------------------
     * error handling while waiting for answer on further ACI
     * function call
     *---------------------------------------------------------------
     */
    ui_signal (UI_SIG_UNHND_CME_ERR, CME_ERR_Unknown);

    ret = AT_CMPL;
  }

  return ret;  
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_SS                   |
| STATE   : code                  ROUTINE : ss_CcwaCnf               |
+--------------------------------------------------------------------+

  PURPOSE : 
*/
GLOBAL void ss_CcwaCnf (T_ACI_AT_CMD cmdId)
{
  TRACE_FUNCTION ("ss_CcwaCnf ()");

  if (cmdId EQ currSsCmd)
    currSsCmd = AT_CMD_NONE;
  else 
    return;

  if (CCWAqryrsn EQ RSN_CHANGE)
  {
    switch (CCWAmode)
    {
      case (KSD_OP_NONE):
        ss_sigCcwa (SS_RSLT_FAIL);
        break;

      case (KSD_OP_ACT):  
        ss_sigCcwa (SS_RSLT_ACT);
        break;

      case (KSD_OP_DEACT):
        ss_sigCcwa (SS_RSLT_DEACT);
        break;
    }

    CCWAmode   = KSD_OP_NONE;
    CCWAqryrsn = RSN_NONE;
  }
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_SS                   |
| STATE   : code                  ROUTINE : ss_CcwaRej               |
+--------------------------------------------------------------------+

  PURPOSE : 
*/
GLOBAL void ss_CcwaRej (T_ACI_AT_CMD  cmdId, 
                        T_ACI_CME_ERR err)
{
  TRACE_FUNCTION ("ss_CcwaRej ()");

  if (cmdId EQ currSsCmd)
    currSsCmd = AT_CMD_NONE;

  ui_signal (UI_SIG_UNHND_CME_ERR, err);

  if (CCWAqryrsn EQ RSN_CHANGE)
  {
    CCWAmode   = KSD_OP_NONE;
    CCWAqryrsn = RSN_NONE;
  }
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_SS                   |
| STATE   : code                  ROUTINE : rAT_PlusCCWA             |
+--------------------------------------------------------------------+

  PURPOSE : 
*/
#ifdef NO_ASCIIZ
GLOBAL void rAT_PlusCCWA (T_ACI_CLSSTAT* clsStatLst,
                          CHAR*          number,
                          T_ACI_TOA*     type,
                          U8             validity,
                          T_ACI_CLASS    class_type,
                          T_ACI_PB_TEXT* pbText)
#else  /* #ifdef NO_ASCIIZ */
GLOBAL void rAT_PlusCCWA (T_ACI_CLSSTAT* clsStatLst,
                          CHAR*          number,
                          T_ACI_TOA*     type,
                          U8             validity,
                          T_ACI_CLASS    class_type,
                          CHAR*          alpha)
#endif /* #ifdef NO_ASCIIZ */
{
#ifdef NO_ASCIIZ
  char  alphaBuf[MAX_ALPHA_LEN], *alpha;

  if ( ( pbText NEQ NULL ) AND ( pbText->len NEQ 0 ) )
  {
    memcpy ( alphaBuf, pbText->data, pbText->len );
    alphaBuf[pbText->len] = '\0';
    alpha = alphaBuf;
  }
  else
  {
    alpha = NULL;
  }
#endif /* #ifdef NO_ASCIIZ */

  TRACE_FUNCTION ("rAT_PlusCCWA ()");

  if (clsStatLst EQ NULL)
  {
    if (alpha NEQ NULL)
      ui_signal (UI_SIG_WAITING_CALL, alpha);
    else if (number NEQ NULL)
      ui_signal (UI_SIG_WAITING_CALL, number);
    else
      ui_signal (UI_SIG_WAITING_CALL, "");
  }
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_SS                   |
| STATE   : code                  ROUTINE : rAT_PlusCSSI             |
+--------------------------------------------------------------------+

  PURPOSE : 
*/
GLOBAL void rAT_PlusCSSI  ( T_ACI_CSSI_CODE code,
                            SHORT           index)
{
  TRACE_FUNCTION ("rAT_PlusCSSI ()");

}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_SS                   |
| STATE   : code                  ROUTINE : rAT_PlusCSSU             |
+--------------------------------------------------------------------+

  PURPOSE : 
*/
GLOBAL void rAT_PlusCSSU  ( T_ACI_CSSU_CODE code,
                            SHORT           index,
                            CHAR           *number,
                            T_ACI_TOA      *type,
                            CHAR           *subaddr,
                            T_ACI_TOS      *satype)
{
  TRACE_FUNCTION ("rAT_PlusCSSU ()");

}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_SS                   |
| STATE   : code                  ROUTINE : rAT_PlusCUSD             |
+--------------------------------------------------------------------+

  PURPOSE : 
*/

GLOBAL void rAT_PlusCUSD  ( T_ACI_CUSD_MOD   m,
                            T_ACI_USSD_DATA *ussd,
                            SHORT            dcs)
{
  TRACE_FUNCTION ("rAT_PlusCUSD ()");

}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_SS                   |
| STATE   : code                  ROUTINE : rAT_PlusCCFC             |
+--------------------------------------------------------------------+

  PURPOSE : 
*/

GLOBAL void rAT_PlusCCFC  ( T_ACI_CCFC_SET * setting )
{
  TRACE_FUNCTION ("rAT_PlusCCFC ()");
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_SS                   |
| STATE   : code                  ROUTINE : rAT_PlusCLCK             |
+--------------------------------------------------------------------+

  PURPOSE : 
*/

GLOBAL void rAT_PlusCLCK  ( T_ACI_CLSSTAT * clsStat )
{
  TRACE_FUNCTION ("rAT_PlusCLCK ()");
}
