/* 
+----------------------------------------------------------------------------- 
|  Project :  GSM-F&D (8411)
|  Modul   :  SMI_UI
+----------------------------------------------------------------------------- 
|  Copyright 2002 Texas Instruments Berlin, AG 
|                 All rights reserved. 
| 
|                 This file is confidential and a trade secret of Texas 
|                 Instruments Berlin, AG 
|                 The receipt of or possession of this file does not convey 
|                 any rights to reproduce or disclose its contents or to 
|                 manufacture, use, or sell anything it may describe, in 
|                 whole, or in part, without the specific written consent of 
|                 Texas Instruments Berlin, AG. 
+----------------------------------------------------------------------------- 
|  Purpose :  This Modul defines the user interface of the
|             SMI componente of the mobile station
+----------------------------------------------------------------------------- 
*/ 

#ifndef SMI_UI_C
#define SMI_UI_C
#endif

#define ENTITY_SMI

/*==== INCLUDES ===================================================*/

#include <string.h>
#include <stdlib.h>
#include <stdarg.h>
#include <stdio.h>
#include "typedefs.h"
#include "vsi.h"
#include "custom.h"
#include "gsm.h"
#include "prim.h"
#include "tok.h"
#include "message.h"

#include "aci_cmh.h"
#include "ksd.h"
#include "aca.h"
#include "smi.h"

#include "gdi.h"
#include "light.h"
#include "pwr.h"
#include "rx.h"
#include "audio.h"
#include "l4_tim.h"

#include "cus_smi.h"

/*==== CONSTANTS ==================================================*/
/*
 *-------------------------------------------------------------------
 * conversion of the keycodes, for adaptation of the UI to different
 * hardware environments
 *-------------------------------------------------------------------
 */
#define convertKeyCode(A) (A)

/*
 *-------------------------------------------------------------------
 * states of UI
 *-------------------------------------------------------------------
 */
#define UI_IDLE    0
#define UI_INPUT   1
#define UI_SELECT  2
#define UI_BLOCKED 9

/*
 *-------------------------------------------------------------------
 * the timer mode is used to identify the current usage of timer
 * with identity TUNIVERSAL
 *-------------------------------------------------------------------
 */

typedef enum
{
  TIMER_MOD_NONE,
  TIMER_MOD_WELCOME,
  TIMER_MOD_LCD_TEST,
  TIMER_MOD_BOOTING,
  TIMER_MOD_NIBT
#ifdef BT_ADAPTER
  , TIMER_MOD_BT
#endif
}
T_TIMER_MOD;

/*
 *-------------------------------------------------------------------
 * input mode
 *-------------------------------------------------------------------
 */
typedef enum
{
  INP_MOD_PIN,        /* input of PIN active                    */
  INP_MOD_NPIN,       /* input of new PIN active                */
  INP_MOD_NPIN_TWICE, /* input of new PIN confirmed             */
  INP_MOD_PUK,        /* input of PUK active                    */
  INP_MOD_DIAL,       /* input of phone number or keystroke     */
                      /* sequence is active                     */
  INP_MOD_SM,         /* input of short message active          */
  INP_MOD_SCA,        /* input of service center address active */
  INP_MOD_DA,         /* input of destination address active    */
  INP_MOD_VP,         /* input of validity period active        */
  INP_MOD_MEM,        /* input of memory type active            */
  INP_MOD_IDX,        /* input of index                         */
  INP_MOD_NONE        /* no input is active                     */
}
T_INP_MOD;

/*
 *-------------------------------------------------------------------
 * menu mode
 *-------------------------------------------------------------------
 */
typedef enum
{
  MEN_MOD_MAIN,       /* main menu is active       */
  MEN_MOD_PLMN,       /* PLMN menu is active       */
  MEN_MOD_CTB,        /* call table menu is active */
  MEN_MOD_SMS_LIST,   /* SMS list is active        */
  MEN_MOD_MSG_VIEW,   /* SM or CBM will be shown   */
  MEN_MOD_NONE        /* no menu is active         */
}
T_MEN_MOD;

/*
 *-------------------------------------------------------------------
 * identifies the input sequence to be carried out by the user
 *-------------------------------------------------------------------
 */
typedef enum
{
  INP_SEQ_SMS_SUBMIT,          /* a short message is send       */
  INP_SEQ_SMS_SUBMIT_FROM_MEM, /* a short message in memory is  */
                               /* send                          */
  INP_SEQ_SMS_WRITE,           /* a short message is written to */
                               /* memory                        */
  INP_SEQ_SMS_DELETE,          /* a short message is deleted    */
  INP_SEQ_SMS_READ,            /* a short message will be read  */
  INP_SEQ_SMS_LIST,            /* all short messages will be    */
                               /* listed                        */
  INP_SEQ_SMS_READ_SINGLE,     /* a short message will be read  */
  INP_SEQ_NONE                 /* no input sequence defined     */
}
T_INP_SEQ;

/*
 *-------------------------------------------------------------------
 * formats of the battery and fieldstrength level output
 *-------------------------------------------------------------------
 */
#define FIELDSTRENGTH_FORMAT_L "F%1d"
#define BATTLEVEL_FORMAT_L     "B%1d"

#define FIELDSTRENGTH_FORMAT_S "%1d"
#define BATTLEVEL_FORMAT_S     "%1d"

/*
 *-------------------------------------------------------------------
 * keypad related constants
 *-------------------------------------------------------------------
 */
#define MAX_KEYPAD_MODE     3

/*
 *-------------------------------------------------------------------
 * area related constants
 *-------------------------------------------------------------------
 */
#define SRVC_WIDTH_L        8
#define SRVC_WIDTH_S        1

/*
 *-------------------------------------------------------------------
 * menu related constants
 *-------------------------------------------------------------------
 */
#define LINES_PER_SMS_LIST  3
#define MENU_WIDTH_L        16
#define MENU_WIDTH_S        16
#define MENU_PRMPT_WIDTH    1
#define MENU_ENTRY_WIDTH_L  (MENU_WIDTH_L - MENU_PRMPT_WIDTH)
#define MENU_ENTRY_WIDTH_S  (MENU_WIDTH_S - MENU_PRMPT_WIDTH)
#define MAX_LINES_PER_MSG   ((MAX_SM_LEN / MENU_ENTRY_WIDTH_S) + 1)

/*==== TYPES ======================================================*/

typedef struct
{
  UBYTE       state;
  T_INP_MOD   inputMode;
  T_INP_SEQ   inputSeq;
  T_TIMER_MOD timerMod;
  BOOL        powerOn;
  BOOL        ringerOn;
  BOOL        waitingOn;
  BOOL        nibtOn;
  BOOL        normalService;
  BOOL        sliderActive;
  BOOL        inSubmenu;
  T_MEN_MOD   menuMode;
  BOOL        sendDTMF;
  BOOL        keyBeep;
  BOOL        backlightOn;
  BOOL        shiftOn;
  UBYTE       lastShiftedKey;
  USHORT      lastKeyIdx;
  USHORT      lastKeyRep;
  USHORT      keypadMode;
  USHORT      mainMenuIdx;
  CHAR        lastDialedNumber[MAX_INPUT_SIZE + 1];
} T_UI_DATA;

typedef struct
{
  UBYTE  keypadCode;
  CHAR  *keyCode[MAX_KEYPAD_MODE];
  BOOL   pressed;
} T_KEYTABLE;

/*==== EXPORT =====================================================*/

/*==== VARIABLES ==================================================*/

LOCAL T_KEYTABLE keyTable[] = {
/*
  keypadcode     normalkey     keycode1    keycode2   pressed
 */
  '0',           "0",          " ",        "\000",    FALSE,
  '1',           "1",          NULL,       "\001",    FALSE,
  '2',           "2",          "ABC",      "\002",    FALSE,
  '3',           "3",          "DEF",      "\003",    FALSE,
  '4',           "4",          "GHI",      "\004",    FALSE,
  '5',           "5",          "JKL",      "\005",    FALSE,
  '6',           "6",          "MNO",      "\006",    FALSE,
  '7',           "7",          "PQRS",     "\007",    FALSE,
  '8',           "8",          "TUV",      "\010",    FALSE,
  '9',           "9",          "WXYZ",     "\011",    FALSE,
  KP_CLEAR,      "b",          NULL,       NULL,      FALSE,
  KP_ON_HOOK,    "c",          NULL,       NULL,      FALSE,
  KP_OFF_HOOK,   "d",          "\177",     NULL,      FALSE,
  KP_SHIFT,      "g",          NULL,       NULL,      FALSE,
  KP_POWER,      "j",          NULL,       NULL,      FALSE,
  KP_UP,         "e",          "h",        NULL,      FALSE,
  KP_DOWN ,      "f",          "a",        NULL,      FALSE,
  KP_STAR,       "*",          "+",        "\012",    FALSE,
  KP_HASH,       "#",          "-",        "\013",    FALSE,
  /* must be the last entry */
  NULL_TERM,     NULL,         NULL,       NULL,      FALSE
};

#if defined (NEW_FRAME)
GLOBAL T_HANDLE t_slider_running;
GLOBAL T_HANDLE t_backlight_running;
GLOBAL T_HANDLE t_shiftkey_running;
GLOBAL T_HANDLE t_universal_running;
#else
GLOBAL T_VSI_THANDLE t_slider_handle;
GLOBAL T_VSI_THANDLE t_backlight_handle;
GLOBAL T_VSI_THANDLE t_shiftkey_handle;
GLOBAL T_VSI_THANDLE t_universal_handle;
#endif

GLOBAL USHORT        screensize_x;
GLOBAL USHORT        screensize_y;
GLOBAL USHORT        keyboardType;

LOCAL T_UI_DATA      ui_data_base;
LOCAL T_UI_DATA*     ui_data = &ui_data_base;
LOCAL UBYTE          oldState;

#ifdef BT_ADAPTER
LOCAL char bd_addr_data[BTI_BD_ADDR_LENGTH*2];
#endif /* BT_ADAPTER */

/*
 *-------------------------------------------------------------------
 * MAIN menu
 *-------------------------------------------------------------------
 */
LOCAL CHAR* menu[4];

/*
 *-------------------------------------------------------------------
 * PLMN menu
 *-------------------------------------------------------------------
 */
LOCAL CHAR* plmnmenu[MAX_OPER + 1];
LOCAL BOOL  plmnIsEmpty = TRUE;

/*
 *-------------------------------------------------------------------
 * CALL TABLE menu
 *-------------------------------------------------------------------
 */
LOCAL CHAR  ctbstring[MAX_CALL_NR + 1][MENU_ENTRY_WIDTH_L + 1];
LOCAL CHAR* ctbmenu  [MAX_CALL_NR + 1];

/*
 *-------------------------------------------------------------------
 * SMS LIST menu
 *-------------------------------------------------------------------
 */
LOCAL CHAR  smsstring[LINES_PER_SMS_LIST + 1][MENU_ENTRY_WIDTH_L + 1];
LOCAL CHAR* smsmenu  [LINES_PER_SMS_LIST + 1];

/*
 *-------------------------------------------------------------------
 * used for reading a single SMS entry
 *-------------------------------------------------------------------
 */
T_ACI_SMS_STOR actMem = SMS_STOR_NotPresent;
UBYTE          actIdx = ACI_NumParmNotPresent;

/*
 *-------------------------------------------------------------------
 * MESSAGE VIEW menu
 *-------------------------------------------------------------------
 */
LOCAL CHAR  msgstring[MAX_LINES_PER_MSG + 1][MENU_ENTRY_WIDTH_L + 1];
LOCAL CHAR* msgmenu  [MAX_LINES_PER_MSG + 1];

/*
 *-------------------------------------------------------------------
 * handles for the output areas
 *-------------------------------------------------------------------
 */
LOCAL USHORT areaUser,      areaProvider, areaMenu,    areaSlider,
             areaState,     areaFS,       areaBAT,     areaCaller,
             areaSubMenu,   areaHook,     areaService, areaShift,
             areaUniversal;

/*
 *-------------------------------------------------------------------
 * state of the power key
 *-------------------------------------------------------------------
 */
LOCAL BOOL powerKeyOn = FALSE;

/*
 *-------------------------------------------------------------------
 * temporary storing of PUK and new PIN input
 *-------------------------------------------------------------------
 */
LOCAL CHAR puk[MAX_PUK_LEN + 1];
LOCAL CHAR pin[MAX_PIN_LEN + 1];

/*==== PROTOTYPES =================================================*/

LOCAL void  ui_processDefaultKey (UBYTE     key_pressed,
                                  UBYTE     key_stat);

#if defined (NEW_FRAME)
LOCAL char *ui_getCauseName      (char*     source,
                                  USHORT    cause);
#else
LOCAL CHAR *ui_getCauseName      (BYTE*     source,
                                  USHORT    cause);
#endif

LOCAL void  ui_startNewInput     (USHORT    aHandle,
                                  UBYTE     row,
                                  UBYTE     col,
                                  CHAR*     prompt,
                                  UBYTE     length,
                                  UBYTE     type,
                                  T_INP_MOD inputMode,
                                  CHAR*     value,
                                  BOOL      show);

/*==== FUNCTIONS ==================================================*/
/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_UI                   |
| STATE   : code                  ROUTINE : ui_handleNibt            |
+--------------------------------------------------------------------+

  PURPOSE :
*/
LOCAL void ui_handleNibt (UBYTE nibt)
{
  if (screensize_y EQ 4)
  {
    wm_clearArea (areaCaller);

    switch (nibt)
    {
      case (NIBT_OFF):
        if (ui_data->nibtOn)
        {
#if defined (NEW_FRAME)
          TIMERSTOP (SMI_TUNIVERSAL);
          t_universal_running = FALSE;
#else
          TIMERSTOP (t_universal_handle);
#endif
          ui_data->timerMod = TIMER_MOD_NONE;
          ui_data->nibtOn   = FALSE;
        }
        break;

      case (NIBT_ALERT):
        wm_areaWrite (areaCaller, 0, 0, "NIBT_ALERT");
        break;

      case (NIBT_DISC):
        wm_areaWrite (areaCaller, 0, 0, "NIBT_DISC");
#if defined (NEW_FRAME)
        TIMERSTART (TNIBT_VALUE, SMI_TUNIVERSAL);
        t_universal_running = TRUE;
#else
        TIMERSTART (TNIBT_VALUE, t_universal_handle);
#endif
        ui_data->timerMod = TIMER_MOD_NIBT;
        ui_data->nibtOn   = TRUE;
        break;

      case (NIBT_BUSY):
        wm_areaWrite (areaCaller, 0, 0, "NIBT_BUSY");
#if defined (NEW_FRAME)
        TIMERSTART (TNIBT_VALUE, SMI_TUNIVERSAL);
        t_universal_running = TRUE;
#else
        TIMERSTART (TNIBT_VALUE, t_universal_handle);
#endif
        ui_data->timerMod = TIMER_MOD_NIBT;
        ui_data->nibtOn   = TRUE;
        break;
   }

    wm_showArea  (areaCaller);
  }
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_UI                   |
| STATE   : code                  ROUTINE : ui_backlight             |
+--------------------------------------------------------------------+

  PURPOSE : This function switches the backlight on/off.
*/
LOCAL void ui_backlight (BOOL on)
{
#if defined (NEW_FRAME)
  if (t_backlight_running NEQ FALSE)
  {
    TIMERSTOP (SMI_TBACKLIGHT)
    t_backlight_running = FALSE;
  }
#else
  if (t_backlight_handle NEQ (T_VSI_THANDLE) VSI_ERROR)
  {
    TIMERSTOP (t_backlight_handle)
  }
#endif

  if (on)
  {
#if !defined (NEW_FRAME)
#if defined WIN32
      char buf[30];
      sprintf (buf, "BACKLIGHT %4x", t_backlight_handle);
      TRACE_FUNCTION (buf);
#endif
#endif
#if defined (NEW_FRAME)
    TIMERSTART (TBACKLIGHT_VALUE, SMI_TBACKLIGHT);
    t_backlight_running = TRUE;
#else
    TIMERSTART (TBACKLIGHT_VALUE, t_backlight_handle);
#endif
  }
  if (ui_data->backlightOn NEQ on)
  {
    light_SetStatus (LIGHT_DEVICE_BACKLIGHT,
                     (UBYTE)((on ? LIGHT_STATUS_ON : LIGHT_STATUS_OFF)));
    TRACE_EVENT ((on ? "backlight on" : "backlight off"));
    ui_data->backlightOn = on;
  }
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_UI                   |
| STATE   : code                  ROUTINE : ui_shift                 |
+--------------------------------------------------------------------+

  PURPOSE : This function switches the shiftkey state on/off and
            indicates the state at the LCD.
*/
LOCAL void ui_shift (BOOL on)
{
#if defined (NEW_FRAME)
  if (t_shiftkey_running NEQ FALSE)
  {
    TIMERSTOP (SMI_TSHIFTKEY)
    t_shiftkey_running = FALSE;
  }
#else
  if (t_shiftkey_handle NEQ (T_VSI_THANDLE) VSI_ERROR)
  {
    TIMERSTOP (t_shiftkey_handle)
  }
  if (on)
#endif

  {
#if defined (NEW_FRAME)
    TIMERSTART (TSHIFTKEY_VALUE, SMI_TSHIFTKEY);
    t_shiftkey_running = TRUE;
#else
    TIMERSTART (TSHIFTKEY_VALUE, t_shiftkey_handle);
#endif
  }

  if (ui_data->shiftOn NEQ on)
  {
    TRACE_EVENT ((on ? "shift on" : "shift off"));

    if (on)
    {
      ui_data->keypadMode = 1; /* select the shifted keypad   */
      wm_areaWrite (areaShift, 0, 0, "S");
    }
    else
    {
      ui_data->keypadMode = 0; /* shifted keypad not selected */
      wm_areaWrite (areaShift, 0, 0, " ");

      ui_processDefaultKey (ui_data->lastShiftedKey, KEY_STAT_PRS);
    }

    ui_data->shiftOn = on;
  }
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_UI                   |
| STATE   : code                  ROUTINE : ui_updateKeystate        |
+--------------------------------------------------------------------+

  PURPOSE : This function manages states for each key.
*/
LOCAL UBYTE ui_updateKeystate (UBYTE key, BOOL pressed)
{
  USHORT      i;
  const CHAR *keyString;

  i = 0;

  while (keyTable[i].keypadCode NEQ NULL_TERM AND
         keyTable[i].keypadCode NEQ key)
    i++;

  if (keyTable[i].keypadCode NEQ NULL_TERM)
  {
    keyTable[i].pressed = pressed;
    if (keyTable[i].pressed)
    {
      if (ui_data->keypadMode < MAX_KEYPAD_MODE)
      {
        keyString = keyTable[i].keyCode[ui_data->keypadMode];

        if (keyString EQ NULL)
        {
          /*
           *---------------------------------------------------------
           * if no keycode defined for this key in the actual
           * keypad mode, use the keycode of the normal mode.
           *---------------------------------------------------------
           */
          keyString = keyTable[i].keyCode[0];

        }
      }
      else
        return 0;

      if (ui_data->lastKeyIdx EQ i)
      {
        ui_data->lastKeyRep++;
        ui_data->lastKeyRep %= strlen (keyString);
      }
      else
      {
        ui_data->lastKeyRep = 0;
        ui_data->lastKeyIdx = i;
      }

      if (keyString[ui_data->lastKeyRep] NEQ POWER_KEY)
      {
        if (ui_data->keyBeep)
          cal_sendAudioToneReq (0x26,
                                csf_getVolume (VT_KEY_BEEP),
                                TONE_SWT_ON_SINGLE);

        ui_backlight (TRUE);
      }
      return keyString[ui_data->lastKeyRep];
    }
    else
    {
      UBYTE dtmfKey;

      if (ui_data->sendDTMF)
      {
        keyString = keyTable[i].keyCode[ui_data->keypadMode];

        dtmfKey = keyString[ui_data->lastKeyRep];

        if ((dtmfKey >= '0' AND dtmfKey <= '9') OR
            (dtmfKey >= 'A' AND dtmfKey <= 'D') OR
            (dtmfKey >= 'a' AND dtmfKey <= 'd') OR
             dtmfKey EQ '#' OR  dtmfKey EQ '*')

           return dtmfKey;
      }
    }
  }
  return 0;
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_UI                   |
| STATE   : code                  ROUTINE : ui_getCauseName          |
+--------------------------------------------------------------------+

  PURPOSE : This function returns the text for a cause value.
*/
#if defined (NEW_FRAME)
LOCAL char *ui_getCauseName (char * source, USHORT cause)
{
  static char buffer [40];

  sprintf (buffer, "%s%4x", source, cause);

  return buffer;
}
#else
LOCAL CHAR *ui_getCauseName (BYTE * source, USHORT cause)
{
  static CHAR buffer [40];

  sprintf (buffer, "%s%4x", source, cause);

  return buffer;
}
#endif
/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_UI                   |
| STATE   : code                  ROUTINE : ui_startFSDisplay        |
+--------------------------------------------------------------------+

  PURPOSE : This function starts the level display of fieldstrength.
*/

LOCAL void ui_startFSDisplay (void)
{
  rx_DCB_Type rx_DCB;
  /*
   *-----------------------------------------------------------------
   * request from the PS an indication on change of the
   * rx level with 10 intervalls
   *-----------------------------------------------------------------
   */
  rx_DCB.Steps    = 10;

  rx_SetConfig (&rx_DCB);
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_UI                   |
| STATE   : code                  ROUTINE : ui_startBATDisplay       |
+--------------------------------------------------------------------+

  PURPOSE : This function starts the level display of the battery
            charge.
*/
LOCAL void ui_startBATDisplay (void)
{
  pwr_DCB_Type pwr_DCB;
  /*
   *-----------------------------------------------------------------
   * request from the driver indication of the battery level
   * with 10 intervalls
   *-----------------------------------------------------------------
   */
  pwr_DCB.RangeMin = 10;
  pwr_DCB.RangeMax = 100;
  pwr_DCB.Steps    = 10;

  pwr_SetConfig (&pwr_DCB);
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_UI                   |
| STATE   : code                  ROUTINE : ui_handlePinAndPuk       |
+--------------------------------------------------------------------+

  PURPOSE : This function manages the input of the PIN or PUK and
            initiates further steps.
*/
LOCAL void ui_handlePinAndPuk (void)
{
  CHAR *input;

  switch (ui_data->inputMode)
  {
    case (INP_MOD_PIN):
      /*
       *-------------------------------------------------------------
       * PIN was entered, PIN evaluation process starts
       *-------------------------------------------------------------
       */
      input = inp_getInput (areaUser);
      inp_stopInput (areaUser);

      ui_data->state     = UI_IDLE;
      ui_data->inputMode = INP_MOD_NONE;

      aca_signal (ACA_SIG_PININPUT, input);
      break;

    case (INP_MOD_PUK):
      /*
       *-------------------------------------------------------------
       * PUK was entered, now new PIN is requested
       *-------------------------------------------------------------
       */
      input = inp_getInput (areaUser);
      inp_stopInput (areaUser);

      strcpy (puk, input);

      ui_startNewInput (areaUser,
                        0,
                        0,
                        "PIN: ",
                        MAX_PIN_LEN,
                        IT_NUMERIC | IT_ECHO | IT_HIDDEN,
                        INP_MOD_NPIN,
                        NULL,
                        TRUE);
      break;

    case (INP_MOD_NPIN):
      /*
       *-------------------------------------------------------------
       * new PIN is entered, now new PIN is requested again
       *-------------------------------------------------------------
       */
      input = inp_getInput (areaUser);
      inp_stopInput (areaUser);

      strcpy (pin, input);

      ui_startNewInput (areaUser,
                        0,
                        0,
                        "PIN: ",
                        MAX_PIN_LEN,
                        IT_NUMERIC | IT_ECHO | IT_HIDDEN,
                        INP_MOD_NPIN_TWICE,
                        NULL,
                        TRUE);
      break;

    case (INP_MOD_NPIN_TWICE):
      /*
       *-------------------------------------------------------------
       * new PIN is entered the second time, now PUK evaluation
       * process starts
       *-------------------------------------------------------------
       */
      input = inp_getInput (areaUser);
      inp_stopInput (areaUser);
      ui_data->state     = UI_IDLE;
      ui_data->inputMode = INP_MOD_NONE;
      aca_signal (ACA_SIG_PUKINPUT, puk, pin, input);

      /*
       *-------------------------------------------------------------
       * forget all about PIN and PUK
       *-------------------------------------------------------------
       */
      memset (puk, NULL_TERM, MAX_PUK_LEN + 1);
      memset (pin, NULL_TERM, MAX_PIN_LEN + 1);
      break;
    }
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_UI                   |
| STATE   : code                  ROUTINE : ui_startBooting          |
+--------------------------------------------------------------------+

  PURPOSE : This function prepares for booting.
*/
LOCAL void ui_startBooting (void)
{
  cal_sendAudioToneReq (CT_RING,
                        csf_getVolume (VT_RINGER),
                        TONE_SWT_ON_SINGLE);

#if defined (NEW_FRAME)
  TIMERSTART (TBOOTING_VALUE, SMI_TUNIVERSAL);
  t_universal_running = TRUE;
#else
  TIMERSTART (TBOOTING_VALUE, t_universal_handle);
#endif
  ui_data->timerMod = TIMER_MOD_BOOTING;
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_UI                   |
| STATE   : code                  ROUTINE : ui_init                  |
+--------------------------------------------------------------------+

  PURPOSE : Initialize the user interface module.
*/
GLOBAL void ui_init (void)
{
  USHORT i;

  csf_getPanelInfo (&screensize_x,
                    &screensize_y,
                    &keyboardType);

  ui_data->state          = UI_IDLE;
  ui_data->powerOn        = FALSE;
  ui_data->inputMode      = INP_MOD_NONE;
  ui_data->inputSeq       = INP_SEQ_NONE;
  ui_data->timerMod       = TIMER_MOD_NONE;
  ui_data->ringerOn       = FALSE;
  ui_data->waitingOn      = FALSE;
  ui_data->nibtOn         = FALSE;
  ui_data->sliderActive   = FALSE;
  ui_data->inSubmenu      = FALSE;
  ui_data->menuMode       = MEN_MOD_NONE;
  ui_data->sendDTMF       = FALSE;
  ui_data->keyBeep        = FALSE;
  ui_data->backlightOn    = FALSE;
  ui_data->shiftOn        = FALSE;
  ui_data->lastShiftedKey = NULL_TERM;
  ui_data->lastKeyIdx     = 0xff;
  ui_data->lastKeyRep     = 0;
  ui_data->keypadMode     = 0;
  oldState                = UI_IDLE;

  ui_data->lastDialedNumber[0] = NULL_TERM;

  /*
   *-----------------------------------------------------------------
   * cleanup the states in the keytable
   *-----------------------------------------------------------------
   */
  i=0;
  while (keyTable[i].keypadCode NEQ NULL_TERM)
    keyTable[i++].pressed = FALSE;

  /*
   *-----------------------------------------------------------------
   * initialize window manager, slider, menu and input modules
   *-----------------------------------------------------------------
   */
  wm_init ();
  men_init ();
  inp_init ();
  sld_init ();

  /*
   *-----------------------------------------------------------------
   * create output areas, screen size will be taken into account
   *-----------------------------------------------------------------
   */
  if (screensize_y EQ 4)
  {
    areaCaller    = wm_createArea ( 0,  3,           14,  1);
    areaProvider  = wm_createArea ( 0,  0,           14,  1);
    areaUser      = wm_createArea ( 0,  2,           14,  1);
    areaState     = wm_createArea ( 0,  1,            8,  1);
    areaService   = wm_createArea ( 8,  1, SRVC_WIDTH_L,  1);
    areaFS        = wm_createArea (14,  2,            2,  1);
    areaBAT       = wm_createArea (14,  0,            2,  1);
    areaMenu      = wm_createArea ( 0,  0, MENU_WIDTH_L,  4);
    areaSubMenu   = wm_createArea ( 0,  0,           12,  4);
    areaSlider    = wm_createArea ( 0,  1,           10,  1);
    areaHook      = wm_createArea (15,  3,            1,  1);
    areaShift     = wm_createArea (14,  3,            1,  1);
    areaUniversal = wm_createArea ( 0,  0,           16,  4);
  }
  else if (screensize_y EQ 2)
  {
    areaCaller    = -1;
    areaProvider  = wm_createArea ( 0,  0,            8,  1);
    areaUser      = wm_createArea ( 0,  1,           10,  1);
    areaState     = wm_createArea ( 8,  0,            8,  1);
    areaService   = wm_createArea (10,  1, SRVC_WIDTH_S,  1);
    areaFS        = wm_createArea (12,  1,            1,  1);
    areaBAT       = wm_createArea (14,  1,            1,  1);
    areaMenu      = wm_createArea ( 0,  0, MENU_WIDTH_S,  2);
    areaSubMenu   = wm_createArea ( 0,  0,           12,  2);
    areaSlider    = wm_createArea ( 0,  0,           10,  1);
    areaHook      = wm_createArea (13,  1,            1,  1);
    areaShift     = wm_createArea (15,  1,            1,  1);
    areaUniversal = wm_createArea ( 0,  0,           16,  2);
  }
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_UI                   |
| STATE   : code                  ROUTINE : ui_topArea               |
+--------------------------------------------------------------------+

  PURPOSE : This function is used to top an area.
*/
LOCAL void ui_topArea (USHORT aHandle)
{
  if (ui_data->timerMod EQ TIMER_MOD_NONE)
    wm_topArea (aHandle);
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_UI                   |
| STATE   : code                  ROUTINE : ui_areaServiceWrite      |
+--------------------------------------------------------------------+

  PURPOSE : This function is used to write text to the service area
            with right alignment.
*/
LOCAL void ui_areaServiceWrite (CHAR* text)
{
  UBYTE areaWidth = (screensize_y < 4 ? SRVC_WIDTH_S : SRVC_WIDTH_L);
  UBYTE textLen   = strlen (text);
  UBYTE column;

  if (textLen > 0)
  {
    column = (textLen >= areaWidth ? 0 : areaWidth - textLen);

    wm_clearArea (areaService);
    wm_areaWrite (areaService, 0, column, text);
  }
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_UI                   |
| STATE   : code                  ROUTINE : ui_startNewInput         |
+--------------------------------------------------------------------+

  PURPOSE : This function is used to stop the actual input and start
            a new one.
*/
LOCAL void ui_startNewInput (USHORT    aHandle,
                             UBYTE     row,
                             UBYTE     col,
                             CHAR*     prompt,
                             UBYTE     length,
                             UBYTE     type,
                             T_INP_MOD inputMode,
                             CHAR*     value,
                             BOOL      show)
{
  /*
   *-----------------------------------------------------------------
   * stop the actual input
   *-----------------------------------------------------------------
   */
  inp_stopInput (aHandle);
  wm_clearArea (aHandle);

  /*
   *-----------------------------------------------------------------
   * set a default input string and start a new input
   *-----------------------------------------------------------------
   */
  inp_startInput (aHandle, row, col, prompt, length, type);

  if (value NEQ NULL)
    inp_setInput (aHandle, value);

  ui_data->state     = UI_INPUT;
  ui_data->inputMode = inputMode;

  /*
   *-----------------------------------------------------------------
   * bring the input area on top
   *-----------------------------------------------------------------
   */
  if (show)
  {
    ui_topArea (aHandle);
    wm_showArea (aHandle);
  }
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_UI                   |
| STATE   : code                  ROUTINE : ui_stopMenu              |
+--------------------------------------------------------------------+

  PURPOSE : This function is used to interrupt a given menu.
*/
LOCAL void ui_stopMenu (void)
{
  /*
   *-----------------------------------------------------------------
   * close the submenu
   *-----------------------------------------------------------------
   */
  if (ui_data->inSubmenu)
  {
    men_stopMenu (areaSubMenu);
    wm_hideArea (areaSubMenu);

    ui_data->inSubmenu = FALSE;
  }

  /*
   *-----------------------------------------------------------------
   * close the menu
   *-----------------------------------------------------------------
   */
  men_stopMenu (areaMenu);
  wm_hideArea (areaMenu);

  ui_data->menuMode = MEN_MOD_NONE;
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_UI                   |
| STATE   : code                  ROUTINE : ui_changeVolume          |
+--------------------------------------------------------------------+

  PURPOSE : This function is used to increment or decrement the
            volume.
*/
LOCAL void ui_changeVolume (BOOL inc)
{
  if (!ui_data->sliderActive)
  {
    sld_startSlider (areaSlider, 0, 255, csf_getVolume (VT_AUDIO_OUT));

    ui_data->sliderActive = TRUE;

    ui_topArea  (areaSlider);
    wm_showArea (areaSlider);
  }

#if defined (NEW_FRAME)
  if (t_slider_running NEQ FALSE)
  {
    TIMERSTOP (SMI_TSLIDER)
    t_slider_running = FALSE;
  }
#else
  if (t_slider_handle NEQ VSI_ERROR)
  {
    TIMERSTOP (t_slider_handle)
  }
#endif

#if defined (NEW_FRAME)
  TIMERSTART (TSLIDER_VALUE, SMI_TSLIDER);
  t_slider_running = TRUE;
#else
  TIMERSTART (TSLIDER_VALUE, t_slider_handle);
#endif

  if (inc)
    sld_upSlider (areaSlider);
  else
    sld_downSlider (areaSlider);

  csf_setVolume (VT_AUDIO_OUT,
                 (UBYTE)sld_getSliderValue (areaSlider));
  csf_setVolume (VT_AUDIO_IN,
                 (UBYTE)sld_getSliderValue (areaSlider));

  aca_signal (ACA_SIG_VOLUME, sld_getSliderValue (areaSlider));
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_UI                   |
| STATE   : code                  ROUTINE : ui_signal                |
+--------------------------------------------------------------------+

  PURPOSE : Process the UI_SIG_xxx signals that are send by PSA or IO
*/
GLOBAL void ui_signal (UBYTE sigCode, ...)
{
  va_list varpars;
  CHAR    buf[25];

  sprintf (buf, "UI-Signal: %d", sigCode);
  TRACE_EVENT (buf);

  switch (sigCode)
  {
    /*
     *---------------------------------------------------------------
     * process signal UI_SIG_ENTER_PIN
     *---------------------------------------------------------------
     */
    case UI_SIG_ENTER_PIN:
      ui_startNewInput (areaUser,
                        0,
                        0,
                        "PIN: ",
                        MAX_PIN_LEN,
                        IT_NUMERIC | IT_ECHO | IT_HIDDEN,
                        INP_MOD_PIN,
                        NULL,
                        TRUE);

      ui_data->inputSeq = INP_SEQ_NONE;
      break;

    /*
     *---------------------------------------------------------------
     * process signal UI_SIG_ENTER_PUK
     *---------------------------------------------------------------
     */
    case UI_SIG_ENTER_PUK:
      /*
       *-------------------------------------------------------------
       * forget anything about entered PINs and PUKs
       *-------------------------------------------------------------
       */
      memset (puk, NULL_TERM, MAX_PUK_LEN + 1);
      memset (pin, NULL_TERM, MAX_PIN_LEN + 1);

      ui_startNewInput (areaUser,
                        0,
                        0,
                        "PUK: ",
                        MAX_PUK_LEN,
                        IT_NUMERIC | IT_ECHO | IT_HIDDEN,
                        INP_MOD_PUK,
                        NULL,
                        TRUE);

      ui_data->inputSeq = INP_SEQ_NONE;
      break;

    /*
     *---------------------------------------------------------------
     * process signal UI_SIG_SIM_NOT_INSERT
     *---------------------------------------------------------------
     */
    case UI_SIG_SIM_NOT_INSERT:
      ui_data->normalService = FALSE;

      wm_areaWrite (areaState, 0, 0, lng_getText (TXT_SIM_NOT_INSERT));
      ui_topArea  (areaState);
      wm_showArea (areaState);

      wm_clearArea (areaProvider);

      ui_startNewInput (areaUser,
                        0,
                        0,
                        ">",
                        MAX_INPUT_SIZE - 10,
                        IT_NUMERIC | IT_ECHO,
                        INP_MOD_DIAL,
                        NULL,
                        TRUE);

      ui_data->inputSeq = INP_SEQ_NONE;
      break;

    /*
     *---------------------------------------------------------------
     * process signal UI_SIG_SIM_WRONG
     *---------------------------------------------------------------
     */
    case UI_SIG_SIM_WRONG:
      ui_data->normalService = FALSE;

      wm_areaWrite (areaState, 0, 0, lng_getText (TXT_SIM_WRONG));
      ui_topArea  (areaState);
      wm_showArea (areaState);

      wm_clearArea (areaProvider);

      ui_startNewInput (areaUser,
                        0,
                        0,
                        ">",
                        MAX_INPUT_SIZE - 10,
                        IT_NUMERIC | IT_ECHO,
                        INP_MOD_DIAL,
                        NULL,
                        TRUE);

      ui_data->inputSeq = INP_SEQ_NONE;
      break;

    /*
     *---------------------------------------------------------------
     * process signal UI_SIG_POWER_UP
     *---------------------------------------------------------------
     */
    case UI_SIG_POWER_UP:
      {
        UBYTE row    = 0;
        UBYTE column = 1;

        ui_init ();

        /*
         *-----------------------------------------------------------
         * build up the screen the first time
         *-----------------------------------------------------------
         */
        ui_backlight (TRUE);
        ui_startFSDisplay  ();
        ui_startBATDisplay ();
        wm_areaWrite (areaHook, 0, 0, "_");
        wm_showArea (areaProvider);
        wm_showArea (areaService);
        wm_showArea (areaUser);
        wm_showArea (areaBAT);
        wm_showArea (areaFS);
        wm_showArea (areaState);
        wm_showArea (areaShift);

        if (screensize_y EQ 4)
          wm_showArea (areaCaller);

        wm_showArea (areaHook);
        ui_data->powerOn = TRUE;

        /*
         *-----------------------------------------------------------
         * enable input before camp on a network
         *-----------------------------------------------------------
         */
        ui_startNewInput (areaUser,
                          0,
                          0,
                          ">",
                          MAX_INPUT_SIZE - 10,
                          IT_NUMERIC | IT_ECHO,
                          INP_MOD_DIAL,
                          NULL,
                          TRUE);

        ui_data->inputSeq = INP_SEQ_NONE;

        /*
         *-----------------------------------------------------------
         * show the welcome window
         *-----------------------------------------------------------
         */
        if (screensize_y EQ 4)
        {
          row    += 1;
/*
          column += 2;
*/
        }

        wm_areaWrite (areaUniversal, row, column, "WELCOME TO THE");
        wm_areaWrite (areaUniversal,
                      (UBYTE)(row + 1), (UBYTE)(column + 1),
                      "WORLD OF GSM");
        ui_topArea   (areaUniversal);
        wm_showArea  (areaUniversal);

#if defined (NEW_FRAME)
        TIMERSTART (TWELCOME_VALUE, SMI_TUNIVERSAL);
        t_universal_running = TRUE;
#else
        TIMERSTART (TWELCOME_VALUE, t_universal_handle);
#endif
        ui_data->timerMod = TIMER_MOD_WELCOME;

        /*
         *-----------------------------------------------------------
         * request RX level indications
         *-----------------------------------------------------------
         */
        rx_Init(rx_callback);
      }
      break;

    /*
     *---------------------------------------------------------------
     * process signal UI_SIG_POWER_DOWN
     *---------------------------------------------------------------
     */
    case UI_SIG_POWER_DOWN:
#if defined (NEW_FRAME)
      if (t_slider_running NEQ FALSE)
      {
        TIMERSTOP (SMI_TSLIDER);
        t_slider_running = FALSE;
      }

      if (t_universal_running NEQ FALSE)
      {
        TIMERSTOP (SMI_TUNIVERSAL);
        t_universal_running = FALSE;

        wm_hideArea (areaUniversal);
      }
#else
      if (t_slider_handle NEQ (T_VSI_THANDLE) VSI_ERROR)
      {
        TIMERSTOP (t_slider_handle);
      }

      if (t_universal_handle NEQ (T_VSI_THANDLE) VSI_ERROR)
      {
        TIMERSTOP (t_universal_handle);

        wm_hideArea (areaUniversal);
      }
#endif
      /*
       *-------------------------------------------------------------
       * switch off the LCD backlight and clean the LCD display
       *-------------------------------------------------------------
       */
      ui_backlight (FALSE);
      wm_hideArea (areaProvider);
      wm_hideArea (areaService);
      wm_hideArea (areaState);
      wm_hideArea (areaFS);
      wm_hideArea (areaBAT);
      wm_hideArea (areaUser);
      wm_hideArea (areaShift);

      if (screensize_y EQ 4)
        wm_hideArea (areaCaller);

      wm_hideArea (areaSlider);
      wm_hideArea (areaHook);

      ui_data->powerOn = FALSE;
      powerKeyOn       = FALSE;

      /*
      *--------------------------------------------------------------
       * stop any tones
       *-------------------------------------------------------------
       */
      audio_Exit ();

      /*
       *-------------------------------------------------------------
       * stop RX level indications
       *-------------------------------------------------------------
       */
      rx_Exit ();

      break;

    /*
     *---------------------------------------------------------------
     * process signal UI_SIG_PLMN
     *---------------------------------------------------------------
     */
    case UI_SIG_PLMN:
    {
      CHAR* oper;

      va_start (varpars, sigCode);
      oper = va_arg (varpars, CHAR*);
      va_end (varpars);

      wm_clearArea (areaProvider);
      wm_areaWrite (areaProvider, 0, 0, oper);

      ui_areaServiceWrite (lng_getText (TXT_NORMAL_SERVICE));

      ui_data->normalService = TRUE;

      break;
    }

    /*
     *---------------------------------------------------------------
     * process signal UI_SIG_SELECT_PLMN
     *---------------------------------------------------------------
     */
    case UI_SIG_SELECT_PLMN:
    {
      CHAR** oper;
      USHORT i;

      va_start (varpars, sigCode);
      oper = va_arg (varpars, CHAR**);
      va_end (varpars);

      if (ui_data->state NEQ UI_SELECT)
      {
        i = 0;

        if (oper[i] EQ NULL)
        {
          /*
           *---------------------------------------------------------
           * the PLMN menu is empty
           *---------------------------------------------------------
           */
          plmnIsEmpty = TRUE;
          plmnmenu[i] = lng_getText (TXT_EMPTY);
          i++;
        }
        else
        {
          /*
           *---------------------------------------------------------
           * the PLMN menu is initialized while not empty
           *---------------------------------------------------------
           */
          plmnIsEmpty = FALSE;

          while (oper[i] NEQ NULL)
          {
            plmnmenu[i] = oper[i];
            i++;
          }
        }

        /*
         *-----------------------------------------------------------
         * the PLMN menu is terminated
         *-----------------------------------------------------------
         */
        plmnmenu[i] = NULL;

        /*
         *-----------------------------------------------------------
         * show the PLMN menu
         *-----------------------------------------------------------
         */
        ui_topArea (areaMenu);
        wm_showArea (areaMenu);
        men_startMenu (areaMenu,
                       lng_getText (TXT_MEN_SELECT_PLMN),
                       plmnmenu);

        oldState          = ui_data->state;
        ui_data->state    = UI_SELECT;
        ui_data->menuMode = MEN_MOD_PLMN;
      }
      break;
    }

    /*
     *---------------------------------------------------------------
     * process signal UI_SIG_SHOW_CALL_TABLE
     *---------------------------------------------------------------
     */
    case UI_SIG_SHOW_CALL_TABLE:
    {
      T_CTB_INFO* ctb;
      USHORT i;

      va_start (varpars, sigCode);
      ctb = va_arg (varpars, T_CTB_INFO*);
      va_end (varpars);

      if (ui_data->state NEQ UI_SELECT)
      {
        i = 0;

        if (ctb[i].index EQ -1)
        {
          /*
           *---------------------------------------------------------
           * the call table is empty
           *---------------------------------------------------------
           */
          CHAR* empty = lng_getText (TXT_EMPTY);

          strncpy (ctbstring[i], empty, MENU_ENTRY_WIDTH_L);
          ctbstring[i][MENU_ENTRY_WIDTH_L] = NULL_TERM;
          i++;
        }
        else
        {
          /*
           *---------------------------------------------------------
           * the call table menu entries are build up
           *---------------------------------------------------------
           */
          while (ctb[i].index NEQ -1)
          {
            CHAR* status;

            switch (ctb[i].status)
            {
              case (CAL_STAT_Wait):
                status = lng_getStatText(STAT_TXT_WAIT);
                break;

              case (CAL_STAT_Active):
                status = lng_getStatText(STAT_TXT_ACTIVE);
                break;

              case (CAL_STAT_Held):
                status = lng_getStatText(STAT_TXT_HELD);
                break;

              case (CAL_STAT_Dial):
                status = lng_getStatText(STAT_TXT_DIAL);
                break;
            }

            sprintf (ctbstring[i], "%d %s", ctb[i].index, status);
            i++;
          }
        }

        /*
         *---------------------------------------------------------
         * the list of call table menu entries is terminated
         *---------------------------------------------------------
         */
        ctbstring[i][0] = NULL_TERM;

        /*
         *-----------------------------------------------------------
         * the call table menu is initialized and terminated
         *-----------------------------------------------------------
         */
        i = 0;

        while (ctbstring[i][0] NEQ NULL_TERM)
        {
          ctbmenu[i] = ctbstring[i];
          i++;
        }

        ctbmenu[i] = NULL;

        /*
         *-----------------------------------------------------------
         * show the call table menu
         *-----------------------------------------------------------
         */
        ui_topArea (areaMenu);
        wm_showArea (areaMenu);
        men_startMenu (areaMenu,
                       lng_getText (TXT_MEN_CALL_TABLE),
                       ctbmenu);

        oldState          = ui_data->state;
        ui_data->state    = UI_SELECT;
        ui_data->menuMode = MEN_MOD_CTB;
      }
      break;
    }

    /*
     *---------------------------------------------------------------
     * process signal UI_SIG_ILLEGAL_MS
     *---------------------------------------------------------------
     */
    case UI_SIG_ILLEGAL_MS:
      wm_clearArea (areaUser);
      wm_areaWrite (areaUser, 0, 0, "####");

      ui_data->state = UI_BLOCKED;
      break;

    /*
     *---------------------------------------------------------------
     * process signal UI_SIG_NO_CELL
     *---------------------------------------------------------------
     */
    case UI_SIG_NO_CELL:
      ui_data->normalService = FALSE;

      wm_clearArea (areaProvider);

      ui_areaServiceWrite (lng_getText (TXT_NO_SERVICE));

/*
      cal_sendAudioToneReq (CT_CALL_DROPPED,
                            csf_getVolume (VT_RINGER),
                            TONE_SWT_ON_SINGLE);

*/
      break;

    /*
     *---------------------------------------------------------------
     * process signal UI_SIG_LIMITED_SERVICE
     *---------------------------------------------------------------
     */
    case UI_SIG_LIMITED_SERVICE:
      ui_data->normalService = FALSE;

      wm_clearArea (areaProvider);

      ui_areaServiceWrite (lng_getText (TXT_LIMITD_SERVICE));

/*
      ui_startNewInput (areaUser,
                        0,
                        0,
                        ">",
                        MAX_INPUT_SIZE - 10,
                        IT_NUMERIC | IT_ECHO,
                        INP_MOD_DIAL,
                        NULL,
                        TRUE);

      ui_data->inputSeq = INP_SEQ_NONE;
*/
      break;

    /*
     *---------------------------------------------------------------
     * process signal UI_SIG_INCOMMING_CALL
     *---------------------------------------------------------------
     */
    case UI_SIG_INCOMMING_CALL:
      ui_backlight (TRUE);

      /*
       *-------------------------------------------------------------
       * switch on the ringer
       *-------------------------------------------------------------
       */
      if (!ui_data->ringerOn)
      {
        cal_sendAudioToneReq (CT_RING,
                              csf_getVolume (VT_RINGER),
                              TONE_SWT_ON_PERIODIC);

        ui_data->ringerOn = TRUE;
      }

      /*
       *-------------------------------------------------------------
       * make an optic indication of the incoming call
       *-------------------------------------------------------------
       */
      wm_clearArea (areaState);
      wm_areaWrite (areaState, 0, 0, lng_getText (TXT_ALERT));
      wm_showArea (areaState);

      /*
       *-------------------------------------------------------------
       * close any menu while a call comes in
       *-------------------------------------------------------------
       */
      if (ui_data->state EQ UI_SELECT)
        ui_stopMenu ();

      /*
       *-------------------------------------------------------------
       * reset the input while a call comes in
       *-------------------------------------------------------------
       */
      ui_startNewInput (areaUser,
                        0,
                        0,
                        ">",
                        MAX_INPUT_SIZE - 10,
                        IT_NUMERIC | IT_ECHO,
                        INP_MOD_DIAL,
                        NULL,
                        TRUE);

      ui_data->inputSeq = INP_SEQ_NONE;
      break;

    /*
     *---------------------------------------------------------------
     * process signal UI_SIG_WAITING_CALL
     *---------------------------------------------------------------
     */
    case UI_SIG_WAITING_CALL:
    {
      CHAR* caller;

      ui_backlight (TRUE);

      /*
       *-------------------------------------------------------------
       * switch on an acoustic indication of the waiting call
       *-------------------------------------------------------------
       */
      if (!ui_data->waitingOn)
      {
        cal_sendAudioToneReq (CT_CALL_WAIT,
                              csf_getVolume (VT_RINGER),
                              TONE_SWT_ON_PERIODIC);

        ui_data->waitingOn = TRUE;
      }

      /*
       *-------------------------------------------------------------
       * make an optic indication of the waiting call
       *-------------------------------------------------------------
       */
      wm_clearArea (areaState);
      wm_areaWrite (areaState, 0, 0, lng_getText (TXT_WAITING_CALL));
      wm_showArea (areaState);

      /*
       *-------------------------------------------------------------
       * show the caller if screen is large enough
       *-------------------------------------------------------------
       */
      if (screensize_y EQ 4)
      {
        va_start (varpars, sigCode);
        caller = va_arg (varpars, CHAR*);
        va_end (varpars);

        wm_clearArea (areaCaller);
        wm_areaWrite (areaCaller, 0, 0, caller);
        wm_showArea  (areaCaller);
      }

      /*
       *-------------------------------------------------------------
       * close any menu while a call is waiting
       *-------------------------------------------------------------
       */
      if (ui_data->state EQ UI_SELECT)
        ui_stopMenu ();

      /*
       *-------------------------------------------------------------
       * reset the input while a call is waiting
       *-------------------------------------------------------------
       */
      ui_startNewInput (areaUser,
                        0,
                        0,
                        ">",
                        MAX_INPUT_SIZE - 10,
                        IT_NUMERIC | IT_ECHO,
                        INP_MOD_DIAL,
                        NULL,
                        TRUE);

      ui_data->inputSeq = INP_SEQ_NONE;
      break;
    }

    /*
     *---------------------------------------------------------------
     * process signal UI_SIG_BUSY
     *---------------------------------------------------------------
     */
    case UI_SIG_BUSY:
    {
      ui_backlight (TRUE);

      /*
       *-------------------------------------------------------------
       * switch on an acoustic indication of the busy subscriber
       *-------------------------------------------------------------
       */
      if (!ui_data->nibtOn)
      {
        ui_handleNibt(NIBT_BUSY);
/*
        cal_sendAudioToneReq (CT_BUSY,
                              csf_getVolume (VT_RINGER),
                              TONE_SWT_ON_PERIODIC);
*/
        ui_data->nibtOn = TRUE;
      }

      /*
       *-------------------------------------------------------------
       * make an optic indication of the busy indication
       *-------------------------------------------------------------
       */
      wm_clearArea (areaState);
      wm_areaWrite (areaState, 0, 0, lng_getText (TXT_USER_BUSY));
      wm_showArea (areaState);

      break;
    }

    /*
     *---------------------------------------------------------------
     * process signal UI_SIG_NO_ANSWER
     *---------------------------------------------------------------
     */
    case UI_SIG_NO_ANSWER:
    {
      ui_backlight (TRUE);

      /*
       *-------------------------------------------------------------
       * make an optic indication when subscriber does not answer
       *-------------------------------------------------------------
       */
      wm_clearArea (areaState);
      wm_areaWrite (areaState, 0, 0, lng_getText (TXT_NO_ANSWER));
      wm_showArea (areaState);

      break;
    }

    /*
     *---------------------------------------------------------------
     * process signal UI_SIG_CLIP
     *---------------------------------------------------------------
     */
    case UI_SIG_CLIP:
    {
      CHAR*  alpha_calladdr;
      CHAR*  subaddr;
      CHAR   callInfo[60];

      ui_backlight (TRUE);

      /*
       *-------------------------------------------------------------
       * make an optic indication of the CLIP
       *-------------------------------------------------------------
       */
      wm_clearArea (areaState);
      wm_areaWrite (areaState, 0, 0, "CLIP");
      wm_showArea (areaState);

      /*
       *-------------------------------------------------------------
       * show the calling line identity if screen is large enough
       *-------------------------------------------------------------
       */
      if (screensize_y EQ 4)
      {
        va_start (varpars, sigCode);
        alpha_calladdr = va_arg (varpars, CHAR*);
        subaddr        = va_arg (varpars, CHAR*);
        va_end (varpars);

        if (strlen (subaddr) > 0)
          sprintf (callInfo, "%s-%s", alpha_calladdr, subaddr);
        else
          strcpy (callInfo, alpha_calladdr);

        wm_clearArea (areaCaller);
        wm_areaWrite (areaCaller, 0, 0, callInfo);
        wm_showArea  (areaCaller);
      }

      break;
    }

    /*
     *---------------------------------------------------------------
     * process signal UI_SIG_COLP
     *---------------------------------------------------------------
     */
    case UI_SIG_COLP:
    {
      CHAR*  alpha_calladdr;
      CHAR*  subaddr;
      CHAR   callInfo[60];

      ui_backlight (TRUE);

      /*
       *-------------------------------------------------------------
       * make an optic indication of the COLP
       *-------------------------------------------------------------
       */
      wm_clearArea (areaState);
      wm_areaWrite (areaState, 0, 0, "COLP");
      wm_showArea (areaState);

      /*
       *-------------------------------------------------------------
       * show the connected line identity if the screen is large
       * enough
       *-------------------------------------------------------------
       */
      if (screensize_y EQ 4)
      {
        va_start (varpars, sigCode);
        alpha_calladdr = va_arg (varpars, CHAR*);
        subaddr        = va_arg (varpars, CHAR*);
        va_end (varpars);

        if (strlen (subaddr) > 0)
          sprintf (callInfo, "%s-%s", alpha_calladdr, subaddr);
        else
          strcpy (callInfo, alpha_calladdr);

        wm_clearArea (areaCaller);
        wm_areaWrite (areaCaller, 0, 0, callInfo);
        wm_showArea  (areaCaller);
      }

      break;
    }

    /*
     *---------------------------------------------------------------
     * process signal UI_SIG_OUTGOING_CALL
     *---------------------------------------------------------------
     */
    case UI_SIG_OUTGOING_CALL:
    {
      CHAR *calledParty;

      va_start (varpars, sigCode);
      calledParty = va_arg (varpars, CHAR*);
      va_end (varpars);

      /*
       *-------------------------------------------------------------
       * make an optic indication of dialing
       *-------------------------------------------------------------
       */
      wm_clearArea (areaState);
      wm_areaWrite (areaState, 0, 0, lng_getText (TXT_DIALING));
      wm_showArea (areaState);

      /*
       *-------------------------------------------------------------
       * display the called party if screen is large enough
       *-------------------------------------------------------------
       */
      if (screensize_y EQ 4)
      {
        wm_clearArea (areaCaller);

        wm_areaWrite (areaCaller, 0, 0, calledParty);
        wm_showArea (areaCaller);
      }
      break;
    }

    /*
     *---------------------------------------------------------------
     * process signal UI_SIG_REGISTERED
     *---------------------------------------------------------------
     */
    case UI_SIG_REGISTERED:
    {
      wm_clearArea (areaState);

/*
      ui_areaServiceWrite (lng_getText (TXT_NORMAL_SERVICE));

      ui_data->normalService = TRUE;
*/

      if (ui_data->inputMode NEQ INP_MOD_PIN        AND
          ui_data->inputMode NEQ INP_MOD_NPIN       AND
          ui_data->inputMode NEQ INP_MOD_NPIN_TWICE AND
          ui_data->inputMode NEQ INP_MOD_PUK           )
      {
        ui_startNewInput (areaUser,
                          0,
                          0,
                          ">",
                          MAX_INPUT_SIZE - 10,
                          IT_NUMERIC | IT_ECHO,
                          INP_MOD_DIAL,
                          NULL,
                          TRUE);

        ui_data->inputSeq = INP_SEQ_NONE;
      }
      break;
    }

    /*
     *---------------------------------------------------------------
     * process signal UI_SIG_REGISTERING
     *---------------------------------------------------------------
     */
    case UI_SIG_REGISTERING:
      wm_clearArea (areaState);
      wm_areaWrite (areaState, 0, 0, lng_getText (TXT_REGISTERING));
      break;

    /*
     *---------------------------------------------------------------
     * process signal UI_SIG_DEREGISTERING
     *---------------------------------------------------------------
     */
    case UI_SIG_DEREGISTERING:
      wm_clearArea (areaState);
      wm_areaWrite (areaState, 0, 0, lng_getText (TXT_DEREGISTERING));
      break;

    /*
     *---------------------------------------------------------------
     * process signal UI_SIG_CONNECTING
     *---------------------------------------------------------------
     */
    case UI_SIG_CONNECTING:
      wm_clearArea (areaState);
      wm_areaWrite (areaState, 0, 0, lng_getText (TXT_CONNECTING));
      break;

    /*
     *---------------------------------------------------------------
     * process signal UI_SIG_DISCONNECTING
     *---------------------------------------------------------------
     */
    case UI_SIG_DISCONNECTING:
      wm_clearArea (areaState);
      wm_areaWrite (areaState, 0, 0, lng_getText (TXT_DISCONNECTING));
      break;

    /*
     *---------------------------------------------------------------
     * process signal UI_SIG_CONNECTED
     *---------------------------------------------------------------
     */
    case UI_SIG_CONNECTED:
      /*
       *-------------------------------------------------------------
       * switch off the ringer
       *-------------------------------------------------------------
       */
      if (ui_data->ringerOn)
      {
        cal_sendAudioToneReq (CT_RING,
                              csf_getVolume (VT_RINGER),
                              TONE_SWT_OFF);

        ui_data->ringerOn = FALSE;
      }

      /*
       *-------------------------------------------------------------
       * make an optic indication
       *-------------------------------------------------------------
       */
      wm_areaWrite (areaHook, 0, 0, "/");

      wm_clearArea (areaState);
      wm_areaWrite (areaState, 0, 0, lng_getText (TXT_CONNECTED));
      break;

    /*
     *---------------------------------------------------------------
     * process signal UI_SIG_DATA_CONNECTED
     *---------------------------------------------------------------
     */
    case UI_SIG_DATA_CONNECTED:
    {
      CHAR   buffer[15];
      USHORT speed;

      va_start (varpars, sigCode);
      speed = va_arg (varpars, USHORT);
      va_end (varpars);

      /*
       *-------------------------------------------------------------
       * switch off the ringer
       *-------------------------------------------------------------
       */
      if (ui_data->ringerOn)
      {
        cal_sendAudioToneReq (CT_RING,
                              csf_getVolume (VT_RINGER),
                              TONE_SWT_OFF);

        ui_data->ringerOn = FALSE;
      }

      /*
       *-------------------------------------------------------------
       * make an optic indication
       *-------------------------------------------------------------
       */
      wm_areaWrite (areaHook, 0, 0, "/");

      sprintf (buffer, "DTA%5d", speed);
      wm_clearArea (areaState);
      wm_areaWrite (areaState, 0, 0, buffer);
      break;
    }

    /*
     *---------------------------------------------------------------
     * process signal UI_SIG_RING_OFF
     *---------------------------------------------------------------
     */
    case UI_SIG_RING_OFF:
    {
      /*
       *-------------------------------------------------------------
       * switch off the ringer
       *-------------------------------------------------------------
       */
      if (ui_data->ringerOn)
      {
        cal_sendAudioToneReq (CT_RING,
                              csf_getVolume (VT_RINGER),
                              TONE_SWT_OFF);

        ui_data->ringerOn = FALSE;
      }
      break;
    }

    /*
     *---------------------------------------------------------------
     * process signal UI_SIG_DISCONNECT
     *---------------------------------------------------------------
     */
    case UI_SIG_DISCONNECT:
    {
      USHORT cause;

      va_start (varpars, sigCode);
      cause = va_arg (varpars, USHORT);
      va_end (varpars);

      /*
       *-------------------------------------------------------------
       * switch off the ringer
       *-------------------------------------------------------------
       */
      if (ui_data->ringerOn)
      {
        cal_sendAudioToneReq (CT_RING,
                              csf_getVolume (VT_RINGER),
                              TONE_SWT_OFF);

        ui_data->ringerOn = FALSE;
      }

      /*
       *-------------------------------------------------------------
       * make an optic indication
       *-------------------------------------------------------------
       */
      if (GET_CAUSE_VALUE(cause) NEQ NOT_PRESENT_8BIT)
      {
        wm_clearArea (areaState);
        wm_areaWrite (areaState, 0, 0,
                      ui_getCauseName ("DISC", cause));
      }

      if (screensize_y EQ 4)
      {
        wm_hideArea (areaCaller);
        wm_clearArea (areaCaller);
      }

      wm_areaWrite (areaHook, 0, 0, "_");

      if (ui_data->normalService)
        ui_areaServiceWrite (lng_getText (TXT_NORMAL_SERVICE));
      else
        ui_areaServiceWrite (lng_getText (TXT_LIMITD_SERVICE));

      /*
       *-------------------------------------------------------------
       * update keypad and DTMF state
       *-------------------------------------------------------------
       */
      if (ui_data->sendDTMF)
      {
        ui_data->keypadMode = 0;
        ui_data->sendDTMF   = FALSE;
      }

      break;
    }

    /*
     *---------------------------------------------------------------
     * process signal UI_SIG_RXLEV
     *---------------------------------------------------------------
     */
    case UI_SIG_RXLEV:
    {
      CHAR  buf[10];
      UBYTE level;

      va_start (varpars, sigCode);
      level = va_arg (varpars, UBYTE);
      va_end (varpars);

      if (screensize_y < 4)
        sprintf (buf, FIELDSTRENGTH_FORMAT_S, level);
      else
        sprintf (buf, FIELDSTRENGTH_FORMAT_L, level);

      wm_areaWrite (areaFS, 0, 0, buf);

      break;
    }

    /*
     *---------------------------------------------------------------
     * process signal UI_SIG_BATLEV
     *---------------------------------------------------------------
     */
    case UI_SIG_BATLEV:
    {
      CHAR  buf[10];
      UBYTE level;

      va_start (varpars, sigCode);
      level = va_arg (varpars, UBYTE);
      va_end (varpars);

      if (screensize_y < 4)
        sprintf (buf, BATTLEVEL_FORMAT_S, level);
      else
        sprintf (buf, BATTLEVEL_FORMAT_L, level);

      wm_areaWrite (areaBAT, 0, 0, buf);

      break;
    }

    /*
     *---------------------------------------------------------------
     * process signal UI_SIG_NIBT
     *---------------------------------------------------------------
     */
    case UI_SIG_NIBT:
    {
      UBYTE nibt;

      va_start (varpars, sigCode);
      nibt = va_arg (varpars, UBYTE);
      va_end (varpars);

      ui_handleNibt (nibt);

      break;
    }

    /*
     *---------------------------------------------------------------
     * process signal UI_SIG_SS
     *---------------------------------------------------------------
     */
    case UI_SIG_SS:
    {
      USHORT cause;

      va_start (varpars, sigCode);
      cause = va_arg (varpars, USHORT);
      va_end (varpars);

      wm_clearArea (areaState);
      wm_areaWrite (areaState, 0, 0,
                    ui_getCauseName ("SS", cause));
      break;
    }

    /*
     *---------------------------------------------------------------
     * process signal UI_SIG_SEC
     *---------------------------------------------------------------
     */
    case UI_SIG_SEC:
    {
      USHORT cause;

      va_start (varpars, sigCode);
      cause = va_arg (varpars, USHORT);
      va_end (varpars);

      wm_clearArea (areaState);
      wm_areaWrite (areaState, 0, 0,
                    ui_getCauseName ("SEC", cause));
      break;
    }

    /*
     *---------------------------------------------------------------
     * process signal UI_SIG_SSWC
     *---------------------------------------------------------------
     */
    case UI_SIG_SSWC:
    {
      USHORT cause;

      va_start (varpars, sigCode);
      cause = va_arg (varpars, USHORT);
      va_end (varpars);

      if (ui_data->waitingOn AND !cal_isCallWaiting())
      {
        cal_sendAudioToneReq (CT_CALL_WAIT,
                              csf_getVolume (VT_RINGER),
                              TONE_SWT_OFF);

        ui_data->waitingOn = FALSE;
      }

      wm_clearArea (areaState);
      wm_areaWrite (areaState, 0, 0,
                    ui_getCauseName ("SSWC", cause));
      break;
    }

    /*
     *---------------------------------------------------------------
     * process signal UI_SIG_UNHND_CME_ERR
     *---------------------------------------------------------------
     */
    case UI_SIG_UNHND_CME_ERR:
    {
      USHORT cause;

      va_start (varpars, sigCode);
      cause = va_arg (varpars, USHORT);
      va_end (varpars);

      /*
       *-------------------------------------------------------------
       * close any menu while an error occured
       *-------------------------------------------------------------
       */
      if (ui_data->state EQ UI_SELECT)
        ui_stopMenu ();

      /*
       *-------------------------------------------------------------
       * reset the input while an error occured
       *-------------------------------------------------------------
       */
      ui_startNewInput (areaUser,
                        0,
                        0,
                        ">",
                        MAX_INPUT_SIZE - 10,
                        IT_NUMERIC | IT_ECHO,
                        INP_MOD_DIAL,
                        NULL,
                        TRUE);

      ui_data->inputSeq = INP_SEQ_NONE;

      /*
       *-------------------------------------------------------------
       * make an optic indication of the error
       *-------------------------------------------------------------
       */
      wm_clearArea (areaState);
      wm_areaWrite (areaState, 0, 0,
                    ui_getCauseName ("EERR", cause));
      break;
    }

    /*
     *---------------------------------------------------------------
     * process signal UI_SIG_UNHND_CMS_ERR
     *---------------------------------------------------------------
     */
    case UI_SIG_UNHND_CMS_ERR:
    {
      USHORT cause;

      va_start (varpars, sigCode);
      cause = va_arg (varpars, USHORT);
      va_end (varpars);

      /*
       *-------------------------------------------------------------
       * close any menu while an error occured
       *-------------------------------------------------------------
       */
      if (ui_data->state EQ UI_SELECT)
        ui_stopMenu ();

      /*
       *-------------------------------------------------------------
       * reset the input while an error occured
       *-------------------------------------------------------------
       */
      ui_startNewInput (areaUser,
                        0,
                        0,
                        ">",
                        MAX_INPUT_SIZE - 10,
                        IT_NUMERIC | IT_ECHO,
                        INP_MOD_DIAL,
                        NULL,
                        TRUE);

      ui_data->inputSeq = INP_SEQ_NONE;

      /*
       *-------------------------------------------------------------
       * make an optic indication of the error
       *-------------------------------------------------------------
       */
      wm_clearArea (areaState);
      wm_areaWrite (areaState, 0, 0,
                    ui_getCauseName ("SERR", cause));
      break;
    }
  }
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_UI                   |
| STATE   : code                  ROUTINE : bt_itoa                  |
+--------------------------------------------------------------------+

  PURPOSE :
*/

#ifdef BT_ADAPTER
LOCAL UBYTE bt_itoa ( char data )
{
  if ( data >= 0x00 AND data <= 0x09)
    return (data + '0');
  if ( data >= 0x0A AND data <= 0x0F)
    return (data + 'A' - 0x0a);
}
#endif /* BT_ADAPTER */


/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_UI                   |
| STATE   : code                  ROUTINE : convert_bd_addr          |
+--------------------------------------------------------------------+

  PURPOSE :
*/

#ifdef BT_ADAPTER
LOCAL void convert_bd_addr ( UBYTE data[BTI_BD_ADDR_LENGTH] )
{
  int i = 0;
  int j = 0;

  while (i<BTI_BD_ADDR_LENGTH)
  {

    bd_addr_data[j]  = ( bt_itoa ( (data[i] >> 4) & 0x0F ));
    j++;
    bd_addr_data[j]  = (bt_itoa ( data[i] & 0x0F ));
    j++;
    i++;
  }
}
#endif /* BT_ADAPTER */


/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_UI                   |
| STATE   : code                  ROUTINE : ui_bt                    |
+--------------------------------------------------------------------+

  PURPOSE : This is used by display Bluetooth related information on
      the display
*/

#ifdef BT_ADAPTER
GLOBAL void ui_bt(T_UI_BT_MES id, UBYTE data[BTI_BD_ADDR_LENGTH])
{
  UBYTE row = 0;
  UBYTE column = 0;
  switch (id)
  {
  case UI_BT_UNKNOWN_DEVICE:

        wm_areaWrite (areaUniversal, row, column, "BLUETOOTH       ");
        wm_areaWrite (areaUniversal, (UBYTE)(row+1), column, "ERROR           ");
        wm_areaWrite (areaUniversal, (UBYTE)(row+2), column, "UNKNOWN PROFILE ");
        wm_areaWrite (areaUniversal, (UBYTE)(row+3), column, "                ");
        ui_topArea   (areaUniversal);
        wm_showArea  (areaUniversal);

#if defined (NEW_FRAME)
        TIMERSTART (TBT_VALUE, SMI_TUNIVERSAL);
        t_universal_running = TRUE;
#else
        TIMERSTART (TBT_VALUE, t_universal_handle);
#endif
        ui_data->timerMod = TIMER_MOD_BT;
    break;


  case UI_BT_HS_INIT_SUCCES:
        wm_areaWrite (areaUniversal, row, column, "BLUETOOTH       ");
        wm_areaWrite (areaUniversal, (UBYTE)(row+1), column, "Headset Profile ");
        wm_areaWrite (areaUniversal, (UBYTE)(row+2), column, "initialized     ");
        wm_areaWrite (areaUniversal, (UBYTE)(row+3), column, "                ");
        ui_topArea   (areaUniversal);
        wm_showArea  (areaUniversal);

#if defined (NEW_FRAME)
        TIMERSTART (TBT_VALUE, SMI_TUNIVERSAL);
        t_universal_running = TRUE;
#else
        TIMERSTART (TBT_VALUE, t_universal_handle);
#endif
        ui_data->timerMod = TIMER_MOD_BT;
    break;


  case UI_BT_HS_INIT_ERROR:

        wm_areaWrite (areaUniversal, row, column, "BLUETOOTH       ");
        wm_areaWrite (areaUniversal, (UBYTE)(row+1), column, "Headset Profile ");
        wm_areaWrite (areaUniversal, (UBYTE)(row+2), column, "init ERROR      ");
        wm_areaWrite (areaUniversal, (UBYTE)(row+3), column, "                ");
        ui_topArea   (areaUniversal);
        wm_showArea  (areaUniversal);

#if defined (NEW_FRAME)
        TIMERSTART (TBT_VALUE, SMI_TUNIVERSAL);
        t_universal_running = TRUE;
#else
        TIMERSTART (TBT_VALUE, t_universal_handle);
#endif
        ui_data->timerMod = TIMER_MOD_BT;
    break;

  case UI_BT_DU_INIT_SUCCES:

        wm_areaWrite (areaUniversal, row, column, "BLUETOOTH       ");
        wm_areaWrite (areaUniversal, (UBYTE)(row+1), column, "Dial-up Profile");
        wm_areaWrite (areaUniversal, (UBYTE)(row+2), column, "initialized    ");
        wm_areaWrite (areaUniversal, (UBYTE)(row+3), column, "                ");
        ui_topArea   (areaUniversal);
        wm_showArea  (areaUniversal);

#if defined (NEW_FRAME)
        TIMERSTART (TBT_VALUE, SMI_TUNIVERSAL);
        t_universal_running = TRUE;
#else
        TIMERSTART (TBT_VALUE, t_universal_handle);
#endif
        ui_data->timerMod = TIMER_MOD_BT;
    break;

  case UI_BT_DU_INIT_ERROR:

        wm_areaWrite (areaUniversal, row, column, "BLUETOOTH       ");
        wm_areaWrite (areaUniversal, (UBYTE)(row+1), column, "Dial-up Profile");
        wm_areaWrite (areaUniversal, (UBYTE)(row+2), column, "init ERROR     ");
        wm_areaWrite (areaUniversal, (UBYTE)(row+3), column, "                ");
        ui_topArea   (areaUniversal);
        wm_showArea  (areaUniversal);

#if defined (NEW_FRAME)
        TIMERSTART (TBT_VALUE, SMI_TUNIVERSAL);
        t_universal_running = TRUE;
#else
        TIMERSTART (TBT_VALUE, t_universal_handle);
#endif
        ui_data->timerMod = TIMER_MOD_BT;
    break;

  case UI_BT_HS_DEINIT_SUCCES:
        wm_areaWrite (areaUniversal, row, column, "BLUETOOTH       ");
        wm_areaWrite (areaUniversal, (UBYTE)(row+1), column, "Headset Profile");
        wm_areaWrite (areaUniversal, (UBYTE)(row+2), column, "de-initialized ");
        wm_areaWrite (areaUniversal, (UBYTE)(row+3), column, "                ");
        ui_topArea   (areaUniversal);
        wm_showArea  (areaUniversal);

#if defined (NEW_FRAME)
        TIMERSTART (TBT_VALUE, SMI_TUNIVERSAL);
        t_universal_running = TRUE;
#else
        TIMERSTART (TBT_VALUE, t_universal_handle);
#endif
        ui_data->timerMod = TIMER_MOD_BT;
    break;


  case UI_BT_HS_DEINIT_ERROR:

        wm_areaWrite (areaUniversal, row, column, "BLUETOOTH       ");
        wm_areaWrite (areaUniversal, (UBYTE)(row+1), column, "Headset Profile");
        wm_areaWrite (areaUniversal, (UBYTE)(row+2), column, "de-init ERROR  ");
        wm_areaWrite (areaUniversal, (UBYTE)(row+3), column, "                ");
        ui_topArea   (areaUniversal);
        wm_showArea  (areaUniversal);

#if defined (NEW_FRAME)
        TIMERSTART (TBT_VALUE, SMI_TUNIVERSAL);
        t_universal_running = TRUE;
#else
        TIMERSTART (TBT_VALUE, t_universal_handle);
#endif
        ui_data->timerMod = TIMER_MOD_BT;
    break;

  case UI_BT_DU_DEINIT_SUCCES:

        wm_areaWrite (areaUniversal, row, column, "BLUETOOTH       ");
        wm_areaWrite (areaUniversal, (UBYTE)(row+1), column, "Dial-up Profile");
        wm_areaWrite (areaUniversal, (UBYTE)(row+2), column, "de-initialized ");
        wm_areaWrite (areaUniversal, (UBYTE)(row+3), column, "                ");
        ui_topArea   (areaUniversal);
        wm_showArea  (areaUniversal);

#if defined (NEW_FRAME)
        TIMERSTART (TBT_VALUE, SMI_TUNIVERSAL);
        t_universal_running = TRUE;
#else
        TIMERSTART (TBT_VALUE, t_universal_handle);
#endif
        ui_data->timerMod = TIMER_MOD_BT;
    break;

  case UI_BT_DU_DEINIT_ERROR:

        wm_areaWrite (areaUniversal, row, column, "BLUETOOTH       ");
        wm_areaWrite (areaUniversal, (UBYTE)(row+1), column, "Dial-up Profile");
        wm_areaWrite (areaUniversal, (UBYTE)(row+2), column, "de-init ERROR  ");
        wm_areaWrite (areaUniversal, (UBYTE)(row+3), column, "                ");
        ui_topArea   (areaUniversal);
        wm_showArea  (areaUniversal);

#if defined (NEW_FRAME)
        TIMERSTART (TBT_VALUE, SMI_TUNIVERSAL);
        t_universal_running = TRUE;
#else
        TIMERSTART (TBT_VALUE, t_universal_handle);
#endif
        ui_data->timerMod = TIMER_MOD_BT;
    break;

  case UI_BT_HS_SEARCH_FINISHED:

        wm_areaWrite (areaUniversal, row, column, "BLUETOOTH       ");
        wm_areaWrite (areaUniversal, (UBYTE)(row+1), column, "Headset search ");
        wm_areaWrite (areaUniversal, (UBYTE)(row+2), column, "Finished       ");
        wm_areaWrite (areaUniversal, (UBYTE)(row+3), column, "                ");
        ui_topArea   (areaUniversal);
        wm_showArea  (areaUniversal);

#if defined (NEW_FRAME)
        TIMERSTART (TBT_VALUE, SMI_TUNIVERSAL);
        t_universal_running = TRUE;
#else
        TIMERSTART (TBT_VALUE, t_universal_handle);
#endif
        ui_data->timerMod = TIMER_MOD_BT;
    break;

  case UI_BT_DU_SEARCH_FINISHED:

        wm_areaWrite (areaUniversal, row, column, "BLUETOOTH       ");
        wm_areaWrite (areaUniversal, (UBYTE)(row+1), column, "Dial-Up search");
        wm_areaWrite (areaUniversal, (UBYTE)(row+2), column, "Finished      ");
        wm_areaWrite (areaUniversal, (UBYTE)(row+3), column, "                ");
        ui_topArea   (areaUniversal);
        wm_showArea  (areaUniversal);

#if defined (NEW_FRAME)
        TIMERSTART (TBT_VALUE, SMI_TUNIVERSAL);
        t_universal_running = TRUE;
#else
        TIMERSTART (TBT_VALUE, t_universal_handle);
#endif
        ui_data->timerMod = TIMER_MOD_BT;
    break;

  case UI_BT_HS_FOUND:

        wm_areaWrite (areaUniversal, row, column, "BLUETOOTH       ");
        wm_areaWrite (areaUniversal, (UBYTE)(row+1), column, "Headset found  ");
        wm_areaWrite (areaUniversal, (UBYTE)(row+2), column, "BT Addr:       ");
        convert_bd_addr(data);
        wm_areaWrite (areaUniversal, (UBYTE)(row+3), column,  bd_addr_data);
        ui_topArea   (areaUniversal);
        wm_showArea  (areaUniversal);

#if defined (NEW_FRAME)
        TIMERSTART (TBT_VALUE, SMI_TUNIVERSAL);
        t_universal_running = TRUE;
#else
        TIMERSTART (TBT_VALUE, t_universal_handle);
#endif
        ui_data->timerMod = TIMER_MOD_BT;
    break;

  case UI_BT_DU_FOUND:

        wm_areaWrite (areaUniversal, row, column, "BLUETOOTH       ");
        wm_areaWrite (areaUniversal, (UBYTE)(row+1), column, "Dial-Up found ");
        wm_areaWrite (areaUniversal, (UBYTE)(row+2), column, "BT Addr:      ");
        convert_bd_addr(data);
        wm_areaWrite (areaUniversal, (UBYTE)(row+3), column, bd_addr_data);
        ui_topArea   (areaUniversal);
        wm_showArea  (areaUniversal);

#if defined (NEW_FRAME)
        TIMERSTART (TBT_VALUE, SMI_TUNIVERSAL);
        t_universal_running = TRUE;
#else
        TIMERSTART (TBT_VALUE, t_universal_handle);
#endif
        ui_data->timerMod = TIMER_MOD_BT;
    break;

  case UI_BT_HS_CONNECTED:

        wm_areaWrite (areaUniversal, row, column, "BLUETOOTH       ");
        wm_areaWrite (areaUniversal, (UBYTE)(row+1), column, "Headset connect");
        wm_areaWrite (areaUniversal, (UBYTE)(row+2), column, "BT Addr:       ");
        convert_bd_addr(data);
        wm_areaWrite (areaUniversal, (UBYTE)(row+3), column, bd_addr_data);
        ui_topArea   (areaUniversal);
        wm_showArea  (areaUniversal);

#if defined (NEW_FRAME)
        TIMERSTART (TBT_VALUE, SMI_TUNIVERSAL);
        t_universal_running = TRUE;
#else
        TIMERSTART (TBT_VALUE, t_universal_handle);
#endif
        ui_data->timerMod = TIMER_MOD_BT;
    break;

  case UI_BT_HS_CONNECT_ERROR:

        wm_areaWrite (areaUniversal, row, column, "BLUETOOTH       ");
        wm_areaWrite (areaUniversal, (UBYTE)(row+1), column, "HS connect error");
        wm_areaWrite (areaUniversal, (UBYTE)(row+2), column, "BT Addr:        ");
        convert_bd_addr(data);
        wm_areaWrite (areaUniversal, (UBYTE)(row+3), column, bd_addr_data);
        ui_topArea   (areaUniversal);
        wm_showArea  (areaUniversal);

#if defined (NEW_FRAME)
        TIMERSTART (TBT_VALUE, SMI_TUNIVERSAL);
        t_universal_running = TRUE;
#else
        TIMERSTART (TBT_VALUE, t_universal_handle);
#endif
        ui_data->timerMod = TIMER_MOD_BT;
    break;

  case UI_BT_DU_CONNECTED:

        wm_areaWrite (areaUniversal, row, column, "BLUETOOTH       ");
        wm_areaWrite (areaUniversal, (UBYTE)(row+1), column, "Dial-Up connect ");
        wm_areaWrite (areaUniversal, (UBYTE)(row+2), column, "BT Addr:        ");
        convert_bd_addr(data);
        wm_areaWrite (areaUniversal, (UBYTE)(row+3), column, bd_addr_data);
        ui_topArea   (areaUniversal);
        wm_showArea  (areaUniversal);

#if defined (NEW_FRAME)
        TIMERSTART (TBT_VALUE, SMI_TUNIVERSAL);
        t_universal_running = TRUE;
#else
        TIMERSTART (TBT_VALUE, t_universal_handle);
#endif
        ui_data->timerMod = TIMER_MOD_BT;
    break;

  case UI_BT_DU_CONNECT_ERROR:

        wm_areaWrite (areaUniversal, row, column, "BLUETOOTH      ");
        wm_areaWrite (areaUniversal, (UBYTE)(row+1), column, "DU connect error");
        wm_areaWrite (areaUniversal, (UBYTE)(row+2), column, "BT Addr:        ");
        convert_bd_addr(data);
        wm_areaWrite (areaUniversal, (UBYTE)(row+3), column, bd_addr_data);
        ui_topArea   (areaUniversal);
        wm_showArea  (areaUniversal);

#if defined (NEW_FRAME)
        TIMERSTART (TBT_VALUE, SMI_TUNIVERSAL);
        t_universal_running = TRUE;
#else
        TIMERSTART (TBT_VALUE, t_universal_handle);
#endif
        ui_data->timerMod = TIMER_MOD_BT;
    break;

  case UI_BT_HS_DISCONNECTED:

        wm_areaWrite (areaUniversal, row, column, "BLUETOOTH      ");
        wm_areaWrite (areaUniversal, (UBYTE)(row+1), column, "Headset discon  ");
        wm_areaWrite (areaUniversal, (UBYTE)(row+2), column, "BT Addr:        ");
        convert_bd_addr(data);
        wm_areaWrite (areaUniversal, (UBYTE)(row+3), column, bd_addr_data);
        ui_topArea   (areaUniversal);
        wm_showArea  (areaUniversal);

#if defined (NEW_FRAME)
        TIMERSTART (TBT_VALUE, SMI_TUNIVERSAL);
        t_universal_running = TRUE;
#else
        TIMERSTART (TBT_VALUE, t_universal_handle);
#endif
        ui_data->timerMod = TIMER_MOD_BT;
    break;

  case UI_BT_DU_DISCONNECTED:

        wm_areaWrite (areaUniversal, row, column, "BLUETOOTH     ");
        wm_areaWrite (areaUniversal, (UBYTE)(row+1), column, "Dial-Up discon");
        wm_areaWrite (areaUniversal, (UBYTE)(row+2), column, "BT Addr:      ");
        convert_bd_addr(data);
        wm_areaWrite (areaUniversal, (UBYTE)(row+3), column, bd_addr_data);
        ui_topArea   (areaUniversal);
        wm_showArea  (areaUniversal);

#if defined (NEW_FRAME)
        TIMERSTART (TBT_VALUE, SMI_TUNIVERSAL);
        t_universal_running = TRUE;
#else
        TIMERSTART (TBT_VALUE, t_universal_handle);
#endif
        ui_data->timerMod = TIMER_MOD_BT;
    break;

    case UI_BT_HS_INIT_START:

        wm_areaWrite (areaUniversal, row, column, "BLUETOOTH     ");
        wm_areaWrite (areaUniversal, (UBYTE)(row+1), column, "Headset Init    ");
        wm_areaWrite (areaUniversal, (UBYTE)(row+2), column, "Started         ");
        wm_areaWrite (areaUniversal, (UBYTE)(row+3), column, "                ");
        ui_topArea   (areaUniversal);
        wm_showArea  (areaUniversal);

#if defined (NEW_FRAME)
        TIMERSTART (TBT_VALUE, SMI_TUNIVERSAL);
        t_universal_running = TRUE;
#else
        TIMERSTART (TBT_VALUE, t_universal_handle);
#endif
        ui_data->timerMod = TIMER_MOD_BT;
    break;
    case UI_BT_DU_INIT_START:

        wm_areaWrite (areaUniversal, row, column, "BLUETOOTH     ");
        wm_areaWrite (areaUniversal, (UBYTE)(row+1), column, "Dial-Up Init    ");
        wm_areaWrite (areaUniversal, (UBYTE)(row+2), column, "Started         ");
        wm_areaWrite (areaUniversal, (UBYTE)(row+3), column, "                ");
        ui_topArea   (areaUniversal);
        wm_showArea  (areaUniversal);

#if defined (NEW_FRAME)
        TIMERSTART (TBT_VALUE, SMI_TUNIVERSAL);
        t_universal_running = TRUE;
#else
        TIMERSTART (TBT_VALUE, t_universal_handle);
#endif
        ui_data->timerMod = TIMER_MOD_BT;
    break;
    case UI_BT_HS_SEARCH_START:

        wm_areaWrite (areaUniversal, row, column, "BLUETOOTH     ");
        wm_areaWrite (areaUniversal, (UBYTE)(row+1), column, "Headset Search  ");
        wm_areaWrite (areaUniversal, (UBYTE)(row+2), column, "Started         ");
        wm_areaWrite (areaUniversal, (UBYTE)(row+3), column, "                ");
        ui_topArea   (areaUniversal);
        wm_showArea  (areaUniversal);

#if defined (NEW_FRAME)
        TIMERSTART (TBT_VALUE, SMI_TUNIVERSAL);
        t_universal_running = TRUE;
#else
        TIMERSTART (TBT_VALUE, t_universal_handle);
#endif
        ui_data->timerMod = TIMER_MOD_BT;
    break;

    case UI_BT_DU_SEARCH_START:

        wm_areaWrite (areaUniversal, row, column, "BLUETOOTH     ");
        wm_areaWrite (areaUniversal, (UBYTE)(row+1), column, "Dial-Up Search  ");
        wm_areaWrite (areaUniversal, (UBYTE)(row+2), column, "Started         ");
        wm_areaWrite (areaUniversal, (UBYTE)(row+3), column, "                ");
        ui_topArea   (areaUniversal);
        wm_showArea  (areaUniversal);

#if defined (NEW_FRAME)
        TIMERSTART (TBT_VALUE, SMI_TUNIVERSAL);
        t_universal_running = TRUE;
#else
        TIMERSTART (TBT_VALUE, t_universal_handle);
#endif
        ui_data->timerMod = TIMER_MOD_BT;
    break;

  case UI_BT_HS_CONNECTING:

        wm_areaWrite (areaUniversal, row, column, "BLUETOOTH     ");
        wm_areaWrite (areaUniversal, (UBYTE)(row+1), column, "HS Connecting  ");
        wm_areaWrite (areaUniversal, (UBYTE)(row+2), column, "BT Addr:       ");
        convert_bd_addr(data);
        wm_areaWrite (areaUniversal, (UBYTE)(row+3), column, bd_addr_data);
        ui_topArea   (areaUniversal);
        wm_showArea  (areaUniversal);

#if defined (NEW_FRAME)
        TIMERSTART (TBT_VALUE, SMI_TUNIVERSAL);
        t_universal_running = TRUE;
#else
        TIMERSTART (TBT_VALUE, t_universal_handle);
#endif
        ui_data->timerMod = TIMER_MOD_BT;
    break;

  case UI_BT_DU_CONNECTING:

        wm_areaWrite (areaUniversal, row, column, "BLUETOOTH     ");
        wm_areaWrite (areaUniversal, (UBYTE)(row+1), column, "DU Connecting  ");
        wm_areaWrite (areaUniversal, (UBYTE)(row+2), column, "BT Addr:       ");
        convert_bd_addr(data);
        wm_areaWrite (areaUniversal, (UBYTE)(row+3), column, bd_addr_data);
        ui_topArea   (areaUniversal);
        wm_showArea  (areaUniversal);

#if defined (NEW_FRAME)
        TIMERSTART (TBT_VALUE, SMI_TUNIVERSAL);
        t_universal_running = TRUE;
#else
        TIMERSTART (TBT_VALUE, t_universal_handle);
#endif
        ui_data->timerMod = TIMER_MOD_BT;
    break;
  case UI_BT_HS_DISCONNECTING:

        wm_areaWrite (areaUniversal, row, column, "BLUETOOTH     ");
        wm_areaWrite (areaUniversal, (UBYTE)(row+1), column, "HS Disconnectin");
        wm_areaWrite (areaUniversal, (UBYTE)(row+2), column, "BT Addr:       ");
        convert_bd_addr(data);
        wm_areaWrite (areaUniversal, (UBYTE)(row+3), column, bd_addr_data);
        ui_topArea   (areaUniversal);
        wm_showArea  (areaUniversal);

#if defined (NEW_FRAME)
        TIMERSTART (TBT_VALUE, SMI_TUNIVERSAL);
        t_universal_running = TRUE;
#else
        TIMERSTART (TBT_VALUE, t_universal_handle);
#endif
        ui_data->timerMod = TIMER_MOD_BT;
    break;

  case UI_BT_DU_DISCONNECTING:

        wm_areaWrite (areaUniversal, row, column, "BLUETOOTH     ");
        wm_areaWrite (areaUniversal, (UBYTE)(row+1), column, "DU Disconnectin");
        wm_areaWrite (areaUniversal, (UBYTE)(row+2), column, "BT Addr:       ");
        convert_bd_addr(data);
        wm_areaWrite (areaUniversal, (UBYTE)(row+3), column, bd_addr_data);
        ui_topArea   (areaUniversal);
        wm_showArea  (areaUniversal);

#if defined (NEW_FRAME)
        TIMERSTART (TBT_VALUE, SMI_TUNIVERSAL);
        t_universal_running = TRUE;
#else
        TIMERSTART (TBT_VALUE, t_universal_handle);
#endif
        ui_data->timerMod = TIMER_MOD_BT;
    break;

  case UI_BT_HS_DEINIT_START:

        wm_areaWrite (areaUniversal, row, column, "BLUETOOTH     ");
        wm_areaWrite (areaUniversal, (UBYTE)(row+1), column, "Headset De-Init ");
        wm_areaWrite (areaUniversal, (UBYTE)(row+2), column, "Started         ");
        wm_areaWrite (areaUniversal, (UBYTE)(row+3), column, "                ");
        ui_topArea   (areaUniversal);
        wm_showArea  (areaUniversal);

#if defined (NEW_FRAME)
        TIMERSTART (TBT_VALUE, SMI_TUNIVERSAL);
        t_universal_running = TRUE;
#else
        TIMERSTART (TBT_VALUE, t_universal_handle);
#endif
        ui_data->timerMod = TIMER_MOD_BT;
    break;

    case UI_BT_DU_DEINIT_START:

        wm_areaWrite (areaUniversal, row, column, "BLUETOOTH     ");
        wm_areaWrite (areaUniversal, (UBYTE)(row+1), column, "Dial-Up De-Init ");
        wm_areaWrite (areaUniversal, (UBYTE)(row+2), column, "Started         ");
        wm_areaWrite (areaUniversal, (UBYTE)(row+3), column, "                ");
        ui_topArea   (areaUniversal);
        wm_showArea  (areaUniversal);

#if defined (NEW_FRAME)
        TIMERSTART (TBT_VALUE, SMI_TUNIVERSAL);
        t_universal_running = TRUE;
#else
        TIMERSTART (TBT_VALUE, t_universal_handle);
#endif
        ui_data->timerMod = TIMER_MOD_BT;
    break;


  default:
        wm_areaWrite (areaUniversal, row, column, "Unknown mes.  ");

        ui_topArea   (areaUniversal);
        wm_showArea  (areaUniversal);

#if defined (NEW_FRAME)
        TIMERSTART (TBT_VALUE, SMI_TUNIVERSAL);
        t_universal_running = TRUE;
#else
        TIMERSTART (TBT_VALUE, t_universal_handle);
#endif
        ui_data->timerMod = TIMER_MOD_BT;
    break;
  }

}
#endif /* BT_ADAPTER */

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_UI                   |
| STATE   : code                  ROUTINE : ui_handleMenuSelection   |
+--------------------------------------------------------------------+

  PURPOSE : This function handles the selection of a menu entry.
*/
LOCAL void ui_handleMenuSelection (void)
{
  TRACE_FUNCTION ("ui_handleMenuSelection()");

  switch (ui_data->menuMode)
  {
    /*
     *---------------------------------------------------------------
     * the following procedure will be executed after a menu
     * selection is made during the view of a single SM or CBM
     * in detail
     *---------------------------------------------------------------
     */
    case (MEN_MOD_MSG_VIEW):
      men_stopMenu (areaMenu);
      wm_hideArea (areaMenu);

      ui_data->state    = oldState;
      ui_data->menuMode = MEN_MOD_NONE;
      ui_data->inputSeq = INP_SEQ_NONE;
      break;

    /*
     *---------------------------------------------------------------
     * the following procedure will be executed after a menu
     * selection is made during the view of the whole list of
     * all saved SM or CBM
     *---------------------------------------------------------------
     */
    case (MEN_MOD_SMS_LIST):
      men_stopMenu (areaMenu);
      wm_hideArea (areaMenu);

      ui_data->state    = oldState;
      ui_data->menuMode = MEN_MOD_NONE;

      switch (ui_data->inputSeq)
      {
        case (INP_SEQ_SMS_DELETE):
          ui_data->inputSeq = INP_SEQ_NONE;

          sms_signal (SMS_SIG_DELETE);
          break;

        case (INP_SEQ_SMS_READ):
          sms_signal (SMS_SIG_START_MSG_VIEW);
          break;

        case (INP_SEQ_SMS_SUBMIT_FROM_MEM):
          {
            CHAR da[MAX_SMS_NUM_LEN];

            sms_getDa (da);

            ui_startNewInput (areaUser,
                              0,
                              0,
                              "DA: ",
                              MAX_SMS_NUM_LEN,
                              IT_NUMERIC | IT_ECHO,
                              INP_MOD_DA,
                              da,
                              TRUE);
          }
          break;

        default:
          ui_data->inputSeq = INP_SEQ_NONE;
          break;
      }
      break;

    /*
     *---------------------------------------------------------------
     * process PLMN menu selection
     *---------------------------------------------------------------
     */
    case (MEN_MOD_PLMN):
      if (!plmnIsEmpty)
        aca_signal (ACA_SIG_REGISTER,
                    plmnmenu[men_getMenuIdx (areaMenu)]);

      plmnIsEmpty = TRUE;

      men_stopMenu (areaMenu);
      wm_hideArea (areaMenu);

      ui_data->state    = oldState;
      ui_data->menuMode = MEN_MOD_NONE;
      break;

    /*
     *---------------------------------------------------------------
     * process call table menu selection
     *---------------------------------------------------------------
     */
    case (MEN_MOD_CTB):
      men_stopMenu (areaMenu);
      wm_hideArea (areaMenu);

      ui_data->state    = oldState;
      ui_data->menuMode = MEN_MOD_NONE;
      break;

    /*
     *---------------------------------------------------------------
     * process main menu selection
     *---------------------------------------------------------------
     */
    case (MEN_MOD_MAIN):
    {
      static CHAR *submenu[6];

      ui_data->mainMenuIdx = men_getMenuIdx (areaMenu);

      if (!ui_data->inSubmenu)
      {
        switch (ui_data->mainMenuIdx)
        {
          case 0: /* language */
            submenu[0] = lng_getText (TXT_MEN_ENGLISH);
            submenu[1] = lng_getText (TXT_MEN_FRENCH);
            submenu[2] = lng_getText (TXT_MEN_GERMAN);
            submenu[3] = lng_getText (TXT_MEN_ITALIAN);
            submenu[4] = lng_getText (TXT_MEN_SPANISH);
            submenu[5] = NULL;
            ui_data->inSubmenu = TRUE;
            break;

          case 1: /* send DTMF */
            submenu[0] = lng_getText (TXT_MEN_START);
            submenu[1] = lng_getText (TXT_MEN_STOP);
            submenu[2] = NULL;
            ui_data->inSubmenu = TRUE;
            break;

          case 2: /* key beep */
            submenu[0] = lng_getText (TXT_MEN_ON);
            submenu[1] = lng_getText (TXT_MEN_OFF);
            submenu[2] = NULL;
            ui_data->inSubmenu = TRUE;
            break;
        }

        if (ui_data->inSubmenu)
        {
          ui_topArea (areaSubMenu);
          men_startMenu (areaSubMenu,
                         menu[ui_data->mainMenuIdx],
                         submenu);
          wm_showArea (areaSubMenu);
        }
        else
        {
          /*
           *---------------------------------------------------------
           * no submenu to open -> stop the main menu
           *---------------------------------------------------------
           */
          men_stopMenu (areaMenu);
          wm_hideArea (areaMenu);

          ui_data->state    = oldState;
          ui_data->menuMode = MEN_MOD_NONE;
        }
      }
      else
      {
        /*
         *---------------------------------------------------------
         * OK pressed in an active submenu
         *---------------------------------------------------------
         */
        switch (ui_data->mainMenuIdx)
        {
          case 0: /* language  */
            switch (men_getMenuIdx (areaSubMenu))
            {
              case 0: /* english */
                csf_setLanguage (LNG_ENGLISH);
                break;
              case 1: /* french  */
                csf_setLanguage (LNG_FRENCH);
                break;
              case 2: /* german  */
                csf_setLanguage (LNG_GERMAN);
                break;
              case 3: /* italien  */
                csf_setLanguage (LNG_ITALIEN);
                break;
              case 4: /* spanish  */
                csf_setLanguage (LNG_SPANISH);
                break;
            }
            break;

          case 1: /* send DTMF */
            switch (men_getMenuIdx (areaSubMenu))
            {
              case 0: /* Start */
                if (cal_isDTMFPossible ())
                  ui_data->sendDTMF = TRUE;
                break;
              case 1: /* Stop  */
                ui_data->sendDTMF = FALSE;
                break;
            }
            break;

          case 2: /* key beep  */
            switch (men_getMenuIdx (areaSubMenu))
            {
              case 0: /* ON */
                ui_data->keyBeep = TRUE;
                break;
              case 1: /* OFF  */
                ui_data->keyBeep = FALSE;
                break;
            }
            break;
        }

        men_stopMenu (areaSubMenu);
        wm_hideArea (areaSubMenu);

        ui_data->inSubmenu = FALSE;

        men_stopMenu (areaMenu);
        wm_hideArea (areaMenu);

        ui_data->state    = oldState;
        ui_data->menuMode = MEN_MOD_NONE;
      }
    }
    break;
  }
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_UI                   |
| STATE   : code                  ROUTINE : ui_processDefaultKey     |
+--------------------------------------------------------------------+

  PURPOSE : This function processes a pressed default key.
*/
LOCAL void ui_processDefaultKey (UBYTE key_pressed, UBYTE stat)
{
  if (ui_data->sendDTMF)
  {
    if ((key_pressed >= '0' AND key_pressed <= '9') OR
        (key_pressed >= 'A' AND key_pressed <= 'D') OR
        (key_pressed >= 'a' AND key_pressed <= 'd') OR
         key_pressed EQ '#' OR  key_pressed EQ '*')
      aca_signal (ACA_SIG_SEND_DTMF, key_pressed, stat );
  }
  else if (ui_data->state EQ UI_INPUT)
    inp_charToInput (areaUser, key_pressed);

  ui_data->lastShiftedKey = NULL_TERM;
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_UI                   |
| STATE   : code                  ROUTINE : ui_keypadInd             |
+--------------------------------------------------------------------+

  PURPOSE : This function is called when a key is pressed or released.

            <key_code>: key code
            <key_stat>: key state
*/
GLOBAL const void ui_keypadInd (UBYTE key_code, UBYTE key_stat)
{
  UBYTE key_pressed; /* holds the key which is pressed */
  CHAR *input;       /* Pointer to input string */

  TRACE_FUNCTION ("ui_keypadInd()");

  key_pressed = ui_updateKeystate
                  (
                    convertKeyCode (key_code),
                    (key_stat EQ KEY_STAT_PRS)
                  );

  if (key_pressed NEQ 0)
  {
    if (!(ui_data->powerOn OR key_pressed EQ POWER_KEY
                           OR key_pressed EQ SHIFT_KEY
                           OR key_pressed EQ '#'      ))
      return;

    TRACE_EVENT_P2 ("key_code: %#x, key_pressed: %#x", key_code, key_pressed);

    switch (key_pressed)
    {
      /*
       *-------------------------------------------------------------
       * process POWER_KEY
       *-------------------------------------------------------------
       */
      case POWER_KEY:
      {
        if (powerKeyOn)
        {
          powerKeyOn = FALSE;
          aca_signal (ACA_SIG_POWER_OFF);
        }
        else
        {
          powerKeyOn = TRUE;
          aca_signal (ACA_SIG_POWER_ON);
        }
        break;
      }

      /*
       *-------------------------------------------------------------
       * process EMERGENCY_KEY
       *-------------------------------------------------------------
       */
      case EMERGENCY_KEY:
        if (!ui_data->normalService)
          aca_signal (ACA_SIG_KEYSEQUENCE, EMERGENCY_SEQUENCE);

        break;

      /*
       *-------------------------------------------------------------
       * process OFF_HOOK_KEY
       *-------------------------------------------------------------
       */
      case OFF_HOOK_KEY:
      {
        if (ui_data->inputMode EQ INP_MOD_PIN        OR
            ui_data->inputMode EQ INP_MOD_NPIN       OR
            ui_data->inputMode EQ INP_MOD_NPIN_TWICE OR
            ui_data->inputMode EQ INP_MOD_PUK          )
          return;

        if (cal_isFirstCallWait ())
        {
          /*
           *---------------------------------------------------------
           * there is only one call waiting
           * no held, active or dialed calls are present
           *---------------------------------------------------------
           */
          input = inp_getInput (areaUser);
          if (!strlen (input))
          {
            if (ui_data->ringerOn)
            {
              cal_sendAudioToneReq (CT_RING,
                                    csf_getVolume (VT_RINGER),
                                    TONE_SWT_OFF);
              ui_data->ringerOn = FALSE;
            }

            /* No key sequence, accept the call */
            aca_signal (ACA_SIG_ACCEPT);
          }
          else
          {
            /* There maybe some key sequence */
            strcpy (ui_data->lastDialedNumber, input);
            aca_signal (ACA_SIG_KEYSEQUENCE, input);
          }
        }
        else
        {
          if (ui_data->state EQ UI_INPUT)
          {
            if (ui_data->inputMode EQ INP_MOD_DIAL)
            {
              input = inp_getInput (areaUser);

              if (!strlen (input))
              {
                inp_setInput (areaUser,
                              ui_data->lastDialedNumber);
                return;
              }
              else
              {
                strcpy (ui_data->lastDialedNumber, input);
                aca_signal (ACA_SIG_KEYSEQUENCE, input);
              }

              /*
               *---------------------------------------------------
               * above recognized key sequence may change the input
               * sequence, so reset of input is done carefully
               *---------------------------------------------------
               */
              if (ui_data->state    EQ UI_INPUT AND
                  ui_data->inputSeq EQ INP_SEQ_NONE)
                ui_startNewInput (areaUser,
                                  0,
                                  0,
                                  ">",
                                  MAX_INPUT_SIZE - 10,
                                  IT_NUMERIC | IT_ECHO,
                                  INP_MOD_DIAL,
                                  NULL,
                                  FALSE);

              /*
               *---------------------------------------------------
               * above recognized key sequence may change to menu
               * mode, so update of views is done carefully
               *---------------------------------------------------
               */
              if (ui_data->state EQ UI_SELECT)
              {
                inp_stopInput (areaUser);
                wm_clearArea (areaUser);

                inp_startInput (areaUser,
                                0,
                                0,
                                ">",
                                MAX_INPUT_SIZE - 10,
                                IT_NUMERIC | IT_ECHO);

                ui_data->inputMode = INP_MOD_DIAL;
              }
              else
              {
                ui_topArea (areaUser);
                wm_showArea (areaUser);
              }
              break;
            }
          }
        }
        break;
      }

      /*
       *-------------------------------------------------------------
       * process ON_HOOK_KEY
       *-------------------------------------------------------------
       */
      case ON_HOOK_KEY:
      {
        if (ui_data->state EQ UI_SELECT)
        {
          ui_stopMenu ();

          if (ui_data->inputSeq NEQ INP_SEQ_NONE)
          {
            /*
             *---------------------------------------------------------
             * stop any input sequences which are started due to
             * former entered key sequences
             *---------------------------------------------------------
             */
            ui_startNewInput (areaUser,
                              0,
                              0,
                              ">",
                              MAX_INPUT_SIZE - 10,
                              IT_NUMERIC | IT_ECHO,
                              INP_MOD_DIAL,
                              NULL,
                              TRUE);

            ui_data->inputSeq = INP_SEQ_NONE;
          }
        }
        else if (ui_data->state    EQ  UI_INPUT     AND
                 ui_data->inputSeq NEQ INP_SEQ_NONE    )
        {
          /*
           *---------------------------------------------------------
           * stop any input sequences which are started due to
           * former entered key sequences
           *---------------------------------------------------------
           */
          ui_startNewInput (areaUser,
                            0,
                            0,
                            ">",
                            MAX_INPUT_SIZE - 10,
                            IT_NUMERIC | IT_ECHO,
                            INP_MOD_DIAL,
                            NULL,
                            TRUE);

          ui_data->inputSeq = INP_SEQ_NONE;
        }
        else if (ui_data->state     EQ UI_INPUT    AND
                 ui_data->inputMode EQ INP_MOD_PIN    )
        {
          if ((!cal_isNoCall () AND !ui_data->nibtOn))
          {
            /*
             *-----------------------------------------------------
             * start normal call release
             *-----------------------------------------------------
             */
            aca_signal (ACA_SIG_RELEASE);
          }
          else if (ui_data->nibtOn)
          {
            ui_handleNibt (NIBT_OFF);
/*
            cal_sendAudioToneReq (CT_BUSY,
                                  csf_getVolume (VT_RINGER),
                                  TONE_SWT_OFF);
*/

            ui_data->nibtOn = FALSE;

            wm_clearArea (areaState);
          }
          else
          {
            /*
             *-------------------------------------------------------
             * cancel the PIN input routine
             *-------------------------------------------------------
             */
            inp_stopInput (areaUser);
            ui_data->state     = UI_IDLE;
            ui_data->inputMode = INP_MOD_NONE;
            aca_signal (ACA_SIG_PININPUT, "");
          }
        }
        else if (ui_data->state EQ UI_INPUT                 AND
                 (
                   ui_data->inputMode EQ INP_MOD_PUK        OR
                   ui_data->inputMode EQ INP_MOD_NPIN       OR
                   ui_data->inputMode EQ INP_MOD_NPIN_TWICE
                 )
                )
        {
          if ((!cal_isNoCall () AND !ui_data->nibtOn))
          {
            /*
             *-----------------------------------------------------
             * start normal call release
             *-----------------------------------------------------
             */
            aca_signal (ACA_SIG_RELEASE);
          }
          else if (ui_data->nibtOn)
          {
            ui_handleNibt (NIBT_OFF);
/*
            cal_sendAudioToneReq (CT_BUSY,
                                  csf_getVolume (VT_RINGER),
                                  TONE_SWT_OFF);
*/

            ui_data->nibtOn = FALSE;

            wm_clearArea (areaState);
          }
          else
          {
            /*
             *-------------------------------------------------------
             * cancel the PUK input routine
             *-------------------------------------------------------
             */
            inp_stopInput (areaUser);
            ui_data->state     = UI_IDLE;
            ui_data->inputMode = INP_MOD_NONE;

            aca_signal (ACA_SIG_PUKINPUT, "", "", "");

            memset (puk, NULL_TERM, MAX_PUK_LEN + 1);
            memset (pin, NULL_TERM, MAX_PIN_LEN + 1);
          }
        }
        else if ((!cal_isNoCall () AND !ui_data->nibtOn) OR
                 cal_isNoCall ())
        {
          /*
           *-------------------------------------------------------
           * start normal call release
           *-------------------------------------------------------
           */
          aca_signal (ACA_SIG_RELEASE);
        }
        else if (ui_data->nibtOn)
        {
          ui_handleNibt (NIBT_OFF);
/*
          cal_sendAudioToneReq (CT_BUSY,
                                csf_getVolume (VT_RINGER),
                                TONE_SWT_OFF);
*/

          ui_data->nibtOn = FALSE;

          wm_clearArea (areaState);
        }
        break;
      }

      /*
       *-------------------------------------------------------------
       * process OK_KEY
       *-------------------------------------------------------------
       */
      case OK_KEY:
      {
        ui_shift (FALSE);

        if (ui_data->state EQ UI_INPUT)
        {
          if (ui_data->inputMode EQ INP_MOD_PIN        OR
              ui_data->inputMode EQ INP_MOD_NPIN       OR
              ui_data->inputMode EQ INP_MOD_NPIN_TWICE OR
              ui_data->inputMode EQ INP_MOD_PUK          )
            ui_handlePinAndPuk ();
          else
          {
            CHAR* input = inp_getInput (areaUser);

            switch (ui_data->inputMode)
            {
              /*
               *---------------------------------------------------
               * process input mode INP_MOD_IDX
               *---------------------------------------------------
               */
              case (INP_MOD_IDX):
                if ( strlen (input) > 0)
                  actIdx = atoi (input);
                else
                  actIdx = ACI_NumParmNotPresent;

                sms_signal (SMS_SIG_READ_SINGLE, actMem, actIdx);

                ui_startNewInput (areaUser,
                                  0,
                                  0,
                                  ">",
                                  MAX_INPUT_SIZE - 10,
                                  IT_NUMERIC | IT_ECHO,
                                  INP_MOD_DIAL,
                                  NULL,
                                  TRUE);

                ui_data->inputSeq = INP_SEQ_NONE;
                break;

              /*
               *---------------------------------------------------
               * process input mode INP_MOD_MEM
               *---------------------------------------------------
               */
              case (INP_MOD_MEM):
                if ( strlen (input) > 0)
                  actMem = atoi (input);
                else
                  actMem = SMS_STOR_NotPresent;

                ui_startNewInput (areaUser,
                                  0,
                                  0,
                                  "IDX: ",
                                  3,
                                  IT_NUMERIC | IT_ECHO,
                                  INP_MOD_IDX,
                                  NULL,
                                  TRUE);
                break;

              /*
               *---------------------------------------------------
               * process input mode INP_MOD_SM
               *---------------------------------------------------
               */
              case (INP_MOD_SM):
                {
                  CHAR da[MAX_SMS_NUM_LEN];

                  sms_getDa (da);

                  sms_setMsg (input);

                  ui_startNewInput (areaUser,
                                    0,
                                    0,
                                    "DA: ",
                                    MAX_SMS_NUM_LEN,
                                    IT_NUMERIC | IT_ECHO,
                                    INP_MOD_DA,
                                    da,
                                    TRUE);
                }
                break;

              /*
               *---------------------------------------------------
               * process input mode INP_MOD_SCA
               *---------------------------------------------------
               */
              case (INP_MOD_SCA):
              {
                CHAR  buf[5];
                SHORT vp = sms_getVp ();

                sms_setSca (input);

                if (vp > ACI_NumParmNotPresent)
                  sprintf (buf, "%d", vp);
                else
                  buf[0] = NULL_TERM;

                ui_startNewInput (areaUser,
                                  0,
                                  0,
                                  "VP: ",
                                  3,
                                  IT_NUMERIC | IT_ECHO,
                                  INP_MOD_VP,
                                  buf,
                                  TRUE);
                break;
              }

              /*
               *---------------------------------------------------
               * process input mode INP_MOD_DA
               *---------------------------------------------------
               */
              case (INP_MOD_DA):
                sms_setDa (input);

                switch (ui_data->inputSeq)
                {
                  case (INP_SEQ_SMS_WRITE):
                    sms_signal (SMS_SIG_WRITE);

                    ui_startNewInput (areaUser,
                                      0,
                                      0,
                                      ">",
                                      MAX_INPUT_SIZE - 10,
                                      IT_NUMERIC | IT_ECHO,
                                      INP_MOD_DIAL,
                                      NULL,
                                      TRUE);

                    ui_data->inputSeq = INP_SEQ_NONE;
                    break;

                  default:
                    {
                      CHAR sca[MAX_SMS_NUM_LEN];

                      sms_getSca (sca);

                      ui_startNewInput (areaUser,
                                        0,
                                        0,
                                        "SCA: ",
                                        MAX_SMS_NUM_LEN,
                                        IT_NUMERIC | IT_ECHO,
                                        INP_MOD_SCA,
                                        sca,
                                        TRUE);
                    }
                    break;
                }

                break;

              /*
               *---------------------------------------------------
               * process input mode INP_MOD_VP
               *---------------------------------------------------
               */
              case (INP_MOD_VP):
                if ( strlen (input) > 0)
                  sms_setVp ((SHORT) atoi (input));
                else
                  sms_setVp (ACI_NumParmNotPresent);

                switch (ui_data->inputSeq)
                {
                  case (INP_SEQ_SMS_SUBMIT):
                    sms_signal (SMS_SIG_SUBMIT);
                    break;

                  case (INP_SEQ_SMS_SUBMIT_FROM_MEM):
                    sms_signal (SMS_SIG_SUBMIT_FROM_MEM);
                    break;

                  default:
                    break;
                }

                ui_startNewInput (areaUser,
                                  0,
                                  0,
                                  ">",
                                  MAX_INPUT_SIZE - 10,
                                  IT_NUMERIC | IT_ECHO,
                                  INP_MOD_DIAL,
                                  NULL,
                                  TRUE);

                ui_data->inputSeq = INP_SEQ_NONE;
                break;
            }
          }
        }
        else if (ui_data->state EQ UI_SELECT)
          ui_handleMenuSelection ();
        break;
      }

      /*
       *-------------------------------------------------------------
       * process '#'-key
       *-------------------------------------------------------------
       */
      case '#':
      {
        if (!powerKeyOn)
        {
          /*
           *---------------------------------------------------------
           * pressing the hash key in power off state
           * leads to a reload of the software
           *---------------------------------------------------------
           */
          ui_startBooting ();
          break;
        }

        if (ui_data->sendDTMF)
        {
          /*
           *---------------------------------------------------------
           * DTMF tone is send
           *---------------------------------------------------------
           */
          aca_signal (ACA_SIG_SEND_DTMF, key_pressed, key_stat);
        }
        else
          if (ui_data->state EQ UI_INPUT)
          {
            if (ui_data->inputMode EQ INP_MOD_PIN        OR
                ui_data->inputMode EQ INP_MOD_PUK        OR
                ui_data->inputMode EQ INP_MOD_NPIN       OR
                ui_data->inputMode EQ INP_MOD_NPIN_TWICE   )
              /*
               *-----------------------------------------------------
               * PIN or PUK input is terminated
               *-----------------------------------------------------
               */
              ui_handlePinAndPuk ();
            else
            {
              CHAR* input = inp_getInput (areaUser);

              if (strlen (input) EQ 1   AND
                  *input         >= '0' AND
                  *input         <= '9'    )
              {
                /*
                 *---------------------------------------------------
                 * start abbreviated dialing
                 *---------------------------------------------------
                 */
                inp_setInput (areaUser,
                              cal_getAbbrDial
                                           ((UBYTE) (*input - '0')));
              }
              else
              {
                /*
                 *---------------------------------------------------
                 * add the character to input
                 *---------------------------------------------------
                 */
                inp_charToInput (areaUser, key_pressed);
              }
            }
          }
        break;
      }

      /*
       *-------------------------------------------------------------
       * process '*'-key
       *-------------------------------------------------------------
       */
      case '*':
      {
        if (ui_data->sendDTMF)
          aca_signal (ACA_SIG_SEND_DTMF, key_pressed, key_stat);
        else if (ui_data->state EQ UI_INPUT)
          inp_charToInput (areaUser, key_pressed);

        break;
      }

      /*
       *-------------------------------------------------------------
       * process F1_KEY
       *-------------------------------------------------------------
       */
      case F1_KEY:
      {
        ui_shift (FALSE);

        if (ui_data->state NEQ UI_SELECT)
        {
          /*
           *---------------------------------------------------------
           * show the main menu
           *---------------------------------------------------------
           */
          menu[0] = lng_getText (TXT_MEN_LANGUAGE);
          menu[1] = lng_getText (TXT_MEN_SEND_DTMF);
          menu[2] = lng_getText (TXT_MEN_KEYBEEP);
          menu[3] = NULL;

          ui_topArea (areaMenu);
          wm_showArea (areaMenu);
          men_startMenu (areaMenu, lng_getText (TXT_OPTIONS), menu);

          oldState          = ui_data->state;
          ui_data->state    = UI_SELECT;
          ui_data->menuMode = MEN_MOD_MAIN;
        }
        else
        {
          /*
           *---------------------------------------------------------
           * if a menu (and a submenu) is active close it
           *---------------------------------------------------------
           */
          if (ui_data->inSubmenu)
          {
            men_stopMenu (areaSubMenu);
            wm_hideArea (areaSubMenu);
            ui_data->inSubmenu = FALSE;
          }

          men_stopMenu (areaMenu);
          wm_hideArea (areaMenu);

          ui_data->state    = oldState;
          ui_data->menuMode = MEN_MOD_NONE;
        }
        break;
      }

      /*
       *-------------------------------------------------------------
       * process SHIFT_KEY
       *-------------------------------------------------------------
       */
      case SHIFT_KEY:
        if (!powerKeyOn)
        {
          /*
           *---------------------------------------------------------
           * pressing the shift key in power off state
           * leads to a reload of the software
           *---------------------------------------------------------
           */
          ui_startBooting ();
        }
        else
          ui_shift (!ui_data->shiftOn);
        break;

      /*
       *-------------------------------------------------------------
       * process UP_KEY
       *-------------------------------------------------------------
       */
      case UP_KEY:
        if (ui_data->state EQ UI_SELECT)
        {
          switch (ui_data->menuMode)
          {
            case (MEN_MOD_SMS_LIST):
              if (men_getMenuIdx (areaMenu) EQ 0)
                sms_signal (SMS_SIG_PREV_LIST_VIEW);
              else
                men_upMenu (areaMenu);
              break;

            default:
              if (ui_data->inSubmenu)
                men_upMenu (areaSubMenu);
              else
                men_upMenu (areaMenu);
              break;
          }
        }
        else
          ui_changeVolume (TRUE);

        break;

      /*
       *-------------------------------------------------------------
       * process DOWN_KEY
       *-------------------------------------------------------------
       */
      case DOWN_KEY:
        if (ui_data->state EQ UI_SELECT)
        {
          switch (ui_data->menuMode)
          {
            case (MEN_MOD_SMS_LIST):
              if (smsmenu[men_getMenuIdx (areaMenu) + 1] EQ NULL)
                sms_signal (SMS_SIG_NEXT_LIST_VIEW);
              else
                men_downMenu (areaMenu);
              break;

            default:
              if (ui_data->inSubmenu)
                men_downMenu (areaSubMenu);
              else
                men_downMenu (areaMenu);
              break;
          }
        }
        else
          ui_changeVolume (FALSE);

        break;

      /*
       *-------------------------------------------------------------
       * process CLEAR_KEY
       *-------------------------------------------------------------
       */
      case CLEAR_KEY:
      {
        if (ui_data->state EQ UI_INPUT)
          inp_charToInput (areaUser, '\b');
        break;
      }

      /*
       *-------------------------------------------------------------
       * process any other keys pressed
       *-------------------------------------------------------------
       */
      default:
      {
        if (ui_data->shiftOn)
          ui_data->lastShiftedKey = key_pressed;
        else
          ui_processDefaultKey (key_pressed, key_stat);

        break;
      }
    }
  }
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)       MODULE  : SMI_UI                     |
| STATE   : code                ROUTINE : ui_timeoutSlider           |
+--------------------------------------------------------------------+

  PURPOSE : This function handles the timeout for timer TSLIDER.
*/
GLOBAL void ui_timeoutSlider (void)
{
  TRACE_FUNCTION ("ui_timeoutSlider()");

  if (ui_data->sliderActive)
  {
    sld_stopSlider (areaSlider);

    wm_hideArea (areaSlider);
    ui_data->sliderActive = FALSE;
  }
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)       MODULE  : SMI_UI                     |
| STATE   : code                ROUTINE : ui_timeoutBacklight        |
+--------------------------------------------------------------------+

  PURPOSE : This function handles the timeout for timer TBACKLIGHT.
*/
GLOBAL void ui_timeoutBacklight (void)
{
  TRACE_FUNCTION ("ui_timeoutBacklight()");

  ui_backlight (FALSE);
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)       MODULE  : SMI_UI                     |
| STATE   : code                ROUTINE : ui_timeoutShiftKey         |
+--------------------------------------------------------------------+

  PURPOSE : This function handles the timeout for timer TSHIFTKEY.
*/

GLOBAL void ui_timeoutShiftKey (void)
{
  TRACE_FUNCTION ("ui_timeoutShiftKey()");

  ui_shift (FALSE);
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)       MODULE  : SMI_UI                     |
| STATE   : code                ROUTINE : ui_timeoutUniversal        |
+--------------------------------------------------------------------+

  PURPOSE : This function handles the timeout for timer TUNIVERSAL.
*/

GLOBAL void ui_timeoutUniversal (void)
{
  TRACE_FUNCTION ("ui_timeoutUniversal()");

  switch (ui_data->timerMod)
  {
    case (TIMER_MOD_WELCOME):
      ui_data->timerMod = TIMER_MOD_NONE;

      wm_hideArea (areaUniversal);
      break;

#ifdef BT_ADAPTER
    case (TIMER_MOD_BT):
      ui_data->timerMod = TIMER_MOD_NONE;

      wm_hideArea (areaUniversal);
      break;
#endif /* BT_ADAPTER */

    case (TIMER_MOD_LCD_TEST):
      ui_startLcdTest();
      break;

    case (TIMER_MOD_BOOTING):
      wm_areaWrite (areaUser, 0, 0, "Booting...");
      wm_showArea  (areaUser);
      break;

    case (TIMER_MOD_NIBT):
      if (screensize_y EQ 4 AND ui_data->nibtOn)
      {
        wm_clearArea (areaCaller);
        wm_areaWrite (areaCaller, 0, 0, "");
        wm_showArea  (areaCaller);

        ui_data->nibtOn = FALSE;
      }
      break;

    default:
      break;
  }
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_UI                   |
| STATE   : code                  ROUTINE : ui_isScrnWidthMin        |
+--------------------------------------------------------------------+

  PURPOSE : This function is used to get information about the screen
            width.

            returns: TRUE if used screen is the smaller of two ones,
                     otherwise FALSE
*/
GLOBAL BOOL ui_isScrnWidthMin (void)
{
  if (screensize_y < 4)
    return TRUE;
  else
    return FALSE;
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_UI                   |
| STATE   : code                  ROUTINE : ui_isScrnHeightMin       |
+--------------------------------------------------------------------+

  PURPOSE : This function is used to get information about screen
            height.

            returns: TRUE if used screen is the smaller of two ones,
                     otherwise FALSE
*/
GLOBAL BOOL ui_isScrnHeightMin (void)
{
  if (screensize_y < 4)
    return TRUE;
  else
    return FALSE;
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)         MODULE  : SMI_UI                   |
| STATE   : code                  ROUTINE : ui_sms_signal            |
+--------------------------------------------------------------------+

  PURPOSE : Process the UI_SMS_SIG_xxx signals that are send by PSA
            or IO
*/
GLOBAL void ui_sms_signal (UBYTE sigCode, ...)
{
  va_list varpars;
  CHAR    buf[30];

  sprintf (buf, "UI-SMS Signal: %d", sigCode);
  TRACE_EVENT (buf);

  switch (sigCode)
  {
    /*
     *---------------------------------------------------------------
     * process the signal UI_SIG_SMS_PROC_SUBMIT
     *---------------------------------------------------------------
     */
    case (UI_SIG_SMS_PROC_SUBMIT):
      ui_startNewInput (areaUser,
                        0,
                        0,
                        "MSG: ",
                        MAX_SM_LEN,
                        IT_ALPHA | IT_ECHO,
                        INP_MOD_SM,
                        NULL,
                        TRUE);

      ui_data->inputSeq = INP_SEQ_SMS_SUBMIT;
      break;

    /*
     *---------------------------------------------------------------
     * process the signal UI_SIG_SMS_PROC_SUBMIT_FROM_MEM
     *---------------------------------------------------------------
     */
    case (UI_SIG_SMS_PROC_SUBMIT_FROM_MEM):
      ui_data->inputSeq = INP_SEQ_SMS_SUBMIT_FROM_MEM;

      sms_signal (SMS_SIG_START_LIST_VIEW);
      break;

    /*
     *---------------------------------------------------------------
     * process the signal UI_SIG_SMS_PROC_WRITE
     *---------------------------------------------------------------
     */
    case (UI_SIG_SMS_PROC_WRITE):
      ui_startNewInput (areaUser,
                        0,
                        0,
                        "MSG: ",
                        MAX_SM_LEN,
                        IT_ALPHA | IT_ECHO,
                        INP_MOD_SM,
                        NULL,
                        TRUE);

      ui_data->inputSeq = INP_SEQ_SMS_WRITE;
      break;

    /*
     *---------------------------------------------------------------
     * process the signal UI_SIG_SMS_PROC_DELETE
     *---------------------------------------------------------------
     */
    case (UI_SIG_SMS_PROC_DELETE):
      ui_data->inputSeq = INP_SEQ_SMS_DELETE;

      sms_signal (SMS_SIG_START_LIST_VIEW);
      break;

    /*
     *---------------------------------------------------------------
     * process the signal UI_SIG_SMS_PROC_READ
     *---------------------------------------------------------------
     */
    case (UI_SIG_SMS_PROC_READ):
      ui_data->inputSeq = INP_SEQ_SMS_READ;

      sms_signal (SMS_SIG_START_LIST_VIEW);
      break;

    /*
     *---------------------------------------------------------------
     * process the signal UI_SIG_SMS_PROC_LIST
     *---------------------------------------------------------------
     */
    case (UI_SIG_SMS_PROC_LIST):
      ui_data->inputSeq = INP_SEQ_SMS_LIST;

      sms_signal (SMS_SIG_START_LIST_VIEW);
      break;

    /*
     *---------------------------------------------------------------
     * process the signal UI_SIG_SMS_PROC_READ_SINGLE
     *---------------------------------------------------------------
     */
    case (UI_SIG_SMS_PROC_READ_SINGLE):
      ui_startNewInput (areaUser,
                        0,
                        0,
                        "MEM: ",
                        1,
                        IT_NUMERIC | IT_ECHO,
                        INP_MOD_MEM,
                        NULL,
                        TRUE);

      ui_data->inputSeq = INP_SEQ_SMS_READ_SINGLE;
      break;

    /*
     *---------------------------------------------------------------
     * process the signal UI_SIG_SMS_SHOW_LIST
     *---------------------------------------------------------------
     */
    case (UI_SIG_SMS_SHOW_LIST):
    {
      T_ACA_SMS_TYP type;
      SHORT         idx;
      CHAR*         address;
      CHAR*         data;
      USHORT        i;

      va_start (varpars, sigCode);
      type = va_arg (varpars, T_ACA_SMS_TYP);
      idx  = va_arg (varpars, SHORT);

      if (type EQ SMS_TYP_SM)
        address = va_arg (varpars, CHAR*);

      data = va_arg (varpars, CHAR*);
      va_end (varpars);

      if (idx <= -1)
      {
        /*
         *-----------------------------------------------------------
         * no entries in the SMS list
         *-----------------------------------------------------------
         */
        CHAR* empty = lng_getText (TXT_EMPTY);

        strncpy (smsstring[0], empty, MENU_ENTRY_WIDTH_L);
        smsstring[0][MENU_ENTRY_WIDTH_L] = NULL_TERM;
        smsstring[1][0]                  = NULL_TERM;
      }
      else
      {
        /*
         *-----------------------------------------------------------
         * prepare a single SMS list entry for viewing, the entry
         * is splitted into several lines and handled like a menu
         *
         * 1. row: index SM
         * 2. row: short message content
         * 3. row: destination address
         *
         * or
         *
         * 1. row: index CBM
         * 2. row: cell broadcast message content
         *-----------------------------------------------------------
         */
        if (type EQ SMS_TYP_SM)
        {
          sprintf (smsstring[0], "%d SM", idx);
          strncpy (smsstring[2], address, MENU_ENTRY_WIDTH_L);
          smsstring[2][MENU_ENTRY_WIDTH_L] = NULL_TERM;
          smsstring[3][0]                  = NULL_TERM;
        }
        else
        {
          sprintf (smsstring[0], "%d CBM", idx);
          smsstring[2][0] = NULL_TERM;
        }

        strncpy (smsstring[1], data, MENU_ENTRY_WIDTH_L);
        smsstring[1][MENU_ENTRY_WIDTH_L] = NULL_TERM;
      }

      /*
       *-------------------------------------------------------------
       * build up the menu to hold the single SMS list entry
       * splitted on several menu items
       *-------------------------------------------------------------
       */
      i = 0;

      while (smsstring[i][0] NEQ NULL_TERM)
      {
        smsmenu[i] = smsstring[i];
        i++;
      }

      smsmenu[i] = NULL;

      /*
       *-----------------------------------------------------------
       * show the SMS list entry
       *-----------------------------------------------------------
       */
      men_stopMenu (areaMenu);

      ui_topArea (areaMenu);
      wm_showArea (areaMenu);
      men_startMenu (areaMenu,
                     lng_getText (TXT_MEN_SMS_LIST),
                     smsmenu);

      oldState          = ui_data->state;
      ui_data->state    = UI_SELECT;
      ui_data->menuMode = MEN_MOD_SMS_LIST;
      break;
    }

    /*
     *---------------------------------------------------------------
     * process the signal UI_SIG_SMS_SHOW_MSG
     *---------------------------------------------------------------
     */
    case (UI_SIG_SMS_SHOW_MSG):
    {
      CHAR*  data;
      CHAR*  token;
      USHORT i, j;
      USHORT menuEntryWidth;

      va_start (varpars, sigCode);
      data = va_arg (varpars, CHAR*);
      va_end (varpars);

      if (data NEQ NULL)
      {
        /*
         *-----------------------------------------------------------
         * prepare a single SMS list entry for detailed viewing,
         * the entry is splitted into several lines and handled like
         * a menu
         *
         * 1.row: characters     0..x   of message
         * 2.row: characters   x+1..2*x of message
         * 3.row: characters 2*x+1..3*x of message
         * 4.row: etc.
         *-----------------------------------------------------------
         */
        i     = 0;
        token = data;

        if (screensize_y EQ 4)
          menuEntryWidth = MENU_ENTRY_WIDTH_L;
        else
          menuEntryWidth = MENU_ENTRY_WIDTH_S;

        while (token NEQ NULL AND i < MAX_LINES_PER_MSG)
        {
          strncpy (msgstring[i], token, menuEntryWidth);
          msgstring[i][menuEntryWidth] = NULL_TERM;

          if (strlen (token) <= menuEntryWidth)
            token = NULL;
          else
            token += menuEntryWidth;

          /*
           *-----------------------------------------------------------
           * <CR> and <LF> are replaced by spaces due to the
           * limited display capabilities
           *-----------------------------------------------------------
           */
          for (j = 0; j < strlen (msgstring[i]); j++)
            if (msgstring[i][j] EQ LF OR
                msgstring[i][j] EQ CR )
                msgstring[i][j] = SPACE;

          i++;
        }

        msgstring[i][0] = NULL_TERM;

        /*
         *-----------------------------------------------------------
         * build up the menu to hold the message content splitted on
         * several menu items.
         *-----------------------------------------------------------
         */
        i = 0;

        while (msgstring[i][0] NEQ NULL_TERM)
        {
          msgmenu[i] = msgstring[i];
          i++;
        }

        msgmenu[i] = NULL;

        /*
         *-----------------------------------------------------------
         * show the message content
         *-----------------------------------------------------------
         */
        men_stopMenu (areaMenu);

        ui_topArea (areaMenu);
        wm_showArea (areaMenu);
        men_startMenu (areaMenu,
                       lng_getText (TXT_MEN_MSG_VIEW),
                       msgmenu);

        oldState          = ui_data->state;
        ui_data->state    = UI_SELECT;
        ui_data->menuMode = MEN_MOD_MSG_VIEW;
      }
      break;
    }

    /*
     *---------------------------------------------------------------
     * process signal UI_SIG_SMS_DELETE
     *---------------------------------------------------------------
     */
    case UI_SIG_SMS_DELETE:
    {
      USHORT cause;

      va_start (varpars, sigCode);
      cause = va_arg (varpars, USHORT);
      va_end (varpars);

      wm_clearArea (areaState);
      wm_areaWrite (areaState, 0, 0,
                    ui_getCauseName ("DEL", cause));
      break;
    }

    /*
     *---------------------------------------------------------------
     * process signal UI_SIG_SMS_SUBMIT
     *---------------------------------------------------------------
     */
    case UI_SIG_SMS_SUBMIT:
    {
      UBYTE mr;

      va_start (varpars, sigCode);
      mr = va_arg (varpars, UBYTE);
      va_end (varpars);

      wm_clearArea (areaState);
      wm_areaWrite (areaState, 0, 0,
                    ui_getCauseName ("SBMT", mr));
      break;
    }

    /*
     *---------------------------------------------------------------
     * process signal UI_SIG_SMS_WRITE
     *---------------------------------------------------------------
     */
    case UI_SIG_SMS_WRITE:
    {
      SHORT index;

      va_start (varpars, sigCode);
      index = va_arg (varpars, SHORT);
      va_end (varpars);

      wm_clearArea (areaState);
      wm_areaWrite (areaState, 0, 0,
                    ui_getCauseName ("WRTE", index));
      break;
    }

    /*
     *---------------------------------------------------------------
     * process signal UI_SIG_SMS_MT_IND
     *---------------------------------------------------------------
     */
    case UI_SIG_SMS_MT_IND:
    {
      T_ACI_SMS_STOR mem;
      UBYTE          index;

      CHAR buffer [40];

      va_start (varpars, sigCode);
      mem   = va_arg (varpars, T_ACI_SMS_STOR);
      index = va_arg (varpars, UBYTE);
      va_end (varpars);

      sprintf (buffer, "MTI%d %d", mem, index);

      wm_clearArea (areaState);
      wm_areaWrite (areaState, 0, 0, buffer);
      break;
    }
  }
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-PS (6147)       MODULE  : SMI_UI                     |
| STATE   : code                ROUTINE : ui_startLcdTest            |
+--------------------------------------------------------------------+

  PURPOSE : This function starts the LCD test.
*/
GLOBAL void ui_startLcdTest (void)
{
  char  test   []   = {" !\"#$%&()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\]^_abcdefghijklmnopqrstuvwxyz{|}~"};
  char  subTest[21];

         UBYTE maxChars  = strlen (test);
  static UBYTE readChars = 0;
         UBYTE actWidth;
         UBYTE actHeigth;

  TRACE_FUNCTION ("ui_startLcdTest()");

  actHeigth = (screensize_y < 4 ?  2 :  4);
  actWidth  = (screensize_y < 4 ? 16 : 16);

  if (readChars < maxChars)
  {
    wm_clearArea (areaUniversal);

    while (readChars < maxChars AND actHeigth > 0)
    {
      strncpy (subTest, test + readChars, actWidth);
      subTest[actWidth] = NULL_TERM;
      readChars += actWidth;

      wm_areaWrite (areaUniversal,
                    (UBYTE)(-1 * (actHeigth - screensize_y)),
                    0,
                    subTest);
      actHeigth--;
    }

    ui_topArea   (areaUniversal);
    wm_showArea  (areaUniversal);

    ui_data->timerMod = TIMER_MOD_LCD_TEST;
#if defined (NEW_FRAME)
    TIMERSTART (TLCD_TEST_VALUE, SMI_TUNIVERSAL);
    t_universal_running = TRUE;
#else
    TIMERSTART (TLCD_TEST_VALUE, t_universal_handle);
#endif
  }
  else
  {
    ui_data->timerMod = TIMER_MOD_NONE;

    wm_hideArea (areaUniversal);
  }
}

