/*******************************************************************************

					CONDAT (UK)

********************************************************************************                                                                              

 This software product is the property of Condat (UK) Ltd and may not be
 disclosed to any third party without the express permission of the owner.                                 
                                                                              
********************************************************************************

 $Project name:	                                                      
 $Project code:	                                                           
 $Module:		
 $File:		  	wap_codec.c
 $Revision:		                                                      
                                                                              
 $Author:		Condat(UK)                                                         
 $Date:		                                                          
                                                                               
********************************************************************************
                                                                              
 Description:
 	Encodes and decodes events sent between the MMI task and the WAP task.
 	This file is compiled twice: one object file lives in the bmi directory, the other
 	in the wapaus directory.
                        
********************************************************************************

 $History: wap_codec.c

	15/05/2003 - SPR#1983 - SH - Updated to latest from 1.6.3 version.
	   
 $End

*******************************************************************************/


#ifdef WAP_TASK
#define ENTITY_WAP

/*==== INCLUDES ===================================================*/
#if defined (NEW_FRAME)

#include <stdarg.h>
#include <string.h>
#include <stdlib.h>
#include <stddef.h>
#include "typedefs.h"
#include "pcm.h"
#include "pconst.cdg"
#include "mconst.cdg"
#include "message.h"
#include "ccdapi.h"
#include "vsi.h"
#include "custom.h"
#include "gsm.h"
#include "prim.h"
#include "cnf_wap.h"
#include "mon_wap.h"
#include "pei.h"
#include "tok.h"

#else  /* NEW_FRAME */

#include <string.h>
#include "stddefs.h"
#include "pconst.cdg"
#include "message.h"
#include "ccdapi.h"
#include "custom.h"
#include "gsm.h"
#include "cnf_wap.h"
#include "mon_wap.h"
#include "prim.h"
#include "vsi.h"
#include "pei.h"
#include "tok.h"

#endif /* NEW_FRAME */

#include "dti.h"
#include "wap.h"
#include "cus_wap.h"
#include "capigle.h"

#include "wap_types.h"
#include "wap_codec.h"
#include "wap_wapmmi.h"
#include "psa_waps.h"

#else  /* WAP_TASK */

#include <stdio.h>
#include <string.h>

#if defined (NEW_FRAME)

#include "typedefs.h"
#include "vsi.h"
#include "pei.h"
#include "custom.h"
#include "gsm.h"

#else  /* NEW_FRAME */

#include "stddefs.h"
#include "custom.h"
#include "gsm.h"
#include "vsi.h"

#endif /* NEW_FRAME */

#include "cus_aci.h"
#include "p_sim.h"
#include "pcm.h"

#include "wap_types.h"
#include "wap_codec.h"
#include "ATBWapACI.h"

#endif /* WAP_TASK */



/* Header files necessary for memory allocation */

#ifdef WAP_TASK
#include "wip_mem.h"
#else
#include "Mfw_mfw.h"
#endif

/* Tracing macros */

#ifdef TRACE_WAP_CODEC
#define trace(x) TRACE_EVENT(x)
#define trace_P1(x,a) TRACE_EVENT_P1(x,a)
#define trace_P2(x,a,b) TRACE_EVENT_P2(x,a,b)
#define trace_P3(x,a,b,c) TRACE_EVENT_P3(x,a,b,c)
#define trace_P4(x,a,b,c,d) TRACE_EVENT_P4(x,a,b,c,d)
#define trace_P5(x,a,b,c,d,e) TRACE_EVENT_P5(x,a,b,c,d,e)
#else
#define trace(x) /* */
#define trace_P1(x,a) /* */
#define trace_P2(x,a,b) /* */
#define trace_P3(x,a,b,c) /* */
#define trace_P4(x,a,b,c,d) /* */
#define trace_P5(x,a,b,c,d,e) /* */
#endif


#define WAP_TRACE 0
//#define TRACE_ATBWAPACI
#include "mmiutilapi.h"
#ifdef WAP_TASK
#define TRACE_EVENT(f) WAP_TRACE_EVENT(("wapaus-wap_codec.c<<<talcon,%s",f));
#define TRACE_FUNCTION(f) TRACE_EVENT(f)
#define trace(f) TRACE_FUNCTION(f)
#else
#define TRACE_EVENT(f) WAP_TRACE_EVENT(("bmi-wap_codec.c<<<talcon,%s",f));
#define TRACE_EVENTUART(f) WAP_TRACE_EVENT(("bmi-wap_codec.c<<<talcon,%s",f));
#define TRACE_FUNCTION(f) TRACE_EVENT(f)
#define trace(f) TRACE_FUNCTION(f)
#endif


/*==== LOCAL TYPE DEFINITIONS ==========================================*/

typedef void (*T_WAP_FUNCTION) (void *parameter);

#ifdef WAP_TASK

typedef struct T_WAP_EVENT_INFO_X
{
	UBYTE		event_id;						/* The ID of the event */
	void		*parameter;						/* Pointer to the event data */
	USHORT		size;							/* The size of the event */
	struct T_WAP_EVENT_INFO_X	*next;
}
T_WAP_EVENT_INFO;


typedef struct T_WAP_CODEC_DATA_X
{
	UBYTE				send_forbidden;			/* TRUE if MMI is busy */
	UBYTE				element_count;			/* Number of elements that are currently buffered */
	USHORT				size;					/* Total size of buffered elements in bytes */
	T_WAP_EVENT_INFO	*first_element;			/* Start of a linked list containing element info */
}
T_WAP_CODEC_DATA;


/*==== LOCAL VARIABLES ===============================================*/

static UBYTE codec_initialised = FALSE;		/* FALSE until codec_data has been initialised */
static T_WAP_CODEC_DATA codec_data;			/* Main data for wap coder */


/*******************************************************************************

 $Function:    	wap_coder/mmi_coder

 $Description:	Encode an event to send to either WAP or MMI

 $Returns:		None
 
 $Arguments:	event_id - The identifier of the event
 				parameter - The data block

*******************************************************************************/


void wap_coder(UBYTE event_id, void* parameter)
{
	/*	- Call wap_coder_pack_event() to pack the event into a memory block
		- Call wap_coder_add_element() to add this event to the list
		- Call wap_coder_check_send() to see if the event is to be sent.
		- If so, call wap_coder_send_buffered_list()
	*/
		
	UBYTE *event_store;
	USHORT size;
	
	trace("wap_coder");
//WAP_TRACE_TASK("wap_coder");
	/* Check if codec_data needs to be initialised, i.e. when starting a new buffer */

	if (!codec_initialised)
	{
		memset(&codec_data, 0, sizeof(codec_data));
		codec_initialised = TRUE;
	}
	
	/* Pack the event into allocated memory */
	
	event_store = wap_coder_pack_event(event_id, parameter, &size);

	/* Only add event to list if created correctly */

	if (event_store)
	{
		wap_coder_add_element(event_id, size, event_store);

		if (wap_coder_check_send())
		{
			wap_coder_send_buffered_list();
		}
	}

	return;
	
}


/*******************************************************************************

 $Function:    	wap_coder_add_element

 $Description:	Adds the supplied event as an element to the buffer.

 $Returns:		None
 
 $Arguments:	event_id	- The ID of the event to be buffered
 				parameter	- Pointer to the data block of the event

*******************************************************************************/

void wap_coder_add_element(UBYTE event_id, USHORT size, UBYTE* parameter)
{
	/*	- Increment the total number of elements
		- Create a new element in the event list
		- Store the event_id, size and pointer
	*/
	T_WAP_EVENT_INFO *new_element;
	T_WAP_EVENT_INFO *element;
	USHORT offsetIndex;
	
	trace("wap_coder_add_element");

	new_element = (T_WAP_EVENT_INFO *)wip_malloc(sizeof(T_WAP_EVENT_INFO)); 

	/* If memory allocation fails, don't continue */
	
	if (!new_element)
	{
		TRACE_EVENT("*** WAP memory allocation failed ***");
		return;
	}
	
	if (codec_data.element_count==0)
	{
		codec_data.first_element = new_element; 
	}
	else
	{
		element = codec_data.first_element; 
		
		while (element->next!=NULL)
		{ 
			element = element->next;
		}

		element->next = new_element;
	}

	codec_data.element_count++;
	codec_data.size += size;
	
	new_element->event_id = event_id;
	new_element->size = size;
	new_element->parameter = parameter;
	new_element->next = NULL;

	return;
}


/*******************************************************************************

 $Function:    	wap_coder_clear_elements

 $Description:	Clears all memory associated with the buffering of events before
 				sending, with the exception of the block to be sent itself.
 				- Destroys blocks associated with each individual event
 				- Destroys linked list containing event info

 $Returns:		None.
 
 $Arguments:	None.
 
*******************************************************************************/

void wap_coder_clear_elements()
{
	/*	- Loop through the elements linked list
		- For each element, free the associated memory
		- Then free the memory of the link itself
	*/

	T_WAP_EVENT_INFO *element;
	T_WAP_EVENT_INFO *next_element;

	trace("wap_coder_clear_elements");
	
	element = codec_data.first_element;

	while (element!=NULL)
	{ 
		wap_coder_free_packed_event(element->parameter, element->size); 
		next_element = element->next;
		wip_free((void *)element);
 
		element = next_element;
	}

	codec_data.element_count = 0;
	codec_data.size = 0;
	codec_data.first_element = NULL;

	return;
}


/*******************************************************************************

 $Function:    	wap_coder_send_buffered_list

 $Description:	Send the first item in the events list
 
 $Returns:		None.
 
 $Arguments:	None.
 
*******************************************************************************/

void wap_coder_send_buffered_list()
{
	USHORT event_index;
	T_WAP_EVENT_INFO *element;
	USHORT offsetIndex;
	UBYTE event_id;
	UBYTE *data;
	USHORT size;

	trace("wap_coder_send_buffered_list");

	/* Check that there are events to send */

	if (!codec_initialised || !codec_data.element_count)
		return;

	element = codec_data.first_element;
	event_id = element->event_id;
	data = (UBYTE *)element->parameter;
	size = element->size;

	trace_P1("Number of elements in list: %d", codec_data.element_count);
	trace_P2("Sending first element, event_id = %d, size = %d", event_id, size);

	/* The second element becomes the first element */
	
	codec_data.first_element = element->next;
	codec_data.element_count--;

	/* Destroy the first element */

	wip_free((void *)element);

#ifdef TRACE_WAP_CODEC
	trace("Sending event:");
	for (offsetIndex = 0; (offsetIndex<size && offsetIndex<200); offsetIndex++)
	{
		trace_P4("%8x %2x %3d %c", data+offsetIndex, *(data+offsetIndex), *(data+offsetIndex), *(data+offsetIndex));
	}
#endif

	codec_data.send_forbidden = TRUE;				/* Don't send anything else until MMI response */
	wap_WAP_MMI_IND (event_id, (void*)data, size);

	/* Free the memory */

	wip_free(data);

	return;
}


/*******************************************************************************

 $Function:    	wap_coder_check_send

 $Description:	Check whether the buffered events should be sent to MMI or not.

 $Returns:		TRUE if events are to be sent, FALSE otherwise
 
 $Arguments:	None.
 
*******************************************************************************/

UBYTE	wap_coder_check_send()
{
	trace("wap_coder_check_send");

	/* Can't send if MMI is busy */
	if (codec_data.send_forbidden)
		return FALSE;
		
	return TRUE;
}

/*******************************************************************************

 $Function:    	wap_coder_free_packed_event

 $Description:	Frees memory allocated for a specific block

 $Returns:		None
 
 $Arguments:	event_store	- Pointer to the data block of the event
 				size		- The size of the block in bytes

*******************************************************************************/

void wap_coder_free_packed_event(UBYTE* event_store, USHORT size)
{
	trace("wap_coder_free_packed_event");

	wip_free(event_store);
	
	return;
}


/*******************************************************************************

 $Function:    	wap_decoder

 $Description:	Decode an event received from MMI

 $Returns:		None
 
 $Arguments:	event_id	- ID of the event
 				parameter	- Pointer to the data block of the event

*******************************************************************************/


void wap_decoder(UBYTE event_id, void *parameter)
{
	WAP_TRACE_EVENT(("wap_decoder:%d", event_id) );

	/* Intercept MMI READY event */

	if (event_id==WAP_READY_IND_ID)
	{
		codec_data.send_forbidden = FALSE;
		wap_coder_send_buffered_list();
		return;	/* Don't need to tell WAP about this event */
	}

	/* Intercept TERMINATE event before passing to WAP */

#ifdef MMS_WAP_ENABLED
	if (event_id==WAP_TERMINATE_IND_ID || event_id==WAP_MMSTERMINATE_IND_ID)
#else
	if (event_id==WAP_TERMINATE_IND_ID)
#endif
	{
		wap_coder_clear_elements();
		codec_initialised = FALSE;
		/* This event will be passed on to WAP */
	}

	/* Process normal events */
	
	wap_decoder_unpack_event(event_id, parameter);
	
	return;

}


#else /* WAP_TASK */
/*******************************************************************************

 $Function:    	mmi_coder

 $Description:	Encode an event and send it to WAP

 $Returns:		None
 
 $Arguments:	event_id	- The ID of the event to be buffered
 				parameter	- Pointer to the data block of the event

*******************************************************************************/

void mmi_coder(UBYTE event_id, void* parameter)
{
	UBYTE *event_store;
	USHORT size;
trace("mmi_coder"); 
	//TRACE_EVENTUART("mmi_coder");
	//WAP_TRACE_QUEUE();

	/* Pack the event into allocated memory */
	
	event_store = mmi_coder_pack_event(event_id, parameter, &size);

	/* Only send event if created correctly */

	if (event_store)
	{
		WAP_TRACE_EVENT(("hyl,mmi send event to wap- size:%d",size));
    		psaMMI_WAP_IND(event_id, (void*)event_store, size);
		WAP_TRACE_EVENT(("hyl,mmi send event to wap end"));

		mmi_coder_free_packed_event(event_store, size);
	}
	//TRACE_EVENTUART("end mmi_coder");
	return;
}


/*******************************************************************************

 $Function:    	[wap_coder/mmi_coder]_free_packed_event

 $Description:	Frees memory allocated for a specific block

 $Returns:		None
 
 $Arguments:	event_store	- Pointer to the data block of the event
 				size		- The size of the block in bytes

*******************************************************************************/

void mmi_coder_free_packed_event(UBYTE* event_store, USHORT size)
{
	trace("mmi_coder_free_packed_event");
	mfwFree(event_store, size);

	return;
}


/*******************************************************************************

 $Function:    	mmi_decoder

 $Description:	Decode an event received from WAP

 $Returns:		None
 
 $Arguments:	 event_id	- ID of the event
 				parameter	- Pointer to the data block of the event

*******************************************************************************/


void mmi_decoder(UBYTE  event_id, void *parameter)
{
	T_MMI_WAP_READY_IND ready;
	
	trace("mmi_decoder"); 
	
	mmi_decoder_unpack_event(event_id, parameter);

	/* Inform WAP that MMI is ready again */

	ready.dummy = 0;
	M_MMI_WAP_READY_IND(&ready);
	
	return;
}
#endif

/* SHARED FUNCTIONS */

/*******************************************************************************

 $Function:    	[wap_coder/mmi_coder]_pack_event

 $Description:	Allocates memory for the event, then packs all its components together
 				in this memory.

 $Returns:		A pointer to the allocated memory
 
 $Arguments:	event_id	- The ID of the event to be buffered
 				parameter	- Pointer to the data block of the event
 				size		- Pointer to USHORT where the size of the block will be returned

*******************************************************************************/

#ifdef WAP_TASK
UBYTE * wap_coder_pack_event(UBYTE event_id, void* parameter, USHORT *size) {
#else
UBYTE * mmi_coder_pack_event(UBYTE event_id, void* parameter, USHORT *size){
#endif
	UBYTE				offsetIndex;
	UBYTE 				*memSrc;
	UBYTE				*memDest;
	UBYTE				*event_store;
	UBYTE				**blockSrcPtr;
	UBYTE				*blockSrc;
	USHORT				block_length;
	UBYTE				unit_size;
	USHORT				block_size;
	USHORT				param_size;
	USHORT				total_size;			/* Total size of packet in bytes */
	USHORT				offset_table[6][2];

#ifdef WAP_TASK
	trace("wap_coder_pack_event");
#else
	trace("mmi_coder_pack_event");
#endif
	
	/* Get the offset index for this event type, and set the total size so far to the structure size */
	
#ifdef WAP_TASK
	wap_offset_table(event_id, offset_table, &param_size);
#else
	mmi_offset_table(event_id, offset_table, &param_size);
#endif

	/* Calculate the total size in bytes, for memory allocation */

	total_size = param_size+sizeof(U32);	/* Total size of parameter block */
	
	for (offsetIndex = 0; offset_table[offsetIndex][0] != 0xFFFF; offsetIndex++)  /* Find all pointers to memory blocks using offsets */
	{
		memSrc = (UBYTE*)parameter+offset_table[offsetIndex][0];				/* Point to size of next block */
		block_length = *memSrc+256*(*(memSrc+1))+1;								/* Find the length of the block in units */
		unit_size = offset_table[offsetIndex][1];								/* Find the unit size in bytes */
		total_size += block_length*(USHORT)unit_size+sizeof(U32);				/* Add to total size, including initial size U32 */
	}

	WAP_TRACE_EVENT(("Alloc memory for event, size: %d", total_size));
	
	/* Allocate the memory and fill it */
	
#ifdef WAP_TASK
	event_store = (UBYTE*)wip_malloc(total_size);	/* Allocate memory using wip_mem */

	/* Do not continue if memory allocation fails */
	
#else
    event_store = (UBYTE*)mfwAlloc(total_size);		/* Allocate memory in MFW */
#endif

	if (!event_store)
	{
		TRACE_EVENT("*** WAP memory allocation failed ***");
		return NULL;
	}
	
	memDest = event_store;							/*  Pointer for writing data (destination) */
	memSrc	= (UBYTE*)parameter;					/*  Pointer for reading data (source) */

	memcpy(memDest, &param_size, sizeof(USHORT));	/*  Store size of data structure in first word */
	memDest += sizeof(USHORT);
	*(memDest++) = 0;
	*(memDest++) = 0;
	
    memcpy(memDest, memSrc, param_size);			/* Copy the data structure */
    memDest += param_size;

	for (offsetIndex = 0; offset_table[offsetIndex][0] != 0xFFFF; offsetIndex++)  /*  Copy each additional block of data (strings etc) */
	{
		WAP_TRACE_EVENT(("Copying block of data #%d", offsetIndex));
		
		memSrc = (UBYTE*)parameter+offset_table[offsetIndex][0];		/* Point to block length */

		block_length = *memSrc+256*(*(memSrc+1))+1;						/* Find the length of the block in units */

		WAP_TRACE_EVENT(( "Block length at %8x is %d",memSrc, block_length));	
		
		memSrc += sizeof(U32);											/* Point to pointer to block */
		
		unit_size = offset_table[offsetIndex][1];						/* Find the unit size in bytes */
		block_size = block_length*(USHORT)unit_size;					/* Find the block size in bytes */

		memcpy(memDest, &block_size, sizeof(USHORT));					/* Store size of block */
		memDest += sizeof(U32);											/* Point to where block is to be stored */

		if (block_size!=0)												/* If the block isn't empty */
		{
			blockSrcPtr = (UBYTE**)memSrc;										
			blockSrc = *blockSrcPtr;									/* Get the pointer to the block */
			
			WAP_TRACE_EVENT(( "Old block pointer is at: %x", blockSrcPtr));
			WAP_TRACE_EVENT(( "Old block at: %x", blockSrc));		

	 		memcpy(memDest, blockSrc, block_size);						/* Copy block */

			WAP_TRACE_EVENT(( "Copy block: %x to %x, offset %d", blockSrc, memDest, memDest-event_store));
			WAP_TRACE_EVENT(( "New block: %x", memDest));		
			
	 		memDest += block_size;										/* Point to next free block */
		}
	}

#ifdef TRACE_WAP_CODEC
	trace_P1( "Size of packed event: %d", total_size);
	trace("Memory dump - packed event");
	
	for (offsetIndex = 0; (offsetIndex<total_size && offsetIndex<80); offsetIndex++)
	{
		trace_P4("%8x %2x %3d %c", event_store+offsetIndex, *(event_store+offsetIndex), *(event_store+offsetIndex), *(event_store+offsetIndex));
	}
#endif /*TRACE_WAP_CODEC */

	/* Store the size of the block */

	*size = total_size;

	return event_store;
}


/*******************************************************************************

 $Function:    	[wap/mmi]_decoder_unpack_event
 
 $Description:	Using the offset table, this function corrects all the pointers in the block
 				of data received (see wap_offset_table for an explanation of the offset
 				table).

 $Returns:		None.
 
 $Arguments:	event_id	- the id of the event
 				data		- the data received with the primitive

*******************************************************************************/

#ifdef WAP_TASK	
void wap_decoder_unpack_event(UBYTE event_id, void* data) {
#else
void mmi_decoder_unpack_event(UBYTE event_id, void* data) {
#endif

	UBYTE 					*parameter;
	UBYTE					*event_store = (UBYTE *) data;
	UBYTE					*destPtr;
	UBYTE					*srcPtr;
	UBYTE					**ptrPtr;
	USHORT					block_size;
	USHORT					param_size;
	USHORT					total_size;
	UBYTE					offsetIndex;
	USHORT					offset_table[6][2];
    UBYTE                  tempLcdStatus;
#ifdef WAP_TASK	
	static T_WAP_FUNCTION	functionList[] = {
	(T_WAP_FUNCTION)W_MMI_WAP_READY_IND,
	(T_WAP_FUNCTION)W_MMI_WAP_START_IND,
	(T_WAP_FUNCTION)W_MMI_WAP_NEW_VIEW_IND,
	(T_WAP_FUNCTION)W_MMI_WAP_TERMINATE_IND,
	(T_WAP_FUNCTION)W_MMI_WAP_CLOSE_VIEW_IND,
	(T_WAP_FUNCTION)W_MMI_WAP_CONFIGURE_IND,
	(T_WAP_FUNCTION)W_MMI_WAP_CONNECTION_CONFIGURE_IND,
	(T_WAP_FUNCTION)W_MMI_WAP_DOWNLOAD_URL_IND,
	(T_WAP_FUNCTION)W_MMI_WAP_BROWSE_CONTROL_IND,
	(T_WAP_FUNCTION)W_MMI_WAP_KEY_SELECTED_IND,
	(T_WAP_FUNCTION)W_MMI_WAP_INPUT_DIALOG_CNF,
	(T_WAP_FUNCTION)W_MMI_WAP_PASSWORD_DIALOG_CNF,
	(T_WAP_FUNCTION)W_MMI_WAP_CONFIRM_DIALOG_CNF,
	(T_WAP_FUNCTION)W_MMI_WAP_INFO_DIALOG_CNF,
	(T_WAP_FUNCTION)W_MMI_WAP_DRAW_CARD_REQ,
	(T_WAP_FUNCTION)W_MMI_WAP_CONNECT_CNF,
	(T_WAP_FUNCTION)W_MMI_WAP_DISCONNECT_IND,
	(T_WAP_FUNCTION)W_MMI_WAP_CACHE_PREPARE_IND,
	(T_WAP_FUNCTION)W_WAP_MMI_CONTROL_IND,
	(T_WAP_FUNCTION)W_WAP_MMI_NEW_KEY_IND,
	(T_WAP_FUNCTION)W_WAP_MMI_INPUT_DIALOG_REQ	,
	(T_WAP_FUNCTION)W_WAP_MMI_PASSWORD_DIALOG_REQ,
	(T_WAP_FUNCTION)W_WAP_MMI_CONFIRM_DIALOG_REQ,
	(T_WAP_FUNCTION)W_WAP_MMI_INFO_DIALOG_REQ,
	(T_WAP_FUNCTION)W_WAP_MMI_NEW_CARD_IND,
	(T_WAP_FUNCTION)W_WAP_MMI_DRAW_CARD_IND,
	(T_WAP_FUNCTION)W_WAP_MMI_CLEAR_CARD_IND,
	(T_WAP_FUNCTION)W_WAP_MMI_SEND_TEXT_IND,
	(T_WAP_FUNCTION)W_WAP_MMI_SEND_IMAGE_IND,
	(T_WAP_FUNCTION)W_WAP_MMI_SEND_TABLE_IND,
	(T_WAP_FUNCTION)W_WAP_MMI_CARD_DRAW_COMPLETED_IND,
	(T_WAP_FUNCTION)W_WAP_MMI_CONNECT_REQ,
	(T_WAP_FUNCTION)W_WAP_MMI_DISCONNECTED_IND,
	(T_WAP_FUNCTION)W_WAP_MMI_READY_CNF
	/*talcon add for mms*/
#ifdef MMS_WAP_ENABLED
	,
	(T_WAP_FUNCTION)W_MMI_WAP_MMSSTART_IND,
	(T_WAP_FUNCTION)W_MMI_WAP_MMSTERMINATE_IND,
	(T_WAP_FUNCTION)W_MMI_WAP_POSTCONTENT_IND,
	(T_WAP_FUNCTION)W_WAP_MMI_CONTENT_REQ,
	(T_WAP_FUNCTION)W_MMI_WAP_POSTMORECONTENT_IND,
	(T_WAP_FUNCTION)W_WAP_MMI_ACKPOSTCONTENT_REQ,
	(T_WAP_FUNCTION)W_MMI_WAP_GETCONTENT_IND,
	(T_WAP_FUNCTION)W_MMI_WAP_ACKCONTENT_IND,
	(T_WAP_FUNCTION)W_MMI_WAP_CANCELCONTENT_IND
#endif
	};
#else
	static T_WAP_FUNCTION		functionList[] = {
	(T_WAP_FUNCTION)M_MMI_WAP_READY_IND,
	(T_WAP_FUNCTION)M_MMI_WAP_START_IND,
	(T_WAP_FUNCTION)M_MMI_WAP_NEW_VIEW_IND,
	(T_WAP_FUNCTION)M_MMI_WAP_TERMINATE_IND,
	(T_WAP_FUNCTION)M_MMI_WAP_CLOSE_VIEW_IND,
	(T_WAP_FUNCTION)M_MMI_WAP_CONFIGURE_IND,
	(T_WAP_FUNCTION)M_MMI_WAP_CONNECTION_CONFIGURE_IND,
	(T_WAP_FUNCTION)M_MMI_WAP_DOWNLOAD_URL_IND,
	(T_WAP_FUNCTION)M_MMI_WAP_BROWSE_CONTROL_IND,
	(T_WAP_FUNCTION)M_MMI_WAP_KEY_SELECTED_IND,
	(T_WAP_FUNCTION)M_MMI_WAP_INPUT_DIALOG_CNF,
	(T_WAP_FUNCTION)M_MMI_WAP_PASSWORD_DIALOG_CNF,
	(T_WAP_FUNCTION)M_MMI_WAP_CONFIRM_DIALOG_CNF,
	(T_WAP_FUNCTION)M_MMI_WAP_INFO_DIALOG_CNF,
	(T_WAP_FUNCTION)M_MMI_WAP_DRAW_CARD_REQ,
	(T_WAP_FUNCTION)M_MMI_WAP_CONNECT_CNF,
	(T_WAP_FUNCTION)M_MMI_WAP_DISCONNECT_IND,
	(T_WAP_FUNCTION)M_MMI_WAP_CACHE_PREPARE_IND,
	(T_WAP_FUNCTION)M_WAP_MMI_CONTROL_IND,
	(T_WAP_FUNCTION)M_WAP_MMI_NEW_KEY_IND,
	(T_WAP_FUNCTION)M_WAP_MMI_INPUT_DIALOG_REQ	,
	(T_WAP_FUNCTION)M_WAP_MMI_PASSWORD_DIALOG_REQ,
	(T_WAP_FUNCTION)M_WAP_MMI_CONFIRM_DIALOG_REQ,
	(T_WAP_FUNCTION)M_WAP_MMI_INFO_DIALOG_REQ,
	(T_WAP_FUNCTION)M_WAP_MMI_NEW_CARD_IND,
	(T_WAP_FUNCTION)M_WAP_MMI_DRAW_CARD_IND,
	(T_WAP_FUNCTION)M_WAP_MMI_CLEAR_CARD_IND,
	(T_WAP_FUNCTION)M_WAP_MMI_SEND_TEXT_IND,
	(T_WAP_FUNCTION)M_WAP_MMI_SEND_IMAGE_IND,
	(T_WAP_FUNCTION)M_WAP_MMI_SEND_TABLE_IND,
	(T_WAP_FUNCTION)M_WAP_MMI_CARD_DRAW_COMPLETED_IND,
	(T_WAP_FUNCTION)M_WAP_MMI_CONNECT_REQ,
	(T_WAP_FUNCTION)M_WAP_MMI_DISCONNECTED_IND,
	(T_WAP_FUNCTION)M_WAP_MMI_READY_CNF
#ifdef MMS_WAP_ENABLED	
	/*talcon add for mms*/
	,
	(T_WAP_FUNCTION)M_MMI_WAP_MMSSTART_IND,
	(T_WAP_FUNCTION)M_MMI_WAP_MMSTERMINATE_IND,
	(T_WAP_FUNCTION)M_MMI_WAP_POSTCONTENT_IND,
	(T_WAP_FUNCTION)M_WAP_MMI_CONTENT_REQ,
	(T_WAP_FUNCTION)M_MMI_WAP_POSTMORECONTENT_IND,
	(T_WAP_FUNCTION)M_WAP_MMI_ACKPOSTCONTENT_REQ,
	(T_WAP_FUNCTION)M_MMI_WAP_GETCONTENT_IND,
	(T_WAP_FUNCTION)M_MMI_WAP_ACKCONTENT_IND,
	(T_WAP_FUNCTION)M_MMI_WAP_CANCELCONTENT_IND
#endif
	};
#endif

#ifndef WAP_TASK
	trace("mmi_decoder_unpack_event");
#else
	trace("wap_decoder_unpack_event");
#endif	

	/* Calculate the offset table */
	
#ifdef WAP_TASK
	wap_offset_table(event_id, offset_table, &param_size);
#else
    tempLcdStatus = dspl_Enable(0);
	mmi_offset_table(event_id, offset_table, &param_size);
#endif
	
	/* Using the offset table, reconnect the pointers of the structure to the other data blocks */
				 
	total_size 			= 0;
	parameter			= event_store+sizeof(U32);								/* 'parameter' is data to be passed to function */
	destPtr				= parameter;											/* Point to start of data block */
	
	for (offsetIndex = 0; offset_table[offsetIndex][0] != 0xFFFF; offsetIndex++)
	{
		trace_P1("Reconnecting pointer %d", offsetIndex);
		srcPtr		= parameter + offset_table[offsetIndex][0]+sizeof(U32);		/* Point to next pointer to be changed */
		block_size	= *(destPtr-4)+256*(*(destPtr-3));							/* Size of block in bytes */
		total_size 	+= block_size+sizeof(U32);
		destPtr		+= block_size+sizeof(U32);									/* Point to beginning of next data block */
		memcpy(srcPtr, &destPtr, sizeof(UBYTE*));								/* Copy the pointer */
	}
	block_size	= *(destPtr-4)+256*(*(destPtr-3));								/* Size of last block in bytes */
	total_size 	+= block_size+sizeof(U32);
	
	WAP_TRACE_EVENT(( "Size of packet received: %d", total_size));

	/* Call the required function */
	
	(*functionList[event_id]) (parameter);

#ifndef WAP_TASK
    dspl_Enable(tempLcdStatus);
#endif	

	return;
}


/*******************************************************************************

 $Function:    	wap_offset_table

 $Description:	The data structure that is transmitted in the event will have pointers
 				that point to blocks of data (e.g. text).  These blocks are bundled
 				together with the data structure itself and sent, but on receipt the
 				pointers are no longer correct.  The offset table stores the *position*
 				of these pointers in the data structure, relative to the base address
 				of the data structure itself.  This permits the decoder to correctly
 				reassign these pointers.

 $Returns:		None.

 $Arguments:	struct_size  - pointer to ushort variable that holds length of structure
 				in bytes
 
*******************************************************************************/

#ifdef WAP_TASK	
void wap_offset_table(UBYTE event_id, USHORT offset_table[][2], USHORT *struct_size) {
#else
void mmi_offset_table(UBYTE event_id, USHORT offset_table[][2], USHORT *struct_size) {
#endif

	UBYTE offsetIndex;
	
	/* On receiving the event, we need to have access to the data structures
	 * in order to decode the data block we receive.
	 */
	
		T_MMI_WAP_NEW_VIEW_IND					*newView;
		T_MMI_WAP_CONFIGURE_IND					*configure;
		T_MMI_WAP_CONNECTION_CONFIGURE_IND		*connectionConfigure;
		T_MMI_WAP_DOWNLOAD_URL_IND 				*downloadURL;
		T_MMI_WAP_INPUT_DIALOG_CNF 				*inputDialogCnf;
		T_MMI_WAP_PASSWORD_DIALOG_CNF 			*passwordDialogCnf;
		T_WAP_MMI_NEW_KEY_IND					*newKey;
		T_WAP_MMI_INPUT_DIALOG_REQ				*inputDialogReq;
		T_WAP_MMI_PASSWORD_DIALOG_REQ			*passwordDialogReq;
		T_WAP_MMI_CONFIRM_DIALOG_REQ			*confirmDialogReq;
		T_WAP_MMI_INFO_DIALOG_REQ				*infoDialogReq;
		T_WAP_MMI_NEW_CARD_IND					*newCard;
		T_WAP_MMI_SEND_TEXT_IND					*sendText;
		T_WAP_MMI_SEND_IMAGE_IND				*sendImage;
		T_WAP_MMI_SEND_TABLE_IND				*sendTable;
			 
#ifdef WAP_TASK	
	trace("wap_offset_table");
#else
	trace("mmi_offset_table");
#endif	

	/* Which type of event is it? */
		
	switch(event_id)
	{
		/* These types have dynamic fields. */

		case WAP_NEW_VIEW_IND_ID:
			offset_table[0][0] = (USHORT)&newView->font_width_length - (USHORT)newView;
			offset_table[0][1] = sizeof(UBYTE);
			offset_table[1][0] = 0xFFFF;
			*struct_size = (USHORT)sizeof(T_MMI_WAP_NEW_VIEW_IND);
			break;
			
		case WAP_CONFIGURE_IND_ID:
			offset_table[0][0] = (USHORT)&configure->length - (USHORT)configure;
			offset_table[0][1] = sizeof(char);
			offset_table[1][0] = 0xFFFF;
			*struct_size = sizeof(T_MMI_WAP_CONFIGURE_IND);
			break;
		
		case WAP_CONNECTION_CONFIGURE_IND_ID:
			offset_table[0][0] = (USHORT)&connectionConfigure->length - (USHORT)connectionConfigure;
			offset_table[0][1] = sizeof(char);
			offset_table[1][0] = 0xFFFF;
			*struct_size = sizeof(T_MMI_WAP_CONNECTION_CONFIGURE_IND);
			break;
			
		case WAP_DOWNLOAD_URL_IND_ID:
			offset_table[0][0] = (USHORT)&downloadURL->url_length - (USHORT)downloadURL;
			offset_table[0][1] = sizeof(char);
			offset_table[1][0] = 0xFFFF;
			*struct_size = sizeof(T_MMI_WAP_DOWNLOAD_URL_IND);
			break;

		case WAP_INPUT_DIALOG_CNF_ID:
			offset_table[0][0] = (USHORT)&inputDialogCnf->text_length - (USHORT)inputDialogCnf;
			offset_table[0][1] = sizeof(USHORT);
			offset_table[1][0] = 0xFFFF;
			*struct_size = sizeof(T_MMI_WAP_INPUT_DIALOG_CNF);
			break;
		
		case WAP_PASSWORD_DIALOG_CNF_ID:
			offset_table[0][0] = (USHORT)&passwordDialogCnf->input_length - (USHORT)passwordDialogCnf;
			offset_table[0][1] = sizeof(char);
			offset_table[1][0] = (USHORT)&passwordDialogCnf->password_length - (USHORT)passwordDialogCnf;
			offset_table[1][1] = sizeof(char);
			offset_table[2][0] = 0xFFFF;
			*struct_size = sizeof(T_MMI_WAP_PASSWORD_DIALOG_CNF);
			break;

		/* These types do not have any dynamic fields. */

		case WAP_READY_IND_ID:
			*struct_size = (USHORT)sizeof(T_MMI_WAP_READY_IND);
			offset_table[0][0] = 0xFFFF;
			break;
		case WAP_START_IND_ID:
			*struct_size = (USHORT)sizeof(T_MMI_WAP_START_IND);
			offset_table[0][0] = 0xFFFF;
			break;
		case WAP_TERMINATE_IND_ID:
			*struct_size = sizeof(T_MMI_WAP_TERMINATE_IND);
			offset_table[0][0] = 0xFFFF;
			break;
		case WAP_CLOSE_VIEW_IND_ID:
			*struct_size = sizeof(T_MMI_WAP_CLOSE_VIEW_IND);
			offset_table[0][0] = 0xFFFF;
			break;
		case WAP_BROWSE_CONTROL_IND_ID:
			*struct_size = sizeof(T_MMI_WAP_BROWSE_CONTROL_IND);
			offset_table[0][0] = 0xFFFF;
			break;
		case WAP_KEY_SELECTED_IND_ID:
			*struct_size = sizeof(T_MMI_WAP_KEY_SELECTED_IND);
			offset_table[0][0] = 0xFFFF;
			break;
		case WAP_CONFIRM_DIALOG_CNF_ID:
			*struct_size = sizeof(T_MMI_WAP_CONFIRM_DIALOG_CNF);
			offset_table[0][0] = 0xFFFF;
			break;
		case WAP_INFO_DIALOG_CNF_ID:
			*struct_size = sizeof(T_MMI_WAP_INFO_DIALOG_CNF);
			offset_table[0][0] = 0xFFFF;
			break;
		case WAP_DRAW_CARD_REQ_ID:
			*struct_size = sizeof(T_MMI_WAP_DRAW_CARD_REQ);
			offset_table[0][0] = 0xFFFF;
			break;
		case WAP_DISCONNECT_IND_ID:
			*struct_size = sizeof(T_MMI_WAP_DISCONNECT_IND);
			offset_table[0][0] = 0xFFFF;
			break;
		case WAP_CONNECT_CNF_ID:
			*struct_size = sizeof(T_MMI_WAP_CONNECT_CNF);
			offset_table[0][0] = 0xFFFF;
			break;
		case WAP_READY_CNF_ID:
			*struct_size = (USHORT)sizeof(T_WAP_MMI_READY_CNF);
			offset_table[0][0] = 0xFFFF;
			break;
		case WAP_CACHE_PREPARE_IND_ID:
			*struct_size = sizeof(T_MMI_WAP_CACHE_PREPARE_IND);
			offset_table[0][0] = 0xFFFF;
			break;
		
		/* These types have dynamic fields. */
		
		case WAP_NEW_KEY_IND_ID:
			offset_table[0][0] = (USHORT)&newKey->label_length - (USHORT)newKey;
			offset_table[0][1] = sizeof(USHORT);
			offset_table[1][0] = 0xFFFF;
			*struct_size = sizeof(T_WAP_MMI_NEW_KEY_IND);
			break;
		
		case WAP_INPUT_DIALOG_REQ_ID:
			offset_table[0][0] = (USHORT)&inputDialogReq->title_length - (USHORT)inputDialogReq;
			offset_table[0][1] = sizeof(USHORT);
			offset_table[1][0] = (USHORT)&inputDialogReq->input_length - (USHORT)inputDialogReq;
			offset_table[1][1] = sizeof(USHORT);
			offset_table[2][0] = (USHORT)&inputDialogReq->format_length - (USHORT)inputDialogReq;
			offset_table[2][1] = sizeof(USHORT);
			offset_table[3][0] = 0xFFFF;
			*struct_size = sizeof(T_WAP_MMI_INPUT_DIALOG_REQ);
			break;
		
		case WAP_PASSWORD_DIALOG_REQ_ID:
			/* SPR#2393 - SH - Change parameters of password event */
			offset_table[0][0] = (USHORT)&passwordDialogReq->realm_length - (USHORT)passwordDialogReq;
			offset_table[0][1] = sizeof(char);
			offset_table[1][0] = 0xFFFF;
			*struct_size = sizeof(T_WAP_MMI_PASSWORD_DIALOG_REQ);
			break;
		
		case WAP_CONFIRM_DIALOG_REQ_ID:
			offset_table[0][0] = (USHORT)&confirmDialogReq->message_length - (USHORT)confirmDialogReq;
			offset_table[0][1] = sizeof(USHORT);
			offset_table[1][0] = (USHORT)&confirmDialogReq->ok_key_length - (USHORT)confirmDialogReq;
			offset_table[1][1] = sizeof(USHORT);
			offset_table[2][0] = (USHORT)&confirmDialogReq->cancel_key_length - (USHORT)confirmDialogReq;
			offset_table[2][1] = sizeof(USHORT);
			offset_table[3][0] = 0xFFFF;
			*struct_size = sizeof(T_WAP_MMI_CONFIRM_DIALOG_REQ);
			break;
		
		case WAP_INFO_DIALOG_REQ_ID:
			offset_table[0][0] = (USHORT)&infoDialogReq->message_length - (USHORT)infoDialogReq;
			offset_table[0][1] = sizeof(USHORT);
			offset_table[1][0] = 0xFFFF;
			*struct_size = sizeof(T_WAP_MMI_INFO_DIALOG_REQ);
			break;

		case WAP_NEW_CARD_IND_ID:
			offset_table[0][0] = (USHORT)&newCard->title_length - (USHORT)newCard;
			offset_table[0][1] = sizeof(USHORT);
			offset_table[1][0] = (USHORT)&newCard->url_length - (USHORT)newCard;
			offset_table[1][1] = sizeof(char);
			offset_table[2][0] = (USHORT)&newCard->history_length - (USHORT)newCard;
			offset_table[2][1] = sizeof(U32);
			offset_table[3][0] = 0xFFFF;
			*struct_size = sizeof(T_WAP_MMI_NEW_CARD_IND);
			break;

		case WAP_SEND_TEXT_IND_ID:
			offset_table[0][0] = (USHORT)&sendText->text_length - (USHORT)sendText;
			offset_table[0][1] = sizeof(USHORT);
			offset_table[1][0] = 0xFFFF;
			*struct_size = sizeof(T_WAP_MMI_SEND_TEXT_IND);
			break;
		
		case WAP_SEND_IMAGE_IND_ID:
			offset_table[0][0] = (USHORT)&sendImage->image_length - (USHORT)sendImage;
			offset_table[0][1] = sizeof(UBYTE);
			offset_table[1][0] = 0xFFFF;
			*struct_size = sizeof(T_WAP_MMI_SEND_IMAGE_IND);
			break;

		case WAP_SEND_TABLE_IND_ID:
			offset_table[0][0] = (USHORT)&sendTable->title_length - (USHORT)sendTable;
			offset_table[0][1] = sizeof(USHORT);
			offset_table[1][0] = (USHORT)&sendTable->cols_length - (USHORT)sendTable;
			offset_table[1][1] = sizeof(SHORT);
			offset_table[2][0] = (USHORT)&sendTable->rows_length - (USHORT)sendTable;
			offset_table[2][1] = sizeof(SHORT);
			offset_table[3][0] = 0xFFFF;
			*struct_size = sizeof(T_WAP_MMI_SEND_TABLE_IND);
			break;

		/* These types do not have any dynamic fields. */

		case WAP_CONTROL_IND_ID:
			*struct_size = sizeof(T_WAP_MMI_CONTROL_IND);
			offset_table[0][0] = 0xFFFF;		
			break;
		case WAP_DRAW_CARD_IND_ID:
			*struct_size = sizeof(T_WAP_MMI_DRAW_CARD_IND);
			offset_table[0][0] = 0xFFFF;
			break;
		case WAP_CLEAR_CARD_IND_ID:
			*struct_size = sizeof(T_WAP_MMI_CLEAR_CARD_IND);
			offset_table[0][0] = 0xFFFF;
			break;
		case WAP_CARD_DRAW_COMPLETED_IND_ID:
			*struct_size = sizeof(T_WAP_MMI_CARD_DRAW_COMPLETED_IND);
			offset_table[0][0] = 0xFFFF;
			break;
		case WAP_CONNECT_REQ_ID:
			*struct_size = sizeof(T_WAP_MMI_CONNECT_REQ);
			offset_table[0][0] = 0xFFFF;
			break;
		case WAP_DISCONNECTED_IND_ID:
			*struct_size = sizeof(T_WAP_MMI_DISCONNECTED_IND);
			offset_table[0][0] = 0xFFFF;
			break;

#ifdef MMS_WAP_ENABLED			
		/*talcon add for mms*/
		case WAP_MMSSTART_IND_ID:
			*struct_size = (USHORT)sizeof(T_MMI_WAP_MMSSTART_IND);
			offset_table[0][0] = 0xFFFF;
			break;
		case WAP_MMSTERMINATE_IND_ID:
			*struct_size = (USHORT)sizeof(T_MMI_WAP_MMSTERMINATE_IND);
			offset_table[0][0] = 0xFFFF;
			break;

		case WAP_POSTCONTENT_IND_ID:
		{
			T_MMI_WAP_POSTCONTENT_IND				*postContent = 0;
			*struct_size = (USHORT)sizeof(T_MMI_WAP_POSTCONTENT_IND);
			offset_table[0][0] = (USHORT)&postContent->addr_length - (USHORT)postContent;
			offset_table[0][1] = sizeof(UBYTE);
			offset_table[1][0] = (USHORT)&postContent->acceptHeader_length - (USHORT)postContent;
			offset_table[1][1] = sizeof(UBYTE);
			offset_table[2][0] = (USHORT)&postContent->data_length - (USHORT)postContent;
			offset_table[2][1] = sizeof(UBYTE);
			offset_table[3][0] = (USHORT)&postContent->contentType_length - (USHORT)postContent;
			offset_table[3][1] = sizeof(UBYTE);
			offset_table[4][0] = (USHORT)&postContent->contentDisp_length - (USHORT)postContent;
			offset_table[4][1] = sizeof(UBYTE);
			offset_table[5][0] = 0xFFFF;
		}
			break;
		case WAP_CONTENT_REQ_ID:
		{
			T_WAP_MMI_CONTENT_REQ					*contentReq;
			*struct_size = (USHORT)sizeof(T_WAP_MMI_CONTENT_REQ);
			offset_table[0][0] = (USHORT)&contentReq->data_length - (USHORT)contentReq;
			offset_table[0][1] = sizeof(UBYTE);
			offset_table[1][0] = (USHORT)&contentReq->contentType_length - (USHORT)contentReq;
			offset_table[1][1] = sizeof(UBYTE);
			offset_table[2][0] = 0xFFFF;
		}
			break;
		case WAP_POSTMORECONTENT_IND_ID:
		{
			T_MMI_WAP_POSTMORECONTENT_IND *postMoreContent;
			*struct_size = (USHORT)sizeof(T_MMI_WAP_POSTMORECONTENT_IND);
			offset_table[0][0] = (USHORT)&postMoreContent->data_length - (USHORT)postMoreContent;
			offset_table[0][1] = sizeof(UBYTE);
			offset_table[1][0] = 0xFFFF;
		}
			break;
		case WAP_ACKPOSTCONTENT_REQ_ID:
			*struct_size = (USHORT)sizeof(T_MMI_WAP_MMSTERMINATE_IND);
			offset_table[0][0] = 0xFFFF;
			break;
		case WAP_GETCONTENT_IND_ID:
		{
			T_MMI_WAP_GETCONTENT_IND *getContent;
			*struct_size = (USHORT)sizeof(T_MMI_WAP_GETCONTENT_IND);
			offset_table[0][0] = (USHORT)&getContent->addr_length- (USHORT)getContent;
			offset_table[0][1] = sizeof(UBYTE);
			offset_table[1][0] = (USHORT)&getContent->acceptHeader_length - (USHORT)getContent;
			offset_table[1][1] = sizeof(UBYTE);
			offset_table[2][0] = 0xFFFF;
		}
			break;
		case WAP_ACKCONTENT_IND_ID:
			*struct_size = (USHORT)sizeof(T_MMI_WAP_ACKCONTENT_IND);
			offset_table[0][0] = 0xFFFF;
			break;
		case WAP_CANCELCONTENT_IND_ID:
			*struct_size = (USHORT)sizeof(T_MMI_WAP_CANCELCONTENT_IND);
			offset_table[0][0] = 0xFFFF;
			break;
		/*end talcon add*/
#endif
	}


#ifdef WAP_TASK	
	trace("wap_offset_table end");
#else
	trace("mmi_offset_table end");
#endif	

	return;
}


