

/*
+-----------------------------------------------------------------------------
|  File     : wap_crpt.c
+-----------------------------------------------------------------------------
|                 Copyright Condat AG 1999-2001, Berlin
|                 All rights reserved.
|
|                 This file is confidential and a trade secret of Condat AG
|                 The receipt of or possession of this file does not convey
|                 any rights to reproduce or disclose its contents or to
|                 manufacture, use, or sell anything it may describe, in
|                 whole, or in part, without the specific written consent of
|                 Condat AG.
+-----------------------------------------------------------------------------
|  Purpose :  callbacks for AUS browser, NULL, NOT IMPLEMENTED YET
|            
+----------------------------------------------------------------------------- 
*/
#ifndef WAP_CRPT_C
#define WAP_CRPT_C
#endif

#define ENTITY_WAP


/*==== INCLUDES ===================================================*/
#if defined (NEW_FRAME)

#include <stdarg.h>
#include <string.h>
#include <stdlib.h>
#include <stddef.h>
#include "typedefs.h"
#include "pcm.h"
#include "pconst.cdg"
#include "mconst.cdg"
#include "message.h"
#include "ccdapi.h"
#include "vsi.h"
#include "custom.h"
#include "gsm.h"
#include "prim.h"
#include "cnf_wap.h"
#include "mon_wap.h"
#include "pei.h"
#include "tok.h"
#include "dti.h"      /* functionality of the dti library */

#else

#include <string.h>
#include "stddefs.h"
#include "pconst.cdg"
#include "message.h"
#include "ccdapi.h"
#include "custom.h"
#include "gsm.h"
#include "cnf_wap.h"
#include "mon_wap.h"
#include "prim.h"
#include "vsi.h"
#include "pei.h"
#include "tok.h"

#endif

#include "wap.h"
#include "cus_wap.h"




#include "aapicrpt.h"
#include "capicrpt.h"

/*************************************************************
 * General functions
 *************************************************************/

/*
 * Perform necessary initialization chores, for example, seed
 * the random number generator.
 * This function must be called once before any other functions
 * in the crypto library are used.
 */
VOID CRYPTa_initialise (UINT16 id)
{
	TRACE_EVENT(">>>>> CRYPTa_initialise");
	CRYPTc_initialiseResponse(id, CRV_OK);
}

/*
 * Terminate the use of the crypto library. This function is called
 * when WTLS is being shut down. Current handles held by the user
 * (e.g., HashHandle and MasterSecretID) are no longer valid
 * after calling this function.
 */
INT16 CRYPTa_terminate (VOID)
{
  return CRV_OK;
}


/***********************************************************************
 * Crypto Methods
 ***********************************************************************/

/*
 * Retrieve lists of crypto parameters that the crypto library supports.
 * This includes cipher methods, key exchange methods, and trusted
 * certificates.
 * The response should be delivered by a call to the connector function
 * CRYPTc_getMethodsResponse. The parameter, "id",
 * should be passed back in this response.
 */
VOID CRYPTa_getMethods (UINT16 id)
{	
  CRYPTc_getMethodsResponse (id, CRV_OK, NULL, 0, NULL, 0, NULL, 0);
}


/***********************************************************************
 * Bulk encryption algorithms
 ***********************************************************************/

/*
 * Encrypt single-part data.
 * For some encryption methods, the input plaintext data has certain
 * length constraints. If these constraints are not satisfied, then
 * CRYPTa_encrypt will fail with return code CRV_DATA_LENGTH.
 * The plaintext and ciphertext can be in the same place, i.e., it is
 * OK if data and encryptedData point to the same location.
 * The ciphertext is always of the same length as the plaintext.
 */
INT16 CRYPTa_encrypt (
                      BulkCipherAlgorithm method,
                      KeyObject key,
                      BYTE *data, 
                      UINT16 dataLen,
                      BYTE *encryptedData
                     )
{
  return CRV_UNSUPPORTED_METHOD;
}

/*
 * Decrypt single-part data.
 * For some decryption methods, the input ciphertext has certain
 * length constraints. If these constraints are not satisfied, then
 * CRYPTa_decrypt will fail with return code CRV_DATA_LENGTH.
 * The ciphertext and plaintext can be in the same place, i.e., it is
 * OK if data and decryptedData point to the same location.
 * The ciphertext is always of the same length as the plaintext.
 */
INT16 CRYPTa_decrypt (
                      BulkCipherAlgorithm method,
                      KeyObject key,
                      BYTE *data, 
                      UINT16 dataLen,
                      BYTE *decryptedData
                     )
{
  return CRV_UNSUPPORTED_METHOD;
}


/***********************************************************************
 * Secure hash algorithms
 ***********************************************************************/

/*
 * Compute a hash digest of given single-part data.
 * The input data and digest output can be in the same place, i.e.,
 * it is OK if "data" and "digest" point to the same location.
 * CRYPTa_hash is equivalent to a call to CRYPTa_hashInit, followed
 * by a sequence of CRYPTa_hashUpdate operations, and terminated
 * by a call to CRYPTa_hashFinal.
 */
INT16 CRYPTa_hash (
                   HashAlgorithm alg,
                   BYTE *data, 
                   UINT16 dataLen,
                   BYTE *digest
                  )
{
  return CRV_UNSUPPORTED_METHOD;
}

/*
 * Initialize a hash operation.
 * "alg" is the hash algorithm to use, and on return "handleptr"
 * will point to a new handle to be used in subsequent operations.
 * After calling CRYPTa_hashInit, one calls CRYPTa_hashUpdate zero or
 * more times, followed by CRYPTa_hashFinal, to digest data in
 * multiple parts. The hash operation is active until one makes
 * a call to CRYPTa_hashFinal to actually obtain
 * the final piece of ciphertext. To process additional data
 * (in single or multiple parts), one must call CRYPTa_hashInit again.
 */
INT16 CRYPTa_hashInit (
                       HashAlgorithm alg,
                       HashHandle *handleptr
                      )
{
  return CRV_UNSUPPORTED_METHOD;
}

/*
 * Continue a multiple-part hash operation, processing another data part.
 * "handle" is the handle of the hash operation, "part" points to the
 * data part, and "partLen" is the length of the data part.
 * The hash operation must have been initialized with CRYPTa_hashInit.
 * A call to CRYPTa_hashUpdate which results in an error terminates
 * the current hash operation.
 */
INT16 CRYPTa_hashUpdate (
                         HashHandle handle,
                         BYTE *part, 
                         UINT16 partLen
                        )
{
  return CRV_INVALID_HANDLE;
}


/*
 * Finish a multiple-part hash operation.
 * "handle" is the handle of the hash operation, and "digest" is the location
 * that receives the computed message digest.
 * The hash operation must have been initialized with CRYPTa_hashInit.
 * A call to CRYPTa_hashFinal always terminates the active hash operation.
 */
INT16 CRYPTa_hashFinal (
                        HashHandle handle,
                        BYTE *digest
                       )
{
  return CRV_INVALID_HANDLE;
}


/***********************************************************************
 * Key exchange
 ***********************************************************************/

/*
 * Perform key exchange, using one of the methods previously
 * reported to be available.
 * Derives the master secret using the pre-master secret produced
 * by the key exchange algorithm, and the 32 random bytes
 * (ClientHello.random + ServerHello.random) supplied
 * as a parameter to this call. The master secret is kept internally in
 * the crypto library, to be used in subsequent operations.
 * The parameter "alg" is the secure hash algorithm to be used.
 * The response should be delivered by a call to the connector function
 * CRYPTc_keyExchangeResponse. The first parameter, "id",
 * should be passed back in this response.
 *
 * RSA:
 * The public key to use is either given explicitly in the parameters,
 * or must be retrieved from a certificate passed to this routine.
 * This function generates a 20-byte value consisting of the given
 * "additionalData" and 19 random bytes. It encrypts the value with
 * the given public key, and stores the result in publicValue.
 * The pre-master secret is the 20-byte value appended with the
 * given public key.
 *
 * Diffie-Hellman: calculate a pre-master secret and a public value
 * to be sent to the server side.
 * The public key to use is given explicitly in the parameters.
 * This function performs a DH calculation based on the given
 * public key and a private key kept in the crypto library.
 *
 * ECDH: calculate a pre-master secret and a public value
 * to be sent to the server side.
 * The public key to use is either given explicitly in the parameters,
 * or must be retrieved from a certificate passed to this routine.
 * This function performs an ECDH calculation based on the given
 * public key and a private key kept in the crypto library.
 *
 * Shared secret: set the pre-master secret to be a pre-defined secret key.
 * Note, that if the library attempts to support the Shared Secret
 * key exchange method, it is required that the routine
 * CRYPTa_getClientKeyExchangeIds pair the method indicator
 * (KEY_EXCH_SHARED_SECRET) with a suitable identifier. That is,
 * an identifier that the gateway can use to retrieve the correct
 * secret key. "identifier" is a copy of that same identifying string.
 * The Crypto Library uses this parameter to select the appropriate
 * secret key.
 *
 * Null: set the pre-master secret to be the empty string.
 *
 */
VOID CRYPTa_keyExchange (
                         UINT16 id, 
                         KeyExchangeParameters param,
                         HashAlgorithm alg,
                         const BYTE *randval
                        )
{
  CRYPTc_keyExchangeResponse (id, CRV_UNSUPPORTED_METHOD, 0, 0, 0);
}

/***********************************************************************
 * Key generation
 ***********************************************************************/

/*
 * Calculate the Pseudo-Random Function defined in section
 * 11.3.2 in the WTLS spec.
 * If the master secret is to be used as first parameter,
 * then "secret" is NULL, and "masterSecretID" indicates which
 * master secret to use. Otherwise, "secret" is provided explicitly.
 * The parameter "label" is a null-terminated character string.
 * The parameter "alg" is the secure hash algorithm to be used,
 * and "outputLen" is the number of bytes of output to produce.
 * The response should be delivered by a call to the connector function
 * CRYPTc_PRFResponse. The first parameter, "id", should be
 * passed back in this response.
 */
VOID CRYPTa_PRF (
                 UINT16 id, HashAlgorithm alg,
                 UINT8 masterSecretID,
                 BYTE *secret, UINT16 seclen,
                 BYTE *label,
                 BYTE *seed, UINT16 seedLen,
                 UINT16 outputLen
                )
{
  CRYPTc_PRFResponse (id, CRV_UNSUPPORTED_METHOD, NULL, 0);
}


/***********************************************************************
 * Certificates
 ***********************************************************************/

/*
 * Verify a chain of certificates.
 * The buffer, "buf", contains a chain of certificates. See section
 * 10.5.2 in the WTLS specification.
 * The response should be delivered by a call to the connector function
 * CRYPTc_verifyCertificateChainResponse. The first parameter, "id",
 * should be passed back in this response.
 */
VOID CRYPTa_verifyCertificateChain (
                                    UINT16 id,
                                    const BYTE *buf, 
                                    UINT16 bufLen,
                                    const BYTE *addr, 
                                    UINT16 addrLen,
                                    const CHAR *domain, 
                                    UINT16 domainLenUINT16
                                   )
{
  CRYPTc_verifyCertificateChainResponse (id, CRV_UNSUPPORTED_METHOD);
}

/*
 * Retrieve a client certificate.
 * The buffer, "buf", contains a list of acceptable certificate
 * authorities, as byte encoded KeyExchangeId:s. See section 10.5.4
 * in the WTLS specification.
 * The response should be delivered by a call to the connector function
 * CRYPTc_getClientCertificateResponse. The first parameter, "id",
 * should be passed back in this response.
 */
VOID CRYPTa_getClientCertificate (
                                  UINT16 id, 
                                  const BYTE *buf, 
                                  UINT16 bufLen
                                 )
{
  CRYPTc_getClientCertificateResponse (id, CRV_UNSUPPORTED_METHOD,
                                       NULL, 0, NULL, 0);
}

/*
 * Compute a digital signature.
 * The buffer, "buf", contains the data to be signed, and the key
 * identifier, "keyId", contains a byte-encoded Identifier.
 * This latter value is either fetched from the Key Exchange Ids
 * passed back in CRYPTc_getMethodsResponse, or is the value
 * passed back in CRYPTc_getClientCertificateResponse (provided that
 * this function has been used). The keyId value indicates
 * which key must be used for signing.
 * The response should be delivered by a call to the connector function
 * CRYPTc_computeSignatureResponse. The first parameter, "id",
 * should be passed back in this response.
 */
VOID CRYPTa_computeSignature (
                              UINT16 id, 
                              const BYTE *keyId, 
                              UINT16 keyIdLen,
                              const BYTE *buf, 
                              UINT16 bufLen
                             )
{
  CRYPTc_computeSignatureResponse (id, CRV_UNSUPPORTED_METHOD, NULL, 0);
}


/***********************************************************************
 * Random number generation
 ***********************************************************************/

/*
 * Generate random (or pseudo-random) data. "randomData" points to
 * the location that receives the data and "randomLen" is the number
 * of bytes of data to be generated.
 */
INT16 CRYPTa_generateRandom (
                             BYTE *randomData, 
                             UINT16 randomLen
                            )
{
  return CRV_UNSUPPORTED_METHOD;
}


/***********************************************************************
 * Session cache
 ***********************************************************************/

/*
 * Initialize the session cache.
 */
VOID CRYPTa_sessionInit (VOID)
{
}

/*
 * Close the session cache.
 */
VOID CRYPTa_sessionClose (VOID)
{
}

/************************************************************
 * 'Peers' methods
 ************************************************************/

/*
 * Delete all peer entries that link to the indicated
 * master secret.
 */
VOID CRYPTa_peerDeleteLinks (UINT8 masterSecretIndex)
{
}

/*
 * Add a peer entry that links to the given master secret.
 * If such an entry already exists, overwite it.
 */
VOID CRYPTa_peerLinkToSession (
                               BYTE *address, UINT16 addrlen,
                               UINT16 portnum,
                               UINT8 masterSecretIndex
                              )
{
}

/*
 * Find a peer with matching address and port number.
 * If none exists, try matching just the address.
 * The result, a master secret index, is returned in
 * CRYPTc_peerLookupResponse.
 */
VOID CRYPTa_peerLookup (
                        UINT16 id,
                        BYTE *address, UINT16 addrlen,
                        UINT16 portnum
                       )
{
  CRYPTc_peerLookupResponse (id, CRV_NOT_FOUND, 0);
}

/************************************************************
 * 'Sessions' methods
 ************************************************************/

/*
 * If "isActive" != 0, then mark the indicated session slot
 * as being "active". Otherwise, mark it as not active.
 * A session slot that is "active", SHOULD NOT be reused.
 * WTLS will keep a session marked as "active" as long as
 * any WTLS connection that is based upon that session is active.
 */
VOID CRYPTa_sessionActive (
                           UINT8 masterSecretIndex, 
                           UINT8 isActive
                          )
{
}

/*
 * Mark a session entry as non-resumable.
 */
VOID CRYPTa_sessionInvalidate (UINT8 masterSecretIndex)
{
}

/*
 * Mark all entries in 'Sessions' as non-resumable.
 */
VOID CRYPTa_sessionClear (VOID)
{
}

/*
 * Fetch the contents of the indicated session entry.
 * The result is returned in CRYPTc_sessionFetchResponse.
 */
VOID CRYPTa_sessionFetch (
                          UINT16 id, 
                          UINT8 masterSecretIndex
                         )
{
  CRYPTc_sessionFetchResponse (id, CRV_NOT_FOUND,
                               0, 0, 0, 0, 0, 0, 0, 0);
}

/*
 * Store new values for a session entry.
 */
VOID CRYPTa_sessionUpdate(
                          UINT8 masterSecretIndex,
                          UINT8 sessionOptions,
                          BYTE *sessionId, UINT8 sessionIdLen,
                          UINT8 macAlg,
                          UINT8 cipherAlg,
                          UINT8 compressionAlg,
                          BYTE *privateKeyId,
                          UINT32 creationTime
                         )
{
}


/*
 * Called when a WTLS connection has been terminated.
 */

void CRYPTa_connectionTerminated (UINT8 objectId, UINT8 channelId)
{
}

void CRYPTa_connectionEstablished(UINT8 objectId, UINT8 channelId,UINT8 masterSecretId)
{
}


