/*
+-----------------------------------------------------------------------------
|  File     : wap_gle.c
+-----------------------------------------------------------------------------
|                 Copyright Condat AG 1999-2001, Berlin
|                 All rights reserved.
|
|                 This file is confidential and a trade secret of Condat AG
|                 The receipt of or possession of this file does not convey
|                 any rights to reproduce or disclose its contents or to
|                 manufacture, use, or sell anything it may describe, in
|                 whole, or in part, without the specific written consent of
|                 Condat AG.
+-----------------------------------------------------------------------------
|  Purpose :  callbacks for GLE
|
|  History: 15/05/2003 - SPR#1983 - SH - Updated to latest from 1.6.3 version.
+----------------------------------------------------------------------------- 
*/

#ifndef WAP_GLE_C
#define WAP_GLE_C
#endif

#define ENTITY_WAP


/*==== INCLUDES ===================================================*/
#if defined (NEW_FRAME)

#include <stdarg.h>
#include <string.h>
#include <stdlib.h>
#include <stddef.h>
#include "typedefs.h"
#include "pcm.h"
#include "pconst.cdg"
#include "mconst.cdg"
#include "message.h"
#include "ccdapi.h"
#include "vsi.h"
#include "custom.h"
#include "gsm.h"
#include "prim.h"
#include "cnf_wap.h"
#include "mon_wap.h"
#include "pei.h"
#include "tok.h"
#include "dti.h"      /* functionality of the dti library */

#else

#include <stdarg.h>
#include <string.h>
#include "stddefs.h"
#include "pconst.cdg"
#include "message.h"
#include "ccdapi.h"
#include "custom.h"
#include "gsm.h"
#include "cnf_wap.h"
#include "mon_wap.h"
#include "prim.h"
#include "vsi.h"
#include "pei.h"
#include "tok.h"

#endif

#include "wap.h"
#include "cus_wap.h"

#include "aapimmi.h"
#include "capimmi.h"
#include "aapiclnt.h"
#include "aapigle.h"
#include "capiclnt.h"
#include "wip_mem.h"

#include "wap_types.h"
#include "wap_wapmmi.h"
#include "gledef.h"

/* Use these to access font height & widths */
static SHORT view_width;
static SHORT view_height;
static UBYTE font_height = 0;
static UBYTE *font_width = NULL;
static UBYTE font_max_width = 0;
static UBYTE unicode_width;
static UBYTE unicode_height;

/* This is provided for debugging use */

void traceWAP(char *string)
{
	TRACE_EVENT(string);
	return;
}

/*******************************************************************************

 $Function:    	setupDisplayData

 $Description:	Stores some information about the target display
 
 $Returns:		None.

 $Arguments:	view_width_in	- width of the WAP view in pixels
 				view_height_in	- height of the WAP view in pixels
 				font_width_in	- array of widths for 256 ascii characters
 				font_height_in	- height of font
 				unicode_width_in  - fixed width of unicode font
 				unicode_height_in  - height of unicode font
 
*******************************************************************************/

void setupDisplayData(USHORT view_width_in, USHORT view_height_in, UBYTE *font_width_in, UBYTE font_height_in, UBYTE unicode_width_in, UBYTE unicode_height_in)
{
	USHORT index;
	
	view_height = view_height_in;
	view_width = view_width_in;
	font_height = font_height_in;
	unicode_width = unicode_width_in;
	unicode_height = unicode_height_in;

	if (font_width==NULL)
	{
		font_width = (UBYTE *)wip_malloc(256*sizeof(UBYTE));
		font_max_width = 0;
		
		for (index = 0; index<256; index++)
		{
			font_width[index] = font_width_in[index];
			if (font_width[index] > font_max_width)
				font_max_width = font_width[index];
		}
	}
	
	return;
}

/*******************************************************************************

 $Function:    	freeDisplayData

 $Description:	Resets display information, freeing memory
 
 $Returns:		None.

 $Arguments:	None
 
*******************************************************************************/

void freeDisplayData(void)
{
	TRACE_FUNCTION("freeDisplayData");
	
	wip_free(font_width);

	font_width = NULL;
	return;
}


/*******************************************************************************

 $Function:    	WAP_ImageCopy

 $Description:	Copies an image, allocating memory for it, and scaling it if necessary
 				SPR#2486 - Added.

 $Returns:		Pointer to the allocated image.

 $Arguments:	oldImage	- Pointer to the original image data
 				oldWidth	- Width of the original image in pixels
 				oldHeight	- Height of the original image in pixels
 				newWidth	- Width of the new image in pixels
 				newHeight	- Height of the new image in pixels
 				
*******************************************************************************/

UBYTE *WAP_ImageCopy(UBYTE *oldImage, UINT16 oldWidth, UINT16 oldHeight,
	UINT16 newWidth, UINT16 newHeight)
{
	UBYTE	*Image;			/* Pointer to the new image data */
	USHORT	heightIndex; 
	USHORT	widthIndex;
	USHORT	oldIndex;
	USHORT	newIndex;
	USHORT	byteWidth;
	USHORT	scrByteWidth;
	UBYTE	oldMask;						/* Pixel shift, shift bytes by this amount */
	UBYTE	newMask;
	USHORT	xScale;
	USHORT	xScaleOld;
	USHORT	yScale;
	USHORT	yScaleOld;

#ifdef TRACE_WAP_GLE
	TRACE_FUNCTION("WAP_ImageCopy");
#endif

	/* Work out the width of the original image in bytes */
	
	byteWidth = oldWidth/8;
	if (oldWidth % 8)
		byteWidth++;

	/* Work out the width of the scaled image in bytes */
	
	scrByteWidth = newWidth/8;
	if (newWidth % 8)
		scrByteWidth++;

	Image = (UBYTE *)wip_malloc(scrByteWidth * newHeight * sizeof(GleBitmap));

	/* If there's no scaling to do, copy the image and that's it! */
	
	if (newWidth==oldWidth && newHeight==oldHeight)
	{
		memcpy(Image, oldImage, byteWidth * oldHeight);
	}
	else
	{
		oldIndex = -1;
		newIndex = -1;
		yScaleOld = 0;
		
		/* Scale the image */
		
		for (heightIndex = 0; heightIndex<oldHeight; heightIndex++)
		{
			yScale = (USHORT) (heightIndex+1)*newHeight/oldHeight;
			
			if (yScale != yScaleOld)
			{
				oldMask = 1;		/* Mask within the byte of the first pixel */
				newMask = 1;
				xScaleOld = 0;

				for (widthIndex = 0; widthIndex<oldWidth; widthIndex++)
				{
					xScale = (USHORT) (widthIndex+1)*newWidth/oldWidth;

					if (oldMask == 1)
					{
						oldMask = 128;
						oldIndex++;
					}
					else
						oldMask /= 2;
					
					if (xScale != xScaleOld)
					{
						if (newMask == 1)
						{
							newMask = 128;
							newIndex++;
							Image[newIndex] = 0;
						}
						else
							newMask /= 2;

						if (oldImage[oldIndex] & oldMask)		/* If old pixel set, */
							Image[newIndex] |= newMask; 		/* set new pixel */
					}
					
					xScaleOld = xScale;
				}
			}
			else
			{
				oldIndex += byteWidth;
			}
			yScaleOld = yScale;
		}
	}

	return Image;
}


/*******************************************************************************

 $Function:    	GLEa_error

 $Description:	The GLE calls this when an error occurred during an operation
 
 $Returns:		None.

 $Arguments:	viewId		- the id of the view
 				errorCode	- the code of the error
 
*******************************************************************************/

VOID GLEa_error(UINT8 viewId, UINT16 errorCode)
{
#ifdef TRACE_WAP_GLE
	TRACE_EVENT("GLEa_error");
	TRACE_EVENT_P1("Error code: %d", errorCode);
#endif 

	return;
}


/*******************************************************************************

 $Function:    	GLEa_status

 $Description:	The GLE calls this when an error occurred during an operation
 
 $Returns:		None.

 $Arguments:	viewId		- the id of the view
 				statusCode	- the status code
 
*******************************************************************************/

VOID GLEa_status(UINT8 viewId, UINT16 statusCode)
{
#ifdef TRACE_WAP_GLE
	TRACE_EVENT("GLEa_status");
	TRACE_EVENT_P1("Status code: %d", statusCode);
#endif
	return;
}


/*******************************************************************************

 $Function:    	GLEa_newCard

 $Description:	A new card has been received
 
 $Returns:		None.

 $Arguments:	viewId			- the id of the view
 				title			- title of the card
 				URL				- URL of the card
 				isBookmarkable 	- TRUE if the URL can be added to the bookmark list
 				history			- pointer to the history list
 
*******************************************************************************/

VOID GLEa_newCard(UINT8 viewId, const GLE_STRING title, const CHAR *URL, BOOL isBookmarkable, const WCHAR * const *history)
{
	T_WAP_MMI_NEW_CARD_IND 		parameter;
	/* USHORT historyIndex; */
	/* USHORT historyIndexIndex; */
	
#ifdef TRACE_WAP_GLE
	TRACE_FUNCTION("GLEa_newCard");
	if (strlen(URL)>60)
	{
		TRACE_EVENT("URL too long to trace");
	}
	else
	{
		TRACE_EVENT_P1("URL: %s", URL);
	}
	TRACE_EVENT_P1("isBookmarkable: %d", isBookmarkable);
#endif

	parameter.object_id 		= viewId;
	parameter.title_length		= (UINT16) GLEa_strlen(title);
	parameter.Title				= (USHORT *)title;
	parameter.Url 				= (CHAR *)URL;
	parameter.url_length 		= (UINT16) strlen(URL)+1;
	parameter.is_bookmarkable 	= isBookmarkable;
	parameter.history_length	= 2;
	/* Currently, we are not interested in the history list as such, rather whether or not the current card
	has a history or not, since this will determine whether the back soft key will appear.  So we only transmit
	four bytes (2 x U32), and if those are zero then this is the first card in the history.*/
	parameter.History 			= (USHORT**)history;
	
	W_WAP_MMI_NEW_CARD_IND(&parameter);
	
	return;

}


/*******************************************************************************

 $Function:    	GLEa_drawCardRequest

 $Description:	Called before GLE draws the content of the view.
 
 $Returns:		FALSE if the WAP application doesn't allow the card to be drawn
 				currently, TRUE if it does.

 $Arguments:	viewId			- the id of the view
 				contentPosX		- x position of the content that is currently displayed
 								at the left of the screen.  Can be used to update a
 								horizontal scrollbar.
 				contentWidth	- The total width of the content in the card.
 				contentPosY		- y position of the content that is currently displayed
 								at the top of the screen.  Can be used to update a
 								vertical scrollbar.
 				contentHeight	- The total height of the content in the card.
 
*******************************************************************************/

BOOL GLEa_drawCardRequest(UINT8 viewId, UINT16 contentPosX, UINT16 contentWidth, UINT16 contentPosY, UINT16 contentHeight)
{

	T_WAP_MMI_DRAW_CARD_IND parameter;

#ifdef TRACE_WAP_GLE
	TRACE_EVENT("GLEa_drawCardRequest");
	TRACE_EVENT_P4("posX, posY, width, height: %d, %d, %d, %d", contentPosX, contentPosY, contentWidth, contentHeight);
#endif	

	parameter.object_id		= viewId;
	parameter.contentPosX	= contentPosX;
	parameter.contentPosY	= contentPosY;
	parameter.contentWidth	= contentWidth;
	parameter.contentHeight	= contentHeight;
	
	W_WAP_MMI_DRAW_CARD_IND(&parameter);
	
	return TRUE;
}


/*******************************************************************************

 $Function:    	GLEa_drawCardCompleted

 $Description:	Called after the GLE has drawn the content of the view corresponding
 				to viewId.  If drawing was directed to an offscreen window, the platform
 				should then copy this to the screen.
 				
 $Returns:		None.

 $Arguments:	viewId			- the id of the view
 				 
*******************************************************************************/
	
VOID GLEa_drawCardCompleted(UINT8 viewId)
{

	T_WAP_MMI_CARD_DRAW_COMPLETED_IND parameter;

#ifdef TRACE_WAP_GLE
	TRACE_EVENT("GLEa_drawCardCompleted");
#endif	

	parameter.object_id = viewId;
	W_WAP_MMI_CARD_DRAW_COMPLETED_IND(&parameter);

	return;
}


/*******************************************************************************

 $Function:    	GLEa_clearCard

 $Description:	Erases the content in the view (window) corresponding to viewId.
 				
 $Returns:		None.

 $Arguments:	viewId			- the id of the view
 				 
*******************************************************************************/

VOID GLEa_clearCard(UINT8 viewId)
{
	T_WAP_MMI_CLEAR_CARD_IND parameter;

#ifdef TRACE_WAP_GLE
	TRACE_EVENT("GLEa_clearCard");
#endif

	parameter.object_id = viewId;
	
	W_WAP_MMI_CLEAR_CARD_IND(&parameter);

	return;
}

/*******************************************************************************

 $Function:    	GLEa_playSoundClick

 $Description:	Play a sound
 				
 $Returns:		None.

 $Arguments:	None.
 				 
*******************************************************************************/

VOID GLEa_playSoundClick(VOID)
{
	T_WAP_MMI_CONTROL_IND parameter;
	
#ifdef TRACE_WAP_GLE
	TRACE_EVENT("GLEa_playSoundClick");
#endif	

	parameter.object_id = 0;
	parameter.status_code = 0;
	parameter.error_code = 0;
	parameter.play_standard_click = TRUE;
	
	W_WAP_MMI_CONTROL_IND(&parameter);
	
	return;
}



/*******************************************************************************

 $Function:    	GLEa_strcpyGle2Wchar

 $Description:	Converts a GLE_STRING to a Unicode string.
 
 $Returns:		the Unicode string

 $Arguments:	dst			- the null terminated destination string
 				src			- the null terminated source string
 
*******************************************************************************/

WCHAR* GLEa_strcpyGle2Wchar(WCHAR* dst, const GLE_STRING src)
{
	UINT16	index;
	USHORT	*srcUnicode = (USHORT *)src;
	USHORT *dstUnicode = (USHORT *)dst;

#ifdef TRACE_WAP_GLE
	TRACE_EVENT("GLEa_strcpyGle2Wchar");
#endif

	for (index=0; srcUnicode[index] != 0; index++)
	{
		dstUnicode[index] = srcUnicode[index];
	}

	dstUnicode[index] = 0;

	return dst;
}


/*******************************************************************************

 $Function:    	GLEa_strcpyWchar2Gle

 $Description:	Converts a Unicode string to a GLE_STRING string.
 
 $Returns:		the GLE_STRING string

 $Arguments:	dst			- the null terminated destination string
				src			- the null terminated source string
 
*******************************************************************************/

GLE_STRING GLEa_strcpyWchar2Gle(GLE_STRING dst, const WCHAR* src)
{
	UINT16	index;
	USHORT	*dstUnicode = (USHORT *)dst;
	USHORT *srcUnicode = (USHORT *)src;

#ifdef TRACE_WAP_GLE
	TRACE_EVENT("GLEa_strcpyWchar2Gle");
#endif

	for (index=0; src[index] != '\0'; index++)
	{
		dstUnicode[index] = srcUnicode[index];
	}

	dstUnicode[index] = 0;

	return dst;
}


/*******************************************************************************

 $Function:    	GLEa_strcpyGle2Gle

 $Description:	Copies a GLE_STRING string to another GLE_STRING string.
 
 $Returns:		the copy

 $Arguments:	dst			- the null terminated destination string
				src			- the null terminated source string
 
*******************************************************************************/

GLE_STRING GLEa_strcpyGle2Gle(GLE_STRING dst, const GLE_STRING src)
{
	UINT16	index;
	USHORT	*srcUnicode = (USHORT *)src;
	USHORT	*dstUnicode = (USHORT *)dst;

#ifdef TRACE_WAP_GLE
	TRACE_EVENT("GLEa_strcpyGle2Gle");
#endif 

	for (index=0; srcUnicode[index] != 0; index++)
	{
		dstUnicode[index] = srcUnicode[index];
	}

	dstUnicode[index] = 0;

	return dst;
}


/*******************************************************************************

 $Function:    	GLEa_strlen

 $Description:	Computes the number of characters in the GLE_STRING string.
 
 $Returns:		the number of characters, not including the null character

 $Arguments:	str			- the null terminated source string
 
*******************************************************************************/

INT16 GLEa_strlen(const GLE_STRING str)
{
	INT16	index = 0;
	USHORT	*strUnicode = (USHORT *)str;
	
#ifdef TRACE_WAP_GLE
	TRACE_EVENT("GLEa_strlen");
#endif	

	while ( strUnicode[index] >= 0x20 )
		index++;

#ifdef TRACE_WAP_GLE
	WAP_trace_ushort_string(strUnicode, index);
#endif	
	return index;
}


/*******************************************************************************

 $Function:    	GLEa_sizeofString

 $Description:	Computes the number of bytes needed to store the GLE_STRING string.
 
 $Returns:		the number of bytes that need to be allocated to convert the Unicode
 				array pointed to text to a null-terminated GLE_STRING string.  Space
 				for the null character is also counted.

 $Arguments:	text			- the null terminated source string
 
*******************************************************************************/

UINT16 GLEa_sizeofString(const WCHAR* str)
{
	UINT16 index = 0;
	USHORT *strUnicode = (USHORT *)str;
	
#ifdef TRACE_WAP_GLE
	TRACE_EVENT("GLEa_sizeofString");
	WAP_trace_ushort_string(strUnicode, 40);
#endif

	while (str[index] != '\0')
		index++;
	
#ifdef TRACE_WAP_GLE	
	TRACE_EVENT_P1("Returned: %d", index*sizeof(USHORT));
#endif	

	return index*sizeof(USHORT);	/* Includes terminating NULL */
}


/*******************************************************************************

 $Function:    	GLEa_strNbrCharsWithoutEndingSpaces

 $Description:	Calculates the number of the first len characters in the passed string.
 
 $Returns:		the number of the len first characters in the str string, except for
 				ending space characters.

 $Arguments:	str			- the source string
				len			- the number of characters to read
 
*******************************************************************************/

INT16 GLEa_strNbrCharsWithoutEndingSpaces(const GLE_STRING str, UINT16 len)
{
	INT16 stringlength = GLEa_strlen(str);
	USHORT	*strUnicode = (USHORT *)str;
	
#ifdef TRACE_WAP_GLE
	TRACE_EVENT("GLEa_strNbrCharsWithoutEndingSpaces");
	WAP_trace_ushort_string((USHORT *)strUnicode, len);
#endif	

	if (len > stringlength)									/* start at len characters, */
		len = stringlength;									/* or the end of the string if smaller. */											

	while (strUnicode[len-1] == 0x0020 && len > 0)			/* if the character is a space, */
		len--;												/* look at the previous character */

#ifdef TRACE_WAP_GLE
	TRACE_EVENT_P1("Returned: %d", len);
#endif 
	
	return (INT16) len;			/* now we have the number of non-space characters */
}


/*******************************************************************************

 $Function:    	GLEa_sizeofNchars

 $Description:	Computes how many bytes are needed to hold the first len characters
 				in the str string, except for ending space characters.
 
 $Returns:		the number of bytes that are needed to hold the first len characters of
 				the string.

 $Arguments:	str			- the source string
				len			- the number of bytes to read
 
*******************************************************************************/

UINT16 GLEa_sizeofNchars(const GLE_STRING str, UINT16 len)
{
	UINT16 sizeNoEndSpaces = GLEa_strNbrCharsWithoutEndingSpaces(str, len);
	
#ifdef TRACE_WAP_GLE
	TRACE_EVENT("GLEa_sizeofNchars");
	WAP_trace_ushort_string((USHORT *)str,sizeNoEndSpaces);
	TRACE_EVENT_P1("Returned: %d", (sizeNoEndSpaces*sizeof(USHORT)));
#endif
	
	return (UINT16) (sizeNoEndSpaces*sizeof(USHORT));
}


/*******************************************************************************

 $Function:    	GLEa_textFontAscent

 $Description:	The rendering algorithm of the GLE uses this function to get a text's
 				ascent.
 
 $Returns:		the ascent in pixels of the corresponding font.

 $Arguments:	format		- the format of the text
				isLink		- TRUE if the text is a link
 
*******************************************************************************/

UINT16 GLEa_textFontAscent(INT8 format, BOOL isLink)
{
#ifdef TRACE_WAP_GLE
	TRACE_EVENT("GLEa_textFontAscent");
#endif

	return font_height;
}


/*******************************************************************************

 $Function:    	GLEa_textFontHeight

 $Description:	The rendering algorithm of the GLE uses this function to get a text's
 				height.
 
 $Returns:		the height in pixels of the corresponding font.  This height includes
 				eventual space to the following line.

 $Arguments:	format		- the format of the text
				isLink		- TRUE if the text is a link
 
*******************************************************************************/

UINT16 GLEa_textFontLineHeight(INT8 format, BOOL isLink)
{
#ifdef TRACE_WAP_GLE
	TRACE_EVENT("GLEa_textFontLineHeight");
	TRACE_EVENT_P1("Returning: %d", font_height);
#endif

	return font_height;
}


/*******************************************************************************

 $Function:    	GLEa_textFontWidth

 $Description:	The rendering algorithm of the GLE uses this function to get a text's
 				width.
 
 $Returns:		the width in pixels of the first len characters in the GLE_STRING text,
 				calculated with the corresponding font.

 $Arguments:	text		- the text whose width is to be returned
				len			- number of characters to be considered
				format		- the format of the text
				isLink		- TRUE if the text is a link
 
*******************************************************************************/

UINT16 GLEa_textFontWidth(const GLE_STRING text, const UINT16 len, INT8 format, BOOL isLink)
{
	UINT16 width;
	INT16 stringlength = GLEa_strlen(text);
	USHORT	*strUnicode = (USHORT *)text;
	UINT16 index;

#ifdef TRACE_WAP_GLE
	TRACE_FUNCTION("GLEa_textFontWidth");
#endif

	width = 0;
	
	for (index=0; index<len && index<stringlength; index++)
	{
		if (strUnicode[index]<256)
			width += font_width[strUnicode[index]];
		else
			width += unicode_width;
	}

#ifdef TRACE_WAP_GLE
	TRACE_EVENT_P1("Returned: %d", width);
#endif

	return width;
}


/*******************************************************************************

 $Function:    	GLEa_textFontWordWrap

 $Description:	Given the null terminated GLE_STRING text, this function returns the
  				number of characters of the whole words that can be displayed within
  				wrapWidth pixels, calculated with the corresponding font.  Ending space
				characters shall be counted as they fit.  If not even one word fits
				within the wrapWidth, then the function returns the number of characters
				of the first word that fits within wrapWidth.

 $Returns:		the number of characters of the whole words that can be displayed.

 $Arguments:	text		- the text to wrap
				wrapWidth 	- maximum width in pixels
				format		- the format of the text
				isLink		- TRUE if the text is a link
 
*******************************************************************************/

UINT16 GLEa_textFontWordWrap(const GLE_STRING text, const UINT16 wrapWidth, INT8 format, BOOL isLink)
{
	UINT16	index;
	UINT16	foundspace 			= 0;
	BOOL	foundnonbreaking	= FALSE;	/* Nonbreaking space character (A0) means cannot wrap until space found */
	UINT16	totalWidth			= 0;
	USHORT	*textUnicode		= (USHORT *)text;

#ifdef TRACE_WAP_GLE
	USHORT *dump				= (USHORT *)text;
	USHORT	dumpIndex;
	
	TRACE_EVENT("GLEa_textFontWordWrap");
	TRACE_EVENT_P3("wrapWidth, format, isLink: %d, %d, %d", wrapWidth, format, isLink);
	for (dumpIndex = 0; dumpIndex < 10; dumpIndex++)
	{
		TRACE_EVENT_P3("%X: %c, %d", dump+dumpIndex, *(dump+dumpIndex), *(dump+dumpIndex));
	}
#endif
	
	for(index = 0; textUnicode[index]>=0x20; index++)
	{
		if (totalWidth>=wrapWidth && !foundnonbreaking)
			break;
			
		totalWidth += GLEa_textFontWidth((GLE_STRING)(textUnicode+index), 1, format, isLink);  /* Width of single character */
		if (textUnicode[index]==0x20)
		{
			foundspace = index;
			foundnonbreaking = FALSE;
		}
		if (textUnicode[index]==0xA0)
		{
			foundnonbreaking = TRUE;
		}
	}

	if (totalWidth>wrapWidth)
		index--;
		
	/* If we haven't found a space, hard-wrap the text */
	
	if (foundspace==0)
	{
		return index;
	}

	/* Otherwise, soft-wrap it */
	
	if (textUnicode[index]==0x20)	/* Space is found */
	{
		return index+1;
	}
	
	if (textUnicode[index]<0x20)	/* End of string is found */
	{
		return index;
	}
	return foundspace+1;
}


/*******************************************************************************

 $Function:    	GLEa_textDraw

 $Description:	This function draws a number of characters of a text in the corresponding
 				font.

 $Returns:		None.

 $Arguments:	text		- the text to draw
				len			- draws the len first characters of the text
				invert		- if TRUE, text should be drawn marked, otherwise not
				textBounds	- the position of the upper left corner and the allowed
								width and height.
				deltaY		- distance from the upper left corner to the top of the
								text
				format		- the format of the text
				isLink		- true if the text is a link
 
*******************************************************************************/

VOID GLEa_textDraw(UINT8 viewId, const GLE_STRING text, const UINT16 len, BOOL invert, GleRectangleType *textBounds, UINT16 deltaY, INT8 format, BOOL isLink)
{
	T_WAP_MMI_SEND_TEXT_IND 	parameter;
	/* USHORT textIndex; */

#ifdef TRACE_WAP_GLE
	TRACE_EVENT("GLEa_textDraw");
	WAP_trace_ushort_string((USHORT *)text,len);
#endif

	/* If our text element is off the edge of the screen, don't send the event */
	
	if (textBounds->topLeft.x > view_width || textBounds->topLeft.y > (view_height-font_height)
		|| (textBounds->topLeft.x+textBounds->extent.x)<0 || textBounds->topLeft.y<0)
		return;

	/* Otherwise, send the event */
	
	parameter.object_id		= viewId;
	parameter.type			= WAP_TEXT_NORM;
	
	parameter.text_length	= len;
	parameter.Text			= (USHORT*)text;

	/* Skip out leading space of string */

	if (parameter.Text[0] == 0x20 || parameter.Text[0] == 0xA0)
	{
		parameter.Text++;
		parameter.text_length--;
	}

	parameter.invert		= invert;
	parameter.pX			= textBounds->topLeft.x;
	parameter.pY			= textBounds->topLeft.y;
	parameter.pWidth		= textBounds->extent.x;
	parameter.pHeight		= textBounds->extent.y;
	parameter.format		= format;
	parameter.selected		= invert;							
	parameter.is_link		= isLink;
	parameter.formatstring_length = 0;
	parameter.Formatstring 	= NULL;
	parameter.deltaY		= deltaY;
	
	W_WAP_MMI_SEND_TEXT_IND(&parameter);

	return;
}


/*******************************************************************************

 $Function:    	GLEa_imageConvert

 $Description:	This function allocates memory and converts an image to a platform
 				image format.
 				SPR#2486 - SH - Now call image scaling routine with actual image
 				dimensions, and specified on-screen dimensions

 $Returns:		a pointer to the converted image

 $Arguments:	identical to MMIa_newImage
 
*******************************************************************************/

GleBitmap* GLEa_imageConvert(const CHAR *imageData, UINT16 imageSize, const CHAR *imageType, INT8* vSpace, INT8* hSpace, INT16* width, INT16* height, INT8 isPercent)
{
	UINT16 index;
	UINT16 shift;
	GleBitmap *Image;
	INT16 origWidth;	/* Original width of image (from image data) */
	INT16 origHeight;	/* Original width of image (from image data) */
	INT16 newWidth;
	INT16 newHeight;
	
#ifdef TRACE_WAP_GLE
	TRACE_EVENT("GLEa_imageConvert");
#endif

	index = 0;
	
	/* First byte should be 0 */
	
	if (imageData[index] != 0)
	{
		TRACE_EVENT("** ERROR - Image not WBMP ***");
		return NULL;
	}

	/* If fixed header is not 0, return */
	
	if (imageData[++index] != 0)
	{
		TRACE_EVENT("*** ERROR - Fixed header not in place ***");
		return NULL;
	}

	origWidth = 0;
	shift = 1;

	do
	{
		index++;
		origWidth |= ((imageData[index] & 0x7F)*shift);
		shift *= 0x80;
	}
	while (imageData[index] & 0x80);
	
	origHeight = 0;
	shift = 1;

	do
	{
		index++;
		origHeight |= ((imageData[index] & 0x7F)*shift);
		shift *= 0x80;
	}
	while (imageData[index] & 0x80);

	index++;

	/* If a width and height are provided, and they're different from the actual
	 * width and height of the image, then the image will need to be scaled */
	 
	if (*width!=0 && *height!=0)
	{
		newWidth = *width;
		newHeight = *height;
	}
	else
	{
		newWidth = origWidth;
		newHeight = origHeight;
	}

	Image = WAP_ImageCopy(
		(UBYTE *)&imageData[index],
		origWidth,
		origHeight,
		newWidth,
		newHeight);

	*width = newWidth;
	*height = newHeight;
	
	return Image;
}


/*******************************************************************************

 $Function:    	GLEa_imageDraw

 $Description:	Draws an image

 $Returns:		None.

 $Arguments:	viewId		- The view id
 				image		- pointer to the image to draw
 				invert		- If the argument is TRUE, the image should be drawn
 								marked, otherwise not
 				imageBounds	- The position of the upper left hand corner and the
 								allowed width and height.
 
*******************************************************************************/

VOID GLEa_imageDraw(UINT8 viewId, const GleBitmap *image, BOOL invert, GleRectangleType* imageBounds)
{

	T_WAP_MMI_SEND_IMAGE_IND	parameter;
	SHORT	byteWidth;

#ifdef TRACE_WAP_GLE
	TRACE_EVENT("GLEa_imageDraw");
#endif

	/* If our image is off the edge of the screen, don't send the event */
	
	if (imageBounds->topLeft.x > view_width || imageBounds->topLeft.y > view_height
		|| (imageBounds->topLeft.x+imageBounds->extent.x)<0 || (imageBounds->topLeft.y+imageBounds->extent.y) < 0 )
		return;

	/* Otherwise, send the event */
	
	parameter.object_id = viewId;

	byteWidth = imageBounds->extent.x/8;
	if (imageBounds->extent.x % 8)
		byteWidth++;
	
	parameter.image_length	= byteWidth * imageBounds->extent.y;
	parameter.Image			= (UBYTE *)image;

	/* Skip out leading space of string */

	parameter.invert		= invert;
	parameter.pX			= imageBounds->topLeft.x;
	parameter.pY			= imageBounds->topLeft.y;
	parameter.pWidth		= imageBounds->extent.x;
	parameter.pHeight		= imageBounds->extent.y;
	parameter.selected		= invert;							
	
	W_WAP_MMI_SEND_IMAGE_IND(&parameter);
	
	return;
}


/*******************************************************************************

 $Function:    	GLEa_imageFree

 $Description:	Deallocates memory occupied by an image

 $Returns:		None.

 $Arguments:	image		- pointer to the image to deallocate
 
*******************************************************************************/

VOID GLEa_imageFree(GleBitmap *image)
{
#ifdef TRACE_WAP_GLE
	TRACE_EVENT("GLEa_imageFree");
#endif

	wip_free((void *)image);
	
	return;
}


/*******************************************************************************

 $Function:    	GLEa_fieldSetDraw

 $Description:	Draw a field set in the view

 $Returns:		None.

 $Arguments:	viewId			- the current view
 				title			- the title of the fieldset
 				fieldSetBounds	- the area of the content that the fieldset shall
 									enclose.
 
*******************************************************************************/

VOID GLEa_fieldSetDraw(UINT8 viewId, const GLE_STRING title, GleRectangleType *fieldSetBounds)
{
	T_WAP_MMI_SEND_TABLE_IND		parameter;

#ifdef TRACE_WAP_GLE
	TRACE_EVENT("GLEa_fieldSetDraw");
#endif

	/* If our fieldset element is off the edge of the screen, don't send the event */
	
	if (fieldSetBounds->topLeft.x > view_width
		|| fieldSetBounds->topLeft.y > (view_height-font_height)
		|| (fieldSetBounds->topLeft.x+fieldSetBounds->extent.x)<0
		|| (fieldSetBounds->topLeft.y+fieldSetBounds->extent.y)<0)
		return;
	
	parameter.object_id		= viewId;
	parameter.title_length	= (UINT16) GLEa_strlen((GLE_STRING)title);
	parameter.Title			= (USHORT*)title;
	
	parameter.pX			= fieldSetBounds->topLeft.x;
	parameter.pY			= fieldSetBounds->topLeft.y;
	parameter.pWidth		= fieldSetBounds->extent.x;
	parameter.pHeight		= fieldSetBounds->extent.y;
	parameter.cols_length	= 0;
	parameter.ColWidth		= NULL;
	parameter.rows_length	= 0;
	parameter.RowHeight		= NULL;
	parameter.noOfCells		= 0;
	W_WAP_MMI_SEND_TABLE_IND(&parameter);

	return;

}


/*******************************************************************************

 $Function:    	GLEa_inputSize

 $Description:	Sets the dimensions of an input field

 $Returns:		None.

 $Arguments:	size			- how many characters should be visible
 				nChars			- how many characters the input field should be able
 									to handle
 				maxWidth		- maximum width possible (or -1 for no maximum)
 				width			- the width in pixels
 				height			- the height in pixels
 				ascent			- the ascent in pixels.  Should be the number of
 									pixels of the input field above the baseline
 									(<=height)
 
*******************************************************************************/

VOID GLEa_inputSize(INT8 size, INT8 nChars, INT16 maxWidth, INT16 *width, INT16 *height, UINT16 *ascent)
{
#ifdef TRACE_WAP_GLE
	TRACE_EVENT("GLEa_inputSize");
	TRACE_EVENT_P3("size %d nChars %d maxWidth %d",size,nChars,maxWidth);
#endif

	/* Include '[' and ']' in width */
	
	if (size>0)
	{
		*width += (INT16) (size+2)*font_max_width;					
	}
	else
		*width = view_width;

	*height = (INT16) GLEa_textFontLineHeight (0, TRUE);
	*ascent = (INT16) GLEa_textFontAscent (0, TRUE);
	
	/* currently no border provided for */
	/*if (*width>maxWidth && maxWidth!=-1)
		*width = maxWidth;*/
#ifdef TRACE_WAP_GLE
	TRACE_EVENT_P3("Width %d Height %d Ascent %d",*width,*height,*ascent);
#endif

	return;
}


/*******************************************************************************

 $Function:    	GLEa_inputDraw

 $Description:	Draws an input field

 $Returns:		None.

 $Arguments:	viewId			- the id of the view
 				text			- the input field text
 				invert			- if TRUE, the text should be drawn marked, otherwise
 									not
 				size			- how many characters should be visible
 				nChars			- how many characters the input field should be able
 									to handle
 				isPassword		- true if the input field is for a password
 				format			- the format of the text
 				inputBounds		- the position of the upper left hand corner and the
 								allowed width and height.
 				
*******************************************************************************/

VOID GLEa_inputDraw(UINT8 viewId, const GLE_STRING text, BOOL invert, INT8 size, INT8 nChars, BOOL isPassword, const GLE_STRING format, GleRectangleType *inputBounds)
{
	T_WAP_MMI_SEND_TEXT_IND 	parameter;
	USHORT						textIndex;

#ifdef TRACE_WAP_GLE
	TRACE_EVENT("GLEa_inputDraw");
	TRACE_EVENT_P5("size %d Px %d Py %d Width %d Height %d",size, inputBounds->topLeft.x, inputBounds->topLeft.y, inputBounds->extent.x, inputBounds->extent.y);
	TRACE_EVENT_P3("size %d nChars %d isPassword %d", size, nChars, isPassword);
#endif

	/* If our input element is off the edge of the screen, don't send the event */
	
	if (inputBounds->topLeft.x > view_width || inputBounds->topLeft.y > (view_height-font_height)
		|| (inputBounds->topLeft.x+inputBounds->extent.x)<0 || inputBounds->topLeft.y<0)
		return;

	parameter.object_id		= viewId;
	parameter.type			= WAP_TEXT_INPUT;
	
	parameter.text_length	= (UINT16) GLEa_strlen((GLE_STRING)text)+2;
	parameter.Text			= (USHORT *)wip_malloc((parameter.text_length+1)*sizeof(USHORT));
	
	/* If the text is a password, make it '*'s */
		
	GLEa_strcpyGle2Gle((GLE_STRING)(parameter.Text+1), text);

	if (isPassword)
	{
		for (textIndex=0; textIndex<parameter.text_length; textIndex++)
			parameter.Text[textIndex] = (USHORT) '*';
	}
		
	parameter.Text[0] = (USHORT) '[';
	parameter.Text[parameter.text_length-1] = (USHORT) ']';
	parameter.Text[parameter.text_length] = (USHORT) '\0';
	
	parameter.invert		= invert;
	parameter.pX			= inputBounds->topLeft.x;
	parameter.pY			= inputBounds->topLeft.y;
	parameter.pWidth		= inputBounds->extent.x;
	parameter.pHeight		= inputBounds->extent.y;

	parameter.formatstring_length = (UINT16) GLEa_strlen((GLE_STRING)format);
	parameter.Formatstring	= (USHORT*)format;
	parameter.selected		= invert;								
	parameter.is_link		= FALSE;
	
	W_WAP_MMI_SEND_TEXT_IND(&parameter);

	wip_free((void *)parameter.Text);
	
	return;
}


/*******************************************************************************

 $Function:    	GLEa_inputDialog

 $Description:	Opens an editor to edit the text of the input field.  The platform
 				mustn't change the text of the input; instead when the user is done
 				editing, the platform calls GLEc_inputDialogResponse with the edited
 				text as a parameter.

 $Returns:		None.

 $Arguments:	inputPtr			- the attributes of the input field
 				
*******************************************************************************/

VOID GLEa_inputDialog(GleInput* inputPtr)
{
	T_WAP_MMI_INPUT_DIALOG_REQ		parameter;

#ifdef TRACE_WAP_GLE
	TRACE_EVENT("GLEa_inputDialog");
#endif	

    /* SPR#2393 - SH - Now set length of NULL strings to 0 */
  
	parameter.object_id		= inputPtr->viewId;
	parameter.dialog_id		= inputPtr->inputId;
	parameter.is_password	= inputPtr->isPassword;
	parameter.empty_ok		= inputPtr->emptyOK;
	parameter.visible		= inputPtr->size;
	parameter.size			= inputPtr->nChars;
	parameter.dialog_pointer   =  (void*)inputPtr;

	/* Title of input dialog */
	
	parameter.Title			= (USHORT*)inputPtr->title;
	if (inputPtr->title==NULL)
		parameter.title_length  = 0;
	else
		parameter.title_length  = (U32) GLEa_strlen((GLE_STRING)inputPtr->title);

	/* Contents of input buffer */
	
	parameter.Input			= (USHORT*)inputPtr->text;
	if (inputPtr->text==NULL)
		parameter.input_length = 0;
	else
		parameter.input_length = (U32) GLEa_strlen((GLE_STRING)inputPtr->text);

	/* Input format string */

	parameter.Format		= (USHORT*)inputPtr->format;
	if (inputPtr->format==NULL)
		parameter.format_length = 0;
	else
		parameter.format_length= (U32) GLEa_strlen((GLE_STRING)inputPtr->format);



	W_WAP_MMI_INPUT_DIALOG_REQ(&parameter);

	return;
}

#ifdef SHOW_KEYS_IN_CONTENT


/*******************************************************************************

 $Function:    	GLEa_newKey

 $Description:	Lets the WAP application choose if the GLE shall take care of a key.

 $Returns:		TRUE if the key is to be handled by the WAP application, FALSE if
 				it is to be handled by the GLE

 $Arguments:	Same as MMIa_newKey
 				
*******************************************************************************/

BOOL GLEa_newKey (UINT8 viewId, UINT8 keyId, const WCHAR *eventType, const WCHAR *label, BOOL isOptional)
{
	T_WAP_MMI_NEW_KEY_IND		parameter;
	USHORT prevString[] = {'p', 'r', 'e', 'v', '\0'};
	
#ifdef TRACE_WAP_GLE
	TRACE_EVENT("GLEa_newKey");
#endif

	if (memcmp((void *)eventType, (void *)prevString, 4)!=0)	/* If the key is not "prev" */
		return FALSE;		/* Tell GLE to draw keys inline */
	
	parameter.object_id		= viewId;
	parameter.keyId			= (UINT16) keyId;
	parameter.key_type		= WAP_KEY_PREV;
	parameter.Label			= (USHORT*)label;
	if (label)
	{
		parameter.label_length = (U32)GLEa_strlen((GLE_STRING)label);
	}
	else
	{
		parameter.label_length = 0;
	}

#ifdef TRACE_WAP_GLE
	WAP_trace_ushort_string(parameter.Label, parameter.label_length);
#endif	

	W_WAP_MMI_NEW_KEY_IND(&parameter);

	return TRUE;		/* TRUE - key handled by WAP application, FALSE - let GLE show keys inline */
}


/*******************************************************************************

 $Function:    	GLEa_keySize

 $Description:	Sets the dimensions of a key
 
 $Returns:		None.

 $Arguments:	label			- the label of the key (used to calculate size)
 				eventType		- identifies the type
 				maxWidth		- maximum width possible (or -1 for no maximum)
 				width			- the width in pixels
 				height			- the height in pixels
 				ascent			- the ascent in pixels.  Should be the number of
 									pixels of the key above the baseline
 									(<=height)
 				
*******************************************************************************/

VOID GLEa_keySize(const GLE_STRING label, const GLE_STRING eventType, INT16 maxWidth, INT16 *width, INT16 *height, UINT16 *ascent)
{
#ifdef TRACE_WAP_GLE
	TRACE_EVENT("GLEa_keySize");
	TRACE_EVENT_P1("Maxwidth: %d", maxWidth);
#endif	

	*width = (INT16) GLEa_textFontWidth (label, (UINT16) GLEa_strlen(label), 0, TRUE);
	*height = (INT16) GLEa_textFontLineHeight (0, TRUE);
	*ascent = (INT16) GLEa_textFontAscent (0, TRUE);
	/* currently no border provided for */
	/* if (*width>maxWidth && maxWidth!=-1)
		*width = maxWidth;*/

#ifdef TRACE_WAP_GLE
	TRACE_EVENT_P3("Width %d, Height %d, MaxWidth %d",*width, *height, maxWidth);
#endif
	
	return;
}


/*******************************************************************************

 $Function:    	GLEa_keyDraw

 $Description:	Draws a key with a label
 
 $Returns:		None.

 $Arguments:	viewId			- the id of the view	
 				label			- the label of the key
 				eventType		- identifies the type
 				invert			- if TRUE, the key should be drawn marked, otherwise
 									not
 				keyBounds		- the position of the upper left hand corner and the
 									allowed width and height
 				
*******************************************************************************/

VOID GLEa_keyDraw(UINT8 viewId, const GLE_STRING label, const GLE_STRING eventType, BOOL invert, GleRectangleType *keyBounds)
{
	T_WAP_MMI_SEND_TEXT_IND 	parameter;

#ifdef TRACE_WAP_GLE
	TRACE_EVENT("GLEa_keyDraw");
	TRACE_EVENT_P4("x, y, width, height: %d, %d, %d, %d", keyBounds->topLeft.x, keyBounds->topLeft.y,
		keyBounds->extent.x, keyBounds->extent.y);
	WAP_trace_ushort_string((USHORT *) label, (USHORT)GLEa_strlen(label));
#endif

	/* If our key is off the edge of the screen, don't send the event */
	
	if (keyBounds->topLeft.x > view_width || keyBounds->topLeft.y > (view_height-font_height)
		|| (keyBounds->topLeft.x+keyBounds->extent.x)<0 || keyBounds->topLeft.y<0)
		return;
	
	parameter.object_id		= viewId;
	parameter.type			= WAP_TEXT_KEY;	

	parameter.text_length	= (UINT16) GLEa_strlen((GLE_STRING)label);
	parameter.Text			= (USHORT*)label;

	parameter.invert		= invert;
	parameter.pX			= keyBounds->topLeft.x;
	parameter.pY			= keyBounds->topLeft.y;
	parameter.pWidth		= keyBounds->extent.x;
	parameter.pHeight		= keyBounds->extent.y;
	parameter.format		= 0;		
	parameter.selected		= 0;
	parameter.is_link		= TRUE;
	
	parameter.formatstring_length = 0;
	parameter.Formatstring 	= NULL;
	
	W_WAP_MMI_SEND_TEXT_IND(&parameter);
	return;
}


#endif

#ifdef SHOW_OPTIONS_IN_CONTENT


/*******************************************************************************

 $Function:    	GLEa_optionSize

 $Description:	Sets the dimensions of an option
 
 $Returns:		None.

 $Arguments:	label			- the label of the option (used to calculate size)
 				multiselect		- if TRUE, the option is in a multiple-choice menu,
 									otherwise it is in a single-choice menu
 				maxWidth		- maximum width possible (or -1 for no maximum)
 				width			- the width in pixels
 				height			- the height in pixels
 				ascent			- the ascent in pixels.  Should be the number of
 									pixels of the option above the baseline
 									(<=height)
 				
*******************************************************************************/

VOID GLEa_optionSize(const GLE_STRING label, BOOL multiSelect, INT16 maxWidth, INT16 *width, INT16 *height, UINT16 *ascent)
{
#ifdef TRACE_WAP_GLE
	TRACE_EVENT("GLEa_optionSize");
	TRACE_EVENT_P1("Maxwidth: %d", maxWidth);
#endif

	/* Add width of checkbox & padding space */

	*width = (INT16) GLEa_textFontWidth (label, (UINT16) GLEa_strlen((GLE_STRING)label), 0, FALSE) \
		+ WAP_CHECKBOX_WIDTH+WAP_CHECKBOX_RIGHT;
		
	*height = (INT16) GLEa_textFontLineHeight (0, FALSE);
	*ascent = (INT16) GLEa_textFontAscent (0, FALSE);
	/* currently no border provided for */
	/*if (*width>maxWidth)
		*width = maxWidth;*/
	return;
}


/*******************************************************************************

 $Function:    	GLEa_optionDraw

 $Description:	Draws an option with a label
 
 $Returns:		None.

 $Arguments:	viewId			- the id of the view
 				label			- the label of the option (used to calculate size)
 				multiselect		- if TRUE, the option is in a multiple-choice menu,
 									otherwise it is in a single-choice menu
 				invert			- if TRUE, the option should be drawn marked,
 									otherwise not
 				selected		- if TRUE, the option should be drawn selected,
 									otherwise not
 				optionBounds	- the position of the upper left hand corner and the
 									allowed width and height
 				
*******************************************************************************/

VOID GLEa_optionDraw(UINT8 viewId, const GLE_STRING label, BOOL multiSelect, BOOL invert, BOOL selected, GleRectangleType *optionBounds)
{
	T_WAP_MMI_SEND_TEXT_IND 	parameter;

#ifdef TRACE_WAP_GLE
	TRACE_EVENT("GLEa_optionDraw");
#endif	

	/* If our option is off the edge of the screen, don't send the event */
	
	if (optionBounds->topLeft.x > view_width || optionBounds->topLeft.y > (view_height-font_height)
		|| (optionBounds->topLeft.x+optionBounds->extent.x)<0 || optionBounds->topLeft.y<0)
		return;
	
	parameter.object_id		= viewId;
	parameter.type			= WAP_TEXT_OPTION;

	parameter.text_length	= (UINT16) GLEa_strlen((GLE_STRING)label);
	parameter.Text			= (USHORT*)label;

	parameter.invert		= invert;
	parameter.pX			= (UINT16) optionBounds->topLeft.x;
	parameter.pY			= (UINT16) optionBounds->topLeft.y;
	parameter.pWidth		= (UINT16) optionBounds->extent.x;
	parameter.pHeight		= (UINT16) optionBounds->extent.y;
	parameter.format		= 0;
	parameter.selected		= selected;
	parameter.is_link		= FALSE;

	parameter.formatstring_length = 0;
	parameter.Formatstring 	= NULL;
	W_WAP_MMI_SEND_TEXT_IND(&parameter);
	
	return;
}


/*******************************************************************************

 $Function:    	GLEa_optionGroupSize

 $Description:	Sets the dimensions of an option group
 
 $Returns:		None.

 $Arguments:	label			- the label of the option (used to calculate size)
 				maxWidth		- maximum width possible (or -1 for no maximum)
 				width			- the width in pixels
 				height			- the height in pixels
 				ascent			- the ascent in pixels.  Should be the number of
 									pixels of the option group above the baseline
 									(<=height)
 				
*******************************************************************************/

VOID GLEa_optionGroupSize(const GLE_STRING label, INT16 maxWidth, INT16 *width, INT16 *height, UINT16 *ascent)
{
#ifdef TRACE_WAP_GLE
	TRACE_EVENT("GLEa_optionGroupSize");
#endif

	*width = (INT16) GLEa_textFontWidth (label, (UINT16) GLEa_strlen((GLE_STRING)label), 0, FALSE);
	*height = (INT16) GLEa_textFontLineHeight (0, FALSE);
	*ascent = (INT16) GLEa_textFontAscent (0, FALSE);
	/* currently no border provided for */
	/* if (*width>maxWidth)
		*width = maxWidth; */
	return;
}


/*******************************************************************************

 $Function:    	GLEa_optionGroupDraw

 $Description:	Draws an option group with a label
 
 $Returns:		None.

 $Arguments:	viewId			- the id of the view
 				label			- the label of the option group (used to calculate size)
 				optionGroupBounds- the position of the upper left hand corner and the
 									allowed width and height
 				
*******************************************************************************/

VOID GLEa_optionGroupDraw(UINT8 viewId, const GLE_STRING label, GleRectangleType *optionGroupBounds)
{
	T_WAP_MMI_SEND_TEXT_IND 	parameter;
	/* unsigned short * UshortText; */

#ifdef TRACE_WAP_GLE
	TRACE_EVENT("GLEa_optionGroupDraw");
#endif

	/* If our option is off the edge of the screen, don't send the event */
	
	if (optionGroupBounds->topLeft.x > view_width || optionGroupBounds->topLeft.y > (view_height-font_height)
		|| (optionGroupBounds->topLeft.x+optionGroupBounds->extent.x)<0 || optionGroupBounds->topLeft.y<0)
		return;
	
	parameter.object_id		= viewId;
	parameter.type			= WAP_TEXT_OPTIONGROUP;
	
	parameter.text_length	= (UINT16) GLEa_strlen((GLE_STRING)label);
	parameter.Text			= (USHORT*)label;
	
	parameter.invert		= FALSE;
	parameter.pX			= (UINT16) optionGroupBounds->topLeft.x;
	parameter.pY			= (UINT16) optionGroupBounds->topLeft.y;
	parameter.pWidth		= (UINT16) optionGroupBounds->extent.x;
	parameter.pHeight		= (UINT16) optionGroupBounds->extent.y;
	parameter.format		= 0;
	parameter.selected		= FALSE;
	parameter.is_link		= FALSE;

	parameter.formatstring_length = 0;
	parameter.Formatstring 	= NULL;
	
	W_WAP_MMI_SEND_TEXT_IND(&parameter);

	return;
}

#else


/*******************************************************************************

 $Function:    	GLEa_selectSize

 $Description:	Sets the dimensions of a select
 
 $Returns:		None

 $Arguments:	label			- the label of the select (used to calculate size)
 				multiselect		- if TRUE, the select is in a multiple-choice menu,
 									otherwise it is in a single-choice menu
 				maxWidth		- maximum width possible (or -1 for no maximum)
 				width			- the width in pixels
 				height			- the height in pixels
 				ascent			- the ascent in pixels.  Should be the number of
 									pixels of the select above the baseline
 									(<=height)
 				
*******************************************************************************/

VOID GLEa_selectSize(const GLE_STRING label, BOOL multiSelect, INT16 maxWidth, INT16 *width, INT16 *height, UINT16 *ascent)
{
#ifdef TRACE_WAP_GLE
	TRACE_EVENT("GLEa_selectSize");
#endif

	*width = (INT16) GLEa_textFontWidth (label, (UINT16) GLEa_strlen((GLE_STRING)label), 0, FALSE);
	*height = (INT16) GLEa_textFontLineHeight (0, FALSE);
	*ascent = (INT16) GLEa_textFontAscent (0, FALSE);
	/* currently no border provided for */
	/* if (*width>maxWidth)
		*width = maxWidth;*/
	return;
}



/*******************************************************************************

 $Function:    	GLEa_selectDraw

 $Description:	Draws a select with a label
 
 $Returns:		None.
 
 $Arguments:	viewId			- the id of the view
 				label			- the label of the select (used to calculate size)
 				multiselect		- if TRUE, the select is in a multiple-choice menu,
 									otherwise it is in a single-choice menu
 				invert			- if TRUE, the select should be drawn marked,
 									otherwise not
 				selectBounds	- the position of the upper left hand corner and the
 									allowed width and height
 				
*******************************************************************************/

VOID GLEa_selectDraw(UINT8 viewId, const GLE_STRING label, BOOL multiSelect, BOOL invert, GleRectangleType *selectBounds)
{
	T_WAP_MMI_SEND_TEXT_IND 	parameter;
	/* unsigned short * UshortText; */

#ifdef TRACE_WAP_GLE
	TRACE_EVENT("GLEa_selectDraw");
#endif

	/* If our option is off the edge of the screen, don't send the event */
	
	if (selectBounds->topLeft.x > view_width || selectBounds->topLeft.y > (view_height-font_height)
		|| (selectBounds->topLeft.x+selectBounds->extent.x)<0 || selectBounds->topLeft.y<0)
		return;
	
	parameter.object_id		= viewId;
	parameter.type			= WAP_TEXT_SELECT;
	
	parameter.text_length	= (UINT16) GLEa_strlen((GLE_STRING)label);
	parameter.Text			= (USHORT*)label;
	
	parameter.invert		= invert;
	parameter.pX			= (USHORT) selectBounds->topLeft.x;
	parameter.pY			= (USHORT) selectBounds->topLeft.y;
	parameter.pWidth		= (USHORT) selectBounds->extent.x;
	parameter.pHeight		= (USHORT) selectBounds->extent.y;
	parameter.format		= NULL;	
	parameter.selected		= 0;
	parameter.is_link		= FALSE;
	parameter.formatstring_length = 0;
	parameter.Formatstring 	= NULL;
	
	W_WAP_MMI_SEND_TEXT_IND(&parameter);

	return;
}


/*******************************************************************************

 $Function:    	GLEa_selectGetLabel

 $Description:	Asks the WAP application for the label to display in the content for the
 				WMLSelect
 
 $Returns:		None.

 $Arguments:	self->multiselect	- if TRUE, the select is in a multiple-choice menu,
 									otherwise it is in a single-choice menu
 				self->next			- next option in the list
 				self->singleSelectedOption -  if list is single-select, this points to the
 									selected option
 				
*******************************************************************************/

VOID GLEa_selectGetLabel(GleSelect* self)
{
#ifdef TRACE_WAP_GLE
	TRACE_EVENT("GLEa_selectGetLabel");
#endif
	
	return;
}

/*******************************************************************************

 $Function:    	GLEa_selectDialog

 $Description:	Opens a dialog box for a WMLSelect
 
 $Returns:		None.

 $Arguments:	viewId		- the id of the view
 				self		- see above
 				
*******************************************************************************/

VOID GLEa_selectDialog(const GleSelect* self)
{
#ifdef TRACE_WAP_GLE
	TRACE_EVENT("GLEa_selectDialog");
#endif

	return;
}

#endif


/*******************************************************************************

 $Function:    	GLEa_tableDraw

 $Description:	Draws the title and cells of a table
 
 $Returns:		None.

 $Arguments:	viewId		- the id of the view
 				pos			- the position of the upper left hand corner
 				title		- the title of the table
 				noOfCells	- the number of cells
 				noOfColumns	- the number of columns
 				noOfRows	- the number of rows
 				colWidth	- array with the widths of columns
 				rowHeight	- array with the height of rows
 				
*******************************************************************************/

VOID GLEa_tableDraw(UINT8 viewId, GlePointType pos, GLE_STRING title, INT8 noOfCells, INT8 noOfColumns, INT8 noOfRows, const INT16 *colWidth, const INT16 *rowHeight)
{
	T_WAP_MMI_SEND_TABLE_IND	parameter;
	SHORT						width;
	SHORT						height;
	USHORT						index;
	
#ifdef TRACE_WAP_GLE
	TRACE_EVENT("GLEa_tableDraw");
#endif

	width = (noOfColumns-1)*gle_cell_hBetween + gle_cell_hBefore + gle_cell_hAfter;
	for (index = 0; index<noOfColumns; index++)
		width += colWidth[index];

	height = (noOfRows-1)*gle_cell_vBetween + gle_cell_vBefore + gle_cell_vAfter;
	for (index = 0; index<noOfRows; index++)
		height += rowHeight[index];
	
	if (pos.x > view_width
		|| pos.y > (view_height-font_height)
		|| (pos.x+width)<0
		|| (pos.y+height)<0)
		return;
	
	parameter.object_id		= viewId;
	parameter.title_length	= (UINT16) GLEa_strlen((GLE_STRING)title);
	parameter.Title			= (USHORT*)title;
	parameter.pX			= pos.x;
	parameter.pY			= pos.y;
	parameter.pWidth		= width;
	parameter.pHeight		= height;
	parameter.cols_length	= (U32)noOfColumns;
	parameter.ColWidth		= (SHORT *)colWidth;
	parameter.rows_length	= (U32)noOfRows;
	parameter.RowHeight		= (SHORT *)rowHeight;
	parameter.noOfCells		= noOfCells;
	W_WAP_MMI_SEND_TABLE_IND(&parameter);

	return;
}

