/*
+-----------------------------------------------------------------------------
|  File     : wap_pei.c
+-----------------------------------------------------------------------------
|                 Copyright Condat AG 1999-2000, Berlin
|                 All rights reserved.
|
|                 This file is confidential and a trade secret of Condat AG
|                 The receipt of or possession of this file does not convey
|                 any rights to reproduce or disclose its contents or to
|                 manufacture, use, or sell anything it may describe, in
|                 whole, or in part, without the specific written consent of
|                 Condat AG.
+-----------------------------------------------------------------------------
|  Purpose  : This module implements the process body interface
|             for the entity wap.                         
|
|             Exported functions:
|              
|             pei_create    - Create the Protocol Stack Entity
|             pei_init      - Initialize Protocol Stack Entity
|             pei_primitive - Process Primitive
|             pei_timeout   - Process Timeout
|             pei_exit      - Close resources and terminate
|             pei_run       - Process Primitive
|             pei_config    - Dynamic Configuration
|             pei_monitor   - Monitoring of physical Parameters
|
| History    :  15/05/2003 - SPR#1983 - SH - Updated to latest from 1.6.3 version.
|
+-----------------------------------------------------------------------------
*/


#ifndef WAP_PEI_C
#define WAP_PEI_C
#endif

#define ENTITY_WAP


/*==== INCLUDES ===================================================*/
#if defined (NEW_FRAME)

#include <stdarg.h>
#include <string.h>
#include <stdlib.h>
#include <stddef.h>
#include "typedefs.h"
#include "pcm.h"
#include "pconst.cdg"
#include "mconst.cdg"
#include "message.h"
#include "ccdapi.h"
#include "vsi.h"
#include "custom.h"
#include "gsm.h"
#include "prim.h"
#include "cnf_wap.h"
#include "mon_wap.h"
#include "pei.h"
#include "tok.h"
#include "dti.h"      /* functionality of the dti library */

#else

#include <string.h>
#include <stdlib.h>
#include <stddef.h>
#include "stddefs.h"
#include "pconst.cdg"
#include "mconst.cdg"
#include "message.h"
#include "ccdapi.h"
#include "custom.h"
#include "gsm.h"
#include "prim.h"
#include "cnf_wap.h"
#include "mon_wap.h"
#include "vsi.h"
#include "pei.h"
#include "tok.h"

#endif

#include "wap.h"

#define WAP_TRACE 1
#include "mmiutilapi.h"
#define TRACE_FUNCTION(f) WAP_TRACE_EVENT((f));
#define TRACE_EVENT(f) TRACE_FUNCTION(f)


/*==== CONST ======================================================*/
/*
 * instance management
 */
#ifdef OPTION_MULTIPLE_INSTANCE
#define GET_INSTANCE(p)           &wap_data_base[p->custom.route.inst_no]
#else
#define GET_INSTANCE(p)           &wap_data_base
#endif


/*==== VAR EXPORT =================================================*/
GLOBAL DTI_HANDLE wap_hDTI;     /* DTI connection for DTI library */
GLOBAL T_HANDLE hCommACI = VSI_ERROR;      /* ACI Communication */
GLOBAL T_HANDLE hCommUDP = VSI_ERROR;      /* UDP Communication */
GLOBAL T_HANDLE wap_handle;

/*==== VAR LOCAL ==================================================*/

#ifdef OPTION_RELATIVE
LOCAL ULONG offset;
#endif

LOCAL USHORT s_flag = 0;         /* signal flag        */
LOCAL BOOL exit_flag = FALSE;
LOCAL BOOL first_access = TRUE;

LOCAL T_MONITOR wap_mon;

#ifdef OPTION_SIGNALS
LOCAL UBYTE *signalDataTable[MAX_SIGNALS];
#endif


/*
 * static buffer for AUS wip_malloc
 */  
LOCAL UBYTE wip_mem[WIP_MEMSIZE]; /* AUS wip memory */

/*==== FUNCTIONS ==================================================*/


/*
+--------------------------------------------------------------------+
| PROJECT : GSM-WAP (8444)             MODULE  : WAP_PEI             |
| STATE   : code                       ROUTINE : pei_handle_signal   |
+--------------------------------------------------------------------+
 
  PURPOSE : Handle the next signal in the signal table
 
*/
LOCAL void pei_handle_signal (void)
{}



/*
+--------------------------------------------------------------------+
| PROJECT : GSM-WAP (8444)             MODULE  : WAP_PEI             |
| STATE   : code                       ROUTINE : pei_not_supported   |
+--------------------------------------------------------------------+
 
  PURPOSE : An unsupported primitive is received.
 
*/
LOCAL const void pei_not_supported (void *data)
{
  TRACE_FUNCTION ("wap pei_not_supported()")

  PFREE (data)
}

/*
 * Use MAK_FUNC_N for non supported opcodes.
 * This entries normally branches to pei_not_supported.
 *
 * Use MAK_FUNC_O for primitives which contains no SDU.
 *
 * Use MAK_FUNC_S for primitives which contains a SDU.
 */

/*
 * jumptable for the entity service access point. Contains
 * the processing-function addresses and opcodes of
 * request and response primitives.
 *
 */

#ifndef TAB_SIZE
#define TAB_SIZE(T) sizeof T / sizeof *T;
#endif

LOCAL const T_FUNC wap_table[] =
  {
    MAK_FUNC_S( mmi_wap_ind,     MMI_WAP_IND),  /* 0x3d00 */
    MAK_FUNC_S( mmi_wap_req,     MMI_WAP_REQ),  /* 0x3d01 */
    MAK_FUNC_S( mmi_wap_cnf,     MMI_WAP_CNF),  /* 0x3d02 */
    MAK_FUNC_0( mmi_wap_dti_req, WAP_DTI_REQ)   /* 0x3d03 */
  };

/*
 * jumptable for the service access point which are used by
 * the entity. Contains the processing-function addresses and
 * opcodes of indication and confirm primitives.
 *
 */
LOCAL const T_FUNC udp_table[] =
  {
    MAK_FUNC_0( udp_bind_cnf,       UDP_BIND_CNF),
    MAK_FUNC_0( udp_closeport_cnf,  UDP_CLOSEPORT_CNF),
    MAK_FUNC_0( udp_error_ind,      UDP_ERROR_IND),
    MAK_FUNC_0( udp_shutdown_ind,   UDP_SHUTDOWN_IND)
  };

LOCAL const T_FUNC dti_dl_table[] = {
  MAK_FUNC_0( pei_dti_dti_connect_ind     ,    DTI2_CONNECT_IND    ),
  MAK_FUNC_0( pei_dti_dti_connect_cnf     ,    DTI2_CONNECT_CNF    ),
  MAK_FUNC_0( pei_dti_dti_disconnect_ind  ,    DTI2_DISCONNECT_IND ),
  MAK_FUNC_0( pei_dti_dti_ready_ind       ,    DTI2_READY_IND      ),
  MAK_FUNC_0( pei_dti_dti_data_ind        ,    DTI2_DATA_IND       )

#if defined (_SIMULATION_)
  ,
  MAK_FUNC_S( pei_dti_dti_data_test_ind   ,    DTI2_DATA_TEST_IND )
#endif  /* _SIMULATION_ */
};

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-WAP (8444)             MODULE  : WAP_PEI             |
| STATE   : code                       ROUTINE : pei_primitive       |
+--------------------------------------------------------------------+
 
  PURPOSE : Process protocol specific primitive.
 
*/
#if defined (NEW_FRAME)
LOCAL SHORT pei_primitive (void * ptr)
#else
GLOBAL T_PEI_RETURN pei_primitive (T_PRIM *prim)
#endif
{
#if defined (NEW_FRAME)
  T_PRIM * prim = ptr;
#endif

  /*
   * @ADAPT@
   *                    |
   *                   MMI                   UPPER LAYER
   *                    |
   *                    v
   *      +-----------(wap)----------+
   *      |                          |
   *      |            WAP           |
   *      |                          |
   *      +-------------^------------+
   *                    |
   *                   UDP                   LOWER LAYER
   *                    |
   *
   */


  if (prim NEQ NULL)
  {
    ULONG            opc = prim->custom.opc;
    USHORT           n;
    const T_FUNC    *table;

    VSI_PPM_REC ((T_PRIM_HEADER *)prim, __FILE__, __LINE__);

    PTRACE_IN (opc);

    wap_data = GET_INSTANCE (prim);

    switch (SAP_NR(opc))
    {
      case WAP_UL:
        table = wap_table;
        n = TAB_SIZE (wap_table);
        break;
      case SAP_NR(UDP_DL):
        table = udp_table;
        n = TAB_SIZE (udp_table);
        break;
      case DTI2_DL:
        {
          table = dti_dl_table;
          n = TAB_SIZE (dti_dl_table);
          /* 
           * to be able to distinguish DTI1/DTI2 opcodes,
           * the ones for DTI2 start at 0x50
           */
          opc -= 0x50;
        }
        break;
      default :
        table = NULL;
        n = 0;
        break;
    }

    if (table NEQ NULL)
    {
      if (PRIM_NR(opc) < n)
      {
        table += PRIM_NR(opc);
#ifdef PALLOC_TRANSITION

        P_SDU(prim) = table->soff ? (T_sdu*) (((char*)&prim->data) + table->soff) : 0;

#ifndef NO_COPY_ROUTING
        P_LEN(prim) = table->size + sizeof (T_PRIM_HEADER);
#endif /* NO_COPY_ROUTING */

#endif /* PALLOC_TRANSITION */
        JUMP (table->func) (P2D(prim));
      }
      else
      {
        pei_not_supported (P2D(prim));
      }
      return PEI_OK;
    }
    /*
     * Primitive is no GSM Primitive
     * then forward to the environment
     */

#ifdef GSM_ONLY
    PFREE (P2D(prim))
    return PEI_ERROR;
#else
    if (opc & SYS_MASK)
      vsi_c_primitive (VSI_CALLER prim);
    else
    {
      PFREE (P2D(prim));
      return PEI_ERROR;
    }
#endif /* GSM_ONLY */
  }
  return PEI_OK;
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-WAP (8444)             MODULE  : WAP_PEI             |
| STATE   : code                       ROUTINE : pei_init            |
+--------------------------------------------------------------------+
 
  PURPOSE : Initialize Protocol Stack Entity
 
*/
#if defined (NEW_FRAME)
LOCAL SHORT pei_init (T_HANDLE handle)
#else
GLOBAL T_PEI_RETURN pei_init (void)
#endif
{
  USHORT wip_initmalloc (void *, ULONG); /* care about this */

#ifdef WAP_USECACHE
#ifndef WAP_CCACHESIZE
  extern const memSizeType cCacheSize;       /* is initialized in capimem.h*/
#else
const memSizeType cCacheSize = WAP_CCACHESIZE;
#endif
#endif

#if defined (NEW_FRAME)
  wap_handle = handle;
#endif
  TRACE_FUNCTION ("wap pei_init()");

  vsi_t_sleep (VSI_CALLER 1);

  if (hCommACI < VSI_OK)
  {
    if ((hCommACI = vsi_c_open (VSI_CALLER ACI_NAME)) < VSI_OK)
      return PEI_ERROR;
  }

  if (hCommUDP < VSI_OK)
  {
    if ((hCommUDP = vsi_c_open (VSI_CALLER UDP_NAME)) < VSI_OK)
      return PEI_ERROR;
  }


  exit_flag = FALSE;

  /*
   * initialize dtilib for this entity
   */
  wap_hDTI = dti_init (
    MAX_INSTANCES,
    handle,
    DTI_DEFAULT_OPTIONS,
    pei_sig_callback
    );
  if(!wap_hDTI)
    return PEI_ERROR;

  /*
   * timer initialization
   */
  if (! wap_init_timer())
    return PEI_ERROR;

  /*
   *  initialize wip_memory for the AUS browser
   */

	wip_initmalloc(wip_mem, WIP_MEMSIZE);

 
  /* initialize wap_data */
#ifdef OPTION_MULTIPLE_INSTANCE
  {
    UBYTE i;  
    for (i = 0 ; i < MAX_INSTANCES; i++)
    {
      wap_data = &wap_data_base[i];
      udp_init(); 
    }
  }
#else
  wap_data = &wap_data_base;
  udp_init();
#endif 
  return PEI_OK;
}



/*
+--------------------------------------------------------------------+
| PROJECT : GSM-WAP (8444)             MODULE  : WAP_PEI             |
| STATE   : code                       ROUTINE : pei_timeout         |
+--------------------------------------------------------------------+
 
  PURPOSE : Process timeout
 
*/
#if defined (NEW_FRAME)
LOCAL SHORT pei_timeout (USHORT index)
#else
GLOBAL T_PEI_RETURN pei_timeout (T_VSI_THANDLE handle)
#endif
{
  wap_timer_expired ();
  return PEI_OK;
}



/*
+--------------------------------------------------------------------+
| PROJECT : GSM-WAP (8444)             MODULE  : WAP_PEI             |
| STATE   : code                       ROUTINE : pei_signal          |
+--------------------------------------------------------------------+
 
  PURPOSE : Process a signal
 
*/
GLOBAL T_PEI_RETURN pei_signal (UBYTE signalNumber,
                                void *signalData)
{
  TRACE_FUNCTION("WAP_Pei_SIGNAL"); 
  return PEI_OK;
}



/*
+--------------------------------------------------------------------+
| PROJECT : GSM-WAP (8444)             MODULE  : WAP_PEI             |
| STATE   : code                       ROUTINE : pei_exit            |
+--------------------------------------------------------------------+
 
  PURPOSE : Close Resources and terminate
 
*/
#if defined (NEW_FRAME)
LOCAL SHORT pei_exit (void)
#else
GLOBAL T_PEI_RETURN pei_exit (void)
#endif
{
#ifdef WAP_USECACHE
#ifndef WAP_CCACHESIZE
  extern const memSizeType cCacheSize;       /* is initialized in capimem.h*/
#else
  const memSizeType cCacheSize = WAP_CCACHESIZE;
#endif
#endif
  
  TRACE_FUNCTION ("wap pei_exit()")

#ifdef OPTION_MULTIPLE_INSTANCE
    ULONG i;
#endif /* OPTION_MULTIPLE_INSTANCE */


  /*
   * Check and clean AUS and WAP-LIB related stuff
   */ 
  /*XXX  not implemented yet XXX*/
     
  
  /*
   * clean up communication
   */
#ifdef OPTION_MULTIPLE_INSTANCE
  /*
   * close channels for all instances
   * should not be neccessary here..
   */
  for (i=0;i<WAP_INSTANCES;i++)
  {
    if (wap_data_base[i].dti_state NEQ DTI_CLOSED)
    {    
      dti_close
      (
        wap_hDTI,
        i,
        WAP_DTI_DN_INTERFACE,
        WAP_DTI_DN_CHANNEL,
        FALSE
      );
      wap_data_base[i].dti_state = DTI_CLOSED;
    }
  }
#else  /* OPTION_MULTIPLE_INSTANCE */
  if (wap_data_base.dti_state NEQ DTI_CLOSED)
  {    
    dti_close
    (
      wap_hDTI,
      WAP_DTI_DN_DEF_INSTANCE,
      WAP_DTI_DN_INTERFACE,
      WAP_DTI_DN_CHANNEL,
      FALSE
    );
    wap_data_base.dti_state = DTI_CLOSED;
  }
#endif /* OPTION_MULTIPLE_INSTANCE */

  vsi_c_close (VSI_CALLER hCommACI);
  hCommACI = VSI_ERROR;

  vsi_c_close (VSI_CALLER hCommUDP);
  hCommUDP = VSI_ERROR;

  /*
   * Shut down dtilib communication
   */
  dti_deinit(wap_hDTI);

  /*
   * Close WAP timer
   */
  wap_close_timer ();

  exit_flag = TRUE;

  return PEI_OK;
}



/*
+--------------------------------------------------------------------+
| PROJECT : GSM-WAP (8444)             MODULE  : WAP_PEI             |
| STATE   : code                       ROUTINE : pei_run             |
+--------------------------------------------------------------------+
 
  PURPOSE : Process Primitives, main loop is located in the
            Protocol Stack Entity
 
*/

#if !defined (NEW_FRAME)
GLOBAL T_PEI_RETURN pei_run (T_VSI_CHANDLE handle)
{
  return PEI_OK;
}
#endif



/*
+------------------------------------------------------------------------------
|  Function    : pei_config
+------------------------------------------------------------------------------
|  Description  : This function is called by the frame when a primitive is
|                 received indicating dynamic configuration.
|
|  Parameters  :  handle            - Communication handle
|
|  Return      :  PEI_OK            - sucessful
|                 PEI_ERROR         - not successful
|
+------------------------------------------------------------------------------
*/
#if !defined (NCONFIG)

  LOCAL KW_DATA kwtab[] = {
#ifdef _SIMULATION_
    UDP_DEF_BIND_REQ,        BIND_REQ,
#endif /* _SIMULATION_ */
    "",                   0
  };

#endif /* !NCONFIG */


LOCAL SHORT pei_config (T_PEI_CONFIG inString)
{
#if !defined (NCONFIG)
  char    *s = inString;
  SHORT    valno;
  SHORT   keyno;
  char    *keyw;
  char    *val [10];
  TRACE_FUNCTION ("wap pei_config()");
  TRACE_EVENT (s);

  tok_init (s);

  /*
   * Parse next keyword and number of variables
   */

  while ((valno = tok_next(&keyw,val)) NEQ TOK_EOCS)
  {
    switch ((keyno = tok_key((KW_DATA *)kwtab, keyw)))
    {
      case TOK_NOT_FOUND:
        TRACE_ERROR ("[PEI_CONFIG]: Illegal Keyword");
        break;

#ifdef _SIMULATION_
      case BIND_REQ:
      {
        void udp_bind_req (UBYTE, UBYTE); /* care about this */

        udp_bind_req(1, 2); /* dummy id, channel */
        break;
      }
#endif /* _SIMULATION_ */

      default:
        break;
    }
  }
#endif /* NCONFIG */
  return PEI_OK;
}


/*
+--------------------------------------------------------------------+
| PROJECT : GSM-WAP (8444)             MODULE  : WAP_PEI             |
| STATE   : code                       ROUTINE : pei_monitor         |
+--------------------------------------------------------------------+
 
  PURPOSE : Monitoring of physical Parameters
 
*/
#if defined (NEW_FRAME)
LOCAL SHORT pei_monitor (void **monitor)
#else
GLOBAL T_PEI_RETURN pei_monitor (void **monitor)
#endif
{
  TRACE_FUNCTION ("wap pei_monitor()")

  wap_mon.version = VERSION_WAP;

  *monitor = &wap_mon;

  return PEI_OK;
}



/*
+--------------------------------------------------------------------+
| PROJECT : GSM-WAP (8444)             MODULE  : WAP_PEI             |
| STATE   : code                       ROUTINE : pei_create          |
+--------------------------------------------------------------------+
 
  PURPOSE : Create the Protocol Stack Entity
 
*/
#if defined (NEW_FRAME)
GLOBAL SHORT wap_pei_create (T_PEI_INFO **info)
{
  static const T_PEI_INFO pei_info =
    {
      "WAP",
      {
        pei_init,
        pei_exit,
        pei_primitive,
        pei_timeout,
        NULL,              /* no signal function  */
        NULL,              /* no run function     */
        pei_config,
        pei_monitor
      },
      6240,                /* stack size          */
      10,                  /* queue entries       */
      179,                 /* priorities          */
      1,                   /* number of timer     */
      0x03                 /* flags               */
    };

  TRACE_FUNCTION ("wap_pei_create()")

  /*
   *  Close Resources if open
   */

  if (first_access)
    first_access = FALSE;
  else
    pei_exit ();

  /*
   *  Export startup configuration data
   */

  * info = (T_PEI_INFO *)&pei_info;

  return PEI_OK;
}

#else

GLOBAL T_PEI_RETURN pei_create (T_VSI_CNAME *name)
{
#ifdef OPTION_RELATIVE
  static T_PEI_RETURN (*compile_addr)() = pei_create;

  offset = (char *)pei_create - (char *)compile_addr;
#endif

  TRACE_FUNCTION ("wap pei_create()")

  /*
  *  Close Resources if open
  */

  if (first_access)
    first_access = FALSE;
  else
    pei_exit ();

  /*
  *  Initialize entity data
  */

  *name = WAP_NAME;

  return PEI_OK;
}

#endif


/*
+------------------------------------------------------------------------------
|    Function    : pei_dti_dti_connect_cnf
+------------------------------------------------------------------------------
|    PURPOSE : Call the process function dti_dti_connect_cnf
+------------------------------------------------------------------------------
*/

GLOBAL const void pei_dti_dti_connect_cnf (
                    T_DTI2_CONNECT_CNF   *dti_connect_cnf
                  )
{
    dti_dti_connect_cnf(wap_hDTI, dti_connect_cnf);  
}

/*
+------------------------------------------------------------------------------
|    Function    : pei_dti_dti_connect_ind
+------------------------------------------------------------------------------
|    PURPOSE : Call the process function dti_dti_connect_ind
+------------------------------------------------------------------------------
*/

GLOBAL const void pei_dti_dti_connect_ind (
                    T_DTI2_CONNECT_IND   *dti_connect_ind
                  )
{
    dti_dti_connect_ind(wap_hDTI, dti_connect_ind);  
}

/*
+------------------------------------------------------------------------------
|    Function    : pei_dti_dti_disconnect_ind
+------------------------------------------------------------------------------
|    PURPOSE : Call the process function dti_dti_disconnect_ind
+------------------------------------------------------------------------------
*/

GLOBAL const void pei_dti_dti_disconnect_ind (
                    T_DTI2_DISCONNECT_IND   *dti_disconnect_ind
                  )
{
    dti_dti_disconnect_ind (wap_hDTI, dti_disconnect_ind);
}

/*
+------------------------------------------------------------------------------
|    Function    : pei_dti_dti_data_ind
+------------------------------------------------------------------------------
|    PURPOSE : Call the process function dti_dti_data_ind
+------------------------------------------------------------------------------
*/

GLOBAL const void pei_dti_dti_data_ind (
                    T_DTI2_DATA_IND   *dti_data_ind
                  )
{
    dti_dti_data_ind (wap_hDTI, dti_data_ind);
}

/*
+------------------------------------------------------------------------------
|    Function    : pei_dti_dti_ready_ind
+------------------------------------------------------------------------------
|    PURPOSE : Call the process function dti_dti_ready_ind
+------------------------------------------------------------------------------
*/

GLOBAL const void pei_dti_dti_ready_ind (
                    T_DTI2_READY_IND   *dti_ready_ind
                  )
{
  dti_dti_ready_ind (wap_hDTI, dti_ready_ind);
}
 
#ifdef _SIMULATION_

/*
+------------------------------------------------------------------------------
|    Function    : pei_dti_dti_data_test_ind
+------------------------------------------------------------------------------
|    PURPOSE : Call the process function dti_dti_data_test_ind
+------------------------------------------------------------------------------
*/

GLOBAL const void pei_dti_dti_data_test_ind (
                    T_DTI2_DATA_TEST_IND   *dti_data_test_ind
                  )
{
  dti_dti_data_test_ind (wap_hDTI, dti_data_test_ind);   
}

#endif /* _SIMULATION_ */


/*
+------------------------------------------------------------------------------
|    Function: pei_sig_callback
+------------------------------------------------------------------------------
|    PURPOSE : Callback function for DTILIB
+------------------------------------------------------------------------------
*/

GLOBAL void pei_sig_callback(U8 instance, U8 interfac, U8 channel, 
                                   U8 reason, T_DTI2_DATA_IND *dti_data_ind)
{
  TRACE_FUNCTION("wap  pei_sig_callback");

#ifdef _SIMULATION_
  if (interfac NEQ WAP_DTI_DN_INTERFACE || channel NEQ WAP_DTI_DN_CHANNEL) 
  {
    TRACE_ERROR("[PEI_SIG_CALLBACK] channel or interface not valid!");
    return; /* error, not found */
  }
#endif /* _SIMULATION_ */

#ifdef OPTION_MULTIPLE_INSTANCE
  wap_data = &wap_data_base[instance];
#else  /* OPTION_MULTIPLE_INSTANCE */
  wap_data = &wap_data_base;
#endif /* OPTION_MULTIPLE_INSTANCE */


  if (wap_hDTI NEQ D_NO_DATA_BASE)
  {
    switch (reason)
    {
      case DTI_REASON_CONNECTION_OPENED:
        sig_dti_udp_connection_opened_ind();
        break;

      case DTI_REASON_CONNECTION_CLOSED:
        sig_dti_udp_connection_closed_ind();
        break;

      case DTI_REASON_DATA_RECEIVED:
        sig_dti_udp_data_received_ind(dti_data_ind);
        break;

      case DTI_REASON_TX_BUFFER_FULL:
        sig_dti_udp_tx_buffer_full_ind();
        break;
      
      case DTI_REASON_TX_BUFFER_READY:
        sig_dti_udp_tx_buffer_ready_ind();
        break;

      default:
        TRACE_ERROR("unknown DTILIB reason parameter");
        break;
    } /* end switch */
  } /* end if */
  else 
  {
    TRACE_ERROR("Pointer to DTILIB database not existing");
  }
} /* pei_sig_callback() */
