/*
+-----------------------------------------------------------------------------
|  File     : wap_udpp.c
+-----------------------------------------------------------------------------
|                 Copyright Condat AG 1999-2000, Berlin
|                 All rights reserved.
|
|                 This file is confidential and a trade secret of Condat AG
|                 The receipt of or possession of this file does not convey
|                 any rights to reproduce or disclose its contents or to
|                 manufacture, use, or sell anything it may describe, in
|                 whole, or in part, without the specific written consent of
|                 Condat AG.
+-----------------------------------------------------------------------------
|  Purpose  : This Modul defines the functions for processing
|             of incomming primitives for the component AUS WAP Browser
|             of the mobile station
|  History   : 15/05/2003 - SPR#1983 - SH - Updated to latest from 1.6.3 version.
+-----------------------------------------------------------------------------
*/

#ifndef WAP_UDPP_C
#define WAP_UDPP_C
#endif


#define ENTITY_WAP

/*==== INCLUDES ===================================================*/
#if defined (NEW_FRAME)

#include <stdarg.h>
#include <string.h>
#include <stdlib.h>
#include <stddef.h>
#include "typedefs.h"
#include "pcm.h"
#include "pconst.cdg"
#include "mconst.cdg"
#include "message.h"
#include "ccdapi.h"
#include "vsi.h"
#include "custom.h"
#include "gsm.h"
#include "prim.h"
#include "cnf_wap.h"
#include "mon_wap.h"
#include "pei.h"
#include "tok.h"
#include "dti.h"      /* functionality of the dti library */

#else

#include <string.h>
#include "stddefs.h"
#include "pconst.cdg"
#include "message.h"
#include "ccdapi.h"
#include "custom.h"
#include "gsm.h"
#include "cnf_wap.h"
#include "mon_wap.h"
#include "prim.h"
#include "vsi.h"
#include "pei.h"
#include "tok.h"

#endif

#include "wap.h"
#include "cus_wap.h"
#include "wap_types.h"

#include "capiclnt.h"
#include "aapiudp.h"
#include "capiudp.h"

/*
 * AUS wip malloc
 */



EXTERN VOID* wip_malloc(UINT32 size);
EXTERN VOID wip_free(VOID *mem);



/****                     ***/

/*==== CONST =======================================================*/

/*==== TYPES =======================================================*/

/*==== VAR EXPORT ==================================================*/

/*==== VAR LOCAL ===================================================*/



/*==== FUNCTIONS ===================================================*/


/*
+--------------------------------------------------------------------+
| PROJECT : GSM-WAP (8444)             MODULE  : WAP_UDPP            |
| STATE   : code                       ROUTINE : udp_bind_cnf        |
+--------------------------------------------------------------------+

  PURPOSE : Process primitive UDP_BIND_CNF received from UDP
*/

GLOBAL const void udp_bind_cnf(
                               T_UDP_BIND_CNF *udp_bind_cnf
                              )
{
  int i;
#ifdef TRACE_WAP_UDP
  TRACE_EVENT("udp_bind_cnf()");
#endif

  PACCESS(udp_bind_cnf);

  if ((i = wap_data->last_bind_req) != -1)
  {
     /*wap_data->last_bind_req = -1;  we need last_bind_req this in case of flow==false */
    switch(udp_bind_cnf->err)
    {
      case UDP_BIND_NOERROR:
        wap_data->ports[i].state = WAP_PORT_BOUND;
        wap_data->ports[i].port = udp_bind_cnf->port;
        break;
      case UDP_BIND_PORTINUSE:
      case UDP_BIND_UDPDOWN:
        wap_data->ports[i].state = WAP_PORT_IDLE;
        wap_data->ports[i].port = 0;
        break;
    }
    PFREE(udp_bind_cnf);
  /*
     *  check flow controll
     */
    if (!(wap_data->flow)) {
      wap_data->connection_done=TRUE;
      udp_send_ready();
    }
    else
    {
    wap_data->last_bind_req=-1;
      wap_lib_bind_callback(wap_data->ports[i].id,wap_data->ports[i].channel, wap_data->ports[i].port,
                         udp_bind_cnf->err);
    }
  }
  else
  {
    TRACE_EVENT("unexpected udp_bind_cnf received")
    PFREE(udp_bind_cnf);
  }
}


/*
+--------------------------------------------------------------------+
| PROJECT : GSM-WAP (8444)             MODULE  : WAP_UDPP            |
| STATE   : code                       ROUTINE : udp_closeport_cnf   |
+--------------------------------------------------------------------+

  PURPOSE : Process primitive UDP_CLOSEPORT_CNF received from UDP
*/

GLOBAL const void  udp_closeport_cnf (
                                       T_UDP_CLOSEPORT_CNF  *udp_closeport_cnf
                                     )
{
  int i;
  UBYTE objectId;
#ifdef TRACE_WAP_UDP
  TRACE_EVENT ("udp_closeport_cnf()");
#endif

  PACCESS (udp_closeport_cnf);

  for (i = 0; i < WAP_MAX_PORTS; i++) {
    if (wap_data->ports[i].state &&
      wap_data->ports[i].state == WAP_PORT_WAIT_CLOSE_CNF)
    {
      wap_data->ports[i].state = WAP_PORT_IDLE;
      wap_data->ports[i].port = 0;
      objectId = wap_data->ports[i].id;
      break;
    }
  }
  wap_lib_closeport_callback(objectId,wap_data->ports[i].channel);
  PFREE (udp_closeport_cnf);
}


/*
+--------------------------------------------------------------------+
| PROJECT : GSM-WAP (8444)             MODULE  : WAP_UDPP            |
| STATE   : code                       ROUTINE : udp_error_ind       |
+--------------------------------------------------------------------+

  PURPOSE : Process primitive UDP_ERROR_IND received from UDP
*/

GLOBAL const void udp_error_ind (
                                 T_UDP_ERROR_IND *udp_error_ind
                                )
{
#ifdef TRACE_WAP_UDP
  TRACE_EVENT ("udp_error_ind()");
#endif

  PACCESS (udp_error_ind);



  UDPc_errorRequest(udp_error_ind->err_msg, udp_error_ind->dst_port);
  PFREE (udp_error_ind);

  {
    PALLOC(udp_error_res, UDP_ERROR_RES);
    PSENDX(UDP, udp_error_res);
  }
  wap_lib_run_aus_callback();
}



/*
+--------------------------------------------------------------------+
| PROJECT : GSM-WAP (8444)             MODULE  : WAP_UDPP            |
| STATE   : code                       ROUTINE : udp_shutdown_ind    |
+--------------------------------------------------------------------+

  PURPOSE : Process primitive UDP_SHUTDOWN_IND received from UDP
*/

GLOBAL const void udp_shutdown_ind (
                                     T_UDP_SHUTDOWN_IND   *udp_shutdown_ind
                                   )
{
  int i;
#ifdef TRACE_WAP_UDP
  TRACE_EVENT ("udp_shutdown_ind()");
#endif

  PACCESS (udp_shutdown_ind);

  for (i = 0; i < WAP_MAX_PORTS; i++)
  {
    switch (wap_data->ports[i].state)
    {
      case WAP_PORT_IDLE:
      case WAP_PORT_WAIT_CLOSE_CNF:
        break;
      case WAP_PORT_WAIT_BIND_CNF:
        wap_lib_bind_callback(wap_data->ports[i].id, wap_data->ports[i].channel, 0, UDP_BIND_UDPDOWN);
        break;
      case WAP_PORT_BOUND:

		{
			void W_WAP_MMI_DISCONNECTED_IND (T_WAP_MMI_DISCONNECTED_IND *); /* care about this */

			T_WAP_MMI_DISCONNECTED_IND parameter;

		#ifdef TRACE_WAP_UDP
			  TRACE_EVENT ("CLNTc_closeConnection()");

				{
					char temp[50];
					sprintf(temp,"wap_data->ports[i].id %d", wap_data->ports[i].id);
					TRACE_EVENT(temp);
				}
		#endif

				/* SPR#2138 - SH */
				CLNTc_setDCHIntConfig (wap_data->ports[i].id, wap_data->ports[i].channel, configONLINE,FALSE);

				CLNTc_closeConnection (wap_data->ports[i].id, wap_data->ports[i].channel, FALSE);

				parameter.object_id = wap_data->ports[i].id;
				parameter.channel = wap_data->ports[i].channel;

				W_WAP_MMI_DISCONNECTED_IND(&parameter);
				wap_lib_run_aus_callback();	
		}
       break;
    }
    wap_data->ports[i].state = WAP_PORT_IDLE;
    wap_data->ports[i].port = 0;
  } /* for */
  wap_data->last_bind_req = -1;

  PFREE (udp_shutdown_ind);


  {
    PALLOC(udp_shutdown_res, UDP_SHUTDOWN_RES);
    PSENDX(UDP, udp_shutdown_res);
  }
}

/*
 * signals from dtilib callback function.
 * these signals are to be seen as primitives
 * wrapped by dtilib and therefore are located
 * here
 */

/*
+------------------------------------------------------------------------+
| PROJECT : GSM-WAP (8444)   MODULE  : WAP_UDPP                          |
| STATE   : code             ROUTINE : sig_dti_udp_connection_opened_ind |
+------------------------------------------------------------------------+

  PURPOSE : Process reason parameter REASON_CONNECTION_OPENED
            received by dtilib callback function
*/
GLOBAL const void sig_dti_udp_connection_opened_ind()
{
  TRACE_FUNCTION("sig_dti_udp_connection_opened_ind()");

  if (wap_data->dti_state EQ DTI_SETUP)
  {
    /*
     * set OPEN and send acknowledgement
     */
    wap_data->dti_state = DTI_IDLE;

    {
      PALLOC (wap_dti_cnf, WAP_DTI_CNF);
      wap_dti_cnf->dti_conn = WAP_CONNECT_DTI;
      PSENDX (ACI, wap_dti_cnf)
    }
  }
}

/*
+-------------------------------------------------------------------------+
| PROJECT : GSM-WAP (8444)    MODULE  : WAP_UDPP                          |
| STATE   : code              ROUTINE : sig_dti_udp_connection_closed_ind |
+-------------------------------------------------------------------------+

  PURPOSE : Process reason parameter REASON_CONNECTION_CLOSED
            received by dtilib callback function
*/
GLOBAL const void sig_dti_udp_connection_closed_ind()
{
  TRACE_FUNCTION("sig_dti_udp_connection_closed_ind()");

  /*
   * set CLOSED and send indication
   */
  wap_data->dti_state = DTI_CLOSED;

  {
    PALLOC (wap_dti_ind, WAP_DTI_IND);
    wap_dti_ind->dti_conn = WAP_DISCONNECT_DTI;
    PSENDX (ACI, wap_dti_ind)
  }

}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-WAP (8444)  MODULE  : WAP_UDPP                       |
| STATE   : code            ROUTINE : sig_dti_udp_tx_buffer_full_ind |
+--------------------------------------------------------------------+

  PURPOSE : Process reason parameter REASON_TX_BUFFER_FULL
            received by dtilib callback function
*/
GLOBAL const void sig_dti_udp_tx_buffer_full_ind()
{
  TRACE_FUNCTION("sig_dti_udp_tx_buffer_full_ind()");

  wap_data->flow = FALSE;
}

/*
+----------------------------------------------------------------------+
| PROJECT : GSM-WAP (8444)   MODULE  : WAP_UDPP                        |
| STATE   : code             ROUTINE : sig_dti_udp_tx_buffer_ready_ind |
+----------------------------------------------------------------------+

  PURPOSE : Process reason parameter REASON_TX_BUFFER_READY
            received by dtilib callback function

            this function replaces former udp_dti_ready_ind
*/
GLOBAL const void sig_dti_udp_tx_buffer_ready_ind()
{
  TRACE_FUNCTION("sig_dti_udp_tx_buffer_ready_ind()");

  wap_data->flow = TRUE;

  if (wap_data->connection_done && wap_data->last_bind_req != -1)
  {
    wap_data->connection_done = FALSE;
    wap_lib_bind_callback(wap_data->ports[wap_data->last_bind_req].id,
             wap_data->ports[wap_data->last_bind_req].channel,
                       wap_data->ports[wap_data->last_bind_req].port,
                       UDP_BIND_NOERROR);
    wap_data->last_bind_req=-1;

  }
  wap_data->aus_runnable = TRUE;
  wap_lib_run_aus_callback();
}

/*
+--------------------------------------------------------------------+
| PROJECT : GSM-WAP (8444)   MODULE  : WAP_UDPP                      |
| STATE   : code             ROUTINE : sig_dti_udp_data_received_ind |
+--------------------------------------------------------------------+

  PURPOSE : Process reason parameter REASON_DATA_RECEIVED
            received by dtilib callback function

            this function replaces former udp_dti_data_ind
*/
GLOBAL const void sig_dti_udp_data_received_ind
                          (
                            T_DTI2_DATA_IND *dti_data_ind
                          )
{
  T_desc2 *desc, *tmp;
  T_SRC_DES *pSrcDes;
  char src_ip[16];
  char des_ip[16];
  USHORT src_len =4;
  USHORT des_len=4;
  USHORT src_port;
  USHORT des_port;
  char* data;
  USHORT data_len, i=0;

  TRACE_FUNCTION("sig_dti_udp_data_received_ind()");
  PACCESS (dti_data_ind);

  /*
   * prevent dtilib from automatically sending flow control primitives
   *
  dti_stop
  (
    wap_hDTI,
    WAP_DTI_DN_DEF_INSTANCE,
    WAP_DTI_DN_INTERFACE,
    WAP_DTI_DN_CHANNEL
  );
   *
   * not necessary here, because we want to send a getdata request
   * anyway, when done with this function ..
   */

#ifdef _SIMULATION_
  /*
   * get the T_SRC_DES out of the first and only descriptor
   * created by dtilib and put it in a descriptor of its own
   */
  if(dti_data_ind->desc_list2.first &&
     dti_data_ind->desc_list2.list_len >= sizeof(T_SRC_DES))
  {
    T_desc2 *old_desc, *addr_desc, *test_desc;
    USHORT  i;

    old_desc = (T_desc2 *)dti_data_ind->desc_list2.first;

    /*
     * build the T_SRC_DES for IP-addresses and ports
     */
    MALLOC (
      addr_desc, (USHORT)(sizeof(T_desc2) - 1 + sizeof(T_SRC_DES))
      );
    addr_desc->len    = sizeof(T_SRC_DES);
    addr_desc->size   = sizeof(T_SRC_DES);
    addr_desc->offset = 0;
    for(i=0; i < addr_desc->len; i++)
      addr_desc->buffer[i] = old_desc->buffer[i];

    /*
     *  Build a desc for the data
     */
    MALLOC (
      test_desc,
      (USHORT)(sizeof(T_desc2) - 1 +
        old_desc->len - sizeof(T_SRC_DES))
      );
    test_desc->len    = old_desc->len - sizeof(T_SRC_DES);
    test_desc->size   = test_desc->len;
    test_desc->offset = 0;
    for(i=0; i < test_desc->len; i++)
      test_desc->buffer[i]
        = old_desc->buffer[i + addr_desc->len];

    dti_data_ind->desc_list2.first = (ULONG)addr_desc;
    addr_desc->next = (ULONG)test_desc;
    test_desc->next = old_desc->next;

    MFREE(old_desc);
  }
#endif /* _SIMULATION_ */

  desc = (T_desc2*)dti_data_ind->desc_list2.first;
  pSrcDes = (T_SRC_DES*)desc->buffer;

  udp_conv_IP_to_str(pSrcDes->src_ip, src_ip);
  udp_conv_IP_to_str(pSrcDes->des_ip, des_ip);

  src_port = (pSrcDes->src_port[1]<<8) + pSrcDes->src_port[0];
  des_port = (pSrcDes->des_port[1]<<8) + pSrcDes->des_port[0];

  /*
   * copy data
   */

  data_len = dti_data_ind->desc_list2.list_len - desc->len;
  if ( 0 == (data = (char*)wip_malloc((ULONG)data_len)))
  {
    PFREE(dti_data_ind);
/*
 * sent automatically by dtilib
 *
    udp_send_ready();
 */
    TRACE_ERROR("wip_malloc() failed");
    return;
  }

  while ((desc = (T_desc2 *)desc->next) != NULL)
  {
    memcpy(data+i, desc->buffer, desc->len);
    i += desc->len;
  }

  /*
   * free descriptors
   */
  desc = (T_desc2 *)dti_data_ind->desc_list2.first;

  do
  {
    tmp = desc;
    desc = (T_desc2 *) (desc->next);
    MFREE(tmp);
  } while(desc != NULL);

  PFREE (dti_data_ind);

  /*
   * call AUS
   */
#ifdef TRACE_WAP_UDP
  TRACE_EVENT("UDPc_receivedRequest");
  {
    char temp[100];
    char* temp2;
    int i, j;

    TRACE_EVENT_P1("RECEIVED %d bytes of data", data_len);
    TRACE_EVENT_P5("Source IP: %u.%u.%u.%u:%u", (unsigned int)src_ip[0],
                                                (unsigned int)src_ip[1],
                                                (unsigned int)src_ip[2],
                                                (unsigned int)src_ip[3],
                                                (unsigned int)src_port);
    TRACE_EVENT_P5("Dest   IP: %u.%u.%u.%u:%u", (unsigned int)des_ip[0],
                                                (unsigned int)des_ip[1],
                                                (unsigned int)des_ip[2],
                                                (unsigned int)des_ip[3],
                                                (unsigned int)des_port);
    /* Dump the data package in human readable form if you want */
    /*
    for (j=0; (j<data_len) && (j<100); j+=20)         // trace max 100 bytes
    {
      temp2 = (char*)temp;
      temp2 += sprintf(temp2,"%04X:  ", j);       
      for (i=j; ((i<data_len)&&(i<j+20)); i++)
        temp2 += sprintf(temp2,"%02X ",data[i]);      // The data itself in HEX
      for (   ; i<=j+20; i++)
        temp2 += sprintf(temp2,"   ");
      temp2 += sprintf(temp2,"  : ");                 // fill to the end of line
      for (i=j; ((i<data_len)&&(i<j+20)); i++)
      {
        char c=data[i];
        if (c>=' ' && c<127)
          *temp2++=c;                                 // add the asci-char
        else
          *temp2++='.';                               // nonprintable char
      }
      *temp2='\0';
      TRACE_EVENT(temp);
    }
    */
  }
#endif /* TRACE_WAP_UDP */ 

  UDPc_receivedRequest(data,
                       data_len,
                       des_ip,
                       (unsigned char) des_len,
                       src_ip,
                       (unsigned char) src_len,
                       des_port,
                       src_port,
                       0);


  ;

  wip_free(data);

/*
 * sent automatically by dtilib
 *
  udp_send_ready();
 */

  wap_lib_run_aus_callback();
}

