/*******************************************************************************

					CONDAT (UK)

********************************************************************************                                                                              

 This software product is the property of Condat (UK) Ltd and may not be
 disclosed to any third party without the express permission of the owner.                                 
                                                                              
********************************************************************************

 $Project name:	                                                      
 $Project code:	                                                           
 $Module:		
 $File:		  	wap_wapmmi.c
 $Revision:		                                                      
                                                                              
 $Author:		Condat(UK)                                                         
 $Date:		                                                          
                                                                               
********************************************************************************
                                                                              
 Description:
 	Create events to send to MMI, and receive events from MMI to forward.
    
                        
********************************************************************************

 $History: wap_wapmmi.c

	20/05/2003 - SPR#1934 - SH - Synchronised latest with 1.7.0 version.
	   
 $End

*******************************************************************************/
#ifndef WAP_WAPMMI_C
#undef WAP_WAPMMI_C
#endif

#define ENTITY_WAP


/*==== INCLUDES ===================================================*/
#if defined (NEW_FRAME)

#include <stdarg.h>
#include <string.h>
#include <stdlib.h>
#include <stddef.h>
#include "typedefs.h"
#include "pcm.h"
#include "pconst.cdg"
#include "mconst.cdg"
#include "message.h"
#include "ccdapi.h"
#include "vsi.h"
#include "custom.h"
#include "gsm.h"
#include "prim.h"
#include "cnf_wap.h"
#include "mon_wap.h"
#include "pei.h"
#include "tok.h"

#else

#include <string.h>
#include "stddefs.h"
#include "pconst.cdg"
#include "message.h"
#include "ccdapi.h"
#include "custom.h"
#include "gsm.h"
#include "cnf_wap.h"
#include "mon_wap.h"
#include "prim.h"
#include "vsi.h"
#include "pei.h"
#include "tok.h"

#endif

#include "dti.h"
#include "wap.h"
#include "cus_wap.h"

#include "aapimmi.h"
#include "capimmi.h"
#include "aapiclnt.h"
#include "aapigle.h"
#include "capiclnt.h"
#include "capigle.h"
#include "wip_mem.h"
#include "capimem.h"
#include "tapigle.h"

#include "wap_types.h"
#include "wap_codec.h"
#include "wap_wapmmi.h"

//#define WAP_TRACE 1
#define TRACE_WAP_WAPMMI
#include "mmiutilapi.h"
#define TRACE_EVENT(f) WAP_TRACE_EVENT(("wapaus-wap_wapmmi.c<<<talcon,%s",f));
#define TRACE_FUNCTION(f) TRACE_EVENT(f)



UBYTE startup_failed = FALSE;	/*  TRUE if WAP start fails */

/* Functions to display strings, for tracing purposes. */
 
void WAP_trace_string(char *String, UBYTE length)
{
	UBYTE stringIndex;
	char trace_string[41];  

	for (stringIndex=0; (stringIndex<40 && stringIndex<length); stringIndex++)
		trace_string[stringIndex] = String[stringIndex];

	trace_string[stringIndex] = 0;

	TRACE_EVENT_P2("%s (%d)", trace_string, length);

	return;
}

void WAP_trace_ushort_string(USHORT *String, USHORT length)
{
	USHORT stringIndex;
	char trace_string[41];
	
	for (stringIndex=0; (stringIndex<40 && stringIndex<length); stringIndex++)
		trace_string[stringIndex] = (char) String[stringIndex];

	trace_string[stringIndex] = 0;

	TRACE_EVENT_P2("%s (%d)", trace_string, length);

	return;
}


/*******************************************************************************

 $Function:    	W_MMI_WAP_READY_IND
 
 $Description:	Informs WAP that MMI is ready to receive events.
 
 $Returns:		None.

 $Arguments:	parameter - data block
 
*******************************************************************************/

void W_MMI_WAP_READY_IND(T_MMI_WAP_READY_IND *parameter)
{

#ifdef TRACE_WAP_WAPMMI
	TRACE_EVENT("W_MMI_WAP_READY_IND");
#endif

	/* Does nothing - already intercepted by wap_codec */
	
return;
}


/*******************************************************************************

 $Function:    	W_MMI_WAP_START_IND
 
 $Description:	Starts the WAP application.
 
 $Returns:		None.

 $Arguments:	parameter - data block
 
*******************************************************************************/

void W_MMI_WAP_START_IND(T_MMI_WAP_START_IND *parameter)
{
	T_WAP_MMI_READY_CNF parameter2;
	
#ifdef TRACE_WAP_WAPMMI
	TRACE_EVENT("W_MMI_WAP_START_IND");
#endif

	/* If startup fails, do not continue */
	
	startup_failed = FALSE;
	GLEc_start();

	if (!startup_failed)
	{ 
		WAP_init_cache();

		MEMc_initCache(WAP_cache_size);

		wap_lib_run_aus_callback();
		 
		parameter2.event_id = WAP_START_IND_ID;
		W_WAP_MMI_READY_CNF(&parameter2);	
	}
	
	return;
}


/*******************************************************************************

 $Function:    	W_MMI_WAP_NEW_VIEW_IND
 
 $Description:	Create a new view
 
 $Returns:		None.

 $Arguments:	parameter - data block
 
*******************************************************************************/

void W_MMI_WAP_NEW_VIEW_IND(T_MMI_WAP_NEW_VIEW_IND *parameter)
{
	T_WAP_MMI_READY_CNF parameter2;
	
#ifdef TRACE_WAP_WAPMMI
	TRACE_EVENT("W_MMI_WAP_NEW_VIEW_IND");
	TRACE_EVENT_P1("WAP object ID: %d",parameter->object_id);
	TRACE_EVENT_P2("WAP id/uamode: %d, %d", parameter->object_id, parameter->uaMode);
	TRACE_EVENT_P2("WAP width/height: %d, %d", (USHORT)parameter->view_width, (USHORT)parameter->view_height);
	TRACE_EVENT_P2("font height: %d, width of 'A': %d", parameter->font_height, parameter->font_width[65]);
	TRACE_EVENT_P2("unicode height: %d, unicode width: %d", parameter->unicode_width, parameter->unicode_height);
#endif

	/* Ensure new view is not set up if startup fails */
	
	if (!startup_failed)
	{
		/* Send display data to wap_gle */
		setupDisplayData(parameter->view_width, parameter->view_height, parameter->font_width,
			parameter->font_height, parameter->unicode_width, parameter->unicode_height);

		GLEc_openView(parameter->object_id, parameter->uaMode, parameter->view_width, parameter->view_height);
	 	
		wap_lib_run_aus_callback();

		parameter2.event_id = WAP_NEW_VIEW_IND_ID;
		W_WAP_MMI_READY_CNF(&parameter2);
	}
	return;
}


/*******************************************************************************

 $Function:    	W_MMI_WAP_TERMINATE_IND
 
 $Description:	Closes the WAP application and frees any memory allocated.
 
 $Returns:		None.

 $Arguments:	parameter - data block
 
*******************************************************************************/

void W_MMI_WAP_TERMINATE_IND(T_MMI_WAP_TERMINATE_IND *parameter)
{
	T_WAP_MMI_READY_CNF parameter2;

#ifdef TRACE_WAP_WAPMMI
	TRACE_EVENT("W_MMI_WAP_TERMINATE_IND");
#endif /*TRACE_WAP_WAPMMI */

	/* Don't terminate if startup failed */

	if (!startup_failed)
	{
		freeDisplayData();

		GLEc_terminate();

		CLNTc_terminate();

		wap_lib_run_aus_callback();
#ifdef TRACE_WAP_WAPMMI
		TRACE_EVENT("W_MMI_WAP_TERMINATE_IND complete");
#endif
	}
	else
	{
		TRACE_EVENT("Startup failed - not terminating.");
	}

	parameter2.event_id = WAP_TERMINATE_IND_ID;
	W_WAP_MMI_READY_CNF(&parameter2);		
	
	return;
}


/*******************************************************************************

 $Function:    	W_MMI_WAP_CLOSE_VIEW_IND
 
 $Description:	Close the specified view.
 
 $Returns:		None.

 $Arguments:	parameter - data block
 
*******************************************************************************/

void W_MMI_WAP_CLOSE_VIEW_IND(T_MMI_WAP_CLOSE_VIEW_IND *parameter)
{
	T_WAP_MMI_READY_CNF parameter2;
	
#ifdef TRACE_WAP_WAPMMI
	TRACE_EVENT("W_MMI_WAP_CLOSE_VIEW_IND");
	TRACE_EVENT_P1("WAP object ID: %d",parameter->object_id);
#endif

	/* Don't close view if startup failed */
	
	if (!startup_failed)
	{
		
		GLEc_closeView(parameter->object_id);

		wap_lib_run_aus_callback();
	}
	
	parameter2.event_id = WAP_CLOSE_VIEW_IND_ID;
	W_WAP_MMI_READY_CNF(&parameter2);	
	
	return;
}


/*******************************************************************************

 $Function:    	W_MMI_WAP_CONFIGURE_IND
 
 $Description:	Sends configuration information to the browser
 
 $Returns:		None

 $Arguments:	parameter - data block
 
*******************************************************************************/

void W_MMI_WAP_CONFIGURE_IND(T_MMI_WAP_CONFIGURE_IND *parameter)
{
#ifdef TRACE_WAP_WAPMMI
	TRACE_EVENT("W_MMI_WAP_CONFIGURE_IND");
	WAP_TRACE_EVENT(("WAP object ID: %d",parameter->object_id));
	TRACE_EVENT_P3("WAP Type, Param, Length: %d, %d, %d", parameter->type, parameter->param, parameter->length);
	if (parameter->type==WAP_IntConfig)
	{
		WAP_TRACE_EVENT(("WAP Integer Value: %d", parameter->intvalue));
	}
	else if (parameter->length>0)
	{
		WAP_trace_string(parameter->strvalue, parameter->length);
	}
#endif

	/* Integer configuration */
	
	if (parameter->type==WAP_IntConfig)
	{
		GLEc_setIntConfig(parameter->object_id, (ConfigInt)parameter->param, parameter->intvalue);
	}

	/* String configuration */
	
	if (parameter->type==WAP_StrConfig)
	{
		CLNTc_setStrConfig(parameter->object_id, (ConfigStr)parameter->param, parameter->strvalue,
			(UINT16)parameter->length);
	}

	wap_lib_run_aus_callback();

	return;
}


/*******************************************************************************


 $Function:    	W_MMI_WAP_CONNECTION_CONFIGURE_IND
 
 $Description:	Sends network configuration information to the browser, such as the
 				bearer, IP addresses, passwords and ids etc.
 
 $Returns:		None.

 $Arguments:	parameter - data block
 
*******************************************************************************/

void W_MMI_WAP_CONNECTION_CONFIGURE_IND(T_MMI_WAP_CONNECTION_CONFIGURE_IND *parameter)
{
#ifdef TRACE_WAP_WAPMMI
	TRACE_EVENT("W_MMI_WAP_CONNECTION_CONFIGURE_IND");
	TRACE_EVENT_P1("WAP object ID: %d",parameter->object_id);
	TRACE_EVENT_P1("WAP param: %d",parameter->param);
	TRACE_EVENT_P3("WAP Type, Channel_id, Length: %d, %d, %d", parameter->type, parameter->channel_id, parameter->length);
	if (parameter->type==WAP_IntConfig)
	{
		TRACE_EVENT_P1("WAP Integer Value: %d", parameter->intvalue);
	}
	else if (parameter->length>0)
	{
		WAP_trace_string(parameter->strvalue, parameter->length);
	}
#endif

	/* These configurations are not sent to AUSB */
								
	if (parameter->param > WAP_additionalConfig)
	{
		switch (parameter->param)
		{
			case WAP_setPort1:
				wap_data->port1 = parameter->intvalue;
			break;

			case WAP_setPort2:
				wap_data->port2 = parameter->intvalue;
			break;
			}

		return;
		}

	/* Integer configuration */
	
	if (parameter->type==WAP_IntConfig)
	{
		CLNTc_setDCHIntConfig(parameter->object_id, parameter->channel_id, (ConfigInt)parameter->param,
			parameter->intvalue);
		}

	/* String configuration */
	
	if (parameter->type==WAP_StrConfig)
	{
		CLNTc_setDCHStrConfig(parameter->object_id, parameter->channel_id, (ConfigStr)parameter->param,
			parameter->strvalue, (UBYTE)parameter->length);
		}

	wap_lib_run_aus_callback();
		
	return;
}


/*******************************************************************************


 $Function:    	W_MMI_WAP_DOWNLOAD_URL_IND
 
 $Description:	Requests the WAP browser to download a particular URL.
 
 $Returns:		None.

 $Arguments:	parameter - data block
 
*******************************************************************************/

void W_MMI_WAP_DOWNLOAD_URL_IND(T_MMI_WAP_DOWNLOAD_URL_IND *parameter)
{		
#ifdef TRACE_WAP_WAPMMI
	TRACE_EVENT("W_MMI_WAP_DOWNLOAD_URL_IND");
	TRACE_EVENT_P1("WAP object ID: %d",parameter->object_id);
	TRACE_EVENT_P2("WAP URL length, Reload: %d, %d", parameter->url_length, parameter->reload);
	WAP_trace_string(parameter->Url, parameter->url_length);
#endif

	MMIc_loadURL(parameter->object_id, parameter->Url, parameter->reload);

	wap_lib_run_aus_callback();
		
	return;
}


/*******************************************************************************


 $Function:    	W_MMI_WAP_BROWSE_CONTROL_IND
 
 $Description:	Sends a control command to the browser, e.g. stop, refresh, redisplay,
 				press a key, go back, block the screen
 
 $Returns:		None.

 $Arguments:	parameter - data block
 
*******************************************************************************/

void W_MMI_WAP_BROWSE_CONTROL_IND(T_MMI_WAP_BROWSE_CONTROL_IND *parameter)
{
	GleEvent event;

#ifdef TRACE_WAP_WAPMMI
	TRACE_EVENT("W_MMI_WAP_BROWSE_CONTROL_IND");
	TRACE_EVENT_P1("WAP object ID: %d",parameter->object_id);
	TRACE_EVENT_P1("Browse command: %d", parameter->browse_command);
#endif

	switch(parameter->browse_command)
	{
		case WAP_DOWNLOAD_STOP:
			GLEc_stop(parameter->object_id);
			break;
		case WAP_CARD_REFRESH:
			MMIc_reload(parameter->object_id);
			break;
		case WAP_CARD_DISPLAY:
			GLEc_drawView(parameter->object_id);
			break;
		case WAP_CARD_GO_BACK:
			MMIc_goBack(parameter->object_id);
			break;
		case WAP_KEY_UP:
			event.eventType = KeyUp;
			GLEc_handleEvent(parameter->object_id, event);
			break;
		case WAP_KEY_DOWN:
			event.eventType = KeyDown;
			GLEc_handleEvent(parameter->object_id, event);
			break;
		case WAP_KEY_LEFT:
			event.eventType = KeyLeft;
			GLEc_handleEvent(parameter->object_id, event);
			break;
		case WAP_KEY_RIGHT:
			event.eventType = KeyRight;
			GLEc_handleEvent(parameter->object_id, event);
			break;
		case WAP_KEY_SELECT:
			event.eventType = KeyYes;
			GLEc_handleEvent(parameter->object_id, event);
			break;
		case WAP_KEY_CANCEL:
			event.eventType = KeyLinkInfo;
			GLEc_handleEvent(parameter->object_id, event);
			break;
		case WAP_KEY_SCROLL:
			GLEc_verticalScrollbarChanged(parameter->object_id, parameter->browse_parameter);
			break;
		case WAP_BLOCK_VIEW:
			break;
		case WAP_UNBLOCK_VIEW:
			break;
		}

	wap_lib_run_aus_callback();
		
	return;
}


/*******************************************************************************


 $Function:    	W_MMI_WAP_KEY_SELECTED_IND
 
 $Description:	Send the id of the selected WAP key to the AUS Browser
 
 $Returns:		None.

 $Arguments:	parameter - data block
 
*******************************************************************************/

void W_MMI_WAP_KEY_SELECTED_IND(T_MMI_WAP_KEY_SELECTED_IND *parameter)
{
#ifdef TRACE_WAP_WAPMMI
	TRACE_EVENT("W_MMI_WAP_KEY_SELECTED_IND");
	TRACE_EVENT_P1("WAP object ID: %d",parameter->object_id);
	TRACE_EVENT_P1("Key ID: %d", parameter->keyId);
#endif

	MMIc_keySelected(parameter->object_id, (UINT8) parameter->keyId);

	wap_lib_run_aus_callback();
		
	return;
}


/*******************************************************************************


 $Function:    	W_MMI_WAP_INPUT_DIALOG_CNF
 
 $Description:	Provides the answer to an input dialog
 
 $Returns:		None.

 $Arguments:	parameter - data block
 
*******************************************************************************/

void W_MMI_WAP_INPUT_DIALOG_CNF(T_MMI_WAP_INPUT_DIALOG_CNF *parameter)
{	

	UINT16 size;
	GLE_STRING text;

#ifdef TRACE_WAP_WAPMMI
	TRACE_EVENT("W_MMI_WAP_INPUT_DIALOG_CNF");
	TRACE_EVENT_P1("WAP object ID: %d",parameter->object_id);
	TRACE_EVENT_P2("WAP dialog_id: %d, edited %d", parameter->dialog_id, parameter->edited);
	WAP_trace_ushort_string(parameter->Text, parameter->text_length);
#endif

	/* SPR#2393 - SH - If dialog pointer exists, it was a GLE dialog. */
	
	if (parameter->dialog_pointer!=NULL)
	{
	GLEc_inputDialogResponse((GleInput*)parameter->dialog_pointer,(GLE_STRING)parameter->Text,parameter->edited);
	}
	else
	{
		MMIc_promptDialogResponse(parameter->object_id, parameter->dialog_id, (WCHAR *)parameter->Text);
	}

	wap_lib_run_aus_callback();
	return;
}


/*******************************************************************************


 $Function:    	W_MMI_WAP_PASSWORD_DIALOG_CNF
 
 $Description:	Provides the answer to a password dialog
 
 $Returns:		None.

 $Arguments:	parameter - data block
 
*******************************************************************************/

void W_MMI_WAP_PASSWORD_DIALOG_CNF(T_MMI_WAP_PASSWORD_DIALOG_CNF *parameter)
{
#ifdef TRACE_WAP_WAPMMI
	TRACE_EVENT("W_MMI_WAP_PASSWORD_DIALOG_CNF");
	TRACE_EVENT_P1("WAP object ID: %d",parameter->object_id);
	TRACE_EVENT_P1("WAP dialog_id: %d", parameter->dialog_id);
	WAP_trace_string(parameter->Input, parameter->input_length);
	WAP_trace_string(parameter->Password, parameter->password_length);
#endif

	MMIc_passwordDialogResponse(parameter->object_id, (UINT16) parameter->dialog_id,
		parameter->Input, parameter->Password);

	wap_lib_run_aus_callback();
	
	return;
}


/*******************************************************************************


 $Function:    	W_MMI_WAP_CONFIRM_DIALOG_CNF
 
 $Description:	Provides the answer to a confirm dialog
 
 $Returns:		None.

 $Arguments:	parameter - data block
 
*******************************************************************************/

void W_MMI_WAP_CONFIRM_DIALOG_CNF(T_MMI_WAP_CONFIRM_DIALOG_CNF *parameter)
{
#ifdef TRACE_WAP_WAPMMI
	TRACE_EVENT("W_MMI_WAP_CONFIRM_DIALOG_CNF");
	TRACE_EVENT_P1("WAP object ID: %d",parameter->object_id);
	TRACE_EVENT_P2("WAP dialog_id: %d, Answer: %d", parameter->dialog_id, parameter->answer);
#endif

	MMIc_confirmDialogResponse(parameter->object_id, parameter->dialog_id, parameter->answer);

	wap_lib_run_aus_callback();
		
	return;
}


/*******************************************************************************


 $Function:    	W_MMI_WAP_INFO_DIALOG_CNF
 
 $Description:	Provides the answer to an info dialog
 
 $Returns:		None.

 $Arguments:	parameter - data block
 
*******************************************************************************/

void W_MMI_WAP_INFO_DIALOG_CNF(T_MMI_WAP_INFO_DIALOG_CNF *parameter)
{
#ifdef TRACE_WAP_WAPMMI
	TRACE_EVENT("W_MMI_WAP_INFO_DIALOG_CNF");
	TRACE_EVENT_P1("WAP object ID: %d",parameter->object_id);
	TRACE_EVENT_P1("WAP dialog_id: %d", parameter->dialog_id);
#endif

	MMIc_alertDialogResponse(parameter->object_id, parameter->dialog_id);

	wap_lib_run_aus_callback();
	
	return;
}


/*******************************************************************************


 $Function:    	W_MMI_WAP_DRAW_CARD_REQ
 
 $Description:	Notifies the WAP that the display events can be sent to the MMI
 
 $Returns:		None.

 $Arguments:	parameter - data block
 
*******************************************************************************/

void W_MMI_WAP_DRAW_CARD_REQ(T_MMI_WAP_DRAW_CARD_REQ *parameter)
{
#ifdef TRACE_WAP_WAPMMI
	TRACE_EVENT("W_MMI_WAP_DRAW_CARD_REQ");
	TRACE_EVENT_P1("WAP object ID: %d",parameter->object_id);
#endif 
	GLEc_drawView(parameter->object_id);	/* Redraw the current view */

	wap_lib_run_aus_callback();
	
	return;
}


/*******************************************************************************


 $Function:    	W_MMI_WAP_CONNECT_CNF
 
 $Description:	MMI confirms that WAP call is connected.
 
 $Returns:		None.

 $Arguments:	parameter - data block
 
*******************************************************************************/

void W_MMI_WAP_CONNECT_CNF(T_MMI_WAP_CONNECT_CNF *parameter)
{
#ifdef TRACE_WAP_WAPMMI
	TRACE_EVENT("W_MMI_WAP_CONNECT_CNF");
	TRACE_EVENT_P3("MMI object ID: %d channel %d success %d",parameter->object_id,parameter->channel,parameter->success);
#endif

	if (parameter->success)
		udp_bind_req(parameter->object_id,parameter->channel);
	else
		CLNTc_setupConnectionDone(parameter->object_id,parameter->channel,FALSE);

	return;
}


/*******************************************************************************


 $Function:    	W_MMI_WAP_DISCONNECT_IND
 
 $Description:	Instruction from MMI to disconnect.
 
 $Returns:		None.

 $Arguments:	parameter - data block
 
*******************************************************************************/

void W_MMI_WAP_DISCONNECT_IND(T_MMI_WAP_DISCONNECT_IND *parameter)
{
#ifdef TRACE_WAP_WAPMMI
	WAP_TRACE_EVENT(("W_MMI_WAP_DISCONNECT_IND: %d", startup_failed));
	TRACE_EVENT_P2("MMI object ID: %d, channel: %d",parameter->object_id, parameter->channel);
#endif

	/* Don't close connection if startup failed */
	
	if (!startup_failed)
	{
		/* Close connection */

		CLNTc_closeConnection(parameter->object_id, parameter->channel, FALSE);

		wap_lib_run_aus_callback();	
	}
	
	return;
}


/*******************************************************************************

 $Function:    	W_MMI_WAP_CACHE_PREPARE_IND
 
 $Description:	Prepare the cache for shutdown.
 
 $Returns:		None.

 $Arguments:	parameter - data block
 
*******************************************************************************/

void W_MMI_WAP_CACHE_PREPARE_IND(T_MMI_WAP_CACHE_PREPARE_IND *parameter)
{
	T_WAP_MMI_READY_CNF parameter2;
	
	#ifdef TRACE_WAP_WAPMMI
	TRACE_EVENT("W_MMI_WAP_CACHE_PREPARE_IND");
#endif

	if (!startup_failed)
	{
		MEMc_prepareCache(WAP_cache_size);

		wap_lib_run_aus_callback();
	}
	else
	{
		/* Startup has failed, so we just acknowledge this step */
		parameter2.event_id = WAP_CACHE_PREPARE_IND_ID;
		W_WAP_MMI_READY_CNF(&parameter2);
	}
	
	return;
}


/*******************************************************************************


 $Function:    	W_WAP_MMI_CONTROL_IND
 
 $Description:	Send control information to MMI.
 
 $Returns:		None.

 $Arguments:	parameter - data block
 
*******************************************************************************/

void W_WAP_MMI_CONTROL_IND(T_WAP_MMI_CONTROL_IND *parameter)
{
#ifdef TRACE_WAP_WAPMMI
	TRACE_EVENT("W_WAP_MMI_CONTROL_IND");
	TRACE_EVENT_P1("WAP object ID: %d",parameter->object_id);
	TRACE_EVENT_P3("Status, Error, Click: %d, %d, %d", parameter->status_code, parameter->error_code,
			parameter->play_standard_click);
#endif

	wap_coder(WAP_CONTROL_IND_ID, (void*)parameter);
	
	return;
}


/*******************************************************************************


 $Function:    	W_WAP_MMI_NEW_KEY_IND
 
 $Description:	Inform MMI a new key is to be created.
 
 $Returns:		None.

 $Arguments:	parameter - data block
 
*******************************************************************************/

void W_WAP_MMI_NEW_KEY_IND(T_WAP_MMI_NEW_KEY_IND *parameter)
{
#ifdef TRACE_WAP_WAPMMI
	TRACE_EVENT("W_WAP_MMI_NEW_KEY_IND");
	TRACE_EVENT_P1("WAP object ID: %d",parameter->object_id);
	TRACE_EVENT_P2("Key ID, Key Type: %d, %d", parameter->keyId, parameter->key_type);
	WAP_trace_ushort_string(parameter->Label, parameter->label_length);
#endif	

	wap_coder(WAP_NEW_KEY_IND_ID, (void*)parameter);
	
	return;
}


/*******************************************************************************


 $Function:    	W_WAP_MMI_INPUT_DIALOG_REQ
 
 $Description:	Request MMI for an input dialog.
 
 $Returns:		None.

 $Arguments:	parameter - data block
 
*******************************************************************************/

void W_WAP_MMI_INPUT_DIALOG_REQ(T_WAP_MMI_INPUT_DIALOG_REQ *parameter)
{
#ifdef TRACE_WAP_WAPMMI
	TRACE_EVENT("W_WAP_MMI_INPUT_DIALOG_REQ");
	TRACE_EVENT_P1("WAP object ID: %d",parameter->object_id);
	TRACE_EVENT_P2("WAP dialog_id, is_password: %d, %d", parameter->dialog_id, parameter->is_password);
	TRACE_EVENT_P3("WAP empty_ok, visible, size: %d, %d, %d", parameter->empty_ok, 
		parameter->visible, parameter->size);
	WAP_trace_ushort_string(parameter->Title, parameter->title_length);
	WAP_trace_ushort_string(parameter->Input, parameter->input_length);
	WAP_trace_ushort_string(parameter->Format, parameter->format_length);
#endif	

	wap_coder(WAP_INPUT_DIALOG_REQ_ID,(void*)parameter);

	return;
}


/*******************************************************************************


 $Function:    	W_WAP_MMI_PASSWORD_DIALOG_REQ
 
 $Description:	Request MMI for a password dialog.
 
 $Returns:		None.

 $Arguments:	parameter - data block
 
*******************************************************************************/

void W_WAP_MMI_PASSWORD_DIALOG_REQ(T_WAP_MMI_PASSWORD_DIALOG_REQ *parameter)
{
#ifdef TRACE_WAP_WAPMMI
	TRACE_EVENT("W_WAP_MMI_PASSWORD_DIALOG_REQ");
	TRACE_EVENT_P1("WAP object ID: %d",parameter->object_id);
	TRACE_EVENT_P2("WAP dialog_id, dialog_type: %d, %d", parameter->dialog_id, parameter->dialog_type);
	WAP_trace_string(parameter->Realm, parameter->realm_length);
#endif	

	wap_coder(WAP_PASSWORD_DIALOG_REQ_ID,(void*)parameter);

	return;
}


/*******************************************************************************


 $Function:    	W_WAP_MMI_CONFIRM_DIALOG_REQ
 
 $Description:	Request MMI for a confirmation dialog.
 
 $Returns:		None.

 $Arguments:	parameter - data block
 
*******************************************************************************/

void W_WAP_MMI_CONFIRM_DIALOG_REQ(T_WAP_MMI_CONFIRM_DIALOG_REQ *parameter)
{
#ifdef TRACE_WAP_WAPMMI
	TRACE_EVENT("W_WAP_MMI_CONFIRM_DIALOG_REQ");
	TRACE_EVENT_P1("WAP object ID: %d",parameter->object_id);
	TRACE_EVENT_P1("WAP dialog_id: %d", parameter->dialog_id);
	WAP_trace_ushort_string(parameter->Message, parameter->message_length);
	WAP_trace_ushort_string(parameter->Ok_key, parameter->ok_key_length);
	WAP_trace_ushort_string(parameter->Cancel_key, parameter->cancel_key_length);
#endif	

	wap_coder(WAP_CONFIRM_DIALOG_REQ_ID, (void*)parameter);
	
	return;
}


/*******************************************************************************


 $Function:    	W_WAP_MMI_INFO_DIALOG_REQ
 
 $Description:	Request MMI for an info/alert dialog.
 
 $Returns:		None.

 $Arguments:	parameter - data block
 
*******************************************************************************/

void W_WAP_MMI_INFO_DIALOG_REQ(T_WAP_MMI_INFO_DIALOG_REQ *parameter)
{
#ifdef TRACE_WAP_WAPMMI
	TRACE_EVENT("W_WAP_MMI_INFO_DIALOG_REQ");
	TRACE_EVENT_P1("WAP object ID: %d",parameter->object_id);
	TRACE_EVENT_P1("WAP dialog_id: %d", parameter->dialog_id);
	WAP_trace_ushort_string(parameter->Message, parameter->message_length);
#endif	

	wap_coder(WAP_INFO_DIALOG_REQ_ID, (void*)parameter);

	return;
}
	
/*******************************************************************************


 $Function:    	W_WAP_MMI_NEW_CARD_IND
 
 $Description:	Inform MMI that a new card is to be created.
 
 $Returns:		None.

 $Arguments:	parameter - data block
 
*******************************************************************************/

void W_WAP_MMI_NEW_CARD_IND(T_WAP_MMI_NEW_CARD_IND *parameter)
{	
#ifdef TRACE_WAP_WAPMMI
	TRACE_EVENT("W_WAP_MMI_NEW_CARD_IND");
	TRACE_EVENT_P1("WAP object ID: %d",parameter->object_id);
	TRACE_EVENT_P1("WAP is_bookmarkable: %d", parameter->is_bookmarkable);
	WAP_trace_ushort_string(parameter->Title, parameter->title_length);
	WAP_trace_string(parameter->Url, parameter->url_length);
#endif
	
	wap_coder(WAP_NEW_CARD_IND_ID, (void*)parameter);

	return;
}


/*******************************************************************************


 $Function:    	W_WAP_MMI_DRAW_CARD_IND

 $Description:	Tells MMI that card is to be drawn.
 
 $Returns:		None.

 $Arguments:	parameter - data block
 
*******************************************************************************/

void W_WAP_MMI_DRAW_CARD_IND(T_WAP_MMI_DRAW_CARD_IND *parameter)
{
#ifdef TRACE_WAP_WAPMMI
	TRACE_EVENT("W_WAP_MMI_DRAW_CARD_IND");
	TRACE_EVENT_P1("WAP object ID: %d",parameter->object_id);
#endif
	wap_coder(WAP_DRAW_CARD_IND_ID, (void*)parameter);

	return;
}


/*******************************************************************************


 $Function:    	W_WAP_MMI_CLEAR_CARD_IND

 $Description:	Tells MMI to clear the card display.
 
 $Returns:		None.

 $Arguments:	parameter - data block
 
*******************************************************************************/

void W_WAP_MMI_CLEAR_CARD_IND(T_WAP_MMI_CLEAR_CARD_IND *parameter)
{
#ifdef TRACE_WAP_WAPMMI
	TRACE_EVENT("W_WAP_MMI_CLEAR_CARD_IND");
	TRACE_EVENT_P1("WAP object ID: %d",parameter->object_id);
#endif	

	wap_coder(WAP_CLEAR_CARD_IND_ID, (void*)parameter);

	return;
}


/*******************************************************************************


 $Function:    	W_WAP_MMI_SEND_TEXT_IND

 $Description:	Sends text to MMI.
 
 $Returns:		None.

 $Arguments:	parameter - data block
 
*******************************************************************************/

void W_WAP_MMI_SEND_TEXT_IND(T_WAP_MMI_SEND_TEXT_IND *parameter)
{
#ifdef TRACE_WAP_WAPMMI
	TRACE_EVENT("W_WAP_MMI_SEND_TEXT_IND");
	TRACE_EVENT_P1("WAP object ID: %d",parameter->object_id);
	TRACE_EVENT_P2("WAP type, invert: %d, %d", parameter->type, parameter->invert);
	TRACE_EVENT_P2("WAP pX, pY: %d, %d", parameter->pX, parameter->pY);
	TRACE_EVENT_P2("WAP pWidth, pHeight: %d, %d", parameter->pWidth, parameter->pHeight);
	TRACE_EVENT_P2("WAP format, selected : %d, %d", parameter->format, \
		parameter->selected);
	TRACE_EVENT_P2("WAP is_link, deltaY : %d, %d", (int)parameter->is_link, parameter->deltaY);
	WAP_trace_ushort_string(parameter->Text, parameter->text_length);
	WAP_trace_ushort_string(parameter->Formatstring, parameter->formatstring_length);
#endif

	wap_coder(WAP_SEND_TEXT_IND_ID, (void*)parameter);

	return;
}


/*******************************************************************************

 $Function:    	W_WAP_MMI_SEND_IMAGE_IND
 
 $Description:	Sends image to MMI.
 
 $Returns:		None.

 $Arguments:	parameter - data block
 
*******************************************************************************/

void W_WAP_MMI_SEND_IMAGE_IND(T_WAP_MMI_SEND_IMAGE_IND *parameter)
{
#ifdef TRACE_WAP_WAPMMI
	TRACE_EVENT("W_WAP_MMI_SEND_IMAGE_IND");
	TRACE_EVENT_P1("WAP object ID: %d",parameter->object_id);
	TRACE_EVENT_P2("Image length: %d, Invert: %d",parameter->image_length, parameter->invert);
	TRACE_EVENT_P4("pX: %d, pY: %d, pWidth: %d, pHeight: %d",parameter->pX, parameter->pY, parameter->pWidth, parameter->pHeight);
#endif

	wap_coder(WAP_SEND_IMAGE_IND_ID, (void*)parameter);

	return;
}


/*******************************************************************************

 $Function:    	W_WAP_MMI_SEND_TABLE_IND

 $Description:	Sends table to MMI.
 
 $Returns:		None.

 $Arguments:	parameter - data block
 
*******************************************************************************/

void W_WAP_MMI_SEND_TABLE_IND(T_WAP_MMI_SEND_TABLE_IND *parameter)
{
#ifdef TRACE_WAP_WAPMMI
	TRACE_EVENT("W_WAP_MMI_SEND_TABLE_IND");
	TRACE_EVENT_P1("WAP object ID: %d",parameter->object_id);
	TRACE_EVENT_P2("Columns: %d, Rows: %d",parameter->cols_length, parameter->rows_length);
	TRACE_EVENT_P4("pX: %d, pY: %d, pWidth: %d, pHeight: %d",parameter->pX, parameter->pY, parameter->pWidth, parameter->pHeight);
	WAP_trace_ushort_string(parameter->Title, parameter->title_length);
#endif

	wap_coder(WAP_SEND_TABLE_IND_ID, (void*)parameter);

	return;
}


/*******************************************************************************

 $Function:    	W_WAP_MMI_CARD_DRAW_COMPLETED_IND
 
 $Description:	Tells MMI that card drawing has completed.
 
 $Returns:		None.

 $Arguments:	parameter - data block
 
*******************************************************************************/

void W_WAP_MMI_CARD_DRAW_COMPLETED_IND(T_WAP_MMI_CARD_DRAW_COMPLETED_IND *parameter)
{
#ifdef TRACE_WAP_WAPMMI
	TRACE_EVENT("W_WAP_MMI_CARD_DRAW_COMPLETED_IND");
	TRACE_EVENT_P1("WAP object ID: %d",parameter->object_id);
#endif

	wap_coder(WAP_CARD_DRAW_COMPLETED_IND_ID, (void*)parameter);

	return;
}


/*******************************************************************************

 $Function:    	W_WAP_MMI_CONNECT_REQ
 
 $Description:	Reqests MMI to connect.
 
 $Returns:		None.

 $Arguments:	parameter - data block
 
*******************************************************************************/

void W_WAP_MMI_CONNECT_REQ(T_WAP_MMI_CONNECT_REQ *parameter)
{
#ifdef TRACE_WAP_WAPMMI
	TRACE_EVENT("W_WAP_MMI_CONNECT_REQ");
	TRACE_EVENT_P1("WAP object ID: %d",parameter->object_id);
	TRACE_EVENT_P1("WAP channel ID: %d",parameter->channel);
#endif

	/* Initialise udp before starting connection */
	
    udp_init();
    
	wap_coder(WAP_CONNECT_REQ_ID,(void*)parameter);

	return;
}


/*******************************************************************************

 $Function:    	W_WAP_MMI_DISCONNECTED_IND
 
 $Description:	Tell MMI that disconnection has occurred.
 
 $Returns:		None.

 $Arguments:	parameter - data block
 
*******************************************************************************/

void W_WAP_MMI_DISCONNECTED_IND(T_WAP_MMI_DISCONNECTED_IND *parameter)
{
#ifdef TRACE_WAP_WAPMMI
	TRACE_EVENT("W_WAP_MMI_DISCONNECTED_IND");
	TRACE_EVENT_P1("WAP object ID: %d",parameter->object_id);
	TRACE_EVENT_P1("WAP channel ID: %d",parameter->channel);
#endif

	wap_coder(WAP_DISCONNECTED_IND_ID,(void*)parameter);

	return;
}

/*******************************************************************************


 $Function:    	W_WAP_MMI_READY_CNF
 
 $Description:	Confirm that WAP operation has completed.
 
 $Returns:		None.

 $Arguments:	parameter - data block
 
*******************************************************************************/

void W_WAP_MMI_READY_CNF(T_WAP_MMI_READY_CNF *parameter)
{
#ifdef TRACE_WAP_WAPMMI
	TRACE_EVENT("W_WAP_MMI_READY_CNF");
#endif

	wap_coder(WAP_READY_CNF_ID,(void*)parameter);

	return;
}

#ifdef  MMS_WAP_ENABLED
/**************talcon add for mms**********/
void W_MMI_WAP_MMSSTART_IND(T_MMI_WAP_MMSSTART_IND *parameter)
{
	T_WAP_MMI_READY_CNF parameter2;
	
#ifdef TRACE_WAP_WAPMMI
	TRACE_EVENT("W_MMI_WAP_MMSSTART_IND");
#endif

	/* If startup fails, do not continue */
	
	startup_failed = FALSE;
	//GLEc_start();
	CLNTc_start();
	if (!startup_failed)
	{ 
		WAP_init_cache();

		MEMc_initCache(WAP_cache_size);

		wap_lib_run_aus_callback();
		 
		parameter2.event_id = WAP_MMSSTART_IND_ID;
		W_WAP_MMI_READY_CNF(&parameter2);	
	}
	
	return;
}

#if 0
void W_MMI_WAP_MMSNEW_VIEW_IND(T_MMI_WAP_MMSNEW_VIEW_IND *parameter)
{
	T_WAP_MMI_READY_CNF parameter2;
	
#ifdef TRACE_WAP_WAPMMI
	TRACE_EVENT("W_MMI_WAP_MMSNEW_VIEW_IND");
#endif

	/* Ensure new view is not set up if startup fails */
	
	if (!startup_failed)
	{
		/*start a useragent*/
		MMIc_startUserAgent(parameter->object_id, parameter->uaMode);
		wap_lib_run_aus_callback();

		parameter2.event_id = WAP_MMSNEW_VIEW_IND_ID;
		W_WAP_MMI_READY_CNF(&parameter2);
	}
	return;
}

void W_MMI_WAP_MMSCLOSE_VIEW_IND(T_MMI_WAP_MMSCLOSE_VIEW_IND *parameter)
{
	T_WAP_MMI_READY_CNF parameter2;
	
#ifdef TRACE_WAP_WAPMMI
	TRACE_EVENT("W_MMI_WAP_MMSCLOSE_VIEW_IND");
	TRACE_EVENT_P1("WAP object ID: %d",parameter->object_id);
#endif

	/* Don't close view if startup failed */
	
	if (!startup_failed)
	{
		MMIc_terminateUserAgent(parameter->object_id);

		wap_lib_run_aus_callback();
	}
	
	parameter2.event_id = WAP_CLOSE_VIEW_IND_ID;
	W_WAP_MMI_READY_CNF(&parameter2);	
	
	return;
}
#endif

void W_MMI_WAP_MMSTERMINATE_IND(T_MMI_WAP_MMSTERMINATE_IND *parameter)
{
	T_WAP_MMI_READY_CNF parameter2;

#ifdef TRACE_WAP_WAPMMI
	TRACE_EVENT("W_MMI_WAP_MMSTERMINATE_IND");
#endif /*TRACE_WAP_WAPMMI */

	/* Don't terminate if startup failed */

	if (!startup_failed)
	{
		freeDisplayData();

		GLEc_terminate();
		CLNTc_terminate();

		wap_lib_run_aus_callback();
#ifdef TRACE_WAP_WAPMMI
		TRACE_EVENT("W_MMI_WAP_TERMINATE_IND complete");
#endif
	}
	else
	{
		TRACE_EVENT("Startup failed - not terminating.");
	}

	parameter2.event_id = WAP_MMSTERMINATE_IND_ID;
	W_WAP_MMI_READY_CNF(&parameter2);		
	
	return;
}


void W_MMI_WAP_POSTCONTENT_IND(T_MMI_WAP_POSTCONTENT_IND *parameter)
{	
 
#ifdef TRACE_WAP_WAPMMI
	WAP_TRACE_EVENT(("W_MMI_WAP_POSTCONTENT_IND-%d,%d",parameter->moreData,parameter->data_length));
#endif
                       
	CLNTc_postContent(parameter->addr, //url
			parameter->urlID,			//urlId
			parameter->reload,						//reload
			parameter->acceptHeader, 
			parameter->data,
			parameter->data_length,
			parameter->moreData,
			parameter->contentType, 
			parameter->sendMode,
			parameter->contentDisp,
			parameter->totalSize);

	 wap_lib_run_aus_callback();

	 return;
}

void W_WAP_MMI_CONTENT_REQ(T_WAP_MMI_CONTENT_REQ *parameter)
{
#ifdef TRACE_WAP_WAPMMI
	TRACE_EVENT("W_WAP_MMI_POSTCONTENT_REQ");
#endif

    
	wap_coder(WAP_CONTENT_REQ_ID,(void*)parameter);

	return;
}

//EXPORT_C VOID    CLNTc_postMoreContent (UINT8 urlID, const CHAR *data, UINT16 dataLen, BOOL moreData)
void W_MMI_WAP_POSTMORECONTENT_IND(T_MMI_WAP_POSTMORECONTENT_IND *parameter)
{	

#ifdef TRACE_WAP_WAPMMI
	WAP_TRACE_EVENT(("W_MMI_WAP_POSTMORECONTENT_IND-%d,%d",parameter->moreData, parameter->data_length));
#endif
                       
	CLNTc_postMoreContent(parameter->urlID, //url
			parameter->data,
			parameter->data_length,
			parameter->moreData);

	 wap_lib_run_aus_callback();

	 return;
}

void W_WAP_MMI_ACKPOSTCONTENT_REQ(T_WAP_MMI_ACKPOSTCONTENT_REQ *parameter)
{
#ifdef TRACE_WAP_WAPMMI
	TRACE_EVENT("W_WAP_MMI_ACKPOSTCONTENT_REQ");
#endif

    
	wap_coder(WAP_ACKPOSTCONTENT_REQ_ID,(void*)parameter);

	return;
}

/*****get content************/
void W_MMI_WAP_GETCONTENT_IND(T_MMI_WAP_GETCONTENT_IND *parameter)
{	
 
#ifdef TRACE_WAP_WAPMMI
	WAP_TRACE_EVENT(("W_MMI_WAP_GETCONTENT_IND:%d, %s,%s",parameter->urlID,parameter->addr,parameter->acceptHeader));
#endif
                       
	CLNTc_getContent(parameter->addr, //url
			parameter->urlID,			//urlId
			parameter->reload,						//reload
			parameter->acceptHeader);

	 wap_lib_run_aus_callback();

	 return;
}
void W_MMI_WAP_ACKCONTENT_IND(T_MMI_WAP_ACKCONTENT_IND *parameter)
{	
 
#ifdef TRACE_WAP_WAPMMI
	WAP_TRACE_EVENT(("W_MMI_WAP_ACKCONTENT_IND"));
#endif
                       
	CLNTc_acknowledgeContent(parameter->urlID);

	 wap_lib_run_aus_callback();

	 return;
}

void W_MMI_WAP_CANCELCONTENT_IND(T_MMI_WAP_CANCELCONTENT_IND *parameter)
{	
 
#ifdef TRACE_WAP_WAPMMI
	WAP_TRACE_EVENT(("W_MMI_WAP_CANCELCONTENT_IND: %d",parameter->urlID));
#endif
                       
	CLNTc_cancelContent(parameter->urlID);

	 wap_lib_run_aus_callback();

	 return;
}
#endif



