/*******************************************************************************

					CONDAT (UK)

********************************************************************************                                                                              

 This software product is the property of Condat (UK) Ltd and may not be
 disclosed to any third party without the express permission of the owner.                                 
                                                                              
********************************************************************************

 $Project name:	                                                      
 $Project code:	                                                           
 $Module:		
 $File:		    ATBWapAUI.c
 $Revision:		                                                      
                                                                              
 $Author:		Condat(UK)                                                         
 $Date:		                                                          
                                                                               
********************************************************************************
                                                                              
 Description:
 	Provides most of the functionality of the ATB, as well as an interface with the AUI.
 	
                        
********************************************************************************

 $History: ATBWapAUI.c

	
	   
 $End

*******************************************************************************/
#define MFW

#ifdef MFW
#define ENTITY_MFW
#else
#ifdef SMI
#define ENTITY_SMI
#else
#define ENTITY_ACI
#endif
#endif



#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#ifdef NEW_FRAME
#include "typedefs.h"
#else  /* NEW_FRAME */
#include "stddefs.h"
#endif /* NEW_FRAME */

#include "Ffs.h"
#include "MmiWapFfs.h"
#include "gledef.h"
#include "capiclnt.h"
#include "aapiclnt.h"
#include "wap_types.h"
#include "ATBWapACI.h"
#include "ATBData.h"
#include "ATBWapAUI.h"
#include "AUIWap.h"
#include "font_bitmaps.h"

#include "cus_aci.h"
#include "p_sim.h"
#include "pcm.h"
#include "dspl.h"
#include "mfw_kbd.h"//zhaowm 2003.1.9
#include "MmiMain.h"  

#define TRACE_ATBWAPAUI
#define WAP_TRACE 1
#include "mmiutilapi.h"
#define TRACE_EVENT(f) WAP_TRACE_EVENT(("bmi--AtbWapAui.c<<<talcon,%s",f));
#define TRACE_FUNCTION(f) TRACE_EVENT(f)



void long_IP_to_char_IP(ULONG IPs,char* IPd);


#define DEFAULT_PROFILE 0							// The default starting profile

static T_WAP_VIEW		*View_header = 0;				// View header points to first element in the view chain
static T_WAP_VIEW		*Current_view = 0;				// Points to the current view
#ifdef MMS_WAP_ENABLED
static T_WAP_VIEW		*Wap_view = 0;				
static T_WAP_VIEW		*MMS_view = 0;				
#endif

static char	debug[50];

// LOCAL FUNCTION PROTOTYPES

void ATB_wap_buffer_image_scale(T_WAP_MMI_SEND_IMAGE_IND *image);



#if defined(TRACE_ATBWAPAUI) || defined(TRACE_ATBWAPACI)

/* Functions to display strings, for tracing purposes.
 * SPR#1437 - SH - Fixed these so they won't overwrite array bounds */
	
void ATB_trace_string(char *String, USHORT length)
{
	UBYTE stringIndex;
	UBYTE trace_string[41];

	for (stringIndex=0; (stringIndex<40 && stringIndex<length); stringIndex++)
		trace_string[stringIndex] = String[stringIndex];

	trace_string[stringIndex] = 0;

	TRACE_EVENT_P2("%s (%d)", trace_string, length);

	return;
}

void ATB_trace_ushort_string(USHORT *String, USHORT length)
{
	USHORT stringIndex;
	UBYTE trace_string[41];
	
	for (stringIndex=0; (stringIndex<40 && stringIndex<length); stringIndex++)
		trace_string[stringIndex] = (char) String[stringIndex];

	trace_string[stringIndex] = 0;

	TRACE_EVENT_P2("%s (%d)", trace_string, length);

	return;
}

#endif


/*******************************************************************************

 $Function:    	ATB_wap_start

 $Description:	Starts the WAP application and initialises the view chain
 
 $Returns:		

 $Arguments:	parameter		- dummy empty parameter
 
*******************************************************************************/

void ATB_wap_start(T_MMI_WAP_START_IND *parameter)
{
#ifdef TRACE_ATBWAPAUI
	TRACE_FUNCTION("ATB_wap_start");
#endif

	M_MMI_WAP_START_IND(parameter);
	// Initialise the pointer to the view chain
#ifdef MMS_WAP_ENABLED
	Wap_view = NULL;
#else
	View_header = NULL;
	Current_view = NULL;
#endif
	
	return;
}


/*******************************************************************************

 $Function:    	ATB_wap_new_view

 $Description:	Starts a new view, defining the graphical size of the screen to
 				be handled.
 
 $Returns:		Pointer to the data structure of the new view

 $Arguments:	parameter		- width and height of the WAP view
 				UIdata			- Optional generic pointer, for use by the AUI.  This
 								can point to window information, etc, but is not used
 								by ATB.
 
*******************************************************************************/

T_WAP_VIEW* ATB_wap_new_view(T_MMI_WAP_NEW_VIEW_IND *parameter, void* UIdata)
{
	T_WAP_VIEW	*View	= (T_WAP_VIEW *) AUI_wap_memory_alloc (sizeof(T_WAP_VIEW));	// Create the view
	T_WAP_VIEW *ViewIndex;

#ifdef TRACE_ATBWAPAUI
	TRACE_FUNCTION("ATB_wap_new_view");
#endif

	//WapData = (FlashDataWap *) AUI_wap_memory_alloc(sizeof(FlashDataWap));		//zhaowm 2002.12.23// Allocate memory for flash data

	View->URL			= (char *)AUI_wap_memory_alloc ((URL_MAX_LEN+1)*sizeof(char)); // space for URL text
	View->Title			= (char *)AUI_wap_memory_alloc ((CARD_TITLE_MAX_LEN+1)*sizeof(char)); // Title of card
	View->cardWidth		= WAP_SCREEN_WIDTH;
	View->cardHeight	= WAP_SCREEN_HEIGHT;
	View->ProfilesList	= ATB_wap_entry_list_create(WAP_PROFILES_LIST, MAX_PROFILES, PROFILENAME_MAX_LEN); // ...profiles names list
	View->object_id		= parameter->object_id;
	View->channel		= WAP_CHANNEL_ID;
	View->UIdata		= UIdata;
	View->ElementHeader	= NULL;													// Element chain is as yet empty
	View->NextView		= NULL;
	View->browser_status = ATB_WAP_NO_STATUS;	/* SPR#1547 - SH - Replaced 'status' with 'browser_status' */
	View->CustSoftKeys	= FALSE;												// Whether we have customised soft keys
	View->Bookmarks		= ATB_wap_entry_list_create(WAP_BOOKMARKS_LIST, MAX_BOOKMARKS, CARD_TITLE_MAX_LEN);		    // Create bookmarks list
	View->BookmarksURL	= ATB_wap_entry_list_create(WAP_URL_LIST, MAX_BOOKMARKS, URL_MAX_LEN);	
	View->History		= ATB_wap_entry_list_create(WAP_HISTORY_LIST, MAX_HISTORY, CARD_TITLE_MAX_LEN+NUMBER_PADDING); // ...history list...
	View->HistoryURL	= ATB_wap_entry_list_create(WAP_URL_LIST, MAX_HISTORY, URL_MAX_LEN);
	
	ATB_wap_profile_names_read(View);					 						// Read in the names of profiles
	ATB_wap_profile_read(View, View->ProfileId);								// Read in the current profile
	
#ifndef MMS_WAP_ENABLED
	if (View_header == NULL)
		View_header = View;														// This is the first entry
	else
	{
		ViewIndex = View_header;
		while (ViewIndex->NextView != NULL)										// If not the last entry,
			ViewIndex = ViewIndex->NextView;									// find the next entry!
		ViewIndex->NextView = View;												// New entry is last entry in chain
	}

	Current_view = View;
#else
	ATB_wap_wiew_add(View);
#endif
	
	M_MMI_WAP_NEW_VIEW_IND(parameter);

	return View;
}


/*******************************************************************************

 $Function:    	ATB_wap_get_view

 $Description:	Returns a pointer to a view, given its object_id
 
 $Returns:		Pointer to the data structure of the specified view, or NULL if not found

 $Arguments:	object_id  - the id of the view
 
*******************************************************************************/

T_WAP_VIEW* ATB_wap_get_view(UBYTE object_id)
{
	T_WAP_VIEW *View = View_header;
	UBYTE Index;

	if (!View_header)
	{
		TRACE_EVENT("blank view header");
		return NULL;
	}

	/* SPR#1497 - SH - If object_id is 0, any view will do */

	if (object_id==0)
		return View;

#ifdef TRACE_ATBWAPAUI
	TRACE_FUNCTION("ATB_wap_get_view");
#endif

	for (Index = 0; View!=NULL && View->object_id!=object_id; Index++)			// Searches for object_id
	{
		View = View->NextView;
	}

	return View;
}


/*******************************************************************************

 $Function:    	ATB_wap_destroy_view

 $Description:	Destroys a view
 
 $Returns:		WAP_OK if successful, WAP_FAIL if otherwise

 $Arguments:	View - pointer to the view to be destroyed
 				
 
*******************************************************************************/

T_WAP_RES ATB_wap_destroy_view(T_WAP_VIEW *View)
{
	T_WAP_VIEW	*ViewIndex = View_header;
	T_WAP_VIEW *Last_view = NULL;

#ifdef TRACE_ATBWAPAUI
	TRACE_FUNCTION("ATB_wap_destroy_view");
#endif

	while (ViewIndex!=View && ViewIndex!=NULL)									// Find the view in the chain
	{
		Last_view = ViewIndex;
		ViewIndex = ViewIndex->NextView;
	}

	if (ViewIndex == NULL)
		return WAP_FAIL;														// Couldn't find view in chain

	if (Last_view == NULL)														// Delete first entry in chain
		View_header = View_header->NextView;
	else
		Last_view->NextView = View->NextView;									// Break link in chain

	ATB_wap_entry_list_destroy(View->ProfilesList);								// Destroy profiles names list
	ATB_wap_entry_list_destroy(View->Bookmarks);								// Destroy bookmarks
	ATB_wap_entry_list_destroy(View->BookmarksURL);
	ATB_wap_entry_list_destroy(View->History);									// Destroy history list
	ATB_wap_entry_list_destroy(View->HistoryURL);

	AUI_wap_memory_free ((UBYTE *)View->URL, (URL_MAX_LEN+1)*sizeof(char));		// Delete URL string
	AUI_wap_memory_free ((UBYTE *)View->Title, (CARD_TITLE_MAX_LEN+1)*sizeof(char)); // Delete card title
	ATB_wap_buffer_clear(View);
	AUI_wap_memory_free ((UBYTE *)View, sizeof(T_WAP_VIEW));					// Delete View

	return WAP_OK;
}


/*******************************************************************************

 $Function:    	ATB_wap_destroy

 $Description:	Destroys the wap application, including all views (calls ATB_wap_terminate)
 
 $Returns:		WAP_OK if successful, WAP_FAIL if otherwise

 $Arguments:	
 				
 
*******************************************************************************/

T_WAP_RES ATB_wap_destroy()
{
	T_MMI_WAP_CLOSE_VIEW_IND	parameter1;
	static T_MMI_WAP_TERMINATE_IND		parameter2;	/* SPR#1569 - SH - Variable is now static */
	T_WAP_VIEW					*ViewIndex;
	T_WAP_VIEW					*NextView;

#ifdef TRACE_ATBWAPAUI
	TRACE_FUNCTION("ATB_wap_destroy");
#endif

	/* SPR#1569 - SH - Modified.  Now destroys an open view each time it is called.
	 * if there are no open views, the WAP broswer is terminated. */
	/* Get the access type (CSD/GPRS) from the root view */

/*talcon remove MMS,it is only for wap*/
#ifndef MMS_WAP_ENABLED___
	if (View_header)
	{
		parameter2.access_type = View_header->Profile->AccessType;
	}
#else
	if(Wap_view)
	{
		parameter2.access_type = Wap_view->Profile->AccessType;
	}
#endif
	
	/* Destroy the view chain.
 	 * If views still exist, destroy the bottom one */
#ifndef MMS_WAP_ENABLED___
	if (View_header!=NULL)														// Until end of chain
#else
	if (Wap_view!=NULL)		
#endif
	{
#ifndef MMS_WAP_ENABLED____
		ViewIndex = View_header;													// Store the current entry pointer
		View_header = ViewIndex->NextView;											// Find the next entry		
#else
		ViewIndex = Wap_view;
		ATB_wap_view_delete(Wap_view);
#endif

		parameter1.object_id = ViewIndex->object_id;
		
		ATB_wap_entry_list_destroy(ViewIndex->ProfilesList);					// Destroy profiles names list
		ATB_wap_entry_list_destroy(ViewIndex->Bookmarks);						// Destroy bookmarks
		ATB_wap_entry_list_destroy(ViewIndex->BookmarksURL);
		ATB_wap_entry_list_destroy(ViewIndex->History);							// Destroy history list
		ATB_wap_entry_list_destroy(ViewIndex->HistoryURL);
		AUI_wap_memory_free ((UBYTE *)ViewIndex->URL, (URL_MAX_LEN+1)*sizeof(char)); // Delete URL string
		AUI_wap_memory_free ((UBYTE *)ViewIndex->Title, (CARD_TITLE_MAX_LEN+1)*sizeof(char));   // Delete card title
		ATB_wap_buffer_clear(ViewIndex);										// Clear the element buffer
		ViewIndex->ElementHeader = ViewIndex->NewElementHeader;					// Just in case a new card is being created
		ATB_wap_buffer_clear(ViewIndex);										// Clear this too!
		AUI_wap_memory_free((UBYTE *)ViewIndex, sizeof(T_WAP_VIEW));			// Destroy the current entry
		M_MMI_WAP_CLOSE_VIEW_IND(&parameter1);
		return WAP_OK;
	}

	else
	{
		// Destroy the profiles information
	//	if (WapData)
	//	{
	//		AUI_wap_memory_free ((UBYTE *)WapData, sizeof(FlashDataWap));
	//		WapData = NULL;
	//	}
#ifndef MMS_WAP_ENABLED___
		View_header = NULL;	
		Current_view = NULL;

		return ATB_wap_terminate(&parameter2);												// Close WAP application
#else
		Current_view = MMS_view;
		if(MMS_view && MMS_view->Profile->AccessType == parameter2.access_type)
		{
			return WAP_FAIL;
		}
		else
		{
			View_header = NULL;	
			Current_view = NULL;
			return ATB_wap_terminate(&parameter2);
		}
#endif
	}
}


/*******************************************************************************

 $Function:    	ATB_wap_UIdata

 $Description:	Returns the UI pointer, for use by AUI
 
 $Returns:		The void pointer

 $Arguments:	
 				
 
*******************************************************************************/

void * ATB_wap_UIdata()
{
#ifndef MMS_WAP_ENABLED
	if (View_header)
		return View_header->UIdata;
#else
	if(Current_view)
		return Current_view->UIdata;
#endif
	else
		return NULL;
}


/*******************************************************************************

 $Function:    	ATB_wap_terminate

 $Description:	Closes the WAP application and frees all the memory allocated
 
 $Returns:		WAP_OK if successful, WAP_FAIL if otherwise

 $Arguments:	
 
*******************************************************************************/

T_WAP_RES ATB_wap_terminate(T_MMI_WAP_TERMINATE_IND *parameter)
{
#ifdef TRACE_ATBWAPAUI
	TRACE_FUNCTION("ATB_wap_terminate");
#endif

	disconnect_wap_call(parameter->access_type); // Patch call to ACI to disconnect the call. It has to be revised.

	M_MMI_WAP_TERMINATE_IND(parameter);
	
	return WAP_OK;
}


/*******************************************************************************

 $Function:    	ATB_wap_configure

 $Description:	Sends configuration information to the browser, the more important being:
 				- ConfigHISTORY_SIZE, the number of URLs stored in the history
 				- ConfigCACHE_AGE, period for the cache to store a card
 				- ConfigDISPLAY_IMAGES, images to be displayed or not
 
 $Returns:		WAP_OK if successful, WAP_FAIL if otherwise

 $Arguments:	parameter		- Properties of configuration

 
*******************************************************************************/

T_WAP_RES ATB_wap_configure(T_MMI_WAP_CONFIGURE_IND *parameter)
{
#ifdef TRACE_ATBWAPAUI
	TRACE_FUNCTION("ATB_wap_configure");
	TRACE_EVENT_P1("MMI object ID: %d",parameter->object_id);
	TRACE_EVENT_P3( "Type, Param, Length: %d, %d, %d", parameter->type, parameter->param, parameter->length);
#endif

	M_MMI_WAP_CONFIGURE_IND(parameter);

	return WAP_OK;
}


/*******************************************************************************

 $Function:    	ATB_wap_connection_configure

 $Description:	Sends network configuration information to the browser, such as the
 				bearer, IP addresses, passwords and ids, etc.
 
 $Returns:		WAP_OK if successful, WAP_FAIL if otherwise

 $Arguments:	parameter		- Properties of configuration

 
*******************************************************************************/

T_WAP_RES ATB_wap_connection_configure(T_MMI_WAP_CONNECTION_CONFIGURE_IND *parameter)
{
#ifdef TRACE_ATBWAPAUI
	TRACE_FUNCTION("ATB_wap_connection_configure");
#endif

	M_MMI_WAP_CONNECTION_CONFIGURE_IND(parameter);

	return WAP_OK;
}


/*******************************************************************************

 $Function:    	ATB_wap_download_url

 $Description:	Requests the WAP browser to download a particular URL
 
 $Returns:		WAP_OK if successful, WAP_FAIL if otherwise

 $Arguments:	parameter		- Properties of the URL download
 
*******************************************************************************/

T_WAP_RES ATB_wap_download_url(T_MMI_WAP_DOWNLOAD_URL_IND *parameter)
{
#ifdef TRACE_ATBWAPAUI
	TRACE_FUNCTION("ATB_wap_download_url");
#endif

	M_MMI_WAP_DOWNLOAD_URL_IND(parameter);
	
	return WAP_OK;
}


/*******************************************************************************

 $Function:    	ATB_wap_stop_download

 $Description:	Requests the WAP browser to stop downloading a particular URL
 
 $Returns:		WAP_OK if successful, WAP_FAIL if otherwise

 $Arguments:	View - the current view
 
*******************************************************************************/

T_WAP_RES ATB_wap_stop_download(T_WAP_VIEW *View)
{
	T_MMI_WAP_BROWSE_CONTROL_IND parameter;

#ifdef TRACE_ATBWAPAUI
	TRACE_FUNCTION("ATB_wap_stop_download");
#endif

	parameter.object_id = View->object_id;
	parameter.browse_command = WAP_DOWNLOAD_STOP;
	M_MMI_WAP_BROWSE_CONTROL_IND(&parameter);

	ATB_wap_status_change(View,ATB_WAP_NO_STATUS);
	return WAP_OK;
}

/*******************************************************************************

 $Function:    	ATB_wap_card_refresh

 $Description:	Requests the WAP browser to refresh the current card from the network
 
 $Returns:		WAP_OK if successful, WAP_FAIL if otherwise

 $Arguments:	View - the current view
 
*******************************************************************************/

T_WAP_RES ATB_wap_card_refresh(T_WAP_VIEW *View)
{
	T_MMI_WAP_BROWSE_CONTROL_IND parameter;

#ifdef TRACE_ATBWAPAUI
	TRACE_FUNCTION("ATB_wap_card_refresh");
#endif

	parameter.object_id = View->object_id;
	parameter.browse_command = WAP_CARD_REFRESH;
	M_MMI_WAP_BROWSE_CONTROL_IND(&parameter);
	
	return WAP_OK;
}


/*******************************************************************************

 $Function:    	ATB_wap_card_display

 $Description:	Requests the WAP browser to redisplay a card held in the cache
 
 $Returns:		WAP_OK if successful, WAP_FAIL if otherwise

 $Arguments:	View - the current view
 
*******************************************************************************/

T_WAP_RES ATB_wap_card_display(T_WAP_VIEW *View)
{
	T_MMI_WAP_BROWSE_CONTROL_IND parameter;

#ifdef TRACE_ATBWAPAUI
	TRACE_FUNCTION("ATB_wap_card_display");
#endif

	parameter.object_id = View->object_id;
	parameter.browse_command = WAP_CARD_DISPLAY;
	M_MMI_WAP_BROWSE_CONTROL_IND(&parameter);
	
	return WAP_OK;
}


/*******************************************************************************

 $Function:    	ATB_wap_card_go_back

 $Description:	Requests the WAP browser to go to the previous card
 
 $Returns:		WAP_OK if successful, WAP_FAIL if otherwise

 $Arguments:	View - the current view
 
*******************************************************************************/

T_WAP_RES ATB_wap_card_go_back(T_WAP_VIEW *View)
{
	T_MMI_WAP_BROWSE_CONTROL_IND parameter;

#ifdef TRACE_ATBWAPAUI
	TRACE_FUNCTION("ATB_wap_card_go_back");
#endif

	parameter.object_id = View->object_id;
	parameter.browse_command = WAP_CARD_GO_BACK;
	M_MMI_WAP_BROWSE_CONTROL_IND(&parameter);
	
	return WAP_OK;
}


/*******************************************************************************

 $Function:    	ATB_wap_card_key_event

 $Description:	Requests the WAP browser react to a key event, up/down etc.
 
 $Returns:		WAP_OK if successful, WAP_FAIL if otherwise

 $Arguments:	View - the current view
 				keyType - the type of key selected
 
*******************************************************************************/

T_WAP_RES ATB_wap_card_key_event(T_WAP_VIEW *View, WAP_CONTROL_TYPES keyType)
{
	T_MMI_WAP_BROWSE_CONTROL_IND	command;
	T_MMI_WAP_DRAW_CARD_REQ			current_card;

#ifdef TRACE_ATBWAPAUI
	TRACE_FUNCTION("ATB_wap_card_key_event");
#endif

	command.object_id = View->object_id;
	command.browse_command = keyType;
	M_MMI_WAP_BROWSE_CONTROL_IND(&command);

	if (keyType == WAP_KEY_SELECT)												// For select, need to redraw card
	{																		// in order to update changed elements
		current_card.object_id = View->object_id;
		M_MMI_WAP_DRAW_CARD_REQ(&current_card);
	}
	
	return WAP_OK;
}


/*******************************************************************************

 $Function:    	ATB_wap_key_selected

 $Description:	Send the id of the selected WAP key to the AUS browser.
 				
 $Returns:		WAP_OK if successful, WAP_FAIL if otherwise

 $Arguments:	View - the current view
 				keyId - Key identifier
 
*******************************************************************************/

T_WAP_RES ATB_wap_key_selected(T_WAP_VIEW *View, USHORT keyId)
{

	T_MMI_WAP_KEY_SELECTED_IND parameter;

#ifdef TRACE_ATBWAPAUI
	TRACE_FUNCTION("ATB_wap_key_selected");
#endif

	parameter.object_id = View->object_id;	
	parameter.keyId = keyId;	
	M_MMI_WAP_KEY_SELECTED_IND(&parameter);

	return WAP_OK;
}


/*******************************************************************************

 $Function:    	ATB_wap_card scroll

 $Description:	Scroll to a y position on the current card
 				
 $Returns:		WAP_OK if successful, WAP_FAIL if otherwise

 $Arguments:	View - the current view
 				scrollPos - Y position on card to which to scroll
 
*******************************************************************************/

T_WAP_RES ATB_wap_card_scroll(T_WAP_VIEW *View, SHORT scrollPos)
{
	T_MMI_WAP_BROWSE_CONTROL_IND	parameter;

	// Make sure the scroll destination is on the card

	if (scrollPos > (View->cardHeight-WAP_SCREEN_HEIGHT))
		scrollPos = View->cardHeight-WAP_SCREEN_HEIGHT;

	if (scrollPos<0)
		scrollPos = 0;
	
	// Send the event
	
	parameter.object_id = View->object_id;
	parameter.browse_command = WAP_KEY_SCROLL;
	parameter.browse_parameter = (USHORT)scrollPos;

	M_MMI_WAP_BROWSE_CONTROL_IND(&parameter);

	return WAP_OK;
}

/*******************************************************************************

 $Function:    	ATB_wap_input_dialog_closed

 $Description:	Provides the answer of an input dialog.
 				
 $Returns:		WAP_OK if successful, WAP_FAIL if otherwise

 $Arguments:	parameter		- Dialog confirmation information
 
*******************************************************************************/

T_WAP_RES ATB_wap_input_dialog_closed(T_MMI_WAP_INPUT_DIALOG_CNF *parameter)
{
	T_MMI_WAP_DRAW_CARD_REQ current_card;

#ifdef TRACE_ATBWAPAUI
	TRACE_FUNCTION("ATB_wap_input_dialog_closed");
#endif

	M_MMI_WAP_INPUT_DIALOG_CNF(parameter);
	current_card.object_id = parameter->object_id;
	M_MMI_WAP_DRAW_CARD_REQ(&current_card);

	return WAP_OK;
}


/*******************************************************************************

 $Function:    	ATB_wap_password_dialog_closed

 $Description:	Provides the answer of a password dialog.
 				
 $Returns:		WAP_OK if successful, WAP_FAIL if otherwise

 $Arguments:	parameter		- Dialog confirmation information
 
*******************************************************************************/

T_WAP_RES ATB_wap_password_dialog_closed(T_MMI_WAP_PASSWORD_DIALOG_CNF *parameter)
{
#ifdef TRACE_ATBWAPAUI
	TRACE_FUNCTION("ATB_wap_password_dialog_closed");
#endif

	M_MMI_WAP_PASSWORD_DIALOG_CNF(parameter);
	
	return WAP_OK;
}


/*******************************************************************************

 $Function:    	ATB_wap_confirm_dialog_closed

 $Description:	Provides the answer of a confirmation dialog.
 				
 $Returns:		WAP_OK if successful, WAP_FAIL if otherwise

 $Arguments:	parameter		- Dialog confirmation information
 
*******************************************************************************/

T_WAP_RES ATB_wap_confirm_dialog_closed(T_MMI_WAP_CONFIRM_DIALOG_CNF *parameter)
{
#ifdef TRACE_ATBWAPAUI
	TRACE_FUNCTION("ATB_wap_confirm_dialog_closed");
#endif

	M_MMI_WAP_CONFIRM_DIALOG_CNF(parameter);
	return WAP_OK;
}


/*******************************************************************************

 $Function:    	ATB_wap_info_dialog_closed

 $Description:	Provides the answer of an info dialog.
 				
 $Returns:		WAP_OK if successful, WAP_FAIL if otherwise

 $Arguments:	parameter		- Dialog confirmation information
 
*******************************************************************************/

T_WAP_RES ATB_wap_info_dialog_closed(T_MMI_WAP_INFO_DIALOG_CNF *parameter)
{
#ifdef TRACE_ATBWAPAUI
	TRACE_FUNCTION("ATB_wap_info_dialog_closed");
#endif

	M_MMI_WAP_INFO_DIALOG_CNF(parameter);
	return WAP_OK;
}


/*******************************************************************************

 $Function:    	ATB_wap_options_menu_select

 $Description:	The AUI will notify to the WAP.ATB the id of the selected item
 
 $Returns:		WAP_OK if successful, WAP_FAIL if otherwise

 $Arguments:	View - the current view
 
*******************************************************************************/

T_WAP_RES ATB_wap_options_menu_select(T_WAP_VIEW *View)
{
#ifdef TRACE_ATBWAPAUI
	TRACE_FUNCTION("ATB_wap_options_menu_select");
#endif

	ATB_wap_card_key_event(View, WAP_KEY_SELECT);
	
	return WAP_OK;
}


/*******************************************************************************

 $Function:    	ATB_wap_buffer_add_element

 $Description:	Add a new element to the element chain
 
 $Returns:		Pointer to the element

 $Arguments:	object_id - the id of the session
 				type	- the type of element (text, fieldset, image)
 
*******************************************************************************/

T_WAP_ELEMENT * ATB_wap_buffer_add_element(UBYTE object_id, WAP_ELEMENT_TYPE type)
{
	T_WAP_VIEW		*View;
	T_WAP_ELEMENT	*Element	= (T_WAP_ELEMENT *)AUI_wap_memory_alloc(sizeof(T_WAP_ELEMENT));// Create the element
	T_WAP_ELEMENT	*ElementIndex;
	static USHORT	elementCount = 0;

#ifdef TRACE_ATBWAPAUI
	TRACE_FUNCTION("ATB_wap_buffer_add_element");
#endif

	if (!(View = ATB_wap_get_view(object_id)))
		return;
	
	if (View->NewElementHeader == NULL)											// If it's the first item in the chain...
	{
		View->NewElementHeader = Element;										// ...element goes at the start
		elementCount = 0;
	}
	else
	{
		ElementIndex = View->NewElementHeader;									// Otherwise, find the end of the chain
		while(ElementIndex->NextElement !=NULL)
			ElementIndex = ElementIndex->NextElement;
		ElementIndex->NextElement = Element;									// ...and put it there
	}
	
	Element->type		= type;
	Element->NextElement = NULL;												// It will be the last item in the chain

	elementCount++;
	
#ifdef TRACE_ATBWAPAUI
	TRACE_EVENT_P1( "Element no.: %d", elementCount);
#endif

	return Element;
}


/*******************************************************************************

 $Function:    	ATB_wap_buffer_display

 $Description:	Displays the card from the buffer
 
 $Returns:		

 $Arguments:	View - the current view
 
*******************************************************************************/

void ATB_wap_buffer_display(T_WAP_VIEW *View)
{
	T_WAP_ELEMENT				*Element		= View->ElementHeader;
	USHORT						elementCount	= 0;							// No. of element
	SHORT						titleX;											// Position of title to centre it
	SHORT						titleWidth;										// Width of the title in pixels
	double						doubleTemp;
	SHORT						scrollBarSize;									// Size of scroll bar in pixels
	SHORT						scrollBarPos;								// Y position of top of scroll bar
	SHORT						lineX;											// First X position of line
	SHORT						lineY;											// First Y position of line
	char						*cardTitle;										// Title of the card
	char						*temp;											// Temporary storage for text strings
	UBYTE						scrollIndex;									// For drawing scrollbar
	
#ifdef TRACE_ATBWAPAUI
	TRACE_FUNCTION("ATB_wap_buffer_display");
#endif
	TRACE_EVENT("ATB_wap_buffer_display_zhaowm");

	// Display the title of the card, centred, over the line

	temp = (char *)AUI_wap_memory_alloc(((CARD_TITLE_MAX_LEN)*2+3)*sizeof(char));		// Allocate memory for card title
	//temp = (char *)AUI_wap_memory_alloc((CARD_TITLE_MAX_LEN+1)*sizeof(char));		// Allocate memory for card title
	/* SPR#1547 - SH - Replaced 'status' with 'browser_status' */
	
	if (ATB_wap_status_get(View, ATB_WAP_DOWNLOADING))										// If the card is loading...
	{
		if ( *AUI_wap_string(WAP_STRING_DOWNLOADING) == 0x80)   //zhaowm 2003.1.7
            wstrcpy(temp, AUI_wap_string(WAP_STRING_DOWNLOADING));
        else    
            strcpy(temp, AUI_wap_string(WAP_STRING_DOWNLOADING));					// ...display "Loading"...
	}

	else if (ATB_wap_status_get(View, ATB_WAP_CARD_READING))										// If the card is loading...
	{
	    if ( *AUI_wap_string(WAP_STRING_UPDATING) == 0x80)   //zhaowm 2003.1.7
            //wstrcpy(temp, AUI_wap_string(WAP_STRING_UPDATING));
            wstrcpy(temp, AUI_wap_string(WAP_STRING_DOWNLOADING));            
        else    

		    strcpy(temp, AUI_wap_string(WAP_STRING_DOWNLOADING));					// ...display "Loading"...
	}

    else
	   {
  
		if (View->Title[0] == 0x80)   //zhaowm 2003.1.7
    	  //wstrncpy(temp, View->Title, wstrlen(View->Title));
           truncateString(temp,View->Title,14,1,".");
        else
          strcpy(temp,View->Title);												// Otherwise, title is the card title
	    }

	//titleWidth = ATB_textWidth(temp, strlen(temp));		
	titleWidth = ATB_textWidth(temp, wstrBitlen(temp));								// Width of the title
	titleX = (WAP_TOTAL_WIDTH - titleWidth)/2;									// Position of title to centre it
	dspl_TextOut(titleX,0,DSPL_TXTATTR_SIGNED_COORDS, temp);					// Draw title text
	
	AUI_wap_memory_free((UBYTE *)temp, ((CARD_TITLE_MAX_LEN)*2+3)*sizeof(char));		// Free card title memory
	//AUI_wap_memory_free((UBYTE *)temp, (CARD_TITLE_MAX_LEN+1)*sizeof(char));		// Free card title memory

	// Horizontal scroll bar and top line
		
	dspl_DrawLine(titleX-2, WAP_TOP_BORDER/2, 0, WAP_TOP_BORDER/2);	// Line to left of title
	dspl_DrawLine(WAP_TOTAL_WIDTH-titleX+1, WAP_TOP_BORDER/2,
		WAP_TOTAL_WIDTH-WAP_VSCROLLBAR_WIDTH, WAP_TOP_BORDER/2);				// Line to right of title

	if (View->cardWidth>WAP_SCREEN_WIDTH)
	{
		doubleTemp = WAP_HSCROLLBAR_WIDTH*(double)WAP_SCREEN_WIDTH/(double)View->cardWidth;
		scrollBarSize = (SHORT)(doubleTemp+0.5);								// Round to nearest int
		if (scrollBarSize>WAP_HSCROLLBAR_WIDTH)
			scrollBarSize = WAP_HSCROLLBAR_WIDTH;

		doubleTemp = WAP_HSCROLLBAR_WIDTH*(double)View->cardXPosition/(double)View->cardWidth;
		scrollBarPos = (SHORT)(doubleTemp+0.5);									// Round to nearest int
		
		for (scrollIndex=0; scrollIndex<scrollBarSize; scrollIndex++)
		{
			dspl_DrawLine(scrollBarPos+scrollIndex, 0,
				scrollBarPos+scrollIndex, WAP_TOP_BORDER/2);
		}

		if (WAP_HSCROLLBAR_WIDTH<(titleX-1))
			dspl_DrawLine( WAP_HSCROLLBAR_WIDTH, 0, WAP_HSCROLLBAR_WIDTH, WAP_TOP_BORDER/2);
	}
	
	// Display the elements of the card one by one
	
	while (Element!=NULL)
	{
		switch(Element->type)
		{
			case WAP_TEXT:														// Text element
				ATB_wap_buffer_text_draw((T_WAP_MMI_SEND_TEXT_IND *)Element->data);
			break;

			case WAP_TABLE:														// Table/fieldset element
				ATB_wap_buffer_table_draw((T_WAP_MMI_SEND_TABLE_IND *)Element->data);
			break;
			
			case WAP_IMAGE:														// Image element
				if (View->Status & WAP_STATUS_SCALEIMAGES)						// Image is to be scaled
					ATB_wap_buffer_image_scale((T_WAP_MMI_SEND_IMAGE_IND *)Element->data);
				else															// Display image normally
					ATB_wap_buffer_image_display((T_WAP_MMI_SEND_IMAGE_IND *)Element->data);
			break;
		}
		elementCount++;
		Element = Element->NextElement;											// Find the next element
	}

	// Vertical scroll bar and right frame

	scrollBarSize = WAP_SCREEN_HEIGHT*WAP_SCREEN_HEIGHT/View->cardHeight;
	if (scrollBarSize>WAP_SCREEN_HEIGHT)
		scrollBarSize = WAP_SCREEN_HEIGHT;

	scrollBarPos = WAP_SCREEN_HEIGHT*View->cardYPosition/View->cardHeight;

	dspl_DrawLine(WAP_TOTAL_WIDTH-WAP_VSCROLLBAR_WIDTH, WAP_TOP_BORDER/2,
		WAP_TOTAL_WIDTH-WAP_VSCROLLBAR_WIDTH, WAP_SCREEN_BOTTOM);						// Line to top of bar

	for (scrollIndex=1; scrollIndex<WAP_VSCROLLBAR_WIDTH; scrollIndex++)
	{
		dspl_DrawLine(WAP_TOTAL_WIDTH-WAP_VSCROLLBAR_WIDTH+scrollIndex, scrollBarPos+WAP_TOP_BORDER,
			WAP_TOTAL_WIDTH-WAP_VSCROLLBAR_WIDTH+scrollIndex, scrollBarPos+WAP_TOP_BORDER+scrollBarSize);
	}
	
#ifdef TRACE_ATBWAPAUI
	TRACE_EVENT_P1( "Elements displayed: %d", elementCount);
#endif

	return;
}


/*******************************************************************************

 $Function:    	ATB_wap_buffer_text_draw

 $Description:	Draws text
 
 $Returns:		

 $Arguments:	parameter	- the send text event
 
*******************************************************************************/

void ATB_wap_buffer_text_draw(T_WAP_MMI_SEND_TEXT_IND *parameter)
{
	char	*temp;
	//char    *temp1;
	SHORT	elementX;
	SHORT	elementY;
	USHORT	textWidth;

	UINT32 oldColor;

	//char    buf[30];  //zhaowm   
	//USHORT  k;

	temp = (char *)AUI_wap_memory_alloc(((parameter->text_length)*2+3)*sizeof(char));  //zhaowm 2002.12.03 add two  // Allocate memory for string
	//temp1 = (char *)AUI_wap_memory_alloc(((parameter->text_length)*2+3)*sizeof(char)); 
	ATB_ushort2char(temp, parameter->Text, (USHORT)parameter->text_length);		// Convert text to CHAR
	//ATB_ushort2char(temp1, parameter->Text, (USHORT)parameter->text_length);		// Convert text to CHAR

	textWidth = parameter->pWidth;
	if ((textWidth + parameter->pX)>WAP_SCREEN_WIDTH)							// if text goes off the screen,
		textWidth = WAP_SCREEN_WIDTH-parameter->pX;								// cut it down to size

	elementX 	= (USHORT)parameter->pX+WAP_LEFT_BORDER;						// Absolute position of text
	elementY 	= (USHORT)parameter->pY+WAP_TOP_BORDER;							// on screen

	switch(parameter->type)
	{
		case WAP_TEXT_OPTION: 													// An options menu item - draw checkbox
			dspl_DrawRect(elementX,
				elementY+WAP_CHECKBOX_TOP, \
				elementX+WAP_CHECKBOX_WIDTH-1, \
				elementY+WAP_CHECKBOX_TOP+WAP_CHECKBOX_HEIGHT-1);
			
			if (parameter->selected)											// Option should be checked
			{
				dspl_DrawRect(elementX+WAP_CHECKBOX_INDENT, \
					elementY+WAP_CHECKBOX_TOP+WAP_CHECKBOX_INDENT, \
					elementX+WAP_CHECKBOX_WIDTH-1-WAP_CHECKBOX_INDENT, \
					elementY+WAP_CHECKBOX_TOP+WAP_CHECKBOX_HEIGHT-1-WAP_CHECKBOX_INDENT);
			}
			
			elementX += (WAP_CHECKBOX_WIDTH+WAP_CHECKBOX_RIGHT);				// Shift to right to make space
			textWidth -= (WAP_CHECKBOX_WIDTH+WAP_CHECKBOX_RIGHT);				// for checkbox
		break;
		
		case WAP_TEXT_OPTIONGROUP:
			// Option group titles are currently not distinguished from normal text
		break;
	}
	
     	//ATB_crop_text((char*)temp, strlen(temp), textWidth);						// Crop the text to the correct width
   // truncateString((char*)temp,(char *)temp1,(unsigned int)textWidth,1,".");  //zhaowm temp

    //zhaowm temp trace
    /*
    for (k=0;k< (parameter->text_length)*2+3;k++)
      {
    	sprintf(buf," zhao=%u,%x",k, temp[k]);//
	    TRACE_EVENT(buf);//
       }//temp
     */
	if(parameter->is_link)
	{
		oldColor = dspl_SetFrgColor(0x0000ff);
	}

	if (!parameter->invert)														// Text is inverted...
		dspl_TextOut(elementX,elementY,DSPL_TXTATTR_SIGNED_COORDS,(char*)temp);
	else																		// ...or isn't inverted
		dspl_TextOut(elementX,elementY,DSPL_TXTATTR_INVERS | DSPL_TXTATTR_SIGNED_COORDS,(char*)temp);

	if(parameter->is_link)
	{
		dspl_DrawLine(elementX  , elementY + parameter->pHeight, elementX  + parameter->pWidth, elementY + parameter->pHeight);
		dspl_SetFrgColor(oldColor );
	}
	AUI_wap_memory_free((UBYTE *)temp, ((parameter->text_length)*2+3)*sizeof(char));     //zhaowm // Free the allocated memory
	//AUI_wap_memory_free((UBYTE *)temp1, ((parameter->text_length)*2+3)*sizeof(char));    
	
	return;
}
				

/*******************************************************************************

 $Function:    	ATB_wap_buffer_table_draw

 $Description:	Draws a table, or a fieldset
 
 $Returns:		

 $Arguments:	parameter	- the send table event
 
*******************************************************************************/

void ATB_wap_buffer_table_draw(T_WAP_MMI_SEND_TABLE_IND *parameter)
{
	SHORT	elementX	= parameter->pX+WAP_LEFT_BORDER;						// Left edge position of table/fieldset
	SHORT	elementY	= parameter->pY+WAP_TOP_BORDER;							// Top edge position of table/fieldset
	SHORT	top			= elementY+WAP_CHAR_HEIGHT/2;							// Top of fieldset
	SHORT	bottom		= elementY+parameter->pHeight-1; 						// Bottom of table/fieldset
	SHORT	right		= elementX+parameter->pWidth-1;							// Right side of table/fieldset
	SHORT	lineX;																// X position of column line
	SHORT	lineY;																// Y position of row line
	char	*temp;																// Temp storage for title
	USHORT	titleWidth;															// Width of the title in pixels
	USHORT	index;																// Index for columns and rows
	char    *temp1;   //zhaowm 2003.1.8

#ifdef TRACE_ATBWAPAUI
	TRACE_FUNCTION("ATB_wap_buffer_table_draw");
	TRACE_EVENT_P4( "Fieldset/table pX, pY, pWidth, pHeight: %d, %d, %d, %d",
		parameter->pX, parameter->pY, parameter->pWidth, parameter->pHeight);
#endif
	TRACE_EVENT("ATB_wap_buffer_table_draw_zhao");

	// Title

	temp = (char *)AUI_wap_memory_alloc(((parameter->title_length)*2+3)*sizeof(char));//zhaowm 2002.1203 add two// Allocate memory for text
	temp1 = (char *)AUI_wap_memory_alloc(((parameter->title_length)*2+3)*sizeof(char));//zhaowm 2002.1203 add two// Allocate memory for text
	
	//ATB_ushort2char(temp, parameter->Title, parameter->title_length);		// Convert to char
	ATB_ushort2char(temp1, parameter->Title, parameter->title_length);		// Convert to char
	
	// Crop title if it overshoots screen

	if ((parameter->pX+gle_fieldset_spaceLeft+ATB_textWidth(temp1, wstrBitlen(temp1))) > WAP_SCREEN_WIDTH)
		//ATB_crop_text(temp, strlen(temp), WAP_SCREEN_WIDTH-parameter->pX-gle_fieldset_spaceLeft);
         truncateString(temp,(char *)temp1,(WAP_SCREEN_WIDTH-parameter->pX-gle_fieldset_spaceLeft)/(WAP_CHAR_WIDTH*2),1,".");

	// Crop title if it overshoots edge of fieldset
	if ((gle_fieldset_spaceLeft+ATB_textWidth(temp1, wstrBitlen(temp1))+gle_fieldset_spaceRight) > parameter->pWidth)
		//ATB_crop_text(temp, strlen(temp), parameter->pWidth-gle_fieldset_spaceLeft-gle_fieldset_spaceRight);
         truncateString(temp,(char *)temp1,(parameter->pWidth-gle_fieldset_spaceLeft-gle_fieldset_spaceRight)/(WAP_CHAR_WIDTH*2),1,".");

	titleWidth = ATB_textWidth(temp,wstrBitlen(temp));								// Width of the title of the fieldset

	if (parameter->pY >= 0 && parameter->pY < WAP_TEXT_LOWEST)					// Provided title is on screen
	{
		dspl_TextOut(elementX+gle_fieldset_spaceLeft, elementY, DSPL_TXTATTR_SIGNED_COORDS, temp);	
	}
	AUI_wap_memory_free((UBYTE *)temp, ((parameter->title_length)*2+3)*sizeof(char));     // Free allocated memory
	AUI_wap_memory_free((UBYTE *)temp1, ((parameter->title_length)*2+3)*sizeof(char));     // Free allocated memory

	// Horizontal lines

	lineY = elementY+gle_cell_vBefore;
	ATB_wap_buffer_line_draw(elementX, bottom, right, bottom);					// Bottom line of table

	if (parameter->rows_length > 0)
		ATB_wap_buffer_line_draw(elementX, lineY, right, lineY);				// Line under title
	else
	{
		ATB_wap_buffer_line_draw(elementX+gle_fieldset_spaceLeft-2, top, elementX, top);     // Line to left of title
		ATB_wap_buffer_line_draw(elementX+gle_fieldset_spaceLeft+titleWidth+2, top, right, top);// right
	}
	
	if (parameter->rows_length > 1)												// Row lines
	{
		for (index=0; index<(parameter->rows_length-1); index++)
		{
			lineY += parameter->RowHeight[index]+gle_cell_spaceToBorder;
			ATB_wap_buffer_line_draw(elementX, lineY, right, lineY);
			lineY += gle_cell_borderExtent+gle_cell_spaceToBorder;
		}
	}
	
	// Vertical lines

	lineX = elementX+gle_cell_hBefore;
	
	if (parameter->rows_length == 0)
		lineY = top;															// For fieldset
	else
		lineY = elementY+gle_cell_vBefore;										// For table

	ATB_wap_buffer_line_draw(elementX, lineY, elementX, bottom);				// Left hand line
	
	if (parameter->cols_length > 1)												// Column lines
	{
		for (index=0; index<(parameter->cols_length-1); index++)
		{
			lineX += parameter->ColWidth[index]+gle_cell_spaceToBorder;
			ATB_wap_buffer_line_draw(lineX, lineY, lineX, bottom);
			lineX += gle_cell_borderExtent+gle_cell_spaceToBorder;
		}
	}
	ATB_wap_buffer_line_draw(right, lineY, right, bottom);						// Right hand line
		
	return;
}


/*******************************************************************************

 $Function:    	ATB_wap_buffer_line_draw

 $Description:	Draws a line, making sure it fits on-screen
 
 $Returns:		

 $Arguments:	x1,y1 - Coordinates of the start of the line
 				x2,y2 - Coordinates of the end of the line
 
*******************************************************************************/

void ATB_wap_buffer_line_draw(SHORT x1, SHORT y1, SHORT x2, SHORT y2)
{
	SHORT deltaX1;
	SHORT deltaY1;
	SHORT deltaX2;
	SHORT deltaY2;
	double m;
	
	ATB_wap_buffer_onscreen(x1,y1, &deltaX1, &deltaY1);
	ATB_wap_buffer_onscreen(x2,y2, &deltaX2, &deltaY2);
	
	if (x2==x1)
	{
		if (deltaX1)
			return;
		y1+=deltaY1;
		y2+=deltaY2;
	}
	else if (y2==y1)
	{
		if (deltaY1)
			return;
		x1+=deltaX1;
		x2+=deltaX2;
	}
	else
	{
		m = (y2-y1)/(x2-x1);	
		if (deltaX1 || deltaY1)
		{
			x1 += deltaX1;
			y1 += m*deltaX1;
			if (!ATB_wap_buffer_onscreen(x1,y1,&deltaX1,&deltaY1))
				return;
		}
		if (deltaX2 || deltaY2)
		{
			x2 += deltaX2;
			y2 += m*deltaX2;
			if (!ATB_wap_buffer_onscreen(x2,y2,&deltaX2,&deltaY2))
				return;
		}
	}
	
	dspl_DrawLine(x1,y1,x2,y2);
	return;
}


/*******************************************************************************

 $Function:    	ATB_wap_buffer_onscreen

 $Description:	Returns TRUE if the point supplied is on-screen
 
 $Returns:		

 $Arguments:	x,y - coordinates of the point
 				deltaX, deltaY - returns offset from screen
 
*******************************************************************************/

static BOOL ATB_wap_buffer_onscreen(SHORT x, SHORT y, SHORT *deltaX, SHORT *deltaY)
{
	BOOL outcome = TRUE;
	*deltaX = 0;
	*deltaY = 0;
	
	if (x<WAP_LEFT_BORDER)
	{
		outcome = FALSE;
		*deltaX = WAP_LEFT_BORDER-x;
	}
	if (x>(WAP_SCREEN_RIGHT-1))
	{
		outcome = FALSE;
		*deltaX = (WAP_SCREEN_RIGHT-1)-x;
	}
	if (y<WAP_TOP_BORDER)
	{
		outcome = FALSE;
		*deltaY = WAP_TOP_BORDER-y;
	}
	if (y>(WAP_SCREEN_BOTTOM-1))
	{
		outcome = FALSE;
		*deltaY = (WAP_SCREEN_BOTTOM-1)-y;
	}

	return outcome;
}


/*******************************************************************************

 $Function:    	ATB_wap_buffer_image_display

 $Description:	Trims an image to fit into the specified space, then displays it
 
 $Returns:		

 $Arguments:	image - the image to display
 
*******************************************************************************/

void ATB_wap_buffer_image_display(T_WAP_MMI_SEND_IMAGE_IND *image)
{
	UBYTE *Region;																// Pointer to trimmed image
	SHORT offsetX = 0;															// Offsets from origin of original image...
	SHORT offsetY = 0;															// ...where trimmed image starts
	SHORT scrWidth = image->pWidth;												// Width of trimmed image on screen
	SHORT scrHeight = image->pHeight;											// Height of trimmed image on screen
	SHORT elementX;																// Its x position on the screen
	SHORT elementY;																// Its y position on the screen
	USHORT byteWidth;															// Width of original in bytes
	USHORT scrByteWidth;														// Width of screen image in bytes
	USHORT	offsetByteWidth;
	USHORT heightIndex;	
	USHORT widthIndex;
	USHORT memIndex = 0;
	UBYTE *Image = image->Image;
	UBYTE pixShift;																// Pixel shift, shift bytes by this amount

#ifdef TRACE_ATBWAPAUI
	TRACE_FUNCTION("ATB_wap_buffer_image_display");
#endif
	TRACE_EVENT("ATB_wap_buffer_image_display_zhao");

	byteWidth = image->pWidth/8;
	if (image->pWidth % 8)
		byteWidth++;
	
	elementX = image->pX;
	elementY = image->pY;
	
	if (elementX < 0)															// If image starts off the left of the screen...
	{
		offsetX = -elementX;													// Set the X offset so it is trimmed appropriately
		scrWidth -= offsetX;													// Change its width
		elementX = 0;															// And place it at the left of the screen
	}
	if (image->pY < 0)															// If image starts above the top of the screen
	{
		offsetY = -elementY;													// Set the Y offset so it is trimmed appropriately
		scrHeight -= offsetY;													// Change its height
		elementY = 0;															// And place it at the top of the screen
	}
	
	if ((elementX+scrWidth)>WAP_SCREEN_WIDTH)									// If the image goes off the right of the screen
		scrWidth = WAP_SCREEN_WIDTH-elementX;									// Reduce its width
	if ((elementY+scrHeight)>WAP_SCREEN_HEIGHT)									// If the image goes off the bottom of the screen
		scrHeight = WAP_SCREEN_HEIGHT-elementY;									// Trim its height

	scrByteWidth = scrWidth/8;													// Work out the width of the trimmed image in bytes
	if (scrWidth % 8)
		scrByteWidth++;
	
	Region = (UBYTE *)AUI_wap_memory_alloc(scrByteWidth*scrHeight);				// Allocate memory for the trimmed image

	offsetByteWidth = offsetX/8;
	pixShift = offsetX % 8;														// Offset within the byte of the first pixel
	
	Image += (offsetY*byteWidth)+offsetByteWidth;										// Point to the first byte of the region to be isolated
		
	// Copy the trimmed region of the image into our allocated memory,
	// inverting it as necessary
	
	for (heightIndex = 0; heightIndex<scrHeight; heightIndex++)
	{
		for (widthIndex = 0; widthIndex<scrByteWidth; widthIndex++)
		{
			Region[memIndex] = Image[widthIndex] << pixShift;
			if ((offsetByteWidth+scrByteWidth)<byteWidth || widthIndex<(scrByteWidth-1))
				Region[memIndex] |= Image[widthIndex+1] >> (8-pixShift);
			
			if (!image->invert)
				Region[memIndex] ^= 0xFF;									// De-invert it if not selected

			memIndex++;
		}
		Image += byteWidth;														// Move down one line
	}

#ifdef TRACE_ATBWAPAUI
	TRACE_EVENT_P2( "byteWidth: %d, scrByteWidth: %d", byteWidth, scrByteWidth);
	TRACE_EVENT_P4( "pX: %d, pY: %d, pWidth: %d, pHeight: %d", image->pX, image->pY, image->pWidth, image->pHeight);
	TRACE_EVENT_P4( "offsetX: %d, offsetY: %d, scrWidth: %d, scrHeight: %d", offsetX, offsetY, scrWidth, scrHeight);
	TRACE_EVENT_P2( "elementX: %d, elementY: %d", elementX, elementY);
	TRACE_EVENT_P2( "Region size: %d, should be: %d", memIndex, scrByteWidth*scrHeight);
#endif

	// Display the image

	DSPL_BitBlt (elementX+WAP_LEFT_BORDER, elementY+WAP_TOP_BORDER, scrWidth, scrHeight, 0, (void *)Region, 0);

	// Free the memory
	AUI_wap_memory_free(Region, scrByteWidth*scrHeight);
	
	return;
}


/*******************************************************************************

 $Function:    	ATB_wap_buffer_image_scale

 $Description:	Scales an image to fit into a specified space
 
 $Returns:		

 $Arguments:	View - pointer to the appropriate view
 
*******************************************************************************/

void ATB_wap_buffer_image_scale(T_WAP_MMI_SEND_IMAGE_IND *image)
{
	T_WAP_MMI_SEND_IMAGE_IND *Scaled = (T_WAP_MMI_SEND_IMAGE_IND *)AUI_wap_memory_alloc(sizeof(T_WAP_MMI_SEND_IMAGE_IND));																// Pointer to scaled image
	SHORT	newWidth;
	SHORT	newHeight;
	USHORT heightIndex;	
	USHORT widthIndex;
	USHORT oldIndex;
	USHORT	newIndex;
	USHORT	byteWidth;
	USHORT	scrByteWidth;
	UBYTE	*Image = image->Image;
	UBYTE	oldMask;																// Pixel shift, shift bytes by this amount
	UBYTE	newMask;
	UBYTE	pixel;
	USHORT	xScale;
	USHORT	xScaleOld;
	USHORT	yScale;
	USHORT	yScaleOld;

#ifdef TRACE_ATBWAPAUI
	TRACE_FUNCTION("ATB_wap_buffer_image_scale");
#endif
	TRACE_EVENT("ATB_wap_buffer_image_scale_zhao");

	if (image->pWidth > WAP_SCREEN_WIDTH)
		newWidth = WAP_SCREEN_WIDTH;											// Squash to fit horizontally
	else
		newWidth = image->pWidth;

	if (image->pHeight > WAP_SCREEN_HEIGHT)
		newHeight = WAP_SCREEN_HEIGHT;											// Squash to fit vertically
	else
		newHeight = image->pHeight;
	
	byteWidth = image->pWidth/8;												// Work out the width of the original image in bytes
	if (image->pWidth % 8)
		byteWidth++;
	
	scrByteWidth = newWidth/8;													// Work out the width of the scaled image in bytes
	if (newWidth % 8)
		scrByteWidth++;
	
	Scaled->Image = (UBYTE *)AUI_wap_memory_alloc(scrByteWidth*newHeight);		// Allocate memory for the scaled image

	oldIndex = -1;
	newIndex = -1;
	yScaleOld = 0;
	
	// Scale the image
	
	for (heightIndex = 0; heightIndex<image->pHeight; heightIndex++)
	{
		yScale = (USHORT) (heightIndex+1)*newHeight/image->pHeight;
		
		if (yScale != yScaleOld)
		{
			oldMask = 1;														// Mask within the byte of the first pixel
			newMask = 1;
			xScaleOld = 0;

			for (widthIndex = 0; widthIndex<image->pWidth; widthIndex++)
			{
				xScale = (USHORT) (widthIndex+1)*newWidth/image->pWidth;

				if (oldMask == 1)
				{
					oldMask = 128;
					oldIndex++;
				}
				else
					oldMask /= 2;
				
				if (xScale != xScaleOld)
				{
					if (newMask == 1)
					{
						newMask = 128;
						newIndex++;
						Scaled->Image[newIndex] = 0;
					}
					else
						newMask /= 2;

					if (Image[oldIndex] & oldMask)							// If old pixel set,
						Scaled->Image[newIndex] |= newMask;					// set new pixel
				}
				
				xScaleOld = xScale;
			}
		}
		else
		{
			oldIndex += byteWidth;
#if defined(TRACE_ATBWAPAUI) 
			TRACE_EVENT_P1( "Skipping line %d.", heightIndex);
#endif
		}
		yScaleOld = yScale;
	}

#ifdef TRACE_ATBWAPAUI
	TRACE_EVENT_P2( "byteWidth: %d, scrByteWidth: %d", byteWidth, scrByteWidth);
	TRACE_EVENT_P4( "pX: %d, pY: %d, pWidth: %d, pHeight: %d",
		image->pX, image->pY, image->pWidth, image->pHeight);
	TRACE_EVENT_P2( "newWidth: %d, newHeight: %d", newWidth, newHeight);

#endif

	// Display the image

	Scaled->object_id = image->object_id;
	Scaled->image_length = scrByteWidth*newHeight;
	Scaled->invert = image->invert;
	Scaled->pX = image->pX;
	Scaled->pY = image->pY;
	Scaled->pWidth = newWidth;
	Scaled->pHeight = newHeight;
	Scaled->selected = image->selected;
	
	ATB_wap_buffer_image_display(Scaled);
	
	// Free the memory
	AUI_wap_memory_free(Scaled->Image, scrByteWidth*newHeight);
	AUI_wap_memory_free((UBYTE *)Scaled, sizeof(T_WAP_MMI_SEND_IMAGE_IND));
	
	return;
}


/*******************************************************************************

 $Function:    	ATB_wap_buffer_clear

 $Description:	Clears the element buffer
 
 $Returns:		

 $Arguments:	View - pointer to the appropriate view
 
*******************************************************************************/

void ATB_wap_buffer_clear(T_WAP_VIEW *View)
{
	T_WAP_ELEMENT					*Element	= View->ElementHeader;
	T_WAP_ELEMENT					*NextElement;
	T_WAP_MMI_SEND_TEXT_IND			*TextElement;
	T_WAP_MMI_SEND_TABLE_IND 		*TableElement;
	T_WAP_MMI_SEND_IMAGE_IND 		*ImageElement;
	USHORT 							elementCount = 0;

#ifdef TRACE_ATBWAPAUI
	TRACE_FUNCTION("ATB_wap_buffer_clear");
#endif

	while (Element != NULL)
	{
		switch (Element->type)
		{
			case WAP_TEXT:
				TextElement = (T_WAP_MMI_SEND_TEXT_IND *)Element->data;
				if (TextElement->text_length != 0)
					AUI_wap_memory_free((UBYTE *)TextElement->Text, TextElement->text_length*sizeof(USHORT));
				if (TextElement->formatstring_length != 0)
				{
					AUI_wap_memory_free((UBYTE *)TextElement->Formatstring,
						TextElement->formatstring_length*sizeof(USHORT));
				}
				AUI_wap_memory_free((UBYTE *)TextElement, sizeof(T_WAP_MMI_SEND_TEXT_IND));
			break;

			case WAP_TABLE:
				TableElement = (T_WAP_MMI_SEND_TABLE_IND *)Element->data;
				if (TableElement->title_length != 0)
					AUI_wap_memory_free((UBYTE *)TableElement->Title, TableElement->title_length*sizeof(USHORT));
				if (TableElement->cols_length != 0)
					AUI_wap_memory_free((UBYTE *)TableElement->ColWidth, TableElement->cols_length*sizeof(SHORT));
				if (TableElement->rows_length != 0)
					AUI_wap_memory_free((UBYTE *)TableElement->RowHeight, TableElement->rows_length*sizeof(SHORT));
				AUI_wap_memory_free((UBYTE *)TableElement, sizeof(T_WAP_MMI_SEND_TABLE_IND));
			break;
				
			case WAP_IMAGE:
				ImageElement = (T_WAP_MMI_SEND_IMAGE_IND *)Element->data;
				if (ImageElement->image_length != 0)
					AUI_wap_memory_free((UBYTE *)ImageElement->Image, ImageElement->image_length*sizeof(UBYTE));
				AUI_wap_memory_free((UBYTE *)ImageElement, sizeof(T_WAP_MMI_SEND_IMAGE_IND));
			break;
		}
		NextElement = Element->NextElement;
		AUI_wap_memory_free((UBYTE *)Element, sizeof(T_WAP_ELEMENT));
		Element = NextElement;
		elementCount++;
	}
	
#ifdef TRACE_ATBWAPAUI
	TRACE_EVENT_P1( "Elements destroyed: %d", elementCount);
#endif

	View->ElementHeader = NULL;
	return;
}


/*******************************************************************************

 $Function:    	ATB_wap_buffer_identify_element

 $Description:	Identifies which element is selected (based on the invert property) and
 				returns an appropriate type.
 
 $Returns:		The type of the element that is selected, or NULL if none

 $Arguments:	View	- pointer to the view
 
*******************************************************************************/

UBYTE ATB_wap_buffer_identify_element(T_WAP_VIEW *View)
{
	T_WAP_ELEMENT				*Element	= View->ElementHeader;
	T_WAP_MMI_SEND_TEXT_IND		*TextElement;
	T_WAP_MMI_SEND_IMAGE_IND	*ImageElement;

#ifdef TRACE_ATBWAPAUI
	TRACE_FUNCTION("ATB_wap_buffer_identify_element");
#endif

	while (Element!=NULL)
	{
		switch (Element->type)
		{
			case WAP_TEXT:
				TextElement = (T_WAP_MMI_SEND_TEXT_IND *)Element->data;
				if (TextElement->invert)
				{
					return TextElement->type;
				}
			break;

			case WAP_IMAGE:
				ImageElement = (T_WAP_MMI_SEND_IMAGE_IND *)Element->data;
				if (ImageElement->invert)
				{
					return WAP_SELECTABLE_IMAGE;
				}
		}
		Element = Element->NextElement;
	}
	
	return NULL;
}


/*******************************************************************************

 $Function:    	ATB_wap_profile_read

 $Description:	Reads in the specified profile and changes the ProfileId
 
 $Returns:		

 $Arguments:	View		- Pointer to the current view
 				ProfileId	- the id of the profile to read from the flash
 
*******************************************************************************/

T_WAP_RES ATB_wap_profile_read(T_WAP_VIEW *View, UBYTE ProfileId)
{
	T_WAP_PROFILE	*Profile		= View->Profile;
	UBYTE 			entryIndex;
	char			*Entry;
	char			*URL;

#ifdef TRACE_ATBWAPAUI
	TRACE_FUNCTION("ATB_wap_profile_read");
#endif

	View->ProfileId	= ProfileId;
	View->Profile	= &(WapData.Profile[View->ProfileId]);						// Points to appropriate profile
		
	return WAP_OK;
}


/*******************************************************************************

 $Function:    	ATB_wap_profile_names_read

 $Description:	Reads into RAM the names of all profiles stored in flash, also
 				the global bookmarks and history lists
 
 $Returns:		

 $Arguments:	Profile		- Pointer to the current profile
 
*******************************************************************************/

T_WAP_RES ATB_wap_profile_names_read(T_WAP_VIEW *View)
{
	T_WAP_PROFILE	*Profile;
	T_WAP_LIST 		*ProfilesList	= View->ProfilesList;
	USHORT			no_of_bookmarks;
	T_WAP_LIST		*Bookmarks		= View->Bookmarks;
	T_WAP_LIST		*BookmarksURL	= View->BookmarksURL;
	USHORT			no_of_history;
	T_WAP_LIST		*History		= View->History;
	T_WAP_LIST		*HistoryURL		= View->HistoryURL;
	UBYTE			entryIndex;
	char			*Entry;
	char			*URL;
	UBYTE			no_of_profiles;

	// Default values are provided.  If the file 'mmi/WapData' isn't found, a file is created
	// with these defaults.
	
#ifdef TRACE_ATBWAPAUI
	TRACE_FUNCTION("ATB_wap_profile_names_read");
#endif
   /*
	if (flash_wap_read() == EFFS_NOTFOUND)
	{
		ATB_wap_profile_default_create();
	}
   */
    //wapffsfile_init();    //zhaowm 2002/12/26
	View->Status 		= WapData.Status;									// Status flags
	View->ProfileId		= WapData.ProfileId;								// Default starting profile

	// Read in profiles list
	
	no_of_profiles = WapData.no_of_profiles;
	ProfilesList->no_of_entries = no_of_profiles;
		
	for (entryIndex = 0; entryIndex<MAX_PROFILES; entryIndex++)
	{
		ProfilesList->Entry[entryIndex] = WapData.Profile[entryIndex].Title;
	}

	// Read in bookmarks
	
	no_of_bookmarks = WapData.no_of_bookmarks;
	Bookmarks->no_of_entries	= no_of_bookmarks;
	BookmarksURL->no_of_entries	= no_of_bookmarks;
	
	for (entryIndex = 0; entryIndex<MAX_BOOKMARKS; entryIndex++)
	{
		Bookmarks->Entry[entryIndex] = WapData.Bookmarks[entryIndex];
		BookmarksURL->Entry[entryIndex] = WapData.BookmarksURL[entryIndex];
	}

	// Read in history entries
	
	no_of_history = WapData.no_of_history;
	History->no_of_entries 	= no_of_history;
	HistoryURL->no_of_entries  = no_of_history;
	
	for (entryIndex = 0; entryIndex<MAX_HISTORY; entryIndex++)
	{
		History->Entry[entryIndex] = WapData.History[entryIndex];
		HistoryURL->Entry[entryIndex] = WapData.HistoryURL[entryIndex];
	}

	return WAP_OK;
}


/*******************************************************************************

 $Function:    	ATB_wap_profile_save

 $Description:	Store the current updated profile in flash
 
 $Returns:		

 $Arguments:	View		- Pointer to the current view
 
*******************************************************************************/

T_WAP_RES ATB_wap_profile_save(T_WAP_VIEW *View)
{	

#ifdef TRACE_ATBWAPAUI
	TRACE_FUNCTION("ATB_wap_profile_save");
#endif
	
	WapData.ProfileId			= View->ProfileId;
	WapData.Status				= View->Status;
	WapData.no_of_bookmarks	= View->Bookmarks->no_of_entries;
	WapData.no_of_history		= View->History->no_of_entries;
	flash_wap_write();

	return WAP_OK;
}


/*******************************************************************************

 $Function:    	ATB_wap_profile_default_create

 $Description:	Creates a default profile in the flash if one isn't already there
 
 $Returns:		

 $Arguments:	
 
*******************************************************************************/

T_WAP_RES ATB_wap_profile_default_create()
{	

#if 0

	UBYTE	no_of_bookmarks		= 10;	
	char	*bookmarksList[] =
	{
		"Yahoo!",
		"AOL",
		"Genie",
		"Financial Times",
		"Cnet tech news",
		"The Guardian",
		"Breathe",
		"WEB.DE",
		"Vizzavi",
		"El Pais"
	};
		
	char 	*bookURLList[]	= 
	{
		"http://wap.yahoo.com",
		"http://wap.aol.co.uk",
		"http://wap.genie.co.uk",
		"http://wap.ft.com",
		"http://wap.cnet.com",
		"http://www.guardian.co.uk/wml#C",
		"http://wap.breathe.com",
		"http://wap.web.de",
		"http://wap.vizzavi.co.uk",
		"http://wap.elpais.es"
	};

	UBYTE	no_of_history		= 0;
	char	*historyList[]		= { "dummy" };
	char	*histURLList[]		= { "dummy" };

	UBYTE	no_of_profiles	= 2;
	char 	*profilesDefault[] 	= { "profile1", "profile2"};
	char	*homepageURL[]		= { "http://wap.monternet.com", "http://wap.yahoo.com"};
	char	*IPAddress1[]		= { "010.000.000.172", "010.000.000.172"};
	char	*IPAddress2[]		= { "010.000.000.172", "010.000.000.172"};
	char	*DialupNumber[]		= { "17266", "17266" };
	char	*APN[]				= { "cmwap", "cmwap"}; 
	char	*Username[]			= { "wap", "wap" };
	char	*Password[]			= { "wap", "wap"};
	USHORT	ResponseTimer[]		= {180,60};
	USHORT	Port1[]				= {9201,9201};
	USHORT	Port2[]				= {9201,9201};


	USHORT entryIndex;
	USHORT	profile;

#ifdef TRACE_ATBWAPAUI
	TRACE_FUNCTION("ATB_wap_profile_default_create");
#endif

	WapData.ProfileId		= 0;
	WapData.Status	= WAP_STATUS_SAVEHISTORY | WAP_STATUS_SCALEIMAGES;

	// Copy bookmarks into WapData structure
	
	WapData.no_of_bookmarks = no_of_bookmarks;

	for (entryIndex = 0; entryIndex<no_of_bookmarks; entryIndex++)
	{
		strcpy(WapData.Bookmarks[entryIndex], bookmarksList[entryIndex]);
		strcpy(WapData.BookmarksURL[entryIndex], bookURLList[entryIndex]);
	}

	//	Copy history list into WapData structure
			
	WapData.no_of_history = no_of_history;
	
	for (entryIndex = 0; entryIndex<no_of_history; entryIndex++)
	{
		sprintf(WapData.History[entryIndex],"%2d. ",entryIndex+1);			// store number
		strcpy(WapData.History[entryIndex]+NUMBER_PADDING, historyList[entryIndex]);
		strcpy(WapData.HistoryURL[entryIndex], histURLList[entryIndex]);
	}

	// Copy profile data into WapData structure
	
	WapData.no_of_profiles    = no_of_profiles;
		
	for (profile = 0; profile<no_of_profiles; profile++)
	{
		strcpy(WapData.Profile[profile].Title, profilesDefault[profile]);
		
		strcpy(WapData.Profile[profile].Homepage, homepageURL[profile]);
		
		WapData.Profile[profile].ConnectionType	= WAP_TEMPORARY;
		WapData.Profile[profile].Security			= FALSE;
		WapData.Profile[profile].AccessType		= WAP_GPRS_DATA;//WAP_CS_DATA;
		WapData.Profile[profile].ResponseTimer		= ResponseTimer[profile];
		WapData.Profile[profile].Port1				= Port1[profile];
		WapData.Profile[profile].Port2				= Port2[profile];
		strcpy(WapData.Profile[profile].IPAddress1, IPAddress1[profile]);
		strcpy(WapData.Profile[profile].IPAddress2, IPAddress2[profile]);
		strcpy(WapData.Profile[profile].DialupNumber, DialupNumber[profile]);
		strcpy(WapData.Profile[profile].APN, APN[profile]);
		strcpy(WapData.Profile[profile].Username, Username[profile]);
		strcpy(WapData.Profile[profile].Password, Password[profile]);
	}

	// Write data to flash
	
	if (flash_wap_write()==EFFS_OK)												// Make sure flash data is written OK
		return WAP_OK;
	else
		return WAP_FAIL;

  #endif		
}


/*******************************************************************************

 $Function:    	ATB_wap_profile_send

 $Description:	Sends the configuration information of the current profile to the WAP task.
 
 $Returns:		

 $Arguments:	View  - the current view
 
*******************************************************************************/

void ATB_wap_profile_send(T_WAP_VIEW *View)
{
	T_MMI_WAP_CONFIGURE_IND					Configure;
	T_MMI_WAP_CONNECTION_CONFIGURE_IND		Connection;
	T_WAP_PROFILE							*Profile = View->Profile;
	ULONG									IPAddressLong;

#ifdef TRACE_ATBWAPAUI
	TRACE_FUNCTION("ATB_wap_profile_send");
#endif

// Configure integer values first - set up some common parameters

	Configure.object_id			= View->object_id;
	Configure.type				= WAP_IntConfig;
	Configure.length			= 0;
	Configure.strvalue			= 0;
	
	// Set max no. of history entries
	
	Configure.param				= configHISTORY_SIZE;
	Configure.intvalue			= (U32) MAX_HISTORY;
	ATB_wap_configure(&Configure);

	// Switch off images, for now
	
	Configure.param				= configDISPLAY_IMAGES;
	Configure.intvalue			= 1;
	ATB_wap_configure(&Configure);
	Configure.param				= configUPDATE_IMAGES;
	Configure.intvalue			= 0;
	ATB_wap_configure(&Configure);

	// Set default channel

	Configure.param				= configDEFAULT_CHANNEL;
	Configure.intvalue			= View->channel;
	ATB_wap_configure(&Configure);

	// Configure some connection parameters now
	Connection.object_id		= View->object_id;
	Connection.type 			= WAP_IntConfig;
	Connection.channel_id		= View->channel;
	Connection.length			= 0;
	Connection.strvalue			= 0;

	// Setup stack mode
	Connection.param			= configSTACKMODE;
	Connection.intvalue			= MODE_CO_WSP;
	ATB_wap_connection_configure(&Connection);

	// Set bearer for data channel
	Connection.param			= configACCESS_TYPE;
	switch(Profile->AccessType)
	{
		case WAP_GPRS_DATA:
			Connection.intvalue	= BEARER_GSM_GPRS;
		break;
		case WAP_CS_DATA:
			Connection.intvalue	= BEARER_GSM_CSD;
		break;
		case WAP_SMS:
			Connection.intvalue	= BEARER_GSM_SMS;
		break;
	}
	ATB_wap_connection_configure(&Connection);

		// Setup timeout
	Connection.param			= configTIMEOUT;
	Connection.intvalue			= Profile->ResponseTimer;
	ATB_wap_connection_configure(&Connection);

	// Set IP address
	Connection.type 			= WAP_StrConfig;
	Connection.param			= configUDP_IP_GW;
	Connection.length			= sizeof(ULONG);				// Length of IP address
	ATB_conv_str_to_IP(Profile->IPAddress1, strlen(Profile->IPAddress1), &IPAddressLong);
	Connection.strvalue = (CHAR *)&IPAddressLong;
	ATB_wap_connection_configure(&Connection);


	// Set WAP Server Authentification name

	Connection.type 			= WAP_StrConfig;
	Connection.param			= configAUTH_ID_GW;
	Connection.length			= strlen(Profile->Username);				
	Connection.strvalue			= Profile->Username;
	ATB_wap_connection_configure(&Connection);

	// Set WAP Server Authentification password

	Connection.type 			= WAP_StrConfig;
	Connection.param			= configAUTH_PASS_GW;
	Connection.length			= strlen(Profile->Password);				
	Connection.strvalue			= Profile->Password;
	ATB_wap_connection_configure(&Connection);

	// Set WAP host

	Connection.type 			= WAP_StrConfig;
	Connection.param			= configADD_HOST;
	Connection.length			= strlen(Profile->Homepage);				
	Connection.strvalue			= Profile->Homepage;
	ATB_wap_connection_configure(&Connection);

	// Set connection port
	Connection.type 			= WAP_IntConfig;
	Connection.param			= configCLIENT_LOCAL_PORT;
	Connection.length			= 0;
	Connection.strvalue			= 0;
	Connection.intvalue			= 3;
	ATB_wap_connection_configure(&Connection);
	// Set connection port 1
	Connection.type 			= WAP_IntConfig;
	Connection.param			= WAP_setPort1;
	Connection.length			= 0;
	Connection.strvalue			= 0;
	Connection.intvalue			= Profile->Port1;
	ATB_wap_connection_configure(&Connection);

	// Set connection port 2
	Connection.type 			= WAP_IntConfig;
	Connection.param			= WAP_setPort2;
	Connection.length			= 0;
	Connection.strvalue			= 0;
	Connection.intvalue			= Profile->Port2;
	ATB_wap_connection_configure(&Connection);
}


/*******************************************************************************

 $Function:    	ATB_wap_entry_list_create

 $Description:	Creates an entries list
 
 $Returns:		

 $Arguments:	type		- type of list
 				max_entries	- maximum number of entries in list
 
*******************************************************************************/

T_WAP_LIST * ATB_wap_entry_list_create(WAP_LIST_TYPE type, UBYTE max_entries, UBYTE entry_size)
{
	T_WAP_LIST *EntryList;

#ifdef TRACE_ATBWAPAUI
	TRACE_FUNCTION("ATB_wap_entry_list_create");
#endif

	EntryList  			= (T_WAP_LIST *)AUI_wap_memory_alloc(sizeof(T_WAP_LIST));	// List structure
	EntryList->Entry	= (char **)AUI_wap_memory_alloc(max_entries*sizeof(char*));	// Pointers to list entries

	EntryList->type		= type;
	EntryList->max_entries = max_entries;
	EntryList->entry_size  = entry_size;
	EntryList->no_of_entries = 0;
	
	return EntryList;
}


/*******************************************************************************

 $Function:    	ATB_wap_entry_list_destroy

 $Description:	Destroys an entries list
 
 $Returns:		

 $Arguments:	EntryList	- pointer to entry list data
 
*******************************************************************************/

T_WAP_RES ATB_wap_entry_list_destroy(T_WAP_LIST *EntryList)
{	
#ifdef TRACE_ATBWAPAUI
	TRACE_FUNCTION("AUI_wap_entry_list_destroy");
#endif

	if (EntryList==NULL)														// Make sure request is meaningful
		return WAP_FAIL;

	// Destroy structure
	AUI_wap_memory_free((UBYTE *)EntryList->Entry, EntryList->max_entries*sizeof(char *));
	AUI_wap_memory_free((UBYTE *)EntryList,sizeof(T_WAP_LIST));
	
	return WAP_OK;
}


/*******************************************************************************

 $Function:    	ATB_wap_entry_add

 $Description:	Adds an entry to an entry list
 
 $Returns:		

 $Arguments:	Entrylist	- Pointer to entries table
 				Entry		- Entry to add
 
*******************************************************************************/

T_WAP_RES ATB_wap_entry_add(T_WAP_LIST *EntryList, char *Entry)
{
	UBYTE entryIndex = EntryList->no_of_entries;								// Where the new entry should go

#ifdef TRACE_ATBWAPAUI
	TRACE_FUNCTION("ATB_wap_entry_add");
	ATB_trace_string(Entry, strlen(Entry));
#endif

	if (entryIndex==EntryList->max_entries)
		return WAP_FAIL;														// No space in table
	
	// Add the entry
	if (Entry[0] == 0x80)   //zhaowm 2003.03.24
       	wstrncpy(EntryList->Entry[entryIndex], Entry, wstrlen(Entry));
    else
	strcpy(EntryList->Entry[entryIndex], Entry);								// Copy string
	EntryList->no_of_entries++;													// Increase bookmark count
	
	return WAP_OK;
}


/*******************************************************************************

 $Function:    	ATB_wap_entry_change

 $Description:	Changes an entry in an entry list, destroying the old entry and
 				creating a new one.
 
 $Returns:		

 $Arguments:	type			- Type of entry list
 				Entrylist 		- Pointer to entries table
 				Entry			- New entry text
 				entryIndex		- Number of entry to change
 
*******************************************************************************/

T_WAP_RES ATB_wap_entry_change(WAP_LIST_TYPE type, T_WAP_LIST *EntryList, char *Entry, UBYTE entryIndex)
{

#ifdef TRACE_ATBWAPAUI
	TRACE_FUNCTION("ATB_wap_entry_change");
#endif

	if (type==WAP_HISTORY_LIST)
	{ if (Entry[0] == 0x80)    //zhaowm 2003.1.7
		wstrncpy(EntryList->Entry[entryIndex], Entry, wstrlen(Entry));
      else
        strcpy(EntryList->Entry[entryIndex], Entry);	
		//sprintf(EntryList->Entry[entryIndex], "%2d. ", entryIndex+1);			// Add "xx. " number at start
		//strcat(EntryList->Entry[entryIndex], Entry);							// Store in History List
	}
	else
	{ if (Entry[0] == 0x80)    //zhaowm 2003.1.7
		wstrncpy(EntryList->Entry[entryIndex], Entry, wstrlen(Entry));
      else
		strcpy(EntryList->Entry[entryIndex], Entry);							// Copy string
	}
	
	return WAP_OK;
}


/*******************************************************************************

 $Function:    	ATB_wap_entry_insert

 $Description:	Insert an entry into an entry list, shifting all subsequent entries up one.
 				If the maximum limit of the entry list is reached, the last entry is
 				deleted.
 
 $Returns:		

 $Arguments:	type			- Type of entry list
 				Entrylist 		- Pointer to entries table
 				Entry			- Entry to insert
 				entryIndex 		- Position at which to insert the new entry
 
*******************************************************************************/

T_WAP_RES ATB_wap_entry_insert(WAP_LIST_TYPE type, T_WAP_LIST *EntryList, char *Entry, UBYTE entryIndex)
{	
	USHORT entryIndex2 = EntryList->no_of_entries;

#ifdef TRACE_ATBWAPAUI
	TRACE_FUNCTION("ATB_wap_entry_remove");
#endif

	if (EntryList==NULL || entryIndex>EntryList->no_of_entries
		|| EntryList->no_of_entries>EntryList->max_entries)  					// Make sure request is meaningful
		return WAP_FAIL;

	if (EntryList->no_of_entries == EntryList->max_entries)	 					// If the list is full...
		entryIndex2--;															// Start shifting from second last entry
	else
		EntryList->no_of_entries++;												// Increase bookmark count

	while (entryIndex2>entryIndex)												// Work back from end of table
	{
	  if (*(EntryList->Entry[entryIndex2-1]) == 0x80)  //zhaowm
        wstrncpy(EntryList->Entry[entryIndex2],EntryList->Entry[entryIndex2-1],wstrlen(EntryList->Entry[entryIndex2-1]));	
      else
	 	strcpy(EntryList->Entry[entryIndex2],EntryList->Entry[entryIndex2-1]);	// Shift the table down to make space
	  entryIndex2--;
	}

	if (type==WAP_HISTORY_LIST)													// For history list, need to include space for number at start
	{
	  if (Entry[0] == 0x80)   //zhaowm 2003.1.7
    	wstrncpy(EntryList->Entry[entryIndex], Entry, wstrlen(Entry));
      else
		//strcpy(EntryList->Entry[entryIndex]+NUMBER_PADDING, Entry);
		strcpy(EntryList->Entry[entryIndex], Entry);
		//ATB_wap_renumber_history(EntryList);
	}
	else
	{
		strcpy(EntryList->Entry[entryIndex], Entry);							// Copy string
	}
	
	return WAP_OK;
}


/*******************************************************************************

 $Function:    	ATB_wap_entry_remove

 $Description:	Removes an entry from the bookmarks or history list and moves
 				other entries up to close the gap.
 
 $Returns:		

 $Arguments:	Entrylist 		- Pointer to entries table
 				entryIndex 		- Number of the entry to delete
 
*******************************************************************************/

T_WAP_RES ATB_wap_entry_remove(T_WAP_LIST *EntryList, UBYTE entryIndex)
{	
#ifdef TRACE_ATBWAPAUI
	TRACE_FUNCTION("ATB_wap_entry_remove");
#endif

	if (EntryList==NULL || entryIndex>EntryList->no_of_entries)					// Make sure request is meaningful
		return WAP_FAIL;
			
	EntryList->no_of_entries--;													// Decrease bookmark count

	while (entryIndex<(EntryList->no_of_entries))								// If we haven't reached end of table...
	{
     if (*(EntryList->Entry[entryIndex+1]) == 0x80)   //zhaowm 2003.03.24
       	wstrncpy(EntryList->Entry[entryIndex], EntryList->Entry[entryIndex+1], wstrlen(EntryList->Entry[entryIndex+1]));
     else	
		strcpy(EntryList->Entry[entryIndex],EntryList->Entry[entryIndex+1]);	// Shift the table up to remove the gap
		entryIndex++;
	}

	return WAP_OK;
}


/*******************************************************************************

 $Function:    	ATB_wap_entry_remove_all

 $Description:	Removes all entries from a Entry List
 
 $Returns:		

 $Arguments:	EntryList 	- Pointer to bookmarks table
 
*******************************************************************************/

T_WAP_RES ATB_wap_entry_remove_all(T_WAP_LIST *EntryList)
{
	UBYTE entryIndex;

#ifdef TRACE_ATBWAPAUI
	TRACE_FUNCTION("ATB_wap_entry_remove_all");
#endif

	if (EntryList==NULL)														// Make sure request is meaningful
		return WAP_FAIL;
		
	EntryList->no_of_entries = 0;												// No entries remaining
	return WAP_OK;
}


/*******************************************************************************

 $Function:    	ATB_wap_renumber_history

 $Description:	Renumber history list, changing prefixes to all entries
 
 $Returns:		

 $Arguments:	History - pointer to history list	
 
*******************************************************************************/

int ATB_wap_renumber_history(T_WAP_LIST *History)
{
	UBYTE entryIndex;

#ifdef TRACE_AUIWAP
	TRACE_FUNCTION("ATB_wap_renumber_history");
#endif

	for (entryIndex = 0; entryIndex<(History->no_of_entries); entryIndex++)
	{
		//sprintf(History->Entry[entryIndex],"%2d.",entryIndex+1);			//zhaowm 2003.1.8	// Overwrite the number prefix
		*(History->Entry[entryIndex]+NUMBER_PADDING-1) = 32;					// Overwrite terminating 0 after prefix with space
	}

	return 1;
}


/*******************************************************************************

 $Function:    	ATB_wap_status_change

 $Description:	Called to change a status flag, i.e. downloading, updating, etc
				SPR#1574 - SH - Removed on/off flag - now selecting a different
				status overrides the previous status.
 
 $Returns:		

 $Arguments:	View			- The current view
				status			- New status
 
*******************************************************************************/

void ATB_wap_status_change(T_WAP_VIEW *View, USHORT status)
{
#ifdef TRACE_ATBWAPAUI
	TRACE_FUNCTION("ATB_wap_status_change");
#endif

	/* SPR#1574 - SH - Replaced 'status' with 'browser_status' */
	View->browser_status = status;

#ifdef MMS_WAP_ENABLED
	if(View->object_id == 128)
	{
		AUI_mms_status_notify(View, status);
	}
	else
	{
		AUI_wap_status_notify(View, status);
	}
#else
	AUI_wap_status_notify(View, status);										// Inform AUI of status change
#endif
}


/*******************************************************************************

 $Function:    	ATB_wap_status_get

 $Description:	
 
 $Returns:		If the browser is in the supplied status (TRUE/FALSE)

 $Arguments:	View			- The current view
				status			- Is the browser in this state?
 
*******************************************************************************/

BOOL ATB_wap_status_get(T_WAP_VIEW *View, USHORT status)
{
#ifdef TRACE_ATBWAPAUI
	TRACE_FUNCTION("ATB_wap_status_get");
#endif

	/* SPR#1574 - SH - Replaced 'status' with 'browser_status' */
	
	if (View->browser_status==status)
		return (TRUE);
	else
		return (FALSE);
}


/*******************************************************************************

 $Function:    	ATB_ushort2char

 $Description:	Converts a USHORT string to a char string
 
 $Returns:		the copy

 $Arguments:	dst			- the destination string
				src			- the source string
				srclen		- the length of the source string
 
*******************************************************************************/

char* ATB_ushort2char_old (char *dst, USHORT *src, USHORT srclen)
{
	UBYTE length;

#ifdef TRACE_ATBWAPAUI
	TRACE_FUNCTION("ATB_ushort2char");
#endif

	for (length = 0; length < srclen; length++)
	{
		dst[length] = (char)(src[length]) & 0x7F;						// Filter out unprintable characters
	}
	
	dst[srclen] = NULL;														// Add terminating NULL
	return dst;
}

 
//zhaowm changed 2002.12.02
char* ATB_ushort2char (char *dst, USHORT *src, USHORT srclen)
{
	//UBYTE length;
	USHORT length,j=1,k;
  //  char buf[30];  //zhaowm   
    TRACE_EVENT("ATB_ushort2char ");

#ifdef TRACE_ATBWAPAUI
	TRACE_FUNCTION("ATB_ushort2char");
#endif
    dst[0] = 0x80;
	for (length = 0; length < srclen; length++)
	{
		//dst[j++] = (char)(((src[length])>>8)&0xff) ;	/* high byte */
		dst[j++] = (char)((src[length])>>8) ;	/* high byte */
		dst[j++] = (char)((src[length])&0xff) ;	/* low byte */


	}
	dst[j] = 0x00;														// Add terminating NULL
    dst[j++] = 0x00;
/*
    for (k=0;k< srclen*2;k++)
      {
    	sprintf(buf," lk=%u,%x",k, dst[k]);//
	    TRACE_EVENT(buf);//
       }
*/
	return dst;
}//zhaowm



/*******************************************************************************

 $Function:    	ATB_char2ushort

 $Description:	Converts a char string to a USHORT string
 
 $Returns:		the length of the destination string

 $Arguments:	dst			- the destination string
				src			- the source string
 
*******************************************************************************/

USHORT ATB_char2ushort (USHORT *dst, char *src)
{
	USHORT length = 0;

#ifdef TRACE_ATBWAPAUI
	TRACE_FUNCTION("ATB_char2ushort");
#endif

	while (src[length]!=NULL)
	{
		dst[length] = (USHORT) src[length];
		length++;
	}
	
	dst[length] = NULL;														// Add terminating NULL
	
	return length;
}


USHORT ATB_uni2ushort (USHORT *dst, char *src)                 //zhaowm 2003 03.31
{
	USHORT length = 0;
    unsigned char * ss=(unsigned char*)src;
    
    if(*ss==0x80)//if unicode
    	ss++;
	while((*ss<<8)+*(ss+1)!=0)
	{
	  dst[length] = (*ss<<8)| (*(ss+1));
      length++;
      ss+=2;
      
	}

	dst[length] = 0x0000;														// Add terminating NULL
	return length;
}

/*******************************************************************************

 $Function:    	ATB_char2char

 $Description:	Copies a CHAR string that is not necessarily terminated with a NULL
 
 $Returns:		the copy

 $Arguments:	dst			- the destination string
				src			- the source string
				srclen		- the length of the source string
 
*******************************************************************************/

char* ATB_char2char (char *dst, char *src, USHORT srclen)
{
	UBYTE length;

#ifdef TRACE_ATBWAPAUI
	TRACE_FUNCTION("ATB_char2char");
#endif

	for (length = 0; length < srclen; length++)
	{
		dst[length] = src[length];
	}
	
	dst[length] = NULL;														// Add terminating NULL
	return dst;
}


/*******************************************************************************

 $Function:    	ATB_char_to_BCD

 $Description:	Converts an ascii string of digits into BCD form, with 4 bits representing
 				each digit.
 
 $Returns:		

 $Arguments:	src 	- source string (ascii)
 				dest	- destination string for BCD digits
 
*******************************************************************************/

void ATB_char_to_BCD(char *dest, char *src)
{
	UBYTE srcIndex = 0;
	UBYTE destIndex = 0;
	BOOL leftbits = TRUE;														// Left or right set of 4 bits
	UBYTE digit;

#ifdef TRACE_ATBWAPAUI
	TRACE_FUNCTION("ATB_char_to_BCD");
#endif

	while (src[srcIndex]!=0)
	{
		digit = src[srcIndex]-'0';											// The digit, 0 to 9.

		if (leftbits)
		{
			dest[destIndex] = digit;
			leftbits = FALSE;
		}
		else
		{
			dest[destIndex] |= digit*16;										// *16 shifts right 4 bits
			leftbits = TRUE;
		}
		srcIndex++;
		destIndex++;
	}
	
	return;
}


/*******************************************************************************

 $Function:    	ATB_textWidth

 $Description:	Returns width of string in pixels
 
 $Returns:		

 $Arguments:	str			- the string
				length		- the length of string to measure
 
*******************************************************************************/

USHORT ATB_textWidth(char *str, USHORT length)
{

#ifdef TRACE_ATBWAPAUI
	TRACE_FUNCTION("ATB_textWidth");
#endif

	return length*WAP_CHAR_WIDTH;
}

/*******************************************************************************

 $Function:    	ATB_charsInWidth

 $Description:	Returns number of characters of the string that can fit into the given
 				width in pixels.
 
 $Returns:		Number of characters

 $Arguments:	str			- the string
				length		- the length of string to attempt to display
				width		- the width in pixels into which the string must fit
 
*******************************************************************************/

USHORT ATB_charsInWidth(char *str, USHORT length, SHORT width)
{
	USHORT strIndex;
	
#ifdef TRACE_ATBWAPAUI
	TRACE_FUNCTION("ATB_charsInWidth");
#endif

	// For now, characters are fixed width, so this is simple.  This must be altered
	// when proportional text is introduced (e.g. as below).
	
	return width/WAP_CHAR_WIDTH;

/*
	for (strIndex = 0; strIndex<length && ATB_textWidth(str, strIndex)<=width; strIndex++)
		;

	return strIndex-1;
	*/
}

/*******************************************************************************

 $Function:    	ATB_crop_text

 $Description:	Crops text if necessary to fit the supplied length
 
 $Returns:		

 $Arguments:	str			- the string
 				length		- the original length
				maxlen		- the maximum length of the new string
 
*******************************************************************************/

void ATB_crop_text(char *str, USHORT length, SHORT width)
{
	USHORT maxlen = ATB_charsInWidth(str, length, width);	// Maximum length of string that will fit
	
#ifdef TRACE_ATBWAPAUI
	TRACE_FUNCTION("ATB_crop_text");
#endif

	if (maxlen<length)
	{
		str[maxlen] = NULL;
		if (maxlen>0)
			str[maxlen-1] = '.';
		if (maxlen>1)
			str[maxlen-2] = '.';												// Adds '..' to indicate text is cropped
	}
	return;
}

/*******************************************************************************

 $Function:    	ATB_conv_str_to_IP

 $Description:	Converts a string IP address to a long type IP address
 
 $Returns:		

 $Arguments:	str			- the string
 				len			- the original length
				ip			- pointer to where the result will be placed
 
*******************************************************************************/

GLOBAL const void ATB_conv_str_to_IP(const char* str, USHORT len, ULONG* ip)
{
	char buf[16];
	char *p, *pLast;
	ULONG i, j, k, m;

#ifdef TRACE_ATBWAPAUI
	TRACE_FUNCTION("ATB_conv_str_to_IP");
#endif

	if (len >= 16)
	{
		*ip = 0;
		return;
	}

	memcpy (buf, str, len);
	buf[len] = '\0';

	p = buf;

	pLast = buf;
	k = 0;
	j = 0;
	m = 1;
	for (i=0; i<len; i++)
	{
		if (*p=='.')
		{
			*p = '\0';
			k |= (atoi(pLast) & 0xFF) * m;
			m*=0x100;
			j++;
			p++;
			pLast = p;
			if (j EQ 3)
				break;
		}
		else
			p++;
	}

	if (j NEQ 3 OR p EQ &buf[len])
	{
		*ip = 0;
		return;
	}

	k |= (atoi(pLast) & 0xFF) * m;

	*ip = k;

	return;
}

/*******************************************************************************

 $Function:    	ATB_data_call_connected

 $Description:	Confirms that a data call has been connected
 
 $Returns:		

 $Arguments:	
 
*******************************************************************************/

void ATB_data_call_connected(void)
{
T_MMI_WAP_CONNECT_CNF parameter;

#ifdef TRACE_ATBWAPAUI
	TRACE_FUNCTION("ATB_data_call_connected");
#endif

#ifndef MMS_WAP_ENABLED
	if (View_header != NULL)
	{
		parameter.object_id = View_header->object_id;
	 	parameter.channel = View_header->channel;
		parameter.success = TRUE;
		M_MMI_WAP_CONNECT_CNF(&parameter);
	}
#else
	if (Current_view != NULL)
	{
		parameter.object_id = Current_view->object_id;
	 	parameter.channel = Current_view->channel;
		parameter.success = TRUE;
		M_MMI_WAP_CONNECT_CNF(&parameter);
	}
#endif
}


/*******************************************************************************

 $Function:    	ATB_wap_start_done

 $Description:	Confirms that WAP_START has completed
 
 $Returns:		

 $Arguments:	
 
*******************************************************************************/

void ATB_wap_start_done(void)
{
	AUI_wap_start_done();
	return;
}

/*******************************************************************************

 $Function:    	ATB_wap_new_view_done

 $Description:	Confirms that WAP_NEW_VIEW has completed
 
 $Returns:		

 $Arguments:	
 
*******************************************************************************/

void ATB_wap_new_view_done(void)
{
	AUI_wap_new_view_done();
	return;
}

/*******************************************************************************

 $Function:    	ATB_wap_close_view_done

 $Description:	Confirms that WAP_CLOSE_VIEW has completed
 
 $Returns:		

 $Arguments:	
 
*******************************************************************************/

void ATB_wap_close_view_done(void)
{
	AUI_wap_close_view_done();
	return;
}

/*******************************************************************************

 $Function:    	ATB_wap_terminate_done

 $Description:	Confirms that WAP_TERMINATE has completed
 
 $Returns:		

 $Arguments:	
 
*******************************************************************************/

void ATB_wap_terminate_done(void)
{
	AUI_wap_terminate_done();
	return;
}


/*******************************************************************************
 $Function:    	ATB_wap_disconnect

 $Description:	SPR#1189 - SH - Added this function
 				Informs WAP browser that a disconnection has occurred
 
 $Returns:		

 $Arguments:	View - the current View
 
*******************************************************************************/

void ATB_wap_disconnect(T_WAP_VIEW *View)
{
	T_MMI_WAP_DISCONNECT_IND parameter;

	TRACE_FUNCTION("ATB_wap_disconnect()");
	
	parameter.object_id = View->object_id;
	parameter.channel = View->channel;
	
	M_MMI_WAP_DISCONNECT_IND(&parameter);

	return;
}


/*******************************************************************************
 $Function:    	long_IP_to_char_IP

 $Description:
 
 $Returns:		

 $Arguments:	
 
*******************************************************************************/

void long_IP_to_char_IP(ULONG IPs,char* IPd)
{
	ULONG tempLong;
	char tempChar;
	int i;
	char temp[50];

#ifdef TRACE_ATBWAPAUI
	sprintf(temp,"IP Address %X",(int)IPs);
	TRACE_EVENT(temp);		
#endif

	for (i=3;i>=0;i--)
	{
		tempLong=((IPs)>>8*i)&(0xFF);
		*IPd = (char)tempLong;
#ifdef TRACE_ATBWAPAUI
		sprintf(temp,"IP %d %X",i,(int)*IPd);	
		TRACE_EVENT(temp);		
#endif
		IPd++;
	}
	
	
}


/*******************************************************************************

 $Function:    	rAT_WAP_PPP_connected

 $Description:
 
 $Returns:		

 $Arguments:	
 
*******************************************************************************/

void rAT_WAP_PPP_connected(SHORT cId,ULONG IPAddress)
{

	char temp[30];
	T_MMI_WAP_CONNECTION_CONFIGURE_IND		Connection;

  TRACE_FUNCTION("rAT_WAP_PPP_connected");

#ifdef TRACE_ATBWAPAUI
  TRACE_EVENT_P1(temp,"Wap Call Id %d",cId);
#endif

	/* SPR#1569 - SH - If there are no open views, disconnect and exit */
	
	if (!View_header)
	{
		TRACE_EVENT("** WAP has closed, disconnecting call **");
		/* SPR#1575 - SH - Call this function to end a GPRS or CSD call */
		AUI_end_call();
		return;
	}
	
	/* SPR#1189 - SH - If the browser is not in the correct state, disconnect */

	if (ATB_wap_status_get(Current_view, ATB_WAP_ONLINE_CONNECTION_CLOSED))
	{
		TRACE_EVENT("** WAP is not in correct status, disconnecting call **");
		AUI_end_call();
		return;
	}
	if (IPAddress)
	{
#ifdef MMS_WAP_ENABLED
	extern ULONG IPAddressSaved ;
		IPAddressSaved = IPAddress;
#endif
		temp[4]='\0';

		long_IP_to_char_IP(IPAddress,(char*)temp);

		// Set IP address host
		Connection.object_id		= Current_view->object_id;
		Connection.channel_id		= Current_view->channel;
		Connection.type 			= WAP_StrConfig;
		Connection.param			= configUDP_IP_SRC;
		Connection.length			= 4;				// Length of IP address
		Connection.strvalue			= temp;
		ATB_wap_connection_configure(&Connection);
	}

	/* SPR#1574 - SH - Removed parameter */
	ATB_wap_status_change(Current_view,ATB_WAP_CONNECTING);

	ATB_data_call_connected();
}


/*******************************************************************************

 $Function:    	rAT_WAP_start_login

 $Description:
 
 $Returns:		

 $Arguments:	
 
*******************************************************************************/

void rAT_WAP_start_login(void)
{

	UBYTE temp;
	TRACE_FUNCTION("rAT_WAP_start_login");
	
	temp = dspl_Enable(0);

	/* SPR#1569 - SH - Only change status if current view exists */
#ifndef MMS_WAP_ENABLED
	if (Current_view)
	{
		/* SPR#1574 - SH - Removed parameter */
		ATB_wap_status_change(Current_view, ATB_WAP_LOGGING_IN);
	}
#else
	if(Wap_view)
		ATB_wap_status_change(Wap_view, ATB_WAP_LOGGING_IN);
	if(MMS_view)
		ATB_wap_status_change(MMS_view, ATB_WAP_LOGGING_IN);
#endif	
	
	dspl_Enable(temp);
}


/*******************************************************************************

 $Function:    	rAT_WAP_start_gprs_login

 $Description:
 
 $Returns:		

 $Arguments:	
 
*******************************************************************************/

void rAT_WAP_start_gprs_login(void)
{

	UBYTE temp;
	TRACE_EVENT("rAT_WAP_start_gprs_login");

	temp = dspl_Enable(0);

	/* SPR#1569 - SH - Only change status if current view exists */
	
	if (View_header && Current_view)
	{
		/* SPR#1574 - SH - Removed parameter */
		/* SPR#1575 - SH - Removed instance of WAP_OBJECT_ID */
		ATB_wap_status_change(Current_view, ATB_WAP_CONNECTING);
	}
	
	dspl_Enable(temp);
}


/*******************************************************************************

 $Function:    	rAT_WAP_call_disconnected

 $Description:
 
 $Returns:		

 $Arguments:	
 
*******************************************************************************/

void rAT_WAP_call_disconnected(SHORT cId)
{
	UBYTE temp;
	T_MMI_WAP_CONNECT_CNF parameter;

	TRACE_FUNCTION("rAT_WAP_call_disconnected");

	temp = dspl_Enable(0);

	/* SPR#1569 - SH - If there are no open views, do nothing */
	
	if (View_header != NULL)
	{
		if (Current_view)
		{
			/* SPR#1574 - SH - Removed parameter */
			ATB_wap_status_change(Current_view, ATB_WAP_ONLINE_CONNECTION_CLOSED);
		}
	
		parameter.object_id = View_header->object_id;
	 	parameter.channel = View_header->channel;
		parameter.success = FALSE;
		M_MMI_WAP_CONNECT_CNF(&parameter);
		Current_view = View_header;
#ifdef MMS_WAP_ENABLED/*It must be add,talcon 2004/06/21*/
#endif
	}

	dspl_Enable(temp);
}
#ifdef MMS_WAP_ENABLED


/*************************talcon add for mms ****************/

/*******************************************************************************

 $Function:    	ATB_mms_start

 $Description:	Starts the WAP application  
 $Returns:		

 $Arguments:	parameter		- dummy empty parameter
 
*******************************************************************************/

void ATB_mms_start(T_MMI_WAP_MMSSTART_IND *parameter)
{
//extern T_WAP_DATA mms_data;
extern T_MMS_VIEW mms_view;
#ifdef TRACE_ATBWAPAUI
	TRACE_FUNCTION("ATB_mms_start");
#endif

	
	M_MMI_WAP_MMSSTART_IND(parameter);

	return;
}


void ATB_mms_start_done(void)
{

	AUI_mms_start_done();
	return;
}
void  ATB_mms_terminate_done()
{
	AUI_mms_terminate_done();
}

/*******************************************
$Description:
	cancel mms
********************************************/
void ATB_mms_cancel(UBYTE urlID)
{

	T_MMI_WAP_CANCELCONTENT_IND parameter;
	parameter.urlID = urlID;
	M_MMI_WAP_CANCELCONTENT_IND(&parameter);

	/***close mms ********/
	//AUI_mms_close();
}

void ATB_mms_profile_send(T_MMS_VIEW *View)
{
//extern T_WAP_PROFILE mms_profile;
	T_MMI_WAP_CONFIGURE_IND					Configure;
	T_MMI_WAP_CONNECTION_CONFIGURE_IND		Connection;
	T_WAP_PROFILE							*Profile =  View->Profile;
	ULONG									IPAddressLong;

#ifdef TRACE_ATBWAPAUI
	TRACE_FUNCTION("ATB_mms_profile_send");
#endif

	WAP_TRACE_EVENT(("%s,%s",View->Profile->Title, View->Profile->Homepage));
	WAP_TRACE_EVENT(("%d,%d,%d,%d",View->Profile->ConnectionType, View->Profile->Security,View->Profile->AccessType,View->Profile->ResponseTimer));
	WAP_TRACE_EVENT(("%s,%s,%s",View->Profile->IPAddress1, View->Profile->IPAddress2,View->Profile->DialupNumber));
	WAP_TRACE_EVENT(("%d,%d",View->Profile->Port1, View->Profile->Port2));
	WAP_TRACE_EVENT(("%s,%s,%s",View->Profile->APN, View->Profile->Username,View->Profile->Password));


// Configure integer values first - set up some common parameters

	Configure.object_id			= View->object_id;
	Configure.type				= WAP_IntConfig;
	Configure.length			= 0;
	Configure.strvalue			= 0;

	// Set default channel

	Configure.param				= configDEFAULT_CHANNEL;
	Configure.intvalue			= View->channel;
	//WAP_TRACE_EVENT(("object_id:%d, channel:%d",View->channel,View->object_id));
	ATB_wap_configure(&Configure);

	// Configure some connection parameters now
	Connection.object_id		= View->object_id;
	Connection.type 			= WAP_IntConfig;
	Connection.channel_id		= View->channel;
	Connection.length			= 0;
	Connection.strvalue			= 0;

	// Setup stack mode
	Connection.param			= configSTACKMODE;
	Connection.intvalue			= MODE_CO_WSP;
	ATB_wap_connection_configure(&Connection);


	// Set bearer for data channel
	Connection.param			= configACCESS_TYPE;
	switch(Profile->AccessType)
	{
		case WAP_GPRS_DATA:
			Connection.intvalue	= BEARER_GSM_GPRS;
		break;
		case WAP_CS_DATA:
			Connection.intvalue	= BEARER_GSM_CSD;
		break;
		case WAP_SMS:
			Connection.intvalue	= BEARER_GSM_SMS;
		break;
	}
	//WAP_TRACE_EVENT(("accesstype:%d",Profile->AccessType));
	ATB_wap_connection_configure(&Connection);

		// Setup timeout
	Connection.param			= configTIMEOUT;
	Connection.intvalue			= Profile->ResponseTimer;
	//WAP_TRACE_EVENT(("ResponseTimer:%d",Profile->ResponseTimer));
	ATB_wap_connection_configure(&Connection);

	// Set IP address
	Connection.type 			= WAP_StrConfig;
	Connection.param			= configUDP_IP_GW;
	Connection.length			= sizeof(ULONG);				// Length of IP address
	ATB_conv_str_to_IP(Profile->IPAddress1, strlen(Profile->IPAddress1), &IPAddressLong);
	Connection.strvalue = (CHAR *)&IPAddressLong;
	//WAP_TRACE_EVENT(("IPAddress1:%s,length:%d",Profile->IPAddress1, strlen(Profile->IPAddress1)));
	ATB_wap_connection_configure(&Connection);


	// Set WAP Server Authentification name

	Connection.type 			= WAP_StrConfig;
	Connection.param			= configAUTH_ID_GW;
	Connection.length			= strlen(Profile->Username);				
	Connection.strvalue			= Profile->Username;
	//WAP_TRACE_EVENT(("Username:%s,length:%d",Profile->Username, strlen(Profile->Username)));
	ATB_wap_connection_configure(&Connection);

	// Set WAP Server Authentification password

	Connection.type 			= WAP_StrConfig;
	Connection.param			= configAUTH_PASS_GW;
	Connection.length			= strlen(Profile->Password);				
	Connection.strvalue			= Profile->Password;
//	WAP_TRACE_EVENT(("Password:%s,length:%d",Profile->Password, strlen(Profile->Password)));
	ATB_wap_connection_configure(&Connection);


	// Set WAP host
	Connection.type 			= WAP_StrConfig;
	Connection.param			= configADD_HOST;
	Connection.length			= strlen(Profile->Homepage);				
	Connection.strvalue			= Profile->Homepage;
	ATB_wap_connection_configure(&Connection);
	

	// Set connection port
	Connection.type 			= WAP_IntConfig;
	Connection.param			= configCLIENT_LOCAL_PORT;
	Connection.length			= 0;
	Connection.strvalue			= 0;
	Connection.intvalue			= 3;
	ATB_wap_connection_configure(&Connection);
	// Set connection port 1
	Connection.type 			= WAP_IntConfig;
	Connection.param			= WAP_setPort1;
	Connection.length			= 0;
	Connection.strvalue			= 0;
	Connection.intvalue			= Profile->Port1;
	ATB_wap_connection_configure(&Connection);

	// Set connection port 2
	Connection.type 			= WAP_IntConfig;
	Connection.param			= WAP_setPort2;
	Connection.length			= 0;
	Connection.strvalue			= 0;
	Connection.intvalue			= Profile->Port2;
	ATB_wap_connection_configure(&Connection);

}



void ATB_wap_content_post(T_MMI_WAP_POSTCONTENT_IND *parameter)
{
	TRACE_FUNCTION("ATB_wap_content_post");
	
	 M_MMI_WAP_POSTCONTENT_IND(parameter);

	 return;
}

void ATB_wap_morecontent_post(T_MMI_WAP_POSTMORECONTENT_IND *parameter)
{
	TRACE_FUNCTION("ATB_wap_morecontent_post");
	
	 M_MMI_WAP_POSTMORECONTENT_IND(parameter);

	 return;
}

/********************************************
$Description:
	insert  a new view into view list
**********************************************/
void ATB_wap_wiew_add(T_WAP_VIEW *View)
{
	TRACE_FUNCTION("ATB_wap_wiew_add");
	if (View_header == NULL)
		View_header = View;														// This is the first entry
	else
	{
		T_WAP_VIEW *ViewIndex = View_header;
		while (ViewIndex->NextView != NULL)										// If not the last entry,
			ViewIndex = ViewIndex->NextView;									// find the next entry!
		ViewIndex->NextView = View;												// New entry is last entry in chain
	}

	Current_view = View;
#ifdef MMS_WAP_ENABLED
	switch(View->object_id)
	{
		case 128:	
			MMS_view = View;
			break;
		default:
			Wap_view = View;
			break;
	}
#endif
}

/********************************************
$Description:
	remove  a view from view list
**********************************************/
void ATB_wap_view_delete(T_WAP_VIEW *View)
{
	T_WAP_VIEW *p1 = View_header;
	T_WAP_VIEW *p2 = View_header;
	
	TRACE_FUNCTION("ATB_wap_view_delete");
	while(p1)
	{
		if(p1 == View)
			break;
		p2 = p1;
		p1 = p1->NextView;
	}
	if(p1 == View_header)
	{
		View_header = View_header->NextView;
	}
	else
	{
		p2->NextView = View->NextView;
	}

	if(Current_view == View)
	{
		Current_view = View_header;
	}
	if(MMS_view == View)
		MMS_view = NULL;
	else if(Wap_view == View)
		Wap_view = NULL;
}


/*******************************************************************************

 $Function:    	ATB_mms_status_change

 $Description:	
 $Returns:		

 $Arguments:	View			- The current view
				status			- New status
 
*******************************************************************************/

void ATB_mms_status_change(T_WAP_VIEW *View, USHORT status)
{
	TRACE_FUNCTION("ATB_mms_status_change");

	View->mms_status = status;

	//AUI_wap_status_notify(View, status);										// Inform AUI of status change
}


/*******************************************************************************

 $Function:    	ATB_mms_status_get

 $Description:	
 
 $Returns:		If the browser is in the supplied status (TRUE/FALSE)

 $Arguments:	View			- The current view
				status			- Is the browser in this state?
 
*******************************************************************************/

BOOL ATB_mms_status_get(T_WAP_VIEW *View, USHORT status)
{
#ifdef TRACE_ATBWAPAUI
	TRACE_FUNCTION("ATB_mms_status_get");
#endif

	
	if (View->mms_status==status)
		return (TRUE);
	else
		return (FALSE);
}


#endif



