/****************************************************************************/
/*                                                                          */
/*  File Name:  ffs_api.h                                                   */
/*                                                                          */
/*                                                                          */
/*  Version   0.1                                                           */
/*                                                                          */
/*  Date                Modification                                        */
/*  ------------------------------------                                    */
/*  28 January   2002   Create                                              */
/*                                                                          */
/*  Author       Stephanie Gerthoux                                         */
/*                                                                          */
/* (C) Copyright 2002 by Texas Instruments Incorporated, All Rights Reserved*/
/****************************************************************************/

#ifndef _FFS_API_H_
#define _FFS_API_H_

   #include "ffs.h"


/******************************************************************************
    * Types
 ******************************************************************************/

   #define FFS_OBJECT_TYPE   T_FFS_OBJECT_TYPE      

   /******************************************************************************
    * Call-back function prototypes
    ******************************************************************************/

   T_FFS_REQ_ID ffs_mkdir_nb (const char  *name,
                              T_RV_RETURN *cp);

   T_FFS_REQ_ID ffs_remove_nb (const char  *namestruct,
                               T_RV_RETURN *cp);

   T_FFS_REQ_ID ffs_rename_nb (const char *oldname,
                               const char *newname,
                              T_RV_RETURN *cp);

   T_FFS_REQ_ID  ffs_open_nb (const char       *name,
                              T_FFS_OPEN_FLAGS option,
                              T_RV_RETURN      *cp);

   T_FFS_REQ_ID ffs_close_nb (T_FFS_FD    fdi,
                              T_RV_RETURN *cp);

   T_FFS_REQ_ID ffs_write_nb (T_FFS_FD    fdi,
                              void        *src,
                              T_FFS_SIZE  size,
                              T_RV_RETURN *cp);

   T_FFS_REQ_ID ffs_seek_nb (T_FFS_FD     fdi,
                             T_FFS_SIZE   offset,
                             T_FFS_WHENCE whence,
                             T_RV_RETURN  *cp); 

   T_FFS_REQ_ID ffs_ftruncate_nb (T_FFS_FD     fdi,
                                  T_FFS_OFFSET length,
                                  T_RV_RETURN  *cp);

   /******************************************************************************
    * No call-back function prototypes
    ******************************************************************************/

   T_FFS_RET ffs_mkdir (const char *name);

   T_FFS_SIZE ffs_opendir (const char *name,
                           T_FFS_DIR  *dir);

   T_FFS_SIZE ffs_readdir (T_FFS_DIR  *dir,
                           char       *name,
                           T_FFS_SIZE size);

   T_FFS_RET ffs_stat (const char *name,
                       T_FFS_STAT *stat);

   T_FFS_RET ffs_remove (const char *name);

   T_FFS_RET ffs_rename (const char *oldname,
                         const char *newname);

   T_FFS_FD  ffs_open (const char       *name,
                       T_FFS_OPEN_FLAGS option);

   T_FFS_RET ffs_close (T_FFS_FD fdi);

   T_FFS_SIZE ffs_write (T_FFS_FD   fdi,
                         void       *src,
                         T_FFS_SIZE amount);

   T_FFS_SIZE ffs_seek (T_FFS_FD     fdi,
                        T_FFS_SIZE   offset,
                        T_FFS_WHENCE whence);

   T_FFS_SIZE ffs_read (T_FFS_FD   fdi,
                        void       *src,
                        T_FFS_SIZE size);

   T_FFS_RET ffs_ftruncate (T_FFS_FD     fdi,
                            T_FFS_OFFSET length); 

#endif
