#
#       Makefile for LCD Riviera driver (ARM compiler) 
#
#       Copyright (c) Texas Instruments 2002
# 
#               Author: Laurent Sollier
#       Date : 17 May 2002
#glowing,2004-07-22, add LCMM_DRV_CHIP, LCMS_DRV_CHIP define

LCM_DRIVERS_FLAGS := -dMODEL=$(MBOARD) -dMAIN_LCD_SIZE=$(LCMMSZ) -dLCDC_DRV_CHIP=$(LCDCCHIP) -dLCMM_DRV_CHIP=$(LCMMCHIP) -dLCMS_DRV_CHIP=$(LCMSCHIP)
LCD_DRV_FLAGS   := -mn -x -mt -o -mw $(LCM_DRIVERS_FLAGS) $(CFLAGSDRIVERS)
LCD_RIV_FLAGS   := -g -mt -mn $(LCM_DRIVERS_FLAGS) $(CFLAGSDRIVERS)
R2D_DRV_FLAGS   := -mf -o2 -mn -x -mt -mw $(LCM_DRIVERS_FLAGS) $(CFLAGSDRIVERS)
R2D_INT_RIV_FLAGS32 :=  -mf -o2 -mn -mw  $(LCM_DRIVERS_FLAGS) $(CFLAGSDRIVERS)
R2D_INT_RIV_FLAGS32_NOOPT :=  -mn -mw $(LCM_DRIVERS_FLAGS) $(CFLAGSDRIVERS)



#Glowing,2004-07-04,add NMODEL to distinguish the product model
#LIB_R2D_INT_RAM := $(OUT_LIB_DIR)/r2d$(NBRD)$(NGOLITE)_int_ram.lib
#LIB_R2D_FLASH := $(OUT_LIB_DIR)/r2d$(NBRD)$(NGOLITE)_flash.lib 
#LIB_CUSTOM_R2D_FLASH := $(OUT_LIB_DIR)/r2d_custom$(NBRD)$(NGOLITE)_flash.lib 
LIB_R2D_INT_RAM := $(OUT_LIB_DIR)/r2d$(NBRD)$(NGOLITE)$(MBRD)_int_ram.lib
LIB_R2D_FLASH := $(OUT_LIB_DIR)/r2d$(NBRD)$(NGOLITE)$(MBRD)_flash.lib 
LIB_CUSTOM_R2D_FLASH := $(OUT_LIB_DIR)/r2d_custom$(NBRD)$(NGOLITE)$(MBRD)_flash.lib 


CFG_LIBS    += $(LIB_R2D_INT_RAM) $(LIB_R2D_FLASH) $(LIB_CUSTOM_R2D_FLASH)


# Source definition


ifdef R2D_SOURCE
SRC_LCD_RIV += $(DRIVERS_DIR)/common/lcd/lcd_functions.c $(DRIVERS_DIR)/common/lcd/lcd_messages.c \
               $(DRIVERS_DIR)/common/lcd/lcd_process.c  $(DRIVERS_DIR)/common/lcd/lcd_gsm_ps_api.c


SRC_R2D_DRIVER += $(DRIVERS_DIR)/common/r2d/r2d.c $(DRIVERS_DIR)/common/r2d/r2d_i.c \
	          $(DRIVERS_DIR)/common/r2d/r2d_inits.c

   SRC_R2D_DRIVER += $(DRIVERS_DIR)/common/r2d/r2d_env.c $(DRIVERS_DIR)/common/r2d/r2d_task.c

SRC_R2D_INT_RAM += $(DRIVERS_DIR)/common/r2d/r2d_refresh.c \
               	   $(DRIVERS_DIR)/common/r2d/r2d_lcd_geometry.c \
                   $(DRIVERS_DIR)/common/r2d/r2d_lcds.c \
                   $(DRIVERS_DIR)/common/r2d/LCDs/r2d_geometry_no2_lcd_i.c \
                   $(DRIVERS_DIR)/common/r2d/r2d_asm_lcds.s \
                   $(DRIVERS_DIR)/common/r2d/r2d_geometry.s \
                   $(DRIVERS_DIR)/common/r2d/r2d_color_color_ia.s
               
SRC_R2D_INT_RAM_NOOPT += $(DRIVERS_DIR)/common/r2d/LCDs/r2d_geometry_no2_lcd_i.c

SRC_LCD_DRV += $(BOARD_DIR)/lcd/uwire.c
endif

SRC_CUSTOM_R2D_DRIVER += $(DRIVERS_DIR)/common/r2d/r2d_fonts.c 


# Object definition
OBJ_R2D_INT_RAM       := $(patsubst %.c,%.obj,$(filter %.c,$(SRC_R2D_INT_RAM)))
OBJ_ASM_R2D_INT_RAM   := $(patsubst %.s,%.obj,$(filter %.s,$(SRC_R2D_INT_RAM)))

CFG_OBJS += $(OBJ_R2D_INT_RAM)
CFG_OBJS += $(OBJ_ASM_R2D_INT_RAM)
CFG_DEPS += $(OBJ_R2D_INT_RAM:.obj=.d)

OBJ_R2D_DRIVER     := $(patsubst %.c,%.obj,$(filter %.c,$(SRC_R2D_DRIVER)))
CFG_OBJS += $(OBJ_R2D_DRIVER)
CFG_DEPS += $(OBJ_R2D_DRIVER:.obj=.d)

OBJ_CUSTOM_R2D_DRIVER     := $(patsubst %.c,%.obj,$(filter %.c,$(SRC_CUSTOM_R2D_DRIVER)))
CFG_OBJS += $(OBJ_CUSTOM_R2D_DRIVER)
CFG_DEPS += $(OBJ_CUSTOM_R2D_DRIVER:.obj=.d)

OBJ_R2D_INT_RAM_NOOPT   := $(patsubst %.c,%.obj,$(filter %.c,$(SRC_R2D_INT_RAM_NOOPT)))

OBJ_LCD_DRV := $(patsubst %.c,%.obj,$(filter %.c,$(SRC_LCD_DRV)))
CFG_OBJS += $(OBJ_LCD_DRV)
CFG_DEPS += $(OBJ_LCD_DRV:.obj=.d)

OBJ_LCD_RIV := $(patsubst %.c,%.obj,$(filter %.c,$(SRC_LCD_RIV)))
CFG_OBJS += $(OBJ_LCD_RIV)
CFG_DEPS += $(OBJ_LCD_RIV:.obj=.d)

######## LIBRARIES #############
$(LIB_CUSTOM_R2D_FLASH) : $(OBJ_CUSTOM_R2D_DRIVER) 
	ar470 r $(LIB_CUSTOM_R2D_FLASH) $(OBJ_CUSTOM_R2D_DRIVER)

$(LIB_R2D_FLASH) : $(OBJ_R2D_DRIVER) $(OBJ_LCD_DRV) $(OBJ_LCD_RIV)
	ar470 r $(LIB_R2D_FLASH) $(OBJ_R2D_DRIVER) $(OBJ_LCD_DRV) $(OBJ_LCD_RIV)

$(LIB_R2D_INT_RAM) : $(OBJ_R2D_INT_RAM) $(OBJ_ASM_R2D_INT_RAM)
	ar470 r $(LIB_R2D_INT_RAM) $(OBJ_R2D_INT_RAM) $(OBJ_ASM_R2D_INT_RAM)

######## OBJECTS #############
$(OBJ_R2D_DRIVER) : %.obj: %.c
	cl470 $(R2D_DRV_FLAGS) -fr$(@D) $<

$(OBJ_CUSTOM_R2D_DRIVER) : %.obj: %.c
	cl470 $(R2D_DRV_FLAGS) -fr$(@D) $<

$(filter-out $(OBJ_R2D_INT_RAM_NOOPT),$(OBJ_R2D_INT_RAM)) : %.obj: %.c
	cl470 $(R2D_INT_RIV_FLAGS32) -fr$(@D) $<

$(OBJ_ASM_R2D_INT_RAM) : %.obj: %.s
	asm470 $(ADEFS) $(ASMFLAGS) $< $@

$(OBJ_R2D_INT_RAM_NOOPT) : %.obj: %.c
	cl470 $(R2D_INT_RIV_FLAGS32_NOOPT) -fr$(@D) $<

$(OBJ_LCD_DRV) : %.obj: %.c
	cl470 $(LCD_DRV_FLAGS) -fr$(@D) $<

$(OBJ_LCD_RIV) : %.obj: %.c
	cl470 $(LCD_RIV_FLAGS) -fr$(@D) $<








