/****************************************************************************
 *
 *		Copyright (C) 2002-2003	YAMAHA CORPORATION. All rights reserved.
 *
 *		Module		: madefs.h
 *
 *		Description : MA Definitions
 *
 *		Version		: 2.0.6 	2003.07.14
 *
 ****************************************************************************/

#ifndef __MADEFS_H__
#define __MADEFS_H__

#define MA_MEMORY_SIZE					(0x6000)	/* 24Kbytes */
#define MA_ROM_SIZE						(0x4000)	/* 16Kbytes */
#define MA_RAM_SIZE						(0x2000)	/*  8Kbytes */

#define MA_ROM_START_ADDRESS			(0x0000)
#define MA_ROM_END_ADDRESS				(0x3FFF)
#define MA_RAM_START_ADDRESS			(0x4000)
#define MA_RAM_END_ADDRESS				(0x5FFF)

#define MA_NORMAL_ROM_ADDRESS			(0x0800)
#define MA_DRUM_ROM_ADDRESS				(0x1000)

/*******************************************************************

	MA-5 Register Definition

	[Naming Rules]

	  MAI_xxx : Intermediate Registers Bank#0
	  MAI3_xx : Intermediate Registers Bank#3
	  MAC_xxx : Control Registers
	  MAB_xxx : Miscelleneous bit definition

********************************************************************/

#define MA_CTRLREG_SIZE					(563)
#define MA_INTERMEDREG_SIZE				(59)


/* Status Flag Reg(WRITE)

        7       6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+-------+
	|  IRQ  |VALID_R|                      REG ID                   |
	+-------+-------+-------+-------+-------+-------+-------+-------+
*/

#define MAB_IRQ_ENABLE					(0x80)

/* Status Flag Reg(READ)

        7       6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+-------+
	| BUSY  |VALID_R|  "0"  |  "0"  |FULL_DW|FULL_W |EMP_DW | EMP_W |
	+-------+-------+-------+-------+-------+-------+-------+-------+
*/

#define MAB_STAT_BUSY					(0x80)
#define MAB_STAT_VALID_R				(0x40)
#define MAB_STAT_FULL_DW				(0x08)
#define MAB_STAT_FULL_W					(0x04)
#define MAB_STAT_EMP_DW					(0x02)
#define MAB_STAT_EMP_W					(0x01)

#define MAI_IRQ_FLAG_1					(0)
/*
        7       6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+-------+
	| FIFO  |  TM#2 |  TM#1 |  TM#0 | SIRQ#1| SIRQ#0| STM#1 | STM#0 |
	+-------+-------+-------+-------+-------+-------+-------+-------+
*/
#define	MAB_IRQFLAG_FIFO				(0x80)
#define	MAB_IRQFLAG_TM2					(0x40)
#define	MAB_IRQFLAG_TM1					(0x20)
#define	MAB_IRQFLAG_TM0					(0x10)
#define	MAB_IRQFLAG_SIRQ1				(0x08)
#define	MAB_IRQFLAG_SIRQ0				(0x04)
#define	MAB_IRQFLAG_STM1				(0x02)
#define	MAB_IRQFLAG_STM0				(0x01)


#define MAI_DELAYED_WRITE				(1)
#define	MAI_READ_BUF					(1)
#define MAI_IMMEDIATE_WRITE				(2)
#define MAI_IMMEDIATE_READ				(3)
#define MAI_BASIC_SETTING				(4)
/*
        7       6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+-------+
	|  RST  |R_FIFO1|R_FIFO0|R_SEQ#1|R_SEQ#0|  "0"  |    B A N K    |
	+-------+-------+-------+-------+-------+-------+-------+-------+
*/
#define	MAB_RST							(0x80)
#define	MAB_RFIFO1						(0x40)
#define	MAB_RFIFO0						(0x20)
#define	MAB_RSEQ1						(0x10)
#define	MAB_RSEQ0						(0x08)

#define MAI_POWER_MANAGEMENT_DIGITAL	(5)
/*
        7       6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+-------+
	|  "0"  |  "0"  |  "0"  |  "0"  |   D1  |   D2  |   D1  |   D0  |
	+-------+-------+-------+-------+-------+-------+-------+-------+
*/
#define MAB_PWM_DP3						(0x08)
#define MAB_PWM_DP2						(0x04)
#define MAB_PWM_DP1						(0x02)
#define MAB_PWM_DP0						(0x01)

#define MAI_POWER_MANAGEMENT_ANALOG		(6)
/*
        7       6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+-------+
	| PLLPD |  "0"  |  AP4R |  AP4L |  AP3  |  AP2  |  AP1  |  AP0  |
	+-------+-------+-------+-------+-------+-------+-------+-------+
*/
#define MAB_PWM_PLLPD					(0x80)
#define MAB_PWM_AP4R					(0x20)
#define MAB_PWM_AP4L					(0x10)
#define MAB_PWM_AP3						(0x08)
#define MAB_PWM_AP2						(0x04)
#define MAB_PWM_AP1						(0x02)
#define MAB_PWM_AP0						(0x01)

#define MAI_ANALOG_EQVOL				(7)
/*
        7       6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+-------+
	|  "0"  |  "0"  |  "0"  |                 EQVOL                 |
	+-------+-------+-------+-------+-------+-------+-------+-------+
*/
#define MAI_ANALOG_HPVOL_L				(8)
/*
        7       6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+-------+
	| MONO  |  "0"  |  "0"  |                HPVOLL                 |
	+-------+-------+-------+-------+-------+-------+-------+-------+
*/
#define MAI_ANALOG_HPVOL_R				(9)
/*
        7       6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+-------+
	|  "0"  |  "0"  |  "0"  |                HPVOLR                 |
	+-------+-------+-------+-------+-------+-------+-------+-------+
*/
#define MAI_ANALOG_SPVOL				(10)
/*
        7       6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+-------+
	| VSEL2 | VSEL1 |  "0"  |                 SPVOL                 |
	+-------+-------+-------+-------+-------+-------+-------+-------+
*/
#define	MAB_VSEL2						(0x80)
#define	MAB_VSEL1						(0x40)

#define MAI_LED_SETTING_1				(11)
/*
        7       6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+-------+
	|  "0"  |  "0"  |  "0"  |  "0"  |  "0"  |  "0"  |  "0"  |  LED  |
	+-------+-------+-------+-------+-------+-------+-------+-------+
*/
#define MAI_LED_SETTING_2				(12)
/*
        7       6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+-------+
	|  "0"  |         FREQ          |  "0"  |         MODE          |
	+-------+-------+-------+-------+-------+-------+-------+-------+
*/
#define MAI_MTR_SETTING_1				(13)
/*
        7       6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+-------+
	|  "0"  |  "0"  |  "0"  |  "0"  |  "0"  |  "0"  |  "0"  |  MTR  |
	+-------+-------+-------+-------+-------+-------+-------+-------+
*/
#define MAI_MTR_SETTING_2				(14)
/*
        7       6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+-------+
	|  "0"  |         FREQ          |  "0"  |         MODE          |
	+-------+-------+-------+-------+-------+-------+-------+-------+
*/
#define	MAI_DIGITAL_OUT_SETTING			(15)
/*
        7       6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+-------+
	|  "0"  |  "0"  |  "0"  | D_BCLK|    D_DACOUT   | D_OEN | D_MODE|
	+-------+-------+-------+-------+-------+-------+-------+-------+
*/
#define	MAB_DDBCLK						(0x10)
#define	MAB_DDACOUT						(0x0C)
#define	MAB_DOEN						(0x02)
#define	MAB_DMODE						(0x01)

#define MAI_IRQ_FLAG_2					(16)
/*
        7       6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+-------+
	|  "0"  |  "0"  |SIRQ#5 |SIRQ#4 |SIRQ#3 |SIRQ#2 |  "0"  |  "0"  |
	+-------+-------+-------+-------+-------+-------+-------+-------+
*/
#define	MAB_IRQFLAG_SIRQ5				(0x20)
#define	MAB_IRQFLAG_SIRQ4				(0x10)
#define	MAB_IRQFLAG_SIRQ3				(0x08)
#define	MAB_IRQFLAG_SIRQ2				(0x04)

#define MAI_IRQ_CONTROL_1				(17)
/*
        7       6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+-------+
	| EFIFO | ETM#2 | ETM#1 | ETM#0 |ESIRQ#1|ESIRQ#0|ESTM#1 |ESTM#0 |
	+-------+-------+-------+-------+-------+-------+-------+-------+
*/
#define	MAB_IRQCTRL_EFIFO				(0x80)
#define	MAB_IRQCTRL_ETM2				(0x40)
#define	MAB_IRQCTRL_ETM1				(0x20)
#define	MAB_IRQCTRL_ETM0				(0x10)
#define	MAB_IRQCTRL_ESIRQ1				(0x08)
#define	MAB_IRQCTRL_ESIRQ0				(0x04)
#define	MAB_IRQCTRL_ESTM1				(0x02)
#define	MAB_IRQCTRL_ESTM0				(0x01)

#define MAI_IRQ_CONTROL_2				(18)
/*
        7       6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+-------+
	|  "0"  |  "0"  |ESIRQ#5|ESIRQ#4|ESIRQ#3|ESIRQ#2|  "0"  |  "0"  |
	+-------+-------+-------+-------+-------+-------+-------+-------+
*/
#define	MAB_IRQCTRL_ESIRQ5				(0x20)
#define	MAB_IRQCTRL_ESIRQ4				(0x10)
#define	MAB_IRQCTRL_ESIRQ3				(0x08)
#define	MAB_IRQCTRL_ESIRQ2				(0x04)

#define	MAI_SOFTINT_RAM_0				(19)
/*
        7       6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+-------+
	|  "0"  |                         RAM#0                         |
	+-------+-------+-------+-------+-------+-------+-------+-------+
*/
#define	MAI_SOFTINT_RAM_1				(20)
/*
        7       6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+-------+
	|  "0"  |                         RAM#1                         |
	+-------+-------+-------+-------+-------+-------+-------+-------+
*/
#define	MAI_SOFTINT_RAM_2				(21)
/*
        7       6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+-------+
	|  "0"  |                         RAM#2                         |
	+-------+-------+-------+-------+-------+-------+-------+-------+
*/
#define	MAI_SOFTINT_RAM_3				(22)
/*
        7       6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+-------+
	|  "0"  |                         RAM#3                         |
	+-------+-------+-------+-------+-------+-------+-------+-------+
*/
#define	MAI_SOFTINT_RAM_4				(23)
/*
        7       6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+-------+
	|  "0"  |                         RAM#4                         |
	+-------+-------+-------+-------+-------+-------+-------+-------+
*/
#define	MAI_SOFTINT_RAM_5				(24)
/*
        7       6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+-------+
	|  "0"  |                         RAM#5                         |
	+-------+-------+-------+-------+-------+-------+-------+-------+
*/

#define	MAI_STREAM_PG_0					(25)
/*
        7       6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+-------+
	|  "0"  |  "0"  |                STREAM PG#0                    |
	+-------+-------+-------+-------+-------+-------+-------+-------+
*/
#define	MAI_STREAM_PG_1					(26)
/*
        7       6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+-------+
	|  "0"  |  "0"  |                STREAM PG#1                    |
	+-------+-------+-------+-------+-------+-------+-------+-------+
*/
#define	MAI_TIMER0_RD_COUNT				(27)
/*
        7       6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+-------+
	|  "0"  |                     TIMER#0 COUNT                     |
	+-------+-------+-------+-------+-------+-------+-------+-------+
*/
#define	MAI_TIMER1_RD_COUNT				(28)
/*
        7       6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+-------+
	|  "0"  |                     TIMER#1 COUNT                     |
	+-------+-------+-------+-------+-------+-------+-------+-------+
*/
#define	MAI_TIMER2_RD_COUNT				(29)
/*
        7       6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+-------+
	|  "0"  |                     TIMER#2 COUNT                     |
	+-------+-------+-------+-------+-------+-------+-------+-------+
*/
#define MAI_PLL_SETTING_1				(30)
/*
        7       6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+-------+
	| CKSEL |  "0"  |  "0"  |                ADJUST1                |
	+-------+-------+-------+-------+-------+-------+-------+-------+
*/
#define MAI_PLL_SETTING_2				(31)
/*
        7       6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+-------+
	|  "0"  |                        ADJUST2                        |
	+-------+-------+-------+-------+-------+-------+-------+-------+
*/
#define	MAI_SEQ_COUNT_H					(32)
/*
        7       6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+-------+
	|  "0"  |  "0"  |  "0"  |        SEQUENCE COUNT (H)             |
	+-------+-------+-------+-------+-------+-------+-------+-------+
*/
#define	MAI_SEQ_COUNT_M					(33)
/*
        7       6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+-------+
	|                        SEQUENCE COUNT (M)                     |
	+-------+-------+-------+-------+-------+-------+-------+-------+
*/
#define	MAI_SEQ_COUNT_L					(34)
/*
        7       6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+-------+
	|                        SEQUENCE COUNT (L)                     |
	+-------+-------+-------+-------+-------+-------+-------+-------+
*/
#define	MAI_SEQ_MSS_H					(35)
/*
        7       6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+-------+
	|                             MS_S (H)                          |
	+-------+-------+-------+-------+-------+-------+-------+-------+
*/
#define	MAI_SEQ_MSS_L					(36)
/*
        7       6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+-------+
	|                   MA_S (L)                    |  "0"  |  "0"  |
	+-------+-------+-------+-------+-------+-------+-------+-------+
*/
#define	MAI_SEQ_IRQPOINT				(37)
/*
        7       6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+-------+
	|  "0"  |  "0"  |  "0"  |  "0"  |  "0"  |       IRQ POINT       |
	+-------+-------+-------+-------+-------+-------+-------+-------+
*/
#define	MAI_SEQ_CTRL					(38)
/*
        7       6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+-------+
	|  "0"  |  "0"     "0"  |  "0"  |  "0"  |  "0"  |  "0"  | START |
	+-------+-------+-------+-------+-------+-------+-------+-------+
*/
#define MAI_MASTER_VOLL					(39)
/*
        7       6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+-------+
	|               MASTER VOLL             |  "0"  |MASTER L SHIFT |
	+-------+-------+-------+-------+-------+-------+-------+-------+
*/
#define MAI_MASTER_VOLR					(40)
/*
        7       6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+-------+
	|               MASTER VOLR             |  "0"  |MASTER R SHIFT |
	+-------+-------+-------+-------+-------+-------+-------+-------+
*/
#define MAI_FM_VOL						(41)
/*
        7       6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+-------+
	|                  FM  VOL              |  "0"  |   FM SHIFT    |
	+-------+-------+-------+-------+-------+-------+-------+-------+
*/
#define MAI_WT_VOL						(42)
/*
        7       6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+-------+
	|                  WT  VOL              |  "0"  |   WT SHIFT    |
	+-------+-------+-------+-------+-------+-------+-------+-------+
*/
#define MAI_STM_VOL						(43)
/*
        7       6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+-------+
	|                 STM  VOL              |  "0"  |   STM SHIFT   |
	+-------+-------+-------+-------+-------+-------+-------+-------+
*/
#define MAI_HV_VOL						(44)
/*
        7       6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+-------+
	|                  HV  VOL              |  "0"  |   HV SHIFT    |
	+-------+-------+-------+-------+-------+-------+-------+-------+
*/
#define MAI_TIMER_MST					(46)
/*
        7       6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+-------+
	|  "0"  |                          MS_T                         |
	+-------+-------+-------+-------+-------+-------+-------+-------+
*/
#define MAI_TIMER0_WR_COUNT				(47)
/*
        7       6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+-------+
	|  "0"  |                        TIMER #0                       |
	+-------+-------+-------+-------+-------+-------+-------+-------+
*/
#define	MAI_TIMER0_CTRL					(48)
/*
        7       6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+-------+
	|  "0"  |  "0"  |  "0"  |  "0"  |  "0"  | TS#0  | ONE#0 | TMST#0|
	+-------+-------+-------+-------+-------+-------+-------+-------+
*/
#define MAI_TIMER1_WR_COUNT				(49)
/*
        7       6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+-------+
	|  "0"  |                        TIMER #1                       |
	+-------+-------+-------+-------+-------+-------+-------+-------+
*/
#define	MAI_TIMER1_CTRL					(50)
/*
        7       6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+-------+
	|  "0"  |  "0"  |  "0"  |  "0"  |  "0"  | TS#1  | ONE#1 | TMST#1|
	+-------+-------+-------+-------+-------+-------+-------+-------+
*/
#define MAI_TIMER2_WR_COUNT				(51)
/*
        7       6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+-------+
	|  "0"  |                        TIMER #2                       |
	+-------+-------+-------+-------+-------+-------+-------+-------+
*/
#define	MAI_TIMER2_CTRL					(52)
/*
        7       6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+-------+
	|  "0"  |  "0"  |  "0"  |  "0"  |  "0"  | TS#2  | ONE#2 |  "0"  |
	+-------+-------+-------+-------+-------+-------+-------+-------+
*/
#define	MAI_TIMER2_TMSTA				(53)
/*
        7       6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+-------+
	|  "0"  |  "0"  |  "0"  |  "0"  |  "0"  |  "0"  |  "0"  |TMST#2A|
	+-------+-------+-------+-------+-------+-------+-------+-------+
*/
#define	MAI_TIMER2_TMSTB				(54)
/*
        7       6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+-------+
	|  "0"  |  "0"  |  "0"  |  "0"  |  "0"  |  "0"  |  "0"  |TMST#2B|
	+-------+-------+-------+-------+-------+-------+-------+-------+
*/
#define	MAI_EXT_CTRL					(55)
/*
        7       6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+-------+
	|     SV_MUTE   |SV_CHV |SV_PAN | MVSEL | NOP2E | FINT  | DADJT |
	+-------+-------+-------+-------+-------+-------+-------+-------+
*/
#define	MAB_EXTCTRL_SVMUTE				(0xC0)
#define	MAB_EXTCTRL_SVCHV				(0x20)
#define	MAB_EXTCTRL_SVPAN				(0x10)
#define	MAB_EXTCTRL_MVSEL				(0x08)
#define	MAB_EXTCTRL_NOP2E				(0x04)
#define	MAB_EXTCTRL_FINT				(0x02)
#define	MAB_EXTCTRL_DADJT				(0x01)


#define	MAI_MODE						(56)
/*
        7       6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+-------+
	|  "0"  |  "0"  |  "0"  |  "0"  |  "0"  |HVMODE |STMMODE|FMMODE |
	+-------+-------+-------+-------+-------+-------+-------+-------+
*/
#define	MAB_HVMODE						(0x04)
#define	MAB_STMMODE						(0x02)
#define	MAB_FMMODE						(0x01)


#define	MAI_STM_IRQ_CTRL				(57)
/*
        7       6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+-------+
	|  "0"  |  "0"  |      SIZE     |  "0"  |  "0"  |  IRQ POINT    |
	+-------+-------+-------+-------+-------+-------+-------+-------+
*/
#define	MAI_STM_CTRL					(58)
/*
        7       6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+-------+
	|  "0"  |  "0"  |  "0"  |STEREO |  "0"  |  "0"  |R_STM#1|R_STM#0|
	+-------+-------+-------+-------+-------+-------+-------+-------+
*/
#define	MAB_STEREO						(0x10)
#define	MAB_RSTM_0						(0x01)
#define	MAB_RSTM_1						(0x02)

/*=====  Bank #3  =====*/

#define	MAI3_POWER_MANAGEMENT_ANALOG	(17)
/*
        7       6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+-------+
	|  "0"  |  "0"  |  "0"  |  AP9  |  AP8  |  AP7  |  AP6  |  AP5  |
	+-------+-------+-------+-------+-------+-------+-------+-------+
*/
#define	MAB_PWM_AP9						(0x10)
#define	MAB_PWM_AP8						(0x08)
#define	MAB_PWM_AP7						(0x04)
#define	MAB_PWM_AP6						(0x02)
#define	MAB_PWM_AP5						(0x01)

#define	MAI3_ANALOG_STATUS				(37)
/*
        7       6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+-------+
	|  "0"  |  "0"  |  "0"  |EX_STBY| EX_RDY|HP_STBY| HP_RDY|VREF_RDY|
	+-------+-------+-------+-------+-------+-------+-------+-------+
*/
#define	MAB_EX_STBY						(0x10)
#define	MAB_EX_RDY						(0x08)
#define	MAB_HP_STBY						(0x04)
#define	MAB_HP_RDY						(0x02)
#define	MAB_VREF_RDY					(0x01)


/*=====================  Control Registers  ======================*/

/* FM Voice#0 - FM Voice#15 */
#define MAC_FM_VOICE_ADDRESS				(0)

/* FM Voice#16 - FM Voice#31 */
#define MAC_EXT_FM_VOICE_ADDRESS			(272)

/*      6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+
	|                      Voice Adr[14:8]                  |
	+-------+-------+-------+-------+-------+-------+-------+
	|                      Voice Adr[7:1]                   |
	+-------+-------+-------+-------+-------+-------+-------+
	|                  VOVOL                | MTRON | LEDON |
	+-------+-------+-------+-------+-------+-------+-------+
	|  EXCH |         BLOCK         |       FNUM(H)         |
	+-------+-------+-------+-------+-------+-------+-------+
	|                       FNUM(L)                         |
	+-------+-------+-------+-------+-------+-------+-------+
	| KeyOn | MUTE  |  RST  |               #CH             |
	+-------+-------+-------+-------+-------+-------+-------+
*/

/* WT Voice#0 - WT Voice#15 */
#define MAC_WT_VOICE_ADDRESS				(96)

/* WT Voice#16 - WT Voice#31 */
#define MAC_EXT_WT_VOICE_ADDRESS			(368)

/*      6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+
	|                      Voice Adr[14:8]                  |
	+-------+-------+-------+-------+-------+-------+-------+
	|                      Voice Adr[7:1]                   |
	+-------+-------+-------+-------+-------+-------+-------+
	|                  VOVOL                | MTRON | LEDON |
	+-------+-------+-------+-------+-------+-------+-------+
	|  EXCH |         BLOCK         |       FNUM(H)         |
	+-------+-------+-------+-------+-------+-------+-------+
	|                       FNUM(L)                         |
	+-------+-------+-------+-------+-------+-------+-------+
	| KeyOn | MUTE  |  RST  |               #CH             |
	+-------+-------+-------+-------+-------+-------+-------+
*/

/* WT Voice#27 - WT Woice#31 */
#define MAC_EXT_WT_VOICE_ADDRESS_27			(434)

#define MAC_CHANNEL_VOLUME				(192)

/*      6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+
	|                CHVOL#0                |  "0"  | DRCT  |
	+-------+-------+-------+-------+-------+-------+-------+
	|                CHVOL#1                |  "0"  | DRCT  |
	+-------+-------+-------+-------+-------+-------+-------+
                                :
                                :
*/

#define MAC_CHANNEL_PANPOT				(208)

/*      6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+
	|                PANPOT#0               |  "0"  | DRCT  |
	+-------+-------+-------+-------+-------+-------+-------+
	|                PANPOT#1               |  "0"  | DRCT  |
	+-------+-------+-------+-------+-------+-------+-------+
                                :
                                :
*/

#define MAC_CHANNEL_SUSTAIN				(224)
#define MAC_CHANNEL_VIBRATO				(224)

/*      6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+
	|EVIB#0 |  "0"  |ESUS#0 |  "0"  |        XVB#0          |
	+-------+-------+-------+-------+-------+-------+-------+
	|EVIB#1 |  "0"  |ESUS#1 |  "0"  |        XVB#1          |
	+-------+-------+-------+-------+-------+-------+-------+
                                :
                                :
*/

#define MAC_CHANNEL_BEND				(240)

/*      6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+
	|  "0"  |  "0"  |     INT #0    |      FRAC(H) #0       |
	+-------+-------+-------+-------+-------+-------+-------+
	|                     FRAC(L) #0                |  "0"  |
	+-------+-------+-------+-------+-------+-------+-------+
	|  "0"  |  "0"  |     INT #1    |      FRAC(H) #1       |
	+-------+-------+-------+-------+-------+-------+-------+
	|                     FRAC(L) #1                |  "0"  |
	+-------+-------+-------+-------+-------+-------+-------+
                                :
                                :
*/
#define MAC_FM_EXTWAVE_15				(464)

/*      6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+
	|                      FM WAVE#15[14:8]                 |
	+-------+-------+-------+-------+-------+-------+-------+
	|                      FM WAVE#15[7:1]                  |
	+-------+-------+-------+-------+-------+-------+-------+
*/
#define MAC_FM_EXTWAVE_23				(466)

/*      6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+
	|                      FM WAVE#23[14:8]                 |
	+-------+-------+-------+-------+-------+-------+-------+
	|                      FM WAVE#23[7:1]                  |
	+-------+-------+-------+-------+-------+-------+-------+
*/
#define MAC_FM_EXTWAVE_31				(468)

/*      6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+
	|                      FM WAVE#31[14:8]                 |
	+-------+-------+-------+-------+-------+-------+-------+
	|                      FM WAVE#31[7:1]                  |
	+-------+-------+-------+-------+-------+-------+-------+
*/

#define	MAC_SOFTINT_RAM					(470)

#define	MAC_SOFTINT_RAM_0				(470)
#define	MAC_SOFTINT_RAM_1				(471)
#define	MAC_SOFTINT_RAM_2				(472)
#define	MAC_SOFTINT_RAM_3				(473)
#define	MAC_SOFTINT_RAM_4				(474)
#define	MAC_SOFTINT_RAM_5				(475)

/*      6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+
	|                        RAM #0                         |
	+-------+-------+-------+-------+-------+-------+-------+
	|                        RAM #1                         |
	+-------+-------+-------+-------+-------+-------+-------+
                                :
                                :
*/

#define	MAC_LED_CTRL					(476)

/*      6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+
	|  "0"  |  "0"  |  "0"  |  "0"  |  "0"  |  "0"  |  LED  |
	+-------+-------+-------+-------+-------+-------+-------+
*/
#define	MAC_MTR_CTRL					(477)

/*      6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+
	|  "0"  |  "0"  |  "0"  |  "0"  |  "0"  |  "0"  |  MTR  |
	+-------+-------+-------+-------+-------+-------+-------+
*/

#define	MAC_NOP2						(478)
#define	MAC_NOP							(479)

#define	MAC_SEQ_STOP					(480)

/*      6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+
	|  "0"  |  "0"  |  "0"  |  "0"  |  "0"  |  "0"  |SEQSTOP|
	+-------+-------+-------+-------+-------+-------+-------+
*/

#define	MAC_ID							(482)

/*      6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+
	|  "0"  |  "1"  |  "0"  |  "0"  |  "0"  |  "0"  |  "1"  |
	+-------+-------+-------+-------+-------+-------+-------+
*/

#define	MAC_EXT_CH_VOL					(483)

/*      6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+
	|                  ExCHVOL              |  "0"  | DRCT  |
	+-------+-------+-------+-------+-------+-------+-------+
*/
#define	MAC_EXT_CH_PAN					(484)

/*      6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+
	|                  ExPANPOT             |  "0"  |  "0"  |
	+-------+-------+-------+-------+-------+-------+-------+
*/
#define	MAC_EXT_CH_SUS					(485)
#define	MAC_EXT_CH_VIBRATO				(485)

/*      6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+
	|  "0"  |  "0"  |ExESUS |  "0"  |        ExXVB          |
	+-------+-------+-------+-------+-------+-------+-------+
*/
#define MAC_EXT_CH_BEND					(486)

/*      6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+
    |  "0"  |  "0"  |     ExINT     |      ExFRAC(H)        |
	+-------+-------+-------+-------+-------+-------+-------+
	|                     ExFRAC(L)                 |  "0"  |
	+-------+-------+-------+-------+-------+-------+-------+
*/


#define	MAC_HV_BASE						(500)

#define	MAC_HV_WS_0						(500)
#define	MAC_HV_WS_1						(501)
#define	MAC_HV_WS_2						(502)
#define	MAC_HV_WS_3						(503)
#define	MAC_HV_WS_4						(504)
#define	MAC_HV_WS_5						(505)
#define	MAC_HV_WS_6						(506)
#define	MAC_HV_WS_7						(507)
/*
        6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+
	|  "0"  |  PAI  |                 WS#0                  |
	+-------+-------+-------+-------+-------+-------+-------+
	|  "0"  |  PAI  |                 WS#1                  |
	+-------+-------+-------+-------+-------+-------+-------+
	                            :
	                            :
*/

#define	MAC_HV_FORMANT_0				(508)
#define	MAC_HV_FORMANT_1				(511)
#define	MAC_HV_FORMANT_2				(514)
#define	MAC_HV_FORMANT_3				(517)
#define	MAC_HV_FORMANT_4				(520)
#define	MAC_HV_FORMANT_5				(523)
#define	MAC_HV_FORMANT_6				(526)
#define	MAC_HV_FORMANT_7				(529)
/*
        6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+
	|                       FMT Level                       |
	+-------+-------+-------+-------+-------+-------+-------+
	|  "0"  |        BLOCK          |       FNUM(H)         |
	+-------+-------+-------+-------+-------+-------+-------+
	|                        FNUL(L)                        |
	+-------+-------+-------+-------+-------+-------+-------+
	                            :
	                            :
*/

#define	MAC_HV_LED_MTR					(532)
/*
        6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+
	|                  "0"                  | MTRON | LEDON |
	+-------+-------+-------+-------+-------+-------+-------+
*/
#define	MAC_HV_PITCH					(533)
/*
        6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+
	| EXCH  |        BLOCK P        |       FNUM P(H)       |
	+-------+-------+-------+-------+-------+-------+-------+
	|                       FNUM P(L)                       |
	+-------+-------+-------+-------+-------+-------+-------+
*/
#define	MAC_HV_CTRL						(536)
/*
        6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+
	| KeyOn | MUTE  |  RST  |              #CH              |
	+-------+-------+-------+-------+-------+-------+-------+
*/


#define	MAC_LPF_BASE					(537)

#define	MAC_LPF_Q						(538)
/*
        6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+
	|  "0"  |  "0"  |                   Q                   |
	+-------+-------+-------+-------+-------+-------+-------+
*/
#define	MAC_LPF_FC3						(545)
/*
        6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+
	|  "0"  |                    Fc3(H)                     |
	+-------+-------+-------+-------+-------+-------+-------+
	|                         Fc3(L)                        |
	+-------+-------+-------+-------+-------+-------+-------+
*/
#define	MAC_LPF_FCOFFSET				(562)
/*
        6       5       4       3       2       1       0
	+-------+-------+-------+-------+-------+-------+-------+
	|LFORST |                  FcOFFSET(H)                  |
	+-------+-------+-------+-------+-------+-------+-------+
	|                      FcOFFSET(L)                      |
	+-------+-------+-------+-------+-------+-------+-------+
*/
#define	MAC_LPF_FIX0					(554)

/*********************************************************************/

#define MA_MAX_CHANNEL					(16)
#define MA_MAX_FM_SLOT					(32)
#define MA_MAX_WT_SLOT					(32)
#define MA_MAX_STREAM_AUDIO				(2)
#define MA_MAX_RAM_ADDRESS				(0x6000)

#define MA_MAX_REG_STREAM_AUDIO			(32)

#define MA_MAX_PG						(1024 - 32)
#define MA_MIN_PG						(0)
#define MA_WAVE_SIZE					(1024 - 32)

#define MA_MIN_ROM_DRUM					(24)
#define MA_MAX_ROM_DRUM					(84)

#define MA_EVENT_INT					(2)
#define MA_SEQEVENT_INT					(3)

#define MA_MAX_ROM_WAVE					(7)

#define MA_RAM_BLOCK_SIZE				(1024)

#define MA_MIN_LENGTH					(20)

#define MA_DIRECT_FIFOSIZE				(64)

#define MA_RESET_RETRYCOUNT				(10)

/* */

#define MASMW_SUCCESS					(0)		/* success 								*/
#define MASMW_ERROR						(-1)	/* error								*/
#define MASMW_ERROR_ARGUMENT			(-2)	/* error of arguments					*/
#define MASMW_ERROR_RESOURCE_OVER		(-3)	/* over specified resources				*/
#define MASMW_ERROR_ID					(-4)	/* error id number 						*/
#define MASMW_ERROR_TIMEOUT				(-5)	/* timeout		 						*/
#define MASMW_ERROR_SOFTRESET			(-6)	/* error of soft reset for MA-5			*/

#define MASMW_ERROR_FILE				(-16)	/* file error							*/
#define MASMW_ERROR_CONTENTS_CLASS		(-17)	/* SMAF Contents Class shows can't play */
#define MASMW_ERROR_CONTENTS_TYPE		(-18)	/* SMAF Contents Type shows can't play	*/
#define MASMW_ERROR_CHUNK_SIZE			(-19)	/* illegal SMAF Chunk Size value		*/
#define MASMW_ERROR_CHUNK				(-20)	/* illegal SMAF Trachk Chunk value		*/
#define MASMW_ERROR_UNMATCHED_TAG		(-21)	/* unmathced specified TAG 				*/
#define MASMW_ERROR_SHORT_LENGTH		(-22)	/* short sequence 						*/
#define MASMW_ERROR_LONG_LENGTH			(-23)	/* long sequence 						*/
#define MASMW_ERROR_UNSUPPORTED			(-24)	/* unsupported format					*/
#define MASMW_ERROR_NO_INFORMATION		(-25)	/* no specified information				*/
#define MASMW_ERROR_HV_CONFLICT			(-26)	/* conflict about HV resource			*/

#define MASMW_ERROR_SMF_FORMAT			(-50)	/* invalid format type != 0/1			*/
#define MASMW_ERROR_SMF_TRACKNUM		(-51)	/* invalid number of tracks				*/
#define MASMW_ERROR_SMF_TIMEUNIT		(-52)	/* invalid time unit					*/
#define MASMW_ERROR_SMF_CMD				(-53)	/* invalid command byte					*/



#define MASMW_NUM_SRMCNV				(13)
#define MASMW_NUM_FILE					(5+1)
#define MASMW_NUM_SEQTYPE				(3)		/* number of sequence type */

#define MASMW_STATE_IDLE				(0)		/* idle state */
#define MASMW_STATE_LOADED				(1)		/* loaded state */
#define MASMW_STATE_OPENED				(2)		/* opened state */
#define MASMW_STATE_READY				(3)		/* ready state */
#define MASMW_STATE_PLAYING				(4)		/* playing state */
#define	MASMW_STATE_PAUSE				(5)		/* pause state */

#define MASMW_REPEAT					(126)
#define MASMW_END_OF_SEQUENCE			(127)
#define MASMW_END_OF_DATA				(128)

/* Sound Sequencer: Control */
#define MASMW_SET_VOLUME				(0)		/* set volume */
#define MASMW_SET_SPEED					(1)		/* set speed */
#define MASMW_SET_KEYCONTROL			(2)		/* set key control */
#define MASMW_GET_TIMEERROR				(3)		/* get time error */
#define MASMW_GET_POSITION				(4)		/* get position */
#define MASMW_GET_LENGTH				(5)		/* get length */
#define MASMW_GET_STATE					(6)		/* get state */
#define MASMW_SEND_MIDIMSG				(7)		/* send midi message */
#define MASMW_SEND_SYSEXMIDIMSG			(8)		/* send sys.ex. midi message */
#define MASMW_SET_BIND					(9)		/* set bind */
#define MASMW_GET_CONTENTSDATA			(10)	/* get contents data */
#define MASMW_GET_PHRASELIST			(11)	/* get phrase list */
#define MASMW_SET_STARTPOINT			(12)	/* set start point */
#define MASMW_SET_ENDPOINT				(13)	/* set end point */
#define MASMW_SET_PANPOT				(14)	/* set panpot */
#define MASMW_GET_LEDSTATUS				(15)	/* get LED status */
#define MASMW_GET_VIBSTATUS				(16)	/* get VIB status */
#define MASMW_SET_EVENTNOTE				(17)	/* set event note */
#define MASMW_GET_CONVERTTIME			(18)	/* get convert time */
#define MASMW_GET_LOADINFO				(19)	/* get load information */
#define MASMW_SET_LOADINFO				(20)	/* set load information */
#define MASMW_SET_HVDICTIONARY			(21)	/* set TTS dictionary for HV */
#define MASMW_SET_HVVOICE				(22)	/* set voice parameter for HV */
#define MASMW_SET_HVPROGRAMNO			(23)	/* set program no. for HV */
#define MASMW_RESET_HVWAIT				(24)	/* reset HVwait */
#define MASMW_GET_HVCONVERTSIZE			(25)	/* get position of HV-Script */
#define MASMW_SET_SPEEDWIDE				(26)	/* set speed (wide range) */
#define MASMW_SET_REPEAT				(27)	/* set play times */
#define MASMW_CHECK_HVDICTIONARY		(28)	/* check TTS dictionary for HV */
#define MASMW_GET_CONTROL_VAL			(29)	/* set control value  */
#define MASMW_SET_CB_INTERVAL			(30)	/* set calback function interval */
#define MASMW_SET_HVVOLUME				(31)	/* set volume for HV-Script */
#define MASMW_GET_HVCHANNEL				(32)	/* get channel for HV-Script */
#define MASMW_GET_HVVOICE				(33)	/* get HV voice parameter */
#define MASMW_GET_HVSCRIPT				(34)	/* get HV-Script */
#define MASMW_GET_HVCHANNELSTATUS		(35)	/* get status of HV channel */
#define MASMW_SET_HVCHANNEL				(36)	/* set channel for HV-Script */
#define MASMW_SET_HVCHANNELSTATUS		(37)	/* set status of HV channel */
#define MASMW_SET_HVLANGUAGE			(38)	/* set hv language type */

/* Sound Sequencer: Device Control */

#define MASMW_PWM_DIGITAL				(0)		/* power management (digital) */
#define MASMW_PWM_ANALOG				(1)		/* power management (analog) */
#define MASMW_EQ_VOLUME					(2)		/* eq volume */
#define MASMW_HP_VOLUME					(3)		/* hp volume */
#define MASMW_SP_VOLUME					(4)		/* sp volume */
#define MASMW_LED_MASTER				(5)		/* LED master select */
#define MASMW_LED_BLINK					(6)		/* LED blink setting */
#define MASMW_LED_DIRECT				(7)		/* LED direct control */
#define MASMW_MTR_MASTER				(8)		/* MOTOR master select */
#define MASMW_MTR_BLINK					(9)		/* MOTOR blink setting */
#define MASMW_MTR_DIRECT				(10)	/* MOTOR direct control */
#define MASMW_GET_PLLOUT				(11)	/* get PLL out */
#define MASMW_HW_INITIALIZE				(13)	/* initialize HW */
#define MASMW_WRITE_IMREG				(17)	/* write into intermediate registers */
#define MASMW_READ_IMREG				(18)	/* read intermediate registers */
#define MASMW_GET_DEVICETYPE			(20)	/* get device type */
#define MASMW_SET_HVTYPE				(21)	/* set hv language type */

#define MASMW_MAX_LED_MASTER			(2)
#define MASMW_MAX_LED_BLINK				(5)
#define MASMW_MAX_LED_DIRECT			(1)

#define MASMW_MAX_MTR_MASTER			(2)
#define MASMW_MAX_MTR_BLINK				(5)
#define MASMW_MAX_MTR_DIRECT			(1)

/* device type */
#define MASMW_DEVICETYPE_MA5			(1)
#define MASMW_DEVICETYPE_MA5S			(2)
#define MASMW_DEVICETYPE_MA5I			(3)

/* HV-Script type */
#define MASMW_HVTYPE_JAPANESE			(0)
#define MASMW_HVTYPE_KOREAN				(1)
#define MASMW_HVTYPE_NONE				(3)
/* default HV language type */
#define MASMW_DEFAULT_HVTYPE			MASMW_HVTYPE_NONE

#endif /*__MADEFS_H__*/
