/****************************************************************************
 *
 *		Copyright 2002-2003	YAMAHA CORPORATION. All rights reserved.
 *
 *		Module		: madevdrv.c
 *
 *		Description	: MA-5 Device Driver
 *
 *
 *		Version		: 2.1.7 	2003.6.23
 *
 *
 ****************************************************************************/
#include "madevdrv.h"

#define MADEVDRV_WAIT_TIMEOUT			(10000)  /* ms */
#define	ENABLE_IRQ						0
#define	DISABLE_IRQ						1

#define	MADEVDRV_STREAM_STATE_IDLE		0
#define	MADEVDRV_STREAM_STATE_STANDBY	1
#define	MADEVDRV_STREAM_STATE_PLAYING	2
#define	MADEVDRV_STREAM_STATE_ENDOFDATA	3

#define	MADEVDRV_ACTIVE_DELAYEDSEQ		1
#define	MADEVDRV_ACTIVE_PHRASESEQ		2
#define	MADEVDRV_ACTIVE_HVSEQ			4


/* Struct Definitions */

typedef struct _MA_SBUF_INFO
{
	UINT32	dWriteNum;
	UINT32	dReadNum;
	UINT32	dBufTotal;
	UINT32	dBufPtr;
	UINT32	dBufSize[MA_SBUF_NUM];
} MA_SBUF_INFO;

typedef struct _MA_STREAM_AUDIO_INFO
{
	UINT8	bState;
	UINT8	bCh;
	UINT32	dFormat;
	UINT8 *	pbWaveData;
	UINT32	dWaveSize;
	UINT32	dEndPoint;
	UINT32	dPrvPoint;
} MA_STREAM_AUDIO_INFO, *PMA_STREAM_AUDIO_INFO;

typedef struct _MADEVDRVINFO
{
	MA_SBUF_INFO	SbufInfo;
	MA_STREAM_AUDIO_INFO	StreamInfo[MA_MAX_STREAM_AUDIO];

	UINT8	bSbufBuffer[MA_SBUF_NUM][MA_FIFO_SIZE];
	void (* pIntFunc[12])(UINT32 dCtrl);

	UINT8	bStatusReg;
	UINT8	bIrqProc;
	UINT8	bMaskInterrupt;
	UINT8	bSlave;
	UINT32	dSeqId;
	UINT32	dEndofSequence;
	UINT32	dActiveSequencer;
	UINT32	dPendingIrq;
	UINT32	dStreamEoInfo;
} MADEVDRVINFO, *PMADEVDRVINFO;


MADEVDRVINFO gMaDevDrvInfo;
static PMADEVDRVINFO pgMaDevDrvInfo = &gMaDevDrvInfo;

extern void 	machdep_ErrHandler				( SINT32 sdError );
extern void 	machdep_Wait					( UINT32 dWaitTime );
extern void	machdep_Sleep					( UINT32 dSleepTime );
extern SINT32	machdep_WaitValidData			( UINT8 bFlag );
extern void 	machdep_WriteStatusFlagReg		( UINT8 bData );
extern UINT8	machdep_ReadStatusFlagReg		( void );
extern void	machdep_WriteDataReg			( UINT8 bData );
extern void	machdep_WriteDataRegN			( UINT8* pbData, UINT32 dSize );
extern void	machdep_WriteDataRegValN		( UINT8 bVal, UINT32 dSize );
extern UINT8	machdep_ReadDataReg				( void );
extern SINT32	machdep_CheckDelayedFifoEmpty	( void );
extern SINT32	machdep_WaitDelayedFifoEmpty 	( void );
extern SINT32	machdep_WaitImmediateFifoEmpty	( void );
extern void 	machdep_GenerateIrqProcTask		( UINT32 dIrqFlag, UINT8 bOldSetting );
extern SINT32	machdep_PowerManagement			( UINT8 bMode );
extern SINT32	machdep_WaitSequencerStop		( void );

extern UINT32	gdPrevAccessType;

static void		dummy_IntFunc( UINT32 dCtrl );
static SINT32	StreamSetup( UINT32 dSaId );
static SINT32	StreamUpdate( UINT32 dSaId );
static SINT32	ControlInterrupt( UINT32 dIrqCtrlReg, UINT32 dCtrl, UINT8 bIntCtrl );
static void		SoftInt0( UINT32 dCtrl );
static void		SoftInt1( UINT32 dCtrl );
static void		SoftInt2( UINT32 dCtrl );
static void		SoftInt3( UINT32 dCtrl );
static void		SoftInt4( UINT32 dCtrl );
static void		StreamPg0( UINT32 dCtrl );
static void		StreamPg1( UINT32 dCtrl );
static void		Timer2( UINT32 dCtrl );
static void		InitRegisters( void );
static UINT8	WriteStatusFlagReg( UINT8 bReg );
static SINT32	WriteStereoAdpcmZeroVal( UINT32 dAddress, UINT32 dSize );

#include "masnddrv.h"
#include "masndseq.h"
#include "maresmgr.h"

#ifdef COLORLED_SUPPORT
#define MADEVDRV_LED_PATTERN_NUM 8
static const UINT8 abLedPattern[MADEVDRV_LED_PATTERN_NUM] =
					{ 1, 2, 3, 4, 5, 6, 7, 0 };
static UINT32 dwLedCount = 0;
#endif

/****************************************************************************
	 dummy Description
 ****************************************************************************/
static void dummy_IntFunc( UINT32 dCtrl )
{
	(void)dCtrl;						/* for unused warning message */

	MADEVDRV_DBGMSG((" dummy_IntFunc: bCtrl=%d\n", dCtrl));
}
/****************************************************************************
 *	MaDevDrv_Initialize
 *
 *	Description:
 *			Initialize the MA Device Driver module.
 *	Arguments:
 *			None
 *	Return:
 *			0		success
 *			-1		error
 *
 ****************************************************************************/
SINT32 MaDevDrv_Initialize( void )
{
	UINT32	i;							/* loop counter */

	MADEVDRV_DBGMSG(("MaDevDrv_Initialize\n"));

	pgMaDevDrvInfo->SbufInfo.dWriteNum = 0;
	pgMaDevDrvInfo->SbufInfo.dReadNum  = 0;
	pgMaDevDrvInfo->SbufInfo.dBufTotal = 0;
	pgMaDevDrvInfo->SbufInfo.dBufPtr   = 0;

	for ( i = 0; i < MA_SBUF_NUM; i++ )
	{
		pgMaDevDrvInfo->SbufInfo.dBufSize[i] = 0;
	}

	/* Initialize for Stream Audio */
	for ( i = 0; i < MA_MAX_STREAM_AUDIO; i++ )
	{
		pgMaDevDrvInfo->StreamInfo[i].bState 		= MADEVDRV_STREAM_STATE_IDLE;
		pgMaDevDrvInfo->StreamInfo[i].bCh			= 0;
		pgMaDevDrvInfo->StreamInfo[i].dFormat		= 0;
		pgMaDevDrvInfo->StreamInfo[i].pbWaveData	= NULL;
		pgMaDevDrvInfo->StreamInfo[i].dWaveSize	= 0;
		pgMaDevDrvInfo->StreamInfo[i].dEndPoint 		= 0;
		pgMaDevDrvInfo->StreamInfo[i].dPrvPoint		= 0;
	}

	pgMaDevDrvInfo->bStatusReg		= 0x80;
	pgMaDevDrvInfo->bIrqProc		= 0;
	pgMaDevDrvInfo->bMaskInterrupt	= 0;
	pgMaDevDrvInfo->bSlave			= 0xff;
	pgMaDevDrvInfo->dSeqId			= 0;
	pgMaDevDrvInfo->dEndofSequence	= 0;
	pgMaDevDrvInfo->dActiveSequencer= 0;
	pgMaDevDrvInfo->dPendingIrq		= 0;
	pgMaDevDrvInfo->dStreamEoInfo	= 0;

	/* register IRQ functions */
	pgMaDevDrvInfo->pIntFunc[0]  = StreamPg0;			/* StreamPG #0 */
	pgMaDevDrvInfo->pIntFunc[1]  = StreamPg1;			/* StreamPG #1 */
	pgMaDevDrvInfo->pIntFunc[2]  = SoftInt0;			/* Soft Int #0 */
	pgMaDevDrvInfo->pIntFunc[3]  = SoftInt1;			/* Soft Int #1 */
	pgMaDevDrvInfo->pIntFunc[4]  = dummy_IntFunc;		/* Timer #0(not used) */
	pgMaDevDrvInfo->pIntFunc[5]  = dummy_IntFunc;		/* Timer #1(not used) */
	pgMaDevDrvInfo->pIntFunc[6]  = Timer2;				/* Timer #2 */
	pgMaDevDrvInfo->pIntFunc[7]  = MaDevDrv_Fifo;		/* FIFO */
	pgMaDevDrvInfo->pIntFunc[8]  = SoftInt2;			/* Soft Int #2 */
	pgMaDevDrvInfo->pIntFunc[9]  = SoftInt3;			/* Soft Int #3 */
	pgMaDevDrvInfo->pIntFunc[10] = SoftInt4;			/* Soft Int #4 */
	pgMaDevDrvInfo->pIntFunc[11] = dummy_IntFunc;		/* Soft Int #5 */
	
	/* Initialize register access flag */
	gdPrevAccessType = MADEVDRV_ACCESS_WRITE;

	return MASMW_SUCCESS;
}
/****************************************************************************
 *	MaDevDrv_Terminate
 *
 *	Description:
 *			Terminate the MA Device Driver module.
 *	Arguments:
 *			None
 *	Return:
 *			0		success
 *			-1		error
 *
 ****************************************************************************/
SINT32 MaDevDrv_Terminate( void )
{
	MADEVDRV_DBGMSG(("MaDevDrv_Terminate\n"));

	return MASMW_SUCCESS;
}
/****************************************************************************
 *	MaDevDrv_HardwareInitialize
 *
 *	Description:
 *			Initialize hardware.
 *	Arguments:
 *			None
 *	Return:
 *			0		success
 *			-1		error
 *
 ****************************************************************************/
SINT32 MaDevDrv_HardwareInitialize( void )
{
	SINT32	sdResult;					/* result of function */

	MADEVDRV_DBGMSG(("MaDevDrv_HardwareInitialize\n"));

	gdPrevAccessType = MADEVDRV_ACCESS_WRITE;

	/* Initialize the uninitialized registers by software reset */
	InitRegisters();

	/* Set the PLL. */
	MaDevDrv_WriteIntermediateReg( MAI_PLL_SETTING_1, MA_ADJUST1_VALUE );
	MaDevDrv_WriteIntermediateReg( MAI_PLL_SETTING_2, MA_ADJUST2_VALUE );

	/* Disable power down mode. */
	sdResult = machdep_PowerManagement( 0 );

	/* error  */
	if ( sdResult != MASMW_SUCCESS )
	{
		return sdResult;
	}

	MaDevDrv_WriteIntermediateReg( MAI_IRQ_CONTROL_1, 
					MAB_IRQCTRL_ETM2 | MAB_IRQCTRL_ESIRQ1 | MAB_IRQCTRL_ESIRQ0  );

	MaDevDrv_WriteIntermediateReg( MAI_IRQ_CONTROL_2, MAB_IRQCTRL_ESIRQ4 | MAB_IRQCTRL_ESIRQ3 | MAB_IRQCTRL_ESIRQ2  );

	/* set stream buffer size and irq point */
	/* buff_size=1024, irq_point=1/4 */
	MaDevDrv_WriteIntermediateReg( MAI_STM_IRQ_CTRL, 0x10 );

	return MASMW_SUCCESS;
}
/****************************************************************************
 *	MaDevDrv_PowerManagement
 *
 *	Description:
 *			Power management.
 *	Argument:
 *			bMode	0: Hardware initialize sequence (normal)
 *					1: Power down change sequence
 *					2: Power down release sequence
 *	Return:
 *			0		success
 *			< 0		error code
 *
 ****************************************************************************/
SINT32 MaDevDrv_PowerManagement( UINT8 bMode )
{
	SINT32	sdResult;					/* sdResult of function */

	MADEVDRV_DBGMSG(("MaDevDrv_PowerManagement: bMode=%d\n", bMode));

	sdResult = machdep_PowerManagement( bMode );
	return sdResult;
}
/****************************************************************************
 *	MaDevDrv_WriteIntermediateReg
 *
 *	Description:
 *			Write data to intermediate register.
 *	Arguments:
 *			dNumber	Register Index
 *			bData	Write Data
 *	Return:
 *			0		success
 *			< 0		error code
 *
 ****************************************************************************/
SINT32 MaDevDrv_WriteIntermediateReg( UINT32 dNumber, UINT8 bData )
{
	/* increment Irq access counter */
	pgMaDevDrvInfo->bMaskInterrupt++;

	/* set specified reg# */
	WriteStatusFlagReg( (UINT8)(dNumber & 0x7F) );

	/* write data */
	machdep_WriteDataReg( bData );

	/* enable interrupt if needed */
	if ( (--pgMaDevDrvInfo->bMaskInterrupt)==0 )
	{
		WriteStatusFlagReg( MAB_IRQ_ENABLE );
	}

	return MASMW_SUCCESS;
}
/****************************************************************************
 *	MaDevDrv_ReadIntermediateReg
 *
 *	Description:
 *			Read data from intermediate register
 *	Arguments:
 *			dNumber	Register Index
 *	Return:
 *			read data
 *
 ****************************************************************************/
UINT8 MaDevDrv_ReadIntermediateReg( UINT32 dNumber )
{
	UINT8 bData;

	/* increment Irq access counter */
	pgMaDevDrvInfo->bMaskInterrupt++;

	/* set specified reg# */
	WriteStatusFlagReg( (UINT8)(dNumber & 0x7F) );

	/* read data */
	bData = machdep_ReadDataReg();

	/* enable interrupt if needed */
	if ( (--pgMaDevDrvInfo->bMaskInterrupt)==0 )
	{
		WriteStatusFlagReg( MAB_IRQ_ENABLE );
	}

	return bData;
}
/****************************************************************************
 *	MaDevDrv_ReceiveData
 *
 *	Description:
 *			Read byte data of register or memory.
 *	Argument:
 *			dAddress	address of the register or memory for read
 *	Return:
 *			read data
 *
 ****************************************************************************/
SINT32 MaDevDrv_ReceiveData( UINT32	dAddress )
{
	UINT8	bRdData;
	SINT32	sdResult;

	MADEVDRV_DBGMSG(("    MaDevDrv_ReceiveData: adrs=%ld\n", dAddress));

	/* increment Irq access counter */
	pgMaDevDrvInfo->bMaskInterrupt++;

	/* set REG_ID #3   */
	WriteStatusFlagReg( MAI_IMMEDIATE_READ );

	/* check status */
	sdResult = machdep_WaitImmediateFifoEmpty();

	/* error  */
	if ( sdResult < MASMW_SUCCESS )
	{
		/* enable interrupt if needed */
		MaDevDrv_EnableIrq();
		return sdResult;
	}

	/* write direct packet for read */
	if		( dAddress < 0x0080 )
	{
		machdep_WriteDataReg( (UINT8)((dAddress >>  0 ) | 0x80) );
	}
	else if ( dAddress < 0x4000 )
	{
		machdep_WriteDataReg( (UINT8)((dAddress >>  0 ) & 0x7F) );
		machdep_WriteDataReg( (UINT8)((dAddress >>  7 ) | 0x80) );
	}
	else
	{
		machdep_WriteDataReg( (UINT8)((dAddress >>  0 ) & 0x7F) );
		machdep_WriteDataReg( (UINT8)((dAddress >>  7 ) & 0x7F) );
		machdep_WriteDataReg( (UINT8)((dAddress >> 14 ) | 0x80) );
	}

	/* Read Buffer # */
	machdep_WriteDataReg( (UINT8)0x80 );

	/* check status */
	sdResult = machdep_WaitValidData( MAB_STAT_VALID_R );

	/* error  */
	if ( sdResult < MASMW_SUCCESS )
	{
		/* enable interrupt if needed */
		MaDevDrv_EnableIrq();
		return sdResult;
	}

	/* clear VALID_R bit and set REG_ID #1   */
	WriteStatusFlagReg( MAB_STAT_VALID_R|MAI_READ_BUF );

	/* read data */
	bRdData = machdep_ReadDataReg();

	/* enable interrupt if needed */
	if ( (--pgMaDevDrvInfo->bMaskInterrupt)==0 )
	{
		WriteStatusFlagReg( MAB_IRQ_ENABLE );
	}

	return ((SINT32)bRdData&0x000000FF);
}
/****************************************************************************
 *	MaDevDrv_SendDirectPacket
 *
 *	Description:
 *			Write direct packets to the REG_ID #2 direct write register.
 *	Argument:
 *			pbData	pointer to the direct packets
 *			dSize	size of the direct packets
 *	Return:
 *			0		success
 *			< 0		error code
 *
 ****************************************************************************/
SINT32 MaDevDrv_SendDirectPacket( const UINT8 * pbData, UINT32 dSize )
{
	UINT32	i, j;
	SINT32	sdResult;

	MADEVDRV_DBGMSG(("    MaDevDrv_SendDirectPacket: pt=%p, sz=%d\n", pbData, dSize));

	if( dSize == 0 )  return MASMW_SUCCESS;

	/* increment Irq access counter */
	pgMaDevDrvInfo->bMaskInterrupt++;

	/* set REG_ID #2   */
	WriteStatusFlagReg( MAI_IMMEDIATE_WRITE );

	/* write data */
	i = dSize/64;

	while ( i-- != 0 )
	{
		/* check status */
		sdResult = machdep_WaitImmediateFifoEmpty();

		/* error  */
		if ( sdResult < MASMW_SUCCESS )
		{
			/* enable interrupt if needed */
			MaDevDrv_EnableIrq();
			return sdResult;
		}

		/* write 64 bytes */
		machdep_WriteDataRegN( (UINT8*)pbData, (UINT32)64 );
		pbData += 64;
	}

	/* write odd data */
	if ( (j=(dSize%64)) != 0 )
	{
		/* check status */
		sdResult = machdep_WaitImmediateFifoEmpty();

		/* error  */
		if ( sdResult < MASMW_SUCCESS )
		{
			/* enable interrupt if needed */
			MaDevDrv_EnableIrq();
			return sdResult;
		}

		/* write data */
		machdep_WriteDataRegN( (UINT8*)pbData, j );
	}

	/* enable interrupt if needed */
	if ( (--pgMaDevDrvInfo->bMaskInterrupt)==0 )
	{
		WriteStatusFlagReg( MAB_IRQ_ENABLE );
	}

	return MASMW_SUCCESS;
}
/****************************************************************************
 *	MaDevDrv_SendDelayedPacket
 *
 *	Description:
 *			Write delayed packets to the REG_ID #1 delayed write register.
 *	Argument:
 *			pbData	pointer to the delayed packets
 *			dSize	size of the delayed packets
 *	Return:
 *			0		success
 *			< 0		error code
 *
 ****************************************************************************/
SINT32 MaDevDrv_SendDelayedPacket( const UINT8 * pbData, UINT32 dSize )
{
	MADEVDRV_DBGMSG(("    MaDevDrv_SendDelayedPacket: pt=%p, sz=%d\n", pbData, dSize));

	if( dSize == 0 )  return MASMW_SUCCESS;

	/* increment Irq access counter */
	pgMaDevDrvInfo->bMaskInterrupt++;

	/* set REG_ID #1   */
	WriteStatusFlagReg( MAI_DELAYED_WRITE );

	/* write data */
	machdep_WriteDataRegN( (UINT8*)pbData, dSize );

	/* enable interrupt if needed */
	if ( (--pgMaDevDrvInfo->bMaskInterrupt)==0 )
	{
		WriteStatusFlagReg( MAB_IRQ_ENABLE );
	}

	return MASMW_SUCCESS;
}
/****************************************************************************
 *	MaDevDrv_SendDirectRamData
 *
 *	Description:
 *			write data to internal RAM.
 *	Arguments:
 *			dAddress  internal RAM address
 *			dType	  type of data 0:8bit data , 1:7bit encoded data
 *			pbData	  pointer ot the write byte data to internal RAM
 *			dSize	  size of write byte data to internal RAM
 *	Return:
 *			0		  success
 *			< 0		  error code
 *
 ****************************************************************************/
SINT32 MaDevDrv_SendDirectRamData( UINT32 dAddress, UINT8 bType, 
											const UINT8 *pbData, UINT32 dSize)
{
	UINT32	i, j;
	UINT32	dWriteSize;
	UINT8	bWriteData;
	UINT8	bTemp = 0;
	SINT32	sdResult;

	MADEVDRV_DBGMSG(("    MaDevDrv_SendDirectRamData: adrs=%04lX type=%d ptr=%p size=%ld\n", dAddress, bType, pbData, dSize));

	if ( dSize == 0 ) return MASMW_SUCCESS;

	/* size */
	if ( bType == 0 )			/* 8bit data */
	{
		dWriteSize = dSize;
	}
	else						/* 7bit encoded data */
	{
		dWriteSize = (dSize / 8) * 7;
		if ( dSize % 8 != 0 )
			dWriteSize = (UINT32)( dWriteSize + (dSize % 8 - 1) );
	}

	if( dAddress+dWriteSize>MA_MEMORY_SIZE ) return MASMW_ERROR;

	/* increment Irq access counter */
	pgMaDevDrvInfo->bMaskInterrupt++;

	/* set REG_ID #2   */
	WriteStatusFlagReg( MAI_IMMEDIATE_WRITE );

	/* check status */
	sdResult = machdep_WaitImmediateFifoEmpty();

	/* error */
	if ( sdResult < MASMW_SUCCESS )
	{
		/* enable interrupt if needed */
		MaDevDrv_EnableIrq();
		return sdResult;
	}

	/* write direct packet */

	/* Address part */
	machdep_WriteDataReg( (UINT8)((dAddress >>  0 ) & 0x7F) );
	machdep_WriteDataReg( (UINT8)((dAddress >>  7 ) & 0x7F) );
	machdep_WriteDataReg( (UINT8)((dAddress >> 14 ) | 0x80) );

	/* size */
	if	( dWriteSize < 0x0080 )
	{
		machdep_WriteDataReg( (UINT8)( (dWriteSize >> 0) | 0x80 ) );
	}
	else
	{
		machdep_WriteDataReg( (UINT8)( (dWriteSize >> 0) & 0x7F ) );
		machdep_WriteDataReg( (UINT8)( (dWriteSize >> 7) | 0x80 ) );
	}

	/* Type 0, 8bit data */
	if ( bType == 0 )
	{
		i = dSize/64;
		while ( (i--) != 0 )
		{
			/* check status */
			sdResult = machdep_WaitImmediateFifoEmpty();

			/* error  */
			if ( sdResult < MASMW_SUCCESS )
			{
				/* enable interrupt if needed */
				MaDevDrv_EnableIrq();
				return sdResult;
			}

			/* write 64 bytes data */
			machdep_WriteDataRegN( (UINT8*)pbData, (UINT32)64 );
			pbData += 64;
		}

		/* write odd data */
		if ( (j=(dSize%64)) != 0 )
		{
			/* check status */
			sdResult = machdep_WaitImmediateFifoEmpty();

			/* error  */
			if ( sdResult < MASMW_SUCCESS )
			{
				/* enable interrupt if needed */
				MaDevDrv_EnableIrq();
				return sdResult;
			}
			/* write data */
			machdep_WriteDataRegN( (UINT8*)pbData, j );
		}
	}
	/* Type 1, 7bit encoded data */
	else
	{
		for ( i = 0; i < dSize; i++ )
		{
			if ( (i & 0xFFFFFFC0) == 0 )
			{
				/* check status */
				sdResult = machdep_WaitImmediateFifoEmpty();

				/* error  */
				if ( sdResult < MASMW_SUCCESS )
				{
					/* enable interrupt if needed */
					MaDevDrv_EnableIrq();
					return sdResult;
				}
			}

			if ( ( i % 8 ) == 0 )
			{
				bTemp = *pbData++;
			}
			else
			{
				bWriteData = *pbData++;
				bWriteData |= (UINT8)( ( bTemp << (i % 8) ) & 0x80 );

				/* write data */
				machdep_WriteDataReg( bWriteData );
			}
		}
	}

	/* enable interrupt if needed */
	if ( (--pgMaDevDrvInfo->bMaskInterrupt)==0 )
	{
		WriteStatusFlagReg( MAB_IRQ_ENABLE );
	}

	return MASMW_SUCCESS;
}
/****************************************************************************
 *	MaDevDrv_SendDirectRamVal
 *
 *	Description:
 *			Write specified data to internal RAM
 *	Arguments:
 *			dAddress  internal RAM address
 *			dSize	  size of write byte data to internal RAM
 *			bVal	  write byte data
 *	Return:
 *			0		  success
 *			< 0		  error code
 *
 ****************************************************************************/
SINT32 MaDevDrv_SendDirectRamVal( UINT32 dAddress, UINT32 dSize, UINT8 bVal )
{
	UINT32	i, j;						/* loop counter */
	SINT32	sdResult;					/* result of function */

	MADEVDRV_DBGMSG(("    MaDevDrv_SendDirectRamVal: adrs=%04lX size=%ld val=%02X\n", dAddress, dSize, bVal));

	if ( dSize == 0 ) return MASMW_SUCCESS;
	if ( dAddress+dSize>MA_MEMORY_SIZE ) return MASMW_ERROR;

	/* increment Irq access counter */
	pgMaDevDrvInfo->bMaskInterrupt++;

	/* set REG_ID #2   */
	WriteStatusFlagReg( MAI_IMMEDIATE_WRITE );

	/* check status */
	sdResult = machdep_WaitImmediateFifoEmpty();

	/* error  */
	if ( sdResult < MASMW_SUCCESS )
	{
		/* enable interrupt if needed */
		MaDevDrv_EnableIrq();
		return sdResult;
	}

	/* write direct packet */

	/* Address */
	machdep_WriteDataReg( (UINT8)((dAddress >>  0 ) & 0x7F) );
	machdep_WriteDataReg( (UINT8)((dAddress >>  7 ) & 0x7F) );
	machdep_WriteDataReg( (UINT8)((dAddress >> 14 ) | 0x80) );

	/* Size */
	if ( dSize < 0x0080 )
	{
		machdep_WriteDataReg( (UINT8)( (dSize >> 0) | 0x80 ) );
	}
	else
	{
		machdep_WriteDataReg( (UINT8)( (dSize >> 0) & 0x7F ) );
		machdep_WriteDataReg( (UINT8)( (dSize >> 7) | 0x80 ) );
	}

	i = dSize/64;

	while ( (i--) != 0 )
	{
		/* check status */
		sdResult = machdep_WaitImmediateFifoEmpty();

		/* error  */
		if ( sdResult < MASMW_SUCCESS )
		{
			/* enable interrupt if needed */
			MaDevDrv_EnableIrq();
			return sdResult;
		}

		/* write 64 bytes data */
		machdep_WriteDataRegValN( bVal, (UINT32)64 );
	}

	/* write remained data(<64) */
	if ( (j=(dSize%64)) != 0 )
	{
		/*check status */
		sdResult = machdep_WaitImmediateFifoEmpty();

		/* error  */
		if ( sdResult < MASMW_SUCCESS )
		{
			/* enable interrupt if needed */
			MaDevDrv_EnableIrq();
			return sdResult;
		}

		/* write data */
		machdep_WriteDataRegValN( bVal, j );
	}

	/* enable interrupt if needed */
	if ( (--pgMaDevDrvInfo->bMaskInterrupt)==0 )
	{
		WriteStatusFlagReg( MAB_IRQ_ENABLE );
	}

	return MASMW_SUCCESS;
}
/****************************************************************************
 *	MaDevDrv_StartSequencer
 *
 *	Description:
 *			Start sequencer.
 *	Argument:
 *			sdSeqId	 sequence id
 *	Return:
 *			0		 success
 *			< 0		 error code
 *
 ****************************************************************************/
SINT32 MaDevDrv_StartSequencer( SINT32 sdSeqId )
{
	SINT32	sdResult;						/* sdResult of function */

	MADEVDRV_DBGMSG(("MaDevDrv_StartSequencer: seq_id=%ld\n", sdSeqId));

	sdResult = MASMW_SUCCESS;

	/* increment Irq access counter */
	pgMaDevDrvInfo->bMaskInterrupt++;

	/* set REG_ID #0   */
	WriteStatusFlagReg( MAI_IRQ_FLAG_1 );

	switch ( sdSeqId )
	{
	case 0:						/* delayed sequence */
		/* Pending FIFO IRQ */
		if( pgMaDevDrvInfo->dPendingIrq & 0x80 ) {
			pgMaDevDrvInfo->pIntFunc[7](0);
			pgMaDevDrvInfo->dPendingIrq &= ~0x80;
		}
		/* Pending SINRQ#2 IRQ */
		if( pgMaDevDrvInfo->dPendingIrq & 0x400 ) {
			pgMaDevDrvInfo->pIntFunc[8](0);
			pgMaDevDrvInfo->dPendingIrq &= ~0x400;
		}
		/* Pending SINRQ#3 IRQ */
		if( pgMaDevDrvInfo->dPendingIrq & 0x800 ) {
			pgMaDevDrvInfo->pIntFunc[9](0);
			pgMaDevDrvInfo->dPendingIrq &= ~0x800;
		}
		/* Enable FIFO IRQ */
		ControlInterrupt( MAI_IRQ_CONTROL_1, ENABLE_IRQ, MAB_IRQCTRL_EFIFO );

		/* Start H/W Sequencer */
		pgMaDevDrvInfo->dActiveSequencer |= MADEVDRV_ACTIVE_DELAYEDSEQ;
		MaDevDrv_WriteIntermediateReg( MAI_SEQ_CTRL, 0x01 );
		break;

	case 1:						/* Phrase */
		/* Start Timer#2A */
		pgMaDevDrvInfo->dActiveSequencer |= MADEVDRV_ACTIVE_PHRASESEQ;
		MaDevDrv_WriteIntermediateReg( MAI_TIMER2_TMSTA, 0x01 );
		break;

	case 2:						/* HV */
		/* Start Timer#2B */
		pgMaDevDrvInfo->dActiveSequencer |= MADEVDRV_ACTIVE_HVSEQ;
		MaDevDrv_WriteIntermediateReg( MAI_TIMER2_TMSTB, 0x01 );
		break;

	default:
		sdResult = MASMW_ERROR_ARGUMENT;
		break;
	}

	/* enable interrupt if needed */
	if ( (--pgMaDevDrvInfo->bMaskInterrupt)==0 )
	{
		WriteStatusFlagReg( MAB_IRQ_ENABLE );
	}

	return sdResult;
}
/****************************************************************************
 *	MaDevDrv_StopSequencer
 *
 *	Description:
 *			Stop sequencer.
 *	Argument:
 *			sdSeqId	 sequence id
 *	Return:
 *			0		 success
 *			< 0		 error code
 *
 ****************************************************************************/
SINT32 MaDevDrv_StopSequencer( SINT32 sdSeqId )
{
	SINT32	sdResult;

	MADEVDRV_DBGMSG(("MaDevDrv_StopSequencer: seq_id=%ld\n", sdSeqId));

	sdResult = MASMW_SUCCESS;

	/* increment Irq access counter */
	pgMaDevDrvInfo->bMaskInterrupt++;

	/* set REG_ID #0   */
	WriteStatusFlagReg( MAI_IRQ_FLAG_1 );

	switch ( sdSeqId )
	{
	case 0:						/* delayed sequence */
		/* Disable FIFO/StreamPG IRQ */
		ControlInterrupt( MAI_IRQ_CONTROL_1, DISABLE_IRQ, MAB_IRQCTRL_EFIFO |
										 MAB_IRQCTRL_ESTM1|MAB_IRQCTRL_ESTM0 );

		pgMaDevDrvInfo->dActiveSequencer &= ~MADEVDRV_ACTIVE_DELAYEDSEQ;
		/* Stop Sequencer */
		MaDevDrv_WriteIntermediateReg( MAI_SEQ_CTRL, 0x00 );
		
#ifdef COLORLED_SUPPORT
		/* output to CPU GPIO */
		Red_Off();
		Blue_Off();
		Green2_Off();
#endif		
	
		break;

	case 1:						/* Phrase */
		pgMaDevDrvInfo->dActiveSequencer &= ~MADEVDRV_ACTIVE_PHRASESEQ;
		/* Stop Timer#2A */
		MaDevDrv_WriteIntermediateReg( MAI_TIMER2_TMSTA, 0x00 );
		break;

	case 2:						/* HV	*/
		pgMaDevDrvInfo->dActiveSequencer &= ~MADEVDRV_ACTIVE_HVSEQ;
		/*Stop Timer#2B */
		MaDevDrv_WriteIntermediateReg( MAI_TIMER2_TMSTB, 0x00 );
		break;

	default:
		sdResult = MASMW_ERROR_ARGUMENT;
		break;
	}

	/* enable interrupt if needed */
	if ( (--pgMaDevDrvInfo->bMaskInterrupt)==0 )
	{
		WriteStatusFlagReg( MAB_IRQ_ENABLE );
	}

	return sdResult;
}
/****************************************************************************
 *	MaDevDrv_EndofSequence
 *
 *	Description:
 *			End of Sequence.
 *	Argument:
 *			None
 *	Return:
 *			0		success
 *			< 0		error code
 *
 ****************************************************************************/
SINT32 MaDevDrv_EndofSequence( void )
{
	MADEVDRV_DBGMSG(("MaDevDrv_EndofSequence\n"));

	pgMaDevDrvInfo->dEndofSequence = 1;
	return MASMW_SUCCESS;
}
/****************************************************************************
 *	MaDevDrv_ClearFifo
 *
 *	Description:
 *			Clear Delayed FIFO.
 *	Argument:
 *			None
 *	Return:
 *			0		success
 *			< 0		error code
 *
 ****************************************************************************/
SINT32 MaDevDrv_ClearFifo( void )
{
	SINT32	i;

	MADEVDRV_DBGMSG(("MaDevDrv_ClearFifo\n"));

	/* increment Irq access counter */
	pgMaDevDrvInfo->bMaskInterrupt++;

	/* set REG_ID #4   */
	WriteStatusFlagReg( MAI_BASIC_SETTING );

	/* set RFIF0 and RSEQ0 bit */
	machdep_WriteDataReg( MAB_RFIFO0|MAB_RSEQ0 );
	/* wait 300ns */
	machdep_Wait( 300 );
	/* reset RFIF0 and RSEQ0 bit */
	machdep_WriteDataReg( 0x00 );

	/* clear Read/Write buffer pointer */
	pgMaDevDrvInfo->SbufInfo.dWriteNum = 0;
	pgMaDevDrvInfo->SbufInfo.dReadNum  = 0;

	/* clear software buffer size */
	for ( i = 0; i < MA_SBUF_NUM; i++ )
	{
		pgMaDevDrvInfo->SbufInfo.dBufSize[i] = 0;
	}

	/* clear EOS flag */
	pgMaDevDrvInfo->dEndofSequence = 0;

	/* Clear Pending IRQ */
	pgMaDevDrvInfo->dPendingIrq = 0;

	/* enable interrupt if needed */
	if ( (--pgMaDevDrvInfo->bMaskInterrupt)==0 )
	{
		WriteStatusFlagReg( MAB_IRQ_ENABLE );
	}

	return MASMW_SUCCESS;
}
/****************************************************************************
 *	MaDevDrv_Fifo
 *
 *	Description:
 *			Update sequence data to sequence FIFO.
 *	Argument:
 *			dCtrl	loop count
 *	Return:
 *			None
 *
 ****************************************************************************/
void MaDevDrv_Fifo( UINT32 dCtrl )
{
	UINT8 *	pbBuf;
	UINT32	dBufSize;
	SINT32	dWroteSize;
	UINT32	i;

	MADEVDRV_DBGMSG(("MaDevDrv_Fifo: dCtrl=%ld\n", dCtrl));

	dWroteSize = 0;
	dBufSize = 0;

	for ( i = 0; i <= dCtrl; i++ )
	{
		/* transfer packet data from S/W buffer to H/W FIFO */
		pbBuf    = pgMaDevDrvInfo->bSbufBuffer[pgMaDevDrvInfo->SbufInfo.dReadNum];
		dBufSize = pgMaDevDrvInfo->SbufInfo.dBufSize[pgMaDevDrvInfo->SbufInfo.dReadNum];

		/* data exist ? */
		if ( dBufSize != 0 )
		{
			/* S/W buffer  H/W FIFO */
			MaDevDrv_SendDelayedPacket( pbBuf, dBufSize );

			pgMaDevDrvInfo->SbufInfo.dBufSize[pgMaDevDrvInfo->SbufInfo.dReadNum] = 0;

			/* update read buffer pointer */
			pgMaDevDrvInfo->SbufInfo.dReadNum++;
			if ( pgMaDevDrvInfo->SbufInfo.dReadNum == MA_SBUF_NUM )
			{
				pgMaDevDrvInfo->SbufInfo.dReadNum = 0;
			}
		}

		if( pgMaDevDrvInfo->dEndofSequence==1 ) continue;

		/* update S/W buffer */
		pbBuf	 = pgMaDevDrvInfo->bSbufBuffer[pgMaDevDrvInfo->SbufInfo.dWriteNum];
		dBufSize = MA_FIFO_SIZE;
		if ( dBufSize != 0 )
		{
			/* update */
			dWroteSize = MaSndDrv_UpdateSequence( MASNDDRV_UPDATE_FIFO, pbBuf, dBufSize  );

			if ( dWroteSize >= 0 )
				pgMaDevDrvInfo->SbufInfo.dBufSize[pgMaDevDrvInfo->SbufInfo.dWriteNum] = dWroteSize;

			/* update write buffer pointer */
			pgMaDevDrvInfo->SbufInfo.dWriteNum++;
			if ( pgMaDevDrvInfo->SbufInfo.dWriteNum == MA_SBUF_NUM )
			{
				pgMaDevDrvInfo->SbufInfo.dWriteNum = 0;
			}
		}
	}
	if( dWroteSize==0 && dBufSize==0 && pgMaDevDrvInfo->dEndofSequence==1 )
	{
		/* Disable FIFO IRQ */
		ControlInterrupt( MAI_IRQ_CONTROL_1, DISABLE_IRQ, MAB_IRQCTRL_EFIFO );
	}

#ifdef COLORLED_SUPPORT
    /* output to CPU GPIO */
	switch( abLedPattern[dwLedCount] ){
		case 0: //RGB -- 000 
			Red_Off();
			Green2_Off();      
			Blue_Off();
			break;
		case 1: //RGB-- 001
			Red_Off();          
			Green2_Off();      
			Blue_On();
			break;
		case 2: //RGB -- 010
			Red_Off();          
			Green2_On();      
			Blue_Off();
			break;
		case 3: //RGB-- 011
			Red_Off();          
			Green2_On();      
			Blue_On();
			break;
		case 4: //RGB -100
			Red_On();          
			Green2_Off();      
			Blue_Off();
			break;
		case 5: //RGB -101
			Red_On();          
			Green2_Off();      
			Blue_On();
			break;
		case 6://RGB -110
			Red_On();          
			Green2_On();      
			Blue_Off();
		case 7://RGB -111
			Red_On();          
			Green2_On();      
			Blue_On();
			break;
		}

    dwLedCount++;
    if ( dwLedCount >= MADEVDRV_LED_PATTERN_NUM ) {
    	dwLedCount = 0;
    }
#endif    
	
}
/****************************************************************************
 *	MaDevDrv_GetSeekBuffer
 *
 *	Description:
 *			Return pointer to buffer and size of buffer for setting control
 *			values when seek.
 *	Arguments:
 *			*pdSize	  pointer to buffer size
 *	Return:
 *			pointer to buffer
 *
 ****************************************************************************/
UINT8 * MaDevDrv_GetSeekBuffer( UINT32 * pdSize )
{
	*pdSize = MA_FIFO_SIZE * MA_SBUF_NUM;

	return pgMaDevDrvInfo->bSbufBuffer[0];
}
/****************************************************************************
 *	MaDevDrv_SeekControl
 *
 *	Description:
 *			Send control values when seek.
 *	Arguments:
 *			dSize	buffer byte size
 *	Return:
 *			0		success
 *			< 0		error code
 *
 ****************************************************************************/
SINT32 MaDevDrv_SeekControl( UINT32 dSize )
{
	UINT8 * pbBuf;					/* pointer to buffer */
	SINT32	sdResult;				/* sdResult of function */

	if( dSize==0 )  return MASMW_SUCCESS;

	/* start the H/W sequencer */
	MaDevDrv_WriteIntermediateReg( MAI_SEQ_CTRL, 0x01 );

	/* write control data to H/W FIFO for seek */
	pbBuf = pgMaDevDrvInfo->bSbufBuffer[0];
	MaDevDrv_SendDelayedPacket( pbBuf, dSize );

	/* waiting for FIFO empty */
	sdResult = machdep_WaitDelayedFifoEmpty();

	/* stop the H/W sequencer */
	MaDevDrv_WriteIntermediateReg( MAI_SEQ_CTRL, 0x00 );

	return sdResult;
}
/****************************************************************************
 *	MaDevDrv_IntHandler
 *
 *	Description:
 *			Interrupt handler for hardware interrupts.
 *	Argument:
 *			None
 *	Return:
 *			None
 *
 ****************************************************************************/
void MaDevDrv_IntHandler( void )
{
	UINT8	bOldSetting;
	UINT8	bIrqFlag1;
	UINT8	bIrqFlag2;
	UINT32	dIrqFlag;

	MADEVDRV_DBGMSG(("    MaDevDrv_IntHandler\n"));


	/* increment access counter */
	pgMaDevDrvInfo->bMaskInterrupt++;

	/* set REG_ID #0   */
	bOldSetting = WriteStatusFlagReg( MAI_IRQ_FLAG_1 );
	/* read IRQ_FLAG_1 */
	bIrqFlag1 = machdep_ReadDataReg();

	/* set REG_ID #16   */
	WriteStatusFlagReg( MAI_IRQ_FLAG_2 );
	/* read IRQ_FLAG_2 */
	bIrqFlag2 = machdep_ReadDataReg();

	dIrqFlag = ((UINT32)bIrqFlag2<<8)|(UINT32)bIrqFlag1;

	if ( dIrqFlag != 0 )
	{
		pgMaDevDrvInfo->bIrqProc = 1;
		machdep_GenerateIrqProcTask(dIrqFlag, bOldSetting);
	}

	--pgMaDevDrvInfo->bMaskInterrupt;

	WriteStatusFlagReg( bOldSetting );
}
/****************************************************************************
 *	MaDevDrv_StreamHandler
 *
 *	Description:
 *			Control the stream audio.
 *	Arguments:
 *			dSeqId	  sequence number
 *			dSaId	  stream audio slot number
 *			dCtrl	  0: setup
 *					  1: update
 *			bRamVal	  SINT RAM register value
 *			dSeek	  Seek Position
 *	Return:
 *			None
 *
 ****************************************************************************/
SINT32 MaDevDrv_StreamHandler( UINT32 dSeqId, UINT32 dSaId, UINT32 dCtrl,
									UINT8 bRamVal, UINT32 dSeek )
{
	UINT8	bPacket[16];
	UINT32	dFormat[2];
	UINT8 *	pbWaveData;
	UINT32	dWaveSize;
	UINT32	dWaveId[2];
	UINT32	dSaSlot[2];
	SINT32	sdResult;
	UINT32	dRegIndex;
	UINT32	dNumSaon;
	UINT32	i,j;
	UINT32	dPp;
	UINT8	bCh;

	static const UINT32 dRegIndexTbl[2] = { 
		MAC_EXT_WT_VOICE_ADDRESS + 9*6 + 5,
		MAC_EXT_WT_VOICE_ADDRESS + 10*6 + 5
	};

	MADEVDRV_DBGMSG(("MaDevDrv_StreamHandler: dSaId=%d ctrl=%d ram_val=%d\n", dSaId, dCtrl, bRamVal));

	/* error  */
	if ( dSaId >= MA_MAX_STREAM_AUDIO ) return MASMW_ERROR;

	sdResult = MASMW_SUCCESS;

	if( dCtrl&0x80 )
	{
		pgMaDevDrvInfo->bSlave = (UINT8)(bRamVal & 0x1F);
		return sdResult;
	}

	pgMaDevDrvInfo->dSeqId = dSeqId;

	dCtrl &= 0x01;

	if ( dCtrl == 0 )						/* setup */
	{
		/* Start or Stop? */
		if ( ( bRamVal & 0x40 ) != 0 )
		{
			/* Start */
			dWaveId[0] = (UINT32)( bRamVal & 0x1F );
			dSaSlot[0] = dSaId;
			dNumSaon   = 1;


			/* master ? */
			if ( bRamVal & 0x20 )
			{
				dSaSlot[1] = (UINT32)( ( dSaId == 0 ) ? 1 : 0 );

				/* read slave wave# */
				dWaveId[1] = (UINT32)pgMaDevDrvInfo->bSlave;
				dNumSaon++;
				pgMaDevDrvInfo->bSlave = 0xff;
			}

			for ( i = 0; i < dNumSaon; i++ )
			{
				/* get stream information */
				sdResult = MaResMgr_GetStreamAudioInfo( dWaveId[i], &dFormat[dSaSlot[i]], &pbWaveData, &dWaveSize );

				/* error  */
				if ( sdResult != MASMW_SUCCESS )
				{
					return sdResult;
				}
				/* data error  */
				if ( pbWaveData==NULL || dWaveSize==0 || dSeek>=dWaveSize )
				{
					return MASMW_ERROR;
				}

				/* save stream information */
				pgMaDevDrvInfo->StreamInfo[dSaSlot[i]].dPrvPoint	= (UINT32)MA_MIN_PG;
				pgMaDevDrvInfo->StreamInfo[dSaSlot[i]].pbWaveData 	= pbWaveData;
				pgMaDevDrvInfo->StreamInfo[dSaSlot[i]].dWaveSize 	= dWaveSize;

				if( dSaSlot[i]==1 && (dFormat[1]&0x80) )
				{
					return MASMW_ERROR;
				}
			}
			if( dNumSaon==2 && ((dFormat[0]&0x80)||(dFormat[1]&0x80)))
			{
				return MASMW_ERROR;
			}


			/* stop stereo stream */
			if( pgMaDevDrvInfo->StreamInfo[0].bState != MADEVDRV_STREAM_STATE_IDLE &&
								(pgMaDevDrvInfo->StreamInfo[0].dFormat&0x80) )
			{
				/* disable streamPG#0 Irq */
				if( dSeqId==0 )
				{
					ControlInterrupt( MAI_IRQ_CONTROL_1, DISABLE_IRQ, (UINT8)(0x01<<0) );
				}
				/* Stream#0 <-> WT#25 */
				dRegIndex = dRegIndexTbl[0];

				sdResult = MaDevDrv_ReceiveData( dRegIndex );
				/* error  */
				if ( sdResult < MASMW_SUCCESS )
				{
					return sdResult;
				}
				bCh = (UINT8)( sdResult & 0x0F );

				bPacket[0] = (UINT8)(  dRegIndex       & 0x7F );	/* address part */
				bPacket[1] = (UINT8)( (dRegIndex >> 7) | 0x80 );
				bPacket[2] = (UINT8)(  0x30 | bCh      | 0x80 );	/* data part, KeyOff */

				dRegIndex += 6;
				bPacket[3] = (UINT8)(  dRegIndex       & 0x7F );	/* address part */
				bPacket[4] = (UINT8)( (dRegIndex >> 7) | 0x80 );
				bPacket[5] = (UINT8)(  0x30 | bCh      | 0x80 );	/* data part, KeyOff */

				/* write packet */
				sdResult = MaDevDrv_SendDirectPacket( bPacket, 6 );

				/* error  */
				if ( sdResult != MASMW_SUCCESS )
				{
					return sdResult;
				}

				/* wait 20us */
				machdep_Wait( 20000 );

				/* clear STEREO bit */
				MaDevDrv_WriteIntermediateReg( MAI_STM_CTRL, 0 );

				pgMaDevDrvInfo->StreamInfo[0].bState = MADEVDRV_STREAM_STATE_IDLE;
			}
			/* stop all streams if NEW stream is stereo */
			else if( dSaId==0 && (dFormat[0]&0x80) )
			{
				for ( i = 0; i < 2 ; i++ )
				{
					/* stop stream if active */
					if ( pgMaDevDrvInfo->StreamInfo[i].bState != MADEVDRV_STREAM_STATE_IDLE )
					{
						/* disable streamPG Irq */
						if( dSeqId==0 )
						{
							ControlInterrupt( MAI_IRQ_CONTROL_1, DISABLE_IRQ, (UINT8)(0x01<<i) );
						}

						/* Stream#0/#1 <-> WT#25/#26 */
						dRegIndex = dRegIndexTbl[i];

						sdResult = MaDevDrv_ReceiveData( dRegIndex );
						/* error  */
						if ( sdResult < MASMW_SUCCESS )
						{
							return sdResult;
						}
						bCh = (UINT8)( sdResult & 0x0F );

						bPacket[0] = (UINT8)(  dRegIndex       & 0x7F );	/* address part */
						bPacket[1] = (UINT8)( (dRegIndex >> 7) | 0x80 );
						bPacket[2] = (UINT8)(  0x30 | bCh      | 0x80 );	/* data part, KeyOff */

						/* write packet */
						sdResult = MaDevDrv_SendDirectPacket( bPacket, 3 );

						/* error  */
						if ( sdResult != MASMW_SUCCESS )
						{
							return sdResult;
						}
						pgMaDevDrvInfo->StreamInfo[i].bState = MADEVDRV_STREAM_STATE_IDLE;
					}
				}
			}
			/* stop overlapping stream */
			else {
				for ( i = 0; i < dNumSaon; i++ )
				{
					if ( pgMaDevDrvInfo->StreamInfo[dSaSlot[i]].bState != MADEVDRV_STREAM_STATE_IDLE )
					{

						/* disable streamPG Irq */
						if( dSeqId==0 )
						{
							ControlInterrupt( MAI_IRQ_CONTROL_1, DISABLE_IRQ, (UINT8)(0x01<<dSaSlot[i]) );
						}

						/* Stream#0/#1 <-> WT#25/#26 */
						dRegIndex = dRegIndexTbl[dSaSlot[i]];


						sdResult = MaDevDrv_ReceiveData( dRegIndex );
						/* error  */
						if ( sdResult < MASMW_SUCCESS )
						{
							return sdResult;
						}
						bCh = (UINT8)( sdResult & 0x0F );

						bPacket[0] = (UINT8)(  dRegIndex       & 0x7F );	/* address part */
						bPacket[1] = (UINT8)( (dRegIndex >> 7) | 0x80 );
						bPacket[2] = (UINT8)(  0x30 | bCh      | 0x80 );	/* data part, KeyOff */

						/* write packet */
						sdResult = MaDevDrv_SendDirectPacket( bPacket, 3 );


						/* error  */
						if ( sdResult != MASMW_SUCCESS )
						{
							return sdResult;
						}
						pgMaDevDrvInfo->StreamInfo[dSaSlot[i]].bState = MADEVDRV_STREAM_STATE_IDLE;
					}
				}
			}

			dPp = 0;
			for ( i = 0; i < dNumSaon ; i++ )
			{
				/* update format info */
				pgMaDevDrvInfo->StreamInfo[dSaSlot[i]].dFormat = dFormat[dSaSlot[i]];

				/* seek */
				pgMaDevDrvInfo->StreamInfo[dSaSlot[i]].pbWaveData 	+= dSeek;
				pgMaDevDrvInfo->StreamInfo[dSaSlot[i]].dWaveSize 	-= dSeek;

				/* Stream Setup */
				sdResult = StreamSetup( dSaSlot[i] );

				/* error  */
				if ( sdResult != MASMW_SUCCESS )
				{
					return sdResult;
				}

				/* make packet for KeyOn */

				/* Stream#0/#1 <-> WT#25/#26 */
				dRegIndex = dRegIndexTbl[dSaSlot[i]];

				/* read channel */
				sdResult = MaDevDrv_ReceiveData( dRegIndex );

				/* error  */
				if ( sdResult < MASMW_SUCCESS )
				{
					return sdResult;
				}
				bCh = (UINT8)( sdResult & 0x0F );

				/* save channel # */

				/* stereo   */
				if( dSaSlot[i]==0 && (pgMaDevDrvInfo->StreamInfo[0].dFormat&0x80) )
				{
					++dRegIndex;

					bPacket[dPp++] = (UINT8)(  dRegIndex       & 0x7F );	/* address part */
					bPacket[dPp++] = (UINT8)( (dRegIndex >> 7) | 0x80 );

					/* read WT#25 regs and copy to WT#26 */

					for( j=0 ; j<5 ; j++ )
					{
						/* read reg */
						sdResult = MaDevDrv_ReceiveData( dRegIndex-6+j );
						/* error  */
						if ( sdResult < MASMW_SUCCESS )
						{
							return sdResult;
						}
						bPacket[dPp++] = (UINT8)sdResult;
					}
					bPacket[dPp++] = (UINT8)(  0x40 | bCh      | 0x80 );	/* data part, KeyOn */

					/* set STEREO bit */
					MaDevDrv_WriteIntermediateReg( MAI_STM_CTRL, MAB_STEREO );

					--dRegIndex;

				}

				bPacket[dPp++] = (UINT8)(  dRegIndex       & 0x7F );	/* address part */
				bPacket[dPp++] = (UINT8)( (dRegIndex >> 7) | 0x80 );
				bPacket[dPp++] = (UINT8)(  0x40 | bCh      | 0x80 );	/* data part, KeyOn */

				/* enable stream PG Irq */
				if( dSeqId==0 )
				{
					ControlInterrupt( MAI_IRQ_CONTROL_1, ENABLE_IRQ, (UINT8)(0x01<<dSaSlot[i]) );
				}

				pgMaDevDrvInfo->StreamInfo[dSaSlot[i]].bState = MADEVDRV_STREAM_STATE_PLAYING;
			}

			/* write packet */
			sdResult = MaDevDrv_SendDirectPacket( bPacket, dPp );


			/* error  */
			if ( sdResult != MASMW_SUCCESS )
			{
				return sdResult;
			}
		}
		else
		{
			/* Stop */
			pgMaDevDrvInfo->StreamInfo[dSaId].bState = MADEVDRV_STREAM_STATE_IDLE;


			/* disable stream PG Irq */
			if( dSeqId==0 )
			{
				ControlInterrupt( MAI_IRQ_CONTROL_1, DISABLE_IRQ, (UINT8)(0x01<<dSaId) );
			}

			/* make packet for KeyOff */

			/* Stream#0/#1 <-> WT#25/#26 */
			dRegIndex = dRegIndexTbl[dSaId];


			sdResult = MaDevDrv_ReceiveData( dRegIndex );
			/* error  */
			if ( sdResult < MASMW_SUCCESS )
			{
				return sdResult;
			}
			bCh = (UINT8)( sdResult & 0x0F );

			bPacket[0] = (UINT8)(  dRegIndex       & 0x7F );	/* address part */
			bPacket[1] = (UINT8)( (dRegIndex >> 7) | 0x80 );
			bPacket[2] = (UINT8)(  0x30 | bCh      | 0x80 );	/* data part, KeyOff */

			/* write packet */
			sdResult = MaDevDrv_SendDirectPacket( bPacket, 3 );

			/* error  */
			if ( sdResult != MASMW_SUCCESS )
			{
				return sdResult;
			}

			/* stereo   */
			if( dSaId==0 && (pgMaDevDrvInfo->StreamInfo[0].dFormat&0x80) )
			{
				dRegIndex += 6;
				bPacket[0] = (UINT8)(  dRegIndex       & 0x7F );	/* address part */
				bPacket[1] = (UINT8)( (dRegIndex >> 7) | 0x80 );
				bPacket[2] = (UINT8)(  0x30 | bCh      | 0x80 );	/* data part, KeyOff */

				/* write packet */
				sdResult = MaDevDrv_SendDirectPacket( bPacket, 3 );

				/* error  */
				if ( sdResult != MASMW_SUCCESS )
				{
					return sdResult;
				}
				/* wait 20us */
				machdep_Wait( 20000 );

				/* clear STEREO bit */
				MaDevDrv_WriteIntermediateReg( MAI_STM_CTRL, 0 );
			}
		}
	}
	else									/* update */
	{
		if ( pgMaDevDrvInfo->StreamInfo[dSaId].bState>MADEVDRV_STREAM_STATE_STANDBY )
		{
			/* Stream Update */
			sdResult = StreamUpdate( dSaId );
		}
	}

	return sdResult;
}
/****************************************************************************
 *	MaDevDrv_IrqProc
 *
 *	Description:
 *			
 *	Argument:
 *			dIrqFlag
 *			dOldSetting
 *	Return:
 *			None
 *
 ****************************************************************************/
void MaDevDrv_IrqProc( UINT32 dIrqFlag, UINT8 bOldSetting )
{
	UINT32	i;
	UINT32	dIrqNum;

	/* IRQ Priority Table */
	static UINT8	bIrqPriority[12] = { 3, 2, 6, 10, 0, 1, 7, 8, 9 ,4, 5, 11};
/*
 *						   0 STMPG#0	stream PG interrupt #0
 *						   1 STMPG#1	stream PG interrupt #1
 *						   2 SIRQ#0		software interrupt #0
 *						   3 SIRQ#1		software interrupt #1
 *						   4 TM#0		timer #0
 *						   5 TM#1		timer #1
 *						   6 TM#2		tiemr #2
 *						   7 FIFO		FIFO
 *						   8 SIRQ#2		software interrupt #2
 *						   9 SIRQ#3		software interrupt #3
 *						  10 SIRQ#4		software interrupt #4
 *						  11 SIRQ#5		software interrupt #5
*/

	/* IRQ Mask Table */
	static UINT32	dIrqMask[12] = {	0x0001,		/* STM#0 	*/
										0x0002,		/* STM#1 	*/
										0x0004,		/* SIRQ#0	*/
										0x0008,		/* SIRQ#1	*/
										0x0010,		/* TM#0		*/
										0x0020,		/* TM#1		*/
										0x0040,		/* TM#2		*/
										0x0080,		/* FIFO		*/
										0x0400,		/* SIRQ#2	*/
										0x0800,		/* SIRQ#3	*/
										0x1000,		/* SIRQ#4	*/
										0x2000 };	/* SIRQ#5	*/
										
	MADEVDRV_DBGMSG(("    MaDevDrv_IrqProc\n"));
	
#if !MASMW_GENERATE_TASK
	(void)bOldSetting;
#endif


	/* call specified Irq Proc */
	for ( i = 0; i < 12; i++ )
	{
		dIrqNum = (UINT32)bIrqPriority[i];

		if ( ( dIrqFlag & dIrqMask[dIrqNum] ) != 0 )
		{
			if ( dIrqNum < 7 )
			{
				/* set REG_ID #0   */
				WriteStatusFlagReg( (UINT8)MAI_IRQ_FLAG_1 );

				/* clear IRQ flag */
				machdep_WriteDataReg( (UINT8)( dIrqMask[dIrqNum] ) );
			}
			else if ( dIrqNum > 7 )
			{
				/* set REG_ID #16   */
				WriteStatusFlagReg( (UINT8)MAI_IRQ_FLAG_2 );

				/* clear IRQ flag */
				machdep_WriteDataReg( (UINT8)( dIrqMask[dIrqNum]>>8 ) );
			}

			switch( dIrqNum ) {
				case 0:		/* STM#0 */
				case 1:		/* STM#1 */
				case 2:		/* SIRQ#0 */
				case 3:		/* SIRQ#1 */
				case 10:	/* SIRQ#4 */
					if( pgMaDevDrvInfo->dActiveSequencer&MADEVDRV_ACTIVE_DELAYEDSEQ ) {
						/* call registered IRQ function */
						pgMaDevDrvInfo->pIntFunc[dIrqNum](0);
					}
					break;
					
				case 6:		/* Timer#2 */
					if( (pgMaDevDrvInfo->dActiveSequencer&MADEVDRV_ACTIVE_PHRASESEQ) ||
				      (pgMaDevDrvInfo->dActiveSequencer&MADEVDRV_ACTIVE_HVSEQ) )
					{
						/* call registered IRQ function */
						pgMaDevDrvInfo->pIntFunc[dIrqNum](0);
					}
					break;
				case 7:		/* FIFO ? */
				case 8:		/* SIRQ#2 ? */
				case 9:		/* SIRQ#3 ? */
					if( !(pgMaDevDrvInfo->dActiveSequencer&MADEVDRV_ACTIVE_DELAYEDSEQ) ) {
						pgMaDevDrvInfo->dPendingIrq |= dIrqMask[dIrqNum];
					}
					else {
						/* call registered IRQ function */
						pgMaDevDrvInfo->pIntFunc[dIrqNum](0);
					}
					break;
				default:
					/* call registered IRQ function */
					pgMaDevDrvInfo->pIntFunc[dIrqNum](0);
					break;
			}

			if ( dIrqNum == 7 )
			{
				/* set REG_ID #0   */
				WriteStatusFlagReg( (UINT8)MAI_IRQ_FLAG_1 );

				/* clear IRQ flag */
				machdep_WriteDataReg( (UINT8)( MAB_IRQFLAG_FIFO ) );
			}
		}
	}
	pgMaDevDrvInfo->bIrqProc = 0;

#if MASMW_GENERATE_TASK
	/* enable irq */
	WriteStatusFlagReg( bOldSetting );
#endif
}
/****************************************************************************
 *	MaDevDrv_ClearIrqProc
 *
 *	Description:
 *			Clear Irq Proc Task
 *	Argument:
 *			None
 *	Return:
 *			None
 *
 ****************************************************************************/
void MaDevDrv_ClearIrqProc( void )
{
	pgMaDevDrvInfo->bStatusReg		= 0x80;	
	pgMaDevDrvInfo->bIrqProc		= 0;
	pgMaDevDrvInfo->bMaskInterrupt	= 0;

	machdep_WriteStatusFlagReg( 0x80 );
}

/****************************************************************************
 *	MaDevDrv_EnableIrq
 *
 *	Description:
 *			Enable Irq
 *	Argument:
 *			None
 *	Return:
 *			None
 *
 ****************************************************************************/
void MaDevDrv_EnableIrq( void )
{
	/* enable interrupt if needed */
	if ( (--pgMaDevDrvInfo->bMaskInterrupt)==0 )
	{
		WriteStatusFlagReg( MAB_IRQ_ENABLE );
	}
}

/****************************************************************************
 *	MaDevDrv_DisableIrq
 *
 *	Description:
 *			Disable Irq
 *	Argument:
 *			None
 *	Return:
 *			None
 *
 ****************************************************************************/
void MaDevDrv_DisableIrq( void )
{
	/*Diasble IRQ if necessary and increment access counter */
	if( pgMaDevDrvInfo->bMaskInterrupt++==0 )
	{
		WriteStatusFlagReg( 0 );
	}
}

/****************************************************************************
 *	MaDevDrv_VerifyRegisters
 *
 *	Description:
 *			Verify the initialized registers by software reset
 *	Argument:
 *			None
 *	Return:
 *			0		success
 *			-1		error
 *
 ****************************************************************************/
SINT32 MaDevDrv_VerifyRegisters( void )
{
	SINT32	sdResult;
	UINT32	i;

	sdResult = MASMW_SUCCESS;

	for( i=MAC_CHANNEL_VOLUME ; i<MAC_CHANNEL_VOLUME+16 ; i++ )
	{
		sdResult = MaDevDrv_ReceiveData( i );
		if ( sdResult < MASMW_SUCCESS )
		{
			return sdResult;
		}
		if( (sdResult&0xff)!=0x60 ) return MASMW_ERROR_SOFTRESET;
	}
	for( i=MAC_CHANNEL_PANPOT ; i<MAC_CHANNEL_PANPOT+16 ; i++ )
	{
		sdResult = MaDevDrv_ReceiveData( i );
		if ( sdResult < MASMW_SUCCESS )
		{
			return sdResult;
		}
		if( (sdResult&0xff)!=0x3c ) return MASMW_ERROR_SOFTRESET;
	}
	return MASMW_SUCCESS;
}

/****************************************************************************
 *	StreamSetup
 *
 *	Description:
 *			Setup wave data to the stream audio.
 *	Arguments:
 *			dSaId	stream audio slot number
 *	Return:
 *			0		success
 *			< 0		error code
 *
 ****************************************************************************/
static SINT32 StreamSetup( UINT32 dSaId )
{
	UINT8	bZeroVal;
	UINT32	dFormat;
	UINT8 *	pbWaveData;
	UINT32	dWaveSize;
	UINT32	dRamAdrs;
	SINT32	sdResult;

	/* error  */
	if ( dSaId >= MA_MAX_STREAM_AUDIO ) return MASMW_ERROR;

	/* clear StreamPG counter */
	MaDevDrv_WriteIntermediateReg( MAI_STM_CTRL, (UINT8)(0x01<<dSaId) );
	MaDevDrv_WriteIntermediateReg( MAI_STM_CTRL, 0 );

	/* get stream info */
	dFormat    = pgMaDevDrvInfo->StreamInfo[dSaId].dFormat;
	pbWaveData = pgMaDevDrvInfo->StreamInfo[dSaId].pbWaveData;
	dWaveSize  = pgMaDevDrvInfo->StreamInfo[dSaId].dWaveSize;

	dRamAdrs = (UINT32)(MA_RAM_START_ADDRESS + (MA_RAM_BLOCK_SIZE * (7-dSaId)) + 0x20);

	if ( dWaveSize == MA_WAVE_SIZE )
	{
		pgMaDevDrvInfo->StreamInfo[dSaId].dEndPoint = dWaveSize;
	}
	else
	{
		if ( (dFormat&0x7f) < 2 )				/* ADPCM */
		{
			pgMaDevDrvInfo->StreamInfo[dSaId].dEndPoint = (dWaveSize % MA_WAVE_SIZE);
		}
		else if( (dFormat&0x7f) <= 3 )			/* 8bit PCM */
		{
			if( dFormat&0x80 )
				pgMaDevDrvInfo->StreamInfo[dSaId].dEndPoint =
					( (dWaveSize + (dWaveSize / MA_WAVE_SIZE)*2 ) % MA_WAVE_SIZE );
			else
				pgMaDevDrvInfo->StreamInfo[dSaId].dEndPoint =
					( (dWaveSize + (dWaveSize / MA_WAVE_SIZE) ) % MA_WAVE_SIZE );
		}
		else 									/* 16bit PCM */
		{
			if( dFormat&0x80 )
				pgMaDevDrvInfo->StreamInfo[dSaId].dEndPoint =
					( (dWaveSize + (dWaveSize / MA_WAVE_SIZE)*4 ) % MA_WAVE_SIZE );
			else
				pgMaDevDrvInfo->StreamInfo[dSaId].dEndPoint =
					( (dWaveSize + (dWaveSize / MA_WAVE_SIZE)*2 ) % MA_WAVE_SIZE );
		}
	}

	pgMaDevDrvInfo->dStreamEoInfo	= 0;

	if ( dWaveSize > MA_WAVE_SIZE )
	{
		/* write wave data to RAM */
		sdResult = MaDevDrv_SendDirectRamData( dRamAdrs, 0, pbWaveData, MA_WAVE_SIZE );

		/* error  */
		if ( sdResult != MASMW_SUCCESS )
		{
			return sdResult;
		}

		if ( (dFormat&0x7f) < 2 )				/* ADPCM */
		{
			dWaveSize   -= MA_WAVE_SIZE;
			pbWaveData  += MA_WAVE_SIZE;
		}
		else if( (dFormat&0x7f) <=3 )				/* 8bit PCM */
		{
			/* stereo */
			if( dFormat&0x80 )
			{
				dWaveSize   -= (MA_WAVE_SIZE - 2);
				pbWaveData  += (MA_WAVE_SIZE - 2);
			}
			/* mono */
			else
			{
				dWaveSize   -= (MA_WAVE_SIZE - 1);
				pbWaveData  += (MA_WAVE_SIZE - 1);
			}
		}
		else						/* 16bit PCM */
		{
			/* stereo */
			if( dFormat&0x80 )
			{
				dWaveSize   -= (MA_WAVE_SIZE - 4);
				pbWaveData  += (MA_WAVE_SIZE - 4);
			}
			/* mono */
			else
			{
				dWaveSize   -= (MA_WAVE_SIZE - 2);
				pbWaveData  += (MA_WAVE_SIZE - 2);
			}
		}
	}
	else
	{
		/* write wave data to RAM */
		sdResult = MaDevDrv_SendDirectRamData( dRamAdrs, 0, pbWaveData, dWaveSize );

		/* error  */
		if ( sdResult != MASMW_SUCCESS )
		{
			return sdResult;
		}

		/* set Zero value */
		switch( dFormat&0x7f ) {
			case 0:					/* MA-2 ADPCM */
			case 1:					/* ADPCM */
			case 2:					/* 8-bit ofset binary */
				bZeroVal = (UINT8)0x80;
				break;
			case 3:					/* 8-bit 2's comp binary */
			case 4:					/* 16-bit 2's comp binary */
				bZeroVal = 0x00;
				break;
			default:
				bZeroVal = 0x00;
				break;
		}

		/* write Zero value */
		if( dFormat==0x80 || dFormat==0x81 )
			sdResult = WriteStereoAdpcmZeroVal( dRamAdrs + dWaveSize, MA_WAVE_SIZE - dWaveSize );
		else
			sdResult = MaDevDrv_SendDirectRamVal( dRamAdrs + dWaveSize, MA_WAVE_SIZE - dWaveSize, bZeroVal );

		/* error  */
		if ( sdResult != MASMW_SUCCESS)
		{
			return sdResult;
		}
		pbWaveData  += dWaveSize;
		dWaveSize   -= dWaveSize;
	}

	/* update stream info */
	pgMaDevDrvInfo->StreamInfo[dSaId].pbWaveData  = pbWaveData;
	pgMaDevDrvInfo->StreamInfo[dSaId].dWaveSize   = dWaveSize;

	return MASMW_SUCCESS;
}
/****************************************************************************
 *	StreamUpdate
 *
 *	Description:
 *			Update wave data to the stream audio.
 *	Arguments:
 *			dSaId	stream audio slot number
 *	Return:
 *			0		success
 *			< 0		error code
 *
 ****************************************************************************/
static SINT32 StreamUpdate( UINT32 dSaId )
{
	SINT32	sdResult = MASMW_SUCCESS;
	UINT32	dFormat;
	UINT8 *	pbWaveData;
	UINT32	dWaveSize;
	UINT32	dPgPoint;
	UINT32	dPrvPoint;
	UINT32	dEndPoint;
	UINT32	dWriteSize;
	UINT32	dRamAdrs;
	UINT8	bZeroVal = 0x00;
	UINT32	dRegIndex;
	UINT8	bPacket[6];
	UINT8	bCh;

	if( pgMaDevDrvInfo->StreamInfo[dSaId].bState==MADEVDRV_STREAM_STATE_IDLE )
		return sdResult;

	/* read current PG */
	dPgPoint = (UINT32)MaDevDrv_ReadIntermediateReg( MAI_STREAM_PG_0 + dSaId );

	dPrvPoint	= pgMaDevDrvInfo->StreamInfo[dSaId].dPrvPoint;
	dFormat 	= pgMaDevDrvInfo->StreamInfo[dSaId].dFormat;

	dPgPoint = (UINT32)((dPgPoint & 0x3F) << 4);

	/* set Zero value */
	switch( dFormat&0x7f ) {
		case 0:					/* MA-2 ADPCM */
		case 1:					/* ADPCM */
		case 2:					/* 8-bit ofset binary */
			bZeroVal = (UINT8)0x80;
			break;
		case 3:					/* 8-bit 2's comp binary */
		case 4:					/* 16-bit 2's comp binary */
			bZeroVal = 0x00;
			break;
		default:
			bZeroVal = 0x00;
			break;
	}
	/* check PG */
	if ( dPgPoint == dPrvPoint )
	{
		return sdResult;
	}

	pbWaveData = pgMaDevDrvInfo->StreamInfo[dSaId].pbWaveData;
	dWaveSize  = pgMaDevDrvInfo->StreamInfo[dSaId].dWaveSize;


	dRamAdrs = dPrvPoint + (MA_RAM_START_ADDRESS + (MA_RAM_BLOCK_SIZE * (7-dSaId)) + 0x20);

	if( dPrvPoint < dPgPoint )
	{
	/*                Prv                Pg
	        0          V                 V                1023
			+--+--------------------------------------------+
			|32|       |xxxxxxxxxxxxxxxxx|                  |
			+--+--------------------------------------------+

				MA_WAVE_SIZE = 1024 -32
				MA_MAX_PG    = 1024 -32
	*/

		dWriteSize = dPgPoint - dPrvPoint;

		if( dWaveSize >= dWriteSize )
		{
		/*            Prv                Pg
	        0          V                 V                1023
			+--+--------------------------------------------+
			|32|       |ddddddddddddddddd|                  |
			+--+--------------------------------------------+
		*/
			/* write data */
			sdResult = MaDevDrv_SendDirectRamData( dRamAdrs, 0, pbWaveData, dWriteSize );
			/* error */
			if ( sdResult != MASMW_SUCCESS )
			{
				return sdResult;
			}
			pbWaveData += dWriteSize;
			dWaveSize  -= dWriteSize;
		}
		else
		{
		/*            Prv                Pg
	        0          V                 V                1023
			+--+--------------------------------------------+
			|32|       |ddddddddddd|00000|                  |
			+--+--------------------------------------------+
		*/
			if( dWaveSize != 0 )
			{
				/* write data */
				sdResult = MaDevDrv_SendDirectRamData( dRamAdrs, 0, pbWaveData, dWaveSize );
				/* error */
				if ( sdResult != MASMW_SUCCESS )
				{
					return sdResult;
				}

				pbWaveData += dWaveSize;
				dWriteSize -= dWaveSize;
				dRamAdrs   += dWaveSize;
				dWaveSize   = 0;
			}

			/* write ZERO value */
			if( dFormat==0x80 || dFormat==0x81 )
				sdResult = WriteStereoAdpcmZeroVal( dRamAdrs, dWriteSize );
			else
				sdResult = MaDevDrv_SendDirectRamVal( dRamAdrs, dWriteSize, bZeroVal );
			/* error */
			if ( sdResult != MASMW_SUCCESS )
			{
				return sdResult;
			}
		}
	}

	else
	{
	/*                 Pg               Prv
	        0          V                 V                1023
			+--+--------------------------------------------+
			|32|xxxxxxx|                 |xxxxxxxxxxxxxxxxxx|
			+--+--------------------------------------------+
	*/
		dWriteSize = MA_MAX_PG - dPrvPoint;
		if( dWaveSize >= dWriteSize )
		{
		/*                 Pg               Prv
		        0          V                 V                1023
				+--+--------------------------------------------+
				|32|       |                 |dddddddddddddddddd|
				+--+--------------------------------------------+
		*/
			/* write data */
			sdResult = MaDevDrv_SendDirectRamData( dRamAdrs, 0, pbWaveData, dWriteSize );
			/* error */
			if ( sdResult != MASMW_SUCCESS )
			{
				return sdResult;
			}

			/* PCM ? */
			if( (dFormat&0x7f)>1 )
			{
				switch( dFormat ) {
					case 0x02:		/* 8bit mono, offset bin */
					case 0x03:		/* 8bit mono, 2's comp */
						--dWriteSize;
						break;
					case 0x82:		/* 8bit stereo, offset bin */
					case 0x83:		/* 8bit stereo, 2's comp */
					case 0x04:		/* 16bit mono */
						dWriteSize -= 2;
						break;
					case 0x84:		/* 16bit stereo */
						dWriteSize -= 4;
						break;
					default:
						break;
				}
			}
			pbWaveData += dWriteSize;
			dWaveSize  -= dWriteSize;
		}
		else
		{
		/*                 Pg               Prv
		        0          V                 V                1023
				+--+--------------------------------------------+
				|32|       |                 |dddddddddddd000000|
				+--+--------------------------------------------+
		*/
			if( dWaveSize != 0 )
			{
				/* write data */
				sdResult = MaDevDrv_SendDirectRamData( dRamAdrs, 0, pbWaveData, dWaveSize );
				/* error */
				if ( sdResult != MASMW_SUCCESS )
				{
					return sdResult;
				}

				pbWaveData += dWaveSize;
				dRamAdrs   += dWaveSize;
				dWriteSize -= dWaveSize;
				dWaveSize   = 0;
			}

			/* write ZERO value */
			if( dFormat==0x80 || dFormat==0x81 )
				sdResult = WriteStereoAdpcmZeroVal( dRamAdrs, dWriteSize );
			else
				sdResult = MaDevDrv_SendDirectRamVal( dRamAdrs, dWriteSize, bZeroVal );
			/* error */
			if ( sdResult != MASMW_SUCCESS )
			{
				return sdResult;
			}
		}

		dWriteSize = dPgPoint;
		dRamAdrs = MA_MIN_PG + (MA_RAM_START_ADDRESS + (MA_RAM_BLOCK_SIZE * (7-dSaId)) + 0x20);

		if( dWaveSize >= dWriteSize )
		{
		/*                 Pg               Prv
		        0          V                 V                1023
				+--+--------------------------------------------+
				|32|ddddddd|                 |                  |
				+--+--------------------------------------------+
		*/
			/* write data */
			sdResult = MaDevDrv_SendDirectRamData( dRamAdrs, 0, pbWaveData, dWriteSize );
			/* error */
			if ( sdResult != MASMW_SUCCESS )
			{
				return sdResult;
			}

			pbWaveData += dWriteSize;
			dWaveSize  -= dWriteSize;
		}
		else
		{
		/*                 Pg               Prv
		        0          V                 V                1023
				+--+--------------------------------------------+
				|32|ddd0000|                 |                  |
				+--+--------------------------------------------+
		*/
			if( dWaveSize != 0 )
			{
				/* write data */
				sdResult = MaDevDrv_SendDirectRamData( dRamAdrs, 0, pbWaveData, dWaveSize );
				/* error */
				if ( sdResult != MASMW_SUCCESS )
				{
					return sdResult;
				}

				pbWaveData += dWaveSize;
				dRamAdrs   += dWaveSize;
				dWriteSize -= dWaveSize;
				dWaveSize   = 0;
			}

			/* write ZERO value */
			if( dFormat==0x80 || dFormat==0x81 )
				sdResult = WriteStereoAdpcmZeroVal( dRamAdrs, dWriteSize );
			else
				sdResult = MaDevDrv_SendDirectRamVal( dRamAdrs, dWriteSize, bZeroVal );
			/* error */
			if ( sdResult != MASMW_SUCCESS )
			{
				return sdResult;
			}
		}
	}

	/* update stream info */

	/* data pointer */
	pgMaDevDrvInfo->StreamInfo[dSaId].pbWaveData = pbWaveData;
	/* data size */
	pgMaDevDrvInfo->StreamInfo[dSaId].dWaveSize  = dWaveSize;
	/* position */
	pgMaDevDrvInfo->StreamInfo[dSaId].dPrvPoint  = dPgPoint;

	if( pgMaDevDrvInfo->StreamInfo[dSaId].bState==MADEVDRV_STREAM_STATE_ENDOFDATA )
	{

		dEndPoint = pgMaDevDrvInfo->StreamInfo[dSaId].dEndPoint;

	/************************************************************

			      Prv        End       Pg
	  Case I       v          v        v
			     ------------------------


			       Pg        Prv      End
	  Case II      v          v        v
			     ------------------------


		    	  End         Pg      Prv
	  Case III     v          v        v
			     ------------------------

	*************************************************************/

		if( (dPrvPoint< dEndPoint && dEndPoint<=dPgPoint)  ||
			(dPgPoint < dPrvPoint && dPrvPoint< dEndPoint) ||
			(dEndPoint<=dPgPoint  && dPgPoint < dPrvPoint) )
		{
			pgMaDevDrvInfo->StreamInfo[dSaId].bState = MADEVDRV_STREAM_STATE_IDLE;

			/* make packet for KeyOff */
			dRegIndex = (UINT32)( MAC_EXT_WT_VOICE_ADDRESS+(9+dSaId)*6 + 5 );

			sdResult = MaDevDrv_ReceiveData( dRegIndex );
			/* error  */
			if ( sdResult < MASMW_SUCCESS )
			{
				return sdResult;
			}
			bCh = (UINT8)( sdResult & 0x0F );

			bPacket[0] = (UINT8)(  dRegIndex       & 0x7F );		/* address part */
			bPacket[1] = (UINT8)( (dRegIndex >> 7) | 0x80 );
			bPacket[2] = (UINT8)(  0x30 | bCh      | 0x80 );		/* data part, KeyOff */

			/* write direct packet */
			sdResult = MaDevDrv_SendDirectPacket( bPacket, 3 );

			/* error */
			if ( sdResult != MASMW_SUCCESS )
			{
				return sdResult;
			}

			/* clear STEREO bit for stereo data */
			if(dSaId==0 && (pgMaDevDrvInfo->StreamInfo[dSaId].dFormat&0x80) )
			{
				dRegIndex += 6;
				bPacket[0] = (UINT8)(  dRegIndex       & 0x7F );		/* address part */
				bPacket[1] = (UINT8)( (dRegIndex >> 7) | 0x80 );
				bPacket[2] = (UINT8)(  0x30 | bCh      | 0x80 );		/* data part, KeyOff */

				/* write direct packet */
				sdResult = MaDevDrv_SendDirectPacket( bPacket, 3 );

				/* error */
				if ( sdResult != MASMW_SUCCESS )
				{
					return sdResult;
				}

				/* wait 20us */
				machdep_Wait( 20000 );

				/* clear STEREO bit */
				MaDevDrv_WriteIntermediateReg( MAI_STM_CTRL, 0 );
			}

			if ( pgMaDevDrvInfo->dSeqId == 1 )
			{
			}
		}
		else
		{
			pgMaDevDrvInfo->StreamInfo[dSaId].dPrvPoint = dPgPoint;
		}
	}
	/* change state to ENDOFDATA */
	else if( dWaveSize==0 )
	{
		pgMaDevDrvInfo->StreamInfo[dSaId].bState=MADEVDRV_STREAM_STATE_ENDOFDATA;
	}

	return sdResult;
}
/****************************************************************************
 *	ControlInterrupt
 *
 *	Description:
 *			Enable or disable the interrupt.
 *	Arguments:
 *			dIrqFlagReg	IRQ control reg
 *			dCtrl		control flag (0: enable, 1: disable)
 *			bIntCtrl	interrupt number
 *	Return:
 *			MASMW_SUCCESS(0)
 *
 ****************************************************************************/
static SINT32 ControlInterrupt( UINT32 dIrqCtrlReg, UINT32 dCtrl, UINT8 bIntCtrl )
{
	UINT8	bData;

	MADEVDRV_DBGMSG(("ControlInterrupt: dIrqCtrlReg=%ld, dCtrl=%ld, bIntCtrl=%02X\n", dIrqCtrlReg, dCtrl, bIntCtrl));

	bData = MaDevDrv_ReadIntermediateReg( dIrqCtrlReg );

	if ( dCtrl==ENABLE_IRQ )		/* enable */
	{
		bData |= bIntCtrl;
	}
	else							/* diable */
	{
		bData &= (UINT8)( ~bIntCtrl );
	}

	MaDevDrv_WriteIntermediateReg( dIrqCtrlReg, bData );
	return MASMW_SUCCESS;
}
/****************************************************************************
 *	SoftInt0
 *
 *	Description:
 *			Software interrupt #0 interrupt handler.
 *	Argument:
 *			dCtrl	unused
 *	Return:
 *			None
 *
 ****************************************************************************/
static void SoftInt0( UINT32 dCtrl )
{
	UINT8	bRamVal;					/* ram value */
	UINT8	bMaster;

	(void)dCtrl;						/* for unused warning message */

	MADEVDRV_DBGMSG(("SoftInt0: dCtrl=%ld\n", dCtrl));

	/* read SoftIntRAM #0 */
	bRamVal = MaDevDrv_ReadIntermediateReg( MAI_SOFTINT_RAM_0 );

	switch( bRamVal&0x60 )
	{
	case 0x00:		/* stop  */
	case 0x40:		/* start */
		MaDevDrv_StreamHandler( 0, 0, 0, bRamVal, 0 );	/* SA #0, setup */
		break;
	case 0x20:		/* slave */
		if( pgMaDevDrvInfo->bSlave!=0xff )
		{
			bMaster = pgMaDevDrvInfo->bSlave;
			pgMaDevDrvInfo->bSlave		= (UINT8)(bRamVal&0x1f);
			MaDevDrv_StreamHandler( 0, 1, 0, bMaster, 0 );	/* SA #1, setup */
		}
		else
		{
			pgMaDevDrvInfo->bSlave		= (UINT8)(bRamVal&0x1f);
		}
		break;
	case 0x60:		/* master */
		if( pgMaDevDrvInfo->bSlave!=0xff )
		{
			MaDevDrv_StreamHandler( 0, 0, 0, bRamVal, 0 );	/* SA #0, setup */
		}
		else
		{
			pgMaDevDrvInfo->bSlave		= (UINT8)(bRamVal);
		}
		break;
	}
}
/****************************************************************************
 *	SoftInt1
 *
 *	Description:
 *			Software interrupt #1 interrupt handler.
 *	Argument:
 *			dCtrl	unused
 *	Return:
 *			None
 *
 ****************************************************************************/
static void SoftInt1( UINT32 dCtrl )
{
	UINT8	bRamVal;					/* ram value */
	UINT8	bMaster;

	(void)dCtrl;						/* for unused warning message */

	MADEVDRV_DBGMSG(("SoftInt1: dCtrl=%ld\n", dCtrl));

	/* read SoftIntRAM#1 */
	bRamVal = MaDevDrv_ReadIntermediateReg( MAI_SOFTINT_RAM_1 );

	switch( bRamVal&0x60 )
	{
	case 0x00:		/* stop  */
	case 0x40:		/* start */
		MaDevDrv_StreamHandler( 0, 1, 0, bRamVal, 0 );	/* SA #1, setup */
		break;
	case 0x20:		/* slave */
		if( pgMaDevDrvInfo->bSlave!=0xff )
		{
			bMaster = pgMaDevDrvInfo->bSlave;
			pgMaDevDrvInfo->bSlave		= (UINT8)(bRamVal&0x1f);
			MaDevDrv_StreamHandler( 0, 0, 0, bMaster, 0 );	/* SA #0, setup */
		}
		else
		{
			pgMaDevDrvInfo->bSlave		= (UINT8)(bRamVal&0x1f);
		}
		break;
	case 0x60:		/* master */
		if( pgMaDevDrvInfo->bSlave!=0xff )
		{
			MaDevDrv_StreamHandler( 0, 1, 0, bRamVal, 0 );	/* SA #1, setup */
		}
		else
		{
			pgMaDevDrvInfo->bSlave		= (UINT8)(bRamVal);
		}
		break;
	}
}
/****************************************************************************
 *	SoftInt2
 *
 *	Description:
 *			Software interrupt #2 interrupt handler.
 *	Argument:
 *			dCtrl	unused
 *	Return:
 *			None
 *
 ****************************************************************************/
static void SoftInt2( UINT32 dCtrl )
{
	UINT8	bRamVal;					/* ram value */

	(void)dCtrl;						/* for unused warning message */

	MADEVDRV_DBGMSG(("SoftInt2: dCtrl=%ld\n", dCtrl));

	/* read SoftIntRAM #2 */
	bRamVal = MaDevDrv_ReadIntermediateReg( MAI_SOFTINT_RAM_2 );

	MaSound_ReceiveMessage( 0, 0, bRamVal );
}
/****************************************************************************
 *	SoftInt3
 *
 *	Description:
 *			Software interrupt #3 interrupt handler.
 *	Argument:
 *			dCtrl	unused
 *	Return:
 *			None
 *
 ****************************************************************************/
static void SoftInt3( UINT32 dCtrl )
{
	UINT8	bRamVal;					/* ram value */
	UINT32	sdResult;

	(void)dCtrl;						/* for unused warning message */

	MADEVDRV_DBGMSG(("SoftInt3: dCtrl=%ld\n", dCtrl));


	sdResult = machdep_WaitSequencerStop();

	/* error  */
	if ( sdResult != MASMW_SUCCESS )
	{
		return;
	}
	/* read SoftIntRAM #3 */
	bRamVal = MaDevDrv_ReadIntermediateReg( MAI_SOFTINT_RAM_3 );

	if( pgMaDevDrvInfo->dEndofSequence==1 )
	{
		/* Delayed FIFO empty ? */
		if( machdep_CheckDelayedFifoEmpty() )
			pgMaDevDrvInfo->dEndofSequence = 0;
	}

	MaSound_ReceiveMessage( 0, 0, bRamVal );
}
/****************************************************************************
 *	SoftInt4
 *
 *	Description:
 *			Software interrupt #4 interrupt handler.
 *	Argument:
 *			dCtrl	unused
 *	Return:
 *			None
 *
 ****************************************************************************/
static void SoftInt4( UINT32 dCtrl )
{
	UINT8	bRamVal;					/* ram value */
	UINT8	bCtrlVal;

	(void)dCtrl;						/* for unused warning message */

	MADEVDRV_DBGMSG(("SoftInt4: dCtrl=%ld\n", dCtrl));

	/* read SoftIntRAM #4 */
	bRamVal = MaDevDrv_ReadIntermediateReg( MAI_SOFTINT_RAM_4 );
	
	if ( bRamVal == 127 )
	{
		bCtrlVal = 191;
	}
	else
	{
		bCtrlVal = (UINT8)(bRamVal + 192);
	}

	MaSound_ReceiveMessage( 0, 0, bCtrlVal );
}
/****************************************************************************
 *	StreamPG #0
 *
 *	Description:
 *			StreamPG #0 interrupt handler.
 *	Argument:
 *			dCtrl	unused
 *	Return:
 *			None
 *
 ****************************************************************************/
static void StreamPg0( UINT32 dCtrl )
{

	(void)dCtrl;						/* for unused warning message */

	MADEVDRV_DBGMSG(("StreamPg#0: dCtrl=%ld\n", dCtrl));

	/* melody mode, stream#0, update  */
	StreamUpdate( 0 );
}
/****************************************************************************
 *	StreamPG #1
 *
 *	Description:
 *			StreamPG #1 interrupt handler.
 *	Argument:
 *			dCtrl	unused
 *	Return:
 *			None
 *
 ****************************************************************************/
static void StreamPg1( UINT32 dCtrl )
{

	(void)dCtrl;						/* for unused warning message */

	MADEVDRV_DBGMSG(("StreamPg#1: dCtrl=%ld\n", dCtrl));

	/* melody mode, stream#1, update  */
	StreamUpdate( 1 );
}

/****************************************************************************
 *	Timer2
 *
 *	Description:
 *			Timer #2 interrupt handler.
 *	Argument:
 *			dCtrl	unused
 *	Return:
 *			None
 *
 ****************************************************************************/
static void Timer2( UINT32 dCtrl )
{
	SINT32	sdResult;

	(void)dCtrl;						/* for unused warning message */

	MADEVDRV_DBGMSG(("Timer#2: dCtrl=%ld\n", dCtrl));

	if( pgMaDevDrvInfo->StreamInfo[0].bState != MADEVDRV_STREAM_STATE_IDLE ) 
	{
		sdResult = StreamUpdate( 0 );

		/* error  */
		if ( sdResult != MASMW_SUCCESS )
		{
			return;
		}
	}
	if( pgMaDevDrvInfo->StreamInfo[1].bState != MADEVDRV_STREAM_STATE_IDLE )
	{
		sdResult = StreamUpdate( 1 );

		/* error  */
		if ( sdResult != MASMW_SUCCESS )
		{
			return;
		}
	}
	if( pgMaDevDrvInfo->dActiveSequencer & MADEVDRV_ACTIVE_PHRASESEQ ) {
		sdResult = MaSndDrv_UpdateSequence( MASNDDRV_UPDATE_TIMER2A, NULL, 0 );
	}
	if( pgMaDevDrvInfo->dActiveSequencer & MADEVDRV_ACTIVE_HVSEQ ) {
		sdResult = MaSndDrv_UpdateSequence( MASNDDRV_UPDATE_TIMER2B, NULL, 0 );
	}
}
/****************************************************************************
 *	InitRegisters
 *
 *	Description:
 *			Initialize the uninitialized registers by software reset
 *	Argument:
 *			None
 *	Return:
 *			None
 *
 ****************************************************************************/
static void InitRegisters( void )
{
	UINT32	dCount;

	static const UINT8 bIndex[10] = { 7, 8, 9, 10, 11, 12, 13, 14, 15, 63 };
#if (MA_DEVICETYPE == MASMW_DEVICETYPE_MA5S)
	static const UINT8 bIndex3[1]  = { 9 };
#elif (MA_DEVICETYPE == MASMW_DEVICETYPE_MA5I)
	static const UINT8 bIndex3[23] = { 27, 28, 29, 30, 31, 32, 33, 34, 35, 
	                                  21, 22, 23, 24, 25, 26, 
	                                  10, 11, 12, 18, 19, 20, 16, 17 };
	static const UINT8 bDefault3[23] = { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	                                     0x10, 0x00, 0x00, 0x00, 0x00, 0x00, 
	                                     0x00, 0x00, 0x0F, 0x03, 0x60, 0x82, 0x87, 0x1F };
#endif

	MADEVDRV_DBGMSG(("InitRegisters:\n"));

	/* set BANK bits of REG_ID #4 basic setting register to '0' */
	machdep_WriteStatusFlagReg( MAI_BASIC_SETTING );
	machdep_WriteDataReg( 0x00 );

	for ( dCount = 0; dCount < 10; dCount++ )
	{
		/* set register index to REG_ID */
		machdep_WriteStatusFlagReg( bIndex[dCount] );

		/* write data to the target register */
		machdep_WriteDataReg( 0x00 );
	}
	
#if (MA_DEVICETYPE == MASMW_DEVICETYPE_MA5S)
	/* MA-5S */
	
	/* set BANK bits of REG_ID #4 basic setting register to '3' */
	machdep_WriteStatusFlagReg( MAI_BASIC_SETTING );
	machdep_WriteDataReg( 0x03 );
	
	/* set register index to REG_ID */
	machdep_WriteStatusFlagReg( bIndex3[0] );

	/* write data to the target register */
	machdep_WriteDataReg( 0x0F );

	/* set BANK bits of REG_ID #4 basic setting register to '0' */
	machdep_WriteStatusFlagReg( MAI_BASIC_SETTING );
	machdep_WriteDataReg( 0x00 );
	
#elif (MA_DEVICETYPE == MASMW_DEVICETYPE_MA5I)
	/* MA-5S */
	
	/* set BANK bits of REG_ID #4 basic setting register to '3' */
	machdep_WriteStatusFlagReg( MAI_BASIC_SETTING );
	machdep_WriteDataReg( 0x03 );
	
	for ( dCount = 0; dCount < 23; dCount++ )
	{
		/* set register index to REG_ID */
		machdep_WriteStatusFlagReg( bIndex3[dCount] );

		/* write data to the target register */
		machdep_WriteDataReg( bDefault3[dCount] );
	}

	/* set BANK bits of REG_ID #4 basic setting register to '0' */
	machdep_WriteStatusFlagReg( MAI_BASIC_SETTING );
	machdep_WriteDataReg( 0x00 );
#endif

}
/****************************************************************************
 *	WriteStatusFlagReg
 *
 *	Description:
 *			Write and save Status Flag Reg
 *	Arguments:
 *			bReg	write data
 *	Return:
 *			previous data
 *
 ****************************************************************************/
static UINT8	WriteStatusFlagReg( UINT8 bReg )
{
	UINT8	bPrevStatus;

	/* previous setting */
	bPrevStatus = pgMaDevDrvInfo->bStatusReg;

	/* save */
	pgMaDevDrvInfo->bStatusReg = bReg;

	if ( pgMaDevDrvInfo->bIrqProc == 1) bReg &= 0x7f;

	/* write data to status flag reg */
	machdep_WriteStatusFlagReg( bReg );

	return (bPrevStatus);
}
/****************************************************************************
 *	WriteStereoAdpcmZeroVal
 *
 *	Description:
 *			Write stereo ADPCM zero value
 *	Arguments:
 *			dAddress	internal RAM address
 *			dSize		data size
 *	Return:
 *			0		  success
 *			< 0		  error code
 *
 ****************************************************************************/
static SINT32	WriteStereoAdpcmZeroVal( UINT32 dAddress, UINT32 dSize )
{
	SINT32 sdResult;
	UINT32 i,j;
	UINT8  bZeroVal;

	if ( dSize == 0 ) return MASMW_SUCCESS;

	if( dAddress+dSize>MA_MEMORY_SIZE ) return MASMW_ERROR;

	/* increment Irq access counter */
	pgMaDevDrvInfo->bMaskInterrupt++;

	/* set REG_ID #2   */
	WriteStatusFlagReg( MAI_IMMEDIATE_WRITE );

	/* check status */
	sdResult = machdep_WaitImmediateFifoEmpty();

	/* error */
	if ( sdResult < MASMW_SUCCESS )
	{
		/* enable interrupt if needed */
		MaDevDrv_EnableIrq();
		return sdResult;
	}

	/* write direct packet */

	/* Address part */
	machdep_WriteDataReg( (UINT8)((dAddress >>  0 ) & 0x7F) );
	machdep_WriteDataReg( (UINT8)((dAddress >>  7 ) & 0x7F) );
	machdep_WriteDataReg( (UINT8)((dAddress >> 14 ) | 0x80) );

	/* size */
	if	( dSize < 0x0080 )
	{
		machdep_WriteDataReg( (UINT8)( (dSize >> 0) | 0x80 ) );
	}
	else
	{
		machdep_WriteDataReg( (UINT8)( (dSize >> 0) & 0x7F ) );
		machdep_WriteDataReg( (UINT8)( (dSize >> 7) | 0x80 ) );
	}

	i = dSize/64;
	while ( i-- != 0 )
	{
		/* check status */
		sdResult = machdep_WaitImmediateFifoEmpty();

		/* error  */
		if ( sdResult < MASMW_SUCCESS )
		{
			/* enable interrupt if needed */
			MaDevDrv_EnableIrq();
			return sdResult;
		}

		/* write 64 bytes data */
		j = 64;
		while( j-- != 0 )
		{
			bZeroVal = (UINT8)(( pgMaDevDrvInfo->dStreamEoInfo&0x01 ) ? 0x00 : 0x88);
			pgMaDevDrvInfo->dStreamEoInfo ^= 0x01;
			machdep_WriteDataReg( bZeroVal );
		}
	}

	/* write odd data */
	if ( (j=(dSize%64)) != 0 )
	{
		/* check status */
		sdResult = machdep_WaitImmediateFifoEmpty();

		/* error  */
		if ( sdResult < MASMW_SUCCESS )
		{
			/* enable interrupt if needed */
			MaDevDrv_EnableIrq();
			return sdResult;
		}
		/* write data */
		while( (j--)!=0 )
		{
			bZeroVal = (UINT8)(( pgMaDevDrvInfo->dStreamEoInfo&0x01 ) ? 0x00 : 0x88);
			pgMaDevDrvInfo->dStreamEoInfo ^= 0x01;
			machdep_WriteDataReg( bZeroVal );
		}
	}

	/* enable interrupt if needed */
	if ( (--pgMaDevDrvInfo->bMaskInterrupt)==0 )
	{
		WriteStatusFlagReg( MAB_IRQ_ENABLE );
	}

	return MASMW_SUCCESS;
}
