/****************************************************************************
 *
 *		Copyright (C) 2003	YAMAHA CORPORATION. All rights reserved.
 *
 *		Module		: mahvsw.c
 *
 *		Description	: MA HV Sequence Wrapper
 *
 *		Version		: 2.0.0 	2003.07.14
 *
 ****************************************************************************/
#include "masndseq.h"
#include "masnddrv.h"

/****************************************************************************
 *	macro
 ****************************************************************************/
#define	MAHVSW_MAKEDWORD(a, b, c, d)	(UINT32)(((UINT32)(a) << 24) | \
						((UINT32)(b) << 16) | ((UINT32)(c) << 8) | (UINT32)(d))

/****************************************************************************
 *	define
 ****************************************************************************/
#define	MAHVSW_MAX_CALLBACK			(3)
#define	MAHVSW_MAX_VOICE			(16)
#define	MAHVSW_MAX_SCRIPT			(64)
#define	MAHVSW_NOTE_OFF				(191)
#define	MAHVSW_MIN_NOTE				(192)
#define	MAHVSW_MAX_NOTE				(MAHVSW_MIN_NOTE + MAHVSW_MAX_SCRIPT)
#define	MAHVSW_SIZE_OF_CHUNKHEADER	(8)

/****************************************************************************
 *	function
 ****************************************************************************/
static SINT32 CallBack_Dummy0(UINT8);
static SINT32 CallBack_Dummy1(UINT8);
static SINT32 CallBack_Dummy2(UINT8);

/****************************************************************************
 *	typedef
 ****************************************************************************/
typedef struct _tagMAHVSWHOOK
{
	SINT32	sdFuncId;
	SINT32	sdFileId;
	SINT32	(*pfnCallbackFunc)(UINT8);
} MAHVSWHOOK, *PMAHVSWHOOK;

typedef struct _tagMAHVSWDATA
{
	UINT8*	pbData;
	UINT32	dSize;
} MAHVSWDATA, *PMAHVSWDATA;

typedef struct _tagMAHVSWINFO
{
	SINT32		sdFuncId;						/*	*/
	SINT32		sdFileId;						/*	*/
	SINT32		sdHvFuncId;						/*	HV func ID				*/
	SINT32		sdHvFileId;						/*	HV file ID				*/
	UINT32		dVolume;						/*	volume		default:100	*/
	UINT32		dHvVolume;						/*	HV volume	default:127	*/
	SINT32		sdKey;							/*	key			default:0	*/
	SINT32		sdKeyCon;						/*	key control 0:disable 1:enable	*/
	MAHVSWHOOK	sHook[MAHVSW_MAX_CALLBACK];		/*	original callback function	*/
	MAHVSWDATA	sData[MAHVSW_MAX_SCRIPT];
	SINT32		sdDefaultHvLanguage;			/*	Default Language		*/
} MAHVSWINFO, *PMAHVSWINFO;

/****************************************************************************
 *	gloval values
 ****************************************************************************/
static MAHVSWINFO	gHvswInfo;

/****************************************************************************
 *	gloval tables
 ****************************************************************************/
static const SINT32 gsdHookId[MASMW_NUM_SRMCNV] = {
	-1, 0, -1, -1, -1, 1, 2, -1, -1, -1, -1, -1, -1
};

static SINT32	(*gpfnCallback[MAHVSW_MAX_CALLBACK])(UINT8) = {
	CallBack_Dummy0,
	CallBack_Dummy1,
	CallBack_Dummy2
};

/****************************************************************************
 *	IsHookFunc
 *
 *	Function:
 *			examine whether hook or not hook callback function.
 *	Argument:
 *			sdFuncId	... Function number.
 *	Return:
 *			>=0	success.
 *			<0	error.
 *
 ****************************************************************************/
static SINT32	IsHookFunc(SINT32 sdFuncId)
{
	if(sdFuncId < 0)					return (-1);
	if(MASMW_NUM_SRMCNV <= sdFuncId)	return (-1);

	return (gsdHookId[sdFuncId]);
}

/****************************************************************************
 *	MixVolume
 *
 *	Function:
 *			Mix master & HV volume.
 *	Argument:
 *			None
 *	Return:
 *			Mixed Volume(0..127)
 *
 ****************************************************************************/
static UINT32 MixVolume(void)
{
	UINT32	dVolume		= gHvswInfo.dVolume;
	UINT32	dHvVolume	= gHvswInfo.dHvVolume;

	if(dVolume > 127)	dVolume		= 127;
	if(dHvVolume > 127)	dHvVolume	= 127;

	return (MaSndDrv_Mix40LogVol(dVolume, dHvVolume));
}

/****************************************************************************
 *	GetLanguageType
 *
 *	Function:
 *			Get HV languate type.
 *	Argument:
 *			pbData	pointer to HV script.
 *			None	HV script size.
 *	Return:
 *			>=0	success.
 *			<0	error.
 *
 ****************************************************************************/
static SINT32 GetLanguageType(UINT8* pbData, UINT32 dSize)
{
	SINT32	sdHvType;

	if(pbData == NULL)	return (MASMW_ERROR_ARGUMENT);
	if(dSize < 4)		return (MASMW_ERROR_ARGUMENT);

	sdHvType = MASMW_HVTYPE_NONE;

	if(	(pbData[0] == 'H')	&&
		(pbData[1] == 'V')	&&
		(pbData[2] == '#')) {

		switch(pbData[3]) {
		case 'J' :
			sdHvType = MASMW_HVTYPE_JAPANESE;
			break;
		case 'K' :
			sdHvType = MASMW_HVTYPE_KOREAN;
			break;
		}
	}

	MASNDSEQ_DBGMSG(("Language Type %d \n", sdHvType));
	return (sdHvType);
}

/****************************************************************************
 *	GetFisrtLanguageType
 *
 *	Function:
 *			Get HV languate type.
 *	Argument:
 *			pbData	pointer to data buffer.
 *			None	data buffer size.
 *	Return:
 *			>=0	success.
 *			<0	error.
 *
 ****************************************************************************/
static SINT32 GetFirstLanguageType(UINT8* pbData, UINT32 dSize)
{
	SINT32	sdLangType;
	UINT32	dId;
	UINT32	dChunkId;
	UINT32	dChunkSize;

	if(pbData == NULL)								return (MASMW_ERROR_ARGUMENT);
	if(dSize < (UINT32)MAHVSW_SIZE_OF_CHUNKHEADER)	return (MASMW_ERROR_ARGUMENT);

	while(dSize >= (UINT32)MAHVSW_SIZE_OF_CHUNKHEADER) {
		dChunkId	= MAHVSW_MAKEDWORD(pbData[0], pbData[1], pbData[2], pbData[3]);
		dChunkSize	= MAHVSW_MAKEDWORD(pbData[4], pbData[5], pbData[6], pbData[7]);

		/*	Check chunk size	*/
		if(dSize < (UINT32)(MAHVSW_SIZE_OF_CHUNKHEADER + dChunkSize)) {
			return (MASMW_ERROR_CHUNK_SIZE);
		}

		pbData	+= (UINT32)MAHVSW_SIZE_OF_CHUNKHEADER;
		dSize	-= (UINT32)MAHVSW_SIZE_OF_CHUNKHEADER;

		if((dChunkId & 0xFFFFFF00) == MAHVSW_MAKEDWORD('M', 's', 'c', 0)) {
			dId = (dChunkId & 0x000000FF);
			if(dId < MAHVSW_MAX_SCRIPT) {
				sdLangType = GetLanguageType(pbData, dChunkSize);
				return (sdLangType);
			}
		}

		pbData	+= dChunkSize;
		dSize	-= dChunkSize;
	}

	return (MASMW_ERROR);
}

/****************************************************************************
 *	LoadHvScript
 *
 *	Function:
 *			Get HV scripts .
 *	Argument:
 *			pbData	pointer to data buffer.
 *			None	data buffer size.
 *	Return:
 *			>=0	success.
 *			<0	error.
 *
 ****************************************************************************/
static SINT32 LoadHvScript(UINT8* pbData, UINT32 dSize)
{
	UINT32	dId;
	UINT32	dChunkId;
	UINT32	dChunkSize;

	if(pbData == NULL)								return (MASMW_ERROR_ARGUMENT);
	if(dSize < (UINT32)MAHVSW_SIZE_OF_CHUNKHEADER)	return (MASMW_ERROR_ARGUMENT);

	while(dSize >= (UINT32)MAHVSW_SIZE_OF_CHUNKHEADER) {
		dChunkId	= MAHVSW_MAKEDWORD(pbData[0], pbData[1], pbData[2], pbData[3]);
		dChunkSize	= MAHVSW_MAKEDWORD(pbData[4], pbData[5], pbData[6], pbData[7]);

		/*	Check chunk size	*/
		if(dSize < (UINT32)(MAHVSW_SIZE_OF_CHUNKHEADER + dChunkSize)) {
			return (MASMW_ERROR_CHUNK_SIZE);
		}

		if((dChunkId & 0xFFFFFF00) == MAHVSW_MAKEDWORD('M', 's', 'c', 0)) {
			dId = (dChunkId & 0x000000FF);
			if(dId < MAHVSW_MAX_SCRIPT) {
				gHvswInfo.sData[dId].pbData	= (pbData + MAHVSW_SIZE_OF_CHUNKHEADER);
				gHvswInfo.sData[dId].dSize	= dChunkSize;
			}
		}

		pbData	+= dChunkSize + (UINT32)MAHVSW_SIZE_OF_CHUNKHEADER;
		dSize	-= dChunkSize + (UINT32)MAHVSW_SIZE_OF_CHUNKHEADER;
	}

	return (MASMW_SUCCESS);
}

/****************************************************************************
 *	MaHvsw_Initialize
 *
 *	Function:
 *			Initialize MAHVSWINFO structure.
 *	Argument:
 *			None
 *	Return:
 *			None
 *
 ****************************************************************************/
static void	MaHvsw_Initialize(void)
{
	UINT32			i;
	PMAHVSWDATA		pData;

	/*	for target format	*/
	gHvswInfo.sdFuncId		= -1;
	gHvswInfo.sdFileId		= -1;

	/*	for HV	*/
	gHvswInfo.sdHvFuncId	= -1;
	gHvswInfo.sdHvFileId	= -1;
	gHvswInfo.dVolume		= 100;
	gHvswInfo.dHvVolume		= 127;
	gHvswInfo.sdKey			= 0;
	gHvswInfo.sdKeyCon		= 0;

	/*	initialize HV script information	*/
	for(i = 0; i < MAHVSW_MAX_SCRIPT; i++) {
		pData = &(gHvswInfo.sData[i]);
		pData->pbData	= NULL;
		pData->dSize	= 0;
	}
}

/****************************************************************************
 *	MaHvsw_HvNoteOn
 *
 *	Function:
 *			Play HV.
 *	Argument:
 *			dId		HV Sequence Number(0..63)
 *			dEvent	Event Number
 *	Return:
 *			0	success.
 *			<0	error code.
 *
 ****************************************************************************/
static SINT32	MaHvsw_HvNoteOn(UINT32 dId)
{
	UINT8		bVolume;
	SINT8		sbKey;
	SINT32		sdRet;
	SINT32		sdFuncId = gHvswInfo.sdHvFuncId;
	SINT32		sdFileId;
	PMAHVSWDATA	psData = &(gHvswInfo.sData[dId]);

	if((sdFuncId < 0L) || (psData->pbData == NULL) || (psData->dSize == 0L)) {
		return (MASMW_ERROR);
	}
	/*	load HV script	*/
	sdRet = __MaSound_Load(sdFuncId, psData->pbData, psData->dSize, 1, NULL, NULL);
	if(sdRet < MASMW_SUCCESS) {
		return (sdRet);
	}
	/*	open HV script	*/
	sdFileId = sdRet;
	sdRet = __MaSound_Open(sdFuncId, sdFileId, 0, NULL);
	if(sdRet < MASMW_SUCCESS) {
		__MaSound_Unload(sdFuncId, sdFileId, NULL);
		return (sdRet);
	}
	/*	standby HV script	*/
	sdRet = __MaSound_Standby(sdFuncId, sdFileId, NULL);
	if(sdRet < MASMW_SUCCESS) {
		__MaSound_Close(sdFuncId, sdFileId, NULL);
		__MaSound_Unload(sdFuncId, sdFileId, NULL);
		return (sdRet);
	}
	/*	set volume	*/
	bVolume = (UINT8)MixVolume();
	__MaSound_Control(sdFuncId, sdFileId, MASMW_SET_VOLUME, &bVolume, NULL);
	/*	set key control	*/
	if(gHvswInfo.sdKeyCon) {
		/*	enable key control	*/
		sbKey = (SINT8)(gHvswInfo.sdKey);
		__MaSound_Control(sdFuncId, sdFileId, MASMW_SET_KEYCONTROL, &sbKey, NULL);
	}
	/*	start HV script	*/
	sdRet = __MaSound_Start(sdFuncId, sdFileId, 1, NULL);
	if(sdRet < MASMW_SUCCESS) {
		__MaSound_Close(sdFuncId, sdFileId, NULL);
		__MaSound_Unload(sdFuncId, sdFileId, NULL);
		return (sdRet);
	}

	/*	remember HVS file id	*/
	gHvswInfo.sdHvFileId = sdFileId;

	return (MASMW_SUCCESS);
}

/****************************************************************************
 *	MaHvsw_HvNoteOff
 *
 *	Function:
 *			Stop HV.
 *	Argument:
 *			none
 *	Return:
 *			0	success.
 *			<0	error code.
 *
 ****************************************************************************/
static SINT32	MaHvsw_HvNoteOff(void)
{
	SINT32	sdRet;
	SINT32	sdFuncId = gHvswInfo.sdHvFuncId;
	SINT32	sdFileId = gHvswInfo.sdHvFileId;

	/*	check func & file id	*/
	if((sdFuncId < 0L) || (sdFileId < 0L)) {
		return (MASMW_ERROR_ID);
	}

	/*	get state	*/
	sdRet = __MaSound_Control(sdFuncId, sdFileId, MASMW_GET_STATE, NULL, NULL);
	if(sdRet < MASMW_SUCCESS) {
		return (sdRet);
	}

	switch(sdRet) {
	case MASMW_STATE_PLAYING	:
	case MASMW_STATE_PAUSE		:
		/*	playing or pause	*/
		__MaSound_Stop(sdFuncId, sdFileId, NULL);
	case MASMW_STATE_READY		:
	case MASMW_STATE_OPENED		:
		/*	opened or ready	*/
		__MaSound_Close(sdFuncId, sdFileId, NULL);
	case MASMW_STATE_LOADED		:
		/*	loaded	*/
		__MaSound_Unload(sdFuncId, sdFileId, NULL);
	default	:
		break;
	}

	gHvswInfo.sdHvFileId = -1;
	return (MASMW_SUCCESS);
}

/****************************************************************************
 *	MaHvsw_CallBack
 *
 *	Function:
 *			Hook Callback.
 *	Argument:
 *			bId	... Event number.
 *	Return:
 *			0	success.
 *			<0	error code.
 *
 ****************************************************************************/
static SINT32	MaHvsw_CallBack(UINT8 bId, SINT32 (*pfnCallbackFunc)(UINT8))
{
	SINT32	sdRet = MASMW_SUCCESS;

	if((MAHVSW_MIN_NOTE <= bId) && (bId < MAHVSW_MAX_NOTE)) {
		MaHvsw_HvNoteOff();
		MaHvsw_HvNoteOn((UINT32)(bId - MAHVSW_MIN_NOTE));
	}
	else if(bId == MAHVSW_NOTE_OFF) {
		MaHvsw_HvNoteOff();
	}
	else {
		if((bId == MASMW_REPEAT) || (bId == MASMW_END_OF_SEQUENCE)) {
			MaHvsw_HvNoteOff();
		}
		if(pfnCallbackFunc) {
			sdRet = pfnCallbackFunc(bId);
		}
	}

	return (sdRet);
}

/****************************************************************************
 *	CallBack_Dummy0/1/2
 *
 *	Function:
 *			Hook callback.
 *	Argument:
 *			bId	... Event number.
 *	Return:
 *			0	success.
 *			<0	error code.
 *
 ****************************************************************************/
static SINT32	CallBack_Dummy0(UINT8 bId)
{
	return (MaHvsw_CallBack(bId, gHvswInfo.sHook[0].pfnCallbackFunc));
}

static SINT32	CallBack_Dummy1(UINT8 bId)
{
	return (MaHvsw_CallBack(bId, gHvswInfo.sHook[1].pfnCallbackFunc));
}

static SINT32	CallBack_Dummy2(UINT8 bId)
{
	return (MaHvsw_CallBack(bId, gHvswInfo.sHook[2].pfnCallbackFunc));
}

/****************************************************************************
 *	MaSound_Create
 *
 *	Function:
 *			Setting the MA Stream Converter and callback function.
 *	Argument:
 *			cnv_id		ID number of MA Stream Converter
 *	Return:
 *			>= 0		function id of registered MA Stream Converter
 *			< 0			error code
 *
 ****************************************************************************/
SINT32 MaSound_Create
(
	UINT8	cnv_id						/* stream converter id */
)
{
	SINT32	sdRet = MASMW_ERROR;		/* result of function */

	if ((cnv_id == MASMW_CNVID_HVS) && (gHvswInfo.sdDefaultHvLanguage == MASMW_HVTYPE_NONE)) {
		return (MASMW_ERROR_ID);
	}
	sdRet = __MaSound_Create(cnv_id);
	if(sdRet < MASMW_SUCCESS) {
		/*	Prohibit access when HV is already created. */
		if((gHvswInfo.sdHvFuncId >= 0) && (gHvswInfo.sdHvFuncId == cnv_id)) {
			return (MASMW_ERROR_HV_CONFLICT);
		}
	}

	return (sdRet);
}

/****************************************************************************
 *	MaSound_Load
 *
 *	Function:
 *			Performs loading processing of Stream Converter.
 *	Arguments:
 *			func_id		function id
 *			file_ptr	pointer to file
 *			file_size	byte size of file
 *			mode		error check mode	0: no check
 *											1: check
 *											2: check only
 *											3: get contents information
 *			func		callback function
 *			ext_args	for future extension
 *	Return:
 *			>= 0		success. file id.
 *			< 0			error code
 *
 ****************************************************************************/
SINT32 MaSound_Load
(
	SINT32	func_id,					/* function id */
	UINT8 *	file_ptr,					/* pointer to file */
	UINT32	file_size,					/* size of file */
	UINT8	mode,						/* error check mode */
	SINT32	(*func)(UINT8 id),			/* callback function */
	void *	ext_args					/* for future extension */
)
{
	SINT32	sdId;
	SINT32	sdRet = MASMW_ERROR;		/* result of function */

	MASNDSEQ_DBGMSG(("MaSound_Load: id=%ld\n", func_id));

	/*	Prohibit access when HV is already created. */
	if(gHvswInfo.sdHvFuncId == func_id) {
		return (MASMW_ERROR);
	}

	/*	Check function id	*/
	sdId = IsHookFunc(func_id);
	if((sdId >= MASMW_SUCCESS) && (mode < 2)) {
		sdRet = __MaSound_Load(func_id, file_ptr, file_size, mode, gpfnCallback[sdId], ext_args);
		if(sdRet >= MASMW_SUCCESS) {
			gHvswInfo.sHook[sdId].sdFuncId			= func_id;
			gHvswInfo.sHook[sdId].sdFileId			= sdRet;
			gHvswInfo.sHook[sdId].pfnCallbackFunc	= func;
		}
	}
	else {
		sdRet = __MaSound_Load(func_id, file_ptr, file_size, mode, func, ext_args);
	}

	return (sdRet);
}

/****************************************************************************
 *	MaSound_Open
 *
 *	Function:
 *			Performs opening processing of Stream Converter.
 *	Arguments:
 *			func_id		function id
 *			file_id		file id
 *			open_mode	file open mode
 *			ext_args	for future extension
 *	Return:
 *			0			success
 *			< 0			error code
 *
 ****************************************************************************/
SINT32 MaSound_Open
(
	SINT32	func_id,					/* function id */
	SINT32	file_id,					/* file id */
	UINT16	open_mode,					/* open mode */
	void *	ext_args					/* for future extension */
)
{
	UINT8				bParam;
	SINT32				sdTmp;
	SINT32				sdHvCh;
	SINT32				sdFuncId;
	SINT32				sdHvType;
	MASMW_HVVOICEPARAM	Hvv;
	MASMW_HVSCRIPT		Hvs;
	SINT32				sdRet = MASMW_ERROR;		/* result of function */

	MASNDSEQ_DBGMSG(("MaSound_Open: id=%ld hd=%ld md=%hd\n", func_id, file_id, open_mode));

	/*	Prohibit access when HV is already created. */
	if(gHvswInfo.sdHvFuncId == func_id) {
		return (MASMW_ERROR);
	}

	sdHvType = gHvswInfo.sdDefaultHvLanguage;

	sdRet = __MaSound_Open(func_id, file_id, open_mode, ext_args);
	if((IsHookFunc(func_id) >= MASMW_SUCCESS) && (sdRet == MASMW_SUCCESS)) {
		/*	Initialize	*/
		MaHvsw_Initialize();
		/*	get HV channel number	*/
		sdHvCh = __MaSound_Control(func_id, file_id, MASMW_GET_HVCHANNEL, NULL, NULL);
		if(sdHvCh < MASMW_SUCCESS) {
			return (sdRet);
		}
		/*	get HV script	*/
		Hvs.pbBuf	= NULL;
		Hvs.dSize	= 0;
		sdTmp = __MaSound_Control(func_id, file_id, MASMW_GET_HVSCRIPT, &Hvs, NULL);
		if(sdTmp == MASMW_SUCCESS) {
			sdHvType = GetFirstLanguageType(Hvs.pbBuf, Hvs.dSize);
			/*	set language type	*/
			if ((gHvswInfo.sdDefaultHvLanguage != MASMW_HVTYPE_JAPANESE) && 
			    (sdHvType == MASMW_HVTYPE_JAPANESE)) {
				__MaSound_DeviceControl(MASMW_SET_HVTYPE, (UINT8)sdHvType, 0, 0);
			}
		}
		else {
			Hvs.pbBuf	= NULL;
			Hvs.dSize	= 0;
		}
		/*	create HVS	*/
		sdFuncId = __MaSound_Create(MASMW_CNVID_HVS);
		if(sdFuncId < MASMW_SUCCESS)
		{
			if (sdFuncId != MASMW_ERROR_ID)	/*--- MASMW_ERROR_ID means No HVS module. ---*/
			{
				__MaSound_Close(func_id, file_id, NULL);
				__MaSound_DeviceControl(MASMW_SET_HVTYPE, (UINT8)gHvswInfo.sdDefaultHvLanguage, 0, 0);
				return (MASMW_ERROR_HV_CONFLICT);
			}
		}
		/*	set HV channel number	*/
		bParam = (UINT8)sdHvCh;
		__MaSound_Control(sdFuncId, 0, MASMW_SET_HVCHANNEL, &bParam, NULL);
		/*	get HV LED/VIB status	*/
		sdTmp = __MaSound_Control(func_id, file_id, MASMW_GET_HVCHANNELSTATUS, NULL, NULL);
		if(sdTmp & 0x00000004) gHvswInfo.sdKeyCon = 1;
		/*	set HV LED/VIB status	*/
		bParam = (UINT8)(sdTmp & 0x00000003);
		__MaSound_Control(sdFuncId, 0, MASMW_SET_HVCHANNELSTATUS, &bParam, NULL);
		/*	get HV extend voice	*/
		Hvv.bProgNo	= 0;
		Hvv.pbBuf	= NULL;
		Hvv.dSize	= 0;
		sdTmp = __MaSound_Control(func_id, file_id, MASMW_GET_HVVOICE, &Hvv, NULL);
		if(sdTmp == MASMW_SUCCESS) {
			/*	set HV extend voice	*/
			Hvv.bProgNo |= 0x80;
			__MaSound_Control(sdFuncId, 0, MASMW_SET_HVVOICE, &Hvv, NULL);
		}
		/*	set HV script	*/
		LoadHvScript(Hvs.pbBuf, Hvs.dSize);

		/*	remember func & file id	*/
		gHvswInfo.sdFuncId		= func_id;
		gHvswInfo.sdFileId		= file_id;
		gHvswInfo.sdHvFuncId	= sdFuncId;
	}

	return (sdRet);
}

/****************************************************************************
 *	MaSound_Control
 *
 *	Function:
 *			Performs controlling processing of Stream Conveter.
 *	Argument:
 *			func_id		function id
 *			file_id		file id
 *			ctrl_num	control number
 *			prm			control parameter
 *			ext_args	for future extension
 *	Return:
 *			0			success
 *			< 0			error code
 *
 ****************************************************************************/
SINT32 MaSound_Control
(
	SINT32	func_id,					/* function id */
	SINT32	file_id,					/* file id */
	UINT8	ctrl_num,					/* control number */
	void *	prm,						/* parameter */
	void *	ext_args					/* for future extension */
)
{
	UINT8	bVolume;
	SINT32	sdRet		= MASMW_ERROR;		/* result of function */
	SINT32	sdFuncId	= gHvswInfo.sdHvFuncId;
	SINT32	sdFileId	= gHvswInfo.sdHvFileId;

	MASNDSEQ_DBGMSG(("MaSound_Control: id=%ld hd=%ld cn=%d\n", func_id, file_id, ctrl_num));

	/*	Prohibit access when HV is already created. */
	if(sdFuncId == func_id) {
		return (MASMW_ERROR);
	}

	if(	(sdFuncId >= 0)					&&
		(gHvswInfo.sdFuncId == func_id)	&&
		(gHvswInfo.sdFileId == file_id)) {

		if(ctrl_num == MASMW_SET_HVVOLUME) {
			/*	Argument Check!!	*/
			if(prm == NULL)		return (MASMW_ERROR_ARGUMENT);
			bVolume = *((UINT8*)prm);
			if(bVolume > 127)	return (MASMW_ERROR_ARGUMENT);

			gHvswInfo.dHvVolume = (UINT32)bVolume;
			if(sdFileId >= 0) {
				bVolume = (UINT8)MixVolume();
				__MaSound_Control(sdFuncId, sdFileId, MASMW_SET_VOLUME, &bVolume, ext_args);
			}
			return (MASMW_SUCCESS);
		}

		sdRet = __MaSound_Control(func_id, file_id, ctrl_num, prm, ext_args);
		if(sdRet >= MASMW_SUCCESS) {
			switch(ctrl_num) {
			case MASMW_SET_VOLUME :
				gHvswInfo.dVolume = (UINT32)*((UINT8*)prm);
				if(sdFileId >= 0) {
					bVolume = (UINT8)MixVolume();
					__MaSound_Control(sdFuncId, sdFileId, ctrl_num, &bVolume, ext_args);
				}
				break;
			case MASMW_SET_KEYCONTROL :
				gHvswInfo.sdKey = (SINT32)*((SINT8*)prm);
				if((sdFileId >= 0) && (gHvswInfo.sdKeyCon)) {
					__MaSound_Control(sdFuncId, sdFileId, ctrl_num, prm, ext_args);
				}
				break;
			default:
				break;
			}
		}
	}
	else {
		sdRet = __MaSound_Control(func_id, file_id, ctrl_num, prm, ext_args);
	}

	return (sdRet);
}

/****************************************************************************
 *	MaSound_Standby
 *
 *	Function:
 *			Performs standby processing of Stream Converter.
 *	Argument:
 *			func_id		function id
 *			file_id		file id
 *			ext_args	for future extension
 *	Return:
 *			0			success
 *			< 0			error code
 *
 ****************************************************************************/
SINT32 MaSound_Standby
(
	SINT32	func_id,					/* function id */
	SINT32	file_id,					/* file id */
	void *	ext_args					/* for future extension */
)
{
	/*	Prohibit access when HV is already created. */
	if(gHvswInfo.sdHvFuncId == func_id) {
		return (MASMW_ERROR);
	}

	return (__MaSound_Standby(func_id, file_id, ext_args));
}

/****************************************************************************
 *	MaSound_Seek
 *
 *	Function:
 *			Performs seeking processing of Stream Converter.
 *	Argument:
 *			func_id		function id
 *			file_id		file id
 *			pos			playing position from top or start point [ms]
 *			flag		unused 0
 *			ext_args	for future extension
 *	Return:
 *			0			success
 *			< 0			error code
 *
 ****************************************************************************/
SINT32 MaSound_Seek
(
	SINT32	func_id,					/* function id */
	SINT32	file_id,					/* file id */
	UINT32	pos,						/* start position (msec) */
	UINT8	flag,						/* flag */
	void *	ext_args					/* for future extension */
)
{
	/*	Prohibit access when HV is already created. */
	if(gHvswInfo.sdHvFuncId == func_id) {
		return (MASMW_ERROR);
	}

	return (__MaSound_Seek(func_id, file_id, pos, flag, ext_args));
}

/****************************************************************************
 *	MaSound_Start
 *
 *	Function:
 *			Performs starting process of Stream Converter.
 *	Argument:
 *			func_id		function id
 *			file_id		file id
 *			play_mode	playing mode
 *			ext_args	for future extension
 *	Return:
 *			0			success
 *			< 0			error code
 *
 ****************************************************************************/
SINT32 MaSound_Start
(
	SINT32	func_id,					/* function id */
	SINT32	file_id,					/* file id */
	UINT16	play_mode,					/* playing mode */
	void *	ext_args					/* for future extension */
)
{
	/*	Prohibit access when HV is already created. */
	if(gHvswInfo.sdHvFuncId == func_id) {
		return (MASMW_ERROR);
	}

	return (__MaSound_Start(func_id, file_id, play_mode, ext_args));
}

/****************************************************************************
 *	MaSound_Pause
 *
 *	Function:
 *			Stops sequence data reproduction temporarily.
 *			For audio systems, this operation is same as Stop.
 *	Argument:
 *			func_id		function id
 *			file_id		file id
 *			ext_args	for future extenstion
 *	Return:
 *			0			success
 *			< 0			error code
 *
 ****************************************************************************/
SINT32 MaSound_Pause
(
	SINT32	func_id,					/* function id */
	SINT32	file_id,					/* file id */
	void *	ext_args					/* for future extension */
)
{
	SINT32	sdRet = MASMW_ERROR;

	MASNDSEQ_DBGMSG(("MaSound_Pause: id=%ld hd=%ld\n", func_id, file_id));

	/*	Prohibit access when HV is already created. */
	if(gHvswInfo.sdHvFuncId == func_id) {
		return (MASMW_ERROR);
	}

	sdRet = __MaSound_Pause(func_id, file_id, ext_args);
	if(	(sdRet == MASMW_SUCCESS)		&&
		(gHvswInfo.sdHvFuncId >= 0)		&&
		(gHvswInfo.sdFuncId == func_id)	&&
		(gHvswInfo.sdFileId == file_id)) {

		MaHvsw_HvNoteOff();
	}

	return (sdRet);
}

/****************************************************************************
 *	MaSound_Restart
 *
 *	Function:
 *			Cancels pause of sequence data reproduction.
 *			For audio systems, this operation is the same as Start.
 *	Argument:
 *			func_id		function id
 *			file_id		file id
 *			ext_args	for future extension
 *	Return:
 *			0			success
 *			< 0			error code
 *
 ****************************************************************************/
SINT32 MaSound_Restart
(
	SINT32	func_id,					/* function id */
	SINT32	file_id,					/* file id */
	void *	ext_args					/* for future extension */
)
{
	/*	Prohibit access when HV is already created. */
	if(gHvswInfo.sdHvFuncId == func_id) {
		return (MASMW_ERROR);
	}

	return (__MaSound_Restart(func_id, file_id, ext_args));
}

/****************************************************************************
 *	MaSound_Stop
 *
 *	Function:
 *			Performs stopping processing of Stream Converter.
 *	Argument:
 *			func_id		function id
 *			file_id		file id
 *			ext_args	for future extension
 *	Return:
 *			0			success
 *			< 0			error code
 *
 ****************************************************************************/
SINT32 MaSound_Stop
(
	SINT32	func_id,					/* function id */
	SINT32	file_id,					/* file id */
	void *	ext_args					/* for future extension */
)
{
	SINT32	sdRet = MASMW_ERROR;

	MASNDSEQ_DBGMSG(("MaSound_Stop: id=%ld hd=%ld\n", func_id, file_id));

	/*	Prohibit access when HV is already created. */
	if(gHvswInfo.sdHvFuncId == func_id) {
		return (MASMW_ERROR);
	}

	sdRet = __MaSound_Stop(func_id, file_id, ext_args);
	if(	(sdRet == MASMW_SUCCESS)		&&
		(gHvswInfo.sdHvFuncId >= 0)		&&
		(gHvswInfo.sdFuncId == func_id)	&&
		(gHvswInfo.sdFileId == file_id)) {

		MaHvsw_HvNoteOff();
	}

	return (sdRet);
}

/****************************************************************************
 *	MaSound_Close
 *
 *	Function:
 *			Performs closing processing of Stream Converter.
 *	Argument:
 *			func_id		function id
 *			file_id		file id
 *			ext_args	for future extension
 *	Return:
 *			0			success
 *			< 0			error code
 *
 ****************************************************************************/
SINT32 MaSound_Close
(
	SINT32	func_id,					/* function id */
	SINT32	file_id,					/* file id */
	void *	ext_args					/* for future extension */
)
{
	SINT32	sdRet = MASMW_ERROR;

	MASNDSEQ_DBGMSG(("MaSound_Close: id=%ld hd=%ld\n", func_id, file_id));

	/*	Prohibit access when HV is already created. */
	if(gHvswInfo.sdHvFuncId == func_id) {
		return (MASMW_ERROR);
	}

	sdRet = __MaSound_Close(func_id, file_id, ext_args);
	if(	(sdRet == MASMW_SUCCESS)		&&
		(gHvswInfo.sdHvFuncId >= 0)		&&
		(gHvswInfo.sdFuncId == func_id)	&&
		(gHvswInfo.sdFileId == file_id)) {

		MaHvsw_HvNoteOff();
		__MaSound_Delete(gHvswInfo.sdHvFuncId);
		__MaSound_DeviceControl(MASMW_SET_HVTYPE, (UINT8)gHvswInfo.sdDefaultHvLanguage, 0, 0);
		MaHvsw_Initialize();
	}

	return (sdRet);
}

/****************************************************************************
 *	MaSound_Unload
 *
 *	Function:
 *			Performs unloading processing of Stream Converter.
 *	Argument:
 *			func_id		function id
 *			file_id		file id
 *			ext_args	for future extension
 *	Return:
 *			0			success
 *			< 0			error code
 *
 ****************************************************************************/
SINT32 MaSound_Unload
(
	SINT32	func_id,					/* function id */
	SINT32	file_id,					/* file id */
	void *	ext_args					/* for future extension */
)
{
	UINT32		i;
	SINT32		sdRet;
	PMAHVSWHOOK	pHook;

	/*	Prohibit access when HV is already created. */
	if(gHvswInfo.sdHvFuncId == func_id) {
		return (MASMW_ERROR);
	}

	sdRet = __MaSound_Unload(func_id, file_id, ext_args);
	if(sdRet < MASMW_SUCCESS)	return (sdRet);

	for(i = 0; i < MAHVSW_MAX_CALLBACK; i++) {
		pHook = &(gHvswInfo.sHook[i]);
		if((pHook->sdFuncId == func_id) && (pHook->sdFileId == file_id)) {
			/*	Initialize hook information	*/
			pHook->sdFuncId			= -1;
			pHook->sdFileId			= -1;
			pHook->pfnCallbackFunc	= NULL;
			break;
		}
	}

	return (sdRet);
}

/****************************************************************************
 *	MaSound_Delete
 *
 *	Function:
 *			Deletes Stream Converter that has been registered with func_id.
 *	Argument:
 *			func_id		function id
 *	Return:
 *			0			success
 *			< 0			error code
 *
 ****************************************************************************/
SINT32 MaSound_Delete
(
	SINT32	func_id						/* function id */
)
{
	/*	Prohibit access when HV is already created. */
	if(gHvswInfo.sdHvFuncId == func_id) {
		return (MASMW_ERROR);
	}

	return (__MaSound_Delete(func_id));
}

/****************************************************************************
 *	MaSound_DeviceControl
 *
 *	Function:
 *			Control the hardware.
 *	Argument:
 *			cmd		command number
 *			param1	parameter 1
 *			param2	parameter 2
 *			param3	parameter 3
 *	Return:
 *			>= 0	success
 *			< 0		error code
 *
 ****************************************************************************/
SINT32 MaSound_DeviceControl
(
	UINT8	cmd,						/* command number */
	UINT8	param1,						/* parameter 1 */
	UINT8	param2,						/* parameter 2 */
	UINT8	param3						/* parameter 3 */
)
{
	return (__MaSound_DeviceControl(cmd, param1, param2, param3));
}

/****************************************************************************
 *	MaSound_Initialize
 *
 *	Function:
 *			Initialize the MA Sound Sequencer.
 *	Argument:
 *			None
 *	Return:
 *			0		success
 *			< 0		error code
 *
 ****************************************************************************/
SINT32 MaSound_Initialize
(
	void
)
{
	UINT32		i;
	SINT32		sdRet;
	PMAHVSWHOOK	pHook;

	sdRet = __MaSound_Initialize();
	if(sdRet == MASMW_SUCCESS) {
		gHvswInfo.sdDefaultHvLanguage = MASMW_DEFAULT_HVTYPE;
		MaHvsw_Initialize();
		for(i = 0; i < MAHVSW_MAX_CALLBACK; i++) {
			pHook = &(gHvswInfo.sHook[i]);
			pHook->sdFuncId			= -1;
			pHook->sdFileId			= -1;
			pHook->pfnCallbackFunc	= NULL;
		}
	}

	return (sdRet);
}

/****************************************************************************
 *	MaSound_Terminate
 *
 *	Function:
 *			Terminate the MA Sound Sequencer.
 *	Argument:
 *			None
 *	Return:
 *			0		success
 *			< 0		error code
 *
 ****************************************************************************/
SINT32 MaSound_Terminate
(
	void
)
{
	return (__MaSound_Terminate());
}
