/****************************************************************************
 *
 *		Copyright (C) 2002-2003	YAMAHA CORPORATION. All rights reserved.
 *
 *		Module		: malib.c
 *
 *		Description	: Libraly Functions
 *
 *		Version		: 2.0.0 	2003.05.14
 *
 ****************************************************************************/

#include "madefs.h"
#include "madebug.h"
#include "mamachdep.h"

#include "mammfcnv.h"
#include "malib.h"

/****************************************************************************
 *	macro
 ****************************************************************************/
#define	MALIB_MAKEDWORD(a, b, c, d)	(UINT32)(((UINT32)(a) << 24) | \
						((UINT32)(b) << 16) | ((UINT32)(c) << 8) | (UINT32)(d))

/****************************************************************************
 *	define
 ****************************************************************************/
#define	MALIB_SIZE_OF_CHUNKHEADER	(8)
#define	MALIB_SIZE_OF_CRC			(2)
#define	MALIB_SIZE_OF_MIN_CNTI		(5)
/* for Phrase */
#define	MALIB_CNTI_CLASS_YAMAHA		(0x00)
#define	MALIB_CNTI_TYPE_PHRASE		(0xF0)
#define	MALIB_PHRASE_TIMEBASE		(20)

/* for Audio */
#define	MALIB_MIN_STREAM_FS			(4000)
#define	MALIB_MAX_4BIT_MONO_FS		(24000)
#define	MALIB_MAX_4BIT_STEREO_FS	(24000)
#define	MALIB_MAX_8BIT_MONO_FS		(12000)
#define	MALIB_MAX_8BIT_STEREO_FS	(12000)

#define	MALIB_AUDIO_SIZE_MAX		(0x100000)		/* 1MByte				*/


/* Chunk ID		*/
#define	MALIB_CHUNKID_MMMD			0x4D4D4D44
#define	MALIB_CHUNKID_CNTI			0x434E5449
#define	MALIB_CHUNKID_OPDA			0x4F504441
#define	MALIB_CHUNKID_DCH			0x44636800
#define	MALIB_CHUNKID_M5P			0x50726F00
#define	MALIB_CHUNKID_MTR			0x4D545200
#define	MALIB_CHUNKID_MSPI			0x4D737049
#define	MALIB_CHUNKID_MTSU			0x4D747375
#define	MALIB_CHUNKID_MTSQ			0x4D747371
#define	MALIB_CHUNKID_MTSP			0x4D747370
#define	MALIB_CHUNKID_MWA			0x4D776100
#define	MALIB_CHUNKID_ATR			0x41545200
#define	MALIB_CHUNKID_ASPI			0x41737049
#define	MALIB_CHUNKID_ATSU			0x41747375
#define	MALIB_CHUNKID_ATSQ			0x41747371
#define	MALIB_CHUNKID_AWA			0x41776100
#define	MALIB_CHUNKID_MTHV			0x4D746876
#define	MALIB_CHUNKID_MHVS			0x4D687673
#define	MALIB_CHUNKID_HVP			0x48565000
#define	MALIB_CHUNKID_MHSC			0x4D687363


/****************************************************************************
 *	table
 ****************************************************************************/
static const UINT16 crctable[256] = {
  0x0000U,0x1021U,0x2042U,0x3063U,0x4084U,0x50A5U,0x60C6U,0x70E7U,
  0x8108U,0x9129U,0xA14AU,0xB16BU,0xC18CU,0xD1ADU,0xE1CEU,0xF1EFU,
  0x1231U,0x0210U,0x3273U,0x2252U,0x52B5U,0x4294U,0x72F7U,0x62D6U,
  0x9339U,0x8318U,0xB37BU,0xA35AU,0xD3BDU,0xC39CU,0xF3FFU,0xE3DEU,
  0x2462U,0x3443U,0x0420U,0x1401U,0x64E6U,0x74C7U,0x44A4U,0x5485U,
  0xA56AU,0xB54BU,0x8528U,0x9509U,0xE5EEU,0xF5CFU,0xC5ACU,0xD58DU,
  0x3653U,0x2672U,0x1611U,0x0630U,0x76D7U,0x66F6U,0x5695U,0x46B4U,
  0xB75BU,0xA77AU,0x9719U,0x8738U,0xF7DFU,0xE7FEU,0xD79DU,0xC7BCU,
  0x48C4U,0x58E5U,0x6886U,0x78A7U,0x0840U,0x1861U,0x2802U,0x3823U,
  0xC9CCU,0xD9EDU,0xE98EU,0xF9AFU,0x8948U,0x9969U,0xA90AU,0xB92BU,
  0x5AF5U,0x4AD4U,0x7AB7U,0x6A96U,0x1A71U,0x0A50U,0x3A33U,0x2A12U,
  0xDBFDU,0xCBDCU,0xFBBFU,0xEB9EU,0x9B79U,0x8B58U,0xBB3BU,0xAB1AU,
  0x6CA6U,0x7C87U,0x4CE4U,0x5CC5U,0x2C22U,0x3C03U,0x0C60U,0x1C41U,
  0xEDAEU,0xFD8FU,0xCDECU,0xDDCDU,0xAD2AU,0xBD0BU,0x8D68U,0x9D49U,
  0x7E97U,0x6EB6U,0x5ED5U,0x4EF4U,0x3E13U,0x2E32U,0x1E51U,0x0E70U,
  0xFF9FU,0xEFBEU,0xDFDDU,0xCFFCU,0xBF1BU,0xAF3AU,0x9F59U,0x8F78U,
  0x9188U,0x81A9U,0xB1CAU,0xA1EBU,0xD10CU,0xC12DU,0xF14EU,0xE16FU,
  0x1080U,0x00A1U,0x30C2U,0x20E3U,0x5004U,0x4025U,0x7046U,0x6067U,
  0x83B9U,0x9398U,0xA3FBU,0xB3DAU,0xC33DU,0xD31CU,0xE37FU,0xF35EU,
  0x02B1U,0x1290U,0x22F3U,0x32D2U,0x4235U,0x5214U,0x6277U,0x7256U,
  0xB5EAU,0xA5CBU,0x95A8U,0x8589U,0xF56EU,0xE54FU,0xD52CU,0xC50DU,
  0x34E2U,0x24C3U,0x14A0U,0x0481U,0x7466U,0x6447U,0x5424U,0x4405U,
  0xA7DBU,0xB7FAU,0x8799U,0x97B8U,0xE75FU,0xF77EU,0xC71DU,0xD73CU,
  0x26D3U,0x36F2U,0x0691U,0x16B0U,0x6657U,0x7676U,0x4615U,0x5634U,
  0xD94CU,0xC96DU,0xF90EU,0xE92FU,0x99C8U,0x89E9U,0xB98AU,0xA9ABU,
  0x5844U,0x4865U,0x7806U,0x6827U,0x18C0U,0x08E1U,0x3882U,0x28A3U,
  0xCB7DU,0xDB5CU,0xEB3FU,0xFB1EU,0x8BF9U,0x9BD8U,0xABBBU,0xBB9AU,
  0x4A75U,0x5A54U,0x6A37U,0x7A16U,0x0AF1U,0x1AD0U,0x2AB3U,0x3A92U,
  0xFD2EU,0xED0FU,0xDD6CU,0xCD4DU,0xBDAAU,0xAD8BU,0x9DE8U,0x8DC9U,
  0x7C26U,0x6C07U,0x5C64U,0x4C45U,0x3CA2U,0x2C83U,0x1CE0U,0x0CC1U,
  0xEF1FU,0xFF3EU,0xCF5DU,0xDF7CU,0xAF9BU,0xBFBAU,0x8FD9U,0x9FF8U,
  0x6E17U,0x7E36U,0x4E55U,0x5E74U,0x2E93U,0x3EB2U,0x0ED1U,0x1EF0U
};

static const UINT8 abMask[8] = {
	0x80, 0x40, 0x20, 0x10, 0x08, 0x04, 0x02, 0x01 };

/*******************************************************************************
 *	malib_MakeCRC
 *
 *	Function:
 *			calculate CRC
 *	Argument:
 *			dSize		size of data
 *			pbData		pointer to data
 *	Return:
 *			value of CRC
 *
*******************************************************************************/
UINT16				malib_MakeCRC(
	UINT32			dSize,
	UINT8*			pbData
)
{
	UINT16			wRes;
	UINT8			bData;

	wRes  = 0xFFFFU;
	while( --dSize >= 2 ){
		bData = *pbData++;
		wRes = (UINT16)((wRes << 8) ^ crctable[(UINT8)(wRes >> 8) ^ bData]);
	}
	return (UINT16)(~wRes & 0xFFFFU);
}


/*******************************************************************************
 *	malib_Decode_7bitData
 *
 *	Function:
 *			change 7bit decode data to 8bit data
 *	Argument:
 *			pb7bit		pointer to 7bit data
 *			d7Size		size of 7bit data
 *			pb8bit		pointer to 8bit data
 *			d8Size		area size of 8bit data
 *	Return:
 *			
 *
*******************************************************************/
UINT32				malib_Decode_7bitData(
	UINT8*			pb7bit,
	UINT32			d7Size,
	UINT8*			pb8bit,
	UINT32			d8Size
)
{
	UINT32			dIndex7, dIndex8, dRemain, i;
	UINT8			bMsb;

	if((pb7bit == 0) || (d7Size == 0) || (pb8bit == 0) || (d8Size == 0))
		return 0;

	for (i = 0; i < d8Size; i ++)
		pb8bit[i] = 0;

	dIndex7	= 0;
	dIndex8	= 0;
	while (d7Size > dIndex7) {
		if (d7Size >= dIndex7 + 8)
			dRemain = 8;
		else
			dRemain = d7Size - dIndex7;
		bMsb	= pb7bit[dIndex7];
		for (i = 1; i < dRemain; i++) {
			if (d8Size <= dIndex8)
				return dIndex8;
			if (bMsb & abMask[i])
				pb8bit[dIndex8]	= (UINT8)(pb7bit[dIndex7 + i] | 0x80);
			else
				pb8bit[dIndex8]	= (UINT8)(pb7bit[dIndex7 + i] & 0x7F);
			dIndex8++;
		}
		dIndex7 += dRemain;
	}
	return dIndex8;
}

/****************************************************************************
 *	PhrChk_VoiceChunkCheck
 *
 *	Function:
 *			Check SMAF/Phrase voice chunk body.
 *	Argument:
 *			pbVoc		Pointer to the head of VOIC
 *			dSize		size of VOIC chunk
 *	Return:
 *			0		success.
 *			< 0		error code.
 *
 ****************************************************************************/
static SINT32	PhrChk_VoiceChunkCheck(UINT8* pbVoc, UINT32 dSize)
{
	UINT32		dChunkId;
	UINT32		dChunkSize;

	while(dSize >= (UINT32)(MALIB_SIZE_OF_CHUNKHEADER)) {
		dChunkId	= MALIB_MAKEDWORD(pbVoc[0], pbVoc[1], pbVoc[2], pbVoc[3]);
		dChunkSize	= MALIB_MAKEDWORD(pbVoc[4], pbVoc[5], pbVoc[6], pbVoc[7]);
		pbVoc		+= (UINT32)MALIB_SIZE_OF_CHUNKHEADER;
		/*	Check chunk size	*/
		if(dSize < (UINT32)(MALIB_SIZE_OF_CHUNKHEADER + dChunkSize)) {
			return (MASMW_ERROR_CHUNK_SIZE);
		}
		/*	Update	*/
		pbVoc	+= dChunkSize;
		dSize	-= (UINT32)(MALIB_SIZE_OF_CHUNKHEADER + dChunkSize);
	}

	return (MASMW_SUCCESS);
}

/****************************************************************************
 *	PhrChk_SequenceChunkCheck
 *
 *	Function:
 *			Check SMAF/Phrase sequence chunk.
 *	Argument:
 *			pbSeq		Pointer to the head of SEQU
 *			dSize		size of SEQU chunk
 *	Return:
 *			>=0		playback time[tick].
 *			< 0		error code.
 *
 ****************************************************************************/
static SINT32	PhrChk_SequenceChunkCheck(UINT8* pbSeq, UINT32 dSize)
{
	UINT32		dData;
	UINT32		dIdx;
	UINT32		dDuration;
	UINT32		dPlayTime;

	/*	Get play back time				*/
	dIdx		= 0L;
	dPlayTime	= 0L;
	while(dIdx < dSize) {
		/*	Check Size	*/
		if(dSize < (dIdx + 2L)) return (MASMW_ERROR_CHUNK_SIZE);

		/*	Duration 1 or 2byte	*/
		dDuration = (UINT32)(pbSeq[dIdx++]);
		if(dDuration & 0x80) {
			dDuration = (UINT32)((((dDuration & 0x7F) << 7)
										+ (UINT32)pbSeq[dIdx++]) + 128);
		}

		/*	Update play back time	*/
		dPlayTime += dDuration;

		/* Event */
		if(dSize < (dIdx + 2L)) return (MASMW_ERROR_CHUNK_SIZE);
		dData = (UINT32)pbSeq[dIdx++];
		switch (dData) {
		case 0x00L:
			/* Control Message */
			dData = (UINT32)pbSeq[dIdx++];
			if((dData & 0x30) == 0x30L) {	/*	3byte	*/
				if(dSize < (dIdx + 1L)) {
					return (MASMW_ERROR_CHUNK_SIZE);
				}
				dIdx++;
			}
			break;

		case 0xFFL:
			/* Exclusive Message or NOP */
			dData = (UINT32)pbSeq[dIdx++];
			if(dData == 0xF0L) {
				if(dSize < (dIdx + 1L)) {
					return (MASMW_ERROR_CHUNK_SIZE);
				}
				dData = (UINT32)pbSeq[dIdx++];
				if(dSize < (UINT32)(dIdx + dData)) {
					return (MASMW_ERROR_CHUNK_SIZE);
				}
				dIdx += dData;
			}
			break;

		default:
			/* Note Message */
			/* Gatetime 1 or 2byte */
			dDuration = (UINT32)pbSeq[dIdx++];
			if(dDuration & 0x80) {
				if(dSize < (dIdx + 1L)) {
					return (MASMW_ERROR_CHUNK_SIZE);
				}
				dDuration = (UINT32)(((dDuration & 0x7F) << 7)
												+ (UINT32)pbSeq[dIdx++] + 128);
			}
			break;
		}
	}

	if(dPlayTime <= 1) {	/*	20ms	*/
		return (MASMW_ERROR_SHORT_LENGTH);
	}

	return (SINT32)(dPlayTime);
}

/****************************************************************************
 *	PhrChk_PhraseBody
 *
 *	Function:
 *			Check SMAF/Phrase chunk.
 *	Argument:
 *			pbData		Pointer to the head of MMMG
 *			dSize		size of MMMG chunk
 *			pPi			pointer to MALIBPHRINFO
 *	Return:
 *			0		success.
 *			< 0		error code.
 *
 ****************************************************************************/
static SINT32	PhrChk_PhraseBody(UINT8* pbData, UINT32 dSize,
															PMALIBPHRINFO pPi)
{
	SINT32	sdRet;
	UINT32	dFound;
	UINT32	dChunkId;
	UINT32	dChunkSize;

	if(dSize < 2) return (MASMW_ERROR_CHUNK_SIZE);

	/*	Check Version		*/
	pPi->dVersion = (UINT32)(*pbData);
	if((pPi->dVersion != 1L) && (pPi->dVersion != 2L)) {
		return (MASMW_ERROR_FILE);
	}

	if(pPi->dErrChk) {
		/*	Check Timer base	*/
		if(*(pbData + 1) != MALIB_PHRASE_TIMEBASE) {
			return (MASMW_ERROR_FILE);
		}
	}

	/*	Search "VOIC", "SEQU" Chunk		*/
	dFound	= 0x00L;
	pbData	+= 2;	/*	Skip	*/
	dSize	-= 2L;
	while(dSize >= (UINT32)MALIB_SIZE_OF_CHUNKHEADER) {
		dChunkId	= MALIB_MAKEDWORD(pbData[0], pbData[1], pbData[2], pbData[3]);
		dChunkSize	= MALIB_MAKEDWORD(pbData[4], pbData[5], pbData[6], pbData[7]);
		pbData		+= (UINT32)MALIB_SIZE_OF_CHUNKHEADER;

		/*	Check chunk size	*/
		if(dSize < (UINT32)(MALIB_SIZE_OF_CHUNKHEADER + dChunkSize)) {
			return (MASMW_ERROR_CHUNK_SIZE);
		}

		if(	(dChunkId == MALIB_MAKEDWORD('V', 'O', 'I', 'C'))	&&
			!(dFound & 0x01)) {
			sdRet = PhrChk_VoiceChunkCheck(pbData, dChunkSize);
			if(sdRet < MASMW_SUCCESS) return (sdRet);
			pPi->pbVoice	= pbData;
			pPi->dVoiceSize	= dChunkSize;
			dFound |= 0x01L;
		}
		else if((dChunkId == MALIB_MAKEDWORD('S', 'E', 'Q', 'U'))	&&
				!(dFound & 0x02)) {
			sdRet = PhrChk_SequenceChunkCheck(pbData, dChunkSize);
			if(sdRet < MASMW_SUCCESS) return (sdRet);
			pPi->pbSeq		= pbData;
			pPi->dSeqSize	= dChunkSize;
			pPi->dPlayTime	= (UINT32)sdRet;
			dFound |= 0x02L;
		}

		/*	Update	*/
		pbData	+= dChunkSize;
		dSize	-= (UINT32)(dChunkSize + MALIB_SIZE_OF_CHUNKHEADER);
	}

	if((dFound & 0x02) == 0x00L) {
		return (MASMW_ERROR_CHUNK);
	}
	return (MASMW_SUCCESS);
}

/****************************************************************************
 *	SmafPhrChecker
 *
 *	Function:
 *			Check SMAF/Phrase data.
 *	Argument:
 *			pbData		Pointer to SMAF/Phrase data
 *			dSize		size of data
 *			pPi			pointer to MALIBPHRINFO
 *	Return:
 *			0		success.
 *			< 0		error code.
 *
 ****************************************************************************/
static SINT32	SmafPhrChecker(UINT8* pbData, UINT32 dSize, PMALIBPHRINFO pPi)
{
	UINT32			dCRC;
	UINT32			dRemain;
	UINT32			dChunkId;
	UINT32			dChunkSize;

	if(dSize < (UINT32)(MALIB_SIZE_OF_CHUNKHEADER + MALIB_SIZE_OF_CRC))
		 return (MASMW_ERROR_CHUNK_SIZE);

	/*	Check header ID and size	*/
	dChunkId	= MALIB_MAKEDWORD(pbData[0], pbData[1], pbData[2], pbData[3]);
	dChunkSize	= MALIB_MAKEDWORD(pbData[4], pbData[5], pbData[6], pbData[7]);

	if(dChunkId != MALIB_MAKEDWORD('M', 'M', 'M', 'D')) {
		return (MASMW_ERROR_CHUNK);
	}
	if((UINT32)(dChunkSize + MALIB_SIZE_OF_CHUNKHEADER) != dSize) {
		return (MASMW_ERROR_CHUNK_SIZE);
	}

	/*	Check CRC			*/
	if(pPi->dErrChk) {
		dCRC = (UINT32)(((UINT32)pbData[dSize - 2L] << 8) |	\
					   ((UINT32)pbData[dSize - 1L] << 0));
		if((UINT32)malib_MakeCRC(dSize, pbData) != dCRC) {
			return (MASMW_ERROR_FILE);
		}
	}

	pbData	+= (UINT32)MALIB_SIZE_OF_CHUNKHEADER;
	dRemain	= dChunkSize;
	if(dRemain < (UINT32)(MALIB_SIZE_OF_CHUNKHEADER
							+ MALIB_SIZE_OF_MIN_CNTI + MALIB_SIZE_OF_CRC)) {
		return (MASMW_ERROR_FILE);
	}

	/*	Check Contents Info ID				*/
	dChunkId	= MALIB_MAKEDWORD(pbData[0], pbData[1], pbData[2], pbData[3]);
	dChunkSize	= MALIB_MAKEDWORD(pbData[4], pbData[5], pbData[6], pbData[7]);
	pbData		+= MALIB_SIZE_OF_CHUNKHEADER;

	if(dChunkId != MALIB_MAKEDWORD('C', 'N', 'T', 'I')) {
		return (MASMW_ERROR_CHUNK);
	}
	if(	(dChunkSize < (UINT32)MALIB_SIZE_OF_MIN_CNTI)	||
		(dRemain < (UINT32)(MALIB_SIZE_OF_CHUNKHEADER
						 + dChunkSize + MALIB_SIZE_OF_CRC))) {
		return (MASMW_ERROR_CHUNK_SIZE);
	}

	/*	Check contents class & type		*/
	if(pPi->dErrChk) {
		if(*(pbData) != MALIB_CNTI_CLASS_YAMAHA) {
			return (MASMW_ERROR_CONTENTS_CLASS);
		}
		if(*(pbData + 1) != MALIB_CNTI_TYPE_PHRASE) {
			return (MASMW_ERROR_CONTENTS_TYPE);
		}
	}

	pbData	+= dChunkSize;
	dRemain -= (UINT32)(dChunkSize + MALIB_SIZE_OF_CHUNKHEADER);

	/*	Phrase Check					*/
	while(dRemain >= (UINT32)(MALIB_SIZE_OF_CHUNKHEADER + MALIB_SIZE_OF_CRC)) {
		/*	Search "MMMG" Chunk				*/
		dChunkId	= MALIB_MAKEDWORD(pbData[0], pbData[1], pbData[2], pbData[3]);
		dChunkSize	= MALIB_MAKEDWORD(pbData[4], pbData[5], pbData[6], pbData[7]);
		pbData		+= (UINT32)MALIB_SIZE_OF_CHUNKHEADER;
		if(dRemain < (UINT32)(MALIB_SIZE_OF_CHUNKHEADER
					 + dChunkSize + MALIB_SIZE_OF_CRC)) {
			return (MASMW_ERROR_CHUNK_SIZE);
		}

		if(dChunkId == MALIB_MAKEDWORD('M', 'M', 'M', 'G')) {
			/*	Support only the first "MMMG" Chunk	*/
			return (PhrChk_PhraseBody(pbData, dChunkSize, pPi));
		}

		/*	Update	*/
		pbData	+= dChunkSize;
		dRemain	-= (UINT32)(dChunkSize + MALIB_SIZE_OF_CHUNKHEADER);
	}

	return (MASMW_ERROR_CHUNK);
}

/****************************************************************************
 *	malib_smafphrase_checker
 *
 *	Function:
 *			Check SMAF/Phrase data.
 *	Argument:
 *			pbData		Pointer to SMAF/Phrase data
 *			dSize		size of data
 *			pvInfo		pointer to MALIBPHRINFO
 *	Return:
 *			0		success.
 *			< 0		error code.
 *
 ****************************************************************************/
SINT32	malib_smafphrase_checker(UINT8* pbData, UINT32 dSize, void* pvInfo)
{
	SINT32			sdRet;
	MALIBPHRINFO	sInfo;
	PMALIBPHRINFO	pPi = (PMALIBPHRINFO)pvInfo;

	if(pbData == NULL)	return (MASMW_ERROR_ARGUMENT);
	if(dSize == 0L)		return (MASMW_ERROR_ARGUMENT);

	/*	call from music API	*/
	if(pPi == NULL) {
		pPi = &sInfo;
		pPi->dErrChk = 1L;	/*	enable error check	*/
	}

	/*	check data format	*/
	sdRet = SmafPhrChecker(pbData, dSize, pPi);
	if(sdRet < MASMW_SUCCESS) {
		return (sdRet);
	}

	/*	tick to ms	*/
	return (SINT32)(pPi->dPlayTime * MALIB_PHRASE_TIMEBASE);
}

/*********************************************************************************
 *	malib_NextChunk
 *
 *	Description:
 *			chunk information
 *	Argument:
 *			pbFile			pointer to data
 *			dFsize			size of data(remain)
 *			dState			chunk search status
 *			pdChunkID		pointer to write chunk id space
 *			pdChunkNo		pointer to write chunk no space
 *	Return:
 *			>=0				success(chunk size)
 *			< 0				error code
 ********************************************************************************/
SINT32					malib_NextChunk(
	UINT8*				pbFile,
	UINT32				dSize,
	UINT32				dState,
	UINT32*				pdChunkID,
	UINT32*				pdChunkNo
)
{
	UINT32				dChunkID, dChunkSize;

	if (dSize < MALIB_SIZE_OF_CHUNKHEADER)
		return MALIB_CHUNK_SIZE_ERROR;

	dChunkID	= MALIB_MAKEDWORD(pbFile[0], pbFile[1], pbFile[2], pbFile[3]);
	dChunkSize	= MALIB_MAKEDWORD(pbFile[4], pbFile[5], pbFile[6], pbFile[7]);

	switch (dChunkID) {
	case MALIB_CHUNKID_MMMD :
		*pdChunkID = MALIB_CHUNKCODE_MMMD;
		if (dState != MALIB_CHUNK_PHASE_MMMD)
			return MALIB_CHUNK_ID_ERROR;
		break;
	case MALIB_CHUNKID_CNTI :
		*pdChunkID = MALIB_CHUNKCODE_CNTI;
		if (dState != MALIB_CHUNK_PHASE_CNTI)
			return MALIB_CHUNK_ID_ERROR;
		break;
	case MALIB_CHUNKID_OPDA :
		*pdChunkID = MALIB_CHUNKCODE_OPDA;
		if (dState != MALIB_CHUNK_PHASE_MMMDSUB)
			return MALIB_CHUNK_ID_ERROR;
		break;
	case MALIB_CHUNKID_MSPI :
		*pdChunkID = MALIB_CHUNKCODE_MSPI;
		if (dState != MALIB_CHUNK_PHASE_MTRSUB)
			return MALIB_CHUNK_ID_ERROR;
		break;
	case MALIB_CHUNKID_MTSU :
		*pdChunkID = MALIB_CHUNKCODE_MTSU;
		if (dState != MALIB_CHUNK_PHASE_MTRSUB)
			return MALIB_CHUNK_ID_ERROR;
		break;
	case MALIB_CHUNKID_MTSQ :
		*pdChunkID = MALIB_CHUNKCODE_MTSQ;
		if (dState != MALIB_CHUNK_PHASE_MTRSUB)
			return MALIB_CHUNK_ID_ERROR;
		break;
	case MALIB_CHUNKID_MTSP :
		*pdChunkID = MALIB_CHUNKCODE_MTSP;
		if (dState != MALIB_CHUNK_PHASE_MTRSUB)
			return MALIB_CHUNK_ID_ERROR;
		break;
	case MALIB_CHUNKID_ASPI :
		*pdChunkID = MALIB_CHUNKCODE_ASPI;
		if (dState != MALIB_CHUNK_PHASE_ATRSUB)
			return MALIB_CHUNK_ID_ERROR;
		break;
	case MALIB_CHUNKID_ATSU :
		*pdChunkID = MALIB_CHUNKCODE_ATSU;
		if (dState != MALIB_CHUNK_PHASE_ATRSUB)
			return MALIB_CHUNK_ID_ERROR;
		break;
	case MALIB_CHUNKID_ATSQ :
		*pdChunkID = MALIB_CHUNKCODE_ATSQ;
		if (dState != MALIB_CHUNK_PHASE_ATRSUB)
			return MALIB_CHUNK_ID_ERROR;
		break;

	case MALIB_CHUNKID_MTHV :
		*pdChunkID = MALIB_CHUNKCODE_MTHV;
		if (dState != MALIB_CHUNK_PHASE_MTRSUB)
			return MALIB_CHUNK_ID_ERROR;
		break;
	case MALIB_CHUNKID_MHVS :
		*pdChunkID = MALIB_CHUNKCODE_MHVS;
		if (dState != MALIB_CHUNK_PHASE_MTHVSUB)
			return MALIB_CHUNK_ID_ERROR;
		break;
	case MALIB_CHUNKID_MHSC :
		*pdChunkID = MALIB_CHUNKCODE_MHSC;
		if (dState != MALIB_CHUNK_PHASE_MTHVSUB)
			return MALIB_CHUNK_ID_ERROR;
		break;

	default :
		*pdChunkNo = (UINT8)(dChunkID & 0x000000FF);
		switch (dChunkID & 0xFFFFFF00) {
		case MALIB_CHUNKID_MTR :
			*pdChunkID = MALIB_CHUNKCODE_MTR;
			if (dState != MALIB_CHUNK_PHASE_MMMDSUB)
				return MALIB_CHUNK_ID_ERROR;
			break;
		case MALIB_CHUNKID_ATR :
			*pdChunkID = MALIB_CHUNKCODE_ATR;
			if (dState != MALIB_CHUNK_PHASE_MMMDSUB)
				return MALIB_CHUNK_ID_ERROR;
			break;

		case MALIB_CHUNKID_DCH :
			*pdChunkID = MALIB_CHUNKCODE_DCH;
			if (dState != MALIB_CHUNK_PHASE_OPDASUB)
				return MALIB_CHUNK_ID_ERROR;
			break;
		case MALIB_CHUNKID_M5P :
			*pdChunkID = MALIB_CHUNKCODE_M5P;
			if (dState != MALIB_CHUNK_PHASE_OPDASUB)
				return MALIB_CHUNK_ID_ERROR;
			break;

		case MALIB_CHUNKID_MWA :
			*pdChunkID = MALIB_CHUNKCODE_MWA;
			if (dState != MALIB_CHUNK_PHASE_MTSPSUB)
				return MALIB_CHUNK_ID_ERROR;
			break;

		case MALIB_CHUNKID_AWA :
			*pdChunkID = MALIB_CHUNKCODE_AWA;
			if (dState != MALIB_CHUNK_PHASE_ATRSUB)
				return MALIB_CHUNK_ID_ERROR;
			break;

		case MALIB_CHUNKID_HVP :
			*pdChunkID = MALIB_CHUNKCODE_HVP;
			if (dState != MALIB_CHUNK_PHASE_MTHVSUB)
				return MALIB_CHUNK_ID_ERROR;
			break;

		default :
			*pdChunkID = MALIB_CHUNKCODE_UNKNOWN;
			break;
		}
		break;
	}

	if (dChunkSize > (dSize - MALIB_SIZE_OF_CHUNKHEADER))
		return MALIB_CHUNK_SIZE_ERROR;
	else
		return (SINT32)dChunkSize;
}

/****************************************************************************
 *	GetWaveInfo_2
 *
 *	Function:
 *			Get data Info from the file
 *	Argument:
 *			pbFile		pointer to the SMAF/Audio data.
 *			dFileSize	size of data.
 *			pAi			pointer to the MALIBAUDINFO structure.
 *	Return:
 *			0		success.
 *			< 0		error.
 *
 ****************************************************************************/
static SINT32	GetWaveInfo_2(UINT8* pbFile, UINT32 dSize, PMALIBAUDINFO pAi)
{
	UINT32	dFmt;
	UINT32	dData;
	UINT8*	pbWave;
	UINT32	dChunkID, dChunkNo;
	UINT32	dIndex;
	UINT32	dFs;
	SINT32	sdChunkSize;
	SINT32	sdSizeOfWave;

	dData = (UINT32)pbFile[2];
	if		(dData == 0x10)		dFs = 4000;		/* Mono, ADPCM, 4KHz	*/
	else if	(dData == 0x11)		dFs = 8000;		/* Mono, ADPCM, 8KHz	*/
	else						return (MASMW_ERROR_CHUNK);	/* Error	*/

	dData = (UINT32)pbFile[3];
	if (dData == 0x00)			dFmt = 0x01;	/* 4bit ADPCM			*/
	else						return (MASMW_ERROR_CHUNK);	/* Error	*/

	dIndex			= 6L;
	pbWave			= NULL;
	sdSizeOfWave	= 0L;
	while(dSize > (dIndex + MALIB_SIZE_OF_CHUNKHEADER)) {
		sdChunkSize = malib_NextChunk(&pbFile[dIndex], (dSize - dIndex),
				MALIB_CHUNK_PHASE_ATRSUB, &dChunkID, &dChunkNo);
		if(sdChunkSize < 0) {
			MAAUDCNV_DBGMSG(("GetDataInfo/Found Error Chunk/\n"));
			return (MASMW_ERROR_CHUNK);
		}
		if ((dChunkID == MALIB_CHUNKCODE_AWA) && (pbWave == NULL)) {
			sdSizeOfWave	= sdChunkSize;
			pbWave			= &(pbFile[dIndex + MALIB_SIZE_OF_CHUNKHEADER]);
		}
		dIndex	+= (UINT32)(sdChunkSize + MALIB_SIZE_OF_CHUNKHEADER);
	}

	if ((UINT32)sdSizeOfWave <= (dFs / 100))	return MASMW_ERROR_SHORT_LENGTH;

	pAi->pbSource		= pbWave;
	pAi->dSizeOfSource	= (UINT32)sdSizeOfWave;
	pAi->dFs			= dFs;
	pAi->dFmt			= dFmt;

	MAAUDCNV_DBGMSG(("MaAudCnv : Fs =  %ld[Hz]\n", pAi->dFs));
	MAAUDCNV_DBGMSG(("MaAudCnv : pData = %08lX\n", pAi->pbSource));
	MAAUDCNV_DBGMSG(("MaAudCnv : Size = %ld[bytes]\n", pAi->dSizeOfSource));

	return (MASMW_SUCCESS);
}

/****************************************************************************
 *	GetWaveInfo_3
 *
 *	Function:
 *			Get data Info from the file
 *	Argument:
 *			pbFile		pointer to the SMAF/Audio data.
 *			dFileSize	size of data.
 *			pAi			pointer to the MALIBAUDINFO structure.
 *	Return:
 *			0		success.
 *			< 0		error.
 *
 ****************************************************************************/
static SINT32	GetWaveInfo_3(UINT8* pbFile, UINT32 dSize, PMALIBAUDINFO pAi)
{
	UINT32	dFmt;
	UINT32	dChunkID, dChunkNo;
	UINT32	dIndex;
	UINT32	dFs;
	SINT32	sdChunkSize;
	UINT8*	pMtsp;
	UINT32	dMtspSize;

	dIndex		= 20;
	dFmt		= 0xFFL;
	pMtsp		= NULL;
	dMtspSize	= 0;

	while(dSize > dIndex + MALIB_SIZE_OF_CHUNKHEADER) {
		sdChunkSize = malib_NextChunk(&(pbFile[dIndex]), (dSize - dIndex),
				MALIB_CHUNK_PHASE_MTRSUB, &dChunkID, &dChunkNo);
		if(sdChunkSize < 0L) {
			MAAUDCNV_DBGMSG(("GetDataInfo/Found Error Chunk/\n"));
			return (MASMW_ERROR_CHUNK);
		}
		if((dChunkID == MALIB_CHUNKCODE_MTSP) && (pMtsp == NULL)) {
			pMtsp		= &(pbFile[dIndex + MALIB_SIZE_OF_CHUNKHEADER]);
			dMtspSize	= sdChunkSize;
		}
		dIndex	+= (UINT32)(sdChunkSize + MALIB_SIZE_OF_CHUNKHEADER);
	}

	if (pMtsp == NULL)		return MASMW_ERROR_SHORT_LENGTH;

	dIndex		= 0;
	while(dMtspSize > dIndex + MALIB_SIZE_OF_CHUNKHEADER) {
		sdChunkSize = malib_NextChunk(&(pMtsp[dIndex]), (dMtspSize - dIndex),
				MALIB_CHUNK_PHASE_MTSPSUB, &dChunkID, &dChunkNo);
		if(sdChunkSize < 4L) {
			MAAUDCNV_DBGMSG(("GetDataInfo/Found Error Chunk/\n"));
			return (MASMW_ERROR_CHUNK);
		}
		if((dChunkID != MALIB_CHUNKCODE_MWA)) {
			dIndex	+= (UINT32)(sdChunkSize + MALIB_SIZE_OF_CHUNKHEADER);
			continue;
		}
		sdChunkSize	-= 3L;
		dFs			=	((UINT32)pMtsp[dIndex + 9] << 8);
		dFs			+=	(UINT32)pMtsp[dIndex + 10];
		if(dFs < MALIB_MIN_STREAM_FS)  return (MASMW_ERROR_CHUNK);

		switch(pMtsp[dIndex + 8]) {
		case 0x01:
			if(dFs > MALIB_MAX_8BIT_MONO_FS)
				return (MASMW_ERROR_CHUNK);
			if((UINT32)sdChunkSize <= (dFs / 50))
				return (MASMW_ERROR_SHORT_LENGTH);
			MAAUDCNV_DBGMSG(("MaAudCnv : 8bit 2's comp mono PCM\n"));
			dFmt = 0x03L;
			break;

		case 0x11:
			if(dFs > MALIB_MAX_8BIT_MONO_FS)
				return (MASMW_ERROR_CHUNK);
			if((UINT32)sdChunkSize <= (dFs / 50))
				return (MASMW_ERROR_SHORT_LENGTH);
			MAAUDCNV_DBGMSG(("MaAudCnv : 8bit offset mono PCM\n"));
			dFmt = 0x02L;
			break;

		case 0x20:
			if(dFs > MALIB_MAX_4BIT_MONO_FS)
				return (MASMW_ERROR_CHUNK);
			if((UINT32)sdChunkSize <= (dFs / 100))
				return (MASMW_ERROR_SHORT_LENGTH);
			MAAUDCNV_DBGMSG(("MaAudCnv : YAMAHA 4bit mono ADPCM\n"));
			dFmt = 0x01L;
			break;

		default:
			MAAUDCNV_DBGMSG(("GetDataInfo/ Wave format error\n"));
			return (MASMW_ERROR_CHUNK);
		}

		pAi->pbSource		= &(pMtsp[dIndex + 11]);
		pAi->dSizeOfSource	= (UINT32)sdChunkSize;
		pAi->dFs			= dFs;
		pAi->dFmt			= dFmt;

		MAAUDCNV_DBGMSG(("MaAudCnv : Fs =  %ld[Hz]\n", pAi->dFs));
		MAAUDCNV_DBGMSG(("MaAudCnv : pData = %08lX\n", pAi->pbSource));
		MAAUDCNV_DBGMSG(("MaAudCnv : Size = %ld[bytes]\n", pAi->dSizeOfSource));
		break;
	}

	if(dFmt == 0xFFL) return (MASMW_ERROR_SHORT_LENGTH);
	return (MASMW_SUCCESS);
}

/****************************************************************************
 *	GetWaveInfo_5
 *
 *	Function:
 *			Get data Info from the file
 *	Argument:
 *			pbFile		pointer to the SMAF/Audio data.
 *			dFileSize	size of data.
 *			pAi			pointer to the MALIBAUDINFO structure.
 *	Return:
 *			0		success.
 *			< 0		error.
 *
 ****************************************************************************/
static SINT32	GetWaveInfo_5(UINT8* pbFile, UINT32 dSize, PMALIBAUDINFO pAi)
{
	UINT32	dFmt;
	UINT32	dChunkID, dChunkNo;
	UINT32	dIndex;
	UINT32	dFs;
	SINT32	sdChunkSize;
	UINT8*	pMtsp;
	UINT32	dMtspSize;

	dIndex		= 20;
	dFmt		= 0xFFL;
	pMtsp		= NULL;
	dMtspSize	= 0;

	while(dSize > dIndex + MALIB_SIZE_OF_CHUNKHEADER) {
		sdChunkSize = malib_NextChunk(&(pbFile[dIndex]), (dSize - dIndex),
				MALIB_CHUNK_PHASE_MTRSUB, &dChunkID, &dChunkNo);
		if(sdChunkSize < 0L) {
			MAAUDCNV_DBGMSG(("GetDataInfo/Found Error Chunk/\n"));
			return (MASMW_ERROR_CHUNK);
		}
		if((dChunkID == MALIB_CHUNKCODE_MTSP) && (pMtsp == NULL)) {
			pMtsp		= &(pbFile[dIndex + MALIB_SIZE_OF_CHUNKHEADER]);
			dMtspSize	= sdChunkSize;
		}
		dIndex	+= (UINT32)(sdChunkSize + MALIB_SIZE_OF_CHUNKHEADER);
	}

	if (pMtsp == NULL)		return MASMW_ERROR_SHORT_LENGTH;

	dIndex	= 0;
	while(dMtspSize > dIndex + MALIB_SIZE_OF_CHUNKHEADER) {
		sdChunkSize = malib_NextChunk(&(pMtsp[dIndex]), (dMtspSize - dIndex),
				MALIB_CHUNK_PHASE_MTSPSUB, &dChunkID, &dChunkNo);
		if(sdChunkSize < 4L) {
			MAAUDCNV_DBGMSG(("GetDataInfo/Found Error Chunk/\n"));
			return (MASMW_ERROR_CHUNK);
		}
		if((dChunkID != MALIB_CHUNKCODE_MWA)) {
			dIndex	+= (UINT32)(sdChunkSize + MALIB_SIZE_OF_CHUNKHEADER);
			continue;
		}

		sdChunkSize	-= 3L;
		dFs			=	((UINT32)pMtsp[dIndex + 9] << 8);
		dFs			+=	(UINT32)pMtsp[dIndex + 10];
		if(dFs < MALIB_MIN_STREAM_FS)  return (MASMW_ERROR_CHUNK);

		switch(pMtsp[dIndex + 8]) {
		case 0x01:
			if(dFs > MALIB_MAX_8BIT_MONO_FS)
				return (MASMW_ERROR_CHUNK);
			if((UINT32)sdChunkSize <= (dFs / 50))
				return (MASMW_ERROR_SHORT_LENGTH);
			MAAUDCNV_DBGMSG(("MaAudCnv : 8bit 2's comp mono PCM\n"));
			dFmt = 0x03L;
			break;

		case 0x11:
			if(dFs > MALIB_MAX_8BIT_MONO_FS)
				return (MASMW_ERROR_CHUNK);
			if((UINT32)sdChunkSize <= (dFs / 50))
				return (MASMW_ERROR_SHORT_LENGTH);
			MAAUDCNV_DBGMSG(("MaAudCnv : 8bit offset mono PCM\n"));
			dFmt = 0x02L;
			break;

		case 0x20:
			if(dFs > MALIB_MAX_4BIT_MONO_FS)
				return (MASMW_ERROR_CHUNK);
			if((UINT32)sdChunkSize <= (dFs / 100))
				return (MASMW_ERROR_SHORT_LENGTH);
			MAAUDCNV_DBGMSG(("MaAudCnv : YAMAHA 4bit mono ADPCM\n"));
			dFmt = 0x01L;
			break;

		case 0x81:
			if(dFs > MALIB_MAX_8BIT_STEREO_FS)
				return (MASMW_ERROR_CHUNK);
			if((UINT32)sdChunkSize <= (dFs / 25))
				return (MASMW_ERROR_SHORT_LENGTH);
			MAAUDCNV_DBGMSG(("MaAudCnv : 8bit 2's comp stereo PCM\n"));
			dFmt = 0x83L;
			break;

		case 0x91:
			if(dFs > MALIB_MAX_8BIT_STEREO_FS)
				return (MASMW_ERROR_CHUNK);
			if((UINT32)sdChunkSize <= (dFs / 25))
				return (MASMW_ERROR_SHORT_LENGTH);
			MAAUDCNV_DBGMSG(("MaAudCnv : 8bit offset stereo PCM\n"));
			dFmt = 0x82L;
			break;

		case 0xA0:
			if(dFs > MALIB_MAX_4BIT_STEREO_FS)
				return (MASMW_ERROR_CHUNK);
			if((UINT32)sdChunkSize <= (dFs / 50))
				return (MASMW_ERROR_SHORT_LENGTH);
			MAAUDCNV_DBGMSG(("MaAudCnv : YAMAHA 4bit stereo ADPCM\n"));
			dFmt = 0x81L;
			break;

		default:
			MAAUDCNV_DBGMSG(("GetDataInfo/ Wave format error\n"));
			return (MASMW_ERROR_CHUNK);
		}

		pAi->pbSource		= &(pMtsp[dIndex + 11]);
		pAi->dSizeOfSource	= (UINT32)sdChunkSize;
		pAi->dFs			= dFs;
		pAi->dFmt			= dFmt;

		MAAUDCNV_DBGMSG(("MaAudCnv : Fs =  %ld[Hz]\n", pAi->dFs));
		MAAUDCNV_DBGMSG(("MaAudCnv : pData = %08lX\n", pAi->pbSource));
		MAAUDCNV_DBGMSG(("MaAudCnv : Size = %ld[bytes]\n", pAi->dSizeOfSource));
		break;
	}

	if(dFmt == 0xFFL) return (MASMW_ERROR_SHORT_LENGTH);
	return (MASMW_SUCCESS);
}

/****************************************************************************
 *	SmafAudChecker
 *
 *	Function:
 *			Get data Info from the file
 *	Argument:
 *			pbFile		pointer to the SMAF/Audio data.
 *			dFileSize	size of data.
 *			bMode		error check mode(0:No 1:Yes 2:Check only)
 *			pAi			pointer to the MALIBAUDINFO structure.
 *	Return:
 *			0		success.
 *			< 0		error.
 *
 ****************************************************************************/
static SINT32	SmafAudChecker(UINT8* pbFile, UINT32 dFileSize, UINT8 bMode,
															PMALIBAUDINFO pAi)
{
	UINT32			dType;
	UINT32			dClass;
	UINT32			dCRC;
	UINT32			dChunkID, dChunkNo;
	UINT32			dIndex;
	SINT32			sdSize;
	SINT32			sdTotalSize;
	UINT8*			pbChunk;
	UINT32			dChunkSize;

	MAAUDCNV_DBGMSG(("GetDataInfo \n"));

	pAi->pbCnti		= NULL;
	pAi->dCntiSize	= 0;
	pAi->pbOpda		= NULL;
	pAi->dOpdaSize	= 0;

	dIndex		= 0L;
	dChunkID	= MALIB_CHUNKCODE_UNKNOWN;
	sdTotalSize = malib_NextChunk(&(pbFile[dIndex]), dFileSize,
			MALIB_CHUNK_PHASE_MMMD, &dChunkID, &dChunkNo);

	if((sdTotalSize < 0L) || (dChunkID != MALIB_CHUNKCODE_MMMD)) {
		MAAUDCNV_DBGMSG(("GetDataInfo/ ID Error\n"));
		return (MASMW_ERROR_FILE);
	}

	sdTotalSize	+= MALIB_SIZE_OF_CHUNKHEADER;
	if(bMode != 0) {
		dCRC = (UINT32)(((UINT32)pbFile[sdTotalSize - 2] << 8) +
			   (UINT32)pbFile[sdTotalSize - 1]);
		if(dCRC != (UINT32)malib_MakeCRC((UINT32)sdTotalSize, pbFile)) {
			MAAUDCNV_DBGMSG(("GetDataInfo/ CRC error\n"));
			return (MASMW_ERROR_FILE);
		}
	}

	dIndex = MALIB_SIZE_OF_CHUNKHEADER;
	sdSize = malib_NextChunk(&pbFile[dIndex], (sdTotalSize - dIndex),
			MALIB_CHUNK_PHASE_CNTI, &dChunkID, &dChunkNo);
	if((sdSize < MALIB_SIZE_OF_MIN_CNTI) || (dChunkID != MALIB_CHUNKCODE_CNTI)) {
		MAAUDCNV_DBGMSG(("GetDataInfo/ ID Error\n"));
		return (MASMW_ERROR_FILE);
	}

	dClass			= (UINT32)pbFile[16];		/* Contents Class			  */
	dType			= (UINT32)pbFile[17];		/* Contents Type			  */

	if(	(dClass != (UINT32)MMF_CONTENTS_CLASS_0)	&&
		(dClass != (UINT32)MMF_CONTENTS_CLASS_1)) {
		return (MASMW_ERROR_CONTENTS_CLASS);
	}

	if		(((dType & 0xF0) == MMF_CONTENTS_TYPE_0) ||
			 ((dType & 0xF0) == MMF_CONTENTS_TYPE_1) ||
			 ((dType & 0xF0) == MMF_CONTENTS_TYPE_2) ){
		pAi->dSMAFType = 0;						/* SMAF MA-1/2				  */
	}
	else if	(((dType & 0xF0) == MMF_CONTENTS_TYPE_3) ||
			 ((dType & 0xF0) == MMF_CONTENTS_TYPE_4) ||
			 ((dType & 0xF0) == MMF_CONTENTS_TYPE_5) ){
		switch (dType & 0x0F) {
		case 0x00 :
		case 0x01 :
			pAi->dSMAFType = 0;					/* SMAF MA-1/2				  */
			break;
		case 0x02 :
		case 0x03 :
			pAi->dSMAFType = 1;					/* SMAF MA-3				  */
			break;
		case 0x04 :
		case 0x05 :
		case 0x06 :
		case 0x07 :
		case 0x08 :
			pAi->dSMAFType = 2;					/* SMAF MA-5				  */
			break;
		default :
			return MASMW_ERROR_CONTENTS_TYPE;
		}
	}
	else {
		return MASMW_ERROR_CONTENTS_TYPE;
	}

	dIndex			+= MALIB_SIZE_OF_CHUNKHEADER;
	pAi->pbCnti		= &(pbFile[dIndex]);
	pAi->dCntiSize	= sdSize;
	dIndex			+= (UINT32)sdSize;
	MAAUDCNV_DBGMSG(("MaAudCnv : CNTISize = %ld\n", sdSize));

	sdSize = malib_NextChunk(&pbFile[dIndex], ((UINT32)sdTotalSize - dIndex),
			MALIB_CHUNK_PHASE_MMMDSUB, &dChunkID, &dChunkNo);
	if(dChunkID == MALIB_CHUNKCODE_OPDA) {		/* Optional Data Chunk		  */
		if((sdSize >= 12L) && (dType >= 0x30)) {
			pAi->pbOpda		= &(pbFile[dIndex + MALIB_SIZE_OF_CHUNKHEADER]);
			pAi->dOpdaSize	= (UINT32)sdSize;
		}
	}

	pbChunk		= NULL;
	dChunkSize	= 0;
	while((UINT32)sdTotalSize >
			(UINT32)(dIndex + MALIB_SIZE_OF_CHUNKHEADER + MALIB_SIZE_OF_CRC)) {
		sdSize = malib_NextChunk(&pbFile[dIndex], ((UINT32)sdTotalSize - dIndex),
				MALIB_CHUNK_PHASE_MMMDSUB, &dChunkID, &dChunkNo);
		if(sdSize < 0L) {
			if (sdSize == MALIB_CHUNK_ID_ERROR)
				return (MASMW_ERROR_FILE);
			else
				return (MASMW_ERROR_CHUNK_SIZE);
		}
		dIndex	+= MALIB_SIZE_OF_CHUNKHEADER;
		switch(dChunkID) {
		case MALIB_CHUNKCODE_MTR:				/* Score Track Chunk		  */
			if		(dChunkNo == 0x05) {		/* MTR5 (MA-3)	*/
				if ((pAi->dSMAFType == 1) && (pbChunk == NULL)) {
					pbChunk		= &pbFile[dIndex];
					dChunkSize	= (UINT32)sdSize;
				}
			}
			else if	(dChunkNo == 0x06) {		/* MTR6 (MA-5)	*/
				if ((pAi->dSMAFType == 2) && (pbChunk == NULL)) {
					pbChunk		= &pbFile[dIndex];
					dChunkSize	= (UINT32)sdSize;
				}
			}
			break;

		case MALIB_CHUNKCODE_ATR:				/* Audio Track Chunk		  */
			if		(dChunkNo == 0x00) {		/* ATR0 (MA-2)	*/
				if ((pAi->dSMAFType == 0) && (pbChunk == NULL)) {
					pbChunk		= &pbFile[dIndex];
					dChunkSize	= (UINT32)sdSize;
				}
			}
			break;

		default :
			break;
		}
		dIndex	+= (UINT32)sdSize;
	}

	if(bMode == 3)					return (MASMW_SUCCESS);

	if (pbChunk == NULL)			return MASMW_ERROR_SHORT_LENGTH;

	switch (pAi->dSMAFType) {
	case 0 :									/* MA-2		*/
		return GetWaveInfo_2(pbChunk, dChunkSize, pAi);
		break;
	case 1 :									/* MA-3		*/
		return GetWaveInfo_3(pbChunk, dChunkSize, pAi);
		break;
	case 2 :									/* MA-5		*/
		return GetWaveInfo_5(pbChunk, dChunkSize, pAi);
		break;
	default :
		return MASMW_ERROR_FILE;
	}
}

/****************************************************************************
 *	malib_smafaudio_checker
 *
 *	Function:
 *			Get data Info from the file
 *	Argument:
 *			pbFile		pointer to the SMAF/Audio data.
 *			dFileSize	size of data.
 *			bMode		error check mode(0:No 1:Yes 2:Check only)
 *			pAi			pointer to the MALIBAUDINFO structure.
 *	Return:
 *			0		success.
 *			< 0		error.
 *
 ****************************************************************************/
SINT32	malib_smafaudio_checker(UINT8* pbData, UINT32 dSize, UINT8 bMode,
																 void* pvInfo)
{
	SINT32			sdRet;
	MALIBAUDINFO	sInfo;
	PMALIBAUDINFO	pAi = (PMALIBAUDINFO)pvInfo;

	if(pbData == NULL)					return (MASMW_ERROR_ARGUMENT);
	if(dSize == 0L)						return (MASMW_ERROR_ARGUMENT);
	if(dSize >= MALIB_AUDIO_SIZE_MAX)	return (MASMW_ERROR_ARGUMENT);
	if(bMode >= 4)						return (MASMW_ERROR_ARGUMENT);

	/*	call from music API	*/
	if(pAi == NULL) pAi = &sInfo;

	/*	check data format	*/
	sdRet = SmafAudChecker(pbData, dSize, bMode, pAi);
	if ((sdRet < MASMW_SUCCESS) || (bMode == 3)) {
		return (sdRet);
	}

	/*	calcurate length[ms]	*/
	if(pAi->dFs == 0) return (MASMW_ERROR);

	switch(pAi->dFmt) {
	case 0x01L:	/*	4bit ADPCM mono		*/
		sdRet = (pAi->dSizeOfSource * 2000L + (pAi->dFs - 1L)) / pAi->dFs;
		break;
	case 0x02L:
	case 0x03L:	/*	8bit PCM mono		*/
	case 0x81L:	/*	4bit ADPCM stereo	*/
		sdRet = (pAi->dSizeOfSource * 1000L + (pAi->dFs - 1L)) / pAi->dFs;
		break;
	case 0x82L:
	case 0x83L:	/*	8bit PCM stereo		*/
		sdRet = (pAi->dSizeOfSource * 500L + (pAi->dFs - 1L)) / pAi->dFs;
		break;
	default:
		return (MASMW_ERROR);
	}

	pAi->dPlayTime = (UINT32)sdRet;
	return (sdRet);
}

/*********************************************************************************
 *	GetContentsData2
 *
 *	Description:
 *			get contents information (from MA-2 contents)
 *	Argument:
 *			psCI			pointer to get contents info structure
 *	Return:
 *			0				success
 *			< 0				error code
 ********************************************************************************/
static SINT32			GetContentsData2(
	PMALIBGETCINFO		psCI
)
{
	UINT8				bCode, fbTagMatch, fbDataEnd, bSkip, bData;
	SINT32				sdInfoSize;
	UINT8*				pbBuf;
	UINT32				dSize, dIndex;

	pbBuf	= (psCI->pbCnti + 5);
	dSize	= (psCI->dCntiSize - 5);
	dIndex	= 0;
	bCode	= psCI->pbCnti[2];

	if ( (pbBuf == NULL) || (dSize < 4) ||
		((psCI->bCode != MALIB_CONTENTS_CODE_BINARY) && (psCI->bCode != bCode)) )
		return MASMW_ERROR_UNMATCHED_TAG;

	while (dSize >= (dIndex + 4)) {
		if ((pbBuf[dIndex]		== psCI->bTag[0]) &&
			(pbBuf[dIndex+1]	== psCI->bTag[1])) {
			fbTagMatch	= 1;
			if (((psCI->bTag[0] == 'L') && (psCI->bTag[1] == '2')) ||
				((psCI->bTag[0] == 'M') && (psCI->bTag[1] == '2'))) {
				psCI->pbBuf[0] = pbBuf[dIndex+3];
				return 1;
			}
		}
		else
			fbTagMatch	= 0;
		fbDataEnd	= 0;	bSkip	= 0;	dIndex	+= 3;	sdInfoSize	= 0;
		while ((dSize >= (dIndex + 1)) && (fbDataEnd == 0)) {
			bData	= pbBuf[dIndex];
			dIndex ++;
			if (bSkip == 0) {
				if (bData == ',') {
					if (fbTagMatch == 1)	return sdInfoSize;
					fbDataEnd = 1;
					continue;
				}
				if (bData == '\\') {
					if ((pbBuf[dIndex] == '\\') || (pbBuf[dIndex] == ','))
						bSkip	= 1;
					continue;
				}
				if (bCode == MALIB_CONTENTS_CODE_SJIS) {
					if (((0x81 <= bData) && (bData <= 0x9F)) ||
						((0xE0 <= bData) && (bData <= 0xFC)))
						bSkip	= 2;
				}
				if (bCode == MALIB_CONTENTS_CODE_UTF8) {
					if ((0xC0 <= bData) && (bData <= 0xDF))		bSkip	= 2;
					if ((0xE0 <= bData) && (bData <= 0xFC))		bSkip	= 3;
				}
			}
			if (fbTagMatch == 1) {
				psCI->pbBuf[sdInfoSize] = bData;
				sdInfoSize ++;
				if (psCI->dBufSize == (UINT32)sdInfoSize)	return sdInfoSize;
			}
			if (bSkip != 0)		bSkip --;
		}
	}
	return MASMW_ERROR_UNMATCHED_TAG;
}


/*********************************************************************************
 *	GetContentsData3
 *
 *	Description:
 *			get contents information (from MA-3 contents)
 *	Argument:
 *			get contents information (from MA-2 contents)
 *	Return:
 *			0				success
 *			< 0				error code
 ********************************************************************************/
static SINT32			GetContentsData3(
	PMALIBGETCINFO		psCI
)
{
	UINT8				*pbOpda, *pbDch, *pbInfo;
	UINT32				dOSize,  dDSize, dISize, dIndex, dTemp;
	SINT32				sdChunkSize ;
	UINT32				dChunkID, dChunkNo, i;

	pbOpda	= psCI->pbOption;
	dOSize	= psCI->dOptionSize;
	pbDch	= NULL;
	dDSize	= 0;
	pbInfo	= NULL;
	dISize	= 0;
	dIndex	= 0;

	if ((pbOpda == NULL) || (dOSize < 12))	return MASMW_ERROR_UNMATCHED_TAG;

	while (dOSize > (dIndex + MALIB_SIZE_OF_CHUNKHEADER)) {
		sdChunkSize = malib_NextChunk(&pbOpda[dIndex], (dOSize-dIndex), 
				MALIB_CHUNK_PHASE_OPDASUB, &dChunkID, &dChunkNo);
		if (sdChunkSize < 0)				return MASMW_ERROR;

		dIndex += MALIB_SIZE_OF_CHUNKHEADER;
		if ((dChunkID == MALIB_CHUNKCODE_DCH) && (dChunkNo == psCI->bCode)) {
			pbDch	= &pbOpda[dIndex];
			dDSize	= (UINT32)sdChunkSize;
		}
		dIndex += sdChunkSize;
	}

	if ((pbDch == NULL) || (dDSize < 4))	return MASMW_ERROR_UNMATCHED_TAG;

	dIndex	= 0;
	while (dDSize >= (dIndex + 4)) {
		dTemp	= (((UINT32)pbDch[dIndex + 2]) << 8) + pbDch[dIndex + 3];
		if (dDSize < dTemp + 4 + dIndex)		return MASMW_ERROR;
		if ((pbDch[dIndex]		== psCI->bTag[0]) &&
			(pbDch[dIndex + 1]	== psCI->bTag[1])) {
			pbInfo	= &(pbDch[dIndex + 4]);
			dISize	= dTemp;
		}
		dIndex += (dTemp + 4);
	}
	if (pbInfo == NULL)						return MASMW_ERROR_UNMATCHED_TAG;

	if (dISize > psCI->dBufSize)			dISize	= psCI->dBufSize;
	for (i = 0; i < dISize; i++)			psCI->pbBuf[i] = pbInfo[i];
	return dISize;
}


/*********************************************************************************
 *	malib_GetContentsData
 *
 *	Description:
 *			get contents information
 *	Argument:
 *			psCI			pointer to get contents info structure
 *	Return:
 *			0				success
 *			< 0				error code
 ********************************************************************************/
SINT32					malib_GetContentsData(
	PMALIBGETCINFO		psCI
)
{
	UINT16				wTag;

	wTag	= (UINT16)((((UINT16)psCI->bTag[0]) << 8) + (UINT16)psCI->bTag[1]);

	if ((psCI->pbBuf == NULL) || (psCI->dBufSize == 0) || (psCI->pbCnti == NULL))
		return MASMW_ERROR_ARGUMENT;

	switch (wTag) {
	case 0x4330 :
		*(psCI->pbBuf)	= psCI->pbCnti[0];
		return 1;
	case 0x4331 :
		*(psCI->pbBuf)	= psCI->pbCnti[1];
		return 1;
	case 0x4332 :
		*(psCI->pbBuf)	= psCI->pbCnti[3];
		return 1;
	case 0x4333 :
		*(psCI->pbBuf)	= psCI->pbCnti[4];
		return 1;
	default :
		if (psCI->pbCnti[1] < 0x30)		return GetContentsData2(psCI);
		else							return GetContentsData3(psCI);
	}
}


/****************************************************************************
 *	malib_hvscript_checker
 *
 *	Function:
 *			Check the data is correct HV-Script or not.
 *	Argument:
 *			pbFile		pointer to the HV-Script data.
 *			dFileSize	size of data.
 *	Return:
 *			0		correct HV-Script.
 *			< 0		error data.
 *
 ****************************************************************************/
SINT32	malib_hvscript_checker(UINT8* pbData, UINT32 dSize)
{
	
	/* argument check */
	if(pbData == NULL || 
	   dSize <= 4 ||
	   dSize >  0x7FFFFFFF)
		return MASMW_ERROR_ARGUMENT;
	
	/* HV-Script Header Information Check */
	if(*pbData != 'H' ||
	   *(pbData + 1) != 'V' ||
	   *(pbData + 2) != '#' ||
	   *(pbData + 3) != 'J')
		return MASMW_ERROR_FILE;

	return MASMW_SUCCESS;
}

