/****************************************************************************
 *
 *		Copyright (C) 2002-2003	YAMAHA CORPORATION. All rights reserved.
 *
 *		Module		: mamidcnv.c
 *
 *		Description	: SMF/GML Stream Converter Module.
 *
 *		Version		: 1.1.2 	2003.07.09
 *
 ****************************************************************************/

#include "mamidcnv.h"

/*--------------------------------------------------------------------------*/
/*   Defines                                                                */
/*--------------------------------------------------------------------------*/
#define	SMF_TIMEBASE_SHIFT			2							/*                         */
#define	SMF_TIMEBASE				(1L<<SMF_TIMEBASE_SHIFT)	/* [ms]                    */

#define	MAX_SMF_MESSAGES			256							/*                          */
#define	MAX_SMF_TRACKS				32							/* Should be <= 32          */
#define SMF_MAX_GAIN				76							/* - 6[dB] : 90             */
																/* -18[dB] : 45             */
#define	MINIMUM_LENGTH				(20)

#define MELODY_MAP					(0)
#define DRUM_MAP					(1)
#define NUM_OF_MAPS					(2)

/*--------------------------------------------------------------------------*/
/*   Types                                                                  */
/*--------------------------------------------------------------------------*/
typedef struct _tagMidChInfo
{
	UINT32					dBank;						/* BankH&L (0x00:00..0x7F7F)       */
	UINT32					dCurrBank;					/* BankH&L (0x00:00..0x7F7F)       */
	UINT32					dProg;						/* ProgramChange (0..127)          */
	UINT32					dVolume;					/* ChannelVolume (0..127)          */
	UINT32					dExpression;				/* Expression (0..127)             */
	UINT32					dModulation;				/* Modulation (0..127)             */
	UINT32					dPitchBend;					/* PitchBendH (0..127)             */
	UINT32					dBendRange;					/* CurrentBendRange (0..24)        */
	UINT32					dPreBendRange;				/* LatestBendRange (0..24)         */
	UINT32					dPanpot;					/* Panpot (0..127)                 */
	UINT32					dHold1;						/* Hold1 (0..127)                  */
	UINT32					dMode;						/* 0:MONO, 1:POLY                  */
	UINT32					dRPN;						/* RPN (0x00:00..0xFF7F)           */
	UINT32					dMipMute;					/* Mute switch (1:mute)            */
	UINT32					dKeyCon;					/* 0:Melady, 1:OFF, 2:ON           */
	UINT32					dLedSync;					/* 0:OFF, 1:ON                     */
	UINT32					dVibSync;					/* 0:OFF, 1:ON                     */
	UINT32					dFineTune;					/* 0..0x3FFF                       */
	UINT32					dCoaseTune;					/* 0..0x7F                         */
} MIDCHINFO, *PMIDCHINFO;

typedef struct _tagMIDPACKET
{
	SINT32					sdDeltaTime;
	UINT32					dMsgID;						
	UINT32					dP1;						
	UINT32					dP2;						
	UINT32					dP3;						
} MIDPACKET, *PMIDPACKET;

typedef struct _tagTrack
{
	UINT32					dSmfCmd;					/* CMD @ now                      */
	UINT32					dSize;						/* [byte] 0 measns nothing in it. */
	UINT8*					pbBase;						/* NULL measns nothing in it.     */
	UINT32					dOffset;					/* offset byte                    */
	SINT32					sdTicks;					/*                                */
} TRACKINFO, *PTRACKINFO;

typedef struct _tagOrderList
{
	struct _tagOrderList*	pPrev;
	struct _tagOrderList*	pNext;
	UINT32					dTrack;
	UINT32					dTicks;
} ORDERLIST, *PORDERLIST;

typedef struct _tagMidInfo
{
	UINT32					dTimeResolution;			/* 0..0x7fff                       */
	UINT8*					pbTitle;					/*                                 */
	UINT32					dSizeTitle;					/*                                 */
	UINT8*					pbCopyright;				/*                                 */
	UINT32					dSizeCopyright;				/*                                 */
	UINT32					dNumOfTracks;				/* 1..32                           */
	UINT32					dSmfFormat;					/* 0..1                            */
	UINT32					dSetupBar;					/* 0:No, 1:Yes                     */
	UINT32					dStart;						/* Index after SetupBar            */
	UINT32					dVibNoteVoice;				/* 0:No, 1:Yes                     */

	SINT32					sdTotalTicks;				/* Total ticks                     */
	SINT32					sdDataEndTime;				/* (22.10)[ms]                     */
	SINT32					sdDelta;					/* (22.10)[ms]                     */

	UINT32					dEndFlag;					/*                                 */
	TRACKINFO				TrackInfo[MAX_SMF_TRACKS];	
	
	struct _tagOrderList*	pTopOrderList;				
	struct _tagOrderList*	pDoneOrderList;				
	struct _tagOrderList*	pBottomOrderList;			
	ORDERLIST				OrderList[MAX_SMF_TRACKS + 3];

	MIDCHINFO				ChInfo[16];					/*                                 */
	UINT32					dValid;						/* 0:none, 1:Valid                 */

	UINT8					bVoiceMap[NUM_OF_MAPS][128];/* 0:Empty, 1:Valid                */
} MIDINFO, *PMIDINFO;

typedef	struct _tagMidGlobals
{
	SINT32		sdSeqID;						/* Sequence ID             */
	SINT32		sdFileID;						/* File ID                 */
	UINT32		dEnable;						/* 0:disable               */
	UINT32		dSetup;							/* 1: Just after seek      */

	UINT32		dRamBase;						/*                         */
	UINT32		dRamOffset;						/*                         */
	UINT32		dRamSize;						/*                         */

	MIDINFO		DataInfo[2];					/*                         */

	SINT32		sdSeekTime;						/* [ms]                    */
	SINT32		sdLastMsgTime;					/* (22.10)[ms]             */
	SINT32		sdSmfCurrentTicks;				/* Ticks @ now             */
	SINT32		sdSmfCurrentTime;				/* (22.10)[ms]             */
	SINT32		sdSmfDataEndTime;				/* (22.10)[ms]             */
	SINT32		sdSmfEndTime;					/* (22.10)[ms]             */
	SINT32		sdSmfDelta;						/* (22.10)[ms]             */

	UINT32		dMaxGain;						/* MaxGain (0..127)        */
	UINT32		dMasterVolume;					/* MsaterVolume (0..127)   */

	UINT32		dHoldMsgs;						/* Number of messages in Q */
	UINT32		dHoldPtrR;						/* Pointer for Read        */
	MIDPACKET	MsgBuffer[MAX_SMF_MESSAGES];	/* Message Q               */

	UINT32		dMuteFlag;						/* 0:Normal, 1:MUTE        */

	UINT32		dSyncNoteCh;					/* 0..15                   */
	UINT32		dSyncNoteKey;					/* 0..127, 255:OFF         */
	UINT32					dVibNote;			/* 0:No VibiNote, 1:Yes,VibNote    */

} MIDGLOBAL, *PMIDGLOBAL;


/*--------------------------------------------------------------------------*/
/*   Consts                                                                 */
/*--------------------------------------------------------------------------*/
#include "mamid4opvoice.h"


/*---------------------------------------------------------------------------*/
/*   Globals                                                                 */
/*---------------------------------------------------------------------------*/
static MIDGLOBAL	gMidInfo;
static PMIDGLOBAL	gpMidInfo;


/*---------------------------------------------------------------------------*/
/*   Functions (internal use only)                                           */
/*---------------------------------------------------------------------------*/

/****************************************************************************
 *	InitializeOrderList(PMIDINFO pI)
 *
 *	Description:
 *			Initialize OrderList.
 *	Param:
 *		pI			... pointer to the data info
 *	Return:
 *			none
 ****************************************************************************/
static void InitializeOrderList(PMIDINFO pI)
{
	UINT32 i;

	for (i = 1; i <= MAX_SMF_TRACKS + 1; i++)
	{
		pI->OrderList[i].pPrev = &pI->OrderList[i - 1];
		pI->OrderList[i].pNext = &pI->OrderList[i + 1];
		pI->OrderList[i].dTrack = 0xFF;
		pI->OrderList[i].dTicks = 0xFFFFFFFFL;
	}
	pI->OrderList[0].pPrev = NULL;
	pI->OrderList[0].pNext = &pI->OrderList[1];
	pI->OrderList[MAX_SMF_TRACKS + 2].pPrev = &pI->OrderList[MAX_SMF_TRACKS + 1];
	pI->OrderList[MAX_SMF_TRACKS + 2].pNext = NULL;
	pI->pTopOrderList = &pI->OrderList[0];
	pI->pDoneOrderList = &pI->OrderList[1];
	pI->pBottomOrderList = &pI->OrderList[MAX_SMF_TRACKS + 2];
}


/****************************************************************************
 *	SortOrderList(PMIDINFO pI)
 *
 *	Description:
 *			Sort OrderList. (Ascending order)
 *	Param:
 *		pI			... pointer to the data info
 *	Return:
 *			none
 ****************************************************************************/
static void SortOrderList(PMIDINFO pI)
{
	PORDERLIST pSlot;
	PORDERLIST pTerget;

	pSlot = (pI->pTopOrderList)->pNext;
	(pSlot->pPrev)->pNext = pSlot->pNext;
	(pSlot->pNext)->pPrev = pSlot->pPrev;
	pSlot->dTicks = ((UINT32)pI->TrackInfo[pSlot->dTrack].sdTicks << 5) + pSlot->dTrack;

	pTerget = pSlot->pNext;
	while (pTerget != pI->pDoneOrderList)
	{
		if (pSlot->dTicks <= pTerget->dTicks) break;
		pTerget = pTerget->pNext;
	}

	(pTerget->pPrev)->pNext = pSlot;
	pSlot->pPrev = pTerget->pPrev;
	pTerget->pPrev = pSlot;
	pSlot->pNext = pTerget;
}


/****************************************************************************
 *	InsertOrderList(PMIDINFO pI, UINT32 dTrack, SINT32 sdTicks)
 *
 *	Description:
 *			Add item to the top of the list.
 *	Param:
 *		pI			... pointer to the data info
 *	Return:
 *			none
 ****************************************************************************/
static void InsertOrderList(PMIDINFO pI, UINT32 dTrack, SINT32 sdTicks)
{
	PORDERLIST pTerget;

	if (pI->dNumOfTracks == 1) return;
	if (((pI->dEndFlag >> dTrack) & 0x01) == 0) return;

	pTerget = pI->pDoneOrderList->pNext;
	if (pTerget == pI->pBottomOrderList) return;
	
	pI->pDoneOrderList->pNext = pTerget->pNext;
	(pTerget->pNext)->pPrev = pI->pDoneOrderList;

	pTerget->dTrack = dTrack;
	pTerget->dTicks = ((UINT32)sdTicks << 5) + dTrack;
	pTerget->pPrev = pI->pTopOrderList;
	pTerget->pNext = (pI->pTopOrderList)->pNext;
	((pI->pTopOrderList)->pNext)->pPrev = pTerget;
	(pI->pTopOrderList)->pNext = pTerget;
	
	SortOrderList(pI);
}


/****************************************************************************
 *	RemoveFromOrderList(PMIDINFO pI)
 *
 *	Description:
 *			delete Item from the top of the list.
 *	Param:
 *		pI			... pointer to the data info
 *	Return:
 *			none
 ****************************************************************************/
static void RemoveFromOrderList(PMIDINFO pI)
{
	PORDERLIST pSlot;
	PORDERLIST pTerget;

	pSlot = (pI->pTopOrderList)->pNext;
	(pSlot->pPrev)->pNext = pSlot->pNext;
	(pSlot->pNext)->pPrev = pSlot->pPrev;
	
	pTerget = pI->pBottomOrderList;
	(pTerget->pPrev)->pNext = pSlot;
	pSlot->pPrev = pTerget->pPrev;
	pTerget->pPrev = pSlot;
	pSlot->pNext = pTerget;
}


/****************************************************************************
 *	GetTrackTime(PMIDINFO pI, UINT32 dTrack)
 *
 *	Description:
 *		Get the 1st DT from the list.
 *	Param:
 *		pI			... pointer to the data info
 *		bTrack		... #Track
 *	Return:
 *		0 : NoError, < 0 : Error code
 ****************************************************************************/
static SINT32	GetTrackTime(PMIDINFO pI, UINT32 dTrack)
{
	UINT32		dTemp;
	SINT32		dTime;
	PTRACKINFO	pMt;

	if (((pI->dEndFlag >> dTrack) & 0x01) == 0) return (-1);

	pMt = &(pI->TrackInfo[dTrack]);

	dTime = 0;
	do
	{
		if (pMt->dOffset >= pMt->dSize)
		{
			pI->dEndFlag &= ~(1L << dTrack);
			return (-1);
		}
		dTemp = (UINT32)pMt->pbBase[pMt->dOffset++];
		dTime = (dTime << 7) + (dTemp & 0x7f);
	} while (dTemp >= 0x80);

	pMt->sdTicks += dTime;

	return (0);
}


/****************************************************************************
 *	UpdateTrackTime(PMIDINFO pI, UINT32 dTrack)
 *
 *	Description:
 *		Update the 1st DT on the Track and OrderList
 *	Param:
 *		pI			... pointer to the data info
 *		bTrack		... #Track
 *	Return:
 *		0 : NoError, < 0 : Error code
 ****************************************************************************/
static SINT32	UpdateTrackTime(PMIDINFO pI, UINT32 dTrack)
{
	UINT32		dTemp;
	SINT32		dTime;
	PTRACKINFO	pMt;

	if (pI->dNumOfTracks == 1)
	{
		/* Single track */
		if (((pI->dEndFlag >> dTrack) & 0x01) == 0)
		{
			return (-1);
		}

		pMt = &(pI->TrackInfo[dTrack]);

		dTime = 0;
		do
		{
			if (pMt->dOffset >= pMt->dSize)
			{
				pI->dEndFlag &= ~(1L << dTrack);
				return (-1);
			}
			dTemp = (UINT32)pMt->pbBase[pMt->dOffset++];
			dTime = (dTime << 7) + (dTemp & 0x7f);
		} while (dTemp >= 0x80);

		pMt->sdTicks += dTime;
	}
	else
	{
		/* Multi track */
		if (((pI->dEndFlag >> dTrack) & 0x01) == 0)
		{
			RemoveFromOrderList(pI);
			return (-1);
		}

		pMt = &(pI->TrackInfo[dTrack]);

		dTime = 0;
		do
		{
			if (pMt->dOffset >= pMt->dSize)
			{
				pI->dEndFlag &= ~(1L << dTrack);
				RemoveFromOrderList(pI);
				return (-1);
			}
			dTemp = (UINT32)pMt->pbBase[pMt->dOffset++];
			dTime = (dTime << 7) + (dTemp & 0x7f);
		} while (dTemp >= 0x80);

		pMt->sdTicks += dTime;
		SortOrderList(pI);
	}

	return (0);
}


/****************************************************************************
 *	ResetTimeInfo(PMIDINFO pI)
 *
 *	Description:
 *		Reset time info
 *	Param:
 *		pI			... pointer to the data info
 *	Return:
 *		none
 ****************************************************************************/
static void	ResetTimeInfo(PMIDINFO pI)
{
	SINT32		sdTrack;
	PTRACKINFO	pMt;

	pI->dEndFlag = 0;

	for (sdTrack = 0; sdTrack < (SINT32)pI->dNumOfTracks; sdTrack++)
	{
		pMt = &(pI->TrackInfo[sdTrack]);

		pMt->dSmfCmd = 0;
		pMt->dOffset = 0;
		pMt->sdTicks = 0;
		if (pMt->dSize > 0) pI->dEndFlag |= (1L << sdTrack);
	}
	
	InitializeOrderList(pI);

	for (sdTrack = 0; sdTrack < (SINT32)pI->dNumOfTracks; sdTrack++)
	{
		GetTrackTime(pI, (UINT32)sdTrack);
		pMt = &(pI->TrackInfo[sdTrack]);
		InsertOrderList(pI,  (UINT32)sdTrack, pMt->sdTicks);
	}
}


/****************************************************************************
 *	GetLeastTimeTrack(PMIDINFO pI)
 *
 *	Description:
 *		Get the track@LeasetTime
 *	Param:
 *		pI			... pointer to the setup storage
 *	Return:
 *		0 : NoError, < 0 : Error
 ****************************************************************************/
static SINT32	GetLeastTimeTrack(PMIDINFO pI)
{
	PORDERLIST	pTerget;

	pTerget = (pI->pTopOrderList)->pNext;
	if (pTerget == pI->pBottomOrderList) return (-1);

	return ((SINT32)pTerget->dTrack);
}


/****************************************************************************
 *	GetSMFInfo(PMIDINFO pI)
 *
 *	Description:
 *		Get SMF info from the file
 *	Param:
 *		pI					... pointer to the setup storage
 *	Return:
 *		0 : NoError, < 0 : Error
 ****************************************************************************/
static SINT32	GetSMFInfo(PMIDINFO pI)
{
	UINT32		dCmd;
	UINT32		dCmd2;
	UINT32		dSize;
	
	UINT32		dTemp;
	UINT32		dTime;
	SINT32		sdTotalTicks;
	SINT32		sdCurrentTime;
	SINT32		sdDelta;
	PMIDCHINFO 	pCh;
	UINT32		dCh;

	UINT32		dSetup;			/* bit0:beat@0, bit1:tempo@0, bit2:GmOn@0, bit3:tempo@1 */
	PTRACKINFO	pMt;
	SINT32		sdTr;

	static UINT32	dBank[16];
	static UINT32	dCurrBank[16];

	MASMFCNV_DBGMSG(("MaMidCnv : GetSMFInfo\n"));

	dSetup = 0;
	sdTotalTicks = 0;
	sdCurrentTime = 0;
	sdDelta = (SINT32)(500 << 10) / pI->dTimeResolution;	/* default=0.5sec */

	pI->pbTitle = NULL;
	pI->dSizeTitle = 0;
	pI->pbCopyright = NULL;
	pI->dSizeCopyright = 0;
	pI->dStart = 0;
	pI->dSetupBar = 0;
	pI->dVibNoteVoice = 0;
	
	for (dCh = 0; dCh < NUM_OF_MAPS; dCh++)
	{
		for (dTemp = 0; dTemp < 128; dTemp++)
		{
			pI->bVoiceMap[dCh][dTemp] = 0;
		}
	}
	pI->bVoiceMap[MELODY_MAP][0] = 1;						/* GM Default Piano */

	for (dCh = 0; dCh < 16; dCh++)
	{
		dBank[dCh] = 0;
		dCurrBank[dCh] = 0;
		pCh = &pI->ChInfo[dCh];
		pCh->dKeyCon = 0;
		pCh->dVibSync = 0;
		pCh->dLedSync = 0;
	}

	ResetTimeInfo(pI);

	if (pI->dSmfFormat != 0) dSetup |= 0x20;

	while (pI->dEndFlag != 0)
	{
		if (pI->dNumOfTracks == 1)
		{
			sdTr = 0;
		}
		else
		{
			sdTr = GetLeastTimeTrack(pI);
			if (sdTr < 0) break;
		}
		pMt = &(pI->TrackInfo[sdTr]);
		
		dTime = pMt->sdTicks - sdTotalTicks;
		sdCurrentTime += dTime * sdDelta;
		sdTotalTicks = pMt->sdTicks;
		if ((sdCurrentTime < 0) || (sdTotalTicks > 0x07FFFFFFL))
		{
			MASMFCNV_DBGMSG(("MaMidCnv : GetSMFInfo Error/ CurrentTime=%ld, Time=%ld, Delta=%ld, Tr=%ld, EFlag=%08lX\n", sdCurrentTime, dTime, sdDelta, sdTr, pI->dEndFlag));
			return (MASMW_ERROR_LONG_LENGTH);
		}

		dCmd = (UINT32)pMt->pbBase[pMt->dOffset++];

		if (dCmd < 0xf0)
		{
			/*--- MidiMsg ---*/
			if (dCmd < 0x80)
			{
				dCmd = pMt->dSmfCmd;
				if (dCmd < 0x80) return (MASMW_ERROR_SMF_CMD);
				pMt->dOffset--;
			} else {
				pMt->dSmfCmd = dCmd;
			}
			
			dCh = dCmd & 0x0f;
			
			switch (dCmd & 0xf0)
			{
			case 0x90:	/* NoteOn */
				switch (dCurrBank[dCh])
				{
				case 0x7800:	/* Drum */
					pI->bVoiceMap[DRUM_MAP][pMt->pbBase[pMt->dOffset] & 0x7F] = 1;
					break;

				default:		/* Unknown: default GM */
					if ((dBank[dCh] >> 8) != 0x79)
					{
						if (dCh == 9)
						{
							/* Unknown: default GM Drum */
							pI->bVoiceMap[DRUM_MAP][pMt->pbBase[pMt->dOffset] & 0x7F] = 1;
						}
					}
				}
				pMt->dOffset += 2;
				break;
				
			case 0xC0:	/* Program change */
				switch (dBank[dCh])
				{
				case 0x7900:	/* Melody */
					pI->bVoiceMap[MELODY_MAP][pMt->pbBase[pMt->dOffset] & 0x7F] = 1;
					break;

				case 0x7906:	/* Vibration */
				 	if (pMt->pbBase[pMt->dOffset] == 0x7C)
					{
						pI->dVibNoteVoice = 1;
					}
					break;

				default:		/* Unknown: default GM Melody */
					if ((dBank[dCh] >> 8) == 0x79)
					{
						/* Melody */
						pI->bVoiceMap[MELODY_MAP][pMt->pbBase[pMt->dOffset] & 0x7F] = 1;
					}
					else
					{
						/* default GM Melody */
						if (dCh != 9)
						{
							pI->bVoiceMap[MELODY_MAP][pMt->pbBase[pMt->dOffset] & 0x7F] = 1;
						}
					}
				}

				dCurrBank[dCh] = dBank[dCh];
				pMt->dOffset++;
				break;
			
			case 0xD0:	/* Channel pressure */
				pMt->dOffset++;
				break;
			
			case 0xB0:	/* Control Change */
				switch (pMt->pbBase[pMt->dOffset])
				{
				case 0x00:	/* Bank select(MSB) */
					dBank[dCh] = (dBank[dCh] & 0x00FF) | (pMt->pbBase[pMt->dOffset + 1] << 8);
					break;
    
    			case 0x20:	/* Bank select (LSB) */
					dBank[dCh] = (dBank[dCh] & 0xFF00) | pMt->pbBase[pMt->dOffset + 1];
					break;
				}
				pMt->dOffset += 2;
				break;
			
			default:
				pMt->dOffset += 2;
			}
		}
		else
		{
			switch (dCmd)
			{
			case 0xF0:			/* SysEx */
			case 0xF7:			/* SysEx */
				pMt->dSmfCmd = 0;
				dSize = 0;
				do
				{
					dTemp = (UINT32)pMt->pbBase[pMt->dOffset++];
					dSize = (dSize << 7) + (dTemp & 0x7f);
				} while (dTemp >= 0x80);
				
				if ((dSize == 5) &&
				    (pMt->pbBase[pMt->dOffset] == 0x7e) &&
				    (pMt->pbBase[pMt->dOffset + 1] == 0x7f) &&
				    (pMt->pbBase[pMt->dOffset + 2] == 0x09) &&
				    (pMt->pbBase[pMt->dOffset + 3] == 0x01))
				{
					/* System On */
					if (sdTotalTicks == 0)
					{
						dSetup |= 0x04;
					}
				}
				else 
				{
					if (pI->dSetupBar == 0)
					{
						if ((dSize == 22) &&
						    (pMt->pbBase[pMt->dOffset] == 0x43) &&
						    (pMt->pbBase[pMt->dOffset + 1] == 0x79) &&
						    (pMt->pbBase[pMt->dOffset + 2] == 0x06) &&
						    (pMt->pbBase[pMt->dOffset + 3] == 0x7C)&&
						    (pMt->pbBase[pMt->dOffset + 4] == 0x02))
						{
							/* Channel status */
							for (dCh = 0; dCh < 16; dCh++)
							{
								pCh = &pI->ChInfo[dCh];
								dTemp = pMt->pbBase[pMt->dOffset + 5 + dCh];
								pCh->dKeyCon = (dTemp >> 2) & 0x03;
								pCh->dVibSync = (dTemp >> 1) & 0x01;
								pCh->dLedSync = dTemp & 0x01;
							}
						}
					}
				}

				pMt->dOffset += dSize;
				break;

			case 0xF1:			/* System Msg */
			case 0xF3:			/* System Msg */
				pMt->dOffset++;
				break;
			
			case 0xF2:			/* System Msg */
				pMt->dOffset += 2;
				break;

			case 0xFF:											/* Meta          */
				dCmd2 = (UINT32)pMt->pbBase[pMt->dOffset++];	/* Meta Command  */
				dSize = 0;										/* Size          */
				do
				{
					dTemp = (UINT32)pMt->pbBase[pMt->dOffset++];
					dSize = (dSize << 7) + (dTemp & 0x7f);
				} while (dTemp >= 0x80);

				switch (dCmd2)
				{
				case 0x02:	/* Copyright */
					if (pI->pbCopyright == NULL)
					{
						pI->pbCopyright = &pMt->pbBase[pMt->dOffset];
						pI->dSizeCopyright = dSize;
					}
					break;

				case 0x03:	/* Title */
					if (pI->pbTitle == NULL)
					{
						pI->pbTitle = &pMt->pbBase[pMt->dOffset];
						pI->dSizeTitle = dSize;
					}
					break;

				case 0x2f:		/* End */
					pI->dEndFlag &= ~(1L << sdTr);
					break;
					
				case 0x51:		/* Set Tempo */
					switch (dSize)
					{
					case 3:
					case 4:
						dTime = ((UINT32)pMt->pbBase[pMt->dOffset] << 16) + 
						        ((UINT32)pMt->pbBase[pMt->dOffset + 1] << 8) +
						         (UINT32)pMt->pbBase[pMt->dOffset + 2];
						if ((sdTotalTicks == 0) && (dTime == 250000)) dSetup |= 0x02;
						if (sdTotalTicks == (SINT32)pI->dTimeResolution) dSetup |= 0x08;
						dTime = (dTime << 7) / 125;
						sdDelta = (SINT32)(dTime / pI->dTimeResolution);
					}
					break;
					
				case 0x58:		/* Set TimeSignature */
					if ((sdTotalTicks == 0) && 
					    (pMt->pbBase[pMt->dOffset] == 1) &&
					    (pMt->pbBase[pMt->dOffset + 1] == 2)) dSetup |= 0x01;
					break;
				}
				pMt->dOffset += dSize;
				break;
			}
		}
		
		UpdateTrackTime(pI, (UINT32)sdTr);
		
		if (dSetup == 0x0F)
		{
			dSetup |= 0x10;
			sdCurrentTime = 0;
			pI->dSetupBar = 1;
			pI->dStart = pI->TrackInfo[0].dOffset;
		}
	}

	pI->sdTotalTicks = sdTotalTicks;
	pI->sdDataEndTime = sdCurrentTime;
	
	if (pI->dSetupBar == 0)
	{
		for (dCh = 0; dCh < 16; dCh++)
		{
			pCh = &pI->ChInfo[dCh];
			pCh->dKeyCon = 0;
			pCh->dVibSync = 0;
			pCh->dLedSync = 0;
		}
	}

	MASMFCNV_DBGMSG(("MaMidCnv : GetSMFInfo/ dTotalTicks  = %ld\n", sdTotalTicks));
	MASMFCNV_DBGMSG(("MaMidCnv : GetSMFInfo/ dCurrentTime = %ld\n", sdCurrentTime));

	if ((pI->sdDataEndTime >> 10) <= MINIMUM_LENGTH) return (MASMW_ERROR_SHORT_LENGTH);

	MASMFCNV_DBGMSG(("MaMidCnv : GetSMFInfo/Done\n"));

	return (MASMW_SUCCESS);
}


/****************************************************************************
 *	CheckSMF(UINT8* fp, UINT32 dFsize, UINT8 dMode)
 *
 *	Description:
 *		Check SMF structure
 *	Param:
 *		fp			... pointer to the data
 *		dFsize		... size fo the data
 *		dMode		... error check (0:No, 1:Yes, 2:ErrorCheck, 3:CNTI)
 *	Return:
 *		0 : NoError, < 0 : Error
 ****************************************************************************/
static SINT32	CheckSMF(UINT8* fp, UINT32 dFsize, UINT32 dMode)
{
	UINT32	dTemp;
	UINT32	dSize;
	PMIDINFO pI;
	UINT32	dFormat;
	UINT32	dNumOfTracks;
	UINT32	i;

	MASMFCNV_DBGMSG(("MaMidCnv : CheckSMF\n"));

	while (dFsize >= 22)
	{
		dTemp = ((UINT32)fp[0] << 24) + ((UINT32)fp[1] << 16) +
		        ((UINT32)fp[2] << 8) + (UINT32)fp[3];
		if (dTemp == 0x4D546864)	break;		/* 'MThd' */
		fp ++;
		dFsize --;
	}

	if (dFsize < 22) 
	{
		MASMFCNV_DBGMSG(("MaMidCnv : CheckSMF Error/ Too small size\n"));
		return (MASMW_ERROR_FILE);
	}
	fp += 4;
	dFsize -= 4;

	/*--- Check size ----------------------------------------------------*/
	dTemp = ((UINT32)fp[0] << 24) + ((UINT32)fp[1] << 16) +
	        ((UINT32)fp[2] << 8) + (UINT32)fp[3];
	if (dTemp != 6)
	{
		MASMFCNV_DBGMSG(("MaMidCnv : CheckSMF Error/ Size != 6\n"));
		return (MASMW_ERROR_CHUNK_SIZE);
	}
	fp += 4;
	dFsize -= 4;

	if (dMode < 2)
	{
		if (gpMidInfo->DataInfo[1].dValid == 1) return (MASMW_ERROR);
		pI = &gpMidInfo->DataInfo[1];
	} else {
		pI = &gpMidInfo->DataInfo[0];
	}
	
	/*--- Check format -------------------------------------------------*/
	dFormat = ((UINT32)fp[0] << 8) + (UINT32)fp[1];
	if (dFormat > 1)
	{
		MASMFCNV_DBGMSG(("MaMidCnv : CheckSMF Error/ Not Format 0 or 1\n"));
		return (MASMW_ERROR_SMF_FORMAT);
	}
	
	/*--- Check number of tracks ---------------------------------------*/
	dNumOfTracks = ((UINT32)fp[2] << 8) + (UINT32)fp[3];
	if (dNumOfTracks == 0)
	{
		MASMFCNV_DBGMSG(("MaMidCnv : CheckSMF Error/ Number of Tracks = 0\n"));
		return (MASMW_ERROR_SMF_TRACKNUM);
	}
	if ((dFormat == 0) && (dNumOfTracks != 1))
	{
		MASMFCNV_DBGMSG(("MaMidCnv : CheckSMF Error/ Number of Tracks > 1\n"));
		return (MASMW_ERROR_SMF_TRACKNUM);
	}
	
	if (dNumOfTracks > MAX_SMF_TRACKS) dNumOfTracks = MAX_SMF_TRACKS;
	pI->dNumOfTracks = (UINT8)dNumOfTracks;

	/*--- Check Time unit --------------------------------------------*/
	dTemp = ((UINT32)fp[4] << 8) + (UINT32)fp[5];
	pI->dTimeResolution = dTemp & 0x7fff;
	if (((dTemp & 0x8000) != 0) || (pI->dTimeResolution == 0))
	{
		MASMFCNV_DBGMSG(("MaMidCnv : CheckSMF Error/ Unknown TimeUnit\n"));
		return (MASMW_ERROR_SMF_TIMEUNIT);
	}
	fp += 6;
	dFsize -= 6;
	
	for (i = 0; i < dNumOfTracks; i++)
	{
		/*--- Check chunk name --------------------------------------------*/
		while (dFsize >= 8)
		{
			dTemp = ((UINT32)fp[0] << 24) + ((UINT32)fp[1] << 16) +
			        ((UINT32)fp[2] << 8) + (UINT32)fp[3];
			if (dTemp == 0x4D54726B)	break;	/* 'MTrk' */
			fp ++;
			dFsize --;
		}

		if (dFsize < 8)
		{
			MASMFCNV_DBGMSG(("MaMidCnv : CheckSMF Error/ Bad size\n"));
			return (MASMW_ERROR_CHUNK_SIZE);
		}

		/*--- Check size ----------------------------------------------------*/
		dSize = ((UINT32)fp[4] << 24) + ((UINT32)fp[5] << 16) +
		        ((UINT32)fp[6] << 8) + (UINT32)fp[7];

		if (dFsize < (dSize + 8))
		{
			MASMFCNV_DBGMSG(("MaMidCnv : CheckSMF Error/ Bad size [%ld] vs [%ld]\n", dFsize, dSize + 22));
			return (MASMW_ERROR_CHUNK_SIZE);
		}
		pI->TrackInfo[i].pbBase = &fp[8];
		pI->TrackInfo[i].dSize = dSize;
		fp += (dSize + 8);
		dFsize -= (dSize + 8);
	}
	pI->dSmfFormat = dFormat;

	return (GetSMFInfo(pI));
}


/****************************************************************************
 *	RegVoice(void)
 *
 *	Desc.
 *		Register default 4op voices.
 *	Param
 *		none
 *	Return
 *      none
 ****************************************************************************/
static void RegVoice(void)
{
	UINT32	i, j;
	UINT8*	pbVoiceParam;
	UINT32	dDrumKey;
	UINT32	dAdr;
	UINT32	dMuteAdr;
	PMIDINFO	pI;

	pI = &gpMidInfo->DataInfo[1];
	dMuteAdr = 0;
	
	/* MutedVoice */
	if ((gpMidInfo->dRamOffset + 30) <= gpMidInfo->dRamSize)
	{
		pbVoiceParam = (UINT8*)&gbMuteVoice[2];
		dMuteAdr = gpMidInfo->dRamBase + gpMidInfo->dRamOffset;
		MaDevDrv_SendDirectRamData(dMuteAdr, 0, pbVoiceParam, 30);
		MaSndDrv_SetVoice(gpMidInfo->sdSeqID, 0x06, 0x7C, 1, 0, dMuteAdr);
		gpMidInfo->dRamOffset += 30;
	}


	/* Melody FM */
	for (i = 0; i < 128; i++)
	{
		if (pI->bVoiceMap[MELODY_MAP][i] == 1)
		{
			if ((gpMidInfo->dRamOffset + 30) <= gpMidInfo->dRamSize)
			{
				pbVoiceParam = (UINT8*)&gbMelodyVoiceTable[i][2];
				dAdr = gpMidInfo->dRamBase + gpMidInfo->dRamOffset;
				MaDevDrv_SendDirectRamData(dAdr, 0, pbVoiceParam, 30);
				MaSndDrv_SetVoice(gpMidInfo->sdSeqID, 0x00, i, 1, 0, dAdr);
				gpMidInfo->dRamOffset += 30;
			}
		}
	}

	/* Drum FM */
	for (i = 35; i < 82; i++)
	{
		if (pI->bVoiceMap[DRUM_MAP][i] == 1)
		{
			j = i - 35;
			if (gbDrumVoiceTable[j][0] == 0)
			{
				if ((gpMidInfo->dRamOffset + 30) <= gpMidInfo->dRamSize)
				{
					pbVoiceParam = (UINT8*)&gbDrumVoiceTable[j][2];
					dDrumKey = (UINT32)gbDrumVoiceTable[j][1];
					dAdr = gpMidInfo->dRamBase + gpMidInfo->dRamOffset;
					MaDevDrv_SendDirectRamData(dAdr, 0, pbVoiceParam, 30);
					MaSndDrv_SetVoice(gpMidInfo->sdSeqID, 0x80, i, 1, dDrumKey, dAdr);
					gpMidInfo->dRamOffset += 30;
				}
			}
		}
	}
}


/****************************************************************************
 *	ErrorFunction(SINT32 sdTime, UINT32 dCh, UINT32 dPara1, UINT32 dPara2)
 *
 *	Desc.
 *		Error function
 *	Param
 *		sdTime		Delta time
 *		dCh			#Channel(0..15)
 *		dPara1		Undefined
 *		dPara2		Undefined
 *	Return
 *      none
 ****************************************************************************/
static void	ErrorFunction(SINT32 sdTime, UINT32 dCh, UINT32 dPara1, UINT32 dPara2)
{
	(void)sdTime;
	(void)dCh;
	(void)dPara1;
	(void)dPara2;

	MASMFCNV_DBGMSG(("MaMidCnv : ErrorFunction\n"));
}


/****************************************************************************
 *	NotSupported(SINT32 sdTime, UINT32 dCh, UINT32 dPara1, UINT32 dPara2)
 *
 *	Desc.
 *		Error function
 *	Param
 *		sdTime		Delta time
 *		dCh			#Channel(0..15)
 *		dPara1		Undefined
 *		dPara2		Undefined
 *	Return
 *      none
 ****************************************************************************/
static void	NotSupported(SINT32 sdTime, UINT32 dCh, UINT32 dPara1, UINT32 dPara2)
{
	(void)sdTime;
	(void)dCh;
	(void)dPara1;
	(void)dPara2;

	MASMFCNV_DBGMSG(("MaMidCnv : NotSupported\n"));
}


/****************************************************************************
 *	SendLedSync(SINT32 sdTime, UINT32 dCh, UINT32 dVal)
 *
 *	Desc.
 *		Send LedSync message
 *	Param
 *		sdTime		Delta time
 *		dCh			#Channel(0..15)
 *		dVal		0:OFF, 1:SYNC_ON
 *	Return
 *      none
 ****************************************************************************/
static void SendLedSync(SINT32 sdTime, UINT32 dCh, UINT32 dVal)
{
	PMIDPACKET	pMsg;

	MASMFCNV_DBGMSG(("MaMidCnv : %8ld:SendLedSync[%d] = %8ld\n", sdTime, dCh, dVal));

	if (sdTime >= 0)
	{
		pMsg = &gpMidInfo->MsgBuffer[gpMidInfo->dHoldMsgs++];
		pMsg->sdDeltaTime = sdTime;
		pMsg->dMsgID = MASNDDRV_CMD_SET_LED_SYNC_CH;
		pMsg->dP1 = dCh;
		pMsg->dP2 = dVal;
		gpMidInfo->sdLastMsgTime += sdTime << (SMF_TIMEBASE_SHIFT + 10);
	}
}


/****************************************************************************
 *	SendVibSync(SINT32 sdTime, UINT32 dCh, UINT32 dVal)
 *
 *	Desc.
 *		Send MotorSync message
 *	Param
 *		sdTime		Delta time
 *		dCh			#Channel(0..15)
 *		dVal		0:OFF, 1:SYNC_ON
 *	Return
 *      none
 ****************************************************************************/
static void SendVibSync(SINT32 sdTime, UINT32 dCh, UINT32 dVal)
{
	PMIDPACKET	pMsg;

	MASMFCNV_DBGMSG(("MaMidCnv : %8ld:SendVibSync[%d] = %8ld\n", sdTime, dCh, dVal));

	if (sdTime >= 0)
	{
		pMsg = &gpMidInfo->MsgBuffer[gpMidInfo->dHoldMsgs++];
		pMsg->sdDeltaTime = sdTime;
		pMsg->dMsgID = MASNDDRV_CMD_SET_MOTOR_SYNC_CH;
		pMsg->dP1 = dCh;
		pMsg->dP2 = dVal;
		gpMidInfo->sdLastMsgTime += sdTime << (SMF_TIMEBASE_SHIFT + 10);
	}
}


/****************************************************************************
 *	SendMasterVolume(SINT32 sdTime, UINT32 dVol)
 *
 *	Desc.
 *		Send MasterVolume message
 *	Param
 *		sdTime		Delta time
 *		dVol		Master volume (0..127)
 *	Return
 *      none
 ****************************************************************************/
static void	SendMasterVolume(SINT32 sdTime, UINT32 dVol)
{
	PMIDPACKET	pMsg;

	MASMFCNV_DBGMSG(("MaMidCnv : %8ld:SendMasterVolume[%ld]\n", sdTime, dVol));

	if (sdTime >= 0)
	{
		if (gpMidInfo->dHoldMsgs < MAX_SMF_MESSAGES)
		{
			pMsg = &gpMidInfo->MsgBuffer[gpMidInfo->dHoldMsgs++];
			pMsg->sdDeltaTime = sdTime;
			pMsg->dMsgID = MASNDDRV_CMD_MASTER_VOLUME;
			pMsg->dP1 = dVol;
			gpMidInfo->sdLastMsgTime += sdTime << (SMF_TIMEBASE_SHIFT + 10);
		}
	}
}


/****************************************************************************
 *	SendUserEvent(SINT32 sdTime, UINT32 dID)
 *
 *	Desc.
 *		Send USER_EVENT message
 *	Param
 *		sdTime		Delta time
 *		dID			#UserEvent(0..127)
 *	Return
 *      none
 ****************************************************************************/
static void	SendUserEvent(SINT32 sdTime, UINT32 dID)
{
	PMIDPACKET	pMsg;

	MASMFCNV_DBGMSG(("MaMidCnv : %8ld:SendUserEvent[%ld]\n", sdTime, dID));

	if (sdTime >= 0)
	{
		pMsg = &gpMidInfo->MsgBuffer[gpMidInfo->dHoldMsgs++];
		pMsg->sdDeltaTime = sdTime;
		pMsg->dMsgID = MASNDDRV_CMD_USER_EVENT;
		pMsg->dP1 = dID;
		gpMidInfo->sdLastMsgTime += sdTime << (SMF_TIMEBASE_SHIFT + 10);
	}
}


/****************************************************************************
 *	SendNoteOff(SINT32 sdTime, UINT32 dCh, UINT32 dKey, UINT32 dVel)
 *
 *	Desc.
 *		Send NoteOff message
 *	Param
 *		sdTime		Delta time
 *		dCh			#Channel(0..15)
 *		dKey		#Key (0..127)
 *		dVel		Velocity (0..127)
 *	Return
 *      none
 ****************************************************************************/
static void	SendNoteOff(SINT32 sdTime, UINT32 dCh, UINT32 dKey, UINT32 dVel)
{
	PMIDPACKET	pMsg;
	(void)dVel;

	MASMFCNV_DBGMSG(("MaMidCnv : %8ld:SendNoteOff[%ld, %ld, %ld]\n", sdTime, dCh, dKey, dVel));

	if (sdTime >= 0)
	{
		pMsg = &gpMidInfo->MsgBuffer[gpMidInfo->dHoldMsgs++];
		pMsg->sdDeltaTime = sdTime;
		pMsg->dMsgID = MASNDDRV_CMD_NOTE_OFF;
		pMsg->dP1 = dCh;
		pMsg->dP2 = dKey;
		gpMidInfo->sdLastMsgTime += sdTime << (SMF_TIMEBASE_SHIFT + 10);
	}
}


/****************************************************************************
 *	SendNoteOn(SINT32 sdTime, UINT32 dCh, UINT32 dKey, UINT32 dVel)
 *
 *	Desc.
 *		Send NoteOn message
 *	Param
 *		sdTime		Delta time
 *		dCh			#Channel(0..15)
 *		dKey		#Key (0..127)
 *		dVel		Velocity (0..127)
 *	Return
 *      none
 ****************************************************************************/
static void	SendNoteOn(SINT32 sdTime, UINT32 dCh, UINT32 dKey, UINT32 dVel)
{
	PMIDPACKET	pMsg;
	PMIDCHINFO	pCh;

	MASMFCNV_DBGMSG(("MaMidCnv : %8ld:SendNoteOn[%ld, %ld, %ld]\n", sdTime, dCh, dKey, dVel));

	if (sdTime >= 0)
	{
		if (dVel > 0)
		{
			pCh = &gpMidInfo->DataInfo[1].ChInfo[dCh];
			if (pCh->dMipMute == 0)									/* MIP support             */
			{
				pMsg = &gpMidInfo->MsgBuffer[gpMidInfo->dHoldMsgs++];
				pMsg->sdDeltaTime = sdTime;
				pMsg->dMsgID = MASNDDRV_CMD_NOTE_ON;
				pMsg->dP1 = dCh;
				pMsg->dP2 = dKey;
				pMsg->dP3 = dVel;
				gpMidInfo->sdLastMsgTime += sdTime << (SMF_TIMEBASE_SHIFT + 10);

				if ((dKey == gpMidInfo->dSyncNoteKey) && (dCh == gpMidInfo->dSyncNoteCh))
					SendUserEvent(0, MASMW_SET_EVENTNOTE);
			}
			else
			{
				if ((dKey == gpMidInfo->dSyncNoteKey) && (dCh == gpMidInfo->dSyncNoteCh))
					SendUserEvent(sdTime, MASMW_SET_EVENTNOTE);
			}
		}
		else
		{
		 	SendNoteOff(sdTime, dCh, dKey, dVel);
		}
	}
}


/****************************************************************************
 *	SendProgram(SINT32 sdTime, UINT32 dCh, UINT32 dProg, UINT32 dPara2)
 *
 *	Desc.
 *		Send ProgramChange message
 *	Param
 *		sdTime		Delta time
 *		dCh			#Channel(0..15)
 *		dProg		#Program (0..127)
 *		dPara2	Undefined
 *	Return
 *		none
 ****************************************************************************/
static void	SendProgram(SINT32 sdTime, UINT32 dCh, UINT32 dProg, UINT32 dPara2)
{
	UINT32		dBk, dPg;
	UINT32		dBb;
	UINT32		dPp;
	PMIDPACKET	pMsg;
	PMIDCHINFO	pCh;
	UINT32		dVibChange;
	(void)dPara2;
	
	MASMFCNV_DBGMSG(("MaMidCnv : %8ld:SendProgram[%ld] = %ld\n", sdTime, dCh, dProg));

	pCh = &gpMidInfo->DataInfo[1].ChInfo[dCh];
	pCh->dProg = dProg;
	dBb = pCh->dBank;
	pCh->dCurrBank = dBb;

	dVibChange = 0;
	if ((dBb == 0x7906) && (dProg == 0x7C))
	{
		if (pCh->dVibSync == 0)
		{
			gpMidInfo->dVibNote = 1;
			dVibChange = 1;
			pCh->dVibSync = 1;
		}
	}
	else
	{
		if (gpMidInfo->dVibNote == 1)
		{
			if (pCh->dVibSync == 1)
			{
				dVibChange = 1;
				pCh->dVibSync = 0;
			}
		}
	}
	
	if (sdTime >= 0)
	{
		dPp = dProg;
		
		switch (dBb)
		{
		case 0x7900:	/* Melody */
			dBk = 0;
			dPg = dPp;
			break;

		case 0x7800:	/* Drum */
			dBk = 128;
			dPg = 0;
			break;

		case 0x7906:	/* Ring Vibrator (SP-MIDI) */
			dBk = 6;
			dPg = 0x7C;
			break;

		default:		/* Unkonwn : Default GM */
			if ((dBb >> 8) == 0x79)
			{
				/* Melody */
				dBk = 0;
				dPg = dPp;
			}
			else
			{
				/* default */
				if (dCh == 9)
				{
					dBk = 128;
					dPg = 0;
				}
				else
				{
					dBk = 0;
					dPg = dPp;
				}
			}
		}
		pMsg = &gpMidInfo->MsgBuffer[gpMidInfo->dHoldMsgs++];
		pMsg->sdDeltaTime = sdTime;
		pMsg->dMsgID = MASNDDRV_CMD_PROGRAM_CHANGE;
		pMsg->dP1 = dCh;
		pMsg->dP2 = dBk;
		pMsg->dP3 = dPg;
		gpMidInfo->sdLastMsgTime += sdTime << (SMF_TIMEBASE_SHIFT + 10);
		
		if (dVibChange == 1) SendVibSync(0, dCh, pCh->dVibSync);
	}
}


/****************************************************************************
 *	SendModDepth(SINT32 sdTime, UINT32 dCh, UINT32 dMod)
 *
 *	Desc.
 *		Send ModDepth message
 *	Param
 *		sdTime		Delta time
 *		dCh			#Channel(0..15)
 *		dMod		Modulation(0..127)
 *	Return
 *      none
 ****************************************************************************/
static void	SendModDepth(SINT32 sdTime, UINT32 dCh, UINT32 dMod)
{
	PMIDPACKET	pMsg;
	PMIDCHINFO	pCh;
	static const UINT8	bModTable[128] = 
	{
		0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
		1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
		2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
		2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
		3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3,
		3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3,
		4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4,
		4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4
	};

	MASMFCNV_DBGMSG(("MaMidCnv : %8ld:SendModDepth[%ld, %ld]\n", sdTime, dCh, dMod));

	if (sdTime >= 0)
	{
		pMsg = &gpMidInfo->MsgBuffer[gpMidInfo->dHoldMsgs++];
		pMsg->sdDeltaTime = sdTime;
		pMsg->dMsgID = MASNDDRV_CMD_MODULATION_DEPTH;
		pMsg->dP1 = dCh;
		pMsg->dP2 = (UINT32)bModTable[dMod & 0x7F];
		gpMidInfo->sdLastMsgTime += sdTime << (SMF_TIMEBASE_SHIFT + 10);
	} else {
		pCh = &gpMidInfo->DataInfo[1].ChInfo[dCh];
		pCh->dModulation = dMod;
	}
}


/****************************************************************************
 *	SendChVol(SINT32 sdTime, UINT32 dCh, UINT32 dVol)
 *
 *	Desc.
 *		Send ChVol message
 *	Param
 *		sdTime		Delta time
 *		dCh			#Channel(0..15)
 *		dVol		ChVolume (0..127)
 *	Return
 *      none
 ****************************************************************************/
static void	SendChVol(SINT32 sdTime, UINT32 dCh, UINT32 dVol)
{
	PMIDPACKET	pMsg;
	PMIDCHINFO	pCh;

	MASMFCNV_DBGMSG(("MaMidCnv : %8ld:SendChVol[%ld] = %ld\n", sdTime, dCh, dVol));

	dVol &= 0x7F;
	if (sdTime >= 0)
	{
		pMsg = &gpMidInfo->MsgBuffer[gpMidInfo->dHoldMsgs++];
		pMsg->sdDeltaTime = sdTime;
		pMsg->dMsgID = MASNDDRV_CMD_CHANNEL_VOLUME;
		pMsg->dP1 = dCh;
		pMsg->dP2 = dVol;
		gpMidInfo->sdLastMsgTime += sdTime << (SMF_TIMEBASE_SHIFT + 10);
	} else {
		pCh = &gpMidInfo->DataInfo[1].ChInfo[dCh];
		pCh->dVolume = dVol;
	}
}


/****************************************************************************
 *	SendExpression(SINT32 sdTime, UINT32 dCh, UINT32 dVol)
 *
 *	Desc.
 *		Send Expression message
 *	Param
 *		sdTime		Delta time
 *		dCh			#Channel(0..15)
 *		dVol		Expression(0..127)
 *	Return
 *      none
 ****************************************************************************/
static void	SendExpression(SINT32 sdTime, UINT32 dCh, UINT32 dVol)
{
	PMIDPACKET	pMsg;
	PMIDCHINFO	pCh;
	
	MASMFCNV_DBGMSG(("MaMidCnv : %8ld:SendExpression[%ld] = %ld, %ld\n", sdTime, dCh, dVol, gpMidInfo->dMasterVolume));

	dVol &= 0x7F;
	pCh = &gpMidInfo->DataInfo[1].ChInfo[dCh];
	pCh->dExpression = dVol;

	if (sdTime >= 0)
	{
		pMsg = &gpMidInfo->MsgBuffer[gpMidInfo->dHoldMsgs++];
		pMsg->sdDeltaTime = sdTime;
		pMsg->dMsgID = MASNDDRV_CMD_EXPRESSION;
		pMsg->dP1 = dCh;
		pMsg->dP2 = MaSndDrv_Mix40LogVol(dVol, gpMidInfo->dMasterVolume);
		gpMidInfo->sdLastMsgTime += sdTime << (SMF_TIMEBASE_SHIFT + 10);
	}
}


/****************************************************************************
 *	SendPanpot(SINT32 sdTime, UINT32 dCh, UINT32 dPan)
 *
 *	Desc.
 *		Send Expression message
 *	Param
 *		sdTime		Delta time
 *		dCh			#Channel(0..15)
 *		dPan		Panpot (0..127)
 *	Return
 *      none
 ****************************************************************************/
static void	SendPanpot(SINT32 sdTime, UINT32 dCh, UINT32 dPan)
{
	PMIDPACKET	pMsg;
	PMIDCHINFO	pCh;

	MASMFCNV_DBGMSG(("MaMidCnv : %8ld:SendPanpot[%ld] = %ld\n", sdTime, dCh, dPan));

	dPan &= 0x7F;

	if (sdTime >= 0)
	{
		pMsg = &gpMidInfo->MsgBuffer[gpMidInfo->dHoldMsgs++];
		pMsg->sdDeltaTime = sdTime;
		pMsg->dMsgID = MASNDDRV_CMD_PANPOT;
		pMsg->dP1 = dCh;
		pMsg->dP2 = dPan;
		gpMidInfo->sdLastMsgTime += sdTime << (SMF_TIMEBASE_SHIFT + 10);
	} else {
		pCh = &gpMidInfo->DataInfo[1].ChInfo[dCh];
		pCh->dPanpot =dPan;
	}
}


/****************************************************************************
 *	SendHold1(SINT32 sdTime, UINT32 dCh, UINT32 dVal)
 *
 *	Desc.
 *		Send Hold1 message
 *	Param
 *		sdTime		Delta time
 *		dCh			#Channel(0..15)
 *		dVal		Hold1 (0..127)
 *	Return
 *      none
 ****************************************************************************/
static void	SendHold1(SINT32 sdTime, UINT32 dCh, UINT32 dVal)
{
	PMIDPACKET	pMsg;
	PMIDCHINFO	pCh;

	MASMFCNV_DBGMSG(("MaMidCnv : %8ld:SendHold1[%ld] = %ld\n", sdTime, dCh, dVal));

	if (sdTime >= 0)
	{
		pMsg = &gpMidInfo->MsgBuffer[gpMidInfo->dHoldMsgs++];
		pMsg->sdDeltaTime = sdTime;
		pMsg->dMsgID = MASNDDRV_CMD_HOLD1;
		pMsg->dP1 = dCh;
		pMsg->dP2 = ((dVal < 64) ? 0 : 1);
		gpMidInfo->sdLastMsgTime += sdTime << (SMF_TIMEBASE_SHIFT + 10);
	} else {
		pCh = &gpMidInfo->DataInfo[1].ChInfo[dCh];
		pCh->dHold1 = dVal;
	}
}


/****************************************************************************
 *	SendBendRange(SINT32 sdTime, UINT32 dCh, UINT32 dVal)
 *
 *	Desc.
 *		Send PitchBendRange message
 *	Param
 *		sdTime		Delta time
 *		dCh			#Channel(0..15)
 *		dVal		BendRange (0..24)
 *	Return
 *      none
 ****************************************************************************/
static void	SendBendRange(SINT32 sdTime, UINT32 dCh, UINT32 dVal)
{
	PMIDPACKET	pMsg;
	PMIDCHINFO	pCh;

	MASMFCNV_DBGMSG(("MaMidCnv : %8ld:SendBendRange[%ld] = %ld\n", sdTime, dCh, dVal));

	if (sdTime >= 0)
	{
		pMsg = &gpMidInfo->MsgBuffer[gpMidInfo->dHoldMsgs++];
		pMsg->sdDeltaTime = sdTime;
		if (dVal <= 24)
		{
			pMsg->dMsgID = MASNDDRV_CMD_BEND_RANGE;
			pMsg->dP1 = dCh;
			pMsg->dP2 = dVal;
		}
		else
		{
			pMsg->dMsgID = MASNDDRV_CMD_NOP;
			pMsg->dP1 = 0;
		}
		gpMidInfo->sdLastMsgTime += sdTime << (SMF_TIMEBASE_SHIFT + 10);
	} else {
		pCh = &gpMidInfo->DataInfo[1].ChInfo[dCh];
		pCh->dPreBendRange = dVal;
	}
}


/****************************************************************************
 *	SendFineTune(SINT32 sdTime, UINT32 dCh, UINT32 dVal)
 *
 *	Desc.
 *		Send FineTune message
 *	Param
 *		sdTime		Delta time
 *		dCh			#Channel(0..15)
 *		dVal		Tune (0..0x3FFF)
 *	Return
 *      none
 ****************************************************************************/
static void	SendFineTune(SINT32 sdTime, UINT32 dCh, UINT32 dVal)
{
	PMIDPACKET	pMsg;

	MASMFCNV_DBGMSG(("MaMidCnv : %8ld:SendFineTune[%ld] = %ld\n", sdTime, dCh, dVal));

	if (sdTime >= 0)
	{
		pMsg = &gpMidInfo->MsgBuffer[gpMidInfo->dHoldMsgs++];
		pMsg->sdDeltaTime = sdTime;

		pMsg->dMsgID = MASNDDRV_CMD_FINE_TUNE;
		pMsg->dP1 = dCh;
		pMsg->dP2 = dVal;

		gpMidInfo->sdLastMsgTime += sdTime << (SMF_TIMEBASE_SHIFT + 10);
	}
}


/****************************************************************************
 *	SendCoaseTune(SINT32 sdTime, UINT32 dCh, UINT32 dVal)
 *
 *	Desc.
 *		Send CoaseTune message
 *	Param
 *		sdTime		Delta time
 *		dCh			#Channel(0..15)
 *		dVal		Tune (0..0x7F)
 *	Return
 *      none
 ****************************************************************************/
static void	SendCoaseTune(SINT32 sdTime, UINT32 dCh, UINT32 dVal)
{
	PMIDPACKET	pMsg;

	MASMFCNV_DBGMSG(("MaMidCnv : %8ld:SendCoaseTune[%ld] = %ld\n", sdTime, dCh, dVal));

	if (sdTime >= 0)
	{
		pMsg = &gpMidInfo->MsgBuffer[gpMidInfo->dHoldMsgs++];
		pMsg->sdDeltaTime = sdTime;

		pMsg->dMsgID = MASNDDRV_CMD_COASE_TUNE;
		pMsg->dP1 = dCh;
		pMsg->dP2 = dVal;

		gpMidInfo->sdLastMsgTime += sdTime << (SMF_TIMEBASE_SHIFT + 10);
	}
}


/****************************************************************************
 *	SendAllSoundOff(SINT32 sdTime, UINT32 dCh)
 *
 *	Desc.
 *		Send AllSoundOff message
 *	Param
 *		sdTime		Delta time
 *		dCh			#Channel(0..15)
 *	Return
 *      none
 ****************************************************************************/
static void	SendAllSoundOff(SINT32 sdTime, UINT32 dCh)
{
	PMIDPACKET	pMsg;

	MASMFCNV_DBGMSG(("MaMidCnv : %8ld:SendAllSoundOff[%ld] \n", sdTime, dCh));

	if (sdTime >= 0)
	{
		pMsg = &gpMidInfo->MsgBuffer[gpMidInfo->dHoldMsgs++];
		pMsg->sdDeltaTime = sdTime;
		pMsg->dMsgID = MASNDDRV_CMD_ALL_SOUND_OFF;
		pMsg->dP1 = dCh;
		gpMidInfo->sdLastMsgTime += sdTime << (SMF_TIMEBASE_SHIFT + 10);
	}
}


/****************************************************************************
 *	SendAllNoteOff(SINT32 sdTime, UINT32 dCh)
 *
 *	Desc.
 *		Send AllNoteOff message
 *	Param
 *		sdTime		Delta time
 *		dCh			#Channel(0..15)
 *	Return
 *      none
 ****************************************************************************/
static void	SendAllNoteOff(SINT32 sdTime, UINT32 dCh)
{
	PMIDPACKET	pMsg;

	MASMFCNV_DBGMSG(("MaMidCnv : %8ld:SendAllNoteOff[%ld]\n", sdTime, dCh));

	if (sdTime >= 0)
	{
		pMsg = &gpMidInfo->MsgBuffer[gpMidInfo->dHoldMsgs++];
		pMsg->sdDeltaTime = sdTime;
		pMsg->dMsgID = MASNDDRV_CMD_ALL_NOTE_OFF;
		pMsg->dP1 = dCh;
		gpMidInfo->sdLastMsgTime += sdTime << (SMF_TIMEBASE_SHIFT + 10);
	}
}


/****************************************************************************
 *	SendResetAllCtl(SINT32 sdTime, UINT32 dCh)
 *
 *	Desc.
 *		Send ResetAllCtl message
 *	Param
 *		sdTime		Delta time
 *		dCh			#Channel(0..15)
 *	Return
 *      none
 ****************************************************************************/
static void	SendResetAllCtl(SINT32 sdTime, UINT32 dCh)
{
	PMIDPACKET	pMsg;
	PMIDCHINFO	pCh;

	MASMFCNV_DBGMSG(("MaMidCnv : %8ld:SendResetAllCtl[%ld] \n", sdTime, dCh));

	pCh = &gpMidInfo->DataInfo[1].ChInfo[dCh];
	pCh->dRPN = 0x7F7F;
	pCh->dExpression = 127;

	if (sdTime >= 0)
	{
		pMsg = &gpMidInfo->MsgBuffer[gpMidInfo->dHoldMsgs++];
		pMsg->sdDeltaTime = sdTime;
		pMsg->dMsgID = MASNDDRV_CMD_RESET_ALL_CONTROLLERS;
		pMsg->dP1 = dCh;
		gpMidInfo->sdLastMsgTime += sdTime << (SMF_TIMEBASE_SHIFT + 10);
		
		SendExpression(0, dCh, pCh->dExpression);
	} else {
		pCh->dModulation = 0;
		pCh->dBendRange = pCh->dPreBendRange;
		pCh->dPitchBend = 0x40;
		pCh->dHold1 = 0;
	}
}


/****************************************************************************
 *	SendMonoOn(SINT32 sdTime, UINT32 dCh, UINT32 dVal)
 *
 *	Desc.
 *		Send MonoOn message
 *	Param
 *		sdTime		Delta time
 *		dCh			#Channel(0..15)
 *		dVal		Shuld be 0
 *	Return
 *      none
 ****************************************************************************/
static void	SendMonoOn(SINT32 sdTime, UINT32 dCh, UINT32 dVal)
{
	PMIDPACKET	pMsg;
	PMIDCHINFO	pCh;
	(void)		dVal;

	MASMFCNV_DBGMSG(("MaMidCnv : %8ld:SendMonoOn[%ld] \n", sdTime, dCh));

	if (sdTime >= 0)
	{
		pMsg = &gpMidInfo->MsgBuffer[gpMidInfo->dHoldMsgs++];
		pMsg->sdDeltaTime = sdTime;
		pMsg->dMsgID = MASNDDRV_CMD_MONO_MODE_ON;
		pMsg->dP1 = dCh;
		gpMidInfo->sdLastMsgTime += sdTime << (SMF_TIMEBASE_SHIFT + 10);
	} else {
		pCh = &gpMidInfo->DataInfo[1].ChInfo[dCh];
		pCh->dMode = 0;
	}
}


/****************************************************************************
 *	SendPolyOn(SINT32 sdTime, UINT32 dCh, UINT32 dVal)
 *
 *	Desc.
 *		Send MonoOn message
 *	Param
 *		sdTime		Delta time
 *		dCh			#Channel(0..15)
 *		dVal		Shuld be 0
 *	Return
 *      none
 ****************************************************************************/
static void	SendPolyOn(SINT32 sdTime, UINT32 dCh, UINT32 dVal)
{
	PMIDPACKET	pMsg;
	PMIDCHINFO	pCh;
	(void)		dVal;

	MASMFCNV_DBGMSG(("MaMidCnv : %8ld:SendPolyOn[%ld] \n", sdTime,dCh));

	if (sdTime >= 0)
	{
		pMsg = &gpMidInfo->MsgBuffer[gpMidInfo->dHoldMsgs++];
		pMsg->sdDeltaTime = sdTime;
		pMsg->dMsgID = MASNDDRV_CMD_POLY_MODE_ON;
		pMsg->dP1 = dCh;
		gpMidInfo->sdLastMsgTime += sdTime << (SMF_TIMEBASE_SHIFT + 10);
	} else {
		pCh = &gpMidInfo->DataInfo[1].ChInfo[dCh];
		pCh->dMode = 1;
	}
}


/****************************************************************************
 *	SendControl(SINT32 sdTime, UINT32 dCh, UINT32 dMsg2, UINT32 dMsg3)
 *
 *	Desc.
 *		Send Control message
 *	Param
 *		sdTime		Delta time
 *		dCh			#Channel(0..15)
 *		dMsg2		Parameter 1
 *		dMsg3		Parameter 2
 *	Return
 *      none
 ****************************************************************************/
static void	SendControl(SINT32 sdTime, UINT32 dCh, UINT32 dMsg2, UINT32 dMsg3)
{
	PMIDCHINFO	pCh;

	switch (dMsg2)
	{
	case 0x07:	/* Channel volume */
		SendChVol(sdTime, dCh, dMsg3);
		break;

	case 0x0B:	/* Expression */
		SendExpression(sdTime, dCh, dMsg3);
		break;

	case 0x0A:	/* Panpot */
		SendPanpot(sdTime, dCh, dMsg3);
		break;

	case 0x40:	/* Hold1 */
		SendHold1(sdTime, dCh, dMsg3);
		break;

	case 0x01:	/* Modulation */
		SendModDepth(sdTime, dCh, dMsg3);
		break;

	case 0x00:	/* Bank select(MSB) */
		pCh = &gpMidInfo->DataInfo[1].ChInfo[dCh];
		pCh->dBank = (pCh->dBank & 0x00FF) | (dMsg3 << 8);
		break;
    
    case 0x20:	/* Bank select (LSB) */
		pCh = &gpMidInfo->DataInfo[1].ChInfo[dCh];
		pCh->dBank = (pCh->dBank & 0xFF00) | dMsg3;
		break;

	case 0x06:	/* Data Entry MSB */
		pCh = &gpMidInfo->DataInfo[1].ChInfo[dCh];
		switch (pCh->dRPN)
		{
		case 0:	/* BendSens */
			SendBendRange(sdTime, dCh, dMsg3);
			break;
		case 1:	/* FineTune */
			pCh->dFineTune = (dMsg3 & 0x7f) << 7;
			SendFineTune(sdTime, dCh, pCh->dFineTune);
			break;
		case 2:	/* CoaseTune */
			pCh->dCoaseTune = dMsg3 & 0x7F;
			SendCoaseTune(sdTime, dCh, pCh->dCoaseTune);
		}
		break;

	case 0x26:	/* Data Entry LSB */
		pCh = &gpMidInfo->DataInfo[1].ChInfo[dCh];
		switch (pCh->dRPN)
		{
		case 1:	/* FineTune */
			pCh->dFineTune &= 0x3F80;
			pCh->dFineTune |= dMsg3 & 0x7f;
			SendFineTune(sdTime, dCh, pCh->dFineTune);
		}
		break;

	case 0x62:	/* NRPN (LSB) */
	case 0x63:	/* NRPN (MSB) */
		pCh = &gpMidInfo->DataInfo[1].ChInfo[dCh];
		pCh->dRPN |= 0x8000;
		break;

	case 0x64:	/* RPN (LSB) */
		pCh = &gpMidInfo->DataInfo[1].ChInfo[dCh];
		pCh->dRPN = (pCh->dRPN & 0x7F00) | dMsg3;
		break;

	case 0x65:	/* RPN (MSB) */
		pCh = &gpMidInfo->DataInfo[1].ChInfo[dCh];
		pCh->dRPN = (pCh->dRPN & 0x007F) | (dMsg3 << 8);
		break;

	case 0x78:
		SendAllSoundOff(sdTime, dCh);
		break;

	case 0x7B:
		SendAllNoteOff(sdTime, dCh);
		break;

	case 0x79:
		SendResetAllCtl(sdTime, dCh);
		break;

	case 0x7e:
		SendAllNoteOff(sdTime, dCh);
		SendMonoOn(0, dCh, dMsg3);
		break;

	case 0x7f:
		SendAllNoteOff(sdTime, dCh);
		SendPolyOn(0, dCh, dMsg3);
		break;
		
	default:
		MASMFCNV_DBGMSG(("MaMidCnv : %8ld:UnknownControl[%ld, %ld, %ld] \n", sdTime, dCh, dMsg2, dMsg3));
	}
}


/****************************************************************************
 *	SendPitchBend(SINT32 sdTime, UINT32 dCh, UINT32 dLl, UINT32 dHh)
 *
 *	Desc.
 *		Send PitchBend message
 *	Param
 *		sdTime		Delta time
 *		dCh			#Channel(0..15)
 *		dLl			Lower 7bit of PitchBend (0..127)
 *		bHh			Upper 7bit of PitchBend (0..127)
 *	Return
 *      none
 ****************************************************************************/
static void SendPitchBend(SINT32 sdTime, UINT32 dCh, UINT32 dLl, UINT32 dHh)
{
	PMIDPACKET	pMsg;
	PMIDCHINFO	pCh;

	MASMFCNV_DBGMSG(("MaMidCnv : %8ld:SendPitchBend[%d] = %02lX:%02lX\n", sdTime, dCh, dHh, dLl));

	if (sdTime >= 0)
	{
		pMsg = &gpMidInfo->MsgBuffer[gpMidInfo->dHoldMsgs++];
		pMsg->sdDeltaTime = sdTime;
		pMsg->dMsgID = MASNDDRV_CMD_PITCH_BEND;
		pMsg->dP1 = dCh;
		pMsg->dP2 = (dHh << 7) + dLl;
		gpMidInfo->sdLastMsgTime += sdTime << (SMF_TIMEBASE_SHIFT + 10);
	} else {
		pCh = &gpMidInfo->DataInfo[1].ChInfo[dCh];
		pCh->dBendRange = pCh->dPreBendRange;
		pCh->dPitchBend = dHh;
	}
}


/****************************************************************************
 *	SendGmOn(SINT32 sdTime)
 *
 *	Desc.
 *		Send GmOn message
 *	Param
 *		sdTime		Delta time
 *	Return
 *      none
 ****************************************************************************/
static void	SendGmOn(SINT32 sdTime)
{
	UINT32		dCh;
	PMIDPACKET	pMsg;
	PMIDCHINFO	pCh;

	MASMFCNV_DBGMSG(("MaMidCnv : %8ld:SendGmOn\n", sdTime));

	gpMidInfo->dMaxGain = SMF_MAX_GAIN;
	gpMidInfo->dMasterVolume = 127;
	gpMidInfo->dVibNote = 0;
	
	for (dCh = 0; dCh < 16; dCh++)
	{
		pCh = &gpMidInfo->DataInfo[1].ChInfo[dCh];
		pCh->dRPN = 0x7f7f;
		pCh->dBank = (dCh == 9) ? 0x7800 : 0x7900;
		pCh->dMipMute = 0;
	}
	
	if (sdTime >= 0)
	{
		SendAllSoundOff(sdTime, 0);
		for (dCh = 1; dCh < 16; dCh++) SendAllSoundOff(0, dCh);

		pMsg = &gpMidInfo->MsgBuffer[gpMidInfo->dHoldMsgs++];
		pMsg->sdDeltaTime = 0;
		pMsg->dMsgID = MASNDDRV_CMD_SYSTEM_ON;
		
		for (dCh = 0; dCh < 16; dCh++)
		{
			pCh = &gpMidInfo->DataInfo[1].ChInfo[dCh];
			SendLedSync(0, dCh, pCh->dLedSync);
			SendVibSync(0, dCh, pCh->dVibSync);
		}
		SendMasterVolume(0, gpMidInfo->dMaxGain);
	} else {
		for (dCh = 0; dCh < 16; dCh++)
		{
			pCh = &gpMidInfo->DataInfo[1].ChInfo[dCh];
			pCh->dCurrBank = pCh->dBank;
			pCh->dProg = 255;
			pCh->dVolume = 255;
			pCh->dExpression = 255;
			pCh->dModulation = 255;
			pCh->dPitchBend = 255;
			pCh->dPreBendRange = 255;
			pCh->dBendRange = 255;
			pCh->dPanpot = 255;
			pCh->dHold1 = 255;
			pCh->dMode = 255;
			pCh->dFineTune = 0x2000;
			pCh->dCoaseTune = 0x40;
		}
	}
}


/****************************************************************************
 *	SendShortMsg(SINT32 sdTime, UINT32 dCmd, UINT32 dCh, UINT32 dP1, UINT32 dP2)
 *
 *	Desc.
 *		Send ShortMsg message
 *	Param
 *		sdTime		Delta time
 *		dCmd		MIDI command
 *		dCh			#Channel(0..15)
 *		dP1			Parameter 1
 *		dP2			Parameter 2
 *	Return
 *		Length of the command[byte]
 ****************************************************************************/
static SINT32	SendShortMsg(SINT32 sdTime, UINT32 dCmd, UINT32 dCh, UINT32 dP1, UINT32 dP2)
{
	static void (*MidiMsg[16])(SINT32 sdTm, UINT32 dChannel, UINT32 dPara1, UINT32 dPara2) =
	{
		ErrorFunction,	ErrorFunction,	ErrorFunction,	ErrorFunction, 
		ErrorFunction, 	ErrorFunction,	ErrorFunction,	ErrorFunction, 
		SendNoteOff,	SendNoteOn,		NotSupported,	SendControl,
		SendProgram,	NotSupported,	SendPitchBend,	NotSupported
	};

	static const SINT32	sdMidiMsgLength[16] = 
	{
		0, 0, 0, 0, 0, 0, 0, 0, 2, 2, 2, 2, 1, 1, 2, 0
	};

	MidiMsg[dCmd](sdTime, dCh, dP1 & 0x7F, dP2 & 0x7F);

	return (sdMidiMsgLength[dCmd]);
}


/****************************************************************************
 *	SendSysEx(SINT32 sdTime, UINT8* pbMsg, UINT32 dSize)
 *
 *	Desc.
 *		Send SysEx message
 *	Param
 *		sdTime		Delta time
 *		pbMsg		Pointer to the message
 *		dSize		Size of the message [byte]
 *	Return
 *		0 : NoError, < 0 : Error code
 ****************************************************************************/
static SINT32	SendSysEx(SINT32 sdTime, UINT8* pbMsg, UINT32 dSize)
{
	PMIDCHINFO	pCh;
	UINT32		dCh;
	UINT32		dNumCh;
	UINT32		i;
	UINT32		dMip;
	UINT32		dMax;
	
	MASMFCNV_DBGMSG(("MaMidCnv : SendSysEx[%ld, %08lX, %ld]\n", sdTime, pbMsg, dSize));

	if (dSize < 4) return (MASMW_SUCCESS);
	
	if ((pbMsg[0] == 0x7e) &&
	    (pbMsg[1] == 0x7f) &&
	    (pbMsg[2] == 0x09) &&
	    ((pbMsg[3] == 0x01) || (pbMsg[3] == 0x03) || (pbMsg[3] == 0x02)) &&
	    (dSize == 4))
	{
		/* GM System On */
		SendGmOn(sdTime);
		return (MASMW_SUCCESS);
	}
	
	if ((pbMsg[0] == 0x7f) &&
	    (pbMsg[1] == 0x7f) &&
	    (pbMsg[2] == 0x04) &&
	    (pbMsg[3] == 0x01) &&
	    (dSize == 6))
	{
		/* Universal Master Volume */
		gpMidInfo->dMasterVolume = (UINT32)(pbMsg[5] & 0x7f);
		pCh = &gpMidInfo->DataInfo[1].ChInfo[0];
		SendExpression(sdTime, 0, pCh->dExpression);
		if (sdTime >= 0) sdTime = 0;
		for (dCh = 1; dCh < 16; dCh++)
		{
			pCh = &gpMidInfo->DataInfo[1].ChInfo[dCh];
			SendExpression(sdTime, dCh, pCh->dExpression);
		}

		return (MASMW_SUCCESS);
	}

	/*---- MIP(SP-MIDI) message ---------------------------------------------*/
	if ((pbMsg[0] == 0x7f) &&
	    (pbMsg[1] == 0x7f) &&
	    (pbMsg[2] == 0x0B) &&
	    (pbMsg[3] == 0x01))
	{
		dNumCh = (dSize - 4) >> 1;
		dMax = 16;
		if ((dNumCh > 0) && (dNumCh <= 16))
		{
			for (i = 0; i < 16; i++)
			{
				pCh = &gpMidInfo->DataInfo[1].ChInfo[i];
				pCh->dMipMute = 1;
			}
			for (i = 0; i < dNumCh; i++)
			{
				dCh = pbMsg[(i << 1) + 4];
				dMip = pbMsg[(i << 1) + 5];
				pCh = &gpMidInfo->DataInfo[1].ChInfo[dCh];
				if (dMip <= dMax) pCh->dMipMute = 0;
			}
		}
	}
	/*---- SP-MIDI message ---------------------------------------------*/

	if ((pbMsg[0] != 0x43) ||
	    (pbMsg[1] != 0x79) ||
	    (pbMsg[2] != 0x06) ||
	    (pbMsg[3] != 0x7f) ||
	    (dSize != 6)) return (MASMW_SUCCESS);

	switch (pbMsg[4])
	{
	case 0x00:		/* Max Gain */
		gpMidInfo->dMaxGain = (UINT32)(pbMsg[5] & 0x7F);
		SendMasterVolume(sdTime, gpMidInfo->dMaxGain);
		return (MASMW_SUCCESS);
	}

	return (MASMW_SUCCESS);
}


/****************************************************************************
 *	GetTimeCount(void)
 *
 *	Desc.
 *		Get Delta time
 *	Param
 *		none
 *	Return
 *		DeltaTime
 ****************************************************************************/
static SINT32 GetTimeCount(void)
{
	SINT32	sdTime;

	if (gpMidInfo->sdSmfCurrentTime > gpMidInfo->sdLastMsgTime)
	{
		sdTime = gpMidInfo->sdSmfCurrentTime - gpMidInfo->sdLastMsgTime;	/* [22.10][ms] */
		sdTime >>= SMF_TIMEBASE_SHIFT;										/*             */
		sdTime += 0x0200;													/* + 0.5       */
		sdTime >>= 10;
	} else {
		sdTime = 0;
	}
	
	return (sdTime);
}


/****************************************************************************
 *	SeekSmfMessage(SINT32 sdSeekTime)
 *
 *	Desc.
 *		Do seek
 *	Param
 *		sdSeekTime :	Time [22.10 ms]  
 *	Return
 *		none
 ****************************************************************************/
static void SeekSmfMessage(SINT32 sdSeekTime)
{
	UINT32		dCmd;
	UINT32		dCmd2;
	UINT32		dCh;

	UINT32		dParam1, dParam2;
	UINT32		dSize;

	UINT32		dTemp;
	UINT32		dTime;
	SINT32		sdTr;
	SINT32		sdCurrTime;
	SINT32		sdPreTime;

	PMIDINFO	pI;
	PTRACKINFO	pMt;

	/*--- Initialize --------------------------*/
	sdPreTime = 0;
	pI = &gpMidInfo->DataInfo[1];
	gpMidInfo->sdSmfCurrentTime = 0;
	gpMidInfo->sdSmfCurrentTicks = 0;
	gpMidInfo->sdSmfDelta = (SINT32)(500 << 10) / pI->dTimeResolution;	/* default=0.5sec */

	ResetTimeInfo(pI);

	SendGmOn(-1);
	
	/*--- Seek till StartPoint ------------------------*/
	if (pI->dSetupBar == 1)
	{
		while (pI->TrackInfo[0].dOffset < pI->dStart)
		{
			/*--- Get Delta time --------------------------*/
			if (pI->dNumOfTracks == 1)
			{
				sdTr = 0;
			}
			else
			{
				sdTr = GetLeastTimeTrack(pI);
				if (sdTr < 0) break;
			}
			pMt = &(pI->TrackInfo[sdTr]);
			dTime = (UINT32)(pMt->sdTicks - gpMidInfo->sdSmfCurrentTicks);
			gpMidInfo->sdSmfCurrentTicks = pMt->sdTicks;
			
			dCmd = (UINT32)pMt->pbBase[pMt->dOffset++];
			if (dCmd < 0xF0)
			{
				if (dCmd < 0x80)
				{
					dCmd = pMt->dSmfCmd;
					pMt->dOffset--;
				} else {
					pMt->dSmfCmd = dCmd;
				}
				
				dCh = dCmd & 0x0F;
				dCmd = (dCmd & 0xf0) >> 4;
				dParam1 = (UINT32)pMt->pbBase[pMt->dOffset];
				dParam2 = (UINT32)pMt->pbBase[pMt->dOffset + 1];
				pMt->dOffset += SendShortMsg(-1, dCmd, dCh, dParam1, dParam2);
			}
			else
			{
				switch (dCmd)
				{
				case 0xFF:			/* Meta */
					dCmd2 = (UINT32)pMt->pbBase[pMt->dOffset++];
					dSize = 0;
					do
					{
						dTemp = (UINT32)pMt->pbBase[pMt->dOffset++];
						dSize = (dSize << 7) + (dTemp & 0x7f);
					} while (dTemp >= 0x80);

					switch (dCmd2)
					{
					case 0x2f:		/* End */
						pI->dEndFlag &= ~(1L << sdTr);
						break;
						
					case 0x51:		/* Set Tempo */
						switch (dSize)
						{
						case 3:
						case 4:
							dTime = ((UINT32)pMt->pbBase[pMt->dOffset] << 16) + 
							        ((UINT32)pMt->pbBase[pMt->dOffset + 1] << 8) +
							         (UINT32)pMt->pbBase[pMt->dOffset + 2];
							dTime = (dTime << 7) / 125;
							gpMidInfo->sdSmfDelta = (SINT32)(dTime / pI->dTimeResolution);
							break;
						}
						break;
					}
					pMt->dOffset += dSize;
					break;

				case 0xF0:			/* SysEx */
					dSize = 0;
					do
					{
						dTemp = (UINT32)pMt->pbBase[pMt->dOffset++];
						dSize = (dSize << 7) + (dTemp & 0x7f);
					} while (dTemp >= 0x80);
					
					SendSysEx(-1, &pMt->pbBase[pMt->dOffset], dSize - 1);
					pMt->dOffset += dSize;
					break;
					
				case 0xF7:			/* SysEx */
					dSize = 0;
					do
					{
						dTemp = (UINT32)pMt->pbBase[pMt->dOffset++];
						dSize = (dSize << 7) + (dTemp & 0x7f);
					} while (dTemp >= 0x80);
					
					SendSysEx(-1, &pMt->pbBase[pMt->dOffset], dSize);
					pMt->dOffset += dSize;
					break;

				case 0xF1:			/* System Msg */
				case 0xF3:			/* System Msg */
					pMt->dOffset++;
					break;
				
				case 0xF2:			/* System Msg */
					pMt->dOffset += 2;
					break;
				}
			}
			UpdateTrackTime(pI, (UINT32)sdTr);
		}
	}
	
	/*--- Seek till SeekTime --------------------------*/
	while (gpMidInfo->sdSmfCurrentTime < sdSeekTime)
	{
		/*--- Get Delta time --------------------------*/
		if (pI->dNumOfTracks == 1)
		{
			sdTr = 0;
		}
		else
		{
			sdTr = GetLeastTimeTrack(pI);
			if (sdTr < 0) break;
		}
		pMt = &(pI->TrackInfo[sdTr]);
		dTime = (UINT32)(pMt->sdTicks - gpMidInfo->sdSmfCurrentTicks);
		sdPreTime = gpMidInfo->sdSmfCurrentTime;
		sdCurrTime = sdPreTime + dTime * gpMidInfo->sdSmfDelta;
		if (sdCurrTime >= sdSeekTime)
		{
			gpMidInfo->sdSmfCurrentTime = sdSeekTime;
			break;
		}

		gpMidInfo->sdSmfCurrentTime = sdCurrTime;
		gpMidInfo->sdSmfCurrentTicks = pMt->sdTicks;
		
		dCmd = (UINT32)pMt->pbBase[pMt->dOffset++];
		if (dCmd < 0xF0)
		{
			if (dCmd < 0x80)
			{
				dCmd = pMt->dSmfCmd;
				pMt->dOffset--;
			} else {
				pMt->dSmfCmd = dCmd;
			}
			
			dCh = dCmd & 0x0f;
			dCmd = (dCmd & 0xf0) >> 4;
			dParam1 = (UINT32)pMt->pbBase[pMt->dOffset];
			dParam2 = (UINT32)pMt->pbBase[pMt->dOffset + 1];
			pMt->dOffset += SendShortMsg(-1, dCmd, dCh, dParam1, dParam2);
		}
		else
		{
			switch (dCmd)
			{
			case 0xFF:			/* Meta */
				dCmd2 = pMt->pbBase[pMt->dOffset++];
				dSize = 0;
				do
				{
					dTemp = pMt->pbBase[pMt->dOffset++];
					dSize = (dSize << 7) + (UINT32)(dTemp & 0x7f);
				} while (dTemp >= 0x80);

				switch (dCmd2)
				{
				case 0x2f:		/* End */
					pI->dEndFlag &= ~(1L << sdTr);
					break;
					
				case 0x51:		/* Set Tempo */
					switch (dSize)
					{
					case 3:
					case 4:
						dTime = ((UINT32)pMt->pbBase[pMt->dOffset] << 16) + 
						        ((UINT32)pMt->pbBase[pMt->dOffset + 1] << 8) +
						         (UINT32)pMt->pbBase[pMt->dOffset + 2];
						dTime = (dTime << 7) / 125;
						gpMidInfo->sdSmfDelta = (SINT32)(dTime / pI->dTimeResolution);
						break;
					}
					break;
				}
				pMt->dOffset += dSize;
				break;

			case 0xF0:			/* SysEx */
				dSize = 0;
				do
				{
					dTemp = (UINT32)pMt->pbBase[pMt->dOffset++];
					dSize = (dSize << 7) + (dTemp & 0x7f);
				} while (dTemp >= 0x80);
				
				SendSysEx(-1, &pMt->pbBase[pMt->dOffset], dSize - 1);
				pMt->dOffset += dSize;
				break;
				
			case 0xF7:			/* SysEx */
				dSize = 0;
				do
				{
					dTemp = (UINT32)pMt->pbBase[pMt->dOffset++];
					dSize = (dSize << 7) + (dTemp & 0x7f);
				} while (dTemp >= 0x80);
				
				SendSysEx(-1, &pMt->pbBase[pMt->dOffset], dSize);
				pMt->dOffset += dSize;
				break;

			case 0xF1:			/* System Msg */
			case 0xF3:			/* System Msg */
				pMt->dOffset++;
				break;
			
			case 0xF2:			/* System Msg */
				pMt->dOffset += 2;
				break;
			}
		}
		UpdateTrackTime(pI, (UINT32)sdTr);
	}

	gpMidInfo->sdLastMsgTime = gpMidInfo->sdSmfCurrentTime;
	gpMidInfo->sdSmfCurrentTime = sdPreTime;
}


/****************************************************************************
 *	GetContentsInfo(SINT32 sdFileID, PMASMW_CONTENTSINFO pInfo)
 *
 *	Desc.
 *			Get contents info
 *	Param
 *		sdFileID		File ID
 *		pInfo			pointer to the command block
 *	Return
 *		0 : NoError, < 0 : Error code
 ****************************************************************************/
static SINT32 GetContentsInfo(SINT32 sdFileID, PMASMW_CONTENTSINFO pInfo)
{
	PMIDINFO	pI;
	UINT8*		pbData;
	UINT32		dSize;
	UINT32		dIndex;
	UINT8*		pbDst;
	UINT32		i;
	
	if ((sdFileID != 0) && (sdFileID != 1)) return (MASMW_ERROR);
	if (pInfo == NULL) return (MASMW_ERROR);
	if (pInfo->size == 0) return (MASMW_ERROR);
	if (gpMidInfo->DataInfo[sdFileID].dValid == 0) return (MASMW_ERROR);
	
	pI = &gpMidInfo->DataInfo[sdFileID];
	dIndex = ((UINT32)pInfo->tag[0] << 8) + (UINT32)pInfo->tag[1];
	pbDst = pInfo->buf;

	switch (dIndex)
	{
	case 0x5354:	/* ST: Title */
		pbData = pI->pbTitle;
		dSize = pI->dSizeTitle;
		break;

	case 0x4352:	/* CR: CopyRight */
		pbData = pI->pbCopyright;
		dSize = pI->dSizeCopyright;
		break;

	case 0x4C44:	/* LD: LED */
		*pbDst = 0;
		for (i = 0; i < 16; i++)
		{
			if (gpMidInfo->DataInfo[sdFileID].ChInfo[i].dLedSync > 0)
			{
				*pbDst = 1;
				break;
			}
		}
		return (1);

	case 0x5642:	/* VB: MTR */
		*pbDst = (UINT8)gpMidInfo->DataInfo[sdFileID].dVibNoteVoice;
		for (i = 0; i < 16; i++)
		{
			if (gpMidInfo->DataInfo[sdFileID].ChInfo[i].dVibSync > 0)
			{
				*pbDst = 1;
				break;
			}
		}
		return (1);

	case 0x4B43:	/* KC: KeyControl */
		*pbDst = 0;
		for (i = 0; i < 16; i++)
		{
			if (gpMidInfo->DataInfo[sdFileID].ChInfo[i].dKeyCon != 1)
			{
				*pbDst = 1;
				break;
			}
		}
		return (1);

	default:		/* Undefined */
		return (MASMW_ERROR);
	}

	if (pbData == NULL) return (MASMW_ERROR);
	if (dSize == 0) return (MASMW_ERROR);
	if (dSize > pInfo->size) dSize = pInfo->size;

	for (i = 0; i < dSize; i++) *pbDst++ = *pbData++;
	
	return (dSize);
}


/****************************************************************************
 *	ConvertOneSmfMessage(void)
 *
 *	Desc.
 *		SMF Message -> MA-5 packets  
 *	Param
 *		none 
 *	Return
 *		0 : No more data, 1 : Continue 
 ****************************************************************************/
static SINT32 ConvertOneSmfMessage()
{
	UINT32		dCmd;
	UINT32		dCmd2;
	UINT32		dCh;

	UINT32		dParam1, dParam2;
	UINT32		dSize;

	UINT32		dTemp;
	UINT32		dTime;
	
	SINT32		sdTr;
	PMIDINFO	pI;
	PTRACKINFO	pMt;

	/*--- Get Delta time --------------------------*/
	pI = &gpMidInfo->DataInfo[1];
	if (pI->dEndFlag == 0) return (0);
	if (pI->dNumOfTracks == 1)
	{
		sdTr = 0;
	}
	else
	{
		sdTr = GetLeastTimeTrack(pI);
		if (sdTr < 0) return (0);
	}
	pMt = &(pI->TrackInfo[sdTr]);
	dTime = pMt->sdTicks - gpMidInfo->sdSmfCurrentTicks;
	gpMidInfo->sdSmfCurrentTime += dTime * gpMidInfo->sdSmfDelta;
	gpMidInfo->sdSmfCurrentTicks = pMt->sdTicks;

	if (gpMidInfo->sdSmfCurrentTime >= gpMidInfo->sdSmfEndTime)
	{
		gpMidInfo->sdSmfCurrentTime = gpMidInfo->sdSmfEndTime;
		return (0);
	}

	dCmd = (UINT32)pMt->pbBase[pMt->dOffset++];
	if (dCmd < 0xF0)
	{
		if (dCmd < 0x80)
		{
			dCmd = pMt->dSmfCmd;
			pMt->dOffset--;
		} else {
			pMt->dSmfCmd = dCmd;
		}
		
		dCh = dCmd & 0x0f;
		dCmd = (dCmd & 0xf0) >> 4;
		dParam1 = (UINT32)pMt->pbBase[pMt->dOffset];
		dParam2 = (UINT32)pMt->pbBase[pMt->dOffset + 1];
		pMt->dOffset += SendShortMsg(GetTimeCount(), dCmd, dCh, dParam1, dParam2);
	}
	else
	{
		switch (dCmd)
		{
		case 0xFF:			/* Meta */
			dCmd2 = (UINT32)pMt->pbBase[pMt->dOffset++];

			dSize = 0;
			do
			{
				dTemp = (UINT32)pMt->pbBase[pMt->dOffset++];
				dSize = (dSize << 7) + (dTemp & 0x7f);
			} while (dTemp >= 0x80);

			switch (dCmd2)
			{
			case 0x2f:		/* End */
				pI->dEndFlag &= ~(1L << sdTr);
				break;
				
			case 0x51:		/* Set Tempo */
				switch (dSize)
				{
				case 3:
				case 4:
					dTime = ((UINT32)pMt->pbBase[pMt->dOffset] << 16) + 
					        ((UINT32)pMt->pbBase[pMt->dOffset + 1] << 8) +
					         (UINT32)pMt->pbBase[pMt->dOffset + 2];
					dTime = (dTime << 7) / 125;
					gpMidInfo->sdSmfDelta = (SINT32)dTime / pI->dTimeResolution;
					break;
				}
				break;
			}
			pMt->dOffset += dSize;
			break;

		case 0xF0:			/* SysEx */
			dSize = 0;
			do
			{
				dTemp = (UINT32)pMt->pbBase[pMt->dOffset++];
				dSize = (dSize << 7) + (dTemp & 0x7f);
			} while (dTemp >= 0x80);
			SendSysEx(GetTimeCount(), &pMt->pbBase[pMt->dOffset], dSize - 1);
			pMt->dOffset += dSize;
			break;
			
		case 0xF7:			/* SysEx */
			dSize = 0;
			do
			{
				dTemp = (UINT32)pMt->pbBase[pMt->dOffset++];
				dSize = (dSize << 7) + (dTemp & 0x7f);
			} while (dTemp >= 0x80);
			
			SendSysEx(GetTimeCount(), &pMt->pbBase[pMt->dOffset], dSize);
			pMt->dOffset += dSize;
			break;

		case 0xF1:			/* System Msg */
		case 0xF3:			/* System Msg */
			pMt->dOffset++;
			break;
		
		case 0xF2:			/* System Msg */
			pMt->dOffset += 2;
			break;
		}
	}
	UpdateTrackTime(pI, (UINT32)sdTr);
	
	return (1);
}


/*---------------------------------------------------------------------------*/
/*   Functions                                                               */
/*---------------------------------------------------------------------------*/

/****************************************************************************
 *	MaMidCnv_Initialize(void)
 *
 *	Desc.
 *		Initialize the converter
 *	Param
 *		none
 *	Return
 *		0 : NoError, < 0 : Error code
 ****************************************************************************/
SINT32	MaMidCnv_Initialize(void)
{
	gpMidInfo = &gMidInfo;
	
	gpMidInfo->sdSeqID = -1;					/* Sequence ID      */
	gpMidInfo->sdFileID = -1;					/* File ID          */
	gpMidInfo->dEnable = 0;						/* 0:disabel        */
	gpMidInfo->DataInfo[0].dValid = 0;
	gpMidInfo->DataInfo[1].dValid = 0;

	return (MASMW_SUCCESS);
}


/****************************************************************************
 *	MaMidCnv_End(void)
 *
 *	Desc.
 *		Finalize the converter
 *	Param
 *		none
 *	Return
 *		0 : NoError, < 0 : Error code
 ****************************************************************************/
SINT32	MaMidCnv_End(void)
{
	return (MASMW_SUCCESS);
}


/****************************************************************************
 *	MaMidCnv_Convert(void)
 *
 *	Desc.
 *			Do convert
 *	Param
 *		none
 *	Return
 *		0 : NoMoreData, 1 : Continue, 0xFFFF: SetupDone
 ****************************************************************************/
static SINT32	MaMidCnv_Convert(void)
{
	PMIDPACKET		pMsg;
	UINT32			i;
	SINT32			sdRet;

	if (gpMidInfo->dEnable == 0) return (0);

	if (gpMidInfo->dHoldMsgs > 0)
	{
		gpMidInfo->dHoldMsgs--;
		pMsg = &gpMidInfo->MsgBuffer[gpMidInfo->dHoldPtrR++];
		MaSndDrv_SetCommand(gpMidInfo->sdSeqID, pMsg->sdDeltaTime, pMsg->dMsgID, pMsg->dP1, pMsg->dP2, pMsg->dP3);
		if ((gpMidInfo->dHoldMsgs == 0) && (gpMidInfo->dMuteFlag == 1)) gpMidInfo->dMuteFlag = 2;
		return (1);
	};
	if (gpMidInfo->dSetup == 1) return (0xFFFF);
	if (gpMidInfo->dMuteFlag == 2)
	{
		gpMidInfo->dMuteFlag = 3;
		return (0);
	}
	else if (gpMidInfo->dMuteFlag == 3)  gpMidInfo->dMuteFlag = 0;
	
	gpMidInfo->dHoldPtrR = 0;
	sdRet = 0;
	while (gpMidInfo->dHoldMsgs == 0)
	{
		sdRet = ConvertOneSmfMessage();
		if (sdRet <= 0)
		{
			if (gpMidInfo->dMuteFlag == 0)
			{
				/*--- Ending... -----------------------*/
				MASMFCNV_DBGMSG((">> MaMidCnv_Convert/ Ending...\n"));
				SendAllSoundOff(GetTimeCount(), 0);
				for (i = 1; i < 16; i++) SendAllSoundOff(0, i);
				SendUserEvent(0, MASMW_END_OF_SEQUENCE);
				gpMidInfo->dMuteFlag = 1;
			}
			break;
		}
	}

	if (gpMidInfo->dHoldMsgs > 0)
	{
		gpMidInfo->dHoldMsgs--;
		pMsg = &gpMidInfo->MsgBuffer[gpMidInfo->dHoldPtrR++];
		MaSndDrv_SetCommand(gpMidInfo->sdSeqID, pMsg->sdDeltaTime, pMsg->dMsgID, pMsg->dP1, pMsg->dP2, pMsg->dP3);
		return (1);
	}
	
	return (0);
}


/****************************************************************************
 *	MaMidCnv_Load(UINT8* pbFile, UINT32 dFileSize, UINT32 dMode, SINT32 (*pfnFunc)(UINT8 bID), void* pvExtArgs)
 *
 *	Desc.
 *		Load SMF data
 *	Param
 *		pbFile			pointer to the data
 *		dFileSize		size fo the data
 *		dMode			error check (0:No, 1:Yes, 2:Check, 3:OnlyInfo)
 *		pfnFunc			pointer of rhe callback function
 *		pvExtArgs		Reserved
 *	Return
 *		>= 0 : FileID, < 0 : Error code
 ****************************************************************************/
SINT32	MaMidCnv_Load(UINT8* pbFile, UINT32 dFileSize, UINT32 dMode, SINT32 (*pfnFunc)(UINT8 bId), void* pvExtArgs)
{
	SINT32	sdRet;
	(void)pvExtArgs;
	(void)pfnFunc;

	if (pbFile == NULL) return (MASMW_ERROR);
	if (dFileSize == 0) return (MASMW_ERROR);

	sdRet = CheckSMF(pbFile, dFileSize, dMode);
	if (sdRet != MASMW_SUCCESS) return (sdRet);
	if (dMode > 1)
	{
		gpMidInfo->DataInfo[0].dValid = 1;
		return (0L);
	}

	gpMidInfo->dEnable = 0;
	gpMidInfo->sdFileID = 1;
	gpMidInfo->DataInfo[gpMidInfo->sdFileID].dValid = 1;

	return (1L);
}


/****************************************************************************
 *	MaMidCnv_Unload(SINT32 sdFileID, void* pvExtArgs)
 *
 *	Desc.
 *		Unload SMF data
 *	Param
 *		sdFileID		File ID
 *		pvExtArgs		Reserved
 *	Return
 *		0 : NoError, < 0 : Error code
 ****************************************************************************/
SINT32	MaMidCnv_Unload(SINT32 sdFileID, void* pvExtArgs)
{
	(void)pvExtArgs;

	if ((sdFileID < 0) || (sdFileID != gpMidInfo->sdFileID))
	{
		if (sdFileID == 0)
		{
			gpMidInfo->DataInfo[sdFileID].dValid = 0;
			return (MASMW_SUCCESS);
		}
		return (MASMW_ERROR);
	}

	gpMidInfo->DataInfo[sdFileID].dValid = 0;
	gpMidInfo->sdFileID = -1;
	gpMidInfo->dEnable = 0;

	return (MASMW_SUCCESS);
}


/****************************************************************************
 *	MaMidCnv_Open(SINT32 sdFileID, void* pvExtArgs)
 *
 *	Desc.
 *			Open SMF data
 *	Param
 *		sdFileID		File ID
 *		pvExtArgs		Reserved
 *	Return
 *		0 : NoError, < 0 : Error code
 ****************************************************************************/
SINT32	MaMidCnv_Open(SINT32 sdFileID, void* pvExtArgs)
{
	UINT32		dRam;
	UINT32		dSize;
	PMIDINFO	pI;
	(void)pvExtArgs;
	
	MASMFCNV_DBGMSG(("MaMidCnv_Open[%d] \n", sdFileID));

	if ((sdFileID < 0) || (sdFileID != gpMidInfo->sdFileID))
	{
		return (MASMW_ERROR);
	}

	gpMidInfo->sdSeqID = MaSndDrv_Create((UINT8)MASMW_SEQTYPE_DELAYED,
								SMF_TIMEBASE,				/* Timebase             */
								4L,							/* 16+16 Mode           */
								7L,							/* 4-OP,GML,MA5VOL      */
								0L,							/* No Streams           */
								255L,						/* No AL                */
								MaMidCnv_Convert,			/*                      */
								&dRam,						/*                      */
								&dSize);					/*                      */

	if (gpMidInfo->sdSeqID < 0)
	{
		MASMFCNV_DBGMSG(("MaMidCnv_Open/ Can't Open SndDrv\n"));
		return (gpMidInfo->sdSeqID);
	}
	MaSndDrv_SetVolShift(gpMidInfo->sdSeqID, 2, 2);			/* WT +12dB             */
	
	pI = &gpMidInfo->DataInfo[1];
	gpMidInfo->dRamBase = dRam;
	gpMidInfo->dRamSize = dSize;
	gpMidInfo->dRamOffset = 0;
	gpMidInfo->dEnable = 0;
	gpMidInfo->dSetup = 0;
	gpMidInfo->dMuteFlag = 0;
	gpMidInfo->sdFileID = 1;
	gpMidInfo->sdSmfDataEndTime = pI->sdDataEndTime + 1;
	gpMidInfo->sdSmfEndTime = gpMidInfo->sdSmfDataEndTime;
	gpMidInfo->dMaxGain = SMF_MAX_GAIN;						/* SMF_MAX_GAIN[dB]     */
	gpMidInfo->dMasterVolume = 127;							/*                      */
	gpMidInfo->dSyncNoteCh = 255;
	gpMidInfo->dSyncNoteKey = 255;

	return (MASMW_SUCCESS);
}


/****************************************************************************
 *	MaMidCnv_Close(SINT32 sdFileID, void* pvExtArgs)
 *
 *	Desc.
 *			Close the data
 *	Param
 *		sdFileID		File ID
 *		pvExtArgs		Reserved
 *	Return
 *		0 : NoError, < 0 : Error code
 ****************************************************************************/
SINT32	MaMidCnv_Close(SINT32 sdFileID, void* pvExtArgs)
{
	SINT32	sdRet;
	(void)pvExtArgs;
	
	MASMFCNV_DBGMSG(("MaMidCnv_Close[%d] \n", sdFileID));

	if ((sdFileID < 0) || (sdFileID != gpMidInfo->sdFileID))
	{
		return (MASMW_ERROR);
	}

	sdRet = MaSndDrv_Free(gpMidInfo->sdSeqID);

	gpMidInfo->sdSeqID = -1;
	gpMidInfo->dEnable = 0;

	return (sdRet);
}


/****************************************************************************
 *	MaMidCnv_Standby(SINT32 sdFileID, void* pvExtArgs)
 *
 *	Desc.
 *			Setup for playing SMF
 *	Param
 *		sdFileID		File ID
 *		pvExtArgs		Reserved
 *	Return
 *		0 : NoError, < 0 : Error code
 ****************************************************************************/
SINT32	MaMidCnv_Standby(SINT32 sdFileID, void* pvExtArgs)
{
	UINT32			dCh;
	PMIDCHINFO		pCh;
	(void)pvExtArgs;

	MASMFCNV_DBGMSG(("MaMidCnv_Standby[%d] \n", sdFileID));

	if ((sdFileID < 0) || (sdFileID != gpMidInfo->sdFileID))
	{
		return (MASMW_ERROR);
	}

	RegVoice();
	for (dCh = 0; dCh < 16; dCh++)
	{
		pCh = &gpMidInfo->DataInfo[1].ChInfo[dCh];
		MaSndDrv_SetKeyControl(gpMidInfo->sdSeqID, dCh, pCh->dKeyCon);
	}
	gpMidInfo->dEnable = 1;

	return (MASMW_SUCCESS);
}


/****************************************************************************
 *	MaMidCnv_Seek(SINT32 sdFileID, UINT32 dPos, void* pvExtArgs)
 *
 *	Desc.
 *			Setup for playing SMF
 *	Param
 *		sdFileID		File ID
 *		dPos			Start Point [ms]
 *		pvExtArgs		Reserved
 *	Return
 *		0 : NoError, < 0 : Error code
 ****************************************************************************/
SINT32	MaMidCnv_Seek(SINT32 sdFileID, UINT32 dPos, void* pvExtArgs)
{
	UINT32			dCh;
	UINT32			dVal;
	UINT32			dTemp;
	PMIDCHINFO		pCh;
	(void)pvExtArgs;

	MASMFCNV_DBGMSG(("MaMidCnv_Seek[%d] = %ld[ms] \n", sdFileID, dPos));

	if ((sdFileID < 0) || (sdFileID != gpMidInfo->sdFileID))
	{
		return (MASMW_ERROR);
	}
	
	gpMidInfo->sdSeekTime = (SINT32)dPos;
	gpMidInfo->dHoldMsgs = 0;
	gpMidInfo->dHoldPtrR = 0;

	SendGmOn(0);

	SeekSmfMessage((SINT32)dPos << 10);
	
	for (dCh = 0; dCh < 16; dCh++)
	{
		pCh = &gpMidInfo->DataInfo[1].ChInfo[dCh];

		dVal = pCh->dProg;
		if (dVal < 255)
		{
			dTemp = pCh->dBank;
			pCh->dBank = pCh->dCurrBank;
			SendProgram(0, dCh, dVal, 0);
			pCh->dBank = dTemp;
		}
		
		dVal = pCh->dVolume;
		if (dVal < 255) SendChVol(0, dCh, dVal);

		dVal = pCh->dExpression;
		if (dVal < 255) SendExpression(0, dCh, dVal);

		dVal = pCh->dModulation;
		if (dVal < 255) SendModDepth(0, dCh, dVal);

		dVal = pCh->dBendRange;
		if (dVal < 255) SendBendRange(0, dCh, dVal);

		dVal = pCh->dPitchBend;
		if (dVal < 255) SendPitchBend(0, dCh, 0, dVal);

		dVal = pCh->dPreBendRange;
		if (dVal != pCh->dBendRange) SendBendRange(0, dCh, dVal);

		dVal = pCh->dPanpot;
		if (dVal < 255) SendPanpot(0, dCh, dVal);

		dVal = pCh->dHold1;
		if (dVal < 255) SendHold1(0, dCh, dVal);

		if (pCh->dMode == 0) SendMonoOn(0, dCh, 0);

		if (pCh->dFineTune != 0x2000) SendFineTune(0, dCh, pCh->dFineTune);
		if (pCh->dCoaseTune != 0x40) SendCoaseTune(0, dCh, pCh->dCoaseTune);

	}
	SendMasterVolume(0, gpMidInfo->dMaxGain);

	gpMidInfo->dSetup = 1;
	gpMidInfo->dMuteFlag = 0;

	return (MaSndDrv_ControlSequencer(gpMidInfo->sdSeqID, 4));
}


/****************************************************************************
 *	MaMidCnv_Start(SINT32 sdFileID, void* pvExtArgs)
 *
 *	Desc.
 *			Start playing
 *	Param
 *		sdFileID		File ID
 *		pvExtArgs		Reserved
 *	Return
 *		0 : NoError, < 0 : Error code
 ****************************************************************************/
SINT32	MaMidCnv_Start(SINT32 sdFileID, void* pvExtArgs)
{
	(void)pvExtArgs;

	MASMFCNV_DBGMSG(("MaMidCnv_Start[%d] \n", sdFileID));

	if ((sdFileID < 0) || (sdFileID != gpMidInfo->sdFileID))
	{
		return (MASMW_ERROR);
	}
	gpMidInfo->dSetup = 0;

	return (MaSndDrv_ControlSequencer(gpMidInfo->sdSeqID, 1));
}


/****************************************************************************
 *	MaMidCnv_Stop(SINT32 sdFileID, void* pvExtArgs)
 *
 *	Desc.
 *			Stop playing
 *	Param
 *		sdFileID		File ID
 *		pvExtArgs		Reserved
 *	Return
 *		0 : NoError, < 0 : Error code
 ****************************************************************************/
SINT32	MaMidCnv_Stop(SINT32 sdFileID, void* pvExtArgs)
{
	(void)pvExtArgs;

	MASMFCNV_DBGMSG(("MaMidCnv_Stop[%d] \n", sdFileID));

	if ((sdFileID < 0) || (sdFileID != gpMidInfo->sdFileID))
	{
		return (MASMW_ERROR);
	}

	return (MaSndDrv_ControlSequencer(gpMidInfo->sdSeqID, 0));
}


/****************************************************************************
 *	MaMidCnv_Control(SINT32 sdFileID, UINT32 dCtrlNum, void* pvPrm, void* pvExtArgs)
 *
 *	Desc.
 *			Send Control message
 *	Param
 *		sdFileID		File ID
 *		dCtrlNum		Command ID
 *		pvPrm			Pointer to Parameters
 *		pvExtArgs		Reserved
 *	Return
 *		0 : NoError, < 0 : Error code
 ****************************************************************************/
SINT32	MaMidCnv_Control(SINT32 sdFileID, UINT32 dCtrlNum, void* pvPrm, void* pvExtArgs)
{
	SINT32	sdRet;
	(void)pvExtArgs;
	
	MASMFCNV_DBGMSG(("MaMidCnv_Control[%d, %d] \n", sdFileID, dCtrlNum));

	if ((sdFileID < 0) || (sdFileID > 1))
	{
		return (MASMW_ERROR);
	}

	switch(dCtrlNum)
	{
	case MASMW_SET_VOLUME :
		if (sdFileID != gpMidInfo->sdFileID)
		{
			return (MASMW_ERROR);
		}
		return (MaSndDrv_SetVolume(gpMidInfo->sdSeqID, (UINT32)*((UINT8*)pvPrm), 3));

	case MASMW_GET_POSITION :
		if (sdFileID != gpMidInfo->sdFileID)
		{
			return (0);
		}
		sdRet = MaSndDrv_GetPos(gpMidInfo->sdSeqID);
		return (sdRet);

	case MASMW_GET_LENGTH :
		return (gpMidInfo->DataInfo[sdFileID].sdDataEndTime >> 10);
	
	case MASMW_GET_CONTENTSDATA :
		return (GetContentsInfo(sdFileID, (MASMW_CONTENTSINFO*)pvPrm));

	case MASMW_SET_SPEED:
		if (sdFileID != gpMidInfo->sdFileID)
		{
			return (MASMW_ERROR);
		}
		if ((*((UINT8*)pvPrm) < 70) || (*((UINT8*)pvPrm) > 130))
		{
			return (MASMW_ERROR_ARGUMENT);
		}
		return (MaSndDrv_SetSpeed(gpMidInfo->sdSeqID, (UINT32)*((UINT8 *)pvPrm)));

	case MASMW_SET_KEYCONTROL:
		if (sdFileID != gpMidInfo->sdFileID)
		{
			return (MASMW_ERROR);
		}
		return MaSndDrv_SetKey(gpMidInfo->sdSeqID, (SINT32)*((SINT8 *)pvPrm));

	case MASMW_SET_EVENTNOTE :
		gpMidInfo->dSyncNoteCh = (UINT32)(*((UINT8*)pvPrm));
		gpMidInfo->dSyncNoteKey = (UINT32)(*((UINT8*)pvPrm + 1));
		break;

	default:
		/*	Undefined	*/
		return (MASMW_ERROR);
	}
	return (MASMW_SUCCESS);
}
