/****************************************************************************
 *
 *		Copyright (C) 2002-2003	YAMAHA CORPORATION. All rights reserved.
 *
 *		Module		: maphrcnv.c
 *
 *		Description	: MA-5 SMAF/Phrase and Audio Stream Converter Module.
 *
 *		Version		: 1.1.2 	2003.07.14
 *
 ****************************************************************************/
#include "mamachdep.h"
#include "madebug.h"
#include "masndseq.h"		/*	MA-X Sound Sequencer	*/
#include "masnddrv.h"		/*	MA-X Sound Driver		*/
#include "maresmgr.h"		/*	MA-X Resource Manager	*/

#include "malib.h"
#include "maphrcnv.h"

/****************************************************************************
 *	macro
 ****************************************************************************/
#define	MAPHR_MAKEDWORD(a, b, c, d)	(UINT32)(((UINT32)(a) << 24) | \
						((UINT32)(b) << 16) | ((UINT32)(c) << 8) | (UINT32)(d))

/****************************************************************************
 *	define
 ****************************************************************************/
/*	for common */
#define MAPHR_MASK_PHRASE			(0x01)
#define MAPHR_MASK_AUDIO			(0x02)
#define	MAPHR_SIZE_OF_CHUNKHEADER	(8)
#define	MAPHR_CTRL_CLEAR_ALL		(0x00000000)
#define	MAPHR_CTRL_SYSTEM_ON		(0x80000000)
#define	MAPHR_CTRL_MASTER_VOLUME	(0x40000000)
#define	MAPHR_CTRL_VOLUME			(0x00000001)
#define	MAPHR_CTRL_PANPOT			(0x00000002)
#define	MAPHR_WAIT_TIMEOUT			(1000)	/* ms */
#define	MAPHR_PHRASE_TIMEBASE		(20)
/* for Phrase */
#define	MAPHR_VERSION_LV1			(1)
#define	MAPHR_VERSION_LV2			(2)
#define	MAPHR_MAX_SLOT				(4)
#define	MAPHR_MAX_CHANNEL			(4)
#define	MAPHR_MAX_WAVES				(16)
#define	MAPHR_MAX_LV1_VOICES		(4)
#define	MAPHR_MAX_LV2_VOICES		MAPLIB_MAX_LV2_VOICES
#define	MAPHR_MA2_4OP_SIZE			(25)
#define	MAPHR_MA5_2OP_SIZE			(16)
#define	MAPHR_MA5_4OP_SIZE			(30)
#define	MAPHR_MA5_WT_SIZE			(13)
#define	MAPHR_END_OF_SEQUENCE		(-1)
#define	MAPHR_REPEAT				(-2)
/* for Audio */
#define	MAPHR_AUDIO_CNV_ID			(4)
#define	MAPHR_MAX_AUDIO_DATA		(2)

/****************************************************************************
 *	function
 ****************************************************************************/
extern void		machdep_memcpy(UINT8*, UINT8*, UINT32);
extern void		machdep_Sleep(UINT32);
static SINT32	MaPhrAud_Convert(void);

/****************************************************************************
 *	typedef
 ****************************************************************************/
typedef enum _MAPHRSTATUS
{
	MAPHR_STATUS_UNNOWN = 0,
	MAPHR_STATUS_NODATA,
	MAPHR_STATUS_READY,
	MAPHR_STATUS_PLAY,
	MAPHR_STATUS_ENDING,
	MAPHR_STATUS_PAUSE
} MAPHRSTATUS;

typedef struct _MAPHRWTBANK
{
	UINT8*			pbWave;			/*	Pointer to the wave data	*/
	UINT32			dSize;			/*	Size of wave data			*/
	UINT32			dRamAdr;
} MAPHRWTBANK, *PMAPHRWTBANK;

typedef struct _MAPHRAPIINFO
{
	MAPHRSTATUS		Status;			/*	Status	*/
	SINT32			sdMasterChannel;
	UINT32			dVolume;		/*	Phrase API Volume(0..127 def:100)	*/
	UINT32			dPan;			/*	Phrase API Panpot(0..127 def:64)	*/
	UINT32			dSlave;
	UINT32			dLength;		/*	play back time[ms]	*/
	UINT32			dLoopCount;		/*	Loop count(0:infinite 1..255)	*/
	UINT32			dRamAdr;		/*	Ram address	*/
	UINT32			dRamEndAdr;		/*	Ram	end address	*/
	MALIBPHRINFO	sDataInfo;		/*	SMAF/Phrase information	*/
} MAPHRAPIINFO, *PMAPHRAPIINFO;

typedef struct _MAPHRCHINFO
{
	UINT32			dKey;			/*	Key number(0..127)	*/
	UINT32			dVelocity;		/*	Velocity(0..127 def:127)	*/
	UINT32			dNewBank;		/*	*/
	UINT32			dBankNo;		/*	Bank Number(0,1,128,129)	*/
	UINT32			dProgNo;		/*	Program Number(0..127 def:0)	*/
	UINT32			dVolume;		/*	Volume(0..127 def:100)	*/
	UINT32			dPan;			/*	Panpot (0..127 def:64)	*/
	UINT32			dExp;			/*	Expression (0..127 def:100)	*/
	UINT32			dMod;			/*	Modulation (0..127 def:0)	*/
	UINT32			dNewRange;		/*	*/
	UINT32			dRange;			/*	Pitch Bend Range 100cent
													unit(0..24 def:2)	*/
	UINT32			dPitch;			/*	Pitch Bend (0..127 def:64)	*/
	UINT32			dExNoteFlag;	/*	*/
	UINT32			dRegL;			/*	*/
	UINT32			dRegH;			/*	*/
	SINT32			sdOctShift;		/*	Octave Shift (-4..0..4 def:0)	*/
	UINT32			dNoteOffTime;	/*	Note off time[tick](def:0xFFFFFFFF)	*/
} MAPHRCHINFO, *PMAPHRCHINFO;

typedef struct _MAPHRINFO
{
	/*	common	*/
	UINT32			dCreateStatus;	/* bit0:SMAF/Phrase bit1:SMAF/Audio	 */
	UINT32			dPlayStatus;
	SINT32			sdSeqId;		/*	Sequencer ID	*/
	UINT32			dCtrlStatus;
	UINT32			dRamAdr;
	UINT32			dRamSize;
	/*	for Phrase	*/
	UINT32			dSlave;
	UINT32			dPhrStatus;
	void			(*pfnEvHandler)(struct event*);
	MAPHRAPIINFO	sApiInfo[MAPHR_MAX_CHANNEL];
	MAPHRCHINFO		sChInfo[MASMW_NUM_CHANNEL];
	/*	for Audio	*/
	UINT32			dVolume;
	UINT32			dPanpot;
	UINT32			dPause;
	UINT32			dAudStatus;
	MALIBAUDINFO	sAudInfo[MAPHR_MAX_AUDIO_DATA];
} MAPHRINFO, *PMAPHRINFO;

/****************************************************************************
 *	gloval values
 ****************************************************************************/
static MAPHRINFO	gPhrInfo;

/*	table	*/
static const UINT32	gdMaskPlay[5] = {
	0x00000001, 0x00000002, 0x00000004, 0x00000008, 0x00000010
};
static const UINT32	gdMaskStart[4] = {
	0x00000001, 0x00000010, 0x00000100, 0x00001000
};
static const UINT32	gdMaskPause[4] = {
	0x00000004, 0x00000040, 0x00000400, 0x00004000
};
static const UINT32	gdMaskRestart[4] = {
	0x00000008, 0x00000080, 0x00000800, 0x00008000
};
static const UINT32	gdMaskStop[4] = {
	0x00000002, 0x00000020, 0x00000200, 0x00002000
};
static const UINT32	gdShortVolTable[16] = {
	 0, 0, 31, 39, 47, 55, 63, 71, 79, 87, 95, 103, 111, 119, 127, 127
};
static const UINT32	gdShortModTable[16] = {
/* +0 +1 +2 +3 +4 +5 +6 +7 +8 +9 +A +B +C +D +E +F	*/
	0, 0, 1, 1, 1, 2, 2, 2, 2, 3, 3, 3, 4, 4, 4, 4
};
static const UINT32	gdNormalModTable[128] = {
/* +0 +1 +2 +3 +4 +5 +6 +7 +8 +9 +A +B +C +D +E +F	*/
	0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
	1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
	2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
	2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
	3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3,
	3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3,
	4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4,
	4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4
};

/****************************************************************************
 *	ResourceCreate
 *
 *	Function:
 *			Create sequencer.
 *	Argument:
 *
 *	Return:
 *			0		success
 *			< 0		error
 *
 ****************************************************************************/
static SINT32	ResourceCreate(void)
{
	SINT32	sdSeqId;
	UINT32	dRamAdr;
	UINT32	dRamSize;

	if(gPhrInfo.sdSeqId >= 0L)	return (MASMW_ERROR);

	sdSeqId = MaSndDrv_Create(
				(UINT32)MASMW_SEQTYPE_DIRECT,
				(UINT32)MAPHR_PHRASE_TIMEBASE,
									/* 20msec */
				1L,					/* Resource mode */
				0x07L,				/* FM mode ... 4op	*/
									/* Drum sound setting ... GM Level1 */
									/* WT volume setting ... Standard mode */
				1L,					/* Number of Streams */
				0xFFL,				/* AL Channel */
				MaPhrAud_Convert,	/* Call back function */
				&dRamAdr,			/* Starting sddress of RAM */
				&dRamSize);			/* Size of RAM */

	if(sdSeqId < 0L) {
		gPhrInfo.sdSeqId = -1L;
		return (sdSeqId);
	}

	gPhrInfo.sdSeqId	= sdSeqId;
	gPhrInfo.dRamAdr	= dRamAdr;
	gPhrInfo.dRamSize	= dRamSize;

	/*	Set Volume	*/
	MaSndDrv_SetVolume(sdSeqId, 0x7FL, 0x07L);		/*	STREAM | WT | FM	*/
	MaSndDrv_SetVolShift(sdSeqId, 2L, 0x02L);		/*	WT +12dB	*/

	/*	Start Sequencer	*/
	gPhrInfo.dCtrlStatus =	MAPHR_CTRL_CLEAR_ALL;
	gPhrInfo.dCtrlStatus |= MAPHR_CTRL_SYSTEM_ON;
	gPhrInfo.dCtrlStatus |= MAPHR_CTRL_MASTER_VOLUME;
	MaSndDrv_ControlSequencer(sdSeqId, 1);

	return (MASMW_SUCCESS);
}

/****************************************************************************
 *	ResourceFree
 *
 *	Function:
 *			Free sequencer.
 *	Argument:
 *
 *	Return:
 *			0		success
 *			< 0		error
 *
 ****************************************************************************/
static SINT32	ResourceFree(void)
{
	SINT32	sdRet;

	if(gPhrInfo.sdSeqId < 0L)	return (MASMW_ERROR);

	/*	Stop Sequencer	*/
	MaSndDrv_ControlSequencer(gPhrInfo.sdSeqId, 0L);

	/*	Free Sequencer	*/
	sdRet = MaSndDrv_Free(gPhrInfo.sdSeqId);
	if(sdRet < 0L) return (sdRet);

	gPhrInfo.sdSeqId	= -1L;
	gPhrInfo.dRamAdr	= 0xFFFFFFFF;
	gPhrInfo.dRamSize	= 0L;

	return (MASMW_SUCCESS);
}

/****************************************************************************
 *	ConvertMA3Voice
 *
 *	Function:
 *			Convert MA-2 FM Voice Parameter to MA-3 format.
 *	Argument:
 *			pbM2V	Pointer to MA-2 FM voice parameter
 *			dM2Size	Size of MA-2 FM voice parameter
 *			pbM3V	Pointer to MA-3 FM voice parameter
 *	Return:
 *			converted size
 *
 ****************************************************************************/
static UINT32	ConvertMA3Voice(UINT8* pbM2V, UINT32 dM2Size, UINT8* pbM3V)
{
	UINT8	bFb;
	UINT8	bAlg;
	UINT8	bNumofOp;
	UINT8	bOp;
	UINT8	bVib;
	UINT8	bEgt;
	UINT8	bSus;
	UINT8	bKsr;
	UINT8	bRR;
	UINT8	bDR;
	UINT8	bDvb;
	UINT8	bDam;
	UINT8	bAm;
	UINT8	bWs;
	UINT32	dSize = 0L;

	bNumofOp = 4;
	bFb	 = (UINT8)((pbM2V[3] & 0x38) >> 3);
	bAlg = (UINT8)(pbM2V[3] & 0x07);
	if((bAlg & 0x06) == 0)				bNumofOp = 2;
	if(dM2Size < MAPHR_MA2_4OP_SIZE)	bNumofOp = 2;

	/*	panpot = 0x10 center	*/
	pbM3V[0] = (UINT8)(0x80 | (pbM2V[4] & 0x03));
	pbM3V[1] = (UINT8)((pbM2V[3] & 0xC0) | bAlg);

	pbM2V += 5;
	pbM3V += 2;
	dSize = 2;
	for(bOp = 0; bOp < bNumofOp; bOp++) {
		bVib = (UINT8)((pbM2V[0] & 0x08) >> 3);
		bEgt = (UINT8)((pbM2V[0] & 0x04) >> 2);
		bSus = (UINT8)((pbM2V[0] & 0x02) >> 1);
		bKsr = (UINT8)((pbM2V[0] & 0x01) >> 0);

		bRR	 = (UINT8)((pbM2V[1] & 0xF0) >> 4);
		bDR	 = (UINT8)(pbM2V[1] & 0x0F);

		bDvb = (UINT8)((pbM2V[4] & 0xC0) >> 6);
		bDam = (UINT8)((pbM2V[4] & 0x30) >> 4);
		bAm	 = (UINT8)((pbM2V[4] & 0x08) >> 3);
		bWs	 = (UINT8)(pbM2V[4] & 0x07);

		/*	SR | DR					*/
		pbM3V[0] = (UINT8)((((bEgt) ? 0x00 : bRR) << 4) | 0x04 | bKsr);
		/*	RR | DR					*/
		pbM3V[1] = (UINT8)((((bSus) ? 0x04 : bRR) << 4) | bDR);
		/*	AR | SL					*/
		pbM3V[2] = pbM2V[2];
		/*	TL | KSL				*/
		pbM3V[3] = pbM2V[3];
		/*	DAM | EAM | DVB | EVB	*/
		pbM3V[4] = (UINT8)((bDam << 5) | (bAm << 4) | (bDvb << 1) | bVib);
		/*	MULTI | DT=0			*/
		pbM3V[5] = (UINT8)(pbM2V[0] & 0xF0);
		/*	WS | FB					*/
		pbM3V[6] = (UINT8)((bWs << 3) | ((bOp == 0) ? bFb : 0x00));

		pbM2V += 5;
		pbM3V += 7;
		dSize += 7;
	}

	return(dSize);
}

/****************************************************************************
 *	GetVoiceInfo
 *
 *	Function:
 *			Get extend voice information.
 *	Argument:
 *			dBankNo	Bank Number(1 or 129)
 *			dProgNo	Program Number(0..127)
 *			pVi		Pointer to the top of voice information list.
 *	Return:
 *			! NULL	Pointer to the voice information.
 *			NULL	error
 *
 ****************************************************************************/
static PMALIBVOCINFO	GetVoiceInfo(UINT32 dBankNo, UINT32 dProgNo,
														 PMALIBVOCINFO pVi)
{
	UINT32	dIdx;

	if(pVi == NULL)									return (NULL);
	if((dBankNo != 0x01L) && (dBankNo != 0x81L))	return (NULL);

	for(dIdx = 0; dIdx < MAPHR_MAX_LV2_VOICES; dIdx++, pVi++) {
		if(	(pVi->dBankNo == dBankNo)	&&
			(pVi->dProgNo == dProgNo)) {
			return (pVi);
		}
		else if(pVi->dBankNo == 0xFFFFFFFFL) {
			return (NULL);
		}
	}

	return (NULL);
}

/****************************************************************************
 *	Velocity
 *
 *	Function:
 *			Update velocity(for LV2).
 *	Argument:
 *			dCh			#Channel(0..15)
 *			dVelocity	Velocity(0..127)
 *	Return:
 *			0		success
 *
 ****************************************************************************/
static SINT32	Velocity(UINT32 dCh, UINT32 dVelocity)
{
	if(dVelocity < 128L) {
		gPhrInfo.sChInfo[dCh].dVelocity = dVelocity;
	}
	return (0L);
}

/****************************************************************************
 *	CnvOctaveShift
 *
 *	Function:
 *			Update OctaveShift(for LV1).
 *	Argument:
 *			dCh		#Channel(0..15)
 *			dOct	OctaveShift(0..4, 129..132)
 *	Return:
 *			0		success
 *
 ****************************************************************************/
static SINT32 OctaveShift(UINT32 dCh, UINT32 dOct)
{
	MAPHRCNV_DBGMSG(("OctaveShift[%d %02X] \n", dCh, dOct));

	switch(dOct) {
	case 0x00L:
	case 0x01L:
	case 0x02L:
	case 0x03L:
	case 0x04L:
		gPhrInfo.sChInfo[dCh].sdOctShift = (SINT32)dOct;
		break;
	case 0x81L:
		gPhrInfo.sChInfo[dCh].sdOctShift = -1L;
		break;
	case 0x82L:
		gPhrInfo.sChInfo[dCh].sdOctShift = -2L;
		break;
	case 0x83L:
		gPhrInfo.sChInfo[dCh].sdOctShift = -3L;
		break;
	case 0x84L:
		gPhrInfo.sChInfo[dCh].sdOctShift = -4L;
		break;
	default:   /* reserved */
		gPhrInfo.sChInfo[dCh].sdOctShift = 0L;
		break;
	}

	return (0L);
}

/****************************************************************************
 *	CnvOctaveShift2
 *
 *	Function:
 *			Update OctaveShift(for LV2).
 *	Argument:
 *			dCh		#Channel(0..15)
 *			dOct	OctaveShift(0..4, 129..132)
 *	Return:
 *			0		success
 *
 ****************************************************************************/
static SINT32 OctaveShift2(UINT32 dCh, UINT32 dOct)
{
	MAPHRCNV_DBGMSG(("OctaveShift2[%d %02X] \n", dCh, dOct));

	switch(dOct) {
	case 0x00L:
	case 0x01L:
	case 0x02L:
	case 0x03L:
	case 0x04L:
		gPhrInfo.sChInfo[dCh].sdOctShift = (SINT32)dOct;
		break;
	case 0x81L:
		gPhrInfo.sChInfo[dCh].sdOctShift = -1L;
		break;
	case 0x82L:
		gPhrInfo.sChInfo[dCh].sdOctShift = -2L;
		break;
	case 0x83L:
		gPhrInfo.sChInfo[dCh].sdOctShift = -3L;
		break;
	case 0x84L:
		gPhrInfo.sChInfo[dCh].sdOctShift = -4L;
		break;
	default:   /* reserved */
		break;
	}

	return (0L);
}

/****************************************************************************
 *	BankSelect
 *
 *	Function:
 *			Update BankSelect(for LV2).
 *	Argument:
 *			dCh		#Channel(0..15)
 *			dBankNo	Bank Number(0,1,128,129)
 *	Return:
 *			0		success
 *
 ****************************************************************************/
static SINT32	BankSelect(UINT32 dCh, UINT32 dBankNo)

{
	if(	(dBankNo == 0x00L)	||
		(dBankNo == 0x01L)	||
		(dBankNo == 0x80L)	||
		(dBankNo == 0x81L)) {

		gPhrInfo.sChInfo[dCh].dNewBank = dBankNo;
	}

	return (0L);
}

/****************************************************************************
 *	CnvProgramChange
 *
 *	Function:
 *			Put Program Change to sound driver(for LV1).
 *	Argument:
 *			dCh		#Channel(0..15)
 *			dProgNo	Program Number(0..127)
 *	Return:
 *			Number of put command.
 *
 ****************************************************************************/
static SINT32	CnvProgramChange(UINT32 dCh, UINT32 dProgNo)
{
	UINT32			dPhrCh;
	UINT32			dBankNo;
	PMALIBVOCINFO	pVi;
	PMAPHRCHINFO	pCi = &(gPhrInfo.sChInfo[dCh]);

	if(dProgNo > 3L)	return (0L);

	pCi->dProgNo	= dProgNo;
	dPhrCh			= (UINT32)(dCh >> 2);
	pVi = &(gPhrInfo.sApiInfo[dPhrCh].sDataInfo.sVocInfo[dProgNo]);

	switch(pVi->dBankNo) {
	case 0x00L :	/*	default melody voice */
		dBankNo = 0x00L;
		dProgNo = pVi->dProgNo;
		break;
	case 0x01L :	/*	extend melody voice	*/
		dBankNo = (UINT32)(1L + dPhrCh);
		break;
	default :	/*	no voice	*/
		dBankNo = 0x00L;
		dProgNo = 0x00L;
		break;
	}

	MaSndDrv_SetCommand(gPhrInfo.sdSeqId, -1L, MASNDDRV_CMD_PROGRAM_CHANGE,
						 dCh, dBankNo, dProgNo);
	return (1L);
}

/****************************************************************************
 *	CnvProgramChange2
 *
 *	Function:
 *			Put Program Change to sound driver(for LV2).
 *	Argument:
 *			dCh		#Channel(0..15)
 *			dProgNo	Program Number(0..127)
 *	Return:
 *			Number of put command.
 *
 ****************************************************************************/
static SINT32	CnvProgramChange2(UINT32 dCh, UINT32 dProgNo)
{
	UINT32			dPhrCh;
	UINT32			dBankNo;
	PMALIBVOCINFO	pVi;
	PMAPHRCHINFO	pCi = &(gPhrInfo.sChInfo[dCh]);

	if(dProgNo > 127L)	return (0L);

	pCi->dProgNo	= dProgNo;
	dPhrCh			= (UINT32)(dCh >> 2);
	switch(pCi->dBankNo) {
	case 0x00L :	/*	default melody voice	*/
		dBankNo	= 0x00L;
		break;
	case 0x01L :	/*	extend melody voice	*/
		pVi = &(gPhrInfo.sApiInfo[dPhrCh].sDataInfo.sVocInfo[0]);
		pVi = GetVoiceInfo(0x01L, dProgNo, pVi);
		if(pVi == NULL) {	/*	not found extend voice	*/
			dBankNo	= 0x00L;
			dProgNo	= 0x00L;
		}
		else {				/*	found extend voice	*/
			dBankNo	= (UINT32)(0x01L + dPhrCh);
		}
		break;
	case 0x80L :	/*	default drum voice	*/
		dBankNo	= 0x80L;
		dProgNo	= 0x00L;
		break;
	case 0x81L :	/*	extend drum voice	*/
		dBankNo	= (UINT32)(0x81L + dPhrCh);
		dProgNo	= 0x00L;
		break;
	default :
		return (0L);
	}

	MaSndDrv_SetCommand(gPhrInfo.sdSeqId, -1L, MASNDDRV_CMD_PROGRAM_CHANGE,
								 dCh, dBankNo, dProgNo);
	return (1L);
}

/****************************************************************************
 *	CnvChannelVolume
 *
 *	Function:
 *			Put Channel Volume to sound driver.
 *	Argument:
 *			dCh		#Channel(0..15)
 *			dVol	Volume(0..127)
 *	Return:
 *			Number of put command.
 *
 ****************************************************************************/
static SINT32	CnvChannelVolume(UINT32 dCh, UINT32 dVol)
{
	PMAPHRCHINFO	pCi = &(gPhrInfo.sChInfo[dCh]);

	if(dVol > 127L)	return (0L);

	if(pCi->dVolume != dVol) {
		MaSndDrv_SetCommand(gPhrInfo.sdSeqId, -1L,
							MASNDDRV_CMD_CHANNEL_VOLUME, dCh, dVol, 0L);
		pCi->dVolume = dVol;	/*	update	*/
		return (1L);
	}
	return (0L);
}

/****************************************************************************
 *	CnvModulation
 *
 *	Function:
 *			Put Modulation Depth to sound driver.
 *	Argument:
 *			dCh		#Channel(0..15)
 *			dMod	Modulation(0..4)
 *	Return:
 *			Number of put command.
 *
 ****************************************************************************/
static SINT32	CnvModulation(UINT32 dCh, UINT32 dMod)
{
	PMAPHRCHINFO	pCi = &(gPhrInfo.sChInfo[dCh]);

	if(dMod > 4L)	return (0L);

	if(pCi->dMod != dMod) {
		MaSndDrv_SetCommand(gPhrInfo.sdSeqId, -1L,
							MASNDDRV_CMD_MODULATION_DEPTH, dCh, dMod, 0L);
		pCi->dMod = dMod;	/*	update	*/
		return (1L);
	}
	return (0L);
}

/****************************************************************************
 *	CnvExpression
 *
 *	Function:
 *			Put Expression to sound driver.
 *	Argument:
 *			dCh		#Channel(0..15)
 *			dExp	Expression(0..127)
 *	Return:
 *			Number of put command.
 *
 ****************************************************************************/
static SINT32	CnvExpression(UINT32 dCh, UINT32 dExp)
{
	PMAPHRCHINFO	pCi = &(gPhrInfo.sChInfo[dCh]);

	if(dExp > 127L)	return (0L);

	if(pCi->dExp != dExp) {
		MaSndDrv_SetCommand(gPhrInfo.sdSeqId, -1L,
							MASNDDRV_CMD_EXPRESSION, dCh, dExp, 0L);
		pCi->dExp = dExp;	/*	update	*/
		return (1L);
	}
	return (0L);
}

/****************************************************************************
 *	CnvPanpot
 *
 *	Function:
 *			Put Panpot to sound driver.
 *	Argument:
 *			dCh		#Channel(0..15)
 *			dPan	Panpot(0..127)
 *	Return:
 *			Number of put command.
 *
 ****************************************************************************/
static SINT32	CnvPanpot(UINT32 dCh, UINT32 dPan)
{
	PMAPHRCHINFO	pCi = &(gPhrInfo.sChInfo[dCh]);

	if(dPan > 127L)	return (0L);

	if(pCi->dPan != dPan) {
		MaSndDrv_SetCommand(gPhrInfo.sdSeqId, -1L,
							MASNDDRV_CMD_PANPOT, dCh, dPan, 0L);
		pCi->dPan = dPan;	/*	update	*/
		return (1L);
	}
	return (0L);
}

/****************************************************************************
 *	CnvPitchBendRange
 *
 *	Function:
 *			Update PitchBend Range.
 *	Argument:
 *			dCh		#Channel(0..15)
 *			dRange	PitchBend Range(0..24)
 *	Return:
 *			Number of put command.
 *
 ****************************************************************************/
static SINT32	PitchBendRange(UINT32 dCh, UINT32 dRange)
{
	if(dRange > 24L)	return (0L);

	gPhrInfo.sChInfo[dCh].dNewRange = dRange;

	return (0L);
}

/****************************************************************************
 *	CnvPitchBend
 *
 *	Function:
 *			Put PitchBend to sound driver.
 *	Argument:
 *			dCh		#Channel(0..15)
 *			dPitch	PitchBend(0..127)
 *	Return:
 *			Number of put command.
 *
 ****************************************************************************/
static SINT32	CnvPitchBend(UINT32 dCh, UINT32 dPitch)
{
	SINT32			sdRet = 1L;
	PMAPHRCHINFO	pCi = &(gPhrInfo.sChInfo[dCh]);

	if(dPitch > 127L)	return (0L);

	if(pCi->dRange != pCi->dNewRange) {
		pCi->dRange = pCi->dNewRange;	/*	update	*/
		MaSndDrv_SetCommand(gPhrInfo.sdSeqId, -1L,
							MASNDDRV_CMD_BEND_RANGE, dCh, pCi->dRange, 0L);
		sdRet++;
	}

	MaSndDrv_SetCommand(gPhrInfo.sdSeqId, -1L,
						MASNDDRV_CMD_PITCH_BEND, dCh, (UINT32)(dPitch << 7), 0L);
	pCi->dPitch = dPitch;	/*	update	*/
	return (sdRet);
}

/****************************************************************************
 *	CnvNoteOff
 *
 *	Function:
 *			Put NoteOff to sound driver.
 *	Argument:
 *			dCh		#Channel(0..15)
 *	Return:
 *			Number of put command.
 *
 ****************************************************************************/
static SINT32	CnvNoteOff(UINT32 dCh)
{
	PMAPHRCHINFO	pCi = &(gPhrInfo.sChInfo[dCh]);

	/*
	MAPHRCNV_DBGMSG(("CnvNoteOff[%d] \n", bCh));
	*/

	if(pCi->dNoteOffTime != 0xFFFFFFFF) {
		MaSndDrv_SetCommand(gPhrInfo.sdSeqId, -1L,
							MASNDDRV_CMD_NOTE_OFF_MA2, dCh, pCi->dKey, 0L);
		pCi->dNoteOffTime = 0xFFFFFFFF;
		return (1L);
	}

	return (0L);
}

/****************************************************************************
 *	CnvNoteOn
 *
 *	Function:
 *			Put NoteOn to sound driver(for LV1).
 *	Argument:
 *			dCh				#Channel(0..15)
 *			dOct			OctaveShift(0..3)
 *			dKey			Key Number(1..12)
 *			dNoteOffTime	NoteOff time[tick]
 *	Return:
 *			Number of put command.
 *
 ****************************************************************************/
static SINT32	CnvNoteOn(UINT32 dCh, UINT32 dOct, UINT32 dKey,
														 UINT32 dNoteOffTime)
{
	SINT32			sdKey;
	SINT32			sdOct;
	PMAPHRCHINFO	pCi = &(gPhrInfo.sChInfo[dCh]);

	sdOct	= (SINT32)(dOct + pCi->sdOctShift + 3L);
	sdKey	= (SINT32)((sdOct * 12L) + dKey);

	/*	Process for when nNote is out of the range (Note[0..127])	*/
	if(sdKey < 0L)			sdKey = 0L;
	else if(sdKey > 127L)	sdKey = 127L;

	/*	Velocity 127 fixed */
	/*
	MAPHRCNV_DBGMSG(("CnvNoteOn[%d %d %02X %08lX] -> [%02X]\n", dCh,
										dOct, dKey, dNoteOffTime, dNote));
	*/

	MaSndDrv_SetCommand(gPhrInfo.sdSeqId, -1L,
						MASNDDRV_CMD_NOTE_ON_MA2, dCh, (UINT32)sdKey, 0x7FL);
	pCi->dKey			= (UINT32)sdKey;
	pCi->dNoteOffTime	= dNoteOffTime;
	return (1L);
}

/****************************************************************************
 *	CnvNoteOn
 *
 *	Function:
 *			Put NoteOn to sound driver(for LV2).
 *	Argument:
 *			dCh				#Channel(0..15)
 *			dOct			OctaveShift(0..3)
 *			dKey			Key Number(1..12)
 *			dNoteOffTime	NoteOff time[tick]
 *	Return:
 *			Number of put command.
 *
 ****************************************************************************/
static SINT32	CnvNoteOn2(UINT32 dCh, UINT32 dOct, UINT32 dKey,
														 UINT32 dNoteOffTime)
{
	SINT32			sdKey;
	SINT32			sdOct;
	PMALIBVOCINFO	pVi;
	PMAPHRCHINFO	pCi = &(gPhrInfo.sChInfo[dCh]);

	if((dKey < 1L) || (12L < dKey)) {
		pCi->dNoteOffTime = 0xFFFFFFFF;
		return (0L);
	}

	switch(pCi->dBankNo) {
	case 0x00L :
	case 0x01L :	/*	default or extend melody voice	*/
		sdOct = (SINT32)(dOct + pCi->sdOctShift + 3L);
		sdKey = (SINT32)((sdOct * 12L) + dKey);
		if(sdKey < 0L)			sdKey = 0L;
		else if(127L < sdKey)	sdKey = 127L;
		break;
	case 0x80L :	/*	default drum voice	*/
		sdKey = (SINT32)(dOct * 12L + dKey + 34L);	/*	1..48 -> 35..82	*/
		break;
	case 0x81L :	/*	extend drum voice	*/
		sdKey	= (SINT32)(dOct * 12L + dKey);		/*	1..48	*/
		pVi		= &(gPhrInfo.sApiInfo[(dCh >> 2)].sDataInfo.sVocInfo[0]);
		pVi		= GetVoiceInfo(0x81L, (UINT32)sdKey, pVi);
		if(pVi == NULL) {	/*	not found extend voice	*/
			pCi->dNoteOffTime = 0xFFFFFFFF;
			return (0L);
		}
		break;
	default :
		pCi->dNoteOffTime = 0xFFFFFFFF;
		return (0L);
	}

	MaSndDrv_SetCommand(gPhrInfo.sdSeqId, -1L, MASNDDRV_CMD_NOTE_ON_MA2,
				dCh, (UINT32)sdKey, pCi->dVelocity);
	pCi->dKey			= (UINT32)sdKey;
	pCi->dNoteOffTime	= dNoteOffTime;
	return (1L);
}

/****************************************************************************
 *	CnvExclusiveMessage
 *
 *	Function:
 *			Interpret the exclusive mesage and
 *			Put command to sound driver(for LV1).
 *	Argument:
 *			dPhrCh	Sequence Number(0..3)
 *			pbExMsg	Pointer to exclusive message.
 *			dSize	Size of exlusive message.
 *	Return:
 *			Number of put command.
 *
 ****************************************************************************/
static SINT32	CnvExclusiveMessage(UINT32 dPhrCh, UINT8* pbExMsg, UINT32 dSize)
{
	UINT32			dCh;
	UINT32			dBlkFnum;
	SINT32			sdRet;
	PMAPHRCHINFO	pCi;

	MAPHRCNV_DBGMSG(("CnvExclusiveMessage[%d %08lX %08lX] \n",
												 dPhrCh, pbExMsg, dSize));

	if(dSize != 6L) return (0L);

	sdRet = 0L;
	if(	(pbExMsg[0] == 0x43)	&&
		(pbExMsg[1] == 0x03)	&&
		(pbExMsg[2] == 0x90)	&&
		(pbExMsg[5] == 0xF7)) {			/*	Extend NoteOn/Off	*/

		dCh	= (UINT32)((pbExMsg[3] & 0x03) + (dPhrCh << 2));
		pCi	= &(gPhrInfo.sChInfo[dCh]);

		if((pbExMsg[3] & 0xF0) == 0xC0) {
			pCi->dExNoteFlag |= 0x02L;
			pCi->dRegH = (UINT32)pbExMsg[4];
		}
		else if((pbExMsg[3] & 0xF0) == 0xB0) {
			pCi->dExNoteFlag |= 0x01L;
			pCi->dRegL = (UINT32)pbExMsg[4];
		}
		else {
			return (0L);
		}

		if((pCi->dExNoteFlag & 0x03L) == 0x03L) {
			dBlkFnum	= (UINT32)(((pCi->dRegH & ~0x20L) << 8)
									 | (pCi->dRegL | 0x80000000));
			if((pCi->dRegH & 0x20L) == 0x20L) {
				MaSndDrv_SetCommand(gPhrInfo.sdSeqId, -1L,
						MASNDDRV_CMD_NOTE_ON_MA2EX, dCh, dBlkFnum, 0x7FL);
			}
			else {
				MaSndDrv_SetCommand(gPhrInfo.sdSeqId, -1L,
						MASNDDRV_CMD_NOTE_OFF_MA2EX, dCh, dBlkFnum, 0L);
			}
			sdRet++;
		}
	}
	else if((pbExMsg[0] == 0x43)	&&
			(pbExMsg[1] == 0x04)	&&
			(pbExMsg[2] == 0x02)	&&
			(pbExMsg[5] == 0xF7)) {		/*	Pitch Bend Range	*/

		dCh	= (UINT32)((pbExMsg[3] & 0x03) + (dPhrCh << 2));
		PitchBendRange(dCh, pbExMsg[4]);
	}

	return (sdRet);
}

/****************************************************************************
 *	CnvAllSoundOff
 *
 *	Function:
 *			Put AllSoundOff to sound driver.
 *	Argument:
 *			dCh		#Channel(0..15)
 *	Return:
 *			Number of put command.
 *
 ****************************************************************************/
static SINT32	CnvAllSoundOff(UINT32 dCh)
{
	MaSndDrv_SetCommand(gPhrInfo.sdSeqId, -1L,
						MASNDDRV_CMD_ALL_SOUND_OFF, dCh, 0L, 0L);
	gPhrInfo.sChInfo[dCh].dNoteOffTime = 0xFFFFFFFF;
	return (1L);
}

/****************************************************************************
 *	CnvAllNoteOff
 *
 *	Function:
 *			Put AllNoteOff to sound driver.
 *	Argument:
 *			dCh		#Channel(0..15)
 *	Return:
 *			Number of put command.
 *
 ****************************************************************************/
static SINT32	CnvAllNoteOff(UINT32 dCh)
{
	MaSndDrv_SetCommand(gPhrInfo.sdSeqId, -1L,
						MASNDDRV_CMD_ALL_NOTE_OFF, dCh, 0L, 0L);
	gPhrInfo.sChInfo[dCh].dNoteOffTime = 0xFFFFFFFF;
	return (1L);
}

/****************************************************************************
 *	PutInitPhraseMessage
 *
 *	Function:
 *			Put initialize command to sound driver.
 *	Argument:
 *			bPhrCh	Sequence Number(0..3)
 *	Return:
 *			Number of put command.
 *
 ****************************************************************************/
static SINT32	PutInitPhraseMessage(UINT32 dPhrCh)
{
	UINT32			i;
	UINT32			dCh;
	SINT32			sdRet;
	PMAPHRCHINFO	pCi;

	MAPHRCNV_DBGMSG(("PutInitPhraseMessage[%ld] \n", dPhrCh));

	sdRet	= 0L;
	for(i = 0; i < MAPHR_MAX_SLOT; i++) {
		dCh = (UINT32)(i + (dPhrCh << 2));
		pCi = &(gPhrInfo.sChInfo[dCh]);

		/*	Direct ON	*/
		MaSndDrv_SetCommand(gPhrInfo.sdSeqId, -1L,
				MASNDDRV_CMD_VOL_DIRECT, dCh, 0L, 0L);
		/*	All Sound Off	*/
		MaSndDrv_SetCommand(gPhrInfo.sdSeqId, -1L,
				MASNDDRV_CMD_ALL_SOUND_OFF, dCh, 0L, 0L);
		/*	Program Change	*/
		if(gPhrInfo.sApiInfo[dPhrCh].sDataInfo.dVersion == (UINT32)MAPHR_VERSION_LV2)
			sdRet += CnvProgramChange2(dCh, pCi->dProgNo);
		else
			sdRet += CnvProgramChange(dCh, pCi->dProgNo);
		/*	Channel Volume	*/
		MaSndDrv_SetCommand(gPhrInfo.sdSeqId, -1L,
				MASNDDRV_CMD_CHANNEL_VOLUME, dCh, pCi->dVolume, 0L);
		/*	Expression		*/
		MaSndDrv_SetCommand(gPhrInfo.sdSeqId, -1L,
				MASNDDRV_CMD_EXPRESSION, dCh, pCi->dExp, 0L);
		/*	Panpot			*/
		MaSndDrv_SetCommand(gPhrInfo.sdSeqId, -1L,
				MASNDDRV_CMD_PANPOT, dCh, pCi->dPan, 0L);
		/*	Modulation		*/
		MaSndDrv_SetCommand(gPhrInfo.sdSeqId, -1L,
				MASNDDRV_CMD_MODULATION_DEPTH, dCh, pCi->dMod, 0L);
		/*	BendRange		*/
		MaSndDrv_SetCommand(gPhrInfo.sdSeqId, -1L,
				MASNDDRV_CMD_BEND_RANGE, dCh, pCi->dRange, 0L);
		/*	PitchBend		*/
		MaSndDrv_SetCommand(gPhrInfo.sdSeqId, -1L,
				MASNDDRV_CMD_PITCH_BEND, dCh, (UINT32)(pCi->dPitch << 7), 0L);
		/*	Direct OFF	*/
		MaSndDrv_SetCommand(gPhrInfo.sdSeqId, -1L,
				MASNDDRV_CMD_VOL_DIRECT, dCh, 1L, 0L);
		sdRet += 9L;
	}

	return (sdRet);
}

/****************************************************************************
 *	InitPhraseInfo
 *
 *	Function:
 *			Initialize MAPHRDATAINFO structure.
 *	Argument:
 *			pPi		pointer to MALIBPHRINFO
 *	Return:
 *
 *
 ****************************************************************************/
static void	InitPhraseInfo(PMALIBPHRINFO pPi)
{
	UINT32			i;
	PMALIBVOCINFO	pVi;

	pPi->dErrChk		= 0L;
	pPi->dVersion		= 0L;
	pPi->pbVoice		= NULL;
	pPi->pbSeq			= NULL;
	pPi->dVoiceSize		= 0L;
	pPi->dSeqSize		= 0L;
	pPi->dPlayTime		= 0L;
	pPi->dTimer			= 0L;
	pPi->dCurrentTime	= 0L;
	pPi->dDataPosition	= 0L;
	for(i = 0; i < 16; i++) {
		pVi = &(pPi->sVocInfo[i]);
		pVi->dBankNo	= 0xFFFFFFFF;
		pVi->dProgNo	= 0xFFFFFFFF;
	}
}

/****************************************************************************
 *	InitApiInfo
 *
 *	Function:
 *			Initialize MAPHRAPIINFO structure.
 *	Argument:
 *			bPhrCh	Sequence Number(0..3)
 *	Return:
 *
 *
 ****************************************************************************/
static void	InitApiInfo(UINT32 dPhrCh)
{
	PMAPHRAPIINFO	pAi;

	if(MAPHR_MAX_CHANNEL <= dPhrCh) return;

	pAi = &(gPhrInfo.sApiInfo[dPhrCh]);
	pAi->Status				= MAPHR_STATUS_NODATA;
	pAi->dVolume			= 100L;
	pAi->dPan				= 64L;
	pAi->sdMasterChannel	= -1L;
	pAi->dSlave				= 0L;
	pAi->dLoopCount			= 0L;
	pAi->dLength			= 0L;
	pAi->dRamAdr			= 0L;
	pAi->dRamEndAdr			= 0L;
	InitPhraseInfo(&(pAi->sDataInfo));
}

/****************************************************************************
 *	InitChInfo
 *
 *	Function:
 *			Initialize MAPHRCHINFO structure.
 *	Argument:
 *			dCh		#Channel(0..15)
 *	Return:
 *
 *
 ****************************************************************************/
static void	InitChInfo(UINT32 dCh)
{
	PMAPHRCHINFO	pCi = &(gPhrInfo.sChInfo[dCh]);

	pCi->dKey			= 0L;
	pCi->dVelocity		= 127L;
	pCi->dProgNo		= 0L;
	pCi->dBankNo		= 1L;
	pCi->dNewBank		= 1L;
	pCi->dVolume		= 100L;
	pCi->dMod			= 0L;
	pCi->dRange			= 2L;
	pCi->dNewRange		= 2L;
	pCi->dPitch			= 64L;
	pCi->dExNoteFlag	= 0x00L;
	pCi->dRegH			= 0x00L;
	pCi->dRegL			= 0x00L;
	pCi->sdOctShift		= 0L;
	pCi->dNoteOffTime	= 0xFFFFFFFF;
	pCi->dPan			= gPhrInfo.sApiInfo[dCh >> 2].dPan;
	pCi->dExp			= gPhrInfo.sApiInfo[dCh >> 2].dVolume;
}

/****************************************************************************
 *	Callback
 *
 *	Function:
 *			Callback.
 *	Argument:
 *			dCh		Sequence Number(0..3)
 *			dEvent	Event Number
 *	Return:
 *			0	success.
 *
 ****************************************************************************/
static SINT32	Callback(UINT32 dCh, UINT32 dEvent)
{
	struct event eve;

	if(gPhrInfo.pfnEvHandler != NULL) {
		eve.ch		= (int)dCh;
		eve.mode	= (int)dEvent;
		gPhrInfo.pfnEvHandler(&eve);
	}

	return (MASMW_SUCCESS);
}

/****************************************************************************
 *	Seek1
 *
 *	Function:
 *			Seek(for LV1).
 *	Argument:
 *			dPhrCh	Sequence Number(0..3)
 *			dPos	Seek Position[ms]
 *	Return:
 *			0	success.
 *			< 0	error code.
 *
 ****************************************************************************/
static SINT32	Seek1(UINT32 dPhrCh, UINT32 dPos)
{
	UINT8*			pbExMsg;
	UINT32			i;
	UINT32			dCh;
	UINT32			dTmp;
	UINT32			dData;
	UINT32			dSeekTime;
	UINT32			dCurrent;
	UINT32			dDuration;
	PMAPHRCHINFO	pCi;
	PMALIBPHRINFO	pDi;

	MAPHRCNV_DBGMSG(("Seek1[%08lX %08lX %08lX] \n", dPhrCh, dPos, gPhrInfo.dPlayStatus));

	dSeekTime = (UINT32)(dPos / MASMW_DIRECT_BASETIME);
	pDi = &(gPhrInfo.sApiInfo[dPhrCh].sDataInfo);
	if(dSeekTime >= pDi->dPlayTime) {
		return (MASMW_ERROR);
	}

	/*	Seek to head position	*/
	pDi->dTimer			= 0L;
	pDi->dCurrentTime	= 0L;
	pDi->dDataPosition	= 0L;

	/*	Initialize each channel info	*/
	for(i = 0; i < MAPHR_MAX_SLOT; i++) {
		dCh = (UINT32)(i + (dPhrCh << 2));
		InitChInfo(dCh);
	}

	/*	Seek the position	*/
	while(pDi->dSeqSize > pDi->dDataPosition) {
		/*	Get Current Time	*/
		dCurrent	= pDi->dDataPosition;
		dDuration	= (UINT32)pDi->pbSeq[dCurrent++];
		if(dDuration & 0x80) {
			dDuration = (UINT32)(((dDuration & 0x7FL) << 7) +
								 (UINT32)pDi->pbSeq[dCurrent++] + 128L);
		}

		if((pDi->dCurrentTime + dDuration) >= dSeekTime) {
			pDi->dTimer = dSeekTime;
			break;
		}

		/*	Update position & time	*/
		pDi->dDataPosition	= dCurrent;
		pDi->dCurrentTime	+= dDuration;

		dData = (UINT32)pDi->pbSeq[pDi->dDataPosition++];
		switch(dData) {
		case 0x00L :
			dData	= pDi->pbSeq[pDi->dDataPosition++];
			dCh		= (UINT32)(((dData & 0xC0) >> 6) + (dPhrCh << 2));
			pCi		= &(gPhrInfo.sChInfo[dCh]);
			dTmp	= (UINT32)(dData & 0x0FL);
			switch(dData & 0x30L) {
			case 0x00L :	/*	Short type Volume	*/
				pCi->dVolume = gdShortVolTable[dTmp];
				break;
			case 0x10L :	/*	Short type PitchBend	*/
				break;
			case 0x20L :	/*	Short type Modulation	*/
				pCi->dMod = gdShortModTable[dTmp];
				break;
			case 0x30 :
				dData = (UINT32)pDi->pbSeq[pDi->dDataPosition++];
				switch(dTmp) {
				case 0x00L :	/*	Program Change	*/
					pCi->dProgNo = (UINT32)(dData & 0x03L);
					break;
				case 0x02L :	/*	Octave Shift	*/
					OctaveShift(dCh, dData);
					break;
				case 0x04L :	/*	Pitch Bend	*/
					pCi->dRange = pCi->dNewRange;
					pCi->dPitch = (UINT32)(dData & 0x7FL);
					break;
				case 0x0AL :	/*	Panpot	*/
					pCi->dPan = (UINT32)(dData & 0x7FL);
					break;
				case 0x0BL :	/*	Normal type Volume	*/
					pCi->dVolume = (UINT32)(dData & 0x7FL);
					break;
				default	  :
					break;
				}
				break;
			default	 :
				break;
			}
			break;
		case 0xFF :
			dData = (UINT32)pDi->pbSeq[pDi->dDataPosition++];
			if(dData == 0xF0L) {			/*	Exclusive Message	*/
				dTmp = (UINT32)pDi->pbSeq[pDi->dDataPosition++];
				pbExMsg	= &(pDi->pbSeq[pDi->dDataPosition]);
				if(	(dTmp == 6L)			&&
					(pbExMsg[0] == 0x43)	&&
					(pbExMsg[1] == 0x03)	&&
					(pbExMsg[2] == 0x90)	&&
					(pbExMsg[5] == 0xF7)) {			/*	Extend NoteOn/Off	*/

					dCh	= (UINT32)((pbExMsg[3] & 0x03) + (dPhrCh << 2));
					pCi	= &(gPhrInfo.sChInfo[dCh]);
					if((pbExMsg[3] & 0xF0) == 0xC0) {
						pCi->dExNoteFlag |= 0x02L;
						pCi->dRegH = (UINT32)pbExMsg[4];
					}
					else if((pbExMsg[3] & 0xF0) == 0xB0) {
						pCi->dExNoteFlag |= 0x01L;
						pCi->dRegL = (UINT32)pbExMsg[4];
					}
				}
				else if((dTmp == 6L)			&&
						(pbExMsg[0] == 0x43)	&&
						(pbExMsg[1] == 0x04)	&&
						(pbExMsg[2] == 0x02)	&&
						(pbExMsg[4] <= 24)		&&
						(pbExMsg[5] == 0xF7)) {		/*	Pitch Bend Range	*/

					dCh	= (UINT32)((pbExMsg[3] & 0x03) + (dPhrCh << 2));
					pCi	= &(gPhrInfo.sChInfo[dCh]);
					pCi->dNewRange	= (UINT32)pbExMsg[4];
				}

				/*	skip exclusive message	*/
				pDi->dDataPosition += dTmp;
			}
			break;
		default	  :
			dDuration = (UINT32)pDi->pbSeq[pDi->dDataPosition++];
			if(dDuration & 0x80L) {
				pDi->dDataPosition++;
			}
			break;
		}
	}

	return (MASMW_SUCCESS);
}

/****************************************************************************
 *	Seek2
 *
 *	Function:
 *			Seek(for LV2)
 *	Argument:
 *			dPhrCh	Sequence Number(0..3)
 *			dPos	Seek Position[ms]
 *	Return:
 *			0	success.
 *			< 0	error code.
 *
 ****************************************************************************/
static SINT32	Seek2(UINT32 dPhrCh, UINT32 dPos)
{
	UINT32			i;
	UINT32			dCh;
	UINT32			dTmp;
	UINT32			dData;
	UINT32			dSeekTime;
	UINT32			dCurrent;
	UINT32			dDuration;
	PMAPHRCHINFO	pCi;
	PMALIBPHRINFO	pDi;

	MAPHRCNV_DBGMSG(("Seek2[%08lX %08lX %08lX] \n", dPhrCh, dPos, gPhrInfo.dPlayStatus));

	dSeekTime = (UINT32)(dPos / MASMW_DIRECT_BASETIME);
	pDi = &(gPhrInfo.sApiInfo[dPhrCh].sDataInfo);
	if(dSeekTime >= pDi->dPlayTime) {
		return (MASMW_ERROR);
	}

	/*	Seek to head position	*/
	pDi->dTimer			= 0L;
	pDi->dCurrentTime	= 0L;
	pDi->dDataPosition	= 0L;

	/*	Initialize each channel info	*/
	for(i = 0; i < MAPHR_MAX_SLOT; i++) {
		dCh = (UINT32)(i + (dPhrCh << 2));
		InitChInfo(dCh);
	}

	/*	Seek the position	*/
	while(pDi->dSeqSize > pDi->dDataPosition) {
		/*	Get Current Time	*/
		dCurrent	= pDi->dDataPosition;
		dDuration	= (UINT32)pDi->pbSeq[dCurrent++];
		if(dDuration & 0x80L) {
			dDuration = (UINT32)(((dDuration & 0x7FL) << 7) +
								 (UINT32)pDi->pbSeq[dCurrent++] + 128L);
		}

		if((pDi->dCurrentTime + dDuration) >= dSeekTime) {
			pDi->dTimer = dSeekTime;
			break;
		}

		/*	Update position & time	*/
		pDi->dDataPosition	= dCurrent;
		pDi->dCurrentTime	+= dDuration;

		dData = (UINT32)pDi->pbSeq[pDi->dDataPosition++];
		switch(dData) {
		case 0x00L :
			dData	= pDi->pbSeq[pDi->dDataPosition++];
			dCh		= (UINT32)(((dData & 0xC0) >> 6) + (dPhrCh << 2));
			pCi		= &(gPhrInfo.sChInfo[dCh]);
			dTmp	= (UINT32)(dData & 0x0FL);
			switch(dData & 0x30) {
			case 0x00L :	/*	Short type Volume	*/
				if((0L < dTmp) && (dTmp < 15L)) {
					pCi->dVolume = gdShortVolTable[dTmp];
				}
				break;
			case 0x10L : /*	Short type PitchBend	*/
				break;
			case 0x20L :	/*	Short type Modulation	*/
				if((0L < dTmp) && (dTmp < 15L)) {
					pCi->dMod = gdShortModTable[dTmp];
				}
				break;
			case 0x30L :
				dData = (UINT32)pDi->pbSeq[pDi->dDataPosition++];
				switch(dTmp) {
				case 0x00L :	/*	Program Change	*/
					if(dData < 128L) {
						/*	update bank number	*/
						pCi->dBankNo = pCi->dNewBank;
						pCi->dProgNo = dData;
					}
					break;
				case 0x01L :	/*	Bank Select	*/
					BankSelect(dCh, dData);
					break;
				case 0x02L :	/*	Octave Shift	*/
					OctaveShift2(dCh, dData);
					break;
				case 0x03L :	/*	Normal type Modulation	*/
					if(dData < 128L) pCi->dMod = gdNormalModTable[dData];
					break;
				case 0x04L :	/*	Pitch Bend	*/
					if(dData < 128L) {
						pCi->dRange = pCi->dNewRange;
						pCi->dPitch = dData;
					}
					break;
				case 0x05L :	/*	Pitch Bend Range	*/
					if(dData <= 24L) pCi->dNewRange = dData;
					break;
				case 0x06L :	/*	Velocity	*/
					Velocity(dCh, dData);
					break;
				case 0x0AL :	/*	Panpot	*/
					if(dData < 128L) pCi->dPan = dData;
					break;
				case 0x0BL :	/*	Normal type Volume	*/
					if(dData < 128L) pCi->dVolume = dData;
					break;
				default	  :
					break;
				}
				break;
			default	 :
				break;
			}
			break;
		case 0xFF :
			dData = (UINT32)pDi->pbSeq[pDi->dDataPosition++];
			if(dData == 0xF0L) {			/*	Exclusive Message	*/
				dTmp = (UINT32)pDi->pbSeq[pDi->dDataPosition++];
				/*	skip exclusive message	*/
				pDi->dDataPosition += dTmp;
			}
			break;
		default	  :
			dDuration = (UINT32)pDi->pbSeq[pDi->dDataPosition++];
			if(dDuration & 0x80L) {
				pDi->dDataPosition++;
			}
			break;
		}
	}

	return (MASMW_SUCCESS);
}

/****************************************************************************
 *	SetVoice
 *
 *	Function:
 *			register voices for LV1.
 *	Argument:
 *			dPhrCh	Sequence Number(0..3)
 *			dIdx	Register Number(0..3)
 *			pbVoc	Pointer to the voice parameter.
 *			dSize	Size of voice parameter.
 *	Return:
 *			>=0	number of registered voices.
 *			< 0	error code.
 *
 ****************************************************************************/
static SINT32	SetVoice(UINT32 dPhrCh, UINT32 dIdx, UINT8* pbVoc, UINT32 dSize)
{
	UINT32			dBankNo;
	UINT32			dProgNo;
	UINT32			dVocSize;
	SINT32			sdRet;
	PMAPHRAPIINFO	pAi;
	PMALIBVOCINFO	pVi;
	static UINT8	bParam[MAPHR_MA5_4OP_SIZE];

	MASMW_ASSERT(dIdx < MAPHR_MAX_LV1_VOICES);

	pAi = &(gPhrInfo.sApiInfo[dPhrCh]);
	pVi = &(pAi->sDataInfo.sVocInfo[dIdx]);
	dBankNo		= 0x00L;
	dProgNo		= 0x00L;
	dVocSize	= 0L;

	/*	MA-2 2op Voice(Shortest voice parameter) */
	if(dSize < 21L) {
		MAPHRCNV_DBGMSG(("Shortest voice parameter[%ld] \n", dSize));
		return (0L);
	}

	/*	check size	*/
	dVocSize = (UINT32)pbVoc[2];
	if((UINT32)(dVocSize + 3L) > dSize) {
		return (0L);
	}

	/*	check header id	*/
	if(	(pbVoc[0] != 0xFF)		||
		(pbVoc[1] != 0xF0)		||
		(pbVoc[3] != 0x43)		||
		(pbVoc[dVocSize + 2L] != 0xF7)) {
		MAPHRCNV_DBGMSG(("Unkown system exclusive message. \n"));
		return (0L);
	}

	/*	MA-2 FM format setting */
	if(	((dVocSize == 18L) || (dVocSize == 28L))	&&
		(pbVoc[4] == 0x03)) {

		dBankNo		= 0x01L;
		dProgNo		= dIdx;
		dVocSize	-= 3L;
		dVocSize	= ConvertMA3Voice(&(pbVoc[5]), dVocSize, &(bParam[0]));
	}
	/*	MA-3 FM format setting	*/
	else if(((dVocSize == 20L) || (dVocSize == 34L))	&&
			(pbVoc[4] == 0x04)							&&
			(pbVoc[5] == 0x01)) {

		dBankNo		= 0x01L;
		dProgNo		= dIdx;
		dVocSize	-= 4;
		if((dVocSize > (UINT32)MAPHR_MA5_2OP_SIZE) && ((pbVoc[7] & 0x07) < 2))
			dVocSize = (UINT32)MAPHR_MA5_2OP_SIZE;
		machdep_memcpy(&(bParam[0]), &(pbVoc[6]), dVocSize);
	}
	else {
		return (0L);
	}

	/*	check ram address	*/
	if((pAi->dRamAdr + dVocSize) > pAi->dRamEndAdr) {
		MAPHRCNV_DBGMSG(("RAM Over!! [%08lX %08lX]\n", pAi->dRamAdr, dVocSize));
		return (0L);
	}

	/*	extend voice setting	*/
	sdRet = MaDevDrv_SendDirectRamData(pAi->dRamAdr, 0, &(bParam[0]), dVocSize);
	if(sdRet < MASMW_SUCCESS) {
		MAPHRCNV_DBGMSG(("MaDevDrv_SendDirectRanData[Return:%ld] \n", sdRet));
		return (sdRet);
	}
	pVi->dBankNo = dBankNo;
	pVi->dProgNo = dProgNo;

	/*	Remove voice	*/
	dBankNo	= (UINT8)(dBankNo + dPhrCh);
	MaSndDrv_SetVoice(	gPhrInfo.sdSeqId,
						dBankNo,		/*	Bank Number				 */
						dProgNo,		/*	Program Number			 */
						1L,				/*	Synth Mode				 */
						0L,				/*	Durum Key				 */
						0xFFFFFFFF);	/*	Registered voice address */
	/*	Set voice	*/
	MaSndDrv_SetVoice(	gPhrInfo.sdSeqId,
						dBankNo,		/*	Bank Number				 */
						dProgNo,		/*	Program Number			 */
						1L,				/*	Synth Mode				 */
						0L,				/*	Durum Key				 */
						pAi->dRamAdr);	/*	Registered voice address */

	MAPHRCNV_DBGMSG(("Register Voice[BK:%02X PG:%02X RAM:%08lX]\n",
										 dBankNo, dProgNo, pAi->dRamAdr));

	/*	update ram address	*/
	pAi->dRamAdr += dVocSize;
	if(pAi->dRamAdr & 0x00000001) pAi->dRamAdr++;
	return (1L);
}

/****************************************************************************
 *	SetupVoice
 *
 *	Function:
 *			register voices for LV1.
 *	Argument:
 *			dPhrCh	Sequence Number(0..3)
 *	Return:
 *			0	success.
 *			< 0	error code.
 *
 ****************************************************************************/
static SINT32	SetupVoice(UINT32 dPhrCh)
{
	UINT8*			pbVoc;
	UINT32			dIdx;
	UINT32			dSize;
	UINT32			dChunkId;
	UINT32			dChunkSize;
	SINT32			sdRet;
	PMAPHRAPIINFO	pAi;
	PMALIBPHRINFO	pDi;
	PMALIBVOCINFO	pVi;

	/*	no voice	*/
	pAi = &(gPhrInfo.sApiInfo[dPhrCh]);
	pDi = &(pAi->sDataInfo);
	if(pDi->pbVoice == NULL) {
		return (MASMW_SUCCESS);
	}

	/*	calucurate ram address	*/
	dSize			= (UINT32)(gPhrInfo.dRamSize >> 2);
	pAi->dRamAdr	= (UINT32)(gPhrInfo.dRamAdr + (dSize * dPhrCh));
	pAi->dRamEndAdr	= (UINT32)(pAi->dRamAdr + dSize);

	MAPHRCNV_DBGMSG(("SetupVoice [%X %08lX-%08lX]\n",
									dPhrCh, pAi->dRamAdr, pAi->dRamEndAdr));

	dIdx	= 0L;
	pbVoc	= pDi->pbVoice;
	dSize	= pDi->dVoiceSize;
	while(	(dSize >= (UINT32)(MAPHR_SIZE_OF_CHUNKHEADER)) &&
			(dIdx < (UINT32)MAPHR_MAX_LV1_VOICES)) {

		dChunkId	= MAPHR_MAKEDWORD(pbVoc[0], pbVoc[1], pbVoc[2], pbVoc[3]);
		dChunkSize	= MAPHR_MAKEDWORD(pbVoc[4], pbVoc[5], pbVoc[6], pbVoc[7]);
		pbVoc		+= (UINT32)MAPHR_SIZE_OF_CHUNKHEADER;

		/*	Check chunk size	*/
#if 0
		if(dSize < (UINT32)(MAPHR_SIZE_OF_CHUNKHEADER + dChunkSize)) {
			return (MASMW_ERROR_CHUNK_SIZE);
		}
#endif

		/*	default voice chunk	*/
		if(dChunkId == MAPHR_MAKEDWORD('D', 'E', 'V', 'O')) {
			pVi = &(pDi->sVocInfo[dIdx]);
			pVi->dBankNo = 0x00L;
			if(dChunkSize == 0L)
				pVi->dProgNo = 0x00L;
			else
				pVi->dProgNo = (UINT32)(pbVoc[0] & 0x7F);
			dIdx++;
		}
		/*	extend voice chunk	*/
		else if(dChunkId == MAPHR_MAKEDWORD('E', 'X', 'V', 'O')) {
			sdRet = SetVoice(dPhrCh, dIdx, pbVoc, dChunkSize);
			if(sdRet < MASMW_SUCCESS) return (sdRet);
			dIdx++;
		}

		/*	Update	*/
		pbVoc	+= dChunkSize;
		dSize	-= (UINT32)(dChunkSize + MAPHR_SIZE_OF_CHUNKHEADER);
	}

	return (MASMW_SUCCESS);
}

/****************************************************************************
 *	SetWtBank
 *
 *	Function:
 *			register voices for LV2.
 *	Argument:
 *			pbWave	Pointer to the wave data.
 *			dSize	Size of wave data.
 *			pWtBank	Pointer to the WT bank.
 *	Return:
 *			0	success.
 *			< 0	error code.
 *
 ****************************************************************************/
static SINT32	SetWtBank(UINT8* pbWave, UINT32 dSize, PMAPHRWTBANK pWtBank)
{
	UINT32	dWaveId;
	UINT32	dWaveSize;

	MAPHRCNV_DBGMSG(("SetWtBank [%08lX]\n", dSize));

	if(dSize < 10L)	return (MASMW_ERROR);

	if(	(pbWave[0] != 0xFF)	||
		(pbWave[1] != 0xF1)	||
		(pbWave[4] != 0x43)	||
		(pbWave[5] != 0x05)	||
		(pbWave[6] != 0x00)	||
		(pbWave[7] >= MAPHR_MAX_WAVES)) {

		return (MASMW_ERROR);
	}

	dWaveId		= (UINT32)pbWave[7];
	dWaveSize	= (UINT32)(((UINT32)pbWave[3] << 8) | (UINT32)pbWave[2]);

	/*	check size	*/
	if(dWaveSize < 5L)					return (MASMW_ERROR);
	if((dWaveSize + 4L) > dSize)		return (MASMW_ERROR);
	if(pbWave[dWaveSize + 3L] != 0xF7)	return (MASMW_ERROR);

	/*	add wt bank	*/
	if(pWtBank[dWaveId].pbWave == NULL) {
		pWtBank[dWaveId].pbWave	= &(pbWave[8]);
		pWtBank[dWaveId].dSize	= (dWaveSize - 5L);
		MAPHRCNV_DBGMSG(("Add WT Bank[ID:%02X %08lX]\n", dWaveId, dWaveSize - 5L));
	}
	else {
		MAPHRCNV_DBGMSG(("Already added wave data[ID:%02X]\n", dWaveId));
	}
	return (MASMW_SUCCESS);
}

/****************************************************************************
 *	SetWaveData
 *
 *	Function:
 *			register WT data for LV2.
 *	Argument:
*			dPhrCh	Sequence Number(0..3)
 *			dWaveId	wave id(0..15)
 *			pWtBank	Pointer to the WT bank.
 *	Return:
 *			>=0	success.
 *			< 0	error code.
 *
 ****************************************************************************/
static SINT32	SetWaveData(UINT32 dPhrCh, UINT32 dWaveId, PMAPHRWTBANK pWtBank)
{
	SINT32			sdRet;
	UINT32			dEndAdr;
	PMAPHRAPIINFO	pAi;
	PMAPHRWTBANK	pWi;

	MAPHRCNV_DBGMSG(("SetWaveData [%02X]\n", dWaveId));

	if(dWaveId >= (UINT32)MAPHR_MAX_WAVES) {
		MAPHRCNV_DBGMSG(("Invalid wave id[%02X] \n", dWaveId));
		return (MASMW_SUCCESS);
	}

	pAi	= &(gPhrInfo.sApiInfo[dPhrCh]);
	pWi	= &(pWtBank[dWaveId]);
	if(pWi->pbWave == NULL) {
		MAPHRCNV_DBGMSG(("No wave data[%02X] \n", dWaveId));
		return (MASMW_SUCCESS);
	}

	/*	already registered	*/
	if(pWi->dRamAdr != 0x00000000) {
		return (SINT32)(pWi->dRamAdr);
	}

	/*	ram check	*/
	dEndAdr = pAi->dRamAdr + pWi->dSize;
	if(dEndAdr & 0x00000001) dEndAdr++;
	dEndAdr += MAPHR_MA5_WT_SIZE;	/*	WT data + voice	*/
	if(dEndAdr > pAi->dRamEndAdr) {
		MAPHRCNV_DBGMSG(("RAM Over!! [%08lX %08lX]\n", pAi->dRamAdr, pWi->dSize));
		return (MASMW_SUCCESS);
	}

	sdRet = MaDevDrv_SendDirectRamData(pAi->dRamAdr, 0, pWi->pbWave, pWi->dSize);
	if(sdRet < MASMW_SUCCESS) {
		MAPHRCNV_DBGMSG(("MaDevDrv_SendDirectRanData[Return:%ld] \n", sdRet));
		return (sdRet);
	}

	/*	update ram address	*/
	pWi->dRamAdr = pAi->dRamAdr;
	pAi->dRamAdr += pWi->dSize;
	if(pAi->dRamAdr & 0x00000001) pAi->dRamAdr++;

	return (SINT32)(pWi->dRamAdr);
}

/****************************************************************************
 *	SetVoice2
 *
 *	Function:
 *			register voices for LV2.
 *	Argument:
 *			dPhrCh	Sequence Number(0..3)
 *			pbVoc	Pointer to the voice parameter.
 *			dSize	Size of voice parameter.
 *			pWtBank	Pointer to the WT bank.
 *	Return:
 *			>=0	number of registered voices.
 *			< 0	error code.
 *
 ****************************************************************************/
static SINT32	SetVoice2(UINT32 dPhrCh, UINT8* pbVoc, UINT32 dSize,
														 PMAPHRWTBANK pWtBank)
{
	UINT32			i;
	UINT32			dSynth;
	UINT32			dWaveId;
	UINT32			dBankNo;
	UINT32			dProgNo;
	UINT32			dDrumKey;
	UINT32			dVocSize;
	SINT32			sdRet;
	SINT32			sdWaveAdr;
	PMAPHRAPIINFO	pAi;
	PMALIBPHRINFO	pDi;
	PMALIBVOCINFO	pVi;
	static UINT8	bParam[MAPHR_MA5_4OP_SIZE];

	pAi	= &(gPhrInfo.sApiInfo[dPhrCh]);
	pDi = &(pAi->sDataInfo);
	dSynth		= 0x00L;
	dBankNo		= 0x00L;
	dProgNo		= 0x00L;
	dWaveId		= 0x00L;
	dDrumKey	= 0L;
	dVocSize	= 0L;

	/*	MA-5 WT Voice(Shortest voice parameter) */
	if(dSize < 25L) {
		MAPHRCNV_DBGMSG(("Shortest voice parameter[%ld] \n", dSize));
		return (0L);
	}

	/*	check size	*/
	dVocSize = (UINT32)pbVoc[2];
	if((UINT32)(dVocSize + 3L) > dSize) {
		return (0L);
	}

	/*	check header id	*/
	if(	(pbVoc[0] != 0xFF)		||
		(pbVoc[1] != 0xF0)		||
		(pbVoc[3] != 0x43)		||
		(pbVoc[4] != 0x05)		||
		(pbVoc[dVocSize + 2] != 0xF7)) {
		MAPHRCNV_DBGMSG(("Unkown system exclusive message. \n"));
		return (0L);
	}

	/*	MA-5 FM format setting */
	if(	((dVocSize == 23L) || (dVocSize == 37L)) && (pbVoc[5] == 0x01)) {
		dSynth		= 1L;
		dBankNo		= (UINT32)pbVoc[6];
		dProgNo		= (UINT32)pbVoc[7];
		dDrumKey	= (UINT32)pbVoc[8];
		dVocSize	-= 7L;
		pbVoc		+= 9;

		if(dDrumKey > 127L) {
			MAPHRCNV_DBGMSG(("Invalid drum key[%08lX] \n", dDrumKey));
			return (0L);
		}

		if(	(dVocSize > (UINT32)MAPHR_MA5_2OP_SIZE)	&&
			((UINT8)(pbVoc[1] & 0x07) < 2)) {
			dVocSize = (UINT32)MAPHR_MA5_2OP_SIZE;
		}
	}
	/*	MA-5 WT format setting	*/
	else if((dVocSize == 22L) && (pbVoc[5] == 0x02)) {
		dSynth		= 2L;
		dBankNo		= (UINT32)pbVoc[6];
		dProgNo		= (UINT32)pbVoc[7];
		dDrumKey	= (UINT32)(((UINT32)pbVoc[8] << 8) | (UINT32)pbVoc[9]);
		dWaveId		= (UINT32)pbVoc[23];
		dVocSize	-= 9L;
		pbVoc		+= 10;

		/*	Check format and drumkey	*/
		if(	((pbVoc[0] & 0x04) || (pbVoc[1] & 0x03)) ||
			((dDrumKey < 1L) || (48000 < dDrumKey))) {
			MAPHRCNV_DBGMSG(("Invalid format[%02X-%02X] or drum key[%08lX] \n",
				((pbVoc[0] >> 2) & 0x01), (pbVoc[1] & 0x03), dDrumKey));
			return (0L);
		}
	}
	else {
		return (MASMW_SUCCESS);
	}

	/*	Check BankNo and ProgNo range	*/
	if(	((dBankNo != 0x01L) && (dBankNo != 0x81L))	||
		((dBankNo == 0x01L) && (dProgNo > 127L))	||
		((dBankNo == 0x81L) && ((dProgNo < 1L) || (47L < dProgNo)))) {

		MAPHRCNV_DBGMSG(("Invalid BankNo[%02X] or ProgNo[%02X] \n",
			 dBankNo, dProgNo));
		return (0L);
	}

	pVi = GetVoiceInfo(dBankNo, dProgNo, &(pDi->sVocInfo[0]));
	if(pVi != NULL) {
		MAPHRCNV_DBGMSG(("Already add voice BankNo[%02X] or ProgNo[%02X] \n",
			 dBankNo, dProgNo));
		return (0L);
	}

	machdep_memcpy(&(bParam[0]), &(pbVoc[0]), dVocSize);

	/*	set wave data	*/
	if(dSynth == 2L) {
		if(dWaveId & 0x80L) {	/*	ROM data	*/
			sdWaveAdr = MaResMgr_GetDefWaveAddress((UINT32)(dWaveId & 0x3FL));
		}
		else {					/*	RAM data	*/
			sdWaveAdr = SetWaveData(dPhrCh, dWaveId, pWtBank);
			if(sdWaveAdr < 0L) return (sdWaveAdr);
		}

		if(sdWaveAdr <= 0L) {
			MAPHRCNV_DBGMSG(("Invalid wave data[%02X] \n", dWaveId));
			return (0L);
		}

		bParam[7] = (UINT8)(((UINT32)sdWaveAdr >> 8) & 0xFF);
		bParam[8] = (UINT8)((UINT32)sdWaveAdr & 0xFF);
	}

	/*	check ram address	*/
	if((pAi->dRamAdr + dVocSize) > pAi->dRamEndAdr) {
		MAPHRCNV_DBGMSG(("RAM Over!! [%08lX %08lX]\n", pAi->dRamAdr, dVocSize));
		return (0L);
	}

	/*	extend voice setting	*/
	sdRet = MaDevDrv_SendDirectRamData(pAi->dRamAdr, 0, &(bParam[0]), dVocSize);
	if(sdRet < MASMW_SUCCESS) {
		MAPHRCNV_DBGMSG(("MaDevDrv_SendDirectRanData[Return:%ld] \n", sdRet));
		return (sdRet);
	}

	/*	add voice infomation	*/
	for(i = 0; i < MAPHR_MAX_LV2_VOICES; i++) {
		pVi = &(pDi->sVocInfo[i]);
		if(pVi->dBankNo == 0xFFFFFFFF) {
			pVi->dBankNo = dBankNo;
			pVi->dProgNo = dProgNo;
			break;
		}
	}
	MASMW_ASSERT(i < MAPHR_MAX_LV2_VOICES);

	/*	Remove voice	*/
	dBankNo	= (UINT32)(dBankNo + dPhrCh);
	MaSndDrv_SetVoice(	gPhrInfo.sdSeqId,
						dBankNo,			/*	Bank Number				 */
						dProgNo,			/*	Program Number			 */
						dSynth,				/*	Synth Mode				 */
						dDrumKey,			/*	Durum Key				 */
						0xFFFFFFFF);		/*	Registered voice address */
	/*	Set voice	*/
	MaSndDrv_SetVoice(	gPhrInfo.sdSeqId,
						dBankNo,			/*	Bank Number				 */
						dProgNo,			/*	Program Number			 */
						dSynth,				/*	Synth Mode				 */
						dDrumKey,			/*	Durum Key				 */
						pAi->dRamAdr);		/*	Registered voice address */

	MAPHRCNV_DBGMSG(("Register Voice[BK:%02X PG:%02X KEY:%08lX RAM:%08lX]\n",
								 dBankNo, dProgNo, dDrumKey, pAi->dRamAdr));

	/*	update ram address	*/
	pAi->dRamAdr += dVocSize;
	if(pAi->dRamAdr & 0x00000001) pAi->dRamAdr++;
	return (1L);
}

/****************************************************************************
 *	SetupVoice2
 *
 *	Function:
 *			register voices for LV2.
 *	Argument:
 *			dPhrCh	Sequence Number(0..3)
 *	Return:
 *			0	success.
 *			< 0	error code.
 *
 ****************************************************************************/
static SINT32	SetupVoice2(UINT32 dPhrCh)
{
	UINT8*				pbVoc;
	UINT32				dIdx;
	UINT32				dSize;
	UINT32				dChunkId;
	UINT32				dChunkSize;
	SINT32				sdRet;
	PMAPHRAPIINFO		pAi;
	PMALIBPHRINFO		pDi;
	static MAPHRWTBANK	sWtBank[MAPHR_MAX_WAVES];

	pAi = &(gPhrInfo.sApiInfo[dPhrCh]);
	pDi = &(pAi->sDataInfo);

	/*	no voice	*/
	if(pDi->pbVoice == NULL) {
		return (MASMW_SUCCESS);
	}

	/*	calucurate ram address	*/
	dSize			= (UINT32)(gPhrInfo.dRamSize >> 2);
	pAi->dRamAdr	= (UINT32)(gPhrInfo.dRamAdr + (dSize * dPhrCh));
	pAi->dRamEndAdr	= (UINT32)(pAi->dRamAdr + dSize);

	MAPHRCNV_DBGMSG(("SetupVoice2 [%X %08lX-%08lX]\n",
									dPhrCh, pAi->dRamAdr, pAi->dRamEndAdr));

	/*	initialize WT bank	*/
	for(dIdx = 0; dIdx < MAPHR_MAX_WAVES; dIdx++) {
		sWtBank[dIdx].pbWave	= NULL;
		sWtBank[dIdx].dSize		= 0L;
		sWtBank[dIdx].dRamAdr	= 0L;
	}

	dIdx	= 0L;
	pbVoc	= pDi->pbVoice;
	dSize	= pDi->dVoiceSize;
	while(	(dSize >= (UINT32)(MAPHR_SIZE_OF_CHUNKHEADER)) &&
			(dIdx < MAPHR_MAX_LV2_VOICES)) {

		dChunkId	= MAPHR_MAKEDWORD(pbVoc[0], pbVoc[1], pbVoc[2], pbVoc[3]);
		dChunkSize	= MAPHR_MAKEDWORD(pbVoc[4], pbVoc[5], pbVoc[6], pbVoc[7]);
		pbVoc		+= (UINT32)MAPHR_SIZE_OF_CHUNKHEADER;

		/*	Check chunk size	*/
#if 0
		if(dSize < (UINT32)(MAPHR_SIZE_OF_CHUNKHEADER + dChunkSize)) {
			return (MASMW_ERROR_CHUNK_SIZE);
		}
#endif

		/*	extend wt data chunk	*/
		if(dChunkId == MAPHR_MAKEDWORD('E', 'X', 'W', 'V')) {
			SetWtBank(pbVoc, dChunkSize, &(sWtBank[0]));
		}
		/*	extend voice chunk	*/
		else if(dChunkId == MAPHR_MAKEDWORD('E', 'X', 'V', 'O')) {
			sdRet = SetVoice2(dPhrCh, pbVoc, dChunkSize, &(sWtBank[0]));
			if(sdRet < MASMW_SUCCESS) return (sdRet);
			if(sdRet > 0L) dIdx++;	/*	update number of voice	*/
		}

		/*	Update	*/
		pbVoc	+= dChunkSize;
		dSize	-= (UINT32)(dChunkSize + MAPHR_SIZE_OF_CHUNKHEADER);
	}

	return (MASMW_SUCCESS);
}


/****************************************************************************
 *	RemoveVoices
 *
 *	Function:
 *			remove voices.
 *	Argument:
 *			dPhrCh	Sequence Number(0..3)
 *	Return:
 *			0	success.
 *			< 0	error code.
 *
 ****************************************************************************/
static SINT32	RemoveVoices(UINT32 dPhrCh)
{
	UINT32			i;
	UINT32			dVoices;
	PMAPHRAPIINFO	pAi;
	PMALIBVOCINFO	pVi;

	pAi = &(gPhrInfo.sApiInfo[dPhrCh]);
	pVi = &(pAi->sDataInfo.sVocInfo[0]);

	if(pAi->sDataInfo.dVersion == (UINT32)MAPHR_VERSION_LV2)
		dVoices	= MAPHR_MAX_LV2_VOICES;
	else
		dVoices	= MAPHR_MAX_LV1_VOICES;

	for(i = 0; i < dVoices; i++) {
		if((pVi[i].dBankNo == 0x01L) || (pVi[i].dBankNo == 0x81L)) {
			/*	Remove voice	*/
			MaSndDrv_SetVoice(	gPhrInfo.sdSeqId,
								pVi[i].dBankNo + dPhrCh,	/*	Bank Number		*/
								pVi[i].dProgNo,				/*	Program Number	*/
								1L,							/*	Synth Mode		*/
								0L,							/*	Durum Key		*/
								0xFFFFFFFF);

			MAPHRCNV_DBGMSG(("Remove Voice[BK:%02X PG:%02X]\n",
										 pVi[i].dBankNo, pVi[i].dProgNo));
		}
	}

	return (MASMW_SUCCESS);
}

/****************************************************************************
 *	PhrAudCnv_Initialize
 *
 *	Function:
 *			Initialize SMAF/Phrase and SMAF/Audio converter.
 *	Argument:
 *
 *	Return:
 *			0	success.
 *
 ****************************************************************************/
SINT32	PhrAudCnv_Initialize(void)
{
	MAPHRCNV_DBGMSG(("PhrAudCnv_Initialize\n"));

	gPhrInfo.sdSeqId		= -1L;
	gPhrInfo.dCreateStatus	= 0x00000000;
	gPhrInfo.dPlayStatus	= 0x00000000;
	gPhrInfo.dCtrlStatus	= 0x00000000;

	return (MASMW_SUCCESS);
}

/****************************************************************************
 *	Phrase_Initialize
 *
 *	Function:
 *			Initialize SMAF/Phrase sequencer.
 *	Argument:
 *
 *	Return:
 *			0	success.
 *			< 0	error.
 *
 ****************************************************************************/
int	Phrase_Initialize(void)
{
	UINT32		i;
	UINT32		dRamAdr;
	UINT32		dRamBlockSize;

	MAPHRCNV_DBGMSG(("Phrase_Initialize\n"));

	/*---	check SMAF/Phrase stream converter ID ----------------------------*/
	if(gPhrInfo.dCreateStatus & (UINT32)MAPHR_MASK_PHRASE) return (-1);

	/*---	Initialize global values -----------------------------------------*/
	gPhrInfo.dPhrStatus		= 0x00000000;
	gPhrInfo.dSlave			= 0x00000000;
	gPhrInfo.pfnEvHandler	= NULL;
	for(i = 0; i < MAPHR_MAX_CHANNEL; i++)	InitApiInfo(i);
	for(i = 0; i < MASMW_NUM_CHANNEL; i++)	InitChInfo(i);

	/*---	MA-5 Sequence Create ---------------------------------------------*/
	if(gPhrInfo.dCreateStatus == 0L) {
		if(ResourceCreate() < MASMW_SUCCESS) {
			return (-1);
		}
	}
	gPhrInfo.dCreateStatus |= (UINT32)MAPHR_MASK_PHRASE;

	/*---	Calucurate RAM address -------------------------------------------*/
	dRamBlockSize = (UINT32)(gPhrInfo.dRamSize >> 2);
	for(i = 0; i < MAPHR_MAX_CHANNEL; i++) {
		dRamAdr = (UINT32)(gPhrInfo.dRamAdr + (dRamBlockSize * i));
		gPhrInfo.sApiInfo[i].dRamAdr = dRamAdr;
		MAPHRCNV_DBGMSG(("Ram Address #%d - %08lX\n", i, dRamAdr));
	}

	return (0);
}

/****************************************************************************
 *	Phrase_GetInfo
 *
 *	Function:
 *			Get device infomation.
 *	Argument:
 *			pDat	pointer to info structure.
 *	Return:
 *			0	success.
 *			< 0	error.
 *
 ****************************************************************************/
int	Phrase_GetInfo(struct info* pDat)
{
	MAPHRCNV_DBGMSG(("Phrase_GetInfo[%08lX] \n", pDat));

	if(pDat == NULL)	return (-1);

	pDat->MakerID		= 0x00430000;			/*	YAMAHA	*/
	pDat->DeviceID		= 0x07;					/*	MA-5	*/
	pDat->VersionID		= 2;
	pDat->MaxVoice		= 4;
	pDat->MaxChannel	= 4;
	pDat->SupportSMAF	= 0x03;					/*	SMAF/Phrase L1 & L2	*/
	pDat->Latency		= (long)(20 * 1000);	/*	20msec	*/

	return (0);
}

/****************************************************************************
 *	Phrase_CheckData
 *
 *	Function:
 *			Check SMAF/Phrase data.
 *	Argument:
 *			pbData	Pointer to the data.
 *			dLen	Size of data
 *	Return:
 *			0		success.
 *			< 0		error
 *
 ****************************************************************************/
int	Phrase_CheckData(unsigned char* pbData, unsigned long dLen)
{
	SINT32	sdRet;

	MAPHRCNV_DBGMSG(("Phrase_CheckData [%08lX %ld] \n", pbData, dLen));

	/*---	check SMAF/Phrase stream converter ID ---------------------------*/
	if(!(gPhrInfo.dCreateStatus & (UINT32)MAPHR_MASK_PHRASE)	||
		(gPhrInfo.sdSeqId < 0L)) {
		return (-1);
	}

	/*---	check parameter -------------------------------------------------*/
	if(pbData == NULL)	return (-1);

	sdRet = malib_smafphrase_checker(pbData, dLen, NULL);
	if(sdRet < MASMW_SUCCESS) {
		return (-1);
	}

	return (0);
}

/****************************************************************************
 *	Phrase_SetData
 *
 *	Function:
 *			Load SMAF/Phrase data.
 *	Argument:
 *			nCh		Sequence Number(0..3)
 *			pbData	Pointer to the data.
 *			dLen	Size of data
 *			nCheck	error check(0:Disable 1:Enable)
 *	Return:
 *			0		success.
 *			< 0		error
 *
 ****************************************************************************/
int	Phrase_SetData(int nCh, unsigned char* pbData,
										unsigned long dLen, int nCheck)
{
	SINT32			sdRet;
	PMAPHRAPIINFO	pAi;

	MAPHRCNV_DBGMSG(("Phrase_SetData [%d %08lX %ld %d] \n",
										 nCh, pbData, dLen, nCheck));

	/*---	check SMAF/Phrase stream converter ID ---------------------------*/
	if(!(gPhrInfo.dCreateStatus & (UINT32)MAPHR_MASK_PHRASE)	||
		(gPhrInfo.sdSeqId < 0L)) {
		return (-1);
	}

	/*---	check parameter -------------------------------------------------*/
	if((nCh < 0) || (MAPHR_MAX_CHANNEL <= nCh))	return (-1);
	if(pbData == NULL)							return (-1);

	/*---	check status (Is status NODATA?) --------------------------------*/
	pAi = &(gPhrInfo.sApiInfo[nCh]);
	if(pAi->Status != MAPHR_STATUS_NODATA)	return (-1);

	/*---	load SMAF/Phrase data -------------------------------------------*/
	InitPhraseInfo(&(pAi->sDataInfo));
	pAi->sDataInfo.dErrChk = (UINT32)((nCheck)? 1 : 0);
	sdRet = malib_smafphrase_checker(pbData, dLen, &(pAi->sDataInfo));
	if(sdRet < MASMW_SUCCESS)	return (-1);

	pAi->dLength = (UINT32)sdRet;

	/*---	setup extend voice ----------------------------------------------*/
	if(pAi->sDataInfo.dVersion == (UINT32)MAPHR_VERSION_LV2) {
		sdRet = SetupVoice2((UINT32)nCh);
		Seek2((UINT32)nCh, 0);	/* seek to top(0ms) */
	}
	else {
		sdRet = SetupVoice((UINT32)nCh);
		Seek1((UINT32)nCh, 0);	/* seek to top(0ms) */
	}

	if(sdRet < MASMW_SUCCESS) return (-1);

	/*---	successful load data --------------------------------------------*/
	pAi->Status	= MAPHR_STATUS_READY;

	/*---	setup volumes ---------------------------------------------------*/
	gPhrInfo.dCtrlStatus |= (UINT32)(MAPHR_CTRL_VOLUME << (nCh << 2));
	return (0);
}

/****************************************************************************
 *	Phrase_Seek
 *
 *	Function:
 *			Seek
 *	Argument:
 *			nCh		Sequence Number(0..3)
 *			dPos	Seek Position[ms]
 *	Return:
 *			0	success.
 *			< 0	error.
 *
 ****************************************************************************/
int	Phrase_Seek(int nCh, long sdPos)
{
	UINT32			i;
	UINT32			dSlave;
	SINT32			sdRet;
	PMAPHRAPIINFO	pAi;

	MAPHRCNV_DBGMSG(("Phrase_Seek[%d %ld] \n", nCh, sdPos));

	/*---	check SMAF/Phrase stream converter ID ---------------------------*/
	if(!(gPhrInfo.dCreateStatus & (UINT32)MAPHR_MASK_PHRASE)	||
		(gPhrInfo.sdSeqId < 0L)) {
		return (-1);
	}
	/*---	check channel ---------------------------------------------------*/
	if((nCh < 0) || (MAPHR_MAX_CHANNEL <= nCh))	return (-1);

	/*---	check status (Is status READY?) ---------------------------------*/
	pAi = &(gPhrInfo.sApiInfo[nCh]);
	if(pAi->Status != MAPHR_STATUS_READY)		return (-1);

	/*---	check position --------------------------------------------------*/
	if((sdPos < 0L) || ((SINT32)pAi->dLength <= sdPos))
		return (-1);

	/*---	is slave ? ------------------------------------------------------*/
	if(gPhrInfo.dSlave & (UINT32)(1L << nCh))	return (-1);

	/*---	seek position ---------------------------------------------------*/
	if(pAi->sDataInfo.dVersion == (UINT32)MAPHR_VERSION_LV2)
		sdRet = Seek2((UINT32)nCh, sdPos);
	else
		sdRet = Seek1((UINT32)nCh, sdPos);

	if(sdRet != MASMW_SUCCESS)	return (-1);

	dSlave = pAi->dSlave;
	for(i = 0; i < MAPHR_MAX_CHANNEL; i++) {
		if(dSlave & (UINT32)(1L << i)) {
			pAi = &(gPhrInfo.sApiInfo[i]);
			if(pAi->Status != MAPHR_STATUS_READY)
				continue;
			if(pAi->sDataInfo.dVersion == (UINT32)MAPHR_VERSION_LV2)
				Seek2(i, sdPos);
			else
				Seek1(i, sdPos);
		}
	}

	return (0);
}

/****************************************************************************
 *	Phrase_Play
 *
 *	Function:
 *			start playback.
 *	Argument:
 *			nCh		Sequence Number(0..3)
 *			nLoop	Number of repeat(0..255, 0:Infinity)
 *	Return:
 *			0	success.
 *			< 0	error.
 *
 ****************************************************************************/
int	Phrase_Play(int nCh, int nLoop)
{
	UINT32			i;
	UINT32			dSlave;
	PMAPHRAPIINFO	pAi;

	MAPHRCNV_DBGMSG(("Phrase_Play[%d %d] \n", nCh, nLoop));

	/*---	check SMAF/Phrase stream converter ID ---------------------------*/
	if(!(gPhrInfo.dCreateStatus & (UINT32)MAPHR_MASK_PHRASE)	||
		(gPhrInfo.sdSeqId < 0L)) {
		return (-1);
	}
	/*---	check channel ---------------------------------------------------*/
	if((nCh < 0) || (MAPHR_MAX_CHANNEL <= nCh))	return (-1);
	if((nLoop < 0) || (255 < nLoop))			return (-1);

	/*---	check status (Is status READY?) ---------------------------------*/
	pAi = &(gPhrInfo.sApiInfo[nCh]);
	if(pAi->Status != MAPHR_STATUS_READY)		return (-1);

	/*---	is slave ? ------------------------------------------------------*/
	if(gPhrInfo.dSlave & (UINT32)(1L << nCh))	return (-1);

	/*---	start playback --------------------------------------------------*/
	pAi->Status			= MAPHR_STATUS_PLAY;
	pAi->dLoopCount		= (UINT32)nLoop;
	gPhrInfo.dPhrStatus |= gdMaskStart[nCh];

	dSlave = pAi->dSlave;
	for(i = 0; i < MAPHR_MAX_CHANNEL; i++) {
		if(dSlave & (UINT32)(1L << i)) {
			pAi = &(gPhrInfo.sApiInfo[i]);
			if(pAi->Status == MAPHR_STATUS_READY) {
				pAi->Status		= MAPHR_STATUS_PLAY;
				pAi->dLoopCount	= (UINT32)nLoop;
				gPhrInfo.dPhrStatus	|= gdMaskStart[i];
			}
		}
	}

	return (0);
}

/****************************************************************************
 *	Phrase_Pause
 *
 *	Function:
 *			pause playback.
 *	Argument:
 *			nCh		Sequence Number(0..3)
 *	Return:
 *			0	success.
 *			< 0	error.
 *
 ****************************************************************************/
int	Phrase_Pause(int nCh)
{
	UINT32			i;
	UINT32			dSlave;
	PMAPHRAPIINFO	pAi;

	MAPHRCNV_DBGMSG(("Phrase_Pause[%d] \n", nCh));

	/*---	check SMAF/Phrase stream converter ID ---------------------------*/
	if(!(gPhrInfo.dCreateStatus & (UINT32)MAPHR_MASK_PHRASE)	||
		(gPhrInfo.sdSeqId < 0L)) {
		return (-1);
	}
	/*---	check channel ---------------------------------------------------*/
	if((nCh < 0) || (MAPHR_MAX_CHANNEL <= nCh))	return (-1);

	/*---	check status (Is status PLAY?) ----------------------------------*/
	pAi = &(gPhrInfo.sApiInfo[nCh]);
	if(pAi->Status != MAPHR_STATUS_PLAY)		return (-1);

	/*---	is slave ? ------------------------------------------------------*/
	if(gPhrInfo.dSlave & (UINT32)(1L << nCh))	return (-1);

	/*---	pause playback --------------------------------------------------*/
	pAi->Status = MAPHR_STATUS_PAUSE;
	if(gPhrInfo.dPhrStatus & gdMaskRestart[nCh])
		gPhrInfo.dPhrStatus &= ~gdMaskRestart[nCh];
	else
		gPhrInfo.dPhrStatus |= gdMaskPause[nCh];

	dSlave = pAi->dSlave;
	for(i = 0; i < MAPHR_MAX_CHANNEL; i++) {
		if(dSlave & (UINT32)(1L << i)) {
			pAi = &(gPhrInfo.sApiInfo[i]);
			if(pAi->Status == MAPHR_STATUS_PLAY) {
				pAi->Status = MAPHR_STATUS_PAUSE;
				if(gPhrInfo.dPhrStatus & gdMaskRestart[i])
					gPhrInfo.dPhrStatus &= ~gdMaskRestart[i];
				else
					gPhrInfo.dPhrStatus |= gdMaskPause[i];
			}
		}
	}

	return (0);
}

/****************************************************************************
 *	Phrase_Restart
 *
 *	Function:
 *			restart playback.
 *	Argument:
 *			nCh		Sequence Number(0..3)
 *	Return:
 *			0	success.
 *			< 0	error.
 *
 ****************************************************************************/
int	Phrase_Restart(int nCh)
{
	UINT32			i;
	UINT32			dSlave;
	PMAPHRAPIINFO	pAi;

	MAPHRCNV_DBGMSG(("Phrase_Restart[%d] \n", nCh));

	/*---	check SMAF/Phrase stream converter ID ---------------------------*/
	if(!(gPhrInfo.dCreateStatus & (UINT32)MAPHR_MASK_PHRASE)	||
		(gPhrInfo.sdSeqId < 0L)) {
		return (-1);
	}
	/*---	check channel ---------------------------------------------------*/
	if((nCh < 0) || (MAPHR_MAX_CHANNEL <= nCh))	return (-1);

	/*---	check status (Is status PAUSE?) ---------------------------------*/
	pAi = &(gPhrInfo.sApiInfo[nCh]);
	if(pAi->Status != MAPHR_STATUS_PAUSE)		return (-1);

	/*---	is slave ? ------------------------------------------------------*/
	if(gPhrInfo.dSlave & (UINT32)(1L << nCh))	return (-1);

	/*---	restart playback ------------------------------------------------*/
	pAi->Status = MAPHR_STATUS_PLAY;
	if(gPhrInfo.dPhrStatus & gdMaskPause[nCh])
		gPhrInfo.dPhrStatus &= ~gdMaskPause[nCh];
	else
		gPhrInfo.dPhrStatus |= gdMaskRestart[nCh];

	dSlave = pAi->dSlave;
	for(i = 0; i < MAPHR_MAX_CHANNEL; i++) {
		if(dSlave & (UINT32)(1L << i)) {
			pAi = &(gPhrInfo.sApiInfo[i]);
			if(pAi->Status == MAPHR_STATUS_PAUSE) {
				pAi->Status = MAPHR_STATUS_PLAY;
				if(gPhrInfo.dPhrStatus & gdMaskPause[i])
					gPhrInfo.dPhrStatus &= ~gdMaskPause[i];
				else
					gPhrInfo.dPhrStatus |= gdMaskRestart[i];
			}
		}
	}

	return (0);
}

/****************************************************************************
 *	Phrase_Stop
 *
 *	Function:
 *			stop playback.
 *	Argument:
 *			nCh		Sequence Number(0..3)
 *	Return:
 *			0	success.
 *			< 0	error.
 *
 ****************************************************************************/
int	Phrase_Stop(int nCh)
{
	UINT32			i;
#if MA_STOPWAIT
	UINT32			dMask;
	SINT32			sdTimeOut;
#else
	UINT32			dCh;
#endif
	UINT32			dSlave;
	PMAPHRAPIINFO	pAi;

	MAPHRCNV_DBGMSG(("Phrase_Stop[%d] \n", nCh));

	/*---	check SMAF/Phrase stream converter ID ---------------------------*/
	if(!(gPhrInfo.dCreateStatus & (UINT32)MAPHR_MASK_PHRASE)	||
		(gPhrInfo.sdSeqId < 0L)) {
		return (-1);
	}
	/*---	check channel ---------------------------------------------------*/
	if((nCh < 0) || (MAPHR_MAX_CHANNEL <= nCh))	return (-1);

	/*---	check status ----------------------------------------------------*/
	pAi = &(gPhrInfo.sApiInfo[nCh]);
	if(	(pAi->Status != MAPHR_STATUS_PLAY)	&&
		(pAi->Status != MAPHR_STATUS_PAUSE)) {
		return (-1);
	}

	/*---	is slave ? ------------------------------------------------------*/
	if(gPhrInfo.dSlave & (1L << nCh))			return (-1);

	/*---	stop playback ---------------------------------------------------*/
#if MA_STOPWAIT
	pAi->Status			= MAPHR_STATUS_ENDING;
	dMask				= gdMaskStop[nCh];
	gPhrInfo.dPhrStatus	|= gdMaskStop[nCh];
#else
	gPhrInfo.dPhrStatus &=
			~(gdMaskStart[nCh] | gdMaskPause[nCh] | gdMaskRestart[nCh]);

	/* Mute */
	dCh = (UINT32)(nCh << 2L);
	CnvAllSoundOff(dCh);
	CnvAllSoundOff((UINT32)(dCh + 1L));
	CnvAllSoundOff((UINT32)(dCh + 2L));
	CnvAllSoundOff((UINT32)(dCh + 3L));

	gPhrInfo.dPlayStatus	&= ~gdMaskPlay[nCh];
	pAi->Status				= MAPHR_STATUS_READY;

	/* seek to top */
	if(pAi->sDataInfo.dVersion == (UINT32)MAPHR_VERSION_LV2)
		Seek2((UINT32)nCh, 0L);
	else
		Seek1((UINT32)nCh, 0L);
#endif

	dSlave = pAi->dSlave;
	for(i = 0; i < MAPHR_MAX_CHANNEL; i++) {
		if(dSlave & (UINT32)(1L << i)) {
			pAi = &(gPhrInfo.sApiInfo[i]);
#if MA_STOPWAIT
			pAi->Status			= MAPHR_STATUS_ENDING;
			dMask				|= gdMaskStop[i];
			gPhrInfo.dPhrStatus	|= gdMaskStop[i];
#else
			gPhrInfo.dPhrStatus &=
					 ~(gdMaskStart[i] | gdMaskPause[i] | gdMaskRestart[i]);

			/* Mute */
			dCh = (UINT32)(i << 2L);
			CnvAllSoundOff(dCh);
			CnvAllSoundOff((UINT32)(dCh + 1L));
			CnvAllSoundOff((UINT32)(dCh + 2L));
			CnvAllSoundOff((UINT32)(dCh + 3L));

			gPhrInfo.dPlayStatus	&= ~gdMaskPlay[i];
			pAi->Status				= MAPHR_STATUS_READY;

			/* seek to top */
			if(pAi->sDataInfo.dVersion == (UINT32)MAPHR_VERSION_LV2)
				Seek2(i, 0L);
			else
				Seek1(i, 0L);
#endif
		}
	}

#if MA_STOPWAIT
	sdTimeOut = MAPHR_WAIT_TIMEOUT;	/*	1000ms	*/
	while((gPhrInfo.dPhrStatus & dMask) && (sdTimeOut > 0)) {
		machdep_Sleep(5);
		sdTimeOut -= 5L;
	}
#endif

	return (0);
}

/****************************************************************************
 *	Phrase_RemoveData
 *
 *	Function:
 *			Unload SMAF/Phrase data.
 *	Argument:
 *			nCh		Sequence Number(0..3)
 *	Return:
 *			0	success.
 *			< 0	error.
 *
 ****************************************************************************/
int	Phrase_RemoveData(int nCh)
{
	UINT32			i;
	SINT32			sdMCh;
	UINT32			dSlave;
	PMAPHRAPIINFO	pAi;

	MAPHRCNV_DBGMSG(("_Phrase_RemoveData[%d] \n", nCh));

	/*---	check SMAF/Phrase stream converter ID ---------------------------*/
	if(!(gPhrInfo.dCreateStatus & (UINT32)MAPHR_MASK_PHRASE)	||
		(gPhrInfo.sdSeqId < 0L)) {
		return (-1);
	}

	/*---	check channel ---------------------------------------------------*/
	if((nCh < 0) || (MAPHR_MAX_CHANNEL <= nCh))	return (-1);

	/*---	check status (Is status READY?) ---------------------------------*/
	pAi = &(gPhrInfo.sApiInfo[nCh]);
	if(pAi->Status != MAPHR_STATUS_READY)		return (-1);

	/*---	free slave setting. ---------------------------------------------*/
	if(gPhrInfo.dSlave & (UINT32)(1L << nCh)) {		/*	slave	*/
		sdMCh	= pAi->sdMasterChannel;
		dSlave	= gPhrInfo.sApiInfo[sdMCh].dSlave & ~((UINT32)(1L << nCh));
		gPhrInfo.sApiInfo[sdMCh].dSlave = dSlave;
		gPhrInfo.dSlave &= ~((UINT32)(1L << nCh));
	} else {								/*	master	*/
		dSlave	= pAi->dSlave;
		for(i = 0; i < MAPHR_MAX_CHANNEL; i++) {
			if(dSlave & (1 << i)) {
				gPhrInfo.sApiInfo[i].sdMasterChannel	= -1L;
				gPhrInfo.sApiInfo[i].dSlave				= 0L;
				gPhrInfo.dSlave &= ~((UINT32)(1L << i));
			}
		}
	}
	pAi->sdMasterChannel	= -1L;
	pAi->dSlave				= 0L;

	/*---	remove all voices!! ----------------------------------------------*/
	RemoveVoices((UINT32)nCh);

	/*---	successful remove data!! -----------------------------------------*/
	pAi->Status = MAPHR_STATUS_NODATA;

	return (0);
}

/****************************************************************************
 *	Phrase_Kill
 *
 *	Function:
 *			Force reset.
 *	Argument:
 *
 *	Return:
 *			0	success.
 *			< 0	error.
 *
 ****************************************************************************/
int	Phrase_Kill(void)
{
	int	i;

	MAPHRCNV_DBGMSG(("Phrase_Kill \n"));

	/*---	check SMAF/Phrase stream converter ID ---------------------------*/
	if(!(gPhrInfo.dCreateStatus & (UINT32)MAPHR_MASK_PHRASE)	||
		(gPhrInfo.sdSeqId < 0L)) {
		return (-1);
	}

	for(i = 0; i < MAPHR_MAX_CHANNEL; i++) {
		Phrase_Stop(i);
	}
	for(i = 0; i < MAPHR_MAX_CHANNEL; i++) {
		Phrase_RemoveData(i);
	}

	return (0);
}

/****************************************************************************
 *	Phrase_SetLink
 *
 *	Function:
 *			Set link.
 *	Argument:
 *			nCh		Seauence Number(0..3)
 *			dSlave	slave
 *	Return:
 *			0	success.
 *			< 0	error.
 *
 ****************************************************************************/
int	Phrase_SetLink(int nCh, unsigned long dSlave)
{
	UINT32			i;
	PMAPHRAPIINFO	pAi;
	PMAPHRAPIINFO	pLi;

	MAPHRCNV_DBGMSG(("Phrase_SetLink [%d %08lX] \n", nCh, dSlave));

	/*---	check SMAF/Phrase stream converter ID ---------------------------*/
	if(!(gPhrInfo.dCreateStatus & (UINT32)MAPHR_MASK_PHRASE)	||
		(gPhrInfo.sdSeqId < 0L)) {
		return (-1);
	}

	/*---	check channel ---------------------------------------------------*/
	if((nCh < 0) || (MAPHR_MAX_CHANNEL <= nCh))	return (-1);

	/*---	check status (Is status READY?) ---------------------------------*/
	pAi = &(gPhrInfo.sApiInfo[nCh]);
	if(pAi->Status != MAPHR_STATUS_READY)		return (-1);

	/*---	is slave ? ------------------------------------------------------*/
	if(gPhrInfo.dSlave & (UINT32)(1L << nCh))	return (-1);

	/*---	check slave	------------------------------------------------------*/
	dSlave	&= ~(1L << nCh);
	for(i = 0; i < MAPHR_MAX_CHANNEL; i++) {
		if(dSlave & (1L << i)) {
			pLi = &(gPhrInfo.sApiInfo[i]);
			if(pLi->dSlave != 0)
				return (-1);	/*	already master	*/
			if(	(pLi->sdMasterChannel != -1L)	&&
				(pLi->sdMasterChannel != (SINT32)nCh))
				return (-1);	/*	already slave of other channel */
			if(pLi->Status != MAPHR_STATUS_READY)
				return (-1);	/*	not ready	*/
			if(pLi->dLength != pAi->dLength)
				return (-1);	/*	not match length	*/
		}
	}

	/*---	free slave ------------------------------------------------------*/
	for(i = 0; i < MAPHR_MAX_CHANNEL; i++) {
		if(pAi->dSlave & (1L << i)) {
			pLi = &(gPhrInfo.sApiInfo[i]);
			pLi->sdMasterChannel	= -1L;
			pLi->dSlave				= 0L;
			gPhrInfo.dSlave			&= ~((UINT32)(1L << i));
		}
	}

	/*---	set slave -------------------------------------------------------*/
	for(i = 0; i < MAPHR_MAX_CHANNEL; i++) {
		if(dSlave & (1L << i)) {
			pLi = &(gPhrInfo.sApiInfo[i]);
			pLi->sdMasterChannel	= (SINT32)nCh;
			pLi->dSlave				= 0L;
			gPhrInfo.dSlave			|= (UINT32)(1L << i);
		}
	}

	pAi->dSlave = dSlave;
	return (0);
}

/****************************************************************************
 *	Phrase_GetLink
 *
 *	Function:
 *			Get link.
 *	Argument:
 *			nCh		Seauence Number(0..3)
 *	Return:
 *			link information.
 *
 ****************************************************************************/
unsigned long	Phrase_GetLink(int nCh)
{
	MAPHRCNV_DBGMSG(("Phrase_GetLink [%d] \n", nCh));

	/*---	check SMAF/Phrase stream converter ID ---------------------------*/
	if(!(gPhrInfo.dCreateStatus & (UINT32)MAPHR_MASK_PHRASE)	||
		(gPhrInfo.sdSeqId < 0L)) {
		return (0);
	}

	/*---	check channel ---------------------------------------------------*/
	if((nCh < 0) || (MAPHR_MAX_CHANNEL <= nCh)) return (0);

	return (gPhrInfo.sApiInfo[nCh].dSlave);
}

/****************************************************************************
 *	Phrase_SetVolume
 *
 *	Function:
 *			Set panpot volume.
 *	Argument:
 *			nCh		Seauence Number(0..3)
 *			bVol	Volume(-1..127)
 *	Return:
 *
 *
 ****************************************************************************/
void	Phrase_SetVolume(int nCh, int nVol)
{
	MAPHRCNV_DBGMSG(("Phrase_SetVolume [%d %d] \n", nCh, nVol));

	/*---	check SMAF/Phrase stream converter ID ---------------------------*/
	if(!(gPhrInfo.dCreateStatus & (UINT32)MAPHR_MASK_PHRASE)	||
		(gPhrInfo.sdSeqId < 0L)) {
		return;
	}

	/*---	check channel ---------------------------------------------------*/
	if((nCh < 0) || (MAPHR_MAX_CHANNEL <= nCh)) return;

	if(nVol == -1)	{
		gPhrInfo.sApiInfo[nCh].dVolume = 100L;	/*	default volume	*/
	}
	else if((0 <= nVol) && (nVol < 128)) {
		gPhrInfo.sApiInfo[nCh].dVolume = (UINT32)nVol;
	}

	gPhrInfo.dCtrlStatus |= (UINT32)(MAPHR_CTRL_VOLUME << (nCh << 2));
}

/****************************************************************************
 *	Phrase_GetVolume
 *
 *	Function:
 *			Get volume.
 *	Argument:
 *			nCh	Seauence Number(0..3)
 *	Return:
 *			>= 0	volume.
 *			< 0		error.
 *
 ****************************************************************************/
int		Phrase_GetVolume(int nCh)
{
	MAPHRCNV_DBGMSG(("Phrase_GetVolume [%d] \n", nCh));

	/*---	check SMAF/Phrase stream converter ID ---------------------------*/
	if(!(gPhrInfo.dCreateStatus & (UINT32)MAPHR_MASK_PHRASE)	||
		(gPhrInfo.sdSeqId < 0L)) {
		return (-1);
	}

	/*---	check channel ---------------------------------------------------*/
	if((nCh < 0) || (MAPHR_MAX_CHANNEL <= nCh)) return (-1);

	return (int)(gPhrInfo.sApiInfo[nCh].dVolume);
}

/****************************************************************************
 *	Phrase_SetPanpot
 *
 *	Function:
 *			Set panpot volume.
 *	Argument:
 *			nCh		Seauence Number(0..3)
 *			bPan	Panpot volume(-1..127)
 *	Return:
 *
 *
 ****************************************************************************/
void	Phrase_SetPanpot(int nCh, int nPan)
{
	MAPHRCNV_DBGMSG(("Phrase_SetPanpot [%d %d] \n", nCh, nPan));

	/*---	check SMAF/Phrase stream converter ID ---------------------------*/
	if(!(gPhrInfo.dCreateStatus & (UINT32)MAPHR_MASK_PHRASE)	||
		(gPhrInfo.sdSeqId < 0L)) {
		return;
	}

	/*---	check channel ---------------------------------------------------*/
	if((nCh < 0) || (MAPHR_MAX_CHANNEL <= nCh)) return;

	if(nPan == -1)	{
		gPhrInfo.sApiInfo[nCh].dPan = 64L;	/*	default volume	*/
	}
	else if((0 <= nPan) && (nPan < 128)) {
		gPhrInfo.sApiInfo[nCh].dPan = (UINT32)nPan;
	}

	gPhrInfo.dCtrlStatus |= (UINT32)(MAPHR_CTRL_PANPOT << (nCh << 2));
}

/****************************************************************************
 *	Phrase_GetPanpot
 *
 *	Function:
 *			Get ppanpot volume.
 *	Argument:
 *			nCh	Seauence Number(0..3)
 *	Return:
 *			>= 0	panpot volume.
 *			< 0		error.
 *
 ****************************************************************************/
int		Phrase_GetPanpot(int nCh)
{
	MAPHRCNV_DBGMSG(("Phrase_GetPanpot [%d] \n", nCh));

	/*---	check SMAF/Phrase stream converter ID ---------------------------*/
	if(!(gPhrInfo.dCreateStatus & (UINT32)MAPHR_MASK_PHRASE)	||
		(gPhrInfo.sdSeqId < 0L)) {
		return (-1);
	}

	/*---	check channel ---------------------------------------------------*/
	if((nCh < 0) || (MAPHR_MAX_CHANNEL <= nCh)) return (-1);

	return (int)(gPhrInfo.sApiInfo[nCh].dPan);
}

/****************************************************************************
 *	Phrase_GetPosition
 *
 *	Function:
 *			Get playback positione[ms].
 *	Argument:
 *			nCh	Seauence Number(0..3)
 *	Return:
 *			>= 0	playback positione[ms].
 *			< 0		error.
 *
 ****************************************************************************/
long	Phrase_GetPosition(int nCh)
{
	SINT32			sdPos;
	PMAPHRAPIINFO	pAi;
/*
	MAPHRCNV_DBGMSG(("Phrase_GetPosition [%d] \n", nCh));
*/
	/*---	check SMAF/Phrase stream converter ID ---------------------------*/
	if(!(gPhrInfo.dCreateStatus & (UINT32)MAPHR_MASK_PHRASE)	||
		(gPhrInfo.sdSeqId < 0L)) {
		return (-1);
	}

	/*---	check channel ---------------------------------------------------*/
	if((nCh < 0) || (MAPHR_MAX_CHANNEL <= nCh)) return (-1);

	/*---	check status ----------------------------------------------------*/
	pAi = &(gPhrInfo.sApiInfo[nCh]);
	if(	(pAi->Status != MAPHR_STATUS_PLAY)	&&
		(pAi->Status != MAPHR_STATUS_PAUSE)) {
		return (-1);
	}

	sdPos = (SINT32)(pAi->sDataInfo.dTimer * MAPHR_PHRASE_TIMEBASE);
	if(sdPos > (SINT32)pAi->dLength) {
		sdPos = (SINT32)pAi->dLength;
	}

	return (long)(sdPos);
}

/****************************************************************************
 *	Phrase_GetLength
 *
 *	Function:
 *			Get playback time[ms].
 *	Argument:
 *			nCh	Seauence Number(0..3)
 *	Return:
 *			>= 0	playback time[ms].
 *			< 0		error.
 *
 ****************************************************************************/
long	Phrase_GetLength(int nCh)
{
	PMAPHRAPIINFO	pAi;

	MAPHRCNV_DBGMSG(("Phrase_GetLength [%d] \n", nCh));

	/*---	check SMAF/Phrase stream converter ID ---------------------------*/
	if(!(gPhrInfo.dCreateStatus & (UINT32)MAPHR_MASK_PHRASE)	||
		(gPhrInfo.sdSeqId < 0L)) {
		return (-1);
	}

	/*---	check channel ---------------------------------------------------*/
	if((nCh < 0) || (MAPHR_MAX_CHANNEL <= nCh)) return (-1);

	/*---	check status ----------------------------------------------------*/
	pAi = &(gPhrInfo.sApiInfo[nCh]);
	if(pAi->Status < MAPHR_STATUS_READY)	return (-1);

	return (long)(pAi->dLength);
}

/****************************************************************************
 *	Phrase_GetStatus
 *
 *	Function:
 *			Get status.
 *	Argument:
 *			nCh	Seauence Number(0..3)
 *	Return:
 *			>= 0	status.
 *			< 0		error.
 *
 ****************************************************************************/
int		Phrase_GetStatus(int nCh)
{
	int	nStatus = 0;

	MAPHRCNV_DBGMSG(("Phrase_GetStatus [%d] \n", nCh));

	/*---	check SMAF/Phrase stream converter ID ---------------------------*/
	if(!(gPhrInfo.dCreateStatus & (UINT32)MAPHR_MASK_PHRASE)	||
		(gPhrInfo.sdSeqId < 0L)) {
		return (-1);
	}

	/*---	check channel ---------------------------------------------------*/
	if((nCh < 0) || (MAPHR_MAX_CHANNEL <= nCh)) return (-1);

	/*---	is slave ? ------------------------------------------------------*/
	if(gPhrInfo.dSlave & (UINT32)(1L << nCh)) nStatus |= (int)0x10;

	switch(gPhrInfo.sApiInfo[nCh].Status) {
	case MAPHR_STATUS_NODATA :
		nStatus |= 1;
		break;
	case MAPHR_STATUS_READY :
		nStatus |= 2;
		break;
	case MAPHR_STATUS_PLAY :
		nStatus |= 3;
		break;
	case MAPHR_STATUS_ENDING :
		nStatus |= 4;
		break;
	case MAPHR_STATUS_PAUSE :
		nStatus |= 5;
		break;
	default:
		break;
	}
	return (nStatus);
}

/****************************************************************************
 *	Phrase_SetEvHandler
 *
 *	Function:
 *			Set callback function.
 *	Argument:
 *			pfnCallback	pointet to the callback function.
 *	Return:
 *			0	success.
 *			< 0	error.
 *
 ****************************************************************************/
int	Phrase_SetEvHandler(void (*pfnCallback)(struct event* pEvent))
{
	MAPHRCNV_DBGMSG(("Phrase_SetEvHandler[%08lX]\n", pfnCallback));

	/*---	check SMAF/Phrase stream converter ID ---------------------------*/
	if(!(gPhrInfo.dCreateStatus & (UINT32)MAPHR_MASK_PHRASE)	||
		(gPhrInfo.sdSeqId < 0L)) {
		return (-1);
	}

	/*--- check parameter ---------------------------------------------------*/
	if(pfnCallback == NULL)	return (-1);

	/*--- regist function ---------------------------------------------------*/
	gPhrInfo.pfnEvHandler = pfnCallback;
	return (0);
}

/****************************************************************************
 *	Phrase_Terminate
 *
 *	Function:
 *			Terminate SMAF/Phrase sequencer.
 *	Argument:
 *
 *	Return:
 *			0	success.
 *			< 0	error.
 *
 ****************************************************************************/
int	Phrase_Terminate(void)
{
	UINT32	i;

	MAPHRCNV_DBGMSG(("Phrase_Terminate\n"));

	/*---	check SMAF/Phrase stream converter ID ---------------------------*/
	if(!(gPhrInfo.dCreateStatus & (UINT32)MAPHR_MASK_PHRASE)	||
		(gPhrInfo.sdSeqId < 0L)) {
		return (-1);
	}

	Phrase_Kill();

	gPhrInfo.dCreateStatus &= ~((UINT32)MAPHR_MASK_PHRASE);
	if(gPhrInfo.dCreateStatus == 0L) {
		ResourceFree();
	}

	/*---	Initialize global values -----------------------------------------*/
	for(i = 0; i < MAPHR_MAX_SLOT; i++) InitApiInfo(i);

	return (0);
}

/****************************************************************************
 *	CnvSequence1
 *
 *	Function:
 *			Convert sequence data to native format(for LV1).
 *	Argument:
 *			dPhrCh	Sequence Number(0..3)
 *	Return:
 *			Number of converted.
 *
 ****************************************************************************/
static SINT32	CnvSequence1(UINT32 dPhrCh)
{
	UINT8*			pbExMsg;
	UINT32			dCh;
	UINT32			dTmp;
	UINT32			dData;
	UINT32			dCurrent;
	UINT32			dDuration;
	SINT32			sdRet;
	PMALIBPHRINFO	pDi;

	/*	Convert Sequence for LV1	*/
	sdRet	= 0L;
	pDi		= &(gPhrInfo.sApiInfo[dPhrCh].sDataInfo);
	while(pDi->dSeqSize > pDi->dDataPosition) {
		/*	Get Current Time	*/
		dCurrent	= pDi->dDataPosition;
		dDuration	= (UINT32)pDi->pbSeq[dCurrent++];
		if(dDuration & 0x80L) {
			dDuration = (UINT32)(((dDuration & 0x7FL) << 7) +
								 (UINT32)pDi->pbSeq[dCurrent++] + 128L);
		}

		if((pDi->dCurrentTime + dDuration) > pDi->dTimer) {
			return (sdRet);
		}

		/*	Update position & time	*/
		pDi->dDataPosition	= dCurrent;
		pDi->dCurrentTime	+= dDuration;

		dData = (UINT32)pDi->pbSeq[pDi->dDataPosition++];
		switch(dData) {
		case 0x00L :
			dData	= pDi->pbSeq[pDi->dDataPosition++];
			dCh		= (UINT32)(((dData & 0xC0L) >> 6) + (dPhrCh << 2));
			dTmp	= (UINT32)(dData & 0x0FL);
			switch(dData & 0x30L) {
			case 0x00L :	/*	Short type Volume	*/
				sdRet += CnvChannelVolume(dCh, gdShortVolTable[dTmp]);
				break;
			case 0x10L : /*	Short type PitchBend	*/
				break;
			case 0x20L :	/*	Short type Modulation	*/
				sdRet += CnvModulation(dCh, gdShortModTable[dTmp]);
				break;
			case 0x30L :
				dData = (UINT32)pDi->pbSeq[pDi->dDataPosition++];
				switch(dTmp) {
				case 0x00L :	/*	Program Change	*/
					sdRet += CnvProgramChange(dCh, (UINT32)(dData & 0x03L));
					break;
				case 0x02L :	/*	Octave Shift	*/
					OctaveShift(dCh, dData);
					break;
				case 0x04L :	/*	Pitch Bend	*/
					sdRet += CnvPitchBend(dCh, (UINT32)(dData & 0x7FL));
					break;
				case 0x0AL :	/*	Panpot	*/
					sdRet += CnvPanpot(dCh, (UINT32)(dData & 0x7FL));
					break;
				case 0x0BL :	/*	Normal type Volume	*/
					sdRet += CnvChannelVolume(dCh, (UINT32)(dData & 0x7FL));
					break;
				default	  :
					break;
				}
				break;
			default	 :
				break;
			}
			break;
		case 0xFF :
			dData = (UINT32)pDi->pbSeq[pDi->dDataPosition++];
			if(dData == 0xF0L) {			/*	Exclusive Message	*/
				dTmp	= (UINT32)pDi->pbSeq[pDi->dDataPosition++];
				pbExMsg	= &(pDi->pbSeq[pDi->dDataPosition]);
				sdRet += CnvExclusiveMessage(dPhrCh, pbExMsg, dTmp);
				/*	skip exclusive message	*/
				pDi->dDataPosition += dTmp;
			}
			else if((dData & 0xF0L) == 0x10L) {	/*	User Event	*/
				Callback(dPhrCh, (UINT32)(dData & 0x0FL));
			}
			break;
		default	  :
			dCh = (UINT32)(((dData & 0xC0L) >> 6) + (dPhrCh << 2));

			dDuration = (UINT32)pDi->pbSeq[pDi->dDataPosition++];
			if(dDuration & 0x80L) {
				dDuration = (UINT32)(((dDuration & 0x7FL) << 7) +
						(UINT32)pDi->pbSeq[pDi->dDataPosition++] + 128L);
			}
			if(dDuration != 0L) {
				dDuration += pDi->dCurrentTime;
				sdRet += CnvNoteOn(	dCh,
									(UINT32)((dData >> 4) & 0x03L),
									(UINT32)(dData & 0x0FL),
									dDuration);
			}
			break;
		}
	}

	return (sdRet);
}

/****************************************************************************
 *	CnvSequence2
 *
 *	Function:
 *			Convert sequence data to native format(for LV2).
 *	Argument:
 *			dPhrCh	Sequence Number(0..3)
 *	Return:
 *			Number of converted.
 *
 ****************************************************************************/
static SINT32	CnvSequence2(UINT32 dPhrCh)
{
	UINT32			dCh;
	UINT32			dTmp;
	UINT32			dData;
	UINT32			dCurrent;
	UINT32			dDuration;
	SINT32			sdRet;
	PMALIBPHRINFO	pDi;

	/*	Convert Sequence for LV2	*/
	sdRet	= 0L;
	pDi		= &(gPhrInfo.sApiInfo[dPhrCh].sDataInfo);
	while(pDi->dSeqSize > pDi->dDataPosition) {
		/*	Get Current Time	*/
		dCurrent	= pDi->dDataPosition;
		dDuration	= (UINT32)pDi->pbSeq[dCurrent++];
		if(dDuration & 0x80) {
			dDuration = (UINT32)(((dDuration & 0x7FL) << 7) +
								 (UINT32)pDi->pbSeq[dCurrent++] + 128L);
		}

		if((pDi->dCurrentTime + dDuration) > pDi->dTimer) {
			return (sdRet);
		}

		/*	Update position & time	*/
		pDi->dDataPosition	= dCurrent;
		pDi->dCurrentTime	+= dDuration;

		dData = (UINT32)pDi->pbSeq[pDi->dDataPosition++];
		switch(dData) {
		case 0x00L :
			dData	= (UINT32)pDi->pbSeq[pDi->dDataPosition++];
			dCh		= (UINT32)(((dData & 0xC0L) >> 6) + (dPhrCh << 2));
			dTmp	= (UINT32)(dData & 0x0FL);
			switch(dData & 0x30L) {
			case 0x00L :	/*	Short type Volume	*/
				if((0L < dTmp) && (dTmp < 15L)) {
					sdRet += CnvChannelVolume(dCh, gdShortVolTable[dTmp]);
				}
				break;
			case 0x10L :	/*	Short type PitchBend	*/
				break;
			case 0x20L :	/*	Short type Modulation	*/
				if((0L < dTmp) && (dTmp < 15L)) {
					sdRet += CnvModulation(dCh, gdShortModTable[dTmp]);
				}
				break;
			case 0x30L :
				dData = (UINT32)pDi->pbSeq[pDi->dDataPosition++];
				switch(dTmp) {
				case 0x00L :	/*	Program Change	*/
					if(dData < 128L) {
						/*	update bank number	*/
						gPhrInfo.sChInfo[dCh].dBankNo = gPhrInfo.sChInfo[dCh].dNewBank;
						sdRet += CnvProgramChange2(dCh, dData);
					}
					break;
				case 0x01L :	/*	Bank Select	*/
					BankSelect(dCh, dData);
					break;
				case 0x02L :	/*	Octave Shift	*/
					OctaveShift2(dCh, dData);
					break;
				case 0x03L :	/*	Normal type Modulation	*/
					if(dData < 128L) {
						dData = gdNormalModTable[dData];
						sdRet += CnvModulation(dCh, dData);
					}
					break;
				case 0x04L :	/*	Pitch Bend	*/
					sdRet += CnvPitchBend(dCh, dData);
					break;
				case 0x05L :	/*	Pitch Bend Range	*/
					PitchBendRange(dCh, dData);
					break;
				case 0x06L :	/*	Velocity	*/
					Velocity(dCh, dData);
					break;
				case 0x0AL :	/*	Panpot	*/
					sdRet += CnvPanpot(dCh, dData);
					break;
				case 0x0BL :	/*	Normal type Volume	*/
					sdRet += CnvChannelVolume(dCh, dData);
					break;
				default	  :
					break;
				}
				break;
			default	 :
				break;
			}
			break;
		case 0xFFL :
			dData = (UINT32)pDi->pbSeq[pDi->dDataPosition++];
			if(dData == 0xF0L) {				/*	Exclusive Message	*/;
				/*	skip exclusive message	*/
				pDi->dDataPosition += (UINT32)pDi->pbSeq[pDi->dDataPosition++];
			}
			else if((dData & 0xF0L) == 0x10L) {	/*	User Event	*/
				Callback(dPhrCh, (UINT32)(dData & 0x0FL));
			}
			break;
		default	  :
			dCh = (UINT32)(((dData & 0xC0L) >> 6) + (dPhrCh << 2));

			dDuration = (UINT32)pDi->pbSeq[pDi->dDataPosition++];
			if(dDuration & 0x80L) {
				dDuration = (UINT32)(((dDuration & 0x7FL) << 7) +
						(UINT32)pDi->pbSeq[pDi->dDataPosition++] + 128L);
			}
			if(dDuration != 0L) {
				dDuration += pDi->dCurrentTime;
				sdRet += CnvNoteOn2(dCh,
									(UINT32)((dData >> 4) & 0x03L),
									(UINT32)(dData & 0x0FL),
									dDuration);
			}
			break;
		}
	}

	return (sdRet);
}

/****************************************************************************
 *	Phrase_Convert
 *
 *	Function:
 *			Convert to native format.
 *	Argument:
 *
 *	Return:
 *			Number of converted.
 *
 ****************************************************************************/
static SINT32	Phrase_Convert(void)
{
	UINT32			i;
	UINT32			j;
	UINT32			dCh;
	UINT32			dContinue;
	UINT32			dMask;
	UINT32			dStatus;
	SINT32			sdRet;
	PMAPHRAPIINFO	pAi;
	PMALIBPHRINFO	pDi;

	dStatus				= gPhrInfo.dPhrStatus;
	gPhrInfo.dPhrStatus	= 0L;
	sdRet				= 0L;

	/* control start/stop/pause/restart */
	if(dStatus) {
		/* Start */
		if(dStatus & 0x00001111) {
			for(i = 0; i < MAPHR_MAX_CHANNEL; i++) {
				if(dStatus & gdMaskStart[i]) {
					gPhrInfo.dPlayStatus |= gdMaskPlay[i];
					sdRet += PutInitPhraseMessage(i);
				}
			}
		}
		/* Restart */
		if(dStatus & 0x00008888) {
			for(i = 0; i < MAPHR_MAX_CHANNEL; i++) {
				if(dStatus & gdMaskRestart[i]) {
					gPhrInfo.dPlayStatus |= gdMaskPlay[i];
				}
			}
		}
		/* Pause */
		if(dStatus & 0x00004444) {
			for(i = 0; i < MAPHR_MAX_CHANNEL; i++) {
				if(dStatus & gdMaskPause[i]) {
					/* Mute */
					dCh = (UINT32)(i << 2);
					CnvAllSoundOff(dCh);
					CnvAllSoundOff((UINT32)(dCh + 1L));
					CnvAllSoundOff((UINT32)(dCh + 2L));
					CnvAllSoundOff((UINT32)(dCh + 3L));
					gPhrInfo.dPlayStatus &= ~gdMaskPlay[i];
					sdRet += 4L;
				}
			}
		}
		/* Stop */
		if(dStatus & 0x00002222) {
#if MA_STOPWAIT
			for(i = 0; i < MAPHR_MAX_CHANNEL; i++) {
				if(dStatus & gdMaskStop[i]) {
					/* Mute */
					dCh = (UINT32)(i << 2);
					CnvAllSoundOff(dCh);
					CnvAllSoundOff((UINT32)(dCh + 1L));
					CnvAllSoundOff((UINT32)(dCh + 2L));
					CnvAllSoundOff((UINT32)(dCh + 3L));
					gPhrInfo.dPlayStatus		&= ~gdMaskPlay[i];
					gPhrInfo.sApiInfo[i].Status	= MAPHR_STATUS_READY;
					sdRet += 4L;

					/* seek to top */
					if(gPhrInfo.sApiInfo[i].sDataInfo.dVersion == (UINT32)MAPHR_VERSION_LV2)
						Seek2(i, 0L);
					else
						Seek1(i, 0L);
				}
			}
#endif
		}
	}

	/*	No played Phrase	*/
	if((gPhrInfo.dPlayStatus & 0x0FL) == 0x00L) {
		return (sdRet);
	}

	/*	Convert	*/
	dContinue = gPhrInfo.dPlayStatus;
	for(i = 0; i < MAPHR_MAX_CHANNEL; i++) {
		if(!(gPhrInfo.dPlayStatus & (UINT32)(1L << i)))
			continue;

		pAi	= &(gPhrInfo.sApiInfo[i]);
		pDi	= &(pAi->sDataInfo);
		if(gPhrInfo.dCtrlStatus) {
			/*	Expression	*/
			dMask = (UINT32)(MAPHR_CTRL_VOLUME << (i << 2));
			if(gPhrInfo.dCtrlStatus & dMask) {
				dCh = (UINT32)(i << 2);
				sdRet += CnvExpression(dCh, pAi->dVolume);
				sdRet += CnvExpression((UINT32)(dCh + 1L), pAi->dVolume);
				sdRet += CnvExpression((UINT32)(dCh + 2L), pAi->dVolume);
				sdRet += CnvExpression((UINT32)(dCh + 3L), pAi->dVolume);
				gPhrInfo.dCtrlStatus &= ~dMask;
			}
			/*	Panpot	*/
			dMask = (UINT32)(MAPHR_CTRL_PANPOT << (i << 2));
			if(gPhrInfo.dCtrlStatus & dMask) {
				dCh = (UINT32)(i << 2);
				MaSndDrv_SetCommand(gPhrInfo.sdSeqId, -1L, MASNDDRV_CMD_PANPOT,
									dCh, pAi->dPan, 0L);
				MaSndDrv_SetCommand(gPhrInfo.sdSeqId, -1L, MASNDDRV_CMD_PANPOT,
									(UINT32)(dCh + 1L), pAi->dPan, 0L);
				MaSndDrv_SetCommand(gPhrInfo.sdSeqId, -1L, MASNDDRV_CMD_PANPOT,
									(UINT32)(dCh + 2L), pAi->dPan, 0L);
				MaSndDrv_SetCommand(gPhrInfo.sdSeqId, -1L, MASNDDRV_CMD_PANPOT,
									(UINT32)(dCh + 3L), pAi->dPan, 0L);
				gPhrInfo.dCtrlStatus &= ~dMask;
				sdRet += 4L;
			}
		}

		/*	Send Note Off	*/
		for(j = 0; j < MAPHR_MAX_CHANNEL; j++) {
			dCh = (UINT32)(j + (i << 2));
			if(gPhrInfo.sChInfo[dCh].dNoteOffTime <= pDi->dTimer) {
				sdRet += CnvNoteOff(dCh);
			}
		}

		/*	Sequence */
		if(pDi->dVersion == (UINT32)MAPHR_VERSION_LV2)
			sdRet += CnvSequence2(i);
		else
			sdRet += CnvSequence1(i);

		/*	increment timer	*/
		pDi->dTimer++;

		/*	End of Data	*/
		if(pDi->dTimer > pDi->dPlayTime) {
			dContinue &= ~(UINT32)(1L << i);
		}
	}

	/*	End of Phrase	*/
	if(dContinue == gPhrInfo.dPlayStatus) {
		return (sdRet);
	}

	/* exist end phrase */
	for(i = 0; i < MAPHR_MAX_CHANNEL; i++) {
		if(	!(gPhrInfo.dPlayStatus & gdMaskPlay[i])	||
			(dContinue & gdMaskPlay[i])) {
			continue;
		}

		MAPHRCNV_DBGMSG(("Phrase%d data end!! \n", i));
		pAi = &(gPhrInfo.sApiInfo[i]);
		pDi	= &(pAi->sDataInfo);

		/*---	stop playback ------------------------------------------------*/
		/* Note Off */
		dCh = (UINT32)(i << 2);
		sdRet += CnvAllNoteOff(dCh);
		sdRet += CnvAllNoteOff((UINT32)(dCh + 1L));
		sdRet += CnvAllNoteOff((UINT32)(dCh + 2L));
		sdRet += CnvAllNoteOff((UINT32)(dCh + 3L));
		gPhrInfo.dPlayStatus &= ~gdMaskPlay[i];

		/* Seek to top */
		if(pDi->dVersion == (UINT32)MAPHR_VERSION_LV2)
			Seek2(i, 0L);
		else
			Seek1(i, 0L);

		if(pAi->dLoopCount != 1L) {	/*	repeat	*/
			if(pAi->dLoopCount != 0L) pAi->dLoopCount--;
			Callback(i, (UINT32)MAPHR_REPEAT);
			/* Start */
			gPhrInfo.dPlayStatus |= gdMaskPlay[i];
			sdRet += PutInitPhraseMessage(i);
		}
		else {					/*	one short	*/
			pAi->Status = MAPHR_STATUS_READY;
			Callback(i, (UINT32)MAPHR_END_OF_SEQUENCE);
		}
	}

	return (sdRet);
}

/****************************************************************************
 *	MaAudCnv_Initialize
 *
 *	Function:
 *			Initialize SMAF/Audio stream converter.
 *	Argument:
 *
 *	Return:
 *			0	success.
 *			< 0	error code.
 *
 ****************************************************************************/
SINT32	MaAudCnv_Initialize(void)
{
	MAAUDCNV_DBGMSG(("MaAudCnv_Initialize \n"));

	gPhrInfo.dAudStatus	= 0x00000000;
	gPhrInfo.dVolume	= 100L;
	gPhrInfo.dPanpot	= 64L;
	gPhrInfo.dPause		= 0L;
	gPhrInfo.sAudInfo[0].dLoaded = 0L;
	gPhrInfo.sAudInfo[1].dLoaded = 0L;

	return (MASMW_SUCCESS);
}

/****************************************************************************
 *	MaAudCnv_End
 *
 *	Function:
 *			Free SMAF/Audio stream converter.
 *	Argument:
 *
 *	Return:
 *			0	success.
 *			< 0	error code.
 *
 ****************************************************************************/
SINT32	MaAudCnv_End(void)
{
	MAAUDCNV_DBGMSG(("MaAudCnv_End \n"));

	return (MASMW_SUCCESS);
}

/****************************************************************************
 *	MaAudCnv_Load
 *
 *	Function:
 *			Load SMAF/Audio data.
 *	Argument:
 *			pbFile		Pointer to the SMAF/Audio data.
 *			dFileSize	Size of SMAF/Audio data.
 *			dMode		error check (0:No 1:Yes 2:Check only)
 *			pfnFunc		Pointer to the callback function.
 *			pvExtArgs	NULL
 *	Return:
 *			>=0	file id.
 *			< 0	error code.
 *
 ****************************************************************************/
SINT32	MaAudCnv_Load(UINT8* pbFile, UINT32 dFileSize, UINT32 dMode,
								SINT32 (*pfnFunc)(UINT8 bId), void* pvExtArgs)
{
	SINT32			sdFileId;
	SINT32			sdRet;
	PMALIBAUDINFO	pAi;

	(void)pfnFunc;
	(void)pvExtArgs;

	MAAUDCNV_DBGMSG(("MaAudCnv_Load[%08lX %08lX %08lX] \n", pbFile,
														dFileSize, dMode));
	/*--- check parameter ---*/
	if(pbFile == NULL)							return (MASMW_ERROR);
	if(dFileSize == 0)							return (MASMW_ERROR);

	sdFileId	= (SINT32)((dMode < 2) ? 1 : 0);
	pAi			= &(gPhrInfo.sAudInfo[sdFileId]);
	if(sdFileId == 0) {
		/*	load OK!!	*/
		pAi->dLoaded = 0L;
	}
	else {
		/*	Already loaded!!	*/
		if(pAi->dLoaded) return (MASMW_ERROR);
	}

	/*	Format check	*/
	sdRet = malib_smafaudio_checker(pbFile, dFileSize, (UINT8)dMode, pAi);
	if(sdRet < MASMW_SUCCESS) return (sdRet);

	pAi->dPlayTime	= (UINT32)sdRet;
	pAi->dLoaded	= 1L;
	return (sdFileId);
}

/****************************************************************************
 *	MaAudCnv_Unload
 *
 *	Function:
 *			Unload SMAF/Audio data.
 *	Argument:
 *			sdFileId	file id(0..1)
 *			pvExtArgs	NULL
 *	Return:
 *			0	success.
 *			< 0	error code.
 *
 ****************************************************************************/
SINT32	MaAudCnv_Unload(SINT32 sdFileId, void* pvExtArgs)
{
	(void)pvExtArgs;

	MAAUDCNV_DBGMSG(("MaAudCnv_Unload[%d] \n", sdFileId));

	if(	(sdFileId < 0)							||
		(sdFileId >= MAPHR_MAX_AUDIO_DATA)		||
		(gPhrInfo.sAudInfo[sdFileId].dLoaded == 0L)) {
		return (MASMW_ERROR);
	}

	gPhrInfo.sAudInfo[sdFileId].dLoaded = 0L;

	return (MASMW_SUCCESS);
}

/****************************************************************************
 *	MaAudCnv_Open
 *
 *	Function:
 *			Allocate playback resource.
 *	Argument:
 *			sdFileId	file id(0..1)
 *			pvExtArgs	NULL
 *	Return:
 *			0	success.
 *			< 0	error code.
 *
 ****************************************************************************/
SINT32	MaAudCnv_Open(SINT32 sdFileId, void* pvExtArgs)
{
	SINT32	sdRet;

	(void)pvExtArgs;

	MAAUDCNV_DBGMSG(("MaAudCnv_Open[%d] \n", sdFileId));

	if(	(sdFileId < 1)							||
		(sdFileId >= MAPHR_MAX_AUDIO_DATA)		||
		(gPhrInfo.sAudInfo[sdFileId].dLoaded == 0L)) {
		return (MASMW_ERROR);
	}

	if(gPhrInfo.dCreateStatus == 0L) {
		sdRet = ResourceCreate();
		if(sdRet < MASMW_SUCCESS) return (sdRet);
	}

	gPhrInfo.dCreateStatus	|= (UINT32)MAPHR_MASK_AUDIO;
	gPhrInfo.dAudStatus		= 0x00000000;
	gPhrInfo.dVolume		= 100L;
	gPhrInfo.dPanpot		= 64L;
	gPhrInfo.dPause			= 0L;
	gPhrInfo.sAudInfo[sdFileId].dTimer = 0L;

	gPhrInfo.dCtrlStatus	|= (MAPHR_CTRL_VOLUME << (MAPHR_AUDIO_CNV_ID << 2));

	return (MASMW_SUCCESS);
}

/****************************************************************************
 *	MaAudCnv_Close
 *
 *	Function:
 *			free playback resource.
 *	Argument:
 *			sdFileId	file id(0..1)
 *			pvExtArgs	NULL
 *	Return:
 *			0	success.
 *			< 0	error code.
 *
 ****************************************************************************/
SINT32	MaAudCnv_Close(SINT32 sdFileId, void* pvExtArgs)
{
	(void)pvExtArgs;

	MAAUDCNV_DBGMSG(("MaAudCnv_Open[%d] \n", sdFileId));

	if(	(sdFileId < 1)							||
		(sdFileId >= MAPHR_MAX_AUDIO_DATA)		||
		(gPhrInfo.sAudInfo[sdFileId].dLoaded == 0L)) {
		return (MASMW_ERROR);
	}

	gPhrInfo.dCreateStatus &= ~((UINT32)MAPHR_MASK_AUDIO);
	if(gPhrInfo.dCreateStatus == 0L) {
		ResourceFree();
	}

	return (MASMW_SUCCESS);
}

/****************************************************************************
 *	MaAudCnv_Standby
 *
 *	Function:
 *			standby playback.
 *	Argument:
 *			sdFileId	file id(0..1)
 *			pvExtArgs	NULL
 *	Return:
 *			0	success.
 *			< 0	error code.
 *
 ****************************************************************************/
SINT32	MaAudCnv_Standby(SINT32 sdFileId, void* pvExtArgs)
{
	PMALIBAUDINFO	pAi;

	(void)pvExtArgs;

	MAAUDCNV_DBGMSG(("MaAudCnv_Standby[%d] \n", sdFileId));

	if(	(sdFileId < 1)							||
		(sdFileId >= MAPHR_MAX_AUDIO_DATA)		||
		(gPhrInfo.sAudInfo[sdFileId].dLoaded == 0L)) {
		return (MASMW_ERROR);
	}

	pAi = &(gPhrInfo.sAudInfo[sdFileId]);
	pAi->dTimer		= 0L;
	pAi->dLoopCount	= 1L;
	return (MaSndDrv_SetStream(gPhrInfo.sdSeqId, 0, pAi->dFmt, pAi->dFs,
										pAi->pbSource, pAi->dSizeOfSource));
}

/****************************************************************************
 *	MaAudCnv_Seek
 *
 *	Function:
 *			seek.
 *	Argument:
 *			sdFileId	file id(0..1)
 *			dPos		start position[ms]
 *			pvExtArgs	NULL
 *	Return:
 *			0	success.
 *			< 0	error code.
 *
 ****************************************************************************/
SINT32	MaAudCnv_Seek(SINT32 sdFileId, UINT32 dPos, void* pvExtArgs)
{
	(void)pvExtArgs;

	MAAUDCNV_DBGMSG(("MaAudCnv_Seek[%08lX] = %ld \n", sdFileId, dPos));

	if(	(sdFileId < 1)							||
		(sdFileId >= MAPHR_MAX_AUDIO_DATA)		||
		(gPhrInfo.sAudInfo[sdFileId].dLoaded == 0L)) {
		return (MASMW_ERROR);
	}

	/*--- No seek on MA-5 ------------------------*/
	if(dPos != 0) return (MASMW_ERROR);
	gPhrInfo.dPause = 0L;
	gPhrInfo.sAudInfo[sdFileId].dTimer = 0L;

	return (MASMW_SUCCESS);
}

/****************************************************************************
 *	MaAudCnv_Start
 *
 *	Function:
 *			start playback.
 *	Argument:
 *			sdFileId	file id(0..1)
 *			pvExtArgs	NULL
 *	Return:
 *			0	success.
 *			< 0	error code.
 *
 ****************************************************************************/
SINT32	MaAudCnv_Start(SINT32 sdFileId, void* pvExtArgs)
{
	(void)pvExtArgs;

	MAAUDCNV_DBGMSG(("MaAudCnv_Start[%d] \n", sdFileId));

	if(	(sdFileId < 1)							||
		(sdFileId >= MAPHR_MAX_AUDIO_DATA)		||
		(gPhrInfo.sAudInfo[sdFileId].dLoaded == 0L)) {
		return (MASMW_ERROR);
	}

	gPhrInfo.dPause = 0L;
	gPhrInfo.dAudStatus |= gdMaskStart[0];

	return (MASMW_SUCCESS);
}

/****************************************************************************
 *	MaAudCnv_Pause
 *
 *	Function:
 *			pause playback.
 *	Argument:
 *			sdFileId	file id(0..1)
 *			pvExtArgs	NULL
 *	Return:
 *			0	success.
 *			< 0	error code.
 *
 ****************************************************************************/
SINT32	MaAudCnv_Pause(SINT32 sdFileId, void* pvExtArgs)
{
	(void)pvExtArgs;

	MAAUDCNV_DBGMSG(("MaAudCnv_Pause[%d] \n", sdFileId));

	if(	(sdFileId < 1)							||
		(sdFileId >= MAPHR_MAX_AUDIO_DATA)		||
		(gPhrInfo.sAudInfo[sdFileId].dLoaded == 0L)) {
		return (MASMW_ERROR);
	}

	if(gPhrInfo.dAudStatus & gdMaskRestart[0]) {
		gPhrInfo.dAudStatus &= ~gdMaskRestart[0];
	}
	else {
		gPhrInfo.dAudStatus |= gdMaskPause[0];
	}
	gPhrInfo.dPause = 1L;

	return (MASMW_SUCCESS);
}

/****************************************************************************
 *	MaAudCnv_Restart
 *
 *	Function:
 *			restart playback.
 *	Argument:
 *			sdFileId	file id(0..1)
 *			pvExtArgs	NULL
 *	Return:
 *			0	success.
 *			< 0	error code.
 *
 ****************************************************************************/
SINT32	MaAudCnv_Restart(SINT32 sdFileId, void* pvExtArgs)
{
	(void)pvExtArgs;

	MAAUDCNV_DBGMSG(("MaAudCnv_Restart[%d] \n", sdFileId));

	if(	(sdFileId < 1)							||
		(sdFileId >= MAPHR_MAX_AUDIO_DATA)		||
		(gPhrInfo.sAudInfo[sdFileId].dLoaded == 0L)) {
		return (MASMW_ERROR);
	}

	if(gPhrInfo.dAudStatus & gdMaskPause[0]) {
		gPhrInfo.dAudStatus &= ~gdMaskPause[0];
	}
	else {
		if(gPhrInfo.dPause)	/*	Normal restart	*/
			gPhrInfo.dAudStatus |= gdMaskRestart[0];
		else			/*	Pause -> Seek -> Restart	*/
			gPhrInfo.dAudStatus |= gdMaskStart[0];
	}
	gPhrInfo.dPause = 0L;

	return (MASMW_SUCCESS);
}

/****************************************************************************
 *	MaAudCnv_Stop
 *
 *	Function:
 *			stop playback.
 *	Argument:
 *			sdFileId	file id(0..1)
 *			pvExtArgs	NULL
 *	Return:
 *			0	success.
 *			< 0	error code.
 *
 ****************************************************************************/
SINT32	MaAudCnv_Stop(SINT32 sdFileId, void* pvExtArgs)
{
	(void)pvExtArgs;

	MAAUDCNV_DBGMSG(("MaAudCnv_Stop[%d %08lX] \n", sdFileId, pvExtArgs));

	if(	(sdFileId < 1)							||
		(sdFileId >= MAPHR_MAX_AUDIO_DATA)		||
		(gPhrInfo.sAudInfo[sdFileId].dLoaded == 0L)) {
		return (MASMW_ERROR);
	}

	/*	stop	*/
	gPhrInfo.dAudStatus = 0L;
	MaSndDrv_SetCommand(gPhrInfo.sdSeqId, -1L,
						MASNDDRV_CMD_STREAM_OFF, 0L, 0L, 0L);
	gPhrInfo.dPlayStatus &= ~gdMaskPlay[MAPHR_AUDIO_CNV_ID];
	gPhrInfo.dPause = 0L;

	return (MASMW_SUCCESS);
}

/****************************************************************************
 *	MaAudCnv_GetPos
 *
 *	Function:
 *			get playback position[ms].
 *	Argument:
 *			sdFileId	file id(0..1)
 *	Return:
 *			0	playback position[ms].
 *			< 0	error code.
 *
 ****************************************************************************/
static SINT32	MaAudCnv_GetPos(SINT32 sdFileId)
{
	SINT32			dLen;
	SINT32			dPos;
	PMALIBAUDINFO	pAi;

	MAAUDCNV_DBGMSG(("MaAudCnv_GetPos[%08lX] \n", sdFileId));

	/* get length of the data */
	pAi = &(gPhrInfo.sAudInfo[sdFileId]);
	dLen = pAi->dPlayTime;

	/* get play position of the data */
	if(pAi->dTimer == 0)
		dPos = 0L;
	else
		dPos = (SINT32)((pAi->dTimer - 1L) * MAPHR_PHRASE_TIMEBASE);

	/* check position */
	if(dPos > dLen) dPos = dLen;

	return (dPos);
}

/****************************************************************************
 *	MaAudCnv_SetVolume
 *
 *	Function:
 *			Set volume.
 *	Argument:
 *			sdFileId	file id(0..1)
 *			bVol		volume(0..127)
 *	Return:
 *			0	success.
 *			< 0	error code.
 *
 ****************************************************************************/
static SINT32	MaAudCnv_SetVolume(SINT32 sdFileId, UINT8 bVol)
{
	(void)sdFileId;

	MAAUDCNV_DBGMSG(("MaAudCnv_SetVolume[%08lX, %d] \n", sdFileId, bVol));

	gPhrInfo.dVolume		= (UINT32)(bVol & 0x7F);
	gPhrInfo.dCtrlStatus	|= (UINT32)(MAPHR_CTRL_VOLUME <<
												 (MAPHR_AUDIO_CNV_ID << 2));

	return (MASMW_SUCCESS);
}

/****************************************************************************
 *	MaAudCnv_SetPanpot
 *
 *	Function:
 *			Set panpot volume.
 *	Argument:
 *			sdFileId	file id(0..1)
 *			bPan		panpot(0..127)
 *	Return:
 *			0	success.
 *			< 0	error code.
 *
 ****************************************************************************/
static SINT32	MaAudCnv_SetPanpot(SINT32 sdFileId, UINT8 bPan)
{
	(void)sdFileId;

	MAAUDCNV_DBGMSG(("MaAudCnv_SetPanpot[%08lX, %d] \n", sdFileId, bPan));

	gPhrInfo.dPanpot = (UINT32)(bPan & 0x7F);

	return (MASMW_SUCCESS);
}

/****************************************************************************
 *	MaAudCnv_Control
 *
 *	Function:
 *
 *	Argument:
 *			sdFileId	file id(0..1)
 *			dCtrlNum	Control command ID
 *			pvPrm		Paramters
 *			pvExtArgs	NULL
 *	Return:
 *			0	success.
 *			< 0	error code.
 *
 ****************************************************************************/
SINT32	MaAudCnv_Control(SINT32 sdFileId, UINT32 dCtrlNum,
											 void* pvPrm, void* pvExtArgs)
{
	MALIBGETCINFO	sCI;
	PMALIBAUDINFO	pAi;

	(void)pvExtArgs;

	MAAUDCNV_DBGMSG(("MaAudCnv_Control[%d, %d] \n", sdFileId, dCtrlNum));

	if(	(sdFileId < 0)							||
		(sdFileId >= MAPHR_MAX_AUDIO_DATA)		||
		(gPhrInfo.sAudInfo[sdFileId].dLoaded == 0L)) {
		return (MASMW_ERROR);
	}

	switch(dCtrlNum) {
	case MASMW_GET_LENGTH :
		if (gPhrInfo.sAudInfo[sdFileId].dPlayTime == 0)
			return (MASMW_ERROR);
		else
			return (gPhrInfo.sAudInfo[sdFileId].dPlayTime);

	case MASMW_GET_CONTENTSDATA :
		MAAUDCNV_DBGMSG(("MaAudCnv_GetContentsInfo[%08lX] \n", sdFileId));
		pAi		= &(gPhrInfo.sAudInfo[sdFileId]);

		sCI.pbCnti		= pAi->pbCnti;
		sCI.dCntiSize	= pAi->dCntiSize;
		sCI.pbOption	= pAi->pbOpda;
		sCI.dOptionSize	= pAi->dOpdaSize;
		sCI.bCode		= ((PMASMW_CONTENTSINFO)pvPrm)->code;
		sCI.bTag[0]		= ((PMASMW_CONTENTSINFO)pvPrm)->tag[0];
		sCI.bTag[1]		= ((PMASMW_CONTENTSINFO)pvPrm)->tag[1];
		sCI.pbBuf		= ((PMASMW_CONTENTSINFO)pvPrm)->buf;
		sCI.dBufSize	= ((PMASMW_CONTENTSINFO)pvPrm)->size;

		return malib_GetContentsData(&sCI);

	case MASMW_SET_VOLUME :
		if(sdFileId < 1) {
			MAAUDCNV_DBGMSG(("MaAudCnv_Control/ File ID Error\n"));
			return (MASMW_ERROR_ID);
		}
		return (MaAudCnv_SetVolume(sdFileId, *((UINT8*)pvPrm)));

	case MASMW_GET_POSITION :
		if(sdFileId < 1) {
			MAAUDCNV_DBGMSG(("MaAudCnv_Control/ File ID Error\n"));
			return (MASMW_ERROR_ID);
		}
		return (MaAudCnv_GetPos(sdFileId));

	case MASMW_SET_PANPOT :
		if(sdFileId < 1) {
			MAAUDCNV_DBGMSG(("MaAudCnv_Control/ File ID Error\n"));
			return (MASMW_ERROR_ID);
		}
		return (MaAudCnv_SetPanpot(sdFileId, *((UINT8*)pvPrm)));

	case MASMW_SET_REPEAT :
		if(sdFileId < 1) {
			MAAUDCNV_DBGMSG(("MaAudCnv_Control/ File ID Error\n"));
			return (MASMW_ERROR_ID);
		}
		MAAUDCNV_DBGMSG(("MASMW_SET_REPEAT [%ld] \n", *((UINT32*)pvPrm)));
		gPhrInfo.sAudInfo[sdFileId].dLoopCount = *((UINT32*)pvPrm);
		return (MASMW_SUCCESS);
	}

	return (MASMW_ERROR);
}

/****************************************************************************
 *	Audio_Convert
 *
 *	Function:
 *			Convert to native format.
 *	Argument:
 *
 *	Return:
 *			Number of converted.
 *
 ****************************************************************************/
static SINT32	Audio_Convert(void)
{
	UINT32			dStatus;
	UINT32			dMask;
	SINT32			sdRet;
	PMALIBAUDINFO	pAi;

	pAi					= &(gPhrInfo.sAudInfo[1]);
	dStatus				= gPhrInfo.dAudStatus;
	gPhrInfo.dAudStatus	= 0L;
	sdRet				= 0L;

	/*	volume	*/
	dMask = (UINT32)(MAPHR_CTRL_VOLUME << (MAPHR_AUDIO_CNV_ID << 2));
	if(gPhrInfo.dCtrlStatus & dMask) {
		MaSndDrv_SetVolume(gPhrInfo.sdSeqId, gPhrInfo.dVolume, 0x04L);
		gPhrInfo.dCtrlStatus &= ~dMask;
	}

	/* control start/stop/pause/restart */
	if(dStatus) {
		/* Start */
		if(dStatus & gdMaskStart[0]) {
			/* Seek to top	*/
			pAi->dTimer = 0L;

			MaSndDrv_SetCommand(gPhrInfo.sdSeqId, -1L,
				MASNDDRV_CMD_STREAM_PANPOT, 0L, gPhrInfo.dPanpot, 0L);
			MaSndDrv_SetCommand(gPhrInfo.sdSeqId, -1L,
				MASNDDRV_CMD_STREAM_ON, 0L, 0L, 127L);
			sdRet += 2L;
			gPhrInfo.dPlayStatus |= gdMaskPlay[MAPHR_AUDIO_CNV_ID];
		}
		/* Restart */
		if(dStatus & gdMaskRestart[0]) {
			MaSndDrv_SetCommand(gPhrInfo.sdSeqId, -1L,
				MASNDDRV_CMD_STREAM_PANPOT, 0L, gPhrInfo.dPanpot, 0L);
			MaSndDrv_SetCommand(gPhrInfo.sdSeqId, -1L,
				MASNDDRV_CMD_STREAM_RESTART, 0L, 0L, 0L);
			gPhrInfo.dPlayStatus |= gdMaskPlay[MAPHR_AUDIO_CNV_ID];
			sdRet += 2L;
		}
		/* Pause */
		if(dStatus & gdMaskPause[0]) {
			MaSndDrv_SetCommand(gPhrInfo.sdSeqId, -1L,
				MASNDDRV_CMD_STREAM_PAUSE, 0L, 0L, 0L);
			gPhrInfo.dPlayStatus &= ~gdMaskPlay[MAPHR_AUDIO_CNV_ID];
			sdRet++;
		}
	}

	if(gPhrInfo.dPlayStatus & gdMaskPlay[MAPHR_AUDIO_CNV_ID]) {
		pAi->dTimer++;	/*	update position	*/

		if(((pAi->dTimer - 1) * MASMW_DIRECT_BASETIME) >= pAi->dPlayTime) {
			/*	Stop	*/
			MaSndDrv_SetCommand(gPhrInfo.sdSeqId, -1L,
						MASNDDRV_CMD_STREAM_OFF, 0L, 0L, 0L);
			sdRet++;
			gPhrInfo.dPlayStatus &= ~gdMaskPlay[MAPHR_AUDIO_CNV_ID];

			if(pAi->dLoopCount != 1L) {	/*	repeat	*/
				if(pAi->dLoopCount != 0L) pAi->dLoopCount--;
				MaSound_ReceiveMessage(gPhrInfo.sdSeqId, 1L, MASMW_REPEAT);
				/*	Start	*/
				gPhrInfo.dAudStatus	|= gdMaskStart[0];
			}
			else {						/*	one shot	*/
				MaSound_ReceiveMessage(gPhrInfo.sdSeqId, 1L, MASMW_END_OF_SEQUENCE);
			}
		}
	}

	return (sdRet);
}

/****************************************************************************
 *	MaPhrAud_Convert
 *
 *	Function:
 *			Convert to native format.
 *	Argument:
 *
 *	Return:
 *			Number of converted.
 *
 ****************************************************************************/
static SINT32	MaPhrAud_Convert(void)
{
	SINT32	sdRet = 0L;

	/*	System-On	*/
	if(gPhrInfo.dCtrlStatus & (UINT32)MAPHR_CTRL_SYSTEM_ON) {
		MaSndDrv_SetCommand(gPhrInfo.sdSeqId, -1L,
								 MASNDDRV_CMD_SYSTEM_ON, 0L, 0L, 0L);
		gPhrInfo.dCtrlStatus &= ~((UINT32)MAPHR_CTRL_SYSTEM_ON);
		sdRet++;
	}
	/*	Master Volume	*/
	if(gPhrInfo.dCtrlStatus & (UINT32)MAPHR_CTRL_MASTER_VOLUME) {
		MaSndDrv_SetCommand(gPhrInfo.sdSeqId, -1L,
								MASNDDRV_CMD_MASTER_VOLUME, 0x7FL, 0L, 0L);
		gPhrInfo.dCtrlStatus &= ~((UINT32)MAPHR_CTRL_MASTER_VOLUME);
		sdRet++;
	}

	if(gPhrInfo.dCreateStatus & (UINT32)MAPHR_MASK_PHRASE) {
		sdRet += Phrase_Convert();
	}
	if(gPhrInfo.dCreateStatus & (UINT32)MAPHR_MASK_AUDIO) {
		sdRet += Audio_Convert();
	}

	if(gPhrInfo.dCreateStatus && (sdRet == 0L)) {
		/*	When no converted data, send NOP as dummy data	*/
		/*	Return 0 makes sequencer to not call Convert	*/
		MaSndDrv_SetCommand(gPhrInfo.sdSeqId, -1L, MASNDDRV_CMD_NOP, 0L, 0L, 0L);
		sdRet++;
	}

	return (sdRet);
}
