/****************************************************************************
 *
 *		Copyright (C) 2002-2003 YAMAHA CORPORATION. All rights reserved.
 *
 *		Module		: maresmgr.c
 *
 *		Description	: MA Resource Manager
 *
 *		Version		: 1.0.3 	2003.01.23
 *
 ****************************************************************************/

#include "maresmgr.h"					/* MA Resource Manager */
#include "mamachdep.h"					/* MA Define machine dependent */



#define MARESMGR_STREAM_INT_POINT	(0)		/* stream irq point  : size x 1/4 */
#define MARESMGR_STREAM_BUF_SIZE	(1)		/* stream buffer size: 1024byte   */
#define MARESMGR_STREAM_WRITE_SIZE	(256)	/* write size (byte) for a int task */

#define MARESMGR_RB_MAP_MASK		(0x000000FF)	/* RAM block */
#define MARESMGR_CH_MAP_MASK		(0x0001FFFF)	/* Channel bit16:Ex Channel */
#define MARESMGR_FM_MAP_MASK		(0xFFFFFFFF)	/* FM voice */
#define MARESMGR_WT_MAP_MASK		(0xFFFFFFFF)	/* WT voice */
#define MARESMGR_SA_MAP_MASK		(0x00000003)	/* Stream Audio */
#define MARESMGR_AL_MAP_MASK		(0x00000001)	/* AL voice */
#define MARESMGR_HV_MAP_MASK		(0x00000001)	/* HV voice */
#define MARESMGR_LD_MAP_MASK		(0x00000001)	/* LED */
#define MARESMGR_MT_MAP_MASK		(0x00000001)	/* Motor */
#define MARESMGR_SQ_MAP_MASK		(0x00000003)	/* Sequencer */
#define MARESMGR_TM_MAP_MASK		(0x0000000F)	/* Timer */
#define MARESMGR_AI_MAP_MASK		(0x00000003)	/* Stream Audio Interrupt */
#define MARESMGR_SI_MAP_MASK		(0x0000003F)	/* Software Interrupt */

#define MA_FM_EXT_MAP_MASK	(0xFFFF0000)	/* FM voice for 2OP */

#define MARESMGR_MAX_WAVEID	(31)			/* maximum value of wave id */
#define MARESMGR_FMMODE_2OP	(0)				/* FM 2OP mode */
#define MARESMGR_FMMODE_4OP	(1)				/* FM 4OP mode */
#define MARESMGR_FMMODE_FREE	(2)				/* FM mode Free */


/* Resource Information */
static MA_RESOURCE_INFO gMaResInfo;
static PMA_RESOURCE_INFO pgMaResInfo = &gMaResInfo;

static const UINT16 wMaRomDrumType[128/16] =
{
					/*           F E D C  B A 9 8  7 6 5 4  3 2 1 0  */
					/* --------+------------------------------------ */
	0x0000,			/*   0- 15 | 0 0 0 0  0 0 0 0  0 0 0 0  0 0 0 0  */
	0x8000,			/*  16- 31 | 1 0 0 0  0 0 0 0  0 0 0 0  0 0 0 0  */
	0xFF5A,			/*  32- 47 | 1 1 1 1  1 1 1 1  0 1 0 1  1 0 1 0  */
	0x0A9F,			/*  48- 63 | 0 0 0 0  1 0 1 0  1 0 0 1  1 1 1 1  */
	0x0000,			/*  64- 79 | 0 0 0 0  0 0 0 0  0 0 0 0  0 0 0 0  */
	0x0000,			/*  80- 95 | 0 0 0 0  0 0 0 0  0 0 0 0  0 0 0 0  */
	0x0000,			/*  96-111 | 0 0 0 0  0 0 0 0  0 0 0 0  0 0 0 0  */
	0x0000,			/* 112-127 | 0 0 0 0  0 0 0 0  0 0 0 0  0 0 0 0  */
};

static const UINT16 wMaRomWaveAdrs[MA_MAX_ROM_WAVE] =
	{ 0x1400, 0x15D6, 0x1BA4, 0x2288, 0x24F4, 0x2B72, 0x34D2 };

static const UINT16 wMaRomDrumKey[61] =
{
	0x0058, 0x001D, 0x003C, 0x0024, 0x0018, 0x0012, 0x0057, 0x3E80, 
	0x005E, 0x2328, 0x0027, 0x2710, 0x2EE0, 0x004A, 0x4A38, 0x0063, 
	0x59D8, 0x1F40, 0x3A98, 0x2710, 0x36B0, 0x30D4, 0x3C8C, 0x3A98, 
	0x4650, 0x3390, 0x5208, 0x3E80, 0x2328, 0x006F, 0x006D, 0x5208, 
	0x0054, 0x36B0, 0x001A, 0x4650, 0x002b, 0x0021, 0x004C, 0x0013, 
	0x0039, 0x0033, 0x002E, 0x004D, 0x0048, 0x0064, 0x006D, 0x0032, 
	0x0030, 0x0031, 0x0030, 0x0045, 0x0049, 0x0044, 0x0050, 0x0010, 
	0x0058, 0x0058, 0x005E, 0x004C, 0x003E
};


/****************************************************************************
 *	MaResMgr_GetResourceInfo
 *
 *	Function:
 *			Gets the currenet mapping of resources.
 *	Argument:
 *			None
 *	Return:
 *			Pointer to the resource information structure.
 *
 ****************************************************************************/
PMA_RESOURCE_INFO MaResMgr_GetResourceInfo( void )
{
	MARESMGR_DBGMSG(("   MaResMgr_GetResourceInfo:\n"));
	
	return pgMaResInfo;
}
/****************************************************************************
 *	MaResMgr_GetDefWaveAddress
 *
 *	Function:
 *			Get internal memory address of specified wave data.
 *	Argument:
 *			dWaveId	wave id number (0..6)
 *	Return:
 *			> 0		internal memory address of specified wave data.
 *			0		no entry
 *			< 0		error code
 *
 ****************************************************************************/
SINT32 MaResMgr_GetDefWaveAddress
(
	UINT32	dWaveId						/* wave id */
)
{
	SINT32	sdResult;
	
	MARESMGR_DBGMSG(("   MaResMgr_GetDefWaveAddress: dWaveId=%d ->", dWaveId));

	if ( dWaveId >= MA_MAX_ROM_WAVE )
	{
		MARESMGR_DBGMSG(("illegal wave id\n"));
		return MASMW_ERROR_ARGUMENT;
	}
	
	sdResult = (SINT32)wMaRomWaveAdrs[dWaveId];

	MARESMGR_DBGMSG((" %04lX\n", sdResult));

	return sdResult;
}
/****************************************************************************
 *	MaResMgr_GetDefVoiceAddress
 *
 *	Function:
 *			Get internal memory address of specified timber.
 *	Argument:
 *			dProg	program number(0..127: melody timbers, 128..255: drum timbers)
 *	Return:
 *			> 0		internal memory address of specified timber.
 *			0		no entry
 *			< 0		error code
 *
 ****************************************************************************/
SINT32 MaResMgr_GetDefVoiceAddress
(
	UINT32	dProg						/* program number */
)
{
	SINT32	sdResult;
	
	MARESMGR_DBGMSG(("   MaResMgr_GetDefVoiceAddress: dProg=%d ->", dProg));
	
	if ( dProg < 128 )
	{
		/* melody */
		sdResult = (SINT32)( MA_NORMAL_ROM_ADDRESS + ( dProg * 16 ) );
	}
	else
	{
		dProg -= 128;

		/* drum */
		if ( ( dProg < MA_MIN_ROM_DRUM ) || ( MA_MAX_ROM_DRUM < dProg ) )
		{
			sdResult = (SINT32)(0);
		}
		else
		{
			sdResult = (SINT32)( MA_DRUM_ROM_ADDRESS + ( ( dProg - MA_MIN_ROM_DRUM ) * 16 ) );
		}
	}
	
	MARESMGR_DBGMSG((" %04lX\n", sdResult));

	return sdResult;
}
/****************************************************************************
 *	MaResMgr_GetDefVoiceSynth
 *
 *	Function:
 *			Get the voice type of specified timber.
 *	Argument:
 *			dProg	program number(0..127: melody timbers, 128..255: drum timbers)
 *	Return:
 *			> 0		voice type of specified timber (1:FM, 2:WT).
 *			0		no entry
 *			< 0		error code
 *
 ****************************************************************************/
SINT32 MaResMgr_GetDefVoiceSynth
(
	UINT32	dProg						/* program number */
)
{
	SINT32	sdResult;
	
	MARESMGR_DBGMSG(("   MaResMgr_GetDefVoiceSynth: dProg=%d ->", dProg));
	
	if ( dProg < 128 )
	{
		sdResult = (SINT32)(1);
	}
	else
	{
		dProg -= 128;

		if ( ( dProg < MA_MIN_ROM_DRUM ) || ( MA_MAX_ROM_DRUM < dProg ) )
		{
			sdResult = (SINT32)(0);
		}
		else
		{
			sdResult = (SINT32)( ( ( wMaRomDrumType[dProg/16] >> (dProg%16) ) & 0x1 ) + 1 );
		}
	}
	
	MARESMGR_DBGMSG((" %lu\n", sdResult));
	
	return sdResult;
}
/****************************************************************************
 *	MaResMgr_GetDefVoiceKey
 *
 *	Function:
 *			Get the key(block:f-number) of specified timber.
 *	Argument:
 *			dProg	program number(0..127: melody timbers, 128..255: drum timbers)
 *	Return:
 *			> 0		key of specified timber.
 *			0		no entry
 *			< 0		error code
 *
 ****************************************************************************/
SINT32 MaResMgr_GetDefVoiceKey
(
	UINT32	dProg						/* program number */
)
{
	SINT32	sdResult;
	
	MARESMGR_DBGMSG(("   MaResMgr_GetDefVoiceKey: dProg=%d ->", dProg));

	if ( dProg < 128 )
	{
		sdResult = (SINT32)MASMW_ERROR;
	}
	else
	{
		dProg -= 128;

		if ( ( dProg < MA_MIN_ROM_DRUM ) || ( MA_MAX_ROM_DRUM < dProg ) )
		{
			sdResult = (SINT32)(0);
		}
		else
		{
			sdResult = (SINT32)wMaRomDrumKey[dProg - MA_MIN_ROM_DRUM];
		}
	}

	MARESMGR_DBGMSG((" %ld\n", sdResult));
	
	return sdResult;
}
/****************************************************************************
 *	MaResMgr_RegStreamAudio
 *
 *	Function:
 *			Regist the wave data for stream audio.
 *	Argument:
 *			dWaveId		wave id (0..31)
 *			dFormat		wave format
 *			pbWave		pointer to the wave data for stream audio
 *			dWaveSize	size of the wave data for stream audio
 *	Return:
 *			0		success
 *			< 0		error codes
 *
 ****************************************************************************/
SINT32 MaResMgr_RegStreamAudio
(
	UINT32	dWaveId,					/* wave id */
	UINT32	dFormat,					/* wave format */
	UINT8 *	pbWave,						/* pointer to the wave data */
	UINT32	dWaveSize					/* size of the wave data */
)
{
	MARESMGR_DBGMSG(("   MaResMgr_RegStreamAudio: id=%d ptr=%p sz=%ld\n", dWaveId, pbWave, dWaveSize));

	/* check argument */

	if ( dWaveId > MARESMGR_MAX_WAVEID )
	{
		MARESMGR_DBGMSG(("illegal wave id\n"));
		return MASMW_ERROR_ARGUMENT;
	}
	if ( (dFormat & 0x0000007F) > 0x00000004 )
	{
		MARESMGR_DBGMSG(("illegal wave format\n"));
		return MASMW_ERROR_ARGUMENT;
	}
	if ( (dFormat & 0xFFFFFF80) > 0x00000080 )
	{
		MARESMGR_DBGMSG(("illegal wave format\n"));
		return MASMW_ERROR_ARGUMENT;
	}
	if ( pbWave == NULL )
	{
		MARESMGR_DBGMSG(("illegal pointer to the wave data\n"));
		return MASMW_ERROR_ARGUMENT;
	}
	if ( dWaveSize == 0 )
	{
		MARESMGR_DBGMSG(("illegal size of the wave data\n"));
		return MASMW_ERROR_ARGUMENT;
	}
	
	/* regist wave data */
	pgMaResInfo->StreamAudio[dWaveId].dFormat	 = dFormat;
	pgMaResInfo->StreamAudio[dWaveId].pbWave	 = pbWave;
	pgMaResInfo->StreamAudio[dWaveId].dWaveSize	 = dWaveSize;

	return MASMW_SUCCESS;
}
/****************************************************************************
 *	MaResMgr_DelStreamAudio
 *
 *	Function:
 *			Delete the wave data for stream audio.
 *	Argument:
 *			dWaveId		wave id
 *	Return:
 *			0		success
 *			< 0		error codes
 *
 ****************************************************************************/
SINT32 MaResMgr_DelStreamAudio
(
	UINT32	dWaveId						/* wave id */
)
{
	MARESMGR_DBGMSG(("   MaResMgr_DelStreamAudio: id=%d\n", dWaveId));

	/* check argument */
	if ( dWaveId > MARESMGR_MAX_WAVEID )
	{
		MARESMGR_DBGMSG(("illegal wave id\n"));
		return MASMW_ERROR_ARGUMENT;
	}

	pgMaResInfo->StreamAudio[dWaveId].pbWave = NULL;
	pgMaResInfo->StreamAudio[dWaveId].dWaveSize = 0;

	return MASMW_SUCCESS;
}
/****************************************************************************
 *	MaResMgr_GetStreamAudioInfo
 *
 *	Function:
 *			return the wave data for stream audio.
 *	Argument:
 *			dWaveId		wave id
 *			pdFormat	pointer to the area put wave format to
 *			ppbWave		pointer to the area put the pointer to 
 *						 the wave data for stream audio to
 *			pdWaveSize	pointer to the area put the size of 
 *						 the wave data for stream audio to
 *	Return:
 *			0		success
 *			< 0		error codes
 *
 ****************************************************************************/
SINT32 MaResMgr_GetStreamAudioInfo
(
	UINT32		dWaveId,				/* wave id */
	UINT32 *	pdFormat,				/* format */
	UINT8 **	ppbWave,				/* pointer to the wave data */
	UINT32 *	pdWaveSize				/* size of the wave data */
)
{
	MARESMGR_DBGMSG(("   MaResMgr_GetStreamAudioInfo: dWaveId=%d\n", dWaveId));
	
	/* check argument */
	if ( dWaveId > MARESMGR_MAX_WAVEID )
	{
		MARESMGR_DBGMSG(("illegal wave id\n"));
		return MASMW_ERROR_ARGUMENT;
	}
	if ( pdFormat == NULL )
	{
		MARESMGR_DBGMSG(("illegal pointer\n"));
		return MASMW_ERROR_ARGUMENT;
	}
	if ( ppbWave == NULL )
	{
		MARESMGR_DBGMSG(("illegal pointer\n"));
		return MASMW_ERROR_ARGUMENT;
	}
	if ( pdWaveSize == NULL )
	{
		MARESMGR_DBGMSG(("illegal pointer\n"));
		return MASMW_ERROR_ARGUMENT;
	}
	
	*pdFormat	= pgMaResInfo->StreamAudio[dWaveId].dFormat;
	*ppbWave	= pgMaResInfo->StreamAudio[dWaveId].pbWave;
	*pdWaveSize = pgMaResInfo->StreamAudio[dWaveId].dWaveSize;

	return MASMW_SUCCESS;
}
/****************************************************************************
 *	MaResMgr_AllocRam
 *
 *	Function:
 *			Allocate the specified RAM block resources.
 *	Argument:
 *			dRbMap		bit mapping of RAM block resources to allocate.
 *	Return:
 *			0		success
 *			< 0		error codes
 *
 ****************************************************************************/
SINT32 MaResMgr_AllocRam
(
	UINT32		dRbMap					/* ram block map */
)
{
	MARESMGR_DBGMSG(("    MaResMgr_AllocRam        : dRbMap=%08lX", dRbMap));

	if ( dRbMap == 0x00000000 ) return MASMW_SUCCESS;

	if ( ( dRbMap & MARESMGR_RB_MAP_MASK ) != dRbMap )
	{
		MARESMGR_DBGMSG(("illegal rb value\n"));
		return MASMW_ERROR_ARGUMENT;
	}
	
	if ( ( pgMaResInfo->dRbMap & dRbMap ) != 0 )
	{
		MARESMGR_DBGMSG(("can't alloc rb\n"));
		return MASMW_ERROR_RESOURCE_OVER;
	}

	pgMaResInfo->dRbMap |= dRbMap;

	MARESMGR_DBGMSG((" -> dRbMap=%08lX\n", pgMaResInfo->dRbMap));

	return MASMW_SUCCESS;
}
/****************************************************************************
 *	MaResMgr_FreeRam
 *
 *	Function:
 *			Free RAM block resources.
 *	Argument:
 *			dRbMap	bit mapping of RAM block resources to free
 *	Return:
 *			0		success
 *			< 0		error codes
 *
 ****************************************************************************/
SINT32 MaResMgr_FreeRam
(
	UINT32	dRbMap						/* ram block map */
)
{
	MARESMGR_DBGMSG(("    MaResMgr_FreeRam         : dRbMap=%08lX", dRbMap));

	if ( dRbMap == 0x00000000 ) return MASMW_SUCCESS;

	if ( ( dRbMap & MARESMGR_RB_MAP_MASK ) != dRbMap )
	{
		MARESMGR_DBGMSG(("illegal rb value\n"));
		return MASMW_ERROR_ARGUMENT;
	}
	
	if ( ( pgMaResInfo->dRbMap & dRbMap ) != dRbMap )
	{
		MARESMGR_DBGMSG(("can't free rb\n"));
		return MASMW_ERROR;
	}

	pgMaResInfo->dRbMap &= ~dRbMap;

	MARESMGR_DBGMSG((" -> dRbMap=%08lX\n", pgMaResInfo->dRbMap));

	return MASMW_SUCCESS;
}
/****************************************************************************
 *	MaResMgr_AllocCh
 *
 *	Function:
 *			Allocate the channel resources.
 *	Argument:
 *			dChMap	bit mapping of channel resources to allocate.
 *	Return:
 *			0		success
 *			< 0		error codes
 *
 ****************************************************************************/
SINT32 MaResMgr_AllocCh
(
	UINT32	dChMap						/* channel map */
)
{
	MARESMGR_DBGMSG(("    MaResMgr_AllocCh         : dChMap=%08lX", dChMap));

	if ( dChMap == 0x00000000 ) return MASMW_SUCCESS;

	if ( ( dChMap & MARESMGR_CH_MAP_MASK ) != dChMap )
	{
		MARESMGR_DBGMSG((" illegal ch value\n"));
		return MASMW_ERROR_ARGUMENT;
	}
	
	if ( ( pgMaResInfo->dChMap & dChMap ) != 0 )
	{
		MARESMGR_DBGMSG((" can't alloc ch\n"));
		return MASMW_ERROR_RESOURCE_OVER;
	}

	pgMaResInfo->dChMap |= dChMap;

	MARESMGR_DBGMSG((" -> dChMap=%08lX\n", pgMaResInfo->dChMap));

	return MASMW_SUCCESS;
}
/****************************************************************************
 *	MaResMgr_FreeCh
 *
 *	Function:
 *			Free the specified channel resources.
 *	Argument:
 *			dChMap	bit mapping of channel resources to free.
 *	Return:
 *			0		success
 *			< 0		error codes
 *
 ****************************************************************************/
SINT32 MaResMgr_FreeCh
(
	UINT32	dChMap						/* channel map */
)
{
	MARESMGR_DBGMSG(("    MaResMgr_FreeCh          : dChMap=%08lX", dChMap));

	if ( dChMap == 0x00000000 ) return MASMW_SUCCESS;

	if ( ( dChMap & MARESMGR_CH_MAP_MASK ) != dChMap )
	{
		MARESMGR_DBGMSG(("illegal ch value\n"));
		return MASMW_ERROR_ARGUMENT;
	}
	
	if ( ( pgMaResInfo->dChMap & dChMap ) != dChMap )
	{
		MARESMGR_DBGMSG(("can't free ch\n"));
		return MASMW_ERROR;
	}

	pgMaResInfo->dChMap &= ~dChMap;

	MARESMGR_DBGMSG((" -> dChMap=%08lX\n", pgMaResInfo->dChMap));

	return MASMW_SUCCESS;
}
/****************************************************************************
 *	MaResMgr_AllocFmVoice
 *
 *	Function:
 *			Allocate the FM voice resources.
 *	Argument:
 *			dFmMap	bit mapping of FM voice resources to allocate
 *			dFmMode	FM mode  0:2OP / 1:4OP
 *	Return:
 *			0		success
 *			< 0		error codes
 *
 ****************************************************************************/
SINT32 MaResMgr_AllocFmVoice
(
	UINT32	dFmMap,						/* FM voice map */
	UINT32	dFmMode						/* FM mode */
)
{
	UINT8	bMaiMode;
	
	MARESMGR_DBGMSG(("    MaResMgr_AllocFmVoice    : dFmMap=%08lX", dFmMap));

	if ( dFmMap == 0x00000000 ) return MASMW_SUCCESS;

	if ( ( dFmMap & MARESMGR_FM_MAP_MASK ) != dFmMap )
	{
		MARESMGR_DBGMSG(("illegal fm voice value\n"));
		return MASMW_ERROR_ARGUMENT;
	}

	if ( dFmMode == MARESMGR_FMMODE_4OP )
	{	/* 4OP */
		if ( ( dFmMap & MA_FM_EXT_MAP_MASK ) != 0 )
		{
			MARESMGR_DBGMSG(("illegal fm mode\n"));
			return MASMW_ERROR_ARGUMENT;
		}
	}
	else if ( dFmMode > MARESMGR_FMMODE_4OP )
	{
		MARESMGR_DBGMSG(("illegal fm mode\n"));
		return MASMW_ERROR_ARGUMENT;
	}

	if ( ( pgMaResInfo->dFmMap & dFmMap ) != 0 )
	{
		MARESMGR_DBGMSG(("can't alloc fm voice\n"));
		return MASMW_ERROR_RESOURCE_OVER;
	}
	
	if ( pgMaResInfo->dFmMode == MARESMGR_FMMODE_FREE ) 
	{
		bMaiMode = MaDevDrv_ReadIntermediateReg( MAI_MODE );
		if ( dFmMode == MARESMGR_FMMODE_4OP )
			bMaiMode &= ~MAB_FMMODE;
		else
			bMaiMode |= MAB_FMMODE;
		MaDevDrv_WriteIntermediateReg( MAI_MODE, bMaiMode );
		pgMaResInfo->dFmMode = dFmMode;
	}
	else if ( pgMaResInfo->dFmMode != dFmMode )
	{
		MARESMGR_DBGMSG(("can't change FM mode\n"));
		return MASMW_ERROR_RESOURCE_OVER;
	}

	pgMaResInfo->dFmMap |= dFmMap;

	MARESMGR_DBGMSG((" -> dFmMap=%08lX\n", pgMaResInfo->dFmMap));

	return MASMW_SUCCESS;
}
/****************************************************************************
 *	MaResMgr_FreeFmVoice
 *
 *	Function:
 *			Free the specified FM voice resources.
 *	Argument:
 *			dFmMap	bit mapping of FM voice resources to free
 *	Return:
 *			0		success
 *			< 0		error codes
 *
 ****************************************************************************/
SINT32 MaResMgr_FreeFmVoice
(
	UINT32	dFmMap						/* FM voice map */
)
{
	UINT8	bMaiMode;

	MARESMGR_DBGMSG(("    MaResMgr_FreeFmVoice     : dFmMap=%08lX", dFmMap));

	if ( dFmMap == 0x00000000 ) return MASMW_SUCCESS;

	if ( ( dFmMap & MARESMGR_FM_MAP_MASK ) != dFmMap )
	{
		MARESMGR_DBGMSG(("illegal fm voice value\n"));
		return MASMW_ERROR_ARGUMENT;
	}
	
	if ( ( pgMaResInfo->dFmMap & dFmMap ) != dFmMap )
	{
		MARESMGR_DBGMSG(("can't free fm map\n"));
		return MASMW_ERROR;
	}

	pgMaResInfo->dFmMap &= ~dFmMap;
	
	if ( pgMaResInfo->dFmMap == 0 )
	{
		bMaiMode = MaDevDrv_ReadIntermediateReg( MAI_MODE );
		bMaiMode &= ~MAB_FMMODE;
		MaDevDrv_WriteIntermediateReg( MAI_MODE, bMaiMode );
		pgMaResInfo->dFmMode = MARESMGR_FMMODE_FREE;
	}

	MARESMGR_DBGMSG((" -> dFmMap=%08lX\n", pgMaResInfo->dFmMap));

	return MASMW_SUCCESS;
}
/****************************************************************************
 *	MaResMgr_AllocWtVoice
 *
 *	Function:
 *			Allocate the specified WT voice resources.
 *	Argument:
 *			dWtMap	bit mapping of WT voice resources to allocate.
 *	Return:
 *			0		success
 *			< 0		error codes
 *
 ****************************************************************************/
SINT32 MaResMgr_AllocWtVoice
(
	UINT32	dWtMap						/* WT voice map */
)
{
	MARESMGR_DBGMSG(("    MaResMgr_AllocWtVoice    : dWtMap=%08lX", dWtMap));

	if ( dWtMap == 0x00000000 ) return MASMW_SUCCESS;

	if ( ( dWtMap & MARESMGR_WT_MAP_MASK ) != dWtMap )
	{
		MARESMGR_DBGMSG(("illegal wt voice value\n"));
		return MASMW_ERROR_ARGUMENT;
	}
	
	if ( ( pgMaResInfo->dWtMap & dWtMap ) != 0 )
	{
		MARESMGR_DBGMSG(("can't alloc wt voice\n"));
		return MASMW_ERROR_RESOURCE_OVER;
	}

	pgMaResInfo->dWtMap |= dWtMap;

	MARESMGR_DBGMSG((" -> dWtMap=%08lX\n", pgMaResInfo->dWtMap));

	return MASMW_SUCCESS;
}
/****************************************************************************
 *	MaResMgr_FreeWtVoice
 *
 *	Function:
 *			Free the specified WT voice resources.
 *	Argument:
 *			dWtMap	bit mapping of WT voice resources to free.
 *	Return:
 *			0		success
 *			< 0		error codes
 *
 ****************************************************************************/
SINT32 MaResMgr_FreeWtVoice
(
	UINT32	dWtMap						/* WT voice map */
)
{
	MARESMGR_DBGMSG(("    MaResMgr_FreeWtVoice     : dWtMap=%08lX", dWtMap));

	if ( dWtMap == 0x00000000 ) return MASMW_SUCCESS;

	if ( ( dWtMap & MARESMGR_WT_MAP_MASK ) != dWtMap )
	{
		MARESMGR_DBGMSG(("illegal wt voice value\n"));
		return MASMW_ERROR_ARGUMENT;
	}
	
	if ( ( pgMaResInfo->dWtMap & dWtMap ) != dWtMap )
	{
		MARESMGR_DBGMSG(("can't free wt map\n"));
		return MASMW_ERROR;
	}

	pgMaResInfo->dWtMap &= ~dWtMap;

	MARESMGR_DBGMSG((" -> dWtMap=%08lX\n", pgMaResInfo->dWtMap));

	return MASMW_SUCCESS;
}
/****************************************************************************
 *	MaResMgr_AllocStreamAudio
 *
 *	Function:
 *			Allocate the specified stream audio resources.
 *	Argument:
 *			dSaMap	bit mapping of Stream Audio resources to allocate.
 *	Return:
 *			0		success
 *			< 0		error code
 *
 ****************************************************************************/
SINT32 MaResMgr_AllocStreamAudio
(
	UINT32	dSaMap						/* stream audio id */
)
{
	SINT32	sdResult;
	UINT8	bMaiMode;
	static const UINT32 dWtResMap[4] = { 0x00000000, 0x02000000, 0x04000000, 0x06000000 };
	
	MARESMGR_DBGMSG(("    MaResMgr_AllocStreamAudio: dSaMap=%08lX\n", dSaMap));

	if ( dSaMap == 0x00000000 ) return MASMW_SUCCESS;

	if ( ( dSaMap & MARESMGR_SA_MAP_MASK ) != dSaMap )
	{
		MARESMGR_DBGMSG(("Illegal dSaMap value.\n"));
		return MASMW_ERROR_ARGUMENT;
	}

	if ( ( pgMaResInfo->dSaMap & dSaMap ) != 0 )
	{
		MARESMGR_DBGMSG(("can't alloc stream audio\n"));
		return MASMW_ERROR_RESOURCE_OVER;
	}

	sdResult = MaResMgr_AllocWtVoice( dWtResMap[dSaMap] );
	if ( sdResult != MASMW_SUCCESS ) return sdResult;

	sdResult = MaResMgr_AllocSoftInt( dSaMap );
	if ( sdResult != MASMW_SUCCESS )
	{
		MaResMgr_FreeWtVoice( dWtResMap[dSaMap] );
		return sdResult;
	}

	sdResult = MaResMgr_AllocStreamInt( dSaMap, MARESMGR_STREAM_INT_POINT, MARESMGR_STREAM_BUF_SIZE );
	if ( sdResult != MASMW_SUCCESS )
	{
		MaResMgr_FreeWtVoice( dWtResMap[dSaMap] );
		MaResMgr_FreeSoftInt( dSaMap );
		return sdResult;
	}

	if ( pgMaResInfo->dSaMap == 0 )
	{
		bMaiMode = MaDevDrv_ReadIntermediateReg( MAI_MODE );
		bMaiMode |= MAB_STMMODE;
		MaDevDrv_WriteIntermediateReg( MAI_MODE, bMaiMode );
	}

	pgMaResInfo->dSaMap |= dSaMap;

	MARESMGR_DBGMSG((" -> dSaMap=%08lX\n", pgMaResInfo->dSaMap));

	return MASMW_SUCCESS;
}
/****************************************************************************
 *	MaResMgr_FreeStreamAudio
 *
 *	Function:
 *			Free the specified stream audio resources.
 *	Argument:
 *			dSaMap	bit mapping of Stream Audio resources to free.
 *	Return:
 *			0		success
 *			< 0		error codes
 *
 ****************************************************************************/
SINT32 MaResMgr_FreeStreamAudio
(
	UINT32	dSaMap						/* stream audio map */
)
{
	SINT32	sdResult;
	UINT8	bMaiMode;
	UINT32	dErrCount;
	static const UINT32 dWtResMap[4] = { 0x00000000, 0x02000000, 0x04000000, 0x06000000 };

	MARESMGR_DBGMSG(("    MaResMgr_FreeStreamAudio : dSaMap=%08lX\n", dSaMap));

	if ( dSaMap == 0x00000000 ) return MASMW_SUCCESS;

	if ( ( dSaMap & MARESMGR_SA_MAP_MASK ) != dSaMap )
	{
		MARESMGR_DBGMSG(("illegal stream audio id\n"));
		return MASMW_ERROR_ARGUMENT;
	}

	if ( ( pgMaResInfo->dSaMap & dSaMap ) != dSaMap )
	{
		MARESMGR_DBGMSG(("can't free sa\n"));
		return MASMW_ERROR;
	}

	dErrCount = 0;
	
	sdResult = MaResMgr_FreeWtVoice( dWtResMap[dSaMap] );
	if ( sdResult != MASMW_SUCCESS ) dErrCount++;

	sdResult = MaResMgr_FreeSoftInt( dSaMap );
	if ( sdResult != MASMW_SUCCESS ) dErrCount++;

	sdResult = MaResMgr_FreeStreamInt( dSaMap );
	if ( sdResult != MASMW_SUCCESS ) dErrCount++;;

	pgMaResInfo->dSaMap &= ~dSaMap;
	
	if ( pgMaResInfo->dSaMap == 0 )
	{
		bMaiMode = MaDevDrv_ReadIntermediateReg( MAI_MODE );
		bMaiMode &= ~MAB_STMMODE;
		MaDevDrv_WriteIntermediateReg( MAI_MODE, bMaiMode );
	}

	MARESMGR_DBGMSG((" -> dSaMap=%08lX\n", pgMaResInfo->dSaMap));

	if ( dErrCount > 0 )
	{
		MARESMGR_DBGMSG(("can't free resource for stream audio\n"));
		return MASMW_ERROR;
	}
		
	return MASMW_SUCCESS;
}
/****************************************************************************
 *	MaResMgr_AllocAl
 *
 *	Function:
 *			Allocate the specified AL voice resources.
 *	Argument:
 *			dWtMap	bit mapping of AL voice resources to allocate.
 *	Return:
 *			0		success
 *			< 0		error codes
 *
 ****************************************************************************/
SINT32 MaResMgr_AllocAl
(
	UINT32	dAlMap						/* AL voice map */
)
{
	MARESMGR_DBGMSG(("    MaResMgr_AllocAl    : dAlMap=%08lX", dAlMap));

	if ( dAlMap == 0x00000000 ) return MASMW_SUCCESS;

	if ( ( dAlMap & MARESMGR_AL_MAP_MASK ) != dAlMap )
	{
		MARESMGR_DBGMSG(("illegal al voice value\n"));
		return MASMW_ERROR_ARGUMENT;
	}
	
	if ( ( pgMaResInfo->dAlMap & dAlMap ) != 0 )
	{
		MARESMGR_DBGMSG(("can't alloc al voice\n"));
		return MASMW_ERROR_RESOURCE_OVER;
	}

	pgMaResInfo->dAlMap |= dAlMap;

	MARESMGR_DBGMSG((" -> dAlMap=%08lX\n", pgMaResInfo->dAlMap));

	return MASMW_SUCCESS;
}
/****************************************************************************
 *	MaResMgr_FreeAl
 *
 *	Function:
 *			Free the specified AL voice resources.
 *	Argument:
 *			dAlMap	bit mapping of AL voice resources to free.
 *	Return:
 *			0		success
 *			< 0		error codes
 *
 ****************************************************************************/
SINT32 MaResMgr_FreeAl
(
	UINT32	dAlMap						/* AL voice map */
)
{
	MARESMGR_DBGMSG(("    MaResMgr_FreeAl     : dAlMap=%08lX", dAlMap));

	if ( dAlMap == 0x00000000 ) return MASMW_SUCCESS;

	if ( ( dAlMap & MARESMGR_AL_MAP_MASK ) != dAlMap )
	{
		MARESMGR_DBGMSG(("illegal al voice value\n"));
		return MASMW_ERROR_ARGUMENT;
	}
	
	if ( ( pgMaResInfo->dAlMap & dAlMap ) != dAlMap )
	{
		MARESMGR_DBGMSG(("can't free al map\n"));
		return MASMW_ERROR;
	}

	pgMaResInfo->dAlMap &= ~dAlMap;

	MARESMGR_DBGMSG((" -> dAlMap=%08lX\n", pgMaResInfo->dAlMap));

	return MASMW_SUCCESS;
}
/****************************************************************************
 *	MaResMgr_AllocHv
 *
 *	Function:
 *			Allocate the specified HV voice resources.
 *	Argument:
 *			dHvMap	bit mapping of HV voice resources to allocate.
 *	Return:
 *			0		success
 *			< 0		error code
 *
 ****************************************************************************/
SINT32 MaResMgr_AllocHv
(
	UINT32	dHvMap						/* HV voice map */
)
{
	SINT32	sdResult;
	UINT8	bMaiMode;
	static const UINT32 dWtResMap = 0x01FF0000;
	
	MARESMGR_DBGMSG(("    MaResMgr_AllocHv: dHvMap=%08lX\n", dHvMap));

	if ( dHvMap == 0x00000000 ) return MASMW_SUCCESS;

	if ( ( dHvMap & MARESMGR_HV_MAP_MASK ) != dHvMap )
	{
		MARESMGR_DBGMSG(("Illegal HV voice map.\n"));
		return MASMW_ERROR_ARGUMENT;
	}

	if ( ( pgMaResInfo->dHvMap & dHvMap ) != 0 )
	{
		MARESMGR_DBGMSG(("can't alloc HV voice\n"));
		return MASMW_ERROR_RESOURCE_OVER;
	}

	sdResult = MaResMgr_AllocWtVoice( dWtResMap );
	if ( sdResult != MASMW_SUCCESS ) return sdResult;
	
	if ( pgMaResInfo->dHvMap == 0 )
	{
		bMaiMode = MaDevDrv_ReadIntermediateReg( MAI_MODE );
		bMaiMode |= MAB_HVMODE;
		MaDevDrv_WriteIntermediateReg( MAI_MODE, bMaiMode );
	}

	pgMaResInfo->dHvMap |= dHvMap;

	MARESMGR_DBGMSG((" -> dHvMap=%08lX\n", pgMaResInfo->dHvMap));

	return MASMW_SUCCESS;
}
/****************************************************************************
 *	MaResMgr_FreeHv
 *
 *	Function:
 *			Free the specified HV voice resources.
 *	Argument:
 *			dHvMap	bit mapping of HV voice resources to free.
 *	Return:
 *			0		success
 *			< 0		error codes
 *
 ****************************************************************************/
SINT32 MaResMgr_FreeHv
(
	UINT32	dHvMap						/* HV voice map */
)
{
	SINT32	sdResult;
	UINT8	bMaiMode;
	static const UINT32 dWtResMap = 0x01FF0000;

	MARESMGR_DBGMSG(("    MaResMgr_FreeHv : dHvMap=%08lX\n", dHvMap));

	if ( dHvMap == 0x00000000 ) return MASMW_SUCCESS;

	if ( ( dHvMap & MARESMGR_HV_MAP_MASK ) != dHvMap )
	{
		MARESMGR_DBGMSG(("illegal HV voice map\n"));
		return MASMW_ERROR_ARGUMENT;
	}

	if ( ( pgMaResInfo->dHvMap & dHvMap ) != dHvMap )
	{
		MARESMGR_DBGMSG(("can't free HV voice\n"));
		return MASMW_ERROR;
	}

	sdResult = MaResMgr_FreeWtVoice( dWtResMap );
	if ( sdResult != MASMW_SUCCESS ) return sdResult;
	
	pgMaResInfo->dHvMap &= ~dHvMap;
	
	if ( pgMaResInfo->dHvMap == 0 )
	{
		bMaiMode = MaDevDrv_ReadIntermediateReg( MAI_MODE );
		bMaiMode &= ~MAB_HVMODE;
		MaDevDrv_WriteIntermediateReg( MAI_MODE, bMaiMode );
	}

	MARESMGR_DBGMSG((" -> dHvMap=%08lX\n", pgMaResInfo->dHvMap));

	return MASMW_SUCCESS;
}
/****************************************************************************
 *	MaResMgr_AllocLed
 *
 *	Function:
 *			Allocate the specified LED.
 *	Argument:
 *			dLedMap	bit mapping of LED resources to allocate.
 *	Return:
 *			0		success
 *			< 0		error codes
 *
 ****************************************************************************/
SINT32 MaResMgr_AllocLed
(
	UINT32	dLedMap						/* led map */
)
{
	MARESMGR_DBGMSG(("    MaResMgr_AllocLed        : dLedMap=%08lX", dLedMap));

	if ( dLedMap == 0x00000000 ) return MASMW_SUCCESS;

	if ( ( dLedMap & MARESMGR_LD_MAP_MASK ) != dLedMap )
	{
		MARESMGR_DBGMSG(("illegal led map\n"));
		return MASMW_ERROR_ARGUMENT;
	}

	if ( ( pgMaResInfo->dLedMap & dLedMap ) != 0 )
	{
		MARESMGR_DBGMSG(("can't alloc led\n"));
		return MASMW_ERROR_RESOURCE_OVER;
	}

	pgMaResInfo->dLedMap |= dLedMap;

	MARESMGR_DBGMSG((" -> dLedMap=%08lX\n", pgMaResInfo->dLedMap));

	return MASMW_SUCCESS;
}
/****************************************************************************
 *	MaResMgr_FreeLed
 *
 *	Function:
 *			Free the specified LED.
 *	Argument:
 *			dLedMap	bit mapping of LED resources to free.
 *	Return:
 *			0		success
 *			< 0		error codes
 *
 ****************************************************************************/
SINT32 MaResMgr_FreeLed
(
	UINT32	dLedMap						/* led map */
)
{
	MARESMGR_DBGMSG(("    MaResMgr_FreeLed         : dLedMap=%08lX", dLedMap));

	if ( dLedMap == 0x00000000 ) return MASMW_SUCCESS;

	if ( ( dLedMap & MARESMGR_LD_MAP_MASK ) != dLedMap )
	{
		MARESMGR_DBGMSG(("illegal led map\n"));
		return MASMW_ERROR_ARGUMENT;
	}
	
	if ( ( pgMaResInfo->dLedMap & dLedMap ) != dLedMap )
	{
		MARESMGR_DBGMSG(("can't free led\n"));
		return MASMW_ERROR;
	}

	pgMaResInfo->dLedMap &= ~dLedMap;

	MARESMGR_DBGMSG((" -> dLedMap=%08lX\n", pgMaResInfo->dLedMap));

	return MASMW_SUCCESS;
}
/****************************************************************************
 *	MaResMgr_AllocMotor
 *
 *	Function:
 *			Allocate the specified motor.
 *	Argument:
 *			dMtrMap	bit mapping of motor resources to allocate.
 *	Return:
 *			0		success
 *			< 0		error codes
 *
 ****************************************************************************/
SINT32 MaResMgr_AllocMotor
(
	UINT32	dMtrMap						/* mtr map */
)
{
	MARESMGR_DBGMSG(("    MaResMgr_AllocMotor      : dMtrMap=%08lX", dMtrMap));

	if ( dMtrMap == 0x00000000 ) return MASMW_SUCCESS;

	if ( ( dMtrMap & MARESMGR_MT_MAP_MASK ) != dMtrMap )
	{
		MARESMGR_DBGMSG(("illegal motor map\n"));
		return MASMW_ERROR_ARGUMENT;
	}
	
	if ( ( pgMaResInfo->dMtrMap & dMtrMap ) != 0 )
	{
		MARESMGR_DBGMSG(("can't alloc motor\n"));
		return MASMW_ERROR_RESOURCE_OVER;
	}

	pgMaResInfo->dMtrMap |= dMtrMap;

	MARESMGR_DBGMSG((" -> dMtrMap=%08lX\n", pgMaResInfo->dMtrMap));

	return MASMW_SUCCESS;
}
/****************************************************************************
 *	MaResMgr_FreeMotor
 *
 *	Function:
 *			Free the specified motor.
 *	Argument:
 *			dMtrMap	bit mapping of motor resources to free.
 *	Return:
 *			0		success
 *			< 0		error codes
 *
 ****************************************************************************/
SINT32 MaResMgr_FreeMotor
(
	UINT32	dMtrMap						/* mtr map */
)
{
	MARESMGR_DBGMSG(("    MaResMgr_FreeMotor       : dMtrMap=%08lX", dMtrMap));

	if ( dMtrMap == 0x00000000 ) return MASMW_SUCCESS;

	if ( ( dMtrMap & MARESMGR_MT_MAP_MASK ) != dMtrMap )
	{
		MARESMGR_DBGMSG(("illegal motor map\n"));
		return MASMW_ERROR_ARGUMENT;
	}
	
	if ( ( pgMaResInfo->dMtrMap & dMtrMap ) != dMtrMap )
	{
		MARESMGR_DBGMSG(("can't free motor\n"));
		return MASMW_ERROR;
	}

	pgMaResInfo->dMtrMap &= ~dMtrMap;

	MARESMGR_DBGMSG((" -> dMtrMap=%08lX\n", pgMaResInfo->dMtrMap));

	return MASMW_SUCCESS;
}
/****************************************************************************
 *	MaResMgr_AllocSequencer
 *
 *	Function:
 *			Allocate the specified sequencer and setting it.
 *	Argument:
 *			dSeqId		sequencer id
 *			wBaseTime	minimum time base of sequencer
 *	Return:
 *			0		success
 *			< 0		error codes
 *
 ****************************************************************************/
SINT32 MaResMgr_AllocSequencer
(
	UINT32	dSeqId,						/* sequencer id */
	UINT16	wBaseTime					/* base time */
)
{
	UINT32	dSeqMap;

	MARESMGR_DBGMSG(("    MaResMgr_AllocSequencer  : id=%d ", dSeqId));

	dSeqMap = (UINT32)( 1 << dSeqId );
	
	if ( ( dSeqMap & MARESMGR_SQ_MAP_MASK ) != dSeqMap )
	{
		MARESMGR_DBGMSG(("illegal sequencer number\n"));
		return MASMW_ERROR_ARGUMENT;
	}
	
	if ( ( pgMaResInfo->dSeqMap & dSeqMap ) != 0 )
	{
		MARESMGR_DBGMSG(("can't alloc sequencer\n"));
		return MASMW_ERROR_RESOURCE_OVER;
	}

	if (wBaseTime > 0xFFFC )
	{
		MARESMGR_DBGMSG(("illegal minimum time base of sequencer\n"));
		return MASMW_ERROR_ARGUMENT;
	}

	pgMaResInfo->dSeqMap |= dSeqMap;

	MARESMGR_DBGMSG(("-> dSeqMap=%08lX\n", pgMaResInfo->dSeqMap));

	if ( dSeqId == 0 )
	{
		MaDevDrv_WriteIntermediateReg( MAI_SEQ_MSS_H, (UINT8)((wBaseTime >> 8) & 0x00FF) );
		MaDevDrv_WriteIntermediateReg( MAI_SEQ_MSS_L, (UINT8)(wBaseTime & 0x00FC) );
		MaDevDrv_WriteIntermediateReg( MAI_SEQ_IRQPOINT, MA_INT_POINT );
		MaDevDrv_WriteIntermediateReg( MAI_SEQ_CTRL, (UINT8)0x02 );
	}

	return MASMW_SUCCESS;
}
/****************************************************************************
 *	MaResMgr_FreeSequencer
 *
 *	Function:
 *			Free the specified sequencer.
 *	Argument:
 *			dSeqId	sequencer id
 *	Return:
 *			0		success
 *			< 0		error codes
 *
 ****************************************************************************/
SINT32 MaResMgr_FreeSequencer
(
	UINT32	dSeqId						/* sequencer id */
)
{
	UINT32	dSeqMap;
	
	MARESMGR_DBGMSG(("    MaResMgr_FreeSequencer   : id=%d", dSeqId));

	dSeqMap = (UINT32)( 1 << dSeqId );

	if ( ( dSeqMap & MARESMGR_SQ_MAP_MASK ) != dSeqMap )
	{
		MARESMGR_DBGMSG(("illegal sequencer number\n"));
		return MASMW_ERROR_ARGUMENT;
	}
	
	if ( ( pgMaResInfo->dSeqMap & dSeqMap ) != dSeqMap )
	{
		MARESMGR_DBGMSG(("can't free seq\n"));
		return MASMW_ERROR;
	}

	pgMaResInfo->dSeqMap &= ~dSeqMap;

	MARESMGR_DBGMSG((" -> dSeqMap=%08lX\n", pgMaResInfo->dSeqMap));

	return MASMW_SUCCESS;
}
/****************************************************************************
 *	MaResMgr_AllocTimer
 *
 *	Function:
 *			Allocate the specified timer.
 *	Arguments:
 *			dTimerId	timer id (0..3)
 *			bBaseTime	base time value (0..127)
 *			bTimeCount		timer count value (0..127)
 *			dMode		0: use base time of timer, 1: use base time of sequencer
 *			dOneShot	0: continuous, 1: one shot
 *	Return:
 *			0		success
 *			< 0		error codes
 *
 ****************************************************************************/
SINT32 MaResMgr_AllocTimer
(
	UINT32	dTimerId,					/* timer id */
	UINT8	bBaseTime,					/* base time */
	UINT8	bTimeCount,					/* timer count value */
	UINT32	dMode,						/* mode */
	UINT32	dOneShot					/* one shot mode */
)
{
	UINT32	dTmMap;
	
	MARESMGR_DBGMSG(("    MaResMgr_AllocTimer      : id=%d bt=%d tm=%d md=%d os=%d", dTimerId, bBaseTime, bTimeCount, dMode, dOneShot));

	if ( dTimerId > 3 )
	{
		MARESMGR_DBGMSG(("illegal timer id\n"));
		return MASMW_ERROR_ARGUMENT;
	}
	if ( dOneShot > 1 )
	{
		MARESMGR_DBGMSG(("illegal mode\n"));
		return MASMW_ERROR_ARGUMENT;
	}
	if ( dMode > 1 )
	{
		MARESMGR_DBGMSG(("illegal one shot mode\n"));
		return MASMW_ERROR_ARGUMENT;
	}

	dTmMap = (UINT32)( 1 << dTimerId );

	if ( ( dTmMap & MARESMGR_TM_MAP_MASK ) != dTmMap )
	{
		MARESMGR_DBGMSG(("illegal timer value\n"));
		return MASMW_ERROR_ARGUMENT;
	}
	
	if ( ( pgMaResInfo->dTmMap & dTmMap ) != 0 )
	{
		MARESMGR_DBGMSG(("can't alloc timer\n"));
		return MASMW_ERROR_RESOURCE_OVER;
	}

	if ( pgMaResInfo->dTmMap == 0 )
		MaDevDrv_WriteIntermediateReg( MAI_TIMER_MST, bBaseTime );

	switch ( dTimerId )
	{
	  case 0:  /* Timer#0 */
		MaDevDrv_WriteIntermediateReg( MAI_TIMER0_WR_COUNT, bTimeCount );
		MaDevDrv_WriteIntermediateReg( MAI_TIMER0_CTRL, (UINT8)((dOneShot << 1) | (dMode << 2)) );
		break;
	  case 1:  /* Timer#1 */
		MaDevDrv_WriteIntermediateReg( MAI_TIMER1_WR_COUNT, bTimeCount );
		MaDevDrv_WriteIntermediateReg( MAI_TIMER1_CTRL, (UINT8)((dOneShot << 1) | (dMode << 2)) );
		break;
	  case 2:  /* Timer#2A */
	  case 3:  /* Timer#2B */
		if ( ( pgMaResInfo->dTmMap & 0x000000C0 ) == 0 )
		{
			MaDevDrv_WriteIntermediateReg( MAI_TIMER2_WR_COUNT, bTimeCount );
			MaDevDrv_WriteIntermediateReg( MAI_TIMER2_CTRL, (UINT8)((dOneShot << 1) | (dMode << 2)) );
		}
		break;
	  default:
		return MASMW_ERROR_ARGUMENT;
	}

	pgMaResInfo->dTmMap |= dTmMap;

	MARESMGR_DBGMSG((" -> dTmMap=%08lX\n", pgMaResInfo->dTmMap));

	return MASMW_SUCCESS;
}
/****************************************************************************
 *	MaResMgr_FreeTimer
 *
 *	Function:
 *			Free the specified timer of.
 *	Argument:
 *			dTimerId	timer id
 *	Return:
 *			0		success
 *			< 0		error code
 *
 ****************************************************************************/
SINT32 MaResMgr_FreeTimer
(
	UINT32	dTimerId					/* timer id */
)
{
	UINT32	dTmMap;
	
	MARESMGR_DBGMSG(("    MaResMgr_FreeTimer       : id=%d", dTimerId));

	if ( dTimerId > 3 )
	{
		MARESMGR_DBGMSG(("illegal timer id\n"));
		return MASMW_ERROR_ARGUMENT;
	}

	dTmMap = (UINT32)( 1 << dTimerId );

	if ( ( dTmMap & MARESMGR_TM_MAP_MASK ) != dTmMap )
	{
		MARESMGR_DBGMSG(("illegal timer value\n"));
		return MASMW_ERROR_ARGUMENT;
	}
	
	if ( ( pgMaResInfo->dTmMap & dTmMap ) != dTmMap )
	{
		MARESMGR_DBGMSG(("can't free timer\n"));
		return MASMW_ERROR;
	}

	switch (dTimerId)
	{
	  case 0:
	  	MaDevDrv_WriteIntermediateReg( MAI_TIMER0_CTRL, 0 );
	  	break;
	  case 1:
	  	MaDevDrv_WriteIntermediateReg( MAI_TIMER1_CTRL, 0 );
	  	break;
	  case 2:
	  	MaDevDrv_WriteIntermediateReg( MAI_TIMER2_TMSTA, 0 );
	  	break;
	  case 3:
	  	MaDevDrv_WriteIntermediateReg( MAI_TIMER2_TMSTB, 0 );
	  	break;
	  default:
	  	return MASMW_ERROR_ARGUMENT;
	}

	pgMaResInfo->dTmMap &= ~dTmMap;

	MARESMGR_DBGMSG((" -> dTmMap=%08lX\n", pgMaResInfo->dTmMap));

	return MASMW_SUCCESS;
}
/****************************************************************************
 *	MaResMgr_AllocStreamInt
 *
 *	Function:
 *			Allocate the specified stream interrupt and setting it.
 *	Argument:
 *			dSaiMap		bit mapping of stream interrupt resource to allocate.
 *			bIntPoint	interrupt point.
 *			bBufSize	size of the wave buffer.
 *	Return:
 *			0		success
 *			< 0		error codes
 *
 ****************************************************************************/
SINT32 MaResMgr_AllocStreamInt
(
	UINT32	dSaiMap,					/* stream interrupt map */
	UINT8	bIntPoint,					/* interrupt point */
	UINT8	bBufSize					/* size of the wave buffer */
)
{
	MARESMGR_DBGMSG(("    MaResMgr_AllocStreamInt  : id=%d ", dSaiMap));

	if ( dSaiMap == 0x00000000 ) return MASMW_SUCCESS;
	
	if ( ( dSaiMap & MARESMGR_AI_MAP_MASK ) != dSaiMap )
	{
		MARESMGR_DBGMSG(("illegal stream interrup map\n"));
		return MASMW_ERROR_ARGUMENT;
	}
	
	if ( ( pgMaResInfo->dSaiMap & dSaiMap ) != 0 )
	{
		MARESMGR_DBGMSG(("can't alloc sequencer\n"));
		return MASMW_ERROR_RESOURCE_OVER;
	}

	if ( bIntPoint > 3 )
	{
		MARESMGR_DBGMSG(("illegal interrupt point\n"));
		return MASMW_ERROR_ARGUMENT;
	}

	if ( bBufSize > 3 )
	{
		MARESMGR_DBGMSG(("illegal size of the wave buffer\n"));
		return MASMW_ERROR_ARGUMENT;
	}

	if ( pgMaResInfo->dSaiMap == 0 )
	{
		MaDevDrv_WriteIntermediateReg( MAI_STM_IRQ_CTRL, (UINT8)(((bBufSize << 4) | bIntPoint) & 0x33) );
	}

	pgMaResInfo->dSaiMap |= dSaiMap;

	MARESMGR_DBGMSG(("-> dSaiMap=%08lX\n", pgMaResInfo->dSaiMap));

	return MASMW_SUCCESS;
}
/****************************************************************************
 *	MaResMgr_FreeStreamInt
 *
 *	Function:
 *			Free the specified stream interrupt.
 *	Argument:
 *			dSaiMap	bit mapping of stream interrupt resource to free.
 *	Return:
 *			0		success
 *			< 0		error codes
 *
 ****************************************************************************/
SINT32 MaResMgr_FreeStreamInt
(
	UINT32	dSaiMap						/* stream interrupt map */
)
{
	MARESMGR_DBGMSG(("    MaResMgr_FreeStreamInt   : id=%d", dSaiMap));

	if ( dSaiMap == 0x00000000 ) return MASMW_SUCCESS;

	if ( ( dSaiMap & MARESMGR_AI_MAP_MASK ) != dSaiMap )
	{
		MARESMGR_DBGMSG(("illegal stream interrupt map\n"));
		return MASMW_ERROR_ARGUMENT;
	}
	
	if ( ( pgMaResInfo->dSaiMap & dSaiMap ) != dSaiMap )
	{
		MARESMGR_DBGMSG(("can't free stream interrupt\n"));
		return MASMW_ERROR;
	}

	pgMaResInfo->dSaiMap &= ~dSaiMap;

	MARESMGR_DBGMSG((" -> dSaiMap=%08lX\n", pgMaResInfo->dSaiMap));

	return MASMW_SUCCESS;
}
/****************************************************************************
 *	MaResMgr_AllocSoftInt
 *
 *	Function:
 *			Allocate the specified software interrupt.
 *	Argument:
 *			dSiMap	bit mapping of software interrupt resources to allocate.
 *	Return:
 *			0		success
 *			< 0		error code
 *
 ****************************************************************************/
SINT32 MaResMgr_AllocSoftInt
(
	UINT32	dSiMap						/* software interrupt map */
)
{
	MARESMGR_DBGMSG(("    MaResMgr_AllocSoftInt    : dSiMap=%08lX", dSiMap));

	if ( dSiMap == 0x00000000 ) return MASMW_SUCCESS;

	if ( ( dSiMap & MARESMGR_SI_MAP_MASK ) != dSiMap )
	{
		MARESMGR_DBGMSG(("illegal soft int map\n"));
		return MASMW_ERROR_ARGUMENT;
	}
	
	if ( ( pgMaResInfo->dSiMap & dSiMap ) != 0 )
	{
		MARESMGR_DBGMSG(("can't alloc soft int\n"));
		return MASMW_ERROR_RESOURCE_OVER;
	}

	pgMaResInfo->dSiMap |= dSiMap;

	MARESMGR_DBGMSG((" -> dSiMap=%08lX\n", pgMaResInfo->dSiMap));

	return MASMW_SUCCESS;
}
/****************************************************************************
 *	MaResMgr_FreeSoftInt
 *
 *	Function:
 *			Free the specified software interrupt.
 *			It is possible to specify some software interrupts at once.
 *	Argument:
 *			dSiMap	bit mapping of software interrupt resources to free.
 *	Return:
 *			0		success
 *			< 0		error code
 *
 ****************************************************************************/
SINT32 MaResMgr_FreeSoftInt
(
	UINT32	dSiMap						/* software interrupt map */
)
{
	MARESMGR_DBGMSG(("    MaResMgr_FreeSoftInt     : dSiMap=%08lX", dSiMap));

	if ( dSiMap == 0x00000000 ) return MASMW_SUCCESS;

	if ( ( dSiMap & MARESMGR_SI_MAP_MASK ) != dSiMap )
	{
		MARESMGR_DBGMSG(("illegal soft int map\n"));
		return MASMW_ERROR_ARGUMENT;
	}

	if ( ( pgMaResInfo->dSiMap & dSiMap ) != dSiMap )
	{
		MARESMGR_DBGMSG(("can't free soft int map\n"));
		return MASMW_ERROR;
	}

	pgMaResInfo->dSiMap &= ~dSiMap;

	MARESMGR_DBGMSG((" -> dSiMap=%08lX\n", pgMaResInfo->dSiMap));

	return MASMW_SUCCESS;
}
/****************************************************************************
 *	MaResMgr_Initialize
 *
 *	Function:
 *			Initialize the MA Resource Manager.
 *	argument:
 *			None
 *	return:
 *			0		success
 *
 ****************************************************************************/
SINT32 MaResMgr_Initialize( void )
{
	UINT32	dWaveId;
	
	MARESMGR_DBGMSG(("MaResMgr_Initialize\n"));

	/* initialize resource info */
	pgMaResInfo->dRbMap = 0x00000000;		/* RAM block */
	pgMaResInfo->dChMap = 0x00000000;		/* channel */
	pgMaResInfo->dFmMap = 0x00000000;		/* FM voice */
	pgMaResInfo->dWtMap = 0x00000000;		/* WT voice */
	pgMaResInfo->dSaMap = 0x00000000;		/* stream audio */
	pgMaResInfo->dAlMap = 0x00000000;		/* AL voice */
	pgMaResInfo->dHvMap = 0x00000000;		/* HV voice */
	pgMaResInfo->dLedMap = 0x00000000;		/* LED */
	pgMaResInfo->dMtrMap = 0x00000000;		/* motor */
	pgMaResInfo->dSeqMap = 0x00000000;		/* sequencer */
	pgMaResInfo->dTmMap = 0x00000000;		/* timer */
	pgMaResInfo->dSaiMap = 0x00000000;		/* stream interrupt */
	pgMaResInfo->dSiMap = 0x00000000;		/* software interrupt */
	pgMaResInfo->dFmMode = MARESMGR_FMMODE_FREE;	/* FM mode : Free */

	/* reset stream audio info */
	for ( dWaveId = 0; dWaveId < MA_MAX_REG_STREAM_AUDIO; dWaveId++ )
	{
		pgMaResInfo->StreamAudio[dWaveId].dFormat    = (UINT32)0;
		pgMaResInfo->StreamAudio[dWaveId].pbWave     = NULL;
		pgMaResInfo->StreamAudio[dWaveId].dWaveSize  = (UINT32)0;
	}
	
	return MASMW_SUCCESS;
}
