/****************************************************************************
 *
 *		Copyright (C) 2002-2003 YAMAHA CORPORATION. All rights reserved.
 *
 *		Module		: maresmgr.h
 *
 *		Description	: MA Resource Manager
 *
 *		Version		: 1.0.3 	2003.01.23
 *
 ****************************************************************************/

#ifndef __MARESMGR_H__
#define __MARESMGR_H__


#include "mamachdep.h"
#include "madebug.h"
#include "madevdrv.h"
#include "madefs.h"


typedef struct _MA_STREAM
{
	UINT32	dFormat;						/* wave format */
	UINT8 *	pbWave;							/* pointer to the wave data */
	UINT32	dWaveSize;						/* size of the wave data */
} MA_STREAM, * PMA_STREAM;

typedef struct _MA_RESOURCE_INFO
{
	UINT32	dRbMap;							/* RAM block */
	UINT32	dChMap;							/* Channle */
	UINT32	dFmMap;							/* FM voice */
	UINT32	dWtMap;							/* WT voice */
	UINT32	dSaMap;							/* Stream audio */
	UINT32	dAlMap;							/* AL voice */
	UINT32	dHvMap;							/* HV voice */
	UINT32	dLedMap;						/* Led */
	UINT32	dMtrMap;						/* Motor */
	UINT32	dSeqMap;						/* Sequencer */
	UINT32	dTmMap;							/* Timer */
	UINT32	dSaiMap;						/* Stream Audio interrupt */
	UINT32	dSiMap;							/* Software interrupt */
	UINT32	dFmMode;						/* FM mode 0:4OP 1:2OP 2:Free */
	MA_STREAM	StreamAudio[MA_MAX_REG_STREAM_AUDIO];

} MA_RESOURCE_INFO, * PMA_RESOURCE_INFO;


PMA_RESOURCE_INFO MaResMgr_GetResourceInfo( void );

SINT32	MaResMgr_Initialize			( void );

SINT32	MaResMgr_GetDefWaveAddress	( UINT32 dWaveId );
SINT32	MaResMgr_GetDefVoiceAddress	( UINT32 dProg );
SINT32	MaResMgr_GetDefVoiceSynth	( UINT32 dProg );
SINT32	MaResMgr_GetDefVoiceKey		( UINT32 dProg );
SINT32	MaResMgr_RegStreamAudio		( UINT32 dWaveId, UINT32 dFormat, UINT8 *pbWave, UINT32 dWaveSize );
SINT32	MaResMgr_DelStreamAudio		( UINT32 dWaveId );
SINT32	MaResMgr_GetStreamAudioInfo	( UINT32 dWaveId, UINT32 *pdFormat, UINT8 **ppbWave, UINT32 *pdWaveSize );

SINT32	MaResMgr_AllocRam			( UINT32 dRbMap );
SINT32 	MaResMgr_FreeRam			( UINT32 dRbMap );
SINT32	MaResMgr_AllocCh			( UINT32 dChMap );
SINT32 	MaResMgr_FreeCh				( UINT32 dChMap );
SINT32 	MaResMgr_AllocFmVoice		( UINT32 dFmMap, UINT32 dFmMode );
SINT32	MaResMgr_FreeFmVoice		( UINT32 dFmMap );
SINT32	MaResMgr_AllocWtVoice		( UINT32 dWtMap );
SINT32 	MaResMgr_FreeWtVoice		( UINT32 dWtMap );
SINT32	MaResMgr_AllocStreamAudio	( UINT32 dSaMap );
SINT32	MaResMgr_FreeStreamAudio	( UINT32 dSaMap );
SINT32	MaResMgr_AllocAl			( UINT32 dAlMap );
SINT32	MaResMgr_FreeAl				( UINT32 dAlMap );
SINT32	MaResMgr_AllocHv			( UINT32 dHvMap );
SINT32	MaResMgr_FreeHv				( UINT32 dHvMap );
SINT32 	MaResMgr_AllocLed			( UINT32 dLedMap );
SINT32 	MaResMgr_FreeLed			( UINT32 dLedMap );
SINT32 	MaResMgr_AllocMotor			( UINT32 dMtrMap );
SINT32 	MaResMgr_FreeMotor			( UINT32 dMtrMap );
SINT32	MaResMgr_AllocSequencer		( UINT32 dSeqId, UINT16 wBaseTime );
SINT32	MaResMgr_FreeSequencer		( UINT32 dSeqId );
SINT32 	MaResMgr_AllocTimer			( UINT32 dTimerId, UINT8 bBaseTime, UINT8 bTimeCount, UINT32 dMode, UINT32 dOneShot );
SINT32 	MaResMgr_FreeTimer			( UINT32 dTimerId );
SINT32	MaResMgr_AllocStreamInt		( UINT32 dSaiMap, UINT8 bIntPoint, UINT8 bBufSize );
SINT32	MaResMgr_FreeStreamInt		( UINT32 dSaiMap );
SINT32 	MaResMgr_AllocSoftInt		( UINT32 dSiMap );
SINT32 	MaResMgr_FreeSoftInt		( UINT32 dSiMap );


#endif /*__MARESMGR_H__*/
