/****************************************************************************
 *
 *		Copyright (C) 2002-2003	YAMAHA CORPORATION. All rights reserved.
 *
 *		Module		: masnddva.c
 *
 *		Description	: DVA module for MA Sound Driver
 *
 *		Version		: 1.0.2 	2003.06.16
 *
 ****************************************************************************/

#include "masnddva.h"

/*--------------------------------------------------------------------------*/
/*   Defines                                                                */
/*--------------------------------------------------------------------------*/
#define	DVA_NUM_CH		16							/* Number of Channnels (== 16)		*/
#define	DVA_NUM_ALT		6							/* Number of Alternate Drums + 1	*/
#define DVA_NUM_FM		32							/* Number of FM slots (== 32)		*/
#define	DVA_NUM_WT		32							/* Number of WT slots (== 32)		*/
#define	DVA_NUM_STM		2							/* Number of STREAM slots (== 2)	*/
#define	DVA_NUM_KEY		128							/* Number of Keys					*/
#define DVA_NUM_MAP		(DVA_NUM_CH*DVA_NUM_KEY)	/* Number of PolyOnMaps				*/

#define DVA_PMAP_FM_ON	0x80
#define DVA_PMAP_WT_ON	0xC0
#define DVA_PMAP_FM_OFF	0x00
#define DVA_PMAP_WT_OFF	0x40

#define DVA_PMAP_MASK	0x7F
#define DVA_PMAP_ONOFF	0x80
#define DVA_PMAP_ON		0x80
#define DVA_PMAP_OFF	0x00

#define DVA_PMAP_SYNTH	0x40
#define DVA_PMAP_FM		0x00
#define DVA_PMAP_WT		0x40

#define DVA_PMAP_NUM	0x3F

/*--------------------------------------------------------------------------*/
/*   Types                                                                  */
/*--------------------------------------------------------------------------*/
typedef	enum _DVACHMODE		{DVA_MONO_CH = 0, DVA_POLY_CH, DVA_DRUM_CH}		DVACHMODE;
typedef	enum _DVASLOTMODE	{DVA_MONO_SLOT = 0, DVA_POLY_SLOT}				DVASLOTMODE;
typedef	enum _DVASYNTH		{DVA_FM_SYNTH = 0, DVA_WT_SYNTH}				DVASYNTH;
typedef	enum _DVASTREAM		{DVA_MONO_STREAM = 0, DVA_STEREO_STREAM}		DVASTREAM;
typedef	enum _DVAALT		{DVA_MA3_ALT = 0, DVA_MA5_ALT}					DVAALT;
typedef	enum _DVANOTEON		{DVA_NOTE_OFF = 0, DVA_NOTE_ON, DVA_NOTE_MUTE}	DVANOTEON;
typedef	enum _DVASTMON		{DVA_STREAM_OFF = 0, DVA_STREAM_ON}				DVASTMON ;

typedef	struct _tagSlotInfo{
	struct _tagSlotInfo*	pNext;
	struct _tagSlotInfo*	pPrev;
	UINT32					dKeyCh;				/* ((NoteOn Key) << 4) + (NoteOn Ch) */
	DVASLOTMODE				Mode;
	DVANOTEON				NoteOn;
	UINT32					dSlotId;
	UINT32					dIndex;
} SLOTINFO, *PSLOTINFO;

typedef	struct _tagChMap{
	DVACHMODE				Mode;
	DVASYNTH				MonoSynth;
	UINT32					dMonoSlot;
	SINT32					sdMonoOn;
} CHMAP, *PCHMAP;

typedef	struct _tagStreamInfo{
	DVASTMON				NoteOn;
	UINT32					dIndex;
	DVASTREAM				Stereo;
} STREAMINFO, *PSTREAMINFO;

typedef	struct _tagDvaGlobals{
	struct _tagSlotInfo		FmSlotInfo[DVA_NUM_FM + 3];
	struct _tagSlotInfo		WtSlotInfo[DVA_NUM_WT + 3];
	struct _tagStreamInfo	StreamSlotInfo[DVA_NUM_STM];
	struct _tagChMap		ChMap[DVA_NUM_CH];
	UINT8					bPolyOnMap[DVA_NUM_MAP];
	SINT32					sdAlt3Map[DVA_NUM_ALT];
	SINT32					sdAlt5Map[DVA_NUM_ALT * DVA_NUM_CH];

	struct _tagSlotInfo*	pFmSlotTop;
	struct _tagSlotInfo*	pWtSlotTop;
	struct _tagSlotInfo*	pFmSlotBottom;
	struct _tagSlotInfo*	pWtSlotBottom;
	struct _tagSlotInfo*	pFmSlotOnTop;
	struct _tagSlotInfo*	pWtSlotOnTop;

	UINT32					dStreamNextID;
	UINT32					dStreamNum;
	DVAALT					DrumAlt;
} DVAGLOBAL, *PDVAGLOBAL;

/*--------------------------------------------------------------------------*/
/*   Consts                                                                 */
/*--------------------------------------------------------------------------*/
static const UINT32	gdAltDrumKey[128] = {
						0, 1, 2, 3, 4, 5, 6, 7, 8, 9,
						10, 11, 12, 13, 14, 15, 16, 17, 18, 19,
						20, 21, 22, 23, 24, 25, 26, 27, 28, 29,
						30, 31, 32, 33, 34, 35, 36, 37, 38, 39,
						40, 41, 42, 43, 42, 45, 42, 47, 48, 49,
						50, 51, 52, 53, 54, 55, 56, 57, 58, 59,
						60, 61, 62, 63, 64, 65, 66, 67, 68, 69,
						70, 71, 71, 73, 73, 75, 76, 77, 78, 78,
						80, 80, 82, 83, 84, 85, 86, 87, 88, 89,
						90, 91, 92, 93, 94, 95, 96, 97, 98, 99,
						100, 101, 102, 103, 104, 105, 106, 107, 108, 109,
						110, 111, 112, 113, 114, 115, 116, 117, 118, 119,
						120, 121, 122, 123, 124, 125, 126, 127};

static const UINT8	gbIsAltDrum[128] = {
						0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
						0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
						0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
						0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
						0, 0, 1, 0, 1, 0, 1, 0, 0, 0,
						0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
						0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
						0, 2, 2, 3, 3, 0, 0, 0, 4, 4,
						5, 5, 0, 0, 0, 0, 0, 0, 0, 0,
						0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
						0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
						0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
						0, 0, 0, 0, 0, 0, 0, 0};

/*---------------------------------------------------------------------------*/
/*   Globals                                                                 */
/*---------------------------------------------------------------------------*/
static DVAGLOBAL	gDvaInfo;
static PDVAGLOBAL	gpDvaInfo;


/*---------------------------------------------------------------------------*/
/*   Functions (internal use only)                                           */
/*---------------------------------------------------------------------------*/

/****************************************************************************
 *	FmTurnOn
 *
 *	Description:
 *			Turn FM-slot on.
 *	Argument:
 *			pSlot		Slot, should be ON.
 *	Return:
 *			none
 ****************************************************************************/
static void FmTurnOn(PSLOTINFO pSlot)
{
	PSLOTINFO pTerget;

	pSlot->NoteOn = DVA_NOTE_ON;
	
	(pSlot->pPrev)->pNext = pSlot->pNext;
	(pSlot->pNext)->pPrev = pSlot->pPrev;
	
	pTerget = gpDvaInfo->pFmSlotBottom->pPrev;
	pTerget->pNext = pSlot;
	pSlot->pPrev = pTerget;
	pTerget = gpDvaInfo->pFmSlotBottom;
	pTerget->pPrev = pSlot;
	pSlot->pNext = pTerget;
}


/****************************************************************************
 *	FmTurnOff
 *
 *	Description:
 *			Turn FM-slot off.
 *	Argument:
 *			pSlot		Slot, should be OFF.
 *	Return:
 *			none
 ****************************************************************************/
static void FmTurnOff(PSLOTINFO pSlot)
{
	PSLOTINFO pTerget;

	if (pSlot->NoteOn != DVA_NOTE_ON)
	{
		return;
	}

	pSlot->NoteOn = DVA_NOTE_OFF;

	(pSlot->pPrev)->pNext = pSlot->pNext;
	(pSlot->pNext)->pPrev = pSlot->pPrev;

	pTerget = gpDvaInfo->pFmSlotOnTop->pPrev;
	pTerget->pNext = pSlot;
	pSlot->pPrev = pTerget;
	pTerget = gpDvaInfo->pFmSlotOnTop;
	pTerget->pPrev = pSlot;
	pSlot->pNext = pTerget;
}


/****************************************************************************
 *	FmMute
 *
 *	Description:
 *			Kill FM-slot.
 *	Argument:
 *			pSlot		Slot, should be MUTE.
 *	Return:
 *			none
 ****************************************************************************/
static void FmMute(PSLOTINFO pSlot)
{
	PSLOTINFO pTerget;

	pSlot->NoteOn = DVA_NOTE_MUTE;

	(pSlot->pPrev)->pNext = pSlot->pNext;
	(pSlot->pNext)->pPrev = pSlot->pPrev;

	pTerget = gpDvaInfo->pFmSlotTop->pNext;
	pTerget->pPrev = pSlot;
	pSlot->pNext = pTerget;
	pTerget = gpDvaInfo->pFmSlotTop;
	pTerget->pNext = pSlot;
	pSlot->pPrev = pTerget;
}


/****************************************************************************
 *	WtTurnOn
 *
 *	Description:
 *			Turn Wt-slot on.
 *	Argument:
 *			pSlot		Slot, should be ON.
 *	Return:
 *			none
 ****************************************************************************/
static void WtTurnOn(PSLOTINFO pSlot)
{
	PSLOTINFO pTerget;

	pSlot->NoteOn = DVA_NOTE_ON;
	
	(pSlot->pPrev)->pNext = pSlot->pNext;
	(pSlot->pNext)->pPrev = pSlot->pPrev;
	
	pTerget = gpDvaInfo->pWtSlotBottom->pPrev;
	pTerget->pNext = pSlot;
	pSlot->pPrev = pTerget;
	pTerget = gpDvaInfo->pWtSlotBottom;
	pTerget->pPrev = pSlot;
	pSlot->pNext = pTerget;
}


/****************************************************************************
 *	WtTurnOff
 *
 *	Description:
 *			Turn WT-slot off.
 *	Argument:
 *			pSlot		Slot, should be OFF.
 *	Return:
 *			none
 ****************************************************************************/
static void WtTurnOff(PSLOTINFO pSlot)
{
	PSLOTINFO pTerget;

	if (pSlot->NoteOn != DVA_NOTE_ON)
	{
		return;
	}

	pSlot->NoteOn = DVA_NOTE_OFF;

	(pSlot->pPrev)->pNext = pSlot->pNext;
	(pSlot->pNext)->pPrev = pSlot->pPrev;

	pTerget = gpDvaInfo->pWtSlotOnTop->pPrev;
	pTerget->pNext = pSlot;
	pSlot->pPrev = pTerget;
	pTerget = gpDvaInfo->pWtSlotOnTop;
	pTerget->pPrev = pSlot;
	pSlot->pNext = pTerget;
}


/****************************************************************************
 *	WtMute
 *
 *	Description:
 *			Kill WT-slot.
 *	Argument:
 *			pSlot		Slot, should be MUTE.
 *	Return:
 *			none
 ****************************************************************************/
static void WtMute(PSLOTINFO pSlot)
{
	PSLOTINFO pTerget;

	pSlot->NoteOn = DVA_NOTE_MUTE;

	(pSlot->pPrev)->pNext = pSlot->pNext;
	(pSlot->pNext)->pPrev = pSlot->pPrev;

	pTerget = gpDvaInfo->pWtSlotTop->pNext;
	pTerget->pPrev = pSlot;
	pSlot->pNext = pTerget;
	pTerget = gpDvaInfo->pWtSlotTop;
	pTerget->pNext = pSlot;
	pSlot->pPrev = pTerget;
}


/*---------------------------------------------------------------------------*/
/*   Functions                                                               */
/*---------------------------------------------------------------------------*/

/****************************************************************************
 *	MaDva_Initialize
 *
 *	Description:
 *			Initialize DVA module.
 *	Argument:
 *			dAl			Number of AL voices.		(0..1)
 *			dFm			Number of FM voices.		(16/32)
 *			dWt			Number of WT voices.		(6/7/8/16/32)
 *			dStream		Number of STREAM voices.	(0..2)
 *			dAlt		Alternate drum mode.		(0: MA-3, 1:MA-5)
 *	Return:
 *			MASMW_SUCCESS	if Success
 *			MASMW_ERROR		if Error
 ****************************************************************************/
SINT32 MaDva_Initialize(UINT32 dAl, UINT32 dFm, UINT32 dWt, UINT32 dStream, UINT32 dAlt)
{
	UINT32		i;
	PSLOTINFO	pSlot;
	PSTREAMINFO	pStm;
	PCHMAP		pCh;

	if (dAl > 1) return (MASMW_ERROR);
	if ((dFm != 16) && (dFm != 32)) return (MASMW_ERROR);
	if ((dWt != 6) && (dWt != 7) && (dWt != 8) && (dWt != 16) && (dWt != 32)) return (MASMW_ERROR);
	if (dStream > 2) return (MASMW_ERROR);
	if (dAlt > 1) return (MASMW_ERROR);

	gpDvaInfo = &gDvaInfo;
	gpDvaInfo->dStreamNum = dStream;
	gpDvaInfo->DrumAlt = (DVAALT)dAlt;
	
	for (i = 0; i < DVA_NUM_CH; i++)
	{
		pCh = &(gpDvaInfo->ChMap[i]);
		pCh->Mode = DVA_POLY_CH;
		pCh->MonoSynth = DVA_FM_SYNTH;
		pCh->sdMonoOn = 0;
		pCh->dMonoSlot = 0;
	}
	
	for (i = 0; i < DVA_NUM_MAP; i++)
	{
		gpDvaInfo->bPolyOnMap[i] = 0;
	}
	
	for (i = 0; i < DVA_NUM_ALT; i++)
	{
		gpDvaInfo->sdAlt3Map[i] = -1;
	}
	
	for (i = 0; i < DVA_NUM_ALT * DVA_NUM_CH; i++)
	{
		gpDvaInfo->sdAlt5Map[i] = -1;
	}
	
	for (i = 0; i < (DVA_NUM_FM + 3); i++)
	{
		pSlot = &(gpDvaInfo->FmSlotInfo[i]);
		pSlot->dKeyCh = 0;
		pSlot->pPrev = NULL;
		pSlot->pNext = NULL;
		pSlot->Mode = DVA_POLY_SLOT;
		pSlot->NoteOn = DVA_NOTE_MUTE;
		pSlot->dSlotId = 0;
		pSlot->dIndex = i;
	}
	
	pSlot = &(gpDvaInfo->FmSlotInfo[0]);
	pSlot->pPrev = NULL;
	pSlot->pNext = &(gpDvaInfo->FmSlotInfo[1]);
	
	pSlot = &(gpDvaInfo->FmSlotInfo[dFm - dAl + 2]);
	pSlot->pPrev = &(gpDvaInfo->FmSlotInfo[dFm - dAl + 1]);
	pSlot->pNext = NULL;
	
	for (i = 1; i < (dFm - dAl + 2); i++)
	{
		pSlot = &(gpDvaInfo->FmSlotInfo[i]);
		pSlot->pPrev = &(gpDvaInfo->FmSlotInfo[i - 1]);
		pSlot->pNext = &(gpDvaInfo->FmSlotInfo[i + 1]);
		pSlot->dSlotId = i + dAl - 1;
	}
	gpDvaInfo->pFmSlotTop = &(gpDvaInfo->FmSlotInfo[0]);
	gpDvaInfo->pFmSlotOnTop = &(gpDvaInfo->FmSlotInfo[dFm - dAl + 1]);
	gpDvaInfo->pFmSlotBottom = &(gpDvaInfo->FmSlotInfo[dFm - dAl + 2]);
	gpDvaInfo->pFmSlotOnTop->dSlotId = 0;
	
	for (i = 0; i < (DVA_NUM_WT + 3); i++)
	{
		pSlot = &(gpDvaInfo->WtSlotInfo[i]);
		pSlot->dKeyCh = 0;
		pSlot->pPrev = NULL;
		pSlot->pNext = NULL;
		pSlot->Mode = DVA_POLY_SLOT;
		pSlot->NoteOn = DVA_NOTE_MUTE;
		pSlot->dSlotId = 0;
		pSlot->dIndex = i;
	}
	
	pSlot = &(gpDvaInfo->WtSlotInfo[0]);
	pSlot->pPrev = NULL;
	pSlot->pNext = &(gpDvaInfo->WtSlotInfo[1]);
	
	pSlot = &(gpDvaInfo->WtSlotInfo[dWt - dAl + 2]);
	pSlot->pPrev = &(gpDvaInfo->WtSlotInfo[dWt - dAl + 1]);
	pSlot->pNext = NULL;
	
	for (i = 1; i < dWt - dAl + 2; i++)
	{
		pSlot = &(gpDvaInfo->WtSlotInfo[i]);
		pSlot->pPrev = &(gpDvaInfo->WtSlotInfo[i - 1]);
		pSlot->pNext = &(gpDvaInfo->WtSlotInfo[i + 1]);
		pSlot->dSlotId = i + dAl - 1;
	}
	gpDvaInfo->pWtSlotTop = &(gpDvaInfo->WtSlotInfo[0]);
	gpDvaInfo->pWtSlotOnTop = &(gpDvaInfo->WtSlotInfo[dWt - dAl + 1]);
	gpDvaInfo->pWtSlotBottom = &(gpDvaInfo->WtSlotInfo[dWt - dAl + 2]);
	gpDvaInfo->pWtSlotOnTop->dSlotId = 0;
	
	for (i = 0; i < DVA_NUM_STM; i++)
	{
		pStm = &(gpDvaInfo->StreamSlotInfo[i]);
		pStm->NoteOn = DVA_STREAM_OFF;
		pStm->Stereo = DVA_MONO_STREAM;
		pStm->dIndex = i;
	}
	gpDvaInfo->dStreamNextID = 0;
	
	return (MASMW_SUCCESS);
}


/****************************************************************************
 *	MaDva_End
 *
 *	Description:
 *			Treminate DVA module.
 *	Argument:
 *			none
 *	Return:
 *			MASMW_SUCCESS	if Success.
 *			MASMW_ERROR		if Error.
 ****************************************************************************/
SINT32 MaDva_End(void)
{
	return (MASMW_SUCCESS);
}


/****************************************************************************
 *	MaDva_GetAlt3Slot
 *
 *	Description:
 *			Get Alternate Info form NoteOn List.
 *	Argument:
 *			dKey		NoteOn Key# (0..127)
 *	Return:
 *			0 or Positive	if Success.	(Key << 4) + Ch)
 *											Key = 0..127
 *											Ch  = 0..15
 *			MASMW_ERROR		if Error.
 ****************************************************************************/
SINT32 MaDva_GetAlt3Slot(UINT32 dKey)
{
	SINT32	sdId;
	
	sdId = gpDvaInfo->sdAlt3Map[gbIsAltDrum[dKey & 0x7f]];
	
	return (sdId);
}


/****************************************************************************
 *	MaDva_GetFmSlot
 *
 *	Description:
 *			Get new FM slot.
 *	Argument:
 *			dCh			NoteOn Ch#	(0..15)
 *			dKey		NoteOn Key#	(0..127)
 *			dMode		NoteOn Mode	(0:Mono, 1:Poly, 2:Drum)
 *	Return:
 *			0 or Positive	if Success.
 *
 *			 31           14 13.......8 7....6 5......0
 *			+---------------+----------+------+--------+
 *          |       0       | OffSlot  | Mode | OnSlot |
 *			+---------------+----------+------+--------+
 *							OffSlot : WT SLOT#, should be MUTE (0..31)
 *							Mode    : 0: New, 1: Continue, 2:Mute&New
 *							OnSlot  : FM SLOT# (0..31)
 *
 *			MASMW_ERROR		if Error.
 ****************************************************************************/
SINT32 MaDva_GetFmSlot(UINT32 dCh, UINT32 dKey, UINT32 dMode)
{
	UINT32		dOnId;
	SINT32		sdId;
	PCHMAP		pCh;
	PSLOTINFO	pSlot;

	sdId = 0;
	pCh = &(gpDvaInfo->ChMap[dCh]);
	
	switch (dMode)
	{
	case 2:		/* DVA_DRUM_CH */
		if (gbIsAltDrum[dKey] > 0)
		{
			if (gpDvaInfo->DrumAlt == DVA_MA5_ALT)
			{
				gpDvaInfo->sdAlt5Map[(gbIsAltDrum[dKey] << 4) + dCh] = (SINT32)((dKey << 4) + dCh);
				dKey = gdAltDrumKey[dKey];
			}
			else
			{
				gpDvaInfo->sdAlt3Map[gbIsAltDrum[dKey]] = (SINT32)((dKey << 4) + dCh);
			}
		}
		/* fall through */

	case 1:		/* DVA_POLY_CH */
		dOnId = (dKey << 4) + dCh;
		if (pCh->Mode == DVA_MONO_CH)
		{
			if (pCh->sdMonoOn > 0)
			{
				pCh->sdMonoOn = 0;
				if (pCh->MonoSynth == DVA_FM_SYNTH)
				{	/* ERROR: Playing MONO.. */
					pCh->Mode = (DVACHMODE)dMode;
					pSlot = &(gpDvaInfo->FmSlotInfo[pCh->dMonoSlot]);
					pSlot->Mode = DVA_POLY_SLOT;
					pSlot->dKeyCh = dOnId;
					sdId = (SINT32)pSlot->dSlotId;
					FmTurnOn(pSlot);
					gpDvaInfo->bPolyOnMap[dOnId] = (UINT8)(pSlot->dIndex + DVA_PMAP_FM_ON);
					return (sdId);
				}
				else
				{	/* ERROR: Playing WT.. */
					pSlot = &(gpDvaInfo->WtSlotInfo[pCh->dMonoSlot]);
					sdId = (SINT32)((pSlot->dSlotId << 8) + 0x80);
					WtTurnOff(pSlot);
				}
			}
		}

		pCh->Mode = (DVACHMODE)dMode;
		switch (gpDvaInfo->bPolyOnMap[dOnId] & 0xC0)
		{
		case DVA_PMAP_FM_ON:	/* FM-On */
			pSlot = &(gpDvaInfo->FmSlotInfo[gpDvaInfo->bPolyOnMap[dOnId] & DVA_PMAP_NUM]);
			sdId = (SINT32)pSlot->dSlotId;
			FmTurnOn(pSlot);
			return (sdId);

		case DVA_PMAP_WT_ON:	/* Wt-On */
			pSlot= &(gpDvaInfo->WtSlotInfo[gpDvaInfo->bPolyOnMap[dOnId] & DVA_PMAP_NUM]);
			sdId = (pSlot->dSlotId << 8) + 0x80;
			WtTurnOff(pSlot);
			/* fall through */
		
		default:	/* Off */
			pSlot = gpDvaInfo->pFmSlotTop->pNext;
			if (pSlot == gpDvaInfo->pFmSlotOnTop) pSlot = pSlot->pNext;
			sdId += pSlot->dSlotId;
			if (pSlot->NoteOn == DVA_NOTE_ON)
			{
				if (pSlot->Mode == DVA_POLY_SLOT)
				{
					gpDvaInfo->bPolyOnMap[pSlot->dKeyCh] &= ~DVA_PMAP_ON;
				}
				else
				{
					gpDvaInfo->ChMap[pSlot->dKeyCh & 0x0F].sdMonoOn = 0;
				}
			}
			FmTurnOn(pSlot);
			gpDvaInfo->bPolyOnMap[dOnId] = (UINT8)(pSlot->dIndex + DVA_PMAP_FM_ON);
			pSlot->dKeyCh = dOnId;
			pSlot->Mode = DVA_POLY_SLOT;
			return (sdId);
		}
		break;

	default:	/* DVA_MONO_CH */
		if (pCh->Mode == DVA_MONO_CH)
		{
			if (pCh->sdMonoOn > 0)
			{
				if (pCh->MonoSynth == DVA_FM_SYNTH)
				{
					pSlot = &(gpDvaInfo->FmSlotInfo[pCh->dMonoSlot]);
					sdId = pSlot->dSlotId + 0x40;
					pCh->sdMonoOn++;
					FmTurnOn(pSlot);
					return (sdId);
				}
				else
				{	/* ERROR: WT mono */
					pSlot = &(gpDvaInfo->WtSlotInfo[pCh->dMonoSlot]);
					sdId = (SINT32)((pSlot->dSlotId << 8) + 0x80);
					WtTurnOff(pSlot);
				}
			}
		}
		
		dOnId = (dKey << 4) + dCh;
		gpDvaInfo->bPolyOnMap[dOnId] = 0;
		
		pSlot = gpDvaInfo->pFmSlotTop->pNext;
		if (pSlot == gpDvaInfo->pFmSlotOnTop) pSlot = pSlot->pNext;
		sdId += pSlot->dSlotId;

		if (pSlot->NoteOn == DVA_NOTE_ON)
		{
			if (pSlot->Mode == DVA_POLY_SLOT)
			{
				gpDvaInfo->bPolyOnMap[pSlot->dKeyCh] &= ~DVA_PMAP_ON;
			}
			else
			{
				gpDvaInfo->ChMap[pSlot->dKeyCh & 0x0F].sdMonoOn = 0;
			}
		}

		pCh->Mode = DVA_MONO_CH;
		pCh->MonoSynth = DVA_FM_SYNTH;
		pCh->sdMonoOn = 1;

		FmTurnOn(pSlot);
		pSlot->dKeyCh = dOnId;
		pSlot->Mode = DVA_MONO_SLOT;
		pCh->dMonoSlot = pSlot->dIndex;
		return (sdId);
	}
	return (MASMW_ERROR);
}


/****************************************************************************
 *	MaDva_GetWtSlot
 *
 *	Description:
 *			Get new WT slot.
 *	Argument:
 *			dCh			NoteOn Ch#	(0..15)
 *			dKey		NoteOn Key#	(0..127)
 *			dMode		NoteOn Mode	(0:Mono, 1:Poly, 2:Drum)
 *	Return:
 *			0 or Positive	if Success.
 *
 *			 31           14 13.......8 7....6 5......0
 *			+---------------+----------+------+--------+
 *          |       0       | OffSlot  | Mode | OnSlot |
 *			+---------------+----------+------+--------+
 *							OffSlot : FM SLOT#, should be MUTE (0..31)
 *							Mode    : 0: New, 1: Continue, 2:Mute&New
 *							OnSlot  : WT SLOT# (0..31)
 *
 *			MASMW_ERROR		if Error.
 ****************************************************************************/
SINT32 MaDva_GetWtSlot(UINT32 dCh, UINT32 dKey, UINT32 dMode)
{
	UINT32		dOnId;
	SINT32		sdId;
	PCHMAP		pCh;
	PSLOTINFO	pSlot;
	
	sdId = 0;
	pCh = &(gpDvaInfo->ChMap[dCh]);
	
	switch (dMode)
	{
	case 2:		/* DVA_DRUM_CH */
		if (gbIsAltDrum[dKey] > 0)
		{
			if (gpDvaInfo->DrumAlt == DVA_MA5_ALT)
			{
				gpDvaInfo->sdAlt5Map[(gbIsAltDrum[dKey] << 4) + dCh] = (SINT32)((dKey << 4) + dCh);
				dKey = gdAltDrumKey[dKey];
			}
			else
			{
				gpDvaInfo->sdAlt3Map[gbIsAltDrum[dKey]] = (SINT32)((dKey << 4) + dCh);
			}
		}
		/* fall through */

	case 1:		/* DVA_POLY_CH */
		dOnId = (dKey << 4) + dCh;
		if (pCh->Mode == DVA_MONO_CH)
		{
			if (pCh->sdMonoOn > 0)
			{
				pCh->sdMonoOn = 0;
				if (pCh->MonoSynth == DVA_WT_SYNTH)
				{	/* ERROR: Playing MONO.. */
					pCh->Mode = (DVACHMODE)dMode;
					pSlot = &(gpDvaInfo->WtSlotInfo[pCh->dMonoSlot]);
					pSlot->Mode = DVA_POLY_SLOT;
					pSlot->dKeyCh = dOnId;
					sdId = (SINT32)pSlot->dSlotId;
					WtTurnOn(pSlot);
					gpDvaInfo->bPolyOnMap[dOnId] = (UINT8)(pSlot->dIndex + DVA_PMAP_WT_ON);
					return (sdId);
				}
				else
				{	/* ERROR: Playing FM.. */
					pSlot = &(gpDvaInfo->FmSlotInfo[pCh->dMonoSlot]);
					sdId = (SINT32)((pSlot->dSlotId << 8) + 0x80);
					FmTurnOff(pSlot);
				}
			}
		}

		pCh->Mode = (DVACHMODE)dMode;
		switch (gpDvaInfo->bPolyOnMap[dOnId] & 0xC0)
		{
		case DVA_PMAP_WT_ON:	/* WT-On */
			pSlot = &(gpDvaInfo->WtSlotInfo[gpDvaInfo->bPolyOnMap[dOnId] & DVA_PMAP_NUM]);
			sdId = (SINT32)pSlot->dSlotId;
			WtTurnOn(pSlot);
			return (sdId);

		case DVA_PMAP_FM_ON:	/* FM-On */
			pSlot= &(gpDvaInfo->FmSlotInfo[gpDvaInfo->bPolyOnMap[dOnId] & DVA_PMAP_NUM]);
			sdId = (pSlot->dSlotId << 8) + 0x80;
			FmTurnOff(pSlot);
			/* fall through */
		
		default:	/* Off */
			pSlot = gpDvaInfo->pWtSlotTop->pNext;
			if (pSlot == gpDvaInfo->pWtSlotOnTop) pSlot = pSlot->pNext;
			sdId += pSlot->dSlotId;
			if (pSlot->NoteOn == DVA_NOTE_ON)
			{
				if (pSlot->Mode == DVA_POLY_SLOT)
				{
					gpDvaInfo->bPolyOnMap[pSlot->dKeyCh] &= ~DVA_PMAP_ON;
				}
				else
				{
					gpDvaInfo->ChMap[pSlot->dKeyCh & 0x0F].sdMonoOn = 0;
				}
			}
			gpDvaInfo->bPolyOnMap[dOnId] = (UINT8)(pSlot->dIndex + DVA_PMAP_WT_ON);
			WtTurnOn(pSlot);
			pSlot->dKeyCh = dOnId;
			pSlot->Mode = DVA_POLY_SLOT;
			return (sdId);
		}

	default:	/* DVA_MONO_CH */
		if (pCh->Mode == DVA_MONO_CH)
		{
			if (pCh->sdMonoOn > 0)
			{
				if (pCh->MonoSynth == DVA_WT_SYNTH)
				{
					pSlot = &(gpDvaInfo->WtSlotInfo[pCh->dMonoSlot]);
					sdId = pSlot->dSlotId + 0x40;
					pCh->sdMonoOn++;
					FmTurnOn(pSlot);
					return (sdId);
				}
				else
				{	/* ERROR: FM mono */
					pSlot = &(gpDvaInfo->FmSlotInfo[pCh->dMonoSlot]);
					sdId = (SINT32)((pSlot->dSlotId << 8) + 0x80);
					FmTurnOff(pSlot);
				}
			}
		}
		
		dOnId = (dKey << 4) + dCh;
		gpDvaInfo->bPolyOnMap[dOnId] = 0;
		
		pSlot = gpDvaInfo->pWtSlotTop->pNext;
		if (pSlot == gpDvaInfo->pWtSlotOnTop) pSlot = pSlot->pNext;
		sdId += pSlot->dSlotId;
		if (pSlot->NoteOn == DVA_NOTE_ON)
		{
			if (pSlot->Mode == DVA_POLY_SLOT)
			{
				gpDvaInfo->bPolyOnMap[pSlot->dKeyCh] &= ~DVA_PMAP_ON;
			}
			else
			{
				gpDvaInfo->ChMap[pSlot->dKeyCh & 0x0F].sdMonoOn = 0;
			}
		}

		pCh->Mode = DVA_MONO_CH;
		pCh->MonoSynth = DVA_WT_SYNTH;
		pCh->sdMonoOn = 1;

		pSlot->dKeyCh = dOnId;
		WtTurnOn(pSlot);
		pSlot->Mode = DVA_MONO_SLOT;
		pCh->dMonoSlot = pSlot->dIndex;
		return (sdId);
	}
	return (MASMW_ERROR);
}


/****************************************************************************
 *	MaDva_GetStreamSlot
 *
 *	Description:
 *			Get new STREAM slot.
 *	Argument:
 *			dCh			NoteOn Ch#	(0..15)
 *			dKey		NoteOn Key#	(0..31)
 *			dMode		NoteOn Mode	(0:MONO, 1:STEREO)
 *	Return:
 *			0 or 1			STREAM SLOT# (0..1)
 *			MASMW_ERROR		if Error.
 ****************************************************************************/
SINT32 MaDva_GetStreamSlot(UINT32 dCh, UINT32 dKey, UINT32 dMode)
{
	UINT32		dOnId;
	SINT32		sdId;
	PSTREAMINFO	pSlot;
	
	sdId = 0;
	dOnId = (dKey << 4) + dCh;
	
	switch (gpDvaInfo->dStreamNum)
	{
	case 2:
		if (dMode == 0)
		{	/* MONO */
			sdId = gpDvaInfo->dStreamNextID;
			pSlot = &(gpDvaInfo->StreamSlotInfo[sdId]);
			pSlot->dIndex = dOnId;
			pSlot->NoteOn = DVA_STREAM_ON;
			pSlot->Stereo = DVA_MONO_STREAM;
			gpDvaInfo->dStreamNextID = (1L - sdId);
			return (sdId);
		}
		else
		{	/* STEREO */
			sdId = 0;
			pSlot = &(gpDvaInfo->StreamSlotInfo[sdId]);
			pSlot->dIndex = dOnId;
			pSlot->NoteOn = DVA_STREAM_ON;
			pSlot->Stereo = DVA_STEREO_STREAM;
			gpDvaInfo->StreamSlotInfo[1].NoteOn = DVA_STREAM_OFF;
			gpDvaInfo->dStreamNextID = 0;
			return (sdId);
		}
		break;

	case 1:
		if (dMode == 0)
		{	/* MONO */
			sdId = 0;
			pSlot = &(gpDvaInfo->StreamSlotInfo[sdId]);
			pSlot->dIndex = dOnId;
			pSlot->NoteOn = DVA_STREAM_ON;
			pSlot->Stereo = DVA_MONO_STREAM;
			return (sdId);
		}
		else
		{	/*STEREO */
			sdId = 0;
			pSlot = &(gpDvaInfo->StreamSlotInfo[sdId]);
			pSlot->dIndex = dOnId;
			pSlot->NoteOn = DVA_STREAM_ON;
			pSlot->Stereo = DVA_STEREO_STREAM;
			return (sdId);
		}
		break;
	}
	return (MASMW_ERROR);
}


/****************************************************************************
 *	MaDva_ReleaseSlot
 *
 *	Description:
 *			Release slot.
 *	Argument:
 *			dCh			NoteOff Ch#	(0..15)
 *			dKey		NoteOff Key#	(0..127)
 *	Return:
 *			0 or Positive	SLOT MAP
 *
 *			 31....................6    5    4.......0
 *			+-----------------------+-------+---------+
 *          |           0           | Synth | OffSlot |
 *			+-----------------------+-------+---------+
 *							Synth    : 0: FM, 1: WT
 *							OnSlot  : SLOT# (0..31)
 *
 *			MASMW_ERROR		if Error.
 ****************************************************************************/
SINT32 MaDva_ReleaseSlot(UINT32 dCh, UINT32 dKey)
{
	UINT32		dOffId;
	SINT32		sdId;
	PCHMAP		pCh;
	PSLOTINFO	pSlot;
	UINT8*		pbIx;
	UINT32		dOrgId;

	sdId = 0;
	pCh = &(gpDvaInfo->ChMap[dCh]);
	
	switch (pCh->Mode)
	{
	case DVA_DRUM_CH:
		if (gpDvaInfo->DrumAlt == DVA_MA5_ALT)
		{
			dOrgId = (gbIsAltDrum[dKey] << 4) + dCh;
			if (dOrgId > 16)
			{
				if (gpDvaInfo->sdAlt5Map[dOrgId] != (SINT32)((dKey << 4) + dCh))
				{
					return (MASMW_ERROR);		/* Nothing to be OFF */
				}
			}
			dKey = gdAltDrumKey[dKey];
			dOffId = (dKey << 4) + dCh;
			pbIx = &(gpDvaInfo->bPolyOnMap[dOffId]);
			if ((*pbIx & DVA_PMAP_ONOFF) == DVA_PMAP_OFF)
			{
				return (MASMW_ERROR);
			}
			
			gpDvaInfo->sdAlt5Map[dOrgId] = -1;
		}
		else
		{
			dOffId = (dKey << 4) + dCh;
			pbIx = &(gpDvaInfo->bPolyOnMap[dOffId]);
			if ((*pbIx & DVA_PMAP_ONOFF) == DVA_PMAP_OFF)
			{
				return (MASMW_ERROR);
			}
			
			if (gbIsAltDrum[dKey] > 0) gpDvaInfo->sdAlt3Map[gbIsAltDrum[dKey]] = -1;
		}
		
		*pbIx &= ~DVA_PMAP_ON;
		if ((*pbIx & DVA_PMAP_SYNTH) == DVA_PMAP_FM)
		{
			pSlot = &(gpDvaInfo->FmSlotInfo[*pbIx & DVA_PMAP_NUM]);
			sdId = (SINT32)pSlot->dSlotId;
			FmTurnOff(pSlot);
			return (sdId);
		}
		else
		{
			pSlot = &(gpDvaInfo->WtSlotInfo[*pbIx & DVA_PMAP_NUM]);
			sdId = (SINT32)pSlot->dSlotId + 32;
			WtTurnOff(pSlot);
			return (sdId);
		}
		break;
	
	case DVA_POLY_CH:
		dOffId = (dKey << 4) + dCh;
		pbIx = &(gpDvaInfo->bPolyOnMap[dOffId]);
		if ((*pbIx & DVA_PMAP_ONOFF) == DVA_PMAP_OFF)
		{
			return (MASMW_ERROR);
		}
		*pbIx &= ~DVA_PMAP_ON;
		if ((*pbIx & DVA_PMAP_SYNTH) == DVA_PMAP_FM)
		{
			pSlot = &(gpDvaInfo->FmSlotInfo[*pbIx & DVA_PMAP_NUM]);
			sdId = (SINT32)pSlot->dSlotId;
			FmTurnOff(pSlot);
			return (sdId);
		}
		else
		{
			pSlot = &(gpDvaInfo->WtSlotInfo[*pbIx & DVA_PMAP_NUM]);
			sdId = (SINT32)pSlot->dSlotId + 32;
			WtTurnOff(pSlot);
			return (sdId);
		}
		break;

	default:	/* DVA_MONO_CH */
		if (pCh->sdMonoOn <= 0)
		{
			return (MASMW_ERROR);		/* Nothing to be OFF */
		}
		pCh->sdMonoOn--;
		if (pCh->sdMonoOn > 0) return (MASMW_ERROR);		/* Contine */
		if (pCh->MonoSynth == DVA_FM_SYNTH)
		{
			pSlot = &(gpDvaInfo->FmSlotInfo[pCh->dMonoSlot]);
			sdId = (SINT32)pSlot->dSlotId;
			FmTurnOff(pSlot);
			return (sdId);
		}
		else
		{
			pSlot = &(gpDvaInfo->WtSlotInfo[pCh->dMonoSlot]);
			sdId = (SINT32)pSlot->dSlotId + 32;
			WtTurnOff(pSlot);
			return (sdId);
		}
	}
	return (MASMW_ERROR);
}


/****************************************************************************
 *	MaDva_ReleaseFmOnSlot
 *
 *	Description:
 *			Release FM slot which is ON.
 *	Argument:
 *			dCh			NoteOff Ch#	(0..15)
 *	Return:
 *			FM-SLOT Off Map
 *
 *			    31    .................      1      0
 *			+-------+---------------------+------+------+
 *          | FM#31 |                     | FM#1 | FM#0 |
 *			+-------+---------------------+------+------+
 *							FM#n    : 0: none, 1: OFF
 *
 *			MASMW_ERROR		if Error.
 ****************************************************************************/
UINT32 MaDva_ReleaseFmOnSlot(UINT32 dCh)
{
	UINT32		dOffMap;
	PSLOTINFO	pSlot;
	PSLOTINFO	pNext;
	UINT32		dIndex;
	
	dOffMap = 0;
	pSlot = gpDvaInfo->pFmSlotOnTop->pNext;
	
	while (pSlot != gpDvaInfo->pFmSlotBottom)
	{
		pNext = pSlot->pNext;
		if ((pSlot->dKeyCh & 0x0F) == dCh)
		{
			switch (pSlot->Mode)
			{
			case DVA_POLY_SLOT:
				gpDvaInfo->bPolyOnMap[pSlot->dKeyCh] &= ~DVA_PMAP_ONOFF;
				if (gpDvaInfo->ChMap[pSlot->dKeyCh & 0x0F].Mode == DVA_DRUM_CH)
				{
					dIndex = gbIsAltDrum[(pSlot->dKeyCh >> 4) & 0x7F];
					gpDvaInfo->sdAlt3Map[dIndex] = -1;
					gpDvaInfo->sdAlt5Map[(dIndex << 4) + (pSlot->dKeyCh & 0x0F)] = -1;
				}
				FmTurnOff(pSlot);
				break;
				
			default: /* DVA_MONO_SLOT */
				gpDvaInfo->ChMap[pSlot->dKeyCh & 0x0F].sdMonoOn = 0;
				FmTurnOff(pSlot);
			}
			dOffMap |= (1L << pSlot->dSlotId);
		}
		pSlot = pNext;
	}
	return (dOffMap);
}


/****************************************************************************
 *	MaDva_ReleaseWtOnSlot
 *
 *	Description:
 *			Release WT slot which is ON.
 *	Argument:
 *			dCh			NoteOff Ch#	(0..15)
 *	Return:
 *			WT-SLOT Off Map
 *
 *			    31    .................      1      0
 *			+-------+---------------------+------+------+
 *          | WT#31 |                     | WT#1 | WT#0 |
 *			+-------+---------------------+------+------+
 *							WT#n    : 0: none, 1: OFF
 ****************************************************************************/
UINT32 MaDva_ReleaseWtOnSlot(UINT32 dCh)
{
	UINT32		dOffMap;
	PSLOTINFO	pSlot;
	PSLOTINFO	pNext;
	UINT32		dIndex;
	
	dOffMap = 0;
	pSlot = gpDvaInfo->pWtSlotOnTop->pNext;
	
	while (pSlot != gpDvaInfo->pWtSlotBottom)
	{
		pNext = pSlot->pNext;
		if ((pSlot->dKeyCh & 0x0F) == dCh)
		{
			switch (pSlot->Mode)
			{
			case DVA_POLY_SLOT:
				gpDvaInfo->bPolyOnMap[pSlot->dKeyCh] &= ~DVA_PMAP_ONOFF;
				if (gpDvaInfo->ChMap[pSlot->dKeyCh & 0x0F].Mode == DVA_DRUM_CH)
				{
					dIndex = gbIsAltDrum[(pSlot->dKeyCh >> 4) & 0x7F];
					gpDvaInfo->sdAlt3Map[dIndex] = -1;
					gpDvaInfo->sdAlt5Map[(dIndex << 4) + (pSlot->dKeyCh & 0x0F)] = -1;
				}
				WtTurnOff(pSlot);
				break;
				
			default: /* DVA_MONO_SLOT */
				gpDvaInfo->ChMap[pSlot->dKeyCh & 0x0F].sdMonoOn = 0;
				WtTurnOff(pSlot);
			}
			dOffMap |= (1L << pSlot->dSlotId);
		}
		pSlot = pNext;
	}
	return (dOffMap);
}


/****************************************************************************
 *	MaDva_ReleaseFmAllSlot
 *
 *	Description:
 *			Release FM slot.
 *	Argument:
 *			dCh			Mute Ch#	(0..15)
 *	Return:
 *			FM-SLOT Off Map
 *
 *			    31    .................      1      0
 *			+-------+---------------------+------+------+
 *          | FM#31 |                     | FM#1 | FM#0 |
 *			+-------+---------------------+------+------+
 *							FM#n    : 0: none, 1: OFF
 ****************************************************************************/
UINT32 MaDva_ReleaseFmAllSlot(UINT32 dCh)
{
	UINT32		dOffMap;
	PSLOTINFO	pSlot;
	PSLOTINFO	pNext;
	UINT32		dIndex;
	
	dOffMap = 0;
	pSlot = gpDvaInfo->pFmSlotTop->pNext;
	if (pSlot == gpDvaInfo->pFmSlotOnTop) pSlot = pSlot->pNext;

	while (pSlot != gpDvaInfo->pFmSlotBottom)
	{
		pNext = pSlot->pNext;
		if (pNext == gpDvaInfo->pFmSlotOnTop) pNext = pNext->pNext;
		if ((pSlot->dKeyCh & 0x0F) == dCh)
		{
			if (pSlot->NoteOn != DVA_NOTE_MUTE)
			{
				switch (pSlot->Mode)
				{
				case DVA_POLY_SLOT:
					gpDvaInfo->bPolyOnMap[pSlot->dKeyCh] &= ~DVA_PMAP_ONOFF;
					if (gpDvaInfo->ChMap[pSlot->dKeyCh & 0x0F].Mode == DVA_DRUM_CH)
					{
						dIndex = gbIsAltDrum[(pSlot->dKeyCh >> 4) & 0x7F];
						gpDvaInfo->sdAlt3Map[dIndex] = -1;
						gpDvaInfo->sdAlt5Map[(dIndex << 4) + (pSlot->dKeyCh & 0x0F)] = -1;
					}
					FmMute(pSlot);
					break;
					
				default: /* DVA_MONO_SLOT */
					gpDvaInfo->ChMap[pSlot->dKeyCh & 0x0F].sdMonoOn = 0;
					FmMute(pSlot);
					break;
				}
				dOffMap |= (1L << pSlot->dSlotId);
			}
		}
		pSlot = pNext;
	}
	return (dOffMap);
}


/****************************************************************************
 *	MaDva_ReleaseWtAllSlot
 *
 *	Description:
 *			Release WT slot.
 *	Argument:
 *			dCh			Mute Ch#	(0..15)
 *	Return:
 *			WT-SLOT Off Map
 *
 *			    31    .................      1      0
 *			+-------+---------------------+------+------+
 *          | WT#31 |                     | WT#1 | WT#0 |
 *			+-------+---------------------+------+------+
 *							WT#n    : 0: none, 1: OFF
 ****************************************************************************/
UINT32 MaDva_ReleaseWtAllSlot(UINT32 dCh)
{
	UINT32		dOffMap;
	PSLOTINFO	pSlot;
	PSLOTINFO	pNext;
	UINT32		dIndex;
	
	dOffMap = 0;
	pSlot = gpDvaInfo->pWtSlotTop->pNext;
	if (pSlot == gpDvaInfo->pWtSlotOnTop) pSlot = pSlot->pNext;
	gpDvaInfo->ChMap[pSlot->dKeyCh & 0x0F].sdMonoOn = 0;
	
	while (pSlot != gpDvaInfo->pWtSlotBottom)
	{
		pNext = pSlot->pNext;
		if (pNext == gpDvaInfo->pWtSlotOnTop) pNext = pNext->pNext;
		if ((pSlot->dKeyCh & 0x0F) == dCh)
		{
			if (pSlot->NoteOn != DVA_NOTE_MUTE)
			{
				if (pSlot->Mode == DVA_POLY_SLOT)
				{
					gpDvaInfo->bPolyOnMap[pSlot->dKeyCh] &= DVA_PMAP_MASK;
					if (gpDvaInfo->ChMap[pSlot->dKeyCh & 0x0F].Mode == DVA_DRUM_CH)
					{
						dIndex = gbIsAltDrum[(pSlot->dKeyCh >> 4) & 0x7F];
						gpDvaInfo->sdAlt3Map[dIndex] = -1;
						gpDvaInfo->sdAlt5Map[(dIndex << 4) + (pSlot->dKeyCh & 0x0F)] = -1;
					}
				}
				WtMute(pSlot);
				dOffMap |= (1L << pSlot->dSlotId);
			}
		}
		pSlot = pNext;
	}
	return (dOffMap);
}


/****************************************************************************
 *	MaDva_ReleaseStreamSlot
 *
 *	Description:
 *			Release STREAM slot.
 *	Argument:
 *			dCh			StreamOff Ch#	(0..15)
 *			dKey		StreamOff Key#	(0..31)
 *	Return:
 *			0..1			Stream Slot#
 *			MASMW_ERROR		if Error.
 ****************************************************************************/
SINT32 MaDva_ReleaseStreamSlot(UINT32 dCh, UINT32 dKey)
{
	SINT32		sdId;
	UINT32		dOffId;
	PSTREAMINFO	pSlot;
	
	if (gpDvaInfo->dStreamNum == 0) return (MASMW_ERROR);
	dOffId = (dKey << 4) + dCh;

	sdId = 0;
	pSlot = &(gpDvaInfo->StreamSlotInfo[sdId]);
	if (pSlot->NoteOn == DVA_STREAM_ON)
	{
		if (pSlot->dIndex == dOffId)
		{
			pSlot->NoteOn = DVA_STREAM_OFF;
			gpDvaInfo->dStreamNextID = (UINT32)sdId;
			return (sdId);
		}
	}
	if (gpDvaInfo->dStreamNum == 1) return (MASMW_ERROR);
	sdId = 1;
	pSlot = &(gpDvaInfo->StreamSlotInfo[sdId]);
	if (pSlot->NoteOn == DVA_STREAM_ON)
	{
		if (pSlot->dIndex == dOffId)
		{
			pSlot->NoteOn = DVA_STREAM_OFF;
			gpDvaInfo->dStreamNextID = (UINT32)sdId;
			return (sdId);
		}
	}
	return (MASMW_ERROR);
}


/****************************************************************************
 *	MaDva_ReleaseStreamAllSlot
 *
 *	Description:
 *			Release STREAM slot.
 *	Argument:
 *			dCh			StreamOff Ch#	(0..15)
 *	Return:
 *			STREAM-SLOT Off Map
 *
 *			    31    .................      1      0
 *			+-----------------------------+------+------+
 *          |            0                | ST#1 | ST#0 |
 *			+-----------------------------+------+------+
 *							STREAM#n    : 0: none, 1: OFF
 ****************************************************************************/
UINT32 MaDva_ReleaseStreamAllSlot(UINT32 dCh)
{
	UINT32		dOffMap;
	PSTREAMINFO	pSlot;
	
	dOffMap = 0;
	if (gpDvaInfo->dStreamNum == 0) return (dOffMap);

	pSlot = &(gpDvaInfo->StreamSlotInfo[0]);
	if (pSlot->NoteOn == DVA_STREAM_ON)
	{
		if ((pSlot->dIndex & 0x0F) == dCh)
		{
			pSlot->NoteOn = DVA_STREAM_OFF;
			gpDvaInfo->dStreamNextID = 0;
			dOffMap = 1;
		}
	}
	
	if (gpDvaInfo->dStreamNum == 1) return (dOffMap);

	pSlot = &(gpDvaInfo->StreamSlotInfo[1]);
	if (pSlot->NoteOn == DVA_STREAM_ON)
	{
		if ((pSlot->dIndex & 0x0F) == dCh)
		{
			pSlot->NoteOn = DVA_STREAM_OFF;
			gpDvaInfo->dStreamNextID = 1;
			dOffMap += 2;
		}
	}
	return (dOffMap);
}


/****************************************************************************
 *	MaDva_GetStreamKey
 *
 *	Description:
 *			Release STREAM slot.
 *	Argument:
 *			dSlot			Slot#	(0..1)
 *	Return:
 *			0..31			Key
 ****************************************************************************/
UINT32 MaDva_GetStreamKey(UINT32 dSlot)
{
	UINT32		dKey;
	
	dKey = (gpDvaInfo->StreamSlotInfo[dSlot].dIndex >> 4) & 0x1F;
	
	return (dKey);
}

