/****************************************************************************
 *
 *		Copyright (C) 2002-2003	YAMAHA CORPORATION. All rights reserved.
 *
 *		Module		: masndseq.c
 *
 *		Description	: MA Sound Sequencer
 *
 *		Version		: 2.1.9 	2003.07.30
 *
 ****************************************************************************/

#include "mamachdep.h"
#include "madefs.h"
#include "masndseq.h"
#include "masnddrv.h"
#include "madevdrv.h"
#include "maresmgr.h"
#include "macnvprf.h"

#define MASNDSEQ_MAX_PLAYMODE	(255)
#define MASNDSEQ_STATUS_USABLE	(0x01)
#define MASNDSEQ_STATUS_NOTINIT	(0xFF)
/*
#define	MaSound_Initialize		__MaSound_Initialize
#define	MaSound_DeviceControl	__MaSound_DeviceControl
#define	MaSound_Create			__MaSound_Create
#define	MaSound_Load			__MaSound_Load
#define	MaSound_Open			__MaSound_Open
#define	MaSound_Control			__MaSound_Control
#define	MaSound_Standby			__MaSound_Standby
#define	MaSound_Seek			__MaSound_Seek
#define	MaSound_Start			__MaSound_Start
#define	MaSound_Pause			__MaSound_Pause
#define	MaSound_Restart			__MaSound_Restart
#define	MaSound_Stop			__MaSound_Stop
#define	MaSound_Close			__MaSound_Close
#define	MaSound_Unload			__MaSound_Unload
#define	MaSound_Delete			__MaSound_Delete
#define	MaSound_Terminate		__MaSound_Terminate
*/
typedef struct _MASNDSEQINFO
{
	UINT32	dSeqStatus;	/* 0xFF:not initialize smw  0x01:usable */
	UINT32	dSaveRepeat[MASMW_NUM_SEQTYPE][MASMW_NUM_FILE];
	UINT32	dCnvCount[MASMW_NUM_SEQTYPE][MASMW_NUM_FILE];
	UINT32	dRepeatCount[MASMW_NUM_SEQTYPE][MASMW_NUM_FILE];
	SINT32	sdFuncId[MASMW_NUM_SEQTYPE];
	SINT32	sdFileId[MASMW_NUM_SEQTYPE];
	UINT32	dSrmCnvMap;
	UINT32	dStartPoint[MASMW_NUM_SRMCNV][MASMW_NUM_FILE];
	UINT32	dEndPoint[MASMW_NUM_SRMCNV][MASMW_NUM_FILE];
	UINT32	dSeekPoint[MASMW_NUM_SRMCNV][MASMW_NUM_FILE];
	UINT32	dSeekPos0[MASMW_NUM_SRMCNV][MASMW_NUM_FILE];
	UINT32	dPlayLength[MASMW_NUM_SRMCNV][MASMW_NUM_FILE];
	UINT32	dLoopLength[MASMW_NUM_SRMCNV][MASMW_NUM_FILE];
	UINT8	bState[MASMW_NUM_SRMCNV][MASMW_NUM_FILE];
	SINT32 (*pfnCallBack[MASMW_NUM_SRMCNV][MASMW_NUM_FILE])(UINT8 id);
	void *	pvStartExtArgs[MASMW_NUM_SRMCNV][MASMW_NUM_FILE];
	UINT32	dHvType;
} MASNDSEQINFO, *PMASNDSEQINFO;




extern void machdep_EnterCriticalSection(void);
extern void machdep_LeaveCriticalSection(void);

/*--------------------------------------------------------------------------*/

static MASNDSEQINFO  gMaSndSeqInfo;
static PMASNDSEQINFO gpMaSndSeqInfo;

static const UINT8 bMaSeqType[MASMW_NUM_SRMCNV] = { 0, 0, 1, 1, 1, 0, 0, 1, 0, 2, 1, 1, 1 };

/*--------------------------------------------------------------------------*/
/* dummy functions                                                          */

SINT32 dummy_Init   ( void )
{
	return MASMW_ERROR_ID;
}

SINT32 dummy_Load   ( UINT8 * pbFilePtr, UINT32 dFileSize, UINT32 dMode, SINT32 (*pfnFunc)(UINT8 id), void * pvExtArgs )
{
	(void)pbFilePtr;
	(void)dFileSize;
	(void)dMode;
	(void)pfnFunc;
	(void)pvExtArgs;
	return MASMW_ERROR_ID;
}

SINT32 dummy_Open   ( SINT32 sdFileId, void * pvExtArgs )
{
	(void)sdFileId;
	(void)pvExtArgs;
	return MASMW_ERROR_ID;
}

SINT32 dummy_Control( SINT32 sdFileId, UINT32 dCtrlNum, void * pvPrm, void * pvExtArgs )
{
	(void)sdFileId;
	(void)dCtrlNum;
	(void)pvPrm;
	(void)pvExtArgs;
	return MASMW_ERROR_ID;
}

SINT32 dummy_Standby( SINT32 sdFileId, void * pvExtArgs )
{
	(void)sdFileId;
	(void)pvExtArgs;
	return MASMW_ERROR_ID;
}

SINT32 dummy_Seek   ( SINT32 sdFileId, UINT32 dPos, void * pvExtArgs )
{
	(void)sdFileId;
	(void)dPos;
	(void)pvExtArgs;
	return MASMW_ERROR_ID;
}

SINT32 dummy_Start  ( SINT32 sdFileId, void * pvExtArgs )
{
	(void)sdFileId;
	(void)pvExtArgs;
	return MASMW_ERROR_ID;
}

SINT32 dummy_Stop   ( SINT32 sdFileId, void * pvExtArgs )
{
	(void)sdFileId;
	(void)pvExtArgs;
	return MASMW_ERROR_ID;
}

SINT32 dummy_Close  ( SINT32 sdFileId, void * pvExtArgs )
{
	(void)sdFileId;
	(void)pvExtArgs;
	return MASMW_ERROR_ID;
}

SINT32 dummy_Unload ( SINT32 sdFileId, void * pvExtArgs )
{
	(void)sdFileId;
	(void)pvExtArgs;
	return MASMW_ERROR_ID;
}

SINT32 dummy_End    ( void )
{
	return MASMW_ERROR_ID;
}

/*--------------------------------------------------------------------------*/

/****************************************************************************
 *	MaSound_ReceiveMessage
 *
 *	Function:
 *			Receive the message from MA Sound Driver.
 *	Argument:
 *			sdSeqId		Sequence Type  (0..2)
 *			sdFileId	file id  (0..5)
 *			bmessage	message type
 *	Return:
 *			0		success
 *			127		end
 *			< 0		error code
 *
 ****************************************************************************/
SINT32 MaSound_ReceiveMessage
(
	SINT32	sdSeqId,					/* sequence id */
	SINT32	sdFileId,					/* file id */
	UINT8	bMessage					/* message number */
)
{
	SINT32	sdFuncId;					/* function id */
	SINT32	sdResult;					/* result of function */

	MASNDSEQ_DBGMSG(("MaSndSeq_ReceiveMessage: id=%ld fid=%ld msg=%d\n", sdSeqId, sdFileId, bMessage));

	sdFuncId = gpMaSndSeqInfo->sdFuncId[sdSeqId];
	if ( sdSeqId != MASMW_SEQTYPE_DIRECT )
		sdFileId = gpMaSndSeqInfo->sdFileId[sdSeqId];

	switch ( bMessage )
	{
	case MASMW_END_OF_DATA:

		if ( gpMaSndSeqInfo->dCnvCount[sdSeqId][sdFileId] == 0xFFFF )
		{
			return MASMW_END_OF_SEQUENCE;
		}

		if ( gpMaSndSeqInfo->dCnvCount[sdSeqId][sdFileId] != 0 )
		{
			gpMaSndSeqInfo->dCnvCount[sdSeqId][sdFileId]--;
			if ( gpMaSndSeqInfo->dCnvCount[sdSeqId][sdFileId] == 0 )
			{
				gpMaSndSeqInfo->dCnvCount[sdSeqId][sdFileId] = 0xFFFF;

				return MASMW_END_OF_SEQUENCE;
			}
		}

		MaSndDrv_ControlSequencer( sdSeqId, MASNDDRV_SEQUENCER_ENTERREPEAT );

		if ( gpMaSndSeqInfo->bState[sdFuncId][sdFileId] == MASMW_STATE_PLAYING )
		{
			sdResult = ma_srmcnv_func[sdFuncId].Stop( sdFileId, NULL );
			if ( sdResult < MASMW_SUCCESS ) return sdResult;
		
			sdResult = ma_srmcnv_func[sdFuncId].Seek( sdFileId, gpMaSndSeqInfo->dStartPoint[sdFuncId][sdFileId], NULL );
			if ( sdResult < MASMW_SUCCESS ) return sdResult;
			sdResult = ma_srmcnv_func[sdFuncId].Start( sdFileId, gpMaSndSeqInfo->pvStartExtArgs[sdFuncId][sdFileId] );
			if ( sdResult < MASMW_SUCCESS ) return sdResult;
		}
		else
		{
			sdResult = ma_srmcnv_func[sdFuncId].Seek( sdFileId, gpMaSndSeqInfo->dStartPoint[sdFuncId][sdFileId], NULL );
			if ( sdResult < MASMW_SUCCESS ) return sdResult;
		}

		MaSndDrv_ControlSequencer( sdSeqId, MASNDDRV_SEQUENCER_LEAVEREPEAT );

		break;
		
	case MASMW_END_OF_SEQUENCE:


		if ( gpMaSndSeqInfo->dRepeatCount[sdSeqId][sdFileId] == 0xFFFF )
		{
			bMessage = MASMW_END_OF_SEQUENCE;
		}
		else 
		{
			if ( gpMaSndSeqInfo->dRepeatCount[sdSeqId][sdFileId] != 0 )
			{
				gpMaSndSeqInfo->dRepeatCount[sdSeqId][sdFileId]--;
			
				if ( ( sdSeqId != 0) ||
					 ( gpMaSndSeqInfo->dRepeatCount[sdSeqId][sdFileId] == 0 ) )
				{
					gpMaSndSeqInfo->dRepeatCount[sdSeqId][sdFileId] = 0xFFFF;
					gpMaSndSeqInfo->dCnvCount[sdSeqId][sdFileId]    = 0xFFFF;
					

					if ( sdSeqId == 0 )
					{
						if ( gpMaSndSeqInfo->bState[sdFuncId][sdFileId] == MASMW_STATE_PLAYING )
						{
							sdResult = ma_srmcnv_func[sdFuncId].Stop( sdFileId, NULL );
							if ( sdResult < MASMW_SUCCESS ) return sdResult;
						}
					}
	
					gpMaSndSeqInfo->bState[sdFuncId][sdFileId] = MASMW_STATE_READY;

					bMessage = MASMW_END_OF_SEQUENCE;
				}
				else
				{
					gpMaSndSeqInfo->dSeekPoint[sdFuncId][sdFileId] =
						gpMaSndSeqInfo->dStartPoint[sdFuncId][sdFileId]
					  + gpMaSndSeqInfo->dSeekPos0[sdFuncId][sdFileId];

					if ( sdSeqId == 0 )
		  				MaSndDrv_ResetPos( sdSeqId );
					bMessage = MASMW_REPEAT;

					if ( sdSeqId == 0 )
					{
						if ( gpMaSndSeqInfo->bState[sdFuncId][sdFileId] == MASMW_STATE_PLAYING )
						{
							MaDevDrv_WriteIntermediateReg( MAI_SEQ_CTRL, 0x01 );
						}
					}
				}
			}
			else
			{
				gpMaSndSeqInfo->dSeekPoint[sdFuncId][sdFileId] =
					gpMaSndSeqInfo->dStartPoint[sdFuncId][sdFileId]
				  + gpMaSndSeqInfo->dSeekPos0[sdFuncId][sdFileId];
			 
				if ( sdSeqId == 0 )
					MaSndDrv_ResetPos( sdSeqId );
				bMessage = MASMW_REPEAT;

				if ( sdSeqId == 0 )
				{
					if ( gpMaSndSeqInfo->bState[sdFuncId][sdFileId] == MASMW_STATE_PLAYING )
					{
						MaDevDrv_WriteIntermediateReg( MAI_SEQ_CTRL, 0x01 );
					}
				}
			}
		}
	
	
	default:

		if ( gpMaSndSeqInfo->pfnCallBack[sdFuncId][sdFileId] != NULL )
		{
			gpMaSndSeqInfo->pfnCallBack[sdFuncId][sdFileId]( bMessage );
		}
		break;
	}
	
	return MASMW_SUCCESS;
}
/****************************************************************************
 *	MaSound_Create
 *
 *	Function:
 *			Setting the MA Stream Converter and callback function.
 *	Argument:
 *			bCnvId		ID number of MA Stream Converter
 *	Return:
 *			>= 0		function id of registered MA Stream Converter
 *			< 0			error code
 *
 ****************************************************************************/
SINT32 MaSound_Create
(
	UINT8	bCnvId						/* stream converter id */
)
{
	UINT32	i;							/* loop counter */
	SINT32	sdResult;					/* result of function */

	MASNDSEQ_DBGMSG(("MaSound_Create: cnv_id=%d\n", bCnvId));
	
	if ( gpMaSndSeqInfo->dSeqStatus != MASNDSEQ_STATUS_USABLE )
	{
		return MASMW_ERROR;
	}

	if ( bCnvId >= MASMW_NUM_SRMCNV )
	{
		return MASMW_ERROR;
	}

	if ( ( gpMaSndSeqInfo->dSrmCnvMap & (UINT32)(1 << bCnvId) ) != 0 )
	{
		return MASMW_ERROR;
	}

	sdResult = ma_srmcnv_func[bCnvId].Init();

	if ( sdResult < MASMW_SUCCESS )
	{
		MASNDSEQ_DBGMSG(("Can't initialize the stream converter\n"));
		return sdResult;
	}

	/* set HV language type */
	if ( bCnvId == MASMW_CNVID_HVS )
	{
		ma_srmcnv_func[bCnvId].Control( 0, MASMW_SET_HVLANGUAGE, (void *)&gpMaSndSeqInfo->dHvType, NULL );
	}

	for ( i = 0; i < MASMW_NUM_FILE; i++ )
	{
		gpMaSndSeqInfo->bState[bCnvId][i] = MASMW_STATE_IDLE;
	}

	gpMaSndSeqInfo->dSrmCnvMap |= (UINT32)(1 << bCnvId);

	return (SINT32)bCnvId;
}
/****************************************************************************
 *	MaSound_Load
 *
 *	Function:
 *			Performs loading processing of Stream Converter.
 *	Arguments:
 *			sdFuncId	function id
 *			pbFilePtr	pointer to file
 *			dFileSize	byte size of file
 *			bMode		error check mode	0: no check
 *											1: check
 *											2: check only
 *											3: get contents information
 *			pfnFunc		callback function
 *			pvExtArgs	for future extension
 *	Return:
 *			>= 0		success. file id.
 *			< 0			error code
 *
 ****************************************************************************/
SINT32 MaSound_Load
(
	SINT32	sdFuncId, 					/* function id */
	UINT8 *	pbFilePtr,					/* pointer to file */
	UINT32	dFileSize,					/* size of file */
	UINT8	bMode,						/* error check mode */
	SINT32	(*pfnFunc)(UINT8 bId),		/* callback function */
	void *	pvExtArgs					/* for future extension */
)
{
	SINT32	sdResult;					/* result of function */
	
	MASNDSEQ_DBGMSG(("MaSound_Load: id=%ld\n", sdFuncId));

	/* check sdFuncId */
	if ( (sdFuncId < 0) || (sdFuncId >= MASMW_NUM_SRMCNV) )
	{
		return MASMW_ERROR;
	}
	if ( ( gpMaSndSeqInfo->dSrmCnvMap & (UINT32)(1 << sdFuncId) ) == 0 )
	{
		return MASMW_ERROR;
	}
	
	/* check mode */
	if ( bMode > 3 )
	{
		return MASMW_ERROR_ARGUMENT;
	}

	/* call a stream converter */
	sdResult = ma_srmcnv_func[sdFuncId].Load( pbFilePtr, dFileSize, (UINT32)bMode, pfnFunc, pvExtArgs );

	if ( sdResult >= MASMW_SUCCESS )
	{
		/* status doesn't change, if sdResult == 0 */
		if ( sdResult == 0 )
		{
			return sdResult;
		}

		/* change status to LOADED */
		gpMaSndSeqInfo->bState[sdFuncId][sdResult] = MASMW_STATE_LOADED;

		/* register a callback function */
		gpMaSndSeqInfo->pfnCallBack[sdFuncId][sdResult] = pfnFunc;
	}
	
	return sdResult;
}
/****************************************************************************
 *	MaSound_Open
 *
 *	Function:
 *			Performs opening processing of Stream Converter.
 *	Arguments:
 *			sdFuncId	function id
 *			sdFileId	file id
 *			wOpenMode	file open mode
 *			pvExtArgs	for future extension
 *	Return:
 *			0			success
 *			< 0 		error code
 *
 ****************************************************************************/
SINT32 MaSound_Open
(
	SINT32	sdFuncId,					/* function id */
	SINT32	sdFileId,					/* file id */
	UINT16	wOpenMode,					/* open mode */
	void *	pvExtArgs					/* for future extension */
)
{
	SINT32	sdResult;					/* result of function */
	SINT32	sdResult2;					/* result of function */
	
	MASNDSEQ_DBGMSG(("MaSound_Open: id=%ld hd=%ld md=%hd\n", sdFuncId, sdFileId, wOpenMode));
	
	(void)wOpenMode;

	/* check sdFuncId and sdFileId */
	if ( (sdFuncId < 0) || (sdFuncId >= MASMW_NUM_SRMCNV) )
	{
		return MASMW_ERROR;
	}
	if ( ( ( gpMaSndSeqInfo->dSrmCnvMap & (UINT32)(1 << sdFuncId) ) == 0 )
		|| ( sdFileId < 0 ) || ( sdFileId >= MASMW_NUM_FILE ) )
	{
		return MASMW_ERROR;
	}

	/* check status */
	if ( gpMaSndSeqInfo->bState[sdFuncId][sdFileId] != MASMW_STATE_LOADED )
	{
		return MASMW_ERROR;
	}

	/* call a stream converter */
	sdResult = ma_srmcnv_func[sdFuncId].Open( sdFileId, pvExtArgs );

	if ( sdResult >= MASMW_SUCCESS )
	{
		if ( bMaSeqType[sdFuncId] != 2 )
		{
			/* set default endpoint */
			
			/* get length of data */
			sdResult = ma_srmcnv_func[sdFuncId].Control( sdFileId, MASMW_GET_LENGTH, NULL, NULL );
			if ( sdResult < MASMW_SUCCESS )
			{
				return sdResult;
			}

			sdResult2 = ma_srmcnv_func[sdFuncId].Control( sdFileId, MASMW_GET_LENGTH, NULL, (void *)(-1) );
			if ( sdResult2 < MASMW_SUCCESS )
			{
				return sdResult2;
			}

			/* set start point and end point */
			gpMaSndSeqInfo->dStartPoint[sdFuncId][sdFileId] = 0;
			gpMaSndSeqInfo->dEndPoint[sdFuncId][sdFileId]   = (UINT32)sdResult;

			/* reset value of seek */
			gpMaSndSeqInfo->dSeekPoint[sdFuncId][sdFileId]  = 0;

			/* set length of data */
			gpMaSndSeqInfo->dPlayLength[sdFuncId][sdFileId] = (UINT32)sdResult;
			gpMaSndSeqInfo->dLoopLength[sdFuncId][sdFileId] = (UINT32)sdResult2;
		}
		else
		{
			gpMaSndSeqInfo->dStartPoint[sdFuncId][sdFileId] = 0;
			gpMaSndSeqInfo->dEndPoint[sdFuncId][sdFileId] = 0xFFFFFFFF;
			gpMaSndSeqInfo->dSeekPoint[sdFuncId][sdFileId] = 0;
			gpMaSndSeqInfo->dPlayLength[sdFuncId][sdFileId] = 0xFFFFFFFF;
			gpMaSndSeqInfo->dLoopLength[sdFuncId][sdFileId] = 0xFFFFFFFF;
		}

		/* change status to OPENED */
		gpMaSndSeqInfo->bState[sdFuncId][sdFileId] = MASMW_STATE_OPENED;

		sdResult = MASMW_SUCCESS;
	}

	return sdResult;
}
/****************************************************************************
 *	MaSound_Control
 *
 *	Function:
 *			Performs controlling processing of Stream Conveter.
 *	Argument:
 *			sdFuncId	function id
 *			sdFileId	file id
 *			bCtrlNum	control number
 *			pvPrm		control parameter
 *			pvExtArgs	for future extension
 *	Return:
 *			0			success
 *			< 0			error code
 *
 ****************************************************************************/
SINT32 MaSound_Control
(
	SINT32	sdFuncId,					/* function id */
	SINT32	sdFileId,					/* file id */
	UINT8	bCtrlNum,					/* control number */
	void *	pvPrm,						/* parameter */
	void *	pvExtArgs					/* for future extension */
)
{
	UINT32	dStartPoint;				/* start point */
	UINT32	dEndPoint;					/* end point */
	UINT32	dPosition;					/* position */
	UINT32	dPlayLength;				/* play length */
	UINT32	dVal;
	SINT32	sdSeqId;					/* sequence type */
	SINT32	sdResult;					/* result of function */

	MASNDSEQ_DBGMSG(("MaSound_Control: id=%ld hd=%ld cn=%d\n", sdFuncId, sdFileId, bCtrlNum));

	/* check sdFuncId and sdFileId */
	if ( (sdFuncId < 0) || (sdFuncId >= MASMW_NUM_SRMCNV) )
	{
		return MASMW_ERROR;
	}
	if ( ( ( gpMaSndSeqInfo->dSrmCnvMap & (UINT32)(1 << sdFuncId) ) == 0 )
		|| ( sdFileId < 0 ) || ( sdFileId >= MASMW_NUM_FILE ) )
	{
		return MASMW_ERROR;
	}
	
	switch ( bCtrlNum )
	{
	case MASMW_GET_STATE:
	
		dVal = (UINT32)gpMaSndSeqInfo->bState[sdFuncId][sdFileId];
		return dVal;

	case MASMW_SET_LOADINFO:
		/* set file information */
		if ( pvPrm == NULL )
		{
			return MASMW_ERROR_ARGUMENT;
		}

		break;

	case MASMW_GET_CONTENTSDATA:

		if ( ( gpMaSndSeqInfo->bState[sdFuncId][sdFileId] < MASMW_STATE_LOADED )
		  && ( sdFileId != 0 ) )
		{
			return MASMW_ERROR;
		}
		if ( (pvPrm == NULL) || (((PMASMW_CONTENTSINFO)pvPrm)->size == 0) )
		{
			return MASMW_ERROR_ARGUMENT;
		}
		break;
	
	case MASMW_GET_PHRASELIST:
	case MASMW_GET_LOADINFO:

		if ( pvPrm == NULL )
		{
			return MASMW_ERROR_ARGUMENT;
		}

	case MASMW_GET_LENGTH:
	case MASMW_GET_HVCHANNEL:

		if ( ( gpMaSndSeqInfo->bState[sdFuncId][sdFileId] < MASMW_STATE_LOADED )
		  && ( sdFileId != 0 ) )
		{
			return MASMW_ERROR;
		}
		break;

	case MASMW_SET_STARTPOINT:

		if ( gpMaSndSeqInfo->bState[sdFuncId][sdFileId] != MASMW_STATE_OPENED )
		{
			return MASMW_ERROR;
		}

		if ( pvPrm == NULL )
		{
			return MASMW_ERROR_ARGUMENT;
		}
		dStartPoint = *((UINT32 *)pvPrm);
		dPlayLength = gpMaSndSeqInfo->dPlayLength[sdFuncId][sdFileId];
		if ( dStartPoint > dPlayLength )
		{
			return MASMW_ERROR;
		}
		else
		{
			gpMaSndSeqInfo->dStartPoint[sdFuncId][sdFileId] = dStartPoint;
			gpMaSndSeqInfo->dSeekPoint[sdFuncId][sdFileId]  = dStartPoint;
		}
		
		return MASMW_SUCCESS;

	case MASMW_SET_ENDPOINT:

		if ( gpMaSndSeqInfo->bState[sdFuncId][sdFileId] != MASMW_STATE_OPENED )
		{
			return MASMW_ERROR;
		}

		if ( pvPrm == NULL )
		{
			return MASMW_ERROR_ARGUMENT;
		}
		dEndPoint = *((UINT32 *)pvPrm);
		dPlayLength = gpMaSndSeqInfo->dPlayLength[sdFuncId][sdFileId];
		if ( dEndPoint > dPlayLength )
		{
			return MASMW_ERROR;
		}
		else
		{
			gpMaSndSeqInfo->dEndPoint[sdFuncId][sdFileId] = dEndPoint;
		}

		break;

	case MASMW_GET_POSITION:

		if ( gpMaSndSeqInfo->bState[sdFuncId][sdFileId] < MASMW_STATE_OPENED )
		{
			return MASMW_ERROR;
		}

		if ( bMaSeqType[sdFuncId] == 0 )
		{
			sdResult = ma_srmcnv_func[sdFuncId].Control( sdFileId, (UINT32)bCtrlNum, pvPrm, pvExtArgs );
			if ( sdResult >= 0 )
			{
				dPosition = gpMaSndSeqInfo->dSeekPoint[sdFuncId][sdFileId] + sdResult;
				dPlayLength = gpMaSndSeqInfo->dLoopLength[sdFuncId][sdFileId];

				if ( (dPlayLength != 0 ) && ( dPosition > dPlayLength ) )
				{
					return dPlayLength;
				}
				else
				{
					return dPosition;
				}
			}
			else
			{
				return sdResult;
			}
		}

		break;

	case MASMW_SET_VOLUME:
	case MASMW_SET_SPEED:
	case MASMW_SET_SPEEDWIDE:
	case MASMW_SET_KEYCONTROL:
	case MASMW_SEND_MIDIMSG:
	case MASMW_SEND_SYSEXMIDIMSG:
	case MASMW_SET_PANPOT:
	case MASMW_SET_EVENTNOTE:
	case MASMW_GET_HVVOICE:
	case MASMW_GET_HVSCRIPT:

		if ( pvPrm == NULL )
		{
			return MASMW_ERROR_ARGUMENT;
		}

	case MASMW_GET_LEDSTATUS:
	case MASMW_GET_VIBSTATUS:
	case MASMW_GET_HVCHANNELSTATUS:

		if ( gpMaSndSeqInfo->bState[sdFuncId][sdFileId] < MASMW_STATE_OPENED )
		{
			return MASMW_ERROR;
		}
		break;
		

	case MASMW_SET_HVDICTIONARY:
	case MASMW_SET_HVVOICE:
	case MASMW_SET_HVCHANNEL:
	case MASMW_SET_HVCHANNELSTATUS:

		if ( gpMaSndSeqInfo->bState[sdFuncId][1] > MASMW_STATE_LOADED )
		{
			return MASMW_ERROR;
		}

		if ( pvPrm == NULL )
		{
			return MASMW_ERROR_ARGUMENT;
		}
		break;
		
	case MASMW_SET_HVPROGRAMNO:
	
		if ( gpMaSndSeqInfo->bState[sdFuncId][sdFileId] != MASMW_STATE_OPENED )
		{
			return MASMW_ERROR;
		}

		if ( pvPrm == NULL )
		{
			return MASMW_ERROR_ARGUMENT;
		}
		break;
		
		
	case MASMW_GET_HVCONVERTSIZE:
								 
		if ( gpMaSndSeqInfo->bState[sdFuncId][sdFileId] < MASMW_STATE_OPENED )
		{
			return MASMW_ERROR;
		}
		break;
		
	
	case MASMW_SET_REPEAT:

		if ( ( gpMaSndSeqInfo->bState[sdFuncId][sdFileId] != MASMW_STATE_READY )
			&& ( gpMaSndSeqInfo->bState[sdFuncId][sdFileId] != MASMW_STATE_PAUSE ) )
		{
			return MASMW_ERROR;
		}

		if ( pvPrm == NULL )
		{
			return MASMW_ERROR_ARGUMENT;
		}

		dVal = (UINT32)(*((UINT8 *)pvPrm));
		if ( dVal > MASNDSEQ_MAX_PLAYMODE )
		{
			return MASMW_ERROR;
		}
		pvPrm = &dVal;

		sdSeqId = bMaSeqType[sdFuncId];
		gpMaSndSeqInfo->dSaveRepeat[sdSeqId][sdFileId] = dVal;
		if ( sdSeqId == 0 )
		{
			if ( gpMaSndSeqInfo->dRepeatCount[sdSeqId][sdFileId] != 0xFFFF )
			{
				sdResult = ma_srmcnv_func[sdFuncId].Control( sdFileId, MASMW_GET_POSITION, NULL, NULL );
				if ( sdResult < MASMW_SUCCESS )
				{
					return MASMW_ERROR;
				}
				dPosition = gpMaSndSeqInfo->dSeekPoint[sdFuncId][sdFileId] + (UINT32)sdResult;
				dPlayLength = gpMaSndSeqInfo->dLoopLength[sdFuncId][sdFileId];
				if ( (dPlayLength != 0 ) && ( dPosition > dPlayLength ) )
					dPosition = dPlayLength;

				sdResult = ma_srmcnv_func[sdFuncId].Seek( sdFileId, dPosition, NULL );
				if ( sdResult < MASMW_SUCCESS )
				{
					return MASMW_ERROR;
				}
			
				gpMaSndSeqInfo->dSeekPoint[sdFuncId][sdFileId] = dPosition + (UINT32)sdResult;
				gpMaSndSeqInfo->dCnvCount[sdSeqId][sdFileId] = dVal;
				gpMaSndSeqInfo->dRepeatCount[sdSeqId][sdFileId] = dVal;
			}
			return MASMW_SUCCESS;
		}

		break;

	case MASMW_GET_CONTROL_VAL:

		if ( gpMaSndSeqInfo->bState[sdFuncId][sdFileId] < MASMW_STATE_READY )
		{
			return MASMW_ERROR;
		}
		break;

	case MASMW_SET_CB_INTERVAL:

		if ( gpMaSndSeqInfo->bState[sdFuncId][sdFileId] != MASMW_STATE_LOADED )
		{
			return MASMW_ERROR;
		}
		break;

	default:
		return MASMW_ERROR;
	}

	/* call a stream converter */
	sdResult = ma_srmcnv_func[sdFuncId].Control( sdFileId, (UINT32)bCtrlNum, pvPrm, pvExtArgs );


	return sdResult;
}
/****************************************************************************
 *	MaSound_Standby
 *
 *	Function:
 *			Performs standby processing of Stream Converter.
 *	Argument:
 *			sdFuncId	function id
 *			sdFileId	file id
 *			pvExtArgs	for future extension
 *	Return:
 *			0			success
 *			< 0			error code
 *
 ****************************************************************************/
SINT32 MaSound_Standby
(
	SINT32	sdFuncId,					/* function id */
	SINT32	sdFileId,					/* file id */
	void *	pvExtArgs					/* for future extension */
)
{
	UINT32	dSeqType;					/* type of sequence */
	UINT32	dPos;						/* position */
	SINT32	sdResult;					/* result of function */
	SINT32	sdResult2;

	MASNDSEQ_DBGMSG(("MaSound_Standby: id=%ld hd=%ld\n", sdFuncId, sdFileId));

	/* check sdFuncId and sdFileId */
	if ( (sdFuncId < 0) || (sdFuncId >= MASMW_NUM_SRMCNV) )
	{
		return MASMW_ERROR;
	}
	if ( ( ( gpMaSndSeqInfo->dSrmCnvMap & (UINT32)(1 << sdFuncId) ) == 0 )
		|| ( sdFileId < 0 ) || ( sdFileId >= MASMW_NUM_FILE ) )
	{
		return MASMW_ERROR;
	}

	/* check status */
	if ( gpMaSndSeqInfo->bState[sdFuncId][sdFileId] != MASMW_STATE_OPENED )
	{
		return MASMW_ERROR;
	}

	/* call a stream converter */
	sdResult = ma_srmcnv_func[sdFuncId].Standby( sdFileId, pvExtArgs );

	if ( sdResult >= MASMW_SUCCESS )
	{
		/* reset counters of loop */
		dSeqType = bMaSeqType[sdFuncId];
		gpMaSndSeqInfo->dSaveRepeat[dSeqType][sdFileId]  = 0xFFFF;
		gpMaSndSeqInfo->dCnvCount[dSeqType][sdFileId]    = 0xFFFF;
		gpMaSndSeqInfo->dRepeatCount[dSeqType][sdFileId] = 0xFFFF;

		/* call seek function of stream converter */
		dPos = gpMaSndSeqInfo->dStartPoint[sdFuncId][sdFileId];
		sdResult2 = ma_srmcnv_func[sdFuncId].Seek( sdFileId, dPos, pvExtArgs );
		if ( sdResult2 < MASMW_SUCCESS )
		{
			return sdResult2;
		}

		/* set seek point */
		gpMaSndSeqInfo->dSeekPos0[sdFuncId][sdFileId]  = (UINT32)sdResult2;
		gpMaSndSeqInfo->dSeekPoint[sdFuncId][sdFileId] = (UINT32)(dPos + (UINT32)sdResult2);

		/* change status to READY */
		gpMaSndSeqInfo->bState[sdFuncId][sdFileId] = MASMW_STATE_READY;
	}

	return sdResult;
}
/****************************************************************************
 *	MaSound_Seek
 *
 *	Function:
 *			Performs seeking processing of Stream Converter.
 *	Argument:
 *			sdFuncId	function id
 *			sdFileId	file id
 *			dPos		playing position from top or start point [ms]
 *			bFlag		unused 0
 *			pvExtArgs	for future extension
 *	Return:
 *			0			success
 *			< 0 		error code
 *
 ****************************************************************************/
SINT32 MaSound_Seek
(
	SINT32	sdFuncId,					/* function id */
	SINT32	sdFileId,					/* file id */
	UINT32	dPos,						/* start position (msec) */
	UINT8	bFlag,						/* flag */
	void *	pvExtArgs					/* for future extension */
)
{
	UINT32	dSeqType;					/* type of sequence */
	UINT32	dStartPoint;				/* start point */
	UINT32	dEndPoint;					/* end point */
	UINT32	dPlayLength;				/* play length */
	SINT32	sdResult;					/* result of function */
	
	MASNDSEQ_DBGMSG(("MaSound_Seek: id=%ld hd=%ld pos=%ld fg=%d\n", sdFuncId, sdFileId, dPos, bFlag));
	
	(void)bFlag;

	/* check sdFuncId and sdFileId */
	if ( (sdFuncId < 0) || (sdFuncId >= MASMW_NUM_SRMCNV) )
	{
		return MASMW_ERROR;
	}
	if ( ( ( gpMaSndSeqInfo->dSrmCnvMap & (UINT32)(1 << sdFuncId) ) == 0 )
		|| ( sdFileId < 0 ) || ( sdFileId >= MASMW_NUM_FILE ) )
	{
		return MASMW_ERROR;
	}

	/* chack status */
	if ( ( gpMaSndSeqInfo->bState[sdFuncId][sdFileId] != MASMW_STATE_READY )
	  && ( gpMaSndSeqInfo->bState[sdFuncId][sdFileId] != MASMW_STATE_PAUSE ) )
	{
		return MASMW_ERROR;
	}

	dStartPoint = gpMaSndSeqInfo->dStartPoint[sdFuncId][sdFileId];
	dEndPoint   = gpMaSndSeqInfo->dEndPoint[sdFuncId][sdFileId];
	dPlayLength = gpMaSndSeqInfo->dPlayLength[sdFuncId][sdFileId];

	dPos += dStartPoint;

	if ( ( dPos > dEndPoint ) || ( dPos > dPlayLength ) )
	{
		return MASMW_ERROR;
	}

	/* call stream converter */
	sdResult = ma_srmcnv_func[sdFuncId].Seek( sdFileId, dPos, pvExtArgs );

	if ( sdResult >= MASMW_SUCCESS )
	{
		gpMaSndSeqInfo->dSeekPoint[sdFuncId][sdFileId] = (UINT32)(dPos + (UINT32)sdResult);

		dSeqType = bMaSeqType[sdFuncId];
		
		if ( dSeqType == 0 )
		{
			gpMaSndSeqInfo->dCnvCount[dSeqType][sdFileId] = gpMaSndSeqInfo->dRepeatCount[dSeqType][sdFileId];
		}
		
		sdResult = MASMW_SUCCESS;
	}


	return sdResult;
}
/****************************************************************************
 *	MaSound_Start
 *
 *	Function:
 *			Performs starting process of Stream Converter.
 *	Argument:
 *			sdFuncId	function id
 *			sdFileId	file id
 *			wPlayMode	playing mode
 *			pvExtArgs	for future extension
 *	Return:
 *			0			success
 *			< 0 		error code
 *
 ****************************************************************************/
SINT32 MaSound_Start
(
	SINT32	sdFuncId,					/* function id */
	SINT32	sdFileId,					/* file id */
	UINT16	wPlayMode,					/* playing mode */
	void * 	pvExtArgs					/* for future extension */
)
{
	UINT32	dSeqType;					/* type of sequence */
	UINT32	dSaveRepeatCount;
	SINT32	sdSaveFuncId;
	SINT32	sdSaveFileId;
	UINT32	dStartPoint;				/* start point */
	UINT32	dEndPoint;					/* end point */
	UINT32	dPMode;
	SINT32	sdResult;					/* result of function */
	
	MASNDSEQ_DBGMSG(("MaSound_Start: id=%ld hd=%ld pm=%d\n", sdFuncId, sdFileId, wPlayMode));

	/* check sdFuncId and sdFileId */
	if ( (sdFuncId < 0) || (sdFuncId >= MASMW_NUM_SRMCNV) )
	{
		return MASMW_ERROR;
	}
	if ( ( ( gpMaSndSeqInfo->dSrmCnvMap & (UINT32)(1 << sdFuncId) ) == 0 )
		|| ( sdFileId < 0 ) || ( sdFileId >= MASMW_NUM_FILE ) )
	{
		return MASMW_ERROR;
	}

	/* check status */
	if ( gpMaSndSeqInfo->bState[sdFuncId][sdFileId] != MASMW_STATE_READY )
	{
		return MASMW_ERROR;
	}
	
	/* check wPlayMode */
	if ( wPlayMode > MASNDSEQ_MAX_PLAYMODE )
	{
		return MASMW_ERROR_ARGUMENT;
	}
	dPMode = (UINT32)wPlayMode;
	
	dSeqType = bMaSeqType[sdFuncId];

	if ( dSeqType != 2 )
	{
		/* check start point and end point */
		dStartPoint = gpMaSndSeqInfo->dStartPoint[sdFuncId][sdFileId];
		dEndPoint   = gpMaSndSeqInfo->dEndPoint[sdFuncId][sdFileId];
		if ( dStartPoint >= dEndPoint )
		{
			return MASMW_ERROR;
		}

		if ( (dEndPoint - dStartPoint) <= MA_MIN_LENGTH )
		{
			return MASMW_ERROR;
		}
	}

	sdSaveFuncId = gpMaSndSeqInfo->sdFuncId[dSeqType];
	sdSaveFileId = gpMaSndSeqInfo->sdFileId[dSeqType];
	dSaveRepeatCount = gpMaSndSeqInfo->dRepeatCount[dSeqType][sdFileId];

	gpMaSndSeqInfo->sdFuncId[dSeqType] = sdFuncId;
	gpMaSndSeqInfo->sdFileId[dSeqType] = sdFileId;

	/* change status to PLAYING */
	gpMaSndSeqInfo->bState[sdFuncId][sdFileId] = MASMW_STATE_PLAYING;

	/* save pvExtArgs */
	gpMaSndSeqInfo->pvStartExtArgs[sdFuncId][sdFileId] = pvExtArgs;

	if ( dSeqType == 0 )
	{	
		/* delayed type */
		if ( gpMaSndSeqInfo->dRepeatCount[dSeqType][sdFileId] == 0xFFFF )
		{
			if ( gpMaSndSeqInfo->dSaveRepeat[dSeqType][sdFileId] == 0xFFFF )
			{
				gpMaSndSeqInfo->dCnvCount[dSeqType][sdFileId] = dPMode;
				gpMaSndSeqInfo->dRepeatCount[dSeqType][sdFileId] = dPMode;
			}
			else
			{
				gpMaSndSeqInfo->dCnvCount[dSeqType][sdFileId] = gpMaSndSeqInfo->dSaveRepeat[dSeqType][sdFileId];
				gpMaSndSeqInfo->dRepeatCount[dSeqType][sdFileId] = gpMaSndSeqInfo->dSaveRepeat[dSeqType][sdFileId];
			}
		}

		/* call stream converter */
		sdResult = ma_srmcnv_func[sdFuncId].Start( sdFileId, pvExtArgs );
	}
	else
	{
		/* direct type */
		if ( gpMaSndSeqInfo->dRepeatCount[dSeqType][sdFileId] == 0xFFFF )
		{
			if ( gpMaSndSeqInfo->dSaveRepeat[dSeqType][sdFileId] == 0xFFFF )
			{
				sdResult = ma_srmcnv_func[sdFuncId].Control( 
								sdFileId, MASMW_SET_REPEAT, &dPMode, NULL );
			}
			else
			{
				sdResult = ma_srmcnv_func[sdFuncId].Control( 
								sdFileId, MASMW_SET_REPEAT, &gpMaSndSeqInfo->dSaveRepeat[dSeqType][sdFileId], NULL );
			}
			if ( sdResult < MASMW_SUCCESS )
			{
				gpMaSndSeqInfo->bState[sdFuncId][sdFileId] = MASMW_STATE_READY;

				gpMaSndSeqInfo->sdFuncId[dSeqType] = sdSaveFuncId;
				gpMaSndSeqInfo->sdFileId[dSeqType] = sdSaveFileId;

				return sdResult;
			}
			gpMaSndSeqInfo->dCnvCount[dSeqType][sdFileId] = 1;
			gpMaSndSeqInfo->dRepeatCount[dSeqType][sdFileId] = 1;
		}
		/* call stream converter */
		sdResult = ma_srmcnv_func[sdFuncId].Start( sdFileId, pvExtArgs );
	}

	if ( sdResult < MASMW_SUCCESS )
	{
		gpMaSndSeqInfo->bState[sdFuncId][sdFileId] = MASMW_STATE_READY;

		gpMaSndSeqInfo->sdFuncId[dSeqType] = sdSaveFuncId;
		gpMaSndSeqInfo->sdFileId[dSeqType] = sdSaveFileId;
		gpMaSndSeqInfo->dRepeatCount[dSeqType][sdFileId] =dSaveRepeatCount;
	}

	return sdResult;
}
/****************************************************************************
 *	MaSound_Pause
 *
 *	Function:
 *			Stops sequence data reproduction temporarily.
 *			For audio systems, this operation is same as Stop.
 *	Argument:
 *			sdFuncId	function id
 *			sdFileId	file id
 *			pvExtArgs	for future extenstion
 *	Return:
 *			0			success
 *			< 0			error code
 *
 ****************************************************************************/
SINT32 MaSound_Pause
(
	SINT32	sdFuncId,					/* function id */
	SINT32	sdFileId,					/* file id */
	void * 	pvExtArgs					/* for future extension */
)
{
	SINT32	sdResult;					/* result of function */
	
	MASNDSEQ_DBGMSG(("MaSound_Pause: id=%ld hd=%ld\n", sdFuncId, sdFileId));

	machdep_EnterCriticalSection();

	/* check sdFuncId and sdFileId */
	if ( (sdFuncId < 0) || (sdFuncId >= MASMW_NUM_SRMCNV) )
	{
		return MASMW_ERROR;
	}
	if ( ( ( gpMaSndSeqInfo->dSrmCnvMap & (UINT32)(1 << sdFuncId) ) == 0 )
		|| ( sdFileId < 0 ) || ( sdFileId >= MASMW_NUM_FILE ) )
	{
		machdep_LeaveCriticalSection();
		return MASMW_ERROR;
	}

	/* check status */
	if ( gpMaSndSeqInfo->bState[sdFuncId][sdFileId] != MASMW_STATE_PLAYING )
	{
		machdep_LeaveCriticalSection();
		return MASMW_ERROR;
	}

	/* call stream converter */
	sdResult = ma_srmcnv_func[sdFuncId].Pause( sdFileId, pvExtArgs );

	if ( sdResult >= MASMW_SUCCESS )
	{
		/* change status to PAUSE */
		gpMaSndSeqInfo->bState[sdFuncId][sdFileId] = MASMW_STATE_PAUSE;
	}

	machdep_LeaveCriticalSection();
	return sdResult;
}
/****************************************************************************
 *	MaSound_Restart
 *
 *	Function:
 *			Cancels pause of sequence data reproduction.
 *			For audio systems, this operation is the same as Start.
 *	Argument:
 *			sdFuncId	function id
 *			sdFileId	file id
 *			pvExtArgs	for future extension
 *	Return:
 *			0			success
 *			< 0			error code
 *
 ****************************************************************************/
SINT32 MaSound_Restart
(
	SINT32	sdFuncId,					/* function id */
	SINT32	sdFileId,					/* file id */
	void * 	pvExtArgs					/* for future extension */
)
{
	SINT32	sdResult;					/* result of function */

	MASNDSEQ_DBGMSG(("MaSound_Restart: id=%ld, hd=%ld\n", sdFuncId, sdFileId));

	/* check sdFuncId and sdFileId */
	if ( (sdFuncId < 0) || (sdFuncId >= MASMW_NUM_SRMCNV) )
	{
		return MASMW_ERROR;
	}
	if ( ( ( gpMaSndSeqInfo->dSrmCnvMap & (UINT32)(1 << sdFuncId) ) == 0 )
		|| ( sdFileId < 0 ) || ( sdFileId >= MASMW_NUM_FILE ) )
	{
		return MASMW_ERROR;
	}

	/* check status */
	if ( gpMaSndSeqInfo->bState[sdFuncId][sdFileId] != MASMW_STATE_PAUSE )
	{
		return MASMW_ERROR;
	}

	/* change status to PLAYING */
	gpMaSndSeqInfo->bState[sdFuncId][sdFileId] = MASMW_STATE_PLAYING;

	/* call stream converter */

	/* save pvExtArgs */
	gpMaSndSeqInfo->pvStartExtArgs[sdFuncId][sdFileId] = pvExtArgs;
	sdResult = ma_srmcnv_func[sdFuncId].Restart( sdFileId, pvExtArgs );

	if ( sdResult < MASMW_SUCCESS )
	{
		/* change status to PAUSE */
		gpMaSndSeqInfo->bState[sdFuncId][sdFileId] = MASMW_STATE_PAUSE;
	}

	return sdResult;
}
/****************************************************************************
 *	MaSound_Stop
 *
 *	Function:
 *			Performs stopping processing of Stream Converter.
 *	Argument:
 *			sdFuncId	function id
 *			sdFileId	file id
 *			pvExtArgs	for future extension
 *	Return:
 *			0			success
 *			< 0 		error code
 *
 ****************************************************************************/
SINT32 MaSound_Stop
(
	SINT32	sdFuncId,					/* function id */
	SINT32	sdFileId,					/* file id */
	void *	pvExtArgs					/* for future extension */
)
{
	SINT32	sdResult;					/* result of function */
	
	MASNDSEQ_DBGMSG(("MaSound_Stop: id=%ld hd=%ld\n", sdFuncId, sdFileId));

	machdep_EnterCriticalSection();

	/* check sdFuncId and sdFileId */
	if ( (sdFuncId < 0) || (sdFuncId >= MASMW_NUM_SRMCNV) )
	{
		return MASMW_ERROR;
	}
	if ( ( ( gpMaSndSeqInfo->dSrmCnvMap & (UINT32)(1 << sdFuncId) ) == 0 )
		|| ( sdFileId < 0 ) || ( sdFileId >= MASMW_NUM_FILE ) )
	{
		machdep_LeaveCriticalSection();
		return MASMW_ERROR;
	}

	/* check status */
	if ( gpMaSndSeqInfo->bState[sdFuncId][sdFileId] == MASMW_STATE_READY )
	{
		machdep_LeaveCriticalSection();
		return MASMW_SUCCESS;
	}

	if ( ( gpMaSndSeqInfo->bState[sdFuncId][sdFileId] != MASMW_STATE_PLAYING )
	  && ( gpMaSndSeqInfo->bState[sdFuncId][sdFileId] != MASMW_STATE_PAUSE ) )
	{
		machdep_LeaveCriticalSection();
		return MASMW_ERROR;
	}

	/* call stream converter */
	sdResult = ma_srmcnv_func[sdFuncId].Stop( sdFileId, pvExtArgs );

	if ( sdResult >= MASMW_SUCCESS )
	{
		/* change status to READY */
		gpMaSndSeqInfo->bState[sdFuncId][sdFileId] = MASMW_STATE_READY;
	}

	machdep_LeaveCriticalSection();

	return sdResult;
}
/****************************************************************************
 *	MaSound_Close
 *
 *	Function:
 *			Performs closing processing of Stream Converter.
 *	Argument:
 *			sdFuncId	function id
 *			sdFileId	file id
 *			pvExtArgs	for future extension
 *	Return:
 *			0			success
 *			< 0			error code
 *
 ****************************************************************************/
SINT32 MaSound_Close
(
	SINT32	sdFuncId,					/* function id */
	SINT32	sdFileId,					/* file id */
	void *	pvExtArgs					/* for future extension */
)
{
	SINT32	sdResult;					/* result of function */

	MASNDSEQ_DBGMSG(("MaSound_Close: id=%ld hd=%ld\n", sdFuncId, sdFileId));

	/* check sdFuncId and sdFileId */
	if ( (sdFuncId < 0) || (sdFuncId >= MASMW_NUM_SRMCNV) )
	{
		return MASMW_ERROR;
	}
	if ( ( ( gpMaSndSeqInfo->dSrmCnvMap & (UINT32)(1 << sdFuncId) ) == 0 )
		|| ( sdFileId < 0 ) || ( sdFileId >= MASMW_NUM_FILE ) )
	{
		return MASMW_ERROR;
	}

	/* check status */
	if ( ( gpMaSndSeqInfo->bState[sdFuncId][sdFileId] != MASMW_STATE_READY )
	  && ( gpMaSndSeqInfo->bState[sdFuncId][sdFileId] != MASMW_STATE_OPENED ) )
	{
		return MASMW_ERROR;
	}

	/* call stream converter */
	sdResult = ma_srmcnv_func[sdFuncId].Close( sdFileId, pvExtArgs );

	if ( sdResult >= MASMW_SUCCESS )
	{
		/* change status to LOADED */
		gpMaSndSeqInfo->bState[sdFuncId][sdFileId] = MASMW_STATE_LOADED;
	}

	return sdResult;
}
/****************************************************************************
 *	MaSound_Unload
 *
 *	Function:
 *			Performs unloading prcessing of Stream Converter.
 *	Argument:
 *			sdFuncId	function id
 *			sdFileId	file id
 *			pvExtArgs	for future extension
 *	Return:
 *			0			success
 *			< 0			error code
 *
 ****************************************************************************/
SINT32 MaSound_Unload
(
	SINT32	sdFuncId,					/* function id */
	SINT32	sdFileId,					/* file id */
	void *	pvExtArgs					/* for future extension */
)
{
	SINT32 sdResult;					/* result of function */

	MASNDSEQ_DBGMSG(("MaSound_Unload: id=%ld hd=%ld\n", sdFuncId, sdFileId));

	/* check sdFuncId and sdFileId */
	if ( (sdFuncId < 0) || (sdFuncId >= MASMW_NUM_SRMCNV) )
	{
		return MASMW_ERROR;
	}
	if ( ( ( gpMaSndSeqInfo->dSrmCnvMap & (UINT32)(1 << sdFuncId) ) == 0 )
		|| ( sdFileId < 0 ) || ( sdFileId >= MASMW_NUM_FILE ) )
	{
		return MASMW_ERROR;
	}

	/* when sdFileId == 0, call stream converter */
	if ( sdFileId == 0 )
	{
		sdResult = ma_srmcnv_func[sdFuncId].Unload( sdFileId, pvExtArgs );
		if ( sdResult >= MASMW_SUCCESS )
		{
			gpMaSndSeqInfo->bState[sdFuncId][sdFileId] = MASMW_STATE_IDLE;
		}
		return MASMW_SUCCESS;
	}

	/* check status */
	if ( gpMaSndSeqInfo->bState[sdFuncId][sdFileId] != MASMW_STATE_LOADED )
	{
		return MASMW_ERROR;
	}

	/* call stream converter */
	sdResult = ma_srmcnv_func[sdFuncId].Unload( sdFileId, pvExtArgs );

	if ( sdResult >= MASMW_SUCCESS )
	{
		/* change status to IDLE */
		gpMaSndSeqInfo->bState[sdFuncId][sdFileId] = MASMW_STATE_IDLE;
	}

	/* free callback function */
	gpMaSndSeqInfo->pfnCallBack[sdFuncId][sdResult] = NULL;

	return sdResult;
}
/****************************************************************************
 *	MaSound_Delete
 *
 *	Function:
 *			Deletes Stream Converter that has been registered with sdFuncId.
 *	Argument:
 *			sdFuncId	function id
 *	Return:
 *			0			success
 *			< 0			error code
 *
 ****************************************************************************/
SINT32 MaSound_Delete
(
	SINT32	sdFuncId					/* function id */
)
{
	UINT32	i;							/* loop counter */
	SINT32	sdResult;					/* result of function */
	
	MASNDSEQ_DBGMSG(("MaSound_Delete: id=%ld\n", sdFuncId));

	/* check sdFuncId */
	if ( (sdFuncId < 0) || (sdFuncId >= MASMW_NUM_SRMCNV) )
	{
		return MASMW_ERROR;
	}
	if ( ( gpMaSndSeqInfo->dSrmCnvMap & (UINT32)(1 << sdFuncId) ) == 0 )
	{
		return MASMW_ERROR;
	}

	/* check status of all file */
	for ( i = 0; i < MASMW_NUM_FILE; i++ )
	{
		if ( gpMaSndSeqInfo->bState[sdFuncId][i] != MASMW_STATE_IDLE )
		{
			return MASMW_ERROR;
		}
	}

	/* call stream converter */
	sdResult = ma_srmcnv_func[sdFuncId].End();

	/* free stream converter */
	gpMaSndSeqInfo->dSrmCnvMap ^= (UINT32)(1 << sdFuncId);

	return sdResult;
}
/****************************************************************************
 *	MaSound_DeviceControl
 *
 *	Function:
 *			Control the hardware.
 *	Argument:
 *			bCmd	command number
 *			bParam1	parameter 1
 *			bParam2	parameter 2
 *			bParam3	parameter 3
 *	Return:
 *			>= 0	success
 *			< 0		error code
 *
 ****************************************************************************/
SINT32 MaSound_DeviceControl
(
	UINT8	bCmd,						/* command number */
	UINT8	bParam1,					/* parameter 1 */
	UINT8	bParam2,					/* parameter 2 */
	UINT8	bParam3						/* parameter 3 */
)
{
	SINT32	sdResult;					/* result of function */

	MASNDSEQ_DBGMSG(("MaSound_DeviceControl: cmd=%d p1=%d p2=%d p3=%d\n", bCmd, bParam1, bParam2, bParam3));

	if ( gpMaSndSeqInfo->dSeqStatus != MASNDSEQ_STATUS_USABLE )
	{
		return MASMW_ERROR;
	}

	/* set HV language type */
	if ( bCmd == MASMW_SET_HVTYPE )
	{
		gpMaSndSeqInfo->dHvType = bParam1;
		
		return MASMW_SUCCESS;
	}

	sdResult = MaSndDrv_DeviceControl( (UINT32)bCmd, (UINT32)bParam1, (UINT32)bParam2, (UINT32)bParam3 );

	return sdResult;
}
/****************************************************************************
 *	MaSound_Initialize
 *
 *	Function:
 *			Initialize the MA Sound Sequencer.
 *	Argument:
 *			None
 *	Return:
 *			0		success
 *			< 0 	error code
 *
 ****************************************************************************/
SINT32 MaSound_Initialize
(
	void
)
{
	UINT32	i, j;						/* loop counter */

	MASNDSEQ_DBGMSG(("MaSound_Initialize\n"));
	
	gpMaSndSeqInfo = &gMaSndSeqInfo;


	/* Initialzie stream converter mapping flag */
	gpMaSndSeqInfo->dSrmCnvMap = 0;

	/* Initillize the all internal status */
	for ( i = 0; i < MASMW_NUM_SRMCNV; i++ )
	{
		for ( j = 0; j < MASMW_NUM_FILE; j++ )
		{
			gpMaSndSeqInfo->bState[i][j] = MASMW_STATE_IDLE;
			gpMaSndSeqInfo->pfnCallBack[i][j] = NULL;
		}
	}

	/* Initialize MA Device Driver */
	MaDevDrv_Initialize();

	/* Initialize MA Resource Manager */
	MaResMgr_Initialize();

	/* Initialize MA Sound Driver */
	MaSndDrv_Initialize();

#if MASMW_SRMCNV_SMAF_PHRASE
	PhrAudCnv_Initialize();
#endif

	/* set default of HV language type */
	gpMaSndSeqInfo->dHvType = MASMW_DEFAULT_HVTYPE;

	gpMaSndSeqInfo->dSeqStatus = MASNDSEQ_STATUS_USABLE;

	return MASMW_SUCCESS;
}
/****************************************************************************
 *	MaSound_Terminate
 *
 *	Function:
 *			Terminate the MA Sound Sequencer.
 *	Argument:
 *			None
 *	Return:
 *			0		success
 *			< 0 	error code
 *
 ****************************************************************************/
SINT32 MaSound_Terminate
(
	void
)
{
	MASNDSEQ_DBGMSG(("MaSound_Terminate\n"));

	/* error if any stream converters are regietered  */
	if ( gpMaSndSeqInfo->dSrmCnvMap != 0 ) 
	{
		return MASMW_ERROR;
	}

	gpMaSndSeqInfo->dSeqStatus = MASNDSEQ_STATUS_NOTINIT;


	return MASMW_SUCCESS;
}

