/****************************************************************************
 *
 *		Copyright (C) 2002-2003	YAMAHA CORPORATION. All rights reserved.
 *
 *		Module		: masndseq.h
 *
 *		Description	: MA Sound Sequencer
 *
 *		Version		: 2.1.9 	2003.07.09
 *
 ****************************************************************************/

#ifndef __MASNDSEQ_H__
#define __MASNDSEQ_H__

#include "mamachdep.h"
#include "madebug.h"

#define MA_STOPWAIT_TIMEOUT	(1000)	/* ms */

#define MASMW_CNVID_MMF				(1)		/* SMAF/MA-1/MA-2/MA-3/MA-5 */
#define MASMW_CNVID_PHR				(2)		/* SMAF/Phrase L1/L2 */
#define	MASMW_CNVID_RMD				(3)		/* Realtime MIDI */
#define	MASMW_CNVID_AUD				(4)		/* SMAF/Audio */
#define	MASMW_CNVID_MID				(5)		/* SMF */
#define	MASMW_CNVID_HVS				(9)		/* HV Script */
#define	MASMW_CNVID_WAV				(11)	/* WAVE */


typedef struct _MASRMCNVFUNC
{
	SINT32	(* Init)	( void );
	SINT32	(* Load)	( UINT8 * file_ptr, UINT32 file_size, UINT32 mode, SINT32 (*func)(UINT8 id), void * ext_args );
	SINT32 	(* Open)	( SINT32 file_id, void * ext_args );
	SINT32	(* Control)	( SINT32 file_id, UINT32 ctrl_num, void * prm, void * ext_args );
	SINT32 	(* Standby)	( SINT32 file_id, void * ext_args );
	SINT32 	(* Seek)	( SINT32 file_id, UINT32 pos, void * ext_args );
	SINT32 	(* Start)	( SINT32 file_id, void * ext_args );
	SINT32 	(* Stop)	( SINT32 file_id, void * ext_args );
	SINT32 	(* Pause)	( SINT32 file_id, void * ext_args );
	SINT32 	(* Restart)	( SINT32 file_id, void * ext_args );
	SINT32 	(* Close)	( SINT32 file_id, void * ext_args );
	SINT32	(* Unload)	( SINT32 file_id, void * ext_args );
	SINT32	(* End)		( void );
} MASRMCNVFUNC, *PMASRMCNVFUNC;

SINT32 dummy_Init   ( void );
SINT32 dummy_Load   ( UINT8 * file_ptr, UINT32 file_size, UINT32 mode, SINT32 (*func)(UINT8 id), void * ext_args );
SINT32 dummy_Open   ( SINT32 file_id, void * ext_args );
SINT32 dummy_Control( SINT32 file_id, UINT32 ctrl_num, void * prm, void * ext_args );
SINT32 dummy_Standby( SINT32 file_id, void * ext_args );
SINT32 dummy_Seek   ( SINT32 file_id, UINT32 pos, void * ext_args );
SINT32 dummy_Start  ( SINT32 file_id, void * ext_args );
SINT32 dummy_Stop   ( SINT32 file_id, void * ext_args );
SINT32 dummy_Close  ( SINT32 file_id, void * ext_args );
SINT32 dummy_Unload ( SINT32 file_id, void * ext_args );
SINT32 dummy_End    ( void );


SINT32	__MaSound_Initialize	( void );
SINT32	__MaSound_Terminate		( void );
SINT32	__MaSound_Create		( UINT8 srm_id );
SINT32	__MaSound_Load			( SINT32 func_id, UINT8 * file_ptr, UINT32 file_size, UINT8 mode, SINT32 (* func)(UINT8 id), void * ext_args );
SINT32	__MaSound_Open			( SINT32 func_id, SINT32 file_id, UINT16 open_mode, void * ext_args );
SINT32	__MaSound_Control		( SINT32 func_id, SINT32 file_id, UINT8 ctrl_num, void * prm, void * ext_args );
SINT32	__MaSound_Standby		( SINT32 func_id, SINT32 file_id, void * ext_args );
SINT32	__MaSound_Seek			( SINT32 func_id, SINT32 file_id, UINT32 pos, UINT8 flag, void * ext_args );
SINT32	__MaSound_Start			( SINT32 func_id, SINT32 file_id, UINT16 play_mode, void * ext_args );
SINT32	__MaSound_Pause			( SINT32 func_id, SINT32 file_id, void * ext_args );
SINT32	__MaSound_Restart		( SINT32 func_id, SINT32 file_id, void * ext_args );
SINT32	__MaSound_Stop			( SINT32 func_id, SINT32 file_id, void * ext_args );
SINT32	__MaSound_Close			( SINT32 func_id, SINT32 file_id, void * ext_args );
SINT32	__MaSound_Unload		( SINT32 func_id, SINT32 file_id, void * ext_args );
SINT32	__MaSound_Delete		( SINT32 func_id );
SINT32	__MaSound_DeviceControl	( UINT8 p1, UINT8 p2, UINT8 p3, UINT8 p4 );

SINT32	MaSound_ReceiveMessage	( SINT32 seq_id, SINT32 file_id, UINT8 event );

#endif /*__MASNDSEQ_H__*/
