#
# Mobile Innovation MIDI driver module makefile for various melody chip.
#
# Description: 
#	All config for MIDI driver come from mobin_var.mak.
#
# History:
#	2004.06.29	Robert.Chen created.
#

MELODY_DIR := $(DRIVERS_DIR)/board/melody
TARID := $(MBRD)$(PRNO)$(MIDIICNO)$(MIDISCH)$(MIDICLEDS)$(MIDISTER)$(MIDITR)
MIDICOPT := 
MIDICOPT += -dTARGET=$(MBOARD)

#
# detecting driver config for MIDI device
#
ifeq ($(MIDICHIP),1)
MIDI_IC_DRV_DIR := $(MELODY_DIR)/762
endif

ifeq ($(MIDICHIP),2)
MIDI_IC_DRV_DIR := $(MELODY_DIR)/765
endif

ifndef MIDI_IC_DRV_DIR
MIDI_IC_DRV_DIR := $(MELODY_DIR)/762
endif

MIDICOPT += -I$(CONDAT_BASE_DIR)/ms/src/mfw
MIDICOPT += -I$(CONDAT_BASE_DIR)/ms/src/bmi

ifeq ($(MIDITRACE),1)
MIDICOPT += -dMIDI_TRACE=1
else
MIDICOPT += -dMIDI_TRACE=0
endif

ifeq ($(MIDIPROCESS),1)
MIDICOPT += -dMIDITASK=1
endif

ifeq ($(MIDIPROCESS),2)
MIDICOPT += -dMIDITASK=2
endif

ifeq ($(MIDIPROCESS),3)		# not supported at present, 2004-06-29
MIDICOPT += -dMIDITASK=3
endif

ifeq ($(MIDICHIP),1)
MIDICOPT += -dMELODY_DEVICE=1
endif

ifeq ($(MIDICHIP),2)
MIDICOPT += -dMELODY_DEVICE=2
endif

ifeq ($(MIDICLED),1)
MIDICOPT += -dCOLORLED_SUPPORT
else
MIDICOPT += -uCOLORLED_SUPPORT
endif

SRC_MIDI_IC_DRV :=	$(MIDI_IC_DRV_DIR)/mamachdep.c \
				$(MIDI_IC_DRV_DIR)/madevdrv.c \
				$(MIDI_IC_DRV_DIR)/masnddrv.c \
				$(MIDI_IC_DRV_DIR)/malib.c \
				$(MIDI_IC_DRV_DIR)/mamidcnv.c \
				$(MIDI_IC_DRV_DIR)/mammfcnv.c \
				$(MIDI_IC_DRV_DIR)/maphrcnv.c \
				$(MIDI_IC_DRV_DIR)/marmdcnv.c \
				$(MIDI_IC_DRV_DIR)/mawavcnv.c \
				$(MIDI_IC_DRV_DIR)/maresmgr.c \
				$(MIDI_IC_DRV_DIR)/masndseq.c \
				$(MIDI_IC_DRV_DIR)/masnddva.c

SRC_MIDI_APP_DRV :=	$(MELODY_DIR)/melody_api.c \
				$(MELODY_DIR)/melody_env.c

LIB_MELODY := $(OUT_LIB_DIR)/melody$(TARID).lib

OBJ_MIDI_IC_DRV  := $(patsubst %.c,%.obj,$(filter %.c,$(SRC_MIDI_IC_DRV)))
OBJ_MIDI_APP_DRV := $(patsubst %.c,%.obj,$(filter %.c,$(SRC_MIDI_APP_DRV)))

CFG_OBJS += $(OBJ_MIDI_IC_DRV) $(OBJ_MIDI_APP_DRV)

CFG_DEPS += $(OBJ_MIDI_IC_DRV:.obj=.d)
CFG_DEPS += $(OBJ_MIDI_APP_DRV:.obj=.d)

CFG_LIBS += $(LIB_MELODY)

$(LIB_MELODY) : CLRAPP $(OBJ_MIDI_IC_DRV) $(OBJ_MIDI_APP_DRV) 
	ar470 r $(LIB_MELODY) $(OBJ_MIDI_IC_DRV) $(OBJ_MIDI_APP_DRV)

# rebuild melody applicaion driver to avoid config conflict
FORCE:
CLRAPP :
	-rm $(OBJ_MIDI_APP_DRV)

$(OBJ_MIDI_IC_DRV) : %.obj: %.c
	cl470 $(DRIVERS_FLAGS) $(MIDICOPT) -fr$(@D) $<

$(OBJ_MIDI_APP_DRV) : %.obj : %.c
	cl470 $(DRIVERS_FLAGS) $(MIDICOPT) -fr$(@D) $<

