//===========================================================================
//	HAL_Regs.H
//
//	This file contains offsets for every 13716 register.
//	The declarations in this file are used to reference registers in
//	virtually every S1D13716 program. Using these constants makes it
//	easier to determine which register the program is attempting to access.
//---------------------------------------------------------------------------
//	Copyright (c) 2003 Epson Research and Development, Inc.
//	All Rights Reserved.
//===========================================================================
// (Tabs set to every 4)


#ifndef __HAL_REGS_H__
#define __HAL_REGS_H__

#define HAL_REGSIZE					200			// Estimated maximum number of registers in HAL_STRUCT
#define HAL_LCDSIZE					256			// Estimated maximum number of LCD entries in HAL_STRUCT
#define HAL_REGBYTES				  1			// Size, in bytes, of register accesses for this chip.
#define HAL_REGCAMSIZE 				200

// 13716 LCD Register Names

#define REG_LCD_BASE				0x0000		// Offset from beginning of 13716 addres space to first LCD register

// Product Information

#define REG0000_PRODINFO0			0x0000
#define REG0001_PRODINFO1			0x0001
#define REG0002_CFGPINS				0x0002
#define REG0008_SOFTRESET			0x0008

// Clock Setting

#define REG0010_PLL0				0x0010
#define REG0011_PLL1				0x0011
#define REG0012_PLL2				0x0012
#define REG0014_CLKSRC				0x0014
#define REG0015_SERLCDCLK			0x0015
#define REG0016_LCDPCLKDIV			0x0016
#define REG0018_POWERSAVE			0x0018

// Host Interface

#define REG0020_HOSTMEMADDR0		0x0020
#define REG0021_HOSTMEMADDR1		0x0021
#define REG0022_HOSTMEMADDR2		0x0022
#define REG0024_HOSTAUTOINC			0x0024
#define REG0028_HOSTMEMPORT0		0x0028
#define REG0029_HOSTMEMPORT1		0x0029

// LCD Panel Interface Generic Settings

#define REG0030_LCDCMD0				0x0030
#define REG0031_LCDCMD1				0x0031
#define REG0032_LCDWRITE0			0x0032
#define REG0033_LCDWRITE1			0x0033
#define REG0034_LCDREAD0			0x0034
#define REG0035_LCDREAD1			0x0035
#define REG0036_LCDTRIG				0x0036
#define REG0038_LCDSTATUS			0x0038
#define REG003A_LCDFRAMEXFER		0x003A
#define REG003C_LCDXFER				0x003C
#define REG003E_XFERDELAY0			0x003E
#define REG003F_XFERDELAY1			0x003F

// Display Setting

#define REG0040_LCD1SETTING			0x0040
#define REG0044_LCD2SETTING			0x0044
#define REG0048_LCD3SETTING			0x0048
#define REG004C_DISPSETTING			0x004C
#define REG004E_SCALEUP				0x004E

// GPIO Registers

#define REG0060_GPIOCONFIG			0x0060
#define REG0062_GPIOPULLDOWN		0x0062
#define REG0064_GPIOSTATUS			0x0064
#define REG0066_GPIOPET				0x0066
#define REG0068_GPIONET				0x0068

// PWM Registers
#define REG0070_PWMCTRL0			0x0070
#define REG0072_PWMCTRL1			0x0072
#define REG0073_PWMDIV				0x0073
#define REG0074_PWMREDON			0x0074
#define REG0075_PWMREDOFF			0x0075
#define REG0076_PWMGREENON			0x0076
#define REG0077_PWMGREENOFF			0x0077
#define REG0078_PWMBLUEON			0x0078
#define REG0079_PWMBLUEOFF			0x0079
#define REG007A_PWMSLOPE0			0x007A
#define REG007B_PWMSLOPE1			0x007B
#define REG007C_PWMCYCLE0			0x007C
#define REG007D_PWMCYCLE1			0x007D
#define REG007E_PWMWHITEPERIOD		0x007E
#define REG007F_PWMWHITECYCLE		0x007F

// I2C Registers

#define REG0080_I2CSTATUS			0x0080
#define REG0082_I2CCOMMAND			0x0082
#define REG0084_I2CSTARTSTOP		0x0084
#define REG0086_I2CREAD				0x0086
#define REG0088_I2CWRITE			0x0088
#define REG008A_I2CREADACK			0x008A

// Camera Interface Registers

#define REG0100_CMCLK				0x0100
#define REG0102_CMSIGNAL			0x0102
#define REG0104_CMMODE0				0x0104
#define REG0106_CMMODE1				0x0106
#define REG0108_CMCONTROL			0x0108
#define REG010A_CMSTATUS			0x010A
#define REG0110_CMSTROBECTRL		0x0110
#define REG0112_CMSTROBEWIDTH0		0x0112
#define REG0113_CMSTROBEWIDTH1		0x0113
#define REG0114_CMSTROBEDELAY0		0x0114
#define REG0115_CMSTROBEDELAY1		0x0115

// Resize Operation

#define REG0200_GRESIZE				0x0200
#define REG0202_RESIZE				0x0202
#define REG0204_RESIZESX0			0x0204
#define REG0205_RESIZESX1			0x0205
#define REG0206_RESIZESY0			0x0206
#define REG0207_RESIZESY1			0x0207
#define REG0208_RESIZEEX0			0x0208
#define REG0209_RESIZEEX1			0x0209
#define REG020A_RESIZEEY0			0x020A
#define REG020B_RESIZEEY1			0x020B
#define REG020C_RESIZEHSCALE		0x020C
#define REG020E_RESIZEVSCALE		0x020E
#define REG0210_RESIZEOP			0x0210
#define REG0212_MEMHORZRES			0x0212
#define REG0214_MEMVERTRES			0x0214
#define REG0216_RYC					0x0216

// YUV / RGB Translate Mode

#define REG0220_YUVRGBXLATE0		0x0220
#define REG0222_YUVRGBXLATE1		0x0222
#define REG0224_YUVWRSTART0			0x0224
#define REG0225_YUVWRSTART1			0x0225
#define REG0226_YUVWRSTART2			0x0226
#define REG0228_XOFFSET0			0x0228
#define REG0229_XOFFSET1			0x0229
#define REG022A_YOFFSET0			0x022A
#define REG022B_YOFFSET1			0x022B
#define REG022C_VDATACLIP			0x022C
#define REG022E_UDATACLIP			0x022E
#define REG0240_OSADDR0				0x0240
#define REG0241_OSADDR1				0x0241
#define REG0242_OSADDR2				0x0242
#define REG0244_OHRES				0x0244
#define REG0246_OVRES				0x0246
#define REG0248_OIMAGE				0x0248
#define REG024A_SWIVELVIEW			0x024A

#define REG0250_MJPEG				0x0250
#define REG0252_BUF2ADDR0			0x0252
#define REG0253_BUF2ADDR1			0x0253
#define REG0254_BUF2ADDR2			0x0254


// Interrupt Control

#define REG0300_IRQSTATUS0			0x0300
#define REG0301_IRQSTATUS1			0x0301
#define REG0302_IRQENABLE0			0x0302
#define REG0303_IRQENABLE1			0x0303
#define REG0304_IRQCONTROL			0x0304
#define REG0306_DEBUGSTATUS			0x0306
#define REG0308_IRQCTRLDEBUG		0x0308
#define REG030A_RESERVEDDEBUG0		0x030A
#define REG030C_RESERVEDDEBUG1		0x030C

// LUT

#define REG0400_LUTRED				0x0400
#define REG0401_LUTGREEN			0x0401
#define REG0402_LUTBLUE				0x0402

// JPEG Function

#define REG0800_JPEGCONTROL			0x0800
#define REG0802_JPEGSTATUS0			0x0802
#define REG0804_JPEGSTATUS1			0x0804
#define REG0806_JPEGRAWSTATUS0		0x0806
#define REG0808_JPEGRAWSTATUS1		0x0808
#define REG080A_JPEGIRQ0			0x080A
#define REG080C_JPEGIRQ1			0x080C
#define REG080E_JPEGRESET			0x080E
#define REG0810_JPEGSTARTSTOP		0x0810
#define REG0814_JPEGPSIZEADJUST		0x0814

// JPEG File Buffer Setting

#define REG0816_JPEGFIFOCTRL		0x0816
#define REG0818_JPEGFIFOSTATUS		0x0818
#define REG081A_JPEGFIFOWRITE		0x081A
#define REG081C_ENCODERESULT0		0x081C
#define REG081D_ENCODERESULT1		0x081D
#define REG081E_ENCODERESULT2		0x081E
#define REG0820_ENCODELIMIT0		0x0820
#define REG0821_ENCODELIMIT1		0x0821
#define REG0822_JPEGFILESIZE0		0x0822
#define REG0823_JPEGFILESIZE1		0x0823
#define REG0824_JPEGFILESIZE2		0x0824
#define REG0825_JPEGLINESTATUS		0x0825
#define REG0826_JPEGLINERAW			0x0826
#define REG0828_JPEGLINERAWC		0x0828
#define REG082A_JPEGLINEIRQ			0x082A
#define REG0900_JPEGLINEWRITE		0x0900

// JPEG CODEC

#define REG1000_OPMODE				0x1000
#define REG1002_COMMAND				0x1002
#define REG1004_JPEGOP				0x1004
#define REG1006_QUANTNUM			0x1006
#define REG1008_HUFFMANNUM			0x1008
#define REG100A_DRI0				0x100A
#define REG100C_DRI1				0x100C
#define REG100E_VERTPIXELSIZE0		0x100E
#define REG1010_VERTPIXELSIZE1		0x1010
#define REG1012_HORZPIXELSIZE0		0x1012
#define REG1014_HORZPIXELSIZE1		0x1014
#define REG1016_DNLVALUE0			0x1016
#define REG1018_DNLVALUE1			0x1018
#define REG101C_RSTOPSETTING		0x101C
#define REG101E_RSTOPSTATUS			0x101E
#define REG1020_JINSMARKER			0x1020
#define REG1200_JQUANT0				0x1200
#define REG1280_JQUANT1				0x1280
#define REG1400_JDCHUFF0REG0		0x1400
#define REG1420_JDCHUFF0REG1		0x1420
#define REG1440_JACHUFF0REG0		0x1440
#define REG1460_JACHUFF0REG1		0x1460
#define REG1600_JDCHUFF1REG0		0x1600
#define REG1620_JDCHUFF1REG1		0x1620
#define REG1640_JACHUFF1REG0		0x1640
#define REG1660_JACHUFF1REG1		0x1660

#define REGAREA_END					0x17FF	// Amount of space the above registers require.

#define REGFLAG_BASE				0xFFF0	// Special reserved flags above this point.
#define REGFLAG_DELAY				0xFFFC	// Indicates a microsecond delay at any point during initialization.
#define REGFLAG_PLLDELAY			0xFFFC	// Indicates a microsecond delay for programming the PLL.
#define REGFLAG_OFFDELAY			0xFFFD	// Indicates a millisecond delay for powering down LCD.
#define REGFLAG_ONDELAY				0xFFFE	// Indicates a millisecond delay for powering up LCD.
#define REGFLAG_ENDOFTABLE			0xFFFF	// End of register table flag.


#endif // __HAL_REGS_H__
