//===========================================================================
// chiplib.c
//
// For this file, set tabs to 4 spaces.
//---------------------------------------------------------------------------
//
//	Copyright (c) 2002 Epson Research and Development, Inc.
//	All Rights Reserved.
//
//===========================================================================

#include <string.h>
#include "stdio.h"
#include "hal.h"
#include "chiplib.h"

//
// Application MUST always call InitChipLib() when starting up.
//
static Boolean gfUsing13716 = FALSE;

//---------------------------------------------------------------------------
//	FUNCTION:		InitChipLib()
//---------------------------------------------------------------------------

void InitChipLib( void )
{
#ifdef EPSON_ORIGINAL
	gfUsing13716 = !strcmp( HalInfo.szChipId, "13716" );
#endif	
}


//---------------------------------------------------------------------------
//	FUNCTION:		is13716Chip()
//---------------------------------------------------------------------------

int is13716Chip( void )
{
	return gfUsing13716;
}

//---------------------------------------------------------------------------
//	FUNCTION:		GetChipName()
//---------------------------------------------------------------------------

char *GetChipName( void )
{
#ifdef EPSON_ORIGINAL
	return HalInfo.szChipId;
#else
	return NULL;
#endif
}

//---------------------------------------------------------------------------
//	FUNCTION:		GetChipId()
//---------------------------------------------------------------------------

void GetChipId( UInt32 *pProductCode, UInt32 *pRevisionCode )
{
	UInt8 Reg00 = halReadReg8( REG0000_PRODINFO0 );

	*pProductCode = (Reg00 >> 2) & 0x3f;
	*pRevisionCode = Reg00 & 0x03;
}


//---------------------------------------------------------------------------
//	FUNCTION:		GetMemSize()
//---------------------------------------------------------------------------

UInt32 GetMemSize( void )
{
	UInt8 reg = halReadReg8( REG0001_PRODINFO1 );

	return ( reg * 4096UL );
}


//---------------------------------------------------------------------------
//	FUNCTION:		SetBitsPerPixel()
//---------------------------------------------------------------------------

void SetBitsPerPixel( WindowDef Window, int bpp )
{
	if ( Window == cl_OVERLAY_WINDOW )
	{
		UInt8 reg = halReadReg8( REG004C_DISPSETTING );

		//
		// Program the bits-per-pixel select bits
		//
		reg &= ~0x20;
		
		if (bpp == 8)
			reg |= 0x20;

		halWriteReg8( REG004C_DISPSETTING, reg );
	}
}


//---------------------------------------------------------------------------
//	FUNCTION:		GetBitsPerPixel()
//---------------------------------------------------------------------------

int GetBitsPerPixel( WindowDef Window )
{
	if ( Window == cl_OVERLAY_WINDOW )
		return (halReadReg8(REG004C_DISPSETTING) & 0x20) ? 8 : 16;
	else
		return 16;
}


//---------------------------------------------------------------------------
//	FUNCTION:		SetLutEntry()
//---------------------------------------------------------------------------

void SetLutEntry( int addr, UInt8 red, UInt8 green, UInt8 blue )
{
	UInt32 reg = REG0400_LUTRED + addr*4;

	halWriteReg8( reg++, red );
	halWriteReg8( reg++, green );
	halWriteReg8( reg, blue );
}

//---------------------------------------------------------------------------
//	FUNCTION:		GetLutEntry()
//---------------------------------------------------------------------------

void GetLutEntry( int addr, pUInt8 pRed, pUInt8 pGreen, pUInt8 pBlue )
{
	UInt32 reg = REG0400_LUTRED + addr*4;

	if ( pRed != NULL )
		*pRed = halReadReg8( reg );

	reg++;

	if ( pGreen != NULL )
		*pGreen = halReadReg8( reg );

	reg++;

	if ( pBlue != NULL )
		*pBlue = halReadReg8( reg );
}

//---------------------------------------------------------------------------
//	FUNCTION:		SetLut()
//---------------------------------------------------------------------------

void SetLut( pUInt8 pLut, int size )
{
	int i;
	UInt32 reg = REG0400_LUTRED;

	for ( i=0; i<size; i++ )
	{
		halWriteReg8( reg++, *pLut++ );
		halWriteReg8( reg++, *pLut++ );
		halWriteReg8( reg++, *pLut++ );
		reg++;	// Must skip over 4 bytes per LUT entry
	}
}

//---------------------------------------------------------------------------
//	FUNCTION:		InitLut()
//---------------------------------------------------------------------------

void InitLut( UInt32 LutMode )
{
#if 0
	int i;
	UInt8 red, green, blue, val8;


	if (LutMode & cl_LUT_TEST_PATTERN)
		{
		LutMode &= ~cl_LUT_TEST_PATTERN;

		switch (LutMode)
			{
			case cl_LUT_NOBYPASS_8BPP_256COLOR_LUT:
				// LUT[0]    - LUT[63]     is red
				// LUT[64]   - LUT[64*2-1] is green
				// LUT[64*2] - LUT[64*3-1] is blue
				// LUT[64*3] - LUT[64*4-1] is gray shades
				for ( i=0; i<64; i++ )
					{
					val8 = (UInt8) (i<<2);
				
					SetLutEntry(i,        val8, 0,    0);
					SetLutEntry(i + 64,   0,    val8, 0);
					SetLutEntry(i + 64*2, 0,    0,    val8);
					SetLutEntry(i + 64*3, val8, val8, val8);
					}
				return;
				break;
			}
		}

	switch (LutMode)
		{
		default:
			break;

		case cl_LUT_NOBYPASS_8BPP_256COLOR_LUT:
			SetLut(LutInfo.lut1, (sizeof(LutInfo.lut1) / sizeof(LutInfo.lut1[0]))/3);
			break;

		case cl_LUT_NOBYPASS_16BPP_565LUT:
			for (i = 0; i < 64; ++i)
				{
				if (i < 32)
					red = blue = (UInt8) ((i * 0xff) / 31);
				else
					red = blue = 0xff;

				green = (UInt8) ((i * 0xff) / 63);

				SetLutEntry(i, red, green, blue);
				}
			break;
		}
#endif
}


//---------------------------------------------------------------------------
//	FUNCTION:		GetStartAddress()
//---------------------------------------------------------------------------

UInt32 GetStartAddress( WindowDef Window )
{
	if ( Window == cl_OVERLAY_WINDOW )
		return ((UInt32)(halReadReg8(REG0242_OSADDR2) & 0x01)<<16) | ((UInt32)(halReadReg8(REG0241_OSADDR1) & 0xFF)<<8) | ((UInt32)(halReadReg8(REG0240_OSADDR0) & 0xFF));
	else
		return ((UInt32)(halReadReg8(REG0226_YUVWRSTART2) & 0x01)<<16) | ((UInt32)(halReadReg8(REG0225_YUVWRSTART1) & 0xFF)<<8) | ((UInt32)(halReadReg8(REG0224_YUVWRSTART0) & 0xFF));
}

//---------------------------------------------------------------------------
//	FUNCTION:		SetStartAddress()
//---------------------------------------------------------------------------

void SetStartAddress( WindowDef Window,UInt32 Address )
{
	if ( Window == cl_OVERLAY_WINDOW )
	{
		halWriteReg8(REG0242_OSADDR2,(UInt8)(Address>>16)); 
		halWriteReg8(REG0241_OSADDR1,(UInt8)(Address>>8));
		halWriteReg8(REG0240_OSADDR0,(UInt8)(Address));
	}
	else
	{
		halWriteReg8(REG0226_YUVWRSTART2,(UInt8)(Address>>16)); 
		halWriteReg8(REG0225_YUVWRSTART1,(UInt8)(Address>>8));
		halWriteReg8(REG0224_YUVWRSTART0,(UInt8)(Address));	
	}
}

//---------------------------------------------------------------------------
//	FUNCTION:		GetMemAddress()
//---------------------------------------------------------------------------

UInt32 GetMemAddress( UInt32 BaseAddress )
{
	return BaseAddress;
}


//---------------------------------------------------------------------------
//	FUNCTION:		GetStride()
//---------------------------------------------------------------------------

UInt32 GetStride( WindowDef Window )
{
	return GetDisplayWidth(Window) * GetBitsPerPixel(Window) / 8;
}


//---------------------------------------------------------------------------
//	FUNCTION:		GetDisplayWidth()
//					GetDisplayHeight()
//---------------------------------------------------------------------------

int GetDisplayWidth( WindowDef Window )
{
	if ( Window == cl_OVERLAY_WINDOW )
	{
		return halReadReg8(REG0244_OHRES) + 1;
	}
	else
	{
		UInt32 xStart, yStart, xEnd, yEnd;
		GetResizerSize( &xStart, &yStart, &xEnd, &yEnd );
		return xEnd - xStart + 1;
	}
}


int GetDisplayHeight( WindowDef Window )
{
	if ( Window == cl_OVERLAY_WINDOW )
	{
		return halReadReg8(REG0246_OVRES) + 1;
	}
	else
	{
		UInt32 xStart, yStart, xEnd, yEnd;
		GetResizerSize( &xStart, &yStart, &xEnd, &yEnd );
		return yEnd - yStart + 1;
	}
}


//---------------------------------------------------------------------------
//	FUNCTION:		SetResizerSize()
//---------------------------------------------------------------------------

void SetResizerSize( UInt32 xStart, UInt32 yStart, UInt32 xEnd, UInt32 yEnd )
{
	halWriteReg8( REG0204_RESIZESX0, (UInt8)xStart );
	halWriteReg8( REG0205_RESIZESX1, (UInt8)(xStart>>8) );

	halWriteReg8( REG0206_RESIZESY0, (UInt8)yStart );
	halWriteReg8( REG0207_RESIZESY1, (UInt8)(yStart>>8) );

	halWriteReg8( REG0208_RESIZEEX0, (UInt8)xEnd );
	halWriteReg8( REG0209_RESIZEEX1, (UInt8)(xEnd>>8) );

	halWriteReg8( REG020A_RESIZEEY0, (UInt8)yEnd );
	halWriteReg8( REG020B_RESIZEEY1, (UInt8)(yEnd>>8) );
}


//---------------------------------------------------------------------------
//	FUNCTION:		GetResizerSize()
//---------------------------------------------------------------------------

void GetResizerSize( pUInt32 xStart, pUInt32 yStart, pUInt32 xEnd, pUInt32 yEnd )
{
	if (xStart != NULL)
		*xStart = halReadReg8(REG0204_RESIZESX0)|(halReadReg8(REG0205_RESIZESX1)<<8);

	if (yStart != NULL)
		*yStart = halReadReg8(REG0206_RESIZESY0)|(halReadReg8(REG0207_RESIZESY1)<<8);

	if (xEnd != NULL)
		*xEnd = halReadReg8(REG0208_RESIZEEX0) | (halReadReg8(REG0209_RESIZEEX1)<<8);

	if (yEnd != NULL)
		*yEnd = halReadReg8(REG020A_RESIZEEY0) | (halReadReg8(REG020B_RESIZEEY1)<<8);
}

//----------------------------------------------------------------------------
// GetPllPowerDown()
//----------------------------------------------------------------------------

Boolean GetPllPowerDown( void )
{
	return (halReadReg8(REG0018_POWERSAVE)&0x01) ? TRUE : FALSE;
}

//---------------------------------------------------------------------------
//	SetPll()
//---------------------------------------------------------------------------

Boolean SetPll( UInt32 freq )
{
	UInt32 ClosestN, ClosestL, ClosestPll, ClosestDiff;
	UInt32 n, l, pll, diff;
	UInt32 ClosestVbits, v, vBits, factor;
	UInt32 Kv;
	Boolean PllProgrammed = FALSE;
	UInt32 PllClk;
	UInt8 val8;

#define MHZ   1000000UL
#define abs32(x, y)  ( (x>y) ? (x-y) : (y-x) )


	// Determine if the PLL clock is the CLKI External Clock Source,
	// or the 32 kHz External Oscillator.
	if ( halReadReg8(REG0014_CLKSRC) & 0x02 )
		PllClk = 32768;  // 32 kHz
	else	
		PllClk = HalInfo.dwClkI;
	
	//
	// Find the closest PLL frequency based on N and L
	//
	ClosestN = 0;
	ClosestL = 0;
	ClosestPll = (ClosestN+1) * (ClosestL+1) * PllClk;
	ClosestDiff = abs32(freq,ClosestPll);
	Kv = 0;


	n = 16;
	while ( n-- )   // n must loop from 15 to 0
		{
		for ( l=0; l<=1023; ++l )
			{
			pll = (n+1) * (l+1) * PllClk;

			diff = abs32(freq,pll);

			if ( diff < ClosestDiff )
				{
				ClosestN = n;
				ClosestL = l;
				ClosestPll = (ClosestN+1) * (ClosestL+1) * PllClk;
				ClosestDiff = abs32(freq,ClosestPll);
				}
			}
		}

	//
	// Determine V based on PLL frequency
	//
	// 100 MHz <= PLL * V <= 410 MHz
	// if V==2, V divider bits=1
	// if V==4, V divider bits=2
	// if V==8, V divider bits=3
	//

	ClosestVbits = 0;

	//
	// If the duty cycle is not important, lower power consumption can be achieved
	// by setting the V-divider bits and VCO Kv Set bits to 0.
	//
	if ( !(HalInfo.dwFlags&fPLL_LOWPOWER) )
		{
		for ( vBits=1; (vBits<=3)&&!PllProgrammed; vBits++ )
			{
			v = 1 << vBits;
		
			factor = ClosestPll * v;
		
			// Check if V is valid
			if ( (100*MHZ<=factor) && (factor<=410*MHZ) )
				{
				ClosestVbits = vBits;
			
				// Determine Kv
				//
				// if (100 MHz <= PLL * V <= 200 MHz) then Kv=2
				// if (200 MHz <  PLL * V <= 300 MHz) then Kv=5
				// if (300 MHz <  PLL * V <= 410 MHz) then Kv=7
			
				factor = ClosestPll * v;
				Kv = 0;
			
				if ( (100*MHZ<=factor) && (factor<=200*MHZ) )
					{
					Kv = 2;
					PllProgrammed = TRUE;
					}
				else if ( (200*MHZ<factor) && (factor<=300*MHZ) )
					{
					Kv = 5;
					PllProgrammed = TRUE;
					}
				else if ( (300*MHZ<factor) && (factor<=410*MHZ) )
					{
					Kv = 7;
					PllProgrammed = TRUE;
					}
				}
			}

		if ( !PllProgrammed )
			{
			// Ensure valid values
			if ( ClosestVbits == 0 )
				ClosestVbits = 1;
		
			if ( Kv==0 )
				Kv = 2;
			}
		}
	else
		PllProgrammed = TRUE;  // Don't need to program V or Kv

	ClosestN &= 0x0f;
	ClosestL &= 0x3ff;
	ClosestVbits &= 0x03;

	// Go into power save before writing to PLL registers
	PowerSaveMode( TRUE );
	halWriteReg8( REG0010_PLL0, (UInt8)((ClosestL<<2)|ClosestVbits) );
	halWriteReg8( REG0011_PLL1, (UInt8)((ClosestN<<4)|((ClosestL>>6)&0x0f)) );
	halWriteReg8( REG0012_PLL2, (UInt8)(Kv<<4) );

	// Enable PLL
	val8 = halReadReg8( REG0018_POWERSAVE );
	val8 |= 0x04;
	halWriteReg8( REG0018_POWERSAVE, val8 );

	// Restore original setting
	halDelayUS( 100 );
	PowerSaveMode( FALSE );

	return PllProgrammed;
}

//---------------------------------------------------------------------------
//	GetPll()
//---------------------------------------------------------------------------

UInt32 GetPll( int *error )
{
	UInt32 N, L, V, pll;
	UInt32 Kv;
	UInt8 reg10, reg11;
	UInt32 factor;
	int tmp;
	UInt32 PllClk;

	// The error would be set to NULL if the caller doesn't
	// want to get the error.
	if ( error == NULL )
		error = &tmp;


	*error = FALSE;

	// Determine if the PLL clock is the CLKI External Clock Source,
	// or the 32 kHz External Oscillator.
	if ( halReadReg8(REG0014_CLKSRC) & 0x02 )
		PllClk = 32768;  // 32 kHz
	else	
		PllClk = HalInfo.dwClkI;
	

	reg10 = halReadReg8( REG0010_PLL0 );
	reg11 = halReadReg8( REG0011_PLL1 );

	N = reg11 >> 4;
	L = ((reg11&0x0f)<<6) | (reg10>>2);

	pll = (N+1) * (L+1) * PllClk;


	//
	// If the duty cycle is not important, lower power consumption can be achieved
	// by setting the V-divider bits and VCO Kv Set bits to 0.
	//
	// Don't check these bits if PLL Low Power mode has been configured.
	//
	if ( !(HalInfo.dwFlags&fPLL_LOWPOWER) )
		{
		// Verify if PLL registers programmed correctly
		V = 1 << (reg10 & 0x03);
		Kv = halReadReg16(REG0012_PLL2) >> 4;
		
		factor = pll * V;
		
		if ( (100*MHZ<=factor) && (factor<=410*MHZ) )
			{
			if ( (100*MHZ<=factor) && (factor<=200*MHZ) )
				{
				if ( Kv != 2 )
					*error = TRUE;
				}
			else if ( (200*MHZ<factor) && (factor<=300*MHZ) )
				{
				if ( Kv != 5 )
					*error = TRUE;
				}
			else if ( (300*MHZ<factor) && (factor<=410*MHZ) )
				{
				if ( Kv != 7 )
					*error = TRUE;
				}
			}
		else
			*error = TRUE;
		}

	return pll;
}

//----------------------------------------------------------------------------
// GetPllMuxOutputFreq()
//----------------------------------------------------------------------------

UInt32 GetPllMuxOutputFreq( SrcClkDef *pSrcClk, PllMuxErrorDef *pPllMuxError )
{
	UInt32 PllMuxOutputFreq = 0;
	UInt8 reg14 = halReadReg8( REG0014_CLKSRC );

	// Default to no PLL Multiplexer errors
	if ( pPllMuxError != NULL )
		*pPllMuxError = PLLMUXERR_NONE;

	if ( reg14 & 0x01 )  // Internal PLL selected
	{
		int PllError;
		PllMuxOutputFreq = GetPll( &PllError );

		// If there is an error determining the PLL frequency,
		// update PLL Multiplexer error (unless pPllMuxError is NULL).
		if ( pPllMuxError != NULL )
		{
			if ( PllError )
				*pPllMuxError = PLLMUXERR_INVALID_VALUE;
			// If the external clock source is selected for the PLL reference, then
			// ensure that this reference frequency is 32.768 kHz.
			else if ( ((reg14&0x02)==0) && (HalInfo.dwClkI!=32768) )
				*pPllMuxError = PLLMUXERR_CLKI_NOT_32768;
		}

		if ( pSrcClk != NULL )
			*pSrcClk = SRC_PLL;
	}
	else
	{
		PllMuxOutputFreq = HalInfo.dwClkI;

		if ( pSrcClk != NULL )
			*pSrcClk = SRC_CLKI;
	}

	return PllMuxOutputFreq;
}

//----------------------------------------------------------------------------
// GetI2cClkDivide()
//----------------------------------------------------------------------------

int GetI2cClkDivide(void)
{
	int val;

	val = (halReadReg8(REG0015_SERLCDCLK)>>3) & 0x0f;

	if ( val == 0 )
		return 1;
	else
		return val * 2;
}

//----------------------------------------------------------------------------
// GetI2cClock()
//----------------------------------------------------------------------------

UInt32 GetI2cClock( SrcClkDef *pSrcClk, PllMuxErrorDef *pPllMuxError )
{
	UInt8 reg18 = halReadReg8( REG0018_POWERSAVE );
	UInt32 PllMuxOutputFreq = GetPllMuxOutputFreq(pSrcClk,pPllMuxError);

	if ( reg18 & 0x01 )
		return 0;  // I2C clock is 0 in powersave mode
	else
	{
	return PllMuxOutputFreq / GetI2cClkDivide();
	}
}

//----------------------------------------------------------------------------
// GetSystemClkDivide()
//----------------------------------------------------------------------------

int GetSystemClkDivide(void)
{
	return ((halReadReg8(REG0014_CLKSRC)>>2)&0x03) + 1;
}

//----------------------------------------------------------------------------
// GetSystemClock()
//----------------------------------------------------------------------------

UInt32 GetSystemClock( SrcClkDef *pSrcClk, PllMuxErrorDef *pPllMuxError )
{
	UInt8 reg18 = halReadReg8( REG0018_POWERSAVE );
	UInt32 PllMuxOutputFreq = GetPllMuxOutputFreq(pSrcClk,pPllMuxError);

	if ( reg18 & 0x01 )
		return 0;  // System clock is 0 in powersave mode
	else
	{
	return PllMuxOutputFreq / GetSystemClkDivide();
	}
}

//----------------------------------------------------------------------------
// GetParallelPanelClkDivide()
//----------------------------------------------------------------------------

UInt32 GetParallelPanelClkDivide( void )
{
	return ((halReadReg8(REG0014_CLKSRC)>>4)+1) * 2;
}

//----------------------------------------------------------------------------
// GetParallelPanelClkFreq()
//----------------------------------------------------------------------------

UInt32 GetParallelPanelClkFreq( void )
{
	return GetSystemClock( NULL, NULL ) / GetParallelPanelClkDivide();
}

//----------------------------------------------------------------------------
// GetSerialLcdClkDivide()
//----------------------------------------------------------------------------

int GetSerialLcdClkDivide( void )
{
	return ((halReadReg8(REG0015_SERLCDCLK)&0x07)+1) * 2;
}

//----------------------------------------------------------------------------
// GetSerialLcdClkFreq()
//----------------------------------------------------------------------------

UInt32 GetSerialLcdClkFreq( void )
{
	return GetSystemClock( NULL, NULL ) / GetSerialLcdClkDivide();
}

//----------------------------------------------------------------------------
// GetCameraClkDivide()
//----------------------------------------------------------------------------

int GetCameraClkDivide( void )
{
	return ((halReadReg8(REG0100_CMCLK)&0x0f)+1) * 2;
}

//----------------------------------------------------------------------------
// GetCameraClkOutFreq()
//----------------------------------------------------------------------------

UInt32 GetCameraClkOutFreq( void )
{
	return GetSystemClock( NULL, NULL ) / GetCameraClkDivide();
}

//----------------------------------------------------------------------------
// SetOutputPort()
//----------------------------------------------------------------------------

Boolean SetOutputPort( OutputPortDef port )
{
	UInt8 reg = (UInt8)(halReadReg8(REG004C_DISPSETTING)&~0x03);

	switch (port)
		{
		case cl_PORT_ALL_OFF:	break;
		case cl_PORT_LCD1:		reg |= 0x01; break;
		case cl_PORT_LCD2:		reg |= 0x02; break;
		case cl_PORT_LCD3:		reg |= 0x03; break;
		default: return FALSE;
		}

	halWriteReg8( REG004C_DISPSETTING, reg );
	return TRUE;
}

//----------------------------------------------------------------------------
// GetOutputPort()
//----------------------------------------------------------------------------

OutputPortDef GetOutputPort( void )
{
	UInt8 PortStatus = (UInt8) (halReadReg8(REG004C_DISPSETTING) & 0x03);

	switch (PortStatus)
		{
		default:
		case 0:  return cl_PORT_ALL_OFF; break;
		case 1:  return cl_PORT_LCD1; break;
		case 2:  return cl_PORT_LCD2; break;
		case 3:  return cl_PORT_LCD3; break;
		}
}


//----------------------------------------------------------------------------
// SetLutBypassEnable()
//----------------------------------------------------------------------------

void SetLutBypassEnable( Boolean enable )
{
	UInt8 reg = halReadReg8(REG004C_DISPSETTING);

	if (enable)
		reg &= ~0x0010;
	else
		reg |= 0x0010;  // LUT bypass disabled

	halWriteReg8(REG004C_DISPSETTING, reg);
}

//----------------------------------------------------------------------------
// GetLutBypassEnable()
//----------------------------------------------------------------------------

int GetLutBypassEnable( WindowDef Window )
{
	if ( Window == cl_OVERLAY_WINDOW )
		return (halReadReg8(REG004C_DISPSETTING) & 0x10) ? FALSE : TRUE;
	else
		return TRUE;
}

//----------------------------------------------------------------------------
// GetLutMode()
//----------------------------------------------------------------------------

UInt32 GetLutMode( WindowDef Window )
{
	int bpp;
	Boolean LutBypass;

	bpp = GetBitsPerPixel(Window);
	LutBypass = GetLutBypassEnable(Window);

	if (LutBypass)
		{
		switch (bpp)
			{
			case 8:  return cl_LUT_BYPASS_8BPP_332RGB;   break;
			case 16: return cl_LUT_BYPASS_16BPP_565RGB;  break;
			default: return cl_LUT_ERROR; break;
			}
		}
	else
		{
		switch (bpp)
			{
			case 8:  return cl_LUT_NOBYPASS_8BPP_256COLOR_LUT;   break;
			case 16: return cl_LUT_NOBYPASS_16BPP_565LUT;  break;
			default: return cl_LUT_ERROR; break;
			}
		}
}

//----------------------------------------------------------------------------
// PowerSaveMode()
//----------------------------------------------------------------------------

void PowerSaveMode( Boolean enable )
{
	UInt8 reg0018;

	reg0018 = halReadReg8( REG0018_POWERSAVE );

	if (enable)   // Enable Power Save Mode
		{
		// Enable Power Save mode
		reg0018 |= 0x01;
		halWriteReg8( REG0018_POWERSAVE, reg0018 );
		}
	else
		{
		// Disable Power Save mode
		reg0018 &= ~0x01;
		halWriteReg8( REG0018_POWERSAVE, reg0018 );
		}
}

//---------------------------------------------------------------------------
//  FUNCTION: isHostIndirectInterface()
//---------------------------------------------------------------------------
Boolean isHostIndirectInterface( void )
{
	return TRUE;	// The S1D13716 is always using the indirect interface
}

//---------------------------------------------------------------------------
//  FUNCTION: clStartOneFrameTransfer()
//---------------------------------------------------------------------------
Boolean clStartOneFrameTransfer( void )
{
	// Don't issue a frame transfer if no port is active
	if ( GetOutputPort() == cl_PORT_ALL_OFF )
		return FALSE;

	// Don't wait for busy bit if in auto frame transfer mode
	if ( (halReadReg8(REG003C_LCDXFER) & 0x01) == 0 )
	{
		while ( 0 != (0x01 & halReadReg8(REG0038_LCDSTATUS)) )
			continue;
	}

	halWriteReg8( REG003A_LCDFRAMEXFER, 1 );

	return TRUE;
}

//---------------------------------------------------------------------------
//	FUNCTION:		SetSwivelViewMode()
//---------------------------------------------------------------------------
void SetSwivelViewMode(WindowDef window, int mode)
{
	UInt8 swivel;
	Boolean fswivel;

	swivel=halReadReg8(REG024A_SWIVELVIEW);
	if(window==cl_OVERLAY_WINDOW)
	{
		fswivel=swivel & 0x03;
		if(!fswivel)
			return;
		else
			halWriteReg8(REG024A_SWIVELVIEW, swivel | 0x40);
	}

	swivel &=0xFc;
	switch(mode)
	{
	case 0:
		halWriteReg8(REG024A_SWIVELVIEW,swivel);
		break;		
	case 90:	
		halWriteReg8(REG024A_SWIVELVIEW,swivel |0x01);
		break;
	case 180:
		halWriteReg8(REG024A_SWIVELVIEW,swivel |0x02);
		break;
	case 270:
		halWriteReg8(REG024A_SWIVELVIEW,swivel |0x03);
		break;
	}
}

//----------------------------------------------------------------------------
// SetWinMirror()
//----------------------------------------------------------------------------
void SetWinMirror(WindowDef window, Boolean Enable)
{
	UInt8 reg;

	switch (window)
		{
		default:
		case cl_MAIN_WINDOW:
			reg = (UInt8) (halReadReg8(REG024A_SWIVELVIEW) & ~0x04);
			SelectHostImage(window);
			if (Enable)
				reg |= 0x04;
			break;

		case cl_OVERLAY_WINDOW:
			reg = (UInt8) (halReadReg8(REG024A_SWIVELVIEW) & ~0x10);
			SelectHostImage(window);
			if (Enable)
				reg |= 0x0010;
			break;
		}

	halWriteReg8(REG024A_SWIVELVIEW, reg);
}

//---------------------------------------------------------------------------
//	FUNCTION:		SetOverlayEnable()
//---------------------------------------------------------------------------
void SetOverlayEnable(Boolean Enable)
{
	UInt8 reg = halReadReg8(REG004C_DISPSETTING);

	reg &= ~0x40;

	if(Enable)
		reg |= 0x40;
	halWriteReg8(REG004C_DISPSETTING, reg);
}

//---------------------------------------------------------------------------
//	FUNCTION:		SetOverlayEnable()
//---------------------------------------------------------------------------
void SelectHostImage(WindowDef window)
{
	if(cl_MAIN_WINDOW==window)
		halWriteReg8(REG0048_LCD3SETTING, 0);
	else
		halWriteReg8(REG0048_LCD3SETTING, 0x10);
}

//---------------------------------------------------------------------------
//	FUNCTION:		SetMainPosition()
//---------------------------------------------------------------------------
void SetMainPosition(UInt32 xStart, UInt32 yStart, UInt32 xEnd, UInt32 yEnd)
{
	UInt8 width;
	UInt8 height;
	Boolean fresize;
	
	width = xEnd-xStart-1;
	height = yEnd-yStart-1;
	
	halWriteReg8(REG0212_MEMHORZRES,width);
	halWriteReg8(REG0214_MEMVERTRES,height);
	halWriteReg8(REG0228_XOFFSET0, (UInt8) xStart);
	halWriteReg8(REG0229_XOFFSET1, (UInt8) xStart>>8);
	halWriteReg8(REG022A_YOFFSET0, (UInt8) yStart);
	halWriteReg8(REG022B_YOFFSET1, (UInt8) yStart>>8);

	//It seem resize width and height must be set same as up setting.

	//if(fresize)
	//{
	//	;
	//}	
}