/*===========================================================================
** MultiFrame.c - Common routines to support MultiFrame JPEG File Format
**---------------------------------------------------------------------------
**	Copyright (c) 2002 Epson Research and Development, Inc.
**	All Rights Reserved.
**===========================================================================
*/

//---------------------------------------------------------------------------
// Include Files
//---------------------------------------------------------------------------
#include <string.h>
#include "multiframe.h"

//---------------------------------------------------------------------------
// Local Data
//---------------------------------------------------------------------------
static const char g_szSignature[] = {"DGJP"};

//---------------------------------------------------------------------------
//	mfInitializeMultiFrame - initializes multiframe structure
//
//	Input:	pJpegMultiFrame		pointer to multiframe object
//			size				exact (result) size of JFIF image in bytes
//
//	Output:	n/a
//
//	Notes:
//---------------------------------------------------------------------------
void mfInitializeMultiFrame( JPEGMULTIFRAME* pJpegMultiFrame, UInt32 size )
{
	memcpy( pJpegMultiFrame->Signature, g_szSignature, sizeof( pJpegMultiFrame->Signature ) );
	pJpegMultiFrame->ResultSize = size;
	pJpegMultiFrame->PaddedSize = (size + sizeof( UInt32) - 1) & 0xFFFFFFC;
}

//---------------------------------------------------------------------------
//	mfValidateMultiFrame - Validates multiframe signature
//
//	Input:	pJpegMultiFrame		pointer to multiframe object
//
//	Output:	TRUE means valid signature
//			FALSE means NOT valid signature
//
//	Notes:
//---------------------------------------------------------------------------
Boolean mfValidateMultiFrame( JPEGMULTIFRAME* pJpegMultiFrame )
{
	int		compare	= strncmp( (const char*)pJpegMultiFrame->Signature, g_szSignature, sizeof( pJpegMultiFrame->Signature ) );
	Boolean fSuccess = (compare == 0) ? TRUE : FALSE;
	return ( fSuccess );
}
