/**
 *	charger_mmi.h
 *		Header file of mmi charging management module interface with LCC, including
 *		all prototype of mmi charging interface.
 *
 *	History:
 *		2004-05-24	Robert.Chen		Init.
 *		
 */

#ifndef _CHARGER_MMI_H_
#define _CHARGER_MMI_H_

/* import this for compatible with MMI charging management method */
extern pwr_Status_Type    pwr_Status;

/* send power information to MMI */
extern void SendVirtualKeyToMMI(U8, U8);

/* register MMI power management callback function to LCC */
void mmi_pwr_register(void);

/* Setting power management information about charging and battery before MMI start */
void mmi_pwr_initStatus(void);

/* MMI power management callback invoked by LCC */
void mmi_pwr_cb_func(void*);

/* check if a FFS DIR exists, if not, make it */
UINT8 mmi_pwr_chkDir(const char * dir);

/* check if a FFS file exists, if not, write it according 'data' and 'size' */
UINT8 mmi_pwr_chkFile(const char * name, void * data, int size);

/* check if all power config DIR and files exist, if not, create them */
void mmi_pwr_chkCfg(void);

/* check charger status */
UINT8 ChargerPlugIn(void);

/* handle low voltage event */
void mmi_pwr_discharging(int Cbat, int Vbat);

#define EN_CHARGER_MMI_TR		  1		/* trace switch for the interface module */

#if (EN_CHARGER_MMI_TR == 1)
extern void MI_dbgPrintf(const char * const format,...);

#define	PWR_MMI_TR(msg)		MI_dbgPrintf msg
#else
#define PWR_MMI_TR(msg)
#endif


#endif	/* _CHARGER_MMI_H_ */
