#
#       Makefile for basic drivers (ARM compiler) 
#
#       Copyright (c) Texas Instruments 2002
# 
#               Author: Laurent Sollier
#       Date : 17 May 2002

#glowing,2004-07-19, add MODEL define
BOARD_FLAGS	:= -dMODEL=$(MBOARD)

DRIVERS_FLAGS   := -g -mn -x -mt -o -mw $(BOARD_FLAGS) $(CFLAGSDRIVERS)
CFLAGS32    := -g -x -o -mw $(CFLAGSDRIVERS)



# Libraries definition
DRIVER_FLASH   := $(OUT_LIB_DIR)/drivers$(NBRD)$(NCHP)$(NANALOG)$(NBLUETOOTH)$(NWCP)$(NGOLITE)$(MBRD)_flash.lib
DRIVER_INT_RAM := $(OUT_LIB_DIR)/drivers$(NBRD)$(NCHP)$(NANALOG)$(NBLUETOOTH)$(NWCP)_int_ram.lib
NIQ32_INT_RAM  := $(OUT_LIB_DIR)/niq32$(NBRD)$(NCHP)$(NANALOG)$(NBLUETOOTH)$(NWCP)$(NGOLITE)$(MBRD)_int_ram.lib
CFG_LIBS      += $(DRIVER_FLASH) $(DRIVER_INT_RAM) $(NIQ32_INT_RAM)

# Source definition
#glowing,2004-06-07, add the hall.c, mmedev.c
SRC_DRIVER_FLASH += $(BOARD_DIR)/buzzer.c      $(BOARD_DIR)/leadapi.c \
                    $(BOARD_DIR)/niq.c         $(BOARD_DIR)/uart.c \
                    $(BOARD_DIR)/inth.c        $(DRIVERS_DIR)/common/sim.c \
                    $(BOARD_DIR)/sim32.c       $(BOARD_DIR)/timer.c \
                    $(BOARD_DIR)/timer1.c      $(BOARD_DIR)/timer2.c \
                    $(BOARD_DIR)/drv_dummy.c   $(BOARD_DIR)/timer_sec.c \
                    $(DRIVERS_DIR)/common/power.c  $(BOARD_DIR)/certificate.c \
                    $(BOARD_DIR)/hall.c		$(BOARD_DIR)/mmedev.c \
#Robert.Chen add, 2004-05-27
SRC_DRIVER_FLASH += $(DRIVERS_DIR)/common/charger_mmi.c
                    

ifeq ($(WCP_PROF),1)
  SRC_DRIVER_FLASH += $(BOARD_DIR)/profiler.c
endif

ifeq ($(DP),1)
  SRC_DRIVER_FLASH += $(BOARD_DIR)/dp/uartfax.c $(BOARD_DIR)/dp/serialswitch.c
else
  SRC_DRIVER_FLASH += $(BOARD_DIR)/uartfax.c    $(BOARD_DIR)/serialswitch.c
endif

ifeq ($(findstring 1,$(TI_PROFILER) $(TI_NUC_MONITOR)),1)
   SRC_NIQ32_INT_RAM += $(BOARD_DIR)/niq32.c
else
   SRC_DRIVER_FLASH += $(BOARD_DIR)/niq32.c
endif

ifeq ($(BOARD),34)
  SRC_DRIVER_INT_RAM += $(BOARD_DIR)/clkm.c  $(BOARD_DIR)/sleep.c \
                        $(BOARD_DIR)/armio.c
else 
  ifeq ($(BOARD),35)
    SRC_DRIVER_INT_RAM += $(BOARD_DIR)/clkm.c  $(BOARD_DIR)/sleep.c \
                          $(BOARD_DIR)/armio.c
  else
    SRC_DRIVER_FLASH += $(BOARD_DIR)/clkm.c \
                        $(BOARD_DIR)/armio.c 
  endif
endif

# Objects definition
OBJ_DRIVER_FLASH := $(patsubst %.c,%.obj,$(filter %.c,$(SRC_DRIVER_FLASH)))
CFG_OBJS += $(OBJ_DRIVER_FLASH)
CFG_DEPS += $(OBJ_DRIVER_FLASH:.obj=.d)

OBJ_DRIVER_INT_RAM := $(patsubst %.c,%.obj,$(filter %.c,$(SRC_DRIVER_INT_RAM)))
CFG_OBJS += $(OBJ_DRIVER_INT_RAM)
CFG_DEPS += $(OBJ_DRIVER_INT_RAM:.obj=.d)

OBJ_NIQ32_INT_RAM:= $(patsubst %.c,%.obj,$(filter %.c,$(SRC_NIQ32_INT_RAM)))
CFG_OBJS += $(OBJ_NIQ_32_INT_RAM)
CFG_DEPS += $(OBJ_NIQ_32_INT_RAM:.obj=.d)

#Sources that needs to be compiled into 32 bit state code  
#They still must be added to the SRC_DRIVER_FLASH due to generation of libraries
SRC32   += $(BOARD_DIR)/sim32.c $(BOARD_DIR)/niq32.c 
OBJ32   := $(patsubst %.c,%.obj,$(filter %.c,$(SRC32)))


######## LIBRARIES #############
$(DRIVER_FLASH) : $(OBJ_DRIVER_FLASH) $(OBJ_RIVIERA_DRIVER)
	ar470 r $(DRIVER_FLASH) $(OBJ_DRIVER_FLASH) $(OBJ_RIVIERA_DRIVER)

$(DRIVER_INT_RAM) : $(OBJ_DRIVER_INT_RAM) $(OBJ_RIVIERA_DRIVER)
	ar470 r $(DRIVER_INT_RAM) $(OBJ_DRIVER_INT_RAM) $(OBJ_RIVIERA_DRIVER)

$(NIQ32_INT_RAM) : $(OBJ_NIQ32_INT_RAM)
	ar470 r $(NIQ32_INT_RAM) $(OBJ_NIQ32_INT_RAM)

######## OBJECTS #############
$(filter-out $(OBJ32), $(OBJ_DRIVER_FLASH)) : %.obj: %.c
	cl470 $(DRIVERS_FLAGS) -fr$(@D) $<

$(filter-out $(OBJ32), $(OBJ_DRIVER_INT_RAM)) : %.obj: %.c
	cl470 $(DRIVERS_FLAGS) -fr$(@D) $<

$(OBJ32) : %.obj: %.c
	cl470 $(CFLAGS32) -fr$(@D) $<

