
#
# Makefile for rebuilding the Nucleus libraries: One being in Flash and the
# other one being loaded in Internal RAM.
# 
#
.SUFFIXES: .obj $(SUFFIXES)

OBJF  = obj_flash
OBJIR = obj_int_ram

# Initialization and Abort on error files
OBJ_INIT = $(OBJF)/inc.obj $(OBJF)/dmi.obj $(OBJF)/evi.obj $(OBJF)/pmi.obj $(OBJF)/mbi.obj $(OBJF)/ioi.obj $(OBJF)/tmi.obj \
           $(OBJF)/qui.obj $(OBJF)/hii.obj $(OBJF)/eri.obj $(OBJF)/pii.obj $(OBJF)/smi.obj $(OBJF)/tci.obj $(OBJF)/erc.obj 
           

# Data used due to the init above
OBJ_DAT = $(OBJF)/mbd.obj $(OBJF)/iod.obj $(OBJF)/hid.obj $(OBJF)/pid.obj $(OBJF)/qud.obj $(OBJF)/erd.obj


# licence Informations files
OBJ_LIC =  $(OBJF)/lic.obj $(OBJF)/rld.obj $(OBJF)/lid.obj 


# not used files
OBJ_NU = $(OBJF)/ioc.obj $(OBJF)/ioce.obj $(OBJF)/iof.obj \
         $(OBJF)/quc.obj $(OBJF)/quce.obj $(OBJF)/quf.obj \
         $(OBJF)/qus.obj $(OBJF)/quse.obj                 \
         $(OBJF)/pic.obj $(OBJF)/pice.obj $(OBJF)/pif.obj \
         $(OBJF)/pis.obj $(OBJF)/pise.obj  \
         $(OBJF)/mbc.obj $(OBJF)/mbce.obj $(OBJF)/mbf.obj \
         $(OBJF)/mbs.obj $(OBJF)/mbse.obj  \
         $(OBJF)/sms.obj $(OBJF)/smse.obj  \
         $(OBJF)/tcs.obj $(OBJF)/tcse.obj  \
         $(OBJF)/evf.obj $(OBJF)/hic.obj   


# not important files
OBJ_NI = $(OBJF)/dmc.obj $(OBJF)/dmce.obj $(OBJF)/dmd.obj $(OBJF)/dmf.obj \
         $(OBJF)/evc.obj $(OBJF)/evce.obj $(OBJF)/evd.obj \
         $(OBJF)/pmf.obj $(OBJF)/smf.obj $(OBJF)/tcf.obj $(OBJF)/tmf.obj \
         $(OBJF)/rlc.obj  


# All files in Flash
OBJ_FLASH = $(OBJ_INIT) $(OBJ_DAT) $(OBJ_LIC) $(OBJ_NU) $(OBJ_NI)


# C files in Internal RAM
COBJS = $(OBJIR)/csc.obj $(OBJIR)/tmc.obj    \
        $(OBJIR)/pmc.obj $(OBJIR)/pmce.obj $(OBJIR)/pmd.obj \
        $(OBJIR)/smc.obj $(OBJIR)/smce.obj $(OBJIR)/smd.obj \
        $(OBJIR)/tcc.obj $(OBJIR)/tcce.obj $(OBJIR)/tcd.obj \
        $(OBJIR)/tms.obj $(OBJIR)/tmse.obj $(OBJIR)/tmd.obj

        
# Assembly files in Internal RAM
AOBJS = $(OBJIR)/tct.obj $(OBJIR)/tmt.obj


# All files in Internal RAM
OBJ_INTERNAL_RAM = $(COBJS) $(AOBJS)


all: all_dir lib/nucleus_flash.lib lib/nucleus_int_ram.lib

all_dir:
	-mkdir $(OBJF)
	-mkdir $(OBJIR)
	-mkdir lib

lib/nucleus_flash.lib: $(OBJ_FLASH) 
	ar470 r lib/nucleus_flash.lib $(OBJ_FLASH)

lib/nucleus_int_ram.lib: $(OBJ_INTERNAL_RAM) 
	ar470 r lib/nucleus_int_ram.lib $(OBJ_INTERNAL_RAM)

$(OBJIR)/tct.obj: tct.s
	asm470 -me tct.s $(OBJIR)/tct.obj

$(OBJIR)/tmt.obj: tmt.s
	asm470 -me tmt.s $(OBJIR)/tmt.obj

$(OBJF)/%.obj: %.c
	cl470 -me -mt -o -mw -fr$(OBJF) $<

$(OBJIR)/%.obj: %.c
	cl470 -me -mt -o -mw -fr$(OBJIR) $<
    
#
#   Clean
#
clean:
	-rm $(OBJ_INIT)        
	-rm $(OBJ_DAT)        
	-rm $(OBJ_LIC)        
	-rm $(OBJ_NU)        
	-rm $(OBJ_NI)        
	-rm $(AOBJS)        
	-rm $(COBJS)        
	-rm lib/nucleus_flash.lib        
	-rm lib/nucleus_int_ram.lib        
