/****************************************************************************
 *
 *		Copyright (C) 2002-2003	YAMAHA CORPORATION. All rights reserved.
 *
 *		Module		: mammfcnv.c
 *
 *		Description	: SMAF Converter
 *
 *		Version		: 2.0.1 	2003.09.02
 *
 ****************************************************************************/

#include "masnddrv.h"
#include "mamachdep.h"
#include "madevdrv.h"
#include "maresmgr.h"
#include "malib.h"
#include "mammfcnv.h"

#define	MMF_MAX_TRACK_NUM			8
#define	MMF_MAX_PHRASE_INFO			7
#define	MMF_MAX_EVENT_NUM			10
#define	MMF_MAX_NOTE_OFF_NUM		256
#define	MMF_MAX_STREAM_DATA_NUM2	62
#define	MMF_MAX_STREAM_DATA_NUM3	32
#define	MMF_MAX_VOICE_DATA_NUM2		16
#define	MMF_MAX_WAVE_DATA_NUM3		127
#define	MMF_CHANNEL_NUM				16

#define	MMF_FUNC_SUCCESS			MASMW_SUCCESS
#define	MMF_FUNC_ERROR				MASMW_ERROR

#define	MMF_ERR_ARGUMENT			MASMW_ERROR_ARGUMENT
#define	MMF_ERR_FILE				MASMW_ERROR_FILE
#define	MMF_ERR_CLASS				MASMW_ERROR_CONTENTS_CLASS
#define	MMF_ERR_TYPE				MASMW_ERROR_CONTENTS_TYPE
#define	MMF_ERR_SIZE				MASMW_ERROR_CHUNK_SIZE
#define	MMF_ERR_CHUNK				MASMW_ERROR_CHUNK
#define	MMF_ERR_NOTAG				MASMW_ERROR_UNMATCHED_TAG
#define	MMF_ERR_SLENGTH				MASMW_ERROR_SHORT_LENGTH
#define	MMF_ERR_LLENGTH				MASMW_ERROR_LONG_LENGTH
#define	MMF_ERR_NODATA				MASMW_ERROR_NO_INFORMATION

#define	MMF_SEQ_ID_NULL				-1

#define	MMF_STATUS_IDLE				0
#define	MMF_STATUS_SAT_PROFILE		1
#define	MMF_STATUS_LOADED			2

#define	MMF_CRC_NULL				0xFFFF0000
#define	MMF_POSITION_OF_CCLASS		16
#define	MMF_POSITION_OF_CTYPE		17
#define	MMF_STSP_OFFSET_NULL		0xFFFFFFFF
#define	MMF_STSP_TIME_NULL			0xFFFFFFFF
#define	MMF_CHUNK_HEADER_SIZE		8
#define	MMF_FILE_CRC_SIZE			2
#define	MMF_ATR_TRACK_NO			MMF_MAX_TRACK_NUM - 1
#define	MMF_MINIMUM_TRACKSIZE2		6
#define	MMF_MINIMUM_TRACKSIZE3		20
#define	MMF_PLAY_TIME_MIN			20
#define	MMF_PLAY_TIME_MAX			0x00200000
#define	MMF_MA2_VOICE_NOTFOUND		0
#define	MMF_MA2_VOICE_FOUND			1
#define	MMF_FM_VOICE_MODE_4OP		0
#define	MMF_FM_VOICE_MODE_2OP		1

#define	MMF_STREAM_ID_INI			0xFF
#define	MMF_STREAM_ID_REGIST		0x00
#define	MMF_MA2_VOICE_NULL			0xFF
#define	MMF_MA3_WAVE_NULL			0xFF

#define	MMF_LED_SYNC_STATUS			0x10
#define	MMF_VIB_SYNC_STATUS			0x20
#define	MMF_KEY_CONTROL_STATUS		0xC0

#define	MMF_AL_VOICE_FLAG			0x02
#define	MMF_FMWT_VOICE_FLAG			0x01
#define	MMF_FM_VOICE_PARAMETER		0x00
#define	MMF_WT_VOICE_PARAMETER		0x01
#define	MMF_AL_VOICE_PARAM_SIZE		27
#define	MMF_2OPFM_VOICE_PARAM_SIZE	17
#define	MMF_4OPFM_VOICE_PARAM_SIZE	31
#define	MMF_WT_VOICE_PARAM_SIZE		16
#define	MMF_MAX_VOICE_PARAM_SIZE	MMF_AL_VOICE_PARAM_SIZE + MMF_4OPFM_VOICE_PARAM_SIZE + 2
#define	MMF_WT_VOICE_WAVE_SIZE		8192

#define	MMF_WAVE_BIN_2S_COMP_8BIT	0x01
#define	MMF_WAVE_BIN_OFFSET_8BIT	0x11
#define	MMF_WAVE_ADPCM_4BIT			0x20
#define	MMF_STREAM_TYPE_8BIT_2SCOMP	3
#define	MMF_STREAM_TYPE_8BIT_BINARY	2
#define	MMF_STREAM_TYPE_4BIT_ADPCM	1
#define	MMF_STREAM_ERROR			0xFF
#define	MMF_WAVE_SF_MINIMUM			4000
#define	MMF_WAVE_SF_4BIT_MAX		24000
#define	MMF_WAVE_SF_8BIT_MAX		12000

#define	MMF_API_MASTERVOL_DEFAULT	100
#define	MMF_MA1_SEQ_MASTERVOLUME	76
#define	MMF_MA2_SEQ_MASTERVOLUME	127
#define	MMF_MA3_SEQ_MASTERVOLUME	45
#define	MMF_MA5_SEQ_MASTERVOLUME	100

#define	MMF_HUFFMAN_TREE_FAILURE	0
#define	MMF_HUFFMAN_TREE_SUCCESS	1

#define	MMF_LOADINFO_SUCCESS		0
#define	MMF_LOADINFO_FAILURE		1
#define	MMF_LOADINFO_ERROR			MMF_FUNC_ERROR

#define	MMF_EXCLUSIVE_CHECK_FAILURE	0

#define	MMF_PHRAZE_SIZE_A			8
#define	MMF_PHRAZE_SIZE_B			12

#define	MMF_TAG_STARTPOINT			0x7374
#define	MMF_TAG_STOPPOINT			0x7370
#define	MMF_TAG_PHRASE_A			0x5041
#define	MMF_TAG_PHRASE_B			0x5042
#define	MMF_TAG_PHRASE_E			0x5045
#define	MMF_TAG_PHRASE_I			0x5049
#define	MMF_TAG_PHRASE_K			0x504B
#define	MMF_TAG_PHRASE_R			0x5052
#define	MMF_TAG_PHRASE_S			0x5053

#define	MMF_SMAF_TYPE_NULL			0
#define	MMF_SMAF_TYPE_MA1			1
#define	MMF_SMAF_TYPE_MA2			2
#define	MMF_SMAF_TYPE_MA3			3
#define	MMF_SMAF_TYPE_MA5			5

#define	MMF_FM_MODE_2OP32			0
#define	MMF_FM_MODE_4OP16			1
#define	MMF_P_SOUNDSET_GMX			0
#define	MMF_P_SOUNDSET_GML1			2
#define	MMF_WT_VOLUME_MA3			0
#define	MMF_WT_VOLUME_MA5			4

#define	MMF_RESOUCE_MODE_MA3		2
#define	MMF_RESOUCE_MODE_MA12		3
#define	MMF_RESOUCE_MODE_MA5		4
#define	MMF_RESOUCE_MODE_MA5_64		5

#define	MMF_SEQUENCETYPE_DERAYED	0
#define	MMF_AL_CHANNEL_NULL			0xFF
#define	MMF_HV_CHANNEL_NULL			16


#define	MMF_BANK_NUMBER_DEF			0
#define	MMF_PROGRAM_NUMBER_DEF		0
#define	MMF_RESONANCE_DEF			0x40
#define	MMF_BRIGHTNESS_DEF			0x40

#define	MMF_SETVOLUME_3RD_PARAM		0x07

#define	MMF_LED_SEQ_SYNC_OFF		0
#define	MMF_LED_SEQ_SYNC_ON			1
#define	MMF_VIB_SEQ_SYNC_OFF		0
#define	MMF_VIB_SEQ_SYNC_ON			1
#define	MMF_KEYCONTROL_ON			2
#define	MMF_KEYCONTROL_OFF			1
#define	MMF_KEYCONTROL_DEFAULT		0

#define	MMF_EVNET_EOS				0xFF

#define	MMF_CHANNEL_MODE_POLY		0
#define	MMF_CHANNEL_MODE_MONO		1

#define	MMF_BANK_NUMBER_DEF			0
#define	MMF_PROGRAM_NUMBER_DEF		0
#define	MMF_RPN_DEF					0x7F
#define	MMF_MODULATION_DEF			0
#define	MMF_CHANNEL_VOLUME_DEF		100
#define	MMF_CHANNEL_PAN_DEF			0x40
#define	MMF_EXPRESSION_DEF			0x7F
#define	MMF_HOLD_DEF				0
#define	MMF_MONO_POLY_MODE_DEF		MMF_CHANNEL_MODE_POLY
#define	MMF_PITCH_BEND_DEF			0x40
#define	MMF_BEND_RANGE_DEF			2
#define	MMF_OCTAVE_SHIFT_DEF		4
#define	MMF_VELOCITY_DEF_MA3		90
#define	MMF_VELOCITY_DEF_MA5		100
#define	MMF_BLOCK_FNUM_DEF			0
#define	MMF_STREAM_PAIR_DEF			0xFF
#define	MMF_STREAM_PAN_DEF			0xFF
#define	MMF_STREAM_PAN_OFF			0x80

#define	SNDDRV_DEF_STREAM_PAN		255
#define	SNDDRV_DEF_RESONANCE		64
#define	SNDDRV_DEF_BRIGHTNESS		64
#define	SNDDRV_DEF_CHANNEL_VOLUME	100
#define	SNDDRV_DEF_CHANNEL_PANPOT	64
#define	SNDDRV_DEF_EXPRESSION		127
#define	SNDDRV_DEF_BENDRANGE		2
#define	SNDDRV_DEF_PITCHBEND		0x2000

#define	MMF_CONVERT_PHASE_PLAY		0x00000000
#define	MMF_CONVERT_PHASE_SEEK_G	0x00010000
#define MMF_CONVERT_PHASE_SEEK_C	0x00020000
#define MMF_CONVERT_PHASE_SEEK_END	0x00030000
#define MMF_CONVERT_PHASE_PLAY_END	0x00FF0000

#define	MMF_SEEK_EVENT_END			0xFFFF

/* OPDA infomation structure	*/
typedef struct _tagOption_Info{
	UINT8 *			pbCnti;					/* pointer to CNTI Body				*/
	UINT32			dCntiSize;				/* size of CNTI Body				*/
	UINT8 *			pbOpda;					/* pointer to OPDA Body				*/
	UINT32			dOpdaSize;				/* size of OPDA Body				*/
} OPTIONINFO, *POPTIONINFO;

/* Track information structure	*/
typedef struct _tagTrack_Info{
	UINT8 *			pbMtr;					/* pointer to MTR(ATR) Body			*/
	UINT32			dMtrSize;				/* size of MTR(ATR) Body			*/
	UINT8 *			pbMspi;					/* pointer to MspI(AspI) Body		*/
	UINT32			dMspiSize;				/* size of MspI(AspI) Body			*/
	UINT8 *			pbMtsu;					/* pointer to Mtsu Body				*/
	UINT32			dMtsuSize;				/* size of Mtsu Body				*/
	UINT8 *			pbMtsq;					/* pointer to Mtsq(Atsq) Body		*/
	UINT32			dMtsqSize;				/* size of Mtsq(Atsq) Body			*/
	UINT8 *			pbMtsp;					/* pointer to Mtsp Body				*/
	UINT32			dMtspSize;				/* size of Mtsp Body				*/
	UINT8 *			pbMthv;					/* pointer to Mthv Body				*/
	UINT32			dMthvSize;				/* size of Mthv Body				*/
	UINT32			dPlayTime;				/* play time (tick)					*/
	UINT32			dTimeBase;				/* time base (msec/tick)			*/
	UINT32			dStartPoint;			/* start point(offset)				*/
	UINT32			dStopPoint;				/* stop point(offset)				*/
	UINT32			dStartTick;				/* start point(tick)				*/
	UINT32			dStopTick;				/* stop point(tick)					*/
} TRACKINFO, *PTRACKINFO;

/* Phrase information structure	*/
typedef struct _tagPhrase_Info{
	UINT32			dStartPoint;			/* start point of phrase(offset)	*/
	UINT32			dStopPoint;				/* stop point of phrase(offset)		*/
	UINT32			dStartTick;				/* start point of phrase(tick)		*/
	UINT32			dStopTick;				/* stop point of phrase(tick)		*/
} PHRASEINFO, *PPHRASEINFO;

/* Huffman information structure	*/
typedef struct _tagHuffman_Info{
	UINT32			dMtsqSize;				/* size of Mtsq(Atsq) Body			*/
	UINT32			dSeqSize;				/* size of sequence data			*/
	UINT32			dReadSize;				/* read data size 					*/
	SINT16			swLeft[512];			/* Huffman Tree (Left)				*/
	SINT16			swRight[512];			/* Huffman Tree (Right)				*/
	UINT8 *			psBuffer;				/* pointer to reference area		*/
	SINT8			sbBitC;					/* counter of reference bit			*/
	UINT8			bByte;					/* value of reference byte			*/
	UINT8 *			psFBuf;					/* pointer to sequence data top		*/
	SINT8			sbFBit;					/* counter of sequence data top bit	*/
	UINT8			bFByte	;				/* value of sequence data top byte	*/
} HUFFMANINFO, *PHUFFMANINFO;

/* HV information structure		*/
typedef struct _tagHv_Info{
	UINT8*			pbVoice;				/* pointer to HVP0 chunk header		*/
	UINT32			dVoiceSize;				/* size of HV voice parameter		*/
	UINT8*			pbScript;				/* pointer to Mhsc body				*/
	UINT32			dScriptSize;			/* size of Mhsc body				*/
	UINT8			bHvChannel;				/* HV channel #						*/
} HVINFO, *PHVINFO;


/* Load information structure	*/
typedef struct _tagLoad_Info{
	UINT8 *			pbMmmd;					/* pointer to MMMD top				*/
	UINT32			dMmmdSize;				/* size of MMMD (whole)				*/
	UINT32			dCrc;					/* file CRC							*/
	UINT32			dSmafType;				/* SMAF Type						*/
	UINT32			dPlayTime;				/* play time (tick)					*/
	UINT32			dStartTime;				/* start time (start point tick)	*/
	UINT32			dTimeBase;				/* time base (msec/tick)			*/
	UINT8			(*pfnGetByte)(PHUFFMANINFO psHuf);
	OPTIONINFO		sOption_Info;
	TRACKINFO		sTrack_Info[MMF_MAX_TRACK_NUM];
	PHRASEINFO		sPhrase_Info[MMF_MAX_PHRASE_INFO];
	HUFFMANINFO		sHuffman_Info;
	HVINFO			sHV_Info;
} LOADINFO, *PLOADINFO;

/* Stream information structure(for MA-2)	*/
typedef struct _tagStream_Info2{
	UINT8			bStrmID;				/* key number of stream				*/
	UINT8 *			pbWave;					/* pointer to Awa body				*/
	UINT32			dWaveSize;				/* size of Awa body					*/
	UINT32			dFs;					/* sampling frequency				*/
} STREAMINFO2, *PSTREAMINFO2;

/* Stream information structure(for MA-3/5)	*/
typedef struct _tagStream_Info3{
	UINT8			fbNote;					/* stream data flag					*/
	UINT8			bPairID;				/* stream pair ID					*/
	UINT8			bPan;					/* stream pan						*/
} STREAMINFO3, *PSTREAMINFO3;

/* Stream information structure	*/
typedef struct _tagStream_Info{
	STREAMINFO2		sStream_Info2[MMF_MAX_STREAM_DATA_NUM2];
	STREAMINFO3		sStream_Info3[MMF_MAX_STREAM_DATA_NUM3];
} STREAMINFO, *PSTREAMINFO;

/* Voice information structure(for MA-1/2)	*/
typedef struct _tagVoice_Info2{
	UINT8			bBank;					/* bank number						*/
	UINT8			bProg;					/* program number					*/
} VOICEINFO2, *PVOICEINFO2;

/* Wave information structure(for MA-3/5)	*/
typedef struct _tagWave_Info3{
	UINT32			dAdrs;					/* wave address						*/
	UINT32			dSize;					/* wave data size					*/
} WAVEINFO3, *PWAVEINFO3;

/* Voice information structure	*/
typedef struct _tagVoice_Info{
	VOICEINFO2		sVoice_Info2[MMF_MAX_VOICE_DATA_NUM2];
	WAVEINFO3		sWave_Info3[MMF_MAX_WAVE_DATA_NUM3];
} VOICEINFO, *PVOICEINFO;

/* Channel information structure	*/
typedef struct _tagChannel_Info{
	UINT8			bBankM;					/* bank select MSB					*/
	UINT8			bBankL;					/* bank select LSB					*/
	UINT8			bBank;					/* bank number (sound driver)		*/
	UINT8			bProg;					/* program change					*/
	UINT8			bRpnM;					/* RPN MSB							*/
	UINT8			bRpnL;					/* RPN LSB							*/
	UINT8			bMod;					/* modulation						*/
	UINT8			bChVol;					/* channel volume					*/
	UINT8			bPan;					/* channel pan						*/
	UINT8			bExp;					/* expression						*/
	UINT8			bHold;					/* hold								*/
	UINT8			bMono;					/* channel mode mono/poly			*/
	UINT8			bPitch;					/* pitch bend (MSB)					*/
	UINT8			bSens1;					/* pitch bend lenge 1				*/
	UINT8			bSens2;					/* pitch bend lenge 2				*/
	UINT8			bOct;					/* octerve shift					*/
	UINT8			bVel;					/* note on velocity					*/
	UINT8			bBlockFnum1;			/* ma-2 pitch bend (0xB0)			*/
	UINT8			bBlockFnum2;			/* ma-2 pitch bend (0xC0)			*/
	UINT8			fbLed;					/* LED synchronous flag				*/
	UINT8			fbVib;					/* Motor synchronous flag			*/
} CHANNELINFO, *PCHANNELINFO;

/* Event information structure	*/
typedef struct _tagEvent_Block{
	UINT32			dEventTime;				/* event activation time			*/
	UINT32			dSeqID;					/* sequencer ID (sound driver)		*/
	UINT32			dCmd;					/* command ID (sound driver)		*/
	UINT32			dParam1;				/* parameter 1						*/
	UINT32			dParam2;				/* parameter 2						*/
	UINT32			dParam3;				/* parameter 3						*/
	UINT8 *			pbSeq;					/* pointer to next event data		*/
	UINT32			dIndex;					/* index of next event				*/
	UINT32			dTrackNo;				/* track no.						*/
	void  *			pvNext;					/* pointer to next event block		*/
} EVENTBLOCK, *PEVENTBLOCK;

/* Note OFF information structure	*/
typedef struct _tagNoteOff_Block{
	UINT32			dOffTime;				/* note off activation time			*/
	UINT32			dSeqID;					/* sequencer ID (sound driver)		*/
	UINT32			dCmd;					/* command ID (sound driver)		*/
	UINT32			dCh;					/* channel no.						*/
	UINT32			dKey;					/* key no.							*/
	void  *			pvNext;					/* pointer to next note off block	*/
} OFFBLOCK, *POFFBLOCK;

/* Playback information structure	*/
typedef struct _tagPlay_Info{
	PEVENTBLOCK		psNextEvent;			/* pointer to next event block		*/
	PEVENTBLOCK		psEmptyEvent;			/* pointer to empty event block		*/
	POFFBLOCK		psNextOff;				/* pointer to next note off block	*/
	POFFBLOCK		psEmptyOff;				/* pointer to empty note off block	*/
	UINT32			dSmafType;				/* SMAF Type						*/
	UINT32			dHWTimeBase;			/* tick to H/W time base			*/
	UINT32			dPlayTime;				/* play time (tick)					*/
	UINT32			dStartTime;				/* start time (start point tick)	*/
	UINT32			dSeekTime;				/* seek time (msec)					*/
	UINT32			dSeekError;				/* seek time error (msec)			*/
	UINT32			dRamAdrs;				/* ram address						*/
	UINT32			dRamSize;				/* ram size							*/
	UINT32			dSeekParam;				/* seek event flag					*/
	UINT32			dPastTime;				/* past time (tick)					*/
	UINT8			bMasterVol;				/* master volume (sequence)			*/
	UINT8			bEos;					/* EOS flag							*/
	UINT16			fwMono;					/* channel mode change flag			*/
	UINT8			bResonance;				/* resonance						*/
	UINT8			bBrightness;			/* brightness						*/
	UINT8			bStreamReserve;			/* stream reserve					*/
	UINT8			bAlChReserve;			/* AL channel number				*/
	UINT8			bHVChannel;				/* HV channel number				*/
	UINT8			bFmVoice;				/* FM Voice Mode 0:4op, 1:2op		*/
	UINT8			(*pfnGetByte)(void);	/* pointer to byte get function		*/
	SINT32			(*pfnGetEvent)(void);	/* pointer to event get function	*/
	STREAMINFO		sStream_Info;
	VOICEINFO		sVoice_Info;
	CHANNELINFO		sChannel_Info[MMF_CHANNEL_NUM + 1];
	EVENTBLOCK		sEvent_Block[MMF_MAX_EVENT_NUM];
	OFFBLOCK		sNoteOff_Block[MMF_MAX_NOTE_OFF_NUM];
} PLAYINFO, *PPLAYINFO;

/* SMAF information structure	*/
typedef struct _tagSmaf_Info{
	SINT32			sdMmfSeqID;				/* sequence id (sound driver)		*/
	UINT32			dStatus;				/* converter status					*/
	LOADINFO		sLoad_Info[2];
	PLAYINFO		sPlay_Info;
} SMAFINFO, *PSMAFINFO;

static			SMAFINFO	gsSmaf_Info;

static	const	UINT8		abBitMaskTable1[8] = {
	0x01,	0x02,	0x04,	0x08,	0x10,	0x20,	0x40,	0x80	};

static	const	UINT16		awBitMaskTable2[8] = {
	0x00FF,	0x01FE, 0x03FC,	0x07F8,	0x0FF0,	0x1FE0,	0x3FC0,	0x7F80	};

static	const	UINT8		abTableA[16] = {
    0, 1, 1, 1, 1, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 15 };

static	const	UINT8		abTableD[16] = {
    0, 1, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14 };

static	const	UINT8		abExpression2[16] = {
0x80,0x00,0x1f,0x27,0x2f,0x37,0x3f,0x47,0x4f,0x57,0x5f,0x67,0x6f,0x77,0x7f,0x80 };

static	const	UINT8		abModulation2[16] = {
0x80,0x00,0x08,0x10,0x18,0x20,0x28,0x30,0x38,0x40,0x48,0x50,0x60,0x70,0x7f,0x80 };



/*********************************************************************************
 *	get_4byte
 *
 *	Description:
 *			SMAF data load (error check and regist)
 *	Argument:
 *			pbBuf			pointer to top data
 *	Return:
 *			nothing
 ********************************************************************************/
static UINT32			get_4byte(
	UINT8*				pbBuf
)
{
	return (UINT32)(	(((UINT32)pbBuf[0]) << 24) +
						(((UINT32)pbBuf[1]) << 16) +
						(((UINT32)pbBuf[2]) <<  8) +
						 ((UINT32)pbBuf[3]));
}


/*********************************************************************************
 *	Check_Initial
 *
 *	Description:
 *			SMAF data load (error check and regist)
 *	Argument:
 *			psLoad			pointer to load information structure
 *	Return:
 *			nothing
 ********************************************************************************/
static void				Check_Initial(
	PLOADINFO			psLoad
)
{
	UINT8				i;

/* Initialize Load information structure	*/
	psLoad->pbMmmd		= NULL;
	psLoad->dMmmdSize	= 0;
	psLoad->dCrc		= MMF_CRC_NULL;
	psLoad->dSmafType	= MMF_SMAF_TYPE_NULL;
	psLoad->dPlayTime	= 0;
	psLoad->dStartTime	= 0;
	psLoad->dTimeBase	= 0;
	psLoad->pfnGetByte	= NULL;

	psLoad->sOption_Info.pbCnti		= NULL;
	psLoad->sOption_Info.dCntiSize	= 0;
	psLoad->sOption_Info.pbOpda		= NULL;
	psLoad->sOption_Info.dOpdaSize	= 0;

	for( i = 0; i < MMF_MAX_TRACK_NUM; i++) {
		psLoad->sTrack_Info[i].pbMtr		= NULL;
		psLoad->sTrack_Info[i].dMtrSize		= 0;
		psLoad->sTrack_Info[i].pbMspi		= NULL;
		psLoad->sTrack_Info[i].dMspiSize	= 0;
		psLoad->sTrack_Info[i].pbMtsu		= NULL;
		psLoad->sTrack_Info[i].dMtsuSize	= 0;
		psLoad->sTrack_Info[i].pbMtsq		= NULL;
		psLoad->sTrack_Info[i].dMtsqSize	= 0;
		psLoad->sTrack_Info[i].pbMtsp		= NULL;
		psLoad->sTrack_Info[i].dMtspSize	= 0;
		psLoad->sTrack_Info[i].pbMthv		= NULL;
		psLoad->sTrack_Info[i].dMthvSize	= 0;
		psLoad->sTrack_Info[i].dPlayTime	= 0;
		psLoad->sTrack_Info[i].dTimeBase	= 0;
		psLoad->sTrack_Info[i].dStartPoint	= MMF_STSP_OFFSET_NULL;
		psLoad->sTrack_Info[i].dStopPoint	= MMF_STSP_OFFSET_NULL;
		psLoad->sTrack_Info[i].dStartTick	= MMF_STSP_TIME_NULL;
		psLoad->sTrack_Info[i].dStopTick	= MMF_STSP_TIME_NULL;
	}

	for( i = 0; i < MMF_MAX_PHRASE_INFO; i++) {
		psLoad->sPhrase_Info[i].dStartPoint	= MMF_STSP_OFFSET_NULL;
		psLoad->sPhrase_Info[i].dStopPoint	= MMF_STSP_OFFSET_NULL;
		psLoad->sPhrase_Info[i].dStartTick	= MMF_STSP_TIME_NULL;
		psLoad->sPhrase_Info[i].dStopTick	= MMF_STSP_TIME_NULL;
	}

	psLoad->sHV_Info.pbVoice		= NULL;
	psLoad->sHV_Info.dVoiceSize		= 0;
	psLoad->sHV_Info.pbScript		= NULL;
	psLoad->sHV_Info.dScriptSize	= 0;
	psLoad->sHV_Info.bHvChannel		= MMF_HV_CHANNEL_NULL;
}


/*********************************************************************************
 *	get_timebase
 *
 *	Description:
 *			get time base (data value -> msec/tick)
 *	Argument:
 *			bData			time base (data value)
 *	Return:
 *			>=0				success(time base (msec/tick))
 *			< 0				error code
 ********************************************************************************/
static SINT32			get_timebase(
	UINT8				bData
)
{
	switch( bData ) {
		case 0x02:	return  4;						/*  4[msec/tick]			*/
		case 0x03:	return  5;						/*  5[msec/tick]			*/
		case 0x10:	return 10;						/* 10[msec/tick]			*/
		case 0x11:	return 20;						/* 20[msec/tick]			*/
		case 0x12:	return 40;						/* 40[msec/tick]			*/
		case 0x13:	return 50;						/* 50[msec/tick]			*/
		default:	return MMF_FUNC_ERROR;			/* Time Base Error			*/
	}
}


/*********************************************************************************
 *	MTR_Check
 *
 *	Description:
 *			score track chunk check (header information and chunk construction)
 *	Argument:
 *			psTrack			pointer to track information structure
 *			bSmafType		SMAF type
 *	Return:
 *			0				success
 *			< 0				error code
 ********************************************************************************/
static SINT32			MTR_Check(
	PTRACKINFO			psTrack,
	UINT8				bSmafType
)
{
	SINT32				sdResult, sdChunkSize;
	UINT32				dSize, dIndex;
	UINT8*				pbBuf;
	UINT32				dChunkID, dChunkNo;

/* Check Format Type				*/
	switch (bSmafType) {
	case MMF_SMAF_TYPE_MA1 :
	case MMF_SMAF_TYPE_MA2 :
		if (psTrack->pbMtr[0] != 0x00) {
			return MMF_ERR_CHUNK;
		}
		break;
	case MMF_SMAF_TYPE_MA3 :
		if ((psTrack->pbMtr[0] != 0x01) && (psTrack->pbMtr[0] != 0x02)) {
			return MMF_ERR_CHUNK;
		}
		break;
	case MMF_SMAF_TYPE_MA5 :
		if (psTrack->pbMtr[0] != 0x02) {
			return MMF_ERR_CHUNK;
		}
		break;
	}

/* Check Sequence Type		*/
	if (psTrack->pbMtr[1] != 0x00) {
		return MMF_ERR_CHUNK;
	}

/* Check Time Base		*/
	if (psTrack->pbMtr[2] != psTrack->pbMtr[3]) {
		return MMF_ERR_CHUNK;
	}
	sdResult = get_timebase(psTrack->pbMtr[2]);
	if (sdResult == MMF_FUNC_ERROR) {
		return MMF_ERR_CHUNK;
	}
	psTrack->dTimeBase = (UINT32)sdResult;

/* Check sub chunk disposition	*/
	if ((bSmafType == MMF_SMAF_TYPE_MA1) || (bSmafType == MMF_SMAF_TYPE_MA2))
		dIndex = MMF_MINIMUM_TRACKSIZE2;
	else
		dIndex = MMF_MINIMUM_TRACKSIZE3;
	pbBuf = psTrack->pbMtr;
	dSize = psTrack->dMtrSize;
	while (dSize > (dIndex + MMF_CHUNK_HEADER_SIZE)) {
		sdChunkSize = malib_NextChunk(&pbBuf[dIndex], (dSize-dIndex),
			MALIB_CHUNK_PHASE_MTRSUB, &dChunkID, &dChunkNo);
		if (sdChunkSize < 0) {
			return MMF_ERR_CHUNK;
		}
		dIndex += MMF_CHUNK_HEADER_SIZE;
		switch (dChunkID) {
		case MALIB_CHUNKCODE_MSPI :
			psTrack->pbMspi		= &(pbBuf[dIndex]);
			psTrack->dMspiSize	= sdChunkSize;
			break;
		case MALIB_CHUNKCODE_MTSU :
			psTrack->pbMtsu		= &(pbBuf[dIndex]);
			psTrack->dMtsuSize	= sdChunkSize;
			break;
		case MALIB_CHUNKCODE_MTSQ :
			psTrack->pbMtsq		= &(pbBuf[dIndex]);
			psTrack->dMtsqSize	= sdChunkSize;
			break;
		case MALIB_CHUNKCODE_MTSP :
			psTrack->pbMtsp		= &(pbBuf[dIndex]);
			psTrack->dMtspSize	= sdChunkSize;
			break;
		case MALIB_CHUNKCODE_MTHV :
			psTrack->pbMthv		= &(pbBuf[dIndex]);
			psTrack->dMthvSize	= sdChunkSize;
			break;
		default :
			break;
		}
		dIndex += sdChunkSize;
	}

	if ((psTrack->pbMtsq == NULL) ||
		(psTrack->dMtsqSize == 0)) {
		return MMF_ERR_SLENGTH;
	}
	return MMF_FUNC_SUCCESS;
}


/*********************************************************************************
 *	ATR_Check
 *
 *	Description:
 *			audio track chunk check (header information and chunk construction)
 *	Argument:
 *			psTrack			pointer to track information structure
 *	Return:
 *			0				success
 *			< 0				error code
 ********************************************************************************/
static SINT32			ATR_Check(
	PTRACKINFO			psTrack
)
{
	SINT32				sdResult, sdChunkSize;
	UINT32				dSize, dIndex;
	UINT8*				pbBuf;
	UINT32				dChunkID, dChunkNo;
	UINT8				fbWave;

/* Check Format Type				*/
	if (psTrack->pbMtr[0] != 0x00) {
		return MMF_ERR_CHUNK;
	}

/* Check Sequence Type		*/
	if (psTrack->pbMtr[1] != 0x00) {
		return MMF_ERR_CHUNK;
	}

/* Check Wave Type			*/
	if (((psTrack->pbMtr[2] != 0x10) && (psTrack->pbMtr[2] != 0x11)) ||
		((psTrack->pbMtr[3] & 0xF0) != 0x00)) {
		return MMF_ERR_CHUNK;
	}

/* Check Time Base		*/
	if (psTrack->pbMtr[4] != psTrack->pbMtr[5]) {
		return MMF_ERR_CHUNK;
	}
	sdResult = get_timebase(psTrack->pbMtr[4]);
	if (sdResult == MMF_FUNC_ERROR) {
		return MMF_ERR_CHUNK;
	}
	psTrack->dTimeBase = (UINT32)sdResult;

	pbBuf	= psTrack->pbMtr;
	dSize	= psTrack->dMtrSize;
	dIndex	= 6;
	fbWave	= MMF_MA2_VOICE_NULL;

/* Check sub chunk disposition	*/
	while (dSize > (dIndex + MMF_CHUNK_HEADER_SIZE)) {
		sdChunkSize = malib_NextChunk(&pbBuf[dIndex], (dSize-dIndex),
			MALIB_CHUNK_PHASE_ATRSUB, &dChunkID, &dChunkNo);
		if (sdChunkSize < 0) {
			return MMF_ERR_CHUNK;
		}
		dIndex += MMF_CHUNK_HEADER_SIZE;
		switch (dChunkID) {
		case MALIB_CHUNKCODE_ASPI :
			psTrack->pbMspi		= &(pbBuf[dIndex]);
			psTrack->dMspiSize	= sdChunkSize;
			break;
		case MALIB_CHUNKCODE_ATSQ :
			psTrack->pbMtsq		= &(pbBuf[dIndex]);
			psTrack->dMtsqSize	= sdChunkSize;
			break;
		case MALIB_CHUNKCODE_AWA :
			if ((0x01 <= dChunkNo) && (dChunkNo <= 0x3E))
				fbWave = MMF_MA2_VOICE_FOUND;
			break;
		default :
			break;
		}
		dIndex += sdChunkSize;
	}

	if ((psTrack->pbMtsq == NULL) ||
		(psTrack->dMtsqSize == 0) ||
		(fbWave == MMF_MA2_VOICE_NULL)) {
		return MMF_ERR_SLENGTH;
	}
	return MMF_FUNC_SUCCESS;
}


/*********************************************************************************
 *	MspI_Check
 *
 *	Description:
 *			seek & phrase info chunk check (get phrase information)
 *	Argument:
 *			psTrack			pointer to track information structure
 *			psPhrase		pointer to phrase information structure
 *	Return:
 *			0				success
 *			< 0				error code
 ********************************************************************************/
static void				MspI_Check(
	PTRACKINFO			psTrack,
	PPHRASEINFO			psPhrase
)
{
	UINT8*				pbBuf;
	UINT32				dSize, dIndex;
	UINT16				wTag;

	if (psTrack->pbMspi == NULL)
		return;

	pbBuf	= psTrack->pbMspi;
	dSize	= psTrack->dMspiSize;
	dIndex	= 0;

	while (dSize >= dIndex + MMF_PHRAZE_SIZE_A) {
		wTag = (UINT16)((((UINT16)pbBuf[dIndex]) << 8) + (UINT16)pbBuf[dIndex+1]);
		switch (wTag) {
		case MMF_TAG_STARTPOINT :				/* start point					*/
			psTrack->dStartPoint	= get_4byte(&(pbBuf[dIndex + 3]));
			dIndex += MMF_PHRAZE_SIZE_A;
			break;
		case MMF_TAG_STOPPOINT :				/* stop point					*/
			psTrack->dStopPoint		= get_4byte(&(pbBuf[dIndex + 3]));
			dIndex += MMF_PHRAZE_SIZE_A;
			break;
		case MMF_TAG_PHRASE_A :					/* A melody						*/
			if (dSize < dIndex + MMF_PHRAZE_SIZE_B)
				return ;
			psPhrase[0].dStartPoint	= get_4byte(&(pbBuf[dIndex + 3]));
			psPhrase[0].dStopPoint	= get_4byte(&(pbBuf[dIndex + 7]));
			dIndex += MMF_PHRAZE_SIZE_B;
			break;
		case MMF_TAG_PHRASE_B :					/* B melody						*/
			if (dSize < dIndex + MMF_PHRAZE_SIZE_B)
				return ;
			psPhrase[1].dStartPoint	= get_4byte(&(pbBuf[dIndex + 3]));
			psPhrase[1].dStopPoint	= get_4byte(&(pbBuf[dIndex + 7]));
			dIndex += MMF_PHRAZE_SIZE_B;
			break;
		case MMF_TAG_PHRASE_E :					/* Ending						*/
			if (dSize < dIndex + MMF_PHRAZE_SIZE_B)
				return ;
			psPhrase[2].dStartPoint	= get_4byte(&(pbBuf[dIndex + 3]));
			psPhrase[2].dStopPoint	= get_4byte(&(pbBuf[dIndex + 7]));
			dIndex += MMF_PHRAZE_SIZE_B;
			break;
		case MMF_TAG_PHRASE_I :					/* Intro						*/
			if (dSize < dIndex + MMF_PHRAZE_SIZE_B)
				return ;
			psPhrase[3].dStartPoint	= get_4byte(&(pbBuf[dIndex + 3]));
			psPhrase[3].dStopPoint	= get_4byte(&(pbBuf[dIndex + 7]));
			dIndex += MMF_PHRAZE_SIZE_B;
			break;
		case MMF_TAG_PHRASE_K :					/* Interlude					*/
			if (dSize < dIndex + MMF_PHRAZE_SIZE_B)
				return ;
			psPhrase[4].dStartPoint	= get_4byte(&(pbBuf[dIndex + 3]));
			psPhrase[4].dStopPoint	= get_4byte(&(pbBuf[dIndex + 7]));
			dIndex += MMF_PHRAZE_SIZE_B;
			break;
		case MMF_TAG_PHRASE_R :					/* Refrain						*/
			if (dSize < dIndex + MMF_PHRAZE_SIZE_B)
				return ;
			psPhrase[5].dStartPoint	= get_4byte(&(pbBuf[dIndex + 3]));
			psPhrase[5].dStopPoint	= get_4byte(&(pbBuf[dIndex + 7]));
			dIndex += MMF_PHRAZE_SIZE_B;
			break;
		case MMF_TAG_PHRASE_S :					/* Bridge						*/
			if (dSize < dIndex + MMF_PHRAZE_SIZE_B)
				return ;
			psPhrase[6].dStartPoint	= get_4byte(&(pbBuf[dIndex + 3]));
			psPhrase[6].dStopPoint	= get_4byte(&(pbBuf[dIndex + 7]));
			dIndex += MMF_PHRAZE_SIZE_B;
			break;
		default :
			return;
		}
	}
	return;
}


/*********************************************************************************
 *	ST_SP_Check
 *
 *	Description:
 *			track chunk check (offset of start point and stop point)
 *	Argument:
 *			psTrack			pointer to track information structure
 *	Return:
 *			0				success
 *			< 0				error code
 ********************************************************************************/
static SINT32			ST_SP_Check(
	PTRACKINFO			psTrack
)
{
	UINT32				dStart, dStop, dSize;

	dSize	= psTrack->dMtsqSize;

	if (psTrack->dStartPoint == MMF_STSP_OFFSET_NULL)
		dStart	= 0;
	else
		dStart	= psTrack->dStartPoint;

	if (psTrack->dStopPoint == MMF_STSP_OFFSET_NULL)
		dStop	= dSize;
	else
		dStop	= psTrack->dStopPoint;

	if ((dStart >= dStop) || (dStart > dSize) || (dStop > dSize))
		return MMF_ERR_CHUNK;

	return MMF_FUNC_SUCCESS;
}


/*********************************************************************************
 *	Mtsu_Check2
 *
 *	Description:
 *			track chunk check (existence voice parameter)
 *	Argument:
 *			psTrack			pointer to track information structure
 *			bSmafType		SMAF type
 *	Return:
 *			0				success
 *			< 0				error code
 ********************************************************************************/
static SINT32			Mtsu_Check2(
	PTRACKINFO			psTrack,
	UINT8				bSmafType
)
{
	UINT8*				pbBuf;
	UINT32				dSize, dIndex;

	if (psTrack->pbMtsu == NULL) {
		return MMF_MA2_VOICE_NOTFOUND;
	}

	pbBuf	= psTrack->pbMtsu;
	dSize	= psTrack->dMtsuSize;
	dIndex	= 0;

	while (dSize > dIndex + 20) {
		if ((pbBuf[dIndex] != 0xFF) || (pbBuf[dIndex + 1] != 0xF0)) {
			return MMF_MA2_VOICE_NOTFOUND;
		}
		if (pbBuf[dIndex + 3] == 0x43) {
			if ((bSmafType == MMF_SMAF_TYPE_MA1) && (pbBuf[dIndex + 4] == 0x02))
				return MMF_MA2_VOICE_FOUND;
			if ((bSmafType == MMF_SMAF_TYPE_MA2) && (pbBuf[dIndex + 4] == 0x03))
				return MMF_MA2_VOICE_FOUND;
		}
		dIndex += (pbBuf[dIndex + 2] + 3);
	}

	return MMF_MA2_VOICE_NOTFOUND;
}


/*********************************************************************************
 *	get_flex2L
 *
 *	Description:
 *			get flex data (duration, gate time)
 *	Argument:
 *			pbBuf			pointer to data top
 *			dSize			size of data (remain)
 *			pdRead			pointer to size of flex data
 *	Return:
 *			>=0				success(flex data value)
 *			< 0				error code
 ********************************************************************************/
static SINT32			get_flex2L(
	UINT8*				pbBuf,
	UINT32				dSize,
	UINT32*				pbRead
)
{
	SINT32				sdTemp;

	if ((dSize < 1) || ((dSize < 2) && (pbBuf[0] >= 0x80)))
		return MMF_FUNC_ERROR;
	if (dSize >= 4) {
		sdTemp = pbBuf[0] + pbBuf[1] + pbBuf[2] + pbBuf[3];
		if (sdTemp == 0)
			return MMF_FUNC_ERROR;
	}
	if (pbBuf[0] >= 0x80) {
		sdTemp = (SINT32)( (((SINT32)(pbBuf[0] & 0x7F)) << 7) +
							((SINT32)(pbBuf[1] & 0x7F)) + 128 );
		*pbRead = 2;
	}
	else {
		sdTemp = (SINT32)(pbBuf[0] & 0x7F);
		*pbRead = 1;
	}
	return sdTemp;
}


/*********************************************************************************
 *	SeqData_Check2
 *
 *	Description:
 *			track chunk check (sequence massage)
 *	Argument:
 *			psTrack			pointer to track information structure
 *			bSmafType		SMAF type
 *	Return:
 *			0				success
 *			< 0				error code
 ********************************************************************************/
static SINT32			SeqData_Check2(
	PTRACKINFO			psTrack,
	UINT8				bSmafType
)
{
	UINT8*				pbBuf;
	UINT32				dSize, dIndex;
	SINT32				sdTemp;
	UINT32				dPast, dGate, dFlexSize;

	if (psTrack->pbMtsq == NULL) {
		return MMF_ERR_SLENGTH;
	}

	dPast	= 0;
	dGate	= 0;
	pbBuf	= psTrack->pbMtsq;
	dSize	= psTrack->dMtsqSize;
	dIndex	= 0;

/* scanning to EOS or stop point					*/
	while ( dSize > dIndex ) {
		if (psTrack->dStartPoint== dIndex)		/* start point	*/
			psTrack->dStartTick	= dPast;
		if (psTrack->dStopPoint == dIndex) {	/* stop point	*/
			psTrack->dStopTick	= dPast;
			break;
		}

		if (dSize >= dIndex + 4) {
			sdTemp=pbBuf[dIndex]+pbBuf[dIndex+1]+pbBuf[dIndex+2]+pbBuf[dIndex+3];
			if (sdTemp == 0) {					/* EOS			*/
				if (bSmafType == MMF_SMAF_TYPE_MA1)
					psTrack->dStopTick	= dPast + dGate;
				else
					psTrack->dStopTick	= dPast;
				break;
			}
		}

		sdTemp = get_flex2L(&pbBuf[dIndex], (dSize - dIndex), &dFlexSize);
		if (sdTemp < 0) {
			return MMF_ERR_CHUNK;
		}
		dPast += sdTemp;
		if (dPast >= MMF_PLAY_TIME_MAX) {
			return MMF_ERR_LLENGTH;
		}

		if ((UINT32)sdTemp >= dGate)			/* calculate remain of GT	*/
			dGate = 0;
		else
			dGate -= sdTemp;
		dIndex += dFlexSize;

		if (dSize < dIndex + 2) {
			return MMF_ERR_CHUNK;
		}

		switch (pbBuf[dIndex]) {
		case 0x00 :
			if ((pbBuf[dIndex + 1] & 0x30) != 0x30)
				dIndex += 2;
			else
				dIndex += 3;
			break;
		case 0xFF :
			switch (pbBuf[dIndex + 1]) {
			case 0x00 :
				dIndex += 2;
				break;
			case 0xF0 :
				if (dSize < dIndex + 3) {
					return MMF_ERR_CHUNK;
				}
				dIndex += (pbBuf[dIndex + 2] + 3);
				if (dSize < dIndex) {
					return MMF_ERR_CHUNK;
				}
				if (pbBuf[dIndex - 1] != 0xF7) {
					return MMF_ERR_CHUNK;
				}
				break;
			default :
				return MMF_ERR_CHUNK;
			}
			break;
		default :
			sdTemp = get_flex2L(&pbBuf[dIndex+1], (dSize - dIndex- 1), &dFlexSize);
			if (sdTemp < 0) {
				return MMF_ERR_CHUNK;
			}
			if (dGate < (UINT32)sdTemp)
				dGate = (UINT32)sdTemp;
			dIndex += (1 + dFlexSize);
			break;
		}
		if (dSize < dIndex) {
			return MMF_ERR_CHUNK;
		}
	}


	if (psTrack->dStartTick  == MMF_STSP_TIME_NULL) {
		if (psTrack->dStartPoint != MMF_STSP_OFFSET_NULL) {
			return MMF_ERR_CHUNK;
		}
		psTrack->dStartTick = 0;
	}

/* check start/stop point potision	*/
	if (psTrack->dStopTick  == MMF_STSP_TIME_NULL) {
		if ((psTrack->dStopPoint != MMF_STSP_OFFSET_NULL) &&
			(psTrack->dStopPoint != dIndex)) {
			return MMF_ERR_CHUNK;
		}
		if (bSmafType == MMF_SMAF_TYPE_MA1)
			psTrack->dStopTick	= dPast + dGate;
		else
			psTrack->dStopTick	= dPast;
	}

/* calculate playback time of this track	*/
	psTrack->dPlayTime = psTrack->dStopTick - psTrack->dStartTick;
	return MMF_FUNC_SUCCESS;
}


/*********************************************************************************
 *	TrackChunk_Check2
 *
 *	Description:
 *			MA-1/2 track chunk check 
 *	Argument:
 *			psLoad			pointer to load information structure
 *	Return:
 *			0				success
 *			< 0				error code
 ********************************************************************************/
static SINT32			TrackChunk_Check2(
	PLOADINFO			psLoad
)
{
	PTRACKINFO			psTrack;
	SINT32				sdResult;
	UINT8				i, fbVoice;

/* delete track information of MA-3/5	*/
		psLoad->sTrack_Info[5].pbMtr	= NULL;
		psLoad->sTrack_Info[5].dMtrSize	= 0;
		psLoad->sTrack_Info[6].pbMtr	= NULL;
		psLoad->sTrack_Info[6].dMtrSize	= 0;

/* fix SMAF Type												*/
	psLoad->dSmafType = MMF_SMAF_TYPE_MA1;
	for (i = 1; i < 5; i++) {
		if (psLoad->sTrack_Info[i].pbMtr != NULL) {
			psLoad->dSmafType = MMF_SMAF_TYPE_MA2;
			break;
		}
	}
	if (psLoad->sTrack_Info[MMF_ATR_TRACK_NO].pbMtr != NULL)
		psLoad->dSmafType = MMF_SMAF_TYPE_MA2;

	if (psLoad->dSmafType == MMF_SMAF_TYPE_MA1) {	/* MA-1			*/
		if (psLoad->sTrack_Info[0].pbMtr == NULL) {
			return MMF_ERR_SLENGTH;
		}
		psTrack = &(psLoad->sTrack_Info[0]);
		if (psTrack->dMtrSize <= MMF_MINIMUM_TRACKSIZE2) {
			return MMF_ERR_CHUNK;
		}
		sdResult = MTR_Check(psTrack, MMF_SMAF_TYPE_MA1);
		if (sdResult != MMF_FUNC_SUCCESS) {
			return sdResult;
		}
		MspI_Check(psTrack, &(psLoad->sPhrase_Info[0]));
		sdResult = ST_SP_Check(psTrack);
		if (sdResult != MMF_FUNC_SUCCESS) {
			return sdResult;
		}
		sdResult = Mtsu_Check2(psTrack, MMF_SMAF_TYPE_MA1);
		if (sdResult != MMF_MA2_VOICE_FOUND) {
			return MMF_ERR_CHUNK;
		}
		sdResult = SeqData_Check2(psTrack, MMF_SMAF_TYPE_MA1);
		if (sdResult != MMF_FUNC_SUCCESS) {
			return sdResult;
		}
		psLoad->dPlayTime	= psTrack->dPlayTime;
		psLoad->dStartTime	= psTrack->dStartTick;
		psLoad->dTimeBase	= psTrack->dTimeBase;
		return MMF_FUNC_SUCCESS;
	}
	else {											/* MA-2			*/
		psLoad->sTrack_Info[0].pbMtr	= NULL;
		psLoad->sTrack_Info[0].dMtrSize	= 0;

		for (i = 1; i < 5; i++) {
			psTrack = &(psLoad->sTrack_Info[i]);
			if (psTrack->pbMtr == NULL)
				continue;
			if (psTrack->dMtrSize <= MMF_MINIMUM_TRACKSIZE2) {
				return MMF_ERR_CHUNK;
			}
			sdResult = MTR_Check(psTrack, MMF_SMAF_TYPE_MA2);
			if (sdResult != MMF_FUNC_SUCCESS) {
				return sdResult;
			}
			MspI_Check(psTrack, &(psLoad->sPhrase_Info[0]));
			sdResult = ST_SP_Check(psTrack);
			if (sdResult != MMF_FUNC_SUCCESS) {
				return sdResult;
			}
			sdResult = SeqData_Check2(psTrack, MMF_SMAF_TYPE_MA2);
			if (sdResult != MMF_FUNC_SUCCESS) {
				return sdResult;
			}
			psLoad->dPlayTime	= psTrack->dPlayTime;
			psLoad->dStartTime	= psTrack->dStartTick;
			psLoad->dTimeBase	= psTrack->dTimeBase;
		}
		if (psLoad->sTrack_Info[MMF_ATR_TRACK_NO].pbMtr != NULL) {
			psTrack = &(psLoad->sTrack_Info[MMF_ATR_TRACK_NO]);

			if (psTrack->dMtrSize <= MMF_MINIMUM_TRACKSIZE2) {
				return MMF_ERR_CHUNK;
			}
			sdResult = ATR_Check(psTrack);
			if (sdResult != MMF_FUNC_SUCCESS) {
				return sdResult;
			}
			MspI_Check(psTrack, &(psLoad->sPhrase_Info[0]));
			sdResult = ST_SP_Check(psTrack);
			if (sdResult != MMF_FUNC_SUCCESS) {
				return sdResult;
			}
			sdResult = SeqData_Check2(psTrack, MMF_SMAF_TYPE_MA2);
			if (sdResult != MMF_FUNC_SUCCESS) {
				return sdResult;
			}
			psLoad->dPlayTime	= psTrack->dPlayTime;
			psLoad->dStartTime	= psTrack->dStartTick;
			psLoad->dTimeBase	= psTrack->dTimeBase;
		}

/* totaling of track information	*/
		for (i = 1; i < MMF_MAX_TRACK_NUM; i++) {
			psTrack = &(psLoad->sTrack_Info[i]);
			if (psTrack->pbMtr == NULL)
				continue;

			if (psLoad->dPlayTime   < psTrack->dPlayTime)
				psLoad->dPlayTime   = psTrack->dPlayTime;
			if (psLoad->dTimeBase  != psTrack->dTimeBase) {
				return MMF_ERR_CHUNK;
			}
			if (psLoad->dStartTime != psTrack->dStartTick) {
				return MMF_ERR_CHUNK;
			}
			if (Mtsu_Check2(psTrack, MMF_SMAF_TYPE_MA2) == MMF_FUNC_SUCCESS)
				fbVoice = MMF_MA2_VOICE_FOUND;
		}

		fbVoice = MMF_MA2_VOICE_FOUND;
		for (i = 1; i < 5; i++) {
			psTrack = &(psLoad->sTrack_Info[i]);
			if (psTrack->pbMtr == NULL)
				continue;
			if (Mtsu_Check2(psTrack, MMF_SMAF_TYPE_MA2) == MMF_MA2_VOICE_FOUND) {
				fbVoice = MMF_MA2_VOICE_FOUND;
				break;
			}
			else
				fbVoice = MMF_MA2_VOICE_NOTFOUND;
		}

		if (fbVoice == MMF_MA2_VOICE_NOTFOUND) {
			return MMF_ERR_CHUNK;
		}
	}
	return MMF_FUNC_SUCCESS;
}


/*********************************************************************************
 *	Decode_GetbitL
 *
 *	Description:
 *			get 1 bit with error check
 *	Argument:
 *			psHuf			pointer to huffman information structure
 *	Return:
 *			read data
 ********************************************************************************/
static UINT8			Decode_GetbitL(
	PHUFFMANINFO		psHuf
)
{
	if (--psHuf->sbBitC < 0) {
		if (psHuf->dReadSize >= psHuf->dMtsqSize)
			return 0;
		psHuf->sbBitC	= 7;
		psHuf->bByte	= *(psHuf->psBuffer++);
		psHuf->dReadSize ++;
	}
	return (UINT8)(psHuf->bByte & abBitMaskTable1[psHuf->sbBitC]);
}


/*********************************************************************************
 *	Decode_Getbits
 *
 *	Description:
 *			get 8 bit
 *	Argument:
 *			psHuf			pointer to huffman information structure
 *	Return:
 *			read data
 ********************************************************************************/
static UINT8			Decode_Getbits(
	PHUFFMANINFO		psHuf
)
{
	UINT16				wTemp;
	UINT8				bData1, bData2;

	if (psHuf->dReadSize >= psHuf->dMtsqSize)
		return 0;
	bData1	= psHuf->bByte;
	bData2	= *(psHuf->psBuffer++);
	psHuf->bByte = bData2;
	psHuf->dReadSize ++;
	wTemp = (UINT16)((((UINT16)bData1) << 8) + ((UINT16)bData2));
	return (UINT8)((wTemp & awBitMaskTable2[psHuf->sbBitC]) >> psHuf->sbBitC);
}


/*********************************************************************************
 *	Decode_tree
 *
 *	Description:
 *			make MA-3 huffman decode tree
 *	Argument:
 *			psHuf			pointer to huffman information structure
 *	Return:
 *			!0				success(sequence data size)
 *			0				error
 ********************************************************************************/
static SINT32			Decode_tree(
	PHUFFMANINFO		psHuf
)
{
	UINT32				dNode, dEmpty, dIndex, i;
	SINT16				*pswLeft, *pswRight, *pswPNode;
	UINT8				bTemp;

    if (Decode_GetbitL(psHuf)) {
		if (psHuf->dReadSize >= psHuf->dMtsqSize)
			return MMF_HUFFMAN_TREE_FAILURE;

		pswLeft	= &(psHuf->swLeft[256]);
		pswRight= &(psHuf->swRight[256]);
		pswPNode= &(psHuf->swRight[0]);
		for (i = 0; i < 256; i++) {
			pswLeft[i]	= -1;
			pswRight[i]	= -1;
			pswPNode[i]	= 0;
		}
		dNode	= 2;
		dEmpty	= 1;
		dIndex	= 0;
	}
	else
		return MMF_HUFFMAN_TREE_FAILURE;

	while (dNode != 0) {
		if ((dEmpty >= 256) || (dNode >= 257))
			return MMF_HUFFMAN_TREE_FAILURE;

		bTemp = Decode_GetbitL(psHuf);
		if (psHuf->dReadSize >= psHuf->dMtsqSize)
			return MMF_HUFFMAN_TREE_FAILURE;

		if (bTemp) {
			dNode ++;
			if (pswLeft[dIndex] == -1)
				pswLeft[dIndex]	= (SINT16)(dEmpty + 256);
			else
				pswRight[dIndex]= (SINT16)(dEmpty + 256);
			pswPNode[dEmpty] = (SINT16)dIndex;
			dIndex = dEmpty;
			dEmpty ++;
		}
		else {
			dNode --;
			bTemp = Decode_Getbits(psHuf);
			if (psHuf->dReadSize >= psHuf->dMtsqSize)
				return MMF_HUFFMAN_TREE_FAILURE;

			if (pswLeft[dIndex] == -1)
				pswLeft[dIndex]	= (SINT16)bTemp;
			else {
				pswRight[dIndex]= (SINT16)bTemp;
				while ((pswRight[dIndex] != -1) && (dIndex != 0))
					dIndex = (UINT32)pswPNode[dIndex];
			}
		}
	}

	for (i = 0; i < 256; i++) {
		if (pswLeft[i]	== -1)
			pswLeft[i]	= 0;
		if (pswRight[i]	== -1)
			pswRight[i]	= 0;
	}
	return MMF_HUFFMAN_TREE_SUCCESS;
}


/*********************************************************************************
 *	Decode_init
 *
 *	Description:
 *			make MA-3 huffman decode tree
 *	Argument:
 *			psHuf			pointer to huffman information structure
 *	Return:
 *			!0				success(sequence data size)
 *			0				error
 ********************************************************************************/
static UINT32			Decode_init(
	PHUFFMANINFO		psHuf
)
{
	UINT32				dSeqSize;

	if (psHuf->dMtsqSize <= 5) {
		return MMF_HUFFMAN_TREE_FAILURE;
	}

	dSeqSize = get_4byte(psHuf->psBuffer);
	psHuf->psBuffer += 4;
	psHuf->dReadSize = 4;
	psHuf->sbBitC = 0;
	psHuf->bByte  = 0;
	if (Decode_tree(psHuf) == MMF_HUFFMAN_TREE_FAILURE)
		return MMF_HUFFMAN_TREE_FAILURE;
	else
		return dSeqSize;
}


/*********************************************************************************
 *	decode_byte3L
 *
 *	Description:
 *			get 1 byte (from compressed data)
 *	Argument:
 *			psHuf			pointer to huffman information structure
 *	Return:
 *			UINT8			success(read data)
 *			0				error code
 ********************************************************************************/
static UINT8			decode_byte3L(
	PHUFFMANINFO		psHuf
)
{
	UINT32				bData, bIndex;

	bIndex = 256;
	while (bIndex >= 256) {
		if (--psHuf->sbBitC < 0) {
			psHuf->dReadSize ++;
			if (psHuf->dReadSize > psHuf->dMtsqSize)
				return 0;
			psHuf->sbBitC	= 7;
			psHuf->bByte	= *(psHuf->psBuffer++);
		}
		bData = (UINT8)(psHuf->bByte & abBitMaskTable1[psHuf->sbBitC]);
		if (bData)
			bIndex = psHuf->swRight[bIndex];
		else
			bIndex = psHuf->swLeft[bIndex];
	}
	return (UINT8)bIndex;
}


/*********************************************************************************
 *	decode_byte3
 *
 *	Description:
 *			get 1 byte (from compressed data)
 *	Argument:
 *			nothing
 *	Return:
 *			UINT8			success(read data)
 ********************************************************************************/
static UINT8			decode_byte3( void )
{
	PHUFFMANINFO		psHuf;
	UINT32				bData, bIndex;

	psHuf = &(gsSmaf_Info.sLoad_Info[1].sHuffman_Info);
	psHuf->dReadSize ++;
	bIndex = 256;
	while (bIndex >= 256) {
		if (--psHuf->sbBitC < 0) {
			psHuf->sbBitC	= 7;
			psHuf->bByte	= *(psHuf->psBuffer++);
		}
		bData = (UINT8)(psHuf->bByte & abBitMaskTable1[psHuf->sbBitC]);
		if (bData)
			bIndex = psHuf->swRight[bIndex];
		else
			bIndex = psHuf->swLeft[bIndex];
	}
	return (UINT8)bIndex;
}


/*********************************************************************************
 *	get_byte3L
 *
 *	Description:
 *			get 1 byte (from normal data)
 *	Argument:
 *			psHuf			pointer to huffman information structure
 *	Return:
 *			UNIT8			success(read data)
 *			0				error code
 ********************************************************************************/
static UINT8			get_byte3L(
	PHUFFMANINFO		psHuf
)
{
	psHuf->dReadSize ++;
	if (psHuf->dReadSize > psHuf->dMtsqSize) {
		return 0;
	}
	return *(psHuf->psBuffer++);
}


/*********************************************************************************
 *	get_byte3
 *
 *	Description:
 *			get 1 byte (from normal data)
 *	Argument:
 *			nothing
 *	Return:
 *			UINT8			success(read data)
 ********************************************************************************/
static UINT8			get_byte3( void )
{
	PHUFFMANINFO		psHuf;

	psHuf = &(gsSmaf_Info.sLoad_Info[1].sHuffman_Info);
	psHuf->dReadSize ++;
	return *(psHuf->psBuffer++);
}


/*********************************************************************************
 *	get_flex3L
 *
 *	Description:
 *			MA-3 sequence data check
 *	Argument:
 *			psLoad			pointer to load information structure
 *			pdRead			pointer to size of flex data
 *	Return:
 *			>=0				success
 *			< 0				error code
 ********************************************************************************/
static SINT32			get_flex3L(
	PLOADINFO			psLoad,
	UINT32*				pdRead
)
{
	UINT32				dTemp, dRead;
	UINT8				bTemp;

	dRead = 1;
	bTemp = psLoad->pfnGetByte( &(psLoad->sHuffman_Info) );
	dTemp = (UINT32)(bTemp & 0x7F);
	while (bTemp & 0x80) {
		if (dRead >= 4)
			return MMF_FUNC_ERROR;
		dRead ++;
		bTemp = psLoad->pfnGetByte( &(psLoad->sHuffman_Info) );
		dTemp = (dTemp << 7) + (UINT32)(bTemp & 0x7F);
	}
	if (dTemp >= MMF_PLAY_TIME_MAX)
		return MMF_FUNC_ERROR;
	*pdRead = dRead;
	return (SINT32)dTemp;
}


/*********************************************************************************
 *	get_flex3
 *
 *	Description:
 *			MA-3 sequence data check
 *	Argument:
 *			nothing
 *	Return:
 *			UINT32			flex value
 ********************************************************************************/
static UINT32			get_flex3( void )
{
	UINT32				dTemp;
	UINT8				bTemp;

	bTemp = gsSmaf_Info.sPlay_Info.pfnGetByte();
	dTemp = (UINT32)(bTemp & 0x7F);
	while (bTemp & 0x80) {
		bTemp = gsSmaf_Info.sPlay_Info.pfnGetByte();
		dTemp = (dTemp << 7) + (UINT32)(bTemp & 0x7F);
	}
	return dTemp;
}


/*********************************************************************************
 *	get_flex5L
 *
 *	Description:
 *			MA-3 sequence data check
 *	Argument:
 *			psLoad			pointer to load information structure
 *			pdRead			pointer to size of flex data
 *	Return:
 *			>=0				success(flex data value)
 *			< 0				error
 ********************************************************************************/
static SINT32			get_flex5L(
	UINT8*				pbBuf,
	UINT32*				pdRead
)
{
	UINT32				dTemp, dRead;
	UINT8				bTemp;

	dRead = 1;
	bTemp = pbBuf[0];
	dTemp = (UINT32)(bTemp & 0x7F);
	while (bTemp & 0x80) {
		if (dRead >= 4)
			return MMF_FUNC_ERROR;
		dRead ++;
		bTemp = pbBuf[dRead - 1];
		dTemp = (dTemp << 7) + (UINT32)(bTemp & 0x7F);
	}
	if (dTemp >= MMF_PLAY_TIME_MAX)
		return MMF_FUNC_ERROR;
	*pdRead = dRead;
	return (SINT32)dTemp;
}


/*********************************************************************************
 *	SeqData_Check3
 *
 *	Description:
 *			MA-3 sequence data check
 *	Argument:
 *			psLoad			pointer to load information structure
 *			bSmafType		SMAF type
 *	Return:
 *			0				success
 *			< 0				error code
 ********************************************************************************/
static SINT32			SeqData_Check3(
	PLOADINFO			psLoad,
	UINT8				bSmafType
)
{
	PTRACKINFO			psTrk;
	PPHRASEINFO			psPhr;
	PHUFFMANINFO		psHuf;
	UINT32				dIndex, fdPhrase, dSize, dPast, dGate, dReadSize, i;
	UINT32				dStartTick, dStopTick;
	SINT32				sdTemp;
	UINT8				bTemp;

	if (bSmafType == MMF_SMAF_TYPE_MA3) {			/* MA-3		*/
		psTrk		= &(psLoad->sTrack_Info[5]);
		dStartTick	= MMF_STSP_TIME_NULL;
		dStopTick	= MMF_STSP_TIME_NULL;
	}
	else {											/* MA-5		*/
		psTrk		= &(psLoad->sTrack_Info[6]);
		dStartTick	= psTrk->dStartTick;
		dStopTick	= psTrk->dStopTick;
		psTrk->dStartTick	= MMF_STSP_TIME_NULL;
		psTrk->dStopTick	= MMF_STSP_TIME_NULL;
	}

	psPhr	= &(psLoad->sPhrase_Info[0]);
	psHuf	= &(psLoad->sHuffman_Info);
	fdPhrase= 0;
	dIndex	= 0;
	dPast	= 0;
	dGate	= 0;
	dSize	= psHuf->dSeqSize;

	if (psHuf->dSeqSize == 0) {
		return MMF_ERR_SLENGTH;
	}

	for (i = 0; i < MMF_MAX_PHRASE_INFO; i++) {
		if (psPhr[i].dStartPoint != MMF_STSP_OFFSET_NULL)
			fdPhrase = 1;
	}

/* scanning sequence data to EOS or stop point	*/
	while (dSize >= dIndex) {

		if (psTrk->dStartPoint	== dIndex)
			psTrk->dStartTick	=  dPast;
		if (psTrk->dStopPoint	== dIndex)			/* stop point		*/
			psTrk->dStopTick	=  dPast;
		if (fdPhrase) {
			for (i = 0; i < MMF_MAX_PHRASE_INFO; i++) {
				if (psPhr[i].dStartPoint== dIndex)
					psPhr[i].dStartTick	=  dPast;
				if (psPhr[i].dStopPoint	== dIndex)
					psPhr[i].dStopTick	=  dPast;
			}
		}

		if ((psTrk->dStopTick != MMF_STSP_TIME_NULL) || (dSize == dIndex))
			break;

		sdTemp = get_flex3L(psLoad, &dReadSize);	/* Duration			*/
		if (sdTemp < 0) {
			return MMF_ERR_CHUNK;
		}
		dPast	+= (UINT32)sdTemp;
		if (dPast >= MMF_PLAY_TIME_MAX) {
			return MMF_ERR_LLENGTH;
		}

		if ((UINT32)sdTemp >= dGate)
			dGate = 0;
		else
			dGate -= (UINT32)sdTemp;
		dIndex += dReadSize;

		bTemp = psLoad->pfnGetByte(psHuf);
		dIndex ++;

		switch (bTemp & 0xF0) {
		case 0x90 :
			psLoad->pfnGetByte(psHuf);
			dIndex ++;
		case 0x80 :
			psLoad->pfnGetByte(psHuf);
			dIndex ++;
			sdTemp = get_flex3L(psLoad, &dReadSize);
			if (sdTemp < 0) {
				return MMF_ERR_CHUNK;
			}
			dIndex += dReadSize;
			if ((UINT32)sdTemp > dGate)
				dGate = sdTemp;
			break;
		case 0xA0 :
		case 0xB0 :
		case 0xE0 :
			bTemp = psLoad->pfnGetByte(psHuf);
			dIndex ++;
		case 0xC0 :
		case 0xD0 :
			bTemp = psLoad->pfnGetByte(psHuf);
			dIndex ++;
			break;
		default :
			switch (bTemp) {
			case 0xF0 :
				sdTemp = get_flex3L(psLoad, &dReadSize);
				if (sdTemp < 0) {
					return MMF_ERR_CHUNK;
				}
				for (i = 0; i < (UINT32)sdTemp; i++)
					bTemp = psLoad->pfnGetByte(psHuf);
				if (bTemp != 0xF7) {
					return MMF_ERR_CHUNK;
				}
				dIndex += (UINT32)sdTemp + dReadSize;
				break;
			case 0xFF :
				bTemp = psLoad->pfnGetByte(psHuf);
				dIndex ++;
				switch (bTemp) {
				case 0x00 :
					break;
				case 0x2F :
					bTemp = psLoad->pfnGetByte(psHuf);
					dIndex ++;
					if (bTemp != 0x00) {
						return MMF_ERR_CHUNK;
					}
					dGate = 0;
					psTrk->dStopTick = dPast;
					dIndex = dSize;
					break;
				default :
					return MMF_ERR_CHUNK;
				}
				break;
			default :
				return MMF_ERR_CHUNK;
			}
			break;
		}
		if ((dSize < dIndex) || (psHuf->dReadSize > psHuf->dMtsqSize)) {
			return MMF_ERR_CHUNK;
		}
	}

	if (bSmafType == MMF_SMAF_TYPE_MA3) {			/* MA-3		*/
/* check start point				*/
		if (psTrk->dStartTick == MMF_STSP_TIME_NULL) {
			if (psTrk->dStartPoint != MMF_STSP_OFFSET_NULL) {
				return MMF_ERR_CHUNK;
			}
			psTrk->dStartPoint	= 0;
			psTrk->dStartTick	= 0;
		}
/* check stop point				*/
		if (psTrk->dStopTick == MMF_STSP_TIME_NULL) {
			if (psTrk->dStopPoint != MMF_STSP_OFFSET_NULL) {
				return MMF_ERR_CHUNK;
			}
			psTrk->dStopPoint	= dSize;
			psTrk->dStopTick	= dPast + dGate;
		}
/* adjust phrase information	*/
		for (i = 0; i < MMF_MAX_PHRASE_INFO; i++) {
			if (psPhr[i].dStartPoint <= psTrk->dStartPoint)
				psPhr[i].dStartTick   = psTrk->dStartTick;
			if (psPhr[i].dStopPoint  >= psTrk->dStopPoint)
				psPhr[i].dStopTick    = psTrk->dStopTick;
			if (psPhr[i].dStopPoint  <= psTrk->dStartPoint)
				psPhr[i].dStopTick    = MMF_STSP_TIME_NULL;
		}
	}
	else {											/* MA-5		*/
/* check stop point				*/
		if (dStopTick > dPast) {
			return MMF_ERR_CHUNK;
		}
		psTrk->dStartTick	= dStartTick;
		psTrk->dStopTick	= dStopTick;
	}

/* calculate playback time of this track	*/
	psTrk->dPlayTime = psTrk->dStopTick - psTrk->dStartTick;
	return MMF_FUNC_SUCCESS;
}


/*********************************************************************************
 *	TrackChunk_Check3
 *
 *	Description:
 *			MA-3 track chunk check 
 *	Argument:
 *			psLoad			pointer to load information structure
 *	Return:
 *			0				success
 *			< 0				error code
 ********************************************************************************/
static SINT32			TrackChunk_Check3(
	PLOADINFO			psLoad
)
{
	PTRACKINFO			psTrack;
	SINT32				sdResult;

	if (psLoad->sTrack_Info[5].pbMtr == NULL) {
		return MMF_ERR_SLENGTH;
	}
	psTrack = &(psLoad->sTrack_Info[5]);
	if (psTrack->dMtrSize <= MMF_MINIMUM_TRACKSIZE3) {
		return MMF_ERR_CHUNK;
	}
	sdResult = MTR_Check(psTrack, MMF_SMAF_TYPE_MA3);
	if (sdResult != MMF_FUNC_SUCCESS) {
		return sdResult;
	}
	MspI_Check(psTrack, &(psLoad->sPhrase_Info[0]));

	psLoad->sHuffman_Info.psBuffer	= psTrack->pbMtsq;
	psLoad->sHuffman_Info.dMtsqSize	= psTrack->dMtsqSize;

/* Initialize Huffman information structure	*/
	if (psTrack->pbMtr[0] == 0x01) {			/* Compressed Foramt	*/
		psLoad->sHuffman_Info.dSeqSize = Decode_init( &(psLoad->sHuffman_Info) );
		if (psLoad->sHuffman_Info.dSeqSize == MMF_HUFFMAN_TREE_FAILURE) {
			return MMF_ERR_CHUNK;
		}
		psLoad->pfnGetByte = decode_byte3L;
		psLoad->sHuffman_Info.psFBuf	= psLoad->sHuffman_Info.psBuffer;
		psLoad->sHuffman_Info.sbFBit	= psLoad->sHuffman_Info.sbBitC;
		psLoad->sHuffman_Info.bFByte	= psLoad->sHuffman_Info.bByte;
	}
	else {										/* No Compressed Foramt	*/
		psLoad->pfnGetByte = get_byte3L;
		psLoad->sHuffman_Info.dSeqSize	= psTrack->dMtsqSize;
		psLoad->sHuffman_Info.dReadSize	= 0;
		psLoad->sHuffman_Info.psFBuf	= psTrack->pbMtsq;
		psLoad->sHuffman_Info.sbFBit	= 0;
		psLoad->sHuffman_Info.bFByte	= 0;
	}

	psTrack->dMtsqSize = psLoad->sHuffman_Info.dSeqSize;
	sdResult = ST_SP_Check(psTrack);
	psTrack->dMtsqSize = psLoad->sHuffman_Info.dMtsqSize;

	if (sdResult != MMF_FUNC_SUCCESS) {
		return sdResult;
	}
	sdResult = SeqData_Check3(psLoad, MMF_SMAF_TYPE_MA3);
	if (sdResult != MMF_FUNC_SUCCESS) {
		return sdResult;
	}
	psLoad->dPlayTime	= psTrack->dPlayTime;
	psLoad->dStartTime	= psTrack->dStartTick;
	psLoad->dTimeBase	= psTrack->dTimeBase;
	return MMF_FUNC_SUCCESS;
}


/*********************************************************************************
 *	CheckM5P
 *
 *	Description:
 *			MA-5 profile data chunk check 
 *	Argument:
 *			psLoad			pointer to load information structure
 *	Return:
 *			0				success
 *			< 0				error code
 ********************************************************************************/
static SINT32			CheckM5P(
	PLOADINFO			psLoad
)
{
	PTRACKINFO			psTrk;
	PPHRASEINFO			psPhr;
	POPTIONINFO			psOptn;
	UINT8				*pbOpda, *pbM5p;
	UINT32				dOSize,  dMSize, dIndex, dPhraseFlag, i;
	UINT32				dChunkID, dChunkNo;
	SINT32				sdChunkSize;

	psOptn	= &(psLoad->sOption_Info);
	psTrk	= &(psLoad->sTrack_Info[6]);
	psPhr	= &(psLoad->sPhrase_Info[0]);
	pbOpda	= psOptn->pbOpda;
	dOSize	= psOptn->dOpdaSize;
	dIndex	= 0;
	pbM5p	= NULL;
	dMSize	= 0;

/* search Pro5 Chunk	*/
	while (dOSize > (dIndex + MMF_CHUNK_HEADER_SIZE)) {
		sdChunkSize = malib_NextChunk(&pbOpda[dIndex], (dOSize - dIndex),
			MALIB_CHUNK_PHASE_OPDASUB, &dChunkID, &dChunkNo);
		if (sdChunkSize < MASMW_SUCCESS)		return MASMW_ERROR;
		dIndex += MMF_CHUNK_HEADER_SIZE;
		if ((dChunkID == MALIB_CHUNKCODE_M5P) && (dChunkNo == 0x05)) {
			pbM5p	= &pbOpda[dIndex];
			dMSize	= (UINT32)sdChunkSize;
			break;
		}
		dIndex += sdChunkSize;
	}

	if ((pbM5p == NULL) || (dMSize < 12))		return MASMW_ERROR;

	dPhraseFlag			= get_4byte(&pbM5p[0]);
	psTrk->dStartTick	= get_4byte(&pbM5p[4]);		/* start point		*/
	psTrk->dStopTick	= get_4byte(&pbM5p[8]);		/* stop point		*/
	dIndex				= 12;

	if (psTrk->dStartTick >= psTrk->dStopTick)	return MASMW_ERROR;

	for (i = 0; i < MMF_MAX_PHRASE_INFO; i ++) {
		if (dMSize < (dIndex + 8))		break;
		if (dPhraseFlag & (0x80000000 >> i)) {
			psPhr[i].dStartTick	= get_4byte(&pbM5p[dIndex]);
			psPhr[i].dStopTick	= get_4byte(&pbM5p[dIndex + 4]);
			if (psPhr[i].dStartTick >= psPhr[i].dStopTick) {
				psPhr[i].dStartTick	= MMF_STSP_TIME_NULL;
				psPhr[i].dStopTick	= MMF_STSP_TIME_NULL;
			}
			if (psPhr[i].dStartTick < psTrk->dStartTick)
				psPhr[i].dStartTick = psTrk->dStartTick;
			if (psPhr[i].dStopTick  > psTrk->dStopTick)
				psPhr[i].dStopTick  = psTrk->dStopTick;
			dIndex += 8;
		}
	}
	return MASMW_SUCCESS;
}


/*********************************************************************************
 *	TrackChunk_Check5
 *
 *	Description:
 *			MA-5 track chunk check 
 *	Argument:
 *			psLoad			pointer to load information structure
 *			dMode			load mode
 *	Return:
 *			0				success
 *			< 0				error code
 ********************************************************************************/
static SINT32			TrackChunk_Check5(
	PLOADINFO			psLoad,
	UINT32				dMode
)
{
	PTRACKINFO			psTrack;
	SINT32				sdResult;

	if (psLoad->sTrack_Info[6].pbMtr == NULL) {
		return MMF_ERR_SLENGTH;
	}
	psTrack = &(psLoad->sTrack_Info[6]);
	if (psTrack->dMtrSize <= MMF_MINIMUM_TRACKSIZE3) {
		return MMF_ERR_CHUNK;
	}
	sdResult = MTR_Check(psTrack, MMF_SMAF_TYPE_MA5);
	if (sdResult != MMF_FUNC_SUCCESS)			return sdResult;

	psLoad->sHuffman_Info.psBuffer	= psTrack->pbMtsq;
	psLoad->sHuffman_Info.dMtsqSize	= psTrack->dMtsqSize;
	psLoad->sHuffman_Info.dSeqSize	= psTrack->dMtsqSize;
	psLoad->sHuffman_Info.dReadSize	= 0;
	psLoad->pfnGetByte				= get_byte3L;

	if (dMode == 0) {
		psTrack->dPlayTime = psTrack->dStopTick - psTrack->dStartTick;
	}
	else {
		sdResult = SeqData_Check3(psLoad, MMF_SMAF_TYPE_MA5);
		if (sdResult != MMF_FUNC_SUCCESS)		return sdResult;
	}
	psLoad->dPlayTime	= psTrack->dPlayTime;
	psLoad->dStartTime	= psTrack->dStartTick;
	psLoad->dTimeBase	= psTrack->dTimeBase;

	return MMF_FUNC_SUCCESS;
}


/*********************************************************************************
 *	Get_HvData
 *
 *	Description:
 *			HV Data Chunk Check
 *	Argument:
 *			psLoad			pointer to load information structure
 *			bCType			Contents Type
 *	Return:
 *			nothing
 ********************************************************************************/
static void				Get_HvData(
	PLOADINFO			psLoad,
	UINT8				bCType
)
{
	UINT8*				pbHvData;
	UINT32				dHvDataSize;

	UINT8*				pbVoice;
	UINT8*				pbScript;
	UINT8*				pbSetup;
	UINT32				dVoiceSize, dScriptSize, dSetupSize, dIndex;
	UINT8				bHvCh;
	UINT16				wTag, wSize;

	UINT32				dChunkID, dChunkNo;
	SINT32				sdChunkSize;

	if ((psLoad->dSmafType == MMF_SMAF_TYPE_MA1) ||
		(psLoad->dSmafType == MMF_SMAF_TYPE_MA2))
		return ;

	if ((bCType & 0x0F) == 0x08)
		return ;

	if (psLoad->dSmafType == MMF_SMAF_TYPE_MA3) {
		pbHvData	= psLoad->sTrack_Info[5].pbMthv;
		dHvDataSize	= psLoad->sTrack_Info[5].dMthvSize;
	}
	else {
		pbHvData	= psLoad->sTrack_Info[6].pbMthv;
		dHvDataSize	= psLoad->sTrack_Info[6].dMthvSize;
	}

	if ((pbHvData == NULL) || (dHvDataSize < MMF_CHUNK_HEADER_SIZE))
		return ;

	pbVoice		= NULL;
	pbScript	= NULL;
	pbSetup		= NULL;
	dVoiceSize	= 0;
	dScriptSize	= 0;
	dSetupSize	= 0;
	bHvCh		= MMF_HV_CHANNEL_NULL;
	dIndex		= 0;

	while (dHvDataSize > (dIndex + MMF_CHUNK_HEADER_SIZE)) {
		sdChunkSize	= malib_NextChunk(&(pbHvData[dIndex]), (dHvDataSize - dIndex),
			MALIB_CHUNK_PHASE_MTHVSUB, &dChunkID, &dChunkNo);
		if (sdChunkSize < 0)		return ;
		dIndex	+= MMF_CHUNK_HEADER_SIZE;
		switch (dChunkID) {
		case MALIB_CHUNKCODE_MHVS :
			pbSetup		= &(pbHvData[dIndex]);
			dSetupSize	= (UINT32)sdChunkSize;
			break;
		case MALIB_CHUNKCODE_HVP :
			if (dChunkNo != 0)		break;
			pbVoice		= &(pbHvData[dIndex - MMF_CHUNK_HEADER_SIZE]);
			dVoiceSize	= (UINT32)(sdChunkSize + MMF_CHUNK_HEADER_SIZE);
			break;
		case MALIB_CHUNKCODE_MHSC :
			pbScript	= &(pbHvData[dIndex]);
			dScriptSize	= (UINT32)sdChunkSize;
			break;
		}
		dIndex	+= sdChunkSize;
	}

	dIndex	= 0;
	while (dSetupSize >= dIndex + 4 ) {
		wTag	= (UINT16)((pbSetup[dIndex    ] << 8) + pbSetup[dIndex + 1]);
		wSize	= (UINT16)((pbSetup[dIndex + 2] << 8) + pbSetup[dIndex + 3]);
		dIndex += 4;
		if (dSetupSize < (dIndex + wSize))			return ;
		if ((wTag == 0x4348) && (wSize == 1))		bHvCh = pbSetup[dIndex];
		dIndex += wSize;
	}

	if ((pbScript == NULL) || (bHvCh >= MMF_HV_CHANNEL_NULL))
		return ;

	psLoad->sHV_Info.pbVoice		= pbVoice;
	psLoad->sHV_Info.dVoiceSize		= dVoiceSize;
	psLoad->sHV_Info.pbScript		= pbScript;
	psLoad->sHV_Info.dScriptSize	= dScriptSize;
	psLoad->sHV_Info.bHvChannel		= bHvCh;

	return ;
}


/*********************************************************************************
 *	Ma_Load
 *
 *	Description:
 *			SMAF data load (error check and regist)
 *	Argument:
 *			pbFile			pointer to SMAF data
 *			dFsize			size of SMAF data
 *			dMode			load mode
 *	Return:
 *			0 or 1			success(file id)
 *			< 0				error code
 ********************************************************************************/
static SINT32			Ma_Load(
	UINT8*				pbFile,
	UINT32				dFSize,
	UINT32				dMode
)
{
	PLOADINFO			psLoad_Info;
	UINT8				bNo;
	UINT32				dChunkID, dChunkNo;
	UINT8*				pbBuf;
	UINT32				dSize, dIndex;
	SINT32				sdChunkSize, sdResult;
	UINT32				dCalcCrc, dFileCrc;

	if ((dMode == 0) || (dMode == 1))
		bNo = 1;
	else
		bNo = 0;
	pbBuf = pbFile;
	dSize = dFSize;
	psLoad_Info = &(gsSmaf_Info.sLoad_Info[bNo]);
	Check_Initial( psLoad_Info );

/* check File Chunk(ID/Size)	*/
	sdChunkSize = malib_NextChunk(pbBuf, dSize, MALIB_CHUNK_PHASE_MMMD,
		&dChunkID, &dChunkNo);
	if ((sdChunkSize < 0) || (dChunkID != MALIB_CHUNKCODE_MMMD)) {
		return MMF_ERR_FILE;
	}
	dSize		= (UINT32)(sdChunkSize + MMF_CHUNK_HEADER_SIZE);
	dCalcCrc	= MMF_CRC_NULL;
	if (dMode != 0) {						/* Load mode-1/2/3	*/
		dCalcCrc = malib_MakeCRC(dSize, pbBuf);
		dFileCrc = (UINT32)((((UINT32)pbBuf[dSize - 2]) << 8) + pbBuf[dSize - 1]);
		if (dCalcCrc != dFileCrc) {
			return MMF_ERR_FILE;
		}
	}

/* check Contents Info Chunk				*/
	dIndex = MMF_CHUNK_HEADER_SIZE;
    sdChunkSize = malib_NextChunk(&pbBuf[dIndex], (dSize-dIndex),
		MALIB_CHUNK_PHASE_CNTI, &dChunkID, &dChunkNo);
	if ((sdChunkSize < 5) || (dChunkID != MALIB_CHUNKCODE_CNTI)) {
		return MMF_ERR_FILE;
	}

/* check Contents Class		*/
	if ((pbBuf[MMF_POSITION_OF_CCLASS] != MMF_CONTENTS_CLASS_0) &&
		(pbBuf[MMF_POSITION_OF_CCLASS] != MMF_CONTENTS_CLASS_1) ) {
		return MMF_ERR_CLASS;
	}

/* check Contents Type		*/
	dIndex += MMF_CHUNK_HEADER_SIZE;
	if		(((pbBuf[MMF_POSITION_OF_CTYPE] & 0xF0) == MMF_CONTENTS_TYPE_0) ||
			 ((pbBuf[MMF_POSITION_OF_CTYPE] & 0xF0) == MMF_CONTENTS_TYPE_1) ||
			 ((pbBuf[MMF_POSITION_OF_CTYPE] & 0xF0) == MMF_CONTENTS_TYPE_2) ){
		psLoad_Info->dSmafType = MMF_SMAF_TYPE_MA2;
	}
	else if	(((pbBuf[MMF_POSITION_OF_CTYPE] & 0xF0) == MMF_CONTENTS_TYPE_3) ||
			 ((pbBuf[MMF_POSITION_OF_CTYPE] & 0xF0) == MMF_CONTENTS_TYPE_4) ||
			 ((pbBuf[MMF_POSITION_OF_CTYPE] & 0xF0) == MMF_CONTENTS_TYPE_5) ){
		switch (pbBuf[MMF_POSITION_OF_CTYPE] & 0x0F) {
		case 0x00 :
		case 0x01 :
			psLoad_Info->dSmafType = MMF_SMAF_TYPE_MA2;
			break;
		case 0x02 :
		case 0x03 :
			psLoad_Info->dSmafType = MMF_SMAF_TYPE_MA3;
			break;
		case 0x04 :
		case 0x05 :
		case 0x06 :
		case 0x07 :
		case 0x08 :
			psLoad_Info->dSmafType = MMF_SMAF_TYPE_MA5;
			break;
		default :
			return MMF_ERR_TYPE;
		}
	}
	else {
		return MMF_ERR_TYPE;
	}

/* get pointer & size of option information	*/
	psLoad_Info->sOption_Info.pbCnti	= &pbBuf[dIndex];
	psLoad_Info->sOption_Info.dCntiSize	= (UINT32)(sdChunkSize);
	dIndex += sdChunkSize;

	if (pbBuf[MMF_POSITION_OF_CTYPE] >= 0x30) {
		sdChunkSize = malib_NextChunk(&pbBuf[dIndex], (dSize-dIndex),
			MALIB_CHUNK_PHASE_MMMDSUB, &dChunkID, &dChunkNo);
		if ((sdChunkSize >= 12) && (dChunkID == MALIB_CHUNKCODE_OPDA)) {
			dIndex += MMF_CHUNK_HEADER_SIZE;
			psLoad_Info->sOption_Info.pbOpda		= &pbBuf[dIndex];
			psLoad_Info->sOption_Info.dOpdaSize	= (UINT32)sdChunkSize;
			dIndex += sdChunkSize;
		}
	}

/* get Track Chunk information	*/
	while (dSize > (dIndex + MMF_CHUNK_HEADER_SIZE + MMF_FILE_CRC_SIZE)) {
		sdChunkSize = malib_NextChunk(&pbBuf[dIndex], (dSize-dIndex),
			MALIB_CHUNK_PHASE_MMMDSUB, &dChunkID, &dChunkNo);
		if (sdChunkSize < 0) {
			if (sdChunkSize == MALIB_CHUNK_ID_ERROR) {
				return MMF_ERR_FILE;
			}
			else {
				return MMF_ERR_SIZE;
			}
		}
		dIndex += MMF_CHUNK_HEADER_SIZE;
		switch (dChunkID) {
		case MALIB_CHUNKCODE_MTR :
			if (dChunkNo > 6)
				break;
			psLoad_Info->sTrack_Info[dChunkNo].pbMtr	= &(pbBuf[dIndex]);
			psLoad_Info->sTrack_Info[dChunkNo].dMtrSize	= (UINT32)sdChunkSize;
			break;
		case MALIB_CHUNKCODE_ATR :
			if (dChunkNo != 0)
				break;
			psLoad_Info->sTrack_Info[MMF_ATR_TRACK_NO].pbMtr	= &(pbBuf[dIndex]);
			psLoad_Info->sTrack_Info[MMF_ATR_TRACK_NO].dMtrSize	= (UINT32)sdChunkSize;
			break;
		default :
			break;
		}
		dIndex += sdChunkSize;
	}

	if (dMode == 3) {
		psLoad_Info->pbMmmd		= pbBuf;
		psLoad_Info->dMmmdSize	= dSize;
		return bNo;
	}

/* Error Check of Track Chunk	*/
	switch (psLoad_Info->dSmafType) {
	case MMF_SMAF_TYPE_MA2 :
		sdResult = TrackChunk_Check2(psLoad_Info);
		break;
	case MMF_SMAF_TYPE_MA3 :
		sdResult = TrackChunk_Check3(psLoad_Info);
		break;
	default :
		if (CheckM5P(psLoad_Info)!= MMF_FUNC_SUCCESS)	return MMF_ERR_CHUNK;
		sdResult = TrackChunk_Check5(psLoad_Info, dMode);
		break;
	}

/* check playback time			*/
	if (sdResult != MMF_FUNC_SUCCESS)		return sdResult;
	if ((psLoad_Info->dPlayTime * psLoad_Info->dTimeBase) <= MMF_PLAY_TIME_MIN) {
		return MMF_ERR_SLENGTH;
	}
	if ((psLoad_Info->dPlayTime * psLoad_Info->dTimeBase) >= MMF_PLAY_TIME_MAX) {
		return MMF_ERR_LLENGTH;
	}

	Get_HvData(psLoad_Info, pbBuf[MMF_POSITION_OF_CTYPE]);

	psLoad_Info->pbMmmd		= pbBuf;
	psLoad_Info->dMmmdSize	= dSize;
	psLoad_Info->dCrc		= (UINT32)dCalcCrc;
	return bNo;
}


/*********************************************************************************
 *	Renewal_Profile
 *
 *	Description:
 *			renew profile data
 *	Argument:
 *			pbFile			pointer to SMAF data
 *	Return:
 *			0 or 1			success(file id)
 *			< 0				error code
 ********************************************************************************/
static	SINT32			Renewal_Profile(
	UINT8*				pbFile
)
{
	PLOADINFO			psLoad;
	POPTIONINFO			psOptn;
	PTRACKINFO			psTrk;
	PHUFFMANINFO		psHuf;

	psLoad	= &(gsSmaf_Info.sLoad_Info[1]);
	psOptn	= &(psLoad->sOption_Info);
	psTrk	= &(psLoad->sTrack_Info[5]);
	psHuf	= &(psLoad->sHuffman_Info);

/* renew pointer  offset to pointer			*/
	psLoad->pbMmmd		= pbFile;
	psLoad->dCrc		= MMF_CRC_NULL;

	psOptn->pbCnti	= &(pbFile[(UINT32)psOptn->pbCnti]);
	psOptn->pbOpda	= &(pbFile[(UINT32)psOptn->pbOpda]);

	psTrk->pbMtr	= &(pbFile[(UINT32)psTrk->pbMtr]);
	psTrk->pbMspi	= &(pbFile[(UINT32)psTrk->pbMspi]);
	psTrk->pbMtsu	= &(pbFile[(UINT32)psTrk->pbMtsu]);
	psTrk->pbMtsq	= &(pbFile[(UINT32)psTrk->pbMtsq]);
	psTrk->pbMtsp	= &(pbFile[(UINT32)psTrk->pbMtsp]);

/* Initialize Huffman information structure	*/
	psHuf->psBuffer		= psTrk->pbMtsq;
	psHuf->dMtsqSize	= psTrk->dMtsqSize;

	if (psTrk->pbMtr[0] == 0x01) {
		psHuf->dSeqSize = Decode_init(psHuf);
		if (psHuf->dSeqSize == MMF_HUFFMAN_TREE_FAILURE) {
			return MMF_FUNC_ERROR;
		}
	}
	psHuf->psFBuf	= psHuf->psBuffer;
	psHuf->sbFBit	= psHuf->sbBitC;
	psHuf->bFByte	= psHuf->bByte;

	return MMF_FUNC_SUCCESS;
}


/*********************************************************************************
 *	get_LM_status2
 *
 *	Description:
 *			get MA-1/2 LED/Motor status
 *	Argument:
 *			nothing
 *	Return:
 *			nothing
 ********************************************************************************/
static void				get_LM_status2( void )
{
	MASMW_CONTENTSINFO	sLed;
	PCHANNELINFO		psChannel;
	SINT32				sdResult;
	UINT8				bBuf1, bBuf2, i;
	UINT16				fwLed, fwVib;

	sLed.code	= MALIB_CONTENTS_CODE_BINARY;
	sLed.tag[0]	= 'L';
	sLed.tag[1]	= '2';
	sLed.buf	= &bBuf1;
	sLed.size	= 1;

	fwLed	= 0;
	fwVib	= 0;
/* Get LED sync infomation(from L2)	*/
	sdResult = MaMmfCnv_Control(1, MASMW_GET_CONTENTSDATA, &sLed, NULL);
	if (sdResult == 1) {
		if ((bBuf1 & 0x0F) <= 0x0B)
			fwLed = (UINT16)(1 << (bBuf1 & 0x0F));
		else if ((bBuf1 & 0x0F) == 0x0C)
			fwLed = 0x000F;
		else if ((bBuf1 & 0x0F) == 0x0D)
			fwLed = 0x00F0;
		else if ((bBuf1 & 0x0F) == 0x0E)
			fwLed = 0x0F00;
		else
			gsSmaf_Info.sPlay_Info.sChannel_Info[16].fbLed = MMF_LED_SEQ_SYNC_ON;
	}

/* get VIB sync information(from channel status)	*/
	if (gsSmaf_Info.sLoad_Info[1].dSmafType == MMF_SMAF_TYPE_MA1) {
		bBuf1 = gsSmaf_Info.sLoad_Info[1].sTrack_Info[0].pbMtr[4];
		bBuf2 = gsSmaf_Info.sLoad_Info[1].sTrack_Info[0].pbMtr[5];
		if (bBuf1 & 0x40)	fwVib |= 0x01;
		if (bBuf1 & 0x04)	fwVib |= 0x02;
		if (bBuf2 & 0x40)	fwVib |= 0x04;
		if (bBuf2 & 0x04)	fwVib |= 0x08;
	}
	else {
		for (i = 0; i < 4; i++) {
			if (gsSmaf_Info.sLoad_Info[1].sTrack_Info[i+1].pbMtr == NULL)
				continue;
			bBuf1 = gsSmaf_Info.sLoad_Info[1].sTrack_Info[i+1].pbMtr[4];
			bBuf2 = gsSmaf_Info.sLoad_Info[1].sTrack_Info[i+1].pbMtr[5];
			if (bBuf1 & 0x40)	fwVib |= (0x01 << (i * 4));
			if (bBuf1 & 0x04)	fwVib |= (0x02 << (i * 4));
			if (bBuf2 & 0x40)	fwVib |= (0x04 << (i * 4));
			if (bBuf2 & 0x04)	fwVib |= (0x08 << (i * 4));
		}
	}

/* set LED/VIB sync information to Channel information structure	*/
	for (i = 0; i < 16; i++) {
		psChannel	= &(gsSmaf_Info.sPlay_Info.sChannel_Info[i]);
		if (fwLed & (0x0001 << i))
			psChannel->fbLed = MMF_LED_SEQ_SYNC_ON;
		else
			psChannel->fbLed = MMF_LED_SEQ_SYNC_OFF;
		if (fwVib & (0x0001 << i))
			psChannel->fbVib = MMF_VIB_SEQ_SYNC_ON;
		else
			psChannel->fbVib = MMF_VIB_SEQ_SYNC_OFF;
	}

	return ;
}


/*********************************************************************************
 *	get_LM_status3
 *
 *	Description:
 *			get MA-3/5 LED/Motor status
 *	Argument:
 *			nothing
 *	Return:
 *			nothing
 ********************************************************************************/
static void				get_LM_status3( void )
{
	PTRACKINFO			psTrk;
	PCHANNELINFO		psChannel;
	UINT8				bChStatus, i;

	if (gsSmaf_Info.sLoad_Info[1].dSmafType == MMF_SMAF_TYPE_MA3)
		psTrk	= &(gsSmaf_Info.sLoad_Info[1].sTrack_Info[5]);
	else
		psTrk	= &(gsSmaf_Info.sLoad_Info[1].sTrack_Info[6]);
	psChannel	= &(gsSmaf_Info.sPlay_Info.sChannel_Info[0]);

/* get & set LED/VIB sync information	*/
	for (i = 0; i < 16; i++) {
		bChStatus = psTrk->pbMtr[4 + i];
		if (bChStatus & MMF_LED_SYNC_STATUS)
			psChannel[i].fbLed	= MMF_LED_SEQ_SYNC_ON;
		else
			psChannel[i].fbLed	= MMF_LED_SEQ_SYNC_OFF;
		if (bChStatus & MMF_VIB_SYNC_STATUS)
			psChannel[i].fbVib	= MMF_VIB_SEQ_SYNC_ON;
		else
			psChannel[i].fbVib	= MMF_VIB_SEQ_SYNC_OFF;
	}

	return ;
}


/*********************************************************************************
 *	Exclusive_Check
 *
 *	Description:
 *			check exclusive message (in setup data chunk)
 *	Argument:
 *			pbBuf			pointer to exclusive message
 *			dSize			setup data chunk size (remain)
 *			bDevID			device ID
 *			pbID			pointer to write message ID
 *			pbOffset		pointer to write offset of data part
 *	Return:
 *			!0				success(exclusive massage size)
 *			0				error
 ********************************************************************************/
static UINT32			Exclusive_Check(
	UINT8*				pbBuf,
	UINT32				dSize,
	UINT8				bDevID,
	UINT8*				pbID,
	UINT8*				pbOffset
)
{
	UINT32				dFlexSize;
	SINT32				sdExSize;

	if ((dSize <= 5) || (pbBuf[0] != 0xF0))
		return MMF_EXCLUSIVE_CHECK_FAILURE;

	sdExSize = get_flex5L(&(pbBuf[1]), &dFlexSize);
	if (sdExSize < 0)
		return MMF_EXCLUSIVE_CHECK_FAILURE;

	sdExSize += dFlexSize + 1;
	if ((UINT32)sdExSize > dSize)
		return MMF_EXCLUSIVE_CHECK_FAILURE;

	if ((pbBuf[dFlexSize+1] != 0x43)   || (pbBuf[dFlexSize+2] != 0x79) ||
		(pbBuf[dFlexSize+3] != bDevID) || (pbBuf[dFlexSize+4] != 0x7F)) {
		*pbID		= 0;
		*pbOffset	= 0;
	}
	else {
		*pbID		= pbBuf[dFlexSize+5];
		*pbOffset	= (UINT8)(dFlexSize+6);
	}
	return (UINT32)sdExSize;
}


/*********************************************************************************
 *	Check_Stream
 *
 *	Description:
 *			check stream reserve
 *	Argument:
 *			psMtr			pointer to track information structure
 *			bDevID			device ID
 *	Return:
 *			0 - 2			success(number of stream reserve)
 ********************************************************************************/
static UINT32			Check_Stream(
	PTRACKINFO			psMtr,
	UINT8				bDevID
)
{
	UINT8				*pbBuf;
	UINT32				dSize, dIndex, dResult, dExSize;
	UINT8				bMessageID, bPointer;

	if (psMtr->pbMtsp == NULL)
		dResult = 0;
	else
		dResult = 2;

	if (psMtr->pbMtsu == NULL)
		return dResult;

	pbBuf	= psMtr->pbMtsu;
	dSize	= psMtr->dMtsuSize;
	dIndex	= 0;

	while (dSize >= dIndex + 9) {
		dExSize = Exclusive_Check(&(pbBuf[dIndex]), (UINT32)(dSize - dIndex),
									bDevID, &bMessageID, &bPointer);
		if (dExSize == MMF_EXCLUSIVE_CHECK_FAILURE)
			return dResult;
		if ((bPointer != 0) && (bMessageID == 0x07) && (dExSize == 0x09)) {
			if (pbBuf[dIndex + bPointer] <= 2)
				return pbBuf[dIndex + bPointer];
		}
		dIndex += dExSize;
	}
	return dResult;
}


/*********************************************************************************
 *	Check_AlChannel
 *
 *	Description:
 *			check stream reserve
 *	Argument:
 *			psMtr			pointer to track information structure
 *			bDevID			device ID
 *	Return:
 *			0 - 15			success(AL channel #)
 *			127				error code
 ********************************************************************************/
static UINT32			Check_AlChannel(
	PTRACKINFO			psMtr,
	UINT8				bDevID
)
{
	UINT8				*pbBuf;
	UINT32				dSize, dIndex, dResult, dExSize;
	UINT8				bMessageID, bPointer;

	dResult = MMF_AL_CHANNEL_NULL;
	if (psMtr->pbMtsu == NULL)
		return dResult;

	pbBuf	= psMtr->pbMtsu;
	dSize	= psMtr->dMtsuSize;
	dIndex	= 0;

	while (dSize >= dIndex + 9) {
		dExSize = Exclusive_Check(&(pbBuf[dIndex]), (UINT32)(dSize - dIndex),
									bDevID, &bMessageID, &bPointer);
		if (dExSize == MMF_EXCLUSIVE_CHECK_FAILURE)
			return dResult;
		if ((bPointer != 0) && (bMessageID == 0x0D) && (dExSize == 0x09)) {
			if (pbBuf[dIndex + bPointer] <= 15)
				return pbBuf[dIndex + bPointer];
		}
		dIndex += dExSize;
	}
	return dResult;
}


/*********************************************************************************
 *	Awa_Check
 *
 *	Description:
 *			MA-2 stream information
 *	Argument:
 *			nothing
 *	Return:
 *			nothing
 ********************************************************************************/
static void				Awa_Check( void )
{
	PTRACKINFO			psAtr;
	PSTREAMINFO2		psStrm2;
	SINT32				sdChunkSize;
	UINT32				dSize, dIndex, dFs;
	UINT8*				pbBuf;
	UINT32				dChunkID, dChunkNo;

	psAtr	= &(gsSmaf_Info.sLoad_Info[1].sTrack_Info[MMF_ATR_TRACK_NO]);
	psStrm2	= &(gsSmaf_Info.sPlay_Info.sStream_Info.sStream_Info2[0]);
	if (psAtr->pbMtr == NULL)
		return ;

/* get FS	*/
	if (psAtr->pbMtr[2] == 0x10)
		dFs = 4000;
	else
		dFs = 8000;

	pbBuf	= psAtr->pbMtr;
	dSize	= psAtr->dMtrSize;
	dIndex	= 6;

/* get stream wave		*/
	while (dSize > (dIndex + MMF_CHUNK_HEADER_SIZE)) {
		sdChunkSize = malib_NextChunk(&pbBuf[dIndex], (dSize-dIndex),
			MALIB_CHUNK_PHASE_ATRSUB, &dChunkID, &dChunkNo);
		dIndex += MMF_CHUNK_HEADER_SIZE;
		switch (dChunkID) {
		case MALIB_CHUNKCODE_AWA :
			if ((0x01 > dChunkNo) || (dChunkNo > 0x3E))
				break;
			dChunkNo --;
			if (psStrm2[dChunkNo].pbWave != NULL)
				break;
			psStrm2[dChunkNo].pbWave	= &pbBuf[dIndex];
			psStrm2[dChunkNo].dWaveSize	= (UINT32)sdChunkSize;
			psStrm2[dChunkNo].dFs		= dFs;
			break;
		default :
			break;
		}
		dIndex += sdChunkSize;
	}
	return ;
}


/*********************************************************************************
 *	SetStream2
 *
 *	Description:
 *			MA-2 stream registration
 *	Argument:
 *			nothing
 *	Return:
 *			nothing
 ********************************************************************************/
static void				SetStream2( void )
{
	PSTREAMINFO2		psStrm2;
	UINT8				i, bStreamID;
	SINT32				sdResult;

	psStrm2	= &(gsSmaf_Info.sPlay_Info.sStream_Info.sStream_Info2[0]);
	bStreamID = 0;
	for (i = 0; i < MMF_MAX_STREAM_DATA_NUM2; i++) {
		if (psStrm2[i].pbWave == NULL)
			continue;
		sdResult = MaSndDrv_SetStream(gsSmaf_Info.sdMmfSeqID, bStreamID, 0,
						psStrm2[i].dFs, psStrm2[i].pbWave, psStrm2[i].dWaveSize);
		if (sdResult < MMF_FUNC_SUCCESS) {
			psStrm2[i].pbWave	= NULL;
			psStrm2[i].dWaveSize= 0;
		}
		else {
			psStrm2[i].bStrmID	= bStreamID;
			bStreamID ++;
			if (bStreamID >= 32)				break;
		}
	}

	for (i = (UINT8)(i + 1); i < MMF_MAX_STREAM_DATA_NUM2; i++) {
		psStrm2[i].pbWave		= NULL;
		psStrm2[i].dWaveSize	= 0;
	}
	return ;
}


/*********************************************************************************
 *	Set_Param_MA1
 *
 *	Description:
 *			MA-1 voice parameter registration
 *	Argument:
 *			bCount			voice #
 *			pbBuf			pointer to voice parameter
 *			bExSize			size of exclusive message
 *	Return:
 *			1				error (fail voice registration)
 *			0				success
 *			!0				error (memory error)
 ********************************************************************************/
static SINT32			Set_Param_MA1(
	UINT8				bCount,
	UINT8*				pbBuf,
	UINT8				bExSize
)
{
	PVOICEINFO2			psVoice;
	UINT8				i, j;
	static UINT8		bBuf[16];
	UINT8				bBO, bML, bVIB, bEGT, bSUS, bRR, bDR, bAR, bSL, bTL;
	UINT8				bWAV, bFL, bSR;
	SINT32				sdResult;

	if ((pbBuf[0] & 0xF8) || (bExSize != 19))
		return MMF_FUNC_SUCCESS + 1;

	bBO = (UINT8)(pbBuf[3] & 0x03);
	bBuf[0] = (UINT8)(bBO | 0x80);
	bBuf[1] = 0x80;

	i = j = 0;
	while ( i < 5) {
		bML  = (UINT8)((pbBuf[4+i] >> 5) & 0x07);
		bVIB = (UINT8)((pbBuf[4+i] >> 4) & 0x01);
		bEGT = (UINT8)((pbBuf[4+i] >> 3) & 0x01);
		bSUS = (UINT8)((pbBuf[4+i] >> 2) & 0x01);
		bRR  = (UINT8)( ((pbBuf[4+i] & 0x03) << 2) | ((pbBuf[5+i] >> 6) & 0x03) );
		bDR  = (UINT8)((pbBuf[5+i] >> 2) & 0x0F);
		bAR  = (UINT8)( ((pbBuf[5+i] & 0x03) << 2) | ((pbBuf[6+i] >> 6) & 0x03) );
		bSL  = (UINT8)((pbBuf[6+i] >> 2) & 0x0F);
		bTL  = (UINT8)( ((pbBuf[6+i] & 0x03) << 4) | ((pbBuf[7+i] >> 4) & 0x0F) );
		bWAV = (UINT8)((pbBuf[7+i] >> 3) & 0x01);
		bFL  = (UINT8)(pbBuf[7+i] & 0x07);

		if (bEGT == 0) {
			bSR = abTableD[bRR];
			if (bSUS == 0)
				bRR = 8;
			else
				bRR = abTableD[bRR];
		}
		else {
			bSR = 0;
			if (bSUS == 0)
				bRR = abTableD[bRR];
			else
				bRR = 5;
		}
		if (i != 0)
			bFL = 0;
		bDR = abTableD[bDR];
		bAR = abTableA[bAR];

		bBuf[2+j] = (UINT8)( (bSR  << 4) & 0xF0);
		bBuf[3+j] = (UINT8)( ((bRR << 4) & 0xF0) + (bDR & 0x0F) );
		bBuf[4+j] = (UINT8)( ((bAR << 4) & 0xF0) + (bSL & 0x0F) );
		bBuf[5+j] = (UINT8)( (bTL  << 2) & 0xFC);
		bBuf[6+j] = (UINT8)( (bVIB & 0x01) + 0x04);
		bBuf[7+j] = (UINT8)( (bML  << 4) & 0xF0);
		bBuf[8+j] = (UINT8)( ((bWAV << 3) & 0xF8) + (bFL & 0x07) );

		i += 4;
		j += 7;
	}

	psVoice	= &(gsSmaf_Info.sPlay_Info.sVoice_Info.sVoice_Info2[bCount]);
	psVoice->bBank = pbBuf[1];
	psVoice->bProg = (UINT8)(pbBuf[2] & 0x7F);

	if (gsSmaf_Info.sPlay_Info.dRamSize < 16)
		return MMF_FUNC_SUCCESS + 1;

	sdResult = MaDevDrv_SendDirectRamData(gsSmaf_Info.sPlay_Info.dRamAdrs,
																	0, bBuf, 16);
	if (sdResult < MMF_FUNC_SUCCESS)		return sdResult;

	sdResult = MaSndDrv_SetVoice(gsSmaf_Info.sdMmfSeqID, 1, (UINT32)bCount,
											1, 0, gsSmaf_Info.sPlay_Info.dRamAdrs);
	if (sdResult < MMF_FUNC_SUCCESS)		return MMF_FUNC_SUCCESS + 1;

	gsSmaf_Info.sPlay_Info.dRamAdrs += 16;
	gsSmaf_Info.sPlay_Info.dRamSize -= 16;

	return MMF_FUNC_SUCCESS;
}


/*********************************************************************************
 *	Set_Param_MA2
 *
 *	Description:
 *			MA-2 voice parameter registration
 *	Argument:
 *			bCount			voice #
 *			pbBuf			pointer to voice parameter
 *			bExSize			size of exclusive message
 *	Return:
 *			1				error (fail voice registration)
 *			0				success
 *			!0				error (memory error)
 ********************************************************************************/
static SINT32			Set_Param_MA2(
	UINT8				bCount,
	UINT8*				pbBuf,
	UINT8				bExSize
)
{
	PVOICEINFO2			psVoice;
	UINT8				i, j, bOperator, bSize;
	static UINT8		bBuf[30];
	UINT8				bLFO, bFB, bALG, bBO, bML, bVIB, bEGT, bSUS, bKSR, bRR;
	UINT8				bDR, bAR, bSL, bTL, bKSL, bDVB, bDAM, bAM, bWS, bSR;
	SINT32				sdResult;

	if (pbBuf[0] & 0xF0)
		return MMF_FUNC_SUCCESS + 1;

	bLFO = (UINT8)((pbBuf[3] >> 6) & 0x03);
	bFB  = (UINT8)((pbBuf[3] >> 3) & 0x07);
	bALG = (UINT8)(pbBuf[3] & 0x07);
	bBO  = (UINT8)(pbBuf[4] & 0x03);
	bBuf[0] = (UINT8)(bBO | 0x80);
	bBuf[1] = (UINT8)( ((bLFO << 6) & 0xC0) + (bALG & 0x07) );

	if (bALG >= 2) {
		bOperator = 4;
		bSize = 30;
	}
	else {
		bOperator = 2;
		bSize = 16;
	}

	if (bExSize < (((bOperator + 1)* 5)+ 3))
		return MMF_FUNC_SUCCESS + 1;

	i = j = 0;
	while (bOperator != 0) {
		bML  = (UINT8)((pbBuf[5+i] >> 4) & 0x0F);
		bVIB = (UINT8)((pbBuf[5+i] >> 3) & 0x01);
		bEGT = (UINT8)((pbBuf[5+i] >> 2) & 0x01);
		bSUS = (UINT8)((pbBuf[5+i] >> 1) & 0x01);
		bKSR = (UINT8)( pbBuf[5+i] & 0x01);
		bRR  = (UINT8)((pbBuf[6+i] >> 4) & 0x0F);
		bDR  = (UINT8)( pbBuf[6+i] & 0x0F);
		bAR  = (UINT8)((pbBuf[7+i] >> 4) & 0x0F);
		bSL  = (UINT8)( pbBuf[7+i] & 0x0F);
		bTL  = (UINT8)((pbBuf[8+i] >> 2) & 0x3F);
		bKSL = (UINT8)( pbBuf[8+i] & 0x03);
		bDVB = (UINT8)((pbBuf[9+i] >> 6) & 0x03);
		bDAM = (UINT8)((pbBuf[9+i] >> 4) & 0x03);
		bAM  = (UINT8)((pbBuf[9+i] >> 3) & 0x01);
		bWS  = (UINT8)( pbBuf[9+i] & 0x07);

		if (bEGT == 0)
			bSR = bRR;
		else
			bSR = 0;
		if (bSUS != 0)
			bRR = 4;

		bBuf[2+j] = (UINT8)( ((bSR  << 4) & 0xF0) + (bKSR & 0x01) );
		bBuf[3+j] = (UINT8)( ((bRR  << 4) & 0xF0) + (bDR  & 0x0F) );
		bBuf[4+j] = (UINT8)( ((bAR  << 4) & 0xF0) + (bSL  & 0x0F) );
		bBuf[5+j] = (UINT8)( ((bTL  << 2) & 0xFC) + (bKSL & 0x03) );
		bBuf[6+j] = (UINT8)( ((bDAM << 5) & 0x60) + ((bAM << 4) & 0x10) +
							 ((bDVB << 1) & 0x06) + (bVIB & 0x01) );
		bBuf[7+j] = (UINT8)((bML << 4) & 0xF0);
		bBuf[8+j] = (UINT8)( ((bWS  << 3) & 0xF8) + (bFB  & 0x07) );

		i += 5;
		j += 7;
		bOperator --;
		bFB = 0;
	}

	psVoice	= &(gsSmaf_Info.sPlay_Info.sVoice_Info.sVoice_Info2[bCount]);
	psVoice->bBank = pbBuf[1];
	psVoice->bProg = (UINT8)(pbBuf[2] & 0x7F);

	if (gsSmaf_Info.sPlay_Info.dRamSize < (UINT32)bSize)
		return MMF_FUNC_SUCCESS + 1;

	sdResult = MaDevDrv_SendDirectRamData(gsSmaf_Info.sPlay_Info.dRamAdrs,
																0, bBuf, bSize);
	if (sdResult < MMF_FUNC_SUCCESS)			return sdResult;

	sdResult = MaSndDrv_SetVoice(gsSmaf_Info.sdMmfSeqID, 1, bCount, 1, 0,
												gsSmaf_Info.sPlay_Info.dRamAdrs);
	if (sdResult < MMF_FUNC_SUCCESS)			return MMF_FUNC_SUCCESS + 1;

	gsSmaf_Info.sPlay_Info.dRamAdrs += bSize;
	gsSmaf_Info.sPlay_Info.dRamSize -= bSize;

	return MMF_FUNC_SUCCESS;
}


/*********************************************************************************
 *	SetVoice2
 *
 *	Description:
 *			MA-1/2 voice parameter registration
 *	Argument:
 *			nothing
 *	Return:
 *			0					success
 *			!0					error(memory error)
 ********************************************************************************/
static SINT32			SetVoice2( void )
{
	UINT8				bVoicecount, i;
	SINT32				sdResult;
	UINT8*				pbBuf;
	UINT32				dIndex, dSize;

	bVoicecount = 0;

	for (i = 0; i < 5; i++) {
		if ((gsSmaf_Info.sLoad_Info[1].sTrack_Info[i].pbMtr  == NULL) ||
			(gsSmaf_Info.sLoad_Info[1].sTrack_Info[i].pbMtsu == NULL))
			continue;
		dIndex	= 0;
		pbBuf	= gsSmaf_Info.sLoad_Info[1].sTrack_Info[i].pbMtsu;
		dSize	= gsSmaf_Info.sLoad_Info[1].sTrack_Info[i].dMtsuSize;

		while (dSize >= dIndex + 21) {
			if ((pbBuf[dIndex]   != 0xFF) || (pbBuf[dIndex+1] != 0xF0) ||
				(pbBuf[dIndex+2] <= 0x11) || (pbBuf[dIndex+3] != 0x43) ||
				(pbBuf[dIndex+4] <= 0x01) || (pbBuf[dIndex+4] >= 0x04)) {
				dIndex += (pbBuf[dIndex+2] + 3);
				continue;
			}
			if (pbBuf[dIndex+4] == 0x02)			/* MA-1 voice	*/
				sdResult = Set_Param_MA1(bVoicecount, &pbBuf[dIndex+5], pbBuf[dIndex+2]);
			else									/* MA-2 voice	*/
				sdResult = Set_Param_MA2(bVoicecount, &pbBuf[dIndex+5], pbBuf[dIndex+2]);

			if (sdResult <  MMF_FUNC_SUCCESS)		return sdResult;
			if (sdResult == MMF_FUNC_SUCCESS)		bVoicecount ++;

			if (bVoicecount >= 16)					return MMF_FUNC_SUCCESS;
			dIndex += (pbBuf[dIndex+2] + 3);
		}
	}
	return MMF_FUNC_SUCCESS;
}


/*********************************************************************************
 *	Standby2
 *
 *	Description:
 *			MA-1/2 standby
 *	Argument:
 *			nothing
 *	Return:
 *			0					success
 *			!0					error(memory error)
 ********************************************************************************/
static SINT32			Standby2( void )
{
	SINT32				sdResult;
	UINT8				bBuf1, bBuf2, i;
	UINT16				fwKey;

	fwKey	= 0;

	if (gsSmaf_Info.sPlay_Info.dSmafType == MMF_SMAF_TYPE_MA1) {
/* MA-1	*/
/* get key control enable channel #	*/
		bBuf1 = gsSmaf_Info.sLoad_Info[1].sTrack_Info[0].pbMtr[4];
		bBuf2 = gsSmaf_Info.sLoad_Info[1].sTrack_Info[0].pbMtr[5];
#if MASMW_MMF13_KCS_IGNORE
		fwKey = 0x000F;
#else
		if (bBuf1 & 0x80)	fwKey |= 0x01;
		if (bBuf1 & 0x08)	fwKey |= 0x02;
		if (bBuf2 & 0x80)	fwKey |= 0x04;
		if (bBuf2 & 0x08)	fwKey |= 0x08;
#endif
	}
	else {
/* MA-2	*/
		for (i = 0; i < 4; i++) {
/* get key control enable channel #	*/
			if (gsSmaf_Info.sLoad_Info[1].sTrack_Info[i+1].pbMtr == NULL)
				continue;
			bBuf1 = gsSmaf_Info.sLoad_Info[1].sTrack_Info[i+1].pbMtr[4];
			bBuf2 = gsSmaf_Info.sLoad_Info[1].sTrack_Info[i+1].pbMtr[5];
			if (bBuf1 & 0x80)	fwKey |= (0x01 << (i * 4));
			if (bBuf1 & 0x08)	fwKey |= (0x02 << (i * 4));
			if (bBuf2 & 0x80)	fwKey |= (0x04 << (i * 4));
			if (bBuf2 & 0x08)	fwKey |= (0x08 << (i * 4));
		}
/* get & set Stream information	*/
		Awa_Check();
		SetStream2();
	}

/* set FM voice	*/
	sdResult = SetVoice2();
	if (sdResult < MMF_FUNC_SUCCESS)		return sdResult;

/* set key control information	*/
	for (i = 0; i < 16; i++) {
		if (fwKey & (0x0001 << i))
			MaSndDrv_SetKeyControl(gsSmaf_Info.sdMmfSeqID,i,MMF_KEYCONTROL_ON);
		else
			MaSndDrv_SetKeyControl(gsSmaf_Info.sdMmfSeqID,i,MMF_KEYCONTROL_OFF);
	}
	return MMF_FUNC_SUCCESS;
}


/*********************************************************************************
 *	SetStream3
 *
 *	Description:
 *			MA-3/5 set stream inforamtion
 *	Argument:
 *			bSmafType			SMAF Type
 *			bReserve			number of stream reserve
 *	Return:
 *			nothing
 ********************************************************************************/
static void				SetStream3(
	UINT8				bSmafType
)
{
	PSTREAMINFO3		psStrm3;
	UINT8*				pbBuf;
	UINT32				dSize, dIndex, dMiniSize;
	SINT32				sdChunkSize, sdResult;
	UINT32				dChunkID, dChunkNo;
	UINT8				bType, bFormat;
	UINT16				wFs, wFsMax;

/* get pointer & size of Mtsp	*/
	if (bSmafType == MMF_SMAF_TYPE_MA3) {
		pbBuf	= gsSmaf_Info.sLoad_Info[1].sTrack_Info[5].pbMtsp;
		dSize	= gsSmaf_Info.sLoad_Info[1].sTrack_Info[5].dMtspSize;
	}
	else {
		pbBuf	= gsSmaf_Info.sLoad_Info[1].sTrack_Info[6].pbMtsp;
		dSize	= gsSmaf_Info.sLoad_Info[1].sTrack_Info[6].dMtspSize;
	}
	psStrm3	= gsSmaf_Info.sPlay_Info.sStream_Info.sStream_Info3;
	dIndex	= 0;

	while (dSize > (dIndex + MMF_CHUNK_HEADER_SIZE)) {
/* srach Mwa	*/
		sdChunkSize = malib_NextChunk(&pbBuf[dIndex], (dSize-dIndex),
			MALIB_CHUNK_PHASE_MTSPSUB, &dChunkID, &dChunkNo);
		if (sdChunkSize < 0)
			return ;
		dIndex += MMF_CHUNK_HEADER_SIZE;
		if ((dChunkID != MALIB_CHUNKCODE_MWA) || (1 > dChunkNo) || (32 < dChunkNo) ||
			(sdChunkSize < 4)){
			dIndex += sdChunkSize;
			continue;
		}
		bType	= (UINT8)(pbBuf[dIndex] & 0x80);
		wFs		= (UINT16)((((UINT16)pbBuf[dIndex+1]) << 8)+ pbBuf[dIndex+2]);
/* check wave format			*/
		switch (pbBuf[dIndex] & 0x7F) {
		case MMF_WAVE_BIN_2S_COMP_8BIT :
			bFormat		= MMF_STREAM_TYPE_8BIT_2SCOMP;
			if (bType)		dMiniSize	= wFs /  25;
			else			dMiniSize	= wFs /  50;
			wFsMax	= MMF_WAVE_SF_8BIT_MAX;
			break;
		case MMF_WAVE_BIN_OFFSET_8BIT :
			bFormat		= MMF_STREAM_TYPE_8BIT_BINARY;
			if (bType)		dMiniSize	= wFs /  25;
			else			dMiniSize	= wFs /  50;
			wFsMax	= MMF_WAVE_SF_8BIT_MAX;
			break;
		case MMF_WAVE_ADPCM_4BIT :
			bFormat		= MMF_STREAM_TYPE_4BIT_ADPCM;
			if (bType)		dMiniSize	= wFs /  50;
			else			dMiniSize	= wFs / 100;
			wFsMax	= MMF_WAVE_SF_4BIT_MAX;
			break;
		default :
			bFormat		= MMF_STREAM_ERROR;
			dMiniSize	= 0;
			wFsMax		= 0;
			break;
		}
		if ( (bFormat == MMF_STREAM_ERROR) ||
			 (wFs > wFsMax) ||
			 (wFs < MMF_WAVE_SF_MINIMUM) ||
			((sdChunkSize - 3) <= (SINT32)dMiniSize)){
			dIndex += sdChunkSize;		continue;
		}

		if ((bType) && (bSmafType == MMF_SMAF_TYPE_MA3)) {
			dIndex += sdChunkSize;		continue;
		}
		else
			bFormat |= bType;

		sdResult = MaSndDrv_SetStream(gsSmaf_Info.sdMmfSeqID, (UINT8)(dChunkNo-1),
			bFormat, (UINT32)wFs, &(pbBuf[dIndex+3]), sdChunkSize-3);

		if (sdResult >= MMF_FUNC_SUCCESS)
			psStrm3[dChunkNo-1].fbNote = MMF_STREAM_ID_REGIST;

		dIndex += sdChunkSize;
	}
	return ;
}


/*********************************************************************************
 *	SetBaseWave
 *
 *	Description:
 *			MA-3/5 regist FM base wave
 *	Argument:
 *			bSmafType			SMAF Type
 *	Return:
 *			0					success
 *			!0					error(memory error)
 ********************************************************************************/
static SINT32			SetBaseWave(
	UINT8				bSmafType
)
{
	UINT8*				pbBuf;
	UINT32				dSize, dIndex, dPdata, dExSize, dWaveSize, i;
	UINT32				dRamadrs, dRamsize;
	UINT8				bMessageID, bPointer, bDevID;
	SINT32				sdResult;
	static UINT8		abWave[2048];

/* get pointer & size of Setup Data Chunk				*/
	if (bSmafType == MMF_SMAF_TYPE_MA3) {
		pbBuf	= gsSmaf_Info.sLoad_Info[1].sTrack_Info[5].pbMtsu;
		dSize	= gsSmaf_Info.sLoad_Info[1].sTrack_Info[5].dMtsuSize;
		bDevID	= 0x06;
	}
	else {
		pbBuf	= gsSmaf_Info.sLoad_Info[1].sTrack_Info[6].pbMtsu;
		dSize	= gsSmaf_Info.sLoad_Info[1].sTrack_Info[6].dMtsuSize;
		bDevID	= 0x07;
	}
	dIndex		= 0;
	dRamadrs	= gsSmaf_Info.sPlay_Info.dRamAdrs;
	dRamsize	= gsSmaf_Info.sPlay_Info.dRamSize;

	while (dSize >= dIndex + 9) {
/* sreach massage of FM Ext Base Wave	*/
		dExSize = Exclusive_Check(&(pbBuf[dIndex]), (UINT32)(dSize - dIndex),
									bDevID, &bMessageID, &bPointer);
		if ((dRamsize < 2048) || (dExSize == MMF_EXCLUSIVE_CHECK_FAILURE)) {
			gsSmaf_Info.sPlay_Info.dRamAdrs	= dRamadrs;
			gsSmaf_Info.sPlay_Info.dRamSize	= dRamsize;
			return MMF_FUNC_SUCCESS;
		}
		dPdata	= dIndex + bPointer;
		dIndex	+= dExSize;
		if ((bPointer == 0) || (bMessageID != 0x0C) || (dExSize < 8))
			continue;

/* check wave #			*/
		if ((pbBuf[dPdata]!=15) && (pbBuf[dPdata]!=23) && (pbBuf[dPdata]!=31))
			continue;

		dWaveSize = dExSize - bPointer - 2;
		if (bSmafType == MMF_SMAF_TYPE_MA3)
			dWaveSize = malib_Decode_7bitData(&pbBuf[dPdata+1],dWaveSize,abWave,2048);
		else {
			if (dWaveSize > 2048)				dWaveSize = 2048;
			for (i = 0; i < dWaveSize; i ++)	abWave[i] = pbBuf[dPdata+1+i];
			for (i = i; i < 2048; i++)			abWave[i] = 0;
		}
		sdResult = MaDevDrv_SendDirectRamData(dRamadrs, 0, abWave, 2048);
		if (sdResult < MMF_FUNC_SUCCESS)		return sdResult;

		MaSndDrv_SetFmExtWave(gsSmaf_Info.sdMmfSeqID, pbBuf[dPdata], dRamadrs);

		dRamadrs += 2048;
		dRamsize -= 2048;
	}
	gsSmaf_Info.sPlay_Info.dRamAdrs	= dRamadrs;
	gsSmaf_Info.sPlay_Info.dRamSize	= dRamsize;
	return MMF_FUNC_SUCCESS;
}


/*********************************************************************************
 *	SetFmVoice
 *
 *	Description:
 *			MA-5 regist FM voice parameter
 *	Argument:
 *			bBank				bank #
 *			bProg				program(key) #
 *			pbParam				pointer to voice parameter
 *			dPSize				size of voice paramerter
 *	Return:
 *			0					success
 *			!0					error (memory error)
 ********************************************************************************/
static SINT32			SetFmVoice(
	UINT8				bBank,
	UINT8				bProg,
	UINT8*				pbParam,
	UINT32				dPSize
)
{
	UINT32				dRamadrs, dRamsize;
	UINT8				bKeyNo, bAlg, bWSize;
	SINT32				sdResult;

	dRamadrs	= gsSmaf_Info.sPlay_Info.dRamAdrs;
	dRamsize	= gsSmaf_Info.sPlay_Info.dRamSize;

	bKeyNo	= pbParam[0];
	bAlg	= (UINT8)(pbParam[2] & 0x07);

	if (bAlg < 2)
		bWSize	= MMF_2OPFM_VOICE_PARAM_SIZE - 1;
	else {
		if (gsSmaf_Info.sPlay_Info.bFmVoice == MMF_FM_VOICE_MODE_2OP)
			return MMF_FUNC_SUCCESS;
		bWSize	= MMF_4OPFM_VOICE_PARAM_SIZE - 1;
	}

	if ((dRamsize < (UINT32)bWSize) || (dPSize < (UINT32)(bWSize + 1)))
		return MMF_FUNC_SUCCESS;

	sdResult = MaDevDrv_SendDirectRamData(dRamadrs, 0, &pbParam[1], bWSize);
	if (sdResult < MMF_FUNC_SUCCESS)			return sdResult;

	sdResult = MaSndDrv_SetVoice(gsSmaf_Info.sdMmfSeqID, bBank, bProg, 1, bKeyNo, dRamadrs);
	if (sdResult < MMF_FUNC_SUCCESS)			return MMF_FUNC_SUCCESS;

	gsSmaf_Info.sPlay_Info.dRamAdrs	+= bWSize;
	gsSmaf_Info.sPlay_Info.dRamSize	-= bWSize;
	return MMF_FUNC_SUCCESS;
}


/*********************************************************************************
 *	SetWaveData
 *
 *	Description:
 *			MA-5 regist WT voice parameter
 *	Argument:
 *			bWaveID				wave id
 *			bSmafType			SMAF Type
 *	Return:
 *			>0					wave address
 *			0					error (wave not found)
 *			<0					error (memory error)
 ********************************************************************************/
static SINT32			SetWaveData(
	UINT8				bWaveID,
	UINT8				bSmafType
)
{
	PWAVEINFO3			psWave3;
	UINT8*				pbBuf;
	UINT32				dSize, dIndex, dPdata, dExSize, dWaveSize;
	UINT32				dRamadrs, dRamsize;
	SINT32				sdResult;
	UINT8				bMessageID, bPointer, bDevID;

/* if the wave was already registered. return address of the wave	*/
	psWave3	= &(gsSmaf_Info.sPlay_Info.sVoice_Info.sWave_Info3[bWaveID]);
	if (psWave3->dAdrs != MMF_MA3_WAVE_NULL)
		return (SINT32)psWave3->dAdrs;

	if (bSmafType == MMF_SMAF_TYPE_MA3) {
		pbBuf	= gsSmaf_Info.sLoad_Info[1].sTrack_Info[5].pbMtsu;
		dSize	= gsSmaf_Info.sLoad_Info[1].sTrack_Info[5].dMtsuSize;
		bDevID	= 0x06;
	}
	else {
		pbBuf	= gsSmaf_Info.sLoad_Info[1].sTrack_Info[6].pbMtsu;
		dSize	= gsSmaf_Info.sLoad_Info[1].sTrack_Info[6].dMtsuSize;
		bDevID	= 0x07;
	}
	dIndex		= 0;

	while (dSize > dIndex + 13) {
		dExSize = Exclusive_Check(&(pbBuf[dIndex]), (UINT32)(dSize - dIndex),
									bDevID, &bMessageID, &bPointer);
		if (dExSize == MMF_EXCLUSIVE_CHECK_FAILURE) {
			return 0;
		}
		dPdata	= dIndex + bPointer;
		dIndex	+= dExSize;
		if ((bPointer == 0) || (bMessageID != 0x03) || (dExSize < 11))
			continue;

		if (pbBuf[dPdata] != bWaveID)
			continue;

		dWaveSize	= dExSize - bPointer - 3;
		dRamadrs	= gsSmaf_Info.sPlay_Info.dRamAdrs;
		dRamsize	= gsSmaf_Info.sPlay_Info.dRamSize;
		if (bSmafType == MMF_SMAF_TYPE_MA3)
			dWaveSize = ((dWaveSize / 8)* 7) + ((dWaveSize % 8) - 1);
		if (dWaveSize % 2)
			dWaveSize ++;
		if (dWaveSize > dRamsize)
			return 0;

		if (bSmafType == MMF_SMAF_TYPE_MA3)
			sdResult = MaDevDrv_SendDirectRamData(dRamadrs, 1, &(pbBuf[dPdata+2]),
						(dExSize - bPointer - 3));
		else
			sdResult = MaDevDrv_SendDirectRamData(dRamadrs, 0, &(pbBuf[dPdata+2]),
						(dExSize - bPointer - 3));

		if (sdResult < MMF_FUNC_SUCCESS)		return sdResult;

		gsSmaf_Info.sPlay_Info.dRamAdrs += dWaveSize;
		gsSmaf_Info.sPlay_Info.dRamSize -= dWaveSize;
		psWave3->dAdrs	= dRamadrs;
		psWave3->dSize	= dWaveSize;
		return dRamadrs;
	}
	return 0;
}


/*********************************************************************************
 *	SetWtVoice
 *
 *	Description:
 *			MA-5 regist WT voice parameter
 *	Argument:
 *			bBank				bank #
 *			bProg				program(key) #
 *			pbParam				pointer to voice parameter
 *			dPSize				size of voice paramerter
 *			bSmafType			SMAF Type
 *	Return:
 *			0					success
 *			!0					error
 ********************************************************************************/
static SINT32			SetWtVoice(
	UINT8				bBank,
	UINT8				bProg,
	UINT8*				pbParam,
	UINT32				dPSize,
	UINT8				bSmafType
)
{
	UINT32				dRamadrs, dRamsize, dFs;
	SINT32				sdWaveadrs, sdResult;
	UINT8				bWaveID, bWFlag;

	dFs		= (UINT32)((((UINT32)pbParam[0]) << 8)+ pbParam[1]);
	bWFlag	= (UINT8)(pbParam[15] & 0x80);
	bWaveID	= (UINT8)(pbParam[15] & 0x7F);

	if (bWFlag)
		sdWaveadrs	= MaResMgr_GetDefWaveAddress(bWaveID);
	else {
		sdWaveadrs	= SetWaveData(bWaveID, bSmafType);
		if (sdWaveadrs <  0)		return sdWaveadrs;
	}
	if (sdWaveadrs <= 0)			return 0;

	dRamadrs	= gsSmaf_Info.sPlay_Info.dRamAdrs;
	dRamsize	= gsSmaf_Info.sPlay_Info.dRamSize;

	if ((dRamsize < 14) || (dPSize < MMF_WT_VOICE_PARAM_SIZE))
		return MMF_FUNC_SUCCESS;

	pbParam[9]	= (UINT8)((sdWaveadrs >> 8)& 0x000000FF);
	pbParam[10]	= (UINT8)( sdWaveadrs & 0x000000FF);

	sdResult = MaDevDrv_SendDirectRamData(dRamadrs,0,&pbParam[2],13);
	if (sdResult < MMF_FUNC_SUCCESS)			return sdResult;

	sdResult = MaSndDrv_SetVoice(gsSmaf_Info.sdMmfSeqID, bBank, bProg, 2, dFs, dRamadrs);
	if (sdResult < MMF_FUNC_SUCCESS)			return MMF_FUNC_SUCCESS;

	gsSmaf_Info.sPlay_Info.dRamAdrs	+= 14;
	gsSmaf_Info.sPlay_Info.dRamSize	-= 14;
	return MMF_FUNC_SUCCESS;
}


/*********************************************************************************
 *	SetVoice3
 *
 *	Description:
 *			MA-3/5 regist FM base wave
 *	Argument:
 *			bSmafType			SMAF Type
 *	Return:
 *			nothing
 ********************************************************************************/
static SINT32			SetVoice3(
	UINT8				bSmafType
)
{
	UINT8*				pbBuf;
	UINT32				dSize, dIndex, dPdata, dExSize, dParamSize, i;
	UINT8				bMessageID, bPointer, bDevID;
	UINT8				bBankM, bBankL, bProg, bNote, bFlag;
	static UINT8		abParam[MMF_MAX_VOICE_PARAM_SIZE];
	SINT32				sdResult;

	if (bSmafType == MMF_SMAF_TYPE_MA3) {
		pbBuf	= gsSmaf_Info.sLoad_Info[1].sTrack_Info[5].pbMtsu;
		dSize	= gsSmaf_Info.sLoad_Info[1].sTrack_Info[5].dMtsuSize;
		bDevID	= 0x06;
	}
	else {
		pbBuf	= gsSmaf_Info.sLoad_Info[1].sTrack_Info[6].pbMtsu;
		dSize	= gsSmaf_Info.sLoad_Info[1].sTrack_Info[6].dMtsuSize;
		bDevID	= 0x07;
	}
	dIndex		= 0;

	while (dSize > dIndex + 13) {
/* sreach massage of voice registration	*/
		dExSize = Exclusive_Check(&(pbBuf[dIndex]), (UINT32)(dSize - dIndex),
									bDevID, &bMessageID, &bPointer);
		if (dExSize == MMF_EXCLUSIVE_CHECK_FAILURE)				return MMF_FUNC_SUCCESS;
		dPdata	= dIndex + bPointer;
		dIndex	+= dExSize;
		if ((bPointer == 0) || (bMessageID != 0x01) || (dExSize < 5))
			continue;

		bBankM	= pbBuf[dPdata];
		bBankL	= pbBuf[dPdata + 1];
		bProg	= pbBuf[dPdata + 2];
		bNote	= pbBuf[dPdata + 3];
		bFlag	= pbBuf[dPdata + 4];

		if (((bBankM != 0x7C) || (bBankL >= 0x0A)) &&
			((bBankM != 0x7D) || (bBankL != 0x00) || (bProg >= 0x0A)))
			continue;

		if (bBankM == 0x7C)		bBankM	= (UINT8)(bBankL +  1);
		else				{	bBankM	= (UINT8)(bProg + 129);	bProg	= bNote; }
		dParamSize	= dExSize - bPointer - 6;
		dPdata		+= 5;
		if (bSmafType == MMF_SMAF_TYPE_MA3)
			dParamSize = malib_Decode_7bitData(&pbBuf[dPdata], dParamSize,
				abParam, MMF_MAX_VOICE_PARAM_SIZE);
		else {
			for (i = 0; i < MMF_MAX_VOICE_PARAM_SIZE; i++)	abParam[i] = 0;
			for (i = 0; i < dParamSize; i++)	abParam[i] = pbBuf[dPdata+i];
		}

		if ((bFlag & MMF_AL_VOICE_FLAG) &&
			(gsSmaf_Info.sPlay_Info.bAlChReserve >= 16))		continue;

		if ((bFlag & MMF_AL_VOICE_FLAG) &&
			(dParamSize >= MMF_WT_VOICE_PARAM_SIZE + MMF_AL_VOICE_PARAM_SIZE)) {
			sdResult = MaSndDrv_SetALVoice(gsSmaf_Info.sdMmfSeqID, bBankM, bProg,
										&pbBuf[dPdata], MMF_AL_VOICE_PARAM_SIZE);

			if (sdResult < MMF_FUNC_SUCCESS)					continue;

			dParamSize	-= MMF_AL_VOICE_PARAM_SIZE;
			dPdata		=  MMF_AL_VOICE_PARAM_SIZE;
			if ((bFlag & MMF_FMWT_VOICE_FLAG) == MMF_FM_VOICE_PARAMETER)
				sdResult = SetFmVoice(bBankM,bProg,&(abParam[dPdata]),dParamSize);
			else
				sdResult = SetWtVoice(bBankM,bProg,&(abParam[dPdata]),dParamSize,bSmafType);
		}
		else {
			if ((bFlag & MMF_FMWT_VOICE_FLAG) == MMF_FM_VOICE_PARAMETER)
				sdResult = SetFmVoice(bBankM, bProg, abParam, dParamSize);
			else
				sdResult = SetWtVoice(bBankM, bProg, abParam, dParamSize, bSmafType);
		}
		if (sdResult < MMF_FUNC_SUCCESS)						return sdResult;
	}
	return MMF_FUNC_SUCCESS;
}


/*********************************************************************************
 *	Standby3
 *
 *	Description:
 *			MA-3/5 standby
 *	Argument:
 *			bSmafType			SMAF Type
 *	Return:
 *			0					success
 *			<0					error
 ********************************************************************************/
static SINT32			Standby3(
	UINT8				bSmafType
)
{
	PTRACKINFO			psTrk;
	UINT8				bChStatus, i;
	SINT32				sdResult;

/* set key control sutatus	*/
	if (bSmafType == MMF_SMAF_TYPE_MA3) {
		psTrk	= &(gsSmaf_Info.sLoad_Info[1].sTrack_Info[5]);
#if MASMW_MMF13_KCS_IGNORE
#else
		for (i = 0; i < 16; i++) {
			bChStatus = psTrk->pbMtr[4 + i];
			MaSndDrv_SetKeyControl(gsSmaf_Info.sdMmfSeqID, i,
					((bChStatus & MMF_KEY_CONTROL_STATUS) >> 6));
		}
#endif
	}
	else {
		psTrk	= &(gsSmaf_Info.sLoad_Info[1].sTrack_Info[6]);
		for (i = 0; i < 16; i++) {
			bChStatus = psTrk->pbMtr[4 + i];
			MaSndDrv_SetKeyControl(gsSmaf_Info.sdMmfSeqID, i,
					((bChStatus & MMF_KEY_CONTROL_STATUS) >> 6));
		}
	}

	if ((psTrk->pbMtsp != NULL) && (gsSmaf_Info.sPlay_Info.bStreamReserve != 0))
		SetStream3(bSmafType);
	if (psTrk->pbMtsu != NULL) {
		sdResult = SetBaseWave(bSmafType);
		if (sdResult < MMF_FUNC_SUCCESS)		return sdResult;

		sdResult = SetVoice3(bSmafType);
		if (sdResult < MMF_FUNC_SUCCESS)		return sdResult;
	}

	return MMF_FUNC_SUCCESS;
}


/*********************************************************************************
 *	GetEvBlock
 *
 *	Description:
 *			get empty event block
 *	Argument:
 *			nothing
 *	Return:
 *			PEVENTBLOCK		pointer to got block
 ********************************************************************************/
static PEVENTBLOCK		GetEvBlock( void )
{
	PEVENTBLOCK			psEv;

	psEv	= gsSmaf_Info.sPlay_Info.psEmptyEvent;
	if (psEv != NULL)
		gsSmaf_Info.sPlay_Info.psEmptyEvent	= (PEVENTBLOCK)psEv->pvNext;

	return psEv;
}


/*********************************************************************************
 *	SetEvBlock
 *
 *	Description:
 *			set event block to wait list
 *	Argument:
 *			psEv			pointer to event block
 *	Return:
 *			nothing
 ********************************************************************************/
static void				SetEvBlock(
	 PEVENTBLOCK		psEv
)
{
	PEVENTBLOCK			psEv1, psEv2;

	psEv1	= gsSmaf_Info.sPlay_Info.psNextEvent;

	if (psEv1 == NULL) {
		gsSmaf_Info.sPlay_Info.psNextEvent = psEv;
		psEv->pvNext	= NULL;
		return ;
	}
	if (psEv1->dEventTime > psEv->dEventTime) {
		gsSmaf_Info.sPlay_Info.psNextEvent = psEv;
		psEv->pvNext	= (void *)psEv1;
		return ;
	}

	psEv2	= (PEVENTBLOCK)(psEv1->pvNext);

	while (psEv2 != NULL) {
		if (psEv2->dEventTime > psEv->dEventTime) {
			psEv1->pvNext	= (void *)psEv;
			psEv->pvNext	= (void *)psEv2;
			return ;
		}
		psEv1 = psEv2;
		psEv2 = (PEVENTBLOCK)(psEv2->pvNext);
	}

	psEv1->pvNext	= (void *)psEv;
	psEv->pvNext	= NULL;
	return ;
}


/*********************************************************************************
 *	GetOffBlock
 *
 *	Description:
 *			get empty note off block
 *	Argument:
 *			nothing
 *	Return:
 *			PEVENTBLOCK		pointer to got block
 ********************************************************************************/
static POFFBLOCK		GetOffBlock( void )
{
	POFFBLOCK			psOff;

	psOff	= gsSmaf_Info.sPlay_Info.psEmptyOff;
	if (psOff != NULL)
		gsSmaf_Info.sPlay_Info.psEmptyOff	= (POFFBLOCK)psOff->pvNext;

	return psOff;
}


/*********************************************************************************
 *	SetOffBlock
 *
 *	Description:
 *			set note off block to wait list
 *	Argument:
 *			psOff			pointer to note off block
 *	Return:
 *			nothing
 ********************************************************************************/
static void				SetOffBlock(
	 POFFBLOCK			psOff
)
{
	POFFBLOCK			psOff1, psOff2;

	psOff1	= gsSmaf_Info.sPlay_Info.psNextOff;

	if (psOff1 == NULL) {
		gsSmaf_Info.sPlay_Info.psNextOff = psOff;
		psOff->pvNext	= NULL;
		return ;
	}
	if (psOff1->dOffTime > psOff->dOffTime) {
		gsSmaf_Info.sPlay_Info.psNextOff = psOff;
		psOff->pvNext	= (void *)psOff1;
		return ;
	}

	psOff2	= (POFFBLOCK)(psOff1->pvNext);

	while (psOff2 != NULL) {
		if (psOff2->dOffTime > psOff->dOffTime) {
			psOff1->pvNext	= (void *)psOff;
			psOff->pvNext	= (void *)psOff2;
			return ;
		}
		psOff1	= psOff2;
		psOff2	= (POFFBLOCK)(psOff2->pvNext);
	}

	psOff1->pvNext	= (void *)psOff;
	psOff->pvNext	= NULL;;
	return ;
}


/*********************************************************************************
 *	Search_OffBlock2
 *
 *	Description:
 *			search off block
 *	Argument:
 *			psOff			pointer to note off block
 *	Return:
 *			!NULL			pointer to note off block
 *			NULL			not found
 ********************************************************************************/
static POFFBLOCK		Search_OffBlock2(
	UINT32				dCh,
	UINT32				dCmd,
	UINT32				dTime
)
{
	POFFBLOCK			psOff1, psOff2;

	psOff1	= gsSmaf_Info.sPlay_Info.psNextOff;

	if (psOff1 == NULL)
		return NULL;
	if ((dCh	== psOff1->dCh) &&
		(dCmd	== psOff1->dCmd) &&
		(dTime	<  psOff1->dOffTime)) {
		gsSmaf_Info.sPlay_Info.psNextOff = (POFFBLOCK)psOff1->pvNext;
		return psOff1;
	}

	psOff2	= (POFFBLOCK)(psOff1->pvNext);

	while (psOff2 != NULL) {
		if ((dCh	== psOff2->dCh) &&
			(dCmd	== psOff2->dCmd) &&
			(dTime	<  psOff2->dOffTime)) {
			psOff1->pvNext	= psOff2->pvNext;
			return psOff2;
		}
		psOff1	= psOff2;
		psOff2	= (POFFBLOCK)(psOff2->pvNext);
	}

	return NULL;
}


/*********************************************************************************
 *	get_flex2
 *
 *	Description:
 *			get flex data (duration, gate time)
 *	Argument:
 *			pbBuf			pointer to data top
 *			pdRead			pointer to size of flex data
 *	Return:
 *			UINT32			flex data value
 ********************************************************************************/
static UINT32			get_flex2(
	UINT8*				pbBuf,
	UINT32*				pbRead
)
{
	UINT32				dTemp;

	if (pbBuf[0] >= 0x80) {
		dTemp = (UINT32)( (((UINT32)(pbBuf[0] & 0x7F)) << 7) +
							((UINT32)(pbBuf[1] & 0x7F)) + 128 );
		*pbRead = 2;
	}
	else {
		dTemp = (UINT32)(pbBuf[0] & 0x7F);
		*pbRead = 1;
	}
	return dTemp;
}


/*********************************************************************************
 *	Ev_Set
 *
 *	Description:
 *			set parameter to event block
 *	Argument:
 *			psCh			pointer to channel information structure
 *			psEv			pointer to event information structure
 *			dCh				channel #
 *	Return:
 *			nothing
 ********************************************************************************/
static void				Ev_Set(
	PEVENTBLOCK			psEv,
	UINT32				dCmd,
	UINT32				dParam1,
	UINT32				dParam2,
	UINT32				dParam3
)
{
	psEv->dCmd		= dCmd;
	psEv->dParam1	= dParam1;
	psEv->dParam2	= dParam2;
	psEv->dParam3	= dParam3;
	return ;
}


/*********************************************************************************
 *	Note_ON2
 *
 *	Description:
 *			MA-1/2 note on
 *	Argument:
 *			psCh			pointer to channel information structure
 *			psEv			pointer to event information structure
 *			dCh				channel #
 *			dOct			octave shift
 *			dKey			key #
 *			dGT				Gate Time
 *	Return:
 *			nothing
 ********************************************************************************/
static void				Note_ON2(
	PCHANNELINFO		psCh,
	PEVENTBLOCK			psEv,
	UINT32				dCh,
	UINT32				dOct,
	UINT32				dKey,
	UINT32				dGT
)
{
	POFFBLOCK			psOff;

/* if gate time = 0. change event to NOP		*/
	if (dGT == 0) {
		Ev_Set(psEv, MASNDDRV_CMD_NOP, 0, 0, 0);
		return ;
	}

	dKey = ((dOct + psCh->bOct) * 12 + dKey);
	if		(dKey <  12)	dKey  =   0;
	else if	(dKey > 139)	dKey  = 127;
	else					dKey -=  12;

	psOff	= Search_OffBlock2(dCh, MASNDDRV_CMD_NOTE_OFF_MA2, psEv->dEventTime);
	if (psOff == NULL)
		psOff	= GetOffBlock();
	if (psOff == NULL) {
		Ev_Set(psEv, MASNDDRV_CMD_NOP, 0, 0, 0);
		return ;
	}

	psOff->dOffTime	= psEv->dEventTime + dGT;
	psOff->dCmd		= MASNDDRV_CMD_NOTE_OFF_MA2;
	psOff->dCh		= dCh;
	psOff->dKey		= dKey;
	SetOffBlock(psOff);
	Ev_Set(psEv, MASNDDRV_CMD_NOTE_ON_MA2, dCh, dKey, 127);
	return ;
}


/*********************************************************************************
 *	Stream_ON2
 *
 *	Description:
 *			MA-2 stream on
 *	Argument:
 *			psEv			pointer to event information structure
 *			dKey			key #
 *			dVel			note on velocity
 *			dGT				Gate Time
 *	Return:
 *			nothing
 ********************************************************************************/
static void				Stream_ON2(
	PEVENTBLOCK			psEv,
	UINT32				dKey,
	UINT32				dVel,
	UINT32				dGT
)
{
	PSTREAMINFO2		psStrm2;
	POFFBLOCK			psOff;

	if ((dKey < 1) || (0x3E < dKey) || (dGT == 0)) {
		Ev_Set(psEv, MASNDDRV_CMD_NOP, 0, 0, 0);
		return ;
	}

	psStrm2	= &(gsSmaf_Info.sPlay_Info.sStream_Info.sStream_Info2[dKey-1]);
	if (psStrm2->pbWave == NULL) {
		Ev_Set(psEv, MASNDDRV_CMD_NOP, 0, 0, 0);
		return ;
	}

	psOff	= Search_OffBlock2(0, MASNDDRV_CMD_STREAM_OFF, psEv->dEventTime);
	if (psOff != NULL) {
		psOff->dOffTime	= psEv->dEventTime + dGT;
		SetOffBlock(psOff);
		return ;
	}

	psOff	= GetOffBlock();
	if (psOff == NULL) {
		Ev_Set(psEv, MASNDDRV_CMD_NOP, 0, 0, 0);
		return ;
	}

	psOff->dOffTime	= psEv->dEventTime + dGT;
	psOff->dCmd		= MASNDDRV_CMD_STREAM_OFF;
	psOff->dCh		= 0;
	psOff->dKey		= psStrm2->bStrmID;
	SetOffBlock(psOff);
	Ev_Set(psEv, MASNDDRV_CMD_STREAM_ON, 0, psStrm2->bStrmID, dVel);
	return ;
}


/*********************************************************************************
 *	Bank_Program2
 *
 *	Description:
 *			MA-1/2 voice search
 *	Argument:
 *			psCh			pointer to channel information structure
 *			psEv			pointer to event information structure
 *			dCh				channel #
 *	Return:
 *			nothing
 ********************************************************************************/
static void				Bank_Program2(
	PCHANNELINFO		psCh,
	PEVENTBLOCK			psEv,
	UINT32				dCh
)
{
	PVOICEINFO2			psVoice2;
	UINT8				i;

	psVoice2	= &(gsSmaf_Info.sPlay_Info.sVoice_Info.sVoice_Info2[0]);
	for (i = 0; i < MMF_MAX_VOICE_DATA_NUM2; i ++) {
		if ((psVoice2[i].bBank	== psCh->bBankM) &&
			(psVoice2[i].bProg	== psCh->bProg))
			break;
	}

	if (i == MMF_MAX_VOICE_DATA_NUM2) {
		psCh->bBank	= MMF_BANK_NUMBER_DEF;
		psCh->bProg	= MMF_PROGRAM_NUMBER_DEF;
	}
	else {
		psCh->bBank	= 1;
		psCh->bProg	= i;
	}

	if (psEv)
		Ev_Set(psEv, MASNDDRV_CMD_PROGRAM_CHANGE, dCh, psCh->bBank, psCh->bProg);
	return ;
}


/*********************************************************************************
 *	Direct_PitchBend2
 *
 *	Description:
 *			MA-1/2 direct pitch bend
 *	Argument:
 *			psEv			pointer to event information structure
 *	Return:
 *			UINT32			read size
 ********************************************************************************/
static UINT32			Direct_PitchBend2(
	PEVENTBLOCK			psEv,
	PCHANNELINFO		psCh
)
{
	UINT8*				pbBuf;
	UINT8				bCh, bEv;
	UINT32				dFnumBlock, dSize;

	pbBuf	= &(psEv->pbSeq[psEv->dIndex]);

	if ( (pbBuf[0] == 0xFF) && (pbBuf[1] == 0xF0)  &&
		 (pbBuf[2] == 0x06) && (pbBuf[3] == 0x43)  &&
		((pbBuf[4] == 0x02) || (pbBuf[4] == 0x03)) &&
		 (pbBuf[5] == 0x90)) {
		bCh		= (UINT8)(pbBuf[6] & 0x0F);
		bEv		= (UINT8)(pbBuf[6] & 0xF0);
		dSize	= 9;
		switch (bEv) {
		case 0xB0 :
			psCh[bCh].bBlockFnum1	= pbBuf[7];
			bEv	= 0xC0;
			break;
		case 0xC0 :
			psCh[bCh].bBlockFnum2	= pbBuf[7];
			bEv	= 0xB0;
			break;
		default :
			Ev_Set(psEv, MASNDDRV_CMD_NOP, 0, 0, 0);
			return dSize;
		}
	}
	else {
		Ev_Set(psEv, MASNDDRV_CMD_NOP, 0, 0, 0);
		if (pbBuf[1] == 0xF0)
			return (pbBuf[2] + 3);
		else
			return 2;
	}

	pbBuf	= &pbBuf[9];
	if ( (pbBuf[0] == 0x00) && (pbBuf[1] == 0xFF) && (pbBuf[2] == 0xF0)  &&
		 (pbBuf[3] == 0x06) && (pbBuf[4] == 0x43)  &&
		((pbBuf[5] == 0x02) || (pbBuf[5] == 0x03)) &&
		 (pbBuf[6] == 0x90) &&
		((pbBuf[7] & 0x0F) == bCh) && ((pbBuf[7] & 0xF0) == bEv)) {
		dSize	= 19;
		if (bEv == 0xB0)
			psCh[bCh].bBlockFnum1	= pbBuf[8];
		else
			psCh[bCh].bBlockFnum2	= pbBuf[8];
	}

	dFnumBlock = (((UINT32)(psCh[bCh].bBlockFnum2 & 0x1F)) << 8) +
					psCh[bCh].bBlockFnum1 + 0x80000000;

	if (psCh[bCh].bBlockFnum2 & 0x20)
		Ev_Set(psEv, MASNDDRV_CMD_NOTE_ON_MA2EX,  bCh, dFnumBlock, 127);
	else
		Ev_Set(psEv, MASNDDRV_CMD_NOTE_OFF_MA2EX, bCh, dFnumBlock,   0);

	return dSize;
}


/*********************************************************************************
 *	NextEvent2_SeekS
 *
 *	Description:
 *			MA-1/2 seek (score track)
 *	Argument:
 *			pbBuf			pointer to event message top
 *			dTrkNo			track #
 *	Return:
 *			UINT32			read size (byte)
 ********************************************************************************/
static UINT32			NextEvent2_SeekS(
	UINT8*				pbBuf,
	UINT32				dTrkNo
)
{
	PCHANNELINFO		psCh;
	UINT32				dRead, dCh, dEv, dVal;

	if (dTrkNo == 0)
		dCh	= 0;
	else
		dCh	= (dTrkNo - 1)* 4;

	psCh	= &(gsSmaf_Info.sPlay_Info.sChannel_Info[0]);
	switch (pbBuf[0]) {
	case 0x00 :								/* control change					*/
		dCh	+= ((pbBuf[1] >> 6)& 0x03);
		if ((pbBuf[1] & 0x30) != 0x30) {
			dRead	= 2;
			dEv		= ((pbBuf[1] >> 4)& 0x03);
			dVal	=  (pbBuf[1] & 0x0F);
			switch (dEv) {
			case 0x00 :						/* expression (short)				*/
				dVal	= abExpression2[dVal];
				if (dVal != 0x80)
					psCh[dCh].bExp	= (UINT8)dVal;
				break;
			case 0x02 :						/* modulation (short)				*/
				dVal	= abModulation2[dVal];
				if		(dVal >= 0x80)	break;
				else if	(dVal == 0x00)	dVal	= 0;
				else if	(dVal <= 0x10)	dVal	= 1;
				else if	(dVal <= 0x30)	dVal	= 2;
				else if	(dVal <= 0x50)	dVal	= 3;
				else					dVal	= 4;
				psCh[dCh].bMod	= (UINT8)dVal;
				break;
			default :
				break;
			}
		}
		else {
			dRead	= 3;
			dEv		= (pbBuf[1] & 0x0F);
			dVal	=  pbBuf[2];
			switch (dEv) {
			case 0x00 :						/* program change					*/
				psCh[dCh].bProg		= (UINT8)dVal;
				Bank_Program2(&psCh[dCh], NULL, dCh);
				break;
			case 0x01 :						/* bank select						*/
				psCh[dCh].bBankM	= (UINT8)dVal;
				break;
			case 0x02 :						/* octav shift						*/
				if		(dVal <= 0x04)	dVal	+= 4;
				else if	(dVal <= 0x80)	dVal	= psCh[dCh].bOct;
				else if (dVal <= 0x84)	dVal	= 0x84 - dVal;
				else					dVal	= psCh[dCh].bOct;
				psCh[dCh].bOct	= (UINT8)dVal;
				break;
			case 0x03 :						/* modulation						*/
				if		(dVal >= 0x80)	break;
				else if	(dVal == 0x00)	dVal	= 0;
				else if	(dVal <= 0x10)	dVal	= 1;
				else if	(dVal <= 0x30)	dVal	= 2;
				else if	(dVal <= 0x50)	dVal	= 3;
				else					dVal	= 4;
				psCh[dCh].bMod	= (UINT8)dVal;
				break;
			case 0x07 :						/* channel volume					*/
				if (dVal < 0x80)
					psCh[dCh].bChVol	= (UINT8)dVal;
				break;
			case 0x0A :						/* channel pan						*/
				if (dVal < 0x80)
					psCh[dCh].bPan		= (UINT8)dVal;
				break;
			case 0x0B :						/* expression						*/
				if (dVal < 0x80)
					psCh[dCh].bExp		= (UINT8)dVal;
				break;
			default :
				break;
			}
		}
		break;
	case 0xFF :								/* exclusive or nop					*/
		switch (pbBuf[1]) {
		case 0xF0 :							/* exclusive						*/
			dRead	= pbBuf[2] + 3;
			if ( (pbBuf[3] == 0x43) &&
				((pbBuf[4] == 0x02) || (pbBuf[4] == 0x03)) &&
				 (pbBuf[5] == 0x90) ) {
				dCh		= pbBuf[6] & 0x0F;
				dVal	= pbBuf[6] & 0xF0;
				if		(dVal == 0xB0)	psCh[dCh].bBlockFnum1 = pbBuf[7];
				else if	(dVal == 0xC0)	psCh[dCh].bBlockFnum2 = pbBuf[7];
			}
			break;
		default :							/* nop								*/
			dRead	= 2;
			break;
		}
		break;
	default :								/* note on							*/
		if (pbBuf[1] & 0x80)	dRead	= 3;
		else					dRead	= 2;
		break;
	}
	return dRead;
}


/*********************************************************************************
 *	NextEvent2_PlayS
 *
 *	Description:
 *			MA-1/2 seek (score track)
 *	Argument:
 *			psEv			pointer to event information structure
 *	Return:
 *			nothing
 ********************************************************************************/
static void				NextEvent2_PlayS(
	PEVENTBLOCK			psEv
)
{
	PCHANNELINFO		psCh;
	UINT32				dRead, dCh, dEv, dVal, dGT, dGTSize;
	UINT8*				pbBuf;

	if (psEv->dTrackNo == 0)
		dCh	= 0;
	else
		dCh	= (psEv->dTrackNo - 1)* 4;

	psCh	= &(gsSmaf_Info.sPlay_Info.sChannel_Info[0]);
	pbBuf	= &(psEv->pbSeq[psEv->dIndex]);
	switch (pbBuf[0]) {
	case 0x00 :								/* control change					*/
		dCh	+= ((pbBuf[1] >> 6)& 0x03);
		if ((pbBuf[1] & 0x30) != 0x30) {
			dRead	= 2;
			dEv		= ((pbBuf[1] >> 4)& 0x03);
			dVal	=  (pbBuf[1] & 0x0F);
			switch (dEv) {
			case 0x00 :						/* expression (short)				*/
				dVal	= abExpression2[dVal];
				if (dVal != 0x80)
					Ev_Set(psEv, MASNDDRV_CMD_EXPRESSION, dCh, dVal, 0);
				else
					Ev_Set(psEv, MASNDDRV_CMD_NOP, 0, 0, 0);
				break;
			case 0x02 :						/* modulation (short)				*/
				dVal	= abModulation2[dVal];
				if		(dVal >= 0x80) {
					Ev_Set(psEv, MASNDDRV_CMD_NOP, 0, 0, 0);
					break;
				}
				else if	(dVal == 0x00)	dVal	= 0;
				else if	(dVal <= 0x10)	dVal	= 1;
				else if	(dVal <= 0x30)	dVal	= 2;
				else if	(dVal <= 0x50)	dVal	= 3;
				else					dVal	= 4;
				Ev_Set(psEv, MASNDDRV_CMD_MODULATION_DEPTH, dCh, dVal, 0);
				break;
			default :
				Ev_Set(psEv, MASNDDRV_CMD_NOP, 0, 0, 0);
				break;
			}
		}
		else {
			dRead	= 3;
			dEv		= (pbBuf[1] & 0x0F);
			dVal	=  pbBuf[2];
			switch (dEv) {
			case 0x00 :						/* program change					*/
				psCh[dCh].bProg		= (UINT8)dVal;
				Bank_Program2(&psCh[dCh], psEv, dCh);
				break;
			case 0x01 :						/* bank select						*/
				psCh[dCh].bBankM	= (UINT8)dVal;
				Ev_Set(psEv, MASNDDRV_CMD_NOP, 0, 0, 0);
				break;
			case 0x02 :						/* octav shift						*/
				if		(dVal <= 0x04)	dVal	+= 4;
				else if	(dVal <= 0x80)	dVal	= psCh[dCh].bOct;
				else if (dVal <= 0x84)	dVal	= 0x84 - dVal;
				else					dVal	= psCh[dCh].bOct;
				psCh[dCh].bOct	= (UINT8)dVal;
				Ev_Set(psEv, MASNDDRV_CMD_NOP, 0, 0, 0);
				break;
			case 0x03 :						/* modulation						*/
				if		(dVal >= 0x80) {
					Ev_Set(psEv, MASNDDRV_CMD_NOP, 0, 0, 0);
					break;
				}
				else if	(dVal == 0x00)	dVal	= 0;
				else if	(dVal <= 0x10)	dVal	= 1;
				else if	(dVal <= 0x30)	dVal	= 2;
				else if	(dVal <= 0x50)	dVal	= 3;
				else					dVal	= 4;
				Ev_Set(psEv, MASNDDRV_CMD_MODULATION_DEPTH, dCh, dVal, 0);
				break;
			case 0x07 :						/* channel volume					*/
				if (dVal < 0x80)
					Ev_Set(psEv, MASNDDRV_CMD_CHANNEL_VOLUME, dCh, dVal, 0);
				else
					Ev_Set(psEv, MASNDDRV_CMD_NOP, 0, 0, 0);
				break;
			case 0x0A :						/* channel pan						*/
				if (dVal < 0x80)
					Ev_Set(psEv, MASNDDRV_CMD_PANPOT, dCh, dVal, 0);
				else
					Ev_Set(psEv, MASNDDRV_CMD_NOP, 0, 0, 0);
				break;
			case 0x0B :						/* expression						*/
				if (dVal < 0x80)
					Ev_Set(psEv, MASNDDRV_CMD_EXPRESSION, dCh, dVal, 0);
				else
					Ev_Set(psEv, MASNDDRV_CMD_NOP, 0, 0, 0);
				break;
			default :
				break;
			}
		}
		break;
	case 0xFF :								/* exclusive or nop					*/
		switch (pbBuf[1]) {
		case 0xF0 :							/* exclusive						*/
			dRead = Direct_PitchBend2(psEv, psCh);
			break;
		default :							/* nop								*/
			dRead	= 2;
			Ev_Set(psEv, MASNDDRV_CMD_NOP, 0, 0, 0);
			break;
		}
		break;
	default :								/* note on							*/
		dCh		+= ((pbBuf[0] >> 6) & 0x03);
		dVal	=  ((pbBuf[0] >> 4) & 0x03);
		dGT		= get_flex2(&pbBuf[1], &dGTSize);
		dRead	= dGTSize + 1;
		Note_ON2(&psCh[dCh], psEv, dCh, dVal, (pbBuf[0] & 0x0F), dGT);
		break;
	}
	psEv->dIndex	+= dRead;
	return ;
}


/*********************************************************************************
 *	NextEvent2_SeekA
 *
 *	Description:
 *			MA-1/2 seek (audio track)
 *	Argument:
 *			pbBuf			pointer to event message top
 *	Return:
 *			UINT32			read size (byte)
 ********************************************************************************/
static UINT32			NextEvent2_SeekA(
	UINT8*				pbBuf
)
{
	PCHANNELINFO		psCh;
	UINT32				dRead;

	psCh	= &(gsSmaf_Info.sPlay_Info.sChannel_Info[16]);
	switch (pbBuf[0]) {
	case 0x00 :								/* control change					*/
		if ((pbBuf[1] & 0x30) != 0x30)
			dRead	= 2;
		else {
			dRead	= 3;
			if ((pbBuf[1] == 0x37) && (pbBuf[2] < 0x80))
				psCh->bVel	= pbBuf[2];
		}
		break;
	case 0xFF :								/* exclusive or nop					*/
		if (pbBuf[1] == 0xF0)
			dRead	= pbBuf[2] + 3;
		else
			dRead	= 2;
		break;
	default :								/* wave on							*/
		if (pbBuf[1] & 0x80)	dRead	= 3;
		else					dRead	= 2;
		break;
	}
	return dRead;
}


/*********************************************************************************
 *	NextEvent2_PlayA
 *
 *	Description:
 *			MA-1/2 seek (audio track)
 *	Argument:
 *			psEv			pointer to event information structure
 *	Return:
 *			nothing
 ********************************************************************************/
static void				NextEvent2_PlayA(
	PEVENTBLOCK			psEv
)
{
	PCHANNELINFO		psCh;
	UINT32				dRead, dGT, dGTSize;
	UINT8*				pbBuf;

	psCh	= &(gsSmaf_Info.sPlay_Info.sChannel_Info[16]);
	pbBuf	= &(psEv->pbSeq[psEv->dIndex]);
	switch (pbBuf[0]) {
	case 0x00 :								/* control change					*/
		if ((pbBuf[1] & 0x30) != 0x30)
			dRead	= 2;
		else {
			dRead	= 3;
			if ((pbBuf[1] == 0x37) && (pbBuf[2] < 0x80))
				psCh->bVel	= pbBuf[2];
		}
		Ev_Set(psEv, MASNDDRV_CMD_NOP, 0, 0, 0);
		break;
	case 0xFF :								/* exclusive or nop					*/
		if (pbBuf[1] == 0xF0)
			dRead	= pbBuf[2] + 3;
		else
			dRead	= 2;
		Ev_Set(psEv, MASNDDRV_CMD_NOP, 0, 0, 0);
		break;
	default :								/* wave on							*/
		dGT		= get_flex2(&pbBuf[1], &dGTSize);
		dRead	= dGTSize + 1;
		if ((pbBuf[0] & 0xC0) != 0x00)
			Ev_Set(psEv, MASNDDRV_CMD_NOP, 0, 0, 0);
		else
			Stream_ON2(psEv, (pbBuf[0] & 0x3F), psCh->bVel, dGT);
		break;
	}
	psEv->dIndex	+= dRead;
	return ;
}


/*********************************************************************************
 *	Seek2
 *
 *	Description:
 *			MA-1/2 seek
 *	Argument:
 *			nothing
 *	Return:
 *			nothing
 ********************************************************************************/
static void				Seek2( void )
{
	PPLAYINFO			psPlay;
	PTRACKINFO			psTrk;
	PEVENTBLOCK			psEv;
	UINT32				dPast, dDelta, i;
	UINT8*				pbBuf;
	UINT32				dSize, dIndex, dRead;

	psPlay	= &(gsSmaf_Info.sPlay_Info);
	psTrk	= &(gsSmaf_Info.sLoad_Info[1].sTrack_Info[0]);

/* seek score track			*/
	for (i = 0; i < 5; i++) {
		if (psTrk[i].pbMtr == NULL)
			continue;
		if (psTrk[i].dStopTick <= psPlay->dPastTime)
			continue;
		pbBuf	= psTrk[i].pbMtsq;
		dSize	= psTrk[i].dMtsqSize;
		dIndex	= 0;
		dPast	= 0;
		dDelta	= get_flex2(pbBuf, &dRead);
		while (((dPast + dDelta) < psPlay->dPastTime) &&
				(dIndex < dSize)) {
			dPast	+= dDelta;
			dIndex	+= dRead;
			dIndex	+= NextEvent2_SeekS(&pbBuf[dIndex], i);
			if (dIndex >= dSize)	break;
			dDelta	= get_flex2(&pbBuf[dIndex], &dRead);
		}
		dIndex	+= dRead;
		psEv	= GetEvBlock();
		if (dIndex >= dSize) {
			psEv->dEventTime	= psTrk[i].dStopTick;
			psEv->pbSeq			= pbBuf;
			psEv->dIndex		= dIndex;
			psEv->dTrackNo		= i;
			psEv->dCmd			= MMF_EVNET_EOS;
		}
		else {
			psEv->dEventTime	= dPast + dDelta;
			psEv->pbSeq			= pbBuf;
			psEv->dIndex		= dIndex;
			psEv->dTrackNo		= i;
			NextEvent2_PlayS(psEv);
		}
		SetEvBlock(psEv);
	}

/* seek audio track				*/
	i	= MMF_ATR_TRACK_NO;
	if (psTrk[i].pbMtr == NULL)						return ;
	if (psTrk[i].dStopTick <= psPlay->dPastTime)	return ;
	pbBuf	= psTrk[i].pbMtsq;
	dSize	= psTrk[i].dMtsqSize;
	dIndex	= 0;
	dPast	= 0;
	dDelta	= get_flex2(pbBuf, &dRead);
	while ((dPast + dDelta) < psPlay->dPastTime) {
		dPast	+= dDelta;
		dIndex	+= dRead;
		dIndex	+= NextEvent2_SeekA(&pbBuf[dIndex]);
		dDelta	= get_flex2(&pbBuf[dIndex], &dRead);
	}
	dIndex	+= dRead;
	psEv	= GetEvBlock();
	psEv->dEventTime	= dPast + dDelta;
	psEv->pbSeq			= pbBuf;
	psEv->dIndex		= dIndex;
	psEv->dTrackNo		= MMF_ATR_TRACK_NO;
	NextEvent2_PlayA(psEv);
	SetEvBlock(psEv);

	return ;
}


/*********************************************************************************
 *	Stream_ON3
 *
 *	Description:
 *			MA-3 stream on
 *	Argument:
 *			psEv			pointer to event information structure
 *			dCh				channel #
 *			dKey			key # (wave id)
 *			dVel			note on velocity
 *			dGT				Gate Time
 *	Return:
 *			nothing
 ********************************************************************************/
static void				Stream_ON3(
	PEVENTBLOCK			psEv1,
	UINT32				dCh,
	UINT32				dKey,
	UINT32				dVel,
	UINT32				dGT
)
{
	PEVENTBLOCK			psEv2;
	POFFBLOCK			psOff1, psOff2;
	PSTREAMINFO3		psStrm3;

/* if key# >= 32. change event to nop				*/
	if (dKey >= 32) {
		Ev_Set(psEv1, MASNDDRV_CMD_NOP, 0, 0, 0);
		return ;
	}

	psStrm3	= &(gsSmaf_Info.sPlay_Info.sStream_Info.sStream_Info3[0]);
	if (psStrm3[dKey].fbNote == MMF_STREAM_ID_INI) {
		Ev_Set(psEv1, MASNDDRV_CMD_NOP, 0, 0, 0);
		return ;
	}

	if (psStrm3[dKey].bPairID == MMF_STREAM_PAIR_DEF) {
		psOff1	= GetOffBlock();
		if (psOff1 == NULL) {
			Ev_Set(psEv1, MASNDDRV_CMD_NOP, 0, 0, 0);
			return ;
		}
		psOff1->dOffTime	= psEv1->dEventTime + dGT;
		psOff1->dCmd		= MASNDDRV_CMD_STREAM_OFF;
		psOff1->dCh			= dCh;
		psOff1->dKey		= dKey;
		SetOffBlock(psOff1);
		Ev_Set(psEv1, MASNDDRV_CMD_STREAM_ON, dCh, dKey, dVel);
	}
	else {
		psOff1	= GetOffBlock();
		psOff2	= GetOffBlock();
		psEv2	= GetEvBlock();
		if ((psOff1 == NULL) || (psOff2 == NULL) || (psEv2 == NULL)) {
			if (psOff1) {
				psOff1->pvNext = (void *)gsSmaf_Info.sPlay_Info.psEmptyOff;
				gsSmaf_Info.sPlay_Info.psEmptyOff = psOff1;
			}
			if (psOff2) {
				psOff2->pvNext = (void *)gsSmaf_Info.sPlay_Info.psEmptyOff;
				gsSmaf_Info.sPlay_Info.psEmptyOff = psOff2;
			}
			if (psEv2) {
				psEv2->pvNext = (void *)gsSmaf_Info.sPlay_Info.psEmptyEvent;
				gsSmaf_Info.sPlay_Info.psEmptyEvent = psEv2;
			}
			Ev_Set(psEv1, MASNDDRV_CMD_NOP, 0, 0, 0);
			return ;
		}
		psOff2->dOffTime	= psEv1->dEventTime + dGT;
		psOff2->dCmd		= MASNDDRV_CMD_STREAM_OFF;
		psOff2->dCh			= dCh;
		psOff2->dKey		= psStrm3[dKey].bPairID;
		SetOffBlock(psOff2);

		psOff1->dOffTime	= psEv1->dEventTime + dGT;
		psOff1->dCmd		= MASNDDRV_CMD_STREAM_OFF;
		psOff1->dCh			= dCh;
		psOff1->dKey		= dKey;
		SetOffBlock(psOff1);

		psEv2->dEventTime	= psEv1->dEventTime;
		psEv2->dSeqID		= psEv1->dSeqID;
		psEv2->dTrackNo		= 5;
		Ev_Set(psEv2, MASNDDRV_CMD_STREAM_SLAVE, dCh, psStrm3[dKey].bPairID, dVel);
		SetEvBlock(psEv2);
		Ev_Set(psEv1, MASNDDRV_CMD_STREAM_ON, dCh, dKey, dVel);
	}
	return ;
}


/*********************************************************************************
 *	Note_ON3
 *
 *	Description:
 *			MA-3 note on
 *	Argument:
 *			psEv			pointer to event information structure
 *			dCh				channel #
 *			dKey			key #
 *			dVel			note on velocity
 *			dGT				Gate Time
 *			dBank			Bank # (sound driver)
 *	Return:
 *			nothing
 ********************************************************************************/
static void				Note_ON3(
	PEVENTBLOCK			psEv,
	UINT32				dCh,
	UINT32				dKey,
	UINT32				dVel,
	UINT32				dGT,
	UINT32				dBank
)
{
	POFFBLOCK			psOff;

	if (dGT == 0) {
		Ev_Set(psEv, MASNDDRV_CMD_NOP, 0, 0, 0);
		return ;
	}

	if (dBank >= 128) {
		if		(dKey <= 0x0C) {
			Stream_ON3(psEv, dCh, dKey, dVel, dGT);
			return ;
		}
		else if	(dKey >= 0x5C) {
			Stream_ON3(psEv, dCh, (dKey - 79), dVel, dGT);
			return ;
		}
	}

	psOff	= GetOffBlock();
	if (psOff == NULL) {
		Ev_Set(psEv, MASNDDRV_CMD_NOP, 0, 0, 0);
		return ;
	}

	psOff->dOffTime	= psEv->dEventTime + dGT;
	psOff->dCmd		= MASNDDRV_CMD_NOTE_OFF;
	psOff->dCh		= dCh;
	psOff->dKey		= dKey;
	SetOffBlock(psOff);
	Ev_Set(psEv, MASNDDRV_CMD_NOTE_ON, dCh, dKey, dVel);
	return ;
}


/*********************************************************************************
 *	Bank_Program3
 *
 *	Description:
 *			MA-3 program change
 *	Argument:
 *			psCh			pointer to channel information structure
 *			psEv			pointer to event information structure
 *			dCh				channel #
 *	Return:
 *			nothing
 ********************************************************************************/
static void				Bank_Program3(
	PCHANNELINFO		psCh,
	PEVENTBLOCK			psEv,
	UINT32				dCh
)
{
	switch (psCh->bBankM) {
	case 0x7C :										/* normal bank				*/
		if (psCh->bBankL < 0x0a)
			psCh->bBank = (UINT8)(psCh->bBankL + 1);
		else
			psCh->bBank = 0;
		break;
	case 0x7D :										/* drum bank				*/
		if ((psCh->bBankL == 0x00) && (psCh->bProg < 0x0A))
			psCh->bBank = (UINT8)(psCh->bProg + 129);
		else
			psCh->bBank = 128;
		break;
	default :
		if (dCh == 9)
			psCh->bBank = 128;
		else
			psCh->bBank = 0;
		break;
	}

	if (psEv != NULL)
		Ev_Set(psEv, MASNDDRV_CMD_PROGRAM_CHANGE, dCh, psCh->bBank, psCh->bProg);
	return ;
}


/*********************************************************************************
 *	All_Off
 *
 *	Description:
 *			MA-3 all note/sound off
 *	Argument:
 *			dCh				channel #
 *			dTime			off time
 *	Return:
 *			nothing
 ********************************************************************************/
static void				All_Off(
	UINT32				dCh,
	UINT32				dTime
)
{
	POFFBLOCK			psOff1, psOff2;

	psOff1	= gsSmaf_Info.sPlay_Info.psNextOff;
	while (psOff1 != NULL) {
		if ((psOff1->dCh == dCh) && (psOff1->dOffTime >= dTime)) {
			gsSmaf_Info.sPlay_Info.psNextOff = (POFFBLOCK)(psOff1->pvNext);
			psOff1->pvNext = (void *)(gsSmaf_Info.sPlay_Info.psEmptyOff);
			gsSmaf_Info.sPlay_Info.psEmptyOff = psOff1;

			psOff1	= gsSmaf_Info.sPlay_Info.psNextOff;
		}
		else
			break;
	}

	if (psOff1 == NULL)		return ;

	psOff2	= (POFFBLOCK)(psOff1->pvNext);
	while (psOff2 != NULL) {
		if ((psOff2->dCh == dCh) && (psOff2->dOffTime >= dTime)) {
			psOff1->pvNext = psOff2->pvNext;
			psOff2->pvNext = (void *)(gsSmaf_Info.sPlay_Info.psEmptyOff);
			gsSmaf_Info.sPlay_Info.psEmptyOff = psOff2;

			psOff2	=(POFFBLOCK)(psOff1->pvNext);
		}
		else {
			psOff1 = psOff2;
			psOff2 = (POFFBLOCK)(psOff1->pvNext);
		}
	}
	return ;
}


/*********************************************************************************
 *	NextEvent3_Seek
 *
 *	Description:
 *			MA-3 seek
 *	Argument:
 *			nothing
 *	Return:
 *			nothing
 ********************************************************************************/
static void				NextEvent3_Seek( void )
{
	PPLAYINFO			psPlay;
	PCHANNELINFO		psCh;
	PSTREAMINFO3		psStrm3;
	UINT32				dExSize, i;
	UINT8				bCh, bStatus, bData0, bData1, bData2, bData3;

	psPlay	= &(gsSmaf_Info.sPlay_Info);
	psCh	= &(psPlay->sChannel_Info[0]);
	bStatus	= psPlay->pfnGetByte();
	bCh		= (UINT8)(bStatus & 0x0F);
	switch (bStatus & 0xF0) {
	case 0x80 :									/* note on						*/
		psPlay->fwMono = 0xFFFF;
		psPlay->pfnGetByte();
		get_flex3();
		break;
	case 0x90 :									/* note on (velocty)			*/
		psPlay->fwMono = 0xFFFF;
		psPlay->pfnGetByte();
		bData0	= (UINT8)(psPlay->pfnGetByte() & 0x7F);
		get_flex3();
		psCh[bCh].bVel	= (UINT8)MaSndDrv_Convert20to40Log((UINT32)bData0);
		break;
	case 0xA0 :									/* nop							*/
		psPlay->pfnGetByte();
		psPlay->pfnGetByte();
		break;
	case 0xB0 :									/* control change				*/
		bData0	= (UINT8)(psPlay->pfnGetByte() & 0x7F);
		bData1	= (UINT8)(psPlay->pfnGetByte() & 0x7F);
		switch (bData0) {
		case 0x00 :								/* bank select (MSB)			*/
			psCh[bCh].bBankM	= bData1;
			break;
		case 0x20 :								/* bank select (LSB)			*/
			psCh[bCh].bBankL	= bData1;
			break;
		case 0x01 :								/* modulation					*/
			if (psPlay->bHVChannel == bCh)		break;
			if		(bData1 == 0x00)	bData1 = 0;
			else if	(bData1 <= 0x1F)	bData1 = 1;
			else if	(bData1 <= 0x3F)	bData1 = 2;
			else if	(bData1 <= 0x5F)	bData1 = 3;
			else						bData1 = 4;
			psCh[bCh].bMod		= bData1;
			break;
		case 0x07 :								/* channel volume				*/
			psCh[bCh].bChVol	= bData1;
			break;
		case 0x0A :								/* channel pan					*/
			psCh[bCh].bPan		= bData1;
			break;
		case 0x0B :								/* expression					*/
			psCh[bCh].bExp		= bData1;
			break;
		case 0x40 :								/* hold							*/
			if	(bData1 <= 0x3F)		bData1 = 0;
			else						bData1 = 1;
			psCh[bCh].bHold		= bData1;
			break;
		case 0x06 :								/* data entry (MSB)				*/
			if ((psCh[bCh].bRpnM == 0x00) && (psCh[bCh].bRpnL == 0x00) &&
				(bData1 <= 0x18))
				psCh[bCh].bSens1= bData1;
			break;
		case 0x64 :								/* RPN (LSB)					*/
			psCh[bCh].bRpnL		= bData1;
			break;
		case 0x65 :								/* RPN (MSB)					*/
			psCh[bCh].bRpnM		= bData1;
			break;
		case 0x79 :								/* reset all controler			*/
			if (bData1 != 0)	break;
			psCh[bCh].bMod		= MMF_MODULATION_DEF;
			psCh[bCh].bExp		= MMF_EXPRESSION_DEF;
			psCh[bCh].bHold		= MMF_HOLD_DEF;
			psCh[bCh].bRpnL		= MMF_RPN_DEF;
			psCh[bCh].bRpnM		= MMF_RPN_DEF;
			psCh[bCh].bPitch	= MMF_PITCH_BEND_DEF;
			psCh[bCh].bVel		= MMF_VELOCITY_DEF_MA3;
			break;
		case 0x7E :								/* mono mode on					*/
			if (bData1 != 1)										break;
			if (psPlay->fwMono & (0x0001 << bCh))	break;
			psCh[bCh].bMono		= MMF_CHANNEL_MODE_MONO;
			psPlay->fwMono |= (0x0001 << bCh);
			break;
		case 0x7F :								/* poly mode on					*/
			psPlay->fwMono |= (0x0001 << bCh);
			break;
		default :								/* nop							*/
			break;
		}
		break;
	case 0xC0 :									/* program change				*/
		bData1 = (UINT8)(psPlay->pfnGetByte() & 0x7F);
		psCh[bCh].bProg		= bData1;
		Bank_Program3(&psCh[bCh], NULL, bCh);
		break;
	case 0xD0 :									/* nop							*/
		psPlay->pfnGetByte();
		break;
	case 0xE0 :									/* pitch bend					*/
		bData1 =         psPlay->pfnGetByte();
		bData2 = (UINT8)(psPlay->pfnGetByte() & 0x7F);
		psCh[bCh].bPitch	= bData2;
		psCh[bCh].bSens2	= psCh[bCh].bSens1;
		break;
	case 0xF0 :									/* nop, eos or exclusive		*/
		if (bCh == 0x0F) {
			bData0 = (UINT8)(psPlay->pfnGetByte() & 0x7F);
			if (bData0 == 0x2F)
				psPlay->dSeekParam = MMF_CONVERT_PHASE_PLAY_END;
			break;
		}
		dExSize	= get_flex3();
		bData0	= psPlay->pfnGetByte();
		bData1	= psPlay->pfnGetByte();
		bData2	= psPlay->pfnGetByte();
		bData3	= psPlay->pfnGetByte();
		if ((bData0 != 0x43) || (bData1 != 0x79) ||
			(bData2 != 0x06) || (bData3 != 0x7F)) {
			for (i = 4; i < dExSize; i++)
				psPlay->pfnGetByte();
			break;
		}
		bData0 = (UINT8)(psPlay->pfnGetByte() & 0x7F);
		switch (bData0) {
		case 0x00 :								/* master volume				*/
			bData1 = (UINT8)(psPlay->pfnGetByte() & 0x7F);
			psPlay->bMasterVol	= bData1;
			for (i = 6; i < dExSize; i++)
				psPlay->pfnGetByte();
			break;
		case 0x08 :								/* stream pair					*/
			psStrm3	= &(psPlay->sStream_Info.sStream_Info3[0]);
			bData1 = (UINT8)(psPlay->pfnGetByte() & 0x7F);
			bData2 = (UINT8)(psPlay->pfnGetByte() & 0x7F);
			bData3 = (UINT8)(psPlay->pfnGetByte() & 0x7F);
			for (i = 8; i < dExSize; i++)
				psPlay->pfnGetByte();
			if ((bData2 < 1) || (32 < bData2) ||
				(bData3 < 1) || (32 < bData3) || (bData2 == bData3))
				break;
			bData2 --;
			bData3 --;
			if (bData1 == 0x00) {
				psStrm3[bData2].bPairID	= bData3;
				psStrm3[bData3].bPairID	= bData2;
			}
			else {
				psStrm3[bData2].bPairID	= MMF_STREAM_PAIR_DEF;
				psStrm3[bData3].bPairID	= MMF_STREAM_PAIR_DEF;
			}
			break;
		case 0x0B :								/* stream pan					*/
			psStrm3	= &(psPlay->sStream_Info.sStream_Info3[0]);
			bData1 = (UINT8)(psPlay->pfnGetByte() & 0x7F);
			bData2 = (UINT8)(psPlay->pfnGetByte() & 0x7F);
			bData3 = (UINT8)(psPlay->pfnGetByte() & 0x7F);
			for (i = 8; i < dExSize; i++)
				psPlay->pfnGetByte();
			if ((bData1 < 1) || (32 < bData1))
				break;
			bData1 --;
			switch (bData2) {
			case 0x01 :
				psStrm3[bData1].bPan	= MMF_STREAM_PAN_DEF;
				break;
			case 0x02 :
				psStrm3[bData1].bPan	= MMF_STREAM_PAN_OFF;
				break;
			default :
				psStrm3[bData1].bPan	= bData3;
				break;
			}
			break;
		default :
			for (i = 5; i < dExSize; i++)
				psPlay->pfnGetByte();
			break;
		}
		break;
	default :
		break;
	}
	return ;
}


/*********************************************************************************
 *	NextEvent3_Play
 *
 *	Description:
 *			MA-3 event convert
 *	Argument:
 *			psEv			pointer to event information structure
 *	Return:
 *			nothing
 ********************************************************************************/
static void				NextEvent3_Play(
	PEVENTBLOCK			psEv
)
{
	PPLAYINFO			psPlay;
	PCHANNELINFO		psCh;
	PSTREAMINFO3		psStrm3;
	PEVENTBLOCK			psEv2;
	UINT32				dExSize, dGT, dPitch, i;
	UINT8				bCh, bStatus, bData0, bData1, bData2, bData3;

	psPlay	= &(gsSmaf_Info.sPlay_Info);
	psCh	= &(psPlay->sChannel_Info[0]);
	bStatus	= psPlay->pfnGetByte();
	bCh		= (UINT8)(bStatus & 0x0F);
	switch (bStatus & 0xF0) {
	case 0x80 :									/* note on						*/
		psPlay->fwMono = 0xFFFF;
		bData0	= (UINT8)(psPlay->pfnGetByte() & 0x7F);
		dGT		= get_flex3();
		Note_ON3(psEv, bCh, bData0, psCh[bCh].bVel, dGT, psCh[bCh].bBank);
		break;
	case 0x90 :									/* note on (velocty)			*/
		psPlay->fwMono = 0xFFFF;
		bData0	= (UINT8)(psPlay->pfnGetByte() & 0x7F);
		bData1	= (UINT8)(psPlay->pfnGetByte() & 0x7F);
		dGT		= get_flex3();
		psCh[bCh].bVel	= (UINT8)MaSndDrv_Convert20to40Log((UINT32)bData1);
		Note_ON3(psEv, bCh, bData0, psCh[bCh].bVel, dGT, psCh[bCh].bBank);
		break;
	case 0xA0 :									/* nop							*/
		psPlay->pfnGetByte();
		psPlay->pfnGetByte();
		Ev_Set(psEv, MASNDDRV_CMD_NOP, 0, 0, 0);
		break;
	case 0xB0 :									/* control change				*/
		bData0	= (UINT8)(psPlay->pfnGetByte() & 0x7F);
		bData1	= (UINT8)(psPlay->pfnGetByte() & 0x7F);
		switch (bData0) {
		case 0x00 :								/* bank select (MSB)			*/
			psCh[bCh].bBankM	= bData1;
			Ev_Set(psEv, MASNDDRV_CMD_NOP, 0, 0, 0);
			break;
		case 0x20 :								/* bank select (LSB)			*/
			psCh[bCh].bBankL	= bData1;
			Ev_Set(psEv, MASNDDRV_CMD_NOP, 0, 0, 0);
			break;
		case 0x01 :								/* modulation					*/
			if (psPlay->bHVChannel == bCh) {
				Ev_Set(psEv, MASNDDRV_CMD_NOP, 0, 0, 0);
				break;
			}
			if		(bData1 == 0x00)	bData1 = 0;
			else if	(bData1 <= 0x1F)	bData1 = 1;
			else if	(bData1 <= 0x3F)	bData1 = 2;
			else if	(bData1 <= 0x5F)	bData1 = 3;
			else						bData1 = 4;
			Ev_Set(psEv, MASNDDRV_CMD_MODULATION_DEPTH, bCh, bData1, 0);
			break;
		case 0x07 :								/* channel volume				*/
			Ev_Set(psEv, MASNDDRV_CMD_CHANNEL_VOLUME, bCh, bData1, 0);
			break;
		case 0x0A :								/* channel pan					*/
			Ev_Set(psEv, MASNDDRV_CMD_PANPOT, bCh, bData1, 0);
			break;
		case 0x0B :								/* expression					*/
			Ev_Set(psEv, MASNDDRV_CMD_EXPRESSION, bCh, bData1, 0);
			break;
		case 0x40 :								/* hold							*/
			if	(bData1 <= 0x3F)		bData1 = 0;
			else						bData1 = 1;
			Ev_Set(psEv, MASNDDRV_CMD_HOLD1, bCh, bData1, 0);
			break;
		case 0x06 :								/* data entry (MSB)				*/
			if ((psCh[bCh].bRpnM == 0x00) && (psCh[bCh].bRpnL == 0x00) &&
				(bData1 <= 0x18))
				Ev_Set(psEv, MASNDDRV_CMD_BEND_RANGE, bCh, bData1, 0);
			else
				Ev_Set(psEv, MASNDDRV_CMD_NOP, 0, 0, 0);
			break;
		case 0x64 :								/* RPN (LSB)					*/
			psCh[bCh].bRpnL		= bData1;
			Ev_Set(psEv, MASNDDRV_CMD_NOP, 0, 0, 0);
			break;
		case 0x65 :								/* RPN (MSB)					*/
			psCh[bCh].bRpnM		= bData1;
			Ev_Set(psEv, MASNDDRV_CMD_NOP, 0, 0, 0);
			break;
		case 0x78 :								/* all sound off				*/
			if (bData1 != 0)
				Ev_Set(psEv, MASNDDRV_CMD_NOP, 0, 0, 0);
			else {
				if (bCh == psPlay->bHVChannel) {
					psEv2	= GetEvBlock();
					if (psEv2 != NULL) {
						psEv2->dEventTime	= psEv->dEventTime;
						psEv2->dSeqID		= psEv->dSeqID;
						psEv2->dTrackNo		= 5;
						Ev_Set(psEv2, MASNDDRV_CMD_HV_EVENT, 127, 0, 0);
						SetEvBlock(psEv2);
					}
				}
				Ev_Set(psEv, MASNDDRV_CMD_ALL_SOUND_OFF, bCh, 0, 0);
				All_Off(bCh, psEv->dEventTime);
			}
			break;
		case 0x79 :								/* reset all controler			*/
			if (bData1 != 0) {
				Ev_Set(psEv, MASNDDRV_CMD_NOP, 0, 0, 0);
				break;
			}
			psCh[bCh].bVel		= MMF_VELOCITY_DEF_MA3;
			Ev_Set(psEv, MASNDDRV_CMD_RESET_ALL_CONTROLLERS, bCh, 0, 0);
			break;
		case 0x7B :								/* all note off					*/
			if (bData1 != 0)
				Ev_Set(psEv, MASNDDRV_CMD_NOP, 0, 0, 0);
			else {
				Ev_Set(psEv, MASNDDRV_CMD_ALL_NOTE_OFF, bCh, 0, 0);
				All_Off(bCh, psEv->dEventTime);
			}
			break;
		case 0x7E :								/* mono mode on					*/
			if (bData1 != 1) {
				Ev_Set(psEv, MASNDDRV_CMD_NOP, 0, 0, 0);
				break;
			}
			if (psPlay->fwMono & (0x0001 << bCh)) {
				Ev_Set(psEv, MASNDDRV_CMD_NOP, 0, 0, 0);
				break;
			}
			psPlay->fwMono |= (0x0001 << bCh);
			Ev_Set(psEv, MASNDDRV_CMD_MONO_MODE_ON, bCh, 0, 0);
			break;
		case 0x7F :								/* poly mode on					*/
			psPlay->fwMono |= (0x0001 << bCh);
			Ev_Set(psEv, MASNDDRV_CMD_NOP, 0, 0, 0);
			break;
		default :								/* nop							*/
			Ev_Set(psEv, MASNDDRV_CMD_NOP, 0, 0, 0);
			break;
		}
		break;
	case 0xC0 :									/* program change				*/
		bData1 = (UINT8)(psPlay->pfnGetByte() & 0x7F);
		psCh[bCh].bProg		= bData1;
		Bank_Program3(&psCh[bCh], psEv, bCh);
		break;
	case 0xD0 :									/* HV ON						*/
		bData1 = (UINT8)(psPlay->pfnGetByte() & 0x7F);
		if ((bData1 <= 63) && (bCh == psPlay->bHVChannel)) {
			psEv2	= GetEvBlock();
			if (psEv2 != NULL) {
				psEv2->dEventTime	= psEv->dEventTime;
				psEv2->dSeqID		= psEv->dSeqID;
				psEv2->dTrackNo		= 5;
				Ev_Set(psEv2, MASNDDRV_CMD_HV_EVENT, bData1, 0, 0);
				SetEvBlock(psEv2);
			}
		}
		Ev_Set(psEv, MASNDDRV_CMD_NOP, 0, 0, 0);
		break;
	case 0xE0 :									/* pitch bend					*/
		bData1 =         psPlay->pfnGetByte();
		bData2 = (UINT8)(psPlay->pfnGetByte() & 0x7F);
		dPitch = (((UINT32)bData2) << 7);
		Ev_Set(psEv, MASNDDRV_CMD_PITCH_BEND, bCh, dPitch, 0);
		break;
	case 0xF0 :									/* nop, eos or exclusive		*/
		if (bCh == 0x0F) {
			bData0 = (UINT8)(psPlay->pfnGetByte() & 0x7F);
			if (bData0 == 0x2F)
				psPlay->pfnGetByte();
			Ev_Set(psEv, MASNDDRV_CMD_NOP, 0, 0, 0);
			break;
		}
		dExSize	= get_flex3();
		bData0	= psPlay->pfnGetByte();
		bData1	= psPlay->pfnGetByte();
		bData2	= psPlay->pfnGetByte();
		bData3	= psPlay->pfnGetByte();
		if ((bData0 != 0x43) || (bData1 != 0x79) ||
			(bData2 != 0x06) || (bData3 != 0x7F)) {
			for (i = 4; i < dExSize; i++)
				psPlay->pfnGetByte();
			Ev_Set(psEv, MASNDDRV_CMD_NOP, 0, 0, 0);
			break;
		}
		bData0 = (UINT8)(psPlay->pfnGetByte() & 0x7F);
		switch (bData0) {
		case 0x00 :								/* master volume				*/
			bData1 = (UINT8)(psPlay->pfnGetByte() & 0x7F);
			for (i = 6; i < dExSize; i++)
				psPlay->pfnGetByte();
			Ev_Set(psEv, MASNDDRV_CMD_MASTER_VOLUME, bData1, 0, 0);
			break;
		case 0x08 :								/* stream pair					*/
			psStrm3	= &(psPlay->sStream_Info.sStream_Info3[0]);
			bData1 = (UINT8)(psPlay->pfnGetByte() & 0x7F);
			bData2 = (UINT8)(psPlay->pfnGetByte() & 0x7F);
			bData3 = (UINT8)(psPlay->pfnGetByte() & 0x7F);
			for (i = 8; i < dExSize; i++)
				psPlay->pfnGetByte();
			Ev_Set(psEv, MASNDDRV_CMD_NOP, 0, 0, 0);
			if ((bData2 < 1) || (32 < bData2) ||
				(bData3 < 1) || (32 < bData3) || (bData2 == bData3)) {
				break;
			}
			bData2 --;
			bData3 --;
			if (bData1 == 0x00) {
				psStrm3[bData2].bPairID	= bData3;
				psStrm3[bData3].bPairID	= bData2;
			}
			else {
				psStrm3[bData2].bPairID	= MMF_STREAM_PAIR_DEF;
				psStrm3[bData3].bPairID	= MMF_STREAM_PAIR_DEF;
			}
			break;
		case 0x0B :								/* stream pan					*/
			psStrm3	= &(psPlay->sStream_Info.sStream_Info3[0]);
			bData1 = (UINT8)(psPlay->pfnGetByte() & 0x7F);
			bData2 = (UINT8)(psPlay->pfnGetByte() & 0x7F);
			bData3 = (UINT8)(psPlay->pfnGetByte() & 0x7F);
			for (i = 8; i < dExSize; i++)
				psPlay->pfnGetByte();
			if ((bData1 < 1) || (32 < bData1)) {
				Ev_Set(psEv, MASNDDRV_CMD_NOP, 0, 0, 0);
				break;
			}
			bData1 --;
			switch (bData2) {
			case 0x01 :
				bData3	= MMF_STREAM_PAN_DEF;
				break;
			case 0x02 :
				bData3	= MMF_STREAM_PAN_OFF;
				break;
			default :
				break;
			}
			Ev_Set(psEv, MASNDDRV_CMD_STREAM_PANPOT, bData1, bData3, 0);
			break;
		case 0x10 :								/* user event					*/
			bData1 = (UINT8)(psPlay->pfnGetByte() & 0x7F);
			for (i = 6; i < dExSize; i++)
				psPlay->pfnGetByte();
			if (bData1 <= 15)
				Ev_Set(psEv, MASNDDRV_CMD_USER_EVENT, bData1, 0, 0);
			else
				Ev_Set(psEv, MASNDDRV_CMD_NOP, 0, 0, 0);
			break;
		default :								/* nop							*/
			for (i = 5; i < dExSize; i++)
				psPlay->pfnGetByte();
			Ev_Set(psEv, MASNDDRV_CMD_NOP, 0, 0, 0);
			break;
		}
		break;
	default :
		Ev_Set(psEv, MASNDDRV_CMD_NOP, 0, 0, 0);
		break;
	}
	return ;
}


/*********************************************************************************
 *	Seek3
 *
 *	Description:
 *			MA-3 seek
 *	Argument:
 *			nothing
 *	Return:
 *			nothing
 ********************************************************************************/
static void				Seek3( void )
{
	PPLAYINFO			psPlay;
	PTRACKINFO			psTrk;
	PEVENTBLOCK			psEv;
	PHUFFMANINFO		psHuf;
	UINT32				dPast;

	psPlay	= &(gsSmaf_Info.sPlay_Info);
	psTrk	= &(gsSmaf_Info.sLoad_Info[1].sTrack_Info[5]);
	psHuf	= &(gsSmaf_Info.sLoad_Info[1].sHuffman_Info);

	psHuf->dReadSize= 0;
	psHuf->psBuffer	= psHuf->psFBuf;
	psHuf->sbBitC	= psHuf->sbFBit;
	psHuf->bByte	= psHuf->bFByte;
	dPast	= get_flex3();
	while (dPast < psPlay->dPastTime) {
		if (psHuf->dSeqSize <= psHuf->dReadSize) {
			psEv	= GetEvBlock();
			psEv->dEventTime	= psPlay->dPastTime;
			psEv->dCmd			= MMF_EVNET_EOS;
			psEv->dIndex		= 0xFFFFFFFF;
			SetEvBlock(psEv);
			psPlay->dSeekParam = MMF_CONVERT_PHASE_PLAY_END;
			return ;
		}
		NextEvent3_Seek();
		dPast	+= get_flex3();
	}

	if (psHuf->dSeqSize <= psHuf->dReadSize) {
		psEv	= GetEvBlock();
		psEv->dEventTime	= psPlay->dPastTime;
		psEv->dCmd			= MMF_EVNET_EOS;
		psEv->dIndex		= 0xFFFFFFFF;
		SetEvBlock(psEv);
		psPlay->dSeekParam = MMF_CONVERT_PHASE_PLAY_END;
		return ;
	}

	psEv	= GetEvBlock();
	psEv->dEventTime	= dPast;
	psEv->dTrackNo		= 5;
	NextEvent3_Play(psEv);
	SetEvBlock(psEv);
	return ;
}


/*********************************************************************************
 *	get_byte5
 *
 *	Description:
 *			get 1 byte (from normal data)
 *	Argument:
 *			nothing
 *	Return:
 *			UNIT8			success(read data)
 *			0				error code
 ********************************************************************************/
static UINT8			get_byte5( void )
{
	PHUFFMANINFO		psHuf;

	psHuf	= &(gsSmaf_Info.sLoad_Info[1].sHuffman_Info);
	psHuf->dReadSize ++;
	if (psHuf->dReadSize > psHuf->dSeqSize) {
		return 0;
	}
	return *(psHuf->psBuffer++);
}


/*********************************************************************************
 *	get_flex5
 *
 *	Description:
 *			get 1 byte (from normal data)
 *	Argument:
 *			nothing
 *	Return:
 *			UNIT32			success(read data)
 *			0				error code
 ********************************************************************************/
static UINT32			get_flex5( void )
{
	PHUFFMANINFO		psHuf;
	UINT32				dTemp, dRead;
	UINT8				bTemp;

	psHuf	= &(gsSmaf_Info.sLoad_Info[1].sHuffman_Info);
	dRead = 1;
	bTemp = get_byte5();
	if (psHuf->dReadSize > psHuf->dSeqSize)			return 0;

	dTemp = (UINT32)(bTemp & 0x7F);
	while (bTemp & 0x80) {
		if (dRead >= 4) {
			psHuf->dReadSize = psHuf->dSeqSize;
			return 0;
		}
		dRead ++;
		bTemp = get_byte5();
		if (psHuf->dReadSize > psHuf->dSeqSize)		return 0;

		dTemp = (dTemp << 7) + (UINT32)(bTemp & 0x7F);
	}
	if (dTemp >= MMF_PLAY_TIME_MAX) {
		psHuf->dReadSize = psHuf->dSeqSize;
		return 0;
	}
	return dTemp;
}


/*********************************************************************************
 *	Stream_ON3
 *
 *	Description:
 *			MA-3 stream on
 *	Argument:
 *			psEv			pointer to event information structure
 *			dCh				channel #
 *			dKey			key # (wave id)
 *			dVel			note on velocity
 *			dGT				Gate Time
 *	Return:
 *			nothing
 ********************************************************************************/
static void				Stream_ON5(
	PEVENTBLOCK			psEv1,
	UINT32				dCh,
	UINT32				dKey,
	UINT32				dVel,
	UINT32				dGT
)
{
	PSTREAMINFO3		psStrm3;
	POFFBLOCK			psOff1;
	UINT32				dVelocity;

/* if key# >= 32. change event to nop				*/
	if (dKey >= 32) {
		Ev_Set(psEv1, MASNDDRV_CMD_NOP, 0, 0, 0);
		return ;
	}

	psStrm3	= &(gsSmaf_Info.sPlay_Info.sStream_Info.sStream_Info3[0]);
	if (psStrm3[dKey].fbNote == MMF_STREAM_ID_INI) {
		Ev_Set(psEv1, MASNDDRV_CMD_NOP, 0, 0, 0);
		return ;
	}

	psOff1	= GetOffBlock();
	if (psOff1 == NULL) {
		Ev_Set(psEv1, MASNDDRV_CMD_NOP, 0, 0, 0);
		return ;
	}
	psOff1->dOffTime	= psEv1->dEventTime + dGT;
	psOff1->dCmd		= MASNDDRV_CMD_STREAM_OFF;
	psOff1->dCh			= dCh;
	psOff1->dKey		= dKey;
	SetOffBlock(psOff1);

	dVelocity = MaSndDrv_Mix40LogVol(dVel, (UINT32)gsSmaf_Info.sPlay_Info.bMasterVol);
	Ev_Set(psEv1, MASNDDRV_CMD_STREAM_ON, dCh, dKey, dVelocity);
	return ;
}


/*********************************************************************************
 *	Note_ON5
 *
 *	Description:
 *			MA-3 note on
 *	Argument:
 *			psEv			pointer to event information structure
 *			dCh				channel #
 *			dKey			key #
 *			dVel			note on velocity
 *			dGT				Gate Time
 *			dBank			Bank # (sound driver)
 *	Return:
 *			nothing
 ********************************************************************************/
static void				Note_ON5(
	PEVENTBLOCK			psEv,
	UINT32				dCh,
	UINT32				dKey,
	UINT32				dVel,
	UINT32				dGT,
	UINT32				dBank
)
{
	POFFBLOCK			psOff;

	if (dGT == 0) {
		Ev_Set(psEv, MASNDDRV_CMD_NOP, 0, 0, 0);
		return ;
	}

	if (dBank >= 128) {
		if		(dKey <= 0x0C) {
			Stream_ON5(psEv, dCh, dKey, dVel, dGT);
			return ;
		}
		else if	(dKey >= 0x5C) {
			Stream_ON5(psEv, dCh, (dKey - 79), dVel, dGT);
			return ;
		}
	}

	psOff	= GetOffBlock();
	if (psOff == NULL) {
		Ev_Set(psEv, MASNDDRV_CMD_NOP, 0, 0, 0);
		return ;
	}

	psOff->dOffTime	= psEv->dEventTime + dGT;
	psOff->dCmd		= MASNDDRV_CMD_NOTE_OFF;
	psOff->dCh		= dCh;
	psOff->dKey		= dKey;
	SetOffBlock(psOff);
	Ev_Set(psEv, MASNDDRV_CMD_NOTE_ON, dCh, dKey, dVel);
	return ;
}


/*********************************************************************************
 *	NextEvent5_Seek
 *
 *	Description:
 *			MA-5 seek
 *	Argument:
 *			nothing
 *	Return:
 *			nothing
 ********************************************************************************/
static void				NextEvent5_Seek( void )
{
	PPLAYINFO			psPlay;
	PCHANNELINFO		psCh;
	PSTREAMINFO3		psStrm3;
	PHUFFMANINFO		psHuf;
	UINT32				dExSize, i;
	UINT8				bCh, bStatus, bData0, bData1, bData2, bData3;

	psPlay	= &(gsSmaf_Info.sPlay_Info);
	psCh	= &(psPlay->sChannel_Info[0]);
	bStatus	= get_byte5();
	bCh		= (UINT8)(bStatus & 0x0F);
	switch (bStatus & 0xF0) {
	case 0x80 :									/* note on						*/
		psPlay->fwMono = 0xFFFF;
		get_byte5();
		get_flex5();
		break;
	case 0x90 :									/* note on (velocty)			*/
		psPlay->fwMono = 0xFFFF;
		get_byte5();
		bData0	= (UINT8)(get_byte5() & 0x7F);
		get_flex5();
		psCh[bCh].bVel	= bData0;
		break;
	case 0xA0 :									/* nop							*/
		get_byte5();
		get_byte5();
		break;
	case 0xB0 :									/* control change				*/
		bData0	= (UINT8)(get_byte5() & 0x7F);
		bData1	= (UINT8)(get_byte5() & 0x7F);
		switch (bData0) {
		case 0x00 :								/* bank select (MSB)			*/
			psCh[bCh].bBankM	= bData1;
			break;
		case 0x20 :								/* bank select (LSB)			*/
			psCh[bCh].bBankL	= bData1;
			break;
		case 0x01 :								/* modulation					*/
			if (psPlay->bHVChannel == bCh)		break;
			if		(bData1 == 0x00)	bData1 = 0;
			else if	(bData1 <= 0x1F)	bData1 = 1;
			else if	(bData1 <= 0x3F)	bData1 = 2;
			else if	(bData1 <= 0x5F)	bData1 = 3;
			else						bData1 = 4;
			psCh[bCh].bMod		= bData1;
			break;
		case 0x07 :								/* channel volume				*/
			psCh[bCh].bChVol	= bData1;
			break;
		case 0x0A :								/* channel pan					*/
			psCh[bCh].bPan		= bData1;
			break;
		case 0x0B :								/* expression					*/
			psCh[bCh].bExp		= bData1;
			break;
		case 0x40 :								/* hold							*/
			if	(bData1 <= 0x3F)		bData1 = 0;
			else						bData1 = 1;
			psCh[bCh].bHold		= bData1;
			break;
		case 0x0F :								/* pitch bend range				*/
			if (bData1 <= 0x18)			psCh[bCh].bSens1 = bData1;
			break;
		case 0x47 :								/* resonance					*/
			if (bCh == psPlay->bAlChReserve)
				psPlay->bResonance	= bData1;
			break;
		case 0x4A :								/* brightness					*/
			if (bCh == psPlay->bAlChReserve)
				psPlay->bBrightness	= bData1;
			break;
		case 0x79 :								/* reset all controler			*/
			if (bData1 != 0)	break;
			psCh[bCh].bMod		= MMF_MODULATION_DEF;
			psCh[bCh].bExp		= MMF_EXPRESSION_DEF;
			psCh[bCh].bHold		= MMF_HOLD_DEF;
			psCh[bCh].bPitch	= MMF_PITCH_BEND_DEF;
			psCh[bCh].bVel		= MMF_VELOCITY_DEF_MA5;
			break;
		case 0x7E :								/* mono mode on					*/
			if (bData1 != 1)										break;
			if (psPlay->fwMono & (0x0001 << bCh))	break;
			psCh[bCh].bMono		= MMF_CHANNEL_MODE_MONO;
			psPlay->fwMono |= (0x0001 << bCh);
			break;
/*
		case 0x7F :
			gsSmaf_Info.sPlay_Info.fwMono |= (0x0001 << bCh);
			break;
*/
		default :
			break;
		}
		break;
	case 0xC0 :									/* program change				*/
		bData1 = (UINT8)(get_byte5() & 0x7F);
		psCh[bCh].bProg		= bData1;
		Bank_Program3(&psCh[bCh], NULL, bCh);
		break;
	case 0xD0 :									/* nop							*/
		get_byte5();
		break;
	case 0xE0 :									/* pitch bend					*/
		bData1 =         get_byte5();
		bData2 = (UINT8)(get_byte5() & 0x7F);
		psCh[bCh].bPitch	= bData2;
		psCh[bCh].bSens2	= psCh[bCh].bSens1;
		break;
	case 0xF0 :									/* nop, eos or exclusive		*/
		if (bCh == 0x0F) {
			bData0 = (UINT8)(get_byte5() & 0x7F);
			if (bData0 == 0x2F) {
				psHuf	= &(gsSmaf_Info.sLoad_Info[1].sHuffman_Info);
				psHuf->dReadSize = psHuf->dSeqSize;
			}
			break;
		}
		dExSize	= get_flex5();
		bData0	= get_byte5();		bData1	= get_byte5();
		bData2	= get_byte5();		bData3	= get_byte5();
		if ((bData0 != 0x43) || (bData1 != 0x79) ||
			(bData2 != 0x07) || (bData3 != 0x7F)) {
			for (i = 4; i < dExSize; i++)			get_byte5();
			break;
		}
		bData0 = (UINT8)(get_byte5() & 0x7F);
		switch (bData0) {
		case 0x00 :								/* master volume				*/
			bData1 = (UINT8)(get_byte5() & 0x7F);
			psPlay->bMasterVol	= bData1;
			for (i = 6; i < dExSize; i++)			get_byte5();
			break;
		case 0x0B :								/* stream pan					*/
			psStrm3	= &(psPlay->sStream_Info.sStream_Info3[0]);
			bData1 = (UINT8)(get_byte5() & 0x7F);
			bData2 = (UINT8)(get_byte5() & 0x7F);
			bData3 = (UINT8)(get_byte5() & 0x7F);
			for (i = 8; i < dExSize; i++)			get_byte5();
			if ((bData1 < 1) || (32 < bData1))
				break;
			bData1 --;
			switch (bData2) {
			case 0x00 :
				psStrm3[bData1].bPan	= bData3;
				break;
			case 0x01 :
				psStrm3[bData1].bPan	= MMF_STREAM_PAN_DEF;
				break;
			case 0x02 :
				psStrm3[bData1].bPan	= MMF_STREAM_PAN_OFF;
				break;
			default :
				break;
			}
			break;
		default :
			for (i = 5; i < dExSize; i++)			get_byte5();
			break;
		}
		break;
	default :
		psHuf	= &(gsSmaf_Info.sLoad_Info[1].sHuffman_Info);
		psHuf->dReadSize = psHuf->dSeqSize;
		break;
	}
	return ;
}


/*********************************************************************************
 *	NextEvent5_Play
 *
 *	Description:
 *			MA-5 event convert
 *	Argument:
 *			psEv			pointer to event information structure
 *	Return:
 *			nothing
 ********************************************************************************/
static void				NextEvent5_Play(
	PEVENTBLOCK			psEv
)
{
	PPLAYINFO			psPlay;
	PCHANNELINFO		psCh;
	PSTREAMINFO3		psStrm3;
	PHUFFMANINFO		psHuf;
	PEVENTBLOCK			psEv2;
	UINT32				dExSize, dGT, dPitch, i, dChVol;
	UINT8				bCh, bStatus, bData0, bData1, bData2, bData3;

	psPlay	= &(gsSmaf_Info.sPlay_Info);
	psCh	= &(psPlay->sChannel_Info[0]);
	bStatus	= get_byte5();
	bCh		= (UINT8)(bStatus & 0x0F);
	switch (bStatus & 0xF0) {
	case 0x80 :									/* note on						*/
		psPlay->fwMono = 0xFFFF;
		bData0	= (UINT8)(get_byte5() & 0x7F);
		dGT		= get_flex5();
		Note_ON5(psEv, bCh, bData0, psCh[bCh].bVel, dGT, psCh[bCh].bBank);
		break;
	case 0x90 :									/* note on (velocty)			*/
		psPlay->fwMono = 0xFFFF;
		bData0	= (UINT8)(get_byte5() & 0x7F);
		bData1	= (UINT8)(get_byte5() & 0x7F);
		dGT		= get_flex5();
		psCh[bCh].bVel	= bData1;
		Note_ON5(psEv, bCh, bData0, psCh[bCh].bVel, dGT, psCh[bCh].bBank);
		break;
	case 0xA0 :									/* nop							*/
		get_byte5();
		get_byte5();
		Ev_Set(psEv, MASNDDRV_CMD_NOP, 0, 0, 0);
		break;
	case 0xB0 :									/* control change				*/
		bData0	= (UINT8)(get_byte5() & 0x7F);
		bData1	= (UINT8)(get_byte5() & 0x7F);
		switch (bData0) {
		case 0x00 :								/* bank select (MSB)			*/
			psCh[bCh].bBankM	= bData1;
			Ev_Set(psEv, MASNDDRV_CMD_NOP, 0, 0, 0);
			break;
		case 0x20 :								/* bank select (LSB)			*/
			psCh[bCh].bBankL	= bData1;
			Ev_Set(psEv, MASNDDRV_CMD_NOP, 0, 0, 0);
			break;
		case 0x01 :								/* modulation					*/
			if (psPlay->bHVChannel == bCh) {
				Ev_Set(psEv, MASNDDRV_CMD_NOP, 0, 0, 0);
				break;
			}
			if		(bData1 == 0x00)	bData1 = 0;
			else if	(bData1 <= 0x1F)	bData1 = 1;
			else if	(bData1 <= 0x3F)	bData1 = 2;
			else if	(bData1 <= 0x5F)	bData1 = 3;
			else						bData1 = 4;
			Ev_Set(psEv, MASNDDRV_CMD_MODULATION_DEPTH, bCh, bData1, 0);
			break;
		case 0x07 :								/* channel volume				*/
			psCh[bCh].bChVol	= bData1;
			dChVol	= MaSndDrv_Mix40LogVol((UINT32)bData1,
							(UINT32)psPlay->bMasterVol);
			Ev_Set(psEv, MASNDDRV_CMD_CHANNEL_VOLUME, bCh, dChVol, 0);
			break;
		case 0x0A :								/* channel pan					*/
			Ev_Set(psEv, MASNDDRV_CMD_PANPOT, bCh, bData1, 0);
			break;
		case 0x0B :								/* expression					*/
			Ev_Set(psEv, MASNDDRV_CMD_EXPRESSION, bCh, bData1, 0);
			break;
		case 0x40 :								/* hold							*/
			if	(bData1 <= 0x3F)		bData1 = 0;
			else						bData1 = 1;
			Ev_Set(psEv, MASNDDRV_CMD_HOLD1, bCh, bData1, 0);
			break;
		case 0x0F :								/* pitch bend range				*/
			if (bData1 <= 0x18)
				Ev_Set(psEv, MASNDDRV_CMD_BEND_RANGE, bCh, bData1, 0);
			else
				Ev_Set(psEv, MASNDDRV_CMD_NOP, 0, 0, 0);
			break;
		case 0x47 :								/* resonance					*/
			if (bCh == psPlay->bAlChReserve)
				Ev_Set(psEv, MASNDDRV_CMD_SET_LPFQ, bCh, bData1, 0);
			else
				Ev_Set(psEv, MASNDDRV_CMD_NOP, 0, 0, 0);
			break;
		case 0x4A :								/* brightness					*/
			if (bCh == psPlay->bAlChReserve)
				Ev_Set(psEv, MASNDDRV_CMD_SET_LPFCOF, bCh, bData1, 0);
			else
				Ev_Set(psEv, MASNDDRV_CMD_NOP, 0, 0, 0);
			break;
		case 0x78 :								/* all sound off				*/
			if (bData1 != 0)
				Ev_Set(psEv, MASNDDRV_CMD_NOP, 0, 0, 0);
			else {
				if (bCh == psPlay->bHVChannel) {
					psEv2	= GetEvBlock();
					if (psEv2 != NULL) {
						psEv2->dEventTime	= psEv->dEventTime;
						psEv2->dSeqID		= psEv->dSeqID;
						psEv2->dTrackNo		= 6;
						Ev_Set(psEv2, MASNDDRV_CMD_HV_EVENT, 127, 0, 0);
						SetEvBlock(psEv2);
					}
				}
				Ev_Set(psEv, MASNDDRV_CMD_ALL_SOUND_OFF, bCh, 0, 0);
				All_Off(bCh, psEv->dEventTime);
			}
			break;
		case 0x79 :								/* reset all controler			*/
			if (bData1 != 0) {
				Ev_Set(psEv, MASNDDRV_CMD_NOP, 0, 0, 0);
				break;
			}
			psCh[bCh].bVel		= MMF_VELOCITY_DEF_MA5;
			Ev_Set(psEv, MASNDDRV_CMD_RESET_ALL_CONTROLLERS, bCh, 0, 0);
			break;
		case 0x7B :								/* all note off					*/
			if (bData1 != 0)
				Ev_Set(psEv, MASNDDRV_CMD_NOP, 0, 0, 0);
			else {
				Ev_Set(psEv, MASNDDRV_CMD_ALL_NOTE_OFF, bCh, 0, 0);
				All_Off(bCh, psEv->dEventTime);
			}
			break;
		case 0x7E :								/* mono mode on					*/
			if (bData1 != 1) {
				Ev_Set(psEv, MASNDDRV_CMD_NOP, 0, 0, 0);
				break;
			}
			if (psPlay->fwMono & (0x0001 << bCh)) {
				Ev_Set(psEv, MASNDDRV_CMD_NOP, 0, 0, 0);
				break;
			}
			psPlay->fwMono |= (0x0001 << bCh);
			Ev_Set(psEv, MASNDDRV_CMD_MONO_MODE_ON, bCh, 0, 0);
			break;
/*
		case 0x7F :
			gsSmaf_Info.sPlay_Info.fwMono |= (0x0001 << bCh);
			Ev_Set(psEv, MASNDDRV_CMD_NOP, 0, 0, 0);
			break;
*/
		default :
			Ev_Set(psEv, MASNDDRV_CMD_NOP, 0, 0, 0);
			break;
		}
		break;
	case 0xC0 :									/* program change				*/
		bData1 = (UINT8)(get_byte5() & 0x7F);
		psCh[bCh].bProg		= bData1;
		Bank_Program3(&psCh[bCh], psEv, bCh);
		break;
	case 0xD0 :									/* nop							*/
		bData1 = (UINT8)(get_byte5() & 0x7F);
		if ((bData1 <= 63) && (bCh == psPlay->bHVChannel)) {
			psEv2	= GetEvBlock();
			if (psEv2 != NULL) {
				psEv2->dEventTime	= psEv->dEventTime;
				psEv2->dSeqID		= psEv->dSeqID;
				psEv2->dTrackNo		= 6;
				Ev_Set(psEv2, MASNDDRV_CMD_HV_EVENT, bData1, 0, 0);
				SetEvBlock(psEv2);
			}
		}
		Ev_Set(psEv, MASNDDRV_CMD_NOP, 0, 0, 0);
		break;
	case 0xE0 :									/* pitch bend					*/
		bData1 =         get_byte5();
		bData2 = (UINT8)(get_byte5() & 0x7F);
		dPitch = (((UINT32)bData2) << 7);
		Ev_Set(psEv, MASNDDRV_CMD_PITCH_BEND, bCh, dPitch, 0);
		break;
	case 0xF0 :									/* nop, eos or exclusive		*/
		if (bCh == 0x0F) {
			bData0 = (UINT8)(get_byte5() & 0x7F);
			if (bData0 == 0x2F) {
				psHuf	= &(gsSmaf_Info.sLoad_Info[1].sHuffman_Info);
				psHuf->dReadSize = psHuf->dSeqSize;
			}
			Ev_Set(psEv, MASNDDRV_CMD_NOP, 0, 0, 0);
			break;
		}
		dExSize	= get_flex5();
		bData0	= get_byte5();		bData1	= get_byte5();
		bData2	= get_byte5();		bData3	= get_byte5();
		if ((bData0 != 0x43) || (bData1 != 0x79) ||
			(bData2 != 0x07) || (bData3 != 0x7F)) {
			for (i = 4; i < dExSize; i++)				get_byte5();
			Ev_Set(psEv, MASNDDRV_CMD_NOP, 0, 0, 0);
			break;
		}
		bData0 = (UINT8)(get_byte5() & 0x7F);
		switch (bData0) {
		case 0x00 :								/* master volume				*/
			bData1 = (UINT8)(get_byte5() & 0x7F);
			for (i = 6; i < dExSize; i++)				get_byte5();
			psPlay->bMasterVol	= bData1;
			Ev_Set(psEv, MASNDDRV_CMD_MASTER_VOLUME, 0, 0, 0);
			break;
		case 0x0B :								/* stream pan					*/
			psStrm3	= &(psPlay->sStream_Info.sStream_Info3[0]);
			bData1 = (UINT8)(get_byte5() & 0x7F);
			bData2 = (UINT8)(get_byte5() & 0x7F);
			bData3 = (UINT8)(get_byte5() & 0x7F);
			for (i = 8; i < dExSize; i++)				get_byte5();
			if ((bData1 < 1) || (32 < bData1)) {
				Ev_Set(psEv, MASNDDRV_CMD_NOP, 0, 0, 0);
				break;
			}
			bData1 --;
			switch (bData2) {
			case 0x00 :
				Ev_Set(psEv, MASNDDRV_CMD_STREAM_PANPOT, bData1, bData3, 0);
				break;
			case 0x01 :
				Ev_Set(psEv, MASNDDRV_CMD_STREAM_PANPOT, bData1, MMF_STREAM_PAN_DEF, 0);
				break;
			case 0x02 :
				Ev_Set(psEv, MASNDDRV_CMD_STREAM_PANPOT, bData1, MMF_STREAM_PAN_OFF, 0);
				break;
			default :
				Ev_Set(psEv, MASNDDRV_CMD_NOP, 0, 0, 0);
				break;
			}
			break;
		case 0x10 :								/* user event					*/
			bData1 = (UINT8)(get_byte5() & 0x7F);
			for (i = 6; i < dExSize; i++)				get_byte5();
			if (bData1 <= 15)
				Ev_Set(psEv, MASNDDRV_CMD_USER_EVENT, bData1, 0, 0);
			else
				Ev_Set(psEv, MASNDDRV_CMD_NOP, 0, 0, 0);
			break;
		default :
			for (i = 5; i < dExSize; i++)				get_byte5();
			Ev_Set(psEv, MASNDDRV_CMD_NOP, 0, 0, 0);
			break;
		}
		break;
	default :
		psHuf	= &(gsSmaf_Info.sLoad_Info[1].sHuffman_Info);
		psHuf->dReadSize = psHuf->dSeqSize;
		Ev_Set(psEv, MASNDDRV_CMD_NOP, 0, 0, 0);
		break;
	}
	return ;
}


/*********************************************************************************
 *	Seek5
 *
 *	Description:
 *			MA-5 seek
 *	Argument:
 *			nothing
 *	Return:
 *			nothing
 ********************************************************************************/
static void				Seek5( void )
{
	PPLAYINFO			psPlay;
	PTRACKINFO			psTrk;
	PHUFFMANINFO		psHuf;
	PEVENTBLOCK			psEv;
	UINT32				dPast;

	psPlay	= &(gsSmaf_Info.sPlay_Info);
	psTrk	= &(gsSmaf_Info.sLoad_Info[1].sTrack_Info[6]);
	psHuf	= &(gsSmaf_Info.sLoad_Info[1].sHuffman_Info);

	psHuf->dSeqSize		= psTrk->dMtsqSize;
	psHuf->dReadSize	= 0;
	psHuf->psBuffer		= psTrk->pbMtsq;

	dPast	= get_flex5();
	while (dPast < psPlay->dPastTime) {
		if (psHuf->dSeqSize <= psHuf->dReadSize) {
			psEv	= GetEvBlock();
			psEv->dEventTime	= psPlay->dPastTime;
			psEv->dCmd			= MMF_EVNET_EOS;
			psEv->dIndex		= 0xFFFFFFFF;
			SetEvBlock(psEv);
			psPlay->dSeekParam = MMF_CONVERT_PHASE_PLAY_END;
			return ;
		}
		NextEvent5_Seek();
		dPast	+= get_flex5();
	}

	if (psHuf->dSeqSize <= psHuf->dReadSize) {
		psEv	= GetEvBlock();
		psEv->dEventTime	= psPlay->dPastTime;
		psEv->dCmd			= MMF_EVNET_EOS;
		psEv->dIndex		= 0xFFFFFFFF;
		SetEvBlock(psEv);
		psPlay->dSeekParam = MMF_CONVERT_PHASE_PLAY_END;
		return ;
	}

	psEv	= GetEvBlock();
	psEv->dEventTime	= dPast;
	psEv->dTrackNo		= 6;
	NextEvent5_Play(psEv);
	SetEvBlock(psEv);
	return ;
}


/*********************************************************************************
 *	GetPhraseList
 *
 *	Description:
 *			get phrase information
 *	Argument:
 *			sdFile_id		file id
 *			pvPrm			pointer to phrase information sturcture
 *	Return:
 *			0				success
 *			< 0				error code
 ********************************************************************************/
static SINT32			GetPhraseList(
	SINT32				sdFile_id,
	PMASMW_PHRASELIST	pvPrm
)
{
	PPHRASEINFO			psPhr;
	UINT16				wTag;
	UINT32				dNo, dTB, dST;

	psPhr	= &(gsSmaf_Info.sLoad_Info[sdFile_id].sPhrase_Info[0]);
	wTag	= (UINT16)((((UINT16)pvPrm->tag[0]) << 8) + (UINT16)pvPrm->tag[1]);
	dNo		= 0;
	switch (wTag) {
	case MMF_TAG_PHRASE_A :
		if ((psPhr[0].dStartTick == MMF_STSP_TIME_NULL) ||
			(psPhr[0].dStopTick  == MMF_STSP_TIME_NULL))
			return MMF_ERR_NOTAG;
		dNo = 0;
		break;
	case MMF_TAG_PHRASE_B :
		if ((psPhr[1].dStartTick == MMF_STSP_TIME_NULL) ||
			(psPhr[1].dStopTick  == MMF_STSP_TIME_NULL))
			return MMF_ERR_NOTAG;
		dNo = 1;
		break;
	case MMF_TAG_PHRASE_E :
		if ((psPhr[2].dStartTick == MMF_STSP_TIME_NULL) ||
			(psPhr[2].dStopTick  == MMF_STSP_TIME_NULL))
			return MMF_ERR_NOTAG;
		dNo = 2;
		break;
	case MMF_TAG_PHRASE_I :
		if ((psPhr[3].dStartTick == MMF_STSP_TIME_NULL) ||
			(psPhr[3].dStopTick  == MMF_STSP_TIME_NULL))
			return MMF_ERR_NOTAG;
		dNo = 3;
		break;
	case MMF_TAG_PHRASE_K :
		if ((psPhr[4].dStartTick == MMF_STSP_TIME_NULL) ||
			(psPhr[4].dStopTick  == MMF_STSP_TIME_NULL))
			return MMF_ERR_NOTAG;
		dNo = 4;
		break;
	case MMF_TAG_PHRASE_R :
		if ((psPhr[5].dStartTick == MMF_STSP_TIME_NULL) ||
			(psPhr[5].dStopTick  == MMF_STSP_TIME_NULL))
			return MMF_ERR_NOTAG;
		dNo = 5;
		break;
	case MMF_TAG_PHRASE_S :
		if ((psPhr[6].dStartTick == MMF_STSP_TIME_NULL) ||
			(psPhr[6].dStopTick  == MMF_STSP_TIME_NULL))
			return MMF_ERR_NOTAG;
		dNo = 6;
		break;
	default :
		return MMF_ERR_NOTAG;
	}

	dTB		= gsSmaf_Info.sLoad_Info[sdFile_id].dTimeBase;
	dST		= gsSmaf_Info.sLoad_Info[sdFile_id].dStartTime;

	pvPrm->start = (psPhr[dNo].dStartTick - dST)* dTB;
	pvPrm->stop  = (psPhr[dNo].dStopTick  - dST)* dTB;
	return MMF_FUNC_SUCCESS;
}


/*********************************************************************************
 *	GetLED
 *
 *	Description:
 *			get LED status
 *	Argument:
 *			sdFile_id		file id
 *	Return:
 *			0				asynchronous
 *			1				synchronous
 ********************************************************************************/
static UINT8			GetLED(
	SINT32				file_id
)
{
	PLOADINFO			psLoad;
	MASMW_CONTENTSINFO	Led_State;
	SINT32				sdRet;
	UINT8				bCh, bBuf;
	UINT8*				pbChStatus;

	psLoad	= &(gsSmaf_Info.sLoad_Info[file_id]);
	if		((psLoad->dSmafType == MMF_SMAF_TYPE_MA1) ||
			 (psLoad->dSmafType == MMF_SMAF_TYPE_MA2)) {
		Led_State.code		= 0xFF;
		Led_State.tag[0]	= 'L';
		Led_State.tag[1]	= '2';
		Led_State.buf		= &bBuf;
		Led_State.size		= 1;
		sdRet = MaMmfCnv_Control(file_id, MASMW_GET_CONTENTSDATA, &Led_State, 0);
		if (sdRet == 1)									return 1;
		return 0;
	}
	else if	(psLoad->dSmafType == MMF_SMAF_TYPE_MA3) {
		pbChStatus = (psLoad->sTrack_Info[5].pbMtr + 4);
		for (bCh = 0; bCh < 16; bCh ++)
			if (pbChStatus[bCh] & MMF_LED_SYNC_STATUS)	return 1;
		return 0;
	}
	else {
		pbChStatus = (psLoad->sTrack_Info[6].pbMtr + 4);
		for (bCh = 0; bCh < 16; bCh ++)
			if (pbChStatus[bCh] & MMF_LED_SYNC_STATUS)	return 1;
		return 0;
	}
}


/*********************************************************************************
 *	GetVIB
 *
 *	Description:
 *			get VIB status
 *	Argument:
 *			sdFile_id		file id
 *	Return:
 *			0				asynchronous
 *			1				synchronous
 ********************************************************************************/
static UINT8			GetVIB(
	SINT32				file_id
)
{
	PLOADINFO			psLoad;
	UINT8				bCh;
	UINT8*				pbChStatus;

	psLoad	= &(gsSmaf_Info.sLoad_Info[file_id]);
	if		(psLoad->dSmafType == MMF_SMAF_TYPE_MA1) {
		pbChStatus = psLoad->sTrack_Info[0].pbMtr;
		if ((pbChStatus[4] & 0x44) || (pbChStatus[5] & 0x44))		return 1;
		return 0;
	}
	else if	(psLoad->dSmafType == MMF_SMAF_TYPE_MA2) {
		for (bCh = 1; bCh < 5; bCh ++) {
			pbChStatus = psLoad->sTrack_Info[bCh].pbMtr;
			if (pbChStatus == NULL)									continue;
			if ((pbChStatus[4] & 0x44) || (pbChStatus[5] & 0x44))	return 1;
		}
		return 0;
	}
	else if	(psLoad->dSmafType == MMF_SMAF_TYPE_MA3) {
		pbChStatus = (psLoad->sTrack_Info[5].pbMtr + 4);
		for (bCh = 0; bCh < 16; bCh ++)
			if (pbChStatus[bCh] & MMF_VIB_SYNC_STATUS)				return 1;
		return 0;
	}
	else {
		pbChStatus = (psLoad->sTrack_Info[6].pbMtr + 4);
		for (bCh = 0; bCh < 16; bCh ++)
			if (pbChStatus[bCh] & MMF_VIB_SYNC_STATUS)				return 1;
		return 0;
	}
}


/*********************************************************************************
 *	GetKCS
 *
 *	Description:
 *			get key control status
 *	Argument:
 *			sdFile_id		file id
 *	Return:
 *			0				disable
 *			1				enable
 ********************************************************************************/
static UINT8			GetKCS(
	SINT32				file_id
)
{
	PLOADINFO			psLoad;
	UINT8				bCh;
	UINT8*				pbChStatus;

	psLoad	= &(gsSmaf_Info.sLoad_Info[file_id]);
	if		(psLoad->dSmafType == MMF_SMAF_TYPE_MA1) {
#if MASMW_MMF13_KCS_IGNORE
		return 1;
#else
		pbChStatus = psLoad->sTrack_Info[0].pbMtr;
		if ((pbChStatus[4] & 0x88) || (pbChStatus[5] & 0x88))		return 1;
		return 0;
#endif
	}
	else if	(psLoad->dSmafType == MMF_SMAF_TYPE_MA2) {
		for (bCh = 1; bCh < 5; bCh ++) {
			pbChStatus = psLoad->sTrack_Info[bCh].pbMtr;
			if (pbChStatus == NULL)									continue;
			if ((pbChStatus[4] & 0x88) || (pbChStatus[5] & 0x88))	return 1;
		}
		return 0;
	}
	else if	(psLoad->dSmafType == MMF_SMAF_TYPE_MA3) {
#if MASMW_MMF13_KCS_IGNORE
		return 1;
#else
		pbChStatus = (psLoad->sTrack_Info[5].pbMtr + 4);
		for (bCh = 0; bCh < 16; bCh ++)
			if ((pbChStatus[bCh] & MMF_KEY_CONTROL_STATUS) != 0x40)	return 1;
		return 0;
#endif
	}
	else {
		pbChStatus = (psLoad->sTrack_Info[6].pbMtr + 4);
		for (bCh = 0; bCh < 16; bCh ++)
			if ((pbChStatus[bCh] & MMF_KEY_CONTROL_STATUS) != 0x40)	return 1;
		return 0;
	}
}


/*********************************************************************************
 *	Get_LoadInfo
 *
 *	Description:
 *			write Load Information
 *	Argument:
 *			sdFile_id		file id
 *			pvWrite			pointer to data write space
 *	Return:
 *			0 or 1			success
 *			< 0				error code
 ********************************************************************************/
static SINT32			Get_LoadInfo(
	SINT32				sdFile_id,
	void*				pvWrite
)
{
	PLOADINFO			psLoad;
	POPTIONINFO			psOptn;
	PTRACKINFO			psTrk;
	PPHRASEINFO			psPhr;
	UINT32*				pdData;
	UINT8*				pbData;
	UINT32				dFAdrs, dTemp, i;

	if (pvWrite == NULL)
		return MMF_LOADINFO_ERROR;

	psLoad	= &(gsSmaf_Info.sLoad_Info[sdFile_id]);
	psOptn	= &(psLoad->sOption_Info);
	psTrk	= &(psLoad->sTrack_Info[5]);
	psPhr	= &(psLoad->sPhrase_Info[0]);
	dFAdrs	= (UINT32)psLoad->pbMmmd;

	if ((psLoad->pbMmmd == NULL) || (psLoad->dCrc == MMF_CRC_NULL))
		return MMF_LOADINFO_ERROR;

	pdData	= (UINT32*)pvWrite;
	pbData	= (UINT8*)pvWrite;

	if (psLoad->dSmafType != MMF_SMAF_TYPE_MA3) {
		pdData[ 0]	= 1;
		pbData[124]	= 0xFF;
		pbData[125]	= 0x00;
		pbData[126]	= 0x00;
		pbData[127]	= 0x00;
		return MMF_LOADINFO_FAILURE;
	}

/*
	pdData[ 0]	= 0;
	pdData[ 1]	= psLoad->dMmmdSize;
	pdData[ 2]	= psLoad->dMmmdSize + 8;
	pdData[ 3]	= psLoad->dCrc;
	pdData[ 4]	= (UINT32)((UINT32)psOptn->pbOpda - dFAdrs);
	pdData[ 5]	= psOptn->dOpdaSize;
	pdData[ 6]	= psLoad->dPlayTime;
*/
	pdData[ 0]	= 0;
	pdData[ 1]	= psLoad->dMmmdSize;
	pdData[ 2]	= psLoad->dCrc;
	pdData[ 3]	= (UINT32)((UINT32)psOptn->pbCnti - dFAdrs);
	pdData[ 4]	= psOptn->dCntiSize;
	pdData[ 5]	= (UINT32)((UINT32)psOptn->pbOpda - dFAdrs);
	pdData[ 6]	= psOptn->dOpdaSize;

	pdData[ 7]	= (UINT32)((UINT32)psTrk->pbMtr  - dFAdrs);
	pdData[ 8]	= psTrk->dMtrSize;
	pdData[ 9]	= (UINT32)((UINT32)psTrk->pbMtsu - dFAdrs);
	pdData[10]	= psTrk->dMtsuSize;
	pdData[11]	= (UINT32)((UINT32)psTrk->pbMtsq - dFAdrs);
	pdData[12]	= psTrk->dMtsqSize;
	pdData[13]	= (UINT32)((UINT32)psTrk->pbMtsp - dFAdrs);
	pdData[14]	= psTrk->dMtspSize;

	pdData[15]	= psTrk->dStartTick;
	pdData[16]	= psTrk->dStopTick;
	pdData[17]	= psPhr[0].dStartTick;
	pdData[18]	= psPhr[0].dStopTick;
	pdData[19]	= psPhr[1].dStartTick;
	pdData[20]	= psPhr[1].dStopTick;
	pdData[21]	= psPhr[2].dStartTick;
	pdData[22]	= psPhr[2].dStopTick;
	pdData[23]	= psPhr[3].dStartTick;
	pdData[24]	= psPhr[3].dStopTick;
	pdData[25]	= psPhr[4].dStartTick;
	pdData[26]	= psPhr[4].dStopTick;
	pdData[27]	= psPhr[5].dStartTick;
	pdData[28]	= psPhr[5].dStopTick;
	pdData[29]	= psPhr[6].dStartTick;
	pdData[30]	= psPhr[6].dStopTick;

	pbData[124]	= 0;
	pbData[125]	= (UINT8)psLoad->dTimeBase;
	pbData[126]	= 0;

	dTemp = 0;
	for (i = 0; i < 127; i++)
		dTemp += pbData[i];

	pbData[127]	= (UINT8)(dTemp & 0x000000FF);
	return MMF_LOADINFO_SUCCESS;
}


/*********************************************************************************
 *	Set_LoadInfo
 *
 *	Description:
 *			read Load Information
 *	Argument:
 *			pvRead			pointer to data read space
 *	Return:
 *			0 or 1			success
 *			< 0				error code
 ********************************************************************************/
static SINT32			Set_LoadInfo(
	void*				pvRead
)
{
	PLOADINFO			psLoad;
	POPTIONINFO			psOptn;
	PTRACKINFO			psTrk;
	PPHRASEINFO			psPhr;
	UINT32*				pdData;
	UINT8*				pbData;
	UINT32				dTemp, i;

	if ((pvRead == NULL) || (gsSmaf_Info.dStatus == MMF_STATUS_LOADED))
		return MMF_LOADINFO_ERROR;

	psLoad	= &(gsSmaf_Info.sLoad_Info[1]);
	psOptn	= &(psLoad->sOption_Info);
	psTrk	= &(psLoad->sTrack_Info[5]);
	psPhr	= &(psLoad->sPhrase_Info[0]);
	pdData	= (UINT32*)pvRead;
	pbData	= (UINT8*)pvRead;

	if ((pbData[124] == 0xFF) &&
		(pbData[125] == 0x00) &&
		(pbData[126] == 0x00) &&
		(pbData[127] == 0x00) )
		return MMF_LOADINFO_FAILURE;

	dTemp = 0;
	for (i = 0; i < 127; i++)
		dTemp += pbData[i];
	if ((UINT8)(dTemp & 0x000000FF) != pbData[127])
		return MMF_LOADINFO_ERROR;

	Check_Initial( psLoad );

/*
	psLoad->dMmmdSize	= pdData[ 1];
	psLoad->dCrc		= pdData[ 3];
	psOptn->pbOpda		= (UINT8*)pdData[ 4];
	psOptn->dOpdaSize	= pdData[ 5];
	psLoad->dPlayTime	= pdData[ 6];
*/

	psLoad->dMmmdSize	= pdData[ 1];
	psLoad->dCrc		= pdData[ 2];
	psOptn->pbCnti		= (UINT8*)pdData[ 3];
	psOptn->dCntiSize	= pdData[ 4];
	psOptn->pbOpda		= (UINT8*)pdData[ 5];
	psOptn->dOpdaSize	= pdData[ 6];

	psTrk->pbMtr 		= (UINT8*)pdData[ 7];
	psTrk->dMtrSize		= pdData[ 8];
	psTrk->pbMtsu		= (UINT8*)pdData[ 9];
	psTrk->dMtsuSize	= pdData[10];
	psTrk->pbMtsq		= (UINT8*)pdData[11];
	psTrk->dMtsqSize	= pdData[12];
	psTrk->pbMtsp		= (UINT8*)pdData[13];
	psTrk->dMtspSize	= pdData[14];

	psTrk->dStartTick	= pdData[15];
	psTrk->dStopTick	= pdData[16];
	psPhr[0].dStartTick	= pdData[17];
	psPhr[0].dStopTick	= pdData[18];
	psPhr[1].dStartTick	= pdData[19];
	psPhr[1].dStopTick	= pdData[20];
	psPhr[2].dStartTick	= pdData[21];
	psPhr[2].dStopTick	= pdData[22];
	psPhr[3].dStartTick	= pdData[23];
	psPhr[3].dStopTick	= pdData[24];
	psPhr[4].dStartTick	= pdData[25];
	psPhr[4].dStopTick	= pdData[26];
	psPhr[5].dStartTick	= pdData[27];
	psPhr[5].dStopTick	= pdData[28];
	psPhr[6].dStartTick	= pdData[29];
	psPhr[6].dStopTick	= pdData[30];

	psLoad->dSmafType	= MMF_SMAF_TYPE_MA3;
	psLoad->dPlayTime	= pdData[16] - pdData[15];
	psLoad->dStartTime	= pdData[15];
	psLoad->dTimeBase	= (UINT32)pbData[125];
	gsSmaf_Info.dStatus	= MMF_STATUS_SAT_PROFILE;

	return MMF_LOADINFO_SUCCESS;
}


/*********************************************************************************
 *	SetSeekParameter
 *
 *	Description:
 *			set seek event
 *	Argument:
 *			nothing
 *	Return:
 *			number of set command
 ********************************************************************************/
static SINT32			SetSeekParameter( void )
{
	PPLAYINFO			psPlay;
	PCHANNELINFO		psCh;
	PSTREAMINFO3		psStrm3;
	UINT8				bPhase, bEv1, bEv2, bEnd;
	UINT32				dPitch, dChVol;
	SINT32				sdSeqID;

	psPlay	= &(gsSmaf_Info.sPlay_Info);
	sdSeqID	=   gsSmaf_Info.sdMmfSeqID ;

	bPhase	= (UINT8)((psPlay->dSeekParam >> 16) & 0x000000FF);
	bEv1	= (UINT8)((psPlay->dSeekParam >>  8) & 0x000000FF);
	bEv2	= (UINT8)( psPlay->dSeekParam        & 0x000000FF);

	switch (bPhase) {
	case 0x01 :
		switch (bEv1) {
		case 0x00 :
			MaSndDrv_SetCommand(sdSeqID, 0, MASNDDRV_CMD_SYSTEM_ON, 0, 0, 0);
/*
			bEv1 = 1;
*/
			bEv2 = 0;
			switch (psPlay->dSmafType) {
			case MMF_SMAF_TYPE_MA5 :					/* MA-5					*/
				bEv1 = 2;								/* stream pan			*/
				break;
			case MMF_SMAF_TYPE_MA3 :					/* MA-3					*/
				bEv1 = 1;								/* master volume		*/
				break;
			default :									/* MA-1/2				*/
				bEv1 = 5;								/* LED status			*/
				break;
			}
			break;
		case 0x01 :
			MaSndDrv_SetCommand(sdSeqID, 0, MASNDDRV_CMD_MASTER_VOLUME, psPlay->bMasterVol, 0, 0);
			bEv1 = 2;	bEv2 = 0;
			break;
/*
			MaSndDrv_SetCommand(sdSeqID, 0, MASNDDRV_CMD_MASTER_VOLUME, psPlay->bMasterVol, 0, 0);
			if (psPlay->dSmafType >= MMF_SMAF_TYPE_MA3) {
				bEv1 = 2;	bEv2 = 0;
			}
			else {
				bEv1 = 5;	bEv2 = 0;
			}
			break;
*/
		case 0x02 :
			psStrm3 = &(gsSmaf_Info.sPlay_Info.sStream_Info.sStream_Info3[bEv2]);
			MaSndDrv_SetCommand(sdSeqID, 0, MASNDDRV_CMD_STREAM_PANPOT, bEv2, psStrm3->bPan, 0);
			if (bEv2 < (MMF_MAX_STREAM_DATA_NUM3 - 1))
				bEv2 ++;
			else {
				if (psPlay->bAlChReserve != MMF_AL_CHANNEL_NULL)
					bEv1 = 3;
				else {
					bPhase = 2;	bEv1 = 0;	bEv2 = 0;
				}
			}
			break;
		case 0x03 :
			MaSndDrv_SetCommand(sdSeqID, 0, MASNDDRV_CMD_SET_LPFQ, psPlay->bAlChReserve,
																	psPlay->bResonance, 0);
			bEv1 = 4;
			break;
		case 0x04 :
			MaSndDrv_SetCommand(sdSeqID, 0, MASNDDRV_CMD_SET_LPFCOF, psPlay->bAlChReserve,
																	psPlay->bBrightness, 0);
			bPhase = 2;	bEv1 = 0;	bEv2 = 0;
			break;
		case 0x05 :
			psCh	= &(psPlay->sChannel_Info[16]);
			if (psCh->fbLed)
				MaSndDrv_SetCommand(sdSeqID, 0, MASNDDRV_CMD_SET_LED_SYNC_CH, 16, 1, 0);
			else
				MaSndDrv_SetCommand(sdSeqID, 0, MASNDDRV_CMD_SET_LED_SYNC_CH, 16, 0, 0);
			bEv1 = 6;
			break;
		case 0x06 :
			MaSndDrv_SetCommand(sdSeqID, 0, MASNDDRV_CMD_SET_MOTOR_SYNC_CH, 16, 0, 0);
			bPhase = 2;	bEv1 = 0;	bEv2 = 0;
			break;
		default :
			MaSndDrv_SetCommand(sdSeqID, 0, MASNDDRV_CMD_NOP, 0, 0, 0);
			bPhase = 2;	bEv1 = 0;	bEv2 = 0;
			break;
		}
		break;
	case 0x02 :
		psCh	= &(psPlay->sChannel_Info[bEv1]);
		switch (bEv2) {
		case 0x00 :
			MaSndDrv_SetCommand(sdSeqID, 0, MASNDDRV_CMD_PROGRAM_CHANGE, bEv1, psCh->bBank, psCh->bProg);
			bEv2 ++;
			break;
		case 0x01 :
			MaSndDrv_SetCommand(sdSeqID, 0, MASNDDRV_CMD_MODULATION_DEPTH, bEv1, psCh->bMod, 0);
			bEv2 ++;
			break;
		case 0x02 :
			if (psPlay->dSmafType == MMF_SMAF_TYPE_MA5) {
				dChVol = MaSndDrv_Mix40LogVol((UINT32)psCh->bChVol, (UINT32)psPlay->bMasterVol);
				MaSndDrv_SetCommand(sdSeqID, 0, MASNDDRV_CMD_CHANNEL_VOLUME, bEv1, dChVol, 0);
			}
			else
				MaSndDrv_SetCommand(sdSeqID, 0, MASNDDRV_CMD_CHANNEL_VOLUME, bEv1, psCh->bChVol, 0);
			bEv2 ++;
			break;
		case 0x03 :
			MaSndDrv_SetCommand(sdSeqID, 0, MASNDDRV_CMD_PANPOT, bEv1, psCh->bPan, 0);
			bEv2 ++;
			break;
		case 0x04 :
			MaSndDrv_SetCommand(sdSeqID, 0, MASNDDRV_CMD_EXPRESSION, bEv1, psCh->bExp, 0);
			bEv2 ++;
			break;
		case 0x05 :
			MaSndDrv_SetCommand(sdSeqID, 0, MASNDDRV_CMD_HOLD1, bEv1, psCh->bHold, 0);
			bEv2 ++;
			break;
		case 0x06 :
			if (psCh->bMono)
				MaSndDrv_SetCommand(sdSeqID, 0, MASNDDRV_CMD_MONO_MODE_ON, bEv1, 0, 0);
			else
				MaSndDrv_SetCommand(sdSeqID, 0, MASNDDRV_CMD_POLY_MODE_ON, bEv1, 0, 0);
			bEv2 ++;
			break;
		case 0x07 :
			MaSndDrv_SetCommand(sdSeqID, 0, MASNDDRV_CMD_BEND_RANGE, bEv1, psCh->bSens2, 0);
			bEv2 ++;
			break;
		case 0x08 :
			dPitch	= (((UINT32)psCh->bPitch) << 7);
			MaSndDrv_SetCommand(sdSeqID, 0, MASNDDRV_CMD_PITCH_BEND, bEv1, dPitch, 0);
			bEv2 ++;
			break;
		case 0x09 :
			MaSndDrv_SetCommand(sdSeqID, 0, MASNDDRV_CMD_BEND_RANGE, bEv1, psCh->bSens1, 0);
			bEv2 ++;
			break;
		case 0x0A :
			MaSndDrv_SetCommand(sdSeqID, 0, MASNDDRV_CMD_SET_LED_SYNC_CH, bEv1, 1, 0);
			bEv2 ++;
			break;
		case 0x0B :
			MaSndDrv_SetCommand(sdSeqID, 0, MASNDDRV_CMD_SET_MOTOR_SYNC_CH, bEv1, 1, 0);
			if (bEv1 < MMF_CHANNEL_NUM - 1) {				bEv1 ++;	bEv2 = 0; }
			else							{	bPhase = 3;	bEv1 = 0;	bEv2 = 0; }
			break;
		default :
			MaSndDrv_SetCommand(sdSeqID, 0, MASNDDRV_CMD_NOP, 0, 0, 0);
			bPhase = 3;	bEv1 = 0;	bEv2 = 0;
			break;
		}
		break;
	case 0x03 :
		psPlay->dSeekParam = MMF_CONVERT_PHASE_PLAY;
		return MMF_SEEK_EVENT_END;
		break;
	case 0xFF :
		MaSndDrv_SetCommand(sdSeqID, 0, MASNDDRV_CMD_SYSTEM_ON, 0, 0, 0);
		bPhase = 3;	bEv1 = 0;	bEv2 = 0;
		break;
	default :
		MaSndDrv_SetCommand(sdSeqID, 0, MASNDDRV_CMD_NOP, 0, 0, 0);
		psPlay->dSeekParam = MMF_CONVERT_PHASE_PLAY;
		break;
	}

	bEnd	 = 1;
	while (bEnd) {
		switch (bPhase) {
		case 0x01 :			/* General Setting			*/
			switch (bEv1) {
			case 0x01 :		/* MA-3 Master Volume		*/
				bEnd	= 0;
				break;
			case 0x02 :		/* MA-3/5 Stream Pan		*/
				psStrm3 = gsSmaf_Info.sPlay_Info.sStream_Info.sStream_Info3;
				for ( ; bEv2 < MMF_MAX_STREAM_DATA_NUM3; bEv2++) {
					if (psStrm3[bEv2].bPan != SNDDRV_DEF_STREAM_PAN )
						break;
				}
				if (bEv2 != MMF_MAX_STREAM_DATA_NUM3) {
					bEnd	= 0;
					break;
				}
				if (psPlay->bAlChReserve != MMF_AL_CHANNEL_NULL)
					bEv1 = 3;
				else {
					bPhase = 2;	bEv1 = 0;	bEv2 = 0;
				}
				break;
			case 0x03 :		/* MA-5 Filter Resonance	*/
				if (psPlay->bResonance != SNDDRV_DEF_RESONANCE) {
					bEnd	= 0;
					break;
				}
				bEv1 = 4;
				break;
			case 0x04 :		/* MA-5 Filter Brightness	*/
				if (psPlay->bBrightness != SNDDRV_DEF_BRIGHTNESS) {
					bEnd	= 0;
					break;
				}
				bPhase = 2;	bEv1 = 0;	bEv2 = 0;
				break;
			case 0x05 :		/* MA-2 Audio Channel LED	*/
				bEnd	= 0;
				break;
			case 0x06 :		/* MA-2 Audio Channel LED	*/
				bEnd	= 0;
				break;
			default :
				bPhase = 2;	bEv1 = 0;	bEv2 = 0;
				break;
			}
			break;
		case 0x02 :			/* Channel Setting			*/
			psCh	= &(psPlay->sChannel_Info[bEv1]);
			switch (bEv2) {
			case 0x00 :		/* Bank Number				*/
				if ((psCh->bBank) || (psCh->bProg)) {
					bEnd	= 0;
					break;
				}
				bEv2 ++;
				break;
			case 0x01 :		/* Modulation Depth			*/
				if (psCh->bMod) {
					bEnd	= 0;
					break;
				}
				bEv2 ++;
				break;
			case 0x02 :		/* Channel Volume			*/
				if (psPlay->dSmafType == MMF_SMAF_TYPE_MA5)
					dChVol = MaSndDrv_Mix40LogVol((UINT32)psCh->bChVol, (UINT32)psPlay->bMasterVol);
				else
					dChVol = psCh->bChVol;

				if (dChVol != SNDDRV_DEF_CHANNEL_VOLUME) {
					bEnd	= 0;
					break;
				}
				bEv2 ++;
				break;
			case 0x03 :		/* Channel Panpot			*/
				if (psCh->bPan != SNDDRV_DEF_CHANNEL_PANPOT) {
					bEnd	= 0;
					break;
				}
				bEv2 ++;
				break;
			case 0x04 :		/* Expression				*/
				if (psCh->bExp != SNDDRV_DEF_EXPRESSION) {
					bEnd	= 0;
					break;
				}
				bEv2 ++;
				break;
			case 0x05 :		/* Hold						*/
				if (psCh->bHold) {
					bEnd	= 0;
					break;
				}
				bEv2 ++;
				break;
			case 0x06 :		/* Channel Mode [Mono/Poly]	*/
				if (psCh->bMono) {
					bEnd	= 0;
					break;
				}
				bEv2 ++;
				break;
			case 0x07 :		/* Bend Range 2				*/
				if (psCh->bSens2 != SNDDRV_DEF_BENDRANGE) {
					bEnd	= 0;
					break;
				}
				bEv2 ++;
				break;
			case 0x08 :		/* Pitch Bend				*/
				dPitch	= (((UINT32)psCh->bPitch) << 7);
				if (dPitch != SNDDRV_DEF_PITCHBEND) {
					bEnd	= 0;
					break;
				}
				bEv2 ++;
				break;
			case 0x09 :		/* Bend Range 1				*/
				if (psCh->bSens1 != SNDDRV_DEF_BENDRANGE) {
					bEnd	= 0;
					break;
				}
				bEv2 ++;
				break;
			case 0x0A :		/* LED Sync					*/
				if (psCh->fbLed) {
					bEnd	= 0;
					break;
				}
				bEv2 ++;
				break;
			case 0x0B :		/* VIB Sync					*/
				if (psCh->fbVib) {
					bEnd	= 0;
					break;
				}
				if (bEv1 < MMF_CHANNEL_NUM - 1) {				bEv1 ++;	bEv2 = 0; }
				else							{	bPhase = 3;	bEv1 = 0;	bEv2 = 0; }
				break;
			default :
				bPhase = 3;	bEv1 = 0;	bEv2 = 0;
				break;
			}
			break;
		case 0x03 :			/* End Setting				*/
			bEnd	= 0;
			break;
		default :
			bPhase = 3;
			bEnd	= 0;
			break;
		}
	}
	psPlay->dSeekParam	= (((UINT32)bPhase) << 16) + (((UINT32)bEv1) << 8) + (UINT32)bEv2;
	return 1;
}


/*********************************************************************************
 *	SetEventEos
 *
 *	Description:
 *			set eos event
 *	Argument:
 *			nothing
 *	Return:
 *			nomber of set command
 ********************************************************************************/
static SINT32			SetEventEos( void )
{
	PPLAYINFO			psPlay;
	UINT32				dDelta;

	psPlay = &(gsSmaf_Info.sPlay_Info);
	if		(psPlay->bEos == 0xFF)
		return 0;
	else if	(psPlay->bEos >= 0x11) {
		MaSndDrv_SetCommand(gsSmaf_Info.sdMmfSeqID, 0, MASNDDRV_CMD_USER_EVENT,
			MASMW_MAX_USEREVENT, 0, 0);
		psPlay->bEos	= 0xFF;
	}
	else if (psPlay->bEos == 0x00) {
		dDelta = psPlay->dStartTime + psPlay->dPlayTime - psPlay->dPastTime;
		if (psPlay->bHVChannel != MMF_HV_CHANNEL_NULL) {
			MaSndDrv_SetCommand(gsSmaf_Info.sdMmfSeqID, (dDelta * psPlay->dHWTimeBase),
				MASNDDRV_CMD_HV_EVENT, 127, 0, 0);
			psPlay->bEos	= 1;
		}
		else {
			MaSndDrv_SetCommand(gsSmaf_Info.sdMmfSeqID, (dDelta * psPlay->dHWTimeBase),
				MASNDDRV_CMD_ALL_SOUND_OFF, 0, 0, 0);
			psPlay->bEos	= 2;
		}
		psPlay->dPastTime	+= dDelta;
	}
	else {
		MaSndDrv_SetCommand(gsSmaf_Info.sdMmfSeqID, 0, MASNDDRV_CMD_ALL_SOUND_OFF,
			(psPlay->bEos - 1), 0, 0);
		psPlay->bEos		++;
	}
	return 1;
}


/*********************************************************************************
 *	SetOffEvent
 *
 *	Description:
 *			set note off event
 *	Argument:
 *			nothing
 *	Return:
 *			nomber of set command
 ********************************************************************************/
static SINT32			SetOffEvent( void )
{
	PPLAYINFO			psPlay;
	POFFBLOCK			psOff;
	UINT32				dDelta;

	psPlay	= &(gsSmaf_Info.sPlay_Info);
	psOff	= psPlay->psNextOff;

	if (psOff->dOffTime >= (psPlay->dStartTime + psPlay->dPlayTime))
		return SetEventEos();

	dDelta	= psOff->dOffTime - psPlay->dPastTime;
	MaSndDrv_SetCommand(psOff->dSeqID, (dDelta * psPlay->dHWTimeBase),
									psOff->dCmd, psOff->dCh, psOff->dKey, 0);

	psPlay->dPastTime	= psOff->dOffTime;
	psPlay->psNextOff	= (POFFBLOCK)(psOff->pvNext);
	psOff->pvNext		= (void *)(psPlay->psEmptyOff);
	psPlay->psEmptyOff	= psOff;

	return 1;
}


/*********************************************************************************
 *	NextEvent2
 *
 *	Description:
 *			set note off event
 *	Argument:
 *			nothing
 *	Return:
 *			nomber of set command
 ********************************************************************************/
static SINT32			NextEvent2( void )
{
	PPLAYINFO			psPlay;
	PTRACKINFO			psTrack;
	PEVENTBLOCK			psEv;
	UINT32				dDelta, dRead;
	UINT8*				pbBuf;

	psPlay	= &(gsSmaf_Info.sPlay_Info);
	psEv	= psPlay->psNextEvent;
	psTrack = &(gsSmaf_Info.sLoad_Info[1].sTrack_Info[psEv->dTrackNo]);

	if ((psEv->dEventTime >= (psPlay->dStartTime + psPlay->dPlayTime)) ||
		(psEv->dCmd == MMF_EVNET_EOS))
		return SetEventEos();

	psPlay->psNextEvent	= (PEVENTBLOCK)(psEv->pvNext);
	dDelta	= psEv->dEventTime - psPlay->dPastTime;
	MaSndDrv_SetCommand(psEv->dSeqID, (dDelta  * psPlay->dHWTimeBase),
						psEv->dCmd, psEv->dParam1, psEv->dParam2, psEv->dParam3);

	psPlay->dPastTime	= psEv->dEventTime;

	pbBuf				= psEv->pbSeq + psEv->dIndex;
	dDelta				= get_flex2(&(psEv->pbSeq[psEv->dIndex]), &dRead);
	psEv->dIndex		+= dRead;
	psEv->dEventTime	+= dDelta;
	if ((psEv->dEventTime	>= psTrack->dStopTick) ||
		(psEv->dIndex		>= psTrack->dMtsqSize)) {
		psEv->pvNext			= (void *)(psPlay->psEmptyEvent);
		psPlay->psEmptyEvent	= psEv;
		return 1;
	}

	if (psEv->dTrackNo == MMF_ATR_TRACK_NO)		NextEvent2_PlayA(psEv);
	else										NextEvent2_PlayS(psEv);
	SetEvBlock(psEv);

	return 1;
}


/*********************************************************************************
 *	NextEvent3
 *
 *	Description:
 *			set note off event
 *	Argument:
 *			nothing
 *	Return:
 *			nomber of set command
 ********************************************************************************/
static SINT32			NextEvent3( void )
{
	PPLAYINFO			psPlay;
	PTRACKINFO			psTrack;
	PEVENTBLOCK			psEv;
	PHUFFMANINFO		psHuf;
	UINT32				dDelta;

	psPlay	= &(gsSmaf_Info.sPlay_Info);
	psEv	= psPlay->psNextEvent;
	psTrack = &(gsSmaf_Info.sLoad_Info[1].sTrack_Info[5]);
	psHuf	= &(gsSmaf_Info.sLoad_Info[1].sHuffman_Info);

	if ((psEv->dEventTime >= (psPlay->dStartTime + psPlay->dPlayTime)) ||
		(psEv->dCmd == MMF_EVNET_EOS))
		return SetEventEos();

	psPlay->psNextEvent	= (PEVENTBLOCK)(psEv->pvNext);
	dDelta	= psEv->dEventTime - psPlay->dPastTime;
	MaSndDrv_SetCommand(psEv->dSeqID, (dDelta  * psPlay->dHWTimeBase),
						psEv->dCmd, psEv->dParam1, psEv->dParam2, psEv->dParam3);

	psPlay->dPastTime	= psEv->dEventTime;

	if ((psEv->dCmd	== MASNDDRV_CMD_STREAM_SLAVE) ||
		(psEv->dCmd	== MASNDDRV_CMD_HV_EVENT) ||
		(psHuf->dSeqSize <= psHuf->dReadSize)) {
		psEv->pvNext			= (void *)(psPlay->psEmptyEvent);
		psPlay->psEmptyEvent	= psEv;
		return 1;
	}

	dDelta				= get_flex3();
	psEv->dEventTime	+= dDelta;
	if (psEv->dEventTime	>= psTrack->dStopTick) {
		psEv->pvNext			= (void *)(psPlay->psEmptyEvent);
		psPlay->psEmptyEvent	= psEv;
		return 1;
	}

	NextEvent3_Play(psEv);
	SetEvBlock(psEv);

	return 1;
}


/*********************************************************************************
 *	NextEvent5
 *
 *	Description:
 *			set note off event
 *	Argument:
 *			nothing
 *	Return:
 *			nomber of set command
 ********************************************************************************/
static SINT32			NextEvent5( void )
{
	PPLAYINFO			psPlay;
	PTRACKINFO			psTrack;
	PEVENTBLOCK			psEv;
	PCHANNELINFO		pcCh;
	PHUFFMANINFO		psHuf;
	UINT32				dDelta, dChVol;

	psPlay	= &(gsSmaf_Info.sPlay_Info);
	psEv	= psPlay->psNextEvent;
	psTrack = &(gsSmaf_Info.sLoad_Info[1].sTrack_Info[6]);
	psHuf	= &(gsSmaf_Info.sLoad_Info[1].sHuffman_Info);

	if ((psEv->dEventTime >= (psPlay->dStartTime + psPlay->dPlayTime)) ||
		(psEv->dCmd == MMF_EVNET_EOS) ||
		(psHuf->dSeqSize <= psHuf->dReadSize))
		return SetEventEos();

	psPlay->psNextEvent	= (PEVENTBLOCK)(psEv->pvNext);
	dDelta	= psEv->dEventTime - psPlay->dPastTime;
	if (psEv->dCmd == MASNDDRV_CMD_MASTER_VOLUME) {
		pcCh	= &(psPlay->sChannel_Info[psEv->dParam1]);
		dChVol	= MaSndDrv_Mix40LogVol((UINT32)psPlay->bMasterVol, (UINT32)pcCh->bChVol);
		MaSndDrv_SetCommand(psEv->dSeqID, (dDelta  * psPlay->dHWTimeBase),
				MASNDDRV_CMD_CHANNEL_VOLUME, psEv->dParam1, dChVol, 0);
	}
	else
		MaSndDrv_SetCommand(psEv->dSeqID, (dDelta  * psPlay->dHWTimeBase),
						psEv->dCmd, psEv->dParam1, psEv->dParam2, psEv->dParam3);

	psPlay->dPastTime	= psEv->dEventTime;

	if (psEv->dCmd	== MASNDDRV_CMD_HV_EVENT) {
		psEv->pvNext			= (void *)(psPlay->psEmptyEvent);
		psPlay->psEmptyEvent	= psEv;
		return 1;
	}
	if ((psEv->dCmd	== MASNDDRV_CMD_MASTER_VOLUME) && (psEv->dParam1	< 15)) {
		Ev_Set(psEv, MASNDDRV_CMD_MASTER_VOLUME, (psEv->dParam1 + 1), 0, 0);
		SetEvBlock(psEv);
		return 1;
	}

	dDelta				= get_flex5();
	psEv->dEventTime	+= dDelta;
	if (psEv->dEventTime	>= psTrack->dStopTick) {
		psEv->pvNext			= (void *)(psPlay->psEmptyEvent);
		psPlay->psEmptyEvent	= psEv;
		return 1;
	}

	NextEvent5_Play(psEv);
	SetEvBlock(psEv);

	return 1;
}


/*********************************************************************************
 *	MaMmfCnv_Convert
 *
 *	Description:
 *			convert one event
 *	Argument:
 *			nothing
 *	Return:
 *			>=0				success(number of setcommand)
 *			< 0				error code
 ********************************************************************************/
static SINT32			MaMmfCnv_Convert( void )
{
	PPLAYINFO			psPlay;
	UINT32				dEvTime, dOffTime;

	if (gsSmaf_Info.sdMmfSeqID == MMF_SEQ_ID_NULL)	return MMF_FUNC_ERROR;

	psPlay = &(gsSmaf_Info.sPlay_Info);
	if (psPlay->dSeekParam)
		return SetSeekParameter();

	if (psPlay->bEos)
		return SetEventEos();

	if (psPlay->psNextEvent)	dEvTime	= psPlay->psNextEvent->dEventTime;
	else						dEvTime	= psPlay->dPlayTime + psPlay->dStartTime;
	if (psPlay->psNextOff)		dOffTime= psPlay->psNextOff->dOffTime;
	else						dOffTime= 0xFFFFFFFF;

	if (dOffTime <= dEvTime)	return SetOffEvent();
	else						return psPlay->pfnGetEvent();
}


/*--- MaMmfCnv    API      -----------------------------------------------------*/

/*********************************************************************************
 *	MaMmfCnv_Initialize
 *
 *	Description:
 *			SMAF Converter Initialize
 *	Argument:
 *			nothing
 *	Return:
 *			0				success
 ********************************************************************************/
SINT32					MaMmfCnv_Initialize( void )
{
	gsSmaf_Info.sdMmfSeqID	= MMF_SEQ_ID_NULL;
	gsSmaf_Info.dStatus		= MMF_STATUS_IDLE;

	gsSmaf_Info.sLoad_Info[0].pbMmmd	= NULL;
	gsSmaf_Info.sLoad_Info[0].dMmmdSize	= 0;
	gsSmaf_Info.sLoad_Info[0].dCrc		= MMF_CRC_NULL;
	gsSmaf_Info.sLoad_Info[0].dSmafType	= MMF_SMAF_TYPE_NULL;

	gsSmaf_Info.sLoad_Info[1].pbMmmd	= NULL;
	gsSmaf_Info.sLoad_Info[1].dMmmdSize	= 0;
	gsSmaf_Info.sLoad_Info[1].dCrc		= MMF_CRC_NULL;
	gsSmaf_Info.sLoad_Info[1].dSmafType	= MMF_SMAF_TYPE_NULL;

	return MMF_FUNC_SUCCESS;
}


/*********************************************************************************
 *	MaMmfCnv_End
 *
 *	Description:
 *			SMAF Converter End
 *	Argument:
 *			nothing
 *	Return:
 *			0				success
 ********************************************************************************/
SINT32					MaMmfCnv_End( void )
{
	gsSmaf_Info.sdMmfSeqID	= MMF_SEQ_ID_NULL;
	gsSmaf_Info.dStatus		= MMF_STATUS_IDLE;

	gsSmaf_Info.sLoad_Info[0].pbMmmd	= NULL;
	gsSmaf_Info.sLoad_Info[0].dMmmdSize	= 0;
	gsSmaf_Info.sLoad_Info[0].dCrc		= MMF_CRC_NULL;
	gsSmaf_Info.sLoad_Info[0].dSmafType	= MMF_SMAF_TYPE_NULL;

	gsSmaf_Info.sLoad_Info[1].pbMmmd	= NULL;
	gsSmaf_Info.sLoad_Info[1].dMmmdSize	= 0;
	gsSmaf_Info.sLoad_Info[1].dCrc		= MMF_CRC_NULL;
	gsSmaf_Info.sLoad_Info[1].dSmafType	= MMF_SMAF_TYPE_NULL;

	return MMF_FUNC_SUCCESS;
}


/*********************************************************************************
 *	MaMmfCnv_Load
 *
 *	Description:
 *			SMAF data load (error check and regist)
 *	Argument:
 *			pbFile			pointer to SMAF data
 *			dFsize			size of SMAF data
 *			dMode			load mode
 *			pFunc			ignore
 *			pvExt_args		ignore
 *	Return:
 *			0 or 1			success(file id)
 *			< 0				error code
 ********************************************************************************/
SINT32					MaMmfCnv_Load(
	UINT8*				pbFile,
	UINT32				dFSize,
	UINT32				dMode,
	SINT32				(*pFunc)(UINT8 bId),
	void*				pvExt_args
)
{
	PLOADINFO			psLoad;
	SINT32				sdResult;
	UINT8*				pbCrc;
	UINT32				dCrc;

	(void)pFunc;
	(void)pvExt_args;

	if ((pbFile == NULL) || (dMode >= 4) || (dFSize == 0)) {
		return MMF_ERR_ARGUMENT;
	}
	if ((dMode <= 1) && (gsSmaf_Info.dStatus == MMF_STATUS_LOADED)) {
		return MMF_FUNC_ERROR;
	}

	if ((dMode == 0) && (gsSmaf_Info.dStatus == MMF_STATUS_SAT_PROFILE)) {
		psLoad	= &(gsSmaf_Info.sLoad_Info[1]);
		if (psLoad->dMmmdSize <= dFSize) {
			pbCrc	= &(pbFile[psLoad->dMmmdSize -2]);
			dCrc	= (UINT32)((((UINT32)pbCrc[0]) << 8) + (UINT32)pbCrc[1]);
		}
		else {
			dCrc	= MMF_CRC_NULL;
		}
		if (dCrc == psLoad->dCrc) {
			if (Renewal_Profile(pbFile) == MMF_FUNC_SUCCESS) {
				gsSmaf_Info.dStatus = MMF_STATUS_LOADED;
				return 1;
			}
		}
	}

	sdResult = Ma_Load(pbFile, dFSize, dMode);

	if (sdResult == 1)
		gsSmaf_Info.dStatus = MMF_STATUS_LOADED;

	return sdResult;
}


/*********************************************************************************
 *	MaMmfCnv_Unload
 *
 *	Description:
 *			SMAF data unload(delete)
 *	Argument:
 *			sdFile_id		file id
 *			pvExt_args		ignore
 *	Return:
 *			0				success
 *			< 0				error code
 ********************************************************************************/
SINT32					MaMmfCnv_Unload(
	SINT32				sdFile_id,
	void*				pvExt_args
)
{
	(void)pvExt_args;

	if ((sdFile_id != 0) && (sdFile_id != 1))
		return MMF_ERR_ARGUMENT;

	Check_Initial( &(gsSmaf_Info.sLoad_Info[sdFile_id]) );

	if (sdFile_id == 1)
		gsSmaf_Info.dStatus = MMF_STATUS_IDLE;

	return MMF_FUNC_SUCCESS;
}


/*********************************************************************************
 *	MaMmfCnv_Open
 *
 *	Description:
 *			resorce(to play) acquisition
 *	Argument:
 *			sdFile_id		file id
 *			pvExt_args		ignore
 *	Return:
 *			0				success
 *			< 0				error code
 ********************************************************************************/
SINT32					MaMmfCnv_Open(
	SINT32				sdFile_id,
	void*				pvExt_args
)
{
	PLOADINFO			psLoad;
	PPLAYINFO			psPlay;
	PCHANNELINFO		psCh;
	UINT32				dResmode, dCnvmode, dStream, dAlch, dSeqvol;
	UINT32				dRamadrs, dRamsize, dTimebase, i;
	SINT32				sdResult;

	(void)pvExt_args;

	if ((sdFile_id != 1) ||
		(gsSmaf_Info.dStatus	!= MMF_STATUS_LOADED) ||
		(gsSmaf_Info.sdMmfSeqID	!= MMF_SEQ_ID_NULL))
		return MMF_ERR_ARGUMENT;

	psLoad	= &(gsSmaf_Info.sLoad_Info[1]);
	psPlay	= &(gsSmaf_Info.sPlay_Info);
	psCh	= psPlay->sChannel_Info;
	for (i = 0; i < MMF_CHANNEL_NUM+1; i++) {
		psCh[i].fbLed			= MMF_LED_SEQ_SYNC_OFF;
		psCh[i].fbVib			= MMF_VIB_SEQ_SYNC_OFF;
	}

	psPlay->bFmVoice	= MMF_FM_VOICE_MODE_4OP;
	switch (psLoad->dSmafType) {
	case MMF_SMAF_TYPE_MA1:
		dResmode	= MMF_RESOUCE_MODE_MA12;
		dCnvmode	= MMF_FM_MODE_4OP16 | MMF_P_SOUNDSET_GMX | MMF_WT_VOLUME_MA5;
		dStream		= 0;
		dAlch		= MMF_AL_CHANNEL_NULL;
		dSeqvol		= MMF_MA1_SEQ_MASTERVOLUME;
		psPlay->pfnGetByte	= NULL;
		psPlay->pfnGetEvent	= NextEvent2;
		get_LM_status2();
		break;
	case MMF_SMAF_TYPE_MA2:
		dResmode	= MMF_RESOUCE_MODE_MA12;
		dCnvmode	= MMF_FM_MODE_4OP16 | MMF_P_SOUNDSET_GMX | MMF_WT_VOLUME_MA5;
		if (psLoad->sTrack_Info[MMF_ATR_TRACK_NO].pbMtr != NULL)
			dStream		= 1;
		else
			dStream		= 0;
		dAlch		= MMF_AL_CHANNEL_NULL;
		dSeqvol		= MMF_MA2_SEQ_MASTERVOLUME;
		psPlay->pfnGetByte	= NULL;
		psPlay->pfnGetEvent	= NextEvent2;
		get_LM_status2();
		break;
	case MMF_SMAF_TYPE_MA3:
		dResmode	= MMF_RESOUCE_MODE_MA3;
		if ((psLoad->pbMmmd[MMF_POSITION_OF_CTYPE] & 0x0F) == 0x02)
			dCnvmode	= MMF_FM_MODE_4OP16 | MMF_P_SOUNDSET_GMX | MMF_WT_VOLUME_MA3;
		else {
			dCnvmode	= MMF_FM_MODE_2OP32 | MMF_P_SOUNDSET_GMX | MMF_WT_VOLUME_MA3;
			psPlay->bFmVoice	= MMF_FM_VOICE_MODE_2OP;
		}
		dStream		= Check_Stream(&(psLoad->sTrack_Info[5]), 0x06);
		dAlch		= MMF_AL_CHANNEL_NULL;
		dSeqvol		= MMF_MA3_SEQ_MASTERVOLUME;
		if (psLoad->sTrack_Info[5].pbMtr[0] == 0x01)
			psPlay->pfnGetByte	= decode_byte3;
		else
			psPlay->pfnGetByte	= get_byte3;
		psPlay->pfnGetEvent	= NextEvent3;
		get_LM_status3();
		break;
	case MMF_SMAF_TYPE_MA5:
		if ((psLoad->pbMmmd[MMF_POSITION_OF_CTYPE] & 0x0F) == 0x08) {
			dResmode	= MMF_RESOUCE_MODE_MA5_64;
			dStream		= 0;
			dAlch		= MMF_AL_CHANNEL_NULL;
		}
		else {
			dResmode	= MMF_RESOUCE_MODE_MA5;
			dStream		= Check_Stream(&(psLoad->sTrack_Info[6]), 0x07);
			dAlch		= Check_AlChannel(&(psLoad->sTrack_Info[6]), 0x07);
		}
		if ((psLoad->pbMmmd[MMF_POSITION_OF_CTYPE] & 0x0F) <= 0x05)
			dCnvmode	= MMF_FM_MODE_4OP16 | MMF_P_SOUNDSET_GMX | MMF_WT_VOLUME_MA5;
		else {
			dCnvmode	= MMF_FM_MODE_2OP32 | MMF_P_SOUNDSET_GMX | MMF_WT_VOLUME_MA5;
			psPlay->bFmVoice	= MMF_FM_VOICE_MODE_2OP;
		}
		dSeqvol		= 127;
		psPlay->pfnGetByte	= NULL;
		psPlay->pfnGetEvent	= NextEvent5;
		get_LM_status3();
		break;
	default :
		return MMF_FUNC_ERROR;
	}

	if (psLoad->dTimeBase <= 10) {
		psPlay->dHWTimeBase	= 1;
		dTimebase			= psLoad->dTimeBase;
	}
	else {
		psPlay->dHWTimeBase = psLoad->dTimeBase / 10;
		dTimebase			= 10;
	}

	psPlay->dSmafType		= psLoad->dSmafType;
	psPlay->dPlayTime		= psLoad->dPlayTime;
	psPlay->dStartTime		= psLoad->dStartTime;
	psPlay->dSeekTime		= 0;
	psPlay->dSeekError		= 0;
	psPlay->bMasterVol		= (UINT8)dSeqvol;
	psPlay->bStreamReserve	= (UINT8)dStream;
	psPlay->bAlChReserve	= (UINT8)dAlch;
	psPlay->bHVChannel		= psLoad->sHV_Info.bHvChannel;

	sdResult = MaSndDrv_Create(	MMF_SEQUENCETYPE_DERAYED, dTimebase, dResmode,
								dCnvmode, dStream, dAlch, MaMmfCnv_Convert,
								&dRamadrs, &dRamsize);

	if (sdResult < MMF_FUNC_SUCCESS)
		return sdResult;

	if ((dRamadrs % 2) && (dRamsize)) {
		dRamadrs ++;
		dRamsize --;
	}
	gsSmaf_Info.sdMmfSeqID = sdResult;
	gsSmaf_Info.sPlay_Info.dRamAdrs = dRamadrs;
	gsSmaf_Info.sPlay_Info.dRamSize = dRamsize;
	MaSndDrv_SetVolume 	(sdResult, MMF_API_MASTERVOL_DEFAULT, MMF_SETVOLUME_3RD_PARAM);
	MaSndDrv_SetCommand	(sdResult, -1, MASNDDRV_CMD_MASTER_VOLUME, dSeqvol, 0, 0);

	if (psLoad->dSmafType == MMF_SMAF_TYPE_MA5)
		MaSndDrv_SetVolShift(sdResult, 2, 0x06);

	return sdResult;
}


/*********************************************************************************
 *	MaMmfCnv_Close
 *
 *	Description:
 *			resorce(to play) opening
 *	Argument:
 *			sdFile_id		file id
 *			pvExt_args		ignore
 *	Return:
 *			0				success
 *			< 0				error code
 ********************************************************************************/
SINT32					MaMmfCnv_Close(
	SINT32				sdFile_id,
	void*				pvExt_args
)
{
	SINT32				sdResult;

	(void)pvExt_args;

	if ((sdFile_id != 1) ||
		(gsSmaf_Info.sdMmfSeqID	== MMF_SEQ_ID_NULL))
		return MMF_ERR_ARGUMENT;

	sdResult = MaSndDrv_Free(gsSmaf_Info.sdMmfSeqID);
	if (sdResult >= 0) {
		sdResult = 0;
		gsSmaf_Info.sdMmfSeqID = MMF_SEQ_ID_NULL;
	}
	return sdResult;
}


/*********************************************************************************
 *	MaMmfCnv_Standby
 *
 *	Description:
 *			voice parameter registration
 *	Argument:
 *			sdFile_id		file id
 *			pvExt_args		ignore
 *	Return:
 *			0				success
 *			< 0				error code
 ********************************************************************************/
SINT32					MaMmfCnv_Standby(
	SINT32				sdFile_id,
	void*				pvExt_args
)
{
	PPLAYINFO			psPlay;
	PSTREAMINFO2		psStrm2;
	PSTREAMINFO3		psStrm3;
	PVOICEINFO2			psVoice;
	PWAVEINFO3			psWave;
	PCHANNELINFO		psCh;
	UINT32				i;

	(void)pvExt_args;

	if ((sdFile_id != 1) ||
		(gsSmaf_Info.sdMmfSeqID	== MMF_SEQ_ID_NULL))
		return MMF_ERR_ARGUMENT;

	psPlay	= &(gsSmaf_Info.sPlay_Info);
	psStrm2	= psPlay->sStream_Info.sStream_Info2;
	psStrm3	= psPlay->sStream_Info.sStream_Info3;
	psVoice	= psPlay->sVoice_Info.sVoice_Info2;
	psWave	= psPlay->sVoice_Info.sWave_Info3;
	psCh	= psPlay->sChannel_Info;

	for (i = 0; i < MMF_MAX_STREAM_DATA_NUM2; i++) {
		psStrm2[i].bStrmID		= MMF_STREAM_ID_INI;
		psStrm2[i].pbWave		= NULL;
		psStrm2[i].dWaveSize	= 0;
	}
	for (i = 0; i < MMF_MAX_STREAM_DATA_NUM3; i++) {
		psStrm3[i].fbNote		= MMF_STREAM_ID_INI;
	}
	for (i = 0; i < MMF_MAX_VOICE_DATA_NUM2; i++) {
		psVoice[i].bBank		= MMF_MA2_VOICE_NULL;
		psVoice[i].bProg		= MMF_MA2_VOICE_NULL;
	}
	for (i = 0; i < MMF_MAX_WAVE_DATA_NUM3; i++) {
		psWave[i].dAdrs			= MMF_MA3_WAVE_NULL;
		psWave[i].dSize			= MMF_MA3_WAVE_NULL;
	}

	switch (psPlay->dSmafType) {
	case MMF_SMAF_TYPE_MA1 :
	case MMF_SMAF_TYPE_MA2 :
		return Standby2();
		break;
	case MMF_SMAF_TYPE_MA3 :
		return Standby3(MMF_SMAF_TYPE_MA3);
		break;
	case MMF_SMAF_TYPE_MA5 :
		return Standby3(MMF_SMAF_TYPE_MA5);
		break;
	default :
		return MMF_FUNC_ERROR;
	}
}


/*********************************************************************************
 *	MaMmfCnv_Seek
 *
 *	Description:
 *			seek
 *	Argument:
 *			sdFile_id		file id
 *			dPos			seek position
 *			pvExt_args		ignore
 *	Return:
 *			0				success
 *			< 0				error code
 ********************************************************************************/
SINT32					MaMmfCnv_Seek(
	SINT32				sdFile_id,
	UINT32				dPos,
	void*				pvExt_args
)
{
	PPLAYINFO			psPlay;
	PEVENTBLOCK			psEv;
	POFFBLOCK			psOff;
	PCHANNELINFO		psCh;
	PSTREAMINFO3		psStrm3;
	UINT32				dSeekTick, i;
	UINT8				bVel;

	(void)pvExt_args;

	if ((sdFile_id != 1) ||
		(gsSmaf_Info.sdMmfSeqID	== MMF_SEQ_ID_NULL))
		return MMF_ERR_ARGUMENT;

	psPlay	= &(gsSmaf_Info.sPlay_Info);
	psEv	= &(psPlay->sEvent_Block[0]);
	psOff	= &(psPlay->sNoteOff_Block[0]);
	psCh	= &(psPlay->sChannel_Info[0]);
	psStrm3	= &(psPlay->sStream_Info.sStream_Info3[0]);

	psPlay->dSeekTime	= dPos;
	psPlay->dSeekError	= 0;
	psPlay->bEos		= 0;
	psPlay->fwMono		= 0;
	psPlay->bResonance	= MMF_RESONANCE_DEF;
	psPlay->bBrightness	= MMF_BRIGHTNESS_DEF;

	if (dPos % gsSmaf_Info.sLoad_Info[1].dTimeBase)
		dSeekTick	= dPos / gsSmaf_Info.sLoad_Info[1].dTimeBase + 1;
	else
		dSeekTick	= dPos / gsSmaf_Info.sLoad_Info[1].dTimeBase;

	switch (psPlay->dSmafType) {
	case MMF_SMAF_TYPE_MA3 :
		psPlay->bMasterVol	= MMF_MA3_SEQ_MASTERVOLUME;
		bVel	= MMF_VELOCITY_DEF_MA3;
		break;
	case MMF_SMAF_TYPE_MA5 :
		psPlay->bMasterVol	= MMF_MA5_SEQ_MASTERVOLUME;
		bVel	= MMF_VELOCITY_DEF_MA5;
		break;
	default :
		bVel	= 0x7F;
		break;
	}

	psEv[0].dEventTime	= 0xFFFFFFFF;
	psEv[0].dSeqID		= gsSmaf_Info.sdMmfSeqID;
	psOff[0].dOffTime	= 0xFFFFFFFF;
	psOff[0].dSeqID		= gsSmaf_Info.sdMmfSeqID;
	for (i = 1; i < MMF_MAX_EVENT_NUM; i++) {
		psEv[i].dEventTime	= 0xFFFFFFFF;
		psEv[i].dSeqID		= gsSmaf_Info.sdMmfSeqID;
		psEv[i].pvNext		= NULL;
		psEv[i-1].pvNext	= &(psEv[i]);
	}
	for (i = 1; i < MMF_MAX_NOTE_OFF_NUM; i++) {
		psOff[i].dOffTime	= 0xFFFFFFFF;
		psOff[i].dSeqID		= gsSmaf_Info.sdMmfSeqID;
		psOff[i].pvNext		= NULL;
		psOff[i-1].pvNext	= &(psOff[i]);
	}

	psPlay->psNextEvent	= NULL;
	psPlay->psEmptyEvent= &(psEv[0]);
	psPlay->psNextOff	= NULL;
	psPlay->psEmptyOff	= &(psOff[0]);

	for (i = 0; i < MMF_CHANNEL_NUM+1; i++) {
		psCh[i].bBankM		= MMF_BANK_NUMBER_DEF;
		psCh[i].bBankL		= MMF_BANK_NUMBER_DEF;
		psCh[i].bBank		= MMF_BANK_NUMBER_DEF;
		psCh[i].bProg		= MMF_PROGRAM_NUMBER_DEF;
		psCh[i].bRpnM		= MMF_RPN_DEF;
		psCh[i].bRpnL		= MMF_RPN_DEF;
		psCh[i].bMod		= MMF_MODULATION_DEF;
		psCh[i].bChVol		= MMF_CHANNEL_VOLUME_DEF;
		psCh[i].bPan		= MMF_CHANNEL_PAN_DEF;
		psCh[i].bExp		= MMF_EXPRESSION_DEF;
		psCh[i].bHold		= MMF_HOLD_DEF;
		psCh[i].bMono		= MMF_MONO_POLY_MODE_DEF;
		psCh[i].bPitch		= MMF_PITCH_BEND_DEF;
		psCh[i].bSens1		= MMF_BEND_RANGE_DEF;
		psCh[i].bSens2		= MMF_BEND_RANGE_DEF;
		psCh[i].bOct		= MMF_OCTAVE_SHIFT_DEF;
		psCh[i].bVel		= bVel;
		psCh[i].bBlockFnum1	= MMF_BLOCK_FNUM_DEF;
		psCh[i].bBlockFnum2	= MMF_BLOCK_FNUM_DEF;
	}
	psCh[ 9].bBank	= 128;
	psCh[16].bVel	=  99;

	for (i = 0; i < MMF_MAX_STREAM_DATA_NUM3; i++) {
		psStrm3[i].bPairID	= MMF_STREAM_PAIR_DEF;
		psStrm3[i].bPan		= MMF_STREAM_PAN_DEF;
	}

	if (dPos >= (psPlay->dPlayTime * gsSmaf_Info.sLoad_Info[1].dTimeBase)) {
		psPlay->dSeekParam	= MMF_CONVERT_PHASE_PLAY_END;
		psPlay->dPastTime	= psPlay->dPlayTime + psPlay->dStartTime;
	}
	else {
		psPlay->dSeekParam	= MMF_CONVERT_PHASE_SEEK_G;
		psPlay->dPastTime	= dSeekTick + psPlay->dStartTime;
		switch (psPlay->dSmafType) {
		case MMF_SMAF_TYPE_MA1 :
		case MMF_SMAF_TYPE_MA2 :
			Seek2();
			break;
		case MMF_SMAF_TYPE_MA3 :
			Seek3();
			break;
		case MMF_SMAF_TYPE_MA5 :
			Seek5();
			break;
		}
	}

	psEv	= GetEvBlock();
	psEv->dEventTime	= psPlay->dPlayTime + psPlay->dStartTime;
	psEv->dCmd			= MMF_EVNET_EOS;
	psEv->dIndex		= 0xFFFFFFFF;
	SetEvBlock(psEv);

	return MaSndDrv_ControlSequencer(gsSmaf_Info.sdMmfSeqID, 4);
}


/*********************************************************************************
 *	MaMmfCnv_Start
 *
 *	Description:
 *			play start
 *	Argument:
 *			sdFile_id		file id
 *			pvExt_args		ignore
 *	Return:
 *			0				success
 *			< 0				error code
 ********************************************************************************/
SINT32					MaMmfCnv_Start(
	SINT32				sdFile_id,
	void*				pvExt_args
)
{
	PPLAYINFO			psPlay;
	UINT32				dTB, dError;

	(void)pvExt_args;

	if ((sdFile_id != 1) ||
		(gsSmaf_Info.sdMmfSeqID	== MMF_SEQ_ID_NULL))
		return MMF_ERR_ARGUMENT;

	psPlay	= &(gsSmaf_Info.sPlay_Info);
	dTB		=   gsSmaf_Info.sLoad_Info[1].dTimeBase;
	dError	= psPlay->dSeekTime % dTB;

	if (psPlay->bEos == 0xFF)	psPlay->bEos		= 0x11;
	if (dError)					psPlay->dSeekError	= dTB - dError;
	else						psPlay->dSeekError	= 0;

	return MaSndDrv_ControlSequencer(gsSmaf_Info.sdMmfSeqID, 1);
}


/*********************************************************************************
 *	MaMmfCnv_Stop
 *
 *	Description:
 *			play stop
 *	Argument:
 *			sdFile_id		file id
 *			pvExt_args		ignore
 *	Return:
 *			0				success
 *			< 0				error code
 ********************************************************************************/
SINT32					MaMmfCnv_Stop(
	SINT32				sdFile_id,
	void*				pvExt_args
)
{
	(void)pvExt_args;

	if ((sdFile_id != 1) ||
		(gsSmaf_Info.sdMmfSeqID	== MMF_SEQ_ID_NULL))
		return MMF_ERR_ARGUMENT;

	return MaSndDrv_ControlSequencer(gsSmaf_Info.sdMmfSeqID, 0);
}


/*********************************************************************************
 *	MaMmfCnv_Control
 *
 *	Description:
 *			play control
 *	Argument:
 *			sdFile_id		file id
 *			dCtrl_num		control #
 *			pvPrm			control parameter
 *			pvExt_args		ignore
 *	Return:
 *			>=0				success(result)
 *			< 0				error code
 ********************************************************************************/
SINT32					MaMmfCnv_Control(
	SINT32				sdFile_id,
	UINT32				dCtrl_num,
	void*				pvPrm,
	void*				pvExt_args
)
{
	PLOADINFO			psLoad;
	PPLAYINFO			psPlay;
	PCHANNELINFO		psCh;
	MALIBGETCINFO		sCI;
	SINT32				sdSeqID, sdPos, i;
	UINT8				bCh, bRet;

	(void)pvExt_args;

	if ((sdFile_id != 0) && (sdFile_id != 1))
		return MMF_ERR_ARGUMENT;

	if ( (gsSmaf_Info.sdMmfSeqID == MMF_SEQ_ID_NULL) && 
		((dCtrl_num	== MASMW_SET_VOLUME) ||
		 (dCtrl_num	== MASMW_SET_SPEED) ||
		 (dCtrl_num	== MASMW_SET_KEYCONTROL) ||
		 (dCtrl_num	== MASMW_GET_POSITION) ||
		 (dCtrl_num	== MASMW_SET_ENDPOINT) ||
		 (dCtrl_num	== MASMW_GET_LEDSTATUS) ||
		 (dCtrl_num	== MASMW_GET_VIBSTATUS) ||
		 (dCtrl_num	== MASMW_GET_HVVOICE) ||
		 (dCtrl_num	== MASMW_GET_HVSCRIPT) ||
		 (dCtrl_num	== MASMW_GET_HVCHANNELSTATUS)) )
		return MMF_ERR_ARGUMENT;

	if ( (sdFile_id == 0) &&
		((dCtrl_num	!= MASMW_GET_LENGTH) &&
		 (dCtrl_num	!= MASMW_GET_CONTENTSDATA) &&
		 (dCtrl_num	!= MASMW_GET_PHRASELIST) &&
		 (dCtrl_num	!= MASMW_GET_LOADINFO) &&
		 (dCtrl_num	!= MASMW_SET_LOADINFO) &&
		 (dCtrl_num	!= MASMW_GET_HVCHANNEL)))
		return MMF_ERR_ARGUMENT;

	sdSeqID	= gsSmaf_Info.sdMmfSeqID;
	switch (dCtrl_num) {
	case MASMW_SET_VOLUME :
		if ((*((UINT8 *)pvPrm)) > 127)
			return MMF_ERR_ARGUMENT;
		return MaSndDrv_SetVolume(sdSeqID, (*((UINT8 *)pvPrm)), MMF_SETVOLUME_3RD_PARAM);

	case MASMW_SET_SPEED :
		if (((*((UINT8 *)pvPrm)) < 70) || (130 < (*((UINT8 *)pvPrm))))
			return MMF_ERR_ARGUMENT;
		return MaSndDrv_SetSpeed(sdSeqID, *((UINT8 *)pvPrm));

	case MASMW_SET_KEYCONTROL :
		if (((*((SINT8 *)pvPrm)) < -12) || (12 < (*((SINT8 *)pvPrm))))
			return MMF_ERR_ARGUMENT;
		return MaSndDrv_SetKey(sdSeqID, *((SINT8 *)pvPrm));

	case MASMW_GET_POSITION :
		sdPos	= MaSndDrv_GetPos(sdSeqID) + gsSmaf_Info.sPlay_Info.dSeekError;
		psLoad	= &(gsSmaf_Info.sLoad_Info[1]);
		psPlay	= &(gsSmaf_Info.sPlay_Info);
		if ((UINT32)sdPos <= psPlay->dPlayTime * psLoad->dTimeBase)
			return sdPos;
		else
			return (SINT32)(psPlay->dPlayTime * psLoad->dTimeBase);

	case MASMW_GET_LENGTH :
		psLoad	= &(gsSmaf_Info.sLoad_Info[sdFile_id]);
		if ((psLoad->pbMmmd == NULL) || (psLoad->dPlayTime == 0))
			return MMF_ERR_ARGUMENT;
		else
			return (psLoad->dPlayTime * psLoad->dTimeBase);

	case MASMW_GET_PHRASELIST :
		return GetPhraseList(sdFile_id, (PMASMW_PHRASELIST)pvPrm);

	case MASMW_SET_ENDPOINT :
		psLoad	= &(gsSmaf_Info.sLoad_Info[1]);
		psPlay	= &(gsSmaf_Info.sPlay_Info);
		if (*((UINT32 *)pvPrm) > (psLoad->dPlayTime * psLoad->dTimeBase))
			return MMF_FUNC_SUCCESS;
		psPlay->dPlayTime = (*((UINT32 *)pvPrm)) / psLoad->dTimeBase;
			return MMF_FUNC_SUCCESS;

	case MASMW_GET_CONTENTSDATA :
		psLoad	= &(gsSmaf_Info.sLoad_Info[sdFile_id]);
		if ((psLoad->pbMmmd == NULL) ||
			(((PMASMW_CONTENTSINFO)pvPrm)->buf == NULL) ||
			(((PMASMW_CONTENTSINFO)pvPrm)->size == 0))	return MMF_ERR_ARGUMENT;

		sCI.pbCnti		= psLoad->sOption_Info.pbCnti;
		sCI.dCntiSize	= psLoad->sOption_Info.dCntiSize;
		sCI.pbOption	= psLoad->sOption_Info.pbOpda;
		sCI.dOptionSize	= psLoad->sOption_Info.dOpdaSize;
		sCI.bCode		= ((PMASMW_CONTENTSINFO)pvPrm)->code;
		sCI.bTag[0]		= ((PMASMW_CONTENTSINFO)pvPrm)->tag[0];
		sCI.bTag[1]		= ((PMASMW_CONTENTSINFO)pvPrm)->tag[1];
		sCI.pbBuf		= ((PMASMW_CONTENTSINFO)pvPrm)->buf;
		sCI.dBufSize	= ((PMASMW_CONTENTSINFO)pvPrm)->size;

		if		((sCI.bTag[0] == 'L') && 
				 (sCI.bTag[1] == 'D')) {			/* LED status				*/
			if (psLoad->dPlayTime == 0)			return MMF_FUNC_ERROR;
			*(sCI.pbBuf) = GetLED(sdFile_id);
			return 1;
		}
		else if	((sCI.bTag[0] == 'V') && 
				 (sCI.bTag[1] == 'B')) {			/* VIB status				*/
			if (psLoad->dPlayTime == 0)			return MMF_FUNC_ERROR;
			*(sCI.pbBuf) = GetVIB(sdFile_id);
			return 1;
		}
		else if	((sCI.bTag[0] == 'K') && 
				 (sCI.bTag[1] == 'C')) {			/* KCS status				*/
			if (psLoad->dPlayTime == 0)			return MMF_FUNC_ERROR;
			*(sCI.pbBuf) = GetKCS(sdFile_id);
			return 1;
		}
		else
			return malib_GetContentsData(&sCI);

	case MASMW_GET_LEDSTATUS :
		psCh	= &(gsSmaf_Info.sPlay_Info.sChannel_Info[0]);
		for (i = 0; i < 17; i ++) {
			if (psCh[i].fbLed == MMF_LED_SEQ_SYNC_ON)	break;
		}
		if (i == 17)
			return 0;
		return MaSndDrv_DeviceControl(MASMW_GET_LEDSTATUS, 0, 0, 0);

	case MASMW_GET_VIBSTATUS :
		psCh	= &(gsSmaf_Info.sPlay_Info.sChannel_Info[0]);
		for (i = 0; i < 17; i ++) {
			if (psCh[i].fbVib == MMF_VIB_SEQ_SYNC_ON)	break;
		}
		if (i == 17)
			return 0;
		return MaSndDrv_DeviceControl(MASMW_GET_VIBSTATUS, 0, 0, 0);

	case MASMW_GET_LOADINFO :
		return Get_LoadInfo(sdFile_id, pvPrm);
	case MASMW_SET_LOADINFO :
		return Set_LoadInfo(pvPrm);

	case MASMW_GET_HVCHANNEL :
		bCh = gsSmaf_Info.sLoad_Info[sdFile_id].sHV_Info.bHvChannel;
		if (bCh >= MMF_HV_CHANNEL_NULL)			return MMF_ERR_NODATA;
		else									return bCh;

	case MASMW_GET_HVVOICE :
		psLoad	= &(gsSmaf_Info.sLoad_Info[sdFile_id]);
		bCh		= psLoad->sHV_Info.bHvChannel;
		if (bCh >= MMF_HV_CHANNEL_NULL)			return MMF_ERR_NODATA;
		if (psLoad->sHV_Info.pbVoice == NULL)	return MMF_ERR_NODATA;
		((PMASMW_HVVOICEPARAM)pvPrm)->bProgNo	= 255;
		((PMASMW_HVVOICEPARAM)pvPrm)->pbBuf		= psLoad->sHV_Info.pbVoice;
		((PMASMW_HVVOICEPARAM)pvPrm)->dSize		= psLoad->sHV_Info.dVoiceSize;
		return MMF_FUNC_SUCCESS;

	case MASMW_GET_HVSCRIPT :
		psLoad	= &(gsSmaf_Info.sLoad_Info[sdFile_id]);
		bCh		= psLoad->sHV_Info.bHvChannel;
		if (bCh >= MMF_HV_CHANNEL_NULL)			return MMF_ERR_NODATA;
		if (psLoad->sHV_Info.pbScript == NULL)	return MMF_ERR_NODATA;
		((PMASMW_HVSCRIPT)pvPrm)->pbBuf	= psLoad->sHV_Info.pbScript;
		((PMASMW_HVSCRIPT)pvPrm)->dSize	= psLoad->sHV_Info.dScriptSize;
		return MMF_FUNC_SUCCESS;

	case MASMW_GET_HVCHANNELSTATUS :
		psLoad	= &(gsSmaf_Info.sLoad_Info[sdFile_id]);
		bCh		= psLoad->sHV_Info.bHvChannel;
		if (bCh >= MMF_HV_CHANNEL_NULL)			return MMF_ERR_NODATA;
		if (psLoad->dSmafType == MMF_SMAF_TYPE_MA3)
			bCh	= psLoad->sTrack_Info[5].pbMtr[bCh + 4];
		else
			bCh	= psLoad->sTrack_Info[6].pbMtr[bCh + 4];

		bRet	=  0;
		if  (bCh & MMF_LED_SYNC_STATUS)					bRet |= 0x01;
		if  (bCh & MMF_VIB_SYNC_STATUS)					bRet |= 0x02;
		if ((bCh & MMF_KEY_CONTROL_STATUS) != 0x40)		bRet |= 0x04;
		return bRet;

	default :
		return MMF_FUNC_ERROR;
	}
}
