/****************************************************************************
 *
 *		Copyright (C) 2002-2003	YAMAHA CORPORATION. All rights reserved.
 *
 *		Module		: masnddrv.c
 *
 *		Description	: MA Sound Driver
 *
 *		Version		: 2.7.13 	2003.08.22
 *
 ****************************************************************************/

#include "mamachdep.h"
#include "madefs.h"
#include "masnddrv.h"
#include "matable.h"
#if MASMW_SRMCNV_HVS
#include "mahvdrv.h"
#endif
#include "masnddva.h"
#include "maresmgr.h"
#include "masndseq.h"
#include "madevdrv.h"


/*--------------------------------------------------------------------------*/
/* macro definitions                                                        */
/*--------------------------------------------------------------------------*/

#define MAKE_TIMER_PART(num, delta_time)							\
{																	\
	if ( delta_time >= 0 )											\
	{																\
		pbPacketBuf[num++] = (UINT8)( 0x80 );						\
	}																\
}

#define MAKE_ADDRESS_PART(num, reg_index)							\
{																	\
	pbPacketBuf[num++] = (UINT8)(  reg_index       & 0x7F );		\
	pbPacketBuf[num++] = (UINT8)( (reg_index >> 7) | 0x80 );		\
}

#define MAKE_KEY_ON(num, voice_id, vo_volume, pitch, ch, mtr, led)	\
{																	\
	pbPacketBuf[num++] = (UINT8)( (voice_id >> 7)  & 0x7F );		\
	pbPacketBuf[num++] = (UINT8)(  voice_id        & 0x7F );		\
	pbPacketBuf[num++] = (UINT8)( (vo_volume       & 0x7C)			\
							   | ((mtr << 1)       & 0x02)			\
							   | ( led             & 0x01));		\
	pbPacketBuf[num++] = (UINT8)( (pitch >> 7)     & 0x7F );		\
	pbPacketBuf[num++] = (UINT8)(  pitch           & 0x7F );		\
	pbPacketBuf[num++] = (UINT8)( (0x40 + ch)      | 0x80 );		\
}

#define MAKE_KEY_OFF(num, ch)										\
{																	\
	pbPacketBuf[num++] = (UINT8)(  ch              | 0x80 );		\
}

#define MAKE_VEL_PITCH(num, vo_volume, pitch, mtr, led)						\
{																	\
	pbPacketBuf[num++] = (UINT8)( (vo_volume       & 0x7C)		\
							   | ((mtr << 1)       & 0x02)			\
							   | ( led             & 0x01));		\
	pbPacketBuf[num++] = (UINT8)( (pitch >> 7)     & 0x7F );		\
	pbPacketBuf[num++] = (UINT8)(  pitch           | 0x80 );		\
}

#define MAKE_PITCH(num, pitch)										\
{																	\
	pbPacketBuf[num++] = (UINT8)( (pitch >> 7)     & 0x7F );		\
	pbPacketBuf[num++] = (UINT8)(  pitch           | 0x80 );		\
}

#define MAKE_MUTE(num, ch)											\
{																	\
	pbPacketBuf[num++] = (UINT8)( (0x20 + ch)  	   | 0x80 );		\
}

#define MAKE_RST(num, ch)											\
{																	\
	pbPacketBuf[num++] = (UINT8)( (0x10 + ch)  	   | 0x80 );		\
}

#define MAKE_RST_MUTE(num, ch)										\
{																	\
	pbPacketBuf[num++] = (UINT8)( (0x30 + ch)  	   | 0x80 );		\
}

#define MAKE_CHANNEL_VOLUME(num, volume, interp)					\
{																	\
	pbPacketBuf[num++] = (UINT8)( (volume & 0x7C)					\
								| (interp & 0x01)  | 0x80 );		\
}

#define MAKE_PANPOT(num, pan, interp)								\
{																	\
	pbPacketBuf[num++] = (UINT8)( (pan & 0x7C)						\
								| (interp & 0x01)  | 0x80 );		\
}

#define MAKE_SFX(num, sfx)											\
{																	\
	pbPacketBuf[num++] = (UINT8)(  sfx             | 0x80 );		\
}

#define MAKE_STREAM_ON(num, ram_adrs, vo_volume, pitch, ch, mtr, led)	\
{																	\
	pbPacketBuf[num++] = (UINT8)( (ram_adrs >> 7)  & 0x7F );		\
	pbPacketBuf[num++] = (UINT8)(  ram_adrs        & 0x7F );		\
	pbPacketBuf[num++] = (UINT8)( (vo_volume       & 0x7C)			\
								| ((mtr << 1)      & 0x02)			\
								| ( led            & 0x01));		\
	pbPacketBuf[num++] = (UINT8)( (pitch >> 7)     & 0x7F );		\
	pbPacketBuf[num++] = (UINT8)( (pitch & 0x7F)   | 0x80 );		\
}

#define MAKE_STREAM_OFF(num, ch)	\
{																	\
	pbPacketBuf[num++] = (UINT8)(  ch | 0x30       | 0x80 );		\
}

#define MAKE_SOFTINT_RAM(num, ram_val)								\
{																	\
	pbPacketBuf[num++] = (UINT8)(  ram_val         | 0x80 );		\
}

#define MAKE_SOFTINT(num, int_val)									\
{																	\
	pbPacketBuf[num++] = (UINT8)(  int_val         | 0x80 );		\
}

#define MAKE_LED(num, led)											\
{																	\
	pbPacketBuf[num++] = (UINT8)(  led             | 0x80 );		\
}

#define MAKE_MOTOR(num, mtr)										\
{																	\
	pbPacketBuf[num++] = (UINT8)(  mtr             | 0x80 );		\
}

#define MAKE_PITCH_BEND(num, bend)									\
{																	\
	pbPacketBuf[num++] = (UINT8)( (bend >> 7)      & 0x7F );		\
	pbPacketBuf[num++] = (UINT8)(  bend            | 0x80 );		\
}

#define MAKE_NOP(num, reg_index)									\
{																	\
	pbPacketBuf[num++] = (UINT8)(  reg_index       & 0x7F );		\
	pbPacketBuf[num++] = (UINT8)( (reg_index >> 7) | 0x80 );		\
	pbPacketBuf[num++] = (UINT8)(  0x00            | 0x80 );		\
}

#define MAKE_FCOFFSET(num, lfo_rst, fc_offset)						\
{																	\
	pbPacketBuf[num++] = (UINT8)( (lfo_rst         & 0x40 )			\
								| ((fc_offset >> 7) & 0x3F ));		\
	pbPacketBuf[num++] = (UINT8)( fc_offset       | 0x80 );			\
}

#define MAKE_LPFQ(num, lpfq)										\
{																	\
	pbPacketBuf[num++] = (UINT8)(  lpfq            | 0x80 );		\
}

#define MAKE_LPFFC3(num, lpfcof)									\
{																	\
	pbPacketBuf[num++] = (UINT8)( (lpfcof >> 7)    & 0x3F );		\
	pbPacketBuf[num++] = (UINT8)( (lpfcof & 0x7F)  | 0x80 );		\
}

#define MAKE_LPFCLR_OFF(num)										\
{																	\
	pbPacketBuf[num++] = (UINT8)( 0xC0 );							\
}

#define MAKE_FIX_DISABLE(num)										\
{																	\
	pbPacketBuf[num++] = (UINT8)( 0x00 );							\
	pbPacketBuf[num++] = (UINT8)( 0x00 );							\
	pbPacketBuf[num++] = (UINT8)( 0x00 );							\
	pbPacketBuf[num++] = (UINT8)( 0x00 );							\
	pbPacketBuf[num++] = (UINT8)( 0x00 );							\
	pbPacketBuf[num++] = (UINT8)( 0x00 );							\
	pbPacketBuf[num++] = (UINT8)( 0x00 );							\
	pbPacketBuf[num++] = (UINT8)( 0x80 );							\
}


/*--------------------------------------------------------------------------*/
/*   Definitions                                                            */
/*--------------------------------------------------------------------------*/

#define MASNDDRV_DEFAULT_MASTER_VOLUME	(100)
#define MASNDDRV_DEFAULT_API_VOLUME		(100)
#define MASNDDRV_DEFAULT_API_BALANCE	(127)
#define MASNDDRV_MAI_EXT_CTRL			(0xBC)
#define MASNDDRV_DEFAULT_HV_CH_VOLUME	(100)

/* control sequener status */
#define MASNDDRV_STATUS_IDLE		(0)
#define MASNDDRV_STATUS_REGISTERED	(1)
#define MASNDDRV_STATUS_PLAYING		(2)
#define MASNDDRV_STATUS_WAITING		(3)

/* for device control */
#define MASNDDRV_LED_MODE_OFF		(0)
#define MASNDDRV_LED_MODE_DIRECT	(1)
#define MASNDDRV_LED_MODE_SYNC		(2)
#define MASNDDRV_MTR_MODE_OFF		(0)
#define MASNDDRV_MTR_MODE_DIRECT	(1)
#define MASNDDRV_MTR_MODE_SYNC		(2)
#define MASNDDRV_LED_SYNC			(1)
#define	MASNDDRV_MTR_SYNC			(2)
#define	MASNDDRV_NUM_DEVICETYPE		(3)
#define	MASNDDRV_NUM_BANK3			(38)

#define MASNDDRV_RESMODE1_NORMAL	(0)
#define MASNDDRV_RESMODE1_MA2		(1)
#define MASNDDRV_RESMODE1_MA3		(2)
#define MASNDDRV_RESMODE2_NORMAL	(0)
#define MASNDDRV_RESMODE2_MA5MONO	(1)
#define MASNDDRV_WTVOL_MODE_MA3		(0)
#define MASNDDRV_WTVOL_MODE_NORMAL	(1)

#define MASNDDRV_DIRECT_TIMERCOUNT	(20)
#define MASNDDRV_HV_TIMERCOUNT		(20)

#define MASNDDRV_FM_VOICE			(1)
#define MASNDDRV_WT_VOICE			(2)
#define MASNDDRV_ST_VOICE			(3)
#define MASNDDRV_AL_VOICE			(4)

#define MASNDDRV_FM_SLOT			(0)
#define MASNDDRV_WT_SLOT			(1)
#define MASNDDRV_ST_SLOT			(2)

#define MASNDDRV_MODE_MONO			(0)
#define MASNDDRV_MODE_POLY			(1)

/* timer */
/* 2.85742 * 7 [ms] */
#define MASNDDRV_TIMER_MST			(0x4D)
#define MASNDDRV_TIMER_COUNT		(7)

/* slot */
#define MASNDDRV_NUM_SLOT			(MA_MAX_FM_SLOT+MA_MAX_WT_SLOT+MA_MAX_STREAM_AUDIO)
#define MASNDDRV_MAX_FM_SLOT		(15)
#define MASNDDRV_MAX_FM_EXT_SLOT	(31)
#define MASNDDRV_MIN_WT_SLOT		(32)
#define MASNDDRV_MAX_WT_SLOT		(47)
#define MASNDDRV_MAX_WT_EXT_SLOT	(63)
#define MASNDDRV_MIN_STM_SLOT		(64)
#define MASNDDRV_MAX_STM_SLOT		(65)
#define MASNDDRV_MASK_FM_SLOT		(0x1F)
#define MASNDDRV_MASK_WT_SLOT		(0x1F)

/* create parameter */
#define MASNDDRV_MAX_CNVMODE		(0x7F)		/* maximum value of cnv_mode */
#define MASNDDRV_MAX_RESMODE		(5)			/* maximum value of res_mode */

/* voice */
#define MASNDDRV_MAX_REG_VOICE		(128)
#define MASNDDRV_MAX_RAM_BANK		(16)
#define MASNDDRV_NUM_VOICEINFO		(2)			/* number of voice information */

#define MASNDDRV_NUM_SETBANK		(16)
#define MASNDDRV_NUM_ALVOICE		(16)
#define MASNDDRV_MASK_ALVOICE		(0xF)
#define MASNDDRV_MASK_ALFLAG		(0x80)
#define MASNDDRV_MASK_ALNOISE		(0x02)

/* stream */
#define MASNDDRV_STMADPCM_ENDPOINT			(0x07C0)	/* steram audio ADPCM end point */
#define MASNDDRV_STMADPCM_ST_ENDPOINT		(0x03E0)	/* stream audio ADPCM stereo end point */
#define MASNDDRV_STMPCM_ENDPOINT			(0x03DF)	/* stream audio PCM end point */
#define MASNDDRV_STMPCM_ST_ENDPOINT			(0x01EF)	/* stream audio PCM stereo end point */
#define MASNDDRV_STM16PCM_ENDPOINT			(0x01EF)	/* stream audio 16bit PCM end point */
#define MASNDDRV_STM16PCM_ST_ENDPOINT		(0x00F7)	/* stream audio 16bit PCM stereo end point */

#define MASNDDRV_MAX_DB						(192)
#define MASNDDRV_FS 						(48000)
#define MASNDDRV_MIN_GML1_DRUM				(35)		/* minimum number of GM Level1 drum */
#define MASNDDRV_MAX_GML1_DRUM				(81)		/* maximum number of GM Level1 drum */

/*--- for DVA ---*/
#define MASNDDRV_MASK_MODE		(0xC0)

/*--- for HV ---*/
#define MASNDDRV_NUM_HVTYPE						(2)
#define MASNDDRV_HV_NOTE_ON						(0)
#define MASNDDRV_HV_NOTE_OFF					(1)
#define MASNDDRV_HV_SET_VOICE					(2)
#define MASNDDRV_HV_PROGRAM_CHANGE				(3)
#define MASNDDRV_HV_PITCH_BEND					(4)
#define MASNDDRV_HV_PITCH_BEND_SENS				(5)
#define MASNDDRV_HV_PROSODIC_PITCH_BEND_SENS	(6)
#define MASNDDRV_HV_FRAME_ON					(37)
#define MASNDDRV_HV_FRAME_OFF					(1)
#define MASNDDRV_HV_FRAME_NORMAL				(28)
#define MASNDDRV_HV_FRAME_CHANGE				(36)
#define MASNDDRV_HV_EXCHANNEL					(127)
#define MASNDDRV_HV_CHANNEL_NUMBER				(0)
#define MASNDDRV_HV_EXIST						(0x80)

#define MASNDDRV_DUMMY_HVDRVFUNC		\
	{									\
		dummy_mahvdrv_Initialize,		\
		dummy_mahvdrv_SetDictionary,	\
		dummy_mahvdrv_SetCommand,		\
		dummy_mahvdrv_Generate,			\
		dummy_mahvdrv_Terminate			\
	}

/*--- for AL ---*/
#define MASNDDRV_MAX_LPFQ			(0x1F)
#define MASNDDRV_MIN_LPFCOF			(0x0008)
#define MASNDDRV_MAX_LPFCOF			(0x1FF8)
#define MASNDDRV_KEY_CENTER			(60)

/* mask for setcommand */
#define MASNDDRV_MASK_CHANNEL		(0x0F)
#define MASNDDRV_MASK_KEY			(0x7F)
#define MASNDDRV_MASK_BANK			(0xFF)
#define MASNDDRV_MASK_PROGRAM		(0x7F)
#define MASNDDRV_MASK_VOLUME		(0x7F)
#define MASNDDRV_MASK_XVB			(0xF8)
#define MASNDDRV_MASK_SUS			(0xEF)
#define MASNDDRV_MASK_BEND			(0x3FFF)
#define MASNDDRV_MASK_DEPTH			(0x07)
#define MASNDDRV_MASK_WAVEID		(0x1F)
#define MASNDDRV_MASK_LPF			(0x7F)
#define MASNDDRV_MASK_HVEVENT		(0x3F)

/* for coase tune, fine tune */
#define	MASNDDRV_MASK_COASETUNE		(0x7F)
#define	MASNDDRV_MASK_FINETUNE		(0x3FFF)
#define	MASNDDRV_MAX_HW_PITCHBEND	(0x0FFF)


/*--------------------------------------------------------------------------*/
/*   Struct Definitions                                                     */
/*--------------------------------------------------------------------------*/

typedef struct _MASNDDRVCHINFO
{
	UINT8	bBankNo;					/* bank number */
	UINT8	bProgNo;					/* program number */
	UINT8	bPoly;						/* poly/mono */
	UINT8	bSfx;						/* sfx */
	UINT8	bVolume;					/* volume */
	UINT8	bExpression;				/* expression */
	UINT8	bBendRange;					/* bend range */
	UINT8	bKeyControl;				/* key control */
	UINT8	bNoteOn;					/* note on */
	UINT8	bSlotNo;					/* slot number */
	UINT8	bAlFlag;					/* al flag  bit7=1:alch, bit0-6=1:al voice number */
	UINT8	bLedSync;					/* led sync flag  1:sync */
	UINT8	bMtrSync;					/* motor sync flag 1:sync */
	UINT8	bVolInterp;					/* volume interpolation 0:off  1:on */
	UINT8	bPcFlag;					/* program change flag  0:none 1:done for AL */
	UINT32	dLpfQ;						/* AL resonance */
	UINT32	dLpfCof;					/* AL brightness */
	SINT32	sdLpfQShift;				/* AL resonance shift value */
	SINT32	sdLpfCofShift;				/* AL brightness shift value */
	UINT32	dPitchBend;					/* pitchbend value [6.10] (default:0x0400) */
	UINT32	dCoaseTune;					/* coase tune value [6.10] (default:0x0400) */
	UINT32	dFineTune;					/* fine tune value [6.10] (default:0x0400) */
	UINT32	dTunePitch;					/* (coase tune * fine tune) [6.10] (default:0x0400) */
} MASNDDRVCHINFO, *PMASNDDRVCHINFO;

typedef struct _MASNDDRVSTREAMINFO
{
	UINT8	bFormat;					/* format */
	UINT8	bPan;						/* pan */
	UINT32	dFrequency;					/* frequency */
	UINT32	dSeekP;						/* seek point (byte) */
} MASNDDRVSTMINFO, * PMASNDDRVSTMINFO;

typedef struct _MASNDDRVSLOTINFO
{
	UINT8	bCh;						/* channel number */
	UINT32	dKey;						/* key number */
} MASNDDRVSLOTINFO;

typedef struct _MASNDDRVVOICEINFO
{
	UINT16	wBankProg;					/* bank, program number */
	UINT8	bSynth;						/* synth type */
	UINT16	wKey;						/* key number */
	UINT16	wRamAdrs;					/* ram address */
} MASNDDRVVOICEINFO, * PMASNDDRVVOICEINFO;

typedef struct _MASNDDRVALVOICEINFO
{
	UINT32	dBankNo;					/* bank number */
	UINT32	dProgNo;					/* program number */
	UINT8  *pAlVoice;					/* pointer to AL voice parameter */
	UINT32	dAlVoiceSize;				/* size of AL voice parameter */
} MASNDDRVALVOICEINFO, * PMASNDDRVALVOICEINFO;

typedef struct _MASNDDRVVINFO
{
	UINT8	bSynth;						/* synth type */
	UINT16	wKey;						/* key number */
	UINT16	wAddress;					/* address */
} MASNDDRVVINFO, * PMASNDDRVVINFO;

typedef struct _MASNDDRVLEDINFO
{
	UINT32	dCtrl;						/* control 0:off 1:direct 2:sync */
	UINT32	dBlink;						/* blink */
	UINT32	dFreq;						/* frequency */
} MASNDDRVLEDINFO, *PMASNDDRVLEDINFO;

typedef struct _MASNDDRVMTRINFO
{
	UINT32	dCtrl;						/* control */
	UINT32	dBlink;						/* blink */
	UINT32	dFreq;						/* frequency */
} MASNDDRVMTRINFO, *PMASNDDRVMTRINFO;

typedef struct _MASNDDRVSEQBUFINFO
{
	UINT8	bSequenceFlag;				/* sequence flag */
	UINT8 *	pbBufPtr;					/* pointer to the buffer */
	UINT32	dBufSize;					/* size of the buffer */
	UINT8	bQueueFlag;					/* queue flag */
	UINT32	dQueueSize;					/* size of data in queue */
	UINT32	dQueueWPtr;					/* write pointer */
	UINT32	dWroteSize;					/* size of wrote data */
	UINT8	bQueue[2048];				/* queue */
} MASNDDRVSEQBUFINFO, *PMASNDDRVSEQBUFINFO;

typedef struct _MASNDDRVINFO
{
	UINT8	bStatus;					/* status */
	UINT8	bDrumMode;					/* drum mode */
	UINT8	bDvaMode;					/* dva mode */
	UINT8	bMelodyMode;				/* melody mode */
	UINT8	bDrumType;					/* drum type */
	UINT8	bWtVolMode;					/* wt volume mode */
	UINT8	bHvType;					/* hv language type */
	UINT8	bBalVolumeL;				/* balance volume L */
	UINT8	bBalVolumeR;				/* balance volume R */
	UINT8	bFmVolume;					/* fm volume */
	UINT8	bWtVolume;					/* wt volume */
	UINT8	bSaVolume;					/* sa volume */
	UINT8	bHvVolume;					/* hv volume */
	UINT8	bMasterVolShiftL;			/* shift value of master volume L */
	UINT8	bMasterVolShiftR;			/* shift value of master volume R */
	UINT8	bFmVolShift;				/* shift value of fm volume */
	UINT8	bWtVolShift;				/* shift value of wt volume */
	UINT8	bSaVolShift;				/* shift value of stream volume */
	UINT8	bHvVolShift;				/* shift value of hv volume */
	UINT8	bMasterVolume;				/* master volume */
	UINT8	bWaveSlave;					/* wave slave number */
	UINT8	bSpeed;						/* speed */
	SINT8	sbKeyOffset;				/* key offset for key control */
	UINT8	bSaLedMtrSync;				/* stream LED/Motor sync flag bit0:LED bit1:Motor 0:off 1:on */
	UINT8	bHvLedMtrSync;				/* HV LED/Motor sync flag bit0:LED bit1:Motor 0:off 1:on */
	UINT8	bHvChannel;					/* channel number for HV */
	UINT32	dTimeBase;					/* time base */
	UINT32	dChNum;						/* number of channel */
	UINT32	dFmNum;						/* number of FM slot */
	UINT32	dWtNum;						/* number of WT slot */
	UINT32	dSaNum;						/* number of stream audio slot */
	UINT32	dAlNum;						/* number of AL slot */
	UINT32	dHvNum;						/* number of HV slot */
	UINT32	dRbNum;						/* number of ram block */
	UINT32	dChMap;						/* bit mapping of channel */
	UINT32	dFmMap;						/* bit mapping of FM slot */
	UINT32	dWtMap;						/* bit mapping of WT slot */
	UINT32	dSaMap;						/* bit mapping of stream audio slot */
	UINT32	dAlMap;						/* bit mapping of AL slot */
	UINT32	dHvMap;						/* bit mapping of HV slot */
	UINT32	dRbMap;						/* bit mapping of ram block */
	UINT32	dLdMap;						/* bit mapping of led */
	UINT32	dMtMap;						/* bit mapping of motor */
	UINT32	dEosFlag;					/* end of sequence flag */
	UINT32	dMinSlot[3];				/* minimum slot number */
	UINT32	dMaxSlot[3];				/* maximum slot number */
} MASNDDRVINFO, *PMASNDDRVINFO;

#define MASNDDRV_STMSTAT_MONO0		(0x01)
#define MASNDDRV_STMSTAT_MONO1		(0x02)
#define MASNDDRV_STMSTAT_STEREO		(0x04)

typedef struct _MASNDDRVSTMCTRLINFO {
	UINT32	dQueFlag;	/* 0:not exist 1:exist */
	UINT32	dType[2];	/* 0x00(00000000):not exist */
						/* 0x01(00000001):Mono   */
						/* 0x02(00000010):Stereo */
						/* 0x80(10000000):ON bit  */
	UINT32	dRamVal[2];
	UINT32	dStreamStatus;	/* ........1 mono#0 */
							/* .......1. mono#1 */
							/* ......1.. stereo */
	UINT32	dWaveId[2];
} MASNDDRVSTMCTRLINFO, *PMASNDDRVSTMCTRLINFO;


typedef struct _MASNDDRVSTMPARAM
{
	UINT8	bPanpot;					/* panpot */
	UINT8	bStm;						/* stream */
	UINT8	bPe;						/* pan enable */
	UINT8	bPanOff;					/* pan off */
	UINT8	bMode;						/* mode */
	UINT8	bSaId;						/* stream audio id */
	UINT8	bVoVolume;					/* voice volume */
	UINT16	wPitch;						/* pitch */
	UINT16	wEndPoint;					/* end point */
} MASNDDRVSTMPARAM, *PMASNDDRVSTMPARAM;

typedef struct _MASNDDRVGROBALINFO
{
	UINT8				bCtrlSequencer[MASMW_NUM_SEQTYPE];
	UINT8				bPacketBuf[MASMW_NUM_SEQTYPE][256];
	UINT32				dHvEventInfo[MASMW_NUM_SEQTYPE];
						/* x.......:exist bit (0:not exist 1:exist) */
						/* .xxxxxxx:ram value                       */
	SINT32 				(* pfnSrmCnv[MASMW_NUM_SEQTYPE])( void );
	MASNDDRVINFO		sSndDrvInfo[MASMW_NUM_SEQTYPE];
	MASNDDRVSTMINFO		sStreamInfo[MASMW_NUM_SEQTYPE][MA_MAX_REG_STREAM_AUDIO];
	MASNDDRVVOICEINFO	sVoiceInfo[MASNDDRV_NUM_VOICEINFO][MASNDDRV_MAX_REG_VOICE*2];
	UINT8				bVoiceIndex[MASNDDRV_MAX_RAM_BANK*2*128];	/* (Bank[16+16]) * Prog   */
															/* index of sVoiceInfo */
															/* NULL if index = 0xff   */
	MASNDDRVALVOICEINFO	sAlVoiceInfo[MASNDDRV_NUM_ALVOICE];
	MASNDDRVCHINFO 		sChInfo[MASNDDRV_NUM_CHANNEL];
	MASNDDRVSLOTINFO	sSlotInfo[MASNDDRV_NUM_SLOT];
	MASNDDRVLEDINFO		sLedInfo;
	MASNDDRVMTRINFO		sMtrInfo;
	MASNDDRVSEQBUFINFO	sSeqbufInfo;
	MASNDDRVSTMCTRLINFO	sStreamCtrlInfo;
} MASNDDRVGROBALINFO, *PMASNDDRVGROBALINFO;


static MASNDDRVGROBALINFO	gMaSndDrvInfo;
static PMASNDDRVGROBALINFO	gpMaSndDrvInfo;

static SINT32 MaSndDrv_Nop					( SINT32 sid, SINT32 dt, UINT32 tp, UINT32 p2,  UINT32 p3 );
static SINT32 MaSndDrv_NoteOn				( SINT32 sid, SINT32 dt, UINT32 ch, UINT32 key, UINT32 vel );
static SINT32 MaSndDrv_NoteOnMa2			( SINT32 sid, SINT32 dt, UINT32 ch, UINT32 key, UINT32 vel );
static SINT32 MaSndDrv_NoteOnMa2Ex			( SINT32 sid, SINT32 dt, UINT32 ch, UINT32 key, UINT32 vel );
static SINT32 MaSndDrv_NoteOff				( SINT32 sid, SINT32 dt, UINT32 ch, UINT32 key, UINT32 p3 );
static SINT32 MaSndDrv_NoteOffMa2			( SINT32 sid, SINT32 dt, UINT32 ch, UINT32 key, UINT32 p3 );
static SINT32 MaSndDrv_NoteOffMa2Ex			( SINT32 sid, SINT32 dt, UINT32 ch, UINT32 key, UINT32 p3 );
static SINT32 MaSndDrv_ProgramChange		( SINT32 sid, SINT32 dt, UINT32 ch, UINT32 bkn, UINT32 prg );
static SINT32 MaSndDrv_ModulationDepth		( SINT32 sid, SINT32 dt, UINT32 ch, UINT32 dph, UINT32 p3 );
static SINT32 MaSndDrv_ChannelVolume		( SINT32 sid, SINT32 dt, UINT32 ch, UINT32 vol, UINT32 p3 );
static SINT32 MaSndDrv_Panpot				( SINT32 sid, SINT32 dt, UINT32 ch, UINT32 pan, UINT32 p3 );
static SINT32 MaSndDrv_Expression			( SINT32 sid, SINT32 dt, UINT32 ch, UINT32 vol, UINT32 p3 );
static SINT32 MaSndDrv_Hold1				( SINT32 sid, SINT32 dt, UINT32 ch, UINT32 val, UINT32 p3 );
static SINT32 MaSndDrv_AllSoundOff			( SINT32 sid, SINT32 dt, UINT32 ch, UINT32 p2,  UINT32 p3 );
static SINT32 MaSndDrv_ResetAllControllers	( SINT32 sid, SINT32 dt, UINT32 ch, UINT32 p2,  UINT32 p3 );
static SINT32 MaSndDrv_AllNoteOff			( SINT32 sid, SINT32 dt, UINT32 ch, UINT32 p2,  UINT32 p3 );
static SINT32 MaSndDrv_MonoModeOn			( SINT32 sid, SINT32 dt, UINT32 ch, UINT32 p2,  UINT32 p3 );
static SINT32 MaSndDrv_PolyModeOn			( SINT32 sid, SINT32 dt, UINT32 ch, UINT32 p2,  UINT32 p3 );
static SINT32 MaSndDrv_PitchBend			( SINT32 sid, SINT32 dt, UINT32 ch, UINT32 val, UINT32 p3 );
static SINT32 MaSndDrv_BendRange			( SINT32 sid, SINT32 dt, UINT32 ch, UINT32 rng, UINT32 p3 );
static SINT32 MaSndDrv_StreamOn				( SINT32 sid, SINT32 dt, UINT32 ch, UINT32 wid, UINT32 vel );
static SINT32 MaSndDrv_StreamOff			( SINT32 sid, SINT32 dt, UINT32 ch, UINT32 wid, UINT32 p3 );
static SINT32 MaSndDrv_StreamSlave			( SINT32 sid, SINT32 dt, UINT32 ch, UINT32 wid, UINT32 vel );
static SINT32 MaSndDrv_StreamPanpot			( SINT32 sid, SINT32 dt, UINT32 wv, UINT32 pan, UINT32 p3 );
static SINT32 MaSndDrv_MasterVolume			( SINT32 sid, SINT32 dt, UINT32 vl, UINT32 p2,  UINT32 p3 );
static SINT32 MaSndDrv_SystemOn				( SINT32 sid, SINT32 dt, UINT32 p1, UINT32 p2,  UINT32 p3 );
static SINT32 MaSndDrv_UserEvent			( SINT32 sid, SINT32 dt, UINT32 vl, UINT32 p2,  UINT32 p3 );
static SINT32 MaSndDrv_LedSyncCh			( SINT32 sid, SINT32 dt, UINT32 p1, UINT32 p2,  UINT32 p3 );
static SINT32 MaSndDrv_MotorSyncCh			( SINT32 sid, SINT32 dt, UINT32 p1, UINT32 p2,  UINT32 p3 );
static SINT32 MaSndDrv_StreamPause			( SINT32 sid, SINT32 dt, UINT32 p1, UINT32 p2,  UINT32 p3 );
static SINT32 MaSndDrv_StreamRestart		( SINT32 sid, SINT32 dt, UINT32 p1, UINT32 p2,  UINT32 p3 );
static SINT32 MaSndDrv_VolumeDirect			( SINT32 sid, SINT32 dt, UINT32 p1, UINT32 p2,  UINT32 p3 );
static SINT32 MaSndDrv_SetLpfQ				( SINT32 sid, SINT32 dt, UINT32 p1, UINT32 p2,  UINT32 p3 );
static SINT32 MaSndDrv_SetLpfCof			( SINT32 sid, SINT32 dt, UINT32 p1, UINT32 p2,  UINT32 p3 );
static SINT32 MaSndDrv_HvNoteOn				( SINT32 sid, SINT32 dt, UINT32 p1, UINT32 p2,  UINT32 p3 );
static SINT32 MaSndDrv_HvNoteOff			( SINT32 sid, SINT32 dt, UINT32 p1, UINT32 p2,  UINT32 p3 );
static SINT32 MaSndDrv_HvProgramChange		( SINT32 sid, SINT32 dt, UINT32 p1, UINT32 p2,  UINT32 p3 );
static SINT32 MaSndDrv_HvPitchBend			( SINT32 sid, SINT32 dt, UINT32 p1, UINT32 p2,  UINT32 p3 );
static SINT32 MaSndDrv_HvBendRange			( SINT32 sid, SINT32 dt, UINT32 p1, UINT32 p2,  UINT32 p3 );
static SINT32 MaSndDrv_HvProsodicBendRange	( SINT32 sid, SINT32 dt, UINT32 p1, UINT32 p2,  UINT32 p3 );
static SINT32 MaSndDrv_HvChannelVolume		( SINT32 sid, SINT32 dt, UINT32 p1, UINT32 p2,  UINT32 p3 );
static SINT32 MaSndDrv_HvPanpot				( SINT32 sid, SINT32 dt, UINT32 p1, UINT32 p2,  UINT32 p3 );
static SINT32 MaSndDrv_HvAllSoundOff		( SINT32 sid, SINT32 dt, UINT32 p1, UINT32 p2,  UINT32 p3 );
static SINT32 MaSndDrv_StreamSeek			( SINT32 sid, SINT32 dt, UINT32 p1, UINT32 p2,  UINT32 p3 );
static SINT32 MaSndDrv_HvEvent				( SINT32 sid, SINT32 dt, UINT32 p1, UINT32 p2,  UINT32 p3 );
static SINT32 MaSndDrv_CoaseTune			( SINT32 sid, SINT32 dt, UINT32 p1, UINT32 p2,  UINT32 p3 );
static SINT32 MaSndDrv_FineTune				( SINT32 sid, SINT32 dt, UINT32 p1, UINT32 p2,  UINT32 p3 );

static SINT32 dummy_mahvdrv_Initialize		( UINT32 dTimeBase );
static SINT32 dummy_mahvdrv_SetDictionary	( UINT8 *pbDict, UINT32 dSize, UINT32 dMode, void *pvExtArgs );
static SINT32 dummy_mahvdrv_SetCommand		( UINT32 dCmd, UINT32 dParam1, UINT32 dParam2 );
static SINT32 dummy_mahvdrv_Generate		( UINT8 *pbPacket );
static SINT32 dummy_mahvdrv_Terminate		( void );

static SINT32 (* ma_snddrv_command[])( SINT32 sid, SINT32 dt, UINT32 p1, UINT32 p2, UINT32 p3 ) =
{
	MaSndDrv_NoteOn,						/*  0 */
	MaSndDrv_NoteOnMa2,						/*  1 */
	MaSndDrv_NoteOnMa2Ex,					/*  2 */
	MaSndDrv_NoteOff,						/*  3 */
	MaSndDrv_NoteOffMa2,					/*  4 */
	MaSndDrv_NoteOffMa2Ex,					/*  5 */
	MaSndDrv_ProgramChange,					/*  6 */
	MaSndDrv_ModulationDepth,				/*  7 */
	MaSndDrv_ChannelVolume,					/*  8 */
	MaSndDrv_Panpot,						/*  9 */
	MaSndDrv_Expression,					/* 10 */
	MaSndDrv_Hold1,							/* 11 */
	MaSndDrv_AllSoundOff,					/* 12 */
	MaSndDrv_ResetAllControllers,			/* 13 */
	MaSndDrv_AllNoteOff,					/* 14 */
	MaSndDrv_MonoModeOn,					/* 15 */
	MaSndDrv_PolyModeOn,					/* 16 */
	MaSndDrv_PitchBend,						/* 17 */
	MaSndDrv_BendRange,						/* 18 */
	MaSndDrv_StreamOn,						/* 19 */
	MaSndDrv_StreamOff,						/* 20 */
	MaSndDrv_StreamSlave,					/* 21 */
	MaSndDrv_StreamPanpot,					/* 22 */
	MaSndDrv_MasterVolume,					/* 23 */
	MaSndDrv_SystemOn,						/* 24 */
	MaSndDrv_Nop,							/* 25 */
	MaSndDrv_Nop,							/* 26 */
	MaSndDrv_Nop,							/* 27 */
	MaSndDrv_Nop,							/* 28 */
	MaSndDrv_UserEvent,						/* 29 */
	MaSndDrv_Nop,							/* 30 */
	MaSndDrv_LedSyncCh,						/* 31 */
	MaSndDrv_MotorSyncCh,					/* 32 */
	MaSndDrv_StreamPause,					/* 33 */
	MaSndDrv_StreamRestart,					/* 34 */
	MaSndDrv_VolumeDirect,					/* 35 */
	MaSndDrv_SetLpfQ,						/* 36 */
	MaSndDrv_SetLpfCof,						/* 37 */
	MaSndDrv_Nop,							/* 38 */
	MaSndDrv_Nop,							/* 39 */
	MaSndDrv_HvNoteOn,						/* 40 */
	MaSndDrv_HvNoteOff,						/* 41 */
	MaSndDrv_HvProgramChange,				/* 42 */
	MaSndDrv_HvPitchBend,					/* 43 */
	MaSndDrv_HvBendRange,					/* 44 */
	MaSndDrv_HvProsodicBendRange,			/* 45 */
	MaSndDrv_Nop,							/* 46 */
	MaSndDrv_HvChannelVolume,				/* 47 */
	MaSndDrv_HvPanpot,						/* 48 */
	MaSndDrv_Nop,							/* 49 */
	MaSndDrv_HvAllSoundOff,					/* 50 */
	MaSndDrv_StreamSeek,					/* 51 */
	MaSndDrv_HvEvent,						/* 52 */
	MaSndDrv_CoaseTune,						/* 53 */
	MaSndDrv_FineTune						/* 54 */
};


/*** Global value for HV ***********************************************************/
typedef struct _MAHVDRVFUNC
{
	SINT32	(* Initialize)		( UINT32 dTimeBase );
	SINT32	(* SetDictionary)	( UINT8 *pbDict, UINT32 dSize, UINT32 dMode, void *pvExtArgs );
	SINT32	(* SetCommand)		( UINT32 dCmd, UINT32 dParam1, UINT32 dParam2 );
	SINT32	(* Generate)		( UINT8 *pbPacket );
	SINT32	(* Terminate)		( void );
} MAHVDRVFUNC, *PMAHVDRVFUNC;

static MAHVDRVFUNC pfnHvFunc[MASNDDRV_NUM_HVTYPE] =
{
#if MASMW_SRMCNV_HVS
	{								/* 0 */
		MaHvDrvJP_Initialize,
		MaHvDrvJP_SetDictionary,
		MaHvDrvJP_SetCommand,
		MaHvDrvJP_Generate,
		MaHvDrvJP_Terminate
	},
#else
	MASNDDRV_DUMMY_HVDRVFUNC,
#endif
	MASNDDRV_DUMMY_HVDRVFUNC		/* 1 */
};


/************************************************************************************/

/*--- for DeviceControl */
#if (MA_DEVICETYPE != MASMW_DEVICETYPE_MA5)
static const UINT8 bImRegJudgeW[MASNDDRV_NUM_DEVICETYPE-1][MASNDDRV_NUM_BANK3] =
{
	{
		/* MA-5S */
		0, 0, 0, 0, 0, 1, 1, 1, 1, 1,
		1, 1, 1, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0
	},
	
	{
		/* MA-5i */
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		1, 1, 1, 0, 0, 0, 1, 1, 1, 1,
		1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
		1, 1, 1, 1, 1, 1, 0, 0
	}
};

static const UINT8 bImRegJudgeR[MASNDDRV_NUM_DEVICETYPE-1][MASNDDRV_NUM_BANK3] =
{
	{
		/* MA-5S */
		0, 0, 0, 0, 0, 0, 0, 0, 1, 1,
		1, 1, 1, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0
	},
	
	{
		/* MA-5i */
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		1, 1, 1, 0, 0, 0, 1, 1, 1, 1,
		1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
		1, 1, 1, 1, 1, 1, 0, 1
	}
};
#endif

/*--- for SetCommand */
static const UINT8 bCmdJudge[MASMW_NUM_SEQTYPE][MASNDDRV_NUM_COMMAND] = 
{
	{
		1, 1, 1, 1, 1, 1, 1, 1, 1, 1,	/* note on - panpot */
		1, 1, 1, 1, 1, 1, 1, 1, 1, 1,	/* expression - stream on */
		1, 1, 1, 1, 1, 0, 0, 0, 0, 1,	/* stream off - user event */
		1, 1, 1, 0, 0, 1, 1, 1, 0, 0,	/* nop - hv reset wait */
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0,	/* hv note on - hv key control */
		0, 0, 1, 1, 1					/* hv all sound off - fine tune */
	},

	{
		1, 1, 1, 1, 1, 1, 1, 1, 1, 1,	/* noteon - panpot */
		1, 1, 1, 1, 1, 1, 1, 1, 1, 1,	/* expression - streamon */
		1, 1, 1, 1, 1, 0, 0, 0, 0, 0,	/* stream off - user event */
		1, 1, 1, 1, 1, 1, 1, 1, 0, 0,	/* nop - hv reset wait */
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0,	/* hv note on - hv key control */
		0, 1, 1, 1, 1					/* hv all sound off - fine tune */
	},

	{
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0,	/* noteon - panpot */
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0,	/* expression - streamon */
		0, 0, 0, 0, 1, 0, 0, 0, 0, 0,	/* stream off - user event */
		0, 1, 1, 0, 0, 0, 0, 0, 0, 0,	/* nop - hv reset wait */
		1, 1, 1, 1, 1, 1, 0, 1, 1, 0,	/* hv note on - hv key control */
		1, 0, 0, 0, 0					/* hv all sound off - fine tune */
	}
};

static const UINT8 bChannelCmd[MASNDDRV_NUM_COMMAND] = 
{
	1, 1, 1, 1, 1, 1, 1, 1, 1, 1,	/* noteon - panpot */
	1, 1, 1, 1, 1, 1, 1, 1, 1, 1,	/* expression - streamon */
	1, 1, 0, 0, 0, 0, 0, 0, 0, 0,	/* stream off - user event */
	0, 0, 0, 0, 0, 1, 1, 1, 0, 0,	/* nop - hv reset wait */
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,	/* hv note on - hv key control */
	0, 0, 0, 1, 1					/* hv all sound off - fine tune */
};


extern void machdep_memcpy( UINT8 *d, UINT8 *s, UINT32 size );
extern void machdep_EnterCriticalSection(void);
extern void machdep_LeaveCriticalSection(void);


/*--------------------------------------------------------------------------*
	Dummy Functions
 *--------------------------------------------------------------------------*/
static	SINT32	dummy_mahvdrv_Initialize( UINT32 dTimeBase )
{
	(void)dTimeBase;
	return MASMW_ERROR;
}

static	SINT32	dummy_mahvdrv_SetDictionary( UINT8 *pbDict, UINT32 dSize, UINT32 dMode, void *pvExtArgs )
{
	(void)pbDict;
	(void)dSize;
	(void)dMode;
	(void)pvExtArgs;
	return MASMW_ERROR;
}

static	SINT32	dummy_mahvdrv_SetCommand( UINT32 dCmd, UINT32 dParam1, UINT32 dParam2 )
{
	(void)dCmd;
	(void)dParam1;
	(void)dParam2;
	return MASMW_SUCCESS;
}

static	SINT32	dummy_mahvdrv_Generate( UINT8 *pbPacket )
{
	(void)pbPacket;
	return MASMW_SUCCESS;
}

static	SINT32	dummy_mahvdrv_Terminate( void )
{
	return MASMW_ERROR;
}

/*--------------------------------------------------------------------------*
	Functions for pitch
 *--------------------------------------------------------------------------*/
/****************************************************************************
 *	GetMa5FmPitch
 *
 *	Description:
 *			Calc Block&Fnum. (FM/MA-5)
 *	Argument:
 *			dBank		NoteOn Bank#				(0..255)
 *			dProg		NoteOn Prog#				(0..127)
 *			dKey		RealKey						(0..127)
 *	Return:
 *			(BLOCK << 10) + FNUM
 ****************************************************************************/
static UINT32 GetMa5FmPitch(UINT32 dBank, UINT32 dProg, UINT32 dKey)
{
	UINT32		dIndex;
	UINT32		dFnum;

	dIndex = (dBank & 0x80) + dProg;
	dFnum  = (gpdMa5FmRate[dIndex])[dKey & MASNDDRV_MASK_KEY];

	return (dFnum);
}

/****************************************************************************
 *	GetMa3WtPitch
 *
 *	Description:
 *			Calc Block&Fnum. (WT/MA-3)
 *	Argument:
 *			dFs			Base Freq[Hz]				(1..48000)
 *			dBank		NoteOn Bank#				(0..255)
 *			dProg		NoteOn Prog#				(0..127)
 *			dKey		RealKey						(0..127)
 *	Return:
 *			(BLOCK << 10) + FNUM
 ****************************************************************************/
static UINT32 GetMa3WtPitch(UINT32 dFs, UINT32 dBank, UINT32 dProg, UINT32 dKey)
{
	UINT32		dIndex;
	UINT32		dMaScale;
	UINT32		dKeyScale;
	UINT32		dBlock;
	UINT32		dFnum;

	dIndex = (dBank & 0x80) + dProg;
	dMaScale   = ((dFs * 0x0800L) / 48000L + 1L) >> 1;
	dKeyScale  = (gpdMa3WtRate[dIndex])[dKey & MASNDDRV_MASK_KEY];
	dMaScale   = (dMaScale * dKeyScale) >> 10;
	if (dMaScale < 0x0400L)
	{
		if (dMaScale > 0x0020)
		{
			dBlock = gdBlockTable[dMaScale >> 5];
			dFnum = (dMaScale << (5 - dBlock)) & 0x3FF;
		}
		else
		{
			dBlock = 0;
			dFnum = 1;
		}
	}
	else
	{
		dBlock = 5;
		dFnum = 0;
	}

	dFnum += dBlock << 10;
	if (dFnum == 0) dFnum = 1;

	return (dFnum);
}

/****************************************************************************
 *	GetMa5WtPitch
 *
 *	Description:
 *			Calc Block&Fnum. (WT/MA-5)
 *	Argument:
 *			dFs			Base Freq[Hz]				(1..48000)
 *			dBank		NoteOn Bank#				(0..255)
 *			dProg		NoteOn Prog#				(0..127)
 *			dKey		RealKey						(0..127)
 *	Return:
 *			(BLOCK << 10) + FNUM
 ****************************************************************************/
static UINT32 GetMa5WtPitch(UINT32 dFs, UINT32 dBank, UINT32 dProg, UINT32 dKey)
{
	UINT32		dIndex;
	UINT32		dMaScale;
	UINT32		dKeyScale;
	UINT32		dBlock;
	UINT32		dFnum;

	dIndex = (dBank & 0x80) + dProg;
	dMaScale   = ((dFs * 0x10000L) / 24000L + 1L) >> 1;

	dKeyScale  = (gpdMa5WtRate[dIndex])[dKey & MASNDDRV_MASK_KEY];
	dMaScale   = dMaScale * dKeyScale;
	if (dMaScale < 0x80000000L)
	{
		if (dMaScale > 0x04000000L)
		{
			dIndex = (dMaScale + (1L << 15)) >> 16;
			dBlock = gdBlockTable[dIndex >> 10];
			dFnum = (dIndex >> dBlock) & 0x3FF;
		}
		else
		{
			dBlock = 0;
			dFnum = 1;
		}
	}
	else
	{
		dBlock = 5;
		dFnum = 0;
	}

	dFnum += dBlock << 10;
	if (dFnum == 0) dFnum = 1;

	return (dFnum);
}

/*--------------------------------------------------------------------------*
	Functions
 *--------------------------------------------------------------------------*/
/****************************************************************************
 *	UpdataStreamStatus
 *
 *	Description:
 *			Update stream playing status
 *	Argument:
 *			none
 *	Return:
 *			0		success
 *			1		error
 *
 ****************************************************************************/
static SINT32 UpdateStreamStatus(void)
{
	PMASNDDRVSTMCTRLINFO	psStreamCtrlInfo;

	psStreamCtrlInfo = &gpMaSndDrvInfo->sStreamCtrlInfo;

	/* ch #1 */
	if( psStreamCtrlInfo->dType[1]!=0 )
	{
		switch( psStreamCtrlInfo->dType[1] )
		{
			/* mono OFF   */
			case 0x01:
				/* mono#1 Off */
				psStreamCtrlInfo->dStreamStatus &= ~(MASNDDRV_STMSTAT_MONO1);
				break;
			/* mono ON    */
			case 0x81:
				/* stereo Off */
				psStreamCtrlInfo->dStreamStatus &= ~(MASNDDRV_STMSTAT_STEREO);
				/* mono#1 On */
				psStreamCtrlInfo->dStreamStatus |= MASNDDRV_STMSTAT_MONO1;
				/* save waveId */
				psStreamCtrlInfo->dWaveId[1] = psStreamCtrlInfo->dRamVal[1];
				break;
		}
	}
	/* ch #0 */
	if( psStreamCtrlInfo->dType[0]!=0 )
	{
		switch( psStreamCtrlInfo->dType[0] )
		{
			/* mono OFF   */
			case 0x01:
				/* mono#0 Off */
				psStreamCtrlInfo->dStreamStatus &= ~(MASNDDRV_STMSTAT_MONO0);
				break;
			/* stereo OFF */
			case 0x02:
				/* stereo Off */
				psStreamCtrlInfo->dStreamStatus &= ~(MASNDDRV_STMSTAT_STEREO);
				break;
			/* mono ON    */
			case 0x81:
				/* stereo Off */
				psStreamCtrlInfo->dStreamStatus &= ~(MASNDDRV_STMSTAT_STEREO);
				/* mono#0 On */
				psStreamCtrlInfo->dStreamStatus |= MASNDDRV_STMSTAT_MONO0;
				/* save waveId */
				psStreamCtrlInfo->dWaveId[0] = psStreamCtrlInfo->dRamVal[0];
				break;
			/* stereo ON  */
			case 0x82:
				/* mono#0/mono#1 Off */
				psStreamCtrlInfo->dStreamStatus &= ~(MASNDDRV_STMSTAT_MONO0|MASNDDRV_STMSTAT_MONO1);
				/* stereo On */
				psStreamCtrlInfo->dStreamStatus |= MASNDDRV_STMSTAT_STEREO;
				/* save waveId */
				psStreamCtrlInfo->dWaveId[0] = psStreamCtrlInfo->dRamVal[0];
				break;
		}
	}
	return MASMW_SUCCESS;
}
/****************************************************************************
 *	SendDelayedPacket
 *
 *	Description:
 *			Send the delayed packet.
 *	Argument:
 *			pbPacket	pointer to the packet data
 *			dPacketSize	size of the packet data
 *	Return:
 *			0		success
 *			1		save to queue
 *			< 0		error code
 *
 ****************************************************************************/
static SINT32 SendDelayedPacket
(
	UINT8 *	pbPacket,					/* pointer to the packet */
	UINT32	dPacketSize					/* size of the packet */
)
{
	PMASNDDRVSEQBUFINFO	psSeqbufInfo;
	
	MASNDDRV_DBGMSG(("SendDelayedPacket: ptr=%p sz=%ld\n", pbPacket, dPacketSize));


	psSeqbufInfo = &gpMaSndDrvInfo->sSeqbufInfo;
	
	if ( ( psSeqbufInfo->dBufSize < dPacketSize ) || ( psSeqbufInfo->bQueueFlag == 1 ) )
	{
		machdep_memcpy( &(psSeqbufInfo->bQueue[psSeqbufInfo->dQueueWPtr]), pbPacket, dPacketSize );

		psSeqbufInfo->bQueueFlag  = 1;
		psSeqbufInfo->dQueueSize += dPacketSize;
		psSeqbufInfo->dQueueWPtr += dPacketSize;

		return 1;
	}

	/* write a packet to the soft buffer */
	
	if ( psSeqbufInfo->pbBufPtr == NULL ) return MASMW_ERROR;

	machdep_memcpy( psSeqbufInfo->pbBufPtr, pbPacket, dPacketSize );
	psSeqbufInfo->pbBufPtr    += dPacketSize;
	psSeqbufInfo->dBufSize   -= dPacketSize;
	psSeqbufInfo->dWroteSize += dPacketSize;

	return MASMW_SUCCESS;
}
/****************************************************************************
 *	GetVoiceInfo
 *
 *	Description:
 *			Return the voice information.
 *	Arguments:
 *			sdSeqId			sequence id (0..1)
 *			dCh				channel number (0..15)
 *			dKey			key number (0..127)
 *			psVoiceInfo		pointer to the voice information
 *	Return:
 *			voice information:
 *			  key number or base frequency (48[KHz] = 1.0[0x0400])
 *			  internal RAM address
 *			  type of synthesize (1:FM, 2:WT)
 *
 ****************************************************************************/
static SINT32 GetVoiceInfo
(
	SINT32			sdSeqId,			/* sequence id */
	UINT32 			dCh,				/* channel number */
	UINT32 			dKey,				/* key number */
	PMASNDDRVVINFO	psVoiceInfo			/* voice information */
)
{
	UINT32	dBankNo;					/* bank number */
	UINT32	dProgNo;					/* program number */
	UINT32	dBankProg;					/* bank & program number */
	UINT32	dType;						/* type */
	UINT32	i;							/* loop counter */
	SINT32	sdResult;					/* result of function */

	MASMW_ASSERT( sdSeqId <= MASMW_SEQTYPE_DIRECT );

	dBankNo = (UINT32)gpMaSndDrvInfo->sChInfo[dCh].bBankNo;
	dProgNo = (UINT32)( ( ( dBankNo < 128 ) ? gpMaSndDrvInfo->sChInfo[dCh].bProgNo : dKey ) & MASNDDRV_MASK_PROGRAM );

	if ( dBankNo < 128 )
	{
		dType = 0;
		dBankProg = (dBankNo << 7) + dProgNo;
	}
	else
	{
		dType = MASNDDRV_MAX_REG_VOICE;
		dBankProg = ((dBankNo - 128 + MASNDDRV_MAX_RAM_BANK) << 7) + dProgNo;
	}

	if ( ( dBankNo & 0x7f ) < MASNDDRV_MAX_RAM_BANK )
	{
		if ( gpMaSndDrvInfo->bVoiceIndex[dBankProg] != 0xff )
		{
			/*--- Yes, Voice Info in RAM ---------*/
			i = gpMaSndDrvInfo->bVoiceIndex[dBankProg];
			psVoiceInfo->bSynth	 = gpMaSndDrvInfo->sVoiceInfo[sdSeqId][i].bSynth;
			psVoiceInfo->wKey	 = gpMaSndDrvInfo->sVoiceInfo[sdSeqId][i].wKey;
			psVoiceInfo->wAddress = (UINT16)(gpMaSndDrvInfo->sVoiceInfo[sdSeqId][i].wRamAdrs >> 1);
			
			return MASMW_SUCCESS;
		}
	}

	if ( gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bDrumType != 0 )
	{
		if ( dBankNo >= 128 )
		{
			if ( ( dProgNo < MASNDDRV_MIN_GML1_DRUM ) || ( MASNDDRV_MAX_GML1_DRUM < dProgNo ) )
			{
				return MASMW_ERROR;
			}
		}
	}

	sdResult = MaResMgr_GetDefVoiceSynth( dType + dProgNo );
	if ( sdResult <= 0 )
		return MASMW_ERROR;
	else
		psVoiceInfo->bSynth = (UINT8)sdResult;

	if ( dType == MASNDDRV_MAX_REG_VOICE )
	{
		sdResult = MaResMgr_GetDefVoiceKey( dType + dProgNo );
		if ( sdResult <= 0 )
			return MASMW_ERROR;
		else
			psVoiceInfo->wKey = (UINT16)sdResult;
	}

	sdResult = MaResMgr_GetDefVoiceAddress( dType + dProgNo );
	if ( sdResult <= 0 )
		return MASMW_ERROR;
	else
		psVoiceInfo->wAddress = (UINT16)(sdResult >> 1);

	return MASMW_SUCCESS;
}
/****************************************************************************
 *	CalcChVolume
 *
 *	Description:
 *			Calculate the channel volume.
 *	Arguments:
 *			dCh			channnel number (0..15)
 *	Return:
 *			Channel volume value.
 *
 ****************************************************************************/
static UINT32 CalcChVolume
(
	UINT32	dCh							/* channel number */
)
{
	UINT32	dDb;
	UINT32	dVolume;

	dDb = (UINT32)db_table[gpMaSndDrvInfo->sChInfo[dCh].bVolume];
	dDb = (UINT32)( dDb + db_table[gpMaSndDrvInfo->sChInfo[dCh].bExpression] );
	if ( dDb > (UINT32)MASNDDRV_MAX_DB ) dDb = (UINT32)MASNDDRV_MAX_DB;

	dVolume = vol_table[dDb];

	return dVolume;
}
/****************************************************************************
 *	CalcVoiceVolume
 *
 *	Description:
 *			Calculate the channel volume.
 *	Arguments:
 *			sdSeqId		sequence id (0..2)
 *			dVol		volume (0..127)
 *			dOffset		offset value
 *	Return:
 *			volume value of slot (0..127).
 *
 ****************************************************************************/
static UINT32 CalcVoiceVolume
(
	SINT32	sdSeqId,					/* sequence id */
	UINT32	dVol,						/* volume value */
	UINT32	dOffset						/* offset value */
)
{
	UINT32	dDb;
	UINT32	dVolume;

	dDb = db_table[dVol];
	dDb = dDb + db_table[gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bMasterVolume];
	dDb = ( dDb >= dOffset) ? (dDb - dOffset) : 0;
	if ( dDb > MASNDDRV_MAX_DB ) dDb = MASNDDRV_MAX_DB;

	dVolume = vol_table[dDb];

	return dVolume;
}
/****************************************************************************
 *	MakeDeltaTime
 *
 *	Description:
 *			make packet of delta time part
 *	Arguments:
 *			pbPacket
 *			sdDeltaTime
 *	Return:
 *			number of packet
 *
 ****************************************************************************/
static UINT32 MakeDeltaTime( UINT8 * pbPacket, SINT32 sdDeltaTime )
{
	UINT32	dNum = 0;

	if ( sdDeltaTime < 0 ) return dNum;

	while ( ( sdDeltaTime > 127 ) && ( dNum < 2 ) )
	{
		*(pbPacket+dNum) = (UINT8)(sdDeltaTime & 0x7F);
		sdDeltaTime >>= 7;
		dNum++;
	}
	*(pbPacket+dNum) = (UINT8)(sdDeltaTime | 0x80 );

	return (UINT32)(dNum+1);
}
/****************************************************************************
 *	GetRealKeyMelody
 *
 *	Description:
 *			Get real key number of melody with key control.
 *	Arguments:
 *			sdSeqId		sequence id (0..1)
 *			dCh			channel number (0..15)
 *			dKey		key number (0..127)
 *	Return:
 *			real key number (0..127)
 *
 ****************************************************************************/
static UINT32 GetRealKeyMelody
(
	SINT32	sdSeqId,					/* sequence id */
	UINT32	dCh,						/* channel number */
	UINT32	dKey						/* key number */
)
{
	SINT32	sdRKey;						/* real key value */
	SINT32	sdKeyOffset;				/* key offset */
	UINT32	dProgNo;
	/*------------------------------------------------------------------------------------*/
	/*  KeyCon Table                                                                      */
	/*------------------------------------------------------------------------------------*/
	static const UINT8 bKeyConRate[128] =
	{
			1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,		/*   0 */
			1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,		/*  16 */
			1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,		/*  32 */
			1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,		/*  48 */
			1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,		/*  64 */
			1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,		/*  80 */
			1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,		/*  96 */
			1, 1, 1, 2, 2, 2, 2, 2, 1, 1, 5,20,10,10,20, 5,		/* 112 */
	};

	sdRKey = (SINT32)dKey;
	sdKeyOffset = (SINT32)(gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].sbKeyOffset);

	switch ( gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bMelodyMode )
	{
	case MASNDDRV_RESMODE1_NORMAL:
	case MASNDDRV_RESMODE1_MA3:

		dProgNo = gpMaSndDrvInfo->sChInfo[dCh].bProgNo;
		if (( dProgNo <= 114 ) || ( dProgNo == 120) || ( dProgNo == 121))	/* 0..114 */
		{
			if ( gpMaSndDrvInfo->sChInfo[dCh].bKeyControl != MASNDDRV_KEYCTRL_OFF )
			{
				sdRKey = sdRKey + sdKeyOffset;
				if ( sdRKey < 0   ) sdRKey = 0;
				else if ( sdRKey > 127 ) sdRKey = 127;
			}
		}
		else
		{
			if ( gpMaSndDrvInfo->sChInfo[dCh].bKeyControl == MASNDDRV_KEYCTRL_ON )
			{
				sdRKey = sdRKey + sdKeyOffset * bKeyConRate[dProgNo];
				if ( sdRKey < 0 )	sdRKey = 0;
				else if ( sdRKey > 127 )	sdRKey = 127;
			}
		}
		break;

	case MASNDDRV_RESMODE1_MA2:

		if ( gpMaSndDrvInfo->sChInfo[dCh].bKeyControl != MASNDDRV_KEYCTRL_OFF )
		{
			sdRKey = sdRKey + sdKeyOffset;
			if ( sdRKey < 0   ) sdRKey = 0;
			else if ( sdRKey > 127 ) sdRKey = 127;
		}
		break;

	default:
		break;
	}
	return (UINT32)sdRKey;
}
/****************************************************************************
 *	GetRealKeyDrum
 *
 *	Description:
 *			Get real key number of drum with key control.
 *	Arguments:
 *			sdSeqId		sequence id (0..1)
 *			dCh			channel number (0..15)
 *			dKey		key number (0..127)
 *	Return:
 *			real key number (0..127)
 *
 ****************************************************************************/
static UINT32 GetRealKeyDrum
(
	SINT32	sdSeqId,					/* sequence id */
	UINT32	dCh,						/* channel number */
	UINT32	dKey						/* key number */
)
{
	SINT32	sdRKey;						/* real key value */
	SINT32	sdKeyOffset;				/* key offset */

	sdRKey = (SINT32)dKey;
	sdKeyOffset = (SINT32)(gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].sbKeyOffset);

	if ( gpMaSndDrvInfo->sChInfo[dCh].bKeyControl == MASNDDRV_KEYCTRL_ON )
	{
		sdRKey = (SINT32)( sdRKey + sdKeyOffset );
		if ( sdRKey < 0   ) sdRKey = 0;
		else if ( sdRKey > 127 ) sdRKey = 127;
	}

	return (UINT32)sdRKey;
}
/****************************************************************************
 *	GetFmBlockFnum
 *
 *	Description:
 *			Return Block:Fnum value for FM slot.
 *	Arguments:
 *			sdSeqId		sequence id (0..1)
 *			dCh			channel number (0..15)
 *			dKey		key number (0..127,0x8000xxxx)
 *			dDrumKey	drum key number (0..127)
 *	Return:
 *			Block:Fnum value
 *
 ****************************************************************************/
static UINT32 GetFmBlockFnum
(
	SINT32	sdSeqId,					/* sequence id */
	UINT32	dCh,						/* channel number */
	UINT32	dKey,						/* key number */
	UINT32	dDrumKey					/* key number for drum */
)
{
	UINT32	dRKey;						/* real key number */
	UINT32	dBankNo;					/* bank number */
	UINT32	dProgNo;					/* program number */
	UINT32	dPitch;						/* pitch = block&fnum */

	MASMW_ASSERT( sdSeqId <= MASMW_SEQTYPE_DIRECT );
	MASMW_ASSERT( dKey <= 0x80000000 );

	dBankNo = gpMaSndDrvInfo->sChInfo[dCh].bBankNo;
	dProgNo = gpMaSndDrvInfo->sChInfo[dCh].bProgNo;

	if ( dBankNo < 128 )
	{
		dRKey = GetRealKeyMelody( sdSeqId, dCh, (UINT8)dKey );
		switch ( gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bMelodyMode )
		{
		  case MASNDDRV_RESMODE1_NORMAL:
		  case MASNDDRV_RESMODE1_MA3:
			dPitch = GetMa5FmPitch( dBankNo, dProgNo, dRKey );
			break;

		case MASNDDRV_RESMODE1_MA2:
		default:
			dPitch = GetMa5FmPitch( 128, dProgNo, dRKey );
			break;
		}
	}
	else
	{
		dRKey = GetRealKeyDrum( sdSeqId, dCh, dDrumKey );
		dPitch = GetMa5FmPitch( 128, dProgNo, dRKey );
	}

	return dPitch;
}
/****************************************************************************
 *	GetFmBlockFnumMa2
 *
 *	Description:
 *			Return MA-2 Block:Fnum value for FM slot.
 *	Arguments:
 *			sdSeqId		sequence id (0..1)
 *			dCh			channel number (0..15)
 *			dKey		key number (0x8000xxxx)
 *			dDrumKey	drum key number (0..127)
 *	Return:
 *			Block:Fnum value
 *
 ****************************************************************************/
static UINT32 GetFmBlockFnumMa2
(
	SINT32	sdSeqId,					/* sequence id */
	UINT8	dCh,						/* channel number */
	UINT32	dKey,						/* key number */
	UINT16	dDrumKey					/* key number for drum */
)
{
	UINT32	dPitch;						/* dPitch = block&fnum */
	UINT32	dBlock;						/* block number */
	UINT32	dFnum;						/* f-number */
	SINT32	sdKeyCon;

	(void)dDrumKey;

	MASMW_ASSERT( sdSeqId <= MASMW_SEQTYPE_DIRECT );
	MASMW_ASSERT( dKey >= 0x80000000 );

	/* compatible mode */

	dBlock = (dKey >> 10) & 0x0007;
	dFnum  =  dKey        & 0x03FF;

	if ( (gpMaSndDrvInfo->sChInfo[dCh].bKeyControl == MASNDDRV_KEYCTRL_ON) ||
		((gpMaSndDrvInfo->sChInfo[dCh].bKeyControl == MASNDDRV_KEYCTRL_NONE) &&
		 (gpMaSndDrvInfo->sChInfo[dCh].bBankNo <= 127))) {
		sdKeyCon = gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].sbKeyOffset + 12;
		dFnum *= ma_ma2ex_fnum[sdKeyCon];
	}
	else
		dFnum *= 0x10911;	/* 49700/48000 */
	
	dFnum >>= 16;
	while ( dFnum > 1023 )
	{
		dFnum >>= 1;
		if ( dBlock < 7 ) dBlock++;
	}

	dPitch = ((dBlock << 10) & 0x1C00)
			| ( dFnum         & 0x03FF);

	return dPitch;
}
/****************************************************************************
 *	GetWtBlockFnum
 *
 *	Description:
 *			Return Block:Fnum value for WT slot.
 *	Arguments:
 *			sdSeqId		sequence id number
 *			dCh			channel number (0..15)
 *			dKey		key number (0..127)
 *			dBaseFs		base fequency
 *	Return:
 *			>=0			Block:Fnum value
 *			< 0			error code
 *
 ****************************************************************************/
static UINT32 GetWtBlockFnum
(
	SINT32	sdSeqId,					/* sequence id number */
	UINT32	dCh,						/* channel number */
	UINT32	dKey,						/* key number */
	UINT32	dBaseFs						/* base frequency */
)
{
	UINT32	dRKey;						/* real key number */
	UINT32	dPitch;						/* dPitch = block&fnum */
	UINT32	dBankNo;					/* bank number */
	UINT32	dProgNo;					/* program number */

	MASMW_ASSERT( sdSeqId <= MASMW_SEQTYPE_DIRECT );

	if ( dKey >= 0x80000000 )
	{
		return 0;
	}

	dBankNo = gpMaSndDrvInfo->sChInfo[dCh].bBankNo;
	dProgNo = gpMaSndDrvInfo->sChInfo[dCh].bProgNo;

	if ( dBankNo < 128 )
	{

		dRKey = GetRealKeyMelody( sdSeqId, dCh, (UINT8)dKey );
		switch ( gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bMelodyMode )
		{
		  case MASNDDRV_RESMODE1_NORMAL:
			dPitch = GetMa5WtPitch( dBaseFs, dBankNo, dProgNo, dRKey );
			break;

		  case MASNDDRV_RESMODE1_MA3:
			dPitch = GetMa3WtPitch( dBaseFs, dBankNo, dProgNo, dRKey );
			break;

		  case MASNDDRV_RESMODE1_MA2:
		  default:
			dPitch = GetMa5WtPitch( dBaseFs, 128, dProgNo, dRKey );
			break;
		}
	}
	else
	{

		dRKey = GetRealKeyDrum( sdSeqId, dCh, 60 );
		switch ( gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bDrumMode )
		{
		  case MASNDDRV_RESMODE1_NORMAL:
		  case MASNDDRV_RESMODE1_MA2:
			dPitch = GetMa5WtPitch( dBaseFs, dBankNo, dProgNo, dRKey );
			break;

		  case MASNDDRV_RESMODE1_MA3:
			dPitch = GetMa3WtPitch( dBaseFs, dBankNo, dProgNo, dRKey );
			break;

		  default:
			dPitch = GetMa5WtPitch( dBaseFs, 128, dProgNo, dRKey );
			break;
		}
	}

	return dPitch;
}
/****************************************************************************
 *	MakeStreamAudioParam
 *
 *	Description:
 *			Make parameter of specified stream audio.
 *	Arguments:
 *			sdSeqId			sequence id (0..2)
 *			dCh				channel number (0..15)
 *			dWaveId			wave id number (0..63)
 *			dVelocity		velocity value (0..127)
 *			psStreamParam	stream parameter of stream audio
 *	Return:
 *			0				success
 *			< 0				error
 *
 ****************************************************************************/
static SINT32 MakeStreamAudioParam
(
	SINT32	sdSeqId,					/* sequence id */
	UINT32	dCh,						/* channel number */
	UINT32	dWaveId,					/* wave id */
	UINT32	dVelocity,					/* velocity */
	PMASNDDRVSTMPARAM	psStreamParam	/* stream audio parameter */
)
{
	UINT32	dFormat;
	UINT32	dMnSt;
	UINT32	dPanpot;
	UINT32	dSaId = 0;
	UINT32	dVoVolume = 0;
	SINT32	sdResult;
	PMASNDDRVSTMINFO	psStreamInfo;
	static UINT8 bWaveMode[8] = { 0, 0, 2, 3, 1, 0, 0, 0 };

	(void)dCh;							/* for unused warning message */

	psStreamInfo = & gpMaSndDrvInfo->sStreamInfo[sdSeqId][dWaveId];
	
	if ( psStreamInfo->dFrequency == 0 )
	{
		return MASMW_ERROR;
	}

	dFormat = (UINT32)(psStreamInfo->bFormat);
	if ( ( dFormat & 0x80 ) == 0 )
		dMnSt = 0;
	else
		dMnSt = 1;
	dPanpot = (UINT32)(psStreamInfo->bPan);

	switch ( gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bDvaMode )
	{
	  case MASNDDRV_RESMODE2_NORMAL:
		sdResult = MaDva_GetStreamSlot( dCh, dWaveId, dMnSt );
		if ( (sdResult != 0) && (sdResult != 1) )
		{
			return MASMW_ERROR;
		}
		dSaId = sdResult;
		break;

	  case MASNDDRV_RESMODE2_MA5MONO:
	  	dSaId = 0;
	  	break;
	}

	if ( gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bWtVolMode == MASNDDRV_WTVOL_MODE_MA3 )
	{
		dVoVolume = CalcVoiceVolume( sdSeqId, dVelocity, (UINT8)(12 << 1) );	/* +12dB */
	}
	else
	{
		dVoVolume = CalcVoiceVolume( sdSeqId, dVelocity, (UINT8)0 );
	}


	psStreamParam->bPanpot	= (UINT8)( (dPanpot & 0x7C) << 1 );
	psStreamParam->bStm		= (UINT8)( 1 << 1 );
	psStreamParam->bPe		= (UINT8)( ( dPanpot == 255 ) ? 0 : 1 );
	if ( ( dPanpot == 128 ) || ( (dFormat & 0x7F) == 0 ) )
		psStreamParam->bPanOff = (UINT8)(1 << 2);
	else
		psStreamParam->bPanOff = 0;
	psStreamParam->bMode		= bWaveMode[dFormat & 0x07];
	dFormat &= 0x7F;
	if ( dFormat <= 1 )	/* ADPCM */
	{
		if ( dMnSt == 0 )	/* ADPCM Mono */
			psStreamParam->wEndPoint	= MASNDDRV_STMADPCM_ENDPOINT;
		else				/* ADPCM Stereo */
			psStreamParam->wEndPoint	= MASNDDRV_STMADPCM_ST_ENDPOINT;
	}
	else if ( dFormat <= 3 )	/* 8bit PCM */
	{
		if ( dMnSt == 0 )	/* 8bit PCM Mono */
			psStreamParam->wEndPoint	= MASNDDRV_STMPCM_ENDPOINT;
		else				/* 8bit PCM Stereo */
			psStreamParam->wEndPoint	= MASNDDRV_STMPCM_ST_ENDPOINT;
	}
	else	/* 16bit PCM */
	{
		if ( dMnSt == 0 )	/* 16bit PCM Mono */
			psStreamParam->wEndPoint	= MASNDDRV_STM16PCM_ENDPOINT;
		else				/* 16bit PCM Stereo */
			psStreamParam->wEndPoint	= MASNDDRV_STM16PCM_ST_ENDPOINT;
	}
	psStreamParam->bSaId		= (UINT8)dSaId;
	psStreamParam->bVoVolume	= (UINT8)dVoVolume;
	psStreamParam->wPitch		= (UINT16)GetMa5WtPitch( psStreamInfo->dFrequency, 128, 0, 60 );

	return MASMW_SUCCESS;
}
/*--------------------------------------------------------------------------*/

/****************************************************************************
 *	MaSndDrv_UpdateSequence
 *
 *	Description:
 *			Update sequence data.
 *	Arguments:
 *			sdCtrlId	control id number
 *			pbBuf		pointer to the buffer
 *			dBufSize	size of the buffer
 *	Return:
 *			>=0			size of written data
 *			< 0			error code
 *
 ****************************************************************************/
SINT32 MaSndDrv_UpdateSequence
(
	SINT32	sdCtrlId,					/* control id */
	UINT8 *	pbBuf,						/* pointer to the buffer */
	UINT32	dBufSize					/* size of the buffer */
)
{
	UINT32	dTimeOut;					/* time out count */
	UINT32	dReadPtr;
	UINT32	dNum;						/* number of packet data */
	UINT32	dRegIndex;					/* register index number */
	SINT32	sdMessage;					/* message */
	SINT32	sdResult;					/* result of function */
	UINT8  *pbPacketBuf;
	PMASNDDRVSEQBUFINFO	psSeqbufInfo;
	static const UINT8 bStopPacket[4] = { 0x80,
										 (UINT8)(MAC_SEQ_STOP & 0x7F), 
										 (UINT8)(MAC_SEQ_STOP >> 7) | 0x80, 
										 0x81
										};

	MASNDDRV_DBGMSG(("MaSndDrv_UpdateSequence\n"));

	switch ( sdCtrlId )
	{
	  case MASNDDRV_UPDATE_FIFO: /* Software Buffer */
	  	if ( gpMaSndDrvInfo->sSndDrvInfo[MASMW_SEQTYPE_DELAYED].bStatus == MASNDDRV_STATUS_IDLE )
	  		return 0;
	  	
	  	MASMW_ASSERT( pbBuf != NULL );
	  	MASMW_ASSERT( dBufSize != 0 );
	  	
	  	psSeqbufInfo = &gpMaSndDrvInfo->sSeqbufInfo;
	  	
		psSeqbufInfo->pbBufPtr  = pbBuf;
		psSeqbufInfo->dBufSize = dBufSize;

		if ( psSeqbufInfo->bQueueFlag == 1 )
		{
			dReadPtr = psSeqbufInfo->dQueueWPtr - psSeqbufInfo->dQueueSize;

			if ( dBufSize < psSeqbufInfo->dQueueSize )
			{
				machdep_memcpy( psSeqbufInfo->pbBufPtr, &(psSeqbufInfo->bQueue[dReadPtr]), dBufSize );

				psSeqbufInfo->dQueueSize -= dBufSize;
				
				return (SINT32)dBufSize;
			}
			else
			{
				machdep_memcpy( psSeqbufInfo->pbBufPtr, &(psSeqbufInfo->bQueue[dReadPtr]), psSeqbufInfo->dQueueSize );

				psSeqbufInfo->pbBufPtr   += psSeqbufInfo->dQueueSize;
				psSeqbufInfo->dBufSize  -= psSeqbufInfo->dQueueSize;
				psSeqbufInfo->dWroteSize = psSeqbufInfo->dQueueSize;

				psSeqbufInfo->dQueueSize = 0;
				psSeqbufInfo->bQueueFlag = 0;
				psSeqbufInfo->dQueueWPtr = 0;
			}
		}
		else
		{
			psSeqbufInfo->dWroteSize = 0;
		}
		
		if ( gpMaSndDrvInfo->sSndDrvInfo[MASMW_SEQTYPE_DELAYED].dEosFlag == 1 )
		{
			if ( psSeqbufInfo->bQueueFlag == 0 )
			{
				gpMaSndDrvInfo->sSndDrvInfo[MASMW_SEQTYPE_DELAYED].dEosFlag = 0;
				sdResult = MaDevDrv_EndofSequence();
				if ( sdResult < 0 )
				{
					return MASMW_ERROR;
				}
			}
			return (SINT32)psSeqbufInfo->dWroteSize;
		}


		dTimeOut = 1024;
		do
		{
			sdResult = gpMaSndDrvInfo->pfnSrmCnv[MASMW_SEQTYPE_DELAYED]();
		}
		while ( ( sdResult > 0 ) && ( psSeqbufInfo->bQueueFlag == 0 ) && ( --dTimeOut !=0 ) );

		if ( ( sdResult == 0 ) || ( dTimeOut == 0 ) )
		{
			sdMessage = MaSound_ReceiveMessage( MASMW_SEQTYPE_DELAYED, 0, MASMW_END_OF_DATA );

			if ( sdMessage == MASMW_END_OF_SEQUENCE )
			{
				sdResult = SendDelayedPacket( (UINT8 *)&bStopPacket[0], 4 );
				if ( sdResult < 0 )
				{
					return MASMW_ERROR;
				}
				else if ( sdResult == 1 )
				{
					gpMaSndDrvInfo->sSndDrvInfo[MASMW_SEQTYPE_DELAYED].dEosFlag = 1;
				}
				else
				{
					sdResult = MaDevDrv_EndofSequence();
					if ( sdResult < 0 )
					{
						return MASMW_ERROR;
					}
				}
			}
			else
			{
				sdResult = SendDelayedPacket( (UINT8 *)&bStopPacket[0], 4 );
				if ( sdResult < 0 )
				{
					return MASMW_ERROR;
				}
			}
		}

		if ( sdResult >= 0 )
		{
			sdResult = (SINT32)psSeqbufInfo->dWroteSize;
		}

		break;

	  case MASNDDRV_UPDATE_TIMER2A: /* Timer #2A */
	  
	  	if ( gpMaSndDrvInfo->sSndDrvInfo[MASMW_SEQTYPE_DIRECT].bStatus == MASNDDRV_STATUS_PLAYING )
	  	{
			sdResult = gpMaSndDrvInfo->pfnSrmCnv[MASMW_SEQTYPE_DIRECT]();
		}
		else
		{
			sdResult = MASMW_SUCCESS;
		}

		break;

	  case MASNDDRV_UPDATE_TIMER2B: /* Timer #2B */

		if ( gpMaSndDrvInfo->sSndDrvInfo[MASMW_SEQTYPE_HV].bStatus == MASNDDRV_STATUS_PLAYING )
		{

			sdResult = gpMaSndDrvInfo->pfnSrmCnv[MASMW_SEQTYPE_HV]();

			if ( sdResult > 0 )
			{
				pbPacketBuf = gpMaSndDrvInfo->bPacketBuf[MASMW_SEQTYPE_HV];
				sdResult = pfnHvFunc[gpMaSndDrvInfo->sSndDrvInfo[MASMW_SEQTYPE_HV].bHvType].Generate( &(pbPacketBuf[2]) );
				switch ( sdResult )
				{
				  case MASNDDRV_HV_FRAME_ON:
				  	dRegIndex = MAC_HV_BASE;
				  	
				  	/* LED & Motor Sync. setting (C_Address:#532) */
				  	pbPacketBuf[34] = (UINT8)( gpMaSndDrvInfo->sSndDrvInfo[MASMW_SEQTYPE_HV].bHvLedMtrSync & 0x03 );
				  	/* set channel number */
				  	if ( gpMaSndDrvInfo->sSndDrvInfo[MASMW_SEQTYPE_HV].bHvChannel != MASNDDRV_HV_EXCHANNEL )
				  	{
						/* not ex channel */
						
						/* clear EXCH bit (C_Address:#533) */
						pbPacketBuf[35] = (UINT8)( pbPacketBuf[35] & 0x3F );
						
						/* set channel number (C_Address:#536) */
						pbPacketBuf[38] = (UINT8)( ( pbPacketBuf[38] & 0x70 )
													| gpMaSndDrvInfo->sSndDrvInfo[MASMW_SEQTYPE_HV].bHvChannel );
					}
				  	break;

				  case MASNDDRV_HV_FRAME_OFF:
				  	dRegIndex = MAC_HV_CTRL;
				  	
				  	if ( gpMaSndDrvInfo->sSndDrvInfo[MASMW_SEQTYPE_HV].bHvChannel != MASNDDRV_HV_EXCHANNEL )
				  	{
						/* not ex channel */

						/* set channel number (C_Address:#536) */
						pbPacketBuf[2] = (UINT8)( ( pbPacketBuf[2] & 0x70 )
													| gpMaSndDrvInfo->sSndDrvInfo[MASMW_SEQTYPE_HV].bHvChannel );
					}
				  	break;

				  case MASNDDRV_HV_FRAME_NORMAL:
				  	dRegIndex = MAC_HV_FORMANT_0;

				  	/* LED & Motor Sync. setting (C_Address:#532) */
				  	pbPacketBuf[26] = (UINT8)( gpMaSndDrvInfo->sSndDrvInfo[MASMW_SEQTYPE_HV].bHvLedMtrSync & 0x03 );
				  	/* set channel number */
				  	if ( gpMaSndDrvInfo->sSndDrvInfo[MASMW_SEQTYPE_HV].bHvChannel != MASNDDRV_HV_EXCHANNEL )
				  	{
						/* not ex channel */
						
						/* clear EXCH bit (C_Address:#533) */
						pbPacketBuf[27] = (UINT8)( pbPacketBuf[27] & 0x3F );
					}
				  	break;

				  case MASNDDRV_HV_FRAME_CHANGE:
				  	dRegIndex = MAC_HV_BASE;

				  	/* LED & Motor Sync. setting (C_Address:#532) */
				  	pbPacketBuf[34] = (UINT8)( gpMaSndDrvInfo->sSndDrvInfo[MASMW_SEQTYPE_HV].bHvLedMtrSync & 0x03 );
				  	/* set channel number */
				  	if ( gpMaSndDrvInfo->sSndDrvInfo[MASMW_SEQTYPE_HV].bHvChannel != MASNDDRV_HV_EXCHANNEL )
				  	{
						/* not ex channel */
						
						/* clear EXCH bit (C_Address:#533) */
						pbPacketBuf[35] = (UINT8)( pbPacketBuf[35] & 0x3F );
					}
				  	break;

				  default:
				  	return 0;
				}

				dNum = 0;
				MAKE_ADDRESS_PART( dNum, dRegIndex );
				dNum += (UINT32)sdResult;
				pbPacketBuf[dNum-1] = (UINT8)(pbPacketBuf[dNum-1] | 0x80);

				sdResult = MaDevDrv_SendDirectPacket( &(pbPacketBuf[0]), dNum );
			}
			else if ( sdResult == 0 )
			{
				return 0;
			}
		}
		else if ( gpMaSndDrvInfo->sSndDrvInfo[MASMW_SEQTYPE_HV].bStatus == MASNDDRV_STATUS_WAITING )
		{
			
			sdResult = gpMaSndDrvInfo->pfnSrmCnv[MASMW_SEQTYPE_HV]();
		}
		else
		{
			sdResult = 0;
		}

	  	break;

	  default:
	  	sdResult = 1;
	}

	return sdResult;
}
/****************************************************************************
 *	MaSndDrv_Nop
 *
 *	Description:
 *			Nop operation.
 *	Arguments:
 *			sdSeqId		sequence id (0..2)
 *			sdDeltaTime	delta time [tick]
 *			dType		NOP type (0..1)
 *			dP2			unused
 *			dP3			unused
 *	Return:
 *			0			success
 *			< 0			error code
 *
 ****************************************************************************/
static SINT32 MaSndDrv_Nop
(
	SINT32	sdSeqId,					/* sequence id */
	SINT32	sdDeltaTime,				/* delta time */
	UINT32	dType,						/* NOP type */
	UINT32	dP2,						/* unused */
	UINT32	dP3							/* unused */
)
{
	UINT32	dNum;						/* number of packet data */
	UINT32	dRegIndex;					/* register index number */
	SINT32	sdResult;					/* result of function */
	UINT8  *pbPacketBuf;
	
	(void)dP2;							/* for unused warning message */
	(void)dP3;
	
	MASNDDRV_DBGMSG((" MaSndDrv_Nop: id=%ld dt=%ld tp=%ld\n", sdSeqId, sdDeltaTime, dType));

	/* check arguments */

	MASMW_ASSERT( dType <= MASNDDRV_MAX_NOPTYPE );

	/* Make packet data */
	pbPacketBuf = gpMaSndDrvInfo->bPacketBuf[sdSeqId];
	dNum = MakeDeltaTime( &pbPacketBuf[0], sdDeltaTime );			/* timer part */

	dRegIndex = ( dType == 0 ) ? MAC_NOP : MAC_NOP2;

	pbPacketBuf[dNum++] = (UINT8)(  dRegIndex       & 0x7F );		/* address part */
	pbPacketBuf[dNum++] = (UINT8)( (dRegIndex >> 7) | 0x80 );
	pbPacketBuf[dNum++] = (UINT8)0x80;								/* data part */

	if ( sdDeltaTime < 0 )				/* direct command */
	{
		sdResult = MaDevDrv_SendDirectPacket( &(pbPacketBuf[0]), dNum );
	}
	else								/* delayed command */
	{
		sdResult = SendDelayedPacket( &(pbPacketBuf[0]), dNum );
	}

	return sdResult;
}
/****************************************************************************
 *	MaSndDrv_NoteOn
 *
 *	Description:
 *			Note On prcessing
 *	Argument:
 *			sdSeqId		sequence id (0..1)
 *			sdDeltaTime	delta time [tick]
 *			dCh			channel number (0..15)
 *			dKey		key number (0..127)
 *			dVelocity	velocity (0..127)
 *	Return:
 *			0			success
 *			< 0			error code
 *
 ****************************************************************************/
static SINT32 MaSndDrv_NoteOn
(
	SINT32	sdSeqId,					/* sequnce id */
	SINT32	sdDeltaTime,				/* delta time */
	UINT32	dCh,						/* channel number */
	UINT32	dKey,						/* key number */
	UINT32	dVelocity					/* velocity */
)
{
	UINT32	dSlotNo;					/* slot number */
	UINT32	dOldSlotNo;
	UINT32	dInfoSlotNo;
	UINT32	dOldCh;						/* old channel number */
	UINT32	dVoVol;						/* voice volume */
	UINT32	dNum;						/* number of packet data */
	UINT32	dRegIndex;					/* register index */
	UINT32	dOldRegIndex;
	UINT32	dPitch;						/* pitch */
	UINT32	dMode;						/* dva mode */
	UINT32	dParam;
	SINT32	sdResult;					/* result of function */
	SINT32	sdLpfParam;
	UINT8  *pbAlParam;					/* pointer to AL voice parameter */
	UINT8	bEvType;					/* event type bit0:1-new note 0-continue, bit1:melody, drum */
	UINT8  *pbPacketBuf;
	PMASNDDRVCHINFO	psChInfo;
	MASNDDRVVINFO	VoiceInfo;			/* voice information */
	
	
	MASNDDRV_DBGMSG((" MaSndDrv_NoteOn: id=%ld dt=%ld ch=%ld key=%ld vel=%ld\n", 
					   sdSeqId, sdDeltaTime, dCh, dKey, dVelocity));

	/* check arguments */

	MASMW_ASSERT( dKey <= MASNDDRV_MAX_KEY );
	MASMW_ASSERT( dVelocity <= MASNDDRV_MAX_VELOCITY );
	
	dKey &= MASNDDRV_MASK_KEY;
	dVelocity &= MASNDDRV_MASK_VOLUME;
	
	pbPacketBuf	= gpMaSndDrvInfo->bPacketBuf[sdSeqId];
	psChInfo 	= &gpMaSndDrvInfo->sChInfo[dCh];

	/* get voice information */

	sdResult = GetVoiceInfo( sdSeqId, dCh, dKey, &VoiceInfo );
	if ( sdResult < MASMW_SUCCESS )
	{
		sdResult = MaSndDrv_Nop( sdSeqId, sdDeltaTime, 0, 0, 0 );
		return sdResult;
	}

	/* check AL channel */
	if ( ( psChInfo->bAlFlag & MASNDDRV_MASK_ALFLAG ) == 0 )
	{
		/*** not AL channel ******************************************************/

		/* alternate assign for MA-3 compatible */

		if ( psChInfo->bBankNo >= 128 )
		{
			/* Drum */
			if ( gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bDrumMode == MASNDDRV_RESMODE1_MA3 )
			{
				sdResult = MaDva_GetAlt3Slot( dKey );
				if ( sdResult >= 0 )
				{
					MaSndDrv_NoteOff( sdSeqId, sdDeltaTime, (UINT32)(sdResult & 0x0f), 
									(UINT32)(sdResult >> 4) & 0x7F, 0 );
					if ( sdDeltaTime >= 0 )
						sdDeltaTime = 0;
				}
			}
		}

		/* get slot number */
		if ( psChInfo->bBankNo >= 128 )
		{
			dMode = 2;
		}
		else
		{
			dMode = psChInfo->bPoly;
		}
		if ( VoiceInfo.bSynth == MASNDDRV_FM_VOICE )
		{
			/* FM */
			sdResult = MaDva_GetFmSlot( dCh, dKey, dMode );
			if ( sdResult < 0 )
			{
				if ( sdDeltaTime > 0 ) {
					MaSndDrv_Nop( sdSeqId, sdDeltaTime, 0, 0, 0 );
					return sdResult;
				}
				return MASMW_SUCCESS;
			}
			dSlotNo = sdResult & MASNDDRV_MASK_FM_SLOT;
			dInfoSlotNo = dSlotNo;

			if ( dSlotNo <= MASNDDRV_MAX_FM_SLOT )
			{
				dRegIndex = (UINT32)((dSlotNo * 6) + MAC_FM_VOICE_ADDRESS);
			}
			else
			{
				dRegIndex = (UINT32)(((dSlotNo & 0x0f) * 6) + MAC_EXT_FM_VOICE_ADDRESS);
			}

			dPitch = GetFmBlockFnum( sdSeqId, (UINT8)dCh, dKey, VoiceInfo.wKey );
			dVoVol = CalcVoiceVolume( sdSeqId, dVelocity, 0 );

			dNum = MakeDeltaTime( &(pbPacketBuf[0]), sdDeltaTime );

			switch ( sdResult & MASNDDRV_MASK_MODE )
			{
			  case 0:	/* new note */

				dOldCh = (gpMaSndDrvInfo->sSlotInfo[dInfoSlotNo].bCh - 1) & 0xF;

				/* reset */
				dRegIndex += 5;
				MAKE_ADDRESS_PART( dNum, dRegIndex )
				MAKE_RST( dNum, dOldCh )
				MAKE_TIMER_PART( dNum, sdDeltaTime )

				dRegIndex -= 5;
				MAKE_ADDRESS_PART( dNum, dRegIndex )
				MAKE_KEY_ON( dNum, VoiceInfo.wAddress, dVoVol, dPitch, dCh,
							 psChInfo->bMtrSync, psChInfo->bLedSync)
			  	break;

			  case 0x40:	/* */

				dRegIndex += 2;
				MAKE_ADDRESS_PART( dNum, dRegIndex )
				MAKE_VEL_PITCH( dNum, dVoVol, dPitch, psChInfo->bMtrSync, psChInfo->bLedSync )
		  		break;

			  case 0x80:	/* change synth */
			  	dOldSlotNo = ( sdResult >> 8 ) & MASNDDRV_MASK_WT_SLOT;
			  	if ( dOldSlotNo <= 15 )
				{
					dOldRegIndex = (UINT32)((dOldSlotNo * 6) + MAC_WT_VOICE_ADDRESS);
				}
				else
				{
					dOldRegIndex = (UINT32)(((dOldSlotNo & 0x0f) * 6) + MAC_EXT_WT_VOICE_ADDRESS);
				}
				dOldCh = (gpMaSndDrvInfo->sSlotInfo[dOldSlotNo+32].bCh - 1) & 0xF;

				/* reset */
				dOldRegIndex += 5;
				MAKE_ADDRESS_PART( dNum, dOldRegIndex )
				MAKE_MUTE( dNum, dOldCh )
				MAKE_TIMER_PART( dNum, sdDeltaTime )

				MAKE_ADDRESS_PART( dNum, dRegIndex )
				MAKE_KEY_ON( dNum, VoiceInfo.wAddress, dVoVol, dPitch, dCh,
							 psChInfo->bMtrSync, psChInfo->bLedSync)
				break;

			  default:
				break;

			}
		}
		else if ( VoiceInfo.bSynth == MASNDDRV_WT_VOICE )
		{
			/* WT */
			sdResult = MaDva_GetWtSlot( dCh, dKey, dMode );
			if ( sdResult < 0 )
			{
				if ( sdDeltaTime > 0 ) {
					MaSndDrv_Nop( sdSeqId, sdDeltaTime, 0, 0, 0 );
					return sdResult;
				}
				return MASMW_SUCCESS;
			}
			dSlotNo = sdResult & MASNDDRV_MASK_WT_SLOT;
			dInfoSlotNo = dSlotNo + 32;

			if ( dSlotNo <= 15 )
			{
				dRegIndex = (UINT32)((dSlotNo * 6) + MAC_WT_VOICE_ADDRESS);
			}
			else
			{
				dRegIndex = (UINT32)(((dSlotNo & 0x0f) * 6) + MAC_EXT_WT_VOICE_ADDRESS);
			}

			dPitch = GetWtBlockFnum( sdSeqId, (UINT8)dCh, dKey, VoiceInfo.wKey );
			if ( gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bWtVolMode == MASNDDRV_WTVOL_MODE_MA3 )
			{
				dVoVol = CalcVoiceVolume( sdSeqId, dVelocity, (UINT8)(12 << 1) );	/* +12dB */
			}
			else
			{
				dVoVol = CalcVoiceVolume( sdSeqId, dVelocity, (UINT8)0 );
			}

			dNum = MakeDeltaTime( &(pbPacketBuf[0]), sdDeltaTime );

			switch ( sdResult & MASNDDRV_MASK_MODE )
			{
			  case 0:	/* new note */

				dOldCh = (gpMaSndDrvInfo->sSlotInfo[dInfoSlotNo].bCh - 1) & 0xF;

				/* reset */
				dRegIndex += 5;
				MAKE_ADDRESS_PART( dNum, dRegIndex )
				MAKE_RST( dNum, dOldCh )
				MAKE_TIMER_PART( dNum, sdDeltaTime )

				dRegIndex -= 5;
				MAKE_ADDRESS_PART( dNum, dRegIndex )
				MAKE_KEY_ON( dNum, VoiceInfo.wAddress, dVoVol, dPitch, dCh,
							 psChInfo->bMtrSync, psChInfo->bLedSync)
			  	break;

			  case 0x40:	/* */

				dRegIndex += 2;
				MAKE_ADDRESS_PART( dNum, dRegIndex )
				MAKE_VEL_PITCH( dNum, dVoVol, dPitch, psChInfo->bMtrSync, psChInfo->bLedSync )
			  	break;

			  case 0x80:	/* change synth */

			  	dOldSlotNo = ( sdResult >> 8 ) & MASNDDRV_MASK_FM_SLOT;
			  	if ( dOldSlotNo <= 15)
				{
					dOldRegIndex = (UINT32)((dOldSlotNo * 6) + MAC_FM_VOICE_ADDRESS);
				}
				else
				{
					dOldRegIndex = (UINT32)(((dOldSlotNo & 0x0f) * 6) + MAC_EXT_FM_VOICE_ADDRESS);
				}
				dOldCh = (gpMaSndDrvInfo->sSlotInfo[dOldSlotNo].bCh - 1) & 0xF;

				/* reset */
				dOldRegIndex += 5;
				MAKE_ADDRESS_PART( dNum, dOldRegIndex )
				MAKE_MUTE( dNum, dOldCh )
				MAKE_TIMER_PART( dNum, sdDeltaTime )

				MAKE_ADDRESS_PART( dNum, dRegIndex )
				MAKE_KEY_ON( dNum, VoiceInfo.wAddress, dVoVol, dPitch, dCh,
							 psChInfo->bMtrSync, psChInfo->bLedSync)
			  	break;

			  default:
			  	break;
			}
		}
		else
		{
			if ( sdDeltaTime > 0 )
				sdResult = MaSndDrv_Nop( sdSeqId, sdDeltaTime, 0, 0, 0 );
			return sdResult;
		}
	} /* end not AL channel */
	else
	{
		/*** AL channel **********************************************************/

		dOldSlotNo = (UINT32)(psChInfo->bSlotNo);
		if ( psChInfo->bBankNo < 128 )
		{	/* melody */
			bEvType = 0x00;
			if ( psChInfo->bPcFlag == 1 )
			{
				/* voice parameter change */
				bEvType |= 0x04;
				if ( psChInfo->bNoteOn == 0 )
				{
					psChInfo->bPcFlag = 0;
				}
			}
		}
		else
		{	/* drum */
			bEvType = 0x01;
			psChInfo->bNoteOn = 0;
			if ( ( psChInfo->bPcFlag == 0 ) && ( dOldSlotNo != 0xFF )
			  && ( gpMaSndDrvInfo->sSlotInfo[dOldSlotNo].dKey == dKey ) )
			{
				bEvType &= ~0x04;
			}
			else
			{
				/* voice parameter change */
				bEvType |= 0x04;
				/* serch AL Parameter */
				for ( dParam = 0; dParam < MASNDDRV_NUM_ALVOICE; dParam++ )
				{
					if ( ( gpMaSndDrvInfo->sAlVoiceInfo[dParam].dProgNo == dKey )
					  && ( gpMaSndDrvInfo->sAlVoiceInfo[dParam].dBankNo == psChInfo->bBankNo ) )
					{
						psChInfo->bAlFlag = (UINT8)(MASNDDRV_MASK_ALFLAG | dParam);
						break;
					}
				}
				if ( dParam == MASNDDRV_NUM_ALVOICE )
				{
					/* not AL Voice */
					psChInfo->bAlFlag |= 0x7F;
				}
				psChInfo->bPcFlag = 0;
			}
		}
		if ( psChInfo->bNoteOn == 0 )
		{	/* new note */
			bEvType |= 0x02;
		}

		dNum = MakeDeltaTime( &(pbPacketBuf[0]), sdDeltaTime );

		/* Set AL parameter */
		if ( ( psChInfo->bAlFlag & 0x7F ) != 0x7F )
		{
			/** AL voice ********************/

			pbAlParam = gpMaSndDrvInfo->sAlVoiceInfo[psChInfo->bAlFlag & MASNDDRV_MASK_ALVOICE].pAlVoice;
			if ( pbAlParam == NULL )
			{
				sdResult = MaSndDrv_Nop( sdSeqId, sdDeltaTime, 0, 0, 0 );
				return sdResult;
			}

			/* set FcOFFSET */
			if ( ( pbAlParam[26] & 0x01 ) != 0 )
			{
				/* FcOFFSET on */
				if ( ( bEvType & 0x01 ) == 0 )
				{
					/* melody */
					dParam = wAlFcOffset[dKey];
				}
				else
				{
					/* drum */
					if ( VoiceInfo.bSynth == MASNDDRV_FM_VOICE )
					{
						dParam = wAlFcOffset[VoiceInfo.wKey & MASNDDRV_MASK_KEY];
					}
					else
					{
						dParam = wAlFcOffset[MASNDDRV_KEY_CENTER];
					}
				}
			}
			else
			{
				/* FcOFFSET off */
				dParam = wAlFcOffset[0];
			}

			if ( ( bEvType & 0x02 ) != 0 )
			{
				/* new note */
				/* reset */
				if ( dOldSlotNo != 0xFF )
				{
					if ( dOldSlotNo == 0 )
					{
						dOldRegIndex = MAC_FM_VOICE_ADDRESS;
					}
					else
					{
						dOldRegIndex = MAC_WT_VOICE_ADDRESS;
					}
					MAKE_ADDRESS_PART( dNum, (dOldRegIndex+5) )
					MAKE_RST( dNum, dCh )
					MAKE_TIMER_PART( dNum, sdDeltaTime )
				}

				/* set AL parameter */
				MAKE_ADDRESS_PART( dNum, MAC_LPF_BASE )
				if ( ( bEvType & 0x04 ) != 0 )
				{
					/* with program change */
					machdep_memcpy( &pbPacketBuf[dNum], pbAlParam, 25 );

					/* set LPFCLR */
					pbPacketBuf[dNum] |= 0x40;

					/* set Q */
					sdLpfParam = (SINT32)(pbAlParam[1] & 0x1F) + psChInfo->sdLpfQShift;
					if ( sdLpfParam > MASNDDRV_MAX_LPFQ )
					{
						sdLpfParam = MASNDDRV_MAX_LPFQ;
					}
					else if ( sdLpfParam < 0 )
					{
						sdLpfParam = 0;
					}
					pbPacketBuf[dNum+1] = (UINT8)sdLpfParam;
					psChInfo->dLpfQ = pbAlParam[1] & 0x1F;

					/* set Fc3 */
					psChInfo->dLpfCof = ((pbAlParam[8] & 0x3F) << 7) | (pbAlParam[9] & 0x7F);
					sdLpfParam = psChInfo->dLpfCof + psChInfo->sdLpfCofShift;
					if ( sdLpfParam > MASNDDRV_MAX_LPFCOF )
					{
						sdLpfParam = MASNDDRV_MAX_LPFCOF;
					}
					else if ( sdLpfParam < MASNDDRV_MIN_LPFCOF )
					{
						sdLpfParam = MASNDDRV_MIN_LPFCOF;
					}
					pbPacketBuf[dNum+8] = (UINT8)((sdLpfParam >> 7) & 0x3F);
					pbPacketBuf[dNum+9] = (UINT8)(sdLpfParam & 0x7F);
					
					dNum += 25;

				}
				else
				{
					/* without program change */
					/* set LPFCLR */
					pbPacketBuf[dNum++] = (UINT8)(0xC0 | pbAlParam[0]);
					MAKE_TIMER_PART( dNum, sdDeltaTime )
					/* set FcOFSSET address */
					MAKE_ADDRESS_PART( dNum, MAC_LPF_FCOFFSET )
				}
				MAKE_FCOFFSET( dNum, pbAlParam[25], dParam )
				MAKE_TIMER_PART( dNum, sdDeltaTime )
			}
			else
			{
				/* continue note */
				/* set FcOFFSET */
				MAKE_ADDRESS_PART( dNum, MAC_LPF_FCOFFSET )
				MAKE_FCOFFSET( dNum, 0x00, dParam )
				MAKE_TIMER_PART( dNum, sdDeltaTime )
			}
		}
		else
		{
			/** not AL voice ****************/

			if ( ( bEvType & 0x02 ) != 0 )
			{
				/* new note */
				/* reset */
				if ( dOldSlotNo != 0xFF )
				{
					if ( dOldSlotNo == 0 )
					{
						dOldRegIndex = MAC_FM_VOICE_ADDRESS;
					}
					else
					{
						dOldRegIndex = MAC_WT_VOICE_ADDRESS;
					}
					MAKE_ADDRESS_PART( dNum, (dOldRegIndex+5) )
					MAKE_RST( dNum, dCh )
					MAKE_TIMER_PART( dNum, sdDeltaTime )
				}

				/* reset AL parameter */
				if ( ( bEvType & 0x04) != 0 )
				{
					/* with program change */
					/* LPFCLR & SYNTH=OFF */
					MAKE_ADDRESS_PART( dNum, MAC_LPF_BASE )
					MAKE_LPFCLR_OFF( dNum )
					MAKE_TIMER_PART( dNum, sdDeltaTime )
					/* FIX disable */
					MAKE_ADDRESS_PART( dNum, MAC_LPF_FIX0 )
					MAKE_FIX_DISABLE( dNum )
					MAKE_TIMER_PART( dNum, sdDeltaTime )
				}
			}
		}

		
		/* set Voice parameter */
		if ( VoiceInfo.bSynth == MASNDDRV_FM_VOICE )
		{
			/* FM */
			dRegIndex = MAC_FM_VOICE_ADDRESS;
			dPitch = GetFmBlockFnum( sdSeqId, (UINT8)dCh, dKey, VoiceInfo.wKey );
			dVoVol = CalcVoiceVolume( sdSeqId, dVelocity, 0 );
			dInfoSlotNo = 0;
		}
		else if ( VoiceInfo.bSynth == MASNDDRV_WT_VOICE )
		{
			/* WT */
			dRegIndex = MAC_WT_VOICE_ADDRESS;
			dPitch = GetWtBlockFnum( sdSeqId, (UINT8)dCh, dKey, VoiceInfo.wKey );
			if ( gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bWtVolMode == MASNDDRV_WTVOL_MODE_MA3 )
			{
				dVoVol = CalcVoiceVolume( sdSeqId, dVelocity, (UINT8)(12 << 1) );	/* +12dB */
			}
			else
			{
				dVoVol = CalcVoiceVolume( sdSeqId, dVelocity, (UINT8)0 );
			}
			dInfoSlotNo = MASNDDRV_MIN_WT_SLOT;
		}
		else
		{
			if ( sdDeltaTime > 0 ) {
				sdResult = MaSndDrv_Nop( sdSeqId, sdDeltaTime, 0, 0, 0 );
				return sdResult;
			}
			return MASMW_SUCCESS;
		}

		psChInfo->bNoteOn++;
		if ( psChInfo->bNoteOn == 1 )
		{
			/* new note */
			/* Key On */
			MAKE_ADDRESS_PART( dNum, dRegIndex )
			MAKE_KEY_ON( dNum, VoiceInfo.wAddress, dVoVol, dPitch, dCh,
						 psChInfo->bMtrSync, psChInfo->bLedSync )
			psChInfo->bSlotNo = (UINT8)dInfoSlotNo;
		}
		else
		{
			/* continuouse note */
			/* pitch & velocity */
			MAKE_ADDRESS_PART( dNum, (dRegIndex+2) )
			MAKE_VEL_PITCH( dNum, dVoVol,dPitch, psChInfo->bMtrSync, psChInfo->bLedSync )
		}
	}/* end if AL or not */

	/* Send packet data */

	if ( sdDeltaTime < 0 )				/* direct command */
	{
		sdResult = MaDevDrv_SendDirectPacket( &(pbPacketBuf[0]), dNum );
	}
	else								/* delayed command */
	{
		sdResult = SendDelayedPacket( &(pbPacketBuf[0]), dNum );
	}

	/* update the slot information */

	gpMaSndDrvInfo->sSlotInfo[dInfoSlotNo].bCh	= (UINT8)( (dCh + 1) | 0x80 );
	gpMaSndDrvInfo->sSlotInfo[dInfoSlotNo].dKey	= dKey;


	return sdResult;
}
/****************************************************************************
 *	MaSndDrv_NoteOnMa2
 *
 *	Description:
 *			MA-2 compatible Note On processing
 *	Argument:
 *			sdSeqId		sequence id (0..1)
 *			sdDeltaTime	delta time [tick]
 *			dCh			channel number (0..15)
 *			dKey		key number (0..127)
 *			dVelocity	velocity (0..127)
 *	Return:
 *			0			success
 *			< 0			error code
 *
 ****************************************************************************/
static SINT32 MaSndDrv_NoteOnMa2
(
	SINT32	sdSeqId,					/* sequnce id */
	SINT32	sdDeltaTime,				/* delta time */
	UINT32	dCh,						/* channel number */
	UINT32	dKey,						/* key number */
	UINT32	dVelocity					/* velocity */
)
{
	UINT32	dSlotNo;					/* slot number */
	UINT32	dInfoSlotNo;
	UINT32	dOldInfoSlotNo;
	UINT32	dOldCh;						/* old channel number */
	UINT32	dVoVolume;					/* voice volume */
	UINT32	dNum;						/* number of packet data */
	UINT32	dRegIndex;					/* register index */
	UINT32	dOldRegIndex;				/* register index */
	UINT32	dPitch;						/* pitch */
	SINT32	sdResult;					/* result of function */
	UINT8  *pbPacketBuf;
	MASNDDRVVINFO	VoiceInfo;			/* voice information */

	MASNDDRV_DBGMSG((" MaSndDrv_NoteOnMa2: id=%ld dt=%ld ch=%ld key=%ld vel=%ld\n", sdSeqId, sdDeltaTime, dCh, dKey, dVelocity));

	/* check arguments */

	MASMW_ASSERT( dKey <= MASNDDRV_MAX_KEY );
	MASMW_ASSERT( dVelocity <= MASNDDRV_MAX_VELOCITY );
	
	dKey &= MASNDDRV_MASK_KEY;
	dVelocity &= MASNDDRV_MASK_VOLUME;

	/* get voice information */

	sdResult = GetVoiceInfo( sdSeqId, dCh, dKey, &VoiceInfo );
	if ( sdResult < MASMW_SUCCESS )
	{
		if ( sdDeltaTime > 0 ) {
			sdResult = MaSndDrv_Nop( sdSeqId, sdDeltaTime, 0, 0, 0 );
			return sdResult;
		}
		return MASMW_SUCCESS;
	}
	
	/* Slot# = Channel# */
	dSlotNo = dCh;

	if ( VoiceInfo.bSynth == MASNDDRV_FM_VOICE )
	{	/* FM */
		/* set register index value and pitch of slot */
		dRegIndex = (dSlotNo * 6) + MAC_FM_VOICE_ADDRESS;
		dPitch = GetFmBlockFnum( sdSeqId, (UINT8)dCh, dKey, VoiceInfo.wKey );
		dVoVolume = CalcVoiceVolume( sdSeqId, dVelocity, 0 );

		dInfoSlotNo = dSlotNo;
	}
	else if ( VoiceInfo.bSynth == MASNDDRV_WT_VOICE )
	{	/* WT */
		/* set register index value and pitch of slot */
		dRegIndex = (dSlotNo * 6) + MAC_WT_VOICE_ADDRESS;
		dPitch = GetWtBlockFnum( sdSeqId, (UINT8)dCh, dKey, VoiceInfo.wKey );
		if ( gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bWtVolMode == MASNDDRV_WTVOL_MODE_MA3 )
		{
			dVoVolume = CalcVoiceVolume( sdSeqId, dVelocity, (UINT8)(12 << 1) );	/* +12dB */
		}
		else
		{
			dVoVolume = CalcVoiceVolume( sdSeqId, dVelocity, (UINT8)0 );
		}

		dInfoSlotNo = dSlotNo + MASNDDRV_MIN_WT_SLOT;
	}
	else
	{
		if ( sdDeltaTime > 0 ) {
			sdResult = MaSndDrv_Nop( sdSeqId, sdDeltaTime, 0, 0, 0 );
			return sdResult;
		}
		return MASMW_SUCCESS;
	}

	/* update channel information */
	gpMaSndDrvInfo->sChInfo[dCh].bNoteOn++;

	dOldCh = gpMaSndDrvInfo->sSlotInfo[dInfoSlotNo].bCh - 1;

	
	pbPacketBuf = gpMaSndDrvInfo->bPacketBuf[sdSeqId];
	dNum = MakeDeltaTime( &(pbPacketBuf[0]), sdDeltaTime );


	if ( gpMaSndDrvInfo->sChInfo[dCh].bBankNo >= 128 )
	{
		dOldInfoSlotNo = gpMaSndDrvInfo->sChInfo[dCh].bSlotNo;
		if ( dOldInfoSlotNo <= MASNDDRV_MAX_FM_SLOT )
		{
			dOldRegIndex = (dSlotNo * 6) + MAC_FM_VOICE_ADDRESS;
		}
		else if ( dOldInfoSlotNo <= MASNDDRV_MAX_WT_SLOT )
		{
			dOldRegIndex = (dSlotNo * 6) + MAC_WT_VOICE_ADDRESS;
		}
		else
		{
			dOldRegIndex = dRegIndex;
		}
		/* KeyOff & RST */
		MAKE_ADDRESS_PART( dNum, (dOldRegIndex + 5) )
		MAKE_RST( dNum, dCh )
		gpMaSndDrvInfo->sSlotInfo[dOldInfoSlotNo].bCh	= (UINT8)(dCh + 1);

		MAKE_TIMER_PART( dNum, sdDeltaTime )
		gpMaSndDrvInfo->sChInfo[dCh].bNoteOn = 1;
	}

	/* KeyOn */

	if ( gpMaSndDrvInfo->sChInfo[dCh].bNoteOn == 1 )		/* new note */
	{
		MAKE_ADDRESS_PART( dNum, dRegIndex )
		MAKE_KEY_ON( dNum, VoiceInfo.wAddress, dVoVolume, dPitch, dCh,
					 gpMaSndDrvInfo->sChInfo[dCh].bMtrSync, gpMaSndDrvInfo->sChInfo[dCh].bLedSync )
		gpMaSndDrvInfo->sChInfo[dCh].bSlotNo = (UINT8)dInfoSlotNo;
	}
	else										/* continuouse note */
	{
		dRegIndex += 2;
		MAKE_ADDRESS_PART( dNum, dRegIndex )
		MAKE_VEL_PITCH( dNum, dVoVolume, dPitch,
						gpMaSndDrvInfo->sChInfo[dCh].bMtrSync, gpMaSndDrvInfo->sChInfo[dCh].bLedSync )
	}

	/* Send packet data */

	if ( sdDeltaTime < 0 )				/* direct command */
	{
		sdResult = MaDevDrv_SendDirectPacket( &(pbPacketBuf[0]), dNum );
	}
	else								/* delayed command */
	{
		sdResult = SendDelayedPacket( &(pbPacketBuf[0]), dNum );
	}

	/* update the slot information */

	gpMaSndDrvInfo->sSlotInfo[dInfoSlotNo].bCh	= (UINT8)( (dCh + 1) | 0x80 );
	gpMaSndDrvInfo->sSlotInfo[dInfoSlotNo].dKey	= dKey;

	return sdResult;
}
/****************************************************************************
 *	MaSndDrv_NoteOnMa2Ex
 *
 *	Description:
 *			MA-2 compatible Note On processing
 *	Argument:
 *			sdSeqId		sequence id (0..1)
 *			sdDeltaTime	delta time [tick]
 *			dCh			channel number (0..15)
 *			dKey		key number (0..127)
 *			dVelocity	velocity (0..127)
 *	Return:
 *			0			success
 *			< 0			error code
 *
 ****************************************************************************/
static SINT32 MaSndDrv_NoteOnMa2Ex
(
	SINT32	sdSeqId,					/* sequnce id */
	SINT32	sdDeltaTime,				/* delta time */
	UINT32	dCh,						/* channel number */
	UINT32	dKey,						/* key number */
	UINT32	dVelocity					/* velocity */
)
{
	UINT32	dSlotNo;					/* slot number */
	UINT32	dInfoSlotNo;
	UINT32	dOldInfoSlotNo;
	UINT32	dOldCh;						/* old channel number */
	UINT32	dVoVolume;					/* voice volume */
	UINT32	dNum;						/* number of packet data */
	UINT32	dRegIndex;					/* register index */
	UINT32	dOldRegIndex;				/* register index */
	UINT32	dPitch;						/* pitch */
	SINT32	sdResult;					/* result of function */
	UINT8  *pbPacketBuf;
	MASNDDRVVINFO	VoiceInfo;			/* voice information */
	
	MASNDDRV_DBGMSG((" MaSndDrv_NoteOnMa2Ex: id=%ld dt=%ld ch=%ld key=%ld vel=%ld\n", sdSeqId, sdDeltaTime, dCh, dKey, dVelocity));

	/* check arguments */

	MASMW_ASSERT( dVelocity <= MASNDDRV_MAX_VELOCITY );
	
	dVelocity &= MASNDDRV_MASK_VOLUME;

	/* get voice information */

	sdResult = GetVoiceInfo( sdSeqId, dCh, dKey, &VoiceInfo );
	if ( sdResult < MASMW_SUCCESS )
	{
		if ( sdDeltaTime > 0 ) {
			sdResult = MaSndDrv_Nop( sdSeqId, sdDeltaTime, 0, 0, 0 );
			return sdResult;
		}
		return MASMW_SUCCESS;
	}

	/* Slot# = Channel# */
	dSlotNo = dCh;

	if ( VoiceInfo.bSynth == MASNDDRV_FM_VOICE )
	{	/* FM */
		/* set register index value and pitch of slot */
		dRegIndex = (dSlotNo * 6) + MAC_FM_VOICE_ADDRESS;
		dPitch = GetFmBlockFnumMa2( sdSeqId, (UINT8)dCh, dKey, VoiceInfo.wKey );
		dVoVolume = CalcVoiceVolume( sdSeqId, dVelocity, 0 );
		
		dInfoSlotNo = dSlotNo;
	}
	else if ( VoiceInfo.bSynth == MASNDDRV_WT_VOICE )
	{	/* WT */
		/* set register index value and pitch of slot */
		dRegIndex = (dSlotNo * 6) + MAC_WT_VOICE_ADDRESS;
		dPitch = GetWtBlockFnum( sdSeqId, (UINT8)dCh, dKey, VoiceInfo.wKey );
		if ( gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bWtVolMode == MASNDDRV_WTVOL_MODE_MA3 )
		{
			dVoVolume = CalcVoiceVolume( sdSeqId, dVelocity, (UINT8)(12 << 1) );	/* +12dB */
		}
		else
		{
			dVoVolume = CalcVoiceVolume( sdSeqId, dVelocity, (UINT8)0 );
		}
		
		dInfoSlotNo = dSlotNo + MASNDDRV_MIN_WT_SLOT;
	}
	else
	{
		if ( sdDeltaTime > 0 ) {
			sdResult = MaSndDrv_Nop( sdSeqId, sdDeltaTime, 0, 0, 0 );
			return sdResult;
		}
		return MASMW_SUCCESS;
	}

	/* update channel information */
	gpMaSndDrvInfo->sChInfo[dCh].bNoteOn++;

	dOldCh = gpMaSndDrvInfo->sSlotInfo[dInfoSlotNo].bCh - 1;


	pbPacketBuf = gpMaSndDrvInfo->bPacketBuf[sdSeqId];
	dNum = MakeDeltaTime( &(pbPacketBuf[0]), sdDeltaTime );


	if ( gpMaSndDrvInfo->sChInfo[dCh].bBankNo >= 128 )
	{
		dOldInfoSlotNo = gpMaSndDrvInfo->sChInfo[dCh].bSlotNo;
		if ( dOldInfoSlotNo <= MASNDDRV_MAX_FM_SLOT )
		{
			dOldRegIndex = (dSlotNo * 6) + MAC_FM_VOICE_ADDRESS;
		}
		else if ( dOldInfoSlotNo <= MASNDDRV_MAX_WT_SLOT )
		{
			dOldRegIndex = (dSlotNo * 6) + MAC_WT_VOICE_ADDRESS;
		}
		else
		{
			dOldRegIndex = dRegIndex;
		}
		/* KeyOff & RST */
		MAKE_ADDRESS_PART( dNum, (dOldRegIndex + 5) )
		MAKE_RST( dNum, dCh )
		gpMaSndDrvInfo->sSlotInfo[dOldInfoSlotNo].bCh	= (UINT8)(dCh + 1);
		
		MAKE_TIMER_PART( dNum, sdDeltaTime )
		gpMaSndDrvInfo->sChInfo[dCh].bNoteOn = 1;
	}

	/* KeyOn */

	if ( gpMaSndDrvInfo->sChInfo[dCh].bNoteOn == 1 )		/* new note */
	{
		MAKE_ADDRESS_PART( dNum, dRegIndex )
		MAKE_KEY_ON( dNum, VoiceInfo.wAddress, dVoVolume, dPitch, dCh,
					 gpMaSndDrvInfo->sChInfo[dCh].bMtrSync, gpMaSndDrvInfo->sChInfo[dCh].bLedSync )
		gpMaSndDrvInfo->sChInfo[dCh].bSlotNo = (UINT8)dInfoSlotNo;
	}
	else										/* continuouse note */
	{
		dRegIndex += 2;
		MAKE_ADDRESS_PART( dNum, dRegIndex )
		MAKE_VEL_PITCH( dNum, dVoVolume, dPitch,
						gpMaSndDrvInfo->sChInfo[dCh].bMtrSync, gpMaSndDrvInfo->sChInfo[dCh].bLedSync )
	}

	/* Send packet data */

	if ( sdDeltaTime < 0 )				/* direct command */
	{
		sdResult = MaDevDrv_SendDirectPacket( &(pbPacketBuf[0]), dNum );
	}
	else								/* delayed command */
	{
		sdResult = SendDelayedPacket( &(pbPacketBuf[0]), dNum );
	}

	/* update the slot information */

	gpMaSndDrvInfo->sSlotInfo[dInfoSlotNo].bCh	= (UINT8)( (dCh + 1) | 0x80 );
	gpMaSndDrvInfo->sSlotInfo[dInfoSlotNo].dKey	= dKey;

	return sdResult;
}
/****************************************************************************
 *	MaSndDrv_NoteOff
 *
 *	Description:
 *			Note Off processing
 *	Arguments:
 *			sdSeqId		sequence id (0..1)
 *			sdDeltaTime	delta time [tick]
 *			dCh			channel number (0..15)
 *			dKey		key number (0..127)
 *			dP3			unused
 *	Return:
 *			0			success
 *			< 0			error code
 *
 ****************************************************************************/
static SINT32 MaSndDrv_NoteOff
(
	SINT32	sdSeqId,					/* sequence id */
	SINT32	sdDeltaTime,				/* delta time */
	UINT32	dCh,						/* channel number */
	UINT32	dKey,						/* key number */
	UINT32	dP3							/* unused */
)
{
	UINT32	dSlotNo = 0xFF;				/* slot number */
	UINT32	dNum = 0;					/* number of packet data */
	UINT32	dRegIndex;					/* register index number */
	SINT32	sdResult;					/* result of function */
	UINT8  *pbPacketBuf;
	MASNDDRVVINFO	VoiceInfo;			/* voice information */

	(void)dP3;							/* for unused warning message */

	MASNDDRV_DBGMSG((" MaSndDrv_NoteOff: id=%ld dt=%ld ch=%ld key=%ld\n", sdSeqId, sdDeltaTime, dCh, dKey));

	/* check arguments */

	MASMW_ASSERT( dKey <= MASNDDRV_MAX_KEY );
	MASMW_ASSERT( gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bDvaMode != MASNDDRV_RESMODE2_MA5MONO );
	
	dKey &= MASNDDRV_MASK_KEY;

	/* get voice information */

	sdResult = GetVoiceInfo( sdSeqId, dCh, dKey, &VoiceInfo );
	if ( sdResult < MASMW_SUCCESS )
	{
		if ( sdDeltaTime > 0 ) {
			sdResult = MaSndDrv_Nop( sdSeqId, sdDeltaTime, 0, 0, 0 );
			return sdResult;
		}
		return MASMW_SUCCESS;
	}

	if ( gpMaSndDrvInfo->sChInfo[dCh].bAlFlag & MASNDDRV_MASK_ALFLAG )
	{
		/* AL voice */
		if ( gpMaSndDrvInfo->sChInfo[dCh].bBankNo >= 128 )
		{
			/* Drum */
			dSlotNo = (UINT32)(gpMaSndDrvInfo->sChInfo[dCh].bSlotNo);
			gpMaSndDrvInfo->sChInfo[dCh].bNoteOn = 0;
		}
		else
		{
			/* Melody */
			if ( gpMaSndDrvInfo->sChInfo[dCh].bNoteOn != 0 )
			{
				gpMaSndDrvInfo->sChInfo[dCh].bNoteOn--;
				if ( gpMaSndDrvInfo->sChInfo[dCh].bNoteOn != 0 )
				{
					if ( sdDeltaTime > 0 ) {
						sdResult = MaSndDrv_Nop( sdSeqId, sdDeltaTime, 0, 0, 0 );
						return sdResult;
					}
					return MASMW_SUCCESS;
				}
			}
			
			dSlotNo = (UINT32)(gpMaSndDrvInfo->sChInfo[dCh].bSlotNo);
		}
	}
	else
	{
		/* not AL */
		sdResult = MaDva_ReleaseSlot( dCh, dKey );
		if ( sdResult < 0 )
		{
			if ( sdDeltaTime > 0 ) {
				sdResult = MaSndDrv_Nop( sdSeqId, sdDeltaTime, 0, 0, 0 );
				return sdResult;
			}
			return MASMW_SUCCESS;
		}
		dSlotNo = sdResult;
	}

	if ( dSlotNo <= MASNDDRV_MAX_FM_SLOT )
	{
		dRegIndex = (dSlotNo * 6) + MAC_FM_VOICE_ADDRESS;
	}
	else if ( dSlotNo <= MASNDDRV_MAX_FM_EXT_SLOT )
	{
		dRegIndex = ((dSlotNo & 0xF) * 6) + MAC_EXT_FM_VOICE_ADDRESS;
	}
	else if ( dSlotNo <= MASNDDRV_MAX_WT_SLOT )
	{
		dRegIndex = ((dSlotNo & 0xF) * 6) + MAC_WT_VOICE_ADDRESS;
	}
	else if ( dSlotNo <= MASNDDRV_MAX_WT_EXT_SLOT )
	{
		dRegIndex = ((dSlotNo & 0xF) * 6) + MAC_EXT_WT_VOICE_ADDRESS;
	}
	else
	{
		if ( sdDeltaTime > 0 )
			sdResult = MaSndDrv_Nop( sdSeqId, sdDeltaTime, 0, 0, 0 );
		else
			sdResult = MASMW_SUCCESS;

		return sdResult;
	}

	/* Create packet data */

	pbPacketBuf = gpMaSndDrvInfo->bPacketBuf[sdSeqId];
	dNum = MakeDeltaTime( &(pbPacketBuf[0]), sdDeltaTime );

	/* KeyOff */
	dRegIndex += 5;
	MAKE_ADDRESS_PART( dNum, dRegIndex )
	MAKE_KEY_OFF( dNum, dCh )

	/* Send packet data */

	if ( sdDeltaTime < 0 )				/* direct command */
	{
		sdResult = MaDevDrv_SendDirectPacket( &(pbPacketBuf[0]), dNum );
	}
	else								/* delayed command */
	{
		sdResult = SendDelayedPacket( &(pbPacketBuf[0]), dNum );
	}

	/* update slot information */

	gpMaSndDrvInfo->sSlotInfo[dSlotNo].bCh	= (UINT8)(dCh + 1);

	return sdResult;
}
/****************************************************************************
 *	MaSndDrv_NoteOffMa2
 *
 *	Description:
 *			Note Off processing
 *	Arguments:
 *			sdSeqId		sequence id (0..1)
 *			sdDeltaTime	delta time [tick]
 *			dCh			channel number (0..15)
 *			dKey		key number
 *			dP3			unused
 *	Return:
 *			0			success
 *			< 0			error code
 *
 ****************************************************************************/
static SINT32 MaSndDrv_NoteOffMa2
(
	SINT32	sdSeqId,					/* sequence id */
	SINT32	sdDeltaTime,				/* delta time */
	UINT32	dCh,						/* channel number */
	UINT32	dKey,						/* key number */
	UINT32	dP3							/* unused */
)
{
	UINT32	dSlotNo = 0xFF;				/* slot number */
	UINT32	dInfoSlotNo;
	UINT32	dNum = 0;					/* number of packet data */
	UINT32	dRegIndex;					/* register index number */
	SINT32	sdResult;					/* result of function */
	UINT8  *pbPacketBuf;
	MASNDDRVVINFO	VoiceInfo;			/* voice information */

	(void)dP3;							/* for unused warning message */
	
	MASNDDRV_DBGMSG((" MaSndDrv_NoteOffMa2: id=%ld dt=%ld ch=%ld key=%ld\n", sdSeqId, sdDeltaTime, dCh, dKey));

	/* check arguments */

	MASMW_ASSERT( dKey <= MASNDDRV_MAX_KEY );
	MASMW_ASSERT( gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bDvaMode == MASNDDRV_RESMODE2_MA5MONO );
	
	dKey &= MASNDDRV_MASK_KEY;

	/* get voice information */

	sdResult = GetVoiceInfo( sdSeqId, dCh, dKey, &VoiceInfo );
	if ( sdResult < MASMW_SUCCESS )
	{
		if ( sdDeltaTime > 0 ) {
			sdResult = MaSndDrv_Nop( sdSeqId, sdDeltaTime, 0, 0, 0 );
			return sdResult;
		}
		return MASMW_SUCCESS;
	}
	
	/* Slot# = Channel# */
	dSlotNo = dCh;
	
	dInfoSlotNo = (UINT32)(gpMaSndDrvInfo->sChInfo[dCh].bSlotNo);

	if ( dInfoSlotNo <= MASNDDRV_MAX_FM_SLOT )
	{	/* FM */
		dRegIndex = (dSlotNo * 6) + MAC_FM_VOICE_ADDRESS;
	}
	else if ( dInfoSlotNo <= MASNDDRV_MAX_WT_SLOT )
	{	/* WT */
		dRegIndex = (dSlotNo * 6) + MAC_WT_VOICE_ADDRESS;
	}
	else
	{
		if ( sdDeltaTime > 0 ) {
			sdResult = MaSndDrv_Nop( sdSeqId, sdDeltaTime, 0, 0, 0 );
			return sdResult;
		}
		return MASMW_SUCCESS;
	}

	gpMaSndDrvInfo->sChInfo[dCh].bNoteOn = 0;

	/* Create packet data */

	pbPacketBuf = gpMaSndDrvInfo->bPacketBuf[sdSeqId];
	dNum = MakeDeltaTime( &(pbPacketBuf[0]), sdDeltaTime );

	/* KeyOff */
	dRegIndex += 5;
	MAKE_ADDRESS_PART( dNum, dRegIndex )
	MAKE_KEY_OFF( dNum, dCh )

	/* Send packet data */

	if ( sdDeltaTime < 0 )				/* direct command */
	{
		sdResult = MaDevDrv_SendDirectPacket( &(pbPacketBuf[0]), dNum );
	}
	else								/* delayed command */
	{
		sdResult = SendDelayedPacket( &(pbPacketBuf[0]), dNum );
	}

	/* update slot information */

	gpMaSndDrvInfo->sSlotInfo[dInfoSlotNo].bCh	= (UINT8)(dCh + 1);
	gpMaSndDrvInfo->sSlotInfo[dInfoSlotNo].dKey	= dKey;

	return sdResult;
}
/****************************************************************************
 *	MaSndDrv_NoteOffMa2Ex
 *
 *	Description:
 *			Note Off processing
 *	Arguments:
 *			sdSeqId		sequence id (0..1)
 *			sdDeltaTime	delta time [tick]
 *			dCh			channel number (0..15)
 *			dKey		key number (0x8000xxxx)
 *			dP3			unused
 *	Return:
 *			0			success
 *			< 0			error code
 *
 ****************************************************************************/
static SINT32 MaSndDrv_NoteOffMa2Ex
(
	SINT32	sdSeqId,					/* sequence id */
	SINT32	sdDeltaTime,				/* delta time */
	UINT32	dCh,						/* channel number */
	UINT32	dKey,						/* key number */
	UINT32	dP3							/* unused */
)
{
	UINT32	dSlotNo = 0xFF;				/* slot number */
	UINT32	dInfoSlotNo;
	UINT32	dNum = 0;					/* number of packet data */
	UINT32	dRegIndex;					/* register index number */
	UINT32	dPitch;						/* pitch */
	SINT32	sdResult;					/* result of function */
	UINT8  *pbPacketBuf;
	MASNDDRVVINFO	VoiceInfo;			/* voice information */

	(void)dP3;							/* for unused warning message */
	
	MASNDDRV_DBGMSG((" MaSndDrv_NoteOffMa2Ex: id=%ld dt=%ld ch=%ld key=%ld\n", sdSeqId, sdDeltaTime, dCh, dKey));

	/* check arguments */

	MASMW_ASSERT( gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bDvaMode == MASNDDRV_RESMODE2_MA5MONO );

	/* get voice information */

	sdResult = GetVoiceInfo( sdSeqId, dCh, dKey, &VoiceInfo );
	if ( sdResult < MASMW_SUCCESS )
	{
		if ( sdDeltaTime > 0 ) {
			sdResult = MaSndDrv_Nop( sdSeqId, sdDeltaTime, 0, 0, 0 );
			return sdResult;
		}
		return MASMW_SUCCESS;
	}

	/* Slot# = Channel# */
	dSlotNo = dCh;
	
	dInfoSlotNo = (UINT32)(gpMaSndDrvInfo->sChInfo[dCh].bSlotNo);

	if ( dInfoSlotNo <= MASNDDRV_MAX_FM_SLOT )
	{	/* FM */
		dRegIndex = (dSlotNo * 6) + MAC_FM_VOICE_ADDRESS;
		dPitch = GetFmBlockFnumMa2( sdSeqId, (UINT8)dCh, dKey, VoiceInfo.wKey );
	}
	else if ( dInfoSlotNo <= MASNDDRV_MAX_WT_SLOT )
	{	/* WT */
		dRegIndex = (dSlotNo * 6) + MAC_WT_VOICE_ADDRESS;
		dPitch = GetWtBlockFnum( sdSeqId, (UINT8)dCh, dKey, VoiceInfo.wKey );
	}
	else
	{
		if ( sdDeltaTime > 0 ) {
			sdResult = MaSndDrv_Nop( sdSeqId, sdDeltaTime, 0, 0, 0 );
			return sdResult;
		}
		return MASMW_SUCCESS;
	}

	gpMaSndDrvInfo->sChInfo[dCh].bNoteOn = 0;

	/* Create packet data */

	pbPacketBuf = gpMaSndDrvInfo->bPacketBuf[sdSeqId];
	dNum = MakeDeltaTime( &(pbPacketBuf[0]), sdDeltaTime );

	/* KeyOff */
	dRegIndex += 5;
	MAKE_ADDRESS_PART( dNum, dRegIndex )
	MAKE_KEY_OFF( dNum, dCh )

	/* pitch */
	dRegIndex -= 2;
	MAKE_TIMER_PART( dNum, sdDeltaTime )
	MAKE_ADDRESS_PART( dNum, dRegIndex )
	MAKE_PITCH( dNum, dPitch )

	/* Send packet data */

	if ( sdDeltaTime < 0 )				/* direct command */
	{
		sdResult = MaDevDrv_SendDirectPacket( &(pbPacketBuf[0]), dNum );
	}
	else								/* delayed command */
	{
		sdResult = SendDelayedPacket( &(pbPacketBuf[0]), dNum );
	}

	/* update slot information */

	gpMaSndDrvInfo->sSlotInfo[dInfoSlotNo].bCh	= (UINT8)(dCh + 1);
	gpMaSndDrvInfo->sSlotInfo[dInfoSlotNo].dKey	= dKey;

	return sdResult;
}
/****************************************************************************
 *	MaSndDrv_ProgramChange
 *
 *	Description:
 *			Sets the bank number and program number of specified channel.
 *	Arguments:
 *			sdSeqId		sequence id (0..1)
 *			sdDeltaTime	delta time [tick]
 *			dCh			channel number (0..15)
 *			dBankNo		bank number (0..15, 128..143)
 *			dProgNo		program number (0..127)
 *	Return:
 *			0			success
 *			< 0			error code
 *
 ****************************************************************************/
static SINT32 MaSndDrv_ProgramChange
(
	SINT32	sdSeqId,					/* sequence id */
	SINT32	sdDeltaTime,				/* delta time */
	UINT32	dCh,						/* channel number */
	UINT32	dBankNo,					/* bank number */
	UINT32	dProgNo						/* program number */
)
{
	UINT32	dAlVoiceNo;
	SINT32	sdResult;					/* result of function */

	(void)sdSeqId;						/* for unused warning message */

	MASNDDRV_DBGMSG((" MaSndDrv_ProgramChange: id=%ld dt=%ld ch=%ld bn=%ld pn=%ld\n", sdSeqId, sdDeltaTime, dCh, dBankNo, dProgNo ));

	/* check arguments */

	MASMW_ASSERT( dBankNo <= MASNDDRV_MAX_BANK );
	MASMW_ASSERT( dProgNo <= MASNDDRV_MAX_PROGRAM );

	/* update channel information */

	gpMaSndDrvInfo->sChInfo[dCh].bBankNo = (UINT8)( dBankNo & MASNDDRV_MASK_BANK );
	gpMaSndDrvInfo->sChInfo[dCh].bProgNo = (UINT8)( dProgNo & MASNDDRV_MASK_PROGRAM );

	if ( ( gpMaSndDrvInfo->sChInfo[dCh].bAlFlag & MASNDDRV_MASK_ALFLAG ) != 0 )
	{
		if ( dBankNo >= 128 )
		{
			/* Drum Bank */
			gpMaSndDrvInfo->sChInfo[dCh].bPcFlag = 1;
		}
		else
		{
			/* Melody Bank */
			/* search AL Voice */
			for ( dAlVoiceNo = 0; dAlVoiceNo < MASNDDRV_NUM_ALVOICE; dAlVoiceNo++ )
			{
				if ( ( gpMaSndDrvInfo->sAlVoiceInfo[dAlVoiceNo].dProgNo == dProgNo )
					&& ( gpMaSndDrvInfo->sAlVoiceInfo[dAlVoiceNo].dBankNo == dBankNo ) )
				{
					/* find AL Voice */
					gpMaSndDrvInfo->sChInfo[dCh].bAlFlag = (UINT8)(MASNDDRV_MASK_ALFLAG | dAlVoiceNo);
					gpMaSndDrvInfo->sChInfo[dCh].bPcFlag = 1;
					break;
				}
			}
			if ( dAlVoiceNo == MASNDDRV_NUM_ALVOICE )
			{
				/* not AL Voice */
				if ( ( gpMaSndDrvInfo->sChInfo[dCh].bAlFlag & 0x7F ) != 0x7F )
				{
					/* change voice from AL voice to normal voice */
					gpMaSndDrvInfo->sChInfo[dCh].bPcFlag = 1;
				}
				gpMaSndDrvInfo->sChInfo[dCh].bAlFlag |= 0x7F;
			}
		}
	}

	if ( dBankNo >= 128 )
	{
		gpMaSndDrvInfo->sChInfo[dCh].bPoly = MASNDDRV_MODE_POLY;
	}

	if ( sdDeltaTime > 0 )
		sdResult = MaSndDrv_Nop( sdSeqId, sdDeltaTime, 0, 0, 0 );
	else
		sdResult = MASMW_SUCCESS;

	return sdResult;
}
/****************************************************************************
 *	MaSndDrv_ModulationDepth
 *
 *	Description:
 *			Sets the depth of vibrato of specified channel.
 *	Arguments:
 *			sdSeqId		sequence id (0..1)
 *			sdDeltaTime	delta time [tick]
 *			dCh			channel number (0..15)
 *			dDepth		depth of vibrato (0: OFF, 1: x1, 2: x2, 3: x4, 4: x8)
 *			dP3			unused
 *	Return:
 *			0			success
 *			< 0			error code
 *
 ****************************************************************************/
static SINT32 MaSndDrv_ModulationDepth
(
	SINT32	sdSeqId,					/* sequence id */
	SINT32	sdDeltaTime,				/* delta time */
	UINT32	dCh,						/* channel number */
	UINT32	dDepth,						/* modulation depth */
	UINT32	dP3
)
{
	UINT32	dSfx;						/* sfx value */
	UINT32	dNum;						/* number of packet data */
	UINT32	dRegIndex;					/* register index number */
	SINT32	sdResult;					/* result of function */
	UINT8  *pbPacketBuf;
	static const UINT32 dXvb[8] = { 0, 1, 2, 4, 6, 0, 0, 0 };

	(void)dP3;							/* for unused warning message */

	MASNDDRV_DBGMSG((" MaSndDrv_ModulationDepth: id=%ld dt=%ld ch=%ld dp=%ld\n", 
						sdSeqId, sdDeltaTime, dCh, dDepth ));

	/* check arguments */

	MASMW_ASSERT( dDepth <= MASNDDRV_MAX_DEPTH );

	/* update channel information */

	dDepth &= MASNDDRV_MASK_DEPTH;

	dSfx = (gpMaSndDrvInfo->sChInfo[dCh].bSfx & MASNDDRV_MASK_XVB) | dXvb[dDepth];
	gpMaSndDrvInfo->sChInfo[dCh].bSfx = (UINT8)dSfx;

	/* Make packet data and send it */
	
	pbPacketBuf = gpMaSndDrvInfo->bPacketBuf[sdSeqId];
	dNum = MakeDeltaTime( &(pbPacketBuf[0]), sdDeltaTime );

	dRegIndex = MAC_CHANNEL_VIBRATO + dCh;

	MAKE_ADDRESS_PART( dNum, dRegIndex )
	MAKE_SFX( dNum, dSfx )
	
	if ( sdDeltaTime < 0 )				/* direct command */
	{
		sdResult = MaDevDrv_SendDirectPacket( &(pbPacketBuf[0]), dNum );
	}
	else								/* delayed command */
	{
		sdResult = SendDelayedPacket( &(pbPacketBuf[0]), dNum );
	}

	return sdResult;
}
/****************************************************************************
 *	MaSndDrv_ChannelVolume
 *
 *	Description:
 *			Sets the volume of the specified channel.
 *	Arguments:
 *			sdSeqId		sequence id (0..1)
 *			sdDeltaTime	delta time [tick]
 *			dCh			channel number (0..15)
 *			dVol		volume value (0..127) default=100
 *			dP3			unused
 *	Return:
 *			0			success
 *			< 0			error code
 *
 ****************************************************************************/
static SINT32 MaSndDrv_ChannelVolume
(
	SINT32	sdSeqId,					/* sequence id */
	SINT32	sdDeltaTime,				/* delta time */
	UINT32	dCh,						/* channel number */
	UINT32	dVol,						/* volume value */
	UINT32	dP3							/* unused */
)
{
	UINT32	dVolume;					/* volume */
	UINT32	dNum;						/* number of packet data */
	UINT32	dRegIndex;					/* register index number */
	UINT8  *pbPacketBuf;
	SINT32	sdResult;					/* result of function */

	(void)dP3;							/* for unused warning message */

	MASNDDRV_DBGMSG((" MaSndDrv_ChannelVolume: id=%ld dt=%ld ch=%ld vol=%ld\n", sdSeqId, sdDeltaTime, dCh, dVol ));

	/* check arguments */

	MASMW_ASSERT( dVol <= MASNDDRV_MAX_VOLUME );

	/* updata channel information */

	gpMaSndDrvInfo->sChInfo[dCh].bVolume = (UINT8)(dVol & MASNDDRV_MASK_VOLUME);
	dVolume = CalcChVolume( dCh );

	/* Make packet data and send it */

	pbPacketBuf = gpMaSndDrvInfo->bPacketBuf[sdSeqId];
	dNum = MakeDeltaTime( &(pbPacketBuf[0]), sdDeltaTime );

	dRegIndex = MAC_CHANNEL_VOLUME + dCh;

	MAKE_ADDRESS_PART( dNum, dRegIndex )
	MAKE_CHANNEL_VOLUME( dNum, dVolume, gpMaSndDrvInfo->sChInfo[dCh].bVolInterp )

	if ( sdDeltaTime < 0 )				/* direct command */
	{
		sdResult = MaDevDrv_SendDirectPacket( &(pbPacketBuf[0]), dNum );
	}
	else								/* delayed command */
	{
		sdResult = SendDelayedPacket( &(pbPacketBuf[0]), dNum );
	}

	return sdResult;
}
/****************************************************************************
 *	MaSndDrv_Panpot
 *
 *	Description:
 *			Sets panpot of the specified channel.
 *	Arguments:
 *			sdSeqId		sequence id number (0..1)
 *			sdDeltaTime	delta time [tick]
 *			dCh			channel number (0..15)
 *			dPan		panpot value (0..127) default=64
 *			dP3
 *	Return:
 *			0			success
 *			< 0			error code
 *
 ****************************************************************************/
static SINT32 MaSndDrv_Panpot
(
	SINT32	sdSeqId,					/* sequence id */
	SINT32	sdDeltaTime,				/* delta time */
	UINT32	dCh,						/* channel number */
	UINT32	dPan,						/* pan */
	UINT32	dP3							/* unused */
)
{
	UINT32	dNum;						/* number of packet data */
	UINT32	dRegIndex;					/* register index number */
	SINT32	sdResult;					/* result of function */
	UINT8  *pbPacketBuf;

	(void)dP3;							/* for unused warning message */

	MASNDDRV_DBGMSG((" MaSndDrv_Panpot: id=%ld dt=%ld ch=%ld pan=%ld\n", sdSeqId, sdDeltaTime, dCh, dPan ));

	/* check arguments */

	MASMW_ASSERT( dPan <= MASNDDRV_MAX_PAN );
	
	dPan &= MASNDDRV_MASK_VOLUME;

	/* Make packet data and send it */

	pbPacketBuf = gpMaSndDrvInfo->bPacketBuf[sdSeqId];
	dNum = MakeDeltaTime( &(pbPacketBuf[0]), sdDeltaTime );

	dRegIndex = MAC_CHANNEL_PANPOT + dCh;

	MAKE_ADDRESS_PART( dNum, dRegIndex )
	MAKE_PANPOT( dNum, dPan, gpMaSndDrvInfo->sChInfo[dCh].bVolInterp )

	if ( sdDeltaTime < 0 )				/* direct command */
	{
		sdResult = MaDevDrv_SendDirectPacket( &(pbPacketBuf[0]), dNum );
	}
	else								/* delayed command */
	{
		sdResult = SendDelayedPacket( &(pbPacketBuf[0]), dNum );
	}

	return sdResult;
}
/****************************************************************************
 *	MaSndDrv_Expression
 *
 *	Description:
 *			Sets the expression of specified channel.
 *	Arguments:
 *			sdSeqId		sequence id (0..1)
 *			sdDeltaTime	delta time [tick]
 *			dCh			channel number (0..15)
 *			val			expression value (0..127) default=127
 *			dP3			unused
 *	Return:
 *			0			success
 *			< 0			error code
 *
 ****************************************************************************/
static SINT32 MaSndDrv_Expression
(
	SINT32	sdSeqId,					/* sequence id */
	SINT32	sdDeltaTime,				/* delta time */
	UINT32	dCh,						/* channel number */
	UINT32	dVol,						/* volume value */
	UINT32	dP3							/* unused */
)
{
	UINT32	dVolume;					/* volume value */
	UINT32	dNum;						/* number of packet data */
	UINT32	dRegIndex;					/* register index number */
	SINT32	sdResult;					/* result of function */
	UINT8  *pbPacketBuf;

	(void)dP3;							/* for unused warning message */

	MASNDDRV_DBGMSG((" MaSndDrv_Expression: id=%ld dt=%ld ch=%ld vol=%ld\n", sdSeqId, sdDeltaTime, dCh, dVol ));

	/* check arguments */
	
	MASMW_ASSERT( dVol <= MASNDDRV_MAX_VOLUME );

	/* update channel information */

	gpMaSndDrvInfo->sChInfo[dCh].bExpression = (UINT8)(dVol & MASNDDRV_MASK_VOLUME);
	dVolume = CalcChVolume( dCh );

	/* Make packet data and send it */

	pbPacketBuf = gpMaSndDrvInfo->bPacketBuf[sdSeqId];
	dNum = MakeDeltaTime( &(pbPacketBuf[0]), sdDeltaTime );

	dRegIndex = MAC_CHANNEL_VOLUME + dCh;

	MAKE_ADDRESS_PART( dNum, dRegIndex )
	MAKE_CHANNEL_VOLUME( dNum, dVolume, gpMaSndDrvInfo->sChInfo[dCh].bVolInterp )

	if ( sdDeltaTime < 0 )				/* direct command */
	{
		sdResult = MaDevDrv_SendDirectPacket( &(pbPacketBuf[0]), dNum );
	}
	else								/* delayed command */
	{
		sdResult = SendDelayedPacket( &(pbPacketBuf[0]), dNum );
	}

	return sdResult;
}
/****************************************************************************
 *	MaSndDrv_Hold1
 *
 *	Description:
 *			Sets the dumpper setting of specified channel.
 *	Arguments:
 *			sdSeqId		sequence id (0..1)
 *			sdDeltaTime	delta time [tick]
 *			dCh			channel number (0..15)
 *			dVal		dumper value (0:OFF, 1:ON) default=0(OFF)
 *			dP3			unused
 *	Return:
 *			0			success
 *			< 0			error code
 *
 ****************************************************************************/
static SINT32 MaSndDrv_Hold1
(
	SINT32	sdSeqId,					/* sequence id */
	SINT32	sdDeltaTime,				/* delta time */
	UINT32	dCh,						/* channle number */
	UINT32	dVal,						/* dumper setting value */
	UINT32	dP3							/* unused */
)
{
	UINT32	dSfx;						/* sfx value */
	UINT32	dNum;						/* number of packet data */
	UINT32	dRegIndex;					/* register index number */
	SINT32	sdResult;					/* result of function */
	UINT8  *pbPacketBuf;

	(void)dP3;							/* for unused warning message */

	MASNDDRV_DBGMSG((" MaSndDrv_Hold1: id=%ld dt=%ld ch=%ld val=%ld\n", sdSeqId, sdDeltaTime, dCh, dVal ));

	/* check arguments */

	MASMW_ASSERT( dVal <= MASNDDRV_MAX_HOLD1 );

	/* updata channel information */

	dSfx = (gpMaSndDrvInfo->sChInfo[dCh].bSfx & MASNDDRV_MASK_SUS) | ((dVal & 0x01) << 4);
	gpMaSndDrvInfo->sChInfo[dCh].bSfx = (UINT8)dSfx;

	/* Make packet data and send it */

	pbPacketBuf = gpMaSndDrvInfo->bPacketBuf[sdSeqId];
	dNum = MakeDeltaTime( &(pbPacketBuf[0]), sdDeltaTime );

	dRegIndex = MAC_CHANNEL_SUSTAIN + dCh;

	MAKE_ADDRESS_PART( dNum, dRegIndex )
	MAKE_SFX( dNum, dSfx )

	if ( sdDeltaTime < 0 )				/* direct command */
	{
		sdResult = MaDevDrv_SendDirectPacket( &(pbPacketBuf[0]), dNum );
	}
	else								/* delayed command */
	{
		sdResult = SendDelayedPacket( &(pbPacketBuf[0]), dNum );
	}

	return sdResult;
}
/****************************************************************************
 *	MaSndDrv_AllSoundOff
 *
 *	Description:
 *			Turn off the all sound of specified channel.
 *	Arguments:
 *			sdSeqId		sequence id (0..1)
 *			sdDeltaTime	delta time [tick]
 *			dCh			channel number (0..15)
 *			dP2			unused
 *			dP3			unused
 *	Return:
 *			0			success
 *			< 0			error code
 *
 ****************************************************************************/
static SINT32 MaSndDrv_AllSoundOff
(
	SINT32	sdSeqId,					/* sequence id */
	SINT32	sdDeltaTime,				/* delta time */
	UINT32	dCh,						/* channel number */
	UINT32	dP2,						/* unused */
	UINT32	dP3							/* unused */
)
{
	UINT32	dSlotNo;					/* slot number */
	UINT32	dMinSlot;					/* minimum slot number */
	UINT32	dMaxSlot;					/* maximum slot number */
	UINT32	dSynthType;					/* type of synth */
	UINT32	dFirst = 0;					/* first flag */
	UINT32	dTNum = 0;					/* total number of packet data */
	UINT32	dNum = 0;					/* number of packet data */
	UINT32	dRegIndex;					/* register index number */
	UINT32	dResMap;					/* bit mapping of resource */
	SINT32	sdResult = MASMW_SUCCESS;	/* result of packet sending */
	SINT32	sdDt;
	UINT8  *pbPacketBuf;

	(void)dP2;							/* for unused warning message */
	(void)dP3;

	MASNDDRV_DBGMSG((" MaSndDrv_AllSoundOff: id=%ld dt=%ld ch=%ld\n", sdSeqId, sdDeltaTime, dCh ));

	MASMW_ASSERT( gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bDvaMode <= MASNDDRV_RESMODE2_MA5MONO );

	pbPacketBuf = gpMaSndDrvInfo->bPacketBuf[sdSeqId];

	gpMaSndDrvInfo->sChInfo[dCh].bNoteOn = 0;

	/* FM & WT */

	dTNum = 0;

	for ( dSynthType = 0; dSynthType < 2; dSynthType++ )
	{
		dMinSlot = gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].dMinSlot[dSynthType];
		dMaxSlot = gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].dMaxSlot[dSynthType];

		for ( dSlotNo = dMinSlot; dSlotNo <= dMaxSlot; dSlotNo++ )
		{
			if ( (gpMaSndDrvInfo->sSlotInfo[dSlotNo].bCh & 0x7F) == (UINT8)(dCh + 1) )
			{
				if ( ( gpMaSndDrvInfo->sSlotInfo[dSlotNo].bCh & 0x80 ) != 0 )
				{
					gpMaSndDrvInfo->sSlotInfo[dSlotNo].bCh = (UINT8)(dCh + 1);
				}

				if ( dSlotNo <= MASNDDRV_MAX_FM_SLOT )
				{
					dRegIndex = (dSlotNo * 6) + MAC_FM_VOICE_ADDRESS;
				}
				else if ( dSlotNo <= MASNDDRV_MAX_FM_EXT_SLOT )
				{
					dRegIndex = ((dSlotNo & 0x0F) * 6) + MAC_EXT_FM_VOICE_ADDRESS;
				}
				else if ( dSlotNo <= MASNDDRV_MAX_WT_SLOT )
				{
					dRegIndex = ((dSlotNo & 0x0F) * 6) + MAC_WT_VOICE_ADDRESS;
				}
				else if ( dSlotNo <= MASNDDRV_MAX_WT_EXT_SLOT )
				{
					dRegIndex = (((dSlotNo & 0x0F) * 6) + MAC_EXT_WT_VOICE_ADDRESS);
				}
				else
				{
					continue;
				}

				if ( dFirst == 0 )
				{
					dNum = MakeDeltaTime( &(pbPacketBuf[0]), sdDeltaTime );

					dFirst = 1;
				}
				else
				{
					dNum = 0;
					MAKE_TIMER_PART( dNum, sdDeltaTime )
				}

				/* KeyOff & MUTE & RST */
				dRegIndex += 5;
				MAKE_ADDRESS_PART( dNum, dRegIndex )
				MAKE_RST_MUTE( dNum, dCh )
				
				if ( ( gpMaSndDrvInfo->sChInfo[dCh].bAlFlag & MASNDDRV_MASK_ALFLAG ) != 0 )
				{
					/* AL channel */
					/* LPFCLR */
					MAKE_TIMER_PART( dNum, sdDeltaTime )
					MAKE_ADDRESS_PART( dNum, MAC_LPF_BASE )
					MAKE_LPFCLR_OFF( dNum )
				}

				if ( sdDeltaTime < 0 )			/* direct command */
				{
					sdResult = MaDevDrv_SendDirectPacket( &(pbPacketBuf[0]), dNum );
				}
				else							/* delayed command */
				{
					sdResult = SendDelayedPacket( &(pbPacketBuf[0]), dNum );
				}
				if ( sdResult < MASMW_SUCCESS )
				{
					return sdResult;
				}

				dTNum += dNum;
			}
		}
	}

	MaDva_ReleaseFmAllSlot( dCh );
	MaDva_ReleaseWtAllSlot( dCh );

	if ( dTNum == 0 )
	{
		if ( sdDeltaTime > 0 )
			sdResult = MaSndDrv_Nop( sdSeqId, sdDeltaTime, 0, 0, 0 );
		else
			sdResult = MASMW_SUCCESS;
	}

	if ( sdSeqId == MASMW_SEQTYPE_DELAYED )
	{
		dResMap = gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].dSaMap;
		for ( dSlotNo = MASNDDRV_MIN_STM_SLOT; dSlotNo <= MASNDDRV_MAX_STM_SLOT; dSlotNo++ )
		{
			if ( ( ( dResMap & (UINT32)(1 << (dSlotNo-MASNDDRV_MIN_STM_SLOT)) ) != 0 )
			  && ( gpMaSndDrvInfo->sSlotInfo[dSlotNo].bCh == (UINT8)( (dCh+1) | 0x80) ) )
			{
				sdDt = (SINT32)( ( sdDeltaTime >= 0 ) ? 0 : -1 );
				sdResult = MaSndDrv_StreamOff( sdSeqId, sdDt, dCh, gpMaSndDrvInfo->sSlotInfo[dSlotNo].dKey, 0 );
				if ( sdResult < MASMW_SUCCESS )
				{
					return sdResult;
				}
			}
		}
		MaDva_ReleaseStreamAllSlot( dCh );
	}

	return sdResult;
}
/****************************************************************************
 *	MaSndDrv_ResetAllControllers
 *
 *	Description:
 *			Resets the control values of specified channel.
 *	Arguments:
 *			sdSeqId		sequence id (0..1)
 *			sdDeltaTime	delta time [tick]
 *			dCh			channel number (0..15)
 *			dP2			unused
 *			dP3			unused
 *	Return:
 *			0			success
 *			< 0			error code
 *
 ****************************************************************************/
static SINT32 MaSndDrv_ResetAllControllers
(
	SINT32	sdSeqId,					/* sequence id */
	SINT32	sdDeltaTime,				/* delta time */
	UINT32	dCh,						/* channel number */
	UINT32	dP2,						/* unused */
	UINT32	dP3							/* unused */
)
{
	UINT32	dVolume;					/* volume value */
	UINT32	dBendRange;					/* bend range */
	UINT32	dBend;						/* bend value */
	UINT32	dNum;						/* number of packet data */
	UINT32	dRegIndex;					/* register index number */
	SINT32	sdResult;					/* result of function */
	UINT8  *pbPacketBuf;

	(void)dP2;							/* for unused warning message */
	(void)dP3;

	MASNDDRV_DBGMSG((" MaSndDrv_ResetAllControllers: id=%ld dt=%ld ch=%ld\n", sdSeqId, sdDeltaTime, dCh ));

	/* updata channel information */

	gpMaSndDrvInfo->sChInfo[dCh].bSfx = 0x00;
	gpMaSndDrvInfo->sChInfo[dCh].bExpression = 127;

	/* Make packet data */

	pbPacketBuf = gpMaSndDrvInfo->bPacketBuf[sdSeqId];
	dNum = MakeDeltaTime( &(pbPacketBuf[0]), sdDeltaTime );

	/* channel volume */
	dVolume = CalcChVolume( dCh );
	dRegIndex = (UINT32)(MAC_CHANNEL_VOLUME + dCh);
	MAKE_ADDRESS_PART( dNum, dRegIndex )
	MAKE_CHANNEL_VOLUME( dNum, dVolume, gpMaSndDrvInfo->sChInfo[dCh].bVolInterp )

	/* sfx */
	dRegIndex = (UINT32)(MAC_CHANNEL_SUSTAIN + dCh);
	MAKE_TIMER_PART( dNum, sdDeltaTime )
	MAKE_ADDRESS_PART( dNum, dRegIndex )
	MAKE_SFX( dNum, gpMaSndDrvInfo->sChInfo[dCh].bSfx )

	/* pitch bend */
	dBendRange = (UINT32)(gpMaSndDrvInfo->sChInfo[dCh].bBendRange);
	dBend = (UINT32)ma_pitchbend[dBendRange][64];
	gpMaSndDrvInfo->sChInfo[dCh].dPitchBend = dBend;

	dBendRange = gpMaSndDrvInfo->sChInfo[dCh].dTunePitch;
	if (dBendRange != 0x0400)
	{
		dBend = (dBend * dBendRange) >> 10;
		if (dBend > MASNDDRV_MAX_HW_PITCHBEND)
		{
			dBend = MASNDDRV_MAX_HW_PITCHBEND;
		}
	}

	dRegIndex = (UINT32)( MAC_CHANNEL_BEND + dCh*2 );
	MAKE_TIMER_PART( dNum, sdDeltaTime )
	MAKE_ADDRESS_PART( dNum, dRegIndex )
	MAKE_PITCH_BEND( dNum, dBend )

	if ( sdDeltaTime < 0 )				/* direct command */
	{
		sdResult = MaDevDrv_SendDirectPacket( &(pbPacketBuf[0]), dNum );
	}
	else								/* delayed command */
	{
		sdResult = SendDelayedPacket( &(pbPacketBuf[0]), dNum );
	}

	return sdResult;
}
/****************************************************************************
 *	MaSndDrv_AllNoteOff
 *
 *	Description:
 *			Off the all notes of specified channel.
 *	Arguments:
 *			sdSeqId		sequence id (0..1)
 *			sdDeltaTime	delta time [tick]
 *			dCh			channel number (0..15)
 *			dP2			unused
 *			dP3			unused
 *	Return:
 *			0			success
 *			< 0			error code
 *
 ****************************************************************************/
static SINT32 MaSndDrv_AllNoteOff
(
	SINT32	sdSeqId,					/* sequence id */
	SINT32	sdDeltaTime,				/* delta time */
	UINT32	dCh,						/* channel number */
	UINT32	dP2,						/* unused */
	UINT32	dP3							/* unused */
)
{
	UINT32	dSlotNo;					/* slot number */
	UINT32	dMinSlot;					/* minimum slot number */
	UINT32	dMaxSlot;					/* maximum slot number */
	UINT32	dSynthType;					/* type of synth */
	UINT32	dFirst = 0;					/* first flag */
	UINT32	dTNum = 0;					/* total number of packet data */
	UINT32	dNum = 0;					/* number of packet data */
	UINT32	dRegIndex;					/* register index number */
	UINT32	dResMap;					/* bit mapping of resource */
	SINT32	sdResult = MASMW_SUCCESS;	/* result of packet sending */
	SINT32	sdDt;
	UINT8  *pbPacketBuf;

	(void)dP2;							/* for unused warning message */
	(void)dP3;

	MASNDDRV_DBGMSG((" MaSndDrv_AllNoteOff: id=%ld dt=%ld ch=%ld\n", sdSeqId, sdDeltaTime, dCh ));

	pbPacketBuf = gpMaSndDrvInfo->bPacketBuf[sdSeqId];

	gpMaSndDrvInfo->sChInfo[dCh].bNoteOn = 0;

	/* FM & WT */

	dTNum = 0;

	for ( dSynthType = 0; dSynthType < 2; dSynthType++ )
	{
		dMinSlot = gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].dMinSlot[dSynthType];
		dMaxSlot = gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].dMaxSlot[dSynthType];

		for ( dSlotNo = dMinSlot; dSlotNo <= dMaxSlot; dSlotNo++ )
		{
			if ( gpMaSndDrvInfo->sSlotInfo[dSlotNo].bCh == (UINT8)( (dCh+1) | 0x80 ) )
			{
				gpMaSndDrvInfo->sSlotInfo[dSlotNo].bCh = (UINT8)(dCh + 1);

				if ( dSlotNo <= MASNDDRV_MAX_FM_SLOT )
				{
					dRegIndex = (dSlotNo * 6) + MAC_FM_VOICE_ADDRESS;
				}
				else if ( dSlotNo <= MASNDDRV_MAX_FM_EXT_SLOT )
				{
					dRegIndex = ((dSlotNo & 0x0F) * 6) + MAC_EXT_FM_VOICE_ADDRESS;
				}
				else if ( dSlotNo <= MASNDDRV_MAX_WT_SLOT )
				{
					dRegIndex = ((dSlotNo & 0x0F) * 6) + MAC_WT_VOICE_ADDRESS;
				}
				else if ( dSlotNo <= MASNDDRV_MAX_FM_EXT_SLOT )
				{
					dRegIndex = ((dSlotNo & 0x0F) * 6) + MAC_EXT_WT_VOICE_ADDRESS;
				}
				else
				{
					continue;
				}

				if ( dFirst == 0 )
				{
					dNum = MakeDeltaTime( &(pbPacketBuf[0]), sdDeltaTime );

					dFirst = 1;
				}
				else
				{
					dNum = 0;
					MAKE_TIMER_PART( dNum, sdDeltaTime )
				}

				/* KeyOff */
				dRegIndex += 5;
				MAKE_ADDRESS_PART( dNum, dRegIndex )
				MAKE_KEY_OFF( dNum, dCh )

				if ( sdDeltaTime < 0 )			/* direct command */
				{
					sdResult = MaDevDrv_SendDirectPacket( &(pbPacketBuf[0]), dNum );
				}
				else							/* delayed command */
				{
					sdResult = SendDelayedPacket( &(pbPacketBuf[0]), dNum );
				}

				dTNum += dNum;
			}
		}
	}
	MaDva_ReleaseFmOnSlot( dCh );
	MaDva_ReleaseWtOnSlot( dCh );

	if ( dTNum == 0 )
	{
		if ( sdDeltaTime > 0 )
			sdResult = MaSndDrv_Nop( sdSeqId, sdDeltaTime, 0, 0, 0 );
		else
			sdResult = MASMW_SUCCESS;
	}

	if ( sdSeqId == MASMW_SEQTYPE_DELAYED )
	{
		dResMap = gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].dSaMap;
		for ( dSlotNo = MASNDDRV_MIN_STM_SLOT; dSlotNo <= MASNDDRV_MAX_STM_SLOT; dSlotNo++ )
		{
			if ( ( ( dResMap & (1 << (dSlotNo-MASNDDRV_MIN_STM_SLOT)) ) != 0 )
			  && ( gpMaSndDrvInfo->sSlotInfo[dSlotNo].bCh == (UINT8)( (dCh+1) | 0x80) ) )
			{
				sdDt = (SINT32)( ( sdDeltaTime >= 0 ) ? 0 : -1 );
				MaSndDrv_StreamOff( sdSeqId, sdDt, dCh, gpMaSndDrvInfo->sSlotInfo[dSlotNo].dKey, 0 );
			}
		}
		MaDva_ReleaseStreamAllSlot( dCh );
	}

	return sdResult;
}
/****************************************************************************
 *	MaSndDrv_MonoModeOn
 *
 *	Description:
 *			Sets the monophonic mode to specified channel.
 *	Arguments:
 *			sdSeqId		sequence id (0..1)
 *			sdDeltaTime	delta time [tick]
 *			dCh			channel number (0..15)
 *			dP2			unused
 *			dP3			unused
 *	Return:
 *			0			success
 *			< 0			error code
 *
 ****************************************************************************/
static SINT32 MaSndDrv_MonoModeOn
(
	SINT32	sdSeqId,					/* sequence id */
	SINT32	sdDeltaTime,				/* delta time */
	UINT32	dCh,						/* channel number */
	UINT32	dP2,						/* unused */
	UINT32	dP3							/* unused */
)
{
	SINT32	sdResult;					/* result of function */

	(void)sdSeqId;						/*for unused warning message */
	(void)sdDeltaTime;
	(void)dP2;
	(void)dP3;

	MASNDDRV_DBGMSG((" MaSndDrv_MonoModeOn: id=%ld dt=%ld ch=%ld\n", sdSeqId, sdDeltaTime, dCh ));

	/* updata channel information */

	if ( gpMaSndDrvInfo->sChInfo[dCh].bBankNo < 128 )
	{
		gpMaSndDrvInfo->sChInfo[dCh].bPoly = MASNDDRV_MODE_MONO;
	}

	/* Make packet data and send it */

	if ( sdDeltaTime > 0 )
		sdResult = MaSndDrv_Nop( sdSeqId, sdDeltaTime, 0, 0, 0 );
	else
		sdResult = MASMW_SUCCESS;

	return sdResult;
}
/****************************************************************************
 *	MaSndDrv_PolyModeOn
 *
 *	Description:
 *			Sets the polyphonic mode to specified channel.
 *	Arguments:
 *			sdSeqId		sequence id (0..1)
 *			sdDeltaTime	delta time [tick]
 *			dCh			channel number (0..15)
 *			dP2			unused
 *			dP3			unused
 *	Return:
 *			0			success
 *			< 0			error code
 *
 ****************************************************************************/
static SINT32 MaSndDrv_PolyModeOn
(
	SINT32	sdSeqId,					/* sequence id */
	SINT32	sdDeltaTime,				/* delta time */
	UINT32	dCh,						/* channel number */
	UINT32	dP2,						/* unused */
	UINT32	dP3							/* unused */
)
{
	SINT32	sdResult;					/* result of function */
	
	(void)sdSeqId;						/*for unused warning message */
	(void)sdDeltaTime;
	(void)dP2;
	(void)dP3;

	MASNDDRV_DBGMSG((" MaSndDrv_PolyModeOn: id=%ld dt=%ld ch=%ld\n", sdSeqId, sdDeltaTime, dCh ));

	/* updata channel information */

	gpMaSndDrvInfo->sChInfo[dCh].bPoly = MASNDDRV_MODE_POLY;

	/* Make packet data and send it */

	if ( sdDeltaTime > 0 )
		sdResult = MaSndDrv_Nop( sdSeqId, sdDeltaTime, 0, 0, 0 );
	else
		sdResult = MASMW_SUCCESS;

	return sdResult;
}
/****************************************************************************
 *	MaSndDrv_PitchBend
 *
 *	Description:
 *			Sets the pitch bend of the specified channel.
 *	Arguments:
 *			sdSeqId		sequence id (0..1)
 *			sdDeltaTime	delta time [tick]
 *			dCh			channel nubmer (0..15)
 *			dBend		pitch bend value (0..2000h..3FFFh) default=2000h
 *			dP3			unused
 *	Return:
 *			0			success
 *			< 0			error code
 *
 ****************************************************************************/
static SINT32 MaSndDrv_PitchBend
(
	SINT32	sdSeqId,					/* sequence id */
	SINT32	sdDeltaTime,				/* delta time */
	UINT32	dCh,						/* channel number */
	UINT32	dBend,						/* bend */
	UINT32	dP3							/* unused */
)
{
	UINT32	dBendRange;					/* bend range */
	UINT32	dPitchBend;					/* pitch bend */
	UINT32	dRegIndex;					/* register index number */
	SINT32	dNum;						/* number of packet data */
	SINT32	sdResult;					/* result of function */
	UINT8  *pbPacketBuf;

	(void)dP3;							/* for unused warning message */

	MASNDDRV_DBGMSG((" MaSndDrv_PitchBend: id=%ld dt=%ld ch=%ld bend=%ld\n", sdSeqId, sdDeltaTime, dCh, dBend ));

	/* check arguments */

	MASMW_ASSERT( dBend <= MASNDDRV_MAX_BEND );

	/* setting the bend value */

	dBend &= MASNDDRV_MASK_BEND;

	dBendRange = (UINT32)(gpMaSndDrvInfo->sChInfo[dCh].bBendRange);
	dPitchBend = (UINT32)ma_pitchbend[dBendRange][dBend >> 7];
	gpMaSndDrvInfo->sChInfo[dCh].dPitchBend = dPitchBend;

	dBendRange = gpMaSndDrvInfo->sChInfo[dCh].dTunePitch;
	if (dBendRange != 0x0400)
	{
		dPitchBend = (dPitchBend * dBendRange) >> 10;
		if (dPitchBend > MASNDDRV_MAX_HW_PITCHBEND)
		{
			dPitchBend = MASNDDRV_MAX_HW_PITCHBEND;
		}
	}

	/* Make packet data and send it */

	pbPacketBuf = gpMaSndDrvInfo->bPacketBuf[sdSeqId];
	dNum = MakeDeltaTime( &(pbPacketBuf[0]), sdDeltaTime );

	dRegIndex = MAC_CHANNEL_BEND + dCh * 2;

	MAKE_ADDRESS_PART( dNum, dRegIndex )
	MAKE_PITCH_BEND( dNum, dPitchBend )

	if ( sdDeltaTime < 0 )				/* direct command */
	{
		sdResult = MaDevDrv_SendDirectPacket( &(pbPacketBuf[0]), dNum );
	}
	else								/* delayed command */
	{
		sdResult = SendDelayedPacket( &(pbPacketBuf[0]), dNum );
	}

	return sdResult;
}
/****************************************************************************
 *	MaSndDrv_BendRange
 *
 *	Description:
 *			
 *	Arguments:
 *			sdSeqId		sequence id (0..1)
 *			sdDeltaTime	delta time [tick]
 *			dCh			channel number (0..15)
 *			dBendRange	bend range (0..24) default=2
 *			dP3			unused
 *	Return:
 *			0			success
 *			< 0			error code
 *
 ****************************************************************************/
static SINT32 MaSndDrv_BendRange
(
	SINT32	sdSeqId,					/* sequence id */
	SINT32	sdDeltaTime,				/* delta time */
	UINT32	dCh,						/* channel number */
	UINT32	dBendRange,					/* bend range */
	UINT32	dP3							/* unused */
)
{
	SINT32	sdResult;					/* result of function */
	
	(void)dP3;							/* for unused warning message */

	MASNDDRV_DBGMSG((" MaSndDrv_BendRange: id=%ld dt=%ld ch=%ld br=%ld\n", sdSeqId, sdDeltaTime, dCh, dBendRange ));

	/* check argument */

	MASMW_ASSERT( dBendRange <= MASNDDRV_MAX_BENDRANGE );

	/* updata channel information */

	if ( dBendRange > MASNDDRV_MAX_BENDRANGE )
	{
		dBendRange = MASNDDRV_MAX_BENDRANGE;
	}
	gpMaSndDrvInfo->sChInfo[dCh].bBendRange = (UINT8)( dBendRange );

	/* Make packet data and send it */

	if ( sdDeltaTime > 0 )
		sdResult = MaSndDrv_Nop( sdSeqId, sdDeltaTime, 0, 0, 0 );
	else
		sdResult = MASMW_SUCCESS;

	return sdResult;
}
/****************************************************************************
 *	MaSndDrv_StreamOn
 *
 *	Description:
 *			Play the stream audio.
 *	Arguments:
 *			sdSeqId		sequence id (0..1)
 *			sdDeltaTime	delta time [tick]
 *			dCh			channel number (0..15)
 *			dWaveId		wave id number (0..31)
 *			dVelocity	velocity value (0..127)
 *	Return:
 *			0			success
 *			< 0			error code
 *
 ****************************************************************************/
static SINT32 MaSndDrv_StreamOn
(
	SINT32	sdSeqId,					/* sequence id */
	SINT32	sdDeltaTime,				/* delta time */
	UINT32	dCh,						/* channel number */
	UINT32	dWaveId,					/* wave id */
	UINT32	dVelocity					/* velocity */
)
{
	UINT32	dSlaveId;					/* slave wave id */
	UINT32	dSlaveSaId;					/* slave sa id */
	UINT32	dNum;						/* number of packet data */
	UINT32	dRegIndex;					/* register index number */
	UINT32	dStartAdrs;					/* start address */
	UINT32	dRamAdrs;					/* ram address */
	UINT32	dLed;						/* led sync setting */
	UINT32	dMtr;						/* motor sync setting */
	SINT32	sdResult;					/* result of function */
	UINT8  *pbPacketBuf;
	MASNDDRVSTMPARAM	sStreamParam;	/* parameter of stream audio */

	MASNDDRV_DBGMSG((" MaSndDrv_StreamOn: id=%ld dt=%ld ch=%ld id=%ld vol=%ld\n", sdSeqId, sdDeltaTime, dCh, dWaveId, dVelocity ));

	/* check arguments */

	MASMW_ASSERT( dWaveId <= MASNDDRV_MAX_WAVEID );
	MASMW_ASSERT( dVelocity <= MASNDDRV_MAX_VELOCITY );
	
	dWaveId &= MASNDDRV_MASK_WAVEID;
	dVelocity &= MASNDDRV_MASK_VOLUME;

	if ( gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].dSaNum == 0 )
	{
		if ( sdDeltaTime > 0 )
			sdResult = MaSndDrv_Nop( sdSeqId, sdDeltaTime, 0, 0, 0 );
		else
			sdResult = MASMW_SUCCESS;
		return sdResult;
	}

	if ( ( gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bWaveSlave != 0x80 )
		&& ( ( gpMaSndDrvInfo->sStreamInfo[sdSeqId][dWaveId].bFormat & 0x80 ) != 0 ) )
	{
		gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bWaveSlave = 0x80;

		if ( sdDeltaTime > 0 )
			sdResult = MaSndDrv_Nop( sdSeqId, sdDeltaTime, 0, 0, 0 );
		else
			sdResult = MASMW_SUCCESS;
		return sdResult;
	}

	/* Make WT parameters */

	sdResult = MakeStreamAudioParam( sdSeqId, dCh, dWaveId, dVelocity, &sStreamParam );
	if ( sdResult < MASMW_SUCCESS )
	{
		if ( sdDeltaTime > 0)
			sdResult = MaSndDrv_Nop( sdSeqId, sdDeltaTime, 0, 0, 0 );
		else
			sdResult = MASMW_SUCCESS;
		return sdResult;
	}
	

	pbPacketBuf = gpMaSndDrvInfo->bPacketBuf[sdSeqId];
	dNum = MakeDeltaTime( &(pbPacketBuf[0]), sdDeltaTime );

	dRamAdrs = MA_RAM_START_ADDRESS + (7-sStreamParam.bSaId) * MA_RAM_BLOCK_SIZE;
	dStartAdrs = dRamAdrs + 0x20;

	pbPacketBuf[dNum++] = (UINT8)(  dRamAdrs        & 0x7F );
	pbPacketBuf[dNum++] = (UINT8)( (dRamAdrs >>  7) & 0x7F );
	pbPacketBuf[dNum++] = (UINT8)( (dRamAdrs >> 14) | 0x80 );

	pbPacketBuf[dNum++] = (UINT8)( 13 | 0x80 );
	pbPacketBuf[dNum++] = (UINT8)( sStreamParam.bPanpot | sStreamParam.bStm | sStreamParam.bPe );
	pbPacketBuf[dNum++] = (UINT8)( sStreamParam.bPanOff | sStreamParam.bMode );
	pbPacketBuf[dNum++] = (UINT8)0x00;
	pbPacketBuf[dNum++] = (UINT8)0xF0;
	pbPacketBuf[dNum++] = (UINT8)0xF0;
	pbPacketBuf[dNum++] = (UINT8)0x00;
	pbPacketBuf[dNum++] = (UINT8)0x00;
	pbPacketBuf[dNum++] = (UINT8)( dStartAdrs >> 8 );
	pbPacketBuf[dNum++] = (UINT8)( dStartAdrs      );
	pbPacketBuf[dNum++] = (UINT8)0x00;
	pbPacketBuf[dNum++] = (UINT8)0x00;
	pbPacketBuf[dNum++] = (UINT8)( sStreamParam.wEndPoint >> 8 );
	pbPacketBuf[dNum++] = (UINT8)( sStreamParam.wEndPoint );


	if ( gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bSaLedMtrSync == 0xFF )
	{
		dLed = gpMaSndDrvInfo->sChInfo[dCh].bLedSync;
		dMtr = gpMaSndDrvInfo->sChInfo[dCh].bMtrSync;
	}
	else
	{
		dLed = gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bSaLedMtrSync & MASNDDRV_LED_SYNC;
		dMtr = (gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bSaLedMtrSync & MASNDDRV_MTR_SYNC ) >> 1;
	}

	dRamAdrs >>= 1;

	dRegIndex = MAC_EXT_WT_VOICE_ADDRESS + (9+sStreamParam.bSaId) * 6;
	MAKE_TIMER_PART( dNum, sdDeltaTime )
	MAKE_ADDRESS_PART( dNum, (dRegIndex+5) )
	MAKE_STREAM_OFF( dNum, (dCh&0xF) )
	MAKE_TIMER_PART( dNum, sdDeltaTime )
	MAKE_NOP( dNum, MAC_NOP2 )
	MAKE_TIMER_PART( dNum, sdDeltaTime )
	MAKE_ADDRESS_PART( dNum, dRegIndex )
	MAKE_STREAM_ON( dNum, dRamAdrs, sStreamParam.bVoVolume, sStreamParam.wPitch, (dCh&0xF),
					dMtr, dLed  )

	if ( ( gpMaSndDrvInfo->sStreamInfo[sdSeqId][dWaveId].bFormat & 0x80 ) != 0 )
	{
		if (sStreamParam.bSaId != 0 )
		{
			if ( sdDeltaTime > 0)
				sdResult = MaSndDrv_Nop( sdSeqId, sdDeltaTime, 0, 0, 0 );
			else
				sdResult = MASMW_SUCCESS;
			return sdResult;
		}
	}

	if ( sdDeltaTime < 0 )				/* direct command */
	{
		sdResult = MaDevDrv_SendDirectPacket( &(pbPacketBuf[0]), dNum );
		if ( sdResult < MASMW_SUCCESS )
			return sdResult;
	}


	if ( gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bWaveSlave != 0x80 )
	{
		dSlaveId = gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bWaveSlave;
		gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bWaveSlave = 0x80;

		dSlaveSaId = ( sStreamParam.bSaId == 0 ) ? 1 : 0;
		
		if ( sdDeltaTime < 0 )				/* direct command */
		{
			sdResult = MaDevDrv_StreamHandler( sdSeqId, dSlaveSaId, 0x80, (UINT8)(dSlaveId | 0x20),
											 gpMaSndDrvInfo->sStreamInfo[sdSeqId][dWaveId].dSeekP );
			gpMaSndDrvInfo->sStreamInfo[sdSeqId][dWaveId].dSeekP = 0;
			if ( sdResult < MASMW_SUCCESS )
			{
				return sdResult;
			}
		}
		else								/* delayed command */
		{



			/* set Queinfo #0 to mono ON */
			gpMaSndDrvInfo->sStreamCtrlInfo.dQueFlag            = 1;
			gpMaSndDrvInfo->sStreamCtrlInfo.dType[dSlaveSaId]   = (0x01|0x80);
			gpMaSndDrvInfo->sStreamCtrlInfo.dRamVal[dSlaveSaId] = (dSlaveId|0x20);


		}
		dWaveId |= 0x20;	/* master */
	}

	if ( sdDeltaTime < 0 )						/* direct command */
	{
		sdResult = MaDevDrv_StreamHandler( sdSeqId, (UINT32)sStreamParam.bSaId, 0x00, (UINT8)(dWaveId | 0x40),
										 gpMaSndDrvInfo->sStreamInfo[sdSeqId][dWaveId].dSeekP );
		gpMaSndDrvInfo->sStreamInfo[sdSeqId][dWaveId].dSeekP = 0;
	}
	else								/* delayed command */
	{

		/* bSaId=0(mono#0 or stereo) */
		if( sStreamParam.bSaId == 0 )
		{
			/* stereo */
			if( gpMaSndDrvInfo->sStreamInfo[sdSeqId][dWaveId].bFormat & 0x80 )
			{
				/* set QueInfo #0 to stereo ON */
				gpMaSndDrvInfo->sStreamCtrlInfo.dType[0]   = (0x02|0x80);
				/* clear QueInfo #1 */
				gpMaSndDrvInfo->sStreamCtrlInfo.dType[1]   = 0;
			}
			/* mono#0 */
			else {
				/* set Queinfo #0 to mono ON */
				gpMaSndDrvInfo->sStreamCtrlInfo.dType[0]   = (0x01|0x80);
			}
			gpMaSndDrvInfo->sStreamCtrlInfo.dQueFlag   = 1;
			gpMaSndDrvInfo->sStreamCtrlInfo.dRamVal[0] = (dWaveId|0x40);
		}
		/* mono #1 */
		else
		{
			/* if stereo ON or stereo OFF has already exist */
			if( gpMaSndDrvInfo->sStreamCtrlInfo.dType[0] & 0x02 )
			{
				/* clear QueInfo #0 */
				gpMaSndDrvInfo->sStreamCtrlInfo.dType[0] = 0;
			}
			/* replace QueInfo #1 to mono ON */
			gpMaSndDrvInfo->sStreamCtrlInfo.dQueFlag   = 1;
			gpMaSndDrvInfo->sStreamCtrlInfo.dType[1]   = (0x01|0x80);
			gpMaSndDrvInfo->sStreamCtrlInfo.dRamVal[1] = (dWaveId|0x40);
		}
		sdResult = SendDelayedPacket( &(pbPacketBuf[0]), dNum );
	}


	gpMaSndDrvInfo->sSlotInfo[MASNDDRV_MIN_STM_SLOT+sStreamParam.bSaId].bCh   = (UINT8)( (dCh + 1) | 0x80 );
	gpMaSndDrvInfo->sSlotInfo[MASNDDRV_MIN_STM_SLOT+sStreamParam.bSaId].dKey  = (UINT8)(dWaveId & 0x1F);

	return sdResult;
}
/****************************************************************************
 *	MaSndDrv_StreamOff
 *
 *	Description:
 *			Stop the stream audio.
 *	Arguments:
 *			sdSeqId		sequence id (0..1)
 *			sdDeltaTime	delta time [tick]
 *			dCh			channel number (0..15)
 *			dWaveId		wave id number (0..31)
 *			dP3
 *	Return:
 *			0			success
 *			< 0			error code
 *
 ****************************************************************************/
static SINT32 MaSndDrv_StreamOff
(
	SINT32	sdSeqId,					/* sequence id */
	SINT32	sdDeltaTime,				/* delta time */
	UINT32	dCh,						/* channel number */
	UINT32	dWaveId,					/* wave id */
	UINT32	dP3							/* unused */
)
{
	UINT32	dSaId = 0;					/* stream audio id */
	SINT32	sdResult;					/* result of function */
	PMASNDDRVSTMCTRLINFO	psStreamCtrlInfo;

	psStreamCtrlInfo = &gpMaSndDrvInfo->sStreamCtrlInfo;

	(void)dP3;							/* for unused warning message */

	MASNDDRV_DBGMSG((" MaSndDrv_StreamOff: id=%ld dt=%ld ch=%ld id=%ld\n", sdSeqId, sdDeltaTime, dCh, dWaveId ));

	/* check arguments */

	MASMW_ASSERT( dWaveId <= MASNDDRV_MAX_WAVEID );
	
	dWaveId &= MASNDDRV_MAX_WAVEID;

	if ( gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].dSaNum == 0 )
	{
		if ( sdDeltaTime > 0 )
			sdResult = MaSndDrv_Nop( sdSeqId, sdDeltaTime, 0, 0, 0 );
		else
			sdResult = MASMW_SUCCESS;
		return sdResult;
	}

	if ( gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bDvaMode == MASNDDRV_RESMODE2_NORMAL )
	{
		sdResult = MaDva_ReleaseStreamSlot( dCh, dWaveId );
		if ( sdResult < 0 )
		{
			/* send sdDeltaTime value if needed */
			if ( sdDeltaTime > 0 )
				sdResult = MaSndDrv_Nop( sdSeqId, sdDeltaTime, 0, 0, 0 );
			else
				sdResult = MASMW_SUCCESS;

			return sdResult;
		}
		dSaId = sdResult;
	}
	else
	{
		dSaId = 0;
	}

	if ( ( gpMaSndDrvInfo->sSlotInfo[MASNDDRV_MIN_STM_SLOT+dSaId].bCh & 0x80 ) == 0 )
	{
		/* send sdDeltaTime value if needed */
		if ( sdDeltaTime > 0 )
			sdResult = MaSndDrv_Nop( sdSeqId, sdDeltaTime, 0, 0, 0 );
		else
			sdResult = MASMW_SUCCESS;
		return sdResult;
	}

	if ( sdDeltaTime < 0 )				/* direct command */
	{
		sdResult = MaDevDrv_StreamHandler( 0, dSaId, 0, (UINT8)dWaveId, 0 );
	}
	else								/* delayed command */
	{
		/* send software interrupt command to stop wave data update */

		/* dSaId=0 */
		if( dSaId == 0 )
		{
			/* stereo */
			if( gpMaSndDrvInfo->sStreamInfo[sdSeqId][dWaveId].bFormat & 0x80 )
			{
				/* 	M0       o----		*/
				/*	S   -----x			*/
				if( psStreamCtrlInfo->dType[0]==0x81 )
				{
				}
				else if( psStreamCtrlInfo->dType[0]==0x82 )
				{
					/* 	S       o-----		*/
					/*	S  -----x			*/
					if( psStreamCtrlInfo->dStreamStatus&MASNDDRV_STMSTAT_STEREO 
								&& psStreamCtrlInfo->dWaveId[0]==dWaveId )
					{
					}
					/*	      o		*/
					/*	S     |		*/
					/*	      x		*/
					else
					{
						/* anyway cancel stereo KeyOn */
						psStreamCtrlInfo->dType[0]   = 0;
						if( psStreamCtrlInfo->dType[1]==0 )
							psStreamCtrlInfo->dQueFlag = 0;

						if( psStreamCtrlInfo->dStreamStatus&MASNDDRV_STMSTAT_STEREO )
						{
							/* stop running stereo stream */
							/*       V			*/
							/* S o-------x		*/
							/*       o			*/
							/* S	 |			*/
							/*		 x			*/
							psStreamCtrlInfo->dQueFlag   = 1;
							psStreamCtrlInfo->dType[0]   = 0x02;
							psStreamCtrlInfo->dRamVal[0] = psStreamCtrlInfo->dWaveId[0];
						}
						else if( psStreamCtrlInfo->dStreamStatus&MASNDDRV_STMSTAT_MONO0 )
						{
							/* stop running Mono#0 stream */
							/*       V			*/
							/* M0o-------x		*/
							/*       o			*/
							/* S	 |			*/
							/*		 x			*/
							psStreamCtrlInfo->dQueFlag   = 1;
							psStreamCtrlInfo->dType[0]   = 0x01;
							psStreamCtrlInfo->dRamVal[0] = psStreamCtrlInfo->dWaveId[0];
						}
						else if( psStreamCtrlInfo->dStreamStatus&MASNDDRV_STMSTAT_MONO1 )
						{
							/* stop running Mono#1 stream */
							/*       V			*/
							/* M1o-------x		*/
							/*       o			*/
							/* S	 |			*/
							/*		 x			*/
							psStreamCtrlInfo->dQueFlag   = 1;
							psStreamCtrlInfo->dType[1]   = 0x01;
							psStreamCtrlInfo->dRamVal[1] = psStreamCtrlInfo->dWaveId[1];
						}
						else
						{
							psStreamCtrlInfo->dQueFlag = 0;
						}
					}
				}
				/* 	M1       o----		*/
				/*	S   -----x			*/
				else if( psStreamCtrlInfo->dType[1]==0x81 )
				{
				}
				/* others */
				else
				{
					if( psStreamCtrlInfo->dStreamStatus&MASNDDRV_STMSTAT_STEREO )
					{
						/* replace QueInfo #0 to stereo OFF */
						psStreamCtrlInfo->dQueFlag   = 1;
						psStreamCtrlInfo->dType[0]   = 0x02;
						psStreamCtrlInfo->dRamVal[0] = dWaveId;
					}
				}
			}
			/* mono #0 */
			else
			{
				if( psStreamCtrlInfo->dType[0]==0x81 )
				{
					/* 	M0       o----		*/
					/*	M0  -----x			*/
					if( psStreamCtrlInfo->dStreamStatus&MASNDDRV_STMSTAT_MONO0
								&& psStreamCtrlInfo->dWaveId[0]==dWaveId )
					{
					}
					/*	      o		*/
					/*	M0    |		*/
					/*	      x		*/
					else
					{
						/* anyway cancel Mono#0 KeyOn */
						psStreamCtrlInfo->dType[0] = 0;
						if( psStreamCtrlInfo->dType[1]==0 )
							psStreamCtrlInfo->dQueFlag = 0;

						if( psStreamCtrlInfo->dStreamStatus&MASNDDRV_STMSTAT_STEREO )
						{
							/* stop running stereo stream */
							/*       V			*/
							/* S o-------x		*/
							/*       o			*/
							/* M0	 |			*/
							/*		 x			*/
							psStreamCtrlInfo->dQueFlag   = 1;
							psStreamCtrlInfo->dType[0]   = 0x02;
							psStreamCtrlInfo->dRamVal[0] = psStreamCtrlInfo->dWaveId[0];
						}
						else if( psStreamCtrlInfo->dStreamStatus&MASNDDRV_STMSTAT_MONO0 )
						{
							/* stop running mono#0 stream */
							/*       V			*/
							/* M0o-------x		*/
							/*       o			*/
							/* M0	 |			*/
							/*		 x			*/
							psStreamCtrlInfo->dQueFlag   = 1;
							psStreamCtrlInfo->dType[0]   = 0x01;
							psStreamCtrlInfo->dRamVal[0] = psStreamCtrlInfo->dWaveId[0];
						}
					}
				}
				/* 	S        o----		*/
				/*	M0  -----x			*/
				else if( psStreamCtrlInfo->dType[0]==0x82 )
				{
				}
				/* others */
				else
				{
					if( psStreamCtrlInfo->dStreamStatus&MASNDDRV_STMSTAT_MONO0 )
					{
						/* replace QueInfo #0 to mono OFF */
						psStreamCtrlInfo->dQueFlag   = 1;
						psStreamCtrlInfo->dType[0]   = 0x01;
						psStreamCtrlInfo->dRamVal[0] = dWaveId;
					}
				}
			}
		}
		/* dSaId=1(mono #1) */
		else if( dSaId == 1 )
		{
			/* 	S        o----		*/
			/*	M1  -----x			*/
			if( psStreamCtrlInfo->dType[0]==0x82 )
			{
			}
			else if( psStreamCtrlInfo->dType[1]==0x81 )
			{
				/* 	M1        o---- 	*/
				/*	M1   -----x			*/
				if( psStreamCtrlInfo->dStreamStatus&MASNDDRV_STMSTAT_MONO1 
							&& psStreamCtrlInfo->dWaveId[1]==dWaveId )
				{
				}
				/*	      o		*/
				/*	M1    |		*/
				/*	      x		*/
				else
				{
					/* anyway cancel Mono#1 KeyOn */
					psStreamCtrlInfo->dType[1] = 0;
					if( psStreamCtrlInfo->dType[0]==0 )
						psStreamCtrlInfo->dQueFlag = 0;

					if( psStreamCtrlInfo->dStreamStatus&MASNDDRV_STMSTAT_STEREO )
					{
						/* stop running stereo stream */
						/*       V			*/
						/* S o-------x		*/
						/*       o			*/
						/* M1	 |			*/
						/*		 x			*/
						psStreamCtrlInfo->dQueFlag   = 1;
						psStreamCtrlInfo->dType[0]   = 0x02;
						psStreamCtrlInfo->dRamVal[0] = psStreamCtrlInfo->dWaveId[0];
					}
					else if( psStreamCtrlInfo->dStreamStatus&MASNDDRV_STMSTAT_MONO1 )
					{
						/* stop running mono#1 stream */
						/*       V			*/
						/* M1o-------x		*/
						/*       o			*/
						/* M1	 |			*/
						/*		 x			*/
						psStreamCtrlInfo->dQueFlag   = 1;
						psStreamCtrlInfo->dType[1]   = 0x01;
						psStreamCtrlInfo->dRamVal[1] = psStreamCtrlInfo->dWaveId[1];
					}
				}
			}
			/* others */
			else
			{
				if( psStreamCtrlInfo->dStreamStatus&MASNDDRV_STMSTAT_MONO1 ) 
				{
					/* replace QueInfo #1 to mono OFF */
					psStreamCtrlInfo->dQueFlag   = 1;
					psStreamCtrlInfo->dType[1]   = 0x01;
					psStreamCtrlInfo->dRamVal[1] = dWaveId;
				}
			}
		}
		sdResult = MaSndDrv_Nop( sdSeqId, sdDeltaTime, 0, 0, 0 );

	}

	/* update slot information */

	gpMaSndDrvInfo->sSlotInfo[MASNDDRV_MIN_STM_SLOT+dSaId].bCh   = (UINT8)(dCh + 1);
	gpMaSndDrvInfo->sSlotInfo[MASNDDRV_MIN_STM_SLOT+dSaId].dKey	= dWaveId;

	return sdResult;
}
/****************************************************************************
 *	MaSndDrv_StreamSlave
 *
 *	Description:
 *			Set slave of the stream audio.
 *	Arguments:
 *			sdSeqId		sequence id (0..1)
 *			sdDeltaTime	delta time [tick]
 *			dCh			channel number (0..15)
 *			dWaveId		wave id number (0..31)
 *			dVelocity	velocity value (0..127)
 *	Return:
 *			0			success
 *			< 0			error code
 *
 ****************************************************************************/
static SINT32 MaSndDrv_StreamSlave
(
	SINT32	sdSeqId,					/* sequence id */
	SINT32	sdDeltaTime,				/* delta time */
	UINT32	dCh,						/* channel number */
	UINT32	dWaveId,					/* wave id */
	UINT32	dVelocity					/* velocity */
)
{
	UINT32	dNum;						/* number of packet data */
	UINT32	dRegIndex;					/* register index number */
	UINT32	dStartAdrs;					/* start address */
	UINT32	dRamAdrs;					/* ram address */
	UINT32	dLed;						/* led sync setting */
	UINT32	dMtr;						/* motor sync setting */
	SINT32	sdResult;					/* result of function */
	UINT8  *pbPacketBuf;
	MASNDDRVSTMPARAM	sStreamParam;	/* stream audio parameter */

	MASNDDRV_DBGMSG((" MaSndDrv_StreamSlave: id=%ld dt=%ld ch=%ld id=%ld vol=%ld\n", sdSeqId, sdDeltaTime, dCh, dWaveId, dVelocity ));

	/* check arguments */

	MASMW_ASSERT( dWaveId <= MASNDDRV_MAX_WAVEID );
	MASMW_ASSERT( dVelocity <= MASNDDRV_MAX_VELOCITY );
	
	dWaveId &= MASNDDRV_MASK_WAVEID;
	dVelocity &= MASNDDRV_MASK_VOLUME;

	if ( gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].dSaNum < 2 )
	{
		if ( sdDeltaTime > 0 ) {
			sdResult = MaSndDrv_Nop( sdSeqId, sdDeltaTime, 0, 0, 0 );
			return sdResult;
		}
		return MASMW_SUCCESS;
	}

	if ( ( gpMaSndDrvInfo->sStreamInfo[sdSeqId][dWaveId].bFormat & 0x80 ) != 0 )
	{
		if ( sdDeltaTime > 0 ) {
			sdResult = MaSndDrv_Nop( sdSeqId, sdDeltaTime, 0, 0, 0 );
			return sdResult;
		}
		return MASMW_SUCCESS;
	}

	/* Make WT parameters */

	sdResult = MakeStreamAudioParam( sdSeqId, dCh, dWaveId, dVelocity, &sStreamParam );
	if ( sdResult < MASMW_SUCCESS )
	{
		sdResult = MaSndDrv_Nop( sdSeqId, sdDeltaTime, 0, 0, 0 );
		return sdResult;
	}


	pbPacketBuf = gpMaSndDrvInfo->bPacketBuf[sdSeqId];
	dNum = MakeDeltaTime( &(pbPacketBuf[0]), sdDeltaTime );
	
	dRamAdrs = MA_RAM_START_ADDRESS + (7-sStreamParam.bSaId) * MA_RAM_BLOCK_SIZE;
	dStartAdrs = dRamAdrs + 0x20;

	pbPacketBuf[dNum++] = (UINT8)(  dRamAdrs        & 0x7F );
	pbPacketBuf[dNum++] = (UINT8)( (dRamAdrs >>  7) & 0x7F );
	pbPacketBuf[dNum++] = (UINT8)( (dRamAdrs >> 14) | 0x80 );

	pbPacketBuf[dNum++] = (UINT8)( 13 | 0x80 );
	pbPacketBuf[dNum++] = (UINT8)( sStreamParam.bPanpot | sStreamParam.bStm | sStreamParam.bPe );
	pbPacketBuf[dNum++] = (UINT8)( sStreamParam.bPanOff | sStreamParam.bMode );
	pbPacketBuf[dNum++] = (UINT8)0x00;
	pbPacketBuf[dNum++] = (UINT8)0xF0;
	pbPacketBuf[dNum++] = (UINT8)0xF0;
	pbPacketBuf[dNum++] = (UINT8)0x00;
	pbPacketBuf[dNum++] = (UINT8)0x00;
	pbPacketBuf[dNum++] = (UINT8)( dStartAdrs >> 8 );
	pbPacketBuf[dNum++] = (UINT8)( dStartAdrs      );
	pbPacketBuf[dNum++] = (UINT8)0x00;
	pbPacketBuf[dNum++] = (UINT8)0x00;
	pbPacketBuf[dNum++] = (UINT8)( sStreamParam.wEndPoint >> 8 );
	pbPacketBuf[dNum++] = (UINT8)( sStreamParam.wEndPoint );


	if ( gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bSaLedMtrSync == 0xFF )
	{
		dLed = gpMaSndDrvInfo->sChInfo[dCh].bLedSync;
		dMtr = gpMaSndDrvInfo->sChInfo[dCh].bMtrSync;
	}
	else
	{
		dLed = gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bSaLedMtrSync & MASNDDRV_LED_SYNC;
		dMtr = (gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bSaLedMtrSync & MASNDDRV_MTR_SYNC ) >> 1;
	}

	dRamAdrs >>= 1;

	dRegIndex = MAC_EXT_WT_VOICE_ADDRESS + (9+(UINT32)sStreamParam.bSaId) * 6;
	MAKE_TIMER_PART( dNum, sdDeltaTime )
	MAKE_ADDRESS_PART( dNum, (dRegIndex+5) )
	MAKE_STREAM_OFF( dNum, (dCh&0xF) )
	MAKE_TIMER_PART( dNum, sdDeltaTime )
	MAKE_NOP( dNum, MAC_NOP2 )
	MAKE_TIMER_PART( dNum, sdDeltaTime )
	MAKE_ADDRESS_PART( dNum, dRegIndex )
	MAKE_STREAM_ON( dNum, dRamAdrs, sStreamParam.bVoVolume, sStreamParam.wPitch, dCh,
					dMtr, dLed  )

	if ( sdDeltaTime < 0 )				/* direct command */
	{
		sdResult = MaDevDrv_SendDirectPacket( &(pbPacketBuf[0]), dNum );
	}
	else								/* delayed command */
	{
		sdResult = SendDelayedPacket( &(pbPacketBuf[0]), dNum );
	}


	gpMaSndDrvInfo->sSlotInfo[MASNDDRV_MIN_STM_SLOT+sStreamParam.bSaId].bCh   = (UINT8)( (dCh + 1) | 0x80 );
	gpMaSndDrvInfo->sSlotInfo[MASNDDRV_MIN_STM_SLOT+sStreamParam.bSaId].dKey  = (UINT8)dWaveId;

	gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bWaveSlave = (UINT8)dWaveId;

	return sdResult;
}
/****************************************************************************
 *	MaSndDrv_StreamPanpot
 *
 *	Description:
 *			Set the panpot value for stream audio data.
 * Arguments:
 *			sdSeqId		sequence id (0..1)
 *			sdDeltaTime	delta time [tick]
 *			dWaveId		wave id number(0..31)
 *			dPan		panpot setting(0..127, 128: center 0dB, 255: use ChPan)
 *			dP3
 *	Return:
 *			0			success
 *			< 0 		error code
 *
 ****************************************************************************/
static SINT32 MaSndDrv_StreamPanpot
(
	SINT32	sdSeqId,					/* sequence id */
	SINT32	sdDeltaTime,				/* delta time */
	UINT32	dWaveId,					/* wave id number */
	UINT32	dPan,						/* pan setting */
	UINT32	dP3
)
{
	SINT32	sdResult;					/* result of function */

	(void)dP3;

	MASNDDRV_DBGMSG((" MaSndDrv_StreamPanpot: id=%ld dt=%ld id=%ld pan=%ld\n", sdSeqId, sdDeltaTime, dWaveId, dPan ));

	MASMW_ASSERT( dWaveId <= MASNDDRV_MAX_WAVEID );
	MASMW_ASSERT( dPan <= MASNDDRV_MAX_STREAMPAN );
	
	dWaveId &= MASNDDRV_MASK_WAVEID;
	if (( dPan != 128 ) && ( dPan != 255 ))
	{
		dPan &= MASNDDRV_MASK_VOLUME;
	}

	if ( gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].dSaNum != 0 )
	{
		gpMaSndDrvInfo->sStreamInfo[sdSeqId][dWaveId].bPan = (UINT8)dPan;
	}

	if ( sdDeltaTime > 0 )
		sdResult = MaSndDrv_Nop( sdSeqId, sdDeltaTime, 0, 0, 0 );
	else
		sdResult = MASMW_SUCCESS;

	return sdResult;
}
/****************************************************************************
 *	MaSndDrv_MasterVolume
 *
 *	Description:
 *			Sets the master volume of the specified sequence.
 *	Argument:
 *			sdSeqId		sequence id (0..2)
 *			sdDeltaTime	delta time [tick]
 *			dVol		volume value (0..127) default=100
 *			dP2			unused
 *			dP3			unused
 *	Return:
 *			0			success
 *			< 0			error code
 *
 ****************************************************************************/
static SINT32 MaSndDrv_MasterVolume
(
	SINT32	sdSeqId,					/* sequence id */
	SINT32	sdDeltaTime,				/* delta time */
	UINT32	dVol,						/* volume value */
	UINT32	dP2,						/* unused */
	UINT32	dP3							/* unused */
)
{
	SINT32	sdResult;					/* result of function */

	(void)dP2;							/* for unused warning message */
	(void)dP3;

	MASNDDRV_DBGMSG((" MaSndDrv_MasterVolume: id=%ld dt=%ld vol=%ld\n", sdSeqId, sdDeltaTime, dVol ));

	/* check argument */
	MASMW_ASSERT( dVol <= MASNDDRV_MAX_VOLUME );

	gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bMasterVolume = (UINT8)( dVol & MASNDDRV_MASK_VOLUME );

	if ( sdDeltaTime > 0 )
		sdResult = MaSndDrv_Nop( sdSeqId, sdDeltaTime, 0, 0, 0 );
	else
		sdResult = MASMW_SUCCESS;


	return sdResult;
}
/****************************************************************************
 *	MaSndDrv_SystemOn
 *
 *	Description:
 *			Initialize the specified sequence.
 *	Argument:
 *			sdSeqId		sequence id (0..2)
 *			sdDeltaTime	delta time [tick]
 *			dP1			unused
 *			dP2			unused
 *			dP3			unused
 *	Return:
 *			0			success
 *			< 0			error code
 *
 ****************************************************************************/
static SINT32 MaSndDrv_SystemOn
(
	SINT32	sdSeqId,					/* sequence id */
	SINT32	sdDeltaTime,				/* delta time */
	UINT32	dP1,						/* unused */
	UINT32	dP2,						/* unused */
	UINT32	dP3							/* unused */
)
{
	UINT32	dCh;						/* channel number */
	UINT32	dMinCh;						/* minimum channel number */
	UINT32	dMaxCh;						/* maximum channel number */
	UINT32	dVolume;					/* volume value */
	UINT32 	dNum;						/* number of packet */
	UINT8	dWaveId;					/* wave id */
	UINT32	dRegIndex;					/* register index number */
	UINT32	dResMap;					/* bit mapping of resource */
	UINT32	dHvType;					/* HV type */
	SINT32	sdResult;					/* result of function */
	UINT8  *pbPacketBuf;
	PMASNDDRVCHINFO	psChInfo;

	(void)dP1;							/* for unused warning message */
	(void)dP2;
	(void)dP3;
	
	MASNDDRV_DBGMSG((" MaSndDrv_SystemOn: id=%ld dt=%ld\n", sdSeqId, sdDeltaTime ));
	
	if ( sdSeqId <= MASMW_SEQTYPE_DIRECT )
	{
		/* Delayed/Direct */


		dMinCh = MASNDDRV_NUM_CHANNEL;
		dMaxCh = 0;
		dResMap = gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].dChMap;
		for ( dCh = 0; dCh < MASNDDRV_NUM_CHANNEL; dCh++ )
		{
			if ( ( dResMap & (UINT32)(1 << dCh) ) != 0 )
			{
				if ( dMinCh > dCh ) dMinCh = dCh;
				if ( dMaxCh < dCh ) dMaxCh = dCh;
			}
		}
		if ( dMinCh == MASNDDRV_NUM_CHANNEL ) return MASMW_SUCCESS;

		for ( dCh = dMinCh; dCh <= dMaxCh; dCh++ )
		{
			psChInfo = &gpMaSndDrvInfo->sChInfo[dCh];
		
			psChInfo->bBankNo 		= (UINT8)( ( dCh == 9 ) ? 128 : 0 );
			psChInfo->bProgNo		= (UINT8)0;
			psChInfo->bPoly 			= (UINT8)MASNDDRV_MODE_POLY;
			psChInfo->bSfx			= (UINT8)0;
			psChInfo->bVolume 		= (UINT8)100;
			psChInfo->bExpression	= (UINT8)127;
			psChInfo->bBendRange 	= (UINT8)2;
			psChInfo->bNoteOn		= (UINT8)0;
			psChInfo->bSlotNo		= (UINT8)0xFF;
			psChInfo->bAlFlag		|= (UINT8)0x7F;
			psChInfo->bLedSync		= (UINT8)0;
			psChInfo->bMtrSync		= (UINT8)0;
			psChInfo->bPcFlag		= (UINT8)0;
			psChInfo->dLpfQ			= 0UL;
			psChInfo->dLpfCof		= 0UL;
			psChInfo->sdLpfQShift	= 0L;
			psChInfo->sdLpfCofShift	= 0L;
			psChInfo->dPitchBend	= 0x0400;
			psChInfo->dCoaseTune	= 0x0400;
			psChInfo->dFineTune		= 0x0400;
			psChInfo->dTunePitch	= 0x0400;
		}

		for ( dWaveId = 0; dWaveId < MA_MAX_REG_STREAM_AUDIO; dWaveId++ )
		{
			gpMaSndDrvInfo->sStreamInfo[sdSeqId][dWaveId].bPan = 255;
			gpMaSndDrvInfo->sStreamInfo[sdSeqId][dWaveId].dSeekP = 0;
		}

		gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bSaLedMtrSync = (UINT8)0xFF;
		gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bHvLedMtrSync = (UINT8)0;

		pbPacketBuf = gpMaSndDrvInfo->bPacketBuf[sdSeqId];
		dNum = MakeDeltaTime( &(pbPacketBuf[0]), sdDeltaTime );

		/* channel volume */

		dRegIndex = MAC_CHANNEL_VOLUME + dMinCh;
		MAKE_ADDRESS_PART( dNum, dRegIndex )

		for ( dCh = dMinCh; dCh < dMaxCh; dCh++ )
		{
			dVolume = CalcChVolume( dCh );
			pbPacketBuf[dNum++] = (UINT8)( dVolume & 0x7C );
		}
		dVolume = CalcChVolume( dCh );
		pbPacketBuf[dNum++] = (UINT8)( (dVolume & 0x7C) | 0x80 );

		/* panpot - 64(center) */

		dRegIndex = MAC_CHANNEL_PANPOT + dMinCh;
		MAKE_TIMER_PART( dNum, sdDeltaTime )
		MAKE_ADDRESS_PART( dNum, dRegIndex )

		for ( dCh = dMinCh; dCh < dMaxCh; dCh++ )
		{
			pbPacketBuf[dNum++] = (UINT8)( 64 & 0x7C );
		}
		pbPacketBuf[dNum++] = (UINT8)( (64  & 0x7C) | 0x80);

		/* hold1            - 0(off) */
		/* modulation depth - 0(off) */

		dRegIndex = MAC_CHANNEL_SUSTAIN + dMinCh;
		MAKE_TIMER_PART( dNum, sdDeltaTime )
		MAKE_ADDRESS_PART( dNum, dRegIndex )

		for ( dCh = dMinCh; dCh < dMaxCh; dCh++ )
		{
			pbPacketBuf[dNum++] = (UINT8)(0 & 0x7F);
		}
		pbPacketBuf[dNum++] = (UINT8)(0 | 0x80);

		/* pitch bend - (1.0) */

		dRegIndex = MAC_CHANNEL_BEND + dMinCh*2;
		MAKE_TIMER_PART( dNum, sdDeltaTime )
		MAKE_ADDRESS_PART( dNum, dRegIndex )


		for ( dCh = dMinCh; dCh < dMaxCh; dCh++ )
		{
			pbPacketBuf[dNum++] = (UINT8)0x08;
			pbPacketBuf[dNum++] = (UINT8)0x00;
		}
		pbPacketBuf[dNum++] = (UINT8)(0x08);
		pbPacketBuf[dNum++] = (UINT8)(0x00 | 0x80);
	
		/* reset LPF setting */
		if ( sdSeqId <= MASMW_SEQTYPE_DIRECT )
		{
			/* LPFCLR & SYNTH=OFF */
			MAKE_TIMER_PART( dNum, sdDeltaTime )
			MAKE_ADDRESS_PART( dNum, MAC_LPF_BASE )
			MAKE_LPFCLR_OFF( dNum )
			/* FIX disable */
			MAKE_TIMER_PART( dNum, sdDeltaTime )
			MAKE_ADDRESS_PART( dNum, MAC_LPF_FIX0 )
			MAKE_FIX_DISABLE( dNum )
		}
	}
	else if ( sdSeqId == MASMW_SEQTYPE_HV )
	{
		/* HV */
		
		/* initialize LED/Motor sync */
		gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bSaLedMtrSync = (UINT8)0xFF;
		gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bHvLedMtrSync = (UINT8)0;
		
		dHvType = (UINT32)gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bHvType;
		
		/* hv program change */
		sdResult = pfnHvFunc[dHvType].SetCommand( MASNDDRV_HV_PROGRAM_CHANGE, MASNDDRV_HV_CHANNEL_NUMBER, 0 );
		if ( sdResult < 0 ) return MASMW_SUCCESS;
		sdResult = pfnHvFunc[dHvType].SetCommand( MASNDDRV_HV_PITCH_BEND, MASNDDRV_HV_CHANNEL_NUMBER, 8192 );
		if ( sdResult < 0 ) return MASMW_SUCCESS;
		sdResult = pfnHvFunc[dHvType].SetCommand( MASNDDRV_HV_PITCH_BEND_SENS, MASNDDRV_HV_CHANNEL_NUMBER, 2 );
		if ( sdResult < 0 ) return MASMW_SUCCESS;
		sdResult = pfnHvFunc[dHvType].SetCommand( MASNDDRV_HV_PROSODIC_PITCH_BEND_SENS, MASNDDRV_HV_CHANNEL_NUMBER, 12 );
		if ( sdResult < 0 ) return MASMW_SUCCESS;


		pbPacketBuf = gpMaSndDrvInfo->bPacketBuf[sdSeqId];
		dNum = MakeDeltaTime( &(pbPacketBuf[0]), sdDeltaTime );

		/* ex channel volume - 100(-4dB) */
		MAKE_ADDRESS_PART( dNum, MAC_EXT_CH_VOL )
		MAKE_CHANNEL_VOLUME( dNum, MASNDDRV_DEFAULT_HV_CH_VOLUME, 0 )
		
		/* ex panpot - 64(center) */
		MAKE_TIMER_PART( dNum, sdDeltaTime )
		MAKE_ADDRESS_PART( dNum, MAC_EXT_CH_PAN )
		pbPacketBuf[dNum++] = (UINT8)( (64  & 0x7C) | 0x80);
		
		/* ex hold1         - 0(off) */
		/* modulation depth - 0(off) */
		MAKE_TIMER_PART( dNum, sdDeltaTime )
		MAKE_ADDRESS_PART( dNum, MAC_EXT_CH_SUS )
		pbPacketBuf[dNum++] = (UINT8)( 0 | 0x80 );
		
		/* ex pitch bend - (1.0) */
		MAKE_TIMER_PART( dNum, sdDeltaTime )
		MAKE_ADDRESS_PART( dNum, MAC_EXT_CH_BEND )
		pbPacketBuf[dNum++] = (UINT8)0x08;
		pbPacketBuf[dNum++] = (UINT8)(0x00 | 0x80);
	}
	else {
		return MASMW_SUCCESS;
	}

	if ( sdDeltaTime < 0 )				/* direct command */
	{
		sdResult = MaDevDrv_SendDirectPacket( &(pbPacketBuf[0]), dNum );
	}
	else								/* delayed command */
	{
		sdResult = SendDelayedPacket( &(pbPacketBuf[0]), dNum );
	}

	return sdResult;
}
/****************************************************************************
 *	MaSndDrv_UserEvent
 *
 *	Description:
 *			User Event operation
 *	Arguments:
 *			sdSeqId		sequence id (0..2)
 *			sdDeltaTime	delta time [tick]
 *			dVal		event value (0..127)
 *			dP2			unused
 *			dP3			unused
 *	Return:
 *			0			success
 *			< 0			error code
 *
 ****************************************************************************/
static SINT32 MaSndDrv_UserEvent
(
	SINT32	sdSeqId,					/* sequence id */
	SINT32	sdDeltaTime,				/* delta time */
	UINT32	dVal,						/* value */
	UINT32	dP2,						/* unused */
	UINT32	dP3							/* unused */
)
{
	UINT32	dNum;						/* number of packet data */
	UINT32	dRegIndex;					/* register index number */
	SINT32	sdResult;					/* result of function */
	UINT8  *pbPacketBuf;

	(void)dP2;							/* for unused warning message */
	(void)dP3;

	MASNDDRV_DBGMSG((" MaSndDrv_UserEvent: id=%ld dt=%ld val=%ld\n", sdSeqId, sdDeltaTime, dVal));

	/* check arguments */

	MASMW_ASSERT( dVal <= MASNDDRV_MAX_USEREVENT );
	
	pbPacketBuf = gpMaSndDrvInfo->bPacketBuf[sdSeqId];

	if( (dVal==MASMW_END_OF_SEQUENCE) && (sdDeltaTime>=0) )
	{
		dNum = 0;

		/* stream control */
		if ( gpMaSndDrvInfo->sStreamCtrlInfo.dQueFlag == 1 )
		{
			UpdateStreamStatus();
			if( gpMaSndDrvInfo->sStreamCtrlInfo.dType[0]!=0 )
			{
				/* make SoftInt#0 packet */
				dRegIndex = MAC_SOFTINT_RAM + 0;
				pbPacketBuf[dNum++] = (UINT8)( 0x80 );	/* Timer part */
				MAKE_ADDRESS_PART( dNum, dRegIndex );
				MAKE_SOFTINT_RAM( dNum, gpMaSndDrvInfo->sStreamCtrlInfo.dRamVal[0] );

				/* clear StreamQueInfo#0 */
				gpMaSndDrvInfo->sStreamCtrlInfo.dType[0] = 0;
			}
			if( gpMaSndDrvInfo->sStreamCtrlInfo.dType[1]!=0 )
			{
				/* make SoftInt#1 packet */
				dRegIndex = MAC_SOFTINT_RAM + 1;
				pbPacketBuf[dNum++] = (UINT8)( 0x80 );	/* Timer part */
				MAKE_ADDRESS_PART( dNum, dRegIndex );
				MAKE_SOFTINT_RAM( dNum, gpMaSndDrvInfo->sStreamCtrlInfo.dRamVal[1] );

				/* clear StreamQueInfo#1 */
				gpMaSndDrvInfo->sStreamCtrlInfo.dType[1] = 0;
			}
			
			/* clear dQueFlag */
			gpMaSndDrvInfo->sStreamCtrlInfo.dQueFlag = 0;
			
		}
		
		/* HV event control */
		if ( gpMaSndDrvInfo->dHvEventInfo[sdSeqId] != 0 )
		{
			/* make SoftInt#4 packet */
			dRegIndex = MAC_SOFTINT_RAM_4;
			pbPacketBuf[dNum++] = (UINT8)( 0x80 );	/* Timer part */
			MAKE_ADDRESS_PART( dNum, dRegIndex );
			MAKE_SOFTINT_RAM( dNum, (gpMaSndDrvInfo->dHvEventInfo[sdSeqId] & 0x7F) );

			/* clear HvEventInfo */
			gpMaSndDrvInfo->dHvEventInfo[sdSeqId]= 0;
		}

		if( dNum!=0 ) {
			/* send SoftInt packet */
			sdResult = SendDelayedPacket( &(pbPacketBuf[0]), dNum );
			if( sdResult < MASMW_SUCCESS ) 
			{
				return sdResult;
			}
		}
	}

	/* Make packet data */

	dNum = MakeDeltaTime( &(pbPacketBuf[0]), sdDeltaTime );			/* timer part */

	/* set RAM */
	if ( dVal == MASMW_END_OF_SEQUENCE )
	{
		dRegIndex = MAC_SOFTINT_RAM + MA_SEQEVENT_INT;
	}
	else
	{
		dRegIndex = (MAC_SOFTINT_RAM + MA_EVENT_INT);
	}

	pbPacketBuf[dNum++] = (UINT8)(  dRegIndex       & 0x7F );		/* address part */
	pbPacketBuf[dNum++] = (UINT8)( (dRegIndex >> 7) | 0x80 );
	pbPacketBuf[dNum++] = (UINT8)(  dVal            | 0x80 );		/* data part */

	if ( sdDeltaTime < 0 )				/* direct command */
	{
		sdResult = MaDevDrv_SendDirectPacket( &(pbPacketBuf[0]), dNum );
	}
	else								/* delayed command */
	{
		sdResult = SendDelayedPacket( &(pbPacketBuf[0]), dNum );
	}

	return sdResult;
}
/****************************************************************************
 *	MaSndDrv_LedSyncCh
 *
 *	Description:
 *			LED synchronization setting.
 *	Argument:
 *			sdSeqId		sequence id (0..2)
 *			sdDeltaTime	delta time [tick]
 *			dCh			channel number (0..15, 16:stream, 127:ExChannel)
 *			dCtrl		set value (0:not synchronize 1:synchronize)
 *			dP3			unused
 *	Return:
 *			0			success
 *			< 0			error code
 *
 ****************************************************************************/
static SINT32 MaSndDrv_LedSyncCh
(
	SINT32	sdSeqId,					/* sequence id */
	SINT32	sdDeltaTime,				/* delta time */
	UINT32	dCh,						/* channel number */
	UINT32	dCtrl,						/* control value */
	UINT32	dP3							/* unused */
)
{
	UINT8  bLedMtrSync;
	SINT32 sdResult;
	
	MASNDDRV_DBGMSG((" MaSndDrv_LedSyncCh: id=%ld dt=%ld ch=%ld ct=%ld\n", sdSeqId, sdDeltaTime, dCh, dCtrl ));

	/* check arguments */

	MASMW_ASSERT( ( dCh <= MASNDDRV_MAX_CHANNEL + 1 ) || ( dCh == MASNDDRV_HV_EXCHANNEL ) );
	MASMW_ASSERT( dCtrl <= 1 );

	(void)dP3;
	
	if ( sdSeqId == MASMW_SEQTYPE_HV )
	{
		/* for HV */
		bLedMtrSync = gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bHvLedMtrSync;
		if ( ( dCtrl & 0x01 ) == MASNDDRV_CTRL_ON )
		{
			/* LED ON */
			gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bHvLedMtrSync = (UINT8)((bLedMtrSync & MASNDDRV_MTR_SYNC) | MASNDDRV_LED_SYNC);
		}
		else
		{
			/* LED OFF */
			gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bHvLedMtrSync = (UINT8)(bLedMtrSync & MASNDDRV_MTR_SYNC);
		}
	}
	else
	{
		/* for Delayed, Direct */
		if ( dCh != MASNDDRV_STM_CH )
		{
			/* FM / WT / Ch Stream */
			if ( ( gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].dChMap & (1 << (dCh&MASNDDRV_MASK_CHANNEL)) ) != 0 )
			{
				gpMaSndDrvInfo->sChInfo[dCh & MASNDDRV_MASK_CHANNEL].bLedSync = (UINT8)(dCtrl & 0x01);
			}
		}
		else
		{
			/* Stream (no ch) */
			bLedMtrSync = gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bSaLedMtrSync;
			if ( ( dCtrl & 0x01 ) == MASNDDRV_CTRL_ON )
			{
				/* LED ON */
				gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bSaLedMtrSync = (UINT8)((bLedMtrSync & MASNDDRV_MTR_SYNC) | MASNDDRV_LED_SYNC);
			}
			else
			{
				/* LED OFF */
				gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bSaLedMtrSync = (UINT8)(bLedMtrSync & MASNDDRV_MTR_SYNC);
			}
		}
	}
	
	if ( sdDeltaTime > 0 )
		sdResult = MaSndDrv_Nop( sdSeqId, sdDeltaTime, 0, 0, 0 );
	else
		sdResult = MASMW_SUCCESS;

	return MASMW_SUCCESS;
}
/****************************************************************************
 *	MaSndDrv_MotorSyncCh
 *
 *	Description:
 *			Motor synchronization setting.
 *	Argument:
 *			sdSeqId		sequence id (0..2)
 *			sdDeltaTime	delta time [tick]
 *			dCh			channel number (0..15, 16:stream, 127:Ex channel)
 *			dCtrl		set value (0:not synchronize 1:synchronize)
 *			dP3			unused
 *	Return:
 *			0			success
 *			< 0			error code
 *
 ****************************************************************************/
static SINT32 MaSndDrv_MotorSyncCh
(
	SINT32	sdSeqId,					/* sequence id */
	SINT32	sdDeltaTime,				/* delta time */
	UINT32	dCh,						/* channel number */
	UINT32	dCtrl,						/* control value */
	UINT32	dP3							/* unused */
)
{
	UINT8  bLedMtrSync;
	SINT32 sdResult;
	
	MASNDDRV_DBGMSG((" MaSndDrv_MotorSyncCh: id=%ld dt=%ld ch=%ld ct=%ld\n", sdSeqId, sdDeltaTime, dCh, dCtrl ));

	/* check arguments */

	MASMW_ASSERT( ( dCh <= MASNDDRV_MAX_CHANNEL + 1 ) || ( dCh == MASNDDRV_HV_EXCHANNEL ) );
	MASMW_ASSERT( dCtrl <= 1 );

	(void)dP3;

	if ( sdSeqId == MASMW_SEQTYPE_HV )
	{
		/* for HV */
		bLedMtrSync = gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bHvLedMtrSync;
		if ( ( dCtrl & 0x01 ) == MASNDDRV_CTRL_ON )
		{
			/* Motor ON */
			gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bHvLedMtrSync = (UINT8)((bLedMtrSync & MASNDDRV_LED_SYNC) | MASNDDRV_MTR_SYNC);
		}
		else
		{
			/* Motor OFF */
			gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bHvLedMtrSync = (UINT8)(bLedMtrSync & MASNDDRV_LED_SYNC);
		}
	}
	else
	{
		/* for Delayed, Direct */
		if ( dCh != MASNDDRV_STM_CH )
		{
			/* FM / WT / Ch Stream */
			if ( ( gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].dChMap & (1 << (dCh&MASNDDRV_MASK_CHANNEL)) ) != 0 )
			{
				gpMaSndDrvInfo->sChInfo[dCh & MASNDDRV_MASK_CHANNEL].bMtrSync = (UINT8)(dCtrl & 0x01);
			}
		}
		else
		{
			/* Stream (no ch) */
			bLedMtrSync = gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bSaLedMtrSync;
			if ( ( dCtrl & 0x01 ) == MASNDDRV_CTRL_ON )
			{
				/* Motor ON */
				gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bSaLedMtrSync = (UINT8)((bLedMtrSync & MASNDDRV_LED_SYNC) | MASNDDRV_MTR_SYNC);
			}
			else
			{
				/* Motor OFF */
				gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bSaLedMtrSync = (UINT8)(bLedMtrSync & MASNDDRV_LED_SYNC);
			}
		}
	}

	if ( sdDeltaTime > 0 )
		sdResult = MaSndDrv_Nop( sdSeqId, sdDeltaTime, 0, 0, 0 );
	else
		sdResult = MASMW_SUCCESS;

	return MASMW_SUCCESS;
}
/****************************************************************************
 *	MaSndDrv_StreamPause
 *
 *	Description:
 *			Pause to play the stream.
 *	Argument:
 *			sdSeqId		sequence id (1)
 *			sdDeltaTime	delta time [tick]
 *			dCh			channel number (0..15)
 *			dWaveId		wave id number (0..31)
 *			dP3			unused
 *	Return:
 *			0			success
 *			< 0			error code
 *
 ****************************************************************************/
static SINT32 MaSndDrv_StreamPause
(
	SINT32	sdSeqId,					/* sequence id */
	SINT32	sdDeltaTime,				/* delta time */
	UINT32	dCh,						/* unused */
	UINT32	dWaveId,					/* unused */
	UINT32	dP3							/* unused */
)
{
	UINT32 dNum;
	UINT32 dRegIndex;
	SINT32 sdResult;
	UINT8 *pbPacketBuf;
	
	MASNDDRV_DBGMSG((" MaSndDrv_StreamPause: id=%ld dt=%ld ch=%ld wa=%ld\n", 
					   sdSeqId, sdDeltaTime, dCh, dWaveId ));

	/* check arguments */

	MASMW_ASSERT( dWaveId <= MASNDDRV_MAX_WAVEID );

	(void)dWaveId;
	(void)dP3;

	pbPacketBuf = gpMaSndDrvInfo->bPacketBuf[sdSeqId];
	dNum = MakeDeltaTime( &(pbPacketBuf[0]), sdDeltaTime );
	dRegIndex = MAC_EXT_WT_VOICE_ADDRESS + 54; /* WT#25 (#422) */

	/* Mute */
	MAKE_ADDRESS_PART( dNum, (dRegIndex + 5) )
	pbPacketBuf[dNum++] = (UINT8)( 0xE0 | (dCh & 0xF) ); /* KeyOn, Mute */
	
	/* FNUM=BLOCK=0 */
	MAKE_TIMER_PART( dNum, sdDeltaTime )
	MAKE_ADDRESS_PART( dNum, (dRegIndex + 3) )
	pbPacketBuf[dNum++] = (UINT8)0x00;
	pbPacketBuf[dNum++] = (UINT8)0x80;
	
	if ( sdDeltaTime < 0 )				/* direct command */
	{
		sdResult = MaDevDrv_SendDirectPacket( &(pbPacketBuf[0]), dNum );
	}
	else								/* delayed command */
	{
		sdResult = SendDelayedPacket( &(pbPacketBuf[0]), dNum );
	}

	return sdResult;
}
/****************************************************************************
 *	MaSndDrv_StreamRestart
 *
 *	Description:
 *			Restart to play the stream.
 *	Argument:
 *			sdSeqId		sequence id (1)
 *			sdDeltaTime	delta time [tick]
 *			dCh			channel number (0..15)
 *			dWaveId		wave id number (0..31)
 *			dP3			unused
 *	Return:
 *			0			success
 *			< 0			error code
 *
 ****************************************************************************/
static SINT32 MaSndDrv_StreamRestart
(
	SINT32	sdSeqId,					/* sequence id */
	SINT32	sdDeltaTime,				/* delta time */
	UINT32	dCh,						/* unused */
	UINT32	dWaveId,					/* unused */
	UINT32	dP3							/* unused */
)
{
	UINT32 dNum;
	UINT32 dRegIndex;
	UINT32 dPitch;
	SINT32 sdResult;
	UINT8 *pbPacketBuf;
	
	MASNDDRV_DBGMSG((" MaSndDrv_StreamRestart: id=%ld dt=%ld ch=%ld wa=%ld\n", 
					   sdSeqId, sdDeltaTime, dCh, dWaveId ));

	/* check arguments */

	MASMW_ASSERT( dWaveId <= MASNDDRV_MAX_WAVEID );
	
	dWaveId &= MASNDDRV_MASK_WAVEID;

	(void)dP3;

	dPitch = GetMa5WtPitch( gpMaSndDrvInfo->sStreamInfo[sdSeqId][dWaveId].dFrequency, 128, 0, 60 );

	pbPacketBuf = gpMaSndDrvInfo->bPacketBuf[sdSeqId];
	dNum = MakeDeltaTime( &(pbPacketBuf[0]), sdDeltaTime );
	dRegIndex = MAC_EXT_WT_VOICE_ADDRESS + 54; /* WT#25 (#422) */
	
	/* set FNUM BLOCK */
	MAKE_ADDRESS_PART( dNum, (dRegIndex + 3) )
	pbPacketBuf[dNum++] = (UINT8)( (dPitch >> 7) & 0x3F );
	pbPacketBuf[dNum++] = (UINT8)( (dPitch & 0x7F) | 0x80 );

	/* reset Mute */
	MAKE_TIMER_PART( dNum, sdDeltaTime )
	MAKE_ADDRESS_PART( dNum, (dRegIndex + 5) )
	pbPacketBuf[dNum++] = (UINT8)( 0xC0 | (dCh & 0xF) ); /* KeyOn, Mute */

	if ( sdDeltaTime < 0 )				/* direct command */
	{
		sdResult = MaDevDrv_SendDirectPacket( &(pbPacketBuf[0]), dNum );
	}
	else								/* delayed command */
	{
		sdResult = SendDelayedPacket( &(pbPacketBuf[0]), dNum );
	}

	return sdResult;
}
/****************************************************************************
 *	MaSndDrv_VolumeDirect
 *
 *	Description:
 *			Control interpolation control.
 *	Argument:
 *			sdSeqId		sequence id (1)
 *			sdDeltaTime	delta time [tick]
 *			dCh			channel number (0..15)
 *			dCtrl		interpolation control 0:OFF 1:ON
 *			dP3			unused
 *	Return:
 *			0			success
 *			< 0			error code
 *
 ****************************************************************************/
static SINT32 MaSndDrv_VolumeDirect
(
	SINT32	sdSeqId,					/* sequence id */
	SINT32	sdDeltaTime,				/* delta time */
	UINT32	dCh,						/* channel number */
	UINT32	dCtrl,						/* control value */
	UINT32	dP3							/* unused */
)
{
	SINT32 sdResult;

	(void)dP3;

	MASNDDRV_DBGMSG((" MaSndDrv_VolumeDirect: id=%ld dt=%ld ch=%ld ct=%ld\n", sdSeqId, sdDeltaTime, dCh, dCtrl ));

	/* check argument */

	MASMW_ASSERT( dCtrl <= 1 );

	gpMaSndDrvInfo->sChInfo[dCh].bVolInterp = (UINT8)(dCtrl & 0x01);
	
	if ( sdDeltaTime > 0 )
		sdResult = MaSndDrv_Nop( sdSeqId, sdDeltaTime, 0, 0, 0 );
	else
		sdResult = MASMW_SUCCESS;

	return sdResult;
}
/****************************************************************************
 *	MaSndDrv_SetLpfQ
 *
 *	Description:
 *			Set the resonance value of the LPF.
 *	Argument:
 *			sdSeqId		sequence id (0..1)
 *			sdDeltaTime	delta time [tick]
 *			dCh			channel number (0..15)
 *			dVal		resonance value (0..64..127)
 *			dP3			unused
 *	Return:
 *			0			success
 *			< 0			error code
 *
 ****************************************************************************/
static SINT32 MaSndDrv_SetLpfQ
(
	SINT32	sdSeqId,					/* sequence id */
	SINT32	sdDeltaTime,				/* delta time */
	UINT32	dCh,						/* channel number */
	UINT32	dVal,						/* resonance value */
	UINT32	dP3							/* unused */
)
{
	UINT32 dNum;
	SINT32 sdQVal;
	SINT32 sdQShiftVal;
	SINT32 sdResult = MASMW_SUCCESS;
	UINT8 *pbPacketBuf;
	
	MASNDDRV_DBGMSG((" MaSndDrv_SetLpfQ: id=%ld dt=%ld ch=%ld val=%ld\n", sdSeqId, sdDeltaTime, dCh, dVal ));

	(void)dP3;

	MASMW_ASSERT( dVal <= MASNDDRV_MAX_LPFQREL );
	
	dVal &= MASNDDRV_MASK_LPF;

	if ( (gpMaSndDrvInfo->sChInfo[dCh].bAlFlag & MASNDDRV_MASK_ALFLAG) == 0 )
	{
		/* not AL channel */
		if ( sdDeltaTime > 0 ) {
			sdResult = MaSndDrv_Nop( sdSeqId, sdDeltaTime, 0, 0, 0 );
			return sdResult;
		}
		return MASMW_SUCCESS;
	}

	sdQShiftVal = sdU127toS64[dVal] / 2;
	gpMaSndDrvInfo->sChInfo[dCh].sdLpfQShift = sdQShiftVal;

	sdQVal = (SINT32)(gpMaSndDrvInfo->sChInfo[dCh].dLpfQ) + sdQShiftVal;
	if ( sdQVal > MASNDDRV_MAX_LPFQ )
	{
		sdQVal = MASNDDRV_MAX_LPFQ;
	}
	else if ( sdQVal < 0 )
	{
		sdQVal = 0;
	}

	/* Make packet data and send it */

	pbPacketBuf = gpMaSndDrvInfo->bPacketBuf[sdSeqId];
	dNum = MakeDeltaTime( &(pbPacketBuf[0]), sdDeltaTime );

	MAKE_ADDRESS_PART( dNum, MAC_LPF_Q )
	MAKE_LPFQ( dNum, sdQVal )
	
	if ( sdDeltaTime < 0 )				/* direct command */
	{
		sdResult = MaDevDrv_SendDirectPacket( &(pbPacketBuf[0]), dNum );
	}
	else								/* delayed command */
	{
		sdResult = SendDelayedPacket( &(pbPacketBuf[0]), dNum );
	}

	return sdResult;
}
/****************************************************************************
 *	MaSndDrv_SetLpfCof
 *
 *	Description:
 *			Set the brightness value of the LPF.
 *	Argument:
 *			sdSeqId		sequence id (0..1)
 *			sdDeltaTime	delta time [tick]
 *			dCh			channel number (0..15)
 *			dVal		brightness value (0..64..127)
 *			dP3			unused
 *	Return:
 *			0			success
 *			< 0			error code
 *
 ****************************************************************************/
static SINT32 MaSndDrv_SetLpfCof
(
	SINT32	sdSeqId,					/* sequence id */
	SINT32	sdDeltaTime,				/* delta time */
	UINT32	dCh,						/* channel number */
	UINT32	dVal,						/* brightness value */
	UINT32	dP3							/* unused */
)
{
	UINT32 dNum;
	SINT32 sdCofVal;
	SINT32 sdShiftVal;
	SINT32 sdResult = MASMW_SUCCESS;
	UINT8 *pbPacketBuf;
	
	MASNDDRV_DBGMSG((" MaSndDrv_SetLpfCof: id=%ld dt=%ld ch=%ld val=%ld\n", sdSeqId, sdDeltaTime, dCh, dVal ));

	(void)dP3;

	MASMW_ASSERT( dVal <= MASNDDRV_MAX_LPFCOFREL );

	dVal &= MASNDDRV_MASK_LPF;

	if ( ( gpMaSndDrvInfo->sChInfo[dCh].bAlFlag & MASNDDRV_MASK_ALFLAG ) == 0 )
	{
		/* not AL channel */
		if ( sdDeltaTime > 0 ) {
			sdResult = MaSndDrv_Nop( sdSeqId, sdDeltaTime, 0, 0, 0 );
			return sdResult;
		}
		return MASMW_SUCCESS;
	}

	sdShiftVal = sdU127toS64[dVal] * 128;
	gpMaSndDrvInfo->sChInfo[dCh].sdLpfCofShift = sdShiftVal;

	sdCofVal = (SINT32)(gpMaSndDrvInfo->sChInfo[dCh].dLpfCof) + sdShiftVal;
	if ( sdCofVal > MASNDDRV_MAX_LPFCOF )
	{
		sdCofVal = MASNDDRV_MAX_LPFCOF;
	}
	else if ( sdCofVal < MASNDDRV_MIN_LPFCOF )
	{
		sdCofVal = MASNDDRV_MIN_LPFCOF;
	}

	/* Make packet data and send it */

	pbPacketBuf = gpMaSndDrvInfo->bPacketBuf[sdSeqId];
	dNum = MakeDeltaTime( &(pbPacketBuf[0]), sdDeltaTime );

	MAKE_ADDRESS_PART( dNum, MAC_LPF_FC3 )
	MAKE_LPFFC3( dNum, sdCofVal )

	if ( sdDeltaTime < 0 )				/* direct command */
	{
		sdResult = MaDevDrv_SendDirectPacket( &(pbPacketBuf[0]), dNum );
	}
	else								/* delayed command */
	{
		sdResult = SendDelayedPacket( &(pbPacketBuf[0]), dNum );
	}

	return sdResult;
}
/****************************************************************************
 *	MaSndDrv_HvNoteOn
 *
 *	Description:
 *			Note On for HV.
 *	Argument:
 *			sdSeqId		sequence id (2)
 *			sdDeltaTime	delta time [tick]
 *			dCh			channel number (0..15, 127)
 *			dpBuf		the pointer to the area of the note data
 *			dP3			unused
 *	Return:
 *			0			success
 *			< 0			error code
 *
 ****************************************************************************/
static SINT32 MaSndDrv_HvNoteOn
(
	SINT32	sdSeqId,					/* sequence id */
	SINT32	sdDeltaTime,				/* delta time */
	UINT32	dCh,						/* channel number */
	UINT32	dpBuf,						/* pointer to data */
	UINT32	dP3							/* unused */
)
{
	SINT32 sdResult;

	MASNDDRV_DBGMSG((" MaSndDrv_HvNoteOn: id=%ld dt=%ld ch=%ld\n", sdSeqId, sdDeltaTime, dCh ));

	MASMW_ASSERT( sdSeqId == MASMW_SEQTYPE_HV );
	MASMW_ASSERT( dCh <= MASNDDRV_MAX_CHANNEL || dCh == MASNDDRV_HV_EXCHANNEL );
	MASMW_ASSERT( dpBuf != 0 );

	(void)sdDeltaTime;
	(void)dP3;

	sdResult = pfnHvFunc[gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bHvType].SetCommand( MASNDDRV_HV_NOTE_ON, MASNDDRV_HV_CHANNEL_NUMBER, dpBuf );
	if ( sdResult < 0 ) return MASMW_SUCCESS;

	gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bHvChannel = (UINT8)dCh;

	return sdResult;
}
/****************************************************************************
 *	MaSndDrv_HvNoteOff
 *
 *	Description:
 *			Note Off for HV.
 *	Argument:
 *			sdSeqId		sequence id (2)
 *			sdDeltaTime	delta time [tick]
 *			dCh			channel number (0..15, 127)
 *			dP2			unused
 *			dP3			unused
 *	Return:
 *			0			success
 *			< 0			error code
 *
 ****************************************************************************/
static SINT32 MaSndDrv_HvNoteOff
(
	SINT32	sdSeqId,					/* sequence id */
	SINT32	sdDeltaTime,				/* delta time */
	UINT32	dCh,						/* channel number */
	UINT32	dP2,						/* unused */
	UINT32	dP3							/* unused */
)
{
	SINT32 sdResult;

	MASNDDRV_DBGMSG((" MaSndDrv_HvNoteOff: id=%ld dt=%ld ch=%ld\n", sdSeqId, sdDeltaTime, dCh ));

	MASMW_ASSERT( sdSeqId == MASMW_SEQTYPE_HV );
	MASMW_ASSERT( dCh <= MASNDDRV_MAX_CHANNEL || dCh == MASNDDRV_HV_EXCHANNEL );

	(void)sdDeltaTime;
	(void)dP2;
	(void)dP3;

	if ( dCh != gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bHvChannel )
	{
		return MASMW_SUCCESS;
	}

	sdResult = pfnHvFunc[gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bHvType].SetCommand( MASNDDRV_HV_NOTE_OFF, MASNDDRV_HV_CHANNEL_NUMBER, 0 );
	if ( sdResult < 0 ) return MASMW_SUCCESS;

	return sdResult;
}
/****************************************************************************
 *	MaSndDrv_HvProgramChange
 *
 *	Description:
 *			Set the program number of the HV.
 *	Argument:
 *			sdSeqId		sequence id (2)
 *			sdDeltaTime	delta time [tick]
 *			dCh			channel number (0..15, 127)
 *			dBank		bank number (0)
 *			dProgNo		program number (0..15, 128..143)
 *	Return:
 *			0			success
 *			< 0			error code
 *
 ****************************************************************************/
static SINT32 MaSndDrv_HvProgramChange
(
	SINT32	sdSeqId,					/* sequence id */
	SINT32	sdDeltaTime,				/* delta time */
	UINT32	dCh,						/* channel number */
	UINT32	dBank,						/* bank number */
	UINT32	dProgNo						/* program number */
)
{
	SINT32 sdResult;

	MASNDDRV_DBGMSG((" MaSndDrv_HvProgramChange: id=%ld dt=%ld ch=%ld bn=%ld pn=%ld\n", 
										sdSeqId, sdDeltaTime, dCh, dBank, dProgNo ));

	MASMW_ASSERT( sdSeqId == MASMW_SEQTYPE_HV );
	MASMW_ASSERT( dCh <= MASNDDRV_MAX_CHANNEL || dCh == MASNDDRV_HV_EXCHANNEL );
	MASMW_ASSERT( dBank == 0 );
	MASMW_ASSERT( ( dProgNo & 0x7F ) <= MASNDDRV_MAX_HV_EFFECTIVE_PROGNO );

	(void)sdDeltaTime;
	(void)dCh;
	(void)dBank;
	
	sdResult = pfnHvFunc[gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bHvType].SetCommand( MASNDDRV_HV_PROGRAM_CHANGE, MASNDDRV_HV_CHANNEL_NUMBER, dProgNo );
	if ( sdResult < 0 ) return MASMW_SUCCESS;

	return sdResult;
}
/****************************************************************************
 *	MaSndDrv_HvPitchBend
 *
 *	Description:
 *			Set the pitch bend of the HV.
 *	Argument:
 *			sdSeqId		sequence id (2)
 *			sdDeltaTime	delta time [tick]
 *			dCh			channel number (0..15, 127)
 *			dBend		pitch bend (0..2000h..3FFFh)
 *			dP3			unused
 *	Return:
 *			0			success
 *			< 0			error code
 *
 ****************************************************************************/
static SINT32 MaSndDrv_HvPitchBend
(
	SINT32	sdSeqId,					/* sequence id */
	SINT32	sdDeltaTime,				/* delta time */
	UINT32	dCh,						/* channel number */
	UINT32	dBend,						/* bend */
	UINT32	dP3							/* unused */
)
{
	SINT32 sdResult;
	
	MASNDDRV_DBGMSG((" MaSndDrv_HvPitchBend: id=%ld dt=%ld ch=%ld bend=%ld\n", sdSeqId, sdDeltaTime, dCh, dBend ));

	MASMW_ASSERT( sdSeqId == MASMW_SEQTYPE_HV );
	MASMW_ASSERT( dCh <= MASNDDRV_MAX_CHANNEL || dCh == MASNDDRV_HV_EXCHANNEL );
	MASMW_ASSERT( dBend <= MASNDDRV_MAX_BEND );

	(void)sdDeltaTime;
	(void)dCh;
	(void)dP3;

	sdResult = pfnHvFunc[gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bHvType].SetCommand( MASNDDRV_HV_PITCH_BEND, MASNDDRV_HV_CHANNEL_NUMBER, dBend );
	if ( sdResult < 0 ) return MASMW_SUCCESS;

	return sdResult;
}
/****************************************************************************
 *	MaSndDrv_HvBendRange
 *
 *	Description:
 *			Set the pitch bend range of the HV.
 *	Argument:
 *			sdSeqId		sequence id (2)
 *			sdDeltaTime	delta time [tick]
 *			dCh			channel number (0..15, 127)
 *			dRange		bend range (0..24)
 *			dP3			unused
 *	Return:
 *			0			success
 *			< 0			error code
 *
 ****************************************************************************/
static SINT32 MaSndDrv_HvBendRange
(
	SINT32	sdSeqId,					/* sequence id */
	SINT32	sdDeltaTime,				/* delta time */
	UINT32	dCh,						/* channel number */
	UINT32	dRange,						/* bend range */
	UINT32	dP3							/* unused */
)
{
	SINT32 sdResult;
	
	MASNDDRV_DBGMSG((" MaSndDrv_HvBendRange: id=%ld dt=%ld ch=%ld br=%ld\n", sdSeqId, sdDeltaTime, dCh, dRange ));

	MASMW_ASSERT( sdSeqId == MASMW_SEQTYPE_HV );
	MASMW_ASSERT( dCh <= MASNDDRV_MAX_CHANNEL || dCh == MASNDDRV_HV_EXCHANNEL );
	MASMW_ASSERT( dRange <= MASNDDRV_MAX_BENDRANGE );

	(void)sdDeltaTime;
	(void)dCh;
	(void)dP3;

	sdResult = pfnHvFunc[gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bHvType].SetCommand( MASNDDRV_HV_PITCH_BEND_SENS, MASNDDRV_HV_CHANNEL_NUMBER, dRange );
	if ( sdResult < 0 ) return MASMW_SUCCESS;

	return sdResult;
}
/****************************************************************************
 *	MaSndDrv_HvProsodicBendRange
 *
 *	Description:
 *			Set the prosodic pitch bend range of the HV.
 *	Argument:
 *			sdSeqId		sequence id (2)
 *			sdDeltaTime	delta time [tick]
 *			dCh			channel number (0..15, 127)
 *			dRange		bend range (0..60)
 *			dP3			unused
 *	Return:
 *			0			success
 *			< 0			error code
 *
 ****************************************************************************/
static SINT32 MaSndDrv_HvProsodicBendRange
(
	SINT32	sdSeqId,					/* sequence id */
	SINT32	sdDeltaTime,				/* delta time */
	UINT32	dCh,						/* channel number */
	UINT32	dRange,						/* prosodic bend range */
	UINT32	dP3							/* unused */
)
{
	SINT32 sdResult;
	
	MASNDDRV_DBGMSG((" MaSndDrv_HvProsodicBendRange: id=%ld dt=%ld ch=%ld br=%ld\n", sdSeqId, sdDeltaTime, dCh, dRange ));

	MASMW_ASSERT( sdSeqId == MASMW_SEQTYPE_HV );
	MASMW_ASSERT( dCh <= MASNDDRV_MAX_CHANNEL || dCh == MASNDDRV_HV_EXCHANNEL );
	MASMW_ASSERT( dRange <= MASNDDRV_MAX_HV_PROSODIC_BENDRANGE );

	(void)sdDeltaTime;
	(void)dCh;
	(void)dP3;

	sdResult = pfnHvFunc[gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bHvType].SetCommand( MASNDDRV_HV_PROSODIC_PITCH_BEND_SENS, MASNDDRV_HV_CHANNEL_NUMBER, dRange );
	if ( sdResult < 0 ) return MASMW_SUCCESS;

	return sdResult;
}
/****************************************************************************
 *	MaSndDrv_HvChannelVolume
 *
 *	Description:
 *			Set the volume of the HV
 *	Argument:
 *			sdSeqId		sequence id (2)
 *			sdDeltaTime	delta time [tick]
 *			dCh			channel number (127)
 *			dVol		volume value (0..127) default=127
 *			dP3			unused
 *	Return:
 *			0			success
 *			< 0			error code
 *
 ****************************************************************************/
static SINT32 MaSndDrv_HvChannelVolume
(
	SINT32	sdSeqId,					/* sequence id */
	SINT32	sdDeltaTime,				/* delta time */
	UINT32	dCh,						/* channel number */
	UINT32	dVol,						/* volume */
	UINT32	dP3							/* unused */
)
{
	UINT32 dNum = 0;
	SINT32 sdResult;
	UINT8 *pbPacketBuf;

	MASNDDRV_DBGMSG((" MaSndDrv_HvChannelVolume: id=%ld dt=%ld ch=%ld vol=%ld\n", sdSeqId, sdDeltaTime, dCh, dVol ));

	/* check arguments */

	MASMW_ASSERT( sdSeqId == MASMW_SEQTYPE_HV );
	MASMW_ASSERT( dCh <= MASNDDRV_MAX_CHANNEL || dCh == MASNDDRV_HV_EXCHANNEL );
	MASMW_ASSERT( dVol <= MASNDDRV_MAX_VOLUME );
	
	dVol &= MASNDDRV_MASK_VOLUME;

	(void)sdDeltaTime;
	(void)dP3;

	/* if dCh == ex channel then do Nop */
	if ( dCh != MASNDDRV_HV_EXCHANNEL )
	{
		return MASMW_SUCCESS;
	}
	
	/* Make packet data and send it */

	pbPacketBuf = gpMaSndDrvInfo->bPacketBuf[sdSeqId];
	MAKE_ADDRESS_PART( dNum, MAC_EXT_CH_VOL );
	MAKE_CHANNEL_VOLUME( dNum, dVol, 0 );

	sdResult = MaDevDrv_SendDirectPacket( &(pbPacketBuf[0]), dNum );

	return sdResult;
}
/****************************************************************************
 *	MaSndDrv_HvPanpot
 *
 *	Description:
 *			Set panpot of the HV
 *	Argument:
 *			sdSeqId		sequence id (2)
 *			sdDeltaTime	delta time [tick]
 *			dCh			channel number (127)
 *			dPan		panpot value (0..127) default=64
 *			dP3			unused
 *	Return:
 *			0			success
 *			< 0			error code
 *
 ****************************************************************************/
static SINT32 MaSndDrv_HvPanpot
(
	SINT32	sdSeqId,					/* sequence id */
	SINT32	sdDeltaTime,				/* delta time */
	UINT32	dCh,						/* channel number */
	UINT32	dPan,						/* pan */
	UINT32	dP3							/* unused */
)
{
	UINT32 dNum = 0;
	SINT32 sdResult;
	UINT8 *pbPacketBuf;

	MASNDDRV_DBGMSG((" MaSndDrv_HvPanpot: id=%ld dt=%ld ch=%ld pan=%ld\n", sdSeqId, sdDeltaTime, dCh, dPan ));

	/* check arguments */

	MASMW_ASSERT( sdSeqId == MASMW_SEQTYPE_HV );
	MASMW_ASSERT( dCh <= MASNDDRV_MAX_CHANNEL || dCh == MASNDDRV_HV_EXCHANNEL );
	MASMW_ASSERT( dPan <= MASNDDRV_MAX_PAN );
	
	dPan &= MASNDDRV_MASK_VOLUME;

	(void)sdDeltaTime;
	(void)dP3;

	/* if dCh == ex channel then do Nop */
	if ( dCh != MASNDDRV_HV_EXCHANNEL )
	{
		return MASMW_SUCCESS;
	}

	/* Make packet data and send it */

	pbPacketBuf = gpMaSndDrvInfo->bPacketBuf[sdSeqId];
	MAKE_ADDRESS_PART( dNum, MAC_EXT_CH_PAN )
	MAKE_PANPOT( dNum, dPan, 0 )

	sdResult = MaDevDrv_SendDirectPacket( &(pbPacketBuf[0]), dNum );

	return sdResult;
}
/****************************************************************************
 *	MaSndDrv_HvAllSoundOff
 *
 *	Description:
 *			All sound off for HV
 *	Argument:
 *			sdSeqId		sequence id (2)
 *			sdDeltaTime	delta time [tick]
 *			dCh			channel number (0..15, 127)
 *			dP2			unused
 *			dP3			unused
 *	Return:
 *			0			success
 *			< 0			error code
 *
 ****************************************************************************/
static SINT32 MaSndDrv_HvAllSoundOff
(
	SINT32	sdSeqId,					/* sequence id */
	SINT32	sdDeltaTime,				/* delta time */
	UINT32	dCh,						/* unused */
	UINT32	dP2,						/* unused */
	UINT32	dP3							/* unused */
)
{
	UINT32 dNum;
	SINT32 sdResult;
	UINT8 *pbPacketBuf;

	MASMW_ASSERT( sdSeqId == MASMW_SEQTYPE_HV );
	MASMW_ASSERT( dCh <= MASNDDRV_MAX_CHANNEL || dCh == MASNDDRV_HV_EXCHANNEL );

	(void)sdDeltaTime;
	(void)dP2;
	(void)dP3;

	dNum = 0;
	pbPacketBuf = gpMaSndDrvInfo->bPacketBuf[sdSeqId];
	MAKE_ADDRESS_PART( dNum, MAC_HV_CTRL );
	MAKE_RST_MUTE( dNum, (dCh & MASNDDRV_MASK_CHANNEL) );

	sdResult = MaDevDrv_SendDirectPacket( &(pbPacketBuf[0]), dNum );

	return MASMW_SUCCESS;
}
/****************************************************************************
 *	MaSndDrv_StreamSeek
 *
 *	Description:
 *			Set Seek point of stream
 *	Argument:
 *			sdSeqId		sequence id (1)
 *			sdDeltaTime	delta time [tick]
 *			dCh			channel number (0..15)
 *			dWaveId		wave id number (0..31)
 *			dSeekP		seek point (byte)
 *	Return:
 *			0			success
 *			< 0			error code
 *
 ****************************************************************************/
static SINT32 MaSndDrv_StreamSeek
(
	SINT32	sdSeqId,					/* sequence id */
	SINT32	sdDeltaTime,				/* delta time */
	UINT32	dCh,						/* unused */
	UINT32	dWaveId,					/* unused */
	UINT32	dSeekP						/* unused */
)
{
	SINT32 sdResult;

	MASNDDRV_DBGMSG((" MaSndDrv_StreamSeek: id=%ld dt=%ld ch=%ld id=%ld sp=%ld\n", sdSeqId, sdDeltaTime, dCh, dWaveId, dSeekP ));

	MASMW_ASSERT( dWaveId <= MASNDDRV_MAX_WAVEID );
	
	dWaveId &= MASNDDRV_MASK_WAVEID;

	(void)dCh;

	if ( gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].dSaNum != 0 )
	{
		gpMaSndDrvInfo->sStreamInfo[sdSeqId][dWaveId].dSeekP = dSeekP;
	}

	if ( sdDeltaTime > 0 )
		sdResult = MaSndDrv_Nop( sdSeqId, sdDeltaTime, 0, 0, 0 );
	else
		sdResult = MASMW_SUCCESS;

	return sdResult;
}
/****************************************************************************
 *	MaSndDrv_HvEvent
 *
 *	Description:
 *			Soft Int operation for HV
 *	Arguments:
 *			sdSeqId		sequence id (0..1)
 *			sdDeltaTime	delta time [tick]
 *			dVal		event value (0..63, 127)
 *			dP2			unused
 *			dP3			unused
 *	Return:
 *			0			success
 *			< 0			error code
 *
 ****************************************************************************/
static SINT32 MaSndDrv_HvEvent
(
	SINT32	sdSeqId,					/* sequence id */
	SINT32	sdDeltaTime,				/* delta time */
	UINT32	dVal,						/* value */
	UINT32	dP2,						/* unused */
	UINT32	dP3							/* unused */
)
{
	SINT32	sdResult;
	
	(void)sdDeltaTime;
	(void)dP2;							/* for unused warning message */
	(void)dP3;

	MASNDDRV_DBGMSG((" MaSndDrv_HvEvent: id=%ld dt=%ld val=%ld\n", sdSeqId, sdDeltaTime, dVal));

	/* check arguments */

	MASMW_ASSERT( ( dVal <= MASNDDRV_MAX_HVSID ) || ( dVal == MASNDDRV_HV_SOUND_OFF ) );
	
	if ( dVal != MASNDDRV_HV_SOUND_OFF )
	{
		dVal &= MASNDDRV_MASK_HVEVENT;
	}
	
	gpMaSndDrvInfo->dHvEventInfo[sdSeqId] = dVal | MASNDDRV_HV_EXIST;
	
	sdResult = MaSndDrv_Nop( sdSeqId, sdDeltaTime, 0, 0, 0 );

	return sdResult;
}
/****************************************************************************
 *	MaSndDrv_CoaseTune
 *
 *	Description:
 *			Sets the coase tune of the specified channel.
 *	Arguments:
 *			sdSeqId		sequence id (0..1)
 *			sdDeltaTime	delta time [tick]
 *			dCh			channel nubmer (0..15)
 *			dTune		tune valuei0..40h..7fhjdefault=40h
 *			dP3			unused
 *	Return:
 *			0			success
 *			< 0			error code
 *
 ****************************************************************************/
static SINT32 MaSndDrv_CoaseTune
(
	SINT32	sdSeqId,					/* sequence id */
	SINT32	sdDeltaTime,				/* delta time */
	UINT32	dCh,						/* channel number */
	UINT32	dTune,						/* tune */
	UINT32	dP3							/* unused */
)
{
	UINT32	dPitchBend;					/* pitch bend */
	UINT32	dTunePitch;					/* tune pitch */
	UINT32	dRegIndex;					/* register index number */
	SINT32	dNum;						/* number of packet data */
	SINT32	sdResult;					/* result of function */
	UINT8  *pbPacketBuf;

	(void)dP3;							/* for unused warning message */

	MASNDDRV_DBGMSG((" MaSndDrv_CoaseTune: id=%ld dt=%ld ch=%ld tune=%ld\n", sdSeqId, sdDeltaTime, dCh, dTune ));

	/* check arguments */

	MASMW_ASSERT( dTune <= MASNDDRV_MAX_COASETUNE );

	dTune &= MASNDDRV_MASK_COASETUNE;
	
	dTune = (UINT32)awCoaseTune[dTune];
	gpMaSndDrvInfo->sChInfo[dCh].dCoaseTune = dTune;
	dTunePitch = (dTune * gpMaSndDrvInfo->sChInfo[dCh].dFineTune) >> 10;
	if (dTunePitch > MASNDDRV_MAX_HW_PITCHBEND)
	{
		dTunePitch = MASNDDRV_MAX_HW_PITCHBEND;
	}
	gpMaSndDrvInfo->sChInfo[dCh].dTunePitch = dTunePitch;

	/* setting the bend value */

	dPitchBend = gpMaSndDrvInfo->sChInfo[dCh].dPitchBend;
	dPitchBend = (dPitchBend * dTunePitch) >> 10;
	if (dPitchBend > MASNDDRV_MAX_HW_PITCHBEND)
	{
		dPitchBend = MASNDDRV_MAX_HW_PITCHBEND;
	}

	/* Make packet data */

	pbPacketBuf = gpMaSndDrvInfo->bPacketBuf[sdSeqId];
	dNum = MakeDeltaTime( &(pbPacketBuf[0]), sdDeltaTime );

	dRegIndex = MAC_CHANNEL_BEND + dCh * 2;

	MAKE_ADDRESS_PART( dNum, dRegIndex )
	MAKE_PITCH_BEND( dNum, dPitchBend )

	if ( sdDeltaTime < 0 )				/* direct command */
	{
		sdResult = MaDevDrv_SendDirectPacket( &(pbPacketBuf[0]), dNum );
	}
	else								/* delayed command */
	{
		sdResult = SendDelayedPacket( &(pbPacketBuf[0]), dNum );
	}
	
	return sdResult;
}


/****************************************************************************
 *	MaSndDrv_FineTune
 *
 *	Description:
 *			Sets the fine tune of the specified channel.
 *	Arguments:
 *			sdSeqId		sequence id (0..1)
 *			sdDeltaTime	delta time [tick]
 *			dCh			channel nubmer (0..15)
 *			dTune		tune value (0..2000h..3FFFh) default=2000h
 *			dP3			unused
 *	Return:
 *			0			success
 *			< 0			error code
 *
 ****************************************************************************/
static SINT32 MaSndDrv_FineTune
(
	SINT32	sdSeqId,					/* sequence id */
	SINT32	sdDeltaTime,				/* delta time */
	UINT32	dCh,						/* channel number */
	UINT32	dTune,						/* tune */
	UINT32	dP3							/* unused */
)
{
	UINT32	dPitchBend;					/* pitch bend */
	UINT32	dTunePitch;					/* tune pitch */
	UINT32	dRegIndex;					/* register index number */
	SINT32	dNum;						/* number of packet data */
	SINT32	sdResult;					/* result of function */
	UINT8  *pbPacketBuf;

	(void)dP3;							/* for unused warning message */

	MASNDDRV_DBGMSG((" MaSndDrv_CoaseTune: id=%ld dt=%ld ch=%ld tune=%ld\n", sdSeqId, sdDeltaTime, dCh, dTune ));

	/* check arguments */

	MASMW_ASSERT( dTune <= MASNDDRV_MAX_FINETUNE );

	dTune &= MASNDDRV_MASK_FINETUNE;
	
	dTune = (UINT32)awFineTune[dTune >> 4];
	gpMaSndDrvInfo->sChInfo[dCh].dFineTune = dTune;
	dTunePitch = (dTune * gpMaSndDrvInfo->sChInfo[dCh].dCoaseTune) >> 10;
	if (dTunePitch > MASNDDRV_MAX_HW_PITCHBEND)
	{
		dTunePitch = MASNDDRV_MAX_HW_PITCHBEND;
	}
	gpMaSndDrvInfo->sChInfo[dCh].dTunePitch = dTunePitch;

	/* setting the bend value */

	dPitchBend = gpMaSndDrvInfo->sChInfo[dCh].dPitchBend;
	dPitchBend = (dPitchBend * dTunePitch) >> 10;
	if (dPitchBend > MASNDDRV_MAX_HW_PITCHBEND)
	{
		dPitchBend = MASNDDRV_MAX_HW_PITCHBEND;
	}

	/* Make packet data */

	pbPacketBuf = gpMaSndDrvInfo->bPacketBuf[sdSeqId];
	dNum = MakeDeltaTime( &(pbPacketBuf[0]), sdDeltaTime );

	dRegIndex = MAC_CHANNEL_BEND + dCh * 2;

	MAKE_ADDRESS_PART( dNum, dRegIndex )
	MAKE_PITCH_BEND( dNum, dPitchBend )

	if ( sdDeltaTime < 0 )				/* direct command */
	{
		sdResult = MaDevDrv_SendDirectPacket( &(pbPacketBuf[0]), dNum );
	}
	else								/* delayed command */
	{
		sdResult = SendDelayedPacket( &(pbPacketBuf[0]), dNum );
	}
	
	return sdResult;
}
/****************************************************************************
 *	MaSndDrv_SetCommand
 *
 *	Description:
 *			
 *	Arguments:
 *			sdSeqId		sequence id (0..2)
 *			sdDeltaTime	delta time [tick]
 *			dCmd		command number
 *			dParam1		parameter 1
 *			dParam2		parameter 2
 *			dParam3		parameter 3
 *	Return:
 *			0			success
 *			< 0			error code
 *
 ****************************************************************************/
SINT32 MaSndDrv_SetCommand
(
	SINT32	sdSeqId,					/* sequence id */
	SINT32	sdDeltaTime,				/* delta time */
	UINT32	dCmd,						/* command number */
	UINT32	dParam1,					/* parameter 1 */
	UINT32	dParam2,					/* parameter 2 */
	UINT32	dParam3						/* parameter 3 */
)
{
	SINT32	sdResult;					/* result of function */

	UINT32	dRegIndex;
	UINT32	dNum;
	UINT8  *pbPacketBuf;

	MASNDDRV_DBGMSG(("MaSndDrv_SetCommand: id=%ld dt=%ld cmd=%ld p1=%ld p2=%ld p3=%ld\n", sdSeqId, sdDeltaTime, dCmd, dParam1, dParam2, dParam3 ));

	MASMW_ASSERT( sdSeqId <= MASNDDRV_MAX_SEQTYPE );
	MASMW_ASSERT( gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bStatus != MASNDDRV_STATUS_IDLE );
	MASMW_ASSERT( dCmd <= MASNDDRV_MAX_COMMAND );

	if ( sdDeltaTime > 0 )
	{
		dNum = 0;
		pbPacketBuf = gpMaSndDrvInfo->bPacketBuf[sdSeqId];

		/* stream control */
		if ( gpMaSndDrvInfo->sStreamCtrlInfo.dQueFlag == 1 )
		{
			UpdateStreamStatus();

			if( gpMaSndDrvInfo->sStreamCtrlInfo.dType[0]!=0 )
			{
				/* make SoftInt#0 packet */
				dRegIndex = MAC_SOFTINT_RAM + 0;
				pbPacketBuf[dNum++] = (UINT8)( 0x80 );	/* Timer part */
				MAKE_ADDRESS_PART( dNum, dRegIndex );
				MAKE_SOFTINT_RAM( dNum, gpMaSndDrvInfo->sStreamCtrlInfo.dRamVal[0] );

				/* clear StreamQueInfo#0 */
				gpMaSndDrvInfo->sStreamCtrlInfo.dType[0] = 0;
			}
			if( gpMaSndDrvInfo->sStreamCtrlInfo.dType[1]!=0 )
			{
				/* make SoftInt#1 packet */
				dRegIndex = MAC_SOFTINT_RAM + 1;
				pbPacketBuf[dNum++] = (UINT8)( 0x80 );	/* Timer part */
				MAKE_ADDRESS_PART( dNum, dRegIndex );
				MAKE_SOFTINT_RAM( dNum, gpMaSndDrvInfo->sStreamCtrlInfo.dRamVal[1] );

				/* clear StreamQueInfo#1 */
				gpMaSndDrvInfo->sStreamCtrlInfo.dType[1] = 0;
			}
		
			/* clear dQueFlag */
			gpMaSndDrvInfo->sStreamCtrlInfo.dQueFlag = 0;
		}
		
		/* HV event control */
		if ( gpMaSndDrvInfo->dHvEventInfo[sdSeqId] != 0 )
		{
			/* make SoftInt#4 packet */
			dRegIndex = MAC_SOFTINT_RAM_4;
			pbPacketBuf[dNum++] = (UINT8)( 0x80 );	/* Timer part */
			MAKE_ADDRESS_PART( dNum, dRegIndex );
			MAKE_SOFTINT_RAM( dNum, (gpMaSndDrvInfo->dHvEventInfo[sdSeqId] & 0x7F) );

			/* clear HvEventInfo */
			gpMaSndDrvInfo->dHvEventInfo[sdSeqId] = 0;
		}
		
		if( dNum!=0 ) {
			/* send SoftInt packet */
			sdResult = SendDelayedPacket( &(pbPacketBuf[0]), dNum );
			if( sdResult < MASMW_SUCCESS ) 
			{
				return sdResult;
			}
		}
	}

	if ( dCmd > MASNDDRV_MAX_COMMAND )
	{	/* Command Error */
		if ( sdDeltaTime > 0 )
		{
			dCmd = MASNDDRV_CMD_NOP;
			dParam1 = 0;						/* NOP_1 */
		}
		else
		{
			return MASMW_SUCCESS;
		}
	}
	else if ( bCmdJudge[sdSeqId][dCmd] == 0 )
	{	/* Sequence Type Error */
		if ( sdDeltaTime > 0 )
		{
			dCmd = MASNDDRV_CMD_NOP;
			dParam1 = 0;						/* NOP_1 */
		}
		else
		{
			return MASMW_SUCCESS;
		}
	}
	else if ( bChannelCmd[dCmd] != 0 )
	{
		MASMW_ASSERT( dParam1 <= MASNDDRV_MAX_CHANNEL );

		dParam1 &= MASNDDRV_MASK_CHANNEL;

		if ( ( gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].dChMap & (UINT32)(1 << dParam1) ) == 0 )
		{
			if ( sdDeltaTime > 0 )
			{
				dCmd = MASNDDRV_CMD_NOP;
				dParam1 = 0;						/* NOP_1 */
			}
			else
			{
				return MASMW_SUCCESS;
			}
		}
	}

	sdResult = ma_snddrv_command[dCmd]( sdSeqId, sdDeltaTime, dParam1, dParam2, dParam3 );

	return sdResult;
}
/****************************************************************************
 *	MaSndDrv_SetHVVoice
 *
 *	Description:
 *			Set timber of HV voice
 *	Argument:
 *			sdSeqId		sequence id (2)
 *			dProgNo		program number (128..143)
 *			pbVoice		pointer to the area of HV parameter
 *			dSize		size of the area of HV parameter
 *	Return:
 *			0			success
 *			< 0			error code
 *
 ****************************************************************************/
SINT32 MaSndDrv_SetHVVoice
(
	SINT32	sdSeqId,					/* sequence id */
	UINT32	dProgNo,					/* program number */
	UINT8  *pbVoice,					/* pointer to param */
	UINT32	dSize						/* size of param */
)
{
	SINT32 sdResult;

	MASNDDRV_DBGMSG(("MaSndDrv_SetHVVoice: id=%ld pb=%d\n", sdSeqId, dProgNo));

	/* check arguments */

	if ( sdSeqId != MASMW_SEQTYPE_HV )			return MASMW_ERROR_ID;
	if ( gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bStatus == MASNDDRV_STATUS_IDLE )	return MASMW_ERROR_ID;
	if ( dSize != MASNDDRV_MAX_HVVOICE_SIZE )		return MASMW_ERROR_ARGUMENT;

	sdResult = pfnHvFunc[gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bHvType].SetCommand( MASNDDRV_HV_SET_VOICE, dProgNo, (UINT32)pbVoice );
	if ( sdResult < 0 )
	{
#if MASMW_SRMCNV_HVS
		switch ( sdResult )
		{
		  case MAHV_ERROR:
		  	return MASMW_ERROR;

		  case MAHV_ERROR_ARGUMENT:
		  case MAHV_ERROR_DATA:
		  case MAHV_ERROR_NODICTIONARY:
		  	return MASMW_ERROR_ARGUMENT;

		  default:
		  	return MASMW_ERROR;
		}
#else
		return MASMW_ERROR;
#endif
	}

	return MASMW_SUCCESS;
}
/****************************************************************************
 *	MaSndDrv_SetHVDictionary
 *
 *	Description:
 *			Set extension TTS Dictionary for HV
 *	Argument:
 *			sdSeqId		sequence id (2)
 *			pbDict		pointer to the area of ex dictionary
 *			dSize		size of the area of ex dictionary
 *			dMode		mode  0:check data 1:register
 *			pvExtArgs	extension argument
 *	Return:
 *			0			success
 *			< 0			error code
 *
 ****************************************************************************/
SINT32 MaSndDrv_SetHVDictionary
(
	SINT32	sdSeqId,					/* sequence id */
	UINT8  *pbDict,						/* pointer to param */
	UINT32	dSize,						/* size of param */
	UINT32	dMode,						/* mode */
	void *pvExtArgs						/* extra parameter */
)
{
	SINT32 sdResult;

	MASNDDRV_DBGMSG(("MaSndDrv_SetHVDictionary: id=%ld\n", sdSeqId));

	if ( sdSeqId != MASMW_SEQTYPE_HV )		return MASMW_ERROR_ID;
	if ( gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bStatus == MASNDDRV_STATUS_IDLE )	return MASMW_ERROR_ID;

	sdResult = pfnHvFunc[gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bHvType].SetDictionary( pbDict, dSize, dMode, pvExtArgs );
	if ( sdResult < 0 )
	{
#if MASMW_SRMCNV_HVS
		switch ( sdResult )
		{
		  case MAHV_ERROR: 	return MASMW_ERROR;
		  default: 			return MASMW_ERROR;
		}
#else
		return MASMW_ERROR;
#endif
	}

	return MASMW_SUCCESS;
}
/****************************************************************************
 *	MaSndDrv_SetALVoice
 *
 *	Description:
 *			Set timber of AL voice
 *	Argument:
 *			sdSeqId		sequence id (0..1)
 *			dBankNo		bank number (0..15: melody, 128..143: drum)
 *			dProgNo		program (key) number (0..127)
 *			pbParam		pointer to the area of AL parameter
 *			dParamSize	size of the area of AL parameter
 *	Return:
 *			0			success
 *			< 0			error code
 *
 ****************************************************************************/
SINT32 MaSndDrv_SetALVoice
(
	SINT32	sdSeqId,					/* sequence id */
	UINT32	dBankNo,					/* bank number */
	UINT32	dProgNo,					/* program number */
	UINT8  *pbParam,					/* pointer to param */
	UINT32	dParamSize					/* size of param */
)
{
	UINT32	dVoiceNo;

	MASNDDRV_DBGMSG(("MaSndDrv_SetALVoice: id=%ld bn=%d pb=%d\n", sdSeqId, dBankNo, dProgNo));

	/* check arguments */

	if ( sdSeqId > MASMW_SEQTYPE_DIRECT )		return MASMW_ERROR_ID;
	if ( gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bStatus == MASNDDRV_STATUS_IDLE )	return MASMW_ERROR_ID;
	if ( (dBankNo & 0x7f) >= MASNDDRV_NUM_SETBANK ) 	return MASMW_ERROR_ARGUMENT;
	if ( dProgNo > MASNDDRV_MAX_PROGRAM )			return MASMW_ERROR_ARGUMENT;

	if ( pbParam == NULL )
	{
		/* delete voice */
		for ( dVoiceNo = 0; dVoiceNo < MASNDDRV_NUM_ALVOICE; dVoiceNo++ )
		{
			if ( ( gpMaSndDrvInfo->sAlVoiceInfo[dVoiceNo].dProgNo == dProgNo )
				&& ( gpMaSndDrvInfo->sAlVoiceInfo[dVoiceNo].dBankNo == dBankNo ) )
			{
				gpMaSndDrvInfo->sAlVoiceInfo[dVoiceNo].dBankNo = 0xFFFF;
				gpMaSndDrvInfo->sAlVoiceInfo[dVoiceNo].dProgNo = 0xFFFF;
				gpMaSndDrvInfo->sAlVoiceInfo[dVoiceNo].pAlVoice = NULL;
				gpMaSndDrvInfo->sAlVoiceInfo[dVoiceNo].dAlVoiceSize = 0;
				break; /* for */
			}
		}
		if ( dVoiceNo == MASNDDRV_NUM_ALVOICE )
		{
			return MASMW_ERROR;
		}
	}
	else
	{
		for ( dVoiceNo = 0; dVoiceNo < MASNDDRV_NUM_ALVOICE; dVoiceNo++ )
		{
			if ( gpMaSndDrvInfo->sAlVoiceInfo[dVoiceNo].pAlVoice == NULL)
			{
				gpMaSndDrvInfo->sAlVoiceInfo[dVoiceNo].dBankNo = dBankNo;
				gpMaSndDrvInfo->sAlVoiceInfo[dVoiceNo].dProgNo = dProgNo;
				gpMaSndDrvInfo->sAlVoiceInfo[dVoiceNo].pAlVoice = pbParam;
				gpMaSndDrvInfo->sAlVoiceInfo[dVoiceNo].dAlVoiceSize = dParamSize;
				break; /* for */
			}
		}
		if ( dVoiceNo == MASNDDRV_NUM_ALVOICE )
		{
			return MASMW_ERROR_RESOURCE_OVER;
		}
	}

	return MASMW_SUCCESS;
}
/****************************************************************************
 *	MaSndDrv_SeekControl
 *
 *	Description:
 *			seek.
 *	Arguments:
 *			sdSeqId		sequence id
 *	Return:
 *			0			success
 *			< 0			error code
 *
 ****************************************************************************/
static SINT32 MaSndDrv_SeekControl
(
	SINT32	sdSeqId						/* sequence id */
)
{
	UINT32	dTimeOut;					/* time out count */
	UINT32	dBufSize;					/* size of buffer */
	SINT32	sdResult;					/* result of function */

	MASNDDRV_DBGMSG(("MaSndDrv_SeekControl: id=%ld\n", sdSeqId ));

	/* get buffer pointer */
	gpMaSndDrvInfo->sSeqbufInfo.pbBufPtr  = MaDevDrv_GetSeekBuffer( &dBufSize );
	gpMaSndDrvInfo->sSeqbufInfo.dBufSize = (UINT16)dBufSize;

	/* clear FIFO */
	MaDevDrv_ClearFifo();

	/* call converter to make seek control data */
	sdResult = 1;
	dTimeOut = 1024;
	while ( ( sdResult > 0 ) && ( sdResult != 0xFFFF ) && ( --dTimeOut !=0 ) )
	{
		sdResult = gpMaSndDrvInfo->pfnSrmCnv[sdSeqId]();
	}

	/* send control data created by converter */
	sdResult = MaDevDrv_SeekControl( gpMaSndDrvInfo->sSeqbufInfo.dWroteSize );

	/* clear FIFO */
	MaDevDrv_ClearFifo();

	return MASMW_SUCCESS;
}
/****************************************************************************
 *	MaSndDrv_ControlSequencer
 *
 *	Description:
 *			control sequencer
 *	Arguments:
 *			sdSeqId		sequence id
 *			dCtrl		control value
 *	Return:
 *			0			success
 *			< 0			error code
 ****************************************************************************/
SINT32 MaSndDrv_ControlSequencer
(
	SINT32	sdSeqId,					/* sequence id */
	UINT32	dCtrl						/* control value */
)
{
	UINT32	dSlotNo;
	UINT32	dCh;
	UINT32	dSaId;
	UINT32	dNum;
	UINT32	dRegIndex;
	UINT32	dResMap;
	UINT32	dAltMode;
	SINT32	sdResult;
	UINT8  *pbPacketBuf;

	MASNDDRV_DBGMSG(("MaSndDrv_ControlSequencer: id=%ld ctrl=%d\n", sdSeqId, dCtrl ));

	if ( sdSeqId > MASNDDRV_MAX_SEQTYPE )										return MASMW_ERROR_ID;
	if ( gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bStatus == MASNDDRV_STATUS_IDLE )	return MASMW_ERROR_ID;

	switch ( dCtrl )
	{
	case MASNDDRV_SEQUENCER_STOP:	/* stop */

		if ( ( gpMaSndDrvInfo->bCtrlSequencer[sdSeqId] == 0 )
		  && ( gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bStatus != MASNDDRV_STATUS_REGISTERED ) )
		{
			MaDevDrv_StopSequencer( sdSeqId );

			dNum = 0;
			pbPacketBuf = gpMaSndDrvInfo->bPacketBuf[sdSeqId];

			if ( sdSeqId <= MASMW_SEQTYPE_DIRECT )
			{
				/* FM all slot sound off */
				dResMap = gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].dFmMap;
				for ( dSlotNo = 0; dSlotNo < MA_MAX_FM_SLOT; dSlotNo++ )
				{
					if ( ( dResMap & (1 << dSlotNo) ) != 0 )
					{
						if ( dSlotNo < 16 )
							dRegIndex = MAC_FM_VOICE_ADDRESS + dSlotNo * 6 + 5;
						else
							dRegIndex = MAC_EXT_FM_VOICE_ADDRESS + (dSlotNo & 0x0F) * 6 + 5 ;

						sdResult = MaDevDrv_ReceiveData( dRegIndex ) & 0x0F ;
						if ( sdResult < MASMW_SUCCESS )
						{
							return sdResult;
						}
						dCh = sdResult;

						/* KeyOff & MUTE & RST */
						MAKE_ADDRESS_PART( dNum, dRegIndex )
						MAKE_RST_MUTE( dNum, dCh )
					}
				}

				/* WT all slot sound off */
				dResMap = gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].dWtMap;
				for ( dSlotNo = 0; dSlotNo < MA_MAX_WT_SLOT; dSlotNo++ )
				{
					if ( ( dResMap & (1 << dSlotNo) ) != 0 )
					{
						if ( dSlotNo < 16 )
							dRegIndex = MAC_WT_VOICE_ADDRESS + dSlotNo * 6 + 5 ;
						else
							dRegIndex = MAC_EXT_WT_VOICE_ADDRESS + (dSlotNo & 0x0F) * 6 + 5;

						sdResult = MaDevDrv_ReceiveData( dRegIndex ) & 0x0F ;
						if ( sdResult < MASMW_SUCCESS )
						{
							return sdResult;
						}
						dCh = sdResult;

						/* KeyOff & MUTE & RST */
						MAKE_ADDRESS_PART( dNum, dRegIndex )
						MAKE_RST_MUTE( dNum, dCh )
					}
				}

				/* AL LPF clear */
				if ( gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].dAlMap != 0 )
				{
					MAKE_ADDRESS_PART( dNum, MAC_LPF_BASE )
					MAKE_LPFCLR_OFF( dNum )
				}

				if ( dNum != 0 )
				{
					MaDevDrv_SendDirectPacket( &(pbPacketBuf[0]), dNum );
				}

				/* SA all slot sound off */
				for ( dSaId = 0; dSaId < MA_MAX_STREAM_AUDIO; dSaId++ )
				{
					if ( ( gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].dSaMap & (UINT32)(1 << dSaId) ) != 0 )
					{
						/* Stream Off */
						MaDevDrv_StreamHandler( sdSeqId, dSaId, 0, 0x00, 0 );
					}
				}

			}
			else if ( sdSeqId == MASMW_SEQTYPE_HV )
			{
				dRegIndex = MAC_HV_CTRL;
				MAKE_ADDRESS_PART( dNum, dRegIndex );
				MAKE_RST_MUTE( dNum, 0 );

				sdResult = MaDevDrv_SendDirectPacket( &(pbPacketBuf[0]), dNum );
			}
		}

		gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bStatus = MASNDDRV_STATUS_REGISTERED;

		sdResult = MASMW_SUCCESS;
		break;

	case MASNDDRV_SEQUENCER_START:	/* start */

		gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bStatus = MASNDDRV_STATUS_PLAYING;

		if ( gpMaSndDrvInfo->bCtrlSequencer[sdSeqId] == 0 )
		{
			if ( sdSeqId == MASMW_SEQTYPE_DELAYED )
			{
				if ( gpMaSndDrvInfo->sSeqbufInfo.bSequenceFlag == (UINT8)(MA_INT_POINT - 2) )
				{
					MaDevDrv_Fifo( gpMaSndDrvInfo->sSeqbufInfo.bSequenceFlag );
				}
				gpMaSndDrvInfo->sSeqbufInfo.bSequenceFlag = 0;
			}
			MaDevDrv_StartSequencer( sdSeqId );
		}
		sdResult = MASMW_SUCCESS;
		break;

	case MASNDDRV_SEQUENCER_ENTERREPEAT:

		gpMaSndDrvInfo->bCtrlSequencer[sdSeqId] = 1;
		sdResult = MASMW_SUCCESS;
		break;

	case MASNDDRV_SEQUENCER_LEAVEREPEAT:

		gpMaSndDrvInfo->bCtrlSequencer[sdSeqId] = 0;
		sdResult = MASMW_SUCCESS;
		break;

	case MASNDDRV_SEQUENCER_SEEK:	/* seek */

		if ( sdSeqId <= MASMW_SEQTYPE_DIRECT )
		{

			for ( dSlotNo = 0; dSlotNo < gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].dFmNum; dSlotNo++ )
			{
				if ( ( gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].dFmMap & (UINT32)(1 << dSlotNo) ) != 0 )
				{
					gpMaSndDrvInfo->sSlotInfo[dSlotNo].bCh   = 0;
				}
			}
			for ( dSlotNo = 0; dSlotNo < gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].dWtNum; dSlotNo++ )
			{
				if ( ( gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].dWtMap & (UINT32)(1 << dSlotNo) ) != 0 )
				{
					gpMaSndDrvInfo->sSlotInfo[dSlotNo+MASNDDRV_MIN_WT_SLOT].bCh   = 0;
				}
			}
			for ( dSlotNo = 0; dSlotNo < gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].dSaNum; dSlotNo++ )
			{
				if ( ( gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].dSaMap & (UINT32)(1 << dSlotNo) ) != 0 )
				{
					gpMaSndDrvInfo->sSlotInfo[dSlotNo+MASNDDRV_MIN_STM_SLOT].bCh   = 0;
				}
			}

			for ( dCh = 0; dCh < gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].dChNum; dCh++ )
			{
				if ( ( gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].dChMap & (UINT32)(1 << dCh) ) != 0 )
				{
					gpMaSndDrvInfo->sChInfo[dCh].bNoteOn = 0;
				}
			}

			MaDva_End();
			if ( gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bDrumMode == MASNDDRV_RESMODE1_NORMAL )
			{
				dAltMode = 1;
			}
			else
			{
				dAltMode = 0;
			}
			MaDva_Initialize((UINT32)(gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].dAlNum),
							 (UINT32)(gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].dFmNum), 
							 (UINT32)(gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].dWtNum), 
							 (UINT32)(gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].dSaNum), dAltMode);

			/* clear QueInfo */
			gpMaSndDrvInfo->sStreamCtrlInfo.dQueFlag = 0;
			gpMaSndDrvInfo->sStreamCtrlInfo.dType[0] = 0;
			gpMaSndDrvInfo->sStreamCtrlInfo.dType[1] = 0;
			gpMaSndDrvInfo->sStreamCtrlInfo.dWaveId[0] = 0;
			gpMaSndDrvInfo->sStreamCtrlInfo.dWaveId[1] = 0;
			gpMaSndDrvInfo->sStreamCtrlInfo.dStreamStatus = 0;
			
			/* clear HvEventInfo */
			gpMaSndDrvInfo->dHvEventInfo[sdSeqId] = 0;
		}

		if ( sdSeqId == MASMW_SEQTYPE_DELAYED )
		{

			if ( gpMaSndDrvInfo->bCtrlSequencer[sdSeqId] == 0 )
			{
				gpMaSndDrvInfo->sSndDrvInfo[MASMW_SEQTYPE_DELAYED].dEosFlag = 0;

				gpMaSndDrvInfo->sSeqbufInfo.pbBufPtr		= NULL;
				gpMaSndDrvInfo->sSeqbufInfo.dBufSize	= 0UL;
				gpMaSndDrvInfo->sSeqbufInfo.bQueueFlag 	= (UINT8)0;
				gpMaSndDrvInfo->sSeqbufInfo.dQueueSize	= 0UL;
				gpMaSndDrvInfo->sSeqbufInfo.dQueueWPtr	= 0UL;
				gpMaSndDrvInfo->sSeqbufInfo.dWroteSize	= 0UL;

				/* volume interpolation OFF */
				for ( dCh = 0; dCh < gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].dChNum; dCh++ )
				{
					if ( ( gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].dChMap & (1 << dCh) ) != 0 )
					{
						gpMaSndDrvInfo->sChInfo[dCh].bVolInterp = 0;
					}
				}

				/* do events for seek */
				MaSndDrv_SeekControl( sdSeqId );

				gpMaSndDrvInfo->sSeqbufInfo.pbBufPtr		= NULL;
				gpMaSndDrvInfo->sSeqbufInfo.dBufSize	= 0UL;
				gpMaSndDrvInfo->sSeqbufInfo.bQueueFlag 	= (UINT8)0;
				gpMaSndDrvInfo->sSeqbufInfo.dQueueSize	= 0UL;
				gpMaSndDrvInfo->sSeqbufInfo.dQueueWPtr	= 0UL;
				gpMaSndDrvInfo->sSeqbufInfo.dWroteSize	= 0UL;

				gpMaSndDrvInfo->sSeqbufInfo.bSequenceFlag = (UINT8)(MA_INT_POINT - 2);

				/* volume interpolation ON */
				for ( dCh = 0; dCh < gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].dChNum; dCh++ )
				{
					if ( ( gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].dChMap & (1 << dCh) ) != 0 )
					{
						gpMaSndDrvInfo->sChInfo[dCh].bVolInterp = 1;
					}
				}
				
				/* reset the sequence counter */
				MaSndDrv_ResetPos( sdSeqId );

			}
		}
		sdResult = MASMW_SUCCESS;
		break;

	default:
		sdResult = MASMW_ERROR;
		break;
	}

	return sdResult;
}
/****************************************************************************
 *	MaSndDrv_SetFmExtWave
 *
 *	Description:
 *			register the FM extra wave
 *	Arguments:
 *			sdSeqId		sequencer id
 *			dWaveNo		wave number
 *			dRamAdrs	internal RAM address
 *	Return:
 *			0			success
 *			< 0			error code
 *
 ****************************************************************************/
SINT32 MaSndDrv_SetFmExtWave
(
	SINT32	sdSeqId,					/* sequence id */
	UINT32	dWaveNo,					/* wave number */
	UINT32	dRamAdrs					/* ram address */
)
{
	UINT8	bPacket[4];
	UINT32	dRegIndex;
	SINT32	sdResult;

	MASNDDRV_DBGMSG(("MaSndDrv_SetFmExtVoice: id=%ld wn=%d ad=%08lx\n", sdSeqId, dWaveNo, dRamAdrs));

	if ( sdSeqId > MASNDDRV_MAX_SEQTYPE )							return MASMW_ERROR_ID;
	if ( gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bStatus == MASNDDRV_STATUS_IDLE )	return MASMW_ERROR_ID;

	if ( dRamAdrs == 0xFFFFFFFF ) return MASMW_SUCCESS;

	switch ( dWaveNo )
	{
	  case 15:	dRegIndex = MAC_FM_EXTWAVE_15;	break;
	  case 23:	dRegIndex = MAC_FM_EXTWAVE_23;	break;
	  case 31:	dRegIndex = MAC_FM_EXTWAVE_31;	break;
	  default:	return MASMW_ERROR_ARGUMENT;
	}

	dRamAdrs >>= 1;

	bPacket[0] = (UINT8)(  dRegIndex       & 0x7F );		/* address part */
	bPacket[1] = (UINT8)( (dRegIndex >> 7) | 0x80 );
	bPacket[2] = (UINT8)( (dRamAdrs >> 7)  & 0x7F );		/* data part */
	bPacket[3] = (UINT8)(  dRamAdrs        | 0x80 );

	sdResult = MaDevDrv_SendDirectPacket( bPacket, 4 );

	return sdResult;
}
/****************************************************************************
 *	MaSndDrv_SetBalance
 *
 *	Description:
 *			Set volume balance of specified sequence.
 *	Argument:
 *			sdSeqId		sequence id
 *			dLVol		L-ch volume (0..127)
 *			dRVol		R-ch volume (0..127)
 *	Return:
 *			0			success
 *			< 0			error code
 *
 ****************************************************************************/
SINT32 MaSndDrv_SetBalance
(
	SINT32	sdSeqId,					/* sequence id */
	UINT32	dLVol,						/* L-ch volume */
	UINT32	dRVol						/* R-ch volume */
)
{
	UINT8 bVolValL;
	UINT8 bVolValR;

	MASNDDRV_DBGMSG(("MaSndDrv_SetBalance: id=%ld vl=%d vr=%d\n", sdSeqId, dLVol, dRVol));

	if ( sdSeqId != MASMW_SEQTYPE_DELAYED ) return MASMW_ERROR_ID;
	if ( dLVol > MASNDDRV_MAX_VOLUME ) return MASMW_ERROR_ARGUMENT;
	if ( dRVol > MASNDDRV_MAX_VOLUME ) return MASMW_ERROR_ARGUMENT;

	bVolValL = (UINT8)(((dLVol << 1) & 0xF8) | gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bMasterVolShiftL);
	bVolValR = (UINT8)(((dRVol << 1) & 0xF8) | gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bMasterVolShiftR);

	MaDevDrv_WriteIntermediateReg( MAI_MASTER_VOLL, bVolValL );
	MaDevDrv_WriteIntermediateReg( MAI_MASTER_VOLR, bVolValR );

	gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bBalVolumeL = (UINT8)dLVol;
	gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bBalVolumeR = (UINT8)dRVol;

	return MASMW_SUCCESS;
}
/****************************************************************************
 *	MaSndDrv_SetVolume
 *
 *	Description:
 *			Set volume of specified sequence.
 *	Argument:
 *			sdSeqId		sequence id
 *			dVolume		volume (0..127)
 *			dCtrl		voice type map
 *	Return:
 *			0			success
 *			< 0			error code
 *
 ****************************************************************************/
SINT32 MaSndDrv_SetVolume
(
	SINT32	sdSeqId,					/* sequence id */
	UINT32	dVolume,					/* volume */
	UINT32	dCtrl						/* voice type map */
)
{
	UINT8 bVolVal;

	MASNDDRV_DBGMSG(("MaSndDrv_SetVolume: id=%ld vl=%d ct=%x\n", sdSeqId, dVolume, dCtrl));

	if ( sdSeqId > MASNDDRV_MAX_SEQTYPE ) return MASMW_ERROR_ID;
	if ( dVolume > MASNDDRV_MAX_VOLUME ) return MASMW_ERROR_ARGUMENT;
	if ( dCtrl > MASNDDRV_MAX_VOICE_TYPE ) return MASMW_ERROR_ARGUMENT;

	if ( dCtrl & MASNDDRV_FM_FLAG_BIT )
	{
		bVolVal = (UINT8)(((dVolume << 1) & 0xF8) | gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bFmVolShift);
		MaDevDrv_WriteIntermediateReg( MAI_FM_VOL, bVolVal );
		gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bFmVolume = (UINT8)dVolume;
	}
	if ( dCtrl & MASNDDRV_WT_FLAG_BIT )
	{
		bVolVal = (UINT8)(((dVolume << 1) & 0xF8) | gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bWtVolShift);
		MaDevDrv_WriteIntermediateReg( MAI_WT_VOL, bVolVal );
		gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bWtVolume = (UINT8)dVolume;
	}
	if ( dCtrl & MASNDDRV_ST_FLAG_BIT )
	{
		bVolVal = (UINT8)(((dVolume << 1) & 0xF8) | gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bSaVolShift);
		MaDevDrv_WriteIntermediateReg( MAI_STM_VOL, bVolVal );
		gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bSaVolume = (UINT8)dVolume;
	}
	if ( dCtrl & MASNDDRV_HV_FLAG_BIT )
	{
		bVolVal = (UINT8)(((dVolume << 1) & 0xF8) | gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bHvVolShift);
		MaDevDrv_WriteIntermediateReg( MAI_HV_VOL, bVolVal );
		gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bHvVolume = (UINT8)dVolume;
	}

	return MASMW_SUCCESS;
}
/****************************************************************************
 *	MaSndDrv_SetVolShift
 *
 *	Description:
 *			Set volume shift of specified sequence.
 *	Argument:
 *			sdSeqId		sequence id
 *			dVal		shift value (0..3)
 *			dCtrl		voice type
 *	Return:
 *			0			success
 *			< 0			error code
 *
 ****************************************************************************/
SINT32 MaSndDrv_SetVolShift
(
	SINT32	sdSeqId,					/* sequence id */
	UINT32	dVal,						/* shift value */
	UINT32	dCtrl						/* voice type map */
)
{
	UINT8 bVolVal;

	MASNDDRV_DBGMSG(("MaSndDrv_SetVolShift: id=%ld vl=%d ct=%x\n", sdSeqId, dVal, dCtrl));

	if ( sdSeqId > MASNDDRV_MAX_SEQTYPE ) return MASMW_ERROR_ID;
	if ( dVal > MASNDDRV_MAX_VOLUME_SHIFT ) return MASMW_ERROR_ARGUMENT;
	if ( dCtrl > MASNDDRV_MAX_VOICE_TYPE ) return MASMW_ERROR_ARGUMENT;

	if ( dCtrl & MASNDDRV_FM_FLAG_BIT )
	{
		bVolVal = (UINT8)(((gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bFmVolume << 1) & 0xF8) | dVal );
		MaDevDrv_WriteIntermediateReg( MAI_FM_VOL, bVolVal );
		gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bFmVolShift	= (UINT8)dVal;
	}
	if ( dCtrl & MASNDDRV_WT_FLAG_BIT )
	{
		bVolVal = (UINT8)(((gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bWtVolume << 1) & 0xF8) | dVal );
		MaDevDrv_WriteIntermediateReg( MAI_WT_VOL, bVolVal );
		gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bWtVolShift	= (UINT8)dVal;
	}
	if ( dCtrl & MASNDDRV_ST_FLAG_BIT )
	{
		bVolVal = (UINT8)(((gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bSaVolume << 1) & 0xF8) | dVal );
		MaDevDrv_WriteIntermediateReg( MAI_STM_VOL, bVolVal );
		gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bSaVolShift	= (UINT8)dVal;
	}
	if ( dCtrl & MASNDDRV_HV_FLAG_BIT )
	{
		bVolVal = (UINT8)(((gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bHvVolume << 1) & 0xF8) | dVal );
		MaDevDrv_WriteIntermediateReg( MAI_HV_VOL, bVolVal );
		gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bHvVolShift	= (UINT8)dVal;
	}
	if ( dCtrl & MASNDDRV_MASTERL_FLAG_BIT )
	{
		bVolVal = (UINT8)(((gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bBalVolumeL << 1) & 0xF8) | dVal );
		MaDevDrv_WriteIntermediateReg( MAI_MASTER_VOLL, bVolVal );
		gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bMasterVolShiftL = (UINT8)dVal;
	}
	if ( dCtrl & MASNDDRV_MASTERR_FLAG_BIT )
	{
		bVolVal = (UINT8)(((gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bBalVolumeR << 1) & 0xF8) | dVal );
		MaDevDrv_WriteIntermediateReg( MAI_MASTER_VOLR, bVolVal );
		gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bMasterVolShiftR = (UINT8)dVal;
	}

	return MASMW_SUCCESS;
}
/****************************************************************************
 *	MaSndDrv_SetVoice
 *
 *	Description:
 *			Set timber of voice
 *	Argument:
 *			sdSeqId		sequence id
 *			dBankNo		bank number (0..15: melody, 128..143: drum)
 *			dProgNo		program (key) number (0..127)
 *			dSynth		synth mode (1:FM, 2:WT)
 *			dKey		key number of drum (FM:0..127 WT:Frequency of C4)
 *			dRamAdrs	internal RAM address
 *	Return:
 *			0			success
 *			< 0			error code
 *
 ****************************************************************************/
SINT32 MaSndDrv_SetVoice
(
	SINT32	sdSeqId,					/* sequence id */
	UINT32	dBankNo,					/* bank number */
	UINT32	dProgNo,					/* program number */
	UINT32	dSynth,						/* synth type */
	UINT32	dKey,						/* key number */
	UINT32	dRamAdrs					/* ram address */
)
{
	UINT32	dState = 0;
	UINT32	dBankProg;					/* bank & program number */
	UINT32	i;
	SINT32	sdResult = MASMW_SUCCESS;
	UINT32	dRealKey;

	MASNDDRV_DBGMSG(("MaSndDrv_SetVoice: id=%ld bn=%d pb=%d sy=%d key=%ld ad=%08lx\n", sdSeqId, dBankNo, dProgNo, dSynth, dKey, dRamAdrs));

	/* check arguments */

	if ( sdSeqId > MASMW_SEQTYPE_DIRECT )			return MASMW_ERROR_ID;
	if ( gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bStatus == MASNDDRV_STATUS_IDLE )	return MASMW_ERROR_ID;
	if ((dBankNo & 0x7f) >= MASNDDRV_MAX_RAM_BANK) 	return MASMW_ERROR_ARGUMENT;
	if ( dProgNo > MASNDDRV_MAX_PROGRAM )			return MASMW_ERROR_ARGUMENT;
	if ( dSynth > MASNDDRV_WT_VOICE )				return MASMW_ERROR_ARGUMENT;
	if ( (dSynth == MASNDDRV_FM_VOICE) && (dKey > MASNDDRV_MAX_KEY ) )	return MASMW_ERROR_ARGUMENT;
	if ( ( dRamAdrs != 0xFFFFFFFF ) && ( ( dRamAdrs < MASNDDRV_MIN_RAMADRS ) || ( MASNDDRV_MAX_RAMADRS < dRamAdrs ) ) )
		return MASMW_ERROR_ARGUMENT;

	if ( dBankNo < 128 )
	{
		dBankProg = (dBankNo << 7) + dProgNo;
	}
	else
	{
		dBankProg = ((dBankNo - 128 + MASNDDRV_MAX_RAM_BANK) << 7) + dProgNo;
	}

	if ( dRamAdrs == 0xFFFFFFFF )
	{
		if ( gpMaSndDrvInfo->bVoiceIndex[dBankProg] != 0xff )
		{
			dState = 1;
			i = gpMaSndDrvInfo->bVoiceIndex[dBankProg];
			gpMaSndDrvInfo->sVoiceInfo[sdSeqId][i].wRamAdrs = 0xFFFF;
			gpMaSndDrvInfo->bVoiceIndex[dBankProg] = 0xff;
		}
	}
	else
	{
		if (gpMaSndDrvInfo->bVoiceIndex[dBankProg] == 0xff)
		{
			dRealKey = dKey;

			/*--- 1st Try ----*/
			i = (dBankNo & 0x80) + dProgNo;
			if ( gpMaSndDrvInfo->sVoiceInfo[sdSeqId][i].wRamAdrs == 0xFFFF )
			{
				dState = 1;
				gpMaSndDrvInfo->sVoiceInfo[sdSeqId][i].bSynth 	= (UINT8)dSynth;
				gpMaSndDrvInfo->sVoiceInfo[sdSeqId][i].wKey	  	= (UINT16)dRealKey;
				gpMaSndDrvInfo->sVoiceInfo[sdSeqId][i].wRamAdrs	= (UINT16)dRamAdrs;
				gpMaSndDrvInfo->bVoiceIndex[dBankProg] = (UINT8)i;
			}
			else
			{
				for ( i = 0; i < (MASNDDRV_MAX_REG_VOICE * 2 - 1); i++ )
				{
					if ( gpMaSndDrvInfo->sVoiceInfo[sdSeqId][i].wRamAdrs == 0xFFFF )
					{
						dState = 1;
						gpMaSndDrvInfo->sVoiceInfo[sdSeqId][i].bSynth 	= (UINT8)dSynth;
						gpMaSndDrvInfo->sVoiceInfo[sdSeqId][i].wKey	  	= (UINT16)dRealKey;
						gpMaSndDrvInfo->sVoiceInfo[sdSeqId][i].wRamAdrs	= (UINT16)dRamAdrs;
						gpMaSndDrvInfo->bVoiceIndex[dBankProg] = (UINT8)i;
						break;
					}
				}
			}
		}
	}

	if ( dState == 0 ) sdResult = MASMW_ERROR;

	return sdResult;
}
/****************************************************************************
 *	MaSndDrv_GetVoice
 *
 *	Description:
 *			Return entry of specified timber.
 *	Argument:
 *			sdSeqId		sequence id
 *			dBankNo		bank number (0,1..127: melody, 128,128..255: drum)
 *			dProgNo		program (key) number (0..127)
 *	Return:
 *			1			entried
 *			0			not entry
 *			< 0			error code
 *
 ****************************************************************************/
SINT32 MaSndDrv_GetVoice
(
	SINT32	sdSeqId,					/* sequence id */
	UINT32	dBankNo,					/* bank number */
	UINT32	dProgNo						/* program number */
)
{
	UINT32	dBankProg;					/* bank & program number */

	MASNDDRV_DBGMSG(("MaSndDrv_GetVoice: id=%ld bn=%d pb=%d\n", sdSeqId, dBankNo, dProgNo));

	/* check arguments */

	if ( sdSeqId > MASMW_SEQTYPE_DIRECT )		return MASMW_ERROR_ID;
	if ( gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bStatus == MASNDDRV_STATUS_IDLE )	return MASMW_ERROR_ID;
	if ( dBankNo > MASNDDRV_MAX_BANK )				return MASMW_ERROR_ARGUMENT;
	if ( dProgNo > MASNDDRV_MAX_PROGRAM )			return MASMW_ERROR_ARGUMENT;

	if ( dBankNo < 128 )
	{
		dBankProg = (dBankNo << 7) + dProgNo;
	}
	else
	{
		dBankProg = ((dBankNo - 128 + MASNDDRV_MAX_RAM_BANK) << 7) + dProgNo;
	}

	if ( (dBankNo & 0x7f) >= MASNDDRV_MAX_RAM_BANK ) return (0);
	if ( gpMaSndDrvInfo->bVoiceIndex[dBankProg] != 0xff ) return (1);

	return (0);
}
/****************************************************************************
 *	MaSndDrv_SetStream
 *
 *	Description:
 *			Set stream audio data.
 *	Arguments:
 *			sdSeqId		sequence id
 *			dWaveId		wave id (0..31)
 *			dFormat		wave format		0: MA-2 ADPCM
 *										1: ADPCM
 *										2: 8bit offset binary PCM
 *										3: 8bit 2's comp binary PCM
 *										4: 16bit 2's comp binary PCM
 *			dFreq		sampling frequency (0, 4000..48000[Hz])
 *			pbWave		pointer to the wave data
 *			dWaveSize	byte size of the wave data
 *	Return:
 *			0			success
 *			< 0			error code
 *
 ****************************************************************************/
SINT32 MaSndDrv_SetStream
(
	SINT32	sdSeqId,					/* sequence id */
	UINT32	dWaveId,					/* wave id */
	UINT32	dFormat,					/* wave format */
	UINT32	dFreq,						/* sampling frequency */
	UINT8 *	pbWave,						/* pointer to the wave data */
	UINT32	dWaveSize					/* byte size of the wave data */
)
{
	SINT32	sdResult;

	MASNDDRV_DBGMSG(("MaSndDrv_SetStream: id=%ld wv=%d ft=%d fs=%ld\n", sdSeqId, dWaveId, dFormat, dFreq));

	/* check arguments */

	if ( sdSeqId > MASMW_SEQTYPE_DIRECT )		return MASMW_ERROR_ID;
	if ( gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bStatus == MASNDDRV_STATUS_IDLE )	return MASMW_ERROR_ID;
	if ( dWaveId > MASNDDRV_MAX_WAVEID )			return MASMW_ERROR_ARGUMENT;
	if ( (dFormat & 0x0000007F) > 0x00000004 )	return MASMW_ERROR_ARGUMENT;
	if ( (dFormat & 0xFFFFFF80) > 0x00000080 )	return MASMW_ERROR_ARGUMENT;
	if ( ( dFreq != 0 )
	  && ( ( dFreq < MASNDDRV_MIN_STREAMFS ) || ( MASNDDRV_MAX_STREAMFS < dFreq ) ) )
		return MASMW_ERROR_ARGUMENT;

	if ( gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].dSaNum == 0 )
	{
		return MASMW_SUCCESS;
	}


	if ( dFreq == 0 )
	{

		sdResult = MaResMgr_DelStreamAudio( dWaveId );
		if ( sdResult != MASMW_SUCCESS ) return sdResult;

		gpMaSndDrvInfo->sStreamInfo[sdSeqId][dWaveId].dFrequency = 0;
	}
	else
	{

		sdResult = MaResMgr_RegStreamAudio( dWaveId, dFormat, pbWave, dWaveSize );
		if ( sdResult != MASMW_SUCCESS ) return sdResult;

		gpMaSndDrvInfo->sStreamInfo[sdSeqId][dWaveId].bFormat 	  = (UINT8)dFormat;
		gpMaSndDrvInfo->sStreamInfo[sdSeqId][dWaveId].bPan		  = (UINT8)MASNDDRV_MAX_STREAMPAN;
		gpMaSndDrvInfo->sStreamInfo[sdSeqId][dWaveId].dFrequency	  = dFreq;
	}

	return MASMW_SUCCESS;
}
/****************************************************************************
 *	MaSndDrv_GetPos
 *
 *	Description:
 *			Get current position.
 *	Arguments:
 *			sdSeqId		sequence id
 *	Return:
 *			>= 0		current play position [ms]
 *			< 0			error code
 *
 ****************************************************************************/
SINT32 MaSndDrv_GetPos
(
	SINT32	sdSeqId						/* sequence id */
)
{
	UINT32	dPosition;
	SINT32	sdPosH, sdPosM, sdPosL;
	SINT32	sdResult;
	MASNDDRV_DBGMSG(("MaSndDrv_GetPos: id=%ld\n", sdSeqId));

	if (sdSeqId != MASMW_SEQTYPE_DELAYED ) return MASMW_ERROR_ID;

	sdPosH = MaDevDrv_ReadIntermediateReg(MAI_SEQ_COUNT_H);
	sdPosM = MaDevDrv_ReadIntermediateReg(MAI_SEQ_COUNT_M);
	sdPosL = MaDevDrv_ReadIntermediateReg(MAI_SEQ_COUNT_L);

	if ( (sdPosH < MASMW_SUCCESS) || (sdPosM < MASMW_SUCCESS) || (sdPosL < MASMW_SUCCESS) )
	{
		return MASMW_ERROR;
	}

	dPosition  = ( (UINT32)sdPosH <<16) + ((UINT32)sdPosM <<8) + (UINT32)sdPosL;

	sdResult = (SINT32)(dPosition * (UINT32)(gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].dTimeBase));

	return sdResult;
}
/****************************************************************************
 *	MaSndDrv_SetSpeed
 *
 *	Description:
 *			Sets speed of play.
 *	Arguments:
 *			sdSeqId		sequence id
 *			dVal		play speed (70..100..130)
 *	Return:
 *			0			success
 *			< 0			error code
 *
 ****************************************************************************/
SINT32 MaSndDrv_SetSpeed
(
	SINT32	sdSeqId,					/* sequence id */
	UINT32	dVal						/* speed value */
)
{
	UINT32	dBaseTime;
	SINT32	sdResult;

	MASNDDRV_DBGMSG(("MaSndDrv_SetSpeed: id=%ld val=%d\n", sdSeqId, dVal));

	if ( sdSeqId != MASMW_SEQTYPE_DELAYED ) return MASMW_ERROR_ID;
	if ( ( MASNDDRV_MIN_SPEED > dVal ) || ( dVal > MASNDDRV_MAX_SPEED ) ) return MASMW_ERROR_ARGUMENT;

	sdResult = MASMW_SUCCESS;

	if ( gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bSpeed != dVal )
	{
		gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bSpeed = (UINT8)dVal;

		/* MA-5 */
		dBaseTime = ( (UINT32)(gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].dTimeBase) * (UINT32)100000000 )
		            / (1085 * (UINT32)dVal);
		if ( dBaseTime > 0x3FFF )
			dBaseTime = 0x3FFF;

		MaDevDrv_WriteIntermediateReg( MAI_SEQ_MSS_H, (UINT8)(dBaseTime>>6) );
		MaDevDrv_WriteIntermediateReg( MAI_SEQ_MSS_L, (UINT8)(dBaseTime<<2) );
		sdResult = MASMW_SUCCESS;
	}

	return sdResult;
}
/****************************************************************************
 *	MaSndDrv_SetKey
 *
 *	Description:
 *			Sets key offset value.
 *	Arguments:
 *			sdSeqId		sequence id (0..2)
 *			sdVal		key offset value (-12..0..+12)
 *	Return:
 *			0			success
 *			< 0			error code
 *
 ****************************************************************************/
SINT32 MaSndDrv_SetKey
(
	SINT32	sdSeqId,					/* sequence id */
	SINT32	sdVal						/* key offset value */
)
{
	MASNDDRV_DBGMSG(("MaSndDrv_SetKey: id=%ld val=%d\n", sdSeqId, sdVal));

	if ( sdSeqId != MASMW_SEQTYPE_DELAYED ) return MASMW_ERROR_ID;
	if ( ( MASNDDRV_MIN_KEYCTRL > sdVal ) || ( sdVal > MASNDDRV_MAX_KEYCTRL ) ) return MASMW_ERROR_ARGUMENT;

	gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].sbKeyOffset = (SINT8)sdVal;

	return MASMW_SUCCESS;
}
/****************************************************************************
 *	MaSndDrv_SetKeyControl
 *
 *	Description:
 *			Sets key control.
 *	Arguments:
 *			sdSeqId		sequence id (0..2)
 *			dCh			channel number (0..15)
 *			dVal		contorol value (0: None, 1: OFF, 2: ON)
 *	Return:
 *			0			success
 *			< 0			error code
 *
 ****************************************************************************/
SINT32 MaSndDrv_SetKeyControl
(
	SINT32	sdSeqId,					/* sequence id */
	UINT32	dCh,						/* channel number */
	UINT32	dVal						/* control value */
)
{
	MASNDDRV_DBGMSG(("MaSndDrv_SetKeyControl: id=%ld ch=%d val=%d\n", sdSeqId, dCh, dVal));

	if ( sdSeqId != MASMW_SEQTYPE_DELAYED ) return MASMW_ERROR_ID;
	if ( dCh > MASNDDRV_MAX_CHANNEL ) return MASMW_ERROR_ARGUMENT;
	if ( dVal > 2 ) return MASMW_ERROR_ARGUMENT;

	if ( ( gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].dChMap & (UINT32)(1 << dCh) ) != 0 )
	{
		gpMaSndDrvInfo->sChInfo[dCh].bKeyControl = (UINT8)dVal;
	}

	return MASMW_SUCCESS;
}
/****************************************************************************
 *	MaSndDrv_Free
 *
 *	Description:
 *			Free the resources.
 *	Argument:
 *			sdSeqId		sequence id number
 *	Return:
 *			0			success
 *			< 0			error code
 *
 ****************************************************************************/
SINT32 MaSndDrv_Free
(
	SINT32	sdSeqId						/* sequence id */
)
{
	MASNDDRV_DBGMSG(("MaSndDrv_Free: id=%ld\n", sdSeqId));

	switch ( sdSeqId )
	{
	  case MASMW_SEQTYPE_DELAYED:

		MaResMgr_FreeSequencer( 0 );

		MaDva_End();

		/* Initialize API Balance */
		MaDevDrv_WriteIntermediateReg( MAI_MASTER_VOLL, (UINT8)((MASNDDRV_DEFAULT_API_BALANCE << 1) & 0xF8) );
		MaDevDrv_WriteIntermediateReg( MAI_MASTER_VOLR, (UINT8)((MASNDDRV_DEFAULT_API_BALANCE << 1) & 0xF8) );

		break;

	  case MASMW_SEQTYPE_DIRECT:

		MaResMgr_FreeTimer( 2 );

		MaDva_End();

		/* Initialize API Balance */
		MaDevDrv_WriteIntermediateReg( MAI_MASTER_VOLL, (UINT8)((MASNDDRV_DEFAULT_API_BALANCE << 1) & 0xF8) );
		MaDevDrv_WriteIntermediateReg( MAI_MASTER_VOLR, (UINT8)((MASNDDRV_DEFAULT_API_BALANCE << 1) & 0xF8) );

		break;

	  case MASMW_SEQTYPE_HV:

		MaResMgr_FreeTimer( 3 );

		/* Terminate HV Driver module */
		pfnHvFunc[gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bHvType].Terminate( );

		break;

	default:
		return MASMW_ERROR_ARGUMENT;
	}

	MaResMgr_FreeCh( gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].dChMap );
	MaResMgr_FreeFmVoice( gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].dFmMap );
	MaResMgr_FreeWtVoice( gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].dWtMap );
	MaResMgr_FreeRam( gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].dRbMap );
	MaResMgr_FreeStreamAudio( gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].dSaMap );
	MaResMgr_FreeAl( gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].dAlMap );
	MaResMgr_FreeHv( gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].dHvMap );
	MaResMgr_FreeLed( gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].dLdMap );
	MaResMgr_FreeMotor( gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].dMtMap );

	gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bStatus = MASNDDRV_STATUS_IDLE;

	return MASMW_SUCCESS;
}
/****************************************************************************
 *	MaSndDrv_Create
 *
 *	Description:
 *			Allocates the resouces.
 *	Arguments:
 *			dSeqType	sequence type
 *			dTimeBase	time base (0, 4..10, 20 [ms])
 *			sdResMode	resource mode (0..5)
 *			sdCnvMode	convert mode
 *						  bit   0: FM mode(0: 2-OP, 1: 4-OP)
 *						  bit   1: percussion sound set type (0: GM-x, 1: GM Level1)
 *						  bit   2: WT volume setting (0: compatible, 1: normal)
 *			dStream		number of stream audio (0..2)
 *			dAlCh		AL channel
 *			pfnFunc		pointer to the converter function
 *			pdRamAdrs	start address of the internal RAM
 *			pdRamSize	byte size of the internal RAM
 *	Return:
 *			>= 0		success. sequence id number
 *			< 0			error code
 *
 ****************************************************************************/
SINT32 MaSndDrv_Create
(
	UINT32	dSeqType,					/* sequence type */
	UINT32	dTimeBase,					/* time base */
	UINT32	dResMode,					/* convert mode */
	UINT32	dCnvMode,					/* resource mode */
	UINT32	dStream,					/* number of stream audio */
	UINT32	dAlCh,						/* channel number for AL */
	SINT32	(*pfnFunc)(void),			/* pointer to converter function */
	UINT32 *pdRamAdrs,					/* ram address */
	UINT32 *pdRamSize					/* byte size of ram */
)
{
	UINT32	dCount;
	UINT32	dChNo;						/* channel number */
	UINT32	dSlotNo;					/* slot number */
	UINT32	dChMap;						/* bit mapping for channel */
	UINT32	dFmMap;						/* bit mapping for FM voices */
	UINT32	dWtMap;						/* bit mapping for WT voices */
	UINT32	dStMap;						/* bit mapping for Stream */
	UINT32	dHvMap;						/* bit mapping for HV voices */
	UINT32	dAlMap;						/* bit mapping for AL */
	UINT32	dRbMap;						/* bit mapping for RAM blocks */
	UINT32	dLedMap;					/* bit mapping for LED */
	UINT32	dMtrMap;					/* bit mapping for Motor */
	UINT32	dFmMode;					/* fm mode flag 0:2OP 1:4OP */
	UINT32	dAltMode;					/* alternate assign mode */
	UINT32	dRamAdrs;					/* address of RAM */
	UINT32	dErrFlag;					/* error flag */
	UINT32	dResFlag;					/* flag of resources */
	SINT32	sdResult;					/* result of allocate resources */
	PMASNDDRVINFO	psSndDrvInfo;
	PMASNDDRVCHINFO	psChInfo;

	MASNDDRV_DBGMSG(("MaSndDrv_Create: st=%d tm=%d rm=%d cm=%02x sa=%d al=%d\n", 
					dSeqType, dTimeBase, dResMode, dCnvMode, dStream, dAlCh));

	if ( dSeqType > MASNDDRV_MAX_SEQTYPE ) return MASMW_ERROR_ARGUMENT;
	if ( dResMode > MASNDDRV_MAX_RESMODE ) return MASMW_ERROR_ARGUMENT;
	if ( dCnvMode > MASNDDRV_MAX_CNVMODE ) return MASMW_ERROR_ARGUMENT;
	if ( dStream  > MASNDDRV_NUM_STREAM )  return MASMW_ERROR_ARGUMENT;
	if ( dAlCh > 0x000000FF ) return MASMW_ERROR_ARGUMENT;
	if ( pfnFunc == NULL ) return MASMW_ERROR_ARGUMENT;

	switch ( ma_resmode_stmap[dResMode] )
	{
	  case 0x00:
	  	if ( dStream > 0 )
	  		return (MASMW_ERROR_ARGUMENT);
	  	break;

	  case 0x03:
	  	if ( dStream > 2 )
	  		return (MASMW_ERROR_ARGUMENT);
	  	break;
	}


	switch ( dSeqType )
	{
	  case MASMW_SEQTYPE_DELAYED:


		if ( ( MASNDDRV_MIN_DLY_BASETIME > dTimeBase ) || ( dTimeBase > MASNDDRV_MAX_DLY_BASETIME ) )
			return MASMW_ERROR_ARGUMENT;

		sdResult = MaResMgr_AllocSequencer( 0, ma_timebase[dTimeBase-1] );
		if ( sdResult != MASMW_SUCCESS )
		{
			return sdResult;
		}

		dLedMap = 1;
		dMtrMap = 1;

		break;

	case MASMW_SEQTYPE_DIRECT:


		if ( dTimeBase == MASNDDRV_DIRECT_BASETIME ) 
		{
			sdResult = MaResMgr_AllocTimer( 2, MASNDDRV_TIMER_MST, MASNDDRV_TIMER_COUNT, 0, 0 );
			if ( sdResult != MASMW_SUCCESS )
			{
				return sdResult;
			}
		}
		else if ( dTimeBase != 0 )
		{
			return MASMW_ERROR;
		}

		dLedMap = 0;
		dMtrMap = 0;

		break;

	case MASMW_SEQTYPE_HV:


		if ( dTimeBase == MASNDDRV_HV_BASETIME )
		{
			sdResult = MaResMgr_AllocTimer( 3, MASNDDRV_TIMER_MST, MASNDDRV_TIMER_COUNT, 0, 0 );
			if ( sdResult != MASMW_SUCCESS )
			{
				return sdResult;
			}
		}
		else
		{
			return MASMW_ERROR_ARGUMENT;
		}

		dLedMap = 0;
		dMtrMap = 0;

		break;

	default:
		return MASMW_ERROR_ARGUMENT;
	}

	dFmMode = (dCnvMode & 0x00000001);


	dChMap = ma_resmode_chmap[dResMode];
	dFmMap = ma_resmode_fmmap[dFmMode][dResMode];
	dWtMap = ma_resmode_wtmap[dResMode];
	dStMap = ma_resmode_stmap[dResMode];
	dHvMap = ma_resmode_hvmap[dResMode];
	dRbMap = ma_resmode_rbmap[dResMode];

	if (dAlCh != 0xFF)
		dAlMap = 0x00000001;
	else
		dAlMap = 0x00000000;

	dErrFlag = 0;
	dResFlag = 0;

	/* alloc CH resource */
	sdResult = MaResMgr_AllocCh( dChMap );
	if ( sdResult != MASMW_SUCCESS )	dErrFlag |= 0x00000001;
	else								dResFlag |= 0x00000001;

	/* alloc FM resource */
	sdResult = MaResMgr_AllocFmVoice( dFmMap, dFmMode );
	if ( sdResult != MASMW_SUCCESS )	dErrFlag |= 0x00000002;
	else								dResFlag |= 0x00000002;

	/* alloc WT resource */
	sdResult = MaResMgr_AllocWtVoice( dWtMap );
	if ( sdResult != MASMW_SUCCESS )	dErrFlag |= 0x00000004;
	else								dResFlag |= 0x00000004;

	/* alloc Stream resource */
	sdResult = MaResMgr_AllocStreamAudio( dStMap );
	if ( sdResult != MASMW_SUCCESS )	dErrFlag |= 0x00000008;
	else								dResFlag |= 0x00000008;

	/* alloc AL resource */
	sdResult = MaResMgr_AllocAl( dAlMap );
	if ( sdResult != MASMW_SUCCESS )	dErrFlag |= 0x00000010;
	else								dResFlag |= 0x00000010;

	/* alloc HV resource */
	sdResult = MaResMgr_AllocHv( dHvMap );
	if ( sdResult != MASMW_SUCCESS )	dErrFlag |= 0x00000020;
	else								dResFlag |= 0x00000020;

	/* alloc RAM resource */
	sdResult = MaResMgr_AllocRam( dRbMap );
	if ( sdResult != MASMW_SUCCESS )	dErrFlag |= 0x00000040;
	else								dResFlag |= 0x00000040;

	/* alloc LED resource */
	sdResult = MaResMgr_AllocLed( dLedMap );
	if ( sdResult != MASMW_SUCCESS )	dErrFlag |= 0x00000080;
	else								dResFlag |= 0x00000080;

	/* alloc Motor resource */
	sdResult = MaResMgr_AllocMotor( dMtrMap );
	if ( sdResult != MASMW_SUCCESS )	dErrFlag |= 0x00000100;
	else								dResFlag |= 0x00000100;

	if ( dErrFlag != 0 )
	{
		switch ( dSeqType )
		{
		case MASMW_SEQTYPE_DELAYED:
			MaResMgr_FreeSequencer( 0 );
			break;
		case MASMW_SEQTYPE_DIRECT:
			MaResMgr_FreeTimer( 2 );
			break;
		case MASMW_SEQTYPE_HV:
			MaResMgr_FreeTimer( 3 );
			break;
		default:
			break;
		}

		if ( ( dResFlag & 0x00000001 ) != 0 )	MaResMgr_FreeCh( dChMap );
		if ( ( dResFlag & 0x00000002 ) != 0 )	MaResMgr_FreeFmVoice( dFmMap );
		if ( ( dResFlag & 0x00000004 ) != 0 )	MaResMgr_FreeWtVoice( dWtMap );
		if ( ( dResFlag & 0x00000008 ) != 0 )	MaResMgr_FreeStreamAudio( dStMap );
		if ( ( dResFlag & 0x00000010 ) != 0 )	MaResMgr_FreeAl( dAlMap );
		if ( ( dResFlag & 0x00000020 ) != 0 )	MaResMgr_FreeHv( dHvMap );
		if ( ( dResFlag & 0x00000040 ) != 0 )	MaResMgr_FreeRam( dRbMap );
		if ( ( dResFlag & 0x00000080 ) != 0 )	MaResMgr_FreeLed( dLedMap );
		if ( ( dResFlag & 0x00000100 ) != 0 )	MaResMgr_FreeMotor( dMtrMap );

		return MASMW_ERROR_RESOURCE_OVER;
	}

	if ( dSeqType == MASMW_SEQTYPE_DELAYED )
	{
		MaDevDrv_ClearFifo();

		/* reset Position counter */
		MaSndDrv_ResetPos( dSeqType );
	}


	if ( ( pdRamAdrs != NULL ) && ( pdRamSize != NULL ) )
	{
		*pdRamSize = 0;
		if ( ma_resmode_rbnum[dResMode] != 0 ) /* no allock ram */
		{
			dRamAdrs = 0;
			for ( dCount = 0; dCount < 8; dCount++ )
			{
				if ( ( dRbMap & (1 << dCount) ) != 0 )
				{
					dRamAdrs = dCount * MA_RAM_BLOCK_SIZE + MA_RAM_START_ADDRESS;
					break;
				}
			}

			if ( dRamAdrs != 0 )
			{
				*pdRamAdrs = dRamAdrs;
				*pdRamSize = ( ma_resmode_rbnum[dResMode] - dStream) * MA_RAM_BLOCK_SIZE;
			}
		}
	}

	/* set sSndDrvInfo **********/
	psSndDrvInfo = &gpMaSndDrvInfo->sSndDrvInfo[dSeqType];

	psSndDrvInfo->dChNum = ma_resmode_chnum[dResMode];
	psSndDrvInfo->dFmNum = ma_resmode_fmnum[dFmMode][dResMode];
	if ( dResMode == 2 )
		psSndDrvInfo->dWtNum = ma_resmode_wtnum[dResMode] - dStream;
	else
		psSndDrvInfo->dWtNum = ma_resmode_wtnum[dResMode];
	psSndDrvInfo->dSaNum = dStream;
	if ( dAlCh == 0xFF )
		psSndDrvInfo->dAlNum = 0;
	else
		psSndDrvInfo->dAlNum = 1;
	psSndDrvInfo->dHvNum = ma_resmode_hvnum[dResMode];
	psSndDrvInfo->dRbNum = ma_resmode_rbnum[dResMode];

	psSndDrvInfo->dChMap = dChMap;
	psSndDrvInfo->dFmMap = dFmMap;
	psSndDrvInfo->dWtMap = dWtMap;
	psSndDrvInfo->dSaMap = dStMap;
	psSndDrvInfo->dAlMap = dAlMap;
	psSndDrvInfo->dHvMap = dHvMap;
	psSndDrvInfo->dRbMap = dRbMap;
	psSndDrvInfo->dLdMap = dLedMap;
	psSndDrvInfo->dMtMap = dMtrMap;

	psSndDrvInfo->bDrumMode   = ma_resmode_drmmode[dResMode];
	psSndDrvInfo->bDvaMode    = ma_resmode_dvamode[dResMode];
	psSndDrvInfo->bMelodyMode = ma_resmode_melmode[dResMode];
	psSndDrvInfo->bDrumType   = (UINT8)((dCnvMode >> 1) & 0x01);
	psSndDrvInfo->bWtVolMode  = (UINT8)((dCnvMode >> 2) & 0x01);
	psSndDrvInfo->bHvType     = (UINT8)((dCnvMode >> 4) & 0x07);
	if ( psSndDrvInfo->bHvType >= MASNDDRV_NUM_HVTYPE )
		psSndDrvInfo->bHvType = 0;

	psSndDrvInfo->dTimeBase   = dTimeBase;

	psSndDrvInfo->bBalVolumeL	= (UINT8)MASNDDRV_DEFAULT_API_BALANCE;
	psSndDrvInfo->bBalVolumeR	= (UINT8)MASNDDRV_DEFAULT_API_BALANCE;
	psSndDrvInfo->bFmVolume		= (UINT8)MASNDDRV_DEFAULT_API_VOLUME;
	psSndDrvInfo->bWtVolume		= (UINT8)MASNDDRV_DEFAULT_API_VOLUME;
	psSndDrvInfo->bSaVolume		= (UINT8)MASNDDRV_DEFAULT_API_VOLUME;
	psSndDrvInfo->bHvVolume		= (UINT8)MASNDDRV_DEFAULT_API_VOLUME;

	psSndDrvInfo->bMasterVolShiftL	= (UINT8)0;
	psSndDrvInfo->bMasterVolShiftR	= (UINT8)0;
	psSndDrvInfo->bFmVolShift 		= (UINT8)0;
	psSndDrvInfo->bWtVolShift 		= (UINT8)0;
	psSndDrvInfo->bSaVolShift 		= (UINT8)0;
	psSndDrvInfo->bHvVolShift 		= (UINT8)0;

	psSndDrvInfo->bMasterVolume		= (UINT8)MASNDDRV_DEFAULT_MASTER_VOLUME;
	psSndDrvInfo->bWaveSlave		= (UINT8)0x80;
	psSndDrvInfo->bSpeed			= (SINT8)100;
	psSndDrvInfo->sbKeyOffset		= (SINT8)0;
	psSndDrvInfo->bSaLedMtrSync		= (UINT8)0xFF;
	psSndDrvInfo->bHvLedMtrSync		= (UINT8)0;
	psSndDrvInfo->bHvChannel		= (UINT8)MASNDDRV_HV_EXCHANNEL;
	psSndDrvInfo->dEosFlag			= 0UL;

	psSndDrvInfo->dMinSlot[MASNDDRV_FM_SLOT] = ma_resmode_fm_minslot[dResMode];
	psSndDrvInfo->dMaxSlot[MASNDDRV_FM_SLOT] = ma_resmode_fm_maxslot[dFmMode][dResMode];
	psSndDrvInfo->dMinSlot[MASNDDRV_WT_SLOT] = ma_resmode_wt_minslot[dResMode];
	psSndDrvInfo->dMaxSlot[MASNDDRV_WT_SLOT] = ma_resmode_wt_maxslot[dResMode];
	if ( psSndDrvInfo->dSaNum != 0 )
	{
		psSndDrvInfo->dMinSlot[MASNDDRV_ST_SLOT] = MASNDDRV_MIN_STM_SLOT;
		psSndDrvInfo->dMaxSlot[MASNDDRV_ST_SLOT] = MASNDDRV_MIN_STM_SLOT + psSndDrvInfo->dSaNum - 1;
	}

	/* initialize sStreamInfo **********/
	for ( dCount = 0; dCount < MA_MAX_REG_STREAM_AUDIO; dCount++ )
	{
		gpMaSndDrvInfo->sStreamInfo[dSeqType][dCount].bFormat	 = (UINT8)0;
		gpMaSndDrvInfo->sStreamInfo[dSeqType][dCount].bPan		 = (UINT8)255;
		gpMaSndDrvInfo->sStreamInfo[dSeqType][dCount].dFrequency = 0UL;
		gpMaSndDrvInfo->sStreamInfo[dSeqType][dCount].dSeekP     = 0UL;
	}

	/* initialize sChInfo **********/
	for ( dChNo = 0; dChNo < MASNDDRV_NUM_CHANNEL; dChNo++ )
	{
		if ( ( dChMap & (UINT32)(1 << dChNo) ) != 0 )
		{
			psChInfo = &gpMaSndDrvInfo->sChInfo[dChNo];
			
			psChInfo->bBankNo	 	 = (UINT8)( ( dChNo == 9 ) ? 128 : 0 );
			psChInfo->bProgNo 		 = (UINT8)0;
			psChInfo->bPoly 		 = (UINT8)MASNDDRV_MODE_POLY;	/* poly mode */
			psChInfo->bSfx 			 = (UINT8)0;					/* all off */
			psChInfo->bVolume 		 = (UINT8)100;
			psChInfo->bExpression 	 = (UINT8)127;
			psChInfo->bBendRange 	 = (UINT8)2;
			psChInfo->bKeyControl	 = (UINT8)0;
			psChInfo->bNoteOn 		 = (UINT8)0;
			psChInfo->bSlotNo		 = (UINT8)0xFF;
			psChInfo->bAlFlag		 = (UINT8)0x7F;
			psChInfo->bLedSync		 = (UINT8)0;
			psChInfo->bMtrSync		 = (UINT8)0;
			psChInfo->bVolInterp	 = (UINT8)1;
			psChInfo->bPcFlag		 = (UINT8)0;
			psChInfo->dLpfQ			 = 0UL;
			psChInfo->dLpfCof		 = 0UL;
			psChInfo->sdLpfQShift	 = 0L;
			psChInfo->sdLpfCofShift	 = 0L;
			psChInfo->dPitchBend	 = 0x0400;
			psChInfo->dCoaseTune	 = 0x0400;
			psChInfo->dFineTune		 = 0x0400;
			psChInfo->dTunePitch	 = 0x0400;
		}
	}
	if ( dAlCh != 0xFF )
	{
		if ( ( dChMap & (UINT32)(1 << dAlCh) ) != 0 )
			gpMaSndDrvInfo->sChInfo[dAlCh].bAlFlag |= MASNDDRV_MASK_ALFLAG;
	}

	/* initialize sSlotInfo **********/
	for ( dSlotNo = 0; dSlotNo < gpMaSndDrvInfo->sSndDrvInfo[dSeqType].dFmNum; dSlotNo++ )
	{
		if ( ( dFmMap & (UINT32)(1 << dSlotNo) ) != 0 )
		{
			gpMaSndDrvInfo->sSlotInfo[dSlotNo].bCh   = 0;
		}
	}
	for ( dSlotNo = 0; dSlotNo < gpMaSndDrvInfo->sSndDrvInfo[dSeqType].dWtNum; dSlotNo++ )
	{
		if ( ( dWtMap & (UINT32)(1 << dSlotNo) ) != 0 )
		{
			gpMaSndDrvInfo->sSlotInfo[dSlotNo+MASNDDRV_MIN_WT_SLOT].bCh   = 0;
		}
	}
	for ( dSlotNo = 0; dSlotNo < gpMaSndDrvInfo->sSndDrvInfo[dSeqType].dSaNum; dSlotNo++ )
	{
		if ( ( dStMap & (UINT32)(1 << dSlotNo) ) != 0 )
		{
			gpMaSndDrvInfo->sSlotInfo[dSlotNo+MASNDDRV_MIN_STM_SLOT].bCh   = 0;
		}
	}

	if ( dSeqType < MASMW_SEQTYPE_HV )
	{
		for ( dCount = 0; dCount < MASNDDRV_MAX_REG_VOICE*2; dCount++ )
		{
			gpMaSndDrvInfo->sVoiceInfo[dSeqType][dCount].wBankProg	= (UINT16)0;
			gpMaSndDrvInfo->sVoiceInfo[dSeqType][dCount].bSynth		= (UINT8)1;
			gpMaSndDrvInfo->sVoiceInfo[dSeqType][dCount].wKey		= (UINT16)0;
			gpMaSndDrvInfo->sVoiceInfo[dSeqType][dCount].wRamAdrs	= (UINT16)0xFFFF;
		}
	}

	for ( dCount = 0; dCount < MA_MAX_REG_STREAM_AUDIO; dCount++ )
	{
		gpMaSndDrvInfo->sStreamInfo[dSeqType][dCount].dFrequency = (UINT16)0;
	}

	gpMaSndDrvInfo->pfnSrmCnv[dSeqType] 	= pfnFunc;

	gpMaSndDrvInfo->bCtrlSequencer[dSeqType]	= 0;

	psSndDrvInfo->bStatus = MASNDDRV_STATUS_REGISTERED;

	switch ( dSeqType )
	{
	  case MASMW_SEQTYPE_DELAYED:

		/* initialize sSeqbufInfo **********/
		gpMaSndDrvInfo->sSeqbufInfo.bSequenceFlag = (UINT8)(MA_INT_POINT - 2);
		gpMaSndDrvInfo->sSeqbufInfo.pbBufPtr	 = NULL;
		gpMaSndDrvInfo->sSeqbufInfo.dBufSize 	 = 0UL;
		gpMaSndDrvInfo->sSeqbufInfo.bQueueFlag	 = (UINT8)0;
		gpMaSndDrvInfo->sSeqbufInfo.dQueueSize	 = 0UL;
		gpMaSndDrvInfo->sSeqbufInfo.dQueueWPtr	 = 0UL;
		gpMaSndDrvInfo->sSeqbufInfo.dWroteSize	 = 0UL;

		/* initialize sStreamCtrlInfo **********/
		gpMaSndDrvInfo->sStreamCtrlInfo.dQueFlag	 = 0;
		gpMaSndDrvInfo->sStreamCtrlInfo.dType[0]	 = 0;
		gpMaSndDrvInfo->sStreamCtrlInfo.dType[1]	 = 0;
		gpMaSndDrvInfo->sStreamCtrlInfo.dWaveId[0]	 = 0;
		gpMaSndDrvInfo->sStreamCtrlInfo.dWaveId[1]	 = 0;
		gpMaSndDrvInfo->sStreamCtrlInfo.dStreamStatus = 0;
		
		/* initialize dHvEventInfo **********/
		gpMaSndDrvInfo->dHvEventInfo[dSeqType] = 0;



	case MASMW_SEQTYPE_DIRECT:

		for ( dCount = 0; dCount < MASNDDRV_NUM_ALVOICE; dCount++ )
		{
			gpMaSndDrvInfo->sAlVoiceInfo[dCount].dBankNo      = 0xFFFF;
			gpMaSndDrvInfo->sAlVoiceInfo[dCount].dProgNo      = 0xFFFF;
			gpMaSndDrvInfo->sAlVoiceInfo[dCount].pAlVoice     = NULL;
			gpMaSndDrvInfo->sAlVoiceInfo[dCount].dAlVoiceSize = 0;
		}

		/* API Balance */
		MaDevDrv_WriteIntermediateReg( MAI_MASTER_VOLL, (UINT8)((MASNDDRV_DEFAULT_API_BALANCE << 1) & 0xF8) );
		MaDevDrv_WriteIntermediateReg( MAI_MASTER_VOLR, (UINT8)((MASNDDRV_DEFAULT_API_BALANCE << 1) & 0xF8) );

		/* API Volume */
		MaDevDrv_WriteIntermediateReg( MAI_FM_VOL, (UINT8)((MASNDDRV_DEFAULT_API_VOLUME << 1) & 0xF8) );
		MaDevDrv_WriteIntermediateReg( MAI_WT_VOL, (UINT8)((MASNDDRV_DEFAULT_API_VOLUME << 1) & 0xF8) );
		MaDevDrv_WriteIntermediateReg( MAI_STM_VOL, (UINT8)((MASNDDRV_DEFAULT_API_VOLUME << 1) & 0xF8) );

		break;

	case MASMW_SEQTYPE_HV:

		/* API Volume */
		MaDevDrv_WriteIntermediateReg( MAI_HV_VOL, (UINT8)((MASNDDRV_DEFAULT_API_VOLUME << 1) & 0xF8) );

		/* initialize HV Driver module */
		pfnHvFunc[psSndDrvInfo->bHvType].Initialize( dTimeBase );
		
		break;

	default:
		break;
	}

	if ( dSeqType != MASMW_SEQTYPE_HV )
	{
		if ( psSndDrvInfo->bDrumMode == MASNDDRV_RESMODE1_NORMAL )
		{
			dAltMode = 1;
		}
		else
		{
			dAltMode = 0;
		}
		MaDva_Initialize((UINT32)(psSndDrvInfo->dAlNum),
						 (UINT32)(psSndDrvInfo->dFmNum), 
						 (UINT32)(psSndDrvInfo->dWtNum), 
						 (UINT32)(psSndDrvInfo->dSaNum), dAltMode);

		for (dChMap = 0; dChMap < (MASNDDRV_MAX_RAM_BANK * 2 * 128); dChMap++)
		{
			gpMaSndDrvInfo->bVoiceIndex[dChMap] = 0xff;
		}

	}

	return (SINT32)dSeqType;
}
/****************************************************************************
 *	MaSndDrv_ResetPos
 *
 *	Description:
 *			Reset the current play position.
 *	Argument:
 *			sdSeqId		sequence id
 *	Return:
 *			0			success
 *
 ****************************************************************************/
SINT32 MaSndDrv_ResetPos
(
	SINT32	sdSeqId						/* sequence id */
)
{
	if ( sdSeqId != MASMW_SEQTYPE_DELAYED )	return MASMW_ERROR_ID;

	if (gpMaSndDrvInfo->sSndDrvInfo[sdSeqId].bStatus == MASNDDRV_STATUS_PLAYING)
		MaDevDrv_WriteIntermediateReg( MAI_SEQ_CTRL, 0x03);
	else
		MaDevDrv_WriteIntermediateReg( MAI_SEQ_CTRL, 0x02);

		return MASMW_SUCCESS;
}
/****************************************************************************
 *	MaSndDrv_DeviceControl
 *
 *	Description:
 *			Control the MA hardware.
 *	Argument:
 *			dCmd	command number
 *			dParam1	parameter 1
 *			dParam2	parameter 2
 *			dParam3	parameter 3
 *	Return:
 *			0		success
 *			< 0		error code
 *
 ****************************************************************************/
SINT32 MaSndDrv_DeviceControl
(
	UINT32	dCmd,						/* command number */
	UINT32	dParam1,					/* parameter 1 */
	UINT32	dParam2,					/* parameter 2 */
	UINT32	dParam3						/* parameter 3 */
)
{
	SINT32	sdResult;

	MASNDDRV_DBGMSG(("MaSndDrv_DeviceControl: cmd=%d p1=%d p2=%d p3=%d\n", dCmd, dParam1, dParam2, dParam3));

	sdResult = MASMW_SUCCESS;
	switch( dCmd )
	{
	  case MASMW_PWM_DIGITAL:
		MaDevDrv_WriteIntermediateReg( MAI_POWER_MANAGEMENT_DIGITAL, (UINT8)dParam1 );
		break;
	  case MASMW_PWM_ANALOG:
		MaDevDrv_WriteIntermediateReg( MAI_POWER_MANAGEMENT_ANALOG, (UINT8)dParam1 );
		break;
	  case MASMW_EQ_VOLUME:
		MaDevDrv_WriteIntermediateReg( MAI_ANALOG_EQVOL, (UINT8)(dParam1&0x1F) );
		break;
	  case MASMW_HP_VOLUME:
	  	if ( dParam1 == 1 )
	  	{
			MaDevDrv_WriteIntermediateReg( MAI_ANALOG_HPVOL_L, (UINT8)( (dParam2&0x1F) | 0x80 ) );
		}
		else
		{
			MaDevDrv_WriteIntermediateReg( MAI_ANALOG_HPVOL_L, (UINT8)(dParam2&0x1F) );
		}
		MaDevDrv_WriteIntermediateReg( MAI_ANALOG_HPVOL_R, (UINT8)(dParam3&0x1F) );
		break;
	  case MASMW_SP_VOLUME:
		MaDevDrv_WriteIntermediateReg( MAI_ANALOG_SPVOL, (UINT8)((MA_VSEL<<6) | (dParam1&0x1F)));
		break;
	  case MASMW_LED_MASTER:
	  case MASMW_LED_BLINK:

	  	if ( dCmd == MASMW_LED_MASTER )
	  	{
			if ( dParam1 > MASMW_MAX_LED_MASTER )
				return MASMW_ERROR_ARGUMENT;
			gpMaSndDrvInfo->sLedInfo.dCtrl = dParam1;

			MaDevDrv_WriteIntermediateReg( MAI_LED_SETTING_1, 0 );
		}
		else
		{
			if ( dParam1 > MASMW_MAX_LED_BLINK )
				return MASMW_ERROR_ARGUMENT;

			if (dParam1 == 0)
				gpMaSndDrvInfo->sLedInfo.dBlink = 0;
			else
			{
				gpMaSndDrvInfo->sLedInfo.dBlink = 1;
				gpMaSndDrvInfo->sLedInfo.dFreq = dParam1 - 1;
			}
		}

		switch ( gpMaSndDrvInfo->sLedInfo.dCtrl )
		{
		  case MASNDDRV_LED_MODE_OFF:
			MaDevDrv_WriteIntermediateReg( MAI_LED_SETTING_2, 0 );
			break;
		  case MASNDDRV_LED_MODE_DIRECT:
		  	MaDevDrv_WriteIntermediateReg( MAI_LED_SETTING_2, 
						  (UINT8)((gpMaSndDrvInfo->sLedInfo.dFreq << 4) | (gpMaSndDrvInfo->sLedInfo.dBlink + 6)) );
			break;
		  case MASNDDRV_LED_MODE_SYNC:
		  	MaDevDrv_WriteIntermediateReg( MAI_LED_SETTING_2,
  						  (UINT8)((gpMaSndDrvInfo->sLedInfo.dFreq << 4) | (gpMaSndDrvInfo->sLedInfo.dBlink + 2)) );
		  	break;
		}

		break;
	  case MASMW_LED_DIRECT:
	  	if ( dParam1 > MASMW_MAX_LED_DIRECT )
	  		return MASMW_ERROR_ARGUMENT;

	  	if ( gpMaSndDrvInfo->sLedInfo.dCtrl == MASNDDRV_LED_MODE_DIRECT )
	  	{
	  		MaDevDrv_WriteIntermediateReg( MAI_LED_SETTING_1, (UINT8)dParam1 );
	  	}

		break;
	  case MASMW_MTR_MASTER:
	  case MASMW_MTR_BLINK:

	  	if ( dCmd == MASMW_MTR_MASTER )
	  	{
			if ( dParam1 > MASMW_MAX_MTR_MASTER )
				return MASMW_ERROR_ARGUMENT;
			gpMaSndDrvInfo->sMtrInfo.dCtrl = dParam1;

			MaDevDrv_WriteIntermediateReg( MAI_MTR_SETTING_1, 0 );
		}
		else
		{
			if ( dParam1 > MASMW_MAX_MTR_BLINK )
				return MASMW_ERROR_ARGUMENT;

			if (dParam1 == 0)
				gpMaSndDrvInfo->sMtrInfo.dBlink = 0;
			else
			{
				gpMaSndDrvInfo->sMtrInfo.dBlink = 1;
				gpMaSndDrvInfo->sMtrInfo.dFreq = dParam1 - 1;
			}
		}

		switch ( gpMaSndDrvInfo->sMtrInfo.dCtrl )
		{
		  case MASNDDRV_MTR_MODE_OFF:
			MaDevDrv_WriteIntermediateReg( MAI_MTR_SETTING_2, 0 );
			break;
		  case MASNDDRV_MTR_MODE_DIRECT:
		  	MaDevDrv_WriteIntermediateReg( MAI_MTR_SETTING_2, 
						  (UINT8)((gpMaSndDrvInfo->sMtrInfo.dFreq << 4) | (gpMaSndDrvInfo->sMtrInfo.dBlink + 6)) );
			break;
		  case MASNDDRV_MTR_MODE_SYNC:
		  	MaDevDrv_WriteIntermediateReg( MAI_MTR_SETTING_2,
		  				  (UINT8)((gpMaSndDrvInfo->sMtrInfo.dFreq << 4) | (gpMaSndDrvInfo->sMtrInfo.dBlink + 2)) );
		  	break;
		}

		break;
	  case MASMW_MTR_DIRECT:
	  	if ( dParam1 > MASMW_MAX_MTR_DIRECT )
	  		return MASMW_ERROR_ARGUMENT;

	  	if ( gpMaSndDrvInfo->sMtrInfo.dCtrl == MASNDDRV_MTR_MODE_DIRECT )
	  	{
	  		MaDevDrv_WriteIntermediateReg( MAI_MTR_SETTING_1, (UINT8)dParam1 );
	  	}

		break;
	  case MASMW_GET_PLLOUT:
	  	sdResult = (SINT32)MA_PLL_OUT;
		break;
	  case MASMW_HW_INITIALIZE:
	  	sdResult = MaDevDrv_HardwareInitialize();
	  	if ( sdResult != MASMW_SUCCESS )
	  	{
			return sdResult;
		}

		/* initialize registers */
		MaDevDrv_WriteIntermediateReg( MAI_EXT_CTRL, (UINT8)MASNDDRV_MAI_EXT_CTRL );
		

		/* Initialize API Balance */
		MaDevDrv_WriteIntermediateReg( MAI_MASTER_VOLL, (UINT8)((MASNDDRV_DEFAULT_API_BALANCE << 1) & 0xF8) );
		MaDevDrv_WriteIntermediateReg( MAI_MASTER_VOLR, (UINT8)((MASNDDRV_DEFAULT_API_BALANCE << 1) & 0xF8) );

	  	break;
	  case MASMW_GET_LEDSTATUS:
	  	if ( gpMaSndDrvInfo->sLedInfo.dCtrl == MASNDDRV_LED_MODE_SYNC )
	  		sdResult = 1;
	  	else
	  		sdResult = 0;
		break;
	  case MASMW_GET_VIBSTATUS:
	  	if ( gpMaSndDrvInfo->sMtrInfo.dCtrl == MASNDDRV_MTR_MODE_SYNC )
	  		sdResult = 1;
	  	else
	  		sdResult = 0;
		break;
	  case MASMW_WRITE_IMREG:
	  
	  	if ( dParam1 == 3 )
	  	{
			/* Bank#3 */
#if (MA_DEVICETYPE == MASMW_DEVICETYPE_MA5)
			/* error, if device type isn't MA-5S or MA-5i */
			return MASMW_ERROR_ARGUMENT;
#else
			
			if ( dParam2 >= MASNDDRV_NUM_BANK3 )
			{
				/* check register index */
				return MASMW_ERROR_ARGUMENT;
			}

			if ( bImRegJudgeW[MA_DEVICETYPE-MASMW_DEVICETYPE_MA5S][dParam2] == 0 )
			{
				/* not writable register */
				return MASMW_ERROR_ARGUMENT;
			}
			
			/* enter the critical section */
			machdep_EnterCriticalSection();

			/* change bank to 3 */
			MaDevDrv_WriteIntermediateReg( MAI_BASIC_SETTING, 3 );
			/* write to the specified register */
			MaDevDrv_WriteIntermediateReg( dParam2, (UINT8)(dParam3&0xFF) );
			/* change bank to 0 */
			MaDevDrv_WriteIntermediateReg( MAI_BASIC_SETTING, 0 );

			/* leave the critical section */
			machdep_LeaveCriticalSection();
#endif /* (MA_DEVICETYPE == MASMW_DEVICETYPE_MA5) */
		}
		else
		{
			/* not Bank#3 */
			return MASMW_ERROR_ARGUMENT;
		}
		
		break;
	  case MASMW_READ_IMREG:
	  
	  	if ( dParam1 == 3 )
	  	{
			/* Bank#3 */
#if (MA_DEVICETYPE == MASMW_DEVICETYPE_MA5)
			/* error, if device type isn't MA-5S or MA-5i */
			return MASMW_ERROR_ARGUMENT;
#else
			
			if ( dParam2 >= MASNDDRV_NUM_BANK3 )
			{
				/* check register index */
				return MASMW_ERROR_ARGUMENT;
			}

			if ( bImRegJudgeR[MA_DEVICETYPE-MASMW_DEVICETYPE_MA5S][dParam2] == 0 )
			{
				/* not writable register */
				return MASMW_ERROR_ARGUMENT;
			}
			
			/* enter the critical section */
			machdep_EnterCriticalSection();

			/* change bank to 3 */
			MaDevDrv_WriteIntermediateReg( MAI_BASIC_SETTING, 3 );
			/* write to the specified register */
			sdResult = (SINT32)MaDevDrv_ReadIntermediateReg( dParam2 );
			/* change bank to 0 */
			MaDevDrv_WriteIntermediateReg( MAI_BASIC_SETTING, 0 );

			/* leave the critical section */
			machdep_LeaveCriticalSection();
#endif /* (MA_DEVICETYPE == MASMW_DEVICETYPE_MA5) */
		}
		else
		{
			/* not Bank#3 */
			return MASMW_ERROR_ARGUMENT;
		}

		break;
	  case MASMW_GET_DEVICETYPE:
	  /* return value in mamachdep.h */
	  	sdResult = (SINT32)MA_DEVICETYPE;
	  	break;
	  default:
		sdResult = MASMW_ERROR;
	}
	return sdResult;
}
/****************************************************************************
 *	MaSndDrv_End
 *
 *	Description:
 *			End processing of the MA Sound Driver.
 *	Argument:
 *			None
 *	Return:
 *			0			success
 *			< 0			error code
 *
 ****************************************************************************/
SINT32 MaSndDrv_End(void)
{
	MASNDDRV_DBGMSG(("MaSndDrv_End:\n"));

	return MASMW_SUCCESS;
}
/****************************************************************************
 *	MaSndDrv_Initialize
 *
 *	Description:
 *			Initialize the MA Sound Driver
 *	Argument:
 *			None
 *	Return:
 *			0			success
 *			< 0			error code
 *
 ****************************************************************************/
SINT32 MaSndDrv_Initialize(void)
{
	UINT32	dSeqId;						/* sequence id */
	UINT32	dCh;						/* channel number */
	UINT32	dVoiceType;					/* voice type */
	UINT32	i;							/* loop counter */
	PMASNDDRVINFO	psSndDrvInfo;
	PMASNDDRVCHINFO	psChInfo;

	MASNDDRV_DBGMSG(("MaSndDrv_Initialize:\n"));

	gpMaSndDrvInfo = &gMaSndDrvInfo;

	for ( dSeqId = 0; dSeqId < MASMW_NUM_SEQTYPE; dSeqId++ )
	{
		/* initialize bCtrlSequencer **********/
		gpMaSndDrvInfo->bCtrlSequencer[dSeqId]	= 0;
		
		/* initialize dHvEventInfo **********/
		gpMaSndDrvInfo->dHvEventInfo[dSeqId] = 0;
		
		/* initialize pfnSrmCnv **********/
		gpMaSndDrvInfo->pfnSrmCnv[dSeqId] = NULL;

		/* initialize gpMaSndDrvInfo **********/
		psSndDrvInfo = &gpMaSndDrvInfo->sSndDrvInfo[dSeqId];
		
		psSndDrvInfo->bStatus				= (UINT8)MASNDDRV_STATUS_IDLE;
		psSndDrvInfo->bDrumMode				= (UINT8)0;
		psSndDrvInfo->bDvaMode				= (UINT8)MASNDDRV_RESMODE2_NORMAL;
		psSndDrvInfo->bMelodyMode			= (UINT8)0;
		psSndDrvInfo->bDrumType				= (UINT8)0;
		psSndDrvInfo->bWtVolMode			= (UINT8)MASNDDRV_WTVOL_MODE_NORMAL;
		psSndDrvInfo->bHvType				= (UINT8)0;
		psSndDrvInfo->bBalVolumeL			= (UINT8)MASNDDRV_DEFAULT_API_BALANCE;
		psSndDrvInfo->bBalVolumeR			= (UINT8)MASNDDRV_DEFAULT_API_BALANCE;
		psSndDrvInfo->bFmVolume				= (UINT8)MASNDDRV_DEFAULT_API_VOLUME;
		psSndDrvInfo->bWtVolume				= (UINT8)MASNDDRV_DEFAULT_API_VOLUME;
		psSndDrvInfo->bSaVolume				= (UINT8)MASNDDRV_DEFAULT_API_VOLUME;
		psSndDrvInfo->bHvVolume				= (UINT8)MASNDDRV_DEFAULT_API_VOLUME;
		psSndDrvInfo->bMasterVolShiftL		= (UINT8)0;
		psSndDrvInfo->bMasterVolShiftR		= (UINT8)0;
		psSndDrvInfo->bFmVolShift			= (UINT8)0;
		psSndDrvInfo->bWtVolShift			= (UINT8)0;
		psSndDrvInfo->bSaVolShift			= (UINT8)0;
		psSndDrvInfo->bHvVolShift			= (UINT8)0;
		psSndDrvInfo->bMasterVolume 		= (UINT8)MASNDDRV_DEFAULT_MASTER_VOLUME;;
		psSndDrvInfo->bWaveSlave			= (UINT8)0;
		psSndDrvInfo->bSpeed				= (UINT8)100;
		psSndDrvInfo->sbKeyOffset			= (SINT8)0;
		psSndDrvInfo->bSaLedMtrSync			= (UINT8)0xFF;
		psSndDrvInfo->bHvLedMtrSync			= (UINT8)0;
		psSndDrvInfo->bHvChannel			= MASNDDRV_HV_EXCHANNEL;
		psSndDrvInfo->dTimeBase				= (UINT8)0;
		psSndDrvInfo->dChNum				= 0UL;
		psSndDrvInfo->dFmNum				= 0UL;
		psSndDrvInfo->dWtNum				= 0UL;
		psSndDrvInfo->dSaNum				= 0UL;
		psSndDrvInfo->dAlNum				= 0UL;
		psSndDrvInfo->dHvNum				= 0UL;
		psSndDrvInfo->dRbNum				= 0UL;
		psSndDrvInfo->dChMap				= 0x00000000UL;
		psSndDrvInfo->dFmMap				= 0x00000000UL;
		psSndDrvInfo->dWtMap				= 0x00000000UL;
		psSndDrvInfo->dSaMap				= 0x00000000UL;
		psSndDrvInfo->dAlMap				= 0x00000000UL;
		psSndDrvInfo->dHvMap				= 0x00000000UL;
		psSndDrvInfo->dRbMap				= 0x00000000UL;
		psSndDrvInfo->dLdMap				= 0x00000000UL;
		psSndDrvInfo->dMtMap				= 0x00000000UL;
		psSndDrvInfo->dEosFlag				= 0UL;

		for ( i = 0; i < 3; i++ )
		{
			psSndDrvInfo->dMinSlot[i]  = 0UL;
			psSndDrvInfo->dMaxSlot[i]  = 0UL;
		}

		/* initialize sStreamInfo **********/
		for ( i = 0; i < MA_MAX_REG_STREAM_AUDIO; i++ )
		{
			gpMaSndDrvInfo->sStreamInfo[dSeqId][i].bFormat	  = (UINT8)0;
			gpMaSndDrvInfo->sStreamInfo[dSeqId][i].bPan		  = (UINT8)255;
			gpMaSndDrvInfo->sStreamInfo[dSeqId][i].dFrequency = 0UL;
			gpMaSndDrvInfo->sStreamInfo[dSeqId][i].dSeekP     = 0UL;
		}
	}

	/* initialize sVoiceInfo **********/
	for ( dVoiceType = 0; dVoiceType < MASNDDRV_NUM_VOICEINFO; dVoiceType++ )
	{
		for ( i = 0; i < MASNDDRV_MAX_REG_VOICE*2; i++ )
		{
			gpMaSndDrvInfo->sVoiceInfo[dVoiceType][i].wBankProg	= (UINT16)0;
			gpMaSndDrvInfo->sVoiceInfo[dVoiceType][i].bSynth	= (UINT8)1;
			gpMaSndDrvInfo->sVoiceInfo[dVoiceType][i].wKey		= (UINT16)0;
			gpMaSndDrvInfo->sVoiceInfo[dVoiceType][i].wRamAdrs	= (UINT16)0xFFFF;
		}
	}
	
	/* initialize bVoiceIndex **********/
	for ( i = 0; i < (MASNDDRV_MAX_RAM_BANK * 2 * 128); i++ )
	{
		gpMaSndDrvInfo->bVoiceIndex[i] = (UINT8)0xFF;
	}

	/* initialize sAlVoiceInfo **********/
	for ( i = 0; i < MASNDDRV_NUM_ALVOICE; i++ )
	{
		gpMaSndDrvInfo->sAlVoiceInfo[i].dBankNo      = 0xFFFF;
		gpMaSndDrvInfo->sAlVoiceInfo[i].dProgNo      = 0xFFFF;
		gpMaSndDrvInfo->sAlVoiceInfo[i].pAlVoice     = NULL;
		gpMaSndDrvInfo->sAlVoiceInfo[i].dAlVoiceSize = 0;
	}
	
	/* initialize sChInfo **********/
	for ( dCh = 0; dCh < MASNDDRV_NUM_CHANNEL; dCh++ )
	{
		psChInfo = &gpMaSndDrvInfo->sChInfo[dCh];
		
		psChInfo->bBankNo		 = (UINT8)( ( dCh == 9 ) ? 128 : 0 );
		psChInfo->bProgNo 		 = (UINT8)0;
		psChInfo->bPoly 		 = (UINT8)MASNDDRV_MODE_POLY;	/* poly mode */
		psChInfo->bSfx 			 = (UINT8)0;					/* all off */
		psChInfo->bVolume 		 = (UINT8)100;
		psChInfo->bExpression	 = (UINT8)127;
		psChInfo->bBendRange	 = (UINT8)2;
		psChInfo->bKeyControl	 = (UINT8)0;
		psChInfo->bNoteOn 		 = (UINT8)0;
		psChInfo->bSlotNo		 = (UINT8)0xFF;
		psChInfo->bAlFlag		 = (UINT8)0x7F;
		psChInfo->bLedSync		 = (UINT8)0;
		psChInfo->bMtrSync		 = (UINT8)0;
		psChInfo->bVolInterp	 = (UINT8)1;
		psChInfo->bPcFlag		 = (UINT8)0;
		psChInfo->dLpfQ			 = 0UL;
		psChInfo->dLpfCof		 = 0UL;
		psChInfo->sdLpfQShift	 = 0L;
		psChInfo->sdLpfCofShift	 = 0L;
		psChInfo->dPitchBend	 = 0x0400;
		psChInfo->dCoaseTune	 = 0x0400;
		psChInfo->dFineTune		 = 0x0400;
		psChInfo->dTunePitch	 = 0x0400;
	}

	/* initialize sSlotInfo **********/
	for ( i = 0; i < MASNDDRV_NUM_SLOT; i++ )
	{
		gpMaSndDrvInfo->sSlotInfo[i].bCh	= (UINT8)0;
		gpMaSndDrvInfo->sSlotInfo[i].dKey	= 0UL;
	}

	/* initialize sLedInfo **********/
	gpMaSndDrvInfo->sLedInfo.dCtrl  = 0UL;
	gpMaSndDrvInfo->sLedInfo.dBlink = 0UL;
	gpMaSndDrvInfo->sLedInfo.dFreq  = 0UL;

	/* initialize sMtrInfo **********/
	gpMaSndDrvInfo->sMtrInfo.dCtrl  = 0UL;
	gpMaSndDrvInfo->sMtrInfo.dBlink = 0UL;
	gpMaSndDrvInfo->sMtrInfo.dFreq  = 0UL;

	/* initialize sStreamCtrlInfo **********/
	gpMaSndDrvInfo->sStreamCtrlInfo.dQueFlag	 = 0;
	gpMaSndDrvInfo->sStreamCtrlInfo.dType[0]	 = 0;
	gpMaSndDrvInfo->sStreamCtrlInfo.dType[1]	 = 0;
	gpMaSndDrvInfo->sStreamCtrlInfo.dRamVal[0]	 = 0;
	gpMaSndDrvInfo->sStreamCtrlInfo.dRamVal[1]	 = 0;
	gpMaSndDrvInfo->sStreamCtrlInfo.dWaveId[0]	 = 0;
	gpMaSndDrvInfo->sStreamCtrlInfo.dWaveId[1]	 = 0;
	gpMaSndDrvInfo->sStreamCtrlInfo.dStreamStatus = 0;



	return MASMW_SUCCESS;
}

/****************************************************************************
 *	MaSndDrv_Mix40LogVol
 *
 *	Description:
 *			Mix volumes[40Log(x/127)].
 *	Arguments:
 *			dVol1		volume (0..127)
 *			dVol2		volume (0..127)
 *	Return:
 *			Mixed volume (0..127).
 ****************************************************************************/
UINT32 MaSndDrv_Mix40LogVol
(
	UINT32	dVol1,						/* volume value */
	UINT32	dVol2						/* volume value */
)
{
	UINT32	dDb;

	dDb = (UINT32)db_table[dVol1 & 0x7F] + (UINT32)db_table[dVol2 & 0x7F];
	if ( dDb > MASNDDRV_MAX_DB ) dDb = MASNDDRV_MAX_DB;

	return ((UINT32)vol_table[dDb]);
}


/****************************************************************************
 *	MaSndDrv_Convert20to40Log
 *
 *	Description:
 *			Convert volume[20Log(x/127)] to volume[40Log(x/127)].
 *	Arguments:
 *			dVol		volume (0..127)
 *	Return:
 *			volume[40Log(x/127)] (0..127).
 ****************************************************************************/
UINT32 MaSndDrv_Convert20to40Log
(
	UINT32	dVol						/* volume value */
)
{
	return ((UINT32)ab20Log40Log_table[dVol & 0x7F]);
}




